(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  OSrules.v                                                               *)
(*                                                                          *)
(*  This file contains the definition of the operational                    *)
(*  semantics rules for lazy PCF, as well as a definition                   *)
(*  of the Ap function and some related properties.                         *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November  1993                                                          *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                OSrules.v                                 *)
(****************************************************************************)

Require PolyList.
Require syntax.
Require environments.

Require typecheck.

Require rename.

(************************************************)
(*  OScons  (abbrev.)				*)
(*    (OScons v t e A) == 			*)
(*     (cons VTT <VT,tm>(<vari,ty>(v,t),e) A)	*)
(************************************************)

Definition OScons :=
    [v:vari][t:ty][e:tm][A:OS_env]
       (cons VTT ((v,t),e) A).


(********************************************************)
(* Ap:							*)
(*    Inductively defines the relation characterized by *)
(*    the Ap function.					*)
(*							*)
(*    (Ap a F A F' n t) <--> Ap(F,a)=<F',[n:t->a]>	*)
(*        New variables may not come from 		*)
(*        the Domain of OS env A.			*)
(*							*)
(********************************************************)

                        
Inductive Ap [a:tm]  : tm -> OS_env -> tm -> vari -> ty -> Prop
    :=  Ap_abs :(nv,v:vari)(t:ty)(e,ne:tm)(A:OS_env)
                (~(member vari nv (OS_Dom A))) ->
                (rename nv v e ne) ->
		(Ap a (abs v t e) A  ne nv t)
    |  Ap_clos: (n,v:vari)(s,t:ty)(e,ne,e1:tm)(A:OS_env)
                (Ap a e (OScons v s e1 A) ne n t) ->
		(Ap a (clos e v s e1) A (clos ne v s e1) n t).


(********************************)
(*   ApNewVar  			*)
(*     Ap(a,fun,A) = 		*)
(*       <b,[n:t->a]> --->	*)
(*     n not in Dom(A)		*)
(********************************)

Goal (a,fun,b:tm)(A:OS_env)(n:vari)(t:ty)
     (Ap a fun A  b n t) ->
     ~(member vari n (OS_Dom A)).

   Induction 1; Intros.
   Assumption.
   Red; Intro; Apply H1; Simpl.
   Right; Assumption.
Save ApNewVar.



(****************************************)
(*  OSrules				*)
(*					*)
(*  Definition of Operational Semantics	*)
(*					*)
(*  <<e,A>> -> <<e',A'>>		*)
(*					*)
(****************************************)

Inductive OSred : config -> config -> Prop
 	:= OS_C0 : (A:OS_env) (OSred (cfg o A) (cfg o A))
	| OS_CT : (A:OS_env) (OSred (cfg ttt A) (cfg ttt A))
	| OS_CF : (A:OS_env) (OSred (cfg fff A) (cfg fff A))
        | OS_L  : (A:OS_env)(e:tm)(t:ty)(x:vari)
	   		(OSred (cfg (abs x t e) A) (cfg (abs x t e) A)) 
	| OS_P0 : (A,A':OS_env)(e:tm)
 		  (OSred (cfg e A)        (cfg o A')) ->
 		  (OSred (cfg (prd e) A) (cfg o A'))
	| OS_P  : (A,A':OS_env)(e,e1:tm)
 		  (OSred (cfg e A)        (cfg (succ e1) A')) ->
 		  (OSred (cfg (prd e) A) (cfg e1 A'))
	| OS_ZT : (A,A':OS_env)(e:tm)
 		  (OSred (cfg e A)        (cfg o A')) ->
 		  (OSred (cfg (is_o e) A) (cfg ttt A'))
	| OS_ZF : (A,A':OS_env)(e,e1:tm)
 		  (OSred (cfg e A)        (cfg (succ e1) A')) ->
 		  (OSred (cfg (is_o e) A) (cfg fff A'))
	| OS_S  : (A,A':OS_env)(e,e1:tm)
 		  	(OSred (cfg e A)        (cfg  e1 A')) ->
 		  	(OSred (cfg (succ e) A) (cfg (succ e1) A'))
	|OS_Var1: (A,A':OS_env)(e,en:tm)(t:ty)(x:vari)
		  (~(member vari x (OS_Dom A))) ->
                        (OSred (cfg e A) (cfg en A')) ->
   			(OSred (cfg (var x) (OScons x t e A))
			       (cfg en (OScons x t en A')))
	|OS_Var2: (A,A':OS_env)(e,en:tm)(t:ty)(x,y:vari)
                  (~x=y) -> (~(member vari x (OS_Dom A))) ->
                        (OSred (cfg (var y) A) (cfg en A')) ->
   			(OSred (cfg (var y) (OScons x t e A))
			       (cfg en (OScons x t e A')))
	|OS_Appl: (A,A',A'':OS_env)(e1,e2,en,en',enf:tm)(n:vari)(t:ty)
                  (OSred (cfg e1 A) (cfg en A')) ->
                  (Ap e2 en A en' n t) ->
		  (OSred (cfg (clos en' n t e2) A') (cfg enf A'')) ->
		  (OSred (cfg (appl e1 e2) A) (cfg enf A''))
	|OS_IfTrue: (A,A',A'':OS_env) (e1,e2,e3,en:tm)
		    (OSred (cfg e1 A) (cfg ttt A')) ->
		    (OSred (cfg e2 A') (cfg en A'')) ->
		    (OSred (cfg (cond e1 e2 e3) A) (cfg en A''))
	|OS_IfFalse:(A,A',A'':OS_env) (e1,e2,e3,en:tm)
		    (OSred (cfg e1 A) (cfg fff A')) ->
		    (OSred (cfg e3 A') (cfg en A'')) ->
		    (OSred (cfg (cond e1 e2 e3) A) (cfg en A''))
	|OS_Fix  :  (A,A':OS_env)(e,e',en:tm)(x,nx:vari)(t:ty)
 		    (~(member vari nx (OS_Dom A))) ->
                    (rename nx x e e') -> 
		    (OSred (cfg (clos e' nx t (fix x t e)) A) (cfg en A')) ->
		    (OSred (cfg (fix x t e) A) (cfg en A'))
	|OS_CL   :  (A,A':OS_env)(e,e1,en,e1':tm)(x:vari)(t:ty)
		    (OSred (cfg e  (OScons x t e1 A))
			   (cfg en (OScons x t e1' A'))) ->
		    (s:ty)(TC (OS_Dom_ty (OScons x t e1 A)) en s) ->
		    (~((s=nat_ty) \/ (s=bool_ty))) ->
		    (OSred (cfg (clos e x t e1) A)
			   (cfg (clos en x t e1') A'))
	|OS_CL'  :  (A,A':OS_env)(e,e1,en,e1':tm)(x:vari)(t:ty)
		    (OSred (cfg e  (OScons x t e1 A))
			   (cfg en (OScons x t e1' A'))) ->
		    (s:ty)(TC (OS_Dom_ty (OScons x t e1 A)) en s) ->
		    ((s=nat_ty) \/ (s=bool_ty)) ->
		    (OSred (cfg (clos e x t e1) A)
			   (cfg en A')).
	

