(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  environments.v                                                          *)
(*                                                                          *)
(*  This file contains the definitions of type environments,                *)
(*  operational semantics environments, and configurations.                 *)
(*  It also contains the function mapsto which interprets                   *)
(*  type environments as partial mappings, and functions that               *)
(*  return the domains of environments.                                     *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November  1993                                                          *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                              environments.v                              *)
(****************************************************************************)

Require syntax.

Require PolyList.

(******************************)
(* (member x L)               *)
(*                            *)
(*  Defines when an element x *)
(*  is contained in a list L. *)
(*                            *)
(******************************)
Definition member :=                                  
 [A:Set][b:A][l:(list A)] (<Prop>Match l with
                  (* nil *)      False 
                  (* cons a m *) [a:A][m:(list A)][P:Prop]
                                      (a=b) \/ P end).

(******************************************************)
(*                                                    *)
(*  ty_env:  type environment, variable-type list.    *)
(*  OS_env:  Operational semantics environment,       *)
(*           variable-type-term list.                 *)
(*                                                    *)
(******************************************************)

Definition VT := vari*ty.
Definition ty_env := (list VT).

Definition VTT := VT*tm.
Definition OS_env := (list VTT).


(****************************************)
(*   Mapsto - 				*)
(*      Implements type environments as	*)
(*      partial mappings.		*)
(*					*)
(*   (mapsto x t H) -> H(x) = t		*)
(****************************************)

Definition mapsto :=
	[indx:vari][val:ty][l:(list VT)]
	(<Prop>Match l with
	(* Nil *) 	False
	(* cons a m *) 	[a:VT][m:(list VT)][P:Prop]
			if (Fst a)=indx
			then (Snd a)=val
			else P end).



(******************************************)
(*   Configurations and their destructors *)
(******************************************)

Inductive config : Set :=
               cfg : tm -> OS_env -> config.

Definition cfgexp := 
        [c:config](<tm>Case c of [e:tm][A:OS_env]e end).

Definition cfgenv := 
        [c:config](<OS_env>Case c of [e:tm][A:OS_env]A end).



(**************************************************)
(*  Domain functions for type and OS environments *)
(*						  *)
(*  TE_Dom    : ty_env -> var list                *)
(*  OS_Dom    : OS_env -> var list                *)
(*  OS_Dom_ty : OS_env -> VT list                 *)
(* 						  *)
(**************************************************)

Definition TE_Dom := 
        [H:ty_env](<(list vari)>Match H with
                  (* nil *)  (nil vari)
           (* cons vt H' *)  [vt:VT][H':ty_env][vl:(list vari)]
                             (cons vari (Fst vt) vl) end).

Definition OS_Dom := 
        [A:OS_env](<(list vari)>Match A with
                  (* nil *)  (nil vari)
          (* cons vtt A' *)  [vtt:VTT][A':OS_env][vl:(list vari)]
                             (cons vari (Fst (Fst vtt)) vl) end).

Definition OS_Dom_ty :=
        [A:OS_env](<(list VT)>Match A with
                    (* nil *)  (nil VT)
            (* cons vtt A' *)  [vtt:VTT][A':OS_env][vtl:(list VT)]
                                  (cons VT (Fst vtt) vtl) end).


