(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*  freevars.v                                                              *)
(*                                                                          *)
(*  This file contains the definition of free variables                     *)
(*  and some related properties.                                            *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November  1993                                                          *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                freevars.v                                *)
(****************************************************************************)

Require syntax.
Require utils.


(********************************************************)
(*   Free Variables, Definition                         *)
(*							*)
(*   (FV x e) -> x in FV(e)				*)
(*							*)
(********************************************************)

Inductive FV [z:vari] : tm -> Prop
        := FV_abs : (e:tm) (FV z e)->
                            (v:vari) ( ~z=v) ->  
                                   (t:ty)   (FV z (abs v t e)) 
        | FV_fix : (e:tm) (FV z e)->
                            (v:vari) ( ~z=v) ->  
                                   (t:ty)  (FV z (fix v t e)) 
        | FV_appl1: (e_1,e_2:tm) (FV z e_1)->  (FV z (appl e_1 e_2))
        | FV_appl2: (e_1,e_2:tm) (FV z e_2)->  (FV z (appl e_1 e_2))
        | FV_cond1: (e_1,e_2,e_3:tm) (FV z e_1) -> (FV z (cond e_1 e_2 e_3))
        | FV_cond2: (e_1,e_2,e_3:tm) (FV z e_2) -> (FV z (cond e_1 e_2 e_3))
        | FV_cond3: (e_1,e_2,e_3:tm) (FV z e_3) -> (FV z (cond e_1 e_2 e_3))
        | FV_var  : (v:vari) (z=v) ->  (FV z (var v))
        | FV_succ : (e:tm)(FV z e) ->  (FV z (succ e)) 
        | FV_prd : (e:tm)(FV z e) ->  (FV z (prd e)) 
        | FV_is_o : (e:tm)(FV z e) ->  (FV z (is_o e)) 
        | FV_closa: (v:vari)(t:ty)(e,e_1:tm) (FV z e_1) -> 
                            (FV z (clos e v t e_1))
        | FV_closb: (v:vari)(t:ty)(e,e_1:tm) (FV z e) -> (~z=v) ->
                            (FV z (clos e v t e_1)).



(****************************************************************)
(*  notFV							*)
(*    These theorems state what ~(FV v e) implies:		*)
(*								*)
(*    ~(FV x (abs v t e)) ---> x=v \/ ~(FV x e)		        *)
(*    ~(FV v (appl e1 e2)) ---> ~(FV v e1) /\ ~(FV v e2)	*)
(*    ~(FV v (cond e1 e2 e3)) ---> 				*)
(*		    ~(FV v e1) /\ ~(FV v e2) /\ ~(FV v e3)	*)
(*    ~(FV v (var x)) --->  ~v=x				*)
(*    ~(FV v (succ e)) ---> ~(FV v e)				*)
(*    ~(FV v (prd e)) ---> ~(FV v e)				*)
(*    ~(FV v (is_o e)) ---> ~(FV v e)				*)
(*    ~(FV x (fix v t e)) ---> x=v \/ ~(FV x e)		        *)
(*    ~(FV x (clos e v t a))->					*)
(*                   ~(FV x a) /\ (x=v \/ ~(FV x e))	        *)
(****************************************************************)


Goal (x,v:vari)(t:ty)(e:tm)
     (~(FV x (abs v t e))) -> ((x=v)\/~(FV x e)).
Intros.
Specialize (Xmidvar x v); Induction 1; Intro A.
Left; Assumption.
Right; Red; Intro; Apply H; Apply FV_abs; Assumption.
Save notFV_abs.


Goal (v:vari)(e1,e2:tm)
     (~(FV v (appl e1 e2))) -> ((~(FV v e1))/\~(FV v e2)).
Intros v e1 e2 N.
Split.
Red; Intro; Apply N; Apply FV_appl1; Assumption.
Red; Intro; Apply N; Apply FV_appl2; Assumption.
Save notFV_appl.

Goal (v:vari)(e1,e2,e3:tm)
     (~(FV v (cond e1 e2 e3))) -> 
     ((~(FV v e1))/\ (~(FV v e2))/\~(FV v e3)).
Intros v e1 e2 e3 N.
Split.
Red; Intro; Apply N; Apply FV_cond1; Assumption.
Split.
Red; Intro; Apply N; Apply FV_cond2; Assumption.
Red; Intro; Apply N; Apply FV_cond3; Assumption.
Save notFV_cond.


Goal (v,x:vari)(~(FV v (var x)))->~v=x.
Intros v x N.
Red; Intro; Apply N; Apply FV_var; Assumption.
Save notFV_var.

Goal (v:vari)(e:tm)(~(FV v (succ e))) -> ~(FV v e).
Intros v e N.
Red; Intro; Apply N; Apply FV_succ; Assumption.
Save notFV_succ.


Goal (v:vari)(e:tm)(~(FV v (prd e))) -> ~(FV v e).
Intros v e N.
Red; Intro; Apply N; Apply FV_prd; Assumption.
Save notFV_prd.

Goal (v:vari)(e:tm)(~(FV v (is_o e))) -> ~(FV v e).
Intros v e N.
Red; Intro; Apply N; Apply FV_is_o; Assumption.
Save notFV_is_o.

Goal (x,v:vari)(t:ty)(e:tm)
     (~(FV x (fix v t e))) -> ((x=v)\/~(FV x e)).
Intros.
Specialize (Xmidvar x v); Induction 1; Intro A.
Left; Assumption.
Right; Red; Intro; Apply H; Apply FV_fix; Assumption.
Save notFV_fix.

Goal (x,v:vari)(t:ty)(e,a:tm)
     (~(FV x (clos e v t a)))->
     ((~(FV x a))/\ ((x=v)\/~(FV x e))).
Intros.
Split.
Red; Intro; Apply H; Apply FV_closa; Assumption.
Specialize (Xmidvar x v); Induction 1; Intro A.
Left; Assumption.
Right; Red; Intro; Apply H; Apply FV_closb; Assumption.
Save notFV_clos.



(********************************************************)
(*  inv_FV						*)
(*     These theorems state what (FV v e) implies.	*)
(*  							*)
(*  (FV v 0) ---> False					*)
(*  (FV v ttt) ---> False				*)
(*  (FV v fff) ---> False				*)
(*  (FV v (abs x t e)) ---> (FV v e) /\ ~v=x		*)
(*  (FV v (fix x t e)) ---> (FV v e) /\ ~v=x	        *)
(*  (FV v (appl e1 e2)) --->  (FV v e1)\/(FV v e2)	*)
(*  (FV v (succ e) ---> (FV v e)			*)
(*  (FV v (prd e) ---> (FV v e)			        *)
(*  (FV v (is_o e) ---> (FV v e)			*)
(*  (FV v (clos e x t a)) ---> (FV v a) \/ 		*)
(*                             (FV v e)/\~v=x	        *)
(*  							*)
(*  They use the function fv : vari->tm->Prop which	*)
(*  defines the free variable relation in terms of the	*)
(*  Match operator.  The theorem FV_fv is proved 	*)
(*  by induction, and each of the individual FV_e	*)
(*  theorems is an instance of the FV_fv theorem.	*)
(*  							*)
(********************************************************)

Definition fv :=
        [v:vari][e:tm]
        (<Prop>Case e of
	(* o *)		False
	(* ttt *)   	False
	(* fff *)   	False
	(* abs x s e *) [x:vari][s:ty][e:tm]
			(FV v e) /\ ~v=x
	(* appl e1 e2 *) [e1,e2:tm]
			 (FV v e1)\/(FV v e2) 
	(* cond e1 e2 e3 *)[e1,e2,e3:tm]
			 (FV v e1)\/(FV v e2)\/(FV v e3) 
	(* var x *)	[x:vari] v=x
	(* succ n *)	[n:tm](FV v n)
	(* prd n *)	[n:tm](FV v n)
	(* is_o n *)	[n:tm](FV v n)
	(* fix x s e *)	[x:vari][s:ty][e:tm]
			(FV v e) /\ ~v=x
	(* clos e x s e1 *)[e:tm]
			   [x:vari][s:ty][e1:tm]
			   (FV v e1) \/ ((FV v e)/\~v=x) end).


Goal (v:vari)(e:tm)(FV v e) -> (fv v e).

Induction 1; Simpl; Intros.
Split; Assumption.
Split; Assumption.
Left; Assumption.
Right; Assumption.
Left; Assumption.
Right; Left; Assumption.
Right; Right; Assumption.
Assumption.
Assumption.
Assumption.
Assumption.
Left; Assumption.
Right; Split; Assumption.
Save FV_fv.


Goal  (v:vari)~(FV v o).
Intro v; Red; Intro F.
Change (fv v o).
Apply FV_fv; Assumption.
Save inv_FV_o.


Goal  (v:vari)~(FV v ttt).
Intro v; Red; Intro F.
Change (fv v ttt).
Apply FV_fv; Assumption.
Save inv_FV_ttt.


Goal  (v:vari)~(FV v fff).
Intro v; Red; Intro F.
Change (fv v fff).
Apply FV_fv; Assumption.
Save inv_FV_fff.

Goal (v,x:vari)(t:ty)(e:tm)
     (FV v (abs x t e)) -> ((FV v e) /\ ~v=x).
Intros v x t e F.
Change (fv v (abs x t e)).
Apply FV_fv; Assumption.
Save inv_FV_abs.

Goal (v,x:vari)(t:ty)(e:tm)
     (FV v (fix x t e)) -> ((FV v e) /\ ~v=x).
Intros v x t e F.
Change (fv v (fix x t e)).
Apply FV_fv; Assumption.
Save inv_FV_fix.

Goal (v:vari)(e1,e2:tm)
     (FV v (appl e1 e2)) -> ((FV v e1)\/(FV v e2)).
Intros v e1 e2 F.
Change (fv v (appl e1 e2)).
Apply FV_fv; Assumption.
Save inv_FV_appl.

Goal (v:vari)(e1,e2,e3:tm)
     (FV v (cond e1 e2 e3)) -> ((FV v e1)\/(FV v e2)\/(FV v e3)).
Intros v e1 e2 e3 F.
Change (fv v (cond e1 e2 e3)).
Apply FV_fv; Assumption.
Save inv_FV_cond.

Goal (v,x:vari)
     (FV v (var x)) -> v=x.
Intros v x F.
Change (fv v (var x)).
Apply FV_fv; Assumption.
Save inv_FV_var.

Goal (v:vari)(e:tm)
     (FV v (succ e)) -> (FV v e).
Intros v e F.
Change (fv v (succ e)).
Apply FV_fv; Assumption.
Save inv_FV_succ.


Goal (v:vari)(e:tm)
     (FV v (prd e)) -> (FV v e).
Intros v e F.
Change (fv v (prd e)).
Apply FV_fv; Assumption.
Save inv_FV_prd.


Goal (v:vari)(e:tm)
     (FV v (is_o e)) -> (FV v e).
Intros v e F.
Change (fv v (is_o e)).
Apply FV_fv; Assumption.
Save inv_FV_is_o.


Goal (v,x:vari)(t:ty)(e,a:tm)
     (FV v (clos e x t a)) -> ((FV v a) \/ ((FV v e)/\~v=x)).
Intros v x t e a F.
Change (fv v (clos e x t a)).
Apply FV_fv; Assumption.
Save inv_FV_clos.


