(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  typecheck.v                                                             *)
(*                                                                          *)
(*  This file contains the definition of the type judgment                  *)
(*  rules and the theorems stating their inverses.                          *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November  1993                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               typecheck.v                                *)
(****************************************************************************)

Require environments.
Require PolyList.

Require syntax.

(****************************************)
(*   Type Judgment Rules		*)
(*					*)
(*   (TC H e t) -> H |- e : t		*)
(****************************************)

Inductive TC : ty_env ->  tm -> ty -> Prop
        :=  TC_o     : (H:ty_env)(TC H o nat_ty)
       	|  TC_ttt   : (H:ty_env)(TC H ttt bool_ty)
 	|  TC_fff   : (H:ty_env)(TC H fff bool_ty)
	|  TC_succ  : (H:ty_env)(e:tm)(TC H e nat_ty) -> (TC H (succ e) nat_ty)
	|  TC_prd  : (H:ty_env)(e:tm)(TC H e nat_ty) -> (TC H (prd e) nat_ty)
	|  TC_is_o  : (H:ty_env)(e:tm)(TC H e nat_ty) -> (TC H (is_o e) bool_ty)
	|  TC_var   : (H:ty_env) (v:vari)(t:ty) 
			    (mapsto v t H) ->  (TC H (var v) t)
	|  TC_appl  : (H:ty_env) (e,e1:tm)(s,t:ty) 
			    (TC H e (arr s t)) -> (TC H e1 s)
                                  ->  (TC H (appl e e1) t)
 	|  TC_abs   : (H:ty_env) (v:vari)(e:tm)(s,t:ty) 
                             (TC (cons VT (v,s) H) e t) 
  				  -> (TC H (abs v s e) (arr s t))
	|  TC_cond  : (H:ty_env) (e1,e2,e3:tm)(t:ty)
			   (TC H e1 bool_ty) -> (TC H e2 t) -> (TC H e3 t)
  				  -> (TC H (cond e1 e2 e3) t)
	|  TC_fix   : (H:ty_env) (e:tm)(t:ty)(v:vari)
                             (TC (cons VT (v,t) H) e t) 
  				  -> (TC H (fix v t e)  t)
	|  TC_clos  : (H:ty_env) (e,e1:tm)(s,t:ty)(v:vari)
			     (TC H e1 s) -> (TC (cons VT (v,s) H) e t) 
				  -> (TC H (clos e v s e1) t).
                                

(****************************************************************)
(*								*)
(*  tc								*)
(*    Uses Match to implement the type judgement rules.		*)
(*								*)
(*  tc_TC 							*)
(*    Proves (tc H e t) -> (TC H e t).  Each of the TC_inv_e	*)
(*    theorems is an instance of this one.			*)
(*								*)
(****************************************************************)

Definition tc :=
        [H:ty_env][e:tm][t:ty]
        (<Prop>Case e of
	(* o *)		(t=nat_ty)
	(* ttt *)   	(t=bool_ty)
	(* fff *)   	(t=bool_ty)
	(* abs v s e *) [v:vari][s:ty][e:tm]
			((Ex [r:ty] (t=(arr s r)) /\
			(TC (cons VT (v,s) H) e r)))
	(* appl e1 e2 *) [e1,e2:tm]
			 ((Ex [s:ty](TC H e1 (arr s t)) /\ (TC H e2 s)))
	(* cond e1 e2 e3 *)[e1,e2,e3:tm]
			   (TC H e1 bool_ty) /\ (TC H e2 t) /\ (TC H e3 t)
	(* var v *)	[v:vari]  (mapsto v t H)
	(* succ n *)	[n:tm](t=nat_ty) /\ (TC H n nat_ty)
	(* prd n *)	[n:tm](t=nat_ty) /\ (TC H n nat_ty)
	(* is_o n *)	[n:tm](t=bool_ty) /\ (TC H n nat_ty)
	(* fix v s e1 *)[v:vari][s:ty][e1:tm]
			((s=t) /\
			 (TC (cons VT (v,s) H) e1 t))
	(* clos e v s e1 *)[e:tm][v:vari][s:ty][e1:tm]
			   (TC H e1 s) /\ (TC (cons VT (v,s) H) e t) end).


Goal (H:ty_env)(e:tm)(t:ty)(TC H e t) -> (tc H e t).
Induction 1; Simpl; Intros.
Reflexivity.
Reflexivity.
Reflexivity.
Split; Reflexivity Orelse Assumption.
Split; Reflexivity Orelse Assumption.
Split; Reflexivity Orelse Assumption.
Assumption.
Exists s; Split; Assumption.
Exists t0; Split; Reflexivity Orelse Assumption.
Split; Assumption Orelse Split; Assumption.
Split; Reflexivity Orelse Assumption.
Split; Assumption.
Save TC_tc.


(********************************************************)
(*							*)
(*   inv_TC_e Rules:					*)
(*	For each term e, inv_TC_e claims the inverse of *)
(*	(TC H e t), meaning if an expression   		*)
(*	is type correct, then the hypotheses of its TC  *)
(*	Rule must hold.					*)
(*							*)
(********************************************************)

Goal (H:ty_env)(t:ty)(TC H o t)->(t=nat_ty).
 Intros H t HTC.  Change (tc H o t).
 Apply TC_tc; Assumption.
Save inv_TC_o.

Goal (H:ty_env)(t:ty)(TC H ttt t)->(t=bool_ty).
 Intros H t HTC.  Change (tc H ttt t).
 Apply TC_tc; Assumption.
Save inv_TC_ttt.

Goal (H:ty_env)(t:ty)(TC H fff t)->(t=bool_ty).
 Intros H t HTC.  Change (tc H fff t).
 Apply TC_tc; Assumption.
Save inv_TC_fff.

Goal (H:ty_env)(t:ty)(e0:tm)
	(TC H (prd e0) t) ->((t=nat_ty)/\ (TC H e0 nat_ty)).
 Intros H t e0 HTC.  Change (tc H (prd e0) t).
 Apply TC_tc; Assumption.
Save inv_TC_prd.

Goal (H:ty_env)(t:ty)(e0:tm)
	(TC H (succ e0) t) ->((t=nat_ty)/\ (TC H e0 nat_ty)).
 Intros H t e0 HTC.  Change (tc H (succ e0) t).
 Apply TC_tc; Assumption.
Save inv_TC_succ.

Goal (H:ty_env)(t:ty)(e0:tm)
	(TC H (is_o e0) t) ->((t=bool_ty)/\ (TC H e0 nat_ty)).
 Intros H t e0 HTC.  Change (tc H (is_o e0) t).
 Apply TC_tc; Assumption.
Save inv_TC_is_o.

Goal (H:ty_env)(t:ty)(v:vari)
	(TC H (var v) t) -> (mapsto v t H).
 Intros H t v HTC.  Change (tc H (var v) t).
 Apply TC_tc; Assumption.
Save inv_TC_var.

Goal (H:ty_env)(t:ty)(e1,e2:tm)
	(TC H (appl e1 e2) t) -> 
		((Ex [s:ty](TC H e1 (arr s t)) /\ (TC H e2 s))).
 Intros H t e1 e2 HTC.  Change (tc H (appl e1 e2) t).
 Apply TC_tc; Assumption.
Save inv_TC_appl.

Goal (H:ty_env)(t,s:ty)(v:vari)(e:tm)
	(TC H (abs v s e) t) -> 
		((Ex [r:ty] (t=(arr s r)) /\ 
		(TC (cons VT (v,s) H) e r))).
 Intros H t s v e HTC.  Change (tc H (abs v s e) t).
 Apply TC_tc; Assumption.
Save inv_TC_abs.

Goal (H:ty_env)(t:ty)(e1,e2,e3:tm)
	(TC H (cond e1 e2 e3) t) -> 
	   ((TC H e1 bool_ty) /\ (TC H e2 t) /\ (TC H e3 t)).
 Intros H t e1 e2 e3 HTC.  Change (tc H (cond e1 e2 e3) t).
 Apply TC_tc; Assumption.
Save inv_TC_cond.


Goal (H:ty_env)(s,t:ty)(e:tm)(v:vari)
	(TC H (fix v s e) t) -> 
			((s=t) /\
			 (TC (cons VT (v,s) H) e t)).
 Intros H s t e v HTC.  Change (tc H (fix v s e) t).
 Apply TC_tc; Assumption.
Save inv_TC_fix.

Goal (H:ty_env)(t,s:ty)(e,e1:tm)(v:vari)
	(TC H (clos e v s e1) t) -> 
		((TC H e1 s) /\ (TC (cons VT (v,s) H) e t)).
 Intros H t s e e1 v HTC.  Change (tc H (clos e v s e1) t).
 Apply TC_tc; Assumption.
Save inv_TC_clos.


