(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  utils.v                                                                 *)
(*                                                                          *)
(*  Contains miscellaneous useful definitions and theorems                  *)
(*  used in later proofs and definitions.                                   *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November  1993                                                          *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                 utils.v                                  *)
(****************************************************************************)

Require syntax.


(********************************************************)
(*							*)
(*   vari_nat:						*)
(*       (m,n:nat)  (x m) = (x n)  --> m = n		*)
(*							*)
(*   valu: a deconstructor for type vari 		*)
(*							*)
(********************************************************)

Definition valu := [v:vari](<nat>Match v with (* x n *) [n:nat]n end).

Goal (m,n:nat)(<vari>(x m)=(x n)) -> (<nat>m=n).
Intros m n Q.  
Replace m with (valu (x m)).
Rewrite -> Q.
Simpl; Reflexivity.
Simpl; Reflexivity.
Save vari_nat.


(********************************************************)
(*							*)
(*   subty_eq:						*)
(*	(s -> r) = (q -> t)  ---->  s = q /\ r = t	*)
(*							*)
(*   Rator_ty, Rand_ty:  Arrow type destructors giving  *)
(*   the types of the operator and operand respectively.*)
(*							*)
(********************************************************)

Definition Rator_ty :=
        [t:ty](<ty>Match t with t t
                        [t1,rt1,t2,rt2:ty] t1 end).
Definition Rand_ty :=
	[t:ty](<ty>Match t with t t
			[t1,rt1,t2,rt2:ty] t2 end).
Goal (s1,t1,s2,t2:ty)
     (<ty>(arr s1 t1)=(arr s2 t2)) -> ((<ty>s1=s2)/\<ty>t1=t2).
Intros s1 t1 s2 t2 Q.
Split.
Change  <ty>(Rator_ty (arr s1 t1))=(Rator_ty (arr s2 t2)).
Apply (f_equal ty ty Rator_ty). Assumption.
Change <ty>(Rand_ty (arr s1 t1))=(Rand_ty (arr s2 t2)).
Apply (f_equal ty ty Rand_ty); Assumption.
Save subty_eq.



(****************************************)
(* Type properties 			*)
(*     Proof that the type constructors	*) 
(*     nat_ty, bool_ty, and arr_ty are	*)
(*     distinct				*)
(****************************************)


Definition is_nat :=
	[t:ty](<Prop>Match t with True 
 				  False
				  [t1:ty][P1:Prop][t2:ty][P2:Prop]False end).
Definition is_bool :=
	[t:ty](<Prop>Match t with False 
				  True 
				  [t1:ty][P1:Prop][t2:ty][P2:Prop]False end).
Definition is_arr :=
	[t:ty](<Prop>Match t with False 
				  False
				  [t1:ty][P1:Prop][t2:ty][P2:Prop]True end).

Goal ~(<ty>nat_ty = bool_ty).
Red; Intro H; Change (is_nat bool_ty); Elim H; Exact I.
Save nat_not_bool.

Goal (t,s:ty)~(<ty>nat_ty = (arr t s)).
Red; Intros t s H; Change (is_nat (arr t s)); Elim H; Exact I.
Save nat_not_arr.

Goal (t,s:ty)~(<ty>bool_ty= (arr t s)).
Red; Intros t s H; Change (is_bool (arr t s)); Elim H; Exact I.
Save bool_not_arr.





(********************************)
(*         			*)
(*  Xmidvar: 			*)
(*      x=v \/ ~x=v		*)
(*          			*)
(*    	Uses Xmidnat		*)
(*          			*)
(********************************)


	(*************************)
	(* Xmidnat   		 *)
	(*     m=n \/ ~m=n       *)
	(*************************)

	Goal (m,n:nat)((<nat>m=n) \/ ~<nat>m=n).

	Induction m.
	Induction n.
	Left; Reflexivity.
	Intros; Right; Apply O_S.
	Intros y H n; Elim n.
	Right; Red; Intro; Apply (O_S y); Symmetry; Assumption.
	Intros y0 I.
	Elim (H y0); Intro E. 
	Left; Elim E; Reflexivity.
	Right; Red; Intro.
	Apply E; Apply eq_add_S; Assumption.
	Save Xmidnat.


(****************)
(*   Xmidvar	*)
(****************)

Goal (v,w:vari)((<vari>v=w) \/ ~<vari>v=w).

Induction v.
Induction w.
Intro. 
Specialize (Xmidnat n n0).
Induction 1.
Intro eq; Left; Elim eq; Reflexivity.
Intro neq; Right; Red; Intro; Apply neq.
Apply vari_nat; Assumption.
Save Xmidvar.



(*****************************************)
(*					 *)
(*  If Properties                        *)
(*				 	 *)
(*  If_T:  (IF A B C) /\ A -> B          *)
(*  If_F:  (IF A B C) /\ not A -> C      *)
(*  T_If:  A /\ B -> (IF A B C)          *)
(*  F_If:  not A /\ C -> (IF A B C)      *)
(*					 *)
(*****************************************)


(*** If_T ***)
Goal (A,B,C:Prop)(IF A B C) -> A -> B.
Unfold IF; Induction 1.
Induction 1; Intros; Assumption.
Induction 1; Intros; Absurd A; Assumption.
Save If_T.

(*** If_F ***)
Goal (A,B,C:Prop)(IF A B C) -> (~A) -> C.
Unfold IF; Induction 1.
Induction 1; Intros; Absurd A; Assumption.
Induction 1; Intros; Assumption.
Save If_F.

(*** T_If ***)
Goal (A,B,C:Prop)A->B->(IF A B C).
Unfold IF; Intros.
Left; Split; Assumption.
Save T_If.

(*** F_If ***)
Goal (A,B,C:Prop)(~A)->C->(IF A B C).
Unfold IF; Intros.
Right; Split; Assumption.
Save F_If.


(********************************)
(* IfA_IfAIfA:  		*)
(*  IF A then B			*)
(*         else C  ---->	*)
(*  IF A then B 		*)
(*       else IF A then D	*)
(*  		   else C.	*)
(********************************)

Goal (A,B,C,D:Prop)
     (IF A B C) -> (IF A B (IF A D C)).

Unfold IF; Induction 1.
Intro T; Left; Assumption.
Intro F; Right; Elim F; Intros; Split.
Assumption.
Right; Split; Assumption.
Save IfA_IfAIfA.

(********************************)
(*  AABC_ABC			*)
(*    A\/(D/\C) --> (D->(A\/B))	*)
(*       --> (A \/ (B/\C))	*)
(********************************)

Goal (A,B,C,D:Prop)(A\/(D/\C)) -> (D->(A\/B)) -> (A \/ (B/\C)).

Intros A B C D A1 A2.
Elim A1.
Intro; Left; Assumption.
Induction 1; Intros DH CH.
Specialize A2 with 1:=DH; Induction 1.
Intro; Left; Assumption.
Intro; Right; Split; Assumption.
Save AABC_ABC.

