(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  ApTypes.v                                                               *)
(*                                                                          *)
(*  This file contains proofs that the Ap function and                      *)
(*  the renaming definition preserve types.  The proof                      *)
(*  that renaming preserves types is quite long and is                      *)
(*  broken down into certain subcases.  Also, the proof                     *)
(*  is first made for a more general case than necessary                    *)
(*  and then the corollary is proved.  The proof that Ap                    *)
(*  preserves types is simple once the theorem about                        *)
(*  renaming is complete.                                                   *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                ApTypes.v                                 *)
(****************************************************************************)

Require TypeThms.

Require PolyList.
Require syntax.
Require environments.
Require utils.
Require freevars.
Require typecheck.
Require rename.
Require OSrules.


(****************************************)
(* TEp_RenExp                           *)
(*   This theorem states that renaming  *)
(*   preserves types.			*)
(*                                      *)
(* The proof proceeds as follows:       *)
(*   Special Cases                      *)
(*   General theorem                    *)
(*   TEp_RenExp (Corollary)             *)
(****************************************)



(****************************************)
(*  ren_case1                           *)
(*    Tool used in first case of abs,   *)
(*    fix, and clos.                    *)
(****************************************)

Goal (nv,v:vari)(HF,H0:ty_env)(e:tm)(t,s,r:ty)
   ((nv=v)
         \/(~(FV nv e))
           /\(~(member vari v (TE_Dom HF)))
             /\~(member vari nv (TE_Dom HF))) ->
   (TC (cons VT (v,t)
       (app VT HF
       (cons VT (v,s) H0)))
       e r) ->
   (TC (cons VT (v,t)
       (app VT HF
       (cons VT (nv,s) H0)))
       e r). 
Intros.
Elim H; Intro B.
Rewrite -> B; Assumption.
Change   (TC (app VT (cons VT (v,t) HF)
                     (cons VT (nv,s) H0)) e r).
Apply TEp_nfvExt with (app VT (cons VT (v,t) HF) H0).
Change (TC (app VT (nil VT)
                   (cons VT (v,t)
                   (app VT HF H0))) e r).
Apply TEp_inv_eqExt with v s
(app VT (nil VT)
(cons VT (v,t)
(app VT HF (cons VT (v,s) H0)))).
Reflexivity.
Simpl; Assumption.
Reflexivity.
Elim B; Intros; Assumption.
Reflexivity.
Save ren_case1.


(****************************************)
(*  ren_case2                           *)
(*    Tool used in second case of abs,  *)
(*    fix, and clos.                    *)
(****************************************)

Goal (nv,v,nx,x:vari)(HF,H0:ty_env)(e1,e2,e3:tm)(t,s,r:ty)
   (~v=nx)->
   (~nx=nv) ->
   (~(FV nx e1)) ->
   (~(FV nv e2)) ->
   ((nv=v)
         \/((nv=x)\/~(FV nv e1))
           /\(~(member vari v (TE_Dom HF)))
             /\~(member vari nv (TE_Dom HF))) ->
   ((HF,H:ty_env) (s,t:ty)
        ((nx=x)
              \/(~(FV nx e1))
                /\(~(member vari x (TE_Dom HF)))
                  /\~(member vari nx (TE_Dom HF))) ->
              (TC (app VT HF (cons VT (x,s) H)) e1 t)->
              (TC (app VT HF (cons VT (nx,s) H)) e2 t)) ->
   ((HF,H:ty_env) (s,t:ty)
        ((nv=v)
              \/(~(FV nv e2))
                /\(~(member vari v (TE_Dom HF)))
                  /\~(member vari nv (TE_Dom HF)))  ->
              (TC (app VT HF (cons VT (v,s) H)) e2 t)->
              (TC (app VT HF (cons VT (nv,s) H)) e3 t)) ->
   (TC (cons VT (x,t)
       (app VT HF
       (cons VT (v,s) H0)))
       e1 r) ->
   (TC (cons VT (nx,t)
       (app VT HF
       (cons VT (nv,s) H0)))
       e3 r). 
Intros.
Change   (TC (app VT (cons VT (nx,t) HF)
                     (cons VT (nv,s) H0)) e3 r).
Apply H6.
Elim H4; Intro A.
Left; Assumption.
Right; Split.
Assumption.
Elim A; Intros F M.
Elim M; Intros Mv Mnv.
Split; Simpl; Unfold IF; Red; Induction 1; Intro N.
Apply H; Symmetry; Assumption.
Apply Mv; Assumption.
Apply H1; Assumption.
Apply Mnv; Assumption.
Change (TC (app VT (nil VT)
                   (cons VT (nx,t)
                   (app VT HF (cons VT (v,s) H0))))
        e2 r).
Apply H5.
Right; Split; Assumption Orelse Split; Simpl; Red; Intro; Assumption.
Simpl; Assumption.
Save ren_case2.



(****************************************)
(*  ren_var1                            *)
(*    Tool used in var_eq case          *)
(****************************************)

Goal (nv,v:vari)(H,HF:ty_env)(s,t:ty)
     ((nv=v) \/ 
      ((~nv=v) /\(~(member vari v (TE_Dom HF)))
                     /\~(member vari nv (TE_Dom HF))))->
     (mapsto v t  (app VT HF  (cons VT (v,s) H))) ->
     (mapsto nv t (app VT HF (cons VT (nv,s) H))).
Induction 1; Intro N.
Intro; Rewrite -> N; Assumption.
Generalize N; Elim HF; Simpl.
Intros A If.
Apply T_If.
Reflexivity.
Apply If_T with (v=v) (mapsto v t H).
Assumption.
Reflexivity.
Induction a; Simpl.
Intros a0 b y IH A If.
Elim A; Intro nvv; Induction 1; Intros a0v a0nv.
Apply F_If.
Red; Intro; Apply a0nv; Left; Assumption.
Apply IH.
Split.  Assumption.
Split.
Red; Intro; Apply a0v; Right; Assumption.
Red; Intro; Apply a0nv; Right; Assumption.
Apply If_F with a0=v b=t.
Assumption.
Red; Intro; Apply a0v; Left; Assumption.
Save ren_var1.


(****************************************)
(*  ren_var2                            *)
(*    Tool used in var_not_eq case      *)
(****************************************)

Goal  (nv,v,x:vari)(H,HF:ty_env)(s,t:ty)
      (~v=x) ->
      ((nv=v) \/ 
      ((~nv=x) /\(~(member vari v (TE_Dom HF)))
                     /\~(member vari nv (TE_Dom HF))))->
      (mapsto x t (app VT HF (cons VT (v,s) H))) ->
      (mapsto x t (app VT HF (cons VT (nv,s) H))).
Intros nv v x H HF s t nvx.
Induction 1; Intro A.
Intro; Rewrite -> A; Assumption.
Generalize A; Elim HF; Simpl.
Intros B If.
Apply F_If.
Elim B; Intros; Assumption.
Apply If_F with v=x s=t; Assumption.
Induction a; Simpl.
Intros a0 b y IH B If.
Specialize (Xmidvar a0 x); Induction 1; Intro ax.
Apply T_If.
Assumption.
Apply If_T with a0=x
      (mapsto x t (app VT y (cons VT (v,s) H)));
Assumption.
Apply F_If.  Assumption.
Apply IH.
Elim B; Intro nx; Induction 1; Intros a0v a0nv.
Split. Assumption.
Split.
Red; Intro; Apply a0v; Right; Assumption.
Red; Intro; Apply a0nv; Right; Assumption.
Apply If_F with a0=x b=t; Assumption.
Save ren_var2.


(************************************************)
(* TEp_RenExpGen                            	*)
(*   [nx/x]e=ne --->                            *)
(*   nx=x \/                                    *)
(*   (nx not in FV(e) /\  x not in dom(HF)      *) 
(*                    /\ nx not in dom(HF)) --> *)
(*   HF[x:s]H  |- e:t  --->                     *)
(*   HF[nx:s]H |- ne:t                          *)
(************************************************)

Goal (nx,x:vari)(e,ne:tm)
   (rename nx x e ne) ->
   (HF,H:ty_env)(s,t:ty)
   ((nx=x)\/
    ((~(FV nx e))/\(~(member vari x (TE_Dom HF)))
                 /\ ~(member vari nx (TE_Dom HF))))->
   (TC (app VT HF (cons VT (x,s) H)) e t) ->
   (TC (app VT HF (cons VT (nx,s) H)) ne t).

Induction 1; Intros.
(* ren_o *)
Specialize inv_TC_o with 1:=H2.
Intro Q; Rewrite -> Q; Apply TC_o.
(* ren_ttt *)
Specialize inv_TC_ttt with 1:=H2.
Intro Q; Rewrite -> Q; Apply TC_ttt.
(* ren_fff *)
Specialize inv_TC_fff with 1:=H2.
Intro Q; Rewrite -> Q; Apply TC_fff.
(* ren_abs1 *)
Specialize inv_TC_abs with 1:=H2.
Induction 1; Induction 1; Intros Q T.
Rewrite -> Q.
Apply TC_abs.
Apply ren_case1.
Specialize 5 AABC_ABC with 1:=H1 B:=~(FV nv e0).
Intro AA; Apply AA.
Intro; Apply notFV_abs with t; Assumption.
Assumption.
(* ren_abs2 *)
Specialize inv_TC_abs with 1:= H10.
Induction 1; Induction 1; Intros Q T.
Rewrite -> Q.
Apply TC_abs.
Apply ren_case2 with v x0 e1 e2.
Red; Intro; Apply H2; Symmetry; Assumption.
Assumption.
Assumption.
Rewrite -> H0; Apply RenNotFree with nx0 e1;
Assumption Orelse  Elim H0; Assumption.
Elim H9; Intro S.
Left; Assumption.
Right; Elim S; Intros F M; Split.
Apply notFV_abs with t; Assumption.
Assumption.
Assumption.
Assumption.
Assumption.
(* ren_abs3 *)
Specialize inv_TC_abs with 1:=H6.
Induction 1; Induction 1; Intros Q T.
Rewrite -> Q; Apply TC_abs.
Change (TC (app VT (cons VT (x0,t) HF)
                   (cons VT (nv,s) H4)) ne0 x1).
Apply H3.
Elim H5; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split.
Red; Intro; Apply F; Apply FV_abs; Assumption.
Elim M; Intros Mv Mnv.
Split; Simpl; Red; Induction 1; Intro A.
Apply H0; Symmetry; Assumption.
Apply Mv; Assumption.
Apply H1; Symmetry; Assumption.
Apply Mnv; Assumption.
Simpl; Assumption.
(* ren_appl *)
Specialize inv_TC_appl with 1:=H6.
Induction 1; Induction 1; Intros T1 T2.
Apply TC_appl with x0.
Apply H1.
Elim H5; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split .
Red; Intro; Apply F; Apply FV_appl1; Assumption.
Assumption.
Assumption.
Apply H3.
Elim H5; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split.
Red; Intro; Apply F; Apply FV_appl2; Assumption.
Assumption.
Assumption.
(* ren_cond *)
Specialize inv_TC_cond with 1:=H8.
Induction 1; Intro T1.
Induction 1; Intros T2 T3.
Apply TC_cond.
Apply H1.
Elim H7; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split.
Red; Intro; Apply F; Apply FV_cond1; Assumption.
Assumption.
Assumption.
Apply H3.
Elim H7; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split.
Red; Intro; Apply F; Apply FV_cond2; Assumption.
Assumption.
Assumption.
Apply H5.
Elim H7; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split.
Red; Intro; Apply F; Apply FV_cond3; Assumption.
Assumption.
Assumption.
(* ren_var_eq *)
Apply TC_var.
Specialize inv_TC_var with 1:= H2.
Intro; Apply ren_var1 with v.
Elim H1; Intro.
Left; Assumption.
Right; Elim H4; Intros; Split.
Apply notFV_var; Assumption.
Assumption.
Assumption.
(* ren_var_not_eq *)
Apply TC_var.
Specialize inv_TC_var with 1:=H3.
Intro; Apply ren_var2 with v.
Assumption.
Elim H2; Intro A.
Left; Assumption.
Right; Elim A; Intros F M; Split.
Apply notFV_var; Assumption.
Assumption.
Assumption.
(* ren_succ *)
Specialize inv_TC_succ with 1:= H4.
Induction 1; Intros Q T; Rewrite -> Q; Apply TC_succ.
Apply H1.
Elim H3; Intro A.
Left; Assumption.
Right; Elim A; Intros F M.
Split; Assumption Orelse  
Red; Intro; Apply F; Apply FV_succ; Assumption.
Assumption.
(* ren_prd *)
Specialize inv_TC_prd with 1:= H4.
Induction 1; Intros Q T; Rewrite -> Q; Apply TC_prd.
Apply H1.
Elim H3; Intro A.
Left; Assumption.
Right; Elim A; Intros F M.
Split; Assumption Orelse  
Red; Intro; Apply F; Apply FV_prd; Assumption.
Assumption.
(* ren_is_o *)
Specialize inv_TC_is_o with 1:= H4.
Induction 1; Intros Q T; Rewrite -> Q; Apply TC_is_o.
Apply H1. 
Elim H3; Intro A.
Left; Assumption.
Right; Elim A; Intros F M.
Split; Assumption Orelse  
Red; Intro; Apply F; Apply FV_is_o; Assumption.
Assumption.
(* ren_fix1 *)
Specialize inv_TC_fix with 1:=H2.
Induction 1; Intros Q T.
Elim Q.
Apply TC_fix.
Pattern 2 t; Rewrite -> Q.
Apply ren_case1.
Specialize 5 AABC_ABC with 1:=H1 B:=~(FV nv e0).
Intro AA; Apply AA.
Intro; Apply notFV_fix with t; Assumption.
Assumption.
(* ren_fix2 *)
Specialize inv_TC_fix with 1:= H10.
Induction 1; Intros Q T.
Elim Q.
Apply TC_fix.
Pattern 2 t; Rewrite -> Q.
Apply ren_case2 with v x0 e1 e2.
Red; Intro; Apply H2; Symmetry; Assumption.
Assumption.
Assumption.
Rewrite -> H0; Apply RenNotFree with nx0 e1;
Assumption Orelse  Elim H0; Assumption.
Elim H9; Intro S.
Left; Assumption.
Right; Elim S; Intros F M; Split.
Apply notFV_fix with t; Assumption.
Assumption.
Assumption.
Assumption.
Assumption.
(* ren_fix3 *)
Specialize inv_TC_fix with 1:=H6.
Induction 1; Intros Q T.
Elim Q; Apply TC_fix.
Pattern 2 t; Rewrite -> Q.
Change (TC (app VT (cons VT (x0,t) HF)
                    (cons VT (nv,s) H4)) ne0 t0).
Apply H3.
Elim H5; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split.
Red; Intro; Apply F; Apply FV_fix; Assumption.
Elim M; Intros Mv Mnv.
Split; Simpl; Red; Induction 1; Intro A.
Apply H0; Symmetry; Assumption.
Apply Mv; Assumption.
Apply H1; Symmetry; Assumption.
Apply Mnv; Assumption.
Simpl; Assumption.
(* ren_clos1 *)
Specialize inv_TC_clos with 1:= H4.
Induction 1; Intros Ta Tb.
Apply TC_clos.
Apply H1.
Elim H3; Intro A.
Left; Assumption.
Right; Elim A; Intros F M; Split.
Red; Intro; Apply F; Apply FV_closa; Assumption.
Assumption.
Assumption.
Apply ren_case1.
Specialize 5 AABC_ABC with 1:=H3 B:=~(FV nv e0).
Intro AA; Apply AA.
Intro F; Specialize notFV_clos with 1:=F.
Induction 1; Intros; Assumption.
Assumption.
(* ren_clos2 *)
Specialize inv_TC_clos with 1:= H12.
Induction 1; Intros Ta Tb.
Apply TC_clos.
Apply H9.
Elim H11; Intro A.
Left; Assumption.
Right; Elim A; Intros F M; Split.
Red; Intro; Apply F; Apply FV_closa; Assumption.
Assumption.
Assumption.
Apply ren_case2 with v x0 e0 e'.
Red; Intro; Apply H2; Symmetry; Assumption.
Assumption.
Assumption.
Rewrite -> H0; Apply RenNotFree with nx0 e0;
Assumption Orelse  Elim H0; Assumption.
Elim H11; Intro S.
Left; Assumption.
Right; Split.
Left; Assumption.
Elim S; Intros; Assumption.
Assumption.
Assumption.
Assumption.
(* ren_clos3 *)
Specialize inv_TC_clos with 1:= H8.
Induction 1; Intros Ta Tb.
Apply TC_clos.
Apply H5.
Elim H7; Intro A.
Left; Assumption.
Right; Elim A; Intros F M; Split.
Red; Intro; Apply F; Apply FV_closa; Assumption.
Assumption.
Assumption.
Change (TC (app VT (cons VT (x0,t) HF)
                   (cons VT (nv,s) H6)) ne0 t0).
Apply H3.
Elim H7; Intro N.
Left; Assumption.
Right; Elim N; Intros F M; Split.
Red; Intro; Apply F; Apply FV_closb; Assumption.
Elim M; Intros Mv Mnv.
Split; Simpl; Red; Induction 1; Intro A.
Apply H0; Symmetry; Assumption.
Apply Mv; Assumption.
Apply H1; Symmetry; Assumption.
Apply Mnv; Assumption.
Simpl; Assumption.
Save TEp_RenExpGen.

(****************************************)
(* TEp_RenExp                           *)
(*   [nx/x]e=ne --->                    *)
(*   nx=x \/ nx not in FV(e) --->       *)
(*   [x:s]H  |- e:t                     *)
(*   [nx:s]H |- ne:t --->               *)
(****************************************)

Goal (nx,x:vari)(e,ne:tm)
        (rename nx x e ne) ->
           ((nx=x)\/~(FV nx e)) ->
           (H:ty_env)(s,t:ty)
           (TC (cons VT (x,s) H) e t) ->
           (TC (cons VT (nx,s) H) ne t).
Intros.
Change (TC (app VT (nil VT) (cons VT (nx,s) H1)) ne t).
Apply TEp_RenExpGen with x e.
Assumption.
Elim H0; Intro N.
Left; Assumption.
Right; Simpl; Split.
Assumption.
Split; Red; Intro; Assumption.
Simpl; Assumption.
Save TEp_RenExp.


(****************************************)
(* TEp_Ap				*)
(*   This theorem states that the AP    *)
(*   function preserves types.		*)
(*                                      *)
(*   Ap(a,e,A) = <ne,[n:t->a]> --->	*)
(*   (x)(x in FV(e) -> x in Dom(A)) ->	*)
(*        H |- e :r->s   --->		*)
(*   [n:t]H |- ne:s  /\ r=t		*)
(****************************************)

Goal (a,e,ne:tm)(A:OS_env)(n:vari)(t:ty)
        (Ap a e A ne n t) ->
	((x:vari)(FV x e) -> (member vari x (OS_Dom A))) ->
        (H:ty_env)(r,s:ty)
        (TC H e (arr r s)) ->
        ((TC (cons VT (n,t) H) ne s) /\ r=t).

Induction 1; Intros.
Specialize inv_TC_abs with 1:=H4.
Induction 1; Induction 1; Intros Ar Te.
Specialize subty_eq with 1:=Ar; Induction 1; Intros Q1 Q2.
Split.  Rewrite ->Q2.
Apply TEp_RenExp with v e0.
Assumption.
Specialize (Xmidvar nv v); Induction 1;Intro N.
Left; Assumption.
Right; Red; Intro F; Apply H0; Apply H2; Apply FV_abs; Assumption.
Assumption.
Assumption.
Specialize inv_TC_clos with 1:=H4.
Induction 1; Intros T1 T0.
Elim H1 with (cons VT (v,s) H3) r s0.
Intros T Q.
Specialize (ApNewVar a e0 ne0 (OScons v s e1 A0) n0 t0) with 1:=H0; 
Intro M.
Split.
Apply TC_clos.
Change (TC (app VT (nil VT)
                   (cons VT (n0,t0) H3)) e1 s).
Apply TEp_nfvExt with H3.
Assumption.
Red; Intro; Apply M; Simpl.
Right; Apply H2; Apply FV_closa; Assumption.
Simpl; Reflexivity.
Change (TC (app  VT (nil VT)
           (cons VT (v,s)
           (cons VT (n0,t0) H3))) ne0 s0).
Apply TEp_swap with (app VT (nil VT)
      (cons VT (n0,t0) (cons VT (v,s) H3))).
Red; Intro; Apply M.
Simpl; Left; Assumption.
Simpl; Assumption.
Reflexivity.
Assumption.
Intros x F; Simpl.
Specialize (Xmidvar x v); Induction 1; Intro N.
Left; Symmetry; Assumption.
Right; Apply H2; Apply FV_closb; Assumption.
Assumption.
Save TEp_Ap.
