(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  NF.v                                                                    *)
(*                                                                          *)
(*  This file contains a definition of the normal forms                     *)
(*  of evaluation and some properties which are used in                     *)
(*  later proofs.                                                           *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                   NF.v                                   *)
(****************************************************************************)

Require syntax.

(********************************************************)
(*							*)
(*	If <<e,A>> -> <<e',A'>>                  	*)
(*	then e' belongs to the following set:		*)
(*							*)
(*	NF = ttt | fff | S | F 				*)
(*           where					*)
(*		S = o | (succ S) 			*)
(*		F = (abs x t e) | (clos F x t e)	*)
(*							*)
(*	which corresponds to:				*)
(*	NF = 0 | true | false | succn(0) | F		*)
(*	 F = \x:t.e | <F,[x:t->e]>			*)
(*							*)
(********************************************************)

Inductive F : tm -> Prop
	 :=  F_abs : (v:vari)(s:ty)(e:tm)(F (abs v s e))
	 |  F_clos: (e,e1:tm)(v:vari)(s:ty)
		    (F e) -> (F (clos e v s e1)).
Inductive Sno : tm -> Prop
	 :=  Sno_o :  (Sno o)
	 |  Sno_s :  (e:tm)(Sno e) -> (Sno (succ e)).

Inductive NF : tm -> Prop
	 :=  NF_ttt :  (NF ttt)
	 |  NF_fff :  (NF fff)
	 |  NF_Sno :  (e:tm)(Sno e) -> (NF e)
	 |  NF_F   :  (e:tm)(F e) -> (NF e).

(********************************************************)
(*							*)
(*   The inverse of NF_Sno and Sno_s are proved here    *)
(*	and used in a later proof.  They require        *)
(*	predicates about (succ e) for inductive proofs. *)
(*							*)
(********************************************************)

Definition NFsucc:= [e:tm]
	<Prop>Case e of
        (* o *)            True
        (* ttt *)          True
        (* fff *)          True
        (* abs v s e *)    [v:vari][s:ty][e:tm]True
        (* appl e1 e2 *)   [e1,e2:tm] True
        (* cond e1 e2 e3 *)[e1,e2,e3:tm] True
        (* var v *)        [v:vari] True
        (* succ n *)       [n:tm](Sno (succ n))
        (* prd n *)        [n:tm]True
        (* is_o n *)       [n:tm]True
        (* fix v s e *)    [v:vari][s:ty][e:tm] True
        (* clos e v s e1 *)[e:tm][v:vari][s:ty][e1:tm] True end.

Goal (e:tm)(NF (succ e)) -> (Sno (succ e)).
Intros e HNF.  Change (NFsucc (succ e)).
Elim HNF; Simpl; Intros;
   Exact I Orelse 
   Elim H; Simpl; Intros; Exact I Orelse 
			  Apply Sno_s; Assumption.
Save inv_NF_Sno.


Definition SnoSucc := [e:tm]
	<Prop>Case e of
        (* o *)                 True
        (* ttt *)               True
        (* fff *)               True
        (* abs v s e *)         [v:vari][s:ty][e:tm] True
        (* appl e1 e2 *)        [e1,e2:tm] True
        (* cond e1 e2 e3 *)     [e1,e2,e3:tm] True
        (* var v *)             [v:vari] True
        (* succ n *)            [n:tm](Sno n)
        (* prd n *)             [n:tm]True
        (* is_o n *)            [n:tm]True
        (* fix v s e *)         [v:vari][s:ty][e:tm] True
        (* clos e v s e1 *)     [e:tm][v:vari][s:ty][e1:tm] True end.

Goal (e:tm)(Sno (succ e)) -> (Sno  e).
Intros e HSno. Change (SnoSucc (succ e)).
Elim HSno; Simpl; Intros; Exact I Orelse Assumption.
Save inv_Sno_s.
