(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(* NF.v                                                                     *)
(*                                                                          *)
(* This file contains proofs of properties of the NF set                    *)
(* and its subsets F and Sno.                                               *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                NFprops.v                                 *)
(****************************************************************************)

Require NF.
Require PolyList.
Require syntax.
Require typecheck.
Require environments.
Require freevars.
Require utils.

(********************************)
(* Fe_not_nat			*)
(*  e in F ---> ~(H |- e:nat)	*)
(********************************)

Goal (e:tm)(F e) -> (H:ty_env)~(TC H e nat_ty).
   Induction 1; Intros.
   Red; Intro.
   Specialize inv_TC_abs with 1:=H1.
   Induction 1; Induction 1; Intros Q T.
   Generalize Q; Exact (nat_not_arr s x).
   Red; Intro.
   Specialize inv_TC_clos with 1:=H3.
   Induction 1; Intros T1 T0.
   Red in H1; Apply H1 with (cons VT (v,s) H2); Assumption.
Save Fe_not_nat.

(********************************)
(* Fe_not_bool			*)
(*  e in F ---> ~(H |- e:bool)	*)
(********************************)

Goal (e:tm)(F e) -> (H:ty_env)~(TC H e bool_ty).
   Induction 1; Intros.
   Red; Intro.
   Specialize inv_TC_abs with 1:=H1.
   Induction 1; Induction 1; Intros Q T.
   Generalize Q; Exact (bool_not_arr s x).
   Red; Intro.
   Specialize inv_TC_clos with 1:=H3.
   Induction 1; Intros T1 T0.
   Red in H1; Apply H1 with (cons VT (v,s) H2); Assumption.
Save Fe_not_bool.


(********************************************************)
(* NFe_Fe						*)
(*							*)
(* e in NF /\ not H |- e:nat /\ not H |- e:bool ---->	*)
(*		e in F					*)
(********************************************************)

Goal (e:tm)(NF e) ->
	   (H:ty_env)(t:ty)(TC H e t) ->
	   (~((t=nat_ty)\/(t=bool_ty))) ->
	   (F e).
 Induction 1; Simpl; Intros.
 Elim H2; Right.
 Apply (inv_TC_ttt H0); Assumption.
 Elim H2;  Right.
 Apply (inv_TC_fff H0); Assumption.
 Elim H3; Left.
 Generalize H2.
 Elim H0.
 Exact (inv_TC_o H1 t).
 Intros.
 Specialize inv_TC_succ with 1:=H6.
 Induction 1; Intros; Assumption.
 Assumption.
Save NFe_Fe.

(************************************************)
(* NFebool_TF 					*)
(*  e in NF /\ H |- e:bool--->  e=ttt \/ e=fff  *)
(************************************************)

Goal (e:tm)(NF e)->
        (H:ty_env)(TC H e bool_ty)->
        ((e=ttt)\/e=fff).
Induction 1; Intros.
Left; Reflexivity.
Right; Reflexivity.
Generalize H2; Elim H0; Intros.
Specialize inv_TC_o with 1:=H3; Intro Q.
Absurd nat_ty=bool_ty.
Exact nat_not_bool.
Symmetry; Assumption.
Specialize inv_TC_succ with 1:=H5; Induction 1;  Intros Q T.
Absurd nat_ty=bool_ty.
Exact nat_not_bool.
Symmetry; Assumption.
Absurd (TC H1 e0 bool_ty).
Apply Fe_not_bool; Assumption.
Assumption.
Save NFebool_TF.


(************************************************)
(* NFenat_Snoe					*)
(*  e in NF /\ H |- e:nat --->  e in Sno	*)
(************************************************)

Goal (e:tm)(NF e)->
	(H:ty_env)(TC H e nat_ty)->
	(Sno e).

   Induction 1; Intros.
   Specialize inv_TC_ttt with 1:=H1; Intro Q.
   Absurd nat_ty=bool_ty;
   Assumption Orelse Exact nat_not_bool.
   Specialize inv_TC_fff  with 1:=H1; Intro Q.
   Absurd nat_ty=bool_ty;
   Assumption Orelse Exact nat_not_bool.
   Assumption.
   Absurd (TC H1 e0 nat_ty).
   Apply Fe_not_nat.
   Assumption.
   Assumption.
Save NFenat_Snoe.


(********************************)
(* Snoe_notFVe			*)
(*  e in Sno --->  e closed 	*)
(********************************)

Goal (e:tm)(Sno e) -> (v:vari)~(FV v e).
  Induction 1; Intros.
  Apply inv_FV_o.
  Red; Intro.
  Red in H1; Apply H1 with v.
  Apply inv_FV_succ; Assumption.
Save Snoe_notFVe.

