(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  TypeThms.v                                                              *)
(*                                                                          *)
(*  This file contains theorems which state some                            *)
(*  conditions under which types are preserved.                             *)
(*  These theorems directly depend on the mapsto                            *)
(*  preservation theorems (mapsto.v).                                       *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                TypeThms.v                                *)
(****************************************************************************)

Require PolyList.
Require syntax.
Require environments.
Require freevars.
Require utils.

Require typecheck.

Require mapsto.


(********************************)
(* TEp_eqExt			*)
(* (HeqExt)  	                *)
(*   v=x --->			*)
(*   H1 |- e:t --->		*)
(*   H1 = HF[v:r]HM+H  --->    	*)
(*   HF[v:r]HM[x:s]H |- e:t 	*)
(********************************)

Goal (v,x:vari)(r,s,t:ty)(H,H1:ty_env)(e:tm)
     (v=x)->
     (TC H1 e t) ->
     (HF,HM:ty_env)
     (H1=(app VT HF (cons VT (v,r) (app VT HM H)))) ->
     (TC (app VT HF (cons VT (v,r)
         (app VT HM (cons VT (x,s) H)))) e t). 

   Induction 2; Simpl; Intros.
   Apply TC_o.
   Apply TC_ttt.
   Apply TC_fff.
   Apply TC_succ.
   Apply H5; Assumption.
   Apply TC_prd.
   Apply H5; Assumption.
   Apply TC_is_o.
   Apply H5; Assumption.
   Apply TC_var.
   Elim H0; Apply Mp_eqExt.
   Elim H5; Assumption.
   Apply TC_appl with s0.
   Apply H5; Assumption.
   Apply H7; Assumption.
   Apply TC_abs.
   Change   (TC (app VT (cons VT (v0,s0) HF)
                        (cons VT (v,r)
                         (app VT HM (cons VT (x,s) H)))) e0 t0).
   Apply H5; Rewrite -> H6; Simpl; Reflexivity.
   Apply TC_cond.
   Apply H5; Assumption.
   Apply H7; Assumption.
   Apply H9; Assumption.
   Apply TC_fix.
   Change   (TC (app VT (cons VT (v0,t0) HF)
                        (cons VT (v,r)
                         (app VT HM (cons VT (x,s) H)))) e0 t0).
   Apply H5; Rewrite -> H6; Simpl; Reflexivity.
   Apply TC_clos.
   Apply H5; Assumption.
   Change   (TC (app VT (cons VT (v0,s0) HF)
                        (cons VT (v,r)
                         (app VT HM (cons VT (x,s) H)))) e0 t0).
   Apply H7; Rewrite -> H8; Simpl; Reflexivity.
Save TEp_eqExt.



(********************************)
(* TEp_inv_eqExt		*)
(* (HeqinvExt) 			*)
(*   v=x --->			*)
(*   H1 |- e:t --->		*)
(*   H1 = HF[v:r]HM[x:s]H --->	*)
(*   HF[v:r]HM*H |- e:t		*)
(********************************)

Goal (v,x:vari)(r,s,t:ty)(H,H1:ty_env)(e:tm)
     (v=x)->
     (TC H1 e t) ->
     (HF,HM:ty_env)
     (H1=(app VT HF (cons VT (v,r) 
                 (app VT HM (cons VT (x,s) H))))) ->
     (TC (app VT HF (cons VT (v,r) (app VT HM H))) e t).

   Induction 2; Intros.
   Apply TC_o.
   Apply TC_ttt.
   Apply TC_fff.
   Apply TC_succ; Apply H5; Assumption.
   Apply TC_prd; Apply H5; Assumption.
   Apply TC_is_o; Apply H5; Assumption.
   Apply TC_var.
   Apply Mp_inv_eqExt with x s; Assumption Orelse Elim H5; Assumption.
   Apply TC_appl with s0.
   Apply H5; Assumption.
   Apply H7; Assumption.
   Apply TC_abs.
   Change (TC (app VT (cons VT (v0,s0) HF)
                      (cons VT (v,r) (app VT HM H))) e0 t0).
   Apply H5.
   Simpl; Elim H6; Reflexivity.
   Apply TC_cond.
   Apply H5; Assumption.
   Apply H7; Assumption.
   Apply H9; Assumption.
   Apply TC_fix.
   Change (TC (app VT (cons VT (v0,t0) HF)
                      (cons VT (v,r) (app VT HM H))) e0 t0).
   Apply H5.
   Simpl; Elim H6; Reflexivity.
   Apply TC_clos.
   Apply H5; Assumption.
   Change (TC (app VT (cons  VT (v0,s0) HF)
                      (cons VT (v,r) (app VT HM H))) e0 t0).
   Apply H7.
   Simpl; Elim H8; Reflexivity.
Save TEp_inv_eqExt.
   
   

(********************************)
(*   TEp_nfvExt			*)
(*   (TyExt)			*)
(*  	H1*H2|-e:t  --->	*)
(*      x not in FV(e) --->	*)
(*	H1[x:s]H2|-e:t		*)
(********************************)

Goal  (H:ty_env)(e:tm)(t:ty)(x:vari)(s:ty)
      (TC H e t) -> 
      (~(FV x e)) -> 
      (H1,H2:ty_env)
      (H=(app VT H1 H2)) ->
      (TC (app VT H1 (cons VT (x,s) H2)) e t).

Induction 1; Simpl; Intros.
Apply TC_o.
Apply TC_ttt.
Apply TC_fff.
Apply TC_succ.
Apply H3; Assumption Orelse Apply notFV_succ; Assumption.
Apply TC_prd.
Apply H3; Assumption Orelse Apply notFV_prd; Assumption.
Apply TC_is_o.
Apply H3; Assumption Orelse Apply notFV_is_o; Assumption.
Apply TC_var.
Simpl; Specialize notFV_var with 1:= H3; Intro NQ.
Apply Mp_nfvExt.
Red; Intro; Apply NQ; Symmetry; Assumption.
Elim H6; Assumption.
Apply TC_appl with s0.
Apply H3; Assumption Orelse
Specialize notFV_appl with 1:=H6; Induction 1; Intros; Assumption.
Apply H5; Assumption Orelse
Specialize notFV_appl with 1:=H6; Induction 1; Intros; Assumption.
Apply TC_abs.
Specialize (Xmidvar x v); Induction 1.
Intro xv; Rewrite -> xv.
Change (TC (app VT (nil VT) 
                   (cons VT (v,s0) 
                   (app VT H5
                           (cons VT (v,s) H6)))) e0 t0).
Apply TEp_eqExt with (app VT (nil VT) 
                             (cons VT (v,s0) 
                             (app VT H5 H6))). 
Reflexivity.
Simpl; Elim H7; Assumption.
Reflexivity.
Intro nxv. 
Change   (TC (app VT (cons VT (v,s0) H5)
                     (cons VT (x,s) H6)) e0 t0).
Apply H3.
Specialize notFV_abs with 1:=H4.
Induction 1; Intro P.
Absurd x=v; Assumption.
Assumption.
Rewrite -> H7; Simpl; Reflexivity.
Apply TC_cond.
Apply H3; Assumption Orelse
Red; Intro; Apply H8; Apply FV_cond1; Assumption.
Apply H5; Assumption Orelse
Red; Intro; Apply H8; Apply FV_cond2; Assumption.
Apply H7; Assumption Orelse
Red; Intro; Apply H8; Apply FV_cond3; Assumption.
Apply TC_fix.
Specialize (Xmidvar x v); Induction 1.
Intro xv; Rewrite -> xv.
Change (TC (app VT (nil VT) 
                   (cons VT (v,t0) 
                   (app VT H5
                           (cons VT (v,s) H6)))) e0 t0).
Apply TEp_eqExt with (app VT (nil VT) 
                             (cons VT (v,t0) 
                             (app VT H5 H6))). 
Reflexivity.
Simpl; Elim H7; Assumption.
Reflexivity.
Intro nxv. 
Change (TC (app VT (cons VT (v,t0) H5)
                   (cons VT (x,s) H6)) e0 t0).
Apply H3.
Specialize notFV_fix with 1:=H4.
Induction 1; Intro P.
Absurd x=v; Assumption.
Assumption.
Rewrite -> H7; Simpl; Reflexivity.

Apply TC_clos.
Apply H3; Assumption Orelse
Red; Intro; Apply H6; Apply FV_closa; Assumption.
Specialize (Xmidvar x v); Induction 1.
Intro xv; Rewrite -> xv.
Change (TC (app VT (nil VT) 
                   (cons VT (v,s0) 
                   (app VT H7 
                           (cons VT (v,s) H8)))) e0 t0).
Apply TEp_eqExt with (app VT (nil VT) 
                             (cons VT (v,s0) 
                             (app VT H7 H8))). 
Reflexivity.
Simpl; Elim H9; Assumption.
Reflexivity.
Intro nxv. 
Change (TC (app VT (cons VT (v,s0) H7) 
                   (cons VT (x,s) H8)) e0 t0).
Apply H5.
Specialize notFV_clos with 1:=H6.
Induction 1; Intro; Induction 1; Intro P.
Absurd x=v; Assumption.
Assumption.
Rewrite -> H9; Simpl; Reflexivity.
Save TEp_nfvExt.



   (*****************************)
   (* TEp_inv_nfvExt		*)
   (* (Gen_inv_TyExt) 		*)
   (*   H1 |- e:t --->		*)
   (*   H1 = HF[x:s]H --->	*)
   (*   x not in FV(e) --->	*)
   (*   HF*H |- e:t		*)
   (*****************************)

   Goal (H1,H:ty_env)(e:tm)(t:ty)(x:vari)(s:ty)
        (TC H1 e t) ->
        (HF:ty_env)
        (H1=(app VT HF (cons VT (x,s) H))) ->
        (~(FV x e)) ->
        (TC (app VT HF H) e t).

   Induction 1; Intros.
   Apply TC_o.
   Apply TC_ttt.
   Apply TC_fff.
   Apply TC_succ; Apply H4;
   Assumption Orelse Red; Intro; Apply H6; Apply FV_succ; Assumption.
   Apply TC_prd; Apply H4;
   Assumption Orelse Red; Intro; Apply H6; Apply FV_prd; Assumption.
   Apply TC_is_o; Apply H4;
   Assumption Orelse Red; Intro; Apply H6; Apply FV_is_o; Assumption.
   Apply TC_var.
   Apply Mp_inv_nfvExt with x s.
   Red; Intro; Apply H5; Apply FV_var; Symmetry; Assumption.
   Elim H4; Assumption.
   Apply TC_appl with s0.
   Apply H4; Assumption Orelse
   Red; Intro; Apply H8; Apply FV_appl1; Assumption.
   Apply H6; Assumption Orelse
   Red; Intro; Apply H8; Apply FV_appl2; Assumption.
   Apply TC_abs.
   Change (TC (app VT (cons VT (v,s0) HF) H) e0 t0).
   Specialize (Xmidvar v x); Induction 1; Intros.
   Simpl; Change (TC (app VT (nil VT) 
                      (cons VT (v,s0) (app VT HF H))) e0 t0).
   Apply TEp_inv_eqExt with x s
   (app VT (nil VT) (cons VT (v,s0)
            (app VT HF (cons VT (x,s) H)))).
   Assumption.
   Elim H5; Assumption.
   Reflexivity.
   Apply H4.  
   Simpl; Elim H5; Reflexivity.
   Red; Intro; Apply H6; Apply FV_abs; Assumption Orelse
   Red; Intro; Apply H8; Symmetry; Assumption.
   Apply TC_cond.
   Apply H4; Assumption Orelse
   Red; Intro; Apply H10; Apply FV_cond1; Assumption.
   Apply H6; Assumption Orelse
   Red; Intro; Apply H10; Apply FV_cond2; Assumption.
   Apply H8; Assumption Orelse
   Red; Intro; Apply H10; Apply FV_cond3; Assumption.
   Apply TC_fix.
   Specialize (Xmidvar v x); Induction 1; Intros.
   Change (TC (app VT (nil VT) 
                 (cons VT (v,t0) (app VT HF H))) e0 t0).
   Apply TEp_inv_eqExt with x s
   (app VT (nil VT) (cons VT (v,t0)
            (app VT HF (cons VT (x,s) H)))).
   Assumption.
   Elim H5; Assumption.
   Reflexivity.
   Change (TC (app VT (cons VT (v,t0) HF) H) e0 t0).
   Apply H4.  
   Simpl; Elim H5; Reflexivity.
   Red; Intro; Apply H6; Apply FV_fix; Assumption Orelse
   Red; Intro; Apply H8; Symmetry; Assumption.
   Apply TC_clos.
   Apply H4;  Assumption Orelse
   Red; Intro; Apply H8; Apply FV_closa; Assumption. 
   Specialize (Xmidvar x v); Induction 1; Intros.
   Change (TC (app VT (nil VT)
                  (cons VT (v,s0) (app VT HF H))) e0 t0).
   Apply TEp_inv_eqExt with x s
   (app VT (nil VT) (cons VT (v,s0)
            (app VT HF (cons VT (x,s) H)))).
   Symmetry; Assumption.
   Elim H7; Assumption.
   Reflexivity.
   Change (TC (app VT (cons VT (v,s0) HF) H) e0 t0).
   Apply H6.
   Simpl; Elim H7; Reflexivity.
   Red; Intro; Apply H8; Apply FV_closb; Assumption.
Save TEp_inv_nfvExt.
   

(********************************)
(* TEp_swap			*)
(* (GenHswap)			*)
(*    ~v=x --->       		*)
(*    H1|-e:r --->	    	*)
(*    H1=HF[x:s,v:t]HB --->	*)
(*    HF[v:t,x:s]HB|-e:r 	*)
(********************************)

Goal (v:vari) (x:vari)
     (~v=x)->
     (r:ty) (s:ty) (t:ty) (e:tm) (H1:ty_env) (HB:ty_env)
     (TC H1 e r)->
     (HF:ty_env)
     ( H1=
          (app VT HF (cons VT (x,s) (cons VT (v,t) HB)))) ->
     (TC  (app VT HF (cons VT (v,t) (cons VT (x,s) HB))) e r).

   Induction 2; Simpl; Intros.
   Apply TC_o.
   Apply TC_ttt.
   Apply TC_fff.
   Apply TC_succ; Apply (H4 HF); Assumption.
   Apply TC_prd; Apply (H4 HF); Assumption.
   Apply TC_is_o; Apply (H4 HF); Assumption.
   Apply TC_var; Apply Mp_swap.
   Red; Intro; Apply H; Symmetry; Assumption.
   Elim H4; Assumption.
   Apply TC_appl with s0.
   Apply (H4 HF); Assumption.
   Apply H6; Assumption.
   Apply TC_abs.
   Change (TC (app VT (cons VT (v0,s0) HF)
                      (cons VT (v,t) 
 		      (cons VT (x,s) HB))) e0 t0).
   Apply (H4 (cons VT (v0,s0) HF)); Simpl.
   Elim H5; Reflexivity.
   Apply TC_cond.
   Apply (H4 HF); Assumption.
   Apply (H6 HF); Assumption.
   Apply (H8 HF); Assumption.
   Apply TC_fix.
   Change (TC (app VT (cons VT (v0,t0) HF)
                      (cons VT (v,t) 
		      (cons VT (x,s) HB))) e0 t0).
   Apply (H4  (cons VT (v0,t0) HF)); Simpl.
   Elim H5; Reflexivity.
   Apply TC_clos.
   Apply (H4 HF); Assumption.
   Change (TC (app VT (cons VT (v0,s0) HF)
                      (cons VT (v,t) 
		      (cons VT (x,s) HB))) e0 t0).
   Apply (H6 (cons VT (v0,s0) HF)); Simpl.
   Elim H7; Reflexivity.
Save TEp_swap.
