(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  envprops.v                                                              *)
(*                                                                          *)
(*  This file contains properties of domains of environments                *)
(*  and their relation to type judgments and evaluations.                   *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                envprops.v                                *)
(****************************************************************************)

Require syntax.
Require PolyList.
Require utils.
Require freevars.
Require typecheck.

Require environments.

Require OSrules.


(********************************)
(*  TEDomDomty_OSDom		*)
(*    Dom(Domt (A)) = Dom(A)	*)
(********************************)

Goal (A:OS_env)(TE_Dom (OS_Dom_ty A))=(OS_Dom A).
   Induction A; Intros.
   Simpl; Reflexivity.
   Simpl; Elim H; Reflexivity.
Save TEDomDomty_OSDom.



(**********************************)
(* vtinH_vinDomH		  *)
(*  (v,t) in H  -->  v in Dom(H)  *)
(**********************************)

Goal (v:vari) (t:ty) (H:ty_env)
        (mapsto v t H)->
                      (member vari v (TE_Dom H)).

Induction H; Simpl.
Intro; Assumption.
Intros; Elim H1; Intro.
Left; Elim H2; Intros; Assumption.
Right; Elim H2; Intro; Exact H0.
Save vtinH_vinDomH.


(****************************************)
(* TCHet_FVeinDomH 			*)
(*					*)
(*   H |- e : t  -->  FV(e) in Dom(H)   *)
(*					*)
(****************************************)

Goal (H:ty_env) (e:tm) (t:ty)
       (TC H e t)->
         (v:vari)(FV  v e)->(member vari v (TE_Dom H)).

Induction 1; Simpl; Intros.
Absurd (FV v o).    Apply inv_FV_o.    Assumption.
Absurd (FV v ttt).  Apply inv_FV_ttt.  Assumption.
Absurd (FV v fff).  Apply inv_FV_fff.  Assumption.
Apply H3.  Apply inv_FV_succ; Assumption.
Apply H3.  Apply inv_FV_prd; Assumption.
Apply H3.  Apply inv_FV_is_o; Assumption.
Specialize inv_FV_var with 1:=H3; Intro Q.
Rewrite -> Q; Apply vtinH_vinDomH with t0; Assumption.
Specialize inv_FV_appl with 1:=H6.
Induction 1; Intro F.
Apply H3; Assumption.
Apply H5; Assumption.
Specialize inv_FV_abs with 1:=H4; Induction 1; Intros.
Elim (H3 v0).
Intro; Absurd v0=v; Assumption Orelse Symmetry; Assumption.
Intro; Assumption.
Assumption.
Specialize inv_FV_cond with 1:=H8; Induction 1.
Intro; Apply H3; Assumption.
Induction 1; Intro.
Apply H5; Assumption.
Apply H7; Assumption.
Specialize inv_FV_fix with 1:=H4; Induction 1; Intros.
Elim (H3 v0).
Intro; Absurd v0=v; Assumption Orelse Symmetry; Assumption.
Intro; Assumption.
Assumption.
Specialize inv_FV_clos with 1:=H6; Induction 1; Intros.
Apply H3; Assumption.
Elim H8; Intros.
Elim (H5 v0).
Intro; Absurd v0=v; Assumption Orelse Symmetry; Assumption.
Intro; Assumption.
Assumption.
Save TCHet_FVeinDomH.



(********************************)
(*  dom_pres			*)
(*    <<e,A>> -> <<n,A'>>  ---> *)
(*	 Domt(A) = Domt(A')	*)
(********************************)

Goal  (c,c':config)(OSred c c')->
      (OS_Dom_ty (cfgenv c)) = (OS_Dom_ty (cfgenv c')).

   Induction 1; Simpl; Intros.
   Reflexivity.  Reflexivity.  Reflexivity.  Reflexivity.
   Assumption.  Assumption.  Assumption.  Assumption.  Assumption.
   Elim H2;  Reflexivity.
   Elim H3; Reflexivity.
   Transitivity (OS_Dom_ty A');  Assumption.
   Transitivity (OS_Dom_ty A');  Assumption.
   Transitivity (OS_Dom_ty A');  Assumption.
   Assumption.
   Replace (OS_Dom_ty A) with 
	   (tail VT (cons VT (x,t) (OS_Dom_ty A))).
   Replace (OS_Dom_ty A') with 
	   (tail VT (cons VT (x,t) (OS_Dom_ty A'))).
   Apply (f_equal (list VT) (list VT) (tail VT)); Assumption.
   Simpl; Reflexivity.
   Simpl; Reflexivity.
   Replace (OS_Dom_ty A) with 
           (tail VT (cons VT (x,t) (OS_Dom_ty A))).
   Replace (OS_Dom_ty A') with 
	   (tail VT (cons VT (x,t) (OS_Dom_ty A'))).
   Apply (f_equal (list VT) (list VT) (tail VT)); Assumption.
   Simpl; Reflexivity.
   Simpl; Reflexivity.
Save dom_pres.


