(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  mapsto.v                                                                *)
(*                                                                          *)
(*  This file contains theorems which describe how                          *)
(*  a type environment may be altered without changing                      *)
(*  the mapsto function.                                                    *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                 mapsto.v                                 *)
(****************************************************************************)

Require PolyList.
Require syntax.
Require environments.

Require utils.


(************************)
(* Mp_nfvExt 		*)
(*   ~v=x --->		*)
(*   HF*H(v) = t  --->	*)
(*   HF[x:s]H(v) = t 	*)
(************************)

Goal (v,x:vari)(t,s:ty)(HF,H:ty_env)
     (~v=x) ->
     (mapsto v t (app VT HF H)) ->
     (mapsto v t (app VT HF (cons VT (x,s) H))). 

   Induction HF; Simpl.
   Intros H neq M.
   Apply F_If.
   Red; Intro; Apply neq; Symmetry; Assumption.
   Assumption.
   Induction a; Simpl; Intros a0 b y IH H0 neq If.
   Specialize (Xmidvar a0 v); Induction 1. Intro T.
   Specialize If_T with 1:=If 2:=T; Intro B.
   Apply T_If; Assumption.
   Intro F.
   Specialize If_F with 1:=If 2:=F; Intro M.
   Apply F_If. Assumption.
   Apply IH; Assumption.
Save Mp_nfvExt.


(********************************)
(* Mp_inv_nfvExt 		*)
(*   ~v=x --->			*)
(*   HF[x:s]H(v) = t --->	*)
(*   HF*H(v) = t		*)
(********************************)

Goal (v,x:vari)(t,s:ty)(HF,H:ty_env)
     (~v=x) ->
     (mapsto v t (app VT HF (cons VT (x,s) H))) ->
     (mapsto v t (app VT HF H)).

   Induction HF; Simpl.
   Intros H neq If.
   Apply If_F with x=v s=t.
   Assumption.
   Red; Intro; Apply neq; Symmetry; Assumption.
   Induction a; Simpl; Intros a0 b y IH H0 neq If.
   Specialize (Xmidvar a0 v); Induction 1. Intro T.
   Specialize If_T with 1:=If 2:=T; Intro B.
   Apply T_If; Assumption.
   Intro F.
   Specialize If_F with 1:=If 2:=F; Intro M.
   Apply F_If. Assumption.
   Apply IH; Assumption.
Save Mp_inv_nfvExt.



(********************************)
(* Mp_eqExt			*)
(*    H[v:s]HM+H'(x)=t --->  	*)
(*    H[v:s]HM[v:r]H'(x)=t   	*)
(********************************)

Goal (x,v:vari)(r,s,t:ty)(H,HM,H':ty_env)
     (mapsto x t  (app VT H (cons VT (v,s) (app VT HM H')))) ->
     (mapsto x t  (app VT H  (cons VT (v,s)
                  (app VT HM (cons VT (v,r) H'))))).

   Induction H; Simpl.
   Induction HM; Simpl.  Intros.
   Apply IfA_IfAIfA.  Assumption.
   Induction a; Simpl; Intros.
   Specialize (Xmidvar v x); Induction 1.
   Intro T.
   Specialize If_T with 1:= H1 2:=T; Intro.
   Apply T_If; Assumption.
   Intro F.
   Specialize If_F with 1:=H1 2:=F; Intro I.
   Apply F_If.
   Assumption. 
   Specialize (Xmidvar y x); Induction 1.
   Intro Q. 
   Specialize If_T with 1:=I 2:=Q; Intro.
   Apply T_If; Assumption.
   Intro nQ.
   Specialize If_F with 1:=I 2:=nQ; Intro.
   Apply F_If.  Assumption.
   Apply If_F with v=x s=t.
   Apply H0.
   Specialize If_F with 1:=H1 2:=F; Intro I2.
   Specialize  If_F with 1:=I2 2:=nQ; Intro.
   Apply F_If; Assumption.
   Assumption. 
   Induction a; Simpl; Intros.
   Specialize (Xmidvar y x); Induction 1.
   Intro T.
   Specialize If_T with 1:=H1 2:=T; Intro.
   Apply T_If; Assumption.
   Intro F.
   Specialize If_F with 1:= H1 2:=F; Intro.
   Apply F_If.  Assumption.
   Apply H0; Assumption.
Save Mp_eqExt.


(********************************)
(* Mp_inv_eqExt 		*)
(*   v=x --->			*)
(*   HF[v:s]HM[x:t]H(y) = r -->	*)
(*   HF[v:s]HM*H(y) = r		*)
(********************************)

Goal (v,x,y:vari)(r,s,t:ty)(HF,HM,H:ty_env)
     (v=x) ->
     (mapsto y r (app VT HF (cons VT (v,s)
                 (app VT HM (cons VT (x,t) H))))) ->
     (mapsto y r (app VT HF (cons VT (v,s)
                 (app VT HM H)))).

   Induction HF; Simpl. Intros HM H Q A.
   Specialize (Xmidvar v y); Induction 1. Intro T.
   Specialize If_T with 1:=A 2:=T; Intro sr.
   Apply T_If; Assumption.
   Intro F.
   Specialize If_F with 1:=A 2:=F; Intro M.
   Apply F_If. Assumption.
   Apply Mp_inv_nfvExt with x t. 
   Elim Q; Red; Intro; Apply F; Symmetry; Assumption.
   Assumption.
   Induction a; Simpl; Intros a0 b y0 IH HM H0 Q If.
   Specialize (Xmidvar a0 y); Induction 1. Intro T.
   Specialize If_T with 1:=If 2:=T; Intro br.
   Apply T_If; Assumption.
   Intro F.
   Specialize If_F with 1:=If 2:=F; Intro M.
   Apply F_If. Assumption.
   Apply IH; Assumption.
Save Mp_inv_eqExt.




(***************************)
(* Mp_swap		   *)
(*    ~x=y --->       	   *)
(*    H[x:s,y:t]H'(v)=r -->*)
(*    H[y:t,x:s]H'(v)=r    *)
(***************************)

Goal (v,x,y:vari)(r,s,t:ty)(H,H':ty_env)
     (~(x=y)) ->
     (mapsto v r (app VT H (cons VT (x,s)
                           (cons VT (y,t) H')))) ->
     (mapsto v r (app VT H (cons VT (y,t)
                           (cons VT (x,s) H')))).

	Induction H.
	Simpl; Intros H' neq If.
        Specialize (Xmidvar y v); Induction 1.
        Intro T.
        Apply T_If. Assumption.
        Apply If_T with y=v (mapsto v r H').
        Apply If_F with x=v s=r.
        Assumption.
        Elim T; Assumption.
        Assumption.
        Intro F.
        Apply F_If. Assumption.
        Specialize (Xmidvar x v); Induction 1.
        Intro TT.
	Specialize If_T with 1:= If 2:=TT; Intro sr.
        Apply T_If; Assumption.
        Intro FF.
        Specialize If_F with 1:=If 2:=FF; Intro If2.
        Specialize If_F with 1:=If2 2:=F; Intro M.
        Apply F_If; Assumption.
	Induction a.
	Simpl; Intros a0 b y0 IH H' N A.
        Specialize (Xmidvar a0 v); Induction 1.
        Intro T.
        Specialize If_T with 1:=A 2:=T; Intro br.
        Apply T_If; Assumption.
        Intro F.
        Specialize If_F with 1:=A 2:=F; Intro M.
        Apply F_If; 
        Assumption Orelse Apply IH; Assumption.
Save Mp_swap.

