(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  subjrfn.v                                                               *)
(*                                                                          *)
(*  This file contains the main theorem, subjr_NF,                          *)
(*  which combines the subject reduction and normal                         *)
(*  form characterization theorems (the combination is                      *)
(*  necessary in order for the induction to go through).                    *)
(*  This proof is followed by individual proofs of the                      *)
(*  subject reduction theorem and normal form character-                    *)
(*  ization theorem as corollaries of the main proof.                       *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                subjrnf.v                                 *)
(****************************************************************************)

Require utils.
Require syntax.
Require environments.
Require typecheck.
Require freevars.
Require PolyList.
Require ApTypes.
Require NF.
Require valid.
Require OSrules.
Require envprops.
Require rename.
Require NFprops.
Require TypeThms.


(********************************************************)
(*   							*)
(*   Subject Reduction + NF 				*)
(*	<<e,A>> -> <<e',A'>>  --->			*)
(*         Valid(A)           --->			*)
(*	   Domt(A)|- e:t      --->			*)
(*         Valid(A') /\  Domt(A')|- e':t /\ (NF e') 	*)
(*   							*)
(********************************************************)

Goal (c,c':config) (OSred c c') ->
     (valid_env (cfgenv c)) ->
     (t:ty) (TC (OS_Dom_ty (cfgenv c)) (cfgexp c) t) ->
     ((valid_env (cfgenv c')) /\
      (TC (OS_Dom_ty (cfgenv c')) (cfgexp c') t) /\
      (NF (cfgexp c'))). 

Induction 1; Simpl; Intros.
(* CO CT CF CL *)
Split. Assumption.  Split. Assumption.  Apply NF_Sno; Apply Sno_o.
Split. Assumption.  Split. Assumption.  Apply NF_ttt.
Split. Assumption.  Split. Assumption.  Apply NF_fff.
Split. Assumption.  Split. Assumption.  Apply NF_F; Apply F_abs.
(* PO *)
Specialize inv_TC_prd with 1:=H3; Induction 1; Intros Q T.
Apply H1. Assumption.
Rewrite -> Q; Assumption.
(* P *)
Specialize inv_TC_prd with 1:=H3; Induction 1; Intros Q T.
Elim H1 with t. Intro V; Induction 1; Intros Ts Nf.
Split. Assumption.
Split.
Specialize inv_TC_succ with 1:=Ts; Induction 1; Intros.
Rewrite -> Q; Assumption.
Apply NF_Sno; Apply inv_Sno_s; Apply inv_NF_Sno; Assumption.
Assumption.
Rewrite -> Q; Assumption.
(* ZT *)
Specialize inv_TC_is_o with 1:=H3; Induction 1; Intros Q T.
Elim H1 with nat_ty. Intro V; Induction 1; Intros Tc Nf.
Split. Assumption.
Split. Rewrite -> Q; Apply TC_ttt.
Apply NF_ttt.
Assumption.
Assumption.
(* ZF *)
Specialize inv_TC_is_o with 1:=H3; Induction 1; Intros Q T.
Elim H1 with nat_ty. Intro V; Induction 1; Intros Tc Nf.
Split. Assumption.
Split. Rewrite -> Q; Apply TC_fff.
Apply NF_fff.
Assumption.
Assumption.
(* S *)
Specialize inv_TC_succ with 1:=H3; Induction 1; Intros Q T.
Elim H1 with nat_ty. Intro V; Induction 1; Intros Te1 Nf.
Split. Assumption.
Split. Rewrite -> Q; Apply TC_succ; Assumption.
Apply NF_Sno;  Apply Sno_s; 
Apply NFenat_Snoe with (OS_Dom_ty A'); Assumption.
Assumption.
Assumption.
(* Var1 *)
Specialize inv_TC_var with 1:=H4; Simpl; Intro M.
Specialize 4 If_T with 1:=M; Intro T.
Cut x=x. Intro R; Specialize T with 1:=R; Intro Q.
Unfold OScons in H3.
Specialize inv_valid_cons with 1:=H3; Induction 1; Intros V Te.
Elim H2 with t. Intro V'; Induction 1; Intros Ten NFen.
Split. Unfold OScons; Apply valid_cons; Assumption.
Split. Change (TC (app VT (nil VT)
              (cons VT (x,t) (OS_Dom_ty A'))) en t0).
Apply TEp_nfvExt with (OS_Dom_ty A').
Elim Q; Assumption.
Red; Intro F; Apply H0.
Elim (TEDomDomty_OSDom A).
Apply TCHet_FVeinDomH with en t.
Replace (OS_Dom_ty A) with (OS_Dom_ty A').  Assumption.
Symmetry; Apply (dom_pres (cfg e A) (cfg en A')).
Assumption.
Assumption.
Reflexivity.
Assumption.
Assumption.
Assumption.
Reflexivity.
(* Var2 *)
Specialize inv_TC_var with 1:=H5; Simpl; Intro M.
Specialize If_F with 1:=M 2:=H0; Intro M1.
Unfold OScons in H4.
Specialize inv_valid_cons with 1:=H4; Induction 1; Intros V Te.
Elim H3 with t0. Intro V'; Induction 1; Intros Ten NFen.
Specialize dom_pres with 1:=H2; Simpl; Intro QA.
Split. Unfold OScons; Apply valid_cons.
Elim QA; Assumption.
Assumption.
Split. Change (TC (app VT (nil VT)
                  (cons VT (x,t) (OS_Dom_ty A'))) en t0).
Apply TEp_nfvExt with (OS_Dom_ty A').
Assumption.
Red; Intro F.
Apply H1.
Elim (TEDomDomty_OSDom A).
Apply TCHet_FVeinDomH with en t0.
Rewrite -> QA; Assumption.
Assumption.
Reflexivity.
Assumption.
Assumption.
Apply TC_var; Assumption.
(* Appl *)
Specialize inv_TC_appl with 1:=H6; Induction 1; Induction 1; Intros T1 T2.
Specialize dom_pres with 1:=H0; Simpl; Intro Q1.
Specialize dom_pres with 1:=H3; Simpl; Intro Q2.
Specialize 7 TEp_Ap with a:=e2 e:=en ne:=en' A:=A n:=n t:=t 1:=H2.
Intro Ap.
Elim H1 with (arr x t0). Intro V'; Induction 1; Intros Ten NFen.
Elim Ap with (OS_Dom_ty A') x t0. Intros Ten' XT.
Apply H4.
Assumption.
Apply TC_clos.
Elim XT; Elim Q1; Assumption.
Assumption.
Specialize ApNewVar with a:=e2 fun:=en b:=en' A:=A n:=n t:=t 1:=H2; 
Intros M x0 F.
Rewrite -> Q1.
Elim (TEDomDomty_OSDom A).
Apply TCHet_FVeinDomH with en (arr x t0).
Rewrite -> Q1; Assumption.
Assumption.
Assumption.
Assumption.
Assumption.
(* IfTrue *)
Specialize inv_TC_cond with 1:=H5.
Induction 1; Intro T1; Induction 1; Intros T2 T3.
Elim H1 with bool_ty. Intro V'; Induction 1; Intros Tt NFt.
Apply H3.  Assumption.
Specialize dom_pres with 1:=H0; Simpl; Intro Q1.
Elim Q1; Assumption.
Assumption.
Assumption.
(* IfFalse *)
Specialize inv_TC_cond with 1:=H5.
Induction 1; Intro T1; Induction 1; Intros T2 T3.
Elim H1 with bool_ty. Intro V'; Induction 1; Intros Tf NFf.
Apply H3.  Assumption.
Specialize dom_pres with 1:=H0; Simpl; Intro Q1.
Elim Q1; Assumption.
Assumption.
Assumption.
(* fix *)
Specialize inv_TC_fix with 1:= H5.
Induction 1; Intros Q Te.
Apply H3.  Assumption.
Apply TC_clos.
Pattern 2 t; Rewrite -> Q; Assumption.
Apply TEp_RenExp with x e.  Assumption.
Specialize (Xmidvar nx x); Induction 1; Intro N.
Left; Assumption.
Right; Red; Intro F; Apply H0.
Elim (TEDomDomty_OSDom A).
Specialize  TCHet_FVeinDomH with 1:=Te 2:=F.
Simpl; Induction 1; Intro R.
Absurd nx=x; Assumption Orelse Symmetry; Assumption.
Assumption.
Assumption.
(* CL *)
Specialize inv_TC_clos with 1:=H5; Induction 1; Intros T1 Te.
Elim H1 with t0. Intro VxA'; Induction 1; Intros Ten Fen.
Unfold OScons in VxA'; 
Specialize inv_valid_cons with 1:=VxA'.
Induction 1; Intros V' T'.
Split.  Assumption.
Split. Apply TC_clos.
Assumption.
Assumption.
Apply NF_F; Apply F_clos;
Apply NFe_Fe with (cons VT (x,t) (OS_Dom_ty A)) s; Assumption.
Unfold OScons; Apply valid_cons.
Assumption.
Assumption.
Assumption.
(* CL' *)
Specialize inv_TC_clos with 1:=H5; Induction 1; Intros T1 Te.
Elim H1 with t0. Intro VxA'; Induction 1; Intros Ten NFen.
Unfold OScons in VxA';
Specialize inv_valid_cons with 1:=VxA'.
Induction 1; Intros V' T'.
Split.  Assumption.
Split.  Change (TC (app VT (nil VT) (OS_Dom_ty A')) en t0).
Apply TEp_inv_nfvExt with (app VT (nil VT) 
               (cons VT (x,t) (OS_Dom_ty A')))
               x t.
Simpl; Assumption.
Reflexivity.
Elim H3; Intro Q.
Apply Snoe_notFVe.
Apply NFenat_Snoe with (cons VT (x,t) (OS_Dom_ty A)).
Assumption.
Elim Q; Assumption.
Specialize 3 NFebool_TF with e:=en 1:=NFen 
                  H:=(cons VT (x,t) (OS_Dom_ty A)).
Induction 1.
Intro T; Rewrite -> T; Apply inv_FV_ttt.
Intro F; Rewrite -> F; Apply inv_FV_fff.
Elim Q; Assumption.
Assumption.
Unfold OScons; Apply valid_cons; Assumption.
Assumption.
Save subjr_NF.


(****************************************)
(*   					*)
(*   Subject Reduction  		*)
(*	<<e,A>> -> <<e',A'>>  --->	*)
(*         Valid(A)           --->	*)
(*	   Domt(A)|- e:t      --->	*)
(*         Domt(A')|- e':t 		*)
(*   					*)
(****************************************)

Goal (e,e':tm)(A,A':OS_env) 
     (OSred (cfg e A) (cfg e' A')) ->
     (valid_env A) ->
     (t:ty) (TC (OS_Dom_ty A) e t) ->
            (TC (OS_Dom_ty A') e' t).

Intros.
Specialize (subjr_NF (cfg e A) (cfg e' A')) with
   1:=H 2:=H0 3:=H1.
Simpl; Induction 1; Intro; Induction 1; Intros; Assumption.
Save subjr_red.


(****************************************)
(*   					*)
(*   Normal Forms			*)
(*	<<e,A>> -> <<e',A'>>  --->	*)
(*         Valid<<e,A>>       --->	*)
(*         e in NF			*)
(*   					*)
(****************************************)

Goal (e,e':tm)(A,A':OS_env) 
     (OSred (cfg e A) (cfg e' A')) ->
     (valid_config (cfg e A)) ->
     (NF e'). 

Intros e e' A A' R VC.
Elim VC; Simpl; Intros V t T.
Specialize (subjr_NF (cfg e A) (cfg e' A'))
 with  1:=R 2:=V 3:=T.
Simpl; Induction 1; Intro; Induction 1; Intros; Assumption.
Save NormalForms.

