(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  valid.v                                                                 *)
(*                                                                          *)
(*  This file contains definitions of valid environments and                *)
(*  configurations, and also a related property.                            *)
(*                                                                          *)
(*                                                                          *)
(*  Jill Seaman                                                             *)
(*  Coq V5.8                                                                *)
(*  November 1993                                                           *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                 valid.v                                  *)
(****************************************************************************)

Require syntax.
Require PolyList.
Require utils.
Require environments.

Require typecheck.

(****************************************************************)
(*							      	*)
(*  Validity functions for OS environments and configurations 	*)
(*							      	*)
(*                   Domt(A)|- e:t  VAL(A) 			*) 
(*  valid_env     : -----------------------			*)
(*                     VAL ([v:t->e]*A)				*)
(*							      	*)
(*                   Domt(A)|- e:t  VAL(A)			*)
(* valid_config   : -----------------------			*)
(*                       VAL<<e,A>> 				*)
(*							      	*)
(****************************************************************)

Inductive valid_env : OS_env -> Prop
       :=  valid_nil  :  (valid_env (nil VTT))
       |  valid_cons :  (v:vari)(t:ty)(e:tm)(A:OS_env)
                        (TC (OS_Dom_ty A) e t) -> (valid_env A) ->
                        (valid_env (cons VTT ((v,t),e) A)).


Inductive valid_config [c:config]:  Prop
       := valid_cfg : (valid_env (cfgenv c)) -> 
                     (t:ty)(TC (OS_Dom_ty (cfgenv c)) (cfgexp c) t) -> 
                     (valid_config c).


(****************************************)
(*  inv_valid_cons		 	*)
(*	Val([v:t->e]A) --->		*)
(*         Val(A) /\ Domt(A)|-e:t	*)
(****************************************)
Definition valid_c :=
     [A:OS_env]
     <Prop>Case A of
          (* nil *)  True
          (* cons *) [vtt:VTT][A':OS_env]
                     (valid_env A')/\
                     (TC (OS_Dom_ty A')
                         (Snd vtt)
			 (Snd (Fst vtt))) end.

Goal (v:vari)(t:ty)(e:tm)(A:OS_env)
     (valid_env (cons VTT ((v,t),e) A)) ->
     ((valid_env A) /\ (TC (OS_Dom_ty A) e t)).

Intros v t e A H.
Change (valid_c (cons VTT ((v,t),e) A)).
Elim H; Simpl;
Exact I Orelse Intros; Split; Assumption.
Save inv_valid_cons.

