(*                              Le_lt_compl.v                               *)

(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)

(* 
 This file gives some additionnal lemmas to the modules
Le and Lt of the distribution: ($COQTH/ARITH/[Le,Lt] )
 
 Please notice that some lemmas and proofs use the constants
 one,two,three,four declared in the "Constants" module
*)

AddPath ".".
Require Arith.
Require Constants.


Lemma lt_or_Zero:(q:nat)(lt O q)\/q=O.
(************************************)
Proof.
  Induction q;Auto.
Qed.

Lemma one_cases:(q:nat)(lt (S O) q)\/q=O\/q=(S O).
(************************************************)
Proof.
 Induction q;Auto.
 Induction n;Auto.
Qed.

Lemma enum1:(n:nat)(lt n two)->n=O\/n=(S O).
(******************************************)
Proof.
 Intro n;Case (one_cases n);[Idtac|Auto].
 Intros; Absurd (le two n);Auto.
Qed.


Lemma le5:(P:nat->Prop)
        (P O)->
        (P one)->
        (P two)->
        (P three)->
        (P four) ->
        ((n:nat)(lt four n)->(P n))->
        (n:nat)(P n).
(************************************)
Proof.
 Do 5 (Induction n;[Auto|Idtac]).
 Intros.
 Apply H4.
 Unfold four;Do 4 Apply lt_n_S;Auto.
Qed.


Lemma enum4:(n:nat)(lt n (S four))->
            n=O\/n=one\/n=two\/n=three\/n=four.
(*********************************************)
Proof.
 Intro n;Pattern n;Apply le5;Auto.
 Intros.
 Absurd (lt n0 (S four));Auto.
Qed.


Lemma not_le_lt:(n,m:nat)~(le n m)->(lt m n).
(*******************************************)
Proof.
 Intros; Case (le_or_lt n m);[Intro|Auto].
 Elim (H H0).
Qed.
Immediate not_le_lt.


Lemma not_lt_le:(n,m:nat)~(lt n m)->(le m n).
(*******************************************)
Proof.
 Intros; Case (le_or_lt m n);[Auto | Intro].
 Elim (H H0).
Qed. 
Immediate not_lt_le.



Lemma lt_not_lt:(n:nat)(p:nat)(lt n p)->~(lt p (S n)).
(***************************************************)
Proof.
 Auto.
Qed.


Lemma lt_a_a_plus_b:(a,b:nat)(lt O b)->(lt a (plus a b)).
(*******************************************************)
Proof.
 Intros.
 Pattern 1 a;Rewrite (plus_n_O a);Auto.
Qed.

Hint lt_a_a_plus_b.




