(* ---                     Mult_compl.v                                 ----*)
(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)


(*
   This file contains some lemmas on multiplication; it is an 
   extension of the module Mult.v *)

AddPath ".".
Require Arith.
Require  Constants.
Require  Le_lt_compl.

(* multiplication is monotonous *)

Lemma mult_le_l:(a,b,c:nat)
                (le b c)->
                (le (mult a b) (mult a c)).
(******************************************)
Proof.
 Induction a;Simpl;[Auto|Intros].
 Apply le_plus_plus;Auto.
Qed.
Hint mult_le_l.


(* multiplication is strictly monotonous *)

Lemma mult_lt_l:(a,b,c:nat)
                (lt O a)->
                (lt b c)->
                (lt (mult a b) (mult a c)).
(******************************************)
Proof.
 Induction a.
 Intros;Absurd (lt O O);Auto.
 Intros;Simpl.
 Apply lt_le_trans with (plus c (mult n b)) .
 Rewrite -> (plus_sym b (mult n b)); 
            Rewrite -> (plus_sym c (mult n b));
            Apply lt_reg_l;Auto.
 Apply le_reg_l;Auto.
Qed.

Hint mult_lt_l.

(* same properties at right *)

Lemma mult_le_r:(a,b,c:nat)
                (le b c)->
                (le (mult b a) (mult c a)).
(*****************************************)
Proof.
 Intros;Rewrite (mult_sym b a);
        Rewrite (mult_sym c a);
        Auto.
Qed.
Hint mult_le_r.


Lemma mult_lt_r:(a,b,c:nat)
                (lt O a)->
                (lt b c)->
                (lt (mult b a) (mult c a)).
(******************************************)
Proof.
 Intros;Rewrite -> (mult_sym b a);
        Rewrite -> (mult_sym c a);
        Auto.
Qed.
Hint mult_lt_r.


Lemma mult_inj_l:(a,b,c:nat)
                 (lt (mult a c) (mult b c))->
                 (lt a b).
(********************************************)
Proof.
 Intros.
 Case (le_or_lt b a);[Intro|Auto].
 Absurd (le (mult b c) (mult a c));Auto.
Qed.


Lemma mult_p_lt_qp:(p,q:nat)
                   (lt O p) ->
                   (lt one q)->
                   (lt p (mult q p)).  
(************************************)
Proof.
 Intros.
 Rewrite -> mult_sym.
 Pattern 1 p ;Rewrite <-  mult_n_1;Auto.
Qed.

Lemma mult_p_le_qp:(p,q:nat)
                   (lt O q)->
                   (le p (mult q p)).
(************************************)
Proof.
 Intros.
 Pattern 1 p; Rewrite <- mult_n_1.
 Rewrite <- (mult_sym p q);Auto.
Qed.

Hint mult_p_lt_qp mult_p_le_qp.
 
Lemma mult_p_lt_pq:(p,q:nat)
                   (lt O p) ->
                   (lt one q)->
                   (lt p (mult p q)).  
(***********************************)
Proof.
 Intros;Rewrite mult_sym;Auto.
Qed.


Lemma mult_p_le_pq:(p,q:nat)
                   (lt O q)->
                   (le p (mult p q)).
(************************************)
Proof.
 Intros;Rewrite mult_sym;Auto.
Qed.

Hint mult_p_lt_pq mult_p_le_pq.


(* exact quotient 
******************)


Lemma quotient_positive:(q,n,n0:nat)
                        n=(mult q n0)->
                        (lt O n)->
                        (lt O q).
(***************************************)
Proof.
 Intros q n n0 H H0.
 Case (lt_or_Zero q);[Auto|Intro H1].
 Absurd (lt O (mult q n0)).
 Rewrite H1;Simpl;Auto.
 Rewrite <- H;Auto.
Qed.

Lemma quotient_gt_one:(q,n,n0:nat)(lt n0 n)->
                                  n=(mult q n0)->
                                  (lt one q).
(*************************************************)
Proof.
 Intros q n n0 H H0; 
  Case (one_cases q); 
  [Auto|(Induction 1;Intro;Absurd (lt n0 n));Auto].
 Rewrite H0;Rewrite H2;Auto.
 Rewrite H0;Rewrite H2;Simpl;Rewrite <- plus_n_O;Auto.
Qed.







