(*  ---                       dicho_strat.v                    --- *)
(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)


(*

 This file presents our favorite strategy, which gave the best results 
 (see the included paper)
 We had some problems to define it in terms of Realizer/Program,
 this will be done in the next release:

 Briefly:


 dicho-strat(n)=
     let l=(ceiling_log2 log2 n)
     in (eucl_dev (two_power (eucl_dev 2 l)) n)
  
  (recall that in the distribution (quotient b a) is the notation
   for the euclidean division of a by b !)

 For instance let n be 87,
 The ceiling logarithm (base 2) of n is l=7 since 64<n<=128 ;
 thus the quotient of 87 by 2^3 is 10.

 Perhaps it's easier to understand this strategy in terms of
binary representation of natural integers:

 For instance, 87 is represented by 1010111; if we cut this
representation in two halves (the shorter at the right hand side,
if the number of digits is odd), we have two words:
   -   1011 -> 10 (it is just dicho_strat(10))
   -    111 -> 7
 Since we have 87=8*10+7,
 the code for (CALL_M 87) will be the  code for (CALL_C 87 10),
 i.e. the concatenation of the code for (CALL_K 10 7), 
 the code for (CALL_M 8), and a MUL statement:
   
    Statement  register  stack

 code for (CALL_K 10 7):

                x         []     
    PUSH        x         [x]
    PUSH        x         [x,x]
    SQR         x^2       [x,x]
    MUL         x^3       [x]
    PUSH        x^3       [x^3,x]
    SWAP        x^3       [x,x^3]
    SQR         x^6       [x,x^3]
    MUL         x^7       [x^3]
    PUSH        x^7       [x^7,x^3]
    SWAP        x^7       [x^3,x^7]
    MUL         x^10      [x^7]
    
 code for (CALL_M 8):

    SQR         x^20      [x^7]
    SQR         x^40      [x^7]
    SQR         x^80      [x^7]

 the final MUL:
    MUL         x^87      []


  The proof is very long, and we hope to shorten it
  quickly.


*)
AddPath ".".
AddPath "$COQTOP/theories/PROGRAMS".
Require strategies.
Require Arith.
Require euclid.
Require shift.
Require two_power.
Require log2_spec.
Require Euclid_proof.
Require Le_lt_compl.
Require Mult_compl.
Require Constants.

Section import_log2.
Variable log2_r:(n:nat)(lt O n)->(log2_spec n). 

Lemma dicho:strategy.
(*******************)
Proof.
 Split.
 Intros n H.
 Elim (ceiling_log2 log2_r n).
 2:Apply lt_trans with four;Unfold one four;Auto.
 Intros x y.
 Elim y;Intros.
 Cut (lt O (S (S O))).
 2:Auto.
 Intro H'.
 (* fails ! !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 Realizer <nat>let (x0:nat)=(quotient (S (S O)) x)
          in (quotient (two_power x0) n).
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *)
 Elim (quotient (S (S O)) H' x);Intros.

 Realizer (quotient (two_power x0) n). 
 (* -------------------------------- *)
 Do 2 Program.
 Auto.
 Program. 
 Elim y1;Intros.
 Elim y0;Intros.
 Elim H2;Intros H4 H5.
 Elim H3;Intros H6 H7.
 Clear y  y0 H2 H3 y1 H'.
 Cut (lt O x0).
 2:Case (lt_or_Zero x0);[Auto|Intro].
 2:Cut (le x one).
 2:Intro.
 2:Absurd (le n two).  
 2:Unfold not;  Intro.
 2:Absurd (lt four two).  
 2:Apply le_not_lt;Auto.
 2:Unfold four two;Auto.
 2:Apply lt_le_trans with n;Auto.
 2:Replace two with (two_power one).
 2:Apply le_trans with (two_power x).
 2:Auto.
 2:Apply two_power_le;Auto.
 2:Simpl;Auto.
 2:Rewrite H6;Rewrite H2;Simpl;Auto.
 Intro x0_pos.
 Cut (lt (S O) (two_power x0)).
 2:Replace (S O) with (two_power O).
 2:Apply two_power_mono;Auto.
 2:Simpl;Auto.
 Intro H8. (* (lt (S O) (two_power x0) *)
 Cut (lt x0 x).
 2:Rewrite H6.
 2:Apply lt_le_trans with (mult x0 (S (S O))).
 2:Rewrite mult_sym;Apply mult_p_lt_qp;Auto.
 2:Auto.
 Intro H9.
 Cut (le x0 (pred x)).
 2:Apply le_S_n.
 2:Elim (S_pred ? ? H9).
 2:Auto.
 Intro H10.
 Cut (lt O n0).
 2:Case (lt_or_Zero n0);[Auto|Intro].
 2:Absurd (lt n (two_power x0)).
 2:Apply le_not_lt.
 2:Apply le_trans with (two_power (pred x)).
 2:Apply two_power_le;Auto.
 2:Auto.
 2:Rewrite H4;Rewrite H2;Simpl.
 2:Auto.
 Intro H11.(* lt O n0 *)
 Split.
 Apply lt_le_trans with (mult n0 (two_power x0)).
 Rewrite mult_sym;Apply mult_p_lt_qp;Auto.
 Rewrite H4;Auto.

 Apply not_lt_le.
 Unfold not;Intro H12.
 Case (enum1 n0 H12);Intro.
 Absurd (lt O n0);[(Rewrite H2;Auto)|Auto].

 Cut (lt n (two_power (S x0))).
 2:Rewrite H4.
 2:Rewrite H2.
 2:Rewrite mult_1_n.
 2:Rewrite two_power_S.
 2:Rewrite mult_sym;Simpl.
 2:Rewrite <- plus_n_O.
 2:Apply lt_reg_l.
 2:Auto.
 Intro.
 Cut (le (two_power x0) n).
 Intro H13.
 Cut (le (S x0) x);[Intro H14|Auto].
 Case (le_lt_or_eq ? ? H14);Intro.
 Cut (le (S x0) (pred x));[Intro|Auto].
 Absurd (lt n n);[Auto|Idtac].
 Apply lt_trans with (two_power (S x0)).
 Auto.
 Apply le_lt_trans with (two_power (pred x)).
 Apply two_power_le;Auto.
 Auto.
 
 Cut (le x0 (S O)).
 Intro H16.
 Absurd (lt n four).
 Apply lt_not_sym.
 Auto.

 Apply lt_le_trans with (two_power (S x0)).
 Auto.
 Replace four with (two_power (S (S O))).
 Apply two_power_le;Auto.
 Unfold four;Simpl;Auto.
 Apply simpl_le_plus_l with x0.
 Replace (plus x0 x0) with (mult x0 (S (S O))).
 Rewrite plus_sym;Simpl.
 Apply le_trans with x.
 Rewrite H6;Auto.
 Elim H15;Auto.
 Rewrite mult_sym;Simpl;Rewrite <- plus_n_O;Auto.
 Rewrite <- (mult_1_n (two_power x0)).
 Elim H2;Rewrite H4;Auto.
Qed.

End import_log2.



