(* ---                              euclid.v                          ----- *)

(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)
(* 
    Some lemmas about qb+r  ... (useful with euclidean division) 
*)


AddPath ".".
Require Arith.
Require Constants.
Require Le_lt_compl.
Require Mult_compl.

Lemma lt_b_qbr:(b,q,r:nat)(lt O r)->
                          (lt O q)->
                          (lt r b)->
                          (lt b (plus (mult q b) r)).
(***************************************************)     
Proof.
 Intros. 
 Apply le_lt_trans with (mult q b) .
 Auto.
 Pattern 1 (mult q b) ;Rewrite -> plus_n_O;Auto.
Qed.

Hint lt_b_qbr.

Lemma  lt_q_qbr:(b,q,r:nat)(lt O r) -> 
                           (lt one b) ->
                           (lt q (plus (mult q b) r)).
(****************************************************)
Proof.
 Induction q;[Auto|Intros].
 Apply lt_trans with (mult (S n) b) .
 Auto.
 Pattern 1 (mult (S n) b) ;Rewrite -> (plus_n_O (mult (S n) b));Auto.
Qed.
Hint lt_q_qbr.



Lemma  lt_q_qbr':(b,q,r:nat)(lt O q)->
                           (lt one b) ->
                           (lt q (plus (mult q b) r)).
(****************************************************)
Proof.
 Intros.
 Apply lt_le_trans with (mult q b);Auto.
Qed.

Hint lt_q_qbr'.


Lemma le_q_qbr:(b,q,r:nat)(lt O b)  -> 
                          (le q (plus (mult q b) r)).
(***************************************************)
Proof.
 Intros.
 Apply le_trans with (mult q b) ;Auto.
Qed.
  
Hint le_q_qbr.


Lemma lt_O_q:(b,q,r:nat)(lt b (plus (mult q b) r))->
                        (lt O r)->
                        (lt r b)->
                        (lt O q).
(***************************************************)
Proof.
 Intros; Case (lt_or_Zero q);[Auto|Intros].
 Absurd (lt b r).
 Apply lt_not_sym;Auto.
 Replace r with (plus (mult q b) r);Auto.
 Rewrite -> H2;Simpl;Auto.
Qed.



