(* -----                          log2_spec.v                          ---- *)

(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)





AddPath ".".
AddPath "$COQTOP/theories/PROGRAMS".
Require Arith.
Require Peano_dec.
Require Constants.
Require Mult_compl.
Require Wf_nat. 
Require euclid.
Require Le_lt_compl.
Require shift.
Require Compare_dec.
Require two_power.

(* specification of logarithm (base 2) *)
(***************************************)
Definition log2_spec:=[n0:nat]
                      {l:nat & {n0=(two_power l)}+
                                 {(lt (two_power l) n0) /\
                                  (lt n0 (two_power (S l)))}}.


Section applications.
 Variable log2:(n:nat)(lt O n)->(log2_spec n).
 
 (* logarithm by excess *)
 (***********************)
 
 Lemma ceiling_log2:(n:nat)(lt one n)->
          {l:nat|(lt (two_power (pred l)) n) /\
                 (le n  (two_power l))}.
 
 (**********************************************)
 Proof.
 Realizer [n:nat]<nat>let (l:nat;b:bool)=(log2 n) 
           in <nat> if b then l else (S l).
 (* -------------------------------------------- *)
  Program_all.
  Rewrite -> e;Split.
  Apply two_power_mono.
  Cut (lt O l).
  Generalize l ;Induction l;Auto.
  Case (lt_or_Zero l);[Auto|Intro eg].
  Absurd (lt one n);[(Rewrite -> e;Rewrite -> eg;Simpl;Auto)|Auto].
  Auto.
  (Unfold pred ;Elim a;Auto).
 Qed.
 
 
 (* logarithm by default *)
 
 Lemma floor_log2:(n:nat)(lt O n)->
         {l:nat|(le (two_power l) n) /\ (lt n (two_power (S l)))}.
 (**************************************************************)
 Proof.
 Realizer [n:nat]<nat>let (l:nat;b:bool)=(log2 n) in  l .
 (* ----------------------------------------------------- *)
  Program_all.
  Elim b;Intro.
  Rewrite y;Split;Auto.
  Elim y;Auto.
 Qed.
End applications.
 
 
 
 
 
 
 
 
 
 








