(* ----                          machine.v                        ----- *)
(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)


(* 
   This file describes the implementation of our powering algorithm:
*)
AddPath ".".
Require monoid.
Require Constants.

(*  Let us imagine an abstract machine for computing on the monoid (M,o,u)
  (see "monoid.v").
    This machine has a register X and a stack S.
*)


Inductive Instr:Set :=
  MUL:Instr         
| SQR:Instr         
| PUSH:Instr        
| SWAP:Instr.       


(* sequences of instructions *)

Inductive Code:Set :=
   End:Code
|  seq:Instr->Code->Code.


(* code appending *)

Recursive Definition app:Code->Code->Code:=
    End c' => c'
  | (seq i  c) c' => (seq i ( app c  c')).


(* semantics *)
(*************)

Section Monoid.
 Variable M:Set.
 Variable MO:(monoid M).
 Local uM := (u ? MO).
 Local oM := (o ? MO).


 Inductive Stack:Set := emptystack:Stack
                      |  push:M->Stack->Stack.

 Definition top:=[s:Stack]<M>Case s of uM
                                      [m:M][_:Stack]m end.
 Definition pop:=[s:Stack]<Stack>Case s of emptystack  
                                          [_:M][r:Stack]r
                                 end.


(* configurations of the abstract machine *)
(******************************************)


 Record Config:Set :=
   config {
         config_X:M;
         config_S:Stack}.


 Lemma Config_inv:(a,a':M)(s,s':Stack)a=a'->s=s'->(config a s)= (config a' s').
 (****************************************************************************)
 Proof.
  Intros.Rewrite H;Rewrite H0;Auto.
 Qed.
 
 Hint Config_inv.




(* Operational semantics of the elementary instructions *)

 Definition Exec1:Instr->Config->Config:=
    [c:Instr][v:Config]<Config>Case v of
                      [m:M][s:Stack]
           <Config> Case c of
           (* MUL *)  (config (oM m (top s)) (pop s)) 
      	       	       	    
           (* SQR *)  (config ( oM m  m) s)

           (* PUSH *) (config m (push m s))
      	       	       	    
           (* SWAP *)  (config m (push (top (pop s)) 
		                   (push  (top s) (pop (pop s)))))
                       end
   end.
 (****************************************************************)



(* Execution of a compound instruction *)

 Recursive Definition Exec:Code->Config->Config:=
     End v => v
  |  (seq i c) v => (Exec c (Exec1 i v)).
 (****************************************)

 (* Semantics of code   appending *)
 (*********************************)

 Lemma Exec_app: (c,c':Code)(v:Config)
                 (Exec (app c  c') v)=(Exec c' (Exec c v)).
 (*******************************************************)
 Proof.
  Induction c;Simpl.
  Auto.
  Intros;Rewrite H;Auto. 
 Qed.

End Monoid.


