(* ---                               main.v                             --- *)
(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)



(*

  This the entry point of all the development;
  The theorem "addchains" states you can compute
  x^n in a given monoid , once you got a strategy
  (see strategies, dicho_strat, binary_strat)

*)


AddPath "$COQTH/PROGRAMS".
AddPath ".".
Require Constants.
Require generation.
Require monoid.
Require machine.
Require strategies.
Require spec.
Require log2_spec.
Require log2_implementation.
Require Compare_dec.
(* for using with Write Caml File *)

Require while.
Require imperative.
Require develop.
Require dicho_strat.
Require binary_strat.
Require trivial.
Require standard.
Require monofun.
Require matrix.

Definition code_gen := [s:strategy](chain_algo s log2_impl).
Definition power_algo_r:=[s:strategy](power_algo s log2_impl).

Theorem addchains:(gamma: strategy)
                  (n:nat)
                  (M:Set)
                  (MO:(monoid M))
                  (a:M)
                  {b:M|b=(power M MO a n)}.
(*******************************************)
Proof.
Realizer [gamma:nat->nat]
          [n:nat][M:Set][MO:(monoid M)] (* (monoid M)] *)
          [a:M]
           <M> if (zerop n) then (u M MO)
           else
             [{H:(lt O n)}](config_X M (Exec M MO 
                                             (power_algo gamma log2_impl n) 
                                             (config M a emptystack))).
 Program_all.
 Rewrite e; Auto.
 Inversion_clear s.
 Rewrite  H0; Simpl;Auto.
Qed.


Definition dic := (dicho log2_impl).



(*  Fibonacci comes back again! *)
(********************************)


Lemma fibonacci:(n:nat){q:nat | q=(Fib n)}.


Realizer [n:nat]<nat>if (zerop n) then (S O)
                     else (M11 (addchains dic n  Mat2 matrix fib_mat )).
Program_all.
Rewrite e;Rewrite Unfold_FibO;Auto.
Rewrite fib_computation.
 Rewrite e;Auto.
Auto.
Qed.

(* generation of an ML File    *)
(*******************************)

Load "$COQTOP/tactics/contrib/path".
Require Extraction.
Write Caml File "Main" [nat_rec state_b state_p state_n log2_spec log2_impl
                        M3 C2M M2q  
                        MMC KMC MMK KMK COND3  while_not Imperative top pop 
                        push nat_rec zerop
                        Exec Exec1 addchains mkmonoid u o ceiling_log2 
                        floor_log2 one 
                        two three  power_algo chain_algo code_gen power_algo_r
                        dicho binary standard funmono trivial matrix 
                         M11 M12 M21 M22 Mat_mult fibonacci
                        dic ].
                         

