(*                     --- shift.v ---                                      *)

(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)

(* 
  Many programmers use left and right shifts in their code;
  so we found useful to provide some definitions and lemmas
  about these operations.
*)

AddPath ".".
AddPath  "$COQTOP/theories/PROGRAMS".

Require Arith.
Require Peano_dec.
Require Constants.
Require Mult_compl.
Require Wf_nat. 
Require euclid.
Require Le_lt_compl.
Require Euclid_proof.
Require Compare_dec.

Lemma plus_n_1:(n:nat)(S n)=(plus n (S O)).
Proof.
 Intro n;Rewrite <- (plus_n_Sm n O);Auto.
Qed.


(* machine arithmetics *)

(* Left shift *)

Definition shift:=[n:nat](mult two n).
(************************************)

(* Right shift (and information on parity) *)

Lemma Unshift:(n:nat){m:nat & {n=(shift m)}+{n=(S (shift m))}}.
(*************************************************************)
Proof.
 Realizer [n:nat]<nat*bool>let (q,r:nat)=(eucl_dev two n)
                in (q,(zerop r)).

 (* ------------------------------------------------------- *)
 Program_all.
 Rewrite e;Rewrite e0;Rewrite <- plus_n_O;Unfold shift;Auto.
 Cut r=(S O).
 Intro eq1.
 Rewrite e;Rewrite eq1.
 Unfold shift.
 Rewrite <- plus_n_1;Auto.
 Case (enum1 r).
 Auto.
 Intro e';Absurd (lt O r);[(Elim e';Auto)|Auto].
 Auto.
Qed.


Lemma half_lt:(a,b:nat)(lt O b)->
                       (b=(shift a) \/ b=(S (shift a)))->
      	       	       (lt a b).
(********************************************************)
Proof.
 Intros;Case (lt_or_Zero a);Intro.
 Case H0;Intro.
 Rewrite H2.
 Unfold shift ;Apply mult_p_lt_qp;Auto.
 Rewrite  H2.
 Rewrite  plus_n_1.
 Unfold shift.
 Rewrite -> mult_sym;Apply lt_q_qbr;Auto.
 Rewrite -> H1;Auto.
Qed.
Hint half_lt.










