(*                           --- while.v ---                                *)
(*                                                                          *)
(* Author: Pierre Casteran.                                                 *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)



Section while_do.
 Variable St:Set. (* states *)
 Variable postcond:St->Prop.
 Variable precond: St->Prop.
 Variable Invar:St->Prop.
 Variable term_order:St->St->Prop.
 Variable LoopExit:St->Prop.
 Hypothesis LoopExit_dec:(s:St)(Invar s)->{(LoopExit s)}+{~(LoopExit s)}.
 Hypothesis Onestep:(s:St)~(LoopExit s)->
                          (Invar s)->
                          {s':St|(Invar s') /\ (term_order s' s)}.
 Hypothesis LoopExit_ok:(s:St)(LoopExit s)->(Invar s)->(postcond s).
 Hypothesis loopstart:(s:St)(precond s)->(Invar s).


 Hypothesis Termi:(well_founded St term_order).

 Lemma while_not:(s:St)(precond s)->{s':St|(postcond s')}.
 (******************)
 Proof.
 
  Remark loopexec: (s:St)(Invar s)->
                       {s':St | (Invar s') /\ (LoopExit s')}.
  Proof.
   Realizer <St>rec loop :: ::  {term_order} [s:St]<St> if (LoopExit_dec s)
                                                   then s
                                                   else (loop (Onestep s)).
   (* ------------------------------------------------------------------ *)
   Program_all.
   Elim y;Auto.
   Elim y;Auto.
  Qed.

  Remark startloop:(s:St)(precond s)->
              {s':St | (Invar s') /\ (LoopExit s')}.
  Proof.
   Realizer [s:St](loopexec s).
   (* ------------------- *)
   Program_all.
  Qed.

  Realizer [s:St](startloop s).
  (* -------------------------- *)
  Program_all.
  Elim a ;Auto.
 Qed.
End while_do.
(*
 Constant while_not:
  (St:Set)
   (postcond:St->Prop)
    (precond:St->Prop)
     (Invar:St->Prop)
      (term_order:St->St->Prop)
       (LoopExit:St->Prop)
        ((s:St)(Invar s)->{(LoopExit s)}+{~(LoopExit s)})
         ->((s:St)
             ~(LoopExit s)
              ->(Invar s)->{s':St | ((Invar s')/\(term_order s' s))})
            ->((s:St)(LoopExit s)->(Invar s)->(postcond s))
               ->((s:St)(precond s)->(Invar s))
                  ->(well_founded St term_order)
                     ->(s:St)(precond s)->{s':St | (postcond s')}

*)

(* Usage:

  Apply while_not with precond invar term_order loopexit x;
    [ Realizer loopexit_dec |
      Realizer one_step |
      Auto or Idtac |
      Auto or Idtac |
      Auto or Idtac |
      Auto or Idtac ].
*)










