(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*Author: Pierre Casteran.                                                  *)
(*    LABRI, URA CNRS 1304,                                                 *)
(*    Departement d'Informatique, Universite Bordeaux I,                    *)
(*    33405 Talence CEDEX,                                                  *)
(*    e-mail:  casteran@labri.u-bordeaux.fr                                 *)
(*                                                                          *)
(*Date: May, 3, 1993                                                        *)
(*                                                                          *)
(*Pro[gramm,v]ing with continuations:A development in Coq.                  *)
(*                                                                          *)
(*(see the file "leavemult.dvi" for more explanations )                     *)
(*                                                                          *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               leavemult.v                                *)
(****************************************************************************)

Require Arith.


(* Binary trees on some domain:*)

Section Domain.

Variable Dom:Set.

Inductive tree : Set := 
     leaf : Dom -> tree
   | cons : tree -> tree -> tree.

End Domain.


(* Binary trees labeled by natural numbers
*)

Definition nat_tree := (tree nat).
Definition nat_cons := (cons nat).
Definition nat_leaf :=(leaf nat).


(* Product of all the leaves of some nat_tree 
*)

Fixpoint leavemult [t:nat_tree] : nat :=
   <nat>Case t of 
        (* (nat_leaf n1) *) [n1:nat]n1
     (* (nat_cons t1 t2) *) [t1:nat_tree][t2:nat_tree]
					(mult (leavemult t1) (leavemult t2)) 
   end.


(* the specification  of our problem:
*)

Definition SPECIF:=[t:nat_tree] { n:nat| n=(leavemult t) }.


(* A (too much) trivial proof 
*)

Theorem trivialalgo: (t:nat_tree)(SPECIF t).

Intro t.
Unfold SPECIF. 
Apply exist with (leavemult t);Auto.
Save.


(* Here we  define a predicate "Has an occurrence of O" 
*)

Fixpoint Has_Zero [t:nat_tree] : Prop :=
   <Prop>Case t of
       (* (nat_leaf n1) *)[n1: nat] n1 = O
    (* (nat_cons t1 t2) *)[t1: nat_tree][t2:nat_tree] 
					    ((Has_Zero t1) \/ (Has_Zero t2))
   end.


(* If some tree t has an occurence of 0, then (leavmult t)=0 
*)

Lemma zero_occ: (t:nat_tree)(Has_Zero t) -> (leavemult t)=O.

Induction t.
Induction d;Simpl;Auto.Intros t1 H1 t2 H2 H.
Simpl.Elim H;Intro H0.
Cut (leavemult t1)=O.Intro H3.Rewrite H3;Simpl;Auto.Auto.
Cut (leavemult t2)=O.Intro H3.Rewrite H3;Simpl.
Symmetry;Apply mult_n_O.Auto.
Save.



(* A  proof of (t:nat_tree)(SPECIF t)
   which uses the preceding lemma
*)

Theorem cpsalgo: (t:nat_tree)(SPECIF t).

Intro.
Cut (Has_Zero t) -> (SPECIF t).
Intro ESCAPE_O.

(*
2 subgoals
  (SPECIF t)
  ============================
    ESCAPE_O : (Has_Zero t)->(SPECIF t)
    t : nat_tree
subgoal 2 is:
  (Has_Zero t)->(SPECIF t)
*)

2:Intro.
2:Unfold SPECIF.
2:Apply exist with O.
2:Symmetry.
2:Apply zero_occ.
2:Auto.

(*
1 subgoal
  (SPECIF t)
  ============================
    ESCAPE_O : (Has_Zero t)->(SPECIF t)
    t : nat_tree

*)
Local subtree_ersatz:=[t',t:nat_tree]((Has_Zero t')->(Has_Zero t)).

Hint Unfold subtree_ersatz.

Local kappa:=[t:nat_tree]
                   [t':nat_tree]
                       (n':nat)(n'=(leavemult t'))-> (SPECIF t).
Hint Unfold kappa.

Cut (t':nat_tree)(subtree_ersatz t' t)->(kappa t t')->(SPECIF t).
Hint Unfold SPECIF.

Intro AUX.
Apply AUX with t.
Auto.
Unfold kappa.
Intros n H.
Unfold SPECIF;Apply exist with n;Auto.

(*
1 subgoal
  (t':nat_tree)(subtree_ersatz t' t)->(kappa t t')->(SPECIF t)
  ============================
    ESCAPE_O : (Has_Zero t)->(SPECIF t)
    t : nat_tree

*)

Induction t'.
Induction d.
Intros H H0.
Apply ESCAPE_O.
Apply H.Simpl.
Auto.
Intros y H1 H2 H3.
Unfold kappa in H3.
Apply H3 with (S y).
Simpl.
Auto.

Intro t1. Intro ind1. Intro t2. Intro ind2.Intros H H0.
Apply ind2.
Unfold subtree_ersatz.
Intro H1.
Unfold subtree_ersatz in H;Apply H.
Unfold Has_Zero.
Unfold Has_Zero in H1.Auto.
(*

1 subgoal
  (kappa t t2)
  ============================
    H0 : (kappa t (cons nat t1 t2))
    H : (subtree_ersatz (cons nat t1 t2) t)
    ind2 : (subtree_ersatz t2 t)->(kappa t t2)->(SPECIF t)
    t2 : (tree nat)
    ind1 : (subtree_ersatz t1 t)->(kappa t t1)->(SPECIF t)
    t1 : (tree nat)
    t' : nat_tree
    ESCAPE_O : (Has_Zero t)->(SPECIF t)
    t : nat_tree



*)


Unfold kappa.
Unfold kappa in H0.
Intros n2 eg2.

(*

1 subgoal
  (SPECIF t)
  ============================
    eg2 : <nat>n2=(leavemult t2)
    n2 : nat
    H0 : (n':nat)(<nat>n'=(leavemult (cons nat t1 t2)))->(SPECIF t)
    H : (subtree_ersatz (cons nat t1 t2) t)
    ind2 : (subtree_ersatz t2 t)->(kappa t t2)->(SPECIF t)
    t2 : (tree nat)
    ind1 : (subtree_ersatz t1 t)->(kappa t t1)->(SPECIF t)
    t1 : (tree nat)
    t' : nat_tree
    ESCAPE_O : (Has_Zero t)->(SPECIF t)
    t : nat_tree


*)


Apply ind1.
Unfold subtree_ersatz.
Intro.
Apply H.
Simpl;Auto.
Unfold kappa.
Intros n1 eg1.
Apply H0 with (mult n1 n2).
Simpl.
Rewrite eg2;Rewrite eg1.
Auto.

Save.


(*

Coq < Extraction cpsalgo.
cpsalgo ==> [t:nat_tree]
             (tree_rec nat kappa->SPECIF
               [d:nat]
                (nat_rec kappa->SPECIF [_:kappa] O
                  [y:nat][_:kappa->SPECIF] ([H3:kappa](H3 (S y))) d)
               [_:(tree nat)]
                ([ind1:kappa->SPECIF]
                  [_:(tree nat)]
                   ([ind2:kappa->SPECIF]
                     [H0:kappa]
                      (ind2 [n2:nat](ind1 [n1:nat](H0 (mult n1 n2))))))
               t [n:nat]n)

Coq < Extraction SPECIF.
SPECIF ==> nat

Coq < Extraction kappa.
kappa ==> nat->SPECIF

*)
