(****************************************************************************)
(*                                                                          *)
(* Jean-Francois.Monin@lannion.cnet.fr                                      *)
(* January 1995, Coq V5.10                                                  *)
(*                                                                          *)
(****************************************************************************)
(*                                  cps_N.v                                 *)
(****************************************************************************)

(* ================================================== *)
(*      Continuation Passing Style translation        *)
(* ================================================== *)


Section defcont.

Variable X:Set.

Definition N_glob := [A:Set](A->X)->X.

Local N := N_glob.

Definition N_unit_glob : (A:Set)A->(N A) := 
       [A:Set][a:A] [k:A->X](k a).


(* For application *)
Definition NF_glob := [A,B:Set] (N A->(N B)).

Local NF := NF_glob.

Definition N_abstr_glob : (A,B:Set)(A->(N B)) ->(NF A B) :=
    [A,B:Set][M:A->(N B)]
      [k:(A->(N B))->X](k [x:A](M x)).

(* equivalent definition 
Definition N_abstr_glob : (A,B:Set)(A->(N B)) ->(NF A B) :=
    [A,B:Set][M:A->(N B)]
      (N_unit_glob A->(N B) M).
*)


Definition N_appli_glob : 
    (A,B:Set)(NF A B)->(N A)->(N B) :=
    [A,B:Set][f_c:(NF A B)][a_c:(N A)]
      [k:B->X](f_c [f:A->(N B)](a_c [a:A](f a k))).

Definition N_lift_glob : 
    (A,B:Set)(A->B)->(N A)->(N B) :=
    [A,B:Set][f:A->B][a_c:(N A)]
      [k:B->X](a_c [a:A](k (f a))).


Definition N_letin_glob : 
    (A,B:Set)(N A)->(NF A B)->(N B) :=
    [A,B:Set][a_c:(N A)][f_c:(NF A B)]
      [k:B->X](a_c [a:A](f_c [f:A->(N B)](f a k))).


(* a;b  <==> let x=a in b  *)

Theorem N_seq_glob : (A,B:Set) (N A)->(N B)->(N B).
  Intros A B a_c b_c.
  Apply N_letin_glob with 1:=a_c.
  Apply N_abstr_glob; Intro x; Exact b_c.
Save.
Transparent N_seq_glob.

    
(* -------------------- *)

(* function and operation, 
   obtained after simplification of : 
Theorem Nfn :
   (A:Set)(f:A->A)(N A)->(N A).
Goal.
  Intros A f x_c.
  Apply N_appli_glob with 2:=x_c.
  Apply N_abstr_glob; Intro x.
  Apply N_unit_glob; Exact (f x).
Save.

Theorem N_op :
   (A:Set)(op:A->A->A)(N A)->(N A)->(N A).
Goal.
  Intros A op x_c y_c.
  Apply N_appli_glob with 2:=x_c.
  Apply N_abstr_glob; Intro x.
  Apply N_appli_glob with 2:=y_c.
  Apply N_abstr_glob; Intro y.
  Apply N_unit_glob; Exact (op x y).
Save.
*)

Definition N_fn_glob :
   (A:Set)(f:A->A)(N A)->(N A) :=
   [A:Set][f:A->A][x_c:(N A)]
       [k:A->X](x_c [x:A](k (f x))).

Definition N_op_glob :
   (A:Set)(op:A->A->A)(N A)->(N A)->(N A) :=
   [A:Set][op:A->A->A] [x_c:(N A)][y_c:(N A)]
       [k:A->X](x_c [x:A](y_c [y:A](k (op x y)))).



(* ============================== *)

(* ---------------------------------------- *)
(*             Control Operators            *)
(* ---------------------------------------- *)


(* continuation catched for escapement *)
Definition Tcontcc_glob := [A:Set] A->(B:Set)(N B).
Local Tcontcc := Tcontcc_glob.

Definition contcc_glob : 
 (A:Set)(A->X)-> (Tcontcc A) :=
 [A:Set][k:A->X]
      [a:A][B:Set][k':B->X](k a).

Definition N_callcc_glob : 
   (A:Set)((Tcontcc A)->(N A))->(N A) :=
   [A:Set][cour:(Tcontcc A)->(N A)]
      [k:A->X](cour [a:A][B:Set][k':B->X](k a) k).


Definition N_throw_glob :
 (A,B:Set)(Tcontcc_glob A)->(N A)->(N B) :=
 [A,B:Set][c:(Tcontcc A)][a_c:(N A)][k':B->X]
     (a_c [a:A](c a B k')).


End defcont.

Require Export applytac.

Require user_comarg.

Require gram1.

