(****************************************************************************)
(*                                                                          *)
(* Jean-Francois.Monin@lannion.cnet.fr                                      *)
(* October 1994, Coq V5.8.3                                                 *)
(*                                                                          *)
(****************************************************************************)
(*                                 cps_Nmx.v                                *)
(****************************************************************************)

(*
   Txval is the type of exceptions and is supplied by the user. 
   Txval is a dependent type which depends on Txlev, another
   user-defined type representing different levels of exception.

Example :
 
 Inductive Definition Txlev : Set := l1 : Txlev | l2 : Txlev.
 Inductive Definition Txval : Txlev->Set :=
   e1 : (Txval l1) |
   e2 : nat->(Txval l2).
*)

(* 
   The following primitives  should be used in a section
   beginning with 

Load DeclareNmx.

*)


Grammar command command8 :=
   [ command7($c1) "Vm" "+"  command5($s) ] -> [ $0=<<(Nmx $c1 $s)>> ].

Section defcont.

Variable Txlev:Set.
Variable Txval : Txlev->Set.

Definition Propxg:=(l:Txlev)(Txval l)->Prop.

Variable X:Set.
Variable P:Propxg.
Variable e:(l:Txlev)(x:(Txval l))(P l x)->X.


Definition Nmxglob := 
  [C:Propxg][A:Set]((l:Txlev)(x:(Txval l))(C l x)->(P l x))->(A->X)->X.

Local Nmx := Nmxglob.

Definition Nmx_unit_glob : (C:Propxg)(A:Set)A->(C Vm+ A) :=
   [C:Propxg][A:Set][a:A] 
     [i:(l:Txlev)(x:(Txval l))(C l x)->(P l x)][k:A->X](k a).

Definition Nmx_raise_glob :
   (C:Propxg)(A:Set)(l:Txlev)(x:(Txval l))(C l x)->(C Vm+ A) :=
   [C:Propxg][A:Set][l:Txlev][x:(Txval l)][c:(C l x)]
     [i:(l:Txlev)(x:(Txval l))(C l x)->(P l x)][k:A->X](e l x (i l x c)).


Definition Nmx_elim_Nmx_glob : 
  (A,A':Set)(C,C':Propxg)
    (A->(C' Vm+ A'))->((l:Txlev)(x:(Txval l))(C l x)->(C' l x))->
      (C Vm+ A)->(C' Vm+ A') :=
  [A,A':Set][C,C':Propxg]
    [f:A->(C' Vm+ A')]
      [j:(l:Txlev)(x:(Txval l))(C l x)->(C' l x)]
        [nx:(C Vm+ A)]
          [i:(l:Txlev)(x:(Txval l))(C' l x)->(P l x)][k:A'->X]
           (nx ([l:Txlev][x:(Txval l)][c:(C l x)](i l x (j l x c))) 
               ([a:A](f a i k))).

Definition Nmx_handle_glob :
  (C:Propxg)(A:Set)
    (Nmx:(C Vm+ A))((l:Txlev)(x:(Txval l))(C l x)->(P l x))->(A->X)->X :=
  [C:Propxg][A:Set][Nmx:(C Vm+ A)]Nmx.

End defcont.



Require user_comarg.

Require gram1.


