(****************************************************************************)
(*                                                                          *)
(* Jean-Francois.Monin@lannion.cnet.fr                                      *)
(* August 1994, Coq V5.8.3                                                  *)
(*                                                                          *)
(****************************************************************************)
(*                               Nmxaccu_ex.v                               *)
(****************************************************************************)

(* Version with 2 exceptions *)

Require tests.
Require user_comarg.
Require gram1.
Require specif.

Require cps_Nmx.

Require decl_exc.

Section algo1.

Load DeclareNmx.

Theorem prop_exc : (Pz,Pd:Prop)Propx.
  Unfold Propx Propxg; Intros Pz Pd l.
  Case l.
    Intro zer; Exact Pz.

    Intro b; 
    Exact (b=(xv2 true)->Pd) /\ 
          (b=(xv2 false)->False).
Save.
Transparent prop_exc.


Definition condsum2_accu_cps := 
  [m,a:nat][t:tree]
   (prop_exc (Pposs_zer t) (P_overweight_accu m a t)) Vm+ (condsum_accu m a t).

Definition condsum2_accu_handle : 
   (m:nat)(t:tree)(nx:(condsum2_accu_cps m O t))
      ((l:Txlev)(x:(Txval l)) 
         (prop_exc (Pposs_zer t) (P_overweight_accu m O t) l x)->(P l x))->
       ((condsum_accu m O t)->X)->X :=
   [m:nat][t:tree][nx:(condsum2_accu_cps m O t)]
     (Nmx_handle (prop_exc (Pposs_zer t) (P_overweight_accu m O t))
               (condsum_accu m O t) nx).

Definition T_aux := [m,n:nat]{n':nat|n=n'&~(le m n')}.


Grammar tactic simple_tactic :=
  [ "Split_condsum" user_comarg($c1) user_comarg($c2) user_comarg($c3) ] -> 
  [$0=<:tactic:<Intro s;Elim s;Clear s; Intros $c1 $c2 $c3>>].

Theorem core_V2 : 
    (m:nat)(t:tree)(condsum2_accu_cps m O t).

Do 2 Intro.
Cut (n:nat) (prop_exc False (le m n)) Vm+ (T_aux m n).
   Intro g.
   Cut (a:nat)(condsum2_accu_cps m a t); [Intro comprec;Apply comprec | Idtac].
   Unfold condsum2_accu_cps; Elim t.
     Intros n a.
     Case (nul_dec n).
       Intro H0; NmxRaise xv1; Simpl; Elim H0; Auto.
       Unfold not; Intro Hn0; NmxElim (g (plus a n));
         [Intro sn'; NmxUnit Realizer sn' | AutoTx].
  
     Hint deb_left_accu_plus deb_right_accu_plus deb_accu_plus_r.
     Intros t1 R1 t2 R2 a.
     NmxElim (R1 a); [Split_condsum an1 Han1t1 Hman1 | AutoTx]. 
     NmxElim (R2 an1); [Split_condsum an Hant2 Hmn | Rewrite Han1t1; AutoTx]. 
     NmxUnit Realizer an.
     Rewrite Hant2; Rewrite Han1t1; Rewrite plus_assoc_r; Simpl; Auto.

     (* definition de g *)

   Intro n; Case (le_dec m n).
     Hint noteq_xv2_true_false. 
     Intro Hlemn; NmxRaise (xv2 true); Simpl; Auto.
     Intro Nlemn; NmxUnit Realizer n.
Save.

End algo1.

Theorem success2 : (m:nat)(t:tree) (condsum_accu m O t)->(RESU2 m t).
  Intros m t ca; Exists (some_b false).
  Elim ca; Simpl.
  Intros n Hn; Rewrite Hn; Auto.
Save.

(* ==> [_:nat] ([_:tree] ([_:nat] (some_b false))) *)


Theorem exc_overweight_accu : 
   (m:nat)(t:tree)
     (l:Txlev)(x:(Txval l))
       (prop_exc (Pposs_zer t) (P_overweight_accu m O t) l x)
        ->(RESU2 m t).

Intros m t l; Case l. 
  Intros x1 Hexc1; Exists exc_b; Auto.

  Intros x2 Hexc2; Exists (some_b (xl2_bool x2)).
  Generalize Hexc2; Clear Hexc2; Rewrite (inv_xv2_xl2_bool x2).
  Elim (xl2_bool x2); Simpl; Intro Hexc2; Elim Hexc2; Intros Ht Hf.
    Cut (le m (plus O (leaveplus t))); [Auto | Elim Ht; Simpl; Auto].

    Elim Hf; Auto.
Save.

(* ==> [_:nat]([_:tree]([l:Txlev]
           <Txval->boolexc>Case l of
                  ([_:Txval] exc_b)
                  ([x2:Txval](some_b (xl2_bool x2)))
                  end))
*)


Theorem F_overweight_V2 : (m:nat)(t:tree)(RESU2 m t).
Intros m t; Apply condsum2_accu_handle with 
        (prop_exc (Pposs_zer t) (P_overweight_accu m O t)) m t.
   Apply core_V2; Exact (exc_overweight_accu m t).
   Intros l x h; Exact h.
   Exact (success2 m t).
Save.

(* ==> [m:nat][t:tree]
         (condsum2_accu_handle boolexc m t
             (core_V2 boolexc (exc_overweight_accu m t) m t) (success2 m t))
*)


Section algo2.

Load DeclareNmx.

Theorem prop_excl : (Pz:Prop)Propx.
  Unfold Propx Propxg; Intros Pz l.
  Elim l.
    Intro zer; Exact Pz.

    Intro b; Exact False.
Save.
Transparent prop_excl.

Definition lcond_cps := 
   [m:nat][lt:ltree]
     (prop_excl (Pposs_zerl lt)) Vm+ (lcond m lt).

Definition lcond_handle : 
   (m:nat)(lt:ltree)(nx:(lcond_cps m lt))
      ((l:Txlev)(x:(Txval l)) 
         (prop_excl (Pposs_zerl lt) l x)->(P l x))->
       ((lcond m lt)->X)->X :=
   [m:nat][lt:ltree][nx:(lcond_cps m lt)]
     (Nmx_handle (prop_excl (Pposs_zerl lt))
               (lcond m lt) nx).

Definition SPEC_resu_aux :=
   [m:nat][t:tree][b:bool]
        (b=true -> (P_overweight_accu m O t))/\
        (b=false->~(P_overweight_accu m O t)).

Definition Tl_aux := 
   [m:nat][t:tree]{b:bool|(SPEC_resu_aux m t b) (* & ~(Pposs_zer t) *)}.
(*                                                 ^^^^^^^^^^^^^^^^
   We cannot guarantee that t has no zero, because another exception
   (overweight) could have been raised before the discovery of a zero.
*)

Theorem catch_overweight :
 (m:nat)(t:tree)
  (l:Txlev)(x:(Txval l))
    (prop_exc (Pposs_zer t) (P_overweight_accu m O t) l x)->
       (prop_excl (Pposs_zer t)) Vm+ (Tl_aux m t).

Intros m t l; Case l; Simpl; Intros v Hv.
   NmxRaise xv1; Simpl; Auto.

   NmxUnit Realizer (xl2_bool v).
   Unfold SPEC_resu_aux; Elim Hv; Intros Ht Hf; Split.
      Intro Hxt; Apply Ht; Elim Hxt; Apply inv_xv2_xl2_bool.
      Intro Hxf; Elim Hf; Elim Hxf; Apply inv_xv2_xl2_bool.
Save.

(* ==> [_:nat]([_:tree]([l:Txlev]
          <Txval->(Nmx Tl_aux)>Case l of
             ([_:Txval] (Nmx_raise X e xl1 xv1 Tl_aux))
             ([v:Txval] (Nmx_unit X Tl_aux (xl2_bool v)))
             end))
*)

Theorem core_list : (m:nat)(lt:ltree)(lcond_cps m lt).
Do 2 Intro; Unfold lcond_cps.
Elim lt.
  NmxUnit Realizer nilb.

  Clear lt; Intros t lt Rlt.
  Cut (prop_excl (Pposs_zer t)) Vm+ (Tl_aux m t).
    Intro g.
    NmxElim g; [Intro aux_b | AutoTx].
      NmxElim Rlt; [Intro Rlb; NmxUnit Realizer (consb aux_b Rlb) | AutoTx].
         Elim s0; Elim x; Unfold P_overweight_accu; Auto.

   (* Definition of g *)

    Apply condsum2_accu_handle with 
         (prop_exc (Pposs_zer t) (P_overweight_accu m O t)) m t.
      Apply core_V2; Exact (catch_overweight m t).

      Intros l x; Intro h; Exact h.
    
      Intro s; Elim s; Simpl; Intros n Hnt Hmn.
      NmxUnit Realizer false.
      Unfold SPEC_resu_aux; Split.
         Intro E; Discriminate  E.
 
         Intro triv; Unfold not; Intro Hov.
         Apply Hmn; Rewrite Hnt; Elim Hov; Auto.
Save.      

End algo2.

Theorem excl_overweight_list : 
   (m:nat)(lt:ltree)(l:Txlev)(x:(Txval l))
      (prop_excl (Pposs_zerl lt) l x)->(RESUL m lt).
Intros m lt l x Hexc; Exists exc_lb.
Generalize l x Hexc; Clear Hexc x l; AutoTx.
Save.

(* ==> [_:nat] ([_:ltree] ([_:Txlev] ([_:Txval] exc_lb))) *)


Theorem success_list : (m:nat)(lt:ltree) (lcond m lt)->(RESUL m lt).
Intros m lt lc; Case lc; Intros lb Hlb.
Exists (some_lb lb); Auto.
Save.

(* ==> [_:nat] ([_:ltree] ([lc:lbool](some_lb lc))) *)


Theorem F_overweight_list : (m:nat)(lt:ltree)(RESUL m lt).
Intros m lt.
Apply lcond_handle with (prop_excl (Pposs_zerl lt)) m lt.
   Apply core_list; Exact (excl_overweight_list m lt).
   Intros l x h; Exact h.
   Exact (success_list m lt).
Save.

(* ==> [m:nat][lt:ltree]
          (lcond_handle lboolexc m lt
             (core_list lboolexc (excl_overweight_list m lt) m lt)
             (success_list m lt))
*)

(* :======================================================= *)
