(****************************************************************************)
(*                                                                          *)
(* Jean-Francois.Monin@lannion.cnet.fr                                      *)
(* August 1994, Coq V5.8.3                                                  *)
(*                                                                          *)
(****************************************************************************)
(*                                decl_exc.v                                *)
(****************************************************************************)

(* 
  Declaration of exceptions.
  To be used by cps_mcont.v
*)

(* 2 levels of exception *)
Inductive Txlev : Set :=  xl1 : Txlev | xl2 : Txlev.

(* The first level carries no value, the second carries a boolean *)
Inductive Txval : Txlev -> Set :=
  xv1 : (Txval xl1) |
  xv2 : bool->(Txval xl2).

Grammar tactic simple_tactic :=
    [ "AutoTx" ] -> 
    [$0=<:tactic:< Intro l; Elim l; Simpl; Intros v Hv; Elim Hv; Auto>>].

Section inverse_of_xv2.

Theorem xl2_bool : (Txval xl2)->bool.
Intro x; Elim x.
  Exact false. (* irrelevant *)
  Intro b; Exact b.
Save.
Transparent xl2_bool.

(* 
  This section aims at proving :
  (v:(Txval xl2)) v=(xv2 (xl2_bool v).
*)

(* Pitfall : do not use this simpler version of l_xl2.
Goal (l:Txlev) l=xl2 ->(Txval l)->(Txval xl2).
  Intros l E v.
  Elim E; Exact v.
Save l_xl2.
*)

Theorem l_xl2 : (l:Txlev) l=xl2 ->(Txval l)->(Txval xl2).
Induction l.
  Intro E; Discriminate  E.
  Intros e v; Exact v.
Save.
Transparent l_xl2.

Theorem inv_xv2_xl2_bool_aux :
    (l:Txlev)(v:(Txval l))(e: l=xl2)
       (l_xl2 l e v)=(xv2 (xl2_bool (l_xl2 l e v))).
Induction v.
  Intros; Absurd xl1=xl2; [Discriminate | Assumption].
  Auto.
Save.

Theorem inv_xv2_xl2_bool : (v:(Txval xl2)) v=(xv2 (xl2_bool v)).
  Intro v.
  Cut xl2=xl2; [Intro e | Auto].
  Cut (l_xl2 xl2 e v)=v; [Intro e'; Elim e' | Auto].
  Apply inv_xv2_xl2_bool_aux.
Save.

End inverse_of_xv2.


Theorem noteq_xv2_true_false : (xv2 true)=(xv2 false)->False.
  Intro E; Discriminate  E.
Save.

(* Hint noteq_xv2_true_false. *)
