(****************************************************************************)
(*                                                                          *)
(* Jean-Francois.Monin@lannion.cnet.fr                                      *)
(* August 1994, Coq V5.8.3                                                  *)
(*                                                                          *)
(****************************************************************************)
(*                                 specif.v                                 *)
(****************************************************************************)

(* ======================================================= *)
(*             Specification and useful lemmas             *)
(* ======================================================= *)

Require Export Le.
Require Export tree_plus.
Require Export Plus.

(* le_plus_trans de la biblio std serait mieux appele le_plus_trans_left *)
Theorem le_plus_trans_right :
     (n:nat)(m:nat)(p:nat)(le n p)->(le n (plus m p)).
Intros; Elim plus_sym; Auto.
Save.
(* Hint le_plus_trans_right. *)

(* ======================================================= *)

Theorem Leibniz_endo : (S:Set)(f:S->S)(n,m:S) n=m -> (f n)=(f m).
Intros S f n m E; Elim E; Auto.
Save.
(* Hint Leibniz_endo. *)

(* ======================================================= *)
(* The first specification *)

Inductive P_overweight [m:nat;t:tree] : Prop :=
   deb_base : (le m (leaveplus t))->(P_overweight m t).
Hint deb_base.
Hint Unfold leaveplus.


Inductive SPEC [m:nat;t:tree] : bool->Prop :=
   St :  (P_overweight m t)->(SPEC m t true)
 | Sf : ~(P_overweight m t)->(SPEC m t false).
Hint St Sf.

Theorem other_Sf: 
  (m,n:nat)(t:tree) n=(leaveplus t) ->~(le m n)->~(P_overweight m t).
Unfold not; Intros m n t Heg Nle_mn Hdeb; Elim Hdeb; Elim Heg; Auto.
Save.

Theorem yet_another_Sf: 
  (m:nat)(t:tree)~(le m (leaveplus t))->~(P_overweight m t).
Unfold not; Intros m t Nle_mt Hdeb; Elim Hdeb; Auto.
Save.

Hint yet_another_Sf.


(* Type of the final result in the simplest case *)

Inductive RESU [m:nat;t:tree] : Set :=
   exres : (b:bool)(SPEC m t b)->(RESU m t).

(* ------------------------------------------------------- *)
(* Specification et lemmes afferents pour l'extraction *)
(* Avec levee d'exception *)

(* acceleration de la levee d'exception via une somme cumulee *)

Inductive condsum_accu [m,a:nat;t:tree] : Set :=
    condsum_accu_intro : (n:nat) n=(plus a (leaveplus t)) ->
                  ~(le m n)->(condsum_accu m a t).

Definition P_overweight_accu := [m,a:nat][t:tree](le m (plus a (leaveplus t))).

Theorem deb_accu_plus_r :
  (m,a,b:nat)(t:tree)
       (P_overweight_accu m a t)->(P_overweight_accu m (plus a b) t).
Unfold P_overweight_accu.
Intros m a b t Hlemt.
Apply le_trans with (plus a (leaveplus t)); Auto.
Save.
Hint deb_accu_plus_r.

Theorem deb_left_accu_plus : 
  (m,a:nat)(t1,t2:tree)(P_overweight_accu m (plus a (leaveplus t2)) t1)->
                       (P_overweight_accu m a (node t1 t2)).
Unfold P_overweight_accu.
Intros m a t1 t2; Simpl.
Pattern (plus (leaveplus t1) (leaveplus t2)); Elim plus_sym; 
    Elim plus_assoc_r; Auto.
Save.
Theorem deb_right_accu_plus : 
  (m,a:nat)(t1,t2:tree)(P_overweight_accu m (plus a (leaveplus t1)) t2)->
                       (P_overweight_accu m a (node t1 t2)).
Unfold P_overweight_accu.
Intros m a t1 t2;
     Elim plus_assoc_l; Intro Hlemt2.
Simpl; Auto.
Save.

(* Hint deb_left_accu_plus deb_right_accu_plus. *)

(* ============================================================== *)

(*
   Pgm avec 2 exceptions de niveaux differents
*)

Section lists.
Variable A : Set.

Mutual Inductive list : Set := nil : list | cons : A -> list -> list.
End lists.

Definition lbool := (list bool).
Definition ltree := (list tree).
Definition nilb := (nil bool).
Definition nilt := (nil tree).
Definition consb := (cons bool).
Definition const := (cons tree).

Inductive Texc : Set :=
     x_zer : Texc.


Inductive lboolexc : Set :=
   exc_lb : lboolexc
 | some_lb : lbool->lboolexc.


(* -------------------------------------- *)
Inductive Pposs_zer : tree->Prop :=
   pz_base : (Pposs_zer (leaf O))
 | pz_left : (t1,t2:tree)(Pposs_zer t1)->(Pposs_zer (node t1 t2))
 | pz_right : (t1,t2:tree)(Pposs_zer t2)->(Pposs_zer (node t1 t2)).
Hint pz_base pz_left pz_right.


Theorem inv_pz_0 :
   (n:nat)(Pposs_zer (leaf n))-> O=n.
Intros n Hposs; Inversion_clear Hposs; Auto.
Save.
(* Hint inv_pz_0. *)


Theorem inv_pz_node :
  (t1,t2:tree)(Pposs_zer (node t1 t2))->
       ((Pposs_zer t1)\/(Pposs_zer t2)).
Intros t1 t2 Hpz; Inversion_clear Hpz; Auto.
Save.
(* Hint inv_pz_node. *)

Theorem not_pz_node :
  (t1,t2:tree)~(Pposs_zer t1)->~(Pposs_zer t2)->~(Pposs_zer (node t1 t2)).
Intros t1 t2 Hnpz1 Hnpz2.
Unfold not; Intro Hpz; Inversion_clear Hpz; Auto.
Save.

(* Hint not_pz_node. *)



(* -------------------------------------- *)

Inductive boolexc : Set :=
   exc_b : boolexc
 | some_b : bool->boolexc.

(* Remarque : non determinisme entre Se2 et St2 *)
Inductive SPEC2 [m:nat;t:tree] : boolexc->Prop :=
   Se2 :  (Pposs_zer t)->(SPEC2 m t exc_b)
 | St2 :  (le m (leaveplus t))->(SPEC2 m t (some_b true))
 | Sf2 : ~(le m (leaveplus t))->(SPEC2 m t (some_b false)).
Hint Se2 St2 Sf2.

Inductive RESU2 [m:nat;t:tree] : Set :=
  cR2 : (r:boolexc)(SPEC2 m t r)->(RESU2 m t).


(* -------------------------------------- *)
Inductive Pposs_zerl : ltree->Prop :=
   base_zerl : (t:tree)(Pposs_zer t)->
              (lt:ltree)(Pposs_zerl (const t lt))
 | rec_zerl : (lt:ltree)(Pposs_zerl lt)->
                   (t:tree)(Pposs_zerl (const t lt)).
Hint base_zerl rec_zerl.

 
Theorem inv_poss_zerl : 
   (t:tree)(lt:ltree)(Pposs_zerl (const t lt))->
            (Pposs_zer t)\/(Pposs_zerl lt).
Intros t lt Hpzl. 
Simple Inversion Hpzl; Unfold const in H0; Injection H0; Intros El Et; Elim El; Elim Et; Auto.
Save.
(* Hint inv_poss_zerl. *)

Theorem inv_poss_zerl_nil : ~(Pposs_zerl nilt).
Unfold not; Intro Hpzn.
Simple Inversion Hpzn; Unfold const nilt in H0; Simplify_eq H0.
Save.
(* Hint inv_poss_zerl_nil. *)


Inductive SPECL [m:nat] : ltree->lboolexc->Prop :=
   Snil : (SPECL m nilt (some_lb nilb))
 | Selz : (lt:ltree)(Pposs_zerl lt)->(SPECL m lt exc_lb)
 | Selt:  (t:tree)(le m (leaveplus t))->
              (lt:ltree)(lb:lbool)(SPECL m lt (some_lb lb))->
                (SPECL m (const t lt) (some_lb (consb true lb)))
 | Self:  (t:tree)~(le m (leaveplus t))->
              (lt:ltree)(lb:lbool)(SPECL m lt (some_lb lb))->
                (SPECL m (const t lt) (some_lb (consb false lb))).
Hint Snil Selz Selt Self.

Inductive RESUL [m:nat;lt:ltree] : Set :=
   cRL : (lb:lboolexc)(SPECL m lt lb)->(RESUL m lt).

Inductive lcond [m:nat;lt:ltree] : Set :=
   clc : (lb:lbool)(SPECL m lt (some_lb lb))->(lcond m lt).


(* ============================================================ *)
