
#open "proof_trees";;
#open "std";;
#open "reduction";;
#open "stdpp";;
#open "initial";;
#open "ast";;
#open "omega";;
#open "names";;
#open "generic";;
#open "term";;
#open "termenv";;
#open "machops";;
#open "tacmach";;
#open "tactics";;
#open "termast";;
#open "timer";;
#open "clenv";;
#open "vernacinterp";;

#infix "THEN";;
#infix "THENS";;

(* FLAGS AND CONTROL *)

let display_time_flag = ref false;;
let display_system_flag = ref false;;
let display_action_flag = ref false;;

vinterp_add ("OmegaFlag",
  fun [VARG_STRING s] () -> 
	let c = try nth_char s 0 with _ -> ` ` in
	let action,flag =
	  match c with 
	     `+` -> (fun b -> b := true),sub_string s 1 (string_length s - 1)
	   | `-` -> (fun b -> b := false),sub_string s 1 (string_length s - 1)
	   | _ -> (fun b -> b := not !b),s in
	match flag with
	   "system" -> action display_system_flag
	 | "action" -> action display_action_flag
	 | "time" -> action display_time_flag
	 | _ -> PPNL [< 'S "Unknown flag "; 'S s >]
   | _ -> failwith "OmegaFlag");;

(* TIMERS *)

let all_time    = timing "Omega     ";;
let solver_time = timing "Solver    ";;
let exact_time  = timing "Rewrites  ";;

let T2 =          timing "Elim      ";;
let T3 =          timing "Simpl     ";;
let T4 =          timing "Generalize";;

(* GENSYMS *)

let new_identifier = 
  let cpt = ref 0 in 
  (fun () -> let s = "Omega" ^ string_of_int !cpt in incr cpt; id_of_string s)
;;

let new_identifier_state = 
  let cpt = ref 0 in 
  (fun () -> let s = make_ident (SA__intern "State") !cpt in incr cpt; s);;

let new_identifier_var = 
  let cpt = ref 0 in 
  (fun () -> let s = "Zvar" ^ string_of_int !cpt in incr cpt; id_of_string s);;

(* TACTICAL: Apply a list of tactics *)

let rec mk_then = 
  function [t] -> t | (t::l) -> t THEN mk_then l | [] -> IDTAC;;

(* EXTRAITS DE TACTICS.ML *)

let absurd_tac c gls =
  (elim_type (ASTnvar (id_of_string "False")) THEN (cut_tac c)
  THENS [cut_tac (applist(pf_global gls (id_of_string "not"),[c]))
         THENS [intros THEN
                  (fun gl ->
                       let (ida,_) = pf_h gl 1
                       and (idna,_) = pf_h gl 2
                       in exact (applist(VAR idna,[VAR ida])) gl);
                   IDTAC];
           IDTAC]) gls;;

let prod_it = it_list (fun c (n,t)  ->  DOP2(Prod,t,DLAM(n,c)));;

let collect_com lbind = 
  map_succeed (function (Com,c)->c | _ -> failwith "Com") lbind;;

let make_clenv_binding_apply wc (c,t) lbind = 
     let largs = collect_com lbind 
     in let lcomargs = length largs 
     in if lcomargs = length lbind then 
               let clause = mk_clenv_from wc (c,t) in
               clenv_constrain_missing_args largs clause
     else if lcomargs = 0 
          then let clause = mk_clenv_rename_from wc (c,t) in
               clenv_match_args lbind clause
          else errorlabstrm "make_clenv_bindings"
          [<'S "Cannot mix bindings and free associations">];;

let resolve_with_bindings_tac  (c,lbind) gl = 
    let (wc,KONT) = StartWalk gl in
    let t = w_hnf_constr wc (w_type_of wc c) in 
    let clause = make_clenv_binding_apply wc (c,t) lbind
    in res_pf KONT clause gl;;

let reduce_to_mind gl t = elimrec t []
    where rec elimrec t l = match whd_castapp_stack t [] with
               (DOPN(MutInd sp,_) as mind,_) -> (mind,prod_it t l)
             | (DOPN(Const _,_),_) -> 
                 (try let t' = strong whd_betaiota (pf_one_step_reduce gl t)
                      in elimrec t' l
                  with UserError _ -> errorlabstrm "tactics__reduce_to_mind"
                                         [< 'S"Not an inductive product" >])
             | (DOPN(MutCase,_),_) ->
             (try let t' = strong whd_betaiota (pf_one_step_reduce gl t)
                      in elimrec t' l
                  with UserError _ -> errorlabstrm "tactics__reduce_to_mind"
                                           [< 'S"Not an inductive product" >])              | (DOP2(Cast,c,_),[]) -> elimrec c l
             | (DOP2(Prod,ty,DLAM(n,t')),[]) -> elimrec t' ((n,ty)::l)
             | _ -> error "Not an inductive product";;

let constructor_tac nconstropt i lbind gl 
    = let cl = pf_concl gl in 
    let (DOPN(MutInd(sp),args) as mind,redcl) = reduce_to_mind gl cl in 
        let nconstr = mis_nconstr (mind_specif_of_mind mind) and
        sigma = Project gl in
         (match nconstropt with SOME expnconstr -> 
                if expnconstr <> nconstr then
                error "Not the expected number of constructors"
                             | _ -> ());
        if i > nconstr then error "Not enough Constructors" 
        else let c = DOPN(MutConstruct(sp,i),args) 
        in let resolve_tac = resolve_with_bindings_tac (c,lbind) in
         (change_pf redcl THEN intros THEN resolve_tac) gl;;

let exists_tac c= constructor_tac (SOME 1) 1 [Com,c];;

(* FIN DE LA COPIE DE TACTICS.ML *)

(* interface to general purpose tactics (with timers) *)
let generalize_tac t = T4 (generalize_tac t);;
let elim t = T2 (elim t);;
let unfold s = unfold_nth_tac [[],id_of_string s];;
let reflexivity = resolve (ASTnvar (id_of_string"refl_equal"));;

(* Tables for hypothesis *)

let rev_assoc k =
  let rec loop = function
     [] -> raise Not_found
   | (v,k')::_ when k = k' -> v
   | _ :: l -> loop l in
  loop;;

let tag_hypothesis,tag_of_hyp, hyp_of_tag =
  let l = ref ([]:(identifier * int) list) in
  (fun h id -> l := (h,id):: !l),
  (fun h -> try assoc h !l with Not_found -> failwith "tag_hypothesis"),
  (fun h -> try rev_assoc h !l with Not_found -> failwith "tag_hypothesis");;

(* Tables for untranslatable expressions *)
let register_constr,find_constr,clear_tables,dump_tables =
  let l = ref ([]:(constr * (identifier * identifier * bool)) list) in
  (fun h id eg b -> l := (h,(id,eg,b)):: !l),
  (fun h -> try assoc h !l with Not_found -> failwith "find_contr"),
  (fun () -> l := []),
  (fun () -> !l);;

(* GENERAL USE PRIMITIVES TO DESTRUCTURATE TERMS *)

(* Some of these should be provided by the base system. May be they are but
 * I didn't find them *)

(* Given an application gives back the function and its argument list *)

let get_applist =
  let rec loop accu = function
     DOPN(AppL,cl) -> 
       begin match list_of_vect cl with
          (h::l) -> loop (l @ accu) h
        | [] -> failwith "get_applist" end
   | DOP2(Cast,c,t) -> loop accu c
   | t -> t,accu in
  loop [];;

exception Destruct;;

(* Given an application of a constant gives back the constant name 
   and its argument list *)

let dest_const_apply t = 
  let f,args = get_applist t in 
  match f with 
     DOPN((Const _ | MutConstruct _ | MutInd _) as s,_) -> id_of_global s,args
   | _ -> raise Destruct ;;

(* Destructurator : it only recognize the kind of patterns used for Omega *)

type result = 
   Kvar of string
 | Kapp of string * constr list
 | Kimp of constr * constr
 (* Yes an object I don't know anything about *)
 | Kufo;;

let destructurate t = 
  match get_applist t with
     DOPN ((Const _ | MutConstruct _ | MutInd _) as s ,_),args ->
       Kapp (string_of_id (id_of_global s),args)
   | VAR id,[] -> Kvar(string_of_id id)
   | DOP2(Prod,typ,DLAM(Anonymous,body)),[] -> Kimp(typ,body)
   | _ -> Kufo;;

(* destructurator for numbers *)
let recognize_number t =
  let rec loop t =
    let f,l = dest_const_apply t in
    match string_of_id f,l with
       "xI",[t] -> 1 + 2 * loop t
     | "xO",[t] -> 2 * loop t 
     | "xH",[] -> 1 in
  let integ t =
    let f,l = dest_const_apply t in
    match string_of_id f,l with
       "POS",[t] -> loop t | "NEG",[t] -> - (loop t) | "ZERO",[] -> 0 in
  try integ t with _ -> error "number";;

(* TERM BUILDERS *)

(* For ast *)
let ast_app f l =
  OP(("APPLIST",[]),ASTnvar (id_of_string f) ::map (bdize (GLOB nil_sign)) l);;

(* For constr *)
let mk_const s =
  let name = id_of_string s in
  let p = nametab__sp_of_id CCI name in
  global_reference (GLOB(initial_sign())) p name;;
let mkc s = let c = mk_const s in (fun l -> applist(c,l));;
let mk_var v = VAR(id_of_string v);;
let mk_lbd x typ c = DOP2(Lambda,typ,DLAM(Name(id_of_string x),c));;
let mk_meta() = DOP0(Meta(newMETA()));;
let mk_integer n =
  let rec loop n = 
    if n=1 then mk_const "xH" else 
      mkc (if n mod 2 = 0 then "xO" else "xI") [loop (n/2)] in
  let c = 
    if n = 0 then mk_const "ZERO" 
    else mkc (if n > 0 then "POS" else "NEG") [loop (abs n)] in
  c;;
let mk_plus t1 t2 = mkc "Zplus" [t1;t2];;
let mk_times t1 t2 = mkc "Zmult" [t1;t2];;
let mk_minus t1 t2 = mkc "Zminus" [t1;t2];;
let mk_eq t1 t2 = mkc "eq" [mk_const "Z"; t1;t2];;
let mk_le t1 t2 = mkc "Zle" [t1;t2];;
let mk_gt t1 t2 = mkc "Zgt" [t1;t2];;
let mk_inv t = mkc "Zinv" [t];;
let mk_and t1 t2 =  mkc "and" [t1;t2];;
let mk_or t1 t2 =  mkc "or" [t1;t2];;
let mk_not t = mkc "not" [t];;
let mk_eq_rel t1 t2 = mkc "eq" [mk_const "relation"; t1;t2];;
let mk_inj t = mkc "inject_nat" [t];;

(* PATHS IN TERMS - only a subset is used *)

type constr_path =
   P_APP of int
 (* Abstraction and product *)
 | P_BODY 
 | P_TYPE
 (* Mutcase *)
 | P_BRANCH of int
 | P_ARITY
 | P_ARG;;

let context operation path (t:constr) =
  let rec loop i = fun
     p (DOP2(Cast,c,t)) -> DOP2(Cast,loop i p c,t)
   | [] t -> operation i t
   | p (DLAM(n,t)) -> DLAM(n,loop (i+1) p t)
   | (P_APP n :: p)  (DOPN(AppL,v) as t) ->
       let f,l = get_applist t in
       let v' = vect_of_list (f::l) in
       v'.(n) <- loop i p v'.(n); (DOPN(AppL,v'))
   | (P_BRANCH n :: p) (DOPN(MutCase,v)) ->
       let v' = copy_vect v in
       v'.(n+2) <- loop i p v.(n+2); (DOPN(AppL,v'))
   | (P_ARITY :: p)  (DOPN(AppL,v)) ->
       let v' = copy_vect v in
       v'.(0) <- loop i p v.(0); (DOPN(AppL,v'))
   | (P_ARG :: p)  (DOPN(AppL,v)) ->
       let v' = copy_vect v in
       v'.(1) <- loop i p v.(1); (DOPN(AppL,v'))
   | p (DOPN(Fix(_,n) as f,v)) ->
       let v' = copy_vect v in
       let l = vect_length v - 1 in
       v'.(l) <- loop i (P_BRANCH n :: p) v.(l); (DOPN(f,v'))
   | (P_BRANCH n :: p) (DLAMV(name,v)) ->
       let v' = copy_vect v in
       v'.(n) <- loop (i+1) p v.(n); DLAMV(name,v')
   | (P_BODY :: p) (DOP2((Prod | Lambda) as k, t,c)) ->
       (DOP2(k,t,loop i p c)) 
   | (P_TYPE :: p) (DOP2((Prod | Lambda) as k, term,c)) ->
       (DOP2(k,loop i p term, c))
   | p t -> 
       PPNL [<printer__prterm t>];
       failwith ("abstract_path " ^ string_of_int(list_length p)) in
  try loop 1 path t
  with e -> PPNL [<printer__prterm t>];raise e;;

let occurence path (t:constr) =
  let rec loop = fun
     p (DOP2(Cast,c,t)) -> loop p c
   | [] t -> t
   | p (DLAM(n,t)) -> loop p t
   | (P_APP n :: p)  (DOPN(AppL,v) as t) ->
       let f,l = get_applist t in loop p v.(n)
   | (P_BRANCH n :: p) (DOPN(MutCase,v)) -> loop p v.(n+2)
   | (P_ARITY :: p)  (DOPN(AppL,v)) -> loop p v.(0)
   | (P_ARG :: p)  (DOPN(AppL,v)) -> loop p v.(1)
   | p (DOPN(Fix(_,n) as f,v)) ->
       let l = vect_length v - 1 in loop (P_BRANCH n :: p) v.(l)
   | (P_BRANCH n :: p) (DLAMV(name,v)) -> loop p v.(n)
   | (P_BODY :: p) (DOP2((Prod | Lambda) as k, t,c)) -> loop p c
   | (P_TYPE :: p) (DOP2((Prod | Lambda) as k, term,c)) -> loop p term
   | p t -> 
       PPNL [<printer__prterm t>];
       failwith ("occurence " ^ string_of_int(list_length p)) in
  loop path t;;

let abstract_path typ path t =
  let term_occur = ref (Rel 0) in
  let abstract = context (fun i t -> term_occur:= t; Rel i) path t in
  mk_lbd "x" typ abstract, !term_occur;;

let focused_simpl path gl =
  let new = context (fun i t -> pf_nf gl t) (rev path) (pf_concl gl) in
  convert_concl new gl;;

let focused_simpl path = T3 (focused_simpl path);;

(* INTERMEDIATE REPRESENTATION FOR FORMULAS *)

type Oformula =
   Oplus of Oformula * Oformula
 | Oinv of  Oformula
 | Otimes of Oformula * Oformula
 | Oatom of  string
 | Oz of int
 | Oufo of constr;;

let rec Oprint = function 
   Oplus(t1,t2) -> 
     print_string "("; Oprint t1; print_string "+"; Oprint t2; print_string ")"
 | Oinv t -> print_string "~"; Oprint t
 | Otimes (t1,t2) -> 
     print_string "("; Oprint t1; print_string "*"; Oprint t2; print_string ")"
 | Oatom s -> print_string s
 | Oz i -> print_int i
 | Oufo f -> print_string "?";;

(* the weight of a formula, used to sort terms in equations *) 
let rec weight = function
   Oatom c -> intern_id c
 | Oz _ -> -1
 | Oinv c -> weight c
 | Otimes(c,_) -> weight c
 | Oplus _ -> failwith "weight"
 | Oufo _ -> -1;;

(* translator from formula to constr *)
let rec val_of = function
   Oatom c -> (VAR (id_of_string c))
 | Oz c -> mk_integer c
 | Oinv c -> mk_inv (val_of c)
 | Otimes (t1,t2) -> mk_times (val_of t1) (val_of t2)
 | Oplus(t1,t2) -> mk_plus(val_of t1) (val_of t2)
 | Oufo c -> c;;

(* translator from formula to omega representation for afine eq *)
let compile name kind = 
  let rec loop accu = function
     Oplus(Otimes(Oatom v,Oz n),r) -> loop ({v=intern_id v; c=n} :: accu) r
   | Oz n ->
      let id = new_id () in
      tag_hypothesis name id;
      {kind = kind; body = rev accu; constant = n; id = id}
   | _ -> error "compile_equation" in
  loop [];;

(* translator from omega representation for afine eq to formula *)
let rec decompile af = 
  let rec loop = function
     ({v=v; c=n}::r) -> Oplus(Otimes(Oatom (unintern_id v),Oz n),loop r) 
   | [] -> Oz af.constant in
  loop af.body;;


let clever_rewrite_base p result theorem gl = 
  let full = pf_concl gl in
  let abstracted,occ = abstract_path (mk_const "Z") (rev p) full in
  let t = 
    applist
      (mk_lbd "P" (arrow (mk_const "Z") (DOP0(Sort prop)))
        (mk_lbd "H" (applist(Rel 1,[result]))
	  (mkc "eq_ind_r" [mk_const "Z"; result;Rel 2; Rel 1; occ; theorem])),
      [abstracted]) in
  exact_time (exact (applist(t,[mk_meta()]))) gl;;

let clever_rewrite_gen p result (s,args) = 
  let theorem = mkc s args in clever_rewrite_base p result theorem;;

let clever_rewrite p vpath s gl = 
  let full = pf_concl gl in
  let abstracted,occ = abstract_path (mk_const "Z") (rev p) full in
  let vargs = map (fun p -> occurence p occ) vpath in
  let t = mkc s (vargs @ [abstracted]) in
  exact_time (exact (applist(t,[mk_meta()]))) gl;;

(* BASIC ALGEBRAIC REWRITERS *)

(* Taking t1 and t2 partially normalized, merge them keeping them sorted. 
 * There may be some redundancies at this level *)

let rec shuffle p (t1,t2)= 
  match t1,t2 with
     Oplus(l1,r1), Oplus(l2,r2) ->
       if weight l1 > weight l2 then begin
	 let (tac,t') = shuffle (P_APP 2 :: p) (r1,t2) in
	 clever_rewrite p 
           [[P_APP 1;P_APP 1]; [P_APP 1; P_APP 2];[P_APP 2]]
           "fast_Zplus_assoc_r"
	 :: tac,
	 Oplus(l1,t')
       end else begin
	 let (tac,t') = shuffle (P_APP 2 :: p) (t1,r2) in
         clever_rewrite p 
           [[P_APP 1];[P_APP 2;P_APP 1];[P_APP 2;P_APP 2]]
           "fast_Zplus_permute"
	 :: tac,
	 Oplus(l2,t')
       end
  | Oplus(l1,r1), t2 -> 
      if weight l1 > weight t2 then
	let (tac,t') = shuffle (P_APP 2 :: p) (r1,t2) in
	clever_rewrite p 
           [[P_APP 1;P_APP 1]; [P_APP 1; P_APP 2];[P_APP 2]]
           "fast_Zplus_assoc_r"
	:: tac, 
	Oplus(l1, t')
      else 
	[clever_rewrite p [[P_APP 1];[P_APP 2]]
                          "fast_Zplus_sym"],
	Oplus(t2,t1)
  | t1,Oplus(l2,r2) -> 
      if weight l2 > weight t1 then
	let (tac,t') = shuffle (P_APP 2 :: p) (t1,r2) in
        clever_rewrite p 
           [[P_APP 1];[P_APP 2;P_APP 1];[P_APP 2;P_APP 2]]
           "fast_Zplus_permute"
	:: tac,
	Oplus(l2,t')
      else [],Oplus(t1,t2)
  | Oz t1,Oz t2 ->
      [focused_simpl p],
      Oz(t1+t2)
  | t1,t2 ->
      if weight t1 < weight t2 then
	[clever_rewrite p [[P_APP 1];[P_APP 2]] "fast_Zplus_sym"],
	Oplus(t2,t1)
      else [],Oplus(t1,t2);;

(* Taking e1 and e2 normalized, multiply them by their respective
   coeff k1 and k2 and merge them keeping them sorted *)

let rec shuffle_mult p_init k1 e1 k2 e2 =
  let rec loop p = function
     (({c=c1;v=v1}::l1) as l1'),(({c=c2;v=v2}::l2) as l2') ->
       if v1 = v2 then
         let tac =
	   clever_rewrite p [[P_APP 1; P_APP 1; P_APP 1; P_APP 1];
			     [P_APP 1; P_APP 1; P_APP 1; P_APP 2];
			     [P_APP 2; P_APP 1; P_APP 1; P_APP 2];
			     [P_APP 1; P_APP 1; P_APP 2];
			     [P_APP 2; P_APP 1; P_APP 2];
			     [P_APP 1; P_APP 2];
			     [P_APP 2; P_APP 2]]
			    "fast_OMEGA10" in
          if k1*c1 + k2 * c2 = 0 then 
            let tac' = 
              clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 2]]
                               "fast_Zred_factor5" in
            tac :: focused_simpl (P_APP 1::P_APP 2:: p) :: tac' :: 
            loop p (l1,l2)
	  else tac :: loop (P_APP 2 :: p) (l1,l2)
       else if v1 > v2 then
	 clever_rewrite p [[P_APP 1; P_APP 1; P_APP 1; P_APP 1];
			   [P_APP 1; P_APP 1; P_APP 1; P_APP 2];
			   [P_APP 1; P_APP 1; P_APP 2];
			   [P_APP 2];
			   [P_APP 1; P_APP 2]]
			  "fast_OMEGA11" ::
          loop (P_APP 2 :: p) (l1,l2')
       else
	 clever_rewrite p [[P_APP 2; P_APP 1; P_APP 1; P_APP 1];
			   [P_APP 2; P_APP 1; P_APP 1; P_APP 2];
			   [P_APP 1];
			   [P_APP 2; P_APP 1; P_APP 2];
			   [P_APP 2; P_APP 2]]
			  "fast_OMEGA12" ::
          loop (P_APP 2 :: p) (l1',l2)
   | ({c=c1;v=v1}::l1), [] -> 
	 clever_rewrite p [[P_APP 1; P_APP 1; P_APP 1; P_APP 1];
			   [P_APP 1; P_APP 1; P_APP 1; P_APP 2];
			   [P_APP 1; P_APP 1; P_APP 2];
			   [P_APP 2];
			   [P_APP 1; P_APP 2]]
			  "fast_OMEGA11" ::
          loop (P_APP 2 :: p) (l1,[])
   | [],({c=c2;v=v2}::l2) -> 
	 clever_rewrite p  [[P_APP 2; P_APP 1; P_APP 1; P_APP 1];
			    [P_APP 2; P_APP 1; P_APP 1; P_APP 2];
			    [P_APP 1];
			    [P_APP 2; P_APP 1; P_APP 2];
			    [P_APP 2; P_APP 2]]
			  "fast_OMEGA12" ::
          loop (P_APP 2 :: p) ([],l2)
   | [],[] -> [focused_simpl p_init] in
  loop p_init (e1,e2);;

(* Same as previous but only e2 is multiplied by a coeff *)
let rec shuffle_mult_right p_init e1 k2 e2 =
  let rec loop p = function
     (({c=c1;v=v1}::l1) as l1'),(({c=c2;v=v2}::l2) as l2') ->
       if v1 = v2 then
         let tac =
	   clever_rewrite p
	     [[P_APP 1; P_APP 1; P_APP 1];
	      [P_APP 1; P_APP 1; P_APP 2];
	      [P_APP 2; P_APP 1; P_APP 1; P_APP 2];
	      [P_APP 1; P_APP 2];
	      [P_APP 2; P_APP 1; P_APP 2];
	      [P_APP 2; P_APP 2]]
	    "fast_OMEGA15" in
          if c1 + k2 * c2 = 0 then 
            let tac' = 
              clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 2]]
                               "fast_Zred_factor5" in
            tac :: focused_simpl (P_APP 1::P_APP 2:: p) :: tac' :: 
            loop p (l1,l2)
	  else tac :: loop (P_APP 2 :: p) (l1,l2)
       else if v1 > v2 then
	 clever_rewrite p [[P_APP 1;P_APP 1]; [P_APP 1; P_APP 2];[P_APP 2]]
               	       	  "fast_Zplus_assoc_r" ::
         loop (P_APP 2 :: p) (l1,l2')
       else
	 clever_rewrite p [[P_APP 2; P_APP 1; P_APP 1; P_APP 1];
			   [P_APP 2; P_APP 1; P_APP 1; P_APP 2];
			   [P_APP 1];
			   [P_APP 2; P_APP 1; P_APP 2];
			   [P_APP 2; P_APP 2]]
			  "fast_OMEGA12" ::
          loop (P_APP 2 :: p) (l1',l2)
   | ({c=c1;v=v1}::l1), [] -> 
	 clever_rewrite p [[P_APP 1;P_APP 1]; [P_APP 1; P_APP 2];[P_APP 2]]
               	       	  "fast_Zplus_assoc_r" ::
          loop (P_APP 2 :: p) (l1,[])
   | [],({c=c2;v=v2}::l2) -> 
	 clever_rewrite p [[P_APP 2; P_APP 1; P_APP 1; P_APP 1];
			   [P_APP 2; P_APP 1; P_APP 1; P_APP 2];
			   [P_APP 1];
			   [P_APP 2; P_APP 1; P_APP 2];
			   [P_APP 2; P_APP 2]]
			  "fast_OMEGA12" ::
          loop (P_APP 2 :: p) ([],l2)
   | [],[] -> [focused_simpl p_init] in
  loop p_init (e1,e2);;

(* Merge two eqs, but the goal is to make them anihilate each other *)
let rec shuffle_cancel p = function 
   [] -> [focused_simpl p]
 | ({c=c1;_}::l1) ->
     let tac = 
       clever_rewrite p [[P_APP 1; P_APP 1; P_APP 1];[P_APP 1; P_APP 2];
			 [P_APP 2; P_APP 2]; 
      	       	       	 [P_APP 1; P_APP 1; P_APP 2; P_APP 1]]
			(if c1 > 0 then "fast_OMEGA13" else "fast_OMEGA14") in
     tac :: shuffle_cancel p l1;;

(* Multiplication by a scalar, the eq is only partially normalized *)

let rec scalar p n = function
   Oplus(t1,t2) -> 
     let tac1,t1' = scalar (P_APP 1 :: p) n t1 and 
         tac2,t2' = scalar (P_APP 2 :: p) n t2 in
     clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 1;P_APP 2];[P_APP 2]]
                    "fast_Zmult_plus_distr" :: 
     (tac1 @ tac2), Oplus(t1',t2')
 | Oinv t ->
     [clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 2]] "fast_Zmult_Zinv_left";
      focused_simpl (P_APP 2 :: p)], Otimes(t,Oz(-n))
 | Otimes(t1,Oz x) -> 
     [clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 1;P_APP 2];[P_APP 2]]
                     "fast_Zmult_assoc_r";
      focused_simpl (P_APP 2 :: p)], 
     Otimes(t1,Oz (n*x))
 | Otimes(t1,t2) -> error "Not handled yet"
 | (Oatom _ as t) -> [], Otimes(t,Oz n)
 | Oz i -> [focused_simpl p],Oz(n*i)
 | Oufo c -> [],(Oufo (mk_times c (mk_integer n)));;

(* Multiplication by a scalar: the equation is normalized *)
let rec scalar_norm p_init = 
  let rec loop p = fun
     [] -> [focused_simpl p_init]
   | (_::l) -> 
       clever_rewrite p
         [[P_APP 1; P_APP 1; P_APP 1];[P_APP 1; P_APP 1; P_APP 2];
          [P_APP 1; P_APP 2];[P_APP 2]]
	 "fast_OMEGA16" :: loop (P_APP 2 :: p) l in
  loop p_init;;

(* Multiplication by a scalar + addition of a constant: the equation is 
   normalized *)
let rec scalar_norm_add p_init =
  let rec loop p = fun
     [] -> [focused_simpl p_init]
   | (_::l) -> 
       clever_rewrite p
         [[P_APP 1; P_APP 1; P_APP 1; P_APP 1];
          [P_APP 1; P_APP 1; P_APP 1; P_APP 2];
	  [P_APP 1; P_APP 1; P_APP 2]; [P_APP 2]; [P_APP 1; P_APP 2]]
	 "fast_OMEGA11" :: loop (P_APP 2 :: p) l in
  loop p_init;;

(* Elimination of Zinv *)
let rec negate p = function
   Oplus(t1,t2) -> 
     let tac1,t1' = negate (P_APP 1 :: p) t1 and 
         tac2,t2' = negate (P_APP 2 :: p) t2 in
     clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 1;P_APP 2]]
                      "fast_Zinv_Zplus" :: 
     (tac1 @ tac2),
     Oplus(t1',t2')
 | Oinv t ->
     [clever_rewrite p [[P_APP 1;P_APP 1]] "fast_Zinv_Zinv"], t
 | Otimes(t1,Oz x) -> 
     [clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 1;P_APP 2]]
                     "fast_Zinv_Zmult_r";
      focused_simpl (P_APP 2 :: p)], Otimes(t1,Oz (-x))
 | Otimes(t1,t2) -> error "Not handled yet"
 | (Oatom _ as t) ->
     let r = Otimes(t,Oz(-1)) in
     [clever_rewrite p [[P_APP 1]] "fast_Zinv_one"], r
 | Oz i -> [focused_simpl p],Oz(-i)
 | Oufo c -> [],(Oufo (mk_inv c));;

(* Takes a constr term, translates it into a normalized formula and gives
 * back the tactic to get this result *)
let rec transform p t = 
  let default () =
    try let v,th,_ = find_constr t in
        [clever_rewrite_base p (VAR v) (VAR th)], Oatom (string_of_id v)
    with _ -> 
        let v = new_identifier_var () and th = new_identifier () in
        register_constr t v th false;
        [clever_rewrite_base p (VAR v) (VAR th)], Oatom (string_of_id v) in
  match destructurate t with
     Kapp("Zplus",[t1;t2]) ->
       let tac1,t1' = transform (P_APP 1 :: p) t1 and 
           tac2,t2' = transform (P_APP 2 :: p) t2 in
       let tac,t' = shuffle p (t1',t2') in
       tac1 @ tac2 @ tac, t'
   | Kapp("Zminus",[t1;t2]) ->
       let tac,t = transform p (mk_plus t1 (mk_inv t2)) in
       unfold "Zminus" :: tac,t
   | Kapp("Zs",[t1]) ->
       let tac,t = transform p (mk_plus t1 (mk_integer 1)) in
       unfold "Zs" :: tac,t
   | Kapp("Zmult",[t1;t2]) ->
       let tac1,t1' = transform (P_APP 1 :: p) t1 and 
           tac2,t2' = transform (P_APP 2 :: p) t2 in
       begin match t1',t2' with
	  (_,Oz n) -> let tac,t' = scalar p n t1' in tac1 @ tac2 @ tac,t'
        | (Oz n,_) ->
	    let sym = 
              clever_rewrite p [[P_APP 1];[P_APP 2]] "fast_Zmult_sym" in
	    let tac,t' = scalar p n t2' in tac1 @ tac2 @ (sym :: tac),t'
	| _ -> default ()
       end
   | Kapp(("POS"|"NEG"|"ZERO"),_) -> [],Oz(recognize_number t)
   | Kvar s -> [],Oatom s
   | Kapp("Zinv",[t]) ->
        let tac,t' = transform (P_APP 1 :: p) t in
        let tac',t'' = negate p t' in
	tac @ tac', t''
   | Kapp("inject_nat",[t']) ->
      begin try 
        let v,th,_ = find_constr t' in 
        [clever_rewrite_base p (VAR v) (VAR th)],Oatom(string_of_id v)
      with _ -> 
        let v = new_identifier_var () and th = new_identifier () in
        register_constr t' v th true;
        [clever_rewrite_base p (VAR v) (VAR th)], Oatom (string_of_id v)
      end
   | _ -> default ();;

(* Coalesce factors on the same variable *)

(* Distribute * and + *)
let shrink_pair p f1 f2 =
  match f1,f2 with
     Oatom v,Oatom _ -> 
       let r = Otimes(Oatom v,Oz 2) in
       clever_rewrite p [[P_APP 1]] "fast_Zred_factor1", r
   | Oatom v, Otimes(_,c2) -> 
       let r = Otimes(Oatom v,Oplus(c2,Oz 1)) in
       clever_rewrite p [[P_APP 1];[P_APP 2;P_APP 2]] "fast_Zred_factor2", r
   | Otimes (v1,c1),Oatom v -> 
       let r = Otimes(Oatom v,Oplus(c1,Oz 1)) in
       clever_rewrite p [[P_APP 2];[P_APP 1;P_APP 2]]
                        "fast_Zred_factor3", r
   | Otimes (Oatom v,c1),Otimes (v2,c2) ->
       let r = Otimes(Oatom v,Oplus(c1,c2)) in
       clever_rewrite p 
         [[P_APP 1;P_APP 1];[P_APP 1;P_APP 2];[P_APP 2;P_APP 2]]
         "fast_Zred_factor4",r
   | t1,t2 -> Oprint t1;print_newline (); Oprint t2; print_newline (); 
              flush std_out; error "shrink.1";;

(* Introduce 1 where necessary, computes coeff after coalescing *)
let reduce_factor p = function
   Oatom v ->
     let r = Otimes(Oatom v,Oz 1) in
     [clever_rewrite p [[]] "fast_Zred_factor0"],r
 | Otimes(Oatom v,Oz n) as f -> [],f
 | Otimes(Oatom v,c) ->
     let rec compute = function
	Oz n -> n
      | Oplus(t1,t2) -> compute t1 + compute t2 
      | _ -> error "condense.1" in
     [focused_simpl (P_APP 2 :: p)], Otimes(Oatom v,Oz(compute c))
 | t -> Oprint t; error "reduce_factor.1";;

let rec condense p = function
   Oplus(f1,(Oplus(f2,r) as t)) ->
     if weight f1 = weight f2 then begin
       let shrink_tac,t = shrink_pair (P_APP 1 :: p) f1 f2 in
       let assoc_tac = 
         clever_rewrite p 
            [[P_APP 1];[P_APP 2;P_APP 1];[P_APP 2;P_APP 2]]
            "fast_Zplus_assoc_l" in
       let tac_list,t' = condense p (Oplus(t,r)) in
       (assoc_tac :: shrink_tac :: tac_list), t'
     end else begin
       let tac,f = reduce_factor (P_APP 1 :: p) f1 in
       let tac',t' = condense (P_APP 2 :: p) t in 
       (tac @ tac'),Oplus(f,t') 
     end
 | (Oplus(f1,Oz n) as t) -> 
      let tac,f1' = reduce_factor (P_APP 1 :: p) f1 in tac,Oplus(f1',Oz n)
 | Oplus(f1,f2) -> 
     if weight f1 = weight f2 then begin
       let tac_shrink,t = shrink_pair p f1 f2 in
       let tac,t' = condense p t in
       tac_shrink :: tac,t'
     end else begin
       let tac,f = reduce_factor (P_APP 1 :: p) f1 in
       let tac',t' = condense (P_APP 2 :: p) f2 in 
       (tac @ tac'),Oplus(f,t') 
     end
 | (Oz _ as t)-> [],t
 | t -> 
     let tac,t' = reduce_factor p t in
     let final = Oplus(t',Oz 0) in
     let tac' = clever_rewrite p [[]] "fast_Zred_factor6" in
     tac @ [tac'], final;;

(* Eliminates 0 *)
let rec clear_zero p = function
   Oplus(Otimes(Oatom v,Oz 0),r) ->
     let tac =
       clever_rewrite p [[P_APP 1;P_APP 1];[P_APP 2]] "fast_Zred_factor5" in
     let tac',t = clear_zero p r in
     tac :: tac',t
 | Oplus(f,r) -> 
     let tac,t = clear_zero (P_APP 2 :: p) r in tac,Oplus(f,t)
 | t -> [],t;;

(* NORMALIZATION OF EQUATIONS *)

let normalize p_initial t =
  let tac,t' = transform p_initial t in
  let tac',t'' = condense p_initial t' in
  let tac'',t''' = clear_zero p_initial t'' in 
  tac @ tac' @ tac'' , t''';;

let normalize_equation id flag theorem pos t t1 t2 (tactic,defs) =
  let p_initial = [P_APP pos ;P_TYPE] in
  let tac,t' = normalize p_initial t in
  let shift_left = 
    generalize_tac [mkc theorem [t1;t2; VAR id]]
    THEN clear_hyp [id] in
  if tac <> [] then
    ((id,(shift_left THEN mk_then tac THEN intros_with [id])) :: tactic,
     compile id flag t' :: defs)
  else (tactic,defs);;

(* RESOLUTION : REPLAY OF THE HISTORY *)

let replay_history tactic_normalisation = 
  let all_eqs = hashtbl__new 7 in
  let aux = id_of_string "auxiliary" in
  let aux1 = id_of_string "auxiliary_1" in
  let aux2 = id_of_string "auxiliary_2" in
  let zero = mk_integer 0 in
  let rec loop t =
    match t with
     (HYP e :: l) -> 
       hashtbl__add all_eqs e.id e;
       begin try assoc (hyp_of_tag e.id) tactic_normalisation THEN loop l
       with Not_found -> loop l end
   | (CONTRADICTION (e1,e2) :: l) -> 
       let eq1 = decompile e1 and eq2 = decompile e2 in 
       let p_initial = [P_APP 2;P_TYPE] in
       let tac = shuffle_cancel p_initial e1.body in
       let solve_le =
	 let superieur = mk_const "SUPERIEUR" in
	 let not_sup_sup = mk_eq_rel superieur superieur in
	 TRY (unfold "Zle" THEN simpl_tac THEN intro THEN
      	 absurd_tac not_sup_sup THENS [ assumption ; reflexivity ]) in
       let theorem =
         mkc "OMEGA2" 
           [val_of eq1; val_of eq2; VAR (hyp_of_tag e1.id);
            VAR (hyp_of_tag e2.id)] in
       generalize_tac [theorem] THEN mk_then tac THEN solve_le

   | (DIVIDE_AND_APPROX (e1,e2,k,d) :: l) ->
       let id = hyp_of_tag e1.id in
       let eq1 = val_of(decompile e1) and eq2 = val_of(decompile e2) in
       let kk = mk_integer k and dd = mk_integer d in
       let rhs = mk_plus (mk_times eq2 kk) dd in
       let state_eg = mk_eq eq1 rhs in
       let tac = scalar_norm_add [P_APP 3] e2.body in
       cut_tac state_eg THENS [
         intros_with [aux] THEN
         generalize_tac [mkc "OMEGA1" [eq1;rhs;VAR aux; VAR id]]
         THEN clear_hyp [aux;id] THEN intros_with [id] THEN
         cut_tac (mk_gt kk dd) THENS [
           cut_tac (mk_gt kk zero) THENS [
	     intros_with [aux1; aux2] THEN
	     generalize_tac 
               [mkc "Zmult_le_approx" [kk;eq2;dd;VAR aux1;VAR aux2; VAR id]]
             THEN clear_hyp [aux1;aux2;id] THEN intros_with [id] THEN loop l;
             unfold "Zgt" THEN simpl_tac THEN reflexivity ];
           unfold "Zgt" THEN simpl_tac THEN reflexivity ];
         mk_then tac THEN reflexivity]

   | (NOT_EXACT_DIVIDE (e1,k) :: l) ->
       let id = hyp_of_tag e1.id in
       let c = floor_div e1.constant k in
       let d = e1.constant - c * k in
       let e2 =  {id=e1.id; kind=true;constant = c; 
                  body = map_eq_linear (fun c -> c / k) e1.body } in
       let eq1 = val_of(decompile e1) and eq2 = val_of(decompile e2) in
       let kk = mk_integer k and dd = mk_integer d in
       let rhs = mk_plus (mk_times eq2 kk) dd in
       let state_eq = mk_eq eq1 rhs in
       let tac = scalar_norm_add [P_APP 2] e2.body in
       cut_tac (mk_gt dd zero) THENS [
         cut_tac (mk_gt kk dd) THENS [
           intros_with [aux2;aux1] THEN
             generalize_tac [mkc "OMEGA4" [dd;kk;eq2;VAR aux1; VAR aux2]]
           THEN clear_hyp [aux1;aux2] THEN unfold "not" THEN intros_with [aux]
	   THEN resolve (ASTnvar aux) THEN mk_then tac THEN assumption;
           unfold "Zgt" THEN simpl_tac THEN reflexivity ];
         unfold "Zgt" THEN simpl_tac THEN reflexivity ]

   | (EXACT_DIVIDE (e1,k) :: l) ->
       let id = hyp_of_tag e1.id in
       let e2 =  map_eq_afine (fun c -> c / k) e1 in
       let eq1 = val_of(decompile e1) and eq2 = val_of(decompile e2) in
       let kk = mk_integer k in
       let rhs = mk_times eq2 kk in
       let state_eq = mk_eq eq1 rhs in
       let tac = scalar_norm [P_APP 3] e2.body in
       cut_tac state_eq THENS [
         cut_tac (mk_gt kk zero) THENS [
           intros_with [aux2;aux1] THEN
           generalize_tac [mkc "OMEGA3" [eq1;eq2;kk;VAR aux2;VAR aux1;VAR id]]
           THEN clear_hyp [aux1;aux2;id] THEN intros_with [id] THEN loop l;
           unfold "Zgt" THEN simpl_tac THEN reflexivity ];
         mk_then tac THEN reflexivity ]

   | (MERGE_EQ(e3,e1,e2)) :: l ->
       let id = new_identifier () in
       tag_hypothesis id e3;
       let id1 = hyp_of_tag e1.id and id2 = hyp_of_tag e2 in
       let eq1 = val_of(decompile e1) and
           eq2 = val_of (decompile (negate_eq e1)) in
       let tac = clever_rewrite [P_APP 3] [[P_APP 1]] "fast_Zinv_one" ::
                 scalar_norm [P_APP 3] e1.body in
       cut_tac (mk_eq eq1 (mk_inv eq2)) THENS [
         intros_with [aux] THEN
         generalize_tac [mkc "OMEGA8" [eq1;eq2;VAR id1;VAR id2; VAR aux]]
         THEN clear_hyp [id1;id2;aux] THEN intros_with [id] THEN loop l;
	 mk_then tac THEN reflexivity]

   | (STATE(new_eq,def,orig,m,sigma) :: l) ->
       let id = new_identifier () and id2 = hyp_of_tag orig.id in
       tag_hypothesis id new_eq.id;
       let eq1 = val_of(decompile def) and eq2 = val_of(decompile orig) in
       let v = unintern_id sigma in
       let vid = id_of_string v in
       let theorem =
         mkc "ex" [mk_const "Z";mk_lbd v (mk_const "Z") (mk_eq (Rel 1) eq1)] in
       let mm = mk_integer m in
       let p_initial = [P_APP 2;P_TYPE] in
       let r = mk_plus eq2 (mk_times (mk_plus (mk_inv (VAR vid)) eq1) mm) in
       let tac = 
         clever_rewrite (P_APP 1 :: P_APP 1 :: P_APP 2 :: p_initial) 
                        [[P_APP 1]] "fast_Zinv_one" ::
         shuffle_mult_right p_initial
           orig.body m ({c= -1;v=sigma}::def.body) in
       cut_tac theorem THENS [
         intros_with [aux] THEN elim (ASTnvar aux) THEN
	 clear_hyp [aux] THEN intros_with [vid; aux] THEN
         generalize_tac
           [mkc "OMEGA9" [VAR vid;eq2;eq1;mm;VAR id2;VAR aux]] THEN
	 mk_then tac THEN clear_hyp [aux] THEN intros_with [id] THEN loop l;
         exists_tac eq1 THEN reflexivity ]

   | (SUM(e3,(k1,e1),(k2,e2)) :: l) ->
       let id = new_identifier () in
       tag_hypothesis id e3;
       let id1 = hyp_of_tag e1.id and id2 = hyp_of_tag e2.id in
       let eq1 = val_of(decompile e1) and eq2 = val_of(decompile e2) in
       if k1 = 1 & e2.kind then
         let kk = mk_integer k2 in
         let p_initial = [P_APP 2;P_TYPE] in
         let tac= shuffle_mult_right p_initial e1.body k2 e2.body in
         generalize_tac[mkc (if e1.kind then "OMEGA5" else "OMEGA6")
                            [eq1;eq2;kk;VAR id1;VAR id2]] THEN
	 mk_then tac THEN intros_with [id] THEN loop l
       else
         let kk1 = mk_integer k1 and kk2 = mk_integer k2 in
         let p_initial = [P_APP 2;P_TYPE] in
	 cut_tac (mk_gt kk1 zero) THENS [
  	   cut_tac (mk_gt kk2 zero) THENS [
	     (let tac= shuffle_mult p_initial k1 e1.body k2 e2.body in
	     intros_with [aux2;aux1] THEN 
             generalize_tac
               [mkc "OMEGA7" [eq1;eq2;kk1;kk2; VAR aux1;VAR aux2;
                              VAR id1;VAR id2]] THEN
	     clear_hyp [aux1;aux2] THEN	mk_then tac
             THEN intros_with [id] THEN
             loop l);
             unfold "Zgt" THEN simpl_tac THEN reflexivity ];
           unfold "Zgt" THEN simpl_tac THEN reflexivity ]

   | (CONSTANT_NOT_NUL(e,k) :: l) -> 
       generalize_tac [VAR (hyp_of_tag e)] THEN discr__discriminate_tac

   | (CONSTANT_NEG(e,k) :: l) ->
       generalize_tac [VAR (hyp_of_tag e)] THEN unfold "Zle" THEN simpl_tac 
       THEN unfold "not" THEN intros_with [aux] THEN resolve (ASTnvar aux) THEN
       reflexivity 

   | _ -> IDTAC in
  loop;;

let destructure_omega gl tac_def id c =
  if atompart_of_id id = SA__intern "State" then tac_def
  else
  try match destructurate c with
     Kapp("eq",[typ;t1;t2]) when destructurate (pf_nf gl typ) = Kapp("Z",[]) ->
       let t = mk_plus t1 (mk_inv t2) in
       normalize_equation id true "Zegal_left" 2 t t1 t2 tac_def
   | Kapp("Zle",[t1;t2]) ->
       let t = mk_plus t2 (mk_inv t1) in
       normalize_equation id false "Zle_left" 2 t t1 t2 tac_def
   | Kapp("Zlt",[t1;t2]) ->
       let t = mk_plus (mk_plus t2 (mk_integer (-1))) (mk_inv t1) in
       normalize_equation id false "Zlt_left" 2 t t1 t2 tac_def
   | Kapp("Zge",[t1;t2]) ->
       let t = mk_plus t1 (mk_inv t2) in
       normalize_equation id false "Zge_left" 2 t t1 t2 tac_def
   | Kapp("Zgt",[t1;t2]) ->
       let t = mk_plus (mk_plus t1 (mk_integer (-1))) (mk_inv t2) in
       normalize_equation id false "Zgt_left" 2 t t1 t2 tac_def
   | _ -> tac_def
  with UserError _ -> tac_def;;

(* RESOLUTION OF A NORMALIZED GOAL *)

let coq_omega gl =
  clear_tables ();
  let tactic_normalisation, system = 
    it_sign (destructure_omega gl) ([],[]) (pf_hyps gl) in
  let prelude,sys = 
    it_list (fun (tac,sys) (t,(v,th,b)) ->
      	       if b then
                 let id = new_identifier () in
  	         let i = new_id () in
                 tag_hypothesis id i;
                 elim (ast_app "intro_Z" [t])
	         THEN intros_with [v; id] THEN elim (ASTnvar id)
                 THEN clear_hyp [id] THEN intros_with [th;id] THEN tac,
                 {kind = false; body = [{v=intern_id (string_of_id v); c=1}]; 
                  constant = 0; id = i} :: sys
               else
                 elim (ast_app "new_var" [t]) THEN intros_with [v;th] THEN tac,
                 sys) 
            (IDTAC,[]) (dump_tables ()) in
  let system = system @ sys in
  if !display_system_flag then display_system system;
  try simplify false system; IDTAC gl
  with UNSOLVABLE -> 
    let path = depend [] [] (history ()) in
    if !display_action_flag then display_action path;
    (prelude THEN replay_history tactic_normalisation path) gl;;

(* COMPUTES A PROOF OF DECIDABILITY OF A PREDICATE *)

let rec decidabilite gl t =
  match destructurate t with
     Kapp("or",[t1;t2]) -> 
       mkc "dec_or" [t1;t2;decidabilite gl t1; decidabilite gl t2]
   | Kapp("and",[t1;t2]) -> 
       mkc "dec_and" [t1;t2;decidabilite gl t1; decidabilite gl t2]
   | Kimp(t1,t2) -> 
       mkc "dec_imp" [t1;t2;decidabilite gl t1; decidabilite gl t2]
   | Kapp("not",[t1]) -> mkc "dec_not" [t1;decidabilite gl t1]
   | Kapp("eq",[typ;t1;t2]) -> 
       begin match destructurate (pf_nf gl typ) with
          Kapp("Z",[]) ->  mkc "dec_eq" [t1;t2]
        | Kapp("nat",[]) ->  mkc "dec_eq_nat" [t1;t2]
        | _ -> PPNL [<printer__prterm typ>]; error "decidabilite.1"
       end
   | Kapp("Zle",[t1;t2]) -> mkc "dec_Zle" [t1;t2]
   | Kapp("Zlt",[t1;t2]) -> mkc "dec_Zlt" [t1;t2]
   | Kapp("Zge",[t1;t2]) -> mkc "dec_Zge" [t1;t2]
   | Kapp("Zgt",[t1;t2]) -> mkc "dec_Zgt" [t1;t2]
   | Kapp("le", [t1;t2]) -> mkc "dec_le"  [t1;t2]
   | Kapp("lt", [t1;t2]) -> mkc "dec_lt"  [t1;t2]
   | Kapp("ge", [t1;t2]) -> mkc "dec_ge"  [t1;t2]
   | Kapp("gt", [t1;t2]) -> mkc "dec_gt"  [t1;t2]
   | Kapp(t,_) -> error ("decidabilite: " ^ t)
   | _ -> error "decidabilite.2";;

(* INJECTS A FORMULA ON NAT IN Z *)

let nat_inject gl =
  let aux = id_of_string "auxiliary" in
  let table = hashtbl__new 7 in
  let hyps = pf_hyps gl in
  let rec explore p t = 
    match destructurate t with
       Kapp("plus",[t1;t2]) ->
         clever_rewrite_gen p (mk_plus (mk_inj t1) (mk_inj t2))
   	                      ("inj_plus",[t1;t2]) THEN 
         explore (P_APP 1 :: p) t1 THEN explore (P_APP 2 :: p) t2
     | Kapp("mult",[t1;t2]) ->
         clever_rewrite_gen p (mk_times (mk_inj t1) (mk_inj t2))
   	                      ("inj_mult",[t1;t2]) THEN 
         explore (P_APP 1 :: p) t1 THEN explore (P_APP 2 :: p) t2
     | Kapp("minus",[t1;t2]) ->
         let id = new_identifier () in
         elim (ast_app "le_gt_dec" [t2;t1]) THEN intros_with [id]
         THENS [
           clever_rewrite_gen p (mk_minus (mk_inj t1) (mk_inj t2))
   	                        ("inj_minus1",[t1;t2;VAR id]) THEN 
	   loop [id] [mkc "le" [t2;t1]] THEN
           explore (P_APP 1 :: p) t1 THEN explore (P_APP 2 :: p) t2;

           clever_rewrite_gen p (mk_integer 0)
   	                        ("inj_minus2",[t1;t2;VAR id]) THEN
	   loop [id] [mkc "gt" [t2;t1]]]

     | Kapp("S",[t']) ->
         let rec is_number t =
	   match destructurate t with 
	      Kapp("S",[t]) -> is_number t
            | Kapp("O",[]) -> true
            | _ -> false in
	 let rec loop p t =
	   match destructurate t with 
              Kapp("S",[t]) ->
                clever_rewrite_gen p (mkc "Zs" [mk_inj t]) ("inj_S",[t]) 
                THEN loop (P_APP 1 :: p) t
            | _ -> explore p t in
         if is_number t' then focused_simpl p else loop p t
     | Kapp("O",[]) -> focused_simpl p
     | _ -> IDTAC and
  loop = fun
     [] [] -> IDTAC
   | (i::li)(t::lt) -> 
       begin match destructurate t with 
          Kapp(("le" | "lt" | "ge" | "gt") as oper,[t1;t2]) ->
	    generalize_tac [mkc ("inj_"^oper) [t1;t2;VAR i]] THEN
            explore [P_APP 1; P_TYPE] t1 THEN explore [P_APP 2; P_TYPE] t2
            THEN clear_hyp [i] THEN intros_with [i] THEN loop li lt
        | Kapp("eq",[typ;t1;t2]) ->
            begin match destructurate (pf_nf gl typ) with
               Kapp("nat",[]) -> 
  	         generalize_tac [mkc ("inj_eq") [t1;t2;VAR i]]
                 THEN explore [P_APP 2; P_TYPE] t1 
                 THEN explore [P_APP 3; P_TYPE] t2
                 THEN clear_hyp [i] THEN intros_with [i] THEN loop li lt
             | _ -> loop li lt
            end
        | _ -> loop li lt
       end in
  loop (rev (fst hyps)) (rev (snd hyps)) gl;;

(* DESTRUCTURATE THE HYPS AND SIMPLIFY THEM, CALL OMEGA *)

let destructure_hyps gl =
  let hyp = pf_hyps gl in
  let rec loop evbd = fun
     [] [] -> nat_inject THEN solver_time coq_omega
   | (i::li) (t::lt) ->
       begin try match destructurate t with
          Kapp(("Zle"|"Zge"|"Zgt"|"Zlt"),[t1;t2]) -> loop evbd li lt
        | Kapp("or",[t1;t2]) ->
	    elim (ASTnvar i) THEN clear_hyp [i] THEN intros_with [i] THENS 
              [ loop evbd (i::li) (t1::lt); loop evbd (i::li) (t2::lt) ]
        | Kapp("and",[t1;t2]) ->
	    let i1 = id_of_string (string_of_id i ^ "_left") in
 	    let i2 = id_of_string (string_of_id i ^ "_right") in
	    elim (ASTnvar i) THEN clear_hyp [i] THEN intros_with [i1;i2] THEN
	    loop (i1::i2::evbd) (i1::i2::li) (t1::t2::lt)
        | Kimp(t1,t2) ->
            generalize_tac [mkc "imp_simp" [t1;t2;decidabilite gl t1;VAR i]]
            THEN clear_hyp [i] THEN intros_with [i] THEN 
            loop evbd (i::li) (mk_or (mk_not t1) t2 :: lt)

        | Kapp("not",[t]) -> 
      	    begin match destructurate t with 
	       Kapp("or",[t1;t2]) -> 
	         generalize_tac [mkc "not_or" [t1;t2;VAR i]] THEN
		 clear_hyp [i] THEN intros_with [i] THEN 
                 loop evbd (i::li) (mk_and (mk_not t1) (mk_not t2) :: lt)
             | Kapp("and",[t1;t2]) -> 
	         generalize_tac
                   [mkc "not_and" [t1;t2;decidabilite gl t1;VAR i]]
                 THEN clear_hyp [i] THEN intros_with [i] THEN 
                 loop evbd (i::li) (mk_or (mk_not t1) (mk_not t2) :: lt)
             | Kimp(t1,t2) ->
	         generalize_tac 
                   [mkc "not_imp" [t1;t2;decidabilite gl t1;VAR i]]
		 THEN clear_hyp [i] THEN intros_with [i] THEN
                 loop evbd (i::li) (mk_and t1 (mk_not t2) :: lt)
	     | Kapp("not",[t]) ->
	         generalize_tac
                   [mkc "not_not" [t;decidabilite gl t;VAR i]]
                 THEN clear_hyp [i] THEN intros_with [i] THEN 
                 loop evbd (i::li) (t :: lt)
             | Kapp(("Zle"|"Zge"|"Zlt"|"Zgt"|"le"|"ge"|"lt"|"gt") as name,
                    [t1;t2]) ->
	         generalize_tac [mkc ("not_" ^ name) [t1;t2;VAR i]]
                 THEN clear_hyp [i] THEN intros_with [i] THEN loop evbd li lt
             | Kapp("eq",[typ;t1;t2]) ->
	         begin match destructurate (pf_nf gl typ) with
		    Kapp("nat",_) -> 
		      elim (ast_app "not_eq" [t1;t2;VAR i]) THEN
                      clear_hyp [i] THEN intros_with [i] THEN loop evbd li lt
                  | Kapp("Z",_) ->
		      elim (ast_app "not_Zeq" [t1;t2;VAR i]) THEN
                      clear_hyp [i] THEN intros_with [i] THEN loop evbd li lt
                  | _ -> loop evbd li lt
                 end
             | _ -> loop evbd li lt
            end 
        | _ -> loop evbd li lt 
      with UserError _  -> loop evbd li lt
      end 
    | _ _ -> error "destructurate_hyps" in
  loop (fst hyp) (fst hyp) (snd hyp) gl;;

(* DESTRUCTURATE THE GOAL : THIS IS THE SOLVER *)

let destructure_goal gl =
  let concl = pf_concl gl in
  let rec loop t =
    match destructurate t with
       Kapp("not",[t1;t2]) -> 
         unfold "not" THEN intro THEN destructure_hyps
     | Kimp(a,b) -> intro THEN loop b
     | Kapp("False",[]) -> destructure_hyps
     | _ ->
        exact (mkc "dec_not_not" [t;decidabilite gl t;mk_meta()])
        THEN intro THEN destructure_hyps in
  (loop concl) gl;;

let omega_solver gl = 
 let result = all_time destructure_goal gl in 
 if !display_time_flag then begin text_time (); flush std_out end;
 result;;

register_atomic_tactic "Omega" omega_solver;;
