
#open "unix";;

let zero_time = 
  {tms_utime = 0.0; tms_stime = 0.0; tms_cutime = 0.0; tms_cstime = 0.0};;

(* operations on time limited to the current process *)
let sub_time {tms_utime = tu; tms_stime = ts; _}
      	     {tms_utime = tu'; tms_stime = ts'; _} =
  {tms_utime = tu -. tu'; tms_stime = ts -. ts';
   tms_cutime = 0.0; tms_cstime = 0.0};;

let add_time {tms_utime = tu; tms_stime = ts; _}
      	     {tms_utime = tu'; tms_stime = ts'; _} =
  {tms_utime = tu +. tu'; tms_stime = ts +. ts';
   tms_cutime = 0.0; tms_cstime = 0.0};;

let (push_elapsed,pop_elapsed,add_elapsed,clear_elapsed) =
  let elapsed_time = ref [] in
  (fun () -> 
     elapsed_time := 
       {tms_utime = 0.0; tms_stime = 0.0; tms_cutime = 0.0; tms_cstime = 0.0}
       :: !elapsed_time),
  (fun () ->
     match !elapsed_time
     with (hd :: tl) -> elapsed_time := tl; hd
        | [] -> failwith "pop_elapsed"),
  (fun t ->
     match !elapsed_time
     with (hd :: tl) -> elapsed_time := (add_time hd t) :: tl
        | [] -> ()),
  (fun () -> elapsed_time := [])
;;

let memory1 = ref (times ());;
let memory2 = ref (!memory1);;

let timing_count = ref 0;;

let table = (hashtbl__new 127 : (string,process_times ref) hashtbl__t);;

(*
let timing timer_name f arg =
  let t1 = times () in
  incr timing_count;
  let cpt1 = !timing_count in
  incr timing_count;
  add_elapsed (sub_time (!memory2) (!memory1));
  memory2 := !memory1;
  push_elapsed ();
  let t2 = times () in
  try (
    let r = f arg in
    let t3 = times () in
    add_elapsed (sub_time (!memory2) (!memory1));
    memory1 := t3;
    let f_elapsed = pop_elapsed () in
    let nb_timing = !timing_count - cpt1 in
    let time_of_f = sub_time (sub_time t3 t2) f_elapsed in
    (try
      let (time_db,nb_timing',nb) = hashtbl__find table timer_name in
      hashtbl__remove table timer_name;
      hashtbl__add table timer_name 
        (add_time time_db time_of_f,nb_timing + nb_timing', nb + 1)
    with Not_found -> hashtbl__add table timer_name (time_of_f,nb_timing,1));
    add_elapsed (add_time (sub_time t2 t1) f_elapsed);
    memory2 := times ();
    r)
  with erreur -> (
    let t3 = times () in
    add_elapsed (sub_time (!memory2) (!memory1));
    memory1 := t3;
    let f_elapsed = pop_elapsed () in
    let nb_timing = !timing_count - cpt1 in
    let time_of_f = sub_time (sub_time t3 t2) f_elapsed in
    (try
      let time_db,nb_timing',nb = hashtbl__find table timer_name in
      hashtbl__remove table timer_name;
      hashtbl__add table timer_name
        (add_time time_db time_of_f,nb_timing + nb_timing', nb + 1)
    with Not_found -> hashtbl__add table timer_name (time_of_f,nb_timing,1));
    add_elapsed (add_time (sub_time t2 t1) f_elapsed);
    memory2 := times ();
    raise erreur)
;;
*)

let rec timing timer_name = 
  let tc = try hashtbl__find table timer_name
           with Not_found -> (
	     let aux = ref zero_time
	     in hashtbl__add table timer_name aux; aux) in
  let rec do_it f arg =
    let x = times() in 
    try (let r = f arg in
         let y = times() in
         tc := add_time !tc (sub_time y x);r)
    with exn -> (
          let y = times () in
	  tc := add_time !tc (sub_time y x);raise exn) in
  do_it;;

let text_time () =
  hashtbl__do_table
    (fun title (ref {tms_utime=tu;tms_stime=ts;_}) ->
      printf__printf "%s :\ttu=%.2f ts=%.2f\n" title tu ts)
    table;
  hashtbl__do_table (fun x r -> r := zero_time) table;;
