(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Ensf_dans.v                                *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


Require Ensf_types.

(*                                                                    *)
(*  APPARTENANCE :                                                    *)
(*    On definit le predicat (dans x E) pour un element x et un       *)
(*    ensemble E.                                                     *)
(*                                                                    *)
 
Inductive dans : Elt -> Ensf -> Prop :=
  dans_add1 : (x:Elt)(e:Ensf) (dans x (add x e))
| dans_add2 : (x,y:Elt)(e:Ensf) (dans x e) -> (dans x (add y e)).
Hint dans_add1 dans_add2.
 
(*  Quelques resultats concernant l'appartenance...  *)
 
Lemma dans_add : (x,y:Elt)(e:Ensf)
   (dans x (add y e)) -> ( (y=x) \/ (dans x e) ).
Intros x y e H.
Simple Inversion H.
Left.
Injection H1.
Intros.
Apply trans_equal with x0 ; [Auto | Assumption].

Intro.
Right.
Injection H2.
Intros.
Rewrite <- H3.
Rewrite <- H1.
Assumption.
Save. 
 
Lemma dans_add_contr : (x,y:Elt)(e:Ensf)
  ~(y=x) -> ~(dans x e) -> ~(dans x (add y e)).
Intros; Red; Intro.
Absurd ( y=x \/ (dans x e)).
2:Apply dans_add; Auto.
Red.
Intro.
Elim H2; Auto.
Save.
 
Lemma empty_empty : (E:Elt)~(dans E empty).
Unfold not; Intros E H.
Simple Inversion H; [Discriminate  H1 | Discriminate  H2].
Save.
Hint empty_empty.
 
Lemma dans_empty_imp_P : (x:Elt)(P:Prop) (dans x empty)->P.
Intros.
ElimType False.
Cut ~(dans x empty); Auto.
Save.
 
Lemma singl2 : (x:Elt) (dans x (singleton x)).
Unfold singleton.
Auto.
Save.
Hint singl2.

Lemma singl2_inv : (x,e:Elt)
  (dans x (singleton e)) -> <Elt>x=e.
Unfold singleton.
Intros x e H.
Simple Inversion H.
Injection H1; Intros.
Rewrite <- H0; Assumption.

Injection H2; Intros.
Apply dans_empty_imp_P with x0.
Rewrite <- H0; Assumption.
Save.
Hint singl2_inv.
