(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Ensf_disj.v                                *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


Require Ensf_types.
Require Ensf_dans.
Require Ensf_union.
Require Ensf_couple.
Require Ensf_inclus.
Require Ensf_map.

(*									*)
(*  UNION DISJOINTE 							*)
(*    L'union disjointe de A et B est definie comme l'ensemble des 	*)
(*    des couples (x,zero) pour x dans A et (x,un) pour x dans B	*)
(*    Pour cela on definit 2 fonctions injgauche : x->(x,zero) et	*)
(*    injdroite : x->(x,un) et on fait l'union de (map injgauche A)	*)
(*    et de (map injdroite B).						*)
(*									*)

Definition injgauche : Elt -> Elt :=
  [e:Elt](couple e zero).

Definition injdroite : Elt -> Elt :=
  [e:Elt](couple e un).

Definition union_disj : Ensf -> Ensf -> Ensf :=
  [e,f:Ensf](union (map injgauche e) (map injdroite f)).


Lemma  dans_map_injg : (e:Ensf)(x:Elt)
  (dans x (map injgauche e)) -> (dans (first x) e).
Intros.
Cut (Ex [y:Elt]( (dans y e) /\ x=(injgauche y) )).
2:Apply dans_map; Auto.
Intro Ht; Elim Ht; Clear Ht.
Intros y Ht; Elim Ht; Clear Ht.
Intros.  
Unfold injgauche in H1.
Replace (first x) with y; Auto.
Symmetry.
Replace y with (first (couple y zero)); Auto.
Apply (f_equal Elt Elt); Auto.
Save.

Lemma dans_map_injd : (e:Ensf)(x:Elt)
  (dans x (map injdroite e)) -> (dans (first x) e).
Intros.
Cut (Ex [y:Elt]( (dans y e) /\ x=(injdroite y) )).
2:Apply dans_map; Auto.
Intro Ht; Elim Ht; Clear Ht.
Intros y Ht; Elim Ht; Clear Ht.
Intros.  
Unfold injdroite in H1.
Replace (first x) with y; Auto.
Symmetry.
Replace y with (first (couple y un)); Auto.
Apply (f_equal Elt Elt); Auto.
Save.

Lemma absurd_injg_injd : (x:Elt)(e,f:Ensf)
  (dans x (map injgauche e)) -> ~(dans x (map injdroite f)).
Intros.
Cut (Ex ([y:Elt]( (dans y e) /\ x=(injgauche y) ))).
2:Apply dans_map; Auto.
Intro Ht; Elim Ht; Clear Ht.
Intros y Ht; Elim Ht; Clear Ht.
Intros.  
Red.
Intro.
Cut (Ex [y':Elt]( (dans y' f) /\ <Elt>x=(injdroite y') )).
2:Apply dans_map; Auto.
Intro Ht; Elim Ht; Clear Ht.
Intros y' Ht; Elim Ht; Clear Ht.
Intros.  
Absurd <Elt>zero=un.
Unfold zero.
Unfold un.
Discriminate .

Unfold injdroite in H4.
Unfold injgauche in H1.
Replace zero with (second (couple y zero)); Auto.
Replace un with (second (couple y' un)); Auto.
Rewrite <- H4.
Rewrite <- H1.
Auto.
Save.

(*									*)
(*  On montre ici que si x est dans l'union disjointe de A et B alors	*)
(*  x est soit de la forme (injgauche y) avec y dans A, soit de la 	*)
(*  forme (injdroite y) avec y dans B					*)
(*									*)

Lemma union_disj1 : (x:Elt)(a,b:Ensf)
  (dans x (union_disj a b)) -> (
      (Ex [y:Elt]  ( (dans y a) /\ (<Elt>x=(injgauche y)) ))
   \/ (Ex [y:Elt]  ( (dans y b) /\ (<Elt>x=(injdroite y)) ))
  ).
Unfold union_disj.
Intros.
Cut ( (dans x (map injgauche a)) \/ (dans x (map injdroite b)) ).
2:Auto.
Intro H0; Elim H0; Clear H0.
Intro; Left.
Apply dans_map; Auto.
Intro; Right.
Apply dans_map; Auto.
Save.

Lemma union_disj_d : (x:Elt)(a,b:Ensf)
  (dans x b) -> (dans (injdroite x) (union_disj a b)).
Intros.
Unfold union_disj.
Apply union_d.
Apply dans_map_inv.
Auto.
Save.

Lemma union_disj_g : (x:Elt)(a,b:Ensf)
  (dans x a) -> (dans (injgauche x) (union_disj a b)).
Intros.
Unfold union_disj.
Apply union_g.
Apply dans_map_inv.
Auto.
Save.

Lemma inclus_union_disj : (a,b,c,d:Ensf)
  (inclus a c) -> (inclus b d) 
    -> (inclus (union_disj a b) (union_disj c d)).
Unfold inclus.
Intros.
Unfold union_disj in H1.
Cut ( (dans x (map injgauche a)) \/ (dans x (map injdroite b)) ); Auto.
Intro Ht; Elim Ht; Clear Ht.

Intro.
Cut (Ex [y:Elt]( (dans y a) /\ x=(injgauche y) )).
2:Apply dans_map; Auto.
Intro Ht; Elim Ht; Clear Ht.
Intros y Ht; Elim Ht; Clear Ht; Intros H3 H4.
Cut (dans y c); Auto.
Intro.
Unfold union_disj.
Apply union_g.
Rewrite H4.
Apply dans_map_inv; Auto.

Intro.
Cut (Ex [y:Elt]( (dans y b) /\ <Elt>x=(injdroite y) )).
2:Apply dans_map; Auto.
Intro Ht; Elim Ht; Clear Ht.
Intros y Ht; Elim Ht; Clear Ht; Intros H3 H4.
Cut (dans y d); Auto.
Intro.
Unfold union_disj.
Apply union_d.
Rewrite H4.
Apply dans_map_inv; Auto.
Save.

(* 									*)
(*  Resultats n'ayant rien a voir avec les ensembles finis mais n'ayant	*)
(*  pas de place dans un fichier particulier.				*)
(*									*)

Lemma pair_equal : (A,B:Set)(x,x':A)(y,y':B)
  (<A>x=x') -> (<B>y=y') -> (<A*B>(<A,B>(x,y))=(<A,B>(x',y'))).
Intros A B x x' y y' X Y.
Rewrite X.
Rewrite Y.
Apply refl_equal.
Save.
Hint pair_equal.
