(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              Ensf_inclus.v                               *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


Require Ensf_types.

Require Ensf_dans.
Require Ensf_union.
Require Ensf_produit.

(*									*)
(*  INCLUSION  								*)
(*    On definit le predicat (inclus E F) par (dans x E)->(dans x F).	*)
(*    On montre ensuite facilement les resultats suivants :		*)
(*    -- inclus empty A							*)
(*    -- inclus A A							*)
(*    -- (inclus a b)->(inclus c d)					*)
(*		->(inclus (prodcart a c) (prodcart b d)			*)
(*									*)

Definition inclus : Ensf -> Ensf -> Prop :=
  [A,B:Ensf]
    (x:Elt) (dans x A) -> (dans x B).

Hint Unfold inclus.

Lemma empty_inclus : (x : Ensf) (inclus empty x).
Unfold inclus; Intros.
Absurd (dans x0 empty); Auto.
Save.
Hint empty_inclus.

Lemma refl_inclus : (x : Ensf)(inclus x x ).
Auto.
Save.
Hint refl_inclus.

Lemma inclus_trans : (a,b,c:Ensf)
  (inclus a b) -> (inclus b c) -> (inclus a c).
Auto.
Save.

Lemma cart_inclus : (a,b,c,d : Ensf)
    (inclus a b) -> (inclus c d) -> (inclus (prodcart a c ) (prodcart b d)).
Unfold inclus.
Intros.
Cut (Ex ([x1:Elt](Ex ([x2:Elt](
	   (dans x1 a)
	/\ (dans x2 c)
	/\ x=(couple x1 x2)
    ))))).
2:Apply coupl3; Auto.
Intro H2; Elim H2; Clear H2.
Intros x1 H2; Elim H2; Clear H2.
Intros x2 H2; Elim H2; Clear H2.
Intros H2 H3; Elim H3; Clear H3.
Intros H3 H4.
Rewrite H4.
Auto.
Save.
Hint cart_inclus.

Lemma inclus_add : (a,b:Ensf)(y:Elt)
  (inclus a b) -> (inclus a (add y b)).
Auto.
Save.
Hint inclus_add.

Lemma singl_inclus_add : (e:Elt)(a:Ensf)
  (inclus (singleton e) (add e a)).
Unfold inclus.
Intros e a x H.
Cut x=e; Auto.
Intro H0.
Rewrite H0; Auto.
Save.
Hint singl_inclus_add.

Lemma inclus_singl : (e:Elt)(a:Ensf)
  (inclus (singleton e) a) -> (dans e a).
Auto.
Save.


Lemma add_inclus : (x:Elt)(a,b:Ensf)
  (dans x b) -> (inclus a b) -> (inclus (add x a) b).
Unfold inclus.
Intros.
Cut ( x=x0 \/ (dans x0 a) ).
2:Apply dans_add; Auto.
Intro H2; Elim H2; Clear H2.
Intro H2; Rewrite <- H2; Auto.
Auto.
Save.
Hint add_inclus.

Lemma dans_trans : (x:Elt)(a,b:Ensf)
  (dans x a) -> (inclus a b) -> (dans x b).
Auto.
Save.

Lemma union_inclus : (a,b,c:Ensf)
  (inclus a c) -> (inclus b c) -> (inclus (union a b) c).
Unfold inclus.
Intros.
Cut ( (dans x a) \/ (dans x b) ); Auto.
Intro H2; Elim H2; Auto.
Save.
Hint union_inclus.

Lemma inclus_g : (a,b:Ensf)
  (inclus a (union a b)).
Auto.
Save.

Lemma inclus_d : (a,b:Ensf)
  (inclus b (union a b)).
Auto.
Save.

Lemma inclus_g2 : (A,B,C:Ensf)
  (inclus A B) -> (inclus A (union B C)).
Auto.
Save.
Hint inclus_g2.

Lemma inclus_d2 : (A,B,C:Ensf)
  (inclus A C) -> (inclus A (union B C)).
Auto.
Save.
Hint inclus_d2.

