(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Ensf_inter.v                               *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


Require Ensf_types.
Require Ensf_dans.
Require Ensf_union.
Require Ensf_inclus.

(*									*)
(*  INTERSECTION 							*)
(*   L'intersection de 2 ensembles est definie comme un predicat sur    *)
(*   3 ensembles A B C en disant que C est l'intresection de A et B     *)
(*   si C est le plus grand ensemble inclus dans A et dans B		*)
(*									*)

Definition inter : Ensf -> Ensf -> Ensf -> Prop :=
  [A,B:Ensf][C:Ensf]
    (inclus C A)
  /\(inclus C B)
  /\((x:Elt) (dans x A)->(dans x B)->(dans x C)).


Lemma union_inter : (a,b,c:Ensf)
  (inter a b empty) -> (inter a c empty)
   -> (inter a (union b c) empty).
Unfold inter.
Intros.
Elim H0; Clear H0.
Intros H0 H1; Elim H1; Clear H1; Intros H1 H2.
Elim H; Clear H.
Intros H3 H4; Elim H4; Clear H4; Intros H4 H5.
Split; Auto.
Split.
Apply empty_inclus.
Intros.
Cut ( (dans x b) \/ (dans x c) ); Auto.
Intro H7; Elim H7; Auto.
Save.

Lemma inter_union : (A,B,C:Ensf)
  (inter A C empty) -> (inter B C empty)
   -> (inter (union A B) C empty).
Unfold inter.
Intros.
Elim H0; Clear H0.
Intros H0 H1; Elim H1; Clear H1; Intros H1 H2.
Elim H; Clear H.
Intros H3 H4; Elim H4; Clear H4; Intros H4 H5.
Split; Auto.
Split; Auto.
Intros.
Cut ( (dans x A) \/ (dans x B) ); Auto.
Intro H7; Elim H7; Auto.
Save.

Lemma inter_dans : (A,B:Ensf)(x:Elt)
  (inter A B empty)
  -> (dans x A)
  -> ~(dans x B).
Unfold inter.
Intros.
Elim H; Clear H; Intros H Ht; Elim Ht; Clear Ht; Intros H1 H2.
Red; Intro.
Cut (dans x empty); Auto.
Intro.
Apply dans_empty_imp_P with x; Auto.
Save.

Lemma sym_inter : (A,B,C:Ensf)
  (inter A B C)
  -> (inter B A C).
Unfold inter.
Intros.
Elim H; Clear H; Intros H Ht; Elim Ht; Clear Ht; Intros H0 H1.
Auto.
Save.
