(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                Ensf_map.v                                *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


Require Ensf_types.
Require Ensf_dans.
Require Ensf_union.
Require Ensf_inclus.

(*									*)
(*  MAP "a la CAML"							*)
(*    On definit une fonction map qui applique une fonction a tous les  *)
(*    elements d'un ensemble et renvoie l'ensemble des resultats	*)
(*    Ceci permet, entre autres, de definir facilement l'union		*)
(*    disjointe (Voir ci-dessous)					*)
(*									*)

Fixpoint map [f:(Elt->Elt); e:Ensf] : Ensf :=
    (<Ensf>Case e of
        (* empty *) empty
        (* ad    *) [y:Elt][e:Ensf] (add (f y) (map f e)) 
    end ).

(*									*)
(*  On montre ici le resultat general suivant :				*)
(*    (dans  x (map f A)) -> il existe y dans A tel que x=f y		*)
(*									*)

Lemma dans_map : (f:Elt->Elt)(a:Ensf)(x:Elt)
  (dans x (map f a)) -> (Ex [y:Elt] ((dans y a) /\ (x=(f y))	
  )).
Intros f.
Induction a.
Simpl.
Intros x H.
Absurd (dans x empty); Auto.

Intros a0 b H x.
Simpl.
Intro.
Cut ( <Elt>(f a0)=x \/ (dans x (map f b)) ).
2:Apply dans_add; Auto.
Intro H1; Elim H1; Clear H1.
Intro; Exists a0; Auto.
Intro.
Cut (Ex [y:Elt](dans y b) /\ (x=(f y))).
Intro H2; Elim H2; Clear H2.
2:Auto.
Intros x0 H2; Elim H2; Clear H2.
Intros.
Exists x0.
Split; Auto.
Save.
Hint dans_map.

Lemma dans_map_inv : (f:Elt->Elt)(x:Elt)(a:Ensf)
  (dans x a) -> (dans (f x) (map f a)).
Intros f x.
Induction a.
Intro.
Apply (dans_empty_imp_P x); Auto.

Intros a0 b H.
Simpl.
Intro H1.
Cut  (<Elt>a0=x \/ (dans x b) ).
2:Apply dans_add; Auto.
Intro H2; Elim H2; Clear H2.
Intro.
Rewrite H0; Auto.
Auto.
Save.
Hint dans_map_inv.

Lemma map_union : (f:Elt->Elt)(a,b:Ensf)
  (<Ensf>(union (map f a) (map f b)) = (map f (union a b)) ).
Intro f.
Induction a; Simpl; Auto.
Save.
Hint map_union.

Lemma dans_map_trans : (x:Elt)(f:Elt->Elt)(a,b:Ensf)
  (dans x (map f a))
  -> (inclus a b)
  -> (dans x (map f b)).
Intros.
Cut (Ex [y:Elt]( (dans y a) /\ <Elt>x=(f y) )).
2:Apply dans_map; Auto. 
Intro Ht; Elim Ht; Clear Ht.
Intros y Ht; Elim Ht; Clear Ht.
Intros.
Cut (dans y b).
2:Apply dans_trans with a; Auto.
Intro.
Rewrite H2.
Apply dans_map_inv; Auto.
Save.

Lemma map_egal : (f,g:Elt->Elt)(E:Ensf)
  ((x:Elt)(dans x E)->(<Elt>(f x)=(g x)))
     -> (<Ensf>(map f E)=(map g E)).
Intros f g.
Induction E; Simpl; Auto.
Save.

Lemma map_inclus : (a,b:Ensf)(f:Elt->Elt)
  (inclus a b) -> (inclus (map f a) (map f b)).
Unfold inclus.
Intros.
Cut (Ex [y:Elt]( (dans y a) /\ <Elt>x=(f y) )).
2:Apply dans_map; Auto.
Intro Ht; Elim Ht; Clear Ht; Intros y Ht; Elim Ht; Clear Ht; Intros.
Cut (dans y b); Auto.
Intro.
Replace x with (f y); Auto.
Save.
