(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              Ensf_produit.v                              *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


Require Ensf_types.
Require Ensf_dans.
Require Ensf_union.
Require Ensf_couple.

(*									*)
(*  PRODUIT CARTESIEN 							*)
(*    On definit ici le produit cartesien de 2 ensembles. Pour cela     *)
(*    on commence par definir (singleprod x E) qui est l'ensemble	*)
(*    des couples (x,y) pour y dans E, puis (prodcart E F) qui est	*)
(*    l'union des (singleprod x F) pour x dans E.			*)
(*									*)

Fixpoint singleprod [x:Elt; A:Ensf] : Ensf :=
  (<Ensf>Case A of
        (* empty *) empty
        (* add   *) [y:Elt][e:Ensf] (add (couple x y) (singleprod x e)) 
  end ).

Fixpoint prodcart [A:Ensf] : Ensf -> Ensf :=
  [B:Ensf]
  (<Ensf>Case A of
	(* empty *) empty
        (* add   *) [x:Elt][e:Ensf] (union (singleprod x B) (prodcart e B))
  end ).

(*  On montre en premier que si (x,y) est dans (singleprod x0 b) alors	*)
(*  on a x=x0 et y dans b						*)

Lemma dans_singleprod : (x,y,x0:Elt)(b:Ensf)
  (dans (couple x y) (singleprod x0 b)) -> ( (<Elt>x=x0) /\ (dans y b) ).
Intros x y x0.
Induction b.
Simpl.
Intro.
Apply (dans_empty_imp_P (couple x y)); Auto.

Intros a b0 H.
Simpl.
Intro.
Cut ( <Elt>(couple x0 a)=(couple x y) \/ (dans (couple x y) (singleprod x0 b0)) ).
2:Apply dans_add; Auto.
Intro H1; Elim H1; Clear H1.
Intro H1.
Injection  H1; Intros.
Split; Auto.
Rewrite H2; Auto.

Intro.
Cut ( <Elt>x=x0 /\ (dans y b0) ); Auto.
Intro H2; Elim H2; Clear H2.
Intros.
Split; Auto.
Save.

(*  On peut ensuite en deduire que si (x,y) est dans AxB alors 		*)
(*  x est dans A et y est dans B.					*)
 
Lemma coupl2 : (x,y:Elt)(a,b:Ensf)
  (dans (couple x y) (prodcart a b)) -> ((dans x a) /\ (dans y b)).
Intros x y.
Induction a.
Intro b.
Simpl.
Intro.
Apply (dans_empty_imp_P (couple x y)); Auto.

Intros a0 b H b0.
Simpl.
Intro.
Cut ( (dans (couple x y) (singleprod a0 b0)) \/ (dans (couple x y) (prodcart b b0)) ).
2:Apply dans_union; Auto.
Intro H1; Elim H1; Clear H1.
Intro H1.
Cut ( <Elt>x=a0 /\ (dans y b0) ).
2:Apply dans_singleprod; Auto.
Intro H2; Elim H2; Clear H2.
Intros.
Rewrite H2.
Split; Auto.
Intro.
Cut ( (dans x b) /\ (dans y b0) ); Auto.
Intro H2; Elim H2; Clear H2.
Intros.
Split; Auto.
Save.

(*  Plus facile : l'inverse...						*)

Lemma dans_single : (x,y:Elt)(a:Ensf)
  (dans y a) -> (dans (couple x y) (singleprod x a)).
Intros x y.
Induction a.
Intro.
Apply (dans_empty_imp_P y); Auto.
Intros a0 b H H1.
Cut ( <Elt>a0=y \/ (dans y b) ).
2:Apply dans_add; Auto.
Intro H2; Elim H2; Clear H2.
Intro.
Simpl.
Rewrite H0; Auto.
Simpl; Auto.
Save.

Lemma coupl2_inv : (x,y:Elt)(a,b:Ensf)
  (dans x a) -> (dans y b) -> (dans (couple x y) (prodcart a b)).
Intros x y.
Induction a.
Intros b H.
Apply (dans_empty_imp_P x); Auto.

Intros a0 b H b0 H0.
Cut (<Elt>a0=x \/ (dans x b) ).
2:Apply dans_add; Auto.
Simpl.
Intro H1; Elim H1; Clear H1.
Intros H1 H2.
Apply dans_union_inv.
Left.
Rewrite H1.
Apply dans_single; Auto.
Intros H1 H2.
Apply dans_union_inv.
Right.
Auto.
Save.
Hint coupl2_inv.

(*  De meme on commence ici par monter que si x est dans		*)
(*  (singleprod x0 b) alors x est de la forme (x0,y) avec y dans b	*)

Lemma dans_singleprod2 : (x,x0:Elt)(b:Ensf)
  (dans x (singleprod x0 b)) -> (Ex [y:Elt](x=(couple x0 y) /\ (dans y b)
  )).
Intros x x0.
Induction b.
Intro.
Apply (dans_empty_imp_P x); Auto.
Intros a b0 H.
Simpl.
Intro.
Cut ( <Elt>(couple x0 a)=x \/ (dans x (singleprod x0 b0)) ).
2:Apply dans_add; Auto.
Intro H1; Elim H1; Clear H1.
Intro.
Exists a; Auto.
Intro.
Cut (Ex [y:Elt](x=(couple x0 y) /\ (dans y b0))); Auto.
Intro H2; Elim H2; Clear H2.
Intros.
Exists x1.
Elim H2; Clear H2.
Intros.
Split; Auto.
Save.

(*  On peut ensuite en deduire que si x est dans AxB alors x est de la	*)
(*  forme (x1,x2) avec x1 dans A et x2 dans B.				*)

Lemma coupl3 : (a,b:Ensf)(x:Elt) 
   (dans x (prodcart a b)) ->
     (Ex [x1:Elt] (Ex ([x2:Elt]
         ((dans x1 a)
          /\ (dans x2 b)
          /\ (x=(couple x1 x2)))))).
Induction a.
Intro b.
Simpl.
Intros x H.
Apply (dans_empty_imp_P x); Auto.

Intros a0 b H b0 x.
Simpl.
Intro.
Cut ( (dans x (singleprod a0 b0)) \/ (dans x (prodcart b b0)) ); Auto.
Intro H1; Elim H1; Clear H1.
Intro.
Cut (Ex [y:Elt](x=(couple a0 y) /\ (dans y b0))).
2:Apply dans_singleprod2; Auto.
Intro H2; Elim H2; Clear H2.
Intros x0 H2.
Exists a0.
Exists x0.
Elim H2; Clear H2.
Intros.
Split; Auto.
Intro.
Cut  (Ex [x1:Elt] (Ex [x2:Elt]
         ((dans x1 b)
          /\ (dans x2 b0)
          /\ (x=(couple x1 x2))))); Auto.
Intro H2; Elim H2; Clear H2.
Intros x0 H2; Elim H2; Clear H2.
Intros x1 H2; Elim H2; Clear H2.
Intros H2 H3; Elim H3; Clear H3.
Intros H4 H5.
Exists x0.
Exists x1.
Split; Auto.
Save.
