(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Ensf_types.v                               *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


(*  On definit 3 "types" mutuellement inductifs : Elt, Ensf et Word      *)
(*  On distingue elt et mot, car on a besoin du type mot plus tard.      *)
(*  Les constructeurs up et word permettent repectivement de considerer  *)
(*  un ensemble ou un mot en tant qu'element.				 *)


Mutual Inductive Ensf : Set :=
   empty   : Ensf
 | add     : Elt -> Ensf -> Ensf

with Elt : Set :=
   natural : nat -> Elt
 | couple  : Elt -> Elt -> Elt
 | up      : Ensf -> Elt
 | word    : Word -> Elt

with Word : Set :=
   nil     : Word
 | cons    : Elt -> Word -> Word.


(*  Inversion de quelques constructeurs...  *)

Definition natural_inv : Elt -> nat :=
  [e:Elt]
    (<nat>Case e of
        (* natural *) [n:nat]n
        (* couple  *) [a:Elt][b:Elt]O
	(* up      *) [e:Ensf]O
	(* word    *) [w:Word]O
    end ).

Lemma nat_invol : (n:nat)(natural_inv (natural n))=n.
Auto.
Save.

Definition word_inv : Elt -> Word :=
  [e:Elt]
    (<Word>Case e of
        (* natural *) [n:nat]nil
        (* couple  *) [a:Elt][b:Elt]nil
	(* up      *) [e:Ensf]nil
	(* word    *) [w:Word]w
    end ).


(*  Quelques resultats triviaux sur les constructeurs...		*)

Lemma add_add : (a,b:Elt)(c,d:Ensf)
  (a=b) -> (c=d)  ->((add a c)=(add b d)).
Intros.
Rewrite H.
Rewrite H0.
Trivial.
Save.
Hint add_add.


Lemma couple_couple : (a,b,c,d:Elt)
  (a=b) -> (c=d) ->((couple a c)=(couple b d)).
Intros.
Rewrite H.
Rewrite H0.
Trivial.
Save.

Lemma word_word : (a,b:Word)  (a=b) -> ((word a)=(word b)).
Intros.
Apply (f_equal Word Elt); Auto.
Save.
Hint word_word.
 
Lemma word_word_inv : (a,b:Word) 
   ((word a)=(word b)) -> (a=b).
Intros a b H.
Injection  H.
Trivial.
Save.

(*  Quelques simplifications  *)

Definition zero : Elt := (natural O).
Definition un   : Elt := (natural (S O)).
Definition singleton : Elt -> Ensf :=
  [e:Elt] (add e empty).

(*  Quelques petits lemmes divers...  *)

Lemma False_imp_P : (P:Prop) False->P.
Intros.
ElimType False.
Assumption.
Save.

Lemma equal_add : (a,b:Ensf)(e:Elt)
  a=b -> (add e a)=(add e b).
Intros.
Apply (f_equal Ensf Ensf); Auto.
Save.
