(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Ensf_union.v                               *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)


Require Ensf_types.
Require Ensf_dans.

(*									*)
(*  UNION :								*)
(*    On definit ici l'union de 2 ensembles. On remarquera qu'un 	*)
(*    peut apparaitre plusieurs fois dans un ensemble, et que c'est     *)
(*    pour cela que l'on utilise 					*)
(*      union E (add x F) -> add x (union E F)				*)
(*    directement.							*)
(*									*)

Fixpoint union [A:Ensf] : Ensf -> Ensf :=
  [B:Ensf]
    (<Ensf>Case A of
        (* empty *) B
        (* add   *) [x:Elt][e:Ensf](add x (union e B))
    end).

Lemma union_a_empty : (a:Ensf) <Ensf>a=(union a empty).
Induction a.
Apply refl_equal.
Intros a0 b H.
Simpl; Auto.
Save.
Hint union_a_empty.


Lemma dans_union : (x:Elt)(a,b:Ensf)
  (dans x (union a b)) -> ( (dans x a) \/ (dans x b) ).
Intros x.
Induction a.
Auto.

Intros a0 b H b0.
Simpl.
Intro H0.
Cut ( <Elt>a0=x \/ (dans x (union b b0)) ).
2:Apply dans_add; Auto.
Intro H1; Elim H1.
Intro; Left.
Rewrite H2; Auto.
Intro.
Cut ((dans x b) \/ (dans x b0)); Auto.
Intro H3; Elim H3; Auto.
Save.
Hint dans_union.


Lemma union_g : (x:Elt)(a,b:Ensf)
  (dans x a) -> (dans x (union a b)).
Intro x.
Induction a.
Intros.
Apply (dans_empty_imp_P x); Auto.
Intros a0 b H b0.
Simpl.
Intro.
Cut ( (<Elt>a0=x) \/ (dans x b) ).
2:Apply dans_add; Auto.
Intro H1; Elim H1; Clear H1.
Intro H1.
Rewrite H1; Auto.
Auto.
Save.
Hint union_g.

Lemma union_d : (x:Elt)(a,b:Ensf)
  (dans x b) -> (dans x (union a b)).
Intro x.
Induction a; Simpl; Auto.
Save.
Hint union_d.


Lemma dans_union_inv : (x:Elt)(a,b:Ensf)
  ( (dans x a) \/ (dans x b) ) -> (dans x (union a b)).
Intros x a b H; Elim H; Auto.
Save.
