(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Max.v                                   *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)

Require Le.
Require Lt.
Require Ensf.


Fixpoint max [n:nat] : nat -> nat :=
  [m:nat] 
   (<nat>Case n of
	(* O    *) m
	(* S n' *) [n':nat](<nat>Case m of
			(* O    *) (S n')
			(* S m' *) [m':nat](S (max n' m'))
			    end)
     end).


Lemma max_SS : (n,m:nat)(S (max n m))=(max (S n) (S m)).
Auto.
Save.

Lemma le_max_l : (n,m:nat)(le n (max n m) ).
Induction n; Intros; Simpl; Auto.
Elim m; Intros; Simpl; Auto.
Save.
Hint le_max_l.

Lemma le_max_r : (n,m:nat)(le m (max n m) ).
Induction n; Simpl; Auto.
Induction m; Simpl; Auto.
Save.
Hint le_max_r.

Lemma lt_max_l : (x,y,z:nat) (lt x y) -> (lt x (max y z)).
Intros.
Apply lt_le_trans with y; Auto.
Save.
Hint lt_max_l.

Lemma lt_max_r : (x,y,z:nat) (lt x z) -> (lt x (max y z)).
Intros.
Apply lt_le_trans with z; Auto.
Save.
Hint lt_max_r.

(*  									*)
(*  Une propriete vraie pour n et m l'est aussi pour max(n,m)  		*)
(*									*)

Lemma max_case : (n,m:nat)(P:nat->Prop)(P n)->(P m)->(P (max n m)).
Induction n; Simpl; Auto.
Induction m; Intros; Simpl; Auto.
Pattern (max n0 n1); Apply H ; Auto.
Save.


(*									*)
(*  On definit maintenant (sup x) pour un ensemble x, qui est		*)
(*    - soit O, si x ne contient pas d'entier				*)
(*    - soit (S n), si n est le plus grand entier de x			*)
(*									*)

Definition Z : Elt -> nat :=
  [x:Elt]
    (<nat>Case x of
	(* natural *) [n:nat](S n)
	(* couple  *) [a:Elt][b:Elt]O
	(* up      *) [e:Ensf]O
	(* word    *) [w:Word]O
    end ).

Fixpoint sup [e:Ensf] : nat :=
    (<nat>Case e of
	(* empty *) O	
	(* add   *) [x:Elt][f:Ensf](max (Z x) (sup f))
    end ).

(*  Par definition on a :  *)

Lemma sup_add : (x:Elt)(e:Ensf) <nat>(sup (add x e))=(max (Z x) (sup e)).
Intros x.
Induction e; Auto.
Save.
Hint sup_add.

(*  Finalement inutile :  *)
(*
Lemma diff_natural : (n,m:nat)~(<nat>n=m)->~(<Elt>(natural n)=(natural m)).
Intros; Red; Intro.
Absurd (<nat>n=m).
Assumption.
Replace n with (natural_inv (natural n)).
2:Auto.
Replace m with (natural_inv (natural m)).
2:Auto.
Elim H0.
Auto.
Save.
*)

(*  Finalement inutile  *)
(*
Lemma lt_diff : (n,m:nat)(lt m n)->~(<nat>n=m).
Intros.
Red.
Intro.
Cut (lt m n); Auto.
Elim H0.
Change ~(lt n n).
Auto.
Save.
*)

Lemma elt_not_sym : (a,b:Elt) ~(<Elt>a=b)->~(<Elt>b=a).
Auto.
Save.

(*  (Z (natural n)) vaut (S n), donc est plus grand que n		*)

Lemma lt_n_Z : (n:nat) (lt n (Z (natural n))).
Intro.
Replace (Z (natural n)) with (S n); Auto.
Save.

(*									*)
(*  On montre d'abord que tout entier dans x est strictement plus petit	*)
(*  que (sup x)								*)
(*									*)

Lemma lt_n_sup : (x:Ensf)(n:nat) 
  (dans (natural n) x) -> (lt n (sup x)).
Induction x.
Intros.
Absurd (dans (natural n) empty); Auto.
Intros a b H n H0.
Replace (sup (add a b)) with (max (Z a) (sup b)).
2:Auto.
Cut ( (lt n (Z a)) \/ (lt n (sup b)) ).
Intro.
Elim H1; Auto.
Cut (<Elt>a=(natural n) \/ (dans (natural n) b) ).
2:Apply dans_add;Auto.
Intro.
Elim H1.
Intro; Left.
Rewrite H2; Apply lt_n_Z.
Intro; Right.
Apply H; Assumption.
Save.

(*								*)	
(*  On en deduit que (natural (sup x)) n'est pas dans x 	*)
(*								*) 

Lemma sup_out : (x:Ensf) ~(dans (natural (sup x)) x).
Intro.
Red.
Intro.
Cut (lt (sup x) (sup x)).
Change ~(lt (sup x) (sup x)).
Apply lt_n_n.
Apply lt_n_sup.
Assumption.
Save.

(*									   *)
(*  Le resultat final : 						   *)
(*      Pout tout ensemble e il existe un element x n'appartenant pas a e  *)
(*	(a savoir (natural (sup x)) )					   *)
(*									   *)

Lemma exist_other : (e:Ensf) (<Elt>Ex ([x:Elt] ~(dans x e))).
Intro.
Exists (natural (sup e)).
Apply sup_out.
Save.
