(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Words.v                                  *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)

Require Ensf.

Parameter alph : Ensf.
Parameter epsilon : Elt.
  Axiom not_dans_epsilon_alph : ~(dans epsilon alph).

(*  On definit le predicat (inmonoid X w) qui signifie que le mot w	*)
(*  est dans le monoide libre engendre par X.				*)

Inductive inmonoid [X:Ensf] : Word->Prop :=
  inmonoid_nil  :  (inmonoid X nil)
| inmonoid_cons : (w:Word)(e:Elt)
                    (inmonoid X w)->(dans e X)->(inmonoid X (cons e w)).
Hint inmonoid_nil.
Hint inmonoid_cons.

(*  Inversion de la definition 						*)

Fixpoint Inmonoid [X:Ensf; w:Word] : Prop :=
  (<Prop>Case w of
      (* nil  *) True
      (* cons *) [a:Elt][w':Word]( (dans a X) /\ (Inmonoid X w') )
  end ).

Lemma i_I : (X:Ensf)(w:Word)
   (inmonoid X w) -> (Inmonoid X w).
Intros X w H.
Elim H.
Red; Simpl; Exact I.
Intros.
Change ( (dans e X) /\ (Inmonoid X w0) ).
Auto.
Save.
Hint i_I.

Lemma I_i : (X:Ensf)(w:Word)
   (Inmonoid X w) -> (inmonoid X w).
Intros X.
Induction w.
Auto.
Intros x w0 H H0.
Cut ( (dans x X) /\ (Inmonoid X w0) ); Auto.
Intro H1; Elim H1; Clear H1.
Auto.
Save.
Hint I_i.

Lemma inmonoid_cons_inv : (X:Ensf)(w:Word)(a:Elt)
  (inmonoid X (cons a w)) -> (inmonoid X w).
Intros.
Cut (Inmonoid X w); Auto.
Cut (Inmonoid X (cons a w)); Auto.
Intro H0.
Cut ( (dans a X) /\ (Inmonoid X w) ); Auto.
Intro H1; Elim H1; Clear H1.
Auto.
Save.

Lemma inmonoid_cons_inv2 : (X:Ensf)(a:Elt)(w:Word)
  (inmonoid X (cons a w)) -> (dans a X).
Intros.
Cut (Inmonoid X (cons a w)); Auto.
Intro.
Cut ( (dans a X) /\ (Inmonoid X w)); Auto.
Intro H1; Elim H1; Clear H1.
Auto.
Save.

Lemma inmonoid_inclus : (E:Ensf)(F:Ensf)(x:Word)
  (inclus E F) -> (inmonoid E x) -> (inmonoid F x).
Intros E F x inclus_E_F inmonoid_E_x.
Elim inmonoid_E_x.
        Trivial.
 
        Intros w e inmonoid_E_w inmonoid_F_w dans_e_E.
        Apply inmonoid_cons; [Assumption | Apply inclus_E_F; Assumption].
Save.


(*									*)
(*  Concatenation de 2 mots : 						*)
(*    (Append w1 w2) est la concatenation de w1 et w2			*)
(*    (append w1 w2 w3) est la proposition "w3 est la conc.de w1 et w2"	*)
(*									*)

Fixpoint Append [w1:Word] : Word -> Word :=
  [w2:Word]
    (<Word>Case w1 of
	(* nil  *) w2
	(* cons *) [a:Elt][w3:Word](cons a (Append w3 w2))
    end ).

Lemma Append_w_nil : (w:Word) <Word>(Append w nil)=w.
Induction w.
Auto.
Intros x w0 H.
Replace (Append (cons x w0) nil) with (cons x (Append w0 nil)); Auto.
Rewrite H; Auto.
Save.

Inductive append : Word -> Word -> Word -> Prop :=
  append_nil  : (w:Word) (append nil w w)
| append_cons : (w1,w2,w3:Word)(a:Elt)
		    (append w1 w2 w3) -> (append (cons a w1) w2 (cons a w3)).


(*  Lemmes sur inmonoid et Append...					*)

Lemma Append_inmonoid_g : (X:Ensf)(w1,w2:Word)
  (inmonoid X (Append w1 w2)) -> (inmonoid X w1).
Intros X.
Induction w1.
Auto.
Intros x w H w2.
Replace (Append (cons x w) w2) with (cons x (Append w w2)); Auto.
Intro.
Apply inmonoid_cons.
Apply (H w2).
Apply inmonoid_cons_inv with x; Auto.
Apply inmonoid_cons_inv2 with (Append w w2); Auto.
Save.

Lemma Append_inmonoid_d : (X:Ensf)(w1,w2:Word)
  (inmonoid X (Append w1 w2)) -> (inmonoid X w2).
Intros X.
Induction w1.
Auto.
Intros x w H w2.
Replace (Append (cons x w) w2) with (cons x (Append w w2)); Auto.
Intro.
Apply (H w2).
Apply inmonoid_cons_inv with x; Auto.
Save.

Lemma inmonoid_Append : (X:Ensf)(w1,w2:Word)
  (inmonoid X w1) -> (inmonoid X w2) 
  -> (inmonoid X (Append w1 w2)).
Intros X.
Induction w1.
Auto.
Intros x w H w2 H0 H1.
Replace (Append (cons x w) w2) with (cons x (Append w w2)); Auto.
Apply inmonoid_cons.
Apply (H w2); Auto.
Apply inmonoid_cons_inv with x; Auto.
Apply inmonoid_cons_inv2 with w; Auto.
Save.


(*									*)
(*  On definit tout d'abord le type wordset, qui est Word->Prop		*)
(*  et qui definit un ensemble de mots par sa fonction caracteristique.	*)
(*  									*)
(*  L'egalite de 2 wordset est definie comme la double implication.	*)
(*									*)


Definition wordset := Word -> Prop.

Definition eqwordset : wordset->wordset->Prop := 
  [l1,l2 : wordset]
    (w : Word)
       (    ((l1 w)->(l2 w))
         /\ ((l2 w)->(l1 w))).

Lemma eqwordset_refl : (L:wordset) (eqwordset L L).
Red.
Auto.
Save.

Lemma eqwordset_sym : (l1,l2:wordset)
  (eqwordset l1 l2) -> (eqwordset l2 l1).
Unfold eqwordset.
Intros.
Elim (H w); Clear H; Intros; Auto.
Save.

Lemma eqwordset_trans : (l1,l2,l3:wordset)
  (eqwordset l1 l2) -> (eqwordset l2 l3) -> (eqwordset l1 l3).
Unfold eqwordset.
Intros.
Elim (H0 w); Clear H0; Intros.
Elim (H w); Clear H; Intros.
Auto.
Save.

(*									*)
(*  Le predicat islangage, defini sur les wordset, dit simplement	*)
(*  que les mots du wordset sont sur l'alphabet alph.			*)
(*									*)

Definition islanguage : Ensf -> wordset -> Prop :=
 [X:Ensf][L:wordset](w : Word)(L w)->(inmonoid X w).


(*									*)
(*  Extension aux mots d'une fonction definie sur les elements		*)
(*									*)

Fixpoint Word_ext [f : Elt -> Elt; w:Word] : Word :=
  (<Word>Case w of
	(* nil  *) nil
	(* cons *) [a:Elt][w':Word](cons (f a) (Word_ext f w'))
   end ).

Lemma inmonoid_map : (f:Elt->Elt)(a:Ensf)(w:Word)
  (inmonoid a w)->(inmonoid (map f a) (Word_ext f w)).
Intros.
Elim H ; [Unfold Word_ext ; Auto | Idtac].
Intros ; Unfold Word_ext ; Simpl.
Apply inmonoid_cons ; Try Apply dans_map_inv ; Auto.
Save.
Hint inmonoid_map.

(*  Un petit lemme bien utile par la suite...				*)

Lemma cons_cons : (x1,x2:Elt)(w1,w2:Word)
  <Elt>x1=x2 -> <Word>w1=w2 -> <Word>(cons x1 w1)=(cons x2 w2).
Intros.
Rewrite H0.
Rewrite H.
Auto.
Save.
Hint cons_cons.

Definition fun_consaw_a : Word -> Elt :=
  [w:Word](<Elt>Case w of
	(* nil  *) zero
	(* cons *) [a:Elt][w':Word]a
  end ).

Definition fun_consaw_w : Word -> Word :=
  [w:Word](<Word>Case w of
	(* nil  *) nil
	(* cons *) [a:Elt][w':Word]w'
  end ).

Lemma cons_cons_inv : (x1,x2:Elt)(w1,w2:Word)
  (cons x1 w1)=(cons x2 w2) -> (x1=x2 /\ w1=w2 ).
Intros.
Split.
Replace x1 with (fun_consaw_a (cons x1 w1)); Auto.
Replace x2 with (fun_consaw_a (cons x2 w2)); Auto.
Apply (f_equal Word Elt); Auto.
Replace w1 with (fun_consaw_w (cons x1 w1)); Auto.
Replace w2 with (fun_consaw_w (cons x2 w2)); Auto.
Apply (f_equal Word Word); Auto.
Save.

Hint cons_cons_inv.

Lemma cons_cons_inv1 : (x1,x2:Elt)(w1,w2:Word)
  <Word>(cons x1 w1)=(cons x2 w2) -> (<Elt>x1=x2).
Intros.
Cut (<Elt>x1=x2 /\ <Word>w1=w2 ) ; [Intuition | Auto].
Save.


Lemma cons_cons_inv2 : (x1,x2:Elt)(w1,w2:Word)
  (cons x1 w1)=(cons x2 w2) -> (w1=w2).
Intros.
Cut (x1=x2 /\ w1=w2 ) ;[ Intuition | Auto ].
Save.


(*									*)
(*  Un mot est soit nil, soit de la forme (cons x w0).			*)
(*									*)

Lemma nil_or_cons : (w:Word)
  (w=nil 
   \/  (Ex [x:Elt](<Word>Ex ([w0:Word](w=(cons x w0)))))).
Induction w.

Left; Auto.

Intros x w0 H.
Right.
Exists x.
Exists w0.
Auto.
Save.
