(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  gram.v                                  *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)

Require Ensf.
Require Words.
Require more_words.
Require need.
Require fonctions.
Require Relations.


Definition Mots := [X:Ensf](a:Elt)(dans a X)->(Ex [w:Word](word w)=a
							)
.


Definition Regles := [X,V,R:Ensf]
(x:Elt)(dans x R)->
		(Ex2 ([A:Elt](dans A V))
                     ([A:Elt](Ex2 ([B:Word]x=(couple A (word B)))
                                  ([B:Word](inmonoid (union X V) B)

					    )
			  )
		)).



Lemma Regles_inv1 : (X,V,R:Ensf)(x,y:Elt)(Regles X V R)->
(dans (couple x y) R)->(dans x V).
Intros X V R x y Regles_R dans_couple_R.
Cut 		(Ex2 ([A:Elt](dans A V))
                     ([A:Elt](Ex2 ([B:Word](couple x y)=(couple A (word B)))
                                  ([B:Word](inmonoid (union X V) B))))).
Intro temp;Elim temp;Clear temp.
Intros x0 dans_x0_V temp; Elim temp;Clear temp.
Intros u eg_couple inmonoid_u.
Replace x with x0 ; Prolog [sym_equal couple_couple_inv1] 3.
	(*Assumption.
	Apply sym_equal.
	Apply couple_couple_inv1 with y (word u); Assumption.*)

Auto.
Save.

Lemma Regles_inv2 : (X,V,R:Ensf)(x:Elt)(u:Word)(Regles X V R)->
(dans (couple x (word u)) R)->(inmonoid (union X V) u).
Intros X V R x u Regles_R dans_couple_R.
(**) Cut 	(Ex2 ([A:Elt](dans A V))

		     ([A:Elt](Ex2 ([B:Word]
                                   (couple x (word u))=(couple A (word B)))
                                  ([B:Word](inmonoid (union X V) B))

					    )
			  )
		).
Intro temp;Elim temp;Clear temp.
Intros x0 dans_x0_V temp; Elim temp;Clear temp.
Intros u0 eg_couple inmonoid_u0.
Replace u with u0 ; Prolog [sym_equal couple_couple_inv2 word_word_inv ] 4.
	(*Assumption.
	Apply word_word_inv.
	Apply couple_couple_inv2 with x0 x;Auto.*)
(**) Auto.
Save.


(* Definition d'une grammaire, *)
(*X : ensemble des terminaux, *)
(*V ensemble des non-terminaux, *)
(*R ensemble des productions A -> w, *)
(*S axiome *)

Definition isGram : Ensf -> Ensf -> Ensf -> Elt -> Prop := 

[X,V,R:Ensf][S:Elt]
(
	(Mots X)
	/\(inter X V empty)
	/\(dans S V)
	/\(Regles X V R)
)
.

Section Easy_lemma_isGram.

Variable X,V,R:Ensf.
Variable S:Elt.
Local H := (isGram X V R S).

Lemma isGram1 : H->(Mots X).
Intro H1.
Elim H1.
Trivial.
Save.

Lemma isGram2 : H->(inter X V empty).
Intro H1.
Elim H1.
Intuition.
Save.

Lemma isGram3 : H->(dans S V).
Intro H1.
Elim H1.
Intuition.
Save.


Lemma isGram4 : H->(Regles X V R).
Intro H1.
Elim H1.
Intuition.
Save.

Lemma isGram5 : (Mots X)->(inter X V empty)->(dans S V)->(Regles X V R)
->H.
Intros.
Red ; Red.
Auto.
Save.


End Easy_lemma_isGram.
(*--------*)

Lemma Regles_R : (X,V,R,R':Ensf)
(inclus R' R)->
(Regles X V R)
->(Regles X V R')
.
Unfold Regles.
Auto.
Save.

Lemma Regles_V : (X,V,R,V':Ensf)
(inclus V V')->
(Regles X V R)->
(Regles X V' R).
Unfold Regles.
Intros X V R V' inclus_V_V' Regles_X_V_R x dans_x_R.
Elim (Regles_X_V_R x dans_x_R).
Intros A dans_A_V temp; Elim temp;Clear temp.
Intros B egal_B inmonoid_B.
Exists A.
Auto.
Exists B.
Assumption.
Apply inmonoid_inclus with (union X V); Auto.
Save.



Lemma Regles_add : (X,V,R:Ensf)(a:Elt)(u:Word)
(Regles X V R)->(dans a V)->(inmonoid (union X V) u)->
(Regles X V (add (couple a (word u))
		R)).
Intros X V R a u R_R dans_a_V inmonoid_u_X_V_u.
Red.
Intros x dans_x_R'.
Cut (<Elt>(couple a (word u))=x)\/(dans x R).(**)
Intuition.
	Intro egal_x_couple.
	Exists a.
		Assumption.
		Exists u;Auto.

(**)Apply dans_add;Assumption.
Save.

Lemma Regles_add2 : (X,V,R:Ensf)(a:Elt)
(Regles X V R)->(Regles X (add a V) R).
Intros.
Apply Regles_V with V;Auto.
Save.


Lemma Regles_union : (X,V,R,R':Ensf)
(Regles X V R)->(Regles X V R')->
(Regles X V (union R R')).

Unfold Regles.
Intros X V R R' R_R R_R' x dans_x_union.
Cut (dans x R)\/(dans x R') ; Intuition.
Save.


Lemma isGram_inclus2 : (X,V,R,R':Ensf)(S:Elt)
(inclus R' R)->
(isGram X V R S)->
(isGram X V R' S).
Prolog [isGram4 Regles_R isGram3 isGram2 isGram1 isGram5] 11.

(*Intros X V R R' S incl isGram_X_V_R_S.
Apply isGram5 .
	Apply isGram1 with V R S; Assumption.
	Apply isGram2 with R S; Assumption.
	Apply isGram3 with X R; Assumption.
	Apply Regles_R with R.
	     Assumption.
	     Apply isGram4 with S; Assumption.*)
Save.

Lemma isGram_inclus3 : (X,V,R:Ensf)(S,a:Elt)
(isGram X V (add a R) S)->(isGram X V R S).
Intros X V R S a isGram_X_V_a_R_S.
Apply isGram_inclus2 with (add a R) ; Auto.
Save.


(*--------------------------*)



(* (Derive R u v) signifie "u se recrit en v par une production de R" *)
Inductive Derive [R:Ensf] : Word -> Word -> Prop :=

(*si A -R-> u alors Av -G-> uv *)
Derive1		: (u,v:Word)(A:Elt)(dans (couple A (word u)) R) 
			-> (Derive R (cons A v) (Append u v))

(*si u -G-> v alors x::u -G-> x::v*)
| Derive2		: (u,v:Word)(x:Elt)(Derive R u v)
			->(Derive R (cons x u) (cons x v))

.

Hint Derive1.
Hint Derive2.


Lemma Derive_inclus : (R1,R2:Ensf)(u,v:Word)
(inclus R1 R2)->(Derive R1 u v)->(Derive R2 u v).
Intros R1 R2 u v inclus_R1_R2 Der_R1.
Elim Der_R1;Auto.
Save.


Definition Derive_inv := [R:Ensf][x,y:Word]
(<Prop>Case x of
     (* nil  *) False
     (* cons *) [A:Elt][w:Word]
       ((Ex2  [u:Word](dans (couple A (word u)) R)
	       [u:Word](Ex2 [v:Word](<Word>(cons A v)=x)
	                    [v:Word](<Word>(Append u v)=y)))

	\/

	(Ex2  [v:Word](Derive R w v) [v:Word]<Word>(cons A v)=y))
end).


Lemma Derive_inv1 : (R:Ensf)(u,v:Word)(Derive R u v)->(Derive_inv R u v).
Intros R x y Der_x_y.
Unfold Derive_inv.
Elim Der_x_y;Prolog [ex_intro2 refl_equal or_intror or_introl] 8.
(* Intros u v A dans_couple.
 Left.
 Exists u; [Assumption | Exists v; Apply refl_equal].

 Intros u v x0 Der_u_v Der_inv_u_v.
 Simpl; Right.
 Exists v; Trivial.*)
Save.

Hint Derive_inv1.

Lemma Derive_inv2 : (R:Ensf)(x,y:Word)(Derive_inv R x y)->
<Elt> Ex ([A:Elt]
	<Word> Ex2 ([w:Word]<Word>(cons A w)=x
		   ,[w:Word]
( (<Word> Ex2 ([u:Word](dans (couple A (word u)) R)
		    ,[u:Word](<Word> Ex2 ([v:Word](<Word>(cons A v)=x)
					 ,[v:Word](<Word>(Append u v)=y)
					)
				)
			)
	)

	\/

	(<Word> Ex2 ([v:Word](Derive R w v)
		    ,[v:Word]<Word>(cons A v)=y
		    )
	))
		)
	)
.
Intros R x y .
Elim x.
Unfold Derive_inv.

Intuition.
(*Intro temp; Elim temp; Clear temp.*)

Intros x0 w Hyp_rec .
Unfold Derive_inv.
(*Simpl.*)
Intuition.
Elim H0 ; Clear H0. (* Elim of an Ex2 *)
Intros x1 dans_couple_R temp; Elim temp; Clear temp.
Intros x2 cons_x0_x2_cons_x0_w Append_x1_x2_y H.
Exists x0.
Exists w.
Trivial.
Apply or_introl.
Exists x1.
 Assumption.
 Exists w.
  Trivial.
  Replace w with x2.
   Assumption.
   Apply cons_cons_inv2 with x0 x0 ; Assumption.
   

Elim H0; Clear H0.
Prolog [ or_intror ex_intro2 refl_equal ex_intro ] 9.
(*Intros x1 der_R cons_x0_x1_y.
Exists x0.
Exists w.
Trivial.
Apply or_intror.
Exists x1;Assumption.*)
Save.

Lemma Derive_inv3 : (R:Ensf)(x,y:Word)(Derive R x y)->
<Elt> Ex ([A:Elt]
	<Word> Ex2 ([w:Word]<Word>(cons A w)=x
		   ,[w:Word]
( (<Word> Ex2 ([u:Word](dans (couple A (word u)) R)
		    ,[u:Word](<Word> Ex2 ([v:Word](<Word>(cons A v)=x)
					 ,[v:Word](<Word>(Append u v)=y)
					)
				)
			)
	)

	\/

	(<Word> Ex2 ([v:Word](Derive R w v)
		    ,[v:Word]<Word>(cons A v)=y
		    )
	))
		)
	)
.
(*
Proof [R:Ensf][x,y:Word][D : (Derive R x y)]
(Derive_inv2 R x y (Derive_inv1 R x y D)).
*)
Prolog [Derive_inv1 Derive Derive_inv2 ] 7.
Save.

Lemma in_mon_X_Der_imp_inmon_X :
(X,V,R:Ensf)
  (u,v:Word)
    (Regles X V R)->
      (Derive R u v)->
        (inmonoid (union X V) u)->
          (inmonoid (union X V) v).

Intros X V1 R1 u v Regles_R1 Der_R1.
Elim Der_R1 ; Prolog [Regles_inv2 inmonoid_cons_inv inmonoid_cons_inv2 inmonoid_cons inmonoid_Append ] 10.
	(*Intros u0 v0 A dans_R1 inmonoid_cons_A_v0.
	Apply inmonoid_Append.
		Apply Regles_inv2 with R1 A;Assumption.
		Apply inmonoid_cons_inv with A; Assumption.

	Intros u0 v0 x Der_R1_u0 imp inmon_cons_x_u0.
	Apply inmonoid_cons.
		Apply imp.
		Apply inmonoid_cons_inv with x;Assumption.

		Apply inmonoid_cons_inv2 with u0;Assumption.*)

Save.


(*  (Derivestar R u v) signifie "u se recrit en v par zero ou plusieurs productions de R" *)


Definition Derivestar :=
[R:Ensf](Rstar Word (Derive R)).

Hint Unfold Derivestar.

Lemma Derivestar_refl : (R:Ensf)(u:Word)
(Derivestar R u u).
Auto.
Save.

Hint Derivestar_refl.

Lemma Derivestar_R : (R:Ensf)(u,v,w:Word)
(Derive R u v)->(Derivestar R v w)->(Derivestar R u w).
Unfold Derivestar.
Prolog [Rstar_R ] 8.
(*Intros.
Apply Rstar_R with v;Assumption.*)
Save.


Lemma Derivestar_inv :
(R:Ensf)(u,v:Word)(Derivestar R u v)->(<Word>u=v\/<Word> Ex2 ([w:Word](Derive R u w),
						[w:Word](Derivestar R w v))).
Unfold Derivestar.
Prolog [Rstar_inv ] 6.
(*Intros R u v Der_R.
Apply Rstar_inv;Assumption.*)
Save.

Hint Derivestar_inv.


Lemma Derivestar_inclus : (R1,R2:Ensf)(u,v:Word)
(inclus R1 R2)->(Derivestar R1 u v)->(Derivestar R2 u v).
Intros R1 R2 u v inclus_R1_R2 Der_R1.
Unfold Derivestar Rstar in Der_R1.
Pattern u v.
Apply Der_R1.
	Auto.
	Intros ; Prolog [Derive_inclus Derivestar_R] 3.
	(*Intros a b c Der_a_b Der_b_c.
	Apply Derivestar_R with b.
		Apply Derive_inclus with R1;Assumption.
		Assumption.*)
Save.


(* LG X V R S est l'ensemble de mots engendre par la grammaire (X V R S) *)

Definition LG : Ensf -> Ensf -> Ensf -> Elt -> wordset :=
	[X,V,R:Ensf][S:Elt][w:Word]
			(Derivestar R (cons S nil) w)
			/\ (inmonoid X w)
.


Lemma LG_inv : (X:Ensf)(V:Ensf)(R:Ensf)(S:Elt)(w:Word)(LG X V R S w)->(inmonoid X w).
Unfold LG.
Intros.
Elim H; Auto.
Save.

(*Pour toute grammaire (X,V,R,S), (LG X V R S) est un langage *)

Lemma LG_langage :
  (X,V,R:Ensf)
    (S:Elt)
      (isGram X V R S)->
        (islanguage X (LG X V R S)).
Intros; Red ; Intros ; Elim H0; Auto.
Save.


(*Reunion de 2 grammaires *)

Definition Gunion := [V1,R1:Ensf][V2,R2:Ensf]
	(<Ensf,Ensf>((union V1 V2),(union R1 R2))).

(*------------------*)
Section injprod.

Local injproduc :=
[f:Elt->Elt][V:Ensf](extension V f).


Definition injproducg : Ensf -> Elt -> Elt :=
	(injproduc injgauche).

Definition injproducd : Ensf -> Elt->Elt :=
	(injproduc injdroite).
					
(*prennent en arguments l'ensemble de non-terminaux V,*)
(*de productions R et rendent*)
(*les injections gauche et droite*)
(*utilisees ensuite pour la definition de G_union_disj_p.*)

End injprod.


Definition Gunion_disj_p := [V1,R1:Ensf][S1:Elt][V2,R2:Ensf][S2:Elt][S:Elt]
   <Ensf,Ensf*Elt>
	((add S (fst Ensf Ensf (Gunion V1 R1 V2 R2))),
	 <Ensf,Elt>((add (couple S (word (cons S1 nil)))
	                 (add (couple S (word (cons S2 nil)))
				(snd Ensf Ensf (Gunion V1 R1 V2 R2))
			 )),
		    S)).


(* image par une fonction d'une grammaire *)

Definition imageGram := [f:Elt->Elt][X,V,R:Ensf][S:Elt]
    <Ensf,Ensf*(Ensf*Elt)>((map f X),
    <Ensf,Ensf*Elt>
    ((map f V),
	<Ensf,Elt>
	((map [P:Elt](couple (f (first P)) 
		([w:Elt](word (Word_ext f (word_inv w))) (second P))) R),
	 (f S)))).


