(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 gram2.v                                  *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)

Require Ensf.
Require Words.
Require more_words.
Require need.
Require fonctions.
Require Relations.
Require gram.

Hint extension_Id.
Section resultats.


Variable X,V1,R1:Ensf.
Variable S1:Elt.
Variable V2,R2:Ensf.
Variable S2:Elt.

Local C := (Gunion V1 R1 V2 R2).
Local Vu := (fst Ensf Ensf C).
Local Ru := (snd Ensf Ensf C).


Lemma inter_X_Vu : (isGram X V1 R1 S1)->(isGram X V2 R2 S2)->(inter X Vu empty).
Intro G1.
Intro G2.
Unfold Vu.
Unfold C.
Unfold Gunion.
Simpl.
Apply union_inter.
Apply isGram2 with R1 S1; Assumption.
Apply isGram2 with R2 S2; Assumption.
Save.


Lemma Gunion_Regles : (isGram X V1 R1 S1)->(isGram X V2 R2 S2)->
(Regles X Vu Ru).
Intros.
Unfold Ru ; Unfold C ; Unfold Gunion ; Simpl.
Apply Regles_union; Unfold Vu; Unfold C; Unfold Gunion; Simpl.
Apply Regles_V with V1;[ Auto | Apply isGram4 with S1; Auto].
Apply Regles_V with V2;[ Auto | Apply isGram4 with S2; Auto].
Save.


Lemma inmon_Der_imp_Der :
(Regles X V1 R1)->
  (Regles X V2 R2)->
    (inter (union X V1) V2 empty)->
      (u,v:Word)
        (Derive Ru u v)->
          (inmonoid (union X V1) u)->
            (Derive R1 u v).
Intros Re_1 Re_2 inter_X_V1_V2_empty u v Der_Ru_u.
Elim Der_Ru_u.
	Intros u0 v0 A dans_couple_Ru inmon_cons_A_v0.
	Apply Derive1.
	Cut (dans (couple A (word u0)) R1)\/(dans (couple A (word u0)) R2).(**)
	Intro temp; Elim temp; Clear temp.
		Auto.

		Intro dans_R2.
		Absurd (inter (union X V1) V2 empty).
			Red.
			Unfold inter.
			Intro temp;Elim temp;Clear temp.
			Intros incl_empty_X_V1 temp;Elim temp;Clear temp.
			Intros incl_empty_V2 imp.
			Apply dans_empty_imp_P with A.
			Apply imp.
				Apply inmonoid_cons_inv2 with v0;Assumption.
				Apply Regles_inv1 with X R2 (word u0);
						Assumption.

		Assumption.

	(**) Apply dans_union;Assumption.

	Intros u0 v0 x Der_Ru imp inmon_cons_x_u0.
	Apply Derive2.
	Apply imp.
	Apply inmonoid_cons_inv with x.
	Assumption.
Save.


Lemma inmon_Der_imp_inmon_R1 :
(u,v:Word)
  (Regles X V1 R1)->
    (Derive R1 u v)->
      (inmonoid (union X V1) u)->
        (inmonoid (union X V1) v).
Intros.
Apply in_mon_X_Der_imp_inmon_X with R1 u;Assumption.
Save.

Lemma inmon_Der_imp_inmon : (u,v:Word)
  (isGram X V1 R1 S1)->(isGram X V2 R2 S2)->
  (inter V1 V2 empty)->
  (inmonoid (union X V1) u)->(Derive Ru u v)->(inmonoid (union X V1) v).
Intros u v G_R1 G_R2 inter_V1_V2_empty inmon_X_V1_u Der_Ru_u_v.
Apply inmon_Der_imp_inmon_R1 with u.
	Apply isGram4 with S1;Assumption.

	Apply inmon_Der_imp_Der.
		Apply isGram4 with S1;Assumption.

		Apply isGram4 with S2;Assumption.

		Apply inter_union.
			Apply isGram2 with R2 S2; Assumption.
			Assumption.

		Assumption.

		Assumption.

	Assumption.

Save.

Lemma Gunion_Derivestar : (isGram X V1 R1 S1)->(isGram X V2 R2 S2)->
(inter V1 V2 empty)->
(u,v:Word)(Derivestar Ru u v)->(inmonoid (union X V1) u)->(Derivestar R1 u v).
Unfold Derivestar .
Intros G_1 G_2 inter_V1_V2_empty u v Derivestar_Ru .
Pattern u v.
Apply Derivestar_Ru.
	Auto.(*Intros.
	Apply Rstar_reflexive.*)

	Intros u0 v0 w Der_Ru inmon_v0_imp_Rstar_R1_v0 inmon_u0. 
(*Prolog [Rstar_R inmon_Der_imp_Der isGram4  inter_union  isGram2 inmon_Der_imp_inmon ] 6. -> raisonnable seulement sur Cray...*)
	Apply Rstar_R with v0.
		Prolog [inmon_Der_imp_Der isGram4  inter_union  isGram2 ] 5.
(*		Apply inmon_Der_imp_Der.
			Apply isGram4 with S1;Assumption.
			Apply isGram4 with S2;Assumption.
			Apply inter_union.
				Apply isGram2 with R2 S2; Assumption.
				Assumption.
			Assumption.
			Assumption.*)

		Prolog [ inmon_Der_imp_inmon ] 3.
(*		Apply inmon_v0_imp_Rstar_R1_v0.
		Apply inmon_Der_imp_inmon with u0;Assumption.*)

Save.


Lemma inmon_Der_imp_Der2 :
(Regles X V1 R1)->
  (Regles X V2 R2)->
    (inter (union X V2) V1 empty)->
      (u,v:Word)(Derive Ru u v)->
        (inmonoid (union X V2) u)->
          (Derive R2 u v).
Intros Re_1 Re_2 inter_X_V2_V1_empty u v Der_Ru_u.
Elim Der_Ru_u.
	Intros u0 v0 A dans_couple_Ru inmon_cons_A_v0.
	Apply Derive1.
	Cut (dans (couple A (word u0)) R1)\/(dans (couple A (word u0)) R2).(**)
	Intro temp ; Elim temp ; Clear temp.
		Intro dans_R1.
		Absurd (inter (union X V2) V1 empty).
			Red.
			Unfold inter.
			Intuition.
		        (*Intros;*)Apply dans_empty_imp_P with A ; Prolog [inmonoid_cons_inv2 Regles_inv1] 3.
(*			Intros incl_empty_V1 imp incl_empty_X_V2.*)


(*			Apply dans_empty_imp_P with A.
			Apply imp.
				Apply inmonoid_cons_inv2 with v0;Assumption.
				Apply Regles_inv1 with X R1 (word u0);
						Assumption.
*)
		Assumption.

	Trivial.

	(**) Auto.

	Prolog [ Derive2 inmonoid_cons_inv ] 10.
(*	Intros u0 v0 x Der_Ru imp inmon_cons_x_u0.
	Apply Derive2.
	Apply imp.
	Apply inmonoid_cons_inv with x.
	Assumption.*)
Save.

Lemma inmon_Der_imp_inmon_R2 : (u,v:Word)
  (Regles X V2 R2)->(Derive R2 u v)->(inmonoid (union X V2) u)
  ->(inmonoid (union X V2) v).
Intros u v Regles_R2 Der_R2.
Elim Der_R2.
	Intros;Prolog [ inmonoid_cons_inv Regles_inv2 inmonoid_Append ] 3.
(*	Intros u0 v0 A dans_R2 inmonoid_cons_A_v0.
	Apply inmonoid_Append.
		Apply Regles_inv2 with R2 A;Assumption.
		Apply inmonoid_cons_inv with A; Assumption.*)

	Intros;Prolog [ inmonoid_cons_inv2 inmonoid_cons_inv inmonoid_cons] 4.
	(*Intros u0 v0 x Der_R2_u0 imp inmon_cons_x_u0.
	Apply inmonoid_cons.
		Apply imp.
		Apply inmonoid_cons_inv with x;Assumption.

		Apply inmonoid_cons_inv2 with u0;Assumption.*)

Save.

Lemma inmon_Der_imp_inmon_2 : (u,v:Word)
  (isGram X V1 R1 S1)->(isGram X V2 R2 S2)->
  (inter V1 V2 empty)->
  (inmonoid (union X V2) u)->(Derive Ru u v)->(inmonoid (union X V2) v).
(* with Prolog...*)
Intros ; Prolog [inmon_Der_imp_inmon_R2 sym_inter isGram2 inter_union isGram4 inmon_Der_imp_Der2 ] 5.

(* without Prolog...*)
(*Intros u v G_R1 G_R2 inter_V1_V2_empty inmon_X_V2_u Der_Ru_u_v.
Apply inmon_Der_imp_inmon_R2 with u.
	Apply isGram4 with S2;Assumption.

	
	Apply inmon_Der_imp_Der2.
		Apply isGram4 with S1;Assumption.

		Apply isGram4 with S2;Assumption.

		Apply inter_union.
			Apply isGram2 with R1 S1; Assumption.
			Apply sym_inter; Assumption.

		Assumption.

		Assumption.

	Assumption.*)
Save.

Lemma Gunion_Derivestar2 : (isGram X V1 R1 S1)->(isGram X V2 R2 S2)->
  (inter V1 V2 empty)->
  (u,v:Word)(Derivestar Ru u v)->(inmonoid (union X V2) u)
			       ->(Derivestar R2 u v).
Unfold Derivestar .
Intros G_1 G_2 inter_V1_V2_empty u v Derivestar_Ru .
Pattern u v.
Apply Derivestar_Ru.
	Auto.(*Intros.
	Apply Rstar_reflexive.*)

	Intros u0 v0 w Der_Ru inmon_v0_imp_Rstar_R2_v0 inmon_u0. 
	Apply Rstar_R with v0.
		Prolog [sym_inter isGram2 inter_union isGram4 inmon_Der_imp_Der2 ] 4.
		(*Apply inmon_Der_imp_Der2.
			Apply isGram4 with S1;Assumption.
			Apply isGram4 with S2;Assumption.
			Apply inter_union.
				Apply isGram2 with R1 S1; Assumption.
				Apply sym_inter;Assumption.
			Assumption.
			Assumption.*)

		Prolog [inmon_Der_imp_inmon_2] 3.
		(*Apply inmon_v0_imp_Rstar_R2_v0.
		Apply inmon_Der_imp_inmon_2 with u0;Assumption.*)
Save.


Lemma Gunion_isGram : (S:Elt)
(isGram X V1 R1 S1)->(isGram X V2 R2 S2)->(dans S Vu)->(isGram X Vu Ru S).

Prolog [ isGram5 isGram1 inter_X_Vu Gunion_Regles ] 7.

(*Intros.
Apply isGram5.

Apply isGram1 with V1 R1 S1 ; Assumption.

Apply inter_X_Vu ; Assumption.

Assumption.

Apply Gunion_Regles; Assumption.*)

Save.


Variable f : Elt -> Elt.

Local wef := (Word_ext f).
Local fet := [w:Elt](word (wef (word_inv w))).
Local fonc := [P:Elt](couple (f (first P))
			    (fet (second P))
		      ).


Local Gim := (imageGram f X V1 R1 S1).
Local Xim := (fst Ensf Ensf*(Ensf*Elt) Gim).
Local Gim2 := (snd Ensf Ensf*(Ensf*Elt) Gim).
Local Vim := (fst Ensf Ensf*Elt Gim2).
Local Gim3 := (snd Ensf Ensf*Elt Gim2).
Local Rim := (fst Ensf Elt Gim3).
Local Sim := (snd Ensf Elt Gim3).

Lemma dans_Rim : (A:Elt)(u:Word)
  (dans (couple A (word u)) R1)->(dans (couple (f A)(word (wef u))) Rim).
Intros.
Unfold Rim Gim3 Gim2 Gim imageGram ; Simpl.
Replace (couple (f A)(word (wef u))) with (fonc (couple A (word u)));Auto.
Save.
Hint dans_Rim.

Lemma image_Regles : (Regles X V1 R1)->(Regles Xim Vim Rim).
Unfold Rim Vim Xim Gim3 Gim2 Gim imageGram Regles ; Simpl.
Intros R_R1 x dans_x.
Cut (<Elt> Ex ([x_ant:Elt]((dans x_ant R1)/\(<Elt>x=(fonc x_ant))))).
Intro temp; Elim temp; Clear temp.
Intros x_ant temp ; Elim temp ; Clear temp ; Intros x_ant_in_R1 x_egal .
Elim (R_R1 x_ant x_ant_in_R1).
Intros A_ant dans_A_ant_V1 temp ; Elim temp; Clear temp.
Intros B_ant x_ant_egal inmonoid_x_ant.
Exists (f A_ant).
	Apply dans_map_inv ; Assumption.
	Exists (Word_ext f B_ant).
	Rewrite x_egal ; Unfold fonc ; Unfold fet.
	Rewrite x_ant_egal; Unfold first ;Unfold second;Unfold word_inv; Simpl.
	Trivial.
	Replace (union (map f X) (map f V1)) with (map f (union X V1));Auto.

Apply dans_map ; Assumption.(*1*)
Save.


Lemma image_isGram : (isGram X V1 R1 S1)->(Mots Xim)->(inter Xim Vim empty)
  ->(isGram Xim Vim Rim Sim).

Unfold Sim Rim Gim3 Vim Gim2 Xim Gim imageGram ; Simpl.
Prolog [isGram4 image_Regles isGram3 dans_map_inv isGram5] 7.

(*Intro is_Gram; Intro Mots ; Intro intersec .
Apply isGram5. 
 Assumption.
 Assumption.
 Apply dans_map_inv ; Apply isGram3 with X R1 ; Assumption.
 Apply image_Regles; Apply isGram4 with S1; Assumption.*)

Save.


Lemma Id_image_X : (Id (union X V1) f)->(<Ensf>Xim=X).

Unfold Xim Gim imageGram; Simpl.
Elim X.
Auto.
Clear X.
Intros a X Hyp Id_a_X_V1_f.
Unfold map.
Simpl.
Apply add_add.
	Auto.

	Apply Hyp.
	Apply Id_inclus with (union (add a X) V1).

		Red; Intros x dans_x_union_X_V1.
		Cut ((dans x X)\/(dans x V1)) ; Intuition.

		Assumption.
Save.

Lemma Id_image_V : (Id (union X V1) f)->(<Ensf>Vim=V1).
Unfold Vim Gim2 Gim imageGram; Simpl.
Generalize V1; Clear V1; Induction V1; Auto.
Clear V1.
Intros a V1 Hyp Id_X_a_V1_f.
Unfold map.
Simpl.
Apply add_add.
	Auto.

	Apply Hyp.
	Apply Id_inclus with (union X (add a V1)).
		Red; Intros x dans_x_union_X_V1.
		Cut ((dans x X)\/(dans x V1)) ; Intuition.

			
		Assumption.
Save.

Lemma Id_image_R : (Id (union X V1) f)->(isGram X V1 R1 S1)->(<Ensf>Rim=R1).
Intros Id_X_V1_f.

Unfold Rim Gim3 Gim2 Gim imageGram; Simpl.
Elim R1.
Auto.
Intros a R Hyp isGram_X_V1_R1_S1.
Unfold map.
Simpl.
Apply add_add.

	Cut (Regles X V1 (add a R)).
	Unfold Regles.
	Intro R_a_R.
	Elim (R_a_R a).
	Intros x dans_x_V1 temp; Elim temp; Clear temp.
	Intros B egal_a inmonoid_B.

	Rewrite egal_a.
	Apply couple_couple; Unfold first second word_inv; Simpl.
		Auto.

		Apply word_word.
		Cut (Id_words (union X V1) (Word_ext f)); Auto.
		

	Trivial.

	Apply isGram4 with S1; Trivial.

	Prolog [isGram_inclus3] 3.
	(*Apply Hyp.
	Apply isGram_inclus3 with a; Assumption.*)
Save.

Lemma Id_image_S : (Id (union X V1) f)->(isGram X V1 R1 S1)->(<Elt>Sim=S1).
Unfold Sim Gim3 Gim2 Gim imageGram ;Simpl.
Prolog [isGram3 union_d] 6.
(*Intros Id_union_X_V1_f isGram_X_V1_R1_S1.
Apply Id_union_X_V1_f.
Apply union_d.
Apply isGram3 with X R1; Assumption.*)
Save.

Lemma Gim_egal :
 <Ensf*(Ensf*(Ensf*Elt))>Gim=(<Ensf,Ensf*(Ensf*Elt)>(Xim,<Ensf,Ensf*Elt>(Vim,
							<Ensf,Elt>(Rim,Sim)))).
Unfold Sim Rim Gim3 Vim Gim2 Xim.
Apply refl_equal.
Save.



Lemma Id_image_G : (Id (union X V1) f)->(isGram X V1 R1 S1)
  ->(<Ensf*(Ensf*(Ensf*Elt))>Gim=(<Ensf,Ensf*(Ensf*Elt)>(X,<Ensf,Ensf*Elt>(V1,
							<Ensf,Elt>(R1,S1))))).
Intros Id_u_X_V1_f isG_X_V1_R1_S1.
Rewrite Gim_egal.

Rewrite (Id_image_S Id_u_X_V1_f isG_X_V1_R1_S1).
Rewrite (Id_image_R Id_u_X_V1_f isG_X_V1_R1_S1).
Rewrite (Id_image_V Id_u_X_V1_f ).
Rewrite (Id_image_X Id_u_X_V1_f ).
Apply refl_equal.
Save.

Lemma Derive_image : (u,v:Word)(Derive R1 u v)->(Derive Rim (wef u) (wef v)).

Intros u v H.
Elim H; Clear H u v.
	Intros u v A dans_A.
	Replace (wef (cons A v)) with (cons (f A) (wef v)).
	Replace (wef (Append u v)) with (Append (wef u) (wef v)).
	Auto.

	Apply sym_equal; Unfold wef ; Apply wef_append ; Auto.

	Auto.

	Intros.
	Replace (wef (cons x u)) with (cons (f x) (wef u)).
	 Replace (wef (cons x v)) with (cons (f x) (wef v));Auto.
	 Auto.
Save.

Hint Derive_image.

Lemma Derivestar_image : (u,v:Word)(Derivestar R1 u v)->(Derivestar Rim (wef u) (wef v)).
Unfold Derivestar.
Intros u v Hyp.
Unfold Derivestar Rstar in Hyp.
Pattern u v.
Apply Hyp.
	Auto.(*Intros; Apply Rstar_reflexive. *)
	Intros a b c DeriveR1 Rst.
	Apply Rstar_R with y:=(wef b);Auto.
Save.

Hint Derivestar_image.


Lemma Reconnait_imageGram : (w:Word)(LG X V1 R1 S1 w)->(LG Xim Vim Rim Sim (wef w)).
 
Intro w.
Unfold LG .
Intro temp ; Split ; Elim temp ; Clear temp ; Intros Der inmo.
	Unfold Sim Gim3 Gim2 Gim imageGram;Simpl.
	Replace (cons (f S1) nil) with (wef (cons S1 nil));Auto.

	Elim inmo.
		Auto.
		Intros wo el.
		Unfold Xim Gim imageGram; Simpl.
		Replace (wef (cons el wo)) with (cons (f el)(wef wo)); Auto.
Save.


End resultats.

Hint dans_Rim.
Hint Derive_image.
Hint Derivestar_image.
Hint Reconnait_imageGram.
