(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 gram3.v                                  *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)

Require Ensf.
Require Words.
Require more_words.
Require need.
Require fonctions.
Require Relations.
Require gram.
Require gram2.


Section resultats_iso_image.

Variable X,V,R : Ensf.
Variable S : Elt.
Variable f : Elt->Elt.

Local Gim := (imageGram f X V R S).
Local Xim := (fst Ensf Ensf*(Ensf*Elt) Gim).
Local Gim2 := (snd Ensf Ensf*(Ensf*Elt) Gim).
Local Vim := (fst Ensf Ensf*Elt Gim2).
Local Gim3 := (snd Ensf Ensf*Elt Gim2).
Local Rim := (fst Ensf Elt Gim3).
Local Sim := (snd Ensf Elt Gim3).

Local invf := (inv (union X V) (union Xim Vim) f).

Local Gim' := (imageGram invf Xim Vim Rim Sim).
Local Xim' := (fst Ensf Ensf*(Ensf*Elt) Gim').
Local Gim2' := (snd Ensf Ensf*(Ensf*Elt) Gim').
Local Vim' := (fst Ensf Ensf*Elt Gim2').
Local Gim3' := (snd Ensf Ensf*Elt Gim2').
Local Rim' := (fst Ensf Elt Gim3').
Local Sim' := (snd Ensf Elt Gim3').

Hypothesis Gram : (isGram X V R S).

Lemma Regles_X_V_R : (Regles X V R).
Proof (isGram4 X V R S Gram).


Hypothesis Mono : (is_mono (union X V) f).

Lemma inter_Xim_Vim_empty : (inter Xim Vim empty).

Unfold Xim Vim;Simpl.
Red.
(*Intuition.*) Split ; [Idtac  | Split ] .

Auto.

Auto.

Intros x dans_x_map_f_X dans_x_map_f_V.
ElimType <Elt>Ex([y:Elt](dans y X)/\(<Elt>x=(f y))).
ElimType <Elt>Ex([y:Elt](dans y V)/\(<Elt>x=(f y))).
Intros v temp ; Elim temp ; Clear temp.
Intros dans_v_V x_egal_f_v x_ant temp ; Elim temp ; Clear temp.
Intros dans_x_ant_X x_egal_f_x_ant.
ElimType (inter X V empty).
Intros incl_empty_X temp ; Elim temp ; Clear temp. Intros incl_empty_V imp.
Apply dans_empty_imp_P with v.
Apply imp.
Replace v with x_ant.
	Assumption.
	Apply Mono.
		Auto.
		Auto.
		Rewrite <- x_egal_f_x_ant;Assumption.
Assumption.
Exact (isGram2 X V R S Gram).
Auto.
Auto.
Save.


Lemma union_Xim_Vim_map_f_union_X_V : 
	<Ensf>(union Xim Vim)=(map f (union X V)).
Unfold Xim Vim Gim2 Gim imageGram; Simpl.
Apply map_union.
Save.

Lemma Iso : (is_iso (union X V) (union Xim Vim) f).
Rewrite union_Xim_Vim_map_f_union_X_V.
Auto.
Save.

Hint Iso.

Local wef := (Word_ext f).
Local weinvf := (Word_ext invf).

Lemma invf_f : (Id (union X V) (comp invf f)).
Unfold invf.
Auto.
Save.

Lemma weinvf_wef : (Id_words (union X V) (comp_word weinvf wef)).
Unfold Id_words.
Unfold weinvf wef.
Intro x.
Rewrite <- (comp_Word_ext invf f x).
Generalize x.
Apply (extension_Id (union X V) (comp invf f)).
Exact invf_f.
Save.

Local Rf := [g:Elt->Elt][P:Elt](couple (g (first P))
				(word (Word_ext g (word_inv (second P))))).

Lemma comp_Rf : (g:Elt->Elt)(f:Elt->Elt)(x:Elt)(dans x R)
  -><Elt>(comp (Rf g) (Rf f) x)=(Rf (comp g f) x).
Clear Mono f.
Intros f g x dans_x_R.
Elim (Regles_X_V_R x dans_x_R).
Intros A dans_A_V temp.
Elim temp; Clear temp; Intros B egal_x inmonoid_B.
Rewrite egal_x.
Unfold  Rf.
Unfold  1 comp. 
 
Apply couple_couple.
 Apply refl_equal.
 Simpl. Rewrite (comp_Word_ext f g B).Apply refl_equal.
Save.


Lemma egalGim'_image_comp : <
    Ensf*(Ensf*(Ensf*Elt))>Gim'=(imageGram (comp invf f) X V R S).
Unfold Gim'.
Unfold Sim Rim Gim3 Vim Gim2 Xim Gim imageGram;Simpl.

Apply pair_equal.
Rewrite (map_map_eg_map_comp invf f X); Auto.

Apply pair_equal.
Rewrite (map_map_eg_map_comp invf f V); Auto.

Apply pair_equal.

Change (<Ensf>(map (Rf invf) (map (Rf f) R))=(map (Rf (comp invf f)) R)).
Replace (map (Rf (comp invf f)) R) with (map (comp (Rf invf) (Rf f)) R).

Apply map_map_eg_map_comp. 
Apply map_egal.
Exact (comp_Rf invf f).

Apply refl_equal.
Save.


Lemma egalG : 
(<Ensf*(Ensf*(Ensf*Elt))>Gim'=(<Ensf,Ensf*(Ensf*Elt)>(X,<Ensf,Ensf*Elt>(V,
							<Ensf,Elt>(R,S))))).
Rewrite egalGim'_image_comp.
Apply Id_image_G.
Unfold invf.
Auto.
Auto.
Save.


Lemma egalS : <Elt>Sim'=S.
Unfold Sim' Rim' Gim3' Vim' Gim2' Xim' .
Rewrite egalG.
Apply refl_equal.
Save.

Lemma egalR : <Ensf>Rim'=R.
Unfold Rim' Gim3' Vim' Gim2' Xim' .
Rewrite egalG.
Apply refl_equal.
Save.

Lemma egalX : <Ensf>Xim'=X.
Unfold Vim' Gim2' Xim' .
Rewrite egalG.
Apply refl_equal.
Save.

Lemma egalV : <Ensf>Vim'=V.
Unfold Vim' Gim2' Xim' .
Rewrite egalG.
Apply refl_equal.
Save.

Lemma Reconnait_imageGram_iso : (w:Word)(inmonoid X w)->
  (LG Xim Vim Rim Sim (wef w))->(LG X V R S w).
Intros w inmonoid_X_w LG_wef_w.
Rewrite <- egalR.
Rewrite <- egalS.
Rewrite <- egalV.
Rewrite <- egalX.

Replace w with (weinvf (wef w)).
	Unfold Sim' Rim' Gim3' Vim' Gim2' Xim' Gim' weinvf.
	Auto.
	Change <Word>(comp_word weinvf wef w)=w.
	Apply Id_words_inv with X .
		Assumption.
		Apply Id_words_inclus with (union X V).
			Auto.
			Exact weinvf_wef.
Save.



Lemma egal_Xim : (Id X f)-><Ensf>Xim=X.
Unfold Xim Gim imageGram;Simpl.
Intros.
Apply map_Id; Assumption.
Save.


Lemma egal_LG : (Id X f)->(l_egal (LG X V R S) (LG Xim Vim Rim Sim)).
Unfold l_egal.
Intro Id_X.
Unfold l_inclus.

Split.
	Intros w LG_X_w.
	Replace w with (wef w).
		Unfold Sim Rim Gim3 Vim Gim2 Xim Gim wef.
		Auto.

		Apply (Id_words_inv X).
			Prolog [ LG_inv ] 2.
			(*Apply LG_inv with V R S;Assumption.*)
			Exact (extension_Id X f Id_X).

	Intros w LG_Xim_w.
	Replace X with Xim'.
	Replace V with Vim'.
	Replace S with Sim'.
	Replace R with Rim'.
	Replace w with (weinvf (wef w)).
		Unfold Sim' Rim' Gim3' Vim' Gim2' Xim' Gim' weinvf.
		Apply Reconnait_imageGram.
		Replace (wef w) with w.
			Assumption.
			Apply sym_equal.
			Apply (Id_words_inv X).
				Rewrite <- (egal_Xim Id_X).
				Prolog [ LG_inv ] 2.
				(*Apply LG_inv with Vim Rim Sim;Assumption.*)
				Exact (extension_Id X f Id_X).
	Pattern 2 w ; Replace w with ((comp_word weinvf wef) w).
		Apply refl_equal.

		Apply Id_words_inv with (union X V).
			Apply inmonoid_inclus with X.
				Auto.
				Replace X with Xim.
				Prolog [ LG_inv ] 2.
				(*Apply LG_inv with Vim Rim Sim;Assumption.*)

				Exact (egal_Xim Id_X).
			Exact weinvf_wef.
	Exact egalR.
	Exact egalS.
	Exact egalV.
	Exact egalX.
Save.
		
End resultats_iso_image.
