(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 gram5.v                                  *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)

Require Ensf.
Require Words.
Require more_words.
Require Rat.
Require need.
Require fonctions.
Require Relations.
Require gram.
Require gram2.
Require gram3.
Require gram4.

Section gram5.

Variable X:Ensf.

Variable V1,R1:Ensf.
Variable S1:Elt.

Variable V2,R2:Ensf.
Variable S2:Elt.

Local S':=zero.

Local G1:=(imageGram (injproducg V1) X V1 R1 S1).
Local G2:=(imageGram (injproducd V2) X V2 R2 S2).
Local X1':=(fst Ensf Ensf*(Ensf*Elt) G1).
Local GG1:=(snd Ensf Ensf*(Ensf*Elt) G1).
Local V1':=(fst Ensf Ensf*Elt GG1).
Local GGG1:=(snd Ensf Ensf*Elt GG1).
Local R1':=(fst Ensf Elt GGG1).
Local S1':=(snd Ensf Elt GGG1).

Local X2':=(fst Ensf Ensf*(Ensf*Elt) G2).
Local GG2:=(snd Ensf Ensf*(Ensf*Elt) G2).
Local V2':=(fst Ensf Ensf*Elt GG2).
Local GGG2:=(snd Ensf Ensf*Elt GG2).
Local R2':=(fst Ensf Elt GGG2).
Local S2':=(snd Ensf Elt GGG2).

Local Gim:=(Gunion_disj_p V1' R1' S1' V2' R2' S2' S').
Local Vu := (fst Ensf (Ensf*Elt) Gim).
Local C':=(snd Ensf (Ensf*Elt) Gim).
Local Ru:=(fst Ensf Elt C').
Local Su:=(snd Ensf Elt C').

Hypothesis Grammaire1 : (isGram X V1 R1 S1).
Hypothesis Grammaire2 : (isGram X V2 R2 S2).
Hint Grammaire1.
Hint Grammaire2.


Lemma Mots_X : (Mots X).
Apply isGram1 with V1 R1 S1.
Auto.
Save.
Hint Mots_X.

Lemma int_X_V1_empty : (inter X V1 empty).
Apply isGram2 with R1 S1.
Auto.
Save.
Hint int_X_V1_empty.

Lemma int_X_V2_empty : (inter X V2 empty).
Apply isGram2 with R2 S2.
Auto.
Save.
Hint int_X_V2_empty.



Definition Gunion_disj := Gim.


Local Vim:=(fst Ensf Ensf*Elt Gunion_disj).
Local GGim:=(snd Ensf Ensf*Elt Gunion_disj).
Local Rim:=(fst Ensf Elt GGim).
Local Sim:=(snd Ensf Elt GGim).


Lemma Id_injproducg1:(x:Elt)(dans x X)-><Elt>((injproducg V1) x)=x.
Unfold Id injproducg .
Simpl.
Intros x dans_x_X.
Apply extension_out.
Apply inter_dans with X;Auto.
Save.
Hint Id_injproducg1.


Lemma Id_injproducd2 :(x:Elt)(dans x X)-><Elt>((injproducd V2) x)=x.
Unfold Id injproducd .
Simpl.
Intros x dans_x_X.
Apply extension_out.
Apply inter_dans with X;Auto.
Save.
Hint Id_injproducd2.

Lemma N_dans_S_X : ~(dans S' X).
Red.
Intro dans_S_X.
ElimType <Word> Ex ([w:Word]<Elt>(word w)=S').
	Intro x.
	Change ~<Elt>(word x)=(natural O).
	Discriminate.
Apply Mots_X;Assumption.
Save.
Hint N_dans_S_X.

Lemma injproducg_V1 : (x:Elt)
  (dans x V1)-><Elt>((injproducg V1) x)=(injgauche x).
Intros x dans_x_V1.
Unfold injproducg extension.
Elim (extension_spec V1 injgauche x).
Intros x0 temp;Elim temp;Clear temp;Intro temp;Elim temp;Clear temp.
	Auto.
	Intros.
	Absurd (dans x V1);Assumption.
Save.
Hint injproducg_V1.


Lemma map_injproducg_V1 : <Ensf>(map (injproducg V1) V1)=(map injgauche V1).
Apply map_egal.
Auto.
Save.
Hint map_injproducg_V1.

Lemma injproducd_V2 : (x:Elt)
  (dans x V2)-><Elt>((injproducd V2) x)=(injdroite x).
Intros x dans_x_V2.
Unfold injproducd extension.
Elim (extension_spec V2 injdroite x).
Intros x0 temp;Elim temp;Clear temp;Intro temp;Elim temp;Clear temp.
	Auto.
	Intros.
	Absurd (dans x V2);Assumption.
Save.
Hint injproducd_V2.


Lemma map_injproducd_V2 : <Ensf>(map (injproducd V2) V2)=(map injdroite V2).
Apply map_egal.
Auto.
Save.
Hint map_injproducd_V2.



Lemma N_dans_S_V1' : ~(dans S' V1').
Red.
Replace V1' with (map injgauche V1).
Intros.
ElimType (<Elt>Ex([y:Elt](dans y V1)/\(<Elt>S'=([e:Elt](couple e zero) y)))).
Intros x temp;Elim temp;Clear temp.
Intros dans_x_V1 S_egal.
Absurd <Elt>S'=(couple x zero).
Unfold S' zero.
Discriminate.
Auto.
Apply (dans_map [e:Elt](couple e zero)).
Assumption.
Unfold V1'.
Simpl.
Auto.
Save.
Hint N_dans_S_V1'.


Lemma N_dans_S_V2' : ~(dans S' V2').
Red.
Replace V2' with (map injdroite V2).
Unfold injdroite.
Intros.
ElimType (<Elt>Ex([y:Elt](dans y V2)/\(<Elt>S'=([e:Elt](couple e un) y)))).
Intros x temp;Elim temp;Clear temp.
Intros dans_x_V2 S_egal.
Absurd <Elt>S'=(couple x un).
Unfold S' zero.
Discriminate.
Auto.
Apply (dans_map [e:Elt](couple e un)).
Assumption.
Unfold V2'.
Simpl.
Auto.
Save.

Hint N_dans_S_V2'.

Lemma is_mono_u_X_V1_injproducg_V1 : (is_mono (union X V1)(injproducg V1)).
Unfold is_mono  .
Intros x y dans_x_u dans_y_u.
ElimType (dans x X)\/(dans x V1).
Intro dans_x.
Replace (injproducg V1 x) with x.

ElimType (dans y X)\/(dans y V1).
Intros dans_y .
Replace (injproducg V1 y) with y.
Auto.
Apply sym_equal;Auto.

Intros dans_y .
Replace (injproducg V1 y) with (injgauche y).
Unfold injgauche.
Elim (Mots_X x dans_x).
Intros x0 egal_x egal2_x.
Absurd <Elt>(word x0)=(couple y zero).
	Discriminate.
	Rewrite egal_x;Assumption.

Apply sym_equal;Auto.

Auto.
Apply sym_equal;Auto.

Intro dans_x.
Replace (injproducg V1 x) with (injgauche x).
ElimType (dans y X)\/(dans y V1).
Intros dans_y .
Replace (injproducg V1 y) with y.
Unfold injgauche.
Elim (Mots_X y dans_y).
Intros x0 egal_y egal2_y.
Absurd <Elt>(word x0)=(couple x zero).
	Discriminate.
	Rewrite egal2_y;Assumption.

Apply sym_equal;Auto.

Intro.
Replace (injproducg V1 y) with (injgauche y).
Unfold injgauche.
Intros.
Apply couple_couple_inv1 with zero zero;Assumption.

Apply sym_equal;Auto.

Auto.

Apply sym_equal;Auto.

Auto.
Save.
Hint is_mono_u_X_V1_injproducg_V1.


Lemma is_mono_u_X_V2_injproducd_V2 : (is_mono (union X V2)(injproducd V2)).
Unfold is_mono  .
Intros x y dans_x_u dans_y_u.
ElimType (dans x X)\/(dans x V2).
Intro dans_x.
Replace (injproducd V2 x) with x.

ElimType (dans y X)\/(dans y V2).
Intros dans_y .
Replace (injproducd V2 y) with y.
Auto.
Apply sym_equal;Auto.

Intros dans_y .
Replace (injproducd V2 y) with (injdroite y).
Unfold injdroite.
Elim (Mots_X x dans_x).
Intros x0 egal_x egal2_x.
Absurd <Elt>(word x0)=(couple y un).
	Discriminate.
	Rewrite egal_x;Assumption.

Apply sym_equal;Auto.

Auto.
Apply sym_equal;Auto.

Intro dans_x.
Replace (injproducd V2 x) with (injdroite x).
ElimType (dans y X)\/(dans y V2).
Intros dans_y .
Replace (injproducd V2 y) with y.
Unfold injdroite.
Elim (Mots_X y dans_y).
Intros x0 egal_y egal2_y.
Absurd <Elt>(word x0)=(couple x un).
	Discriminate.
	Rewrite egal2_y;Assumption.

Apply sym_equal;Auto.

Intro.
Replace (injproducd V2 y) with (injdroite y).
Unfold injdroite.
Intros.
Apply couple_couple_inv1 with un un;Assumption.

Apply sym_equal;Auto.

Auto.

Apply sym_equal;Auto.

Auto.
Save.
Hint is_mono_u_X_V2_injproducd_V2.



Lemma egal_LG_1_1' : (l_egal (LG X V1 R1 S1) (LG X V1' R1' S1')).
Pattern 2 X.
Replace X with X1'.
Unfold X1' V1' R1' S1' GGG1 GG1 G1.
Apply egal_LG.
Auto.
Auto.
Red;Auto.

Unfold X1'; Simpl.
Apply map_Id.
Red.
Auto.

Save.
Hint egal_LG_1_1'.


Lemma egal_LG_2_2' : (l_egal (LG X V2 R2 S2) (LG X V2' R2' S2')).
Pattern 2 X.
Replace X with X2'.

Unfold X2' V2' R2' S2' GGG2 GG2 G2.
Apply egal_LG.
Auto.
Auto.
Red;Auto.

Unfold X2'; Simpl.
Apply map_Id.
Red.
Auto.

Save.
Hint egal_LG_2_2'.



Lemma egal_X_X1' : <Ensf>X1'=X.
Unfold X1'. Simpl.
Apply map_Id.
Red.
Auto.
Save.

Lemma egal_X_X2' : <Ensf>X2'=X.
Unfold X2'. Simpl.
Apply map_Id.
Red.
Auto.
Save.


Lemma Grammaire1' : (isGram X V1' R1' S1').
Rewrite <- egal_X_X1'.
Unfold X1' V1' R1' S1' GGG1 GG1 G1.
Apply image_isGram.
	Auto.

	Change (Mots X1').
	Rewrite egal_X_X1'.
	Auto.

	Apply inter_Xim_Vim_empty;Auto.

Save.
Hint Grammaire1'.


Lemma Grammaire2' : (isGram X V2' R2' S2').
Rewrite <- egal_X_X2'.
Unfold X2' V2' R2' S2' GGG2 GG2 G2.
Apply image_isGram.
	Auto.

	Change (Mots X2').
	Rewrite egal_X_X2'.
	Auto.

	Apply inter_Xim_Vim_empty;Auto.

Save.
Hint Grammaire2'.

Lemma inter_V1'_V2'_empty : (inter V1' V2' empty).
Unfold V1' V2';Simpl.
Unfold inter.
Split; [Auto | Split ].
Auto.

Replace (map (injproducg V1) V1) with (map injgauche V1).
Replace (map (injproducd V2) V2) with (map injdroite V2).
Intros x dans_map_1 dans_map_2.
ElimType (<Elt>Ex ([y:Elt](dans y V1)/\(<Elt>x=(injgauche y)))).
ElimType (<Elt>Ex ([y:Elt](dans y V2)/\(<Elt>x=(injdroite y)))).
Intros x2 temp;Elim temp;Clear temp;Intros dans_x2_V2 egal_x2.
Intros x1 temp;Elim temp;Clear temp;Intros dans_x1_V1 egal_x1.
Absurd <Elt>zero=un.
Unfold zero un.
Red.
Intro.
Cut <nat>O=(S O).
Change ~(<nat>O=(S O)).
Auto.
Change <nat>(natural_inv (natural O))=(natural_inv (natural (S O))) .
Apply (f_equal Elt nat natural_inv);Assumption.
Apply couple_couple_inv2 with x1 x2.
Replace (couple x1 zero) with x.
Auto.
Apply dans_map;Assumption.
Apply dans_map;Assumption.
Auto.
Auto.
Save.

Hint inter_V1'_V2'_empty.



Lemma egal_LG_u_1'_2' : (l_egal (LG X Vu Ru S')
			(lunion (LG X V1' R1' S1')
				(LG X V2' R2' S2'))
			).
Unfold Ru Vu C' Gim.
Apply Gunion_disj_LG.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Save.
Hint egal_LG_u_1'_2'.



Theorem union_is_LCF : (l_egal (LG X Vu Ru S')
                               (lunion (LG X V1 R1 S1)
                               (LG X V2 R2 S2))).
ElimType (l_egal (LG X V1 R1 S1) (LG X V1' R1' S1')).
ElimType (l_egal (LG X V2 R2 S2) (LG X V2' R2' S2')).
ElimType (l_egal (LG X Vu Ru S')
			(lunion (LG X V1' R1' S1')
				(LG X V2' R2' S2'))
			).

Intros incl_i_u_1'_2' incl_u_1'_2'_u_i incl_2_2' incl_2'_2 incl_1_1' incl_1'_1.

Unfold l_egal; Split;Unfold l_inclus.
	Intros w LG_Ru_w.
	ElimType ((lunion (LG X V1' R1' S1')
		(LG X V2' R2' S2'))
		w);
Unfold lunion.
Intro Hyp; Apply or_introl.
Apply incl_1'_1;Assumption.

Intro Hyp; Apply or_intror.
Apply incl_2'_2;Assumption.

Change ((lunion (LG X V1' R1' S1') (LG X V2' R2' S2')) w).
Apply incl_i_u_1'_2';Assumption.

Intros w lunion_LG_1_2_w.
Apply incl_u_1'_2'_u_i.
Unfold lunion.
Elim lunion_LG_1_2_w; Intro LG_w.
Apply or_introl; Apply incl_1_1';Assumption.
Apply or_intror; Apply incl_2_2';Assumption.

Change (l_egal (LG X Vu Ru S')
			(lunion (LG X V1' R1' S1')
				(LG X V2' R2' S2'))
			).

Auto.

Change (l_egal (LG X V2 R2 S2) (LG X V2' R2' S2')).
Auto.

Change (l_egal (LG X V1 R1 S1) (LG X V1' R1' S1')).
Auto.

Save.


End gram5.
