(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               more_words.v                               *)
(****************************************************************************)
(*              Formal Language Theory                                      *)
(*                                                                          *)
(*              Judicael Courant - Jean-Christophe Filliatre                *)
(*                                                                          *)
(*              Developped in V5.8  June-July 1993                          *)
(*              Ported     to V5.10 October 1994                            *)
(****************************************************************************)

Require Ensf.
Require Words.

Hint Unfold eqwordset conj.

Definition l_inclus : wordset->wordset->Prop :=
[l1,l2:wordset](w:Word)(l1 w)->(l2 w).

Hint Unfold l_inclus.

Lemma refl_l_inclus : (l1:wordset)(l_inclus l1 l1).
Auto.
Save.
Hint refl_l_inclus.

Lemma trans_l_inclus : (l1,l2,l3:wordset)(l_inclus l1 l2)->(l_inclus l2 l3)->(l_inclus l1 l3).
Auto.
Save.


Definition l_egal : wordset->wordset->Prop :=
[l1,l2:wordset]((l_inclus l1 l2)/\(l_inclus l2 l1)).

Hint Unfold l_egal.

(*predicat equivalent a eqwordset*)
(*demonstration :                *)

Lemma equiv_l_egal_eqwordset : (a,b:wordset)(l_egal a b)<->(eqwordset a b).
Intros a b.
Unfold iff.
Split.
Intro Hyp; Elim Hyp; Auto.
Intros Hyp.
Split ; Unfold l_inclus ;Intro w ; Elim (Hyp w);Auto.
Save.


Lemma refl_l_egal : (l1:wordset)(l_egal l1 l1).
Auto.
Save.
Hint refl_l_egal.




Section more_about_words.

Variable f:Elt->Elt. 

Local wef := (Word_ext f).

(* 
Lemma wef_cons : (a:Elt)(u:Word)(wef (cons a u))=(cons (f a) (wef u)).
Proof [a:Elt][u:Word](refl_equal Word (wef (cons a u))).
*)

Lemma wef_append : (u,v:Word)(wef (Append u v))=(Append (wef u) (wef v)).
Intros u v.
Elim u.

	Trivial.

	Unfold wef.
	Intros x w H.
	Simpl.

	Rewrite <- H.
	Reflexivity.

Save.

Lemma wef_nil : (a:Word) (wef a)=nil -> a=nil.
Intro a.
Case a.
Auto.

Unfold wef ; Simpl ; Intros x w H ; Discriminate  H.

Save.


Lemma wef_cons :
(a,b:Word)(e:Elt)
(cons e a)=(wef b)->
(Ex [x:Elt](Ex2 ([w:Word](cons x w)=b)
		([w:Word]((f x)=e /\ (wef w)=a))
			      )
		   ).

Intros a b e.
Unfold wef.
Case b.
	Simpl ; Intro H ; Discriminate  H.

	Simpl ; Intros x w H.
	Exists x .
	Exists w.
		Trivial.
		Injection H ; Auto.

Save.

End more_about_words.

Hint wef_cons.

Lemma Append_assoc : (a,b,c:Word)(Append a (Append b c))=(Append (Append a b) c).
Intros a b c. 
Unfold Append.
Elim a ;Auto.
Save.
Hint Append_assoc.
