(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              July 1st 1995                               *)
(*                                                                          *)
(****************************************************************************)
(****************************************************************)
(*      Proof of a multiplier circuit                           *)
(*      C. Paulin-Mohring, June 95                              *)
(****************************************************************)
(*              Circ.v                                          *)
(*              Representation of Circuits                      *)
(****************************************************************)

Require GFP.
Require Streams.
Require Linear.

Section Circuits.

Variables TI,TR,TO : Set.

Variable output : TI->TR->TO.
Variable update : TI->TR->TR.

Definition CIRC : TR->(Str TI)->(Str TO) :=
   [ri:TR][si:(Str TI)]
   (StrCoIt [x:(Str TI)*TR]
            <TO*((Str TI)*TR)>let (s,r) = x in 
            ((output (Hd s) r),((Tl s),(update (Hd s) r)))
            (si,ri)).
Local Circ := CIRC.

Theorem Hd_Circ : (r:TR)(si:(Str TI))(Hd (Circ r si))=(output (Hd si) r).
Trivial.
Qed.

Theorem Tl_Circ : (r:TR)(si:(Str TI))(Tl (Circ r si))=(Circ (update (Hd si) r) (Tl si)).
Trivial.
Qed.

Definition Reg : ((Str TI)*TR)->TR.
Destruct 1; Trivial.
Defined.

Definition Input : ((Str TI)*TR)->(Str TI).
Destruct 1; Trivial.
Defined.

Section CircProof.
Variable i0 : (Str TI).
Variable r0 : TR.
Variable P : nat->TO->Prop.

Variable  inv : nat->TR->Prop.
Hypothesis inv_init : (inv O r0).
Hint inv_init.

Local inv_aux : nat->(Str TO)->Prop :=
  [n:nat][s:(Str TO)](Ex [r:TR](inv n r)/\(s=(Circ r (NthTl i0 n)))).

Remark inv_aux_init : (inv_aux O (Circ r0 i0)).
Exists r0; Auto.
Qed.
Hint inv_aux_init.

Hypothesis inv_stable : 
(n:nat)(r:TR)(inv n r)
  ->(P n (output (Nth i0 n) r)) /\ (inv (S n) (update (Nth i0 n) r)).

Remark inv_aux_stable : 
   (n:nat)(s:(Str TO))(inv_aux n s)->(P n (Hd s))/\(inv_aux (S n) (Tl s)).
Induction 1; Induction 1;Intros.
Rewrite H2; Rewrite Tl_Circ; Rewrite Hd_Circ.
Elim (inv_stable n x); Intros; Trivial.
Split; Trivial.
Exists (update (Nth i0 n) x); Auto.
Qed.
Hint inv_aux_stable.

Theorem Circ_property : (n:nat)(P n (Nth (Circ r0 i0) n)).
Intro; Apply All_Q_Nth with Inv:=inv_aux; Auto.
Qed.

End CircProof.

End Circuits.

Syntactic Definition Circ := (CIRC ? ? ?).
Syntax constr circ <<(CIRC $1 $2 $3 $t1)>> 1 <$t:"term":L>
   with $t := (APPLIST Circ $t1).

