Require Acc_and_Inf.

Section Arithmetic_with_an_explicit_infinity.

CoInductive Set Nat := 
    Z    : Nat |
    SUCC : Nat->Nat.

Theorem Nat_unfold : (x:Nat)(x=(<Nat>Case x of Z SUCC end)).
Intros.
Case x.
Try Trivial.
Intros.
Try Trivial.
Qed.

CoInductive EqNat : Nat->Nat->Prop :=
   eqZ    : (EqNat Z Z) |
   eqSUCC : (n,m:Nat)(EqNat n m)->(EqNat (SUCC n) (SUCC m)). 

Lemma EqNat_reflex : (x:Nat)(EqNat x x).
Cofix u.
Destruct x.
Apply eqZ.
 
Intros.
Apply eqSUCC.
Apply u.
Qed.

Lemma EqNat_inj : (x,y:Nat)x=y->(EqNat x y).
Intros.
Rewrite -> H.
Apply EqNat_reflex.
Qed.

Inductive Le  [n:Nat] :  Nat -> Prop
    := Le_n : (Le n (SUCC n))
     | Le_S : (m:Nat)(Le n m)->(Le n (SUCC m)).


Lemma Nat_Induction : 
  (x:Nat)(Acc Nat Le x)->
    (P:Nat->Prop)(P Z)->((x:Nat)(P x)->(P (SUCC x)))->(P x).
Intros x H.
Elim H.
Intros x0.
Case x0.
Intros.
Assumption.
Intros.
Apply H3.
Apply (H1 n).
Apply Le_n.
Assumption.
Assumption.
Qed.


Lemma is_definitional_for_finite_numbers :
 (x:Nat)(Acc Nat Le x)->(y:Nat)(EqNat x y)->x=y.
Intros x H.
Apply (Nat_Induction x H).
Intros.
Inversion_clear H0.
Try Trivial.
Intros.
Inversion_clear H1.
Cut (eq ? x0 m).
Intros.
Rewrite -> H1.
Try Trivial.
Apply H0.
Assumption.
Qed.


CoFixpoint Plus  : Nat->Nat->Nat :=
 [n,m:Nat]<Nat>Case n of
		    m
		    [p:Nat](SUCC (Plus p m))
	       end.

Fixpoint ntoN [n:nat] : Nat :=
  <Nat>Case n of
	    Z
	    [m:nat](SUCC (ntoN m))
	end.

Lemma natInj : (n,m:nat)(ntoN n)=(ntoN m)->n=m.
Fix 1.
Case n.
Destruct m.
Try Trivial.
 
Simpl.
Intros.
Cut (not (eq ? Z (SUCC (ntoN n0)))).
Intros.
Case H0.
Assumption.
Discriminate.
 
Destruct m.
Simpl.
Intros.
Cut (not (eq ? (SUCC (ntoN n0)) Z)).
Intros.
Case H0.
Assumption.
Discriminate.
 
Simpl.
Intros.
Cut (eq ? n0 n1).
Intros.
Rewrite -> H0.
Try Trivial.
Apply (natInj n0 n1).
Injection H 1.
Qed.


Section The_Infinity.

CoFixpoint OO : Nat := (SUCC OO).

Definition OO_unfold : OO=(SUCC OO) := (Nat_unfold OO).


Lemma Plus_Idemp : (x:Nat)(EqNat (Plus OO x) OO).
Cofix u.
Intros.
Rewrite -> OO_unfold.
Rewrite -> (Nat_unfold (Plus (SUCC OO) x)).
Simpl.
Apply eqSUCC.
Apply u.
Qed.


Lemma OO_is_infinite : (Inf Nat Le OO).
Cofix u.
Rewrite -> OO_unfold.
Apply (inf Nat Le (SUCC OO) OO).
Apply (Le_n  OO).
Apply u.
Qed.


Lemma OO_is_not_Acc : ~(Acc Nat Le OO).
Red.
Intros.
Apply (NotBoth Nat Le OO).
Assumption.
Apply OO_is_infinite.
Qed.

Lemma only_OO_expands_forever : (x:Nat)(x=(SUCC x))->(EqNat x OO).
Cofix u.
Intros.
Rewrite -> H.
Rewrite -> OO_unfold.
Apply eqSUCC.
Apply u.
Assumption.
Qed.

Lemma Z_is_minimum : (x:Nat)(Acc Nat Le x)->(Le Z (SUCC x)).
Intros.
Apply (Nat_Induction x H).
Apply Le_n.
Intros.
Apply Le_S.
Assumption.
Qed.

(* 

This lemma is not true if we take Le as being Inductive :

Lemma OO_is_maximum : (x:Nat)(Acc Nat Le x)->(Le x OO).

*)


Lemma only_OO_is_grater_than_OO :
  (y:Nat)(Le OO y)->(EqNat OO y).
Cofix u.
Intros y H.
Case H.
Apply EqNat_inj.
Apply OO_unfold.
Intros.
Rewrite -> OO_unfold.
Apply eqSUCC.
Apply u.
Assumption.
Qed.


Lemma all_infinite_is_OO :
  (x:Nat)(Inf Nat Le x)->(EqNat x OO).
Cofix u.
Rewrite -> OO_unfold.
Intros.
Case H.
Intros.
Inversion_clear H0.
Apply eqSUCC.
Apply u.
Assumption.
Apply eqSUCC.
Apply u.
Apply (inf Nat Le m y).
Assumption.
Assumption.
Qed.

(* Strange phenomena *)

Lemma OO_is_infiniteII : 
  (Inf Nat [x,y:Nat](Le y x) OO).
Cofix u.
Apply (inf Nat [x,y:Nat](Le y x) OO OO).
Pattern 2 OO .
Rewrite -> OO_unfold.
Apply Le_n.
Apply u.
Qed.


End The_Infinity.

End Arithmetic_with_an_explicit_infinity.
