(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                                June 1995                                 *)
(*                                                                          *)
(****************************************************************************)
(*                              Protocol.v                                  *)
(****************************************************************************)
(*           An interpreter for a Calculus of Broadcasting Systems          *)
(*                                                                          *)
(*                            Eduardo Gimenez                               *)
(*                                                                          *)
(****************************************************************************)


Require Export Bool.
Require Export Streams.
Require Export Process.


(***********************************)
  Section Alternating_Bit_Protocol.
(***********************************)

(* The process will transmit on two different channels. 
   Channel one is for syncornisation, channel two is to communicate outside *)

(* The language of actions spoken by the processes *)

Variable A : Set.

Inductive Set Act := 
	Send  : bool->A->Act |
	Ack   : bool->Act    |
	Relay : A   -> Act.


(* The Process who sends the messages *)

CoFixpoint send :bool->(Stream A)->(Process Act) := 
 [b:bool]
   [s:(Stream A)]
     (TALK Act  (Send b (hd s)) (sending b s) [_:Act](send b s))

with sending : bool->(Stream A)->(Process Act) :=
 [b:bool]
   [s:(Stream A)] 
       (TALK Act 
             (Send b (hd s)) (sending b s) 
      	     [v:Act]<(Process Act)>Case v of
			       [b1:bool][a:A](sending b s)
			       [b1:bool]<(Process Act)>if (eqb b1 b) 
					         then (send (neg b) (tl s)) 
					         else (send b s)
			       [a:A] (sending b s)
			      end  ).




(*  The Process which re-transmites the values in channel 2     *) 
(*   and acknowledges the sender in channel 1                   *)

CoFixpoint acking :  bool->(Process Act) :=
 [b:bool]
    (LISTEN Act 
      	    [v:Act]<(Process Act)>Case v of
			      [b1:bool][a:A]<(Process Act)>if (eqb b1 (neg b)) 
					             then (out (neg b) a)
					             else (ack b)
			      [_:bool](acking b)
			      [_:A]   (acking b)
			     end  )
with out   :  bool->A->(Process Act) := 
 [b1:bool][a:A](TALK Act (Relay a) (ack b1) [_:Act](out b1 a))

with ack  : bool->(Process Act) := 
 [b:bool](TALK Act (Ack b) (acking b) [_:Act](ack b) ).
 

(* The protocol itself *)

Definition abp := [b:bool][s:(Stream A)](PAR Act (sending b (cons (hd s) s))  (ack b)).

End  Alternating_Bit_Protocol.


CoFixpoint from : nat->(Stream nat) :=
  [n:nat](cons O (from (S n))).

Definition example := (abp nat true (from O)).
