(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*                           Solange Coupet-Grimal                          *)
(*                                                                          *)
(*                                                                          *)                      
(*                   Laboratoire d'Informatique de Marseille                *)
(*                   URA CNRS 1787                                          *)
(*                   Technopole de Chateau-Gombert                          *)
(*                   39, Rue F. Joliot Curie                                *)
(*                   13453 MARSEILLE Cedex 13                               *)
(*                   e-mail:solange@gyptis.univ-mrs.fr                      *)
(*                                                                          *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                               Feb 7th 1995                               *)
(*                                                                          *)
(****************************************************************************)
(*                            Transition systems                            *)
(****************************************************************************)


Section TS.

Variables process, action: Set.
Variable T:action.
Variable transition:process->action->process->Prop.
Hypothesis eqT_dec:(a:action)(a=T)\/~(a=T).

(*************************** Derivatives **********************************)

Inductive weak_derivative :process->action->process->Prop:=
eps:(p:process)(weak_derivative p T p)|
w_single:(a:action)(p,q:process)(transition p a q)->(weak_derivative p a q)|
w_tau_left:(a:action)(p,p',q:process)(transition p T p')->
                                   (weak_derivative p' a q)->
                                   (weak_derivative p a q)|
w_tau_right:(a:action)(p,q,q':process)(transition q' T q)->
                                  (weak_derivative p a q')->
                                  (weak_derivative p a q).
Hint w_single eps.

Inductive derivative :process->action->process->Prop:=
single:(a:action)(p,q:process)(transition p a q)->(derivative p a q)|
tau_left:(a:action)(p,p',q:process)(transition p T p')->(derivative p' a q)->
                                                           (derivative p a q)|
tau_right:(a:action)(p,q,q':process)(transition q' T q)->(derivative p a q')->
                                                           (derivative p a q).

Hint single.

Lemma deriv_weak_deriv:(p,p':process)(a:action)
                       (derivative p a p')->(weak_derivative p a p').

(Intros p p' a H; Elim H; Clear  H; Auto).
Clear  p p' a.
Intros a p p' q tr de we.
(Apply w_tau_left with p'; Try Trivial).

(Clear  p p' a; Intros a p q q' tr de we).
(Apply w_tau_right with q'; Try Trivial).

Save.

Hint deriv_weak_deriv.

Lemma weak_deriv_deriv:(p,p':process)(a:action)
                       (weak_derivative p a p')->~a=T->(derivative p a p').
(Intros p p' a H; Elim H; Clear  H p p' a).
Unfold not .
(Intros p H; Cut False; Auto).
(Intros H1; Elim H1).

(Intros a p q tr H; Auto).

Intros a p p' q tr we H_ind not_eq.
(Apply tau_left with p'; Auto).

Intros a p q q' tr we H_ind notEq.
(Apply tau_right with q'; Auto).
Save.

Hint weak_deriv_deriv.

Lemma weak_deriv_tau_left: (p,p',p'':process)(a:action)
    (weak_derivative p T p')-> (weak_derivative p' a p'')->
                               (weak_derivative p a p'').

Cut (p:process)
     (a:action)
      (p':process)
       (weak_derivative p a p')
        ->(eq ? a T)
           ->(p'':process)
              (b:action)
               (weak_derivative p' b p'')->(weak_derivative p b p'').
(Intros H p p' p'' a we we'; Apply (H p T p'); Auto).

Induction 1.
(Induction 1; Auto).

Clear  H p' a p.
Intros a p q Tpq eqT p'' b we.
Rewrite eqT in Tpq.
(Apply w_tau_left with q; Auto).

Clear  H p p' a.
Intros a p p' q Tpq we H_rec eqT p'' b we'.
(Apply w_tau_left with p'; Auto).

Clear  H p a p'.
Intros a p q q' Tr we H_rec eqT p'' b we'.
Apply (H_rec eqT).
(Apply w_tau_left with q; Auto).

Save.

Lemma weak_deriv_tau_right:(p,p',p'':process)(a:action)
       (weak_derivative p a p') -> (weak_derivative p' T p'')
                                -> (weak_derivative p a p'').

Cut (p':process)
     (a:action)
      (p'':process)
       (weak_derivative p' a p'')
        ->(eq ? a T)
           ->(p:process)
              (b:action)
               (weak_derivative p b p')->(weak_derivative p b p'').
(Intros H p p' p'' a we we1; Apply (H p' T p''); Auto).

(Intros p' a p'' H; Elim H; Clear  H a p' p''; Auto).
Intros a p q tr eqT p0 b we.
(Apply w_tau_right with p; Auto).
(Rewrite eqT in tr; Auto).

Intros a p q q' tr we H_rec eqT p0 b we'.
Apply (H_rec eqT).
(Apply w_tau_right with p; Auto).

Intros a p q q' tr we H_rec eqT p0 b we'.
(Apply w_tau_right with q'; Auto).

Save.


Lemma w_deriv_tau_left:(p,p',p'':process)(a:action)
(weak_derivative p T p')->(derivative p' a p'')->(derivative p a p'').

Cut (p:process)
     (a:action)
      (p':process)
       (weak_derivative p a p')
        ->(eq ? a T)
           ->(p'':process)
              (b:action)(derivative p' b p'')->(derivative p b p'').
(Intros H p p' p'' a de de1; Apply (H p T p'); Auto).

(Intros p a p' H; Elim H; Clear  H a p p'; Auto).
Intros a p q Tpq eqT p'' b de.
Rewrite eqT in Tpq.
(Apply tau_left with q; Auto).

Intros a p p' q tr de H_ind eqT p'' d de'.
(Apply tau_left with p'; Auto).

Intros a p q q' Tr de H_ind eqT p'' b de'.
Apply (H_ind eqT).
(Apply tau_left with q; Auto).

Save.

Lemma w_deriv_tau_right:(p,p',p'':process)(a:action)
(derivative p a p')->(weak_derivative p' T p'')->
(derivative p a p'').

Cut (p':process)
     (a:action)
      (p'':process)
       (weak_derivative p' a p'')
        ->(eq ? a T)
           ->(p:process)
              (b:action)(derivative p b p')->(derivative p b p'').
(Intros H p p' p'' a de de1; Apply (H p' T p''); Auto).

(Intros p' a p'' H; Elim H; Clear  H a p' p'').
Auto.

Intros a p q tr eqT p0 b we.
(Apply tau_right with p; Auto).
(Rewrite eqT in tr; Auto).

Intros a p q q' tr we H_rec eqT p0 b we'.
Apply (H_rec eqT).
(Apply tau_right with p; Auto).

Intros a p q q' tr we H_rec eqT p0 b we'.
(Apply tau_right with q'; Auto).

Save.

Lemma deriv_tau_left:(p,p',p'':process)(a:action)
(derivative p T p')->(derivative p' a p'')->(derivative p a p'').

Intros p p' p'' a pp' p_p''.
(Apply w_deriv_tau_left with p'; Auto).
Save.

Lemma deriv_tau_right:(p,p',p'':process)(a:action)
(derivative p a p')->(derivative p' T p'')->
(derivative p a p'').

Intros p p' p'' a de1 de2.
(Apply w_deriv_tau_right with p'; Auto).
Save.

(************************* Strong equivalence ******************************)

CoInductive strong_eq:process->process->Prop:=
str_eq:(p,q:process)((a:action)(p':process)(transition p a p')->
        (Ex [q':process] (transition q a q')/\(strong_eq p' q')))->
((a:action)(q':process)(transition q a q')->
        (Ex [p':process] (transition p a p')/\(strong_eq p' q')))->
(strong_eq p q).

(******************* strong_eq is an equivalence relation ******************)


Lemma refl_strong_eq:(p:process) (strong_eq p p).
Cofix.
(Intros p; Apply str_eq).
(Intros a p' trans; Exists  p'; Split; Auto).
(Intros a p' trans; Exists  p'; Split; Auto).
Save.

Hint refl_strong_eq.

Lemma sym_strong_eq:(p,q:process) (strong_eq p q)->(strong_eq q p).
Cofix.
(Intros p q H; Inversion_clear H).
Apply str_eq.
Intros a q'.
Intros trans.
(Elim (H1 a q' trans); Clear  H1).
(Intros p' H; Elim H; Exists  p'; Split; Auto).
Intros a p' trans.
(Elim (H0 a p' trans); Clear  H0).
(Intros q' H; Elim H; Clear  H).
Intros trans1 str.
(Exists  q'; Split; Auto).
Save.

Immediate sym_strong_eq.

Lemma trans_strong_eq:(p,q,r:process) (strong_eq p q)->(strong_eq q r)->
(strong_eq p r).

Cofix.
Intros p q r pq qr.
(Inversion_clear pq; Inversion_clear qr).
Apply str_eq.
Intros a p' tp.
Cut (Ex [q':process](and (transition q a q') (strong_eq p' q'))).
(Intros G; Elim G; Clear  G).
(Intros q' G; Elim G).
(Intros tq pq'; Elim (H1 a q' tq)).
Clear  G.
(Intros r' G; Elim G; Clear  G).
(Intros tr qr'; Exists  r'; Split; Auto).
(Apply trans_strong_eq with q'; Auto).
Elim (H a p' tp).
(Intros q' G; Elim G; Clear  G).
Intros tq pq'.
(Exists  q'; Split; Auto).
Intros a r' tr.
Cut (Ex [q':process](and (transition q a q') (strong_eq q' r'))).
(Intros G; Elim G; Clear  G).
(Intros q' G; Elim G; Clear  G).
Intros tq qr'.
Elim (H0 a q' tq).
(Intros p' G; Elim G; Clear  G).
(Intros tp pq'; Exists  p'; Split; Auto).
(Apply trans_strong_eq with q'; Auto).
Elim (H2 a r' tr).
(Intros q' G; Elim G; Clear  G).
(Intros tq qr'; Exists  q'; Split; Auto).
Save.

(**************************** Weak equivalence *****************************)

CoInductive weak_eq:process->process->Prop:=
w_eq:(p,q:process)

((a:action)(p':process)(transition p a p')->
        (Ex [q':process] (weak_derivative q a q')/\(weak_eq p' q')))->

((a:action)(q':process)(transition q a q')->
        (Ex [p':process] (weak_derivative p a p')/\(weak_eq p' q')))->

(weak_eq p q).

(****************** weak_eq is an equivalence Relation *********************)

Lemma refl_weak_eq:(p:process) (weak_eq p p).
Cofix.
(Intros p; Apply w_eq).
(Intros a p' trans; Exists  p'; Split; Auto).
(Intros a p' trans; Exists  p'; Split; Auto).
Save.

Hint refl_weak_eq.

Lemma sym_weak_eq:(p,q:process) (weak_eq p q)->(weak_eq q p).
Cofix.
(Intros p q H; Inversion_clear H).
Apply w_eq.
Intros a q'.
Intros trans.
(Elim (H1 a q' trans); Clear  H1).
(Intros p' H; Elim H; Exists  p'; Split; Auto).
Intros a p' trans.
(Elim (H0 a p' trans); Clear  H0).
(Intros q' H; Elim H; Clear  H).
Intros trans1 str.
(Exists  q'; Split; Auto).
Save.

Immediate sym_weak_eq.


        (*___________ Proof of Transitivity ______________*)


Remark Hint_Trans:(q,q',r:process)(a:action)(weak_derivative q a q')->
(weak_eq q r)->(Ex [r':process] (weak_derivative r a r')/\(weak_eq q' r')).

Intros q q' r a H.
(Generalize r ; Clear  r).
(Elim H; Clear  H).
(Intros p r we; Exists  r; Split; Auto).

(Clear  a q q'; Intros a p q tr r we).
Inversion_clear we.
Apply (H a q tr).

(Clear  q q' a; Intros a p p' q tr wd H_ind r we).
Cut (Ex [r'':process](and (weak_derivative r T r'') (weak_eq p' r''))).
Intros H.
(Elim H; Clear  H).
(Intros r'' Hr''; Elim Hr''; Clear  Hr''; Intros rTr'' we').
Cut (Ex [r':process](and (weak_derivative r'' a r') (weak_eq q r'))).
(Intros H; Elim H; Clear  H).
(Intros r' Hr'; Elim Hr'; Clear  Hr'; Intros wde we'').
(Exists  r'; Split; Auto).
(Apply (weak_deriv_tau_left r r'' r' a); Auto).

(Apply (H_ind r''); Auto).

Inversion_clear we.
(Apply (H T p' tr); Auto).

(Clear  q q' a; Intros a p q q' tr wd H_ind r we).
Cut (Ex [r'':process](and (weak_derivative r a r'') (weak_eq q' r''))).
(Intros H; Elim H; Clear  H).
(Intros r'' Hr''; Elim Hr''; Clear  Hr''; Intros wd' we').
Cut (Ex [r':process](and (weak_derivative r'' T r') (weak_eq q r'))).
(Intros H; Elim H; Clear  H).
(Intros r' Hr'; Elim Hr'; Intros wd'' we'').
(Exists  r'; Split; Auto).
(Apply (weak_deriv_tau_right r r'' r' a); Auto).

(Inversion_clear we'; Auto).

(Apply (H_ind r we); Auto).

Save.

Lemma trans_weak_eq:(p,q,r:process) (weak_eq p q)->(weak_eq q r)->
(weak_eq p r).
Cofix.
Intros p q r pq qr.
Apply w_eq.

Intros a p' tr.
Cut (Ex [q':process](and (weak_derivative q a q') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H; Intros q' Hq'; Elim Hq'; Clear  Hq';
 Intros wd we).
Cut (Ex [r':process](and (weak_derivative r a r') (weak_eq q' r'))).
(Intros Hr'; Elim Hr'; Clear  Hr'; Intros r' Hr'; Elim Hr'; Clear  Hr';
 Intros wd' we').
(Exists  r'; Split; Auto).
(Apply trans_weak_eq with q'; Auto).
(Apply (Hint_Trans q q' r); Auto).
(Inversion_clear pq; Auto).

Intros a r' tr.
Cut (Ex [q':process](and (weak_derivative q a q') (weak_eq q' r'))).
(Intros H; Elim H; Clear  H; Intros q' Hq'; Elim Hq'; Clear  Hq';
 Intros wd we).
Cut (Ex [p':process](and (weak_derivative p a p') (weak_eq q' p'))).
(Intros H; Elim H; Clear  H; Intros p' Hp'; Elim Hp'; Clear  Hp';
 Intros wd' we').
(Exists  p'; Split; Auto).
(Apply trans_weak_eq with q'; Auto).
(Apply (Hint_Trans q q' p a); Auto).
(Inversion_clear qr; Auto).
Save.


(******** weaq_eq1 : an equivalent definition for weak equivalence *********)


CoInductive weak_eq1:process->process->Prop:=
w_eq1:(p,q:process)

((a:action)(p':process)(derivative p a p')->
        (Ex [q':process] (weak_derivative q a q')/\(weak_eq1 p' q')))->

((a:action)(q':process)(derivative q a q')->
        (Ex [p':process] (weak_derivative p a p')/\(weak_eq1 p' q')))->

(weak_eq1 p q).

Lemma weak_eq_deriv:(p,q:process)(weak_eq p q)->
((a:action)(p':process)(derivative p a p')->
        (Ex [q':process] (weak_derivative q a q')/\(weak_eq p' q'))).

Intros p q we a p' de.
Generalize we .
Generalize q .
(Elim de; Clear  de we a p p' q).
Intros a p p' tr q we.
Inversion we.
(Elim (H a p' tr); Clear  H H0).
(Intros q' H; Elim H; Clear  H; Intros wde we').
(Exists  q'; Split; Assumption).

Intros a p p1 p' tr de H_ind q we.
Cut (Ex [q1:process](and (weak_derivative q T q1) (weak_eq p1 q1))).
(Intros H; Elim H; Clear  H).
(Intros q1 H; Elim H; Clear  H; Intros wde we').
Cut (Ex [q':process](and (weak_derivative q1 a q') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H).
(Intros q' H; Elim H; Clear  H; Intros wde1 we1).
(Exists  q'; Split; Try Try Try Trivial).
(Apply weak_deriv_tau_left with q1; Try Try Trivial).

(Apply (H_ind q1); Try Try Trivial).

Inversion_clear we.
(Apply (H T p1); Try Try Trivial).

Intros a p p' p1 tr de H_ind q we.
Cut (Ex [q1:process](and (weak_derivative q a q1) (weak_eq p1 q1))).
(Intros H; Elim H; Clear  H).
(Intros q1 H; Elim H; Clear  H; Intros wde we1).
Cut (Ex [q':process](and (weak_derivative q1 T q') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H).
(Intros q' H; Elim H; Clear  H; Intros wde1 we').
(Exists  q'; Split; Try Try Try Trivial).
(Apply weak_deriv_tau_right with q1; Try Try Trivial).

Inversion_clear we1.
(Apply (H T p'); Try Try Trivial).

Apply (H_ind q we).
Save.

Lemma weak_eq_deriv_sym:(p,q:process)(weak_eq p q)->
((a:action)(q':process)(derivative q a q')->
        (Ex [p':process] (weak_derivative p a p')/\(weak_eq p' q'))).

Intros p q we a q' de.
Cut (weak_eq q p);Auto.
Intros we'.
Elim (weak_eq_deriv q p we' a q' de).
(Intros p' H; Elim H; Intros w_de we'').
(Exists  p'; Split; Auto).

Save.


Lemma weak_eq1_weak_eq:(p,q:process)(weak_eq1 p q)->(weak_eq p q).
Cofix.
Intros p q we1.
Apply w_eq.
Intros a p' tr.
Inversion we1.
Elim (H a p').
(Intros q' G; Elim G; Clear  G; Intros wde we1').
(Exists  q'; Split; Try Try Try Trivial).
(Apply weak_eq1_weak_eq; Assumption).

(Apply single; Assumption).

Intros a q' tr.
Inversion we1.
Elim (H0 a q').
(Intros p' G; Elim G; Intros wde we1').
(Exists  p'; Split; Try Try Try Trivial).
(Apply weak_eq1_weak_eq; Assumption).

(Apply single; Try Try Trivial).
Save.

Immediate weak_eq1_weak_eq.

Lemma weak_eq_weak_eq1:(p,q:process)(weak_eq p q)->(weak_eq1 p q).

Cofix.
Intros p q we.
Apply w_eq1.
Intros a p' de.
Cut (Ex [q':process](and (weak_derivative q a q') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H).
(Intros q' H; Elim H; Clear  H; Intros wde we').
(Exists  q'; Split; Try Try Try Trivial).
(Apply weak_eq_weak_eq1; Try Try Trivial).

Elim (weak_eq_deriv p q we a p' de).
(Intros q' H; Exists  q'; Assumption).

Intros a q' de.
Cut (Ex [p':process](and (weak_derivative p a p') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H).
(Intros p' H; Elim H; Clear  H; Intros wde we').
(Exists  p'; Split; Try Try Try Try Trivial).
(Apply weak_eq_weak_eq1; Try Try Try Trivial).

Elim (weak_eq_deriv_sym p q we a q' de).
(Intros p' H; Exists  p'; Assumption).

Save.

Immediate weak_eq_weak_eq1.


(******************** weak_eq1 is an equivalence relation ******************)

Lemma refl_weak_eq1:(p:process) (weak_eq1 p p).

Auto.
Save.

Lemma sym_weak_eq1:(p,q:process) (weak_eq1 p q)->(weak_eq1 q p).

Intros.
Apply weak_eq_weak_eq1.
(Apply sym_weak_eq; Auto).

Save.

Lemma trans_weak_eq1:(p,q,r:process) (weak_eq1 p q)->(weak_eq1 q r)->
(weak_eq1 p r).

Intros p q r pq qr.
Apply weak_eq_weak_eq1.
(Apply trans_weak_eq with q; Auto).

Save.

(****** Observational equivalence: definition, reflexivity,symmetry ******)

Definition obs_eq:process->process->Prop:= [p,q:process]
((a:action)(p':process)(transition p a p')->
        (Ex [q':process] (derivative q a q')/\(weak_eq p' q')))/\
((a:action)(q':process)(transition q a q')->
        (Ex [p':process] (derivative p a p')/\(weak_eq p' q'))).

Lemma refl_obs_eq:(p:process) (obs_eq p p).
Unfold obs_eq .
(Split; Intros a p' trans; Exists  p'; Split; Auto).
Save.

Immediate refl_obs_eq.

Lemma sym_obs_eq:(p,q:process) (obs_eq p q)->(obs_eq q p).
Unfold obs_eq .
Intros p q.
(Intros H; Elim H; Clear  H; Intros Hp Hq).
Split.
Auto.
Intros a q' tr.
(Elim (Hq a q' tr); Clear  Hq).
(Intros p' H; Elim H; Clear  H; Intros de we; Exists  p'; Split; Auto).
Intros a p' tr.
(Elim (Hp a p' tr); Clear  Hp).
(Intros q' H; Elim H; Clear  H; Intros de we; Exists  q'; Split; Auto).
Save.

Immediate sym_obs_eq.


(**** obs_eq1 : an equivalent definition for observational equivalence ****)


Definition obs_eq1:process->process->Prop:= [p,q:process]
((a:action)(p':process)(derivative p a p')->
        (Ex [q':process] (derivative q a q')/\(weak_eq p' q')))/\
((a:action)(q':process)(derivative q a q')->
        (Ex [p':process] (derivative p a p')/\(weak_eq p' q'))).


Lemma obs_eq1_obs_eq:(p,q:process)(obs_eq1 p q)->(obs_eq p q).
Intros p q oe.
(Unfold obs_eq ; Split; Elim oe; Auto).
Save.

Hint obs_eq1_obs_eq.

Remark half_obs_eq_obs_eq1:(p,q:process)(obs_eq p q)->
       (a:action)(p':process)(derivative p a p')->
       (Ex [q':process](derivative q a q')/\(weak_eq p' q')).

(Intros p q oe a p' de; Generalize oe ; Generalize q ; Clear  oe q).
(Elim de; Clear  de a p p').
Intros a p p' tr q oe.
(Unfold obs_eq  in oe; Elim oe; Clear  oe; Intros oe_l oe_r; Clear  oe_r).
Auto.

Intros a p p1 p' tr de H_ind q oe.
Cut (Ex [q1:process](and (derivative q T q1) (weak_eq p1 q1))).
(Intros H; Elim H; Clear  H).
(Intros q1 H; Elim H; Clear  H; Intros de' we).
Cut (Ex [q':process](and (weak_derivative q1 a q') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H).
(Intros q' H; Elim H; Intros de'' we').
(Exists  q'; Split; Auto).
Elim eqT_dec with a.
Intros a_T.
Rewrite a_T in de''.
Rewrite -> a_T.
(Apply w_deriv_tau_right with q1; Auto).

Intros dif_a_T.
Apply weak_deriv_deriv.
(Apply weak_deriv_tau_left with q1; Auto).

Auto.

(Cut (weak_eq1 p1 q1); Auto).
Intros we1.
Cut (Ex [q':process](and (weak_derivative q1 a q') (weak_eq1 p' q'))).
(Intros H; Elim H; Clear  H; Intros q' H; Elim H; Intros de'' we1';
 Exists  q'; Split; Auto).

(Inversion_clear we1; Auto).

(Unfold obs_eq  in oe; Elim oe; Auto).

Intros a p p' p1 tr de H_ind q oe.
Cut (Ex [q1:process](and (derivative q a q1) (weak_eq p1 q1))).
(Intros H; Elim H; Clear  H).
(Intros q1 H; Elim H; Clear  H; Intros de1 we1).
Cut (Ex [q':process](and (weak_derivative q1 T q') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H; Intros q' H; Elim H; Intros de2 we).
(Exists  q'; Split; Auto).
(Apply w_deriv_tau_right with q1; Auto).

(Inversion_clear we1; Auto).

(Apply (H_ind q); Auto).

Save.

Lemma obs_eq_obs_eq1:(p,q:process)(obs_eq p q)->(obs_eq1 p q).

Intros p q oe.
(Unfold obs_eq1 ; Split).
Exact (half_obs_eq_obs_eq1 p q oe).

Cut (a:action)
     (q':process)
      (derivative q a q')
       ->(Ex [p':process](and (derivative p a p') (weak_eq q' p'))).
Intros H a q' de.
(Elim (H a q' de); Clear  H).
(Intros p' H; Elim H; Clear  H; Intros; Exists  p'; Split; Auto).

(Cut (obs_eq q p); Auto).
Exact (half_obs_eq_obs_eq1 q p).

Save.

Hint obs_eq_obs_eq1.

(************* obs_eq1 ans obs_eq are equivalence relations ***************)

Lemma refl_obs_eq1:(p:process) (obs_eq1 p p).

Auto.
Save.

Immediate refl_obs_eq1.

Lemma sym_obs_eq1:(p,q:process) (obs_eq1 p q)->(obs_eq1 q p).
Intros.
Apply obs_eq_obs_eq1.
Apply sym_obs_eq.
Auto.
Save.

Immediate sym_obs_eq1.

Remark half_trans_obs_eq1:(p,q,r:process)(obs_eq1 p q)->(obs_eq1 q r)->
       (a:action)(p':process)(derivative p a p')->
                 (Ex [r':process] (derivative r a r')/\(weak_eq p' r')).

Intros p q r pq qr a p' de.
Cut (Ex [q':process](and (derivative q a q') (weak_eq p' q'))).
(Intros H; Elim H; Clear  H; Intros q' H; Elim H; Intros de' we;
 Clear  H).
(Elim qr; Clear  qr; Intros qr_l qr_r; Clear  qr_r).
Elim (qr_l a q' de').
(Intros r' H; Elim H; Clear  H; Intros de'' we'; Exists  r'; Split; Auto).
(Apply trans_weak_eq with q'; Auto).

(Elim pq; Intros pq_l pq_r; Clear  pq_r);Auto.

Save.

Lemma trans_obs_eq1:(p,q,r:process)(obs_eq1 p q)->(obs_eq1 q r)->(obs_eq1 p r).

(Intros p q r pq qr; Unfold obs_eq ; Split).
Exact (half_trans_obs_eq1 p q r pq qr).

Cut (a:action)
     (r':process)
      (derivative r a r')
       ->(Ex [p':process](and (derivative p a p') (weak_eq r' p'))).
Intros H a r' de.
(Elim (H a r' de); Clear  H; Intros p' H; Elim H; Clear  H; Intros;
 Exists  p'; Split; Auto).

(Cut (obs_eq1 q p); Auto).
(Cut (obs_eq1 r q); Auto).
Exact (half_trans_obs_eq1 r q p).
Save.

Lemma trans_obs_eq:(p,q,r:process)(obs_eq p q)->(obs_eq q r)->(obs_eq p r).
Intros p q r pq qr.
Apply obs_eq1_obs_eq.
(Apply trans_obs_eq1 with q; Auto).

Save.

(*************************** p~q->p=q->p~~q *******************************)

Lemma strong_weak:(p,q:process) (strong_eq p q)->(weak_eq p q).
Cofix.
Intros p q.
Intros H.
Inversion_clear H.
Apply w_eq.
Intros a p' trans.
Elim (H0 a p' trans).
Clear  H0.
(Intros q' H; Elim H; Clear  H).
Intros trans1 Str.
Exists  q'.
Split.
Apply w_single.
Try Try Trivial.

(Apply strong_weak; Try Try Trivial).

Clear  H0.
Intros a q' trans.
(Elim (H1 a q' trans); Clear  H1).
(Intros p' H; Elim H; Clear  H).
(Intros trans1 Str; Exists  p'; Split).
(Apply w_single; Try Try Trivial).

(Apply strong_weak; Try Try Trivial).
Save.

Hint strong_weak.

Lemma strong_obs:(p,q:process) (strong_eq p q)->(obs_eq p q).

Intros p q H.
Inversion_clear H.
(Unfold obs_eq ; Split).
(Clear  H1; Intros a p' trans).
(Elim (H0 a p' trans); Clear  H0).
(Intros q' H; Elim H; Clear  H).
(Intros trans1 Str; Exists  q'; Auto).
(Clear  H0; Intros a q' trans).
(Elim (H1 a q' trans); Clear  H1).
(Intros p' H; Elim H; Clear  H).
(Intros trans1 Str; Exists  p'; Split; Auto).
Save.

Hint strong_obs.

Lemma obs_weak:(p,q:process) (obs_eq p q)->(weak_eq p q).

Unfold obs_eq .
(Intros p q H; Elim H; Clear  H; Intros H1 H2).
Apply w_eq.
Intros a p' tr.
Elim (H1 a p' tr).
(Intros q' H; Elim H; Clear  H; Intros de we).
(Exists  q'; Split; Auto).
Intros a q' tr.
Elim (H2 a q' tr).
(Intros p' H; Elim H; Clear  H; Intros de we).
(Exists  p'; Split; Auto).
Save.


End TS.
