(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              July 1st 1995                               *)
(*                                                                          *)
(****************************************************************************)

(************************************************************************)
(**************************** Comp_Circ.v *******************************)
(************************************************************************)
(*               Solange COUPET-GRIMAL and Line JAKUBIEC                *)
(*                                                                      *)
(*               Laboratoire d'Informatique de Marseille                *)
(*               URA CNRS 1787                                          *)
(*               Technopole de Chateau-Gombert                          *)
(*               39, Rue F. Joliot Curie                                *)
(*               13453 MARSEILLE Cedex 13                               *)
(*                                                                      *)
(*               July 1994                                              *)
(************************************************************************)


(** Compile Module Bool. **)
Require Export Bool.
Require Export Arith.

(** Multiplexeur **)

Definition Mux := [b:bool][n,m:nat]<nat>Case b of n m end.
                                  



Definition ZeroTest := [n:nat]<bool>Case n of true 
                                              [x:nat]false end.
    

(****)

Lemma bool_dec : (b:bool){b=true}+{b=false}.
(Induction b;Auto).
Save.
Hint bool_dec.

(****)

Lemma orb_sym : (a,b:bool)(orb a b)=(orb b a).
(Induction a;Induction b;Auto).
Save.

(****)

Lemma Zerotest_eq_O : (n:nat)(ZeroTest n)=true->n=O.
Induction n;Auto.
Intros.
(Absurd (eq ? (ZeroTest (S n0)) true);Auto).
Save.
Hint Zerotest_eq_O.

(****)

Lemma Zerotest_no_eq_O : (n:nat)(ZeroTest n)=false->~n=O.
(Induction n;Auto).
Intros.
(Absurd (eq ? (ZeroTest O) false);Auto).
Save.
Hint Zerotest_no_eq_O.

(****)

Lemma O_ZeroTest : (n:nat)n=O->(ZeroTest n)=true.
(Intros n H;Rewrite -> H;Auto).
Save.
Hint O_ZeroTest.

(****)

Lemma no_O_ZeroTest : (n:nat)~n=O->(ZeroTest n)=false.
(Induction n;Auto).
Intro.
(Absurd (eq ? O O);Auto).
Save.
Hint no_O_ZeroTest.

(****)

Lemma not_S_eq : (n,m:nat)~(S n)=(S m)->~n=m.
(Red;Intros).
(Apply H;Auto).
Save.
Hint not_S_eq.

(****)

Lemma pred_diff_O : (n:nat)~n=O->~n=(S O)->~(pred n)=O.
(Induction n;Auto).
Save.
Hint pred_diff_O.

(****)

Lemma pred_mult : (n,m:nat)
                  (lt O n)->
                  (mult n m)=(plus (mult (pred n) m) m).
Intros.
Elim H.
Simpl.
Auto.
Intros.
Simpl.
(Elim plus_sym;Auto).
Save.
Hint pred_mult.

(****)

Lemma lt_neq_O : (n:nat)~n=O->(lt O n).
(Induction n;Auto).
Save.
Hint lt_neq_O.

(****)

Lemma pred_diff_lt : (n:nat)~n=O->~n=(S O)->(lt O (pred n)).
(Intros;Apply lt_neq_O;Auto).
Save.
Hint pred_diff_lt.

(****)

Lemma ZeroTest_eq : (b:bool)(n,m:nat)
                    (ZeroTest (Mux b n m))=
                    (orb (andb (ZeroTest n) b)
                         (andb (ZeroTest m) (negb b))).
(Induction b;Induction n;Induction m;Auto).
Save.

(****)

Lemma andb_true_l : (a,b:bool)(andb a b)=true->(a=true).
(Induction a;Auto).
Save.

(****)

Lemma and_sym : (a,b:bool)(andb a b)=(andb b a).
(Induction a;Induction b;Auto).
Save.
Hint and_sym.

(****)

Lemma andb_negb_true_r : (a,b:bool)(andb a (negb b))=true->(b=false).
(Induction b;Auto).
(Elim and_sym;Auto).
Save.

(****)

