(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              July 1st 1995                               *)
(*                                                                          *)
(****************************************************************************)

(************************************************************************)
(**************************** Factorielle.v *****************************)
(************************************************************************)
(*               Solange COUPET-GRIMAL and Line JAKUBIEC                *)
(*                                                                      *)
(*               Laboratoire d'Informatique de Marseille                *)
(*               URA CNRS 1787                                          *)
(*               Technopole de Chateau-Gombert                          *)
(*               39, Rue F. Joliot Curie                                *)
(*               13453 MARSEILLE Cedex 13                               *)
(*                                                                      *)
(*               July 1994                                              *)
(************************************************************************)

(** Compile Module Comp_Circ. **)
Require Export Comp_Circ. 


(************** Specification de la fonction factorielle ****************)

Fixpoint FactR [n:nat] : nat := 
           <nat>Case n of
                (*O*) (S O)
                (*Sp*) [p:nat](mult (S p) (FactR p)) end.


(********** Description d'un circuit calculant la factorielle ************)

Parameter DontCare : nat.

Fixpoint J [t:nat] : nat -> nat := [i:nat]<nat>Case t of
           (*O*) O
           (*Sn*) [n:nat]
                  (Mux (ZeroTest (J n i)) i (pred (J n i)))
           end.


Fixpoint R [t:nat] : nat -> nat := [i:nat]<nat>Case t of
           (*O*) DontCare
           (*Sn*) [n:nat]
                  (Mux (ZeroTest (J n i)) (S O)
                                          (mult (R n i) (J n i)))
           end.


(********************** Invariant du circuit ****************************)

Lemma fact_pred : (n:nat)
                  (ZeroTest n)=false->
                  (FactR n)=(mult n (FactR (pred n))).
(Induction n;Auto).
Save.
Hint fact_pred.

                               (****)


Lemma Inv_Fact : (t,i:nat)
                 (mult (R (S t) i) (FactR (J (S t) i)))=(FactR i).
Induction t.
Simpl;Auto.
Intros.
Cut (sumbool (eq ? (ZeroTest (J (S n) i)) true)
      (eq ? (ZeroTest (J (S n) i)) false)).
Intro.
Change (eq ?
         (mult
           (Mux (ZeroTest (J (S n) i)) (S O)
             (mult (R (S n) i) (J (S n) i)))
           (FactR (Mux (ZeroTest (J (S n) i)) i (pred (J (S n) i)))))
         (FactR i)).
(Elim H0;Clear H0;Intros H0).
Rewrite -> H0;Simpl;Auto.
Rewrite -> H0.
Change (eq ?
         (mult (mult (R (S n) i) (J (S n) i)) (FactR (pred (J (S n) i))))
         (FactR i)).
Rewrite -> mult_assoc_r.
(Elim fact_pred;Auto).
Auto.
Save.


(********************** Correction du circuit ****************************)

Theorem Circuit_Correct : (t,i:nat)
                          (J (S t) i)=O->
                          (R (S t) i)=(FactR i).
Intros.
Elim (Inv_Fact t i).
Rewrite -> H.
(Replace (FactR O) with (S O);Auto).
Save.

(*************************************************************************)

