(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              July 1st 1995                               *)
(*                                                                          *)
(****************************************************************************)

(************************************************************************)
(**************************** Multiplieur.v *****************************)
(************************************************************************)
(*               Solange COUPET-GRIMAL and Line JAKUBIEC                *)
(*                                                                      *)
(*               Laboratoire d'Informatique de Marseille                *)
(*               URA CNRS 1787                                          *)
(*               Technopole de Chateau-Gombert                          *)
(*               39, Rue F. Joliot Curie                                *)
(*               13453 MARSEILLE Cedex 13                               *)
(*                                                                      *)
(*               July 1994                                              *)
(************************************************************************)


(** Compile Module Comp_Circ. **)
Require Export Comp_Circ.


Definition Pair := (pair bool nat).
Definition First := (fst bool nat).
Definition Second := (snd bool nat).

(************** Description du circuit du multiplieur  *****************)

Parameters DontCare1, DontCare2 : nat.

Fixpoint Prod_DONE_L2[t:nat] : nat -> nat -> bool*nat :=
     [i,j:nat]<bool*nat>Case t of
        (*O*) (Pair true DontCare1)
        (*Sp*)[p:nat] 
              (Pair
                (orb (ZeroTest j)
                     (ZeroTest (Mux (First (Prod_DONE_L2 p i j))
                                    (pred i)
                            (pred (pred (Second (Prod_DONE_L2 p i j)))))))
                (Mux (First (Prod_DONE_L2 p i j)) 
                     i
                     (pred (Second (Prod_DONE_L2 p i j)))))
     end.


Definition L2 := [t,i,j:nat](Second (Prod_DONE_L2 t i j)).


Definition DONE := [t,i,j:nat](First (Prod_DONE_L2 t i j)).
 

Fixpoint OUT[t:nat] : nat -> nat -> nat :=
    [i,j:nat]<nat>Case t of
              (*O*) DontCare2
              (*Sp*) [p:nat]
                     (Mux (DONE p i j)
                          (Mux (ZeroTest i) O j)
                          (plus (Mux (ZeroTest i) O j)
                                (OUT p i j)))
    end.


(********************** Invariant du circuit ****************************)

(**       DONE_false est utilise dans la preuve de l'invariant        **)


                                (****)

Lemma DONE_false : (t,i,j:nat)
                   ~i=O->
                   ~j=O->
                   ~i=(S O)->
                   (DONE (S t) i j)=false->
                   ~(pred (L2 (S t) i j ))=O.
Intros t i j H H0 H1.
Change (eq ?
         (orb (ZeroTest j)
           (ZeroTest
             (Mux (DONE t i j) (pred i) (pred (pred (L2 t i j))))))
         false)
        ->(not (eq ? (pred (L2 (S t) i j)) O)).
Cut (eq ? (ZeroTest j) false).
(Intros H2;Rewrite -> H2;Simpl).
Change (eq ?
         (ZeroTest (Mux (DONE t i j) (pred i) (pred (pred (L2 t i j)))))
         false)
        ->(not (eq ? (pred (Mux (DONE t i j) i (pred (L2 t i j)))) O)).
Cut (sumbool (eq ? (DONE t i j) true) (eq ? (DONE t i j) false)).
(Intros H4;Elim H4;Clear H4;Intros H4;Rewrite -> H4;Simpl).
Auto.
Auto.
Auto.
(Apply no_O_ZeroTest;Auto).
Save.


                                (****)


Lemma Inv_Mult : (t,i,j:nat)
                   ~i=O->
                   ~j=O->
                   ~i=(S O)->
                   (plus (mult (pred (L2 (S t) i j )) j) (OUT (S t) i j))
                    =(mult i j).
(Induction t;Intros).
Change (eq ?
         (plus (mult (pred (Mux (DONE O i j) i (pred (L2 O i j)))) j)
           (OUT (S O) i j))
         (mult i j)).
Simpl.
Cut (eq ? (ZeroTest i) false).
(Intros H2;Rewrite -> H2;Simpl).
(Elim pred_mult;Auto).

Auto.

Change (eq ?
         (plus
           (mult (pred (Mux (DONE (S n) i j) i (pred (L2 (S n) i j)))) j)
           (Mux (DONE (S n) i j) (Mux (ZeroTest i) O j)
             (plus (Mux (ZeroTest i) O j) (OUT (S n) i j))))
         (mult i j)).
Cut (sumbool (eq ? (DONE (S n) i j) true) (eq ? (DONE (S n) i j) false)).
(Intros H3;Elim H3;Clear  H3;Intros H3;Rewrite -> H3).
Cut (eq ? (ZeroTest i) false).
(Intros H4;Rewrite -> H4;Simpl).
(Elim pred_mult;Auto).
Auto.
Cut (eq ? (ZeroTest i) false).
(Intros H4;Rewrite -> H4).
Change (eq ?
         (plus (mult (pred (pred (L2 (S n) i j))) j)
           (plus j (OUT (S n) i j)))
         (mult i j)).
Cut (sumbool (eq ? (DONE n i j) true) (eq ? (DONE n i j) false)).
(Intros H5;Elim H5;Clear  H5;Intros H5).
Change (eq ?
         (plus
           (mult (pred (pred (Mux (DONE n i j) i (pred (L2 n i j))))) j)
           (plus j
             (Mux (DONE n i j) (Mux (ZeroTest i) O j)
               (plus (Mux (ZeroTest i) O j) (OUT n i j)))))
         (mult i j)).
(Rewrite -> H4;Rewrite -> H5;Simpl).
Rewrite -> plus_assoc_l.
Elim pred_mult.
(Elim pred_mult;Auto).
(Apply pred_diff_lt;Auto).
Rewrite -> plus_assoc_l.
Elim pred_mult.
Auto.
Apply lt_neq_O.
(Apply DONE_false;Auto).
Auto.
Auto.
Auto.
Save.


(*************************************************************************)
(********************** Correction du circuit ****************************)

(**   La forme du circuit nous conduit a prouver d'abord sa correction  **)
(**           pour les cas particuliers ou i=0, i=1 et j=0,             **)
(**                    puis pour le cas general                         **)
(*************************************************************************)

(** i_O_DONE_true, i_SO_DONE_true et j_O_DONE_true sont utilises dans   **)
(** la preuve de correction pour les cas particuliers i=0, i=1 et j=0   **)

                                (****)


Lemma i_O_DONE_true : (t,i,j:nat)
                      i=O->
                      (DONE t i j)=true.
(Induction t;Auto).
Intros.
Change (eq ?
         (orb (ZeroTest j)
           (ZeroTest
             (Mux (DONE n i j) (pred i) (pred (pred (L2 n i j))))))
         true).
(Rewrite -> H0;Simpl).
(Rewrite -> H;Simpl;Auto).
(Elim orb_sym;Auto).
Save.
Hint i_O_DONE_true.


                                (****)


Lemma i_SO_DONE_true : (t,i,j:nat)
                      i=(S O)->
                      (DONE t i j)=true.
Induction t;Auto.
Intros.
Change (eq ?
         (orb (ZeroTest j)
           (ZeroTest
             (Mux (DONE n i j) (pred i) (pred (pred (L2 n i j))))))
         true).
Rewrite -> H0;Simpl.
Rewrite -> H;Simpl;Auto.
Elim orb_sym;Auto.
Save.
Hint i_SO_DONE_true.

                                (****)


Lemma j_O_DONE_true : (t,i,j:nat)
                      j=O->
                      (DONE t i j)=true.
(Induction t;Auto).
Intros.
Change (eq ?
         (orb (ZeroTest j)
           (ZeroTest
             (Mux (DONE n i j) (pred i) (pred (pred (L2 n i j))))))
         true).
(Rewrite -> H0;Auto).
Save.
Hint j_O_DONE_true.


                                (****)


(** Cas ou i=0 **)

Theorem Circuit_Correct_i_O : (t,i,j:nat)
                              i=O->
                              (DONE (S t) i j)=true->  
                              (OUT (S t) i j)=(mult i j).
Intros.
(Rewrite -> H;Simpl).
Cut (eq ? (DONE t O j) true).
(Intros H1;Rewrite -> H1;Simpl;Auto).
Auto.
Save.


(** Cas ou i=1 **)

Theorem Circuit_Correct_i_SO : (t,i,j:nat)
                              i=(S O)->
                              (DONE (S t) i j)=true->
                              (OUT (S t) i j)=(mult i j).
Intros.
Rewrite -> H;Simpl.
Cut (eq ? (DONE t (S O) j) true).
Intros H1;Rewrite -> H1;Simpl;Auto.
Auto.
Save.


(** Cas ou j=0 **)

Theorem Circuit_Correct_j_O : (t,i,j:nat)
                              j=O->
                              (DONE (S t) i j)=true->
                              (OUT (S t) i j)=(mult i j).
Intros.
(Rewrite -> H;Simpl).
Cut (eq ? (DONE t i O) true).
(Intros H1;Rewrite -> H1;Simpl;Auto).
Elim mult_n_O.
Cut (sumbool (eq ? (ZeroTest i) true) (eq ? (ZeroTest i) false)).
(Intros H2;Elim H2;Clear  H2;Intros H2;Rewrite -> H2;Auto).
Auto.
Auto.
Save.


(** Cas general **)

Theorem Circuit_Correct : (t,i,j:nat)
                          ~i=O->
                          ~j=O->
                          ~i=(S O)->
                          (DONE (S t) i j)=true->
                          (OUT (S t) i j)=(mult i j).
Intros t i j H H0 H1.
Change (eq ?
         (orb (ZeroTest j)
           (ZeroTest
             (Mux (DONE t i j) (pred i) (pred (pred (L2 t i j))))))
         true)
        ->(eq ? (OUT (S t) i j) (mult i j)).
Cut (eq ? (ZeroTest j) false).
(Intros H2;Rewrite -> H2;Simpl).
Rewrite -> ZeroTest_eq.
Cut (eq bool (ZeroTest (pred i)) false).
(Intros H3;Rewrite -> H3;Simpl).
Intro.
Cut (eq ? (ZeroTest i) false).
(Intros H5;Rewrite -> H5;Simpl).
Elim (Inv_Mult t i j).
Change (eq ? (Mux (DONE t i j) j (plus j (OUT t i j)))
         (plus (mult (pred (Mux (DONE t i j) i (pred (L2 t i j)))) j)
           (OUT (S t) i j))).
Cut (eq ? (DONE t i j) false).
Cut (eq ? (pred (pred (L2 t i j))) O).
Intros.
(Rewrite -> H7;Simpl).
(Rewrite -> H6;Simpl).
(Rewrite -> H7;Simpl).
(Rewrite -> H5;Simpl;Auto).
Apply Zerotest_eq_O.
(Apply andb_true_l with (negb (DONE t i j)) ;Auto).
(Apply andb_negb_true_r with (ZeroTest (pred (pred (L2 t i j)))) ;Auto).
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Save.


(*************************************************************************)

