(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*      Coq V5.8                                                            *)
(*                                                                          *)
(*                                                                          *)
(*      First-order Unification                                             *)
(*                                                                          *)
(*      Joseph Rouyer                                                       *)
(*                                                                          *)
(*      November 1992                                                       *)
(*                                                                          *)
(****************************************************************************)
(*                       is_in_quasiterm_term_subst.v                       *)
(****************************************************************************)

Require Arith.
Require nat_complements.
Require nat_term_eq_quasiterm.

(***********************************************************************)
(******************* VARIABLE IN A QUASITERM *********************)
(***********************************************************************)

Inductive is_in [x:var]:quasiterm->Prop
:=is_in_init : (is_in x (V x))
|is_in_Root : (l:fun)(t:quasiterm)(is_in x t)->(is_in x (Root l t))
|is_in_ConsArg1 : (t1,t2:quasiterm)(is_in x t1)->(is_in x (ConsArg t1 t2))
|is_in_ConsArg2 : (t1,t2:quasiterm)(is_in x t2)
                   ->(is_in x (ConsArg t1 t2)).

Fixpoint IS_IN [x:var; t:quasiterm] : Prop
:= 
(<Prop>Case t of
        [y:var](x=y)
        [l:fun]False
        [l:fun][t:quasiterm](IS_IN x t)
        [t1:quasiterm][t2:quasiterm]((IS_IN x t1)\/(IS_IN x t2)) end).


Lemma IS_IN_is_in : (x:var)(t:quasiterm)(IS_IN x t)->(is_in x t).
Induction t.
Intros.
Elim H;Apply is_in_init.
Intros;Elim H.
Intros;Apply is_in_Root;Auto.
Intros;Elim H1;Intros.
Apply is_in_ConsArg1;Auto.
Apply is_in_ConsArg2;Auto.
Save.


Lemma is_in_IS_IN : (x:var)(t:quasiterm)(is_in x t)->(IS_IN x t).
Intros;Elim H;Simpl;Auto.
Save.

(***********************************************************************)
(************************* IS_IN decidability *****************************)
(***********************************************************************)

Lemma IS_IN_decS : (x:var)(t:quasiterm){(IS_IN x t)}+{~(IS_IN x t)}.
Induction t.
Intro;Elim (var_eq_decS x v);Intros;Simpl;Auto.
Intros;Right;Auto.
Intros f y h;Elim h;Intros;Simpl;Auto.
Intros y h y0 h0;Elim h;Elim h0;Intros;Simpl;Auto.
Right ; Tauto.
Save.

Lemma IS_IN_decP : (x:var)(t:quasiterm)(IS_IN x t)\/(~(IS_IN x t)).
Intros;Elim (IS_IN_decS x t);Intros;Auto.
Save.

(**********************************************************************)
(************************** End of IS_IN *******************************)
(**********************************************************************)

(***********************************************************************)
(********************** QUASISUBSTITUTIONS *************************)
(***********************************************************************)

Definition quasisubst:=var->quasiterm.

Fixpoint Subst [f:quasisubst;t:quasiterm] : quasiterm
:=(<quasiterm>Case t 
of f
     C 
     [l:fun][t:quasiterm](Root l (Subst f t))
     [t1,t2:quasiterm](ConsArg (Subst f t1) (Subst f t2)) end).

(*---------------------------------------------*)
(*------------ Verifications on the terms:begin -------------*)
(*---------------------------------------------*)

Inductive term_subst[l:list_nat;f:quasisubst]:Prop:=
term_subst_init:((x:var)(term l (f x)))->(term_subst l f).

(***********************************************************************)
(********************** If (term_subst l f), *******************************)
(********* then Length, SIMPLE, (L_TERM l) and (term l) ****************) 
(******************* are stable by (Subst f). ******************************)
(***********************************************************************)

Lemma term_subst_eq_Length : (l:list_nat)(f:quasisubst)(t:quasiterm)
(term_subst l f)-><nat>(Length t)=(Length (Subst f t)).
Induction t;Intros;Elim H;Simpl;Auto.
Intros h;Elim (h v).
Elim (f v);Simpl;Auto.
Intros.
Absurd False;Auto.
Intros.
Elim H;Elim H0;Auto.
Save.

Lemma SIMPLE_term_subst : (l:list_nat)(f:quasisubst)(term_subst l f)
->(t:quasiterm)(SIMPLE t)->(SIMPLE (Subst f t)).
Induction t;Simpl;Auto.
Intros;Elim H;Intros h;Elim (h v);Intros;Auto.
Save.

Lemma L_TERM_term_subst : (l:list_nat)(t:quasiterm)(L_TERM l t)
->(f:quasisubst)(term_subst l f)
->(L_TERM l (Subst f t)).
Intros l t H;Cut (l_term l t).
2:Try Apply L_TERM_l_term;Auto.
Intros h;Elim h;Simpl;Auto.
Intros x f h0;Elim h0.
Intros h1;Elim (h1 x);Auto.
Intros.
Split;Auto.
Replace (arity l f) with (Length t0).
Apply  term_subst_eq_Length with l;Auto.
Intros.
Split;Auto.
Apply SIMPLE_term_subst with l;Auto.
Save.

Lemma term_term_subst : (l:list_nat)(t:quasiterm)(term l t)
->(f:quasisubst)(term_subst l f)
->(term l (Subst f t)).
Intros;Apply term_init.
Elim H;Intros;Apply L_TERM_term_subst;Auto.
Elim H;Intros;Apply SIMPLE_term_subst with l;Auto.
Save.

(*---------------------------------------------*)
(*------------ Verifications on the terms:end --------------*)
(*---------------------------------------------*)

(***********************************************************************)
(************** Properties linked to IS_IN and Subst ***********************)
(***********************************************************************)

Lemma n_IS_IN_Subst :
(f:quasisubst)(t:quasiterm)(x:var)((y:var)(~(IS_IN x (f y))))
->~(IS_IN x (Subst f t)).
Induction t;Simpl;Auto.
Intros;Unfold not;Intros h;Elim h;Intros.
Elim (H x);Auto.
Elim (H0 x);Auto.
Save.

Lemma IS_IN_Subst_IS_IN : (f:quasisubst)(t:quasiterm)(x,x0:var)(IS_IN x (f x0))
->(IS_IN x0 t)->(IS_IN x (Subst f t)).
Induction t;Simpl;Auto.
Intros.
Elim H0;Auto.
Intros.
Elim H2;Intros.
Left;Apply (H x x0);Auto.
Right;Apply (H0 x x0);Auto.
Save.

(***********************************************************************)
(**** If two quasisubstitutions have the same restriction ********************)
(************** on the set of the variables of t, **************************)
(************** then they transform t in the same quasiterm. **************)
(***********************************************************************)

Lemma eq_restriction_s_t : (t:quasiterm)(f,g:quasisubst)
((x:var)(IS_IN x t)-><quasiterm>(f x)=(g x))
-><quasiterm>(Subst f t)=(Subst g t).
Induction t;Simpl;Auto.
Intros.
Elim (H f0 g);Auto.
Intros.
Elim (H f g);Elim (H0 f g);Auto.
Save.

(***********************************************************************)
(******* If two quasisubstitutions transform t in the same quasiterm, ********)
(***** then they have the same restriction on the set of the variables of t.****)
(***********************************************************************)

Lemma eq_restriction_t_s :
(t:quasiterm)(f,g:quasisubst)((Subst f t)=(Subst g t))
->((x:var)(IS_IN x t)-><quasiterm>(f x)=(g x)).
Induction t;Simpl;Intros.
Rewrite H0 ; Auto.
Contradiction.                         
Apply H;[Injection H0 ; Auto | Auto].
Elim H2;Intros.
Apply H; [ Injection H1 ; Auto | Auto].
Apply H0 ; [Injection H1 ; Auto | Auto].
Save.

(***********************************************************************)
(********************* End of quasisubstitutions **************************)
(***********************************************************************)

