(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*      Coq V5.8                                                            *)
(*                                                                          *)
(*                                                                          *)
(*      First-order Unification                                             *)
(*                                                                          *)
(*      Joseph Rouyer                                                       *)
(*                                                                          *)
(*      November 1992                                                       *)
(*                                                                          *)
(****************************************************************************)
(*                             listv_is_in_lv.v                             *)
(****************************************************************************)

Require Arith.
Require nat_complements.
Require nat_term_eq_quasiterm.
Require is_in_quasiterm_term_subst.

(***********************************************************************)
(*********************** Lists of variables : *****************************)
(***********************************************************************)

Inductive listv : Set := Nilv : listv | Consv : var -> listv -> listv.

Definition BNilv:listv->Prop:=[l:listv](<Prop>Case l 
of True 
   [x:var][l:listv]False end).

Definition BConsv:listv->Prop:=[l:listv](<Prop>Case l 
of False
   [x:var][l:listv]True end).

Fixpoint Appv [l0:listv] : listv -> listv := [l1:listv](<listv>Case l0
of l1 
   [x:var][l2:listv](Consv x (Appv l2 l1)) end).

Definition Headv:var->listv->var:=[x:var][l:listv](<var>Case l
of x
     [y:var][_:listv]y end).

Definition Tailv:listv->listv:=[l:listv](<listv>Case l
of Nilv
     [_:var][l1:listv]l1 end).

(***********************************************************************)
(**************** Predicate "to be member of a list" : *********************)
(***********************************************************************)

Inductive is_in_lv[x:var]:listv->Prop:=
is_in_lv_init:(l:listv)(is_in_lv x (Consv x l))
|is_in_lv_Consv:(l:listv)(x0:var)(is_in_lv x l)
->(is_in_lv x (Consv x0 l)).

Fixpoint IS_IN_LV [x:var;l:listv] : Prop
:=(<Prop>Case l
of False
   [y:var][l2:listv](IS_IN_LV x l2)\/<var>x=y end).

Lemma is_in_lv_IS_IN_LV : (l:listv)(x:var)(is_in_lv x l)->(IS_IN_LV x l).
Intros;Elim H;Simpl;Auto.
Save.

Lemma IS_IN_LV_is_in_lv : (l:listv)(x:var)(IS_IN_LV x l)->(is_in_lv x l).
Induction l;Simpl;Intros.
Elim H.
Elim H0;Intros.
Apply is_in_lv_Consv;Auto.
Elim H1;Apply is_in_lv_init;Auto.
Save.

(***********************************************************************)
(************ Properties 1) if x belongs to l and is not the head, ***********)
(************************* then x belongs to the tail. *********************)
(***********************************************************************)

Lemma IS_IN_LV_n_eq :
(x,x0:var)(l:listv)(IS_IN_LV x (Consv x0 l))->(~<var>x=x0)->(IS_IN_LV x l).
Induction l;Simpl;Intros.
Elim H;Intros;Auto;Elim H0;Auto.
Elim H0;Intros;Auto.
Save.

(***********************************************************************)
(********************** 2) [x0] contains only x0 . ************************)
(***********************************************************************)

Lemma IS_IN_LV_Consv_Nilv_eq :
(x,x0:var)(IS_IN_LV x (Consv x0 Nilv))-><var>x=x0.
Intros;Elim H;Intros h;Elim h;Auto.
Save.

(***********************************************************************)
(********************** 3) Decidability of IS_IN_LV : *********************)
(***********************************************************************)

Lemma IS_IN_LV_decS : (x:var)(l:listv){(IS_IN_LV x l)}+{~(IS_IN_LV x l)}.
Induction l;Simpl;Intros.
(*l=Nilv*)
Auto.
(*l=(Consv v y)*)
Elim H;Intros.
Auto.
Elim (var_eq_decS x v);Intros.
(* ... <var>x=v *)
Auto.
(*... ~<var>x=v*)
Right;Tauto.
Save.

Lemma IS_IN_LV_decP : (x:var)(l:listv)(IS_IN_LV x l)\/~(IS_IN_LV x l).
Intros;Elim (IS_IN_LV_decS x l);Intros;Auto.
Save.

(***********************************************************************)
(*********************** IS_IN_LV and Appv *****************************)
(***********************************************************************)

Lemma IS_IN_LV_Appv1 :
(l,l0:listv)(x:var)(IS_IN_LV x l)->(IS_IN_LV x (Appv l l0)).
Induction l;Simpl;Intros.
Elim H;Auto.
Elim H0;Intros;Auto.
Save.

Lemma IS_IN_LV_Appv2 :
(l,l0:listv)(x:var)(IS_IN_LV x l0)->(IS_IN_LV x (Appv l l0)).
Intros l;Elim l;Simpl;Auto.
Save.

Lemma IS_IN_LV_Appv_IS_IN_LV :
(l,l0:listv)(x:var)(IS_IN_LV x (Appv l l0))->((IS_IN_LV x l)\/(IS_IN_LV x l0)).
Induction l;Simpl;Intros;Auto.
Elim H0;Intros;Auto.
Elim (H l1 x);Auto.
Save.

(***********************************************************************)

