(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*      Coq V5.8                                                            *)
(*                                                                          *)
(*                                                                          *)
(*      First-order Unification                                             *)
(*                                                                          *)
(*      Joseph Rouyer                                                       *)
(*                                                                          *)
(*      November 1992                                                       *)
(*                                                                          *)
(*               New theorems and lemmas on naturals.                       *)
(*                                                                          *)
(****************************************************************************)
(*                            nat_complements.v                             *)
(****************************************************************************)

Require Arith.

(*********************************************************************)
(************* Lemmas on naturals proved in Nat.v and *****************)
(******************** used with the command "auto". *******************)
(*********************************************************************)
(*O_S      :(n:nat)(~O=(S n))*)
(*eq_S     :(n:nat)(m:nat)(n=m)->((S n)=(S m))*)
(*eq_add_S :(n:nat)(m:nat)((S n)=(S m))->(n=m)*)
(*le_pred_n:(n:nat)(le (pred n) n)*)
(*le_n_Sn  :(n:nat)(le n (S n))*)
(*le_S_n   :(n:nat)(m:nat)(le (S n) (S m))->(le n m)*)
(*le_Sn_O  :(n:nat)(~(le (S n) O))*)
(*le_n     :(n:nat)(le n n)*)
(*le_n_S   :(n:nat)(m:nat)(le n m)->(le (S n) (S m))*)
(*********************************************************************)

(*********************************************************************)
(********** Logic tools : To translate P:Prop into Q:Set : ****************)
(*********************************************************************)

Definition P_S:(A:Set)(P:A->Prop)A->Set:=
[A:Set][P:A->Prop][x:A]{a:A|(a=x)&(P a)}.

Lemma P_S_proof1 : (A:Set)(a:A)(P:A->Prop)(P a)->(P_S A P a).
Intros;Unfold P_S;Exists a;Auto.
Save.

Lemma P_S_proof2 : (A:Set)(a:A)(P:A->Prop)(P_S A P a)->(P a).
Unfold P_S;Intros A a P h;Elim h;Intros x h0;Elim h0;Auto.
Save.

(*********************************************************************)
(***************** Logic tools  To negate an equality : ******************)
(*********************************************************************)

Lemma Diff : (A:Set)(f:A->Prop)(a,b:A)(f a)->(~(f b))->~a=b.
Unfold not;Intros A f a b H H0 H1;Elim H0;Elim H1;Auto.
Save.

(*** Replace "Apply h;Auto" by "auto" when the Type of h is False : ***)

Lemma n_False : ~False.
Unfold not;Auto.
Save.
Hint n_False.
(*********************************************************************)

(*********************************************************************)
(********************** Section complement_nat. *********************)
(*********************************************************************)

(*Section complement_nat.*)

(*********************************************************************)
(**************** Decidability of the equality in the Set nat : *************)
(*********************************************************************)

Lemma nat_eq_decS : (x,y:nat){x=y}+{~x=y}.
Induction x.           
(*case x=O*)
Induction y;Auto.
(*case x=(S y)*)
Induction y.
(*... case y0=O*)
Right;Discriminate.
(*... case y0=(S y1)*)
Intros y1 h;Elim (H y1);Intros.
(*... ... case y=y1*)
Auto.(*Apply eq_S*)
(*case not y=y1*)
Right; Simplify_eq ; Auto.
Save.

Lemma nat_eq_decP : (x,y:nat)(x=y)\/(~x=y).
Intros x y;Elim (nat_eq_decS x y);Auto.
Save.

(*********************************************************************)
(************** General induction (with le) : ****************************)
(*********************************************************************)

Lemma ind_leS : (n:nat)(P:nat->Set)(P O)->((p:nat)((q:nat)(le q p)->(P q))
->(P (S p)))->(P n).
Intros n P;Cut ((m:nat)(le m n)->(P m))->(P n).
2:Auto.(*Apply le_n*)
Intros h h0 h1;Apply h.
Elim n.
(*case n=O*)
(*... case m=O*)
Induction m.
Intros ; Auto.
(*... case m=(S y)*)    
Intros y Hyp1 Hyp2 ;Absurd (le (S y) O);Auto.(*le_Sn_O*)
(*case n=(S y)*)
Induction m.
(*... case m=O*)
Auto.
(*... case m=(S y0)*)    
Intros y0 hyp1 hyp2 ;Apply (h1 y0).
Intros;Apply H.
Apply le_trans  with y0;Auto.(*le_S_n*)
Save.

Lemma ind_leP : (n:nat)(P:nat->Prop)(P O)->((p:nat)((q:nat)(le q p)->(P q))
->(P (S p)))->(P n).
Intros;Apply (P_S_proof2 nat n P).
Apply (ind_leS n (P_S nat P)); [ Apply P_S_proof1; Assumption
			       | Intros;Apply P_S_proof1].
Apply H0;Intros;Elim (H1 q);[Intros x eg_x_q | Auto].
Rewrite eg_x_q ;Auto.
Save.

(*********************************************************************)
(********** Reasoning by cases with the natural constructors : ************)
(*********************************************************************)

Lemma pred_or : (m:nat)(O=m)\/(m=(S (pred m))).
Intro;Elim m;Auto.
Save.

Lemma nat_caseS : (x:nat)(P:nat->Set)(P O)->((n:nat)(P (S n)))->(P x).
Intros;Elim x;Auto.
Save.

(*********************************************************************)
(************** Decidability of le : ************************************)
(*********************************************************************)

Lemma le_decS : (n,p:nat){(le n p)}+{~(le n p)}.
Induction n.
(*case n=O*)
Auto.
(*case n=(S y)*)
Induction p.
Auto.           (*Apply le_Sn_O*)
Intros y0 le_or_not_le ;Elim (H y0).
Auto.             (*Apply le_n_S*)
Intro not_le_n0_y0 ; Right;Unfold not.
Intros;Elim not_le_n0_y0 ;Auto.        (*Apply le_S_n*)
Save.

Lemma le_decP : (n,p:nat)(le n p)\/(~(le n p)).
Intros;Elim (le_decS n p);Intros;Auto.
Save.

Lemma le_S_eqS : (n,p:nat)(le n p)->{(le (S n) p)}+{<nat>n=p}.
Induction n.
Induction p;Auto.
Intros n0 H p ; Elim p.
Intros;Absurd (le (S n0) O);Auto.
Intros n1 H1 H2 ;Elim (H n1);Auto.(*le_n_S*)
Save.

Lemma le_S_eqP : (n,p:nat)(le n p)->((le (S n) p)\/(<nat>n=p)).
Intros;Elim (le_S_eqS n p);Intros;Auto.
Save.
