(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*      Coq V5.8                                                            *)
(*                                                                          *)
(*                                                                          *)
(*      First-order Unification                                             *)
(*                                                                          *)
(*      Joseph Rouyer                                                       *)
(*                                                                          *)
(*      November 1992                                                       *)
(*                                                                          *)
(****************************************************************************)
(*                         nat_term_eq_quasiterm.v                          *)
(****************************************************************************)

Require Arith.
Require nat_complements.

(*---------------------------------------------*)
(*------------ Verifications on the terms:begin -------------*)
(*---------------------------------------------*)

Section nat_sequence.

Inductive list_nat:Set:=
nil_nat:list_nat
|cons_nat:nat->list_nat->list_nat.

Fixpoint At_last [x:nat;l:list_nat] : list_nat
:= <list_nat> Case l of
	(* nil_nat *)(cons_nat x nil_nat)
        (* cons_nat c ll *)[c:nat][ll:list_nat](cons_nat c (At_last x ll))
	     end.


Fixpoint List_queue_nat [l:list_nat] : list_nat
:= <list_nat> Case l of
	(* nil_nat *)nil_nat
	(* cons_nat c ll *)[c:nat][tail:list_nat]
				(At_last c (List_queue_nat tail))
	      end
.




Fixpoint Constr_f [l:list_nat] : nat->nat
:=[n:nat]
<nat>Case l of
    (* nil_nat *)O
    [m:nat][tail:list_nat](<nat> Case n of
				(* O *) m
				(* (S p) *)[p:nat](Constr_f tail p)
				end)
     end.

(***********************************************************************)
(********************** Properties of  Constr_f ***************************)
(***********************************************************************)

Inductive OK_Constr_f[f:list_nat->nat->nat]:Prop:=
OK_Constr_f_init:((x:nat)<nat>O=(f nil_nat x))
->((val:nat)(l:list_nat)<nat>val=(f (cons_nat val l) O))
->((x,val:nat)(l:list_nat)<nat>(f l x)=(f (cons_nat val l) (S x)))
->(OK_Constr_f f).

Lemma Constr_f_OK : (OK_Constr_f Constr_f).
Apply OK_Constr_f_init;Simpl;Auto;Induction x;Simpl;Auto.
Save.

Inductive OK_At_last[f:nat->list_nat->list_nat]:Prop:=
OK_At_last_init:
((x:nat)<list_nat>(cons_nat x nil_nat)=(f x nil_nat))
->((x,y:nat)<list_nat>(cons_nat y (cons_nat x nil_nat))
     =(f x (f y nil_nat)))
->(OK_At_last f).

Lemma At_last_OK : (OK_At_last At_last).
Apply OK_At_last_init;Simpl;Auto.
Save.

Inductive OK_List_queue_nat[f:list_nat->list_nat]:Prop:=
OK_List_queue_nat_init:(<list_nat>nil_nat=(f nil_nat))
->((x:nat)(l:list_nat)<list_nat>(At_last x (f l))=(f (cons_nat x l)))
->(OK_List_queue_nat f).

Lemma List_queue_nat_OK : (OK_List_queue_nat List_queue_nat).
Apply OK_List_queue_nat_init;Simpl;Auto.
Save.

End nat_sequence.

(**********************************************************************)
(******************* Specifications of terms ******************************)
(**********************************************************************)

Section terms.

Definition fun:Set:=nat.

Definition var:Set:=nat.

Lemma var_eq_decP : (x,x0:var)(<var>x=x0)\/(~<var>x=x0).
Intros;Elim (nat_eq_decP x x0);Auto.
Save.

Lemma var_eq_decS : (x,x0:var){<var>x=x0}+{~<var>x=x0}.
Intros;Elim (nat_eq_decS x x0);Auto.
Save.

Lemma fun_eq_decP : (x,x0:fun)(<fun>x=x0)\/(~<fun>x=x0).
Intros;Elim (nat_eq_decP x x0);Auto.
Save.

Lemma fun_eq_decS : (x,x0:fun){<fun>x=x0}+{~<fun>x=x0}.
Intros;Elim (nat_eq_decS x x0);Auto.
Save.

 
Inductive quasiterm:Set:=
V:var->quasiterm                                 (*variable quasiterm*)
|C:fun->quasiterm                                (*constant quasiterm*)
|Root : fun->quasiterm->quasiterm                           (*rooting*)
|ConsArg  : quasiterm->quasiterm->quasiterm.
                                 (*building the pairs of quasiterms*)

(**********************************************************************)
(*********************** The arity function *****************************)
(**********************************************************************)

Hypothesis list_arity:list_nat.

Definition arity:fun->nat:=(Constr_f (List_queue_nat list_arity)).

(**********************************************************************)
(********************** Length of a quasiterm ***************************)
(**********************************************************************)




Fixpoint Length [t:quasiterm] : nat:=
<nat>Case t of
	[x:var](S O)
	[f:fun](S O)
	[f:fun][u:quasiterm](S O)
	[t1,t2:quasiterm](plus (Length t1) (Length t2))
(*(<nat>Match (Length t1) with (Length t2) [x:nat]S end)*)
end.


(***********************************************************************)
(********************** Predicate SIMPLE *******************************)
(****** Simple term is constant term or variable term or rooted term. *******)
(***********************************************************************)

Definition SIMPLE:quasiterm->Prop:=
[t:quasiterm](<Prop>Case t of
                 [x:var]True
                 [l:fun]True
                 [l:fun][u:quasiterm]True
                 [t1:quasiterm][t2:quasiterm]False end).

(***********************************************************************)
(***************** Predicates l_term and L_TERM : ***********************)
(***********************************************************************)

Inductive l_term:quasiterm->Prop:=
l_term_initV:(x:var)(l_term (V x))
|l_term_initC:(f:fun)(<nat>O=(arity f))->(l_term (C f))
|l_term_Root:(f:fun)(t:quasiterm)(l_term t)
->(<nat>(arity f)=(Length t))->(l_term (Root f t))
|l_term_ConsArg:(t1,t2:quasiterm)(l_term t1)->(l_term t2)
->(SIMPLE t1)->(l_term (ConsArg t1 t2)).

Fixpoint L_TERM [t:quasiterm] : Prop :=
<Prop>Case t of
     [x:var]True 
     [c:fun](O=(arity c))
     [l:fun][t:quasiterm](L_TERM t)/\ ((arity l)=(Length t))
     [t1:quasiterm][t2:quasiterm]
     (SIMPLE t1)/\(L_TERM t1)/\(L_TERM t2)
end.

(***********************************************************************)
(******************** L_TERM is equivalent to l_term : *******************)
(***********************************************************************)

Lemma L_TERM_l_term : (t:quasiterm)(L_TERM t)->(l_term t).
Intro;Elim t;Simpl;Intros.
Apply l_term_initV;Auto.
Apply l_term_initC;Auto.
Apply l_term_Root.
Apply H;Elim H0;Auto.
Elim H0;Auto.
Apply l_term_ConsArg.
Apply H;Elim H1;Intros H2 H3;Elim H3;Intros;Auto.
Apply H0;Elim H1;Intros H2 H3;Elim H3;Intros;Auto.
Elim H1;Auto.
Save.

Lemma l_term_L_TERM : (t:quasiterm)(l_term t)->(L_TERM t).
Intros;Elim H;Simpl;Auto.
Save.

(***********************************************************************)
(******************** Predicate term : ***********************************)
(***********************************************************************)

Inductive term[t:quasiterm]:Prop:=
term_init:(L_TERM t)->(SIMPLE t)->(term t).

(***********************************************************************)
(************ Decidability of L_TERM and decidability of term : ************)
(***********************************************************************)

Lemma SIMPLE_decS : (t:quasiterm){(SIMPLE t)}+{~(SIMPLE t)}.
Intros;Elim t;Simpl;Auto.
Save.

Lemma L_TERM_decS : (t:quasiterm){(L_TERM t)}+{~(L_TERM t)}.
Induction t;Simpl;Auto.
Intros f;Elim (nat_eq_decS O (arity f));Intros;Simpl;Auto.
Intros f y h0;Elim (nat_eq_decS (arity f) (Length y));Intros h.
Elim h;Elim h0;Intros;Auto.
Right;Tauto.

Right;Tauto.

Intros y h y0 h0;Elim h;Intros.
Elim h0;Intros.
Elim (SIMPLE_decS y);Intros h1.
Auto.
Right;Tauto.
Right;Tauto.
Right;Tauto.
Save.

Lemma term_decS : (t:quasiterm){(term t)}+{~(term t)}.
Intro;Elim (L_TERM_decS t);Intros h1;Elim (SIMPLE_decS t);Intros h2.
Left;Apply term_init;Auto.
Right;Unfold not;Intros h3;Elim h2;Elim h3;Intros;Auto.
Right;Unfold not;Intros h3;Elim h1;Elim h3;Intros;Auto.
Right;Unfold not;Intros h3;Elim h1;Elim h3;Intros;Auto.
Save.

(***********************************************************************)
(******************* Properties of  Length *******************************)
(***********************************************************************)

Lemma Length_n_O : (t:quasiterm){x:nat|<nat>(Length t)=(S x)}.
Induction t.
Exists O;Simpl;Auto.
Exists O;Simpl;Auto.
Exists O;Simpl;Auto.
Intros qx Hqx qy Hqy.
Elim Hqy;Elim Hqx;Intros x Eqx y Eqy.
Exists (plus x (Length qy)).
Simpl.
Rewrite Eqx ; Auto.
Save.
 
Lemma n_SO_Length_ConsArg : (t,t0:quasiterm)~<nat>(S O)=(Length (ConsArg t t0)).
Intros;Elim (Length_n_O t);Elim (Length_n_O t0);Intros;Simpl.
Replace (Length t) with (S x0);
Replace (Length t0) with (S x).
Elim x0;Simpl.
Discriminate.
Intros;Discriminate.
Save .

Lemma SIMPLE_SO : (t:quasiterm)(SIMPLE t)-><nat>(S O)=(Length t).
Induction t;Simpl;Intros;Auto.
Absurd False;Auto.
Save .

Lemma Length_SO_term : (t:quasiterm)(L_TERM t)
->(<nat>(S O)=(Length t))->(term t).
Induction t;Intros;Apply term_init;Simpl;Auto.
Apply (n_SO_Length_ConsArg q q0);Auto.
Save .

Lemma term_L_TERM_Length : (t:quasiterm)
(term t)->((L_TERM t)/\ (<nat>(S O)=(Length t))).
Induction t.
Intros;Simpl;Auto.
Intros;Simpl.
Split;Auto.
Elim H;Simpl;Intros h;Elim h;Auto.
Simpl;Intros.
Split;Auto.
Elim H0;Simpl;Intros h;Elim h;Intros;Split;Auto.
Simpl;Intros.
Split.
Elim H1;Simpl;Intros;Auto.
Elim H1;Simpl;Intros.
Absurd False;Auto.
Save.

End terms.

Section eq_quasiterm.

(***********************************************************************)
(************************ Structural predicates ***************************)
(***********************************************************************)

Definition BC:quasiterm->Prop:=
[t:quasiterm](<Prop>Case t
			of
			[_:var]False
			[_:fun]True
			[_:fun][_:quasiterm]False
			[_:quasiterm][t2:quasiterm]False
		    end)
.

Definition BV:quasiterm->Prop:=
[t:quasiterm](<Prop>Case t
			of
			[_:var]True
			[_:fun]False
			[_:fun][_:quasiterm]False
			[_:quasiterm][t2:quasiterm]False
		    end)
.

Definition BRoot:quasiterm->Prop:=
[t:quasiterm](<Prop>Case t
			of
			[_:var]False
			[_:fun]False
			[_:fun][_:quasiterm]True
			[_:quasiterm][t2:quasiterm]False
		    end)
.

Definition BConsArg:quasiterm->Prop:=
[t:quasiterm](<Prop>Case t
			of
			[_:var]False
			[_:fun]False
			[_:fun][_:quasiterm]False
			[_:quasiterm][t2:quasiterm]True
		    end)
.

(***********************************************************************)
(********************** Destroyers (Destructors): *************************)
(***********************************************************************)
Definition Destr1:=
[t:quasiterm](<quasiterm>Case t of
				V
				C
				[_:fun][p:quasiterm]p
				[p,_:quasiterm]p end).

Definition Destr2:=
[t:quasiterm](<quasiterm>Case t of
				V
				C
				[_:fun][p:quasiterm]p
				[_,q:quasiterm]q end).

Definition Destrvar:=[X:var][t:quasiterm](<var>Case t
of   [x:var]x
     [_:fun]X
     [_:fun][_:quasiterm]X
     [_,_:quasiterm]X end).


Definition Destrfun:=[F:fun][t:quasiterm](<fun>Case t
of   [_:var]F
     [l:fun]l
     [l:fun][_:quasiterm]l
     [p,q:quasiterm]F end).

(**********************************************************************)
(******************** Equal in the Set of quasiterms : *******************)
(**********************************************************************)

Lemma proj_C : (l1,l2:fun)(<quasiterm>(C l1)=(C l2))->(<fun>l1=l2).
Intros;Replace l1 with (Destrfun l1 (C l1));Auto.
Replace l2 with (Destrfun l1 (C l2));Auto.
Elim H;Auto.
Save.

Lemma proj_V : (x1,x2:var)(<quasiterm>(V x1)=(V x2))->(<var>x1=x2).
Intros;Replace x1 with (Destrvar x1 (V x1));Auto.
Replace x2 with (Destrvar x1 (V x2));Auto.
Elim H;Auto.
Save.

Lemma proj_Root1 : (t1,t2:quasiterm)(l1,l2:fun)
(<quasiterm>(Root l1 t1)=(Root l2 t2))->(<fun>l1=l2).
Intros;Replace l1 with (Destrfun l1 (Root l1 t1));Auto.
Replace l2 with (Destrfun l1 (Root l2 t2));Auto.
Elim H;Auto.
Save.

Lemma proj_Root2 : (t1,t2:quasiterm)(l1,l2:fun)
(<quasiterm>(Root l1 t1)=(Root l2 t2))->(<quasiterm>t1=t2).
Intros;Replace t1 with (Destr1 (Root l1 t1));Auto.
Replace t2 with (Destr2 (Root l2 t2));Auto.
Elim H;Auto.
Save.

Lemma proj_ConsArg1 : (t1,t2,t3,t4:quasiterm)
(<quasiterm>(ConsArg t1 t2)=(ConsArg t3 t4))->(<quasiterm>t1=t3).
Intros;Replace t1 with (Destr1 (ConsArg t1 t2));Auto.
Replace t3 with (Destr1 (ConsArg t3 t4));Auto.
Elim H;Auto.
Save.

Lemma proj_ConsArg2 : (t1,t2,t3,t4:quasiterm)
(<quasiterm>(ConsArg t1 t2)=(ConsArg t3 t4))->(<quasiterm>t2=t4).
Intros;Replace t2 with (Destr2 (ConsArg t1 t2));Auto.
Replace t4 with (Destr2 (ConsArg t3 t4));Auto.
Elim H;Auto.
Save.

(**********************************************************************)
(****************** Not equal in the Set of the quasiterms : **************)
(**********************************************************************)
Lemma C_diff_C : (l,l0:fun)(~<fun>l=l0)->(~(<quasiterm>(C l)=(C l0))).
Unfold not;Intros;Elim H;Apply proj_C;Auto.
Save.

Lemma V_diff_V : (x,y:var)(~(<var>x=y))->(~(<quasiterm>(V x)=(V y))).
Unfold not;Intros;Elim H;Apply proj_V;Auto.
Save.

Lemma ConsArg_diff_ConsArg :
(t,t0,t1,t2:quasiterm)((~(<quasiterm>t=t1))\/(~(<quasiterm>t0=t2)))
->(~(<quasiterm>(ConsArg t t0)=(ConsArg t1 t2))).
Unfold not;Intros;Elim H;Intros;Elim H1.
Apply proj_ConsArg1 with t0 t2;Auto.
Apply proj_ConsArg2 with t t1;Auto.
Save.

Lemma Root_diff_Root :
(l,l0:fun)(t,t0:quasiterm)((~(<fun>l=l0))\/(~(<quasiterm>t=t0)))
->(~(<quasiterm>(Root l t)=(Root l0 t0))).
Unfold not;Intros;Elim H;Intros;Elim H1.
Apply proj_Root1 with t t0;Auto.
Apply proj_Root2 with l l0;Auto.
Save.

(**********************************************************************)
(*********** Decidability of the equality in the Set of quasiterms : *********)
(**********************************************************************)
Lemma quasiterm_eq_decS :
(t,t0:quasiterm){<quasiterm>t=t0}+{~(<quasiterm>t=t0)}.
Induction t.
Induction t0.
(*t=(V ...)*)
Intros ;Elim (var_eq_decS v v0);Intros H.
Elim H;Auto.
Right;Apply V_diff_V;Auto.
Intros;Right;Apply (Diff quasiterm BV);Simpl;Auto.
Intros;Right;Apply (Diff quasiterm BV);Simpl;Auto.
Intros;Right;Apply (Diff quasiterm BV);Simpl;Auto.
(*t=(C ...)*)
Induction t0.
Intros;Right;Apply (Diff quasiterm BC);Simpl;Auto.
Intros;Elim (fun_eq_decS f f0);Intros H.
Elim H;Auto.
Right;Apply C_diff_C;Auto.
Intros;Right;Apply (Diff quasiterm BC);Simpl;Auto.
Intros;Right;Apply (Diff quasiterm BC);Simpl;Auto.
(*t=(Root...)*)
Induction t0.
Intros;Right;Apply (Diff quasiterm BRoot);Simpl;Auto.
Intros;Right;Apply (Diff quasiterm BRoot);Simpl;Auto.
Intros.
Elim (H q0);Intros y.
Rewrite y;Elim (fun_eq_decS f f0);Intros y0.
Rewrite y0;Auto.
Right;Simplify_eq ; Auto.
Right;Simplify_eq ; Auto.
Intros;Right;Simplify_eq.
(*t=(ConsArg ...)*)
Induction t0.
Intros;Right; Simplify_eq.
Intros;Right; Simplify_eq.
Intros;Right; Simplify_eq.
Intros y1 H1 y2 H2.
Elim (H y1);Intros E1.
Elim (H0 y2);Intros E2.
Rewrite E1 ; Rewrite E2 ; Auto.
Right ; Simplify_eq ; Tauto.
Right ; Simplify_eq ; Tauto.
Save.

Lemma quasiterm_eq_decP :
(t,t0:quasiterm)(<quasiterm>t=t0)\/(~<quasiterm>t=t0).
Intros;Elim (quasiterm_eq_decS t t0);Intros;Auto.
Save .

(**********************************************************************)
(********************* End of equality in quasiterm **********************)
(**********************************************************************)


(***********************************************************************)
(************************* End eq_quasiterm. ***************************)
(***********************************************************************)
End eq_quasiterm.
