(***************************************************************************)
(* File: applications.v                                                    *)
(* auteur: alex@sysal.ibp.fr                                               *)
(* date: 19/10/95                                                          *)
(***************************************************************************)
Require Export couples.

(***************************************************************************)
(***************************************************************************)
(* Relations n-aires                                                       *)
(***************************************************************************)
(***************************************************************************)
Definition n_aire := [n:nat][a:E][F:E->Prop]
  (subset F (cart_power n a))
.

Lemma lem_subset :
  (a,b,y:E)(F:E->Prop)
    (a=(subset F b))
    ->
    ((In y a) -> (In y b))
.
(* Pas de reciproque (dixit pagano pb des ensembles non decidables)        *)
(* Proof of lem_subset                                                     *)
(Intros;Elim (axs_comprehension F b y);Intros).
(Generalize H1;Rewrite <- H;Clear H H1 H2;Intros).
(Elim H1;Intros;Auto).

Save.
(* End of proof of lem_subset                                              *)

(***************************************************************************)
(***************************************************************************)
(* applications                                                            *)
(***************************************************************************)
(***************************************************************************)
Definition fun := [f:E]
  (Ex[a:E](Ex[b:E]((inc f (cartesien a b)))))
  /\  
 ((x,y,z:E)
    (In (couple x y) f)
    ->
    (In (couple x z) f)
    ->
    y=z
  )
.

(***************************************************************************)
(* Domaine d'une application                                               *)
(***************************************************************************)
Definition dom := [f:E]
  (subset [v0:E]((fun f) -> (Ex[v1:E]((In (couple v0 v1) f)))) (reunion (reunion f)))
.

(***************************************************************************)
(* Image d'une application                                                 *)
(***************************************************************************)
Definition Img := [f:E]
  (subset [v1:E]((fun f) -> (Ex[v0:E]((In (couple v0 v1) f)))) (reunion (reunion f)))
.

(***************************************************************************)
(* f est une application de a vers b                                       *)
(***************************************************************************)
Definition fun_sig := [f,a,b:E]
  (fun f) /\ ((dom f)=a) /\ (inc (Img f) b)
.

(***************************************************************************)
(* Trois lemmes evidents mais utiles                                       *)
(***************************************************************************)
Lemma lem_fun_sig_is_fun :
  (f,a,b:E)(fun_sig f a b)->(fun f)
.
(* Proof of lem_fun_sig_is_fun                                             *)
(Unfold fun_sig;Intros).
(Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_fun_sig_is_fun                                      *)

Lemma lem_fun_sig_dom :
  (f,a,b:E)(fun_sig f a b)->(dom f)=a
.
(* Proof of lem_fun_sig_dom                                                *)
(Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Auto).

Save.
(* End of proof of lem_fun_sig_dom                                         *)

Lemma lem_fun_sig_img :
  (f,a,b:E)(fun_sig f a b)->(inc (Img f) b)
.
(* Proof of lem_fun_sig_img                                                *)
(Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Auto).

Save.
(* End of proof of lem_fun_sig_img                                         *)

(***************************************************************************)
(* f est une application surjective de a vers b                            *)
(***************************************************************************)
Definition fun_surj := [f,a,b:E]
  (fun f) /\ ((dom f)=a) /\ ((Img f)=b)
.

(***************************************************************************)
(* f est une application injective                                         *)
(***************************************************************************)
Definition fun_inj := [f,a,b:E]
  (fun f) 
  /\  
  ((dom f)=a) 
  /\  
  (inc (Img f) b)
  /\ 
  ((b:E)(In b (Img f))->(ExU[a:E]((In (couple a b) f))))
.

(***************************************************************************)
(* f est une bijection                                                     *)
(***************************************************************************)
Definition fun_bij := [f,a,b:E]
  (fun f)
  /\ 
  ((dom f)=a) 
  /\  
  ((Img f)=b)
  /\ 
  ((b:E)(In b (Img f))->(ExU[a:E]((In (couple a b) f))))
.

(***************************************************************************)
(* Re-belote: Trois lemmes debiles ... mais utiles                         *)
(***************************************************************************)
Lemma lem_surj_to_sig :
  (f,a,b:E)(fun_surj f a b)->(fun_sig f a b)
.
(* Proof of lem_surj_to_sig                                                *)
(Unfold fun_surj;Unfold fun_sig;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Split;[Auto|Split;[Auto|Unfold inc;Intros]]).
(Generalize H2;Rewrite -> H1;Tauto).

Save.
(* End of proof of lem_surj_to_sig                                         *)

Lemma lem_inj_to_sig :
  (f,a,b:E)(fun_inj f a b)->(fun_sig f a b)
.
(* Proof of lem_inj_to_sig                                                 *)
(Unfold fun_inj;Unfold fun_sig;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Split;[Auto|Split;Auto]).

Save.
(* End of proof of lem_inj_to_sig                                          *)

Lemma lem_bij_to_sig :
  (f,a,b:E)(fun_bij f a b)->(fun_sig f a b)
.
(* Proof of lem_bij_to_sig                                                 *)
(Unfold fun_bij;Unfold fun_sig;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Split;[Auto|Split;[Auto|Unfold inc;Intros]]).
(Rewrite <- H1;Auto).

Save.
(* End of proof of lem_bij_to_sig                                          *)

(***************************************************************************)
(* Une bijection est une injection                                         *)
(***************************************************************************)
Lemma lem_bij_is_inj : 
  (f,a,b:E)
    (fun_bij f a b) -> (fun_inj f a b)
.
(* Proof of lem_bij_is_inj                                                 *)
Unfold fun_bij fun_inj;Intros.
Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros.
(Split;[Auto|Split;[Auto|Split;[Unfold inc;Intros;Rewrite <- H1;Auto|Auto]]]).

Save.
(* End of proof of lem_bij_is_inj                                          *)

(***************************************************************************)
(* Une bijection est une surjection                                        *)
(***************************************************************************)
Lemma lem_bij_is_surj : 
  (f,a,b:E)
    (fun_bij f a b) -> (fun_surj f a b)
.
(* Proof of lem_bij_is_inj                                                 *)
(Unfold fun_bij fun_surj;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Split;[Auto|Split;Auto]).

Save.
(* End of proof of lem_bij_is_inj                                          *)

(***************************************************************************)
(* Une injection surjective est une bijection                              *)
(***************************************************************************)
Lemma lem_inj_surj_is_bij :
  (f,a,b:E)
    (fun_surj f a b) -> (fun_inj f a b) -> (fun_bij f a b)
.
(* Proof of lem_inj_surj_is_bij                                            *)
Unfold fun_surj fun_inj fun_bij ;Intros.
Elim H;Clear H;Intros;Clear H;Elim H1;Clear H1;Intros;Clear H.
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Split;[Auto|Split;[Auto|Split;Auto]]).

Save.
(* End of proof of lem_inj_surj_is_bij                                     *)

(***************************************************************************)
(***************************************************************************)
(* L'application Vide                                                      *)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(* Vide est une application                                                *)
(***************************************************************************)
Lemma lem_vide_application :
  (fun Vide)
.
(* Proof of lem_vide_application                                           *)
(Unfold fun;Split;[Exists Vide;Exists Vide;Exact (lem_vide_is_inc (cartesien Vide Vide))|Intros]).
(Unfold Vide in H;Elim (lem_vide_propertie (couple x y) var_vide);Auto).

Save.
(* End of proof of lem_vide_application                                    *)

(***************************************************************************)
(* Le domaine de l'application Vide est Vide                               *)
(***************************************************************************)
Lemma lem_dom_vide :
  (dom Vide)=Vide
.
(* Proof of lem_dom_vide                                                   *)
(Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros;Unfold dom in H).
(Elim (axs_comprehension [v0:E](fun Vide)->(Ex [v1:E](In (couple v0 v1) Vide)) (reunion (reunion Vide)) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
Generalize H;Rewrite -> lem_reunion_vide;Rewrite -> lem_reunion_vide;Tauto.

(Elim (lem_vide_propertie v2 var_vide);Auto).

Save.
(* End of proof of  lem_dom_vide                                           *)

(***************************************************************************)
(* L'image de l'application Vide est Vide                                  *)
(***************************************************************************)
Lemma lem_img_vide :
  (Img Vide)=Vide
.
(* Proof of lem_img_vide                                                   *)
(Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros;Unfold Img in H).
(Elim (axs_comprehension [v1:E](fun Vide)->(Ex [v0:E](In (couple v0 v1) Vide)) (reunion (reunion Vide)) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
Elim H;Clear H;Intros.
(Generalize H;Rewrite -> lem_reunion_vide;Rewrite -> lem_reunion_vide;Tauto).

(Unfold Vide in H;Elim (lem_vide_propertie v2 var_vide);Auto).

Save.
(* End of proof of lem_img_vide                                            *)

(***************************************************************************)
(* Pour tout ensemble a, Vide est une application de Vide vers a           *)
(***************************************************************************)
Lemma lem_fun_vide_sig :
  (a:E)(fun_sig Vide Vide a)
.
(* Proof of lem_fun_vide_sig                                               *)
(Unfold fun_sig;Intros).
(Split;[Exact lem_vide_application|Split;[Exact lem_dom_vide|Rewrite -> lem_img_vide;Exact (lem_vide_is_inc a)]]).

Save.
(* End of proof of lem_fun_vide_sig                                        *)

(***************************************************************************)
(***************************************************************************)
(* Quelques proprietes sur application, domaine et image                   *)
(***************************************************************************)
(***************************************************************************)
Lemma lem_fun_and_dom :
  (f,a,b:E)
    (fun f) -> (In (couple a b) f) -> (In a (dom f))
.
(* Proof of lem_fun_and_dom                                                *)
(Intros;Unfold dom).
(Elim (axs_comprehension [v0:E](fun f)->(Ex [v1:E](In (couple v0 v1) f)) (reunion (reunion f)) a);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Intros;Exists b;Auto]).
(Elim (axs_reunion (reunion f) a);Intros;Apply H2;Clear H1 H2;Intros).
(Exists (singleton a);Split;[Idtac|Exact (lem_x_in_sing_x a)]).
(Elim (axs_reunion f (singleton a));Intros;Apply H2;Clear H1 H2;Intros).
(Exists (couple a b);Split;[Auto|Unfold couple;Elim (axs_paire (singleton a) (paire a b) (singleton a));Intros;Apply H2;Left;Reflexivity]).

Save.
(* End of proof of lem_fun_and_dom                                         *)

Lemma lem_fun_and_img :
  (f,a,b:E)
    (fun f) -> (In (couple a b) f) -> (In b (Img f))
.
(* Proof of lem_fun_and_img                                                *)
(Intros;Unfold Img).
(Elim (axs_comprehension [v1:E](fun f)->(Ex [v0:E](In (couple v0 v1) f)) (reunion (reunion f)) b);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Intros;Exists a;Auto]).
(Elim (axs_reunion (reunion f) b);Intros;Apply H2;Clear H1 H2;Intros;Exists (paire a b)).
(Split;[Idtac|Elim (axs_paire a b b);Intros;Apply H2;Right;Reflexivity]).
(Elim (axs_reunion f (paire a b));Intros;Apply H2;Clear H1 H2;Intros).
(Exists (couple a b);Split;[Auto|Unfold couple;Elim (axs_paire (singleton a) (paire a b) (paire a b));Intros;Apply H2;Right;Reflexivity]).

Save.
(* End of proof of lem_fun_and_img                                         *)

(***************************************************************************)
(* Si f est une application et x est un element du domaine de f alors il   *)
(* existe un y tel que (x,y) in f                                          *)
(***************************************************************************)
Lemma lem_App :
  (f,x:E)
    (fun f)
    ->
    (In x (dom f))
    ->
    (Ex[y:E]((In y (Img f)) /\ (In (couple x y) f)))
.
(* Proof of lem_App                                                   *)
Intros;Unfold dom in H0.
(Elim (axs_comprehension [v0:E](fun f)->(Ex [v1:E](In (couple v0 v1) f)) (reunion (reunion f)) x);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Generalize (H1 H);Clear H1;Intros).
(Elim H1;Clear H1;Intros).
(Exists x0;Split;[Exact (lem_fun_and_img f x x0 H H1)|Auto]).

Save.
(* End of proof of lem_App                                            *)

(***************************************************************************)
(* Si f est une application et que y est un element de l'image de f alors  *)
(* il existe au moins un x du domaine de f tel que (x,y) in f              *)
(***************************************************************************)
Lemma lem_App_rev :
  (f,y:E)
    (fun f)
    ->
    (In y (Img f))
    ->
    (Ex[x:E]((In x (dom f)) /\ (In (couple x y) f)))
.
(* Proof of lem_App_rev                                               *)
(Intros;Unfold Img in H0).
(Elim (axs_comprehension [v1:E](fun f)->(Ex [v0:E](In (couple v0 v1) f)) (reunion (reunion f)) y);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Generalize (H1 H);Clear H1;Intros;Elim H1;Clear H1;Intros).
(Exists x;Split;[Exact (lem_fun_and_dom f x y H H1)|Auto]).

Save.
(* End of proof of lem_App_rev                                        *)

(***************************************************************************)
(***************************************************************************)
(* Composition d'application                                               *)
(***************************************************************************)
(***************************************************************************)
Definition comp := [f,g,a,b,c:E][Af:(fun_sig f a b)][Ag:(fun_sig g b c)]
  (subset
    [c:E](Ex[y:E]((In (couple (first c) y) f) /\ (In (couple y (second c)) g)))
    (cartesien a c)
  )
.

(***************************************************************************)
(* Il peut paraitre necessaire de reformuler la definition de la           *)
(* composition.                                                            *)
(***************************************************************************)
Definition comp2 := [f,g,a,b,c:E]
  (subset
    [x:E](fun_sig f a b)->(fun_sig g b c)->(Ex[y:E]((In (couple (first x) y) f) /\ (In (couple y (second x)) g)))
    (cartesien a c)
  )
.

Lemma lem_comp2_eq_comp :
  (f,g,a,b,c:E)(Af:(fun_sig f a b))(Ag:(fun_sig g b c))
  (comp f g a b c Af Ag)=(comp2 f g a b c)
.
(* Proof of lem_comp2_eq_comp                                              *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold comp in H.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold comp2.
(Elim (axs_comprehension [x:E](fun_sig f a b)->(fun_sig g b c)->(Ex[y:E](and (In (couple (first x) y) f) (In (couple y (second x)) g))) (cartesien a c) v2);Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Auto|Intros;Exists x;Split;Auto]).

Unfold comp2 in H.
(Elim (axs_comprehension [x:E](fun_sig f a b)->(fun_sig g b c)->(Ex[y:E](and (In (couple (first x) y) f) (In (couple y (second x)) g))) (cartesien a c) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim (H0 Af Ag);Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold comp.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) v2);Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Auto|Exists x;Split;Auto]).

Save.
(* End of proof of lem_comp2_eq_comp                                       *)

(***************************************************************************)
(* comp est une application.                                               *)
(***************************************************************************)
Lemma lem_comp_is_fun :
  (f,g,a,b,c:E)(Af:(fun_sig f a b))(Ag:(fun_sig g b c))
    (fun (comp f g a b c Af Ag))
.
(* Proof of lem_comp_is_fun                                                *)
Intros.
(Unfold fun;Split;[Exists a;Exists c|Intros]).
(Unfold inc;Intros).
Unfold comp in H.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear  H;Intros;Elim H0;Clear  H0;Intros;Elim H0;Clear  H0;Intros).
(Unfold fun_sig in Af;Unfold fun_sig in Ag).
(Elim Af;Clear Af;Intros).
(Elim H3;Clear H3;Intros).
(Elim Ag;Clear Ag;Intros).
(Elim H6;Clear H6;Intros).
Auto.

(Unfold comp in H;Unfold comp in H0).
(Elim (axs_comprehension [c0:E](Ex[y0:E](and (In (couple (first c0) y0) f) (In (couple y0 (second c0)) g))) (cartesien a c) (couple x y));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) (couple x z));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim H0;Clear H0;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Generalize H3;Clear H3;Rewrite -> (lem_first_propertie x z);Intros).
(Generalize H4;Clear H4;Rewrite -> (lem_second_propertie x z);Intros).
(Generalize H1;Clear H1;Rewrite -> (lem_first_propertie x y);Intros).
(Generalize H2;Clear H2;Rewrite -> (lem_second_propertie x y);Intros).
(Unfold fun_sig in Af;Elim Af;Intros).
(Unfold fun_sig in Ag;Elim Ag;Intros).
(Unfold fun in H5;Unfold fun in H7;Clear H6 H8).
(Elim H5;Clear H5;Intros;Clear H5).
(Elim H7;Clear H7;Intros;Clear H5).
(Generalize (H6 x x0 x1 H1 H3);Clear H1 H3;Intros).
(Generalize H4;Clear H4;Rewrite <- H1;Clear H1;Intros).
Exact (H7 x0 y z H2 H4).

Save.
(* End of proof of lem_comp_is_fun                                         *)

(***************************************************************************)
(* Le domaine de g o f est le domaine de f                                 *)
(***************************************************************************)
Lemma lem_comp_dom :
  (f,g,a,b,c:E)(Af:(fun_sig f a b))(Ag:(fun_sig g b c))
    (dom (comp f g a b c Af Ag)) = (dom f)
.
(* Proof of lem_comp_dom                                                   *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold dom in H.
(Elim (axs_comprehension [v0:E](fun (comp f g a b c Af Ag))->(Ex [v1:E](In (couple v0 v1) (comp f g a b c Af Ag))) (reunion (reunion (comp f g a b c Af Ag))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Generalize (H0 (lem_comp_is_fun f g a b c Af Ag));Clear H0;Intros).
(Elim H0;Clear H0;Intros).
Unfold comp in H0.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Generalize H1;Clear H1 H2;Rewrite -> (lem_first_propertie v2 x);Intros).
(Unfold fun_sig in Af;Elim Af;Intros).
Exact (lem_fun_and_dom f v2 x0 H2 H1).

(Generalize Af Ag;Intros).
(Unfold fun_sig in Af0;Unfold fun_sig in Ag0).
(Elim Af;Clear Af;Intros;Elim H1;Clear H1;Intros).
(Elim Ag;Clear Ag;Intros;Elim H4;Clear H4;Intros).
(Generalize (lem_App f v2 H0 H);Intros;Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros).
Unfold inc in H2.
(Generalize (H2 x);Intros).
(Generalize (H8 H6);Clear H6 H8;Intros).
(Cut (In x (dom g));[Intros|Rewrite -> H4;Auto]).
(Generalize (lem_App g x H3 H8);Intros).
(Elim H9;Clear H9;Intros;Elim H9;Clear H9;Intros).
Unfold dom.
(Elim (axs_comprehension [v0:E](fun (comp f g a b c Af0 Ag0))->(Ex [v1:E](In (couple v0 v1) (comp f g a b c Af0 Ag0))) (reunion (reunion (comp f g a b c Af0 Ag0))) v2);Intros;Apply H12;Clear H11 H12;Intros).
Split.
(Elim (axs_reunion (reunion (comp f g a b c Af0 Ag0)) v2);Intros;Apply H12;Clear H11 H12;Intros).
(Exists (singleton v2);Split;[Idtac|Apply lem_x_in_sing_x]).
(Elim (axs_reunion (comp f g a b c Af0 Ag0) (singleton v2));Intros;Intros;Apply H12;Clear H11 H12;Intros).
(Exists (couple v2 x0);Split;[Idtac|Unfold couple;Elim (axs_paire (singleton v2) (paire v2 x0) (singleton v2));Intros;Apply H12;Left;Reflexivity]).
Unfold comp.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) (couple v2 x0));Intros;Apply H12;Clear H11 H12;Intros).
(Split;[Idtac|Exists x]).
(Elim (lem_cartesian_propertie a c (couple v2 x0));Intros;Apply H12;Clear H11 H12;Intros).
(Exists v2;Exists x0).
(Rewrite <- H1;Split;[Auto|Split;[Idtac|Reflexivity]]).
(Unfold inc in H5;Generalize (H5 x0);Intros;Exact (H11 H9)).

(Rewrite -> (lem_first_propertie v2 x0);Rewrite -> (lem_second_propertie v2 x0);Split;Auto).

(Intros;Exists x0).
Unfold comp.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) (couple v2 x0));Intros;Apply H13;Clear H12 H13;Intros).
(Split;[Idtac|Exists x]).
(Elim (lem_cartesian_propertie a c (couple v2 x0));Intros;Apply H13;Clear H12 H13;Intros).
Exists v2;Exists x0.
Rewrite <- H1;Split;[Auto|Split;[Idtac|Reflexivity]].
Unfold inc in H5;Generalize (H5 x0);Intros;Exact (H12 H9).

Rewrite -> (lem_first_propertie v2 x0);Rewrite -> (lem_second_propertie v2 x0);Split;Auto.

Save.
(* End of proof of lem_comp_dom                                            *)

(***************************************************************************)
(* L'image de g o f est incluse dans l'image de f                          *)
(***************************************************************************)
Lemma lem_comp_img :
  (f,g,a,b,c:E)(Af:(fun_sig f a b))(Ag:(fun_sig g b c))
    (inc (Img (comp f g a b c Af Ag)) (Img g))
.
(* Proof of lem_comp_img                                                   *)
(Intros;Unfold inc;Intros).
(Elim Af;Intros;Elim H1;Clear H1;Intros).
(Elim Ag;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_App_rev (comp f g a b c Af Ag) v0 (lem_comp_is_fun f g a b c Af Ag) H);Intros).
(Elim H6;Clear H6;Intros).
Unfold comp in H7.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) (couple x v0));Intros;Generalize (H8 H7);Clear H7 H8 H9;Intros).
(Elim H7;Clear H7;Intros;Elim H8;Clear H8;Intros;Elim H8;Clear H8;Intros).
(Generalize H9;Clear H9;Rewrite -> (lem_second_propertie x v0);Intros).
Exact (lem_fun_and_img g x0 v0 H3 H9).

Save.
(* End of proof of lem_comp_img                                            *)

(***************************************************************************)
(* (x,y) in f et (y,z) in g -> (x,z) in g o f                              *)
(***************************************************************************)
Lemma lem_comp_make :
  (f,g,a,b,c,x,y,z:E)(Af:(fun_sig f a b))(Ag:(fun_sig g b c))
    (In (couple x y) f)
    ->
    (In (couple y z) g)
    ->
    (In (couple x z) (comp f g a b c Af Ag))
.
(* Proof of lem_comp_make                                                  *)
(Intros;Unfold comp).
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) (couple x z));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists y;Rewrite -> (lem_first_propertie x z);Rewrite -> (lem_second_propertie x z);Split;Auto]).
(Elim Af;Clear Af;Intros;Elim H2;Clear H2;Intros).
(Elim Ag;Clear Ag;Intros;Elim H5;Clear H5;Intros).
(Elim (lem_cartesian_propertie a c (couple x z));Intros;Apply H8;Clear H7 H8;Intros).
(Exists x;Exists z;Split;[Rewrite <- H2;Exact (lem_fun_and_dom f x y H1 H)|Split;[Idtac|Reflexivity]]).
(Generalize (lem_fun_and_img g y z H4 H0);Intros).
(Unfold inc in H6;Exact (H6 z H7)).

Save.
(* End of proof of lem_comp_make                                           *)

(***************************************************************************)
(* (x,z) in g o f -> il existe un y tel que (x,y) in f et (y,z) in g       *)
(***************************************************************************)
Lemma lem_comp_intermed :
  (f,g,a,b,c,x,z:E)(Af:(fun_sig f a b))(Ag:(fun_sig g b c))
    (In (couple x z) (comp f g a b c Af Ag))
    ->
    (Ex[y:E]((In (couple x y) f) /\ (In (couple y z) g)))
.
(* Proof of lem_comp_intermed                                              *)
Intros;Unfold comp in H.
(Elim (axs_comprehension [c0:E](Ex[y:E](and (In (couple (first c0) y) f) (In (couple y (second c0)) g))) (cartesien a c) (couple x z));Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Generalize H0;Clear H0;Rewrite -> (lem_first_propertie x z);Intros).
(Generalize H1;Clear H1;Rewrite -> (lem_second_propertie x z);Intros).
(Exists x0;Split;Auto).

Save.
(* End of proof of lem_comp_intermed                                       *)

(***************************************************************************)
(***************************************************************************)
(* Redefinissons maintenant une application dans le style f(x) = y         *)
(***************************************************************************)
(***************************************************************************)
Definition App := [f:E][Af:(fun f)][x:E]
  (subset 
    [z:E](Ex[y:E]((In (couple x y) f) /\ (In z y)))
    (reunion (Img f))
  )
.

Lemma lem_eval_propertie :
  (f,x,y:E)(Af:(fun f))
    (In (couple x y) f)
    ->
    y=(App f Af x)
.
(* Proof of lem_eval_propertie                                             *)
(Intros;Apply axs_extensionnalite;Intros).
(Unfold iff;Split;Intros;Unfold App ).
(Elim (axs_comprehension [z:E](Ex [y:E](and (In (couple x y) f) (In z y))) (reunion (Img f)) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists y;Split;Auto]).
(Elim (axs_reunion (Img f) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Exists y;Split;[Exact (lem_fun_and_img f x y Af H)|Auto]).

Unfold App in H0.
(Elim (axs_comprehension [z:E](Ex [y:E](and (In (couple x y) f) (In z y))) (reunion (Img f)) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim Af;Clear Af;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Generalize (H4 x x0 y H1 H);Clear H4;Intros).
(Rewrite <- H4;Auto).

Save.
(* End of proof of lem_eval_propertie                                      *)

Lemma lem_eval_prop2 :
  (f,x:E)(Af:(fun f))
    (In x (dom f))
    ->
    (In (couple x (App f Af x)) f)
.
(* Proof of lem_eval_prop2                                                 *)
(Unfold dom;Intros).
(Elim (axs_comprehension [v0:E](fun f)->(Ex [v1:E](In (couple v0 v1) f)) (reunion (reunion f)) x);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear  H;Intros).
(Generalize (H0 Af);Clear H0;Intros;Elim H0;Clear H0;Intros).
(Generalize (lem_eval_propertie f x x0 Af H0);Intros).
(Rewrite <- H1;Auto).

Save.
(* End of proof of lem_eval_prop2                                          *)

(***************************************************************************)
(* Si v2 est dans f, alors v2 est un couple ...                            *)
(***************************************************************************)
Lemma lem_x_in_fun_form :
  (f:E)(Af:(fun f))(x:E)
  (In x f)
  ->
  (Ex[a:E](Ex[b:E]((x=(couple a b)))))
.
(* Proof of lem_x_in_fun_form                                              *)
Intros.
(Elim Af;Clear Af;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Generalize (H0 x H);Clear H H0;Intros).
(Elim (lem_cartesian_propertie x0 x1 x);Intros;Generalize (H0 H);Clear H H0 H2;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Exists x2;Exists x3;Auto).

Save.
(* End of proof of lem_x_in_fun_form                                       *)

(***************************************************************************)
(***************************************************************************)
(* Image d'une partie du domaine d'une application f par f                 *)
(***************************************************************************)
(***************************************************************************)
Definition fun_img_directe := [f:E][Af:(fun f)][c:E]
  (subset 
    [x:E]((In c (parties (dom f)))
    ->(Ex[y:E]((In y c) /\ (x=(App f Af y)))))
    (Img f)
  )
.

(***************************************************************************)
(* On peut definir l'image reciproque d'une partie de l'image d'une        *)
(* application f.                                                          *)
(***************************************************************************)
Definition fun_img_inverse := [f:E][Af:(fun f)][d:E]
  (subset
    [x:E]((In d (parties (Img f)))->(In (App f Af x) d))
    (dom f)
  )
.

(***************************************************************************)
(* Ces deux notions permettent de definir une application des parties de   *)
(* (dom f) dans les parties de (Img f)                                     *)
(***************************************************************************)
Definition fun_part := [f:E][Af:(fun f)]
  (subset
    [x:E](Ex[y:E](x=(couple y (fun_img_directe f Af y))))
    (cartesien (parties (dom f)) (parties (Img f)))
  )
.

(***************************************************************************)
(* fun_part est une application                                            *)
(***************************************************************************)
Lemma lem_fun_part_is_fun :
  (f:E)(Af:(fun f))
    (fun (fun_part f Af))
.
(* Proof of lem_fun_part_is_fun                                             *)
(Intros;Unfold fun;Split;Intros).
(Exists (parties (dom f));Exists (parties (Img f));Unfold inc;Intros).
Unfold fun_part in H.
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

(Unfold fun_part in H;Unfold fun_part in H0).
(Elim (axs_comprehension [x0:E](Ex [y0:E](eq ? x0 (couple y0 (fun_img_directe f Af y0)))) (cartesien (parties (dom f)) (parties (Img f))) (couple x y));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H1;Clear H1;Intros).
(Elim (axs_comprehension [x0:E](Ex [y:E](eq ? x0 (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple x z));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x0 y (fun_img_directe f Af x0));Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros).
(Rewrite -> H3;Clear H3;Rewrite <- H1;Clear H1).
(Elim (lem_couple_propertie x x1 z (fun_img_directe f Af x1));Intros;Generalize (H1 H2);Clear H2 H3 H1;Intros).
(Elim H1;Clear H1;Intros).
(Rewrite -> H2;Clear H2;Rewrite <- H1;Clear H1;Reflexivity).

Save.
(* End of proof of lem_fun_part_is_fun                                     *)

(***************************************************************************)
(* Le domaine de (fun_part f ?) est l'ensemble des parties du domaine de f *)
(***************************************************************************)
Lemma lem_fun_part_dom :
  (f:E)(Af:(fun f))
    (dom (fun_part f Af))=(parties (dom f))
.
(* Proof of lem_fun_part_dom                                               *)
(Intros;Apply axs_extensionnalite;Intros).
(Unfold iff;Split;Intros).
Unfold dom in H.
(Elim (axs_comprehension [v0:E](fun (fun_part f Af))->(Ex [v1:E](In (couple v0 v1) (fun_part f Af))) (reunion (reunion (fun_part f Af))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Generalize (H0 (lem_fun_part_is_fun f Af));Clear H0;Intros).
(Elim H0;Clear H0;Intros).
Unfold fun_part in H0.
(Elim (axs_comprehension [x0:E](Ex [y:E](eq ? x0 (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
(Elim (lem_cartesian_propertie (parties (dom f)) (parties (Img f)) (couple v2 x));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Rewrite -> H3;Auto).

Unfold dom.
(Elim (axs_comprehension [v0:E](fun (fun_part f Af))->(Ex [v1:E](In (couple v0 v1) (fun_part f Af))) (reunion (reunion (fun_part f Af))) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Split;Intros).
(Elim (axs_reunion (reunion (fun_part f Af)) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Exists (singleton v2);Split;[Idtac|Apply lem_x_in_sing_x]).
(Elim (axs_reunion (fun_part f Af) (singleton v2));Intros;Apply H1;Clear H0 H1;Intros).
(Exists (couple v2 (fun_img_directe f Af v2));Split;[Idtac|Unfold couple;Elim (axs_paire (singleton v2) (paire v2 (fun_img_directe f Af v2)) (singleton v2));Intros;Apply H1;Left;Reflexivity]).
Unfold fun_part.
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple v2 (fun_img_directe f Af v2)));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists v2;Reflexivity]).
(Elim (lem_cartesian_propertie (parties (dom f)) (parties (Img f)) (couple v2 (fun_img_directe f Af v2)));Intros;Apply H1;Clear H0 H1;Intros).
(Exists v2;Exists (fun_img_directe f Af v2);Split;[Auto|Split;[Idtac|Reflexivity]]).
(Elim (axs_parties (Img f) (fun_img_directe f Af v2));Intros;Apply H1;Clear H0 H1;Intros).
Unfold fun_img_directe in H0.
(Elim (axs_comprehension [x:E](In v2 (parties (dom f)))->(Ex [y:E](and (In y v2) (eq ? x (App f Af y)))) (Img f) v3);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Auto).

(Exists (fun_img_directe f Af v2);Unfold fun_part).
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple v2 (fun_img_directe f Af v2)));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists v2;Reflexivity]).
(Elim (lem_cartesian_propertie (parties (dom f)) (parties (Img f)) (couple v2 (fun_img_directe f Af v2)));Intros;Apply H2;Clear H1 H2;Intros).
(Exists v2;Exists (fun_img_directe f Af v2);Split;[Auto|Split;[Idtac|Reflexivity]]).
(Elim (axs_parties (Img f) (fun_img_directe f Af v2));Intros;Apply H2;Clear H1 H2;Intros).
Unfold fun_img_directe in H1.
(Elim (axs_comprehension [x:E](In v2 (parties (dom f)))->(Ex [y:E](and (In y v2) (eq ? x (App f Af y)))) (Img f) v3);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Auto).

Save.
(* End of proof of lem_fun_part_dom                                        *)

(***************************************************************************)
(* L'image de (fun_part f ?) est egale aux parties de (Img f)              *)
(***************************************************************************)
Lemma lem_fun_part_img :
  (f:E)(Af:(fun f))
    (Img (fun_part f Af))=(parties (Img f))
.
(* Proof of lem_fun_part_img                                               *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold Img in H.
(Elim (axs_comprehension [v1:E](fun (fun_part f Af))->(Ex [v0:E](In (couple v0 v1) (fun_part f Af))) (reunion (reunion (fun_part f Af))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Generalize (H0 (lem_fun_part_is_fun f Af));Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold fun_part in H0.
(Elim (axs_comprehension [x0:E](Ex [y:E](eq ? x0 (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple x v2));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
(Elim H1;Clear H1;Intros).
(Elim (lem_cartesian_propertie (parties (dom f)) (parties (Img f)) (couple x v2));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x1 v2 x2);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros).
(Rewrite -> H4;Auto).

Unfold Img.
(Elim (axs_comprehension [v1:E](fun (fun_part f Af))->(Ex [v0:E](In (couple v0 v1) (fun_part f Af))) (reunion (reunion (fun_part f Af))) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Split;Intros).
(Elim (axs_reunion (reunion (fun_part f Af)) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Exists (paire (fun_img_inverse f Af v2) v2);Split;[Idtac|Elim (axs_paire (fun_img_inverse f Af v2) v2 v2);Intros;Apply H1;Right;Reflexivity]).
(Elim (axs_reunion (fun_part f Af) (paire (fun_img_inverse f Af v2) v2));Intros;Apply H1;Clear H0 H1;Intros).
(Exists (couple (fun_img_inverse f Af v2) v2);Split;[Idtac|Unfold couple;Elim (axs_paire (singleton (fun_img_inverse f Af v2)) (paire (fun_img_inverse f Af v2) v2) (paire (fun_img_inverse f Af v2) v2));Intros;Apply H1;Right;Reflexivity]).
Unfold fun_part.
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple (fun_img_inverse f Af v2) v2));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists (fun_img_inverse f Af v2)]).
(Elim (lem_cartesian_propertie (parties (dom f)) (parties (Img f)) (couple (fun_img_inverse f Af v2) v2));Intros;Apply H1;Clear H0 H1;Intros).
(Exists (fun_img_inverse f Af v2);Exists v2;Split;[Idtac|Split;[Auto|Reflexivity]]).
(Elim (axs_parties (dom f) (fun_img_inverse f Af v2));Intros;Apply H1;Clear H0 H1;Intros).
Unfold fun_img_inverse in H0.
(Elim (axs_comprehension [x:E](In v2 (parties (Img f)))->(In (App f Af x) v2) (dom f) v3);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Auto).

(Elim (lem_couple_propertie (fun_img_inverse f Af v2) (fun_img_inverse f Af v2) v2 (fun_img_directe f Af (fun_img_inverse f Af v2)));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Reflexivity|Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros]).
Unfold fun_img_directe.
(Elim (axs_comprehension [x:E](In (fun_img_inverse f Af v2) (parties (dom f)))->(Ex[y:E](and (In y (fun_img_inverse f Af v2)) (eq ? x (App f Af y)))) (Img f) v0);Intros;Apply H2;Clear H1 H2;Intros).
(Split;Intros).
(Elim (axs_parties (Img f) v2);Intros;Generalize (H1 H);Clear H1;Intros).
Exact (H1 v0 H0).

(Elim (axs_parties (Img f) v2);Intros;Generalize (H2 H);Clear H2 H3;Intros).
(Generalize (H2 v0 H0);Clear H2;Intros).
(Generalize (lem_App_rev f v0 Af H2);Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Exists x;Split).
Unfold fun_img_inverse.
(Elim (axs_comprehension [x0:E](In v2 (parties (Img f)))->(In (App f Af x0) v2) (dom f) x);Intros;Apply H6;Clear H5 H6;Intros).
(Split;[Auto|Intros]).
(Generalize (lem_eval_propertie f x v0 Af H4);Intros).
(Rewrite <- H6;Auto).

Exact (lem_eval_propertie f x v0 Af H4).

Unfold fun_img_directe in H0.
(Elim (axs_comprehension [x:E](In (fun_img_inverse f Af v2) (parties (dom f)))->(Ex[y:E](and (In y (fun_img_inverse f Af v2)) (eq ? x (App f Af y)))) (Img f) v0);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
(Cut (In (fun_img_inverse f Af v2) (parties (dom f)));Intros).
(Generalize (H1 H2);Clear H1 H2;Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
Unfold fun_img_inverse in H1.
(Elim (axs_comprehension [x0:E](In v2 (parties (Img f)))->(In (App f Af x0) v2) (dom f) x);Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros;Generalize (H3 H);Clear H3;Intros).
(Rewrite -> H2;Auto).

(Clear H1;Elim (axs_parties (dom f) (fun_img_inverse f Af v2));Intros;Apply H2;Clear H1 H2;Intros).
Unfold fun_img_inverse in H1.
(Elim (axs_comprehension [x:E](In v2 (parties (Img f)))->(In (App f Af x) v2) (dom f) v3);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Auto).

Exists (fun_img_inverse f Af v2).
Unfold fun_part.
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple (fun_img_inverse f Af v2) v2));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists (fun_img_inverse f Af v2)]).
(Elim (lem_cartesian_propertie (parties (dom f)) (parties (Img f)) (couple (fun_img_inverse f Af v2) v2));Intros;Apply H2;Clear H1 H2;Intros).
(Exists (fun_img_inverse f Af v2);Exists v2;Split;[Idtac|Split;[Auto|Reflexivity]]).
(Elim (axs_parties (dom f) (fun_img_inverse f Af v2));Intros;Apply H2;Clear H1 H2;Intros).
Unfold fun_img_inverse in H1.
(Elim (axs_comprehension [x:E](In v2 (parties (Img f)))->(In (App f Af x) v2) (dom f) v3);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Generalize (H2 H);Clear H2;Intros;Auto).

(Elim (lem_couple_propertie (fun_img_inverse f Af v2) (fun_img_inverse f Af v2) v2 (fun_img_directe f Af (fun_img_inverse f Af v2)));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Reflexivity|Apply axs_extensionnalite;Unfold iff;Split;Intros]).
Unfold fun_img_directe.
(Elim (axs_comprehension [x:E](In (fun_img_inverse f Af v2) (parties (dom f)))->(Ex[y:E](and (In y (fun_img_inverse f Af v2)) (eq ? x (App f Af y)))) (Img f) v0);Intros;Apply H3;Clear H2 H3;Intros).
(Split;Elim (axs_parties (Img f) v2);Intros).
Exact (H2 H v0 H1).

(Elim (axs_parties (Img f) v2);Intros).
(Generalize (H5 H v0 H1);Clear H5 H6;Intros).
(Generalize (lem_App_rev f v0 Af H5);Intros).
(Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros).
(Exists x;Split).
Unfold fun_img_inverse.
(Elim (axs_comprehension [x0:E](In v2 (parties (Img f)))->(In (App f Af x0) v2) (dom f) x);Intros;Apply H9;Clear H8 H9;Intros).
(Split;[Auto|Intros]).
(Generalize (lem_eval_propertie f x v0 Af H7);Intros).
(Rewrite <- H9;Auto).

Exact (lem_eval_propertie f x v0 Af H7).

Unfold fun_img_directe in H1.
(Elim (axs_comprehension [x:E](In (fun_img_inverse f Af v2) (parties (dom f)))->(Ex[y:E](and (In y (fun_img_inverse f Af v2)) (eq ? x (App f Af y)))) (Img f) v0);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros).
(Cut (In (fun_img_inverse f Af v2) (parties (dom f)));Intros).
(Generalize (H2 H3);Clear H2 H3;Intros).
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
Unfold fun_img_inverse in H2.
(Elim (axs_comprehension [x0:E](In v2 (parties (Img f)))->(In (App f Af x0) v2) (dom f) x);Intros;Generalize (H4 H2);Clear H2 H4 H5;Intros).
(Elim H2;Clear H2;Intros;Generalize (H4 H);Clear H4;Intros).
(Rewrite -> H3;Auto).

(Clear H2;Elim (axs_parties (dom f) (fun_img_inverse f Af v2));Intros;Apply H3;Clear H2 H3;Intros).
Unfold fun_img_inverse in H2.
(Elim (axs_comprehension [x:E](In v2 (parties (Img f)))->(In (App f Af x) v2) (dom f) v3);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros;Auto).

Save.
(* End of proof of lem_fun_part_img                                        *)

(***************************************************************************)
(* Les lemmes dont on avait besoin dans la preuve d'avant                  *)
(***************************************************************************)
(***************************************************************************)
(* (fun_img_inverse f Af x) est un element de (parties (dom f))            *)
(***************************************************************************)
Lemma lem_fun_img_inv_is_in_pdom :
  (f:E)(Af:(fun f))(x:E)
    (In x (parties (Img f)))
    ->
    (In (fun_img_inverse f Af x) (parties (dom f)))
.
(* Proof of lem_fun_img_inv_is_in_pdom                                     *)
Intros.
(Elim (axs_parties (dom f) (fun_img_inverse f Af x));Intros;Apply H1;Clear H0 H1;Intros).
Unfold fun_img_inverse in H0.
(Elim (axs_comprehension [x0:E](In x (parties (Img f)))->(In (App f Af x0) x) (dom f) v3);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Auto).

Save.
(* End of proof of lem_fun_img_inv_is_in_pdom                              *)

Lemma lem_fun_img_prop1 :
  (f:E)(Af:(fun f))(x:E)
    (In x (parties (Img f)))
    ->
    (fun_img_directe f Af (fun_img_inverse f Af x))=x
.
(* Proof of lem_fun_img_prop1                                              *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold fun_img_directe in H0.
(Elim (axs_comprehension [x0:E](In (fun_img_inverse f Af x) (parties (dom f)))->(Ex[y:E](and (In y (fun_img_inverse f Af x)) (eq ? x0 (App f Af y)))) (Img f) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
(Generalize (H1 (lem_fun_img_inv_is_in_pdom f Af x H));Clear H1;Intros).
(Elim H1;Clear H1;Intros).
(Elim H1;Clear H1;Intros).
Unfold fun_img_inverse in H1.
(Elim (axs_comprehension [x1:E](In x (parties (Img f)))->(In (App f Af x1) x) (dom f) x0);Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros;Generalize (H3 H);Clear H3;Intros).
(Rewrite -> H2;Auto).

Unfold fun_img_directe.
(Elim (axs_comprehension [x0:E](In (fun_img_inverse f Af x) (parties (dom f)))->(Ex[y:E](and (In y (fun_img_inverse f Af x)) (eq ? x0 (App f Af y)))) (Img f) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;Intros).
(Elim (axs_parties (Img f) x);Intros;Generalize (H1 H);Clear H1 H2;Intros;Auto).

(Elim (axs_parties (Img f) x);Intros;Generalize (H2 H);Clear H2 H3;Intros).
(Generalize (H2 v2 H0);Clear H2;Intros).
(Generalize (lem_App_rev f v2 Af H2);Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Exists x0;Split).
Unfold fun_img_inverse.
(Elim (axs_comprehension [x1:E](In x (parties (Img f)))->(In (App f Af x1) x) (dom f) x0);Intros;Apply H6;Clear H5 H6;Intros).
(Split;[Auto|Intros;Clear H5]).
(Generalize (lem_eval_propertie f x0 v2 Af H4);Intros).
(Rewrite <- H5;Auto).

Exact (lem_eval_propertie f x0 v2 Af H4).

Save.
(* End of proof of lem_fun_img_prop1                                       *)

(***************************************************************************)
(* (fun_img_directe f Af x) est element de (parties (Img f))               *)
(***************************************************************************)
Lemma lem_fun_img_dir_is_in_pimg :
  (f:E)(Af:(fun f))(x:E)
    (In x (parties (dom f)))
    ->
    (In (fun_img_directe f Af x) (parties (Img f)))
.
(* Proof of lem_fun_img_dir_is_in_pimg                                     *)
Intros.
(Elim (axs_parties (Img f) (fun_img_directe f Af x));Intros;Apply H1;Clear H0 H1;Intros).
Unfold fun_img_directe in H0.
(Elim (axs_comprehension [x0:E](In x (parties (dom f)))->(Ex [y:E](and (In y x) (eq ? x0 (App f Af y)))) (Img f) v3);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Auto).

Save.
(* End of proof of lem_fun_img_dir_is_in_pimg                              *)

Lemma lem_fun_img_prop2 :
  (f:E)(Af:(fun f))(x:E)
    (In x (parties (dom f)))
    ->
    (inc x (fun_img_inverse f Af (fun_img_directe f Af x)))
.
(* Proof of lem_fun_img_prop2                                              *)
(Intros;Unfold inc;Intros).
Unfold fun_img_inverse.
(Elim (axs_comprehension [x0:E](In (fun_img_directe f Af x) (parties (Img f)))->(In (App f Af x0) (fun_img_directe f Af x)) (dom f) v0);Intros;Apply H2;Clear H1 H2;Intros).
(Split;Intros).
(Elim (axs_parties (dom f) x);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
Exact (H v0 H0).

Unfold fun_img_directe.
(Elim (axs_comprehension [x0:E](In x (parties (dom f)))->(Ex [y:E](and (In y x) (eq ? x0 (App f Af y)))) (Img f) (App f Af v0));Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Idtac|Intros;Clear H2]).
(Elim (axs_parties (dom f) x);Intros;Generalize (H2 H);Clear H2 H3;Intros).
(Generalize (H2 v0 H0);Clear H2;Intros).
(Generalize (lem_eval_prop2 f v0 Af H2);Intros).
Exact (lem_fun_and_img f v0 (App f Af v0) Af H3).

(Exists v0;Split;[Auto|Reflexivity]).

Save.
(* End of proof of lem_fun_img_prop2                                       *)

(***************************************************************************)
(* fun_img_directe est un evaluateur de fun_part                           *)
(***************************************************************************)
Lemma lem_fun_part_eval_prop :
  (f:E)(Af:(fun f))(x:E)
    (In x (dom (fun_part f Af)))
    ->
    (App (fun_part f Af) (lem_fun_part_is_fun f Af) x)=(fun_img_directe f Af x)
.
(* Proof of lem_fun_part_eval_prop                                         *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold App in H0.
(Elim (axs_comprehension [z:E](Ex [y:E](and (In (couple x y) (fun_part f Af)) (In z y))) (reunion (Img (fun_part f Af))) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
Unfold fun_part in H1.
(Elim (axs_comprehension [x1:E](Ex [y:E](eq ? x1 (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple x x0));Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros;Elim H3;Clear H3;Intros).
(Elim (lem_couple_propertie x x1 x0 (fun_img_directe f Af x1));Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Rewrite -> H3;Rewrite <- H4;Auto).

(Generalize H;Rewrite -> (lem_fun_part_dom f Af);Intros).
Unfold fun_img_directe in H0.
(Elim (axs_comprehension [x0:E](In x (parties (dom f)))->(Ex [y:E](and (In y x) (eq ? x0 (App f Af y)))) (Img f) v2);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear  H0;Intros;Generalize (H2 H1) ;Clear  H2;Intros).
(Elim H2;Clear  H2;Intros;Elim H2;Clear  H2;Intros).
Unfold App.
(Elim (axs_comprehension [z:E](Ex [y:E](and (In (couple x y) (fun_part f Af)) (In z y))) (reunion (Img (fun_part f Af))) v2);Intros;Apply H5;Clear H4 H5;Intros).
Split.
Rewrite -> (lem_fun_part_img f Af).
Rewrite -> (lem_reunion_parties (Img f)).
Auto.

(Generalize (lem_fun_part_is_fun f Af) ;Intros).
(Generalize (lem_App (fun_part f Af) x H4 H);Intros).
(Elim H5;Clear H5;Intros;Elim H5;Clear H5;Intros).
(Exists x1;Split;[Auto|Idtac]).
Unfold fun_part in H6.
(Elim (axs_comprehension [x0:E](Ex [y:E](eq ? x0 (couple y (fun_img_directe f Af y)))) (cartesien (parties (dom f)) (parties (Img f))) (couple x x1));Intros;Generalize (H7 H6);Clear H6 H7 H8;Intros).
(Elim H6;Clear H6;Intros;Elim H7;Clear H7;Intros).
(Elim (lem_couple_propertie x x2 x1 (fun_img_directe f Af x2));Intros;Generalize (H8 H7);Clear H7 H8 H9;Intros).
(Elim H7;Clear H7;Intros).
(Rewrite -> H8;Rewrite <- H7).
Unfold fun_img_directe.
(Elim (axs_comprehension [x0:E](In x (parties (dom f)))->(Ex [y:E](and (In y x) (eq ? x0 (App f Af y)))) (Img f) v2);Intros;Apply H10;Clear H9 H10;Intros).
(Split;[Auto|Intros;Exists x0;Split;Auto]).

Save.
(* End of proof of lem_fun_part_eval_prop                                  *)

(***************************************************************************)
(***************************************************************************)
(* Reciproque: application des parties de l'image de f dans les parties du *)
(* domaine de f                                                            *)
(***************************************************************************)
(***************************************************************************)
Definition fun_part_inv := [f:E][Af:(fun f)]
  (subset
    [x:E](Ex[y:E](x=(couple y (fun_img_inverse f Af y))))
    (cartesien (parties (Img f)) (parties (dom f)))
  )
.

(***************************************************************************)
(* fun_part_inv est une application                                        *)
(***************************************************************************)
Lemma lem_fun_part_inv_is_fun :
  (f:E)(Af:(fun f))
    (fun (fun_part_inv f Af))
.
(* Proof of lem_fun_part_inv_is_fun                                        *)
(Intros;Unfold fun ;Split;Intros).
(Exists (parties (Img f));Exists (parties (dom f));Unfold inc;Intros).
Unfold fun_part_inv in H.
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

(Unfold fun_part_inv in H;Unfold fun_part_inv in H0).
(Elim (axs_comprehension [x0:E](Ex [y0:E](eq ? x0 (couple y0 (fun_img_inverse f Af y0)))) (cartesien (parties (Img f)) (parties (dom f))) (couple x y));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim (axs_comprehension [x0:E](Ex [y:E](eq ? x0 (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple x z));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H1;Clear H1;Intros).
(Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x0 y (fun_img_inverse f Af x0));Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim (lem_couple_propertie x x1 z (fun_img_inverse f Af x1));Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Rewrite -> H3;Rewrite -> H4;Rewrite <- H1;Rewrite <- H2;Reflexivity).

Save.
(* End of proof of lem_fun_part_inv_is_fun                                 *)

(***************************************************************************)
(* Le domaine de fun_part_inv est l'ensemble des parties de f              *)
(***************************************************************************)
Lemma lem_fun_part_inv_dom :
  (f:E)(Af:(fun f))
    (dom (fun_part_inv f Af))=(parties (Img f))
.
(* Proof of lem_fun_part_inv_dom                                           *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold dom in H.
(Elim (axs_comprehension [v0:E](fun (fun_part_inv f Af))->(Ex [v1:E](In (couple v0 v1) (fun_part_inv f Af))) (reunion (reunion (fun_part_inv f Af))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Generalize (H0 (lem_fun_part_inv_is_fun f Af));Clear H0;Intros).
(Elim H0;Clear H0;Intros).
Unfold fun_part_inv in H0.
(Elim (axs_comprehension [x0:E](Ex [y:E](eq ? x0 (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_cartesian_propertie (parties (Img f)) (parties (dom f)) (couple v2 x));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie v2 x1 x x2);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros).
(Rewrite -> H3;Auto).

(Elim (axs_parties (Img f) v2);Intros;Generalize (H0 H);Clear H0 H1;Intros).
Unfold dom.
(Elim (axs_comprehension [v0:E](fun (fun_part_inv f Af))->(Ex [v1:E](In (couple v0 v1) (fun_part_inv f Af))) (reunion (reunion (fun_part_inv f Af))) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;Intros).
(Elim (axs_reunion (reunion (fun_part_inv f Af)) v2);Intros;Apply H2;Clear H1 H2;Intros).
Exists (paire v2 (fun_img_inverse f Af v2)).
(Split;[Idtac|Elim (axs_paire v2 (fun_img_inverse f Af v2) v2);Intros;Apply H2;Left;Reflexivity]).
(Elim (axs_reunion (fun_part_inv f Af) (paire v2 (fun_img_inverse f Af v2)));Intros;Apply H2;Clear H1 H2;Intros).
Exists (couple v2 (fun_img_inverse f Af v2)).
(Split;[Idtac|Unfold couple;Elim (axs_paire (singleton v2) (paire v2 (fun_img_inverse f Af v2)) (paire v2 (fun_img_inverse f Af v2)));Intros;Apply H2;Right;Reflexivity]).
Unfold fun_part_inv.
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple v2 (fun_img_inverse f Af v2)));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists v2;Auto]).
(Elim (lem_cartesian_propertie (parties (Img f)) (parties (dom f)) (couple v2 (fun_img_inverse f Af v2)));Intros;Apply H2;Clear H1 H2;Intros).
(Exists v2;Exists (fun_img_inverse f Af v2);Split;[Auto|Split;[Idtac|Reflexivity]]).
(Elim (axs_parties (dom f) (fun_img_inverse f Af v2));Intros;Apply H2;Clear H1 H2;Intros).
Unfold fun_img_inverse in H1.
(Elim (axs_comprehension [x:E](In v2 (parties (Img f)))->(In (App f Af x) v2) (dom f) v3);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Auto).

(Exists (fun_img_inverse f Af v2);Unfold fun_part_inv).
(Elim (axs_comprehension [x:E](Ex [y:E](eq ? x (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple v2 (fun_img_inverse f Af v2)));Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Idtac|Exists v2;Reflexivity]).
(Elim (lem_cartesian_propertie (parties (Img f)) (parties (dom f)) (couple v2 (fun_img_inverse f Af v2)));Intros;Apply H3;Clear H2 H3;Intros).
(Exists v2;Exists (fun_img_inverse f Af v2);Split;[Auto|Split;[Idtac|Reflexivity]]).
(Elim (axs_parties (dom f) (fun_img_inverse f Af v2));Intros;Apply H3;Clear H2 H3;Intros).
Unfold fun_img_inverse in H2.
(Elim (axs_comprehension [x:E](In v2 (parties (Img f)))->(In (App f Af x) v2) (dom f) v3);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros;Auto).

Save.
(* End of proof of lem_fun_part_inv_dom                                    *)

(***************************************************************************)
(* L'image de fun_img_inv est l'ensemble des parties du domaine de f       *)
(***************************************************************************)
Lemma lem_fun_part_inv_img :
  (f:E)(Af:(fun f))
    (inc (Img (fun_part_inv f Af)) (parties (dom f)))
.
(* Proof of lem_fun_part_inv_img                                           *)
(Intros;Unfold inc;Intros).
Unfold Img in H.
(Elim (axs_comprehension [v1:E](fun (fun_part_inv f Af))->(Ex [v2:E](In (couple v2 v1) (fun_part_inv f Af))) (reunion (reunion (fun_part_inv f Af))) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Generalize (H0 (lem_fun_part_inv_is_fun f Af));Clear H0;Intros).
(Elim H0;Clear H0;Intros).
Unfold fun_part_inv in H0.
(Elim (axs_comprehension [x0:E](Ex [y:E](eq ? x0 (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple x v0));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_cartesian_propertie (parties (Img f)) (parties (dom f)) (couple x v0));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x1 v0 x2);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros).
(Rewrite -> H4;Auto).

Save.
(* End of proof of lem_fun_part_inv_img                                    *)

(***************************************************************************)
(* fun_img_inverse est un evaluateur de fun_part_inv                       *)
(***************************************************************************)
Lemma lem_fun_part_inv_eval_prop :
  (f:E)(Af:(fun f))(x:E)
    (In x (dom (fun_part_inv f Af)))
    ->
    (App (fun_part_inv f Af) (lem_fun_part_inv_is_fun f Af) x)=(fun_img_inverse f Af x)
.
(* Proof of lem_fun_part_inv_eval_prop                                     *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold App in H0.
(Elim (axs_comprehension [z:E](Ex [y:E](and (In (couple x y) (fun_part_inv f Af)) (In z y))) (reunion (Img (fun_part_inv f Af))) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
Unfold fun_part_inv in H1.
(Elim (axs_comprehension [x1:E](Ex [y:E](eq ? x1 (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple x x0));Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros;Elim H3;Clear H3;Intros).
(Elim (lem_couple_propertie x x1 x0 (fun_img_inverse f Af x1));Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros).
(Rewrite -> H3;Rewrite <- H4;Auto).

Unfold fun_img_inverse in H0.
(Elim (axs_comprehension [x0:E](In x (parties (Img f)))->(In (App f Af x0) x) (dom f) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
(Generalize H;Rewrite -> (lem_fun_part_inv_dom f Af);Intros).
(Generalize (H1 H2);Clear H1 H2;Intros).
Unfold App.
(Elim (axs_comprehension [z:E](Ex [y:E](and (In (couple x y) (fun_part_inv f Af)) (In z y))) (reunion (Img (fun_part_inv f Af))) v2);Intros;Apply H3;Clear H2 H3;Intros).
(Generalize (lem_fun_part_inv_is_fun f Af);Intros).
(Elim (lem_App (fun_part_inv f Af) x H2 H);Intros).
(Elim H3;Clear H3;Intros).
Split.
(Elim (axs_reunion (Img (fun_part_inv f Af)) v2);Intros;Apply H6;Clear H5 H6;Intros).
Exists x0.
(Split;[Auto|Idtac]).
Unfold fun_part_inv in H4.
(Elim (axs_comprehension [x1:E](Ex [y:E](eq ? x1 (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple x x0));Intros;Generalize (H5 H4);Clear H4 H5 H6;Intros).
(Elim H4;Clear H4;Intros;Elim H5;Clear H5;Intros).
(Elim (lem_couple_propertie x x1 x0 (fun_img_inverse f Af x1));Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Rewrite -> H6;Rewrite <- H5;Clear H5 H6).
Unfold fun_img_inverse.
(Elim (axs_comprehension [x0:E](In x (parties (Img f)))->(In (App f Af x0) x) (dom f) v2);Intros;Apply H6;Clear H5 H6;Intros).
(Split;Intros;Auto).

(Exists x0;Split;[Auto|Idtac]).
Unfold fun_part_inv in H4.
(Elim (axs_comprehension [x1:E](Ex [y:E](eq ? x1 (couple y (fun_img_inverse f Af y)))) (cartesien (parties (Img f)) (parties (dom f))) (couple x x0));Intros;Generalize (H5 H4);Clear H4 H5 H6;Intros).
(Elim H4;Clear H4;Intros;Elim H5;Clear H5;Intros).
(Elim (lem_couple_propertie x x1 x0 (fun_img_inverse f Af x1));Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Rewrite -> H6;Rewrite <- H5;Clear H5 H6).
Unfold fun_img_inverse.
(Elim (axs_comprehension [x0:E](In x (parties (Img f)))->(In (App f Af x0) x) (dom f) v2);Intros;Apply H6;Clear H5 H6;Intros).
(Split;Intros;Auto).

Save.
(* End of proof of lem_fun_part_inv_eval_prop                              *)

(***************************************************************************)
(***************************************************************************)
(* Relation inverse d'une application et applications au cas des bijections*)
(***************************************************************************)
(***************************************************************************)
(***************************************************************************)
(* On peut definir la relation inverse d'une application                   *)
(***************************************************************************)
Definition fun_recip := [f:E][Af:(fun f)]
  (subset
    [c:E](Ex[x:E](Ex[y:E]((In (couple x y) f) /\ (c=(couple y x)))))
    (cartesien (Img f) (dom f))
  )
.

(***************************************************************************)
(* fun_recip est une application ssi f est une injection                   *)
(***************************************************************************)
Lemma lem_fun_recip_is_fun_under_inj :
  (f:E)(Af:(fun f))
    (fun_inj f (dom f) (Img f))
    <->
    (fun (fun_recip f Af))
.
(* Proof of lem_fun_recip_is_fun_under_inj                                 *)
Intros;Unfold iff;Split;Intros.
Unfold fun_inj in H.
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Clear H0;Elim H1;Clear H1;Intros;Clear H0).
(Unfold fun;Split;Intros).
Exists (Img f);Exists (dom f);Unfold inc;Intros.
Unfold fun_recip in H0.
(Elim (axs_comprehension [c:E](Ex[x:E](Ex [y:E](and (In (couple x y) f) (eq ? c (couple y x))))) (cartesien (Img f) (dom f)) v0);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Auto).

(Unfold fun_recip in H0;Unfold fun_recip in H2).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex[y0:E](and (In (couple x0 y0) f) (eq ? c (couple y0 x0))))) (cartesien (Img f) (dom f)) (couple x y));Intros;Generalize (H3 H0);Clear H0 H3 H4;Intros).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) f) (eq ? c (couple y x0))))) (cartesien (Img f) (dom f)) (couple x z));Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H0;Clear H0;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Elim H2;Clear H2;Intros;Elim H5;Clear H5;Intros;Elim H5;Clear H5;Intros;Elim H5;Clear H5;Intros).
(Elim (lem_couple_propertie x x1 y x0);Intros;Generalize (H7 H4);Clear H4 H7 H8;Intros;Elim H4;Clear H4;Intros).
(Generalize H3;Rewrite <- H4;Rewrite <- H7;Clear H3 H4 H7;Intros).
(Elim (lem_couple_propertie x x3 z x2);Intros;Generalize (H4 H6);Clear H4 H6 H7;Intros;Elim H4;Clear H4;Intros).
(Generalize H5;Rewrite <- H4;Rewrite <- H6;Clear H4 H5 H6;Intros).
(Generalize (lem_fun_and_img f y x Af H3);Intros).
(Generalize (H1 x H4);Unfold exUniq;Intros).
(Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros;Generalize (H7 y H3) (H7 z H5);Intros).
(Rewrite <- H8;Rewrite <- H9;Reflexivity).

(Elim H;Clear H;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros).
Unfold fun_inj.
(Split;[Auto|Split;[Reflexivity|Split;[Unfold inc;Intros;Auto|Unfold exUniq;Intros]]]).
(Generalize (lem_App_rev f b Af H1);Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Exists x1;Split;[Auto|Intros]).
(Generalize (lem_fun_and_dom f y b Af H4);Intros).
(Cut (In (couple b x1) (fun_recip f Af));Intros).
(Cut (In (couple b y) (fun_recip f Af));Intros).
Exact (H0 b x1 y H6 H7).

Unfold fun_recip.
(Elim (axs_comprehension [c:E](Ex[x:E](Ex [y0:E](and (In (couple x y0) f) (eq ? c (couple y0 x))))) (cartesien (Img f) (dom f)) (couple b y));Intros;Apply H8;Clear H7 H8;Intros).
(Split;[Idtac|Exists y;Exists b;Split;[Auto|Reflexivity]]).
(Elim (lem_cartesian_propertie (Img f) (dom f) (couple b y));Intros;Apply H8;Clear H7 H8;Intros).
(Exists b;Exists y;Split;[Auto|Split;[Auto|Reflexivity]]).

Unfold fun_recip.
(Elim (axs_comprehension [c:E](Ex[x:E](Ex [y:E](and (In (couple x y) f) (eq ? c (couple y x))))) (cartesien (Img f) (dom f)) (couple b x1));Intros;Apply H7;Clear H6 H7;Intros).
(Split;[Idtac|Exists x1;Exists b;Split;[Auto|Reflexivity]]).
(Elim (lem_cartesian_propertie (Img f) (dom f) (couple b x1));Intros;Apply H7;Clear H6 H7;Intros).
(Exists b;Exists x1;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_fun_recip_is_fun_under_inj                          *)

(***************************************************************************)
(* Si f est une application et que (x,y) est dans f alors (y,x) est dans   *)
(* (fun_recip f)                                                           *)
(***************************************************************************)
Lemma lem_xyinf_yxinrecipf :
  (f:E)(Af:(fun f))(x,y:E)
    (In (couple x y) f) -> (In (couple y x) (fun_recip f Af))
.
(* Proof of lem_xyinf_yxinrecipf                                           *)
(Intros;Unfold fun_recip).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex[y0:E](and (In (couple x0 y0) f) (eq ? c (couple y0 x0))))) (cartesien (Img f) (dom f)) (couple y x));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists x;Exists y;Split;[Auto|Reflexivity]]).
(Generalize (lem_fun_and_dom f x y Af H);Intros).
(Generalize (lem_fun_and_img f x y Af H);Intros).
(Elim (lem_cartesian_propertie (Img f) (dom f) (couple y x));Intros;Apply H3;Clear H2 H3;Intros).
(Exists y;Exists x;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_xyinf_yxinrecipf                                    *)

(***************************************************************************)
(* Si f est une bijection alors fun_recip est une bijection                *)
(***************************************************************************)
Lemma lem_fun_recip_is_bij_under_bij :
  (f:E)(Af:(fun f))
    (fun_bij f (dom f) (Img f))
    ->
    (fun_bij (fun_recip f Af) (Img f) (dom f))
.
(* Proof of lem_fun_recip_is_bij_under_bij                                 *)
Intros.
(Generalize (lem_bij_is_inj f (dom f) (Img f) H);Intros).
(Elim (lem_fun_recip_is_fun_under_inj f Af);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
Unfold fun_bij in H.
(Elim H;Clear H;Intros;Clear H;Elim H1;Clear H1;Intros;Clear H;Elim H1;Clear H1;Intros;Clear H).
Unfold fun_bij .
(Split;[Auto|Split]).
(Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold dom in H.
(Elim (axs_comprehension [v0:E](fun (fun_recip f Af))->(Ex [v1:E](In (couple v0 v1) (fun_recip f Af))) (reunion (reunion (fun_recip f Af))) v2);Intros;Generalize (H2 H);Clear H H2 H3;Intros).
(Elim H;Clear H;Intros;Generalize (H2 H0);Clear H2;Intros;Elim H2;Clear H2;Intros).
Unfold fun_recip in H2.
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) f) (eq ? c (couple y x0))))) (cartesien (Img f) (dom f)) (couple v2 x));Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros).
(Elim (lem_cartesian_propertie (Img f) (dom f) (couple v2 x));Intros;Generalize (H4 H2);Clear H2 H4 H5;Intros).
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H4;Clear  H4;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Rewrite -> H5;Auto).

(Generalize (H1 v2 H);Intros).
Unfold exUniq in H2.
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Generalize (lem_xyinf_yxinrecipf f Af x v2 H2);Intros).
Exact (lem_fun_and_dom (fun_recip f Af) v2 x H0 H4).

Split.
(Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold Img in H.
(Elim (axs_comprehension [v1:E](fun (fun_recip f Af))->(Ex [v0:E](In (couple v0 v1) (fun_recip f Af))) (reunion (reunion (fun_recip f Af))) v2);Intros;Generalize (H2 H);Clear H H2 H3;Intros).
(Elim H;Clear H;Intros;Generalize (H2 H0);Clear H2;Intros;Elim H2;Clear H2;Intros).
Unfold fun_recip in H2.
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) f) (eq ? c (couple y x0))))) (cartesien (Img f) (dom f)) (couple x v2));Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros).
(Elim (lem_cartesian_propertie (Img f) (dom f) (couple x v2));Intros;Generalize (H4 H2);Clear H2 H4 H5;Intros).
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_couple_propertie x x0 v2 x1);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros;Rewrite -> H6;Auto).

(Generalize (lem_App f v2 Af H);Intros).
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Generalize (lem_xyinf_yxinrecipf f Af v2 x H3);Intros).
Exact (lem_fun_and_img (fun_recip f Af) x v2 H0 H4).

Intros.
(Unfold exUniq;Intros).
Unfold Img in H.
(Elim (axs_comprehension [v1:E](fun (fun_recip f Af))->(Ex [v0:E](In (couple v0 v1) (fun_recip f Af))) (reunion (reunion (fun_recip f Af))) b);Intros;Generalize (H2 H);Clear H H2 H3;Intros).
(Elim H;Clear H;Intros;Generalize (H2 H0);Clear H2;Intros;Elim H2;Clear H2;Intros).
(Exists x;Split;[Auto|Intros]).
(Unfold fun_recip in H2;Unfold fun_recip in H3).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) f) (eq ? c (couple y x0))))) (cartesien (Img f) (dom f)) (couple x b));Intros;Generalize (H4 H2);Clear H2 H4 H5;Intros).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex [y0:E](and (In (couple x y0) f) (eq ? c (couple y0 x))))) (cartesien (Img f) (dom f)) (couple y b));Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H2;Clear H2;Intros;Elim H4;Clear H4;Intros;Elim H4;Clear H4;Intros;Elim H4;Clear H4;Intros).
(Elim H3;Clear H3;Intros;Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros).
(Elim (lem_couple_propertie x x1 b x0);Intros;Generalize (H8 H5);Clear H5 H8 H9;Intros;Elim H5;Clear H5;Intros).
(Elim (lem_couple_propertie y x3 b x2);Intros;Generalize (H9 H7);Clear H7 H9 H10;Intros;Elim H7;Clear H7;Intros).
(Generalize H4 H6;Clear H4 H6;Rewrite <- H8;Rewrite <- H9;Rewrite <- H5;Rewrite <- H7;Clear H5 H7 H8 H9;Intros).
(Generalize Af;Intros).
(Elim Af0;Clear Af0;Intros).
Exact (H7 b x y H4 H6).

Save.
(* End of proof of lem_fun_recip_is_bij_under_bij                          *)

(***************************************************************************)
(* Si f est une injection de (dom f) dans (Img f) alors c'est une          *)
(* bijection de (dom f) dans (Img f)                                       *)
(***************************************************************************)
Lemma lem_fun_inj_dom_img_is_bij :
  (f:E)(Af:(fun f))
    (fun_inj f (dom f) (Img f))
    ->
    (fun_bij f (dom f) (Img f))
.
(* Proof of lem_fun_inj_dom_img_is_bij                                     *)
(Unfold fun_inj fun_bij;Intros).
(Elim H;Clear H Af;Intros;Elim H0;Clear H0;Intros;Clear H0;Elim H1;Clear H1;Intros;Clear H0).
(Split;[Auto|Split;[Reflexivity|Split;[Reflexivity|Auto]]]).

Save.
(* End of proof of lem_fun_inj_dom_img_is_bij                              *)

(***************************************************************************)
(* Si f est une bijection alors fun_recip de fun_recip de f est egale a f  *)
(***************************************************************************)
Lemma lem_fun_recip_involutive_under_bij :
  (f:E)(Af:(fun f))(Abijf:(fun_bij f (dom f) (Img f)))(Arec:(fun (fun_recip f Af)))
    ((fun_recip (fun_recip f Af) Arec)=f)
.
(* Proof of lem_fun_recip_involutive_under_bij                             *)
(Intros;Apply axs_extensionnalite;Unfold iff ;Split;Intros).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](and (In (couple x y) (fun_recip f Af)) (eq ? c (couple y x))))) (cartesien (Img (fun_recip f Af)) (dom (fun_recip f Af))) v2);Intros).
(Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold fun_recip in H0.
(Elim (axs_comprehension [c:E](Ex[x1:E](Ex [y:E](and (In (couple x1 y) f) (eq ? c (couple y x1))))) (cartesien (Img f) (dom f)) (couple x x0));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x2 x0 x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros).
(Generalize H2;Clear H2;Rewrite <- H3;Rewrite <- H4;Rewrite <- H1;Auto).

(Generalize Arec;Clear Arec;Generalize Af;Intros).
(Elim Af;Clear Af;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold inc in H0.
(Generalize (H0 v2 H);Clear H0;Intros).
(Elim (lem_cartesian_propertie x x0 v2);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Generalize H ;Rewrite -> H3;Intros).
(Generalize (lem_xyinf_yxinrecipf f Af0 x1 x2 H4);Intros).
(Exact (lem_xyinf_yxinrecipf (fun_recip f Af0) Arec x2 x1 H5);Intros).

Save.
(* End of proof of lem_fun_recip_involutive_under_bij                      *)

(***************************************************************************)
(***************************************************************************)
(* Exponantiation de a par b                                               *)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(* exp a b est l'ensemble des applications de b dans a                     *)
(***************************************************************************)
Definition exp := [a,b:E]
  (subset
    [f:E]((x:E)(In x b)->(ExU[y:E]((In (couple x y) f))))
    (parties (cartesien b a))
  )
.

(***************************************************************************)
(* Si f est un element de (exp a b) alors f est une application de b dans a*)
(***************************************************************************)
Lemma lem_exp_elem_is_fun :
  (a,b,f:E)
    (In f (exp a b))
    ->
    (fun f)
.
(* Proof of lem_exp_is_fun                                                 *)
(Intros;Unfold fun;Split;Intros).
Unfold exp in H.
(Elim (axs_comprehension [f0:E](x:E)(In x b)->(exUniq E [y:E](In (couple x y) f0)) (parties (cartesien b a)) f);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Exists b;Exists a;Unfold inc;Intros).
(Elim (axs_parties (cartesien b a) f);Intros;Generalize (H2 H);Clear H H2 H3;Intros).
Exact (H v0 H1).

Unfold exp in H.
(Elim (axs_comprehension [f0:E](x:E)(In x b)->(exUniq E [y:E](In (couple x y) f0)) (parties (cartesien b a)) f);Intros;Generalize (H2 H);Clear H H2 H3;Intros).
(Elim H;Clear H;Intros).
(Elim (axs_parties (cartesien b a) f);Intros;Generalize (H3 H);Clear H H3 H4;Intros).
(Generalize (H (couple x y) H0);Intros).
(Elim (lem_cartesian_propertie b a (couple x y));Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Generalize H3;Clear H3;Rewrite <- H5;Intros).
(Generalize (H2 x H3);Clear H2 H3;Intros).
Unfold exUniq in H2.
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Generalize (H3 y H0);Intros).
(Generalize (H3 z H1);Intros).
(Rewrite <- H7;Rewrite <- H8;Reflexivity).

Save.
(* End of proof of lem_exp_is_fun                                          *)

(***************************************************************************)
(* Le domaine d'un element de (exp a b) est b                              *)
(***************************************************************************)
Lemma lem_exp_elem_dom :
  (a,b,f:E)
    (In f (exp a b))
    ->
    (dom f)=b
.
(* Proof of lem_exp_elem_dom                                               *)
Intros.
(Generalize (lem_exp_elem_is_fun a b f H);Intros;Unfold exp in H).
(Elim (axs_comprehension [f0:E](x:E)(In x b)->(exUniq E [y:E](In (couple x y) f0)) (parties (cartesien b a)) f);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros).
(Generalize H;Intros).
(Elim (axs_parties (cartesien b a) f);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Generalize (lem_App f v2 H0 H3);Intros).
(Elim H4;Clear H4;Intros;Elim H4;Clear H4;Intros).
(Generalize (H2 (couple v2 x) H5);Clear H2 H5;Intros).
(Elim (lem_cartesian_propertie b a (couple v2 x));Intros;Generalize (H5 H2);Clear H2 H5 H6;Intros).
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H5;Clear H5;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H7 H6);Clear H6 H7 H8;Intros).
(Elim H6;Clear H6;Intros).
(Rewrite -> H6;Auto).

(Generalize (H1 v2 H3);Clear H1 H3;Intros).
Unfold exUniq in H1.
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
Exact (lem_fun_and_dom f v2 x H0 H1).

Save.
(* End of proof of lem_exp_elem_dom                                        *)

(***************************************************************************)
(* L'image d'un element de (exp a b) est incluse dans a                    *)
(***************************************************************************)
Lemma lem_exp_elem_img :
  (a,b,f:E)
    (In f (exp a b))
    ->
    (inc (Img f) a)
.
(* Proof of lem_exp_elem_img                                               *)
(Intros;Unfold inc;Intros).
(Generalize (lem_exp_elem_is_fun a b f H);Intros).
Unfold exp in H.
(Elim (axs_comprehension [f0:E](x:E)(In x b)->(exUniq E [y:E](In (couple x y) f0)) (parties (cartesien b a)) f);Intros;Generalize (H2 H);Clear H H2 H3;Intros).
(Elim H;Clear H;Intros).
(Elim (axs_parties (cartesien b a) f);Intros;Generalize (H3 H);Clear H H3 H4;Intros).
(Generalize (lem_App_rev f v0 H1 H0);Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Generalize (H (couple x v0) H4);Clear H H4;Intros).
(Elim (lem_cartesian_propertie b a (couple x v0));Intros;Generalize (H4 H) ;Clear H H4 H5;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H4;Clear  H4;Intros).
(Elim (lem_couple_propertie x x0 v0 x1);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Rewrite -> H6;Auto).

Save.
(* End of proof of lem_exp_elem_img                                        *)

(***************************************************************************)
(* La conclusion: les elements de (exp a b) sont les applications de b     *)
(* dans a                                                                  *)
(***************************************************************************)
Lemma lem_exp_elem_prop :
  (a,b,f:E)
    (In f (exp a b))
    ->
    (fun_sig f b a)
.
(* Proof of lem_exp_elem_prop                                              *)
Intros.
(Generalize (lem_exp_elem_is_fun a b f H);Intros).
(Generalize (lem_exp_elem_dom a b f H);Intros).
(Generalize (lem_exp_elem_img a b f H);Intros).
Unfold fun_sig.
(Split;[Auto|Split;Auto]).

Save.
(* End of proof of lem_exp_elem_prop                                       *)

(***************************************************************************)
(* Un lemme a la xxx: si f est une application et que c est un element de  *)
(* f alors il existe x et y tel que c=(x,y)                                *)
(***************************************************************************)
Lemma lem_fun_elem_is_2tupple :
  (f:E)(Af:(fun f))(c:E)
    (In c f)->(Ex[x:E](Ex[y:E]((In (couple x y) f) /\ c=(couple x y))))
.
(* Proof of lem_fun_elem_is_2tupple                                        *)
Intros.
(Elim Af;Clear Af;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold inc in H0.
(Generalize (H0 c H);Clear H0;Intros).
(Elim (lem_cartesian_propertie x x0 c);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Elim H2;Clear H2;Intros).
(Exists x1;Exists x2;Rewrite <- H3;Split;[Auto|Reflexivity]).

Save.
(* End of proof of lem_fun_elem_is_2tupple                                 *)

(***************************************************************************)
(* La reciproque : Si f est une application de b dans a alors f est un     *)
(* element de (exp a b).                                                   *)
(***************************************************************************)
Lemma lem_exp_elem_prop_recip :
  (a,b,f:E)
  (fun_sig f b a)
  ->
  (In f (exp a b))
.
(* Proof of lem_exp_elem_prop_recip                                        *)
(Unfold fun_sig;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
Unfold exp.
(Elim (axs_comprehension [f0:E](x:E)(In x b)->(exUniq E [y:E](In (couple x y) f0)) (parties (cartesien b a)) f);Intros;Apply H3;Clear H2 H3;Intros).
(Split;Intros).
(Elim (axs_parties (cartesien b a) f);Intros;Apply H3;Clear H2 H3;Intros).
(Generalize (lem_fun_elem_is_2tupple f H v3 H2);Clear H2;Intros).
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Generalize (lem_fun_and_dom f x x0 H H2);Intros).
(Generalize (lem_fun_and_img f x x0 H H2);Intros).
(Elim (lem_cartesian_propertie b a v3);Intros;Apply H7;Clear H6 H7;Intros).
(Exists x;Exists x0;Split).
(Rewrite <- H0;Auto).

(Unfold inc in H1;Generalize (H1 x0 H5);Clear H1 H5;Intros).
(Split;Auto).

(Unfold exUniq;Intros).
(Generalize H2;Clear H2;Rewrite <- H0;Intros).
(Generalize (lem_App f x H H2);Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Exists x0;Split;[Auto|Intros]).
(Elim H;Intros).
Exact (H7 x x0 y H4 H5).

Save.
(* End of proof of lem_exp_elem_prop_recip                                 *)

(***************************************************************************)
(***************************************************************************)
(* Famille d'ensembles indexee                                             *)
(***************************************************************************)
(***************************************************************************)
Definition index := [a:E][I:E][funa:(fun a)][doma:((dom a)=I)]
  (subset [x:E]True (Img a))
.

Lemma lem_index_prop :
  (a:E)(I:E)(funa:(fun a))(doma:((dom a)=I))
    (index a I funa doma)=(Img a)
.
(* Proof of lem_index_prop                                                 *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold index in H.
(Elim (axs_comprehension [_:E]True (Img a) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

Unfold index.
(Elim (axs_comprehension [_:E]True (Img a) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Split;Auto).

Save.
(* End of proof of lem_index_prop                                          *)

Lemma lem_reunion_index_prop :
  (a:E)(I:E)(funa:(fun a))(doma:((dom a)=I))(x:E)
    (In x (reunion (index a I funa doma)))
    ->
    (Ex[i:E]((In i I) /\ (In x (App a funa i))))
.
(* Proof of lem_reunion_index_prop                                         *)
Intros.
(Generalize (lem_index_prop a I funa doma);Intros).
(Generalize H;Rewrite -> H0;Clear H H0;Intros).
(Elim (axs_reunion (Img a) x);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros).
(Generalize (lem_App_rev a x0 funa H);Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Generalize H1;Clear  H1;Rewrite -> doma;Intros).
(Generalize (lem_eval_propertie a x1 x0 funa H2);Intros).
(Generalize H0;Clear H0;Rewrite -> H3;Clear H3;Intros).
(Exists x1;Split;Auto).

Save.
(* End of proof of lem_reunion_index_prop                                  *)

Lemma lem_index_inter :
  (a:E)(I:E)(funa:(fun a))(doma:((dom a)=I))(Ine:(~(I=Vide)))(x:E)
  (indNe:~((index a I funa doma)=Vide))
    (In x (Einter (index a I funa doma) indNe))
    ->
    ((i:E)(In i I) -> (In x (App a funa i)))
.
(* Proof of lem_index_inter                                                *)
Intros.
Unfold App.
(Elim (axs_comprehension [z:E](Ex [y:E](and (In (couple i y) a) (In z y))) (reunion (Img a)) x);Intros;Apply H2;Clear H1 H2;Intros).
Unfold Einter in H.
(Elim (axs_comprehension [v0:E](v3:E)(In v3 (index a I funa doma))->(In v0 v3) (reunion (index a I funa doma)) x);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros).
(Generalize H;Clear H;Rewrite -> (lem_index_prop a I funa doma);Intros).
(Split;[Auto|Exists (App a funa i);Split]).
(Generalize H0;Rewrite <- doma;Intros).
(Generalize (lem_App a i funa H2);Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Generalize (lem_eval_propertie a i x0 funa H4);Intros).
(Rewrite <- H5;Auto).

(Apply (H1 (App a funa i));Clear H1;Intros).
Unfold index.
(Elim (axs_comprehension [_:E]True (Img a) (App a funa i));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Auto]).
(Generalize H0;Rewrite <- doma;Intros).
(Generalize (lem_eval_prop2 a i funa H1);Intros).
Exact (lem_fun_and_img a i (App a funa i) funa H2).

Save.
(* End of proof of lem_index_inter                                         *)

(***************************************************************************)
(***************************************************************************)
(* Produit d'une famille d'ensembles                                       *)
(***************************************************************************)
(***************************************************************************)
Definition produit := 
  [a:E][I:E][funa:(fun a)][doma:((dom a)=I)]
    (subset
      [f:E]((i:E)(In i I)->(ExU[y:E]((In (couple i y) f) /\ (In y (App a funa i)))))
      (exp (reunion (index a I funa doma)) I)
    )
.

Lemma lem_f_in_prod_is_fun :
  (a:E)(I:E)(funa:(fun a))(doma:((dom a)=I))(f:E)
    (In f (produit a I funa doma))
    ->
    (fun f)
.
(* Proof of lem_f_in_prod_is_fun                                           *)
(Unfold produit;Intros).
(Elim (axs_comprehension [f0:E](i:E)(In i I)->(exUniq E [y:E](and (In (couple i y) f0) (In y (App a funa i)))) (exp (reunion (index a I funa doma)) I) f);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
Exact (lem_exp_elem_is_fun (reunion (index a I funa doma)) I f H).

Save.
(* End of proof of lem_f_in_prod_is_fun                                    *)

Lemma lem_dom_of_f_in_prod_is_I :
  (a:E)(I:E)(funa:(fun a))(doma:((dom a)=I))(f:E)
    (In f (produit a I funa doma))
    ->
    (dom f)=I
.
(* Proof of lem_dom_of_f_in_prod_is_I                                      *)
Intros.
(Generalize (lem_f_in_prod_is_fun a I funa doma f H);Intros).
Unfold produit in H.
(Elim (axs_comprehension [f0:E](i:E)(In i I)->(exUniq E [y:E](and (In (couple i y) f0) (In y (App a funa i)))) (exp (reunion (index a I funa doma)) I) f);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros).
Exact (lem_exp_elem_dom (reunion (index a I funa doma)) I f H).

Save.
(* End of proof of lem_dom_of_f_in_prod_is_I                               *)

(***************************************************************************)
(*                         Next : axs_choice.v                             *)
(***************************************************************************)
