(***************************************************************************)
(* File: axs_comprehension.v                                               *)
(* auteur: alex@sysal.ibp.fr                                               *)
(* date: 19/10/95                                                          *)
(***************************************************************************)
Require Export axs_parties.

(***************************************************************************)
(* Schema d'axiome de comprehension                                        *)
(***************************************************************************)
Variable subset : (E->Prop)->E->E.
Axiom axs_comprehension :
  (F:E->Prop)(a:E)(v0:E)
    ((In v0 (subset F a))
    <->
    ((In v0 a) /\ (F v0)))
.

(***************************************************************************)
(* Pour les tactiques, on splite l'axiome precedent en deux lemmes orientes*)
(***************************************************************************)
Lemma lem_comprehension_lr :
  (F:E->Prop)(a:E)(v0:E)
    (In v0 (subset F a))
    ->
    ((In v0 a) /\ (F v0))
.
(* Proof of lem_comprehension_lr                                           *)
(Intros;Elim (axs_comprehension F a v0);Intros;Exact (H0 H)).

Save.
(* End of proof of lem_comprehension_lr                                    *)

Lemma lem_comprehension_rl :
  (F:E->Prop)(a:E)(v0:E)
    ((In v0 a) /\ (F v0))
    ->
    (In v0 (subset F a))
.
(* Proof of lem_comprehension_rl                                           *)
(Intros;Elim (axs_comprehension F a v0);Intros;Exact (H1 H)).

Save.
(* End of proof of lem_comprehension_rl                                    *)

(***************************************************************************)
(* Consequences                                                            *)
(***************************************************************************)

(***************************************************************************)
(* Existence de l'ensemble vide                                            *)
(***************************************************************************)
Definition def_F_vide := [v0:E]
  ~(v0 = v0)
.

Definition vide := [a:E]
  (subset def_F_vide a)
.

Lemma lem_vide_propertie :
  (v0:E)(v1:E)~(In v0 (vide v1))
.
(* Proof of lem_vide_propertie                                             *)
(Intros;Unfold vide ;Unfold def_F_vide ;Unfold not ;Intros).
(Elim (axs_comprehension def_F_vide v1 v0);Unfold def_F_vide ;Unfold not ;Intros).
(Elim H0;Clear  H0;Intros).
(Clear  H;Clear  H1;Clear  H0;Apply H2;Clear  H2;Reflexivity).

(Apply H1;Clear  H1).
Split.
(Elim (axs_comprehension def_F_vide v1 v0);Unfold def_F_vide ;Unfold not ;Intros).
(Elim H0;Clear  H0;Intros;Assumption).

(Elim (axs_comprehension def_F_vide v1 v0);Unfold def_F_vide ;Unfold not ;Intros).
(Elim H0;Clear  H0;Intros).
(Apply H3;Reflexivity).

Assumption.

Save.
(* End of proof of lem_vide_existence                                      *)

Lemma lem_vide_unicite :
  (v0,v1:E)
    (vide v0) = (vide v1)
.
(* Proof of lem_vide_unicite                                               *)
(Intros;Apply axs_extensionnalite;Unfold iff ;Split;Intros).
(Generalize (lem_vide_propertie v2 v0) ;Intros;Elim H0;Assumption).

(Generalize (lem_vide_propertie v2 v1) ;Intros;Elim H0;Assumption).

Save.
(* End of proof of lem_vide_unicite                                        *)

Variable var_vide : E.
Definition Vide := (vide var_vide).

Lemma lem_x_inc_vide :
  (x:E)(inc x Vide)->x=Vide
.
(* Proof of lem_x_inc_vide                                                 *)
Intros.
Unfold inc in H.
(Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Exact (H v2 H0).

(Generalize (lem_vide_propertie v2 var_vide);Intros).
Unfold Vide in H0.
(Absurd (In v2 (vide var_vide));Auto).

Save.
(* End of proof of lem_x_inc_vide                                          *)

Lemma lem_x_noteq_vide :
  (x,y:E)(In y x)->(~(x=Vide))
.
(* Proof of lem_x_noteq_vide                                               *)
Intros.
(Unfold not;Intros).
(Generalize H;Clear H;Rewrite -> H0;Intros).
(Generalize (lem_vide_propertie y var_vide);Intros;Unfold Vide in H).
(Absurd (In y (vide var_vide));Auto).

Save.
(* End of proof of lem_x_noteq_vide                                        *)

Lemma lem_paire_not_empty :
  (a,b:E)~((paire a b)=Vide)
.
(* Proof of lem_paire_not_empty                                            *)
Intros.
(Cut (In a (paire a b));Intros).
(Unfold not ;Intros).
(Generalize H;Clear H;Rewrite -> H0;Intros).
(Generalize (lem_vide_propertie a var_vide);Intros;Unfold Vide in H).
(Absurd (In a (vide var_vide));Auto).

(Elim (axs_paire a b a);Intros;Apply H0;Left;Reflexivity).

Save.
(* End of proof of lem_paire_not_empty                                     *)

Lemma lem_not_vide_in_vide :
  ~(In Vide Vide)
.
(* Proof of lem_not_vide_in_vide                                           *)
(Unfold not ;Intros).
(Absurd (eq ? Vide Vide);[Unfold not ;Intros|Reflexivity]).
(Generalize (lem_x_noteq_vide Vide Vide H);Intros).
Tauto.

Save.
(* End of proof of lem_not_vide_in_vide                                    *)

(***************************************************************************)
(* Intersection de deux ensembles                                          *)
(***************************************************************************)
Definition def_F_inter := [s:E][x:E]
  (In x s)
.

Definition inter := [a:E][b:E]
  (subset (def_F_inter b) a)
.

Lemma lem_inter_propertie :
  (a,b:E)(x:E)
    ((In x (inter a b))
    <->
    ((In x a) /\ (In x b)))
.
(* Proof of lem_inter_propertie                                            *)
(Intros;Unfold iff ;Unfold inter ;Unfold def_F_inter ;Split;Intros).
(Elim (axs_comprehension (def_F_inter b) a x);Unfold def_F_inter ;Intros;Auto).

(Elim (axs_comprehension (def_F_inter b) a x);Unfold def_F_inter ;Intros;Auto).

Save.
(* End of proof of lem_inter_propertie                                     *)

Lemma lem_inter_propertie_lr :
  (a,b:E)(x:E)
    ((In x (inter a b))
    ->
    ((In x a) /\ (In x b)))
.
(* Proof of lem_inter_propertie_lr                                         *)
(Intros;Elim (lem_inter_propertie a b x);Intros;Exact (H0 H)).

Save.
(* End of proof of lem_inter_propertie_lr                                  *)

Lemma lem_inter_propertie_rl :
  (a,b:E)(x:E)
    ((In x a) /\ (In x b))
    ->
    (In x (inter a b))
.
(* Proof of lem_inter_propertie_rl                                         *)
(Intros;Elim (lem_inter_propertie a b x);Intros;Exact (H1 H)).

Save.
(* End of proof of lem_inter_propertie_rl                                  *)

Lemma lem_inter_unicite :
  (a,b:E)(i:E)
    ((x:E)((In x i) <-> ((In x a) /\ (In x b))))
    ->
    (i = (inter a b))
.
(* Proof of lem_inter_unicite                                              *)
(Intros;Apply axs_extensionnalite;Intros).
Elim (H v2);Clear  H;Intros.
(Elim (lem_inter_propertie a b v2);Intros;Split;Intros;Auto).

Save.
(* End of proof of lem_inter_unicite                                       *)

Lemma lem_inter_vide_prop :
  (x:E)(inter x Vide)=Vide
.
(* Proof of lem_inter_vide_prop                                            *)
(Intros;Unfold inter;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
(Elim (axs_comprehension (def_F_inter Vide) x v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Unfold def_F_inter;Intros;Auto).

(Elim (axs_comprehension (def_F_inter Vide) x v2);Intros;Apply H1;Clear H0 H1;Intros).
Unfold def_F_inter.
(Split;[Idtac|Auto]).
(Generalize (lem_vide_propertie v2 var_vide);Intros).
Unfold Vide in H.
(Absurd (In v2 (vide var_vide));Auto).

Save.
(* End of proof of lem_inter_vide_prop                                     *)

(***************************************************************************)
(* L'ensemble de tous les ensembles ... n'existe pas !!!                   *)
(***************************************************************************)
Theorem notEEexists :
  ~(Ex[v0:E]((v1:E)(In v1 v0)))
.
(* Proof of notEEexists                                                    *)
(Unfold not ;Intros;Elim H;Clear  H;Intros).
(Elim (axs_comprehension [v0:E](not (In v0 v0)) x
        (subset [v0:E](not (In v0 v0)) x));Intros).
(Generalize (H (subset [v0:E](not (In v0 v0)) x)) ;Intros).
(Absurd (In (subset [v0:E](not (In v0 v0)) x)
          (subset [v0:E](not (In v0 v0)) x));Tauto).

Save.
(* End of proof of notEEexists                                             *)

(***************************************************************************)
(* Intersection d'un ensemble a                                            *)
(* a est un ensemble non vide. On recherche l'ensemble dont tous les       *)
(* elements sont elements des elements de a.                               *)
(***************************************************************************)
Definition Einter := [a:E][Ine:(~(a=Vide))]
  (subset ([v0:E]((v3:E)((In v3 a) -> (In v0 v3)))) (reunion a))
.

Lemma lem_Einter_propertie :
  (a:E)(v0:E)(Ine:(~(a=Vide)))
    ((In v0 (Einter a Ine))
    ->
    ((v3:E)((In v3 a) -> (In v0 v3))))
.
(* Proof of lem_Einter_propertie                                           *)
(Unfold Einter;Intros).
(Elim (axs_comprehension [v1:E](v3:E)(In v3 a)->(In v1 v3) (reunion a) v0);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros).
Exact (H1 v3 H0).

Save.
(* End of proof of lem_Einter_propertie                                    *)

Lemma lem_Einter_sing :
  (a:E)((Einter (singleton a) (lem_paire_not_empty a a))=a)
.
(* Proof of lem_Einter_sing                                                *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold Einter in H.
(Elim (axs_comprehension [v0:E](v3:E)(In v3 (singleton a))->(In v0 v3) (reunion (singleton a)) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim (axs_reunion (singleton a) v2);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros).
Unfold singleton in H.
(Elim (axs_paire a a x);Intros;Generalize (H2 H);Clear H H2 H3;Intros).
(Elim H;Clear H;Intros;Generalize H1;Rewrite -> H;Tauto).

Unfold Einter.
(Elim (axs_comprehension [v0:E](v3:E)(In v3 (singleton a))->(In v0 v3) (reunion (singleton a)) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Split;Intros).
(Elim (axs_reunion (singleton a) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Exists a;Split;[Apply lem_x_in_sing_x|Idtac];Auto).

Unfold singleton in H0.
(Elim (axs_paire a a v3);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Rewrite -> H0;Auto).

Save.
(* End of proof of lem_Einter_sing                                         *)

Lemma lem_Einter_paire :
  (a,b:E)((Einter (paire a b) (lem_paire_not_empty a b))=(inter a b))
.
(* Proof of lem_Einter_paire                                               *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold Einter in H.
(Elim (axs_comprehension [v0:E](v3:E)(In v3 (paire a b))->(In v0 v3) (reunion (paire a b)) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim (axs_reunion (paire a b) v2);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros).
(Generalize (H0 a);Generalize (H0 b);Clear H0;Intros).
(Cut (In a (paire a b));Cut (In b (paire a b));Intros).
(Generalize (H0 H3) (H2 H4);Clear H0 H2 H3 H4;Intros).
Unfold inter.
(Elim (axs_comprehension (def_F_inter b) a v2);Intros;Apply H4;Clear H3 H4;Intros).
(Unfold def_F_inter;Split;Auto).

(Elim (axs_paire a b b);Intros;Apply H4;Right;Reflexivity).

(Elim (axs_paire a b a);Intros;Apply H5;Left;Reflexivity).

(Elim (axs_paire a b b);Intros;Apply H4;Right;Reflexivity).

(Unfold inter in H;Unfold def_F_inter in H).
(Elim (axs_comprehension [x:E](In x b) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
Unfold Einter.
(Elim (axs_comprehension [v0:E](v3:E)(In v3 (paire a b))->(In v0 v3) (reunion (paire a b)) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;Intros).
(Elim (lem_union_propertie a b v2);Unfold union;Intros;Apply H2;Clear H1 H2;Intros).
(Left;Auto).

(Elim (axs_paire a b v3);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Rewrite -> H1;Auto).

Save.
(* End of proof of lem_Einter_paire                                        *)

Lemma lem_Einter_vide :
  (x:E)(Nv:(~(x=Vide)))(In Vide x)->(Einter x Nv)=Vide
.
(* Proof of lem_Einter_vide                                                *)
Intros.
(Apply lem_x_inc_vide;Intros).
(Unfold inc;Unfold Einter;Intros).
(Elim (axs_comprehension [v1:E](v3:E)(In v3 x)->(In v1 v3) (reunion x) v0);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
Exact (H1 Vide H).

Save.
(* End of proof of lem_Einter_vide                                         *)
(***************************************************************************)
(* Complementaire d'un ensemble                                            *)
(***************************************************************************)
Definition Comp := [a,b:E]
  (subset [y:E](~(In y b)) a)
.

Variable OComp : E->E->E.

Axiom def_OComp :
  (a,b:E)(y:E)
    (In y (OComp a b))
    <->
    ((In y a) /\ ~(In y b))
.

Lemma lem_Comp_eq_OComp :
  (a,b:E)((OComp a b)=(Comp a b))
.
(* Proof of lem_Comp_eq_OComp                                              *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (def_OComp a b v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
Unfold Comp.
(Elim (axs_comprehension [y:E](not (In y b)) a v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;Auto).

Unfold Comp in H.
(Elim (axs_comprehension [y:E](not (In y b)) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim (def_OComp a b v2);Intros;Apply H1;Clear H0 H1;Intros;Auto).

Save.
(* End of proof of lem_Comp_eq_OComp                                       *)

(***************************************************************************)
(* Difference symetrique de deux ensembles                                 *)
(***************************************************************************)
Definition Delta := [a,b:E]
  (union (Comp a b) (Comp b a))
.

(***************************************************************************)
(* Quelques proprietes agreables                                           *)
(***************************************************************************)
Lemma lem_inter_commut :
  (a,b:E)((inter a b) = (inter b a))
.
(* Proof of lem_inter_commut                                               *)
(Intros;Apply axs_extensionnalite;Intros).
(Elim (lem_inter_propertie a b v2);Intros).
(Elim (lem_inter_propertie b a v2);Intros).
(Unfold iff ;Split;Intros).
(Clear H0;Clear H1;Apply H2;Apply lem_and_sym;Apply H;Auto).

(Clear H;Clear H2;Apply H0;Apply lem_and_sym;Apply H1;Auto).

Save.
(* End of proof of lem_inter_commut                                        *)

Lemma lem_inter_assoc :
  (a,b,c:E)((inter (inter a b) c) = (inter a (inter b c)))
.
(* Proof of lem_inter_assoc                                                *)
Intros;Apply axs_extensionnalite;Intros.
Elim (lem_inter_propertie (inter a b) c v2);Intros.
Elim (lem_inter_propertie a (inter b c) v2);Intros.
Elim (lem_and_rew_l (In v2 c) (In v2 (inter a b))
       (and (In v2 a) (In v2 b)));Intros.
Elim (lem_and_rew_r (In v2 a) (In v2 (inter b c))
       (and (In v2 b) (In v2 c)));Intros.
Elim (lem_and_assoc_l (In v2 a) (In v2 b) (In v2 c));Intros.
Unfold iff ;Split;Intros.
(Clear H0;Clear H1;Clear H4;Clear H5;Clear H8).
(Apply H2;Clear  H2;Apply H6;Clear  H6;Apply H7;Clear  H7).
(Apply H3;Clear  H3;Apply H;Clear  H;Auto).

(Clear H;Clear H2;Clear H3;Clear H6;Clear H7).
(Apply H0;Clear  H0;Apply H4;Clear  H4;Apply H8;Clear  H8).
(Apply H5;Clear  H5;Apply H1;Clear  H1;Auto).

Apply (lem_inter_propertie b c v2).

Apply (lem_inter_propertie a b v2).

Save.
(* End of proof of lem_inter_assoc                                         *)

Lemma lem_union_commut :
  (a,b:E)((union a b) = (union b a))
.
(* Proof of lem_union_commut                                               *)
(Intros;Apply axs_extensionnalite;Intros).
(Elim (lem_union_propertie a b v2);Intros).
(Elim (lem_union_propertie b a v2);Intros).
(Unfold iff ;Split;Intros).
(Clear  H0;Clear  H1).
(Apply H2;Clear  H2;Apply lem_or_sym;Apply H;Clear  H;Auto).

(Clear  H;Clear  H2).
(Apply H0;Clear  H0;Apply lem_or_sym;Apply H1;Clear  H1;Auto).

Save.
(* End of proof of lem_union_commut                                        *)

Lemma lem_union_assoc :
  (a,b,c:E)
    ((union (union a b) c) = (union a (union b c)))
.
(* Proof of lem_union_assoc                                                *)
(Intros;Apply axs_extensionnalite;Intros).
(Elim (lem_union_propertie (union a b) c v2);Intros).
(Elim (lem_union_propertie a (union b c) v2);Intros).
(Elim (lem_or_rew_l (In v2 c) (In v2 (union a b))
        (or (In v2 a) (In v2 b)));Intros).
(Elim (lem_or_rew_r (In v2 a) (In v2 (union b c))
        (or (In v2 b) (In v2 c)));Intros).
(Elim (lem_or_assoc_l (In v2 a) (In v2 b) (In v2 c));Intros).
(Unfold iff ;Split;Intros).
(Clear H0;Clear H1;Clear H4;Clear H5;Clear H8).
(Apply H2;Clear  H2;Apply H6;Clear  H6;Apply H7;Clear  H7).
(Apply H3;Clear  H3;Apply H;Clear  H;Auto).

(Clear H;Clear H2;Clear H3;Clear H6;Clear H7).
(Apply H0;Clear  H0;Apply H4;Clear  H4;Apply H8;Clear  H8).
(Apply H5;Clear  H5;Apply H1;Clear  H1;Auto).

Apply (lem_union_propertie b c v2).

Apply (lem_union_propertie a b v2).

Save.
(* End of proof of lem_union_assoc                                         *)

Lemma lem_inter_dist :
  (a,b,c:E)
    ((inter a (union b c)) = (union (inter a b) (inter a c)))
.
(* Proof of lem_inter_dist                                                 *)
Intros;Apply axs_extensionnalite;Intros.
Elim (lem_inter_propertie a (union b c) v2);Intros.
Elim (lem_union_propertie (inter a b) (inter a c) v2);Intros.
Elim (lem_or_rew_lr (In v2 (inter a b)) (In v2 (inter a c))
       (and (In v2 a) (In v2 b)) (and (In v2 a) (In v2 c)));Intros.
Elim (lem_or_rew_lr (and (In v2 a) (In v2 b)) (and (In v2 a) (In v2 c))
       (and (In v2 b) (In v2 a)) (and (In v2 c) (In v2 a)));Intros.
Elim (lem_and_rew_r (In v2 a) (In v2 (union b c))
       (or (In v2 b) (In v2 c)));Intros.
Unfold iff ;Split;Intros.
(Clear H0;Clear H1;Clear H3;Clear H5;Clear H8).
(Apply H2;Clear  H2;Apply H4;Clear  H4).
(Apply H6;Clear  H6;Apply lem_and_fact).
(Apply lem_and_sym;Apply H7;Clear  H7;Apply H;Clear  H;Auto).

(Clear H;Clear H2;Clear H4;Clear H6;Clear H7).
(Apply H0;Clear  H0;Apply H8;Clear  H8).
(Apply lem_and_sym;Apply lem_and_dist).
(Apply H5;Clear  H5;Apply H3;Clear  H3;Apply H1;Clear  H1;Auto).

Apply lem_union_propertie.

(Unfold iff ;Split;Intros;Apply lem_and_sym;Auto).

(Unfold iff ;Split;Intros;Apply lem_and_sym;Auto).

Apply lem_inter_propertie.

Apply lem_inter_propertie.

Save.
(* End of proof of lem_inter_dist                                          *)

Lemma lem_union_dist :
  (a,b,c:E)
    ((union a (inter b c)) = (inter (union a b) (union a c)))
.
(* Proof of lem_union_dist                                                 *)
(Intros;Apply axs_extensionnalite;Intros).
(Elim (lem_union_propertie a (inter b c) v2);Intros).
(Elim (lem_inter_propertie (union a b) (union a c) v2);Intros).
(Elim (lem_and_rew_lr (In v2 (union a b)) (In v2 (union a c))
        (or (In v2 a) (In v2 b)) (or (In v2 a) (In v2 c)));Intros;[Idtac|Apply lem_union_propertie|Apply lem_union_propertie]).
(Elim (lem_or_rew_r (In v2 a) (In v2 (inter b c))
        (and (In v2 b) (In v2 c)));Intros;[Idtac|Apply lem_inter_propertie]).
(Unfold iff ;Split;Intros).
(Apply H2;Clear  H2;Apply H4;Clear  H4).
(Elim (lem_and_rew_lr (or (In v2 a) (In v2 b)) (or (In v2 a) (In v2 c))
        (or (In v2 b) (In v2 a)) (or (In v2 c) (In v2 a)));Intros;[Idtac|Unfold iff ;Split;Intros;Apply lem_or_sym;Auto|Unfold iff ;Split;Intros;Apply lem_or_sym;Auto]).
(Apply H4;Clear  H4;Apply lem_or_fact;Apply lem_or_sym;Apply H5;Clear  H5;Apply H;Clear  H;Auto).

(Apply H0;Clear  H0;Apply H6;Clear  H6;Apply lem_or_sym;Apply lem_or_dist).
(Elim (lem_and_rew_lr (or (In v2 b) (In v2 a)) (or (In v2 c) (In v2 a))
        (or (In v2 a) (In v2 b)) (or (In v2 a) (In v2 c)));Intros;[Idtac|Unfold iff ;Split;Intros;Apply lem_or_sym;Auto|Unfold iff ;Split;Intros;Apply lem_or_sym;Auto]).
(Apply H6;Clear  H6;Apply H3;Clear  H3;Apply H1;Clear  H1;Auto).

Save.
(* End of proof of lem_union_dist                                          *)

(***************************************************************************)
(* Quelques proprietes de l'ensemble Vide                                  *)
(***************************************************************************)
Lemma lem_reunion_vide :
  (reunion Vide) = Vide
.
(* Proof of lem_reunion_vide                                               *)
(Apply axs_extensionnalite;Intros).
(Unfold iff;Split;Intros).
(Elim (axs_reunion Vide v2);Intros).
(Elim H0;Clear H0 H1;Intros;[Idtac|Auto]).
(Elim H0;Clear H0;Intros;Unfold Vide;Unfold Vide in H;Elim (lem_vide_propertie x var_vide);Auto).

(Unfold Vide in H;Elim (lem_vide_propertie v2 var_vide);Auto).

Save.
(* End of proof of lem_reunion_vide                                        *)

Lemma lem_union_vide :
  (a:E)(union a Vide)=a
.
(* Proof of lem_union_vide                                                 *)
(Intros;Apply axs_extensionnalite;Intros).
(Elim (lem_union_propertie a Vide v2);Intros).
(Unfold iff ;Split;Intros).
(Elim H;Clear  H H0;Intros;[Auto|Unfold Vide  in
H1;Elim (lem_vide_propertie v2 var_vide);Auto|Auto]).

(Apply H0;Clear  H H0;Left;Auto).

Save.
(* End of proof of lem_union_vide                                          *)

Lemma lem_inter_vide :
  (a:E)(inter a Vide)=Vide
.
(* Proof of lem_inter_vide                                                 *)
(Intros;Apply axs_extensionnalite;Intros).
(Elim (lem_inter_propertie a Vide v2);Intros).
(Unfold iff ;Split;Intros).
(Elim H;Clear  H H0;Intros;Auto).

(Apply H0;Clear  H H0;Unfold Vide  in
H1;Elim (lem_vide_propertie v2 var_vide);Auto).

Save.
(* End of proof of lem_inter_vide                                          *)

(***************************************************************************)
(* Proprietes de l'inclusion                                               *)
(***************************************************************************)
Lemma lem_vide_is_inc :
  (a:E)(inc Vide a)
.
(* Proof of lem_vide_is_inc                                                *)
(Unfold inc ;Intros).
(Unfold Vide  in H;Elim (lem_vide_propertie v0 var_vide);Auto).

Save.
(* End of proof of lem_vide_is_inc                                         *)

(***************************************************************************)
(* Definition de l'inclusion stricte                                       *)
(***************************************************************************)
Definition incNotEq : E->E->Prop := [a:E][b:E]
  ((v0:E)((In v0 a) -> (In v0 b)))
  /\ (Ex[y:E]((In y b) /\ ~(In y a)))
.

(***************************************************************************)
(*                     Next : axs_remplacement.v                           *)
(***************************************************************************)
