(***************************************************************************)
(* File: axs_paire.v                                                       *)
(* auteur: alex@sysal.ibp.fr                                               *)
(* date: 19/10/95                                                          *)
(***************************************************************************)
Require Export axs_extensionnalite.

(***************************************************************************)
(* Axiome de la paire                                                      *)
(***************************************************************************)
Variable paire : E->E->E.
Axiom axs_paire :
  (v0,v1:E)(v3:E)((In v3 (paire v0 v1))<->((v3=v0) \/ (v3=v1)))
.

(***************************************************************************)
(* Etant donne a et b, {a,b} est unique.                                   *)
(***************************************************************************)
Lemma lem_paire_unique :
  (a,b,p:E)
    ((y:E)((In y p) <-> (y=a \/ y=b)))
    -> (p = (paire a b))
.
(* Proof of lem_paire_unique                                               *)
Intros.
(Apply axs_extensionnalite;Intros).
(Generalize (H v2) ;Clear  H;Unfold iff ;Intros).
(Elim H;Clear  H;Intros).
(Generalize (axs_paire a b) ;Unfold iff ;Intros).
(Generalize (H1 v2) ;Clear  H1;Intros).
(Elim H1;Clear  H1;Intros).
(Split;Intros).
(Clear  H0;Clear  H1;Auto).

(Clear  H;Clear  H2;Auto).

Save.
(* End of proof of lem_paire_unique                                        *)

(***************************************************************************)
(* {a,b} = {b,a}                                                           *)
(***************************************************************************)
Lemma lem_paire_sym :
  (a,b:E)
    (paire a b)=(paire b a)
.
(* Proof of lem_paire_sym                                                  *)
Intros.
Apply axs_extensionnalite;Intros;Unfold iff ;Intros.
Elim (axs_paire b a v2);Intros.
Elim (axs_paire a b v2);Intros.
(Split;Intros).
(Clear  H;Clear  H2;Apply H0;Clear  H0;Apply lem_or_sym;Auto).

(Clear  H0;Clear  H1;Apply H2;Clear  H2;Apply lem_or_sym;Auto).

Save.
(* End of proof of lem_paire_sym                                           *)

(***************************************************************************)
(* Propriete de la paire                                                   *)
(***************************************************************************)
Lemma lem_paire_propertie :
  (a0,b0,a1,b1:E)
    ((paire a0 b0) = (paire a1 b1))
    <->
    (((a0 = a1) /\ (b0 = b1)) \/ ((a0 = b1) /\ (b0 = a1)))
.
(* Proof of lem_paire_propertie                                            *)
Intros;Split;Intros.
Apply lem_or_and_conv_r.
(Generalize axs_paire ;Intros).
Split.
(Elim (H0 a1 b1 a0);Intros).
(Clear  H0;Clear  H2;Apply H1;Clear  H1).
Rewrite <- H.
(Elim (axs_paire a0 b0 a0);Intros).
(Clear  H0;Apply H1;Clear  H1;Left;Reflexivity).

Split.
Apply lem_or_sym.
(Elim (H0 a0 b0 b1);Clear  H0;Intros).
Clear  H1.
(Elim (lem_eq_or a0 b0 b1);Intros).
(Clear  H1;Apply H2;Clear  H2).
(Apply H0;Clear  H0).
Rewrite -> H.
(Elim (axs_paire a1 b1 b1);Intros).
(Clear  H0;Apply H1;Clear  H1;Right;Reflexivity).

Split.
(Elim (lem_eq_or a0 b0 a1);Intros).
(Clear  H1;Apply H2;Clear  H2).
(Elim (H0 a0 b0 a1);Clear  H0;Intros).
(Clear  H1;Apply H0;Clear  H0;Rewrite -> H).
(Elim (axs_paire a1 b1 a1);Intros).
(Clear  H0;Apply H1;Clear  H1;Left;Reflexivity).

Apply lem_or_sym.
(Elim (H0 a1 b1 b0);Clear  H0;Intros).
(Clear  H1;Apply H0;Clear  H0;Rewrite <- H;Clear  H).
(Elim (axs_paire a0 b0 b0);Intros).
(Clear  H;Apply H0;Clear  H0;Right;Reflexivity).

(Elim H;Clear  H;Intros).
(Elim H;Clear  H;Intros).
(Rewrite -> H;Clear  H;Rewrite -> H0;Clear  H0;Reflexivity).

(Elim H;Clear  H;Intros).
(Rewrite -> H;Clear  H;Rewrite -> H0;Clear  H0;Elim (lem_paire_sym a1 b1);Intros;Reflexivity).

Save.
(* End of proof of lem_paire_propertie                                     *)

(***************************************************************************)
(* Singleton                                                               *)
(***************************************************************************)
Definition singleton := [a:E]
  (paire a a)
.

Hint Unfold singleton.

(***************************************************************************)
(* {a} = {a'} <=> a = a'                                                   *)
(***************************************************************************)
Lemma lem_singleton_propertie :
  (a,b:E)
    ((singleton a) = (singleton b))
    <->
    (a = b)
.
(* Proof of lem_singleton_propertie                                        *)
Intros;Unfold singleton .
Elim (lem_paire_propertie a a b b);Intros.
Unfold iff ;Split;Intros.
(Apply lem_and_contract;Apply lem_or_contract;Auto).

Rewrite -> H1;Reflexivity.

Save.
(* End of proof of lem_singleton_propertie                                 *)

Lemma lem_in_singleton :
  (a:E)(x:E)
    (In x (singleton a))
    <->
    (x = a)
.
(* Proof of lem_in_singleton                                               *)
(Intros;Elim (axs_paire a a x);Intros).
(Unfold singleton ;Unfold iff ;Split;Intros;[Elim H;Intros;Auto|Apply H0;Left;Auto]).

Save.
(* End of proof of lem_in_singleton                                        *)

Lemma lem_x_in_sing_x :
  (x:E)(In x (singleton x))
.
(* Proof of lem_x_in_sing_x                                                *)
(Intros;Unfold singleton;Elim (axs_paire x x x);Intros;Apply H0;Clear H0 H;Left;Reflexivity).

Save.
(* End of proof of lem_x_in_sing_x                                         *)

(***************************************************************************)
(*                        Next : axs_reunion.v                             *)
(***************************************************************************)
