(***************************************************************************)
(* File: axs_reunion.v                                                     *)
(* auteur: alex@sysal.ibp.fr                                               *)
(* date: 19/10/95                                                          *)
(***************************************************************************)
Require Export axs_paire.

(***************************************************************************)
(* Axiome de la reunion                                                    *)
(***************************************************************************)
Variable reunion : E->E.
Axiom axs_reunion :
  (v0:E)((v2:E)((In v2 (reunion v0))<->(Ex[v3:E]
    ((In v3 v0) /\ (In v2 v3)))))
.

(***************************************************************************)
(* Unicite de la reunion de d'ensemble                                     *)
(* a est la reunion de b                                                   *)
(***************************************************************************)
Lemma lem_reunion_unique :
  (v0:E)
    (v1:E)((v2:E)(In v2 v1) <-> (Ex[v3:E]((In v3 v0) /\ (In v2 v3))))
    -> ( v1 = (reunion v0))
.

(* Preuve de lem_reunion_unique                                            *)
Intros.
(Generalize (axs_reunion v0) ;Intros).
(Apply axs_extensionnalite;Intros).
(Generalize (H v2) ;Clear  H;Unfold iff ;Intros).
(Elim H;Clear  H;Intros).
(Generalize (H0 v2) ;Clear  H0;Unfold iff ;Intros).
(Elim H0;Clear  H0;Intros).
(Split;Intros).
(Clear  H0;Clear  H1;Auto).

(Clear  H;Clear  H2;Auto).

Save.
(* Fin de la preuve de lem_reunion_unique                                  *)

(* Autre version                                                           *)


(***************************************************************************)
(* Notion d'union de deux ensembles                                        *)
(***************************************************************************)
Definition union := [a,b:E]
  (reunion (paire a b))
.

Hint Unfold union.

Lemma lem_union_propertie :
  (a,b:E)(v0:E)((In v0 (union a b)) <-> ((In v0 a) \/ (In v0 b)))
.

(* Proof of lem_union_propertie                                            *)
Intros;Unfold union .
(Elim (axs_reunion (paire a b) v0);Intros).
(Unfold iff ;Split;Intros).
(Elim H;Clear  H;Intros).
(Clear  H0;Clear  H1).
(Elim (axs_paire a b x);Unfold iff ;Intros).
(Elim H;Clear  H;Intros).
(Clear  H1;Elim H0;Clear  H0;Intros).
(Clear  H;Rewrite <- H0;Left;Assumption).

(Clear  H;Rewrite <- H0;Right;Assumption).

(Clear  H2;Assumption).

(Clear  H0;Assumption).

(Apply H0;Elim H1;Clear  H1;Intros).
(Clear  H;Clear  H0;Exists a;Split).
(Clear  H1;Elim (axs_paire a b a);Intros;Auto).

Assumption.

(Clear  H;Clear  H0;Exists b;Split).
(Clear H1;Elim (axs_paire a b b);Intros;Auto).

Assumption.

Save.
(* End of proof of lem_union_propertie                                     *)

Lemma lem_union_propertie_lr :
  (a,b:E)(v0:E)((In v0 (union a b)) -> ((In v0 a) \/ (In v0 b)))
.
(* Proof of lem_union_propertie_lr                                         *)
(Intros;Elim (lem_union_propertie a b v0);Intros;Exact (H0 H)).

Save.
(* End of proof of lem_union_propertie_lr                                  *)

Lemma lem_union_propertie_rl :
  (a,b:E)(v0:E)((In v0 a) \/ (In v0 b))->(In v0 (union a b))
.
(* Proof of lem_union_propertie_rl                                         *)
(Intros;Elim (lem_union_propertie a b v0);Intros;Exact (H1 H)).

Save.
(* End of proof of lem_union_propertie_rl                                  *)

(***************************************************************************)
(*                        Next : axs_parties.v                             *)
(***************************************************************************)
