(***************************************************************************)
(* File: couples.v                                                         *)
(* auteur: alex@sysal.ibp.fr                                               *)
(* date: 19/10/95                                                          *)
(***************************************************************************)
Require Export axs_remplacement.

Definition couple := [a,b:E]
  (paire (singleton a) (paire a b))
.

(***************************************************************************)
(* Un lemme intermediaire sur les paires : {a,b}={a,c} ==> b=c             *)
(***************************************************************************)
Lemma lem_paire_prop2 :
  (a,b,c:E)
    ((paire a b)=(paire a c))
    <->
    (b=c)
.
(* Proof of lem_paire_prop2                                                *)
(Intros;Unfold iff;Split;Intros).
Elim (lem_paire_propertie a b a c);Intros.
Elim H0;Intros;[Idtac|Idtac|Auto];Elim H2;Intros;[Auto|Rewrite <- H3;Rewrite -> H4;Reflexivity].

(Rewrite -> H;Reflexivity).

Save.
(* End of proof of lem_paire_prop2                                         *)

(***************************************************************************)
(* {a} = {b,c} <==> (a=b) /\ (a=c)                                         *)
(***************************************************************************)
Lemma lem_sing_eq_paire :
  (a,b,c:E)
    ((singleton a) = (paire b c))
    <->
    ((a=b) /\ (a=c))
.
(* Proof of lem_sing_eq_paire                                              *)
(Intros;Unfold singleton ;Unfold iff ;Split;Intros).
(Elim (lem_paire_propertie a a b c);Intros).
(Elim H0;Intros;[Auto|Elim H2;Intros;Split;Auto|Auto]).

(Elim H;Intros;Rewrite <- H0;Rewrite <- H1;Reflexivity).

Save.
(* End of proof of lem_sing_eq_paire                                       *)

(***************************************************************************)
(* (a,b)=(c,d) <==> (a=c /\ b=d)                                           *)
(***************************************************************************)
Lemma lem_couple_propertie :
  (al,ar,bl,br:E)
    ((couple al bl)=(couple ar br))
    <->
    ((al=ar) /\ (bl=br))
.
(* Proof of lem_couple_propertie                                           *)
(Unfold couple;Intros).
(Unfold iff;Split;Intros).
(Elim (lem_paire_propertie (singleton al) (paire al bl) (singleton ar) (paire ar br));Intros).
(Clear H1;Elim H0;Clear H0;Intros;[Idtac|Idtac|Auto];Elim H0;Clear H0;Intros).
(Elim (lem_singleton_propertie al ar);Intros).
(Generalize H1;Clear H1;Elim H2;Clear H2;Intros;[Idtac|Auto]).
(Elim (lem_paire_prop2 al bl br);Intros).
(Split;[Reflexivity|Auto]).

(Elim (lem_sing_eq_paire al ar br);Intros).
(Elim (lem_sing_eq_paire ar al bl);Intros).
(Cut (eq ? (singleton ar) (paire al bl));Intros;[Idtac|Symmetry;Auto]).
(Elim H2;Intros;[Idtac|Auto]).
(Elim H4;Intros;[Idtac|Auto]).
(Rewrite <- H10;Rewrite <- H7;Rewrite <- H8;Split;Reflexivity).

(Elim H;Clear H;Intros).
(Rewrite -> H;Rewrite -> H0;Reflexivity).

Save.
(* End of proof of lem_couple_propertie                                    *)

Lemma lem_couple_propertie_lr :
  (al,ar,bl,br:E)
    ((couple al bl)=(couple ar br))
    ->
    ((al=ar) /\ (bl=br))
.
(* Proof of lem_couple_propertie_lr                                        *)
(Intros;Elim (lem_couple_propertie al ar bl br);Intros;Exact (H0 H)).

Save.
(* End of proof of lem_couple_propertie_lr                                 *)

Lemma lem_couple_propertie_rl :
  (al,ar,bl,br:E)
    ((al=ar) /\ (bl=br))
    ->
    ((couple al bl)=(couple ar br))
.
(* Proof of lem_couple_propertie_rl                                        *)
(Intros;Elim (lem_couple_propertie al ar bl br);Intros;Exact (H1 H)).

Save.
(* End of proof of lem_couple_propertie_rl                                 *)

(***************************************************************************)
(* Produit cartesien de deux ensembles                                     *)
(***************************************************************************)
Definition cartesien := [a,b:E]
  (subset 
    ([u:E](Ex[x:E](Ex[y:E]((In x a) /\ (In y b) /\ (u = (couple x y))))))
    (parties (parties (union a b)))
  )
.

Lemma lem_cartesian_propertie :
  (a,b:E)(x:E)
    (In x (cartesien a b))
    <->
    (Ex[v1:E](Ex[v2:E]((In v1 a) /\ (In v2 b) /\ (x = (couple v1 v2)))))
.
(* Proof of lem_cartesian_propertie                                        *)
(Intros;Unfold cartesien).
(Elim (axs_comprehension [u:E](Ex[x0:E](Ex[y:E](and (In x0 a) (and (In y b) (eq ? u (couple x0 y)))))) (parties (parties (union a b))) x);Intros).
(Unfold iff;Split;Intros).
(Elim H;Clear H;Clear H0;Intros;[Idtac|Auto]).
(Do 2 (Elim H0;Clear H0;Intros);Exists x0;Exists x1;Auto).

(Do 2 (Elim H1;Clear H1;Intros);Clear H;Apply H0;Clear H0).
(Split;[Idtac|Exists x0;Exists x1;Auto]).
(Elim H1;Clear H1;Intros;Elim H0;Clear H0;Intros).
(Elim (axs_parties (parties (union a b)) x);Intros).
(Clear H2;Apply H3;Clear H3;Intros).
(Elim (axs_parties (union a b) v3);Intros).
(Clear H3;Apply H4;Clear H4;Intros).
(Elim (lem_union_propertie a b v0);Intros).
(Clear H4;Apply H5;Clear H5;Intros).
Unfold couple in H1.
(Elim (axs_paire (singleton x0) (paire x0 x1) v3);Intros).
(Generalize H2;Rewrite -> H1;Intros).
(Clear H5;Elim H4;Clear H4;Intros;[Idtac|Idtac|Auto]).
(Generalize H3;Rewrite -> H4;Intros).
(Elim (lem_in_singleton x0 v0);Intros).
(Generalize H;Clear H8;Elim H7;Clear H7;Intros;[Idtac|Auto];Left;Auto).

(Generalize H3;Rewrite -> H4;Intros).
(Elim (axs_paire x0 x1 v0);Intros).
(Clear H8;Elim H7;Clear H7;Intros;[Rewrite -> H7;Left;Auto|Rewrite -> H7;Right;Auto|Auto]).

Save.
(* End of proof of lem_cartesian_propertie                                 *)

Lemma lem_cartesian_cons :
  (x,y,a,b:E)
    (In x a)
    ->
    (In y b)
    ->
    (In (couple x y) (cartesien a b))
.
(* Proof of lem_cartesian_cons                                             *)
Intros;Unfold cartesien.
(Elim (axs_comprehension [u:E](Ex[x0:E](Ex[y0:E](and (In x0 a) (and (In y0 b) (eq ? u (couple x0 y0)))))) (parties (parties (union a b))) (couple x y));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists x;Exists y;Split;[Auto|Split;[Auto|Reflexivity]]]).
(Elim (axs_parties (parties (union a b)) (couple x y));Intros;Apply H2;Clear H1 H2;Intros).
(Elim (axs_parties (union a b) v3);Intros;Apply H3;Clear H2 H3;Intros).
(Unfold couple in H1;Generalize (axs_paire (singleton x) (paire x y) v3);Unfold iff;Intros).
(Elim H3;Clear H3;Intros).
(Generalize (H3 H1);Clear H3 H4;Intros).
(Elim H3;Clear H3;Intros).
(Generalize H2;Clear H2;Rewrite -> H3;Clear H3;Intros).
(Elim (lem_in_singleton x v0);Intros).
(Generalize (H3 H2);Clear H2 H3 H4;Intros;Rewrite -> H2).
(Elim (lem_union_propertie a b x);Intros).
(Apply H4;Left;Auto).

(Generalize H2;Clear H2;Rewrite -> H3;Clear H3;Intros).
(Elim (axs_paire x y v0);Intros).
(Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros).
(Rewrite -> H2;Clear H2;Elim (lem_union_propertie a b x);Intros).
(Apply H3;Left;Auto).

(Rewrite -> H2;Clear H2;Elim (lem_union_propertie a b y);Intros).
(Apply H3;Right;Auto).

Save.
(* End of proof of lem_cartesian_cons                                      *)

(***************************************************************************)
(* Somme disjointe de deux ensembles                                       *)
(***************************************************************************)
(* Old Release                                                             *)
Variable Odisj_l : E->E.
Axiom axs_Odisj_l :
  (a:E)(x:E)
    (In x (Odisj_l a))
    <->
    (Ex[y:E]((In y a) /\ (x = (couple y Vide))))
.
Variable Odisj_r : E->E.
Axiom axs_Odisj_r :
  (a:E)(x:E)
    (In x (Odisj_r a))
    <->
    (Ex[y:E]((In y a) /\ (x = (couple y (singleton Vide)))))
.

Definition Osomme_disj := [a,b:E]
  (union (Odisj_l a) (Odisj_r b))
.

(* New Release                                                             *)
Definition disj_l := [a:E]
  (subset
    ([y:E](Ex[x:E]((y=(couple x Vide)) /\ (In x a))))
    (cartesien a (singleton Vide))
  )
.

Definition disj_r := [a:E]
  (subset
    ([y:E](Ex[x:E]((y=(couple x (singleton Vide))) /\ (In x a))))
    (cartesien a (singleton (singleton Vide)))
  )
.

Definition somme_disj := [a,b:E]
  (union (disj_l a) (disj_r b))
.

(* Old and New                                                             *)
Lemma lem_OaN :
  (a,b:E)((somme_disj a b)=(Osomme_disj a b))
.
(* Proof of lem_OaN                                                        *)
(Intros;Apply axs_extensionnalite;Intros).
(Unfold somme_disj;Unfold Osomme_disj).
Elim (lem_union_propertie (disj_l a) (disj_r b) v2);Intros.
Elim (lem_union_propertie (Odisj_l a) (Odisj_r b) v2);Intros.
Elim (axs_Odisj_l a v2);Intros.
Elim (axs_Odisj_r b v2);Intros.
Elim (axs_comprehension [y:E](Ex [x:E](and (eq ? y (couple x Vide)) (In x a))) (cartesien a (singleton Vide)) v2);Intros.
Elim (axs_comprehension [y:E](Ex [x:E](and (eq ? y (couple x (singleton Vide))) (In x b))) (cartesien b (singleton (singleton Vide))) v2);Intros.
Unfold iff ;Split;Intros.
(Elim H;Clear H H0;Intros;[Clear H11;Apply H2;Clear H1 H2;Intros|Clear H11;Apply H2;Clear H1 H2;Intros|Auto]).
Unfold disj_l in H.
(Elim H7;Clear H7 H8;Intros;[Clear H|Auto]).
(Elim H1;Clear H1;Intros).
(Elim H;Clear H;Intros).
(Left;Apply H4;Clear H3 H4;Intros).
(Exists x;Split;Auto).

Unfold disj_r in H.
(Elim H9;Clear H9 H10;Intros;[Clear H|Auto]).
(Elim H1;Clear H1;Intros).
(Elim H;Clear H;Intros).
(Right;Apply H6;Clear H5 H6;Intros).
(Exists x;Split;Auto).

(Apply H0;Clear H H0;Intros).
(Elim H1;Clear H1 H2;Intros;[Clear H11|Clear H11|Auto]).
(Left;Unfold disj_l;Apply H8;Clear H7 H8;Intros).
(Apply lem_and_sym;Elim H3;Clear H3 H4;Intros;[Clear H|Auto]).
(Elim H0;Clear H0;Intros).
(Split;[Exists x;Split;Auto|Idtac]).
(Elim (lem_cartesian_propertie a (singleton Vide) v2);Intros).
(Apply H2;Clear H1 H2;Exists x;Exists Vide;Split;[Auto|Apply lem_and_sym;Split;[Auto|Idtac]]).
(Unfold singleton;Elim (axs_paire Vide Vide Vide);Intros).
(Apply H2;Clear H1 H2;Left;Reflexivity).

(Right;Unfold disj_r;Apply H10;Clear H9 H10;Intros).
(Apply lem_and_sym;Elim H5;Clear H5 H6;Intros;[Clear H|Auto]).
(Elim H0;Clear H0;Intros).
(Split;[Exists x;Split;Auto|Idtac]).
(Elim (lem_cartesian_propertie b (singleton (singleton Vide)) v2);Intros).
(Apply H2;Clear  H1 H2;Exists x;Exists (singleton Vide);Split;[Auto|Apply lem_and_sym;Split;[Auto|Idtac]]).
(Unfold singleton ;Elim (axs_paire (paire Vide Vide) (paire Vide Vide) (paire Vide Vide));Intros).
(Apply H2;Left;Reflexivity).

Save.
(* End of proof of lem_OaN                                                 *)

(***************************************************************************)
(* Extension des constructions d'ensembles                                 *)
(***************************************************************************)
(*
Lemma lem_ext_set :
  (f:E->E)(P:E->Prop)
  (Ex[ns:E]((y:E)((In y ns) <-> (Ex[t:E]((y=(f t)) /\ (P t))))))
.
(* Proof of lem_ext_set                                                    *)
????
(* End of proof of lem_ext_set                                             *)
*)
(***************************************************************************)
(*                     Projections et couples                              *)
(***************************************************************************)
(* Un lemme auxiliaire: La reunion d'un couple (a,b) est la paire {a,b}    *)
Lemma lem_reunion_couple :
  (a,b:E)((reunion (couple a b))=(paire a b))
.
(* Proof of lem_reunion_couple                                             *)
(Intros;Apply axs_extensionnalite;Intros).
Unfold couple.
(Elim (axs_reunion (paire (singleton a) (paire a b)) v2);Intros).
(Elim (axs_paire a b v2);Intros).
(Unfold iff;Split;Intros).
(Elim H;Clear H H0;Intros;[Clear H3|Auto]).
(Elim H;Clear H;Intros).
(Apply H2;Clear H1 H2;Intros).
(Elim (axs_paire (singleton a) (paire a b) x);Intros).
(Elim H1;Clear H1 H2;Intros;[Clear H|Clear H|Auto]).
(Left;Generalize H0;Rewrite -> H1;Clear H1 H0;Intros).
Unfold singleton in H0.
(Elim (axs_paire a a v2);Intros).
(Apply lem_or_contract;Apply H;Auto).

(Generalize H0;Rewrite -> H1;Clear H1 H0;Intros).
(Elim (axs_paire a b v2);Auto).

(Apply H0;Clear H0 H;Intros).
(Exists (paire a b);Split;[Idtac|Auto]).
(Clear H1 H2 H3;Elim (axs_paire (singleton a) (paire a b) (paire a b));Auto).

Save.
(* End of proof of lem_reunion_couple                                      *)

(* Definitions des projections par comprehension                           *)
Definition first := [c:E]
  (subset 
    [x:E](Ex[a:E](Ex[b:E]((c=(couple a b)) /\ (In x a)))) 
    (reunion (reunion c))
  )
.
Definition second := [c:E]
  (subset 
    [x:E](Ex[a:E](Ex[b:E]((c=(couple a b)) /\ (In x b)))) 
    (reunion (reunion c))
  )
.

Lemma lem_first_propertie :
  (a,b:E)((first (couple a b))=a)
.
(* Proof of lem_first_propertie                                            *)
(Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros).
Unfold first in H.
(Elim (axs_comprehension [x:E](Ex[a0:E](Ex [b0:E](and (eq ? (couple a b) (couple a0 b0)) (In x a0)))) (reunion (reunion (couple a b))) v2);Intros).
(Elim H0;Clear H0 H1;Intros;[Clear  H|Auto]).
(Elim H1;Clear H1;Intros).
(Elim H;Clear H;Intros).
(Elim H;Clear H;Intros).
(Elim (lem_couple_propertie a x b x0);Intros).
(Elim H2;Clear H2 H3;Intros;[Clear H|Auto]).
(Rewrite -> H2;Auto).

Unfold first.
(Elim (axs_comprehension [x:E](Ex[a0:E](Ex [b0:E](and (eq ? (couple a b) (couple a0 b0)) (In x a0)))) (reunion (reunion (couple a b))) v2);Intros).
(Apply H1;Clear H1 H0;Intros).
Rewrite -> (lem_reunion_couple a b).
(Split;[Idtac|Exists a;Exists b;Split;[Reflexivity|Auto]]).
(Elim (lem_union_propertie a b v2);Intros).
(Clear H0;Unfold union in H1;Apply H1;Clear H1;Intros).
(Left;Auto).

Save.
(* End of proof of lem_first_prop                                          *)

Lemma lem_second_propertie :
  (a,b:E)((second (couple a b))=b)
.
(* Proof of lem_second_propertie                                           *)
Intros;Apply axs_extensionnalite;Intros;Unfold iff;Split;Intros.
Unfold second in H.
Elim (axs_comprehension [x:E](Ex[a0:E](Ex [b0:E](and (eq ? (couple a b) (couple a0 b0)) (In x b0)))) (reunion (reunion (couple a b))) v2);Intros.
Elim H0;Clear H0 H1;Intros;[Clear H|Auto].
Elim H1;Clear H1;Intros.
Elim H;Clear H;Intros.
Elim H;Clear H;Intros.
Elim (lem_couple_propertie a x b x0);Intros.
Elim H2;Clear H2 H3;Intros;[Clear H|Auto].
Rewrite -> H3;Auto.

Unfold second.
Elim (axs_comprehension [x:E](Ex[a0:E](Ex [b0:E](and (eq ? (couple a b) (couple a0 b0)) (In x b0)))) (reunion (reunion (couple a b))) v2);Intros.
Apply H1;Clear H1 H0;Intros.
Rewrite -> (lem_reunion_couple a b).
Split;[Idtac|Exists a;Exists b;Split;[Reflexivity|Auto]].
Elim (lem_union_propertie a b v2);Intros.
Clear H0;Unfold union in H1;Apply H1;Clear H1;Intros.
(Right;Auto).

Save.
(* End of proof of lem_second_propertie                                    *)

(***************************************************************************)
(* Couples et reunion                                                      *)
(***************************************************************************)
Lemma lem_couple_reunion_prop :
  (x,y,z:E)
    (In (couple x y) z)
    ->
    ((In x (reunion (reunion z))) /\ (In y (reunion (reunion z))))
.
(* Proof of lem_couple_reunion_prop                                        *)
Intros.
Split.
(Elim (axs_reunion (reunion z) x);Intros;Apply H1;Clear H0 H1;Intros).
(Exists (singleton x);Split;[Idtac|Apply lem_x_in_sing_x]).
(Elim (axs_reunion z (singleton x));Intros;Apply H1;Clear H0 H1;Intros).
(Exists (couple x y);Split;[Auto|Elim (axs_paire (singleton x) (paire x y) (singleton x));Intros;Apply H1;Left;Reflexivity]).

(Elim (axs_reunion (reunion z) y);Intros;Apply H1;Clear H0 H1;Intros).
(Exists (paire x y);Split;[Idtac|Elim (axs_paire x y y);Intros;Apply H1;Right;Reflexivity]).
(Elim (axs_reunion z (paire x y));Intros;Apply H1;Clear H0 H1;Intros).
(Exists (couple x y);Split;[Auto|Elim (axs_paire (singleton x) (paire x y) (paire x y));Intros;Apply H1;Right;Reflexivity]).

Save.
(* End of proof of lem_couple_reunion_prop                                 *)

(***************************************************************************)
(*                        Les n-uplets                                     *)
(***************************************************************************)
Inductive isTupple : nat->E->Prop :=
  c_0_tupple : (isTupple O Vide)
 |c_1_tupple : (a:E)(isTupple (S O) a)
 |c_2_tupple : (a,b:E)(isTupple (S (S O)) (couple a b))
 |c_n_tupple : (a,b:E)(n:nat)((isTupple (S (S n)) b)->(isTupple (S (S (S n))) (couple a b)))
.

Recursive Definition cart_power : nat->E->E :=
  O a => Vide
 |(S O) a => a
 |(S (S n)) a => (cartesien a (cart_power (S n) a))
.

Lemma lem_eq_cart_power :
  (a:E)((cart_power (S (S O)) a)=(cartesien a a))
.
(* Proof of lem_eq_cart_power                                              *)
(Intros;Rewrite -> (cart_power_eq3 O a);Rewrite -> (cart_power_eq2 a);Reflexivity).

Save.
(* End of proof of lem_eq_cart_power                                       *)

(***************************************************************************)
(*                     Next : relations.v                                  *)
(***************************************************************************)
