(***************************************************************************)
(* File: useful.v                                                          *)
(* auteur: alex@sysal.ibp.fr                                               *)
(* date: 19/10/95                                                          *)
(***************************************************************************)
Require nothing.

(***************************************************************************)
(* Many useful lemmas                                                      *)
(***************************************************************************)

(***************************************************************************)
(* Distributivity and factorisation of /\ and \/                           *)
(***************************************************************************)
Lemma lem_and_fact :
  (P0,P1,P2:Prop)
    (((P0 \/ P1) /\ P2) -> ((P0 /\ P2) \/ (P1 /\ P2)))
.
(* Proof of lem_and_fact                                                   *)
(Intros;Tauto).
Save.
(* End of proof of lem_and_fact                                            *)

Lemma lem_and_dist :
  (P0,P1,P2:Prop)
    (((P0 /\ P2) \/ (P1 /\ P2)) -> ((P0 \/ P1) /\ P2))
.
(* Proof of lem_and_dist                                                   *)
(Intros;Tauto).

Save.
(* End of proof of lem_and_dist                                            *)

Lemma lem_or_fact :
  (P0,P1,P2:Prop)
    (((P0 /\ P1) \/ P2) -> ((P0 \/ P2) /\ (P1 \/ P2)))
.
(* Proof of lem_or_fact                                                    *)
(Intros;Tauto).

Save.
(* End of proof of lem_or_fact                                             *)

Lemma lem_or_dist :
  (P0,P1,P2:Prop)
    (((P0 \/ P2) /\ (P1 \/ P2)) -> ((P0 /\ P1) \/ P2))
.
(* Proof of lem_or_dist                                                    *)
(Intros;Tauto).

Save.
(* End of proof of lem_or_dist                                             *)

(***************************************************************************)
(* Rewriting                                                               *)
(***************************************************************************)
Lemma lem_or_and_conv_l :
  (a,b,c,d:Prop)
    ((a /\ b) \/ (c /\ d))
    ->
    ((a \/ c) /\ (b \/ c) /\ (a \/ d) /\ (b \/ d))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

Lemma lem_or_and_conv_r :
  (a,b,c,d:Prop)
    ((a \/ c) /\ (b \/ c) /\ (a \/ d) /\ (b \/ d))
    ->
    ((a /\ b) \/ (c /\ d))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

Lemma lem_and_rew_r :
  (P,Q1,Q2:Prop)
    (Q1 <-> Q2)
    ->
    ((P /\ Q1) <-> (P /\ Q2))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

Lemma lem_and_rew_l :
  (P,Q1,Q2:Prop)
    (Q1 <-> Q2)
    ->
    ((Q1 /\ P) <-> (Q2 /\ P))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

Lemma lem_and_assoc_l :
  (a,b,c:Prop)
    ((a /\ b) /\ c)
    <-> 
    (a /\ b /\ c)
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

(* Note : le /\ est associatif a droite en CoQ don pas de lemme symmetrique*)

Lemma lem_or_rew_l :
  (P,Q1,Q2:Prop)
    (Q1 <-> Q2)
    ->
    ((Q1 \/ P) <-> (Q2 \/ P))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

Lemma lem_or_rew_r :
  (P,Q1,Q2:Prop)
    (Q1 <-> Q2)
    ->
    ((P \/ Q1) <-> (P \/ Q2))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

Lemma lem_or_assoc_l :
  (a,b,c:Prop)
    ((a \/ b) \/ c)
    <-> 
    (a \/ b \/ c)
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

(* Note : le \/ est associatif a droite en CoQ don pas de lemme symmetrique*)

Lemma lem_and_rew_lr :
  (al,bl,ar,br:Prop)
  ((al <-> ar)
  ->
  (bl <-> br)
  -> 
  ((al /\ bl) <-> (ar /\ br)))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

Lemma lem_or_rew_lr :
  (al,bl,ar,br:Prop)
  ((al <-> ar)
  ->
  (bl <-> br)
  -> 
  ((al \/ bl) <-> (ar \/ br)))
.
(* Proof                                                                   *)
(Intros;Tauto).
Save.
(* End of proof                                                            *)

(***************************************************************************)
(* Symmetry properties                                                     *)
(***************************************************************************)
Lemma lem_iff_sym :
  (a,b:Prop)((a <-> b) -> (b <-> a))
.
(* Proof of lem_iff_sym                                                    *)
(Intros;Tauto).
Save.
(* End of proof of lem_iff_sym                                             *)

Lemma lem_or_sym :
  (a,b:Prop)
    (a \/ b) -> (b \/ a)
.
(* Proof of lem_or_sym                                                     *)
Intros.
(Elim H;Clear  H;Intros).
(Right;Assumption).

(Left;Assumption).

Save.
(* End of proof of lem_or_sym                                              *)

Lemma lem_and_sym :
  (a,b:Prop)
    (a /\ b) -> (b /\ a)
.
(* Proof of lem_and_sym                                                    *)
(Intros;Tauto).

Save.
(* End of proof of lem_and_sym                                             *)

(***************************************************************************)
(* Contraction and expansion                                               *)
(***************************************************************************)
Lemma lem_or_expand :
  (a:Prop)
    a -> (a \/ a)
.
(* Proof of lem_or_expand                                                  *)
(Intros;Tauto).

Save.
(* End of proof of lem_or_expand                                           *)

Lemma lem_and_expand :
  (a:Prop)
    a -> (a /\ a)
.
(* Proof of lem_and_expand                                                 *)
(Intros;Tauto).

Save.
(* End of proof of lem_and_expand                                          *)

Lemma lem_or_contract :
  (a:Prop)
    (a \/ a) -> a
.
(* Proof of lem_or_contract                                                *)
(Intros;Tauto).

Save.
(* End of proof of lem_or_contract                                         *)

Lemma lem_and_contract :
  (a:Prop)
    (a /\ a) -> a
.
(* Proof of lem_and_contract                                               *)
(Intros;Tauto).

Save.
(* End of proof of lem_and_contract                                        *)

(***************************************************************************)
(* Existenciel et unicite                                                  *)
(***************************************************************************)
Definition exUniq := [A:Set][P:A->Prop]
  (Ex[x:A]((P x) /\ ((y:A)((P y)->(x=y)))))
.

Syntactic Definition ExU :=
  (exUniq ?)
.

(***************************************************************************)
(*                     Next : any file you want                            *)
(***************************************************************************)
