(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Lci.v                                   *)
(****************************************************************************)

(* Proprie'te's des lois de composition interne *)

(*****************************************************************************)
Section Internal.

Variable S: Set.
Variable G: S -> Prop.
Variable Add: S -> S -> S.
Variable O, I: S.
Variable Opp, Inv: S -> S.
(*****************************************************************************)

(***************)
Definition intern := (x, y: S) (G x) -> (G y) -> (G (Add x y)).

(**********************)
Definition commutativity := (x, y: S) ((Add x y) = (Add y x)).

(**********************)
Definition associativity := 
  (x, y, z: S) (<S> (Add x (Add y z)) = (Add (Add x y) z)).

(****************)
Definition neutral := [S: Set] [G: S -> Prop] [Add: S -> S -> S] [O: S]
  (G O) /\ (x: S) (G x) -> (((Add x O) = x) /\ ((Add O x) = x)).

(****************)
Lemma neutral_add: (neutral S G Add O) -> O = (Add O O).

Goal.
Intros; Symmetry.
Elim H; Intros; Elim (H1 O H0); Trivial.
Save.

(********************)
Definition is_opposite := 
  [x, y: S] (G x) /\ (G y) /\ ((Add x y) = O) /\ ( (Add y x) = O).

(************)
Lemma opp_com: commutativity -> 
  (x, y: S) ((G x) -> (G y) -> ( (Add x y) = O) -> (is_opposite x y)).

Goal.
Intros. Unfold is_opposite. 
Split. Exact H0. Split. Exact H1. Split. Exact H2. Elim (H x y). Exact H2.
Save.

(*****************)
Definition opposite := 
  (x: S) (G x) -> (is_opposite x (Opp x)).

(***********************)
Definition distributivity := [S: Set] [Add, Mult: S -> S -> S] 
  (x, y, z: S) ((Mult (Add x y) z) = (Add (Mult x z) (Mult y z))) /\
               ((Mult x (Add y z)) = (Add (Mult x y) (Mult x z))).

End Internal.
(*****************************************************************************)
