(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              Z_succ_pred.v                               *)
(****************************************************************************)
Require Arith.
Require Zbase.

(* Succ and Pred on Z *)

(**************)
Definition succZ := 
  [x: Z] (<Z> Case x of
    (* OZ *)    IZ 
    (* pos n *) [n: nat] (pos (S n))
    (* neg n *) [n: nat] (<Z> Case n of
       (* O *)   OZ
       (* S m *) [m: nat] (neg m)
       end) 
    end).

(**************)
Definition predZ := 
  [x: Z] (<Z> Case x of
    (* OZ *)    (neg O)
    (* pos n *) [n: nat] (<Z> Case n of
       (* O *)   OZ
       (* S m *) [m: nat](pos m)
       end)
    (* neg n *) [n: nat] (neg (S n)) 
    end).
    
(***************)
Lemma pred_succZ: (x: Z) ((predZ (succZ x)) = x).

Goal.
Intros; Pattern x; Elim x.
(* OZ *)
Simpl; Reflexivity.
(* pos n *)
Simpl; Reflexivity.
(* neg n *)
Intros; Elim n.
Simpl; Reflexivity.
Intros; Simpl; Reflexivity.
Save.

(***************)
Lemma succ_predZ: (x: Z) (succZ (predZ x)) = x.

Goal.
Intros; Pattern x; Elim x.
(* OZ *)
Simpl; Reflexivity.
(* pos n *)
Intros; Elim n.
Simpl; Reflexivity.
Intros; Simpl; Reflexivity.
(* neg n *)
Simpl; Reflexivity.
Save.

(*************************)
Lemma succ_pred_pred_succZ: (x: Z) (succZ (predZ x)) = (predZ (succZ x)).

Goal.
Intros; Rewrite (pred_succZ x); Exact (succ_predZ x).
Save.

(******************)
Lemma tech_pred_posZ: 
  (n: nat) (lt O n) -> (predZ (pos n)) = (pos (pred n)).

Goal.
Intro; Elim n; Intro. Elim (lt_n_O O); Exact H. Intros; Simpl; Reflexivity.
Save.

(********************)
Lemma tech_succ_posOZ: (n: nat) (succZ (posOZ n)) = (pos n).

Goal.
Intros; Elim n; Simpl; Reflexivity; Simpl; Reflexivity.
Save.
