(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Zbase.v                                  *)
(****************************************************************************)

(* Basic definitions on type Z *)

(*************)
Inductive  Z : Set := OZ: Z | pos: nat -> Z | neg: nat -> Z.

(***********)
Definition IZ := (pos O).

(****************)
Recursive Definition is_posn : Z -> Z -> Prop :=
  OZ       y => False
| (pos n) OZ => False 
| (pos n) (pos m) => (eq nat n m)
| (pos n) (neg m) => False
| (neg n) y => False.

(* was 
  [x, y: Z] (<Prop> Match x with
    (* OZ *) False
    (* pos n *) [n: nat] (<Prop> Match y with
       (* OZ *) False
       (* pos m *) [m: nat] (n = m)
       (* neg m *) [m: nat] False 
       end)
    (* neg n *) [n: nat] False
    end).
**)

(**********************)
Lemma tech_pos_not_posZ: 
  (n, m: nat) ~( n = m) -> ~((pos n) = (pos m)).


Unfold not; Intros.
Cut (is_posn (pos n) (pos m)).
Simpl; Exact H. Rewrite H0; Simpl; Reflexivity.
Save.

(**************)
Lemma eq_OZ_dec: (x: Z) {x = OZ}+{~(x = OZ)}.


Intros; Elim x.
Left; Reflexivity.
Intros; Right; Discriminate.
Intros; Right; Discriminate.
Save.

(**************)
Definition posOZ := 
  [n: nat] (<Z> Case n of
     (* O *) OZ
     (* S n' *) [n': nat](pos n') 
     end).

(**************)
Definition negOZ := 
  [n: nat] (<Z> Case n of
     (* O *) OZ
     (* S n' *) [n': nat](neg n') 
     end).

(*************)
Definition absZ :=
  [x: Z] (<Z> Case x of
    (* OZ *)    OZ 
    (* pos n *) [n: nat] (pos n)
    (* neg n *) [n: nat] (pos n) 
    end).

(*************)
Definition sgnZ := 
  [x: Z] (<Z> Case x of
    (* OZ *) OZ
    (* pos n *) [n: nat] (pos O)
    (* neg n *) [n: nat] (neg O) 
    end).

