(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Zdiv.v                                  *)
(****************************************************************************)
Require Zbase.
Require Zadd.
Require Zmult.
Require Zle.
Require Minus.
Require Le.
Require Lt.
Require Gt.
Require Euclid_proof.
Require Nat_complements.
Require Z_succ_pred.

(********************)
Definition is_diveuclZ := [a, b, q, r: Z] 
  ~(b = OZ) /\ (leZ OZ r) /\ (ltZ r (absZ b)) /\ (a = (addZ (multZ b q) r)).

(***************************)
Inductive diveuclZ [a, b: Z]: Set
  := divexZ: (q, r: Z) (is_diveuclZ a b q r) -> (diveuclZ a b).

(************************)
Lemma divZ: (a, b: Z) ~(b = OZ) -> (diveuclZ a b).

Realizer 
[a,b:Z] 
  <Z*Z>Case b of
    	(except (Z*Z))
        [m:nat]<Z*Z>Case a of
                  <Z,Z>(OZ,OZ)
        	  [n:nat]<Z*Z>let (q,r) = (eucl_dev (S m) (S n)) in
            			((posOZ q),(posOZ r))
        	  [n:nat]<Z*Z>let (q,r) = (eucl_dev (S m) (S n)) in
                	  <Z*Z>if (eq_gt_O_dec r)
               		  then ((negOZ q),OZ)
                	  else ((neg q),(pos (minus m r)))
                end
        [m:nat]<Z*Z>Case a of
         	  <Z,Z>(OZ,OZ)
        	  [n:nat]<Z*Z>let (q,r) = (eucl_dev (S m) (S n)) in
            			((negOZ q),(posOZ r))
        	  [n:nat]<Z*Z>let (q,r) = (eucl_dev (S m) (S n)) in
                	  <Z*Z>if (eq_gt_O_dec r)
                	  then ((posOZ q),OZ)
                	  else ((pos q),(pos (minus m r)))
		end
   end.
Program_all.
(* Elim H; Reflexivity. *)
(* (is_diveuclZ OZ (pos m) OZ OZ) *)
Split. Discriminate. Split. Exact I. 
Split. Exact (le_O_n m).
Rewrite -> (mult_OZ (pos m)); Reflexivity.

(* (is_diveuclZ (pos n) (pos m) (posOZ q) (posOZ r)) *)
Split. Discriminate. Split. Apply (tech_posOZ_pos r).
Split. Unfold ltZ; Rewrite -> (tech_succ_posOZ r).
Exact (gt_S_le r m g).
Exact (tech_div1 n m q r e).

(* (is_diveuclZ (neg n) (pos m) (negOZ q) OZ) *)
Split. Discriminate. Split. Exact I.
Split. Exact (le_O_n m).
Rewrite -> (add_OZ (multZ (pos m) (negOZ q))). Apply (tech_div2 n m q).
Rewrite e; Rewrite e0; Auto.

(* (is_diveuclZ (neg n) (pos m) (neg q) (pos (minus m r))) *)
Split. Discriminate. Split. Exact I.
Split. 
Exact (lt_le_S (minus m r) m (lt_minus m r (gt_S_le r m g) g0)).
Exact (tech_div3 n m q r e g).

(* (is_diveuclZ OZ (neg m) OZ OZ) *)
Split. Discriminate. Split. Exact I.
Split. Exact (le_O_n m).
Rewrite -> (mult_OZ (neg m)); Reflexivity.

(* (is_diveuclZ (pos n) (neg m) (negOZ q) (posOZ r)) *)
Split. Discriminate. Split. Apply (tech_posOZ_pos r).
Split. 
Unfold ltZ; Rewrite -> (tech_succ_posOZ r); Exact (gt_S_le r m g).
Exact (tech_div4 n m q r e).

(* (is_diveuclZ (neg n) (neg m) (posOZ q) OZ) *)
Unfold is_diveuclZ. Split. Discriminate. Split. Exact I.
Split. Exact (le_O_n m).
Rewrite -> (add_OZ (multZ (neg m) (posOZ q))). 
Apply (tech_div5 n m q).
Rewrite e; Rewrite e0; Auto.

(* (is_diveuclZ (neg n) (neg m) (pos q) (pos (minus m r))) *)
Split. Discriminate. Split. Exact I.
Split.
Exact (lt_le_S (minus m r) m (lt_minus m r (gt_S_le r m g) g0)).
Exact (tech_div6 n m q r e g).
Save.
