(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Zgcd.v                                  *)
(****************************************************************************)
Require misc.
Require Zadd.
Require rings.
Require Nat_complements.
Require Zbase.
Require Z_succ_pred.
Require Zle.
Require Euclid_proof.
Require Peano_dec.
Require Zrec.
Require Zmult.
Require Zdiv.
Require groups.
Require Lci.

(***************************)
Lemma gcd_unicity_apart_sign :
 (a, b, d1, d2: Z) 
  (is_gcd Z IdZ multZ OZ a b d1) -> (is_gcd Z IdZ multZ OZ a b d2) -> 
  (d2 = d1) \/ (d2 = (oppZ d1)).

Intros.
Elim (gcd_unicity_apart_unities Z IdZ addZ multZ OZ IZ oppZ Z_unitary_commutative_ring integrityZ a b d1 d2 H H0).
Intros.
Elim (inversibleZ x); Intros.
Left. Elim H1; Intros; Elim H4; Intros. Rewrite H6. Rewrite H2. 
Exact (mult_IZ d1).
Right. Elim H1; Intros; Elim H4; Intros. Rewrite H6. Rewrite H2.
Simpl; Exact (mult_mIZ d1).
Elim H1; Intros; Exact H2.
Save.

(***********)
Lemma gcd_OZ_absZ : (b: Z) (is_gcd Z IdZ multZ OZ OZ b (absZ b)).

Intros. Elim (abs_eq_or_oppZ b); Intro y.
(* |b|=b *)
Rewrite y.
Unfold is_gcd; Split.
Unfold divide; Unfold IdZ; Split. Exact I. Split. Exact I. Left; Reflexivity.
Split; Unfold divide; Unfold IdZ. Split. Exact I. Split. Exact I.
Elim (eq_OZ_dec b); Intro y0. 
Left; Exact y0.
Right. Split. Exact y0. Exists IZ. Split. Exact I. Symmetry; Exact (mult_IZ b).
Intros; Exact H0.
(* |b|=(-b) *)
Rewrite y.
Unfold is_gcd; Split.
Unfold divide; Unfold IdZ; Split. Exact I. Split. Exact I. Left; Reflexivity.
Split; Unfold divide; Unfold IdZ; Split. Exact I. Split. Exact I.
Elim (eq_OZ_dec b); Intro y0. 
Left; Exact y0.
Right. Split.
Unfold not; Intros; Elim y0.
Exact (opp_O Z IdZ addZ multZ OZ oppZ Z_ring b I H).
Exists (oppZ IZ); Split. Exact I.
Rewrite (mult_opp_opp Z IdZ addZ multZ OZ oppZ Z_ring b IZ I I).
Symmetry; Exact (mult_IZ b). Exact I. Split. Exact I.
Elim H0; Intros; Elim H2; Intros; Elim H4; Intros. Rewrite H5. 
Left; Reflexivity.
Right; Split. Elim H5; Intros; Exact H6. Elim H5; Intros; Elim H7; Intros.
Exists (oppZ x). Split. Exact I. Elim H8; Intros; Rewrite H10.
Symmetry; Exact (mult_opp_r Z IdZ addZ multZ OZ oppZ Z_ring q x I I).
Save.

(******************)
Inductive is_gcdZ : Z -> Z -> Z -> Prop :=
  gcd_OZ: (b: Z) (is_gcdZ OZ b (absZ b))
| gcd_mod: (b, a, d, q, r: Z) ~(b = OZ) -> (is_diveuclZ a b q r) -> 
               (is_gcdZ r b d) -> (is_gcdZ b a d).

(******************)
Definition have_gcdZ := [a,b:Z]{d:Z|(is_gcdZ a b d)}.

Definition gcdZ_i := [a,b:Z](exist Z (is_gcdZ a b)).

(* Inductive have_gcdZ [a, b: Z]: Set
  := gcdZ_i: (d: Z) (is_gcdZ a b d) -> (have_gcdZ a b). *)

(*******************)
Lemma gcdZ_exists : (a, b: Z) (have_gcdZ a b).

Realizer 
[a:Z](recZ (Z->Z) 
        [q:Z][f:Z->Z->Z] 
     	  <Z->Z>if (eq_OZ_dec q)
          then absZ
          else [b:Z]<Z>let (div,rem) = (divZ b q) in (f rem q)
      a).
Program_all.
(* is_gcdZ (q, x, |x|) *)
Rewrite e; Apply (gcd_OZ x).
(* lt_absZ rem q *)
Elim i; Intros; Elim H0; Intros; Elim H2; Intros.
Unfold lt_absZ; Rewrite (tech_le_pos_abs rem H1); Exact H3.
(* is_gcdZ (n, b0, x) with is_gcdZ (r, n, x) *)
Apply gcd_mod with 2:=i; Trivial.

Save.

(****************)
Lemma gcdZ_is_gcd : (a, b, d: Z) (is_gcdZ a b d) -> (is_gcd Z IdZ multZ OZ a b d).

Intros. Elim H; Intros. Apply (gcd_OZ_absZ b0). Unfold is_gcd.
Split. Elim H3; Intros; Elim H5; Intros; Exact H6.
Split. Elim H1; Intros; Elim H5; Intros; Elim H7; Intros; Rewrite H9.
Apply (div_add Z IdZ addZ multZ OZ oppZ Z_ring (multZ b0 q) r d0).
Elim H3; Intros; Elim H11; Intros. 
Exact (div_mult Z IdZ addZ multZ OZ oppZ Z_ring b0 q d0 H12 I).
Elim H3; Intros; Exact H10.
Intros. Elim H3; Intros; Elim H7; Intros. Apply (H9 q0).
Cut (r = (addZ a0 (oppZ (multZ b0 q)))); Intros. Rewrite H10.
Apply (div_add Z IdZ addZ multZ OZ oppZ Z_ring a0 (oppZ (multZ b0 q)) q0 H5).
Apply (div_opp Z IdZ addZ multZ OZ oppZ Z_ring (multZ b0 q) q0).
Exact (div_mult Z IdZ addZ multZ OZ oppZ Z_ring b0 q q0 H4 I).
Elim H1; Intros; Elim H11; Intros; Elim H13; Intros; Rewrite H15.
Elim (addZ_commutativity r (multZ b0 q)).
Elim (addZ_associativity r (multZ b0 q) (oppZ (multZ b0 q))).
Elim (addZ_opposite (multZ b0 q) I); Intros. 
Elim H17; Intros. Elim H19; Intros. Rewrite H20. Symmetry. Exact (add_OZ r).
Exact H4.
Save.

(*************)
Definition gcdZ := 
  [a, b: Z] (pi1 Z (is_gcdZ a b) (gcdZ_exists a b)).

(*******************)
Theorem gcdZ_correct : (a, b: Z) (is_gcdZ a b (gcdZ a b)).
Proof [a, b: Z] (pi2 Z (is_gcdZ a b) (gcdZ_exists a b)).

(*********************)
Lemma positive_is_gcdZ : (a, b, d: Z) (is_gcdZ a b d) -> (leZ OZ d).

Intros; Elim H; Intros. Apply (sign_absZ b0). Exact H3.
Save.

(********************)
Lemma unicity_is_gcdZ : (a, b, d1, d2: Z) (is_gcdZ a b d1) -> (is_gcdZ a b d2) ->  d2 = d1.

Intros.
Elim (gcd_unicity_apart_sign a b d1 d2 (gcdZ_is_gcd a b d1 H) (gcdZ_is_gcd a b d2 H0)).
Intros; Exact H1.
Intros; Apply (le_opp_OZ2 d2 d1 H1 (positive_is_gcdZ a b d2 H0) (positive_is_gcdZ a b d1 H)).
Save.

(*****************)
Lemma gcdZ_is_gcdZ : (a, b, d: Z) (is_gcdZ a b d) -> (d = (gcdZ a b)).

Intros. Apply (unicity_is_gcdZ a b (gcdZ a b) d (gcdZ_correct a b) H).
Save.

(*************)
Lemma gcd_modZ : (a, b, q, r: Z) ~(b = OZ) -> (is_diveuclZ a b q r) -> (gcdZ r b) = (gcdZ b a). 

Intros. Apply (gcdZ_is_gcdZ b a (gcdZ r b)).
Apply (gcd_mod b a (gcdZ r b) q r H H0 (gcdZ_correct r b)).
Save.

(*********************************)
Inductive verify_BezoutZ [a, b: Z]: Set :=
  Bezout_i: (u, v: Z) (addZ (multZ a u) (multZ b v)) = (gcdZ a b) -> 
            (verify_BezoutZ a b).

(********************)
Lemma Bezout_exists : (a, b: Z) (verify_BezoutZ a b).

Realizer 
[a:Z](recZ (Z->Z*Z) 
      [q:Z][f:Z->Z->Z*Z] 
     	<Z->Z*Z>if (eq_OZ_dec q)
        then [b:Z](IZ,(sgnZ b))
        else [b:Z]<Z*Z>let (div,rem) = (divZ b q) in
                   <Z*Z>let (u,v) = (f rem q) in 
                     ((addZ v (oppZ (multZ div u))), u) 
      a).
Program_all.
(* (addZ (multZ q IZ) (multZ b (sgnZ b)))=(gcdZ q b) *)
Rewrite e. Simpl. Rewrite (sgn_abs b). 
Apply (gcdZ_is_gcdZ OZ b (absZ b)); Apply gcd_OZ.
(* (lt_absZ rem q) *)
Unfold lt_absZ. Elim i; Intros; Elim H0; Intros. 
Rewrite (tech_le_pos_abs rem H1). Elim H2; Trivial.
(* (addZ (multZ q (addZ v (oppZ (multZ div u)))) (multZ b u))=(gcdZ q b) *)
Elim i; Intros; Elim H0; Intros; Elim H2; Intros. Pattern 1 b; Rewrite H4.
(*Clear H0 H1 H2 H3 H4 H5.*)
Elim (mult_add_distributivity q v (oppZ (multZ div u))); Intros.
Rewrite H6.(*H1; Clear H0 H1.*)
Elim (mult_add_distributivity (multZ q div) rem u); Intros. 
Rewrite H7.(*H0; Clear H0 H1.*)
Rewrite (mult_opp_r Z IdZ addZ multZ OZ oppZ Z_ring q (multZ div u) I I).
Elim (addZ_commutativity (multZ rem u) (multZ (multZ q div) u)).
Rewrite (add_add Z addZ addZ_commutativity addZ_associativity (multZ q v) (oppZ (multZ q (multZ div u))) (multZ rem u) (multZ (multZ q div) u)).
Elim (addZ_commutativity (multZ rem u) (multZ q v)). Rewrite e.
Elim (multZ_associativity q div u).
Elim (addZ_opposite (multZ q (multZ div u)) I); Intros. 
Elim H10(*H1*); Intros; Elim H12(*H3*); Intros; Rewrite H14(*H5; Clear H0 H1 H2 H3 H4 H5*). 
Rewrite (add_OZ (gcdZ rem q)). Exact (gcd_modZ b q div rem H i).
Save.

(******************)
Definition congruentZ := 
  [x, y, n: Z] (divide Z IdZ multZ OZ n (addZ x (oppZ y))).

(*****************)
Lemma divide_selfZ : (x: Z) (divide Z IdZ multZ OZ x x).

Intros. Unfold divide. Split. Exact I. Split. Exact I.
Elim (eq_OZ_dec x); Intros. Left; Exact y.
Right; Split. Exact y. Exists IZ. Split. Exact I. Symmetry; Exact (mult_IZ x).
Save.

(**********************************)
Theorem chinese_remaindering_theorem : (a, b, x, y: Z) (gcdZ a b) = IZ -> 
  (Ex [z: Z] (congruentZ z x a) /\ (congruentZ z y b)).

Intros. Elim (Bezout_exists a b); Intros.
Exists (addZ (multZ x (multZ b v)) (multZ y (multZ a u))).
Unfold congruentZ; Split.
(* congruentZ (z, x, a) *)
Replace (multZ b v) with (addZ IZ (oppZ (multZ a u))).
Elim (mult_add_distributivity x IZ (oppZ (multZ a u))); Intros.
Rewrite H1; Clear H0 H1. Rewrite (mult_IZ x).
Elim (mult_opp_r Z IdZ addZ multZ OZ oppZ Z_ring a u I I).
Rewrite (multZ_associativity x a (oppZ u)). Elim (multZ_commutativity a x).
Elim (multZ_associativity a x (oppZ u)). Rewrite (multZ_associativity y a u).
Elim (multZ_commutativity a y). Elim (multZ_associativity a y u).
Elim (addZ_associativity x (multZ a (multZ x (oppZ u))) (multZ a (multZ y u))).
Elim (addZ_commutativity (addZ (multZ a (multZ x (oppZ u))) (multZ a (multZ y u))) x).
Elim (addZ_associativity (addZ (multZ a (multZ x (oppZ u))) (multZ a (multZ y u))) x (oppZ x)).
Elim (addZ_opposite x I); Intros. Elim H1; Intros. Elim H3; Intros. 
Rewrite H4; Clear H0 H1 H2 H3 H4 H5.
Rewrite (add_OZ (addZ (multZ a (multZ x (oppZ u))) (multZ a (multZ y u)))).
Apply (div_add Z IdZ addZ multZ OZ oppZ Z_ring (multZ a (multZ x (oppZ u))) (multZ a (multZ y u)) a).
Apply (div_mult Z IdZ addZ multZ OZ oppZ Z_ring a (multZ x (oppZ u)) a (divide_selfZ a) I).
Apply (div_mult Z IdZ addZ multZ OZ oppZ Z_ring a (multZ y u) a (divide_selfZ a) I).
Elim H. Elim e.
Elim (addZ_commutativity (multZ b v) (multZ a u)).
Elim (addZ_associativity (multZ b v) (multZ a u) (oppZ (multZ a u))).
Elim (addZ_opposite (multZ a u) I); Intros. Elim H1; Intros. Elim H3; Intros.
Rewrite H4; Clear H0 H1 H2 H3 H4 H5. Exact (add_OZ (multZ b v)).
(* congruentZ (z y b) *)
Cut (multZ a u) = (addZ IZ (oppZ (multZ b v))); Intros.
Rewrite H0; Clear H0.
Elim (mult_add_distributivity y IZ (oppZ (multZ b v))); Intros.
Rewrite H1; Clear H0 H1. Rewrite (mult_IZ y).
Elim (mult_opp_r Z IdZ addZ multZ OZ oppZ Z_ring b v I I).
Rewrite (multZ_associativity y b (oppZ v)). Elim (multZ_commutativity b y).
Elim (multZ_associativity b y (oppZ v)). Rewrite (multZ_associativity x b v).
Elim (multZ_commutativity b x). Elim (multZ_associativity b x v).
Elim (addZ_commutativity (multZ b (multZ y (oppZ v))) y).
Rewrite (addZ_associativity (multZ b (multZ x v)) (multZ b (multZ y (oppZ v))) y).
Elim (addZ_associativity (addZ (multZ b (multZ x v)) (multZ b (multZ y (oppZ v)))) y (oppZ y)).
Elim (addZ_opposite y I); Intros. Elim H1; Intros. Elim H3; Intros.
Rewrite H4; Clear H0 H1 H2 H3 H4 H5.
Rewrite (add_OZ (addZ (multZ b (multZ x v)) (multZ b (multZ y (oppZ v))))).
Apply (div_add Z IdZ addZ multZ OZ oppZ Z_ring (multZ b (multZ x v)) (multZ b (multZ y (oppZ v))) b).
Apply (div_mult Z IdZ addZ multZ OZ oppZ Z_ring b (multZ x v) b (divide_selfZ b) I).
Apply (div_mult Z IdZ addZ multZ OZ oppZ Z_ring b (multZ y (oppZ v)) b (divide_selfZ b) I).
Elim H. Elim e.
Elim (addZ_associativity (multZ a u) (multZ b v) (oppZ (multZ b v))).
Elim (addZ_opposite (multZ b v) I); Intros. Elim H1; Intros. Elim H3; Intros.
Rewrite H4; Clear H0 H1 H2 H3 H4 H5. Symmetry; Exact (add_OZ (multZ a u)).
Save.

