(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Zrec.v                                  *)
(****************************************************************************)
Require Arith.
Require Nat_complements.
Require misc.
Require Zbase.
Require Zle.
Require Z_succ_pred.

(**********)
Lemma lt_OZ: (m: Z) ~(ltZ (absZ m) OZ).

Intros; Unfold ltZ; Elim m.
Unfold not; Simpl; Intros; Exact H.
Unfold not; Simpl; Intros; Exact H.
Unfold not; Simpl; Intros; Exact H.
Save.

(**********)
Lemma Zrec1: (P: Z -> Set) 
  ((n: Z) ((m: Z) (lt_absZ m n) -> (P m)) -> (P n)) -> (P OZ).

Intros; Apply (H OZ); Intros.
Unfold lt_absZ in H0; Unfold 2 absZ in H0; Elim (lt_OZ m); Assumption.
Save.

(**********************************)
Inductive and_set_set_set [S1: Set; S2: Set] : Set :=
  and_set_set_set_i: S2 -> S1 -> (and_set_set_set S1 S2).

(*****************)
Definition and_recZ :=
  [p: nat] [P: Z -> Set]  
  (n: nat) (le n p) -> (and_set_set_set (P (pos n)) (P (neg n))).

(**********)
Lemma Zrec2: (P: Z -> Set) 
  ((n: Z) ((m: Z) (lt_absZ m n) -> (P m)) -> (P n)) -> (and_recZ O P).

Unfold and_recZ; Intros; Apply and_set_set_set_i.
(* neg n *)
Elim (le_n_O_eq n H0). Apply (H (neg O)); Intros.
Rewrite -> (tech_lt_abs_OZ m). Apply (Zrec1 P H). Exact H1.
(* pos n *)
Elim (le_n_O_eq n H0). Apply (H (pos O)); Intros.
Rewrite -> (tech_lt_abs_OZ m). Apply (Zrec1 P H). Exact H1.
Save.

(**********)
Lemma Zrec3: (P: Z -> Set) (p: nat)
  ((n: Z) ((m: Z) (lt_absZ m n) -> (P m)) -> (P n)) ->
  ((and_recZ p P) -> (and_recZ (S p) P)).

Split.
(* P (neg n) *)
Elim (lt_succ n p H1); Intros. Elim (H0 n y); Intros; Trivial.
Rewrite -> y. Apply (H (neg (S p))).
Destruct m. Intros; Apply (Zrec1 P H).
Unfold lt_absZ; Unfold absZ; Unfold ltZ; Unfold leZ; Intros.
Elim (H0 n0); Auto.
Unfold lt_absZ; Unfold absZ; Unfold ltZ; Unfold leZ; Intros.
Elim (H0 n0); Auto.
(* P (pos n) *)
Elim (lt_succ n p H1); Intros. Elim (H0 n); Intros; Trivial.
Rewrite -> y. Apply (H (pos (S p))).
Destruct m. Intros; Apply (Zrec1 P H).
Unfold lt_absZ; Unfold absZ; Unfold ltZ; Unfold leZ; Intros.
Elim (H0 n0); Auto.
Unfold lt_absZ; Unfold absZ; Unfold ltZ; Unfold leZ; Intros.
Elim (H0 n0); Auto.
Save.

(*************)
Theorem Zrec4: (P: Z -> Set)
  ((n: Z) ((m: Z) (lt_absZ m n) -> (P m)) -> (P n)) -> (p: nat) (and_recZ p P).

Intros; Elim p. 
Exact (Zrec2 P H). 
Intros; Apply Zrec3; Trivial.
Save.

(************)
Theorem recZ: (P: Z -> Set)
  ((n: Z) ((m: Z) (lt_absZ m n) -> (P m)) -> (P n)) -> (p: Z) (P p).

Intros; Elim p.
(* OZ *)
Exact (Zrec1 P H).
(* pos n *)
Intro n; Cut (and_recZ n P). Intros.
Elim (H0 n); Auto.
Apply Zrec4; Trivial.
(* neg n *)
Intro n; Cut (and_recZ n P). Intros.
Elim (H0 n); Auto.
Apply Zrec4; Trivial.
Save.
