(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Demo.v                                  *)
(****************************************************************************)

(* A short demo of coq *)

Fixpoint Plus [n:nat] : nat -> nat := [m:nat]
     <nat>Case n of
        (* O *) m 
      (* S p *) [p:nat](S (Plus p m)) end.


(**********************************************)
(***      A few elementary properties       ***)
(**********************************************)

Goal (n:nat)n=(Plus n O).
Induction n; Simpl; Auto.
Save Plus_n_O.
Hint Plus_n_O.

Goal (m,n:nat)(S (Plus m n))=(Plus m (S n)).
Induction m; Simpl; Auto.
Save Plus_S.
Hint Plus_S.

Goal (n,m:nat)(Plus n m)=(Plus m n).
Induction m; Simpl; Auto.
Induction 1; Auto.
Save Plus_com.
Immediate Plus_com.

Goal (n,m,p:nat)((Plus n m)=(Plus n p))-><nat>m=p.
Induction n; Simpl; Auto.
Save Plus_simpl.

Goal (n,m,p:nat)(Plus n (Plus m p))=(Plus (Plus n m) p).
Induction n; Simpl; Auto.
Save Plus_assoc.
Hint Plus_assoc.

Goal (n,m,p:nat)(Plus (Plus n m) p)=(Plus n (Plus m p)).
Auto.
Save assoc_Plus.


(************************************)
(***         Trees                ***)
(************************************)

Inductive tree : Set := 
    tip  : tree 
  | node : tree -> tree -> tree.  

Fixpoint size [t:tree] : nat := 
 <nat>Case t of
   (* tip *)      (S O)
   (* node u v *) [u,v:tree]
                  (Plus (size u) (size v)) end.

Goal (t,u,v:tree)(size (node t (node u v))) = (size (node (node t u) v)).

Simpl; Auto.
Save size_assoc.
