(*-- Tactics Tauto & Intuition --*)

(*-- Tauto: Tactic for automating proof in Intuionnistic Propositional 
            Caulculus, based on the contraction-free LJT of Dickhoff 
     Ref:   Roy Dyckhoff, The Journal of Symbolic Logic Volume 57,
            Number 3, Sept. 1992                                      --*) 

Parameter A,B,C:Prop.
Parameter even:nat -> Prop.
Parameter P:nat -> Prop.

(*-- Examples of intuitionistcs tautologies 
     Ref: Lincoln Wallen, Automated Deduction in Nonclassical Logics, 1990 
          Sthephen Klenne, Introduction to Metamathematics,1952. --*)

Theorem Ex_Wallen: (A->(B/\C)) -> ((A->B)\/(A->C)).
Tauto.
Save.

Theorem Ex_Klenne: ~(~(A \/ ~A)).
Tauto.
Save.

(*-- Example with a first ordre formula --*)  
Theorem Ex_Klenne' : (n:nat)(~(~((even n) \/ ~(even n)))).
Intro.
Tauto.
Save.

(*-- Example with a first ordre tautologic formula --*)
Theorem Ex_Klenne'' : ~(~(((n:nat)(even n)) \/ ~((m:nat)(even m)))).
Tauto.
Save.

(*-- Intuition: Tactics for simplifications of buts, based on LJT calcul.
     Ref      : Cesar Munoz, Rapport de stage de DEA 1993. --*)

Theorem Intu:(((x:nat)(P x)) /\ B) ->
              (((y:nat)(P y)) /\ (P O)) \/ (B /\ (P O)).
Intuition.
Save.

(*  A private club has the following rules :
 *
 *  . rule 1 : Every non-scottish member wears red socks
 *  . rule 2 : Every member wears a kilt or doesn't wear red socks
 *  . rule 3 : The married members don't go out on sunday
 *  . rule 4 : A member goes out on sunday if and only if he is scottish
 *  . rule 5 : Every member who wears a kilt is scottish and married
 *  . rule 6 : Every scottish member wears a kilt
 *
 *  Actually, no one can be accepted !
 *)

Section club.

Variable Scottish, RedSocks, WearKilt, Married, GoOutSunday : Prop.

Hypothesis rule1 : ~Scottish -> RedSocks.
Hypothesis rule2 : WearKilt \/ ~RedSocks.
Hypothesis rule3 : Married -> ~GoOutSunday.
Hypothesis rule4 : GoOutSunday <-> Scottish.
Hypothesis rule5 : WearKilt -> (Scottish /\ Married).
Hypothesis rule6 : Scottish -> WearKilt.

Lemma NoMember : False.
Tauto.
Save.

End club.

Theorem tauto:((x:nat)(P x)) -> ((y:nat)(P y)).
Tauto.
Save.

Theorem tauto1:(A -> A).
Tauto.
Save.

Theorem tauto2:(A -> B -> C) -> (A -> B) -> A -> C.
Tauto.
Save.

Theorem a : (x0: (A \/ B))(x1:(B /\ C))(A -> B).
Tauto.
Save.

Theorem a2 :  ((A -> (B /\ C)) -> ((A -> B) \/ (A -> C))).
Tauto.
Save.

Theorem a4 : (~A -> ~A).
Tauto.
Save.

Theorem e2 : ~(~(A \/ ~A)).
Tauto.
Save.

Theorem e4 : ~(~((A \/ B) -> (A \/ B))).
Tauto.
Save.

Theorem y0 : (x0:A)(x1: ~A)(x2:(A -> B))(x3:(A \/ B))(x4:(A /\ B))(A -> False).
Tauto.
Save.

Theorem y1 : (x0:((A /\ B) /\ C))B.
Tauto.
Save.

Theorem y2 : (x0:A)(x1:B)(C \/ B).
Tauto.
Save.

Theorem y3 : (x0:(A /\ B))(B /\ A).
Tauto.
Save.

Theorem y5 : (x0:(A \/ B))(B \/ A).
Tauto.
Save.

Theorem y6 : (x0:(A -> B))(x1:A) B.
Tauto.
Save.

Theorem y7 : (x0 : ((A /\ B) -> C))(x1 : B)(x2 : A) C.
Tauto.
Save.

Theorem y8 : (x0 : ((A \/ B) -> C))(x1 : A) C.
Tauto.
Save.

Theorem y9 : (x0 : ((A \/ B) -> C))(x1 : B) C.
Tauto.
Save.

Theorem y10 : (x0 : ((A -> B) -> C))(x1 : B) C.
Tauto.
Save.

