(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             ProPre_sorting.v                             *)
(****************************************************************************)


(* == Quelques fonctions utiles == *)

(* Inductive Set bool := true : bool | false : bool. *)

Recursive Definition si [X:Set] : bool -> X -> X -> X :=
   true x y  => x
 | false x y => y.

(* Inductive Set nat : O:nat | S:nat->nat. *)

Recursive Definition egal_nat : nat -> nat -> bool :=
   O     O     => true
 | O     (S m) => false
 | (S n) O     => false
 | (S n) (S m) => (egal_nat n m).

Recursive Definition inf_egal : nat -> nat -> bool :=
   O     m     => true
 | (S n) O     => false
 | (S n) (S m) => (inf_egal n m).

Inductive Set list [X:Set] :=
   Nil  : (list X)
 | Cons : X -> (list X) -> (list X).

Recursive Definition cdr [X:Set] : (list X) -> (list X) :=
   (Nil X)       => (Nil X)
 | (Cons X x xs) => xs.

Recursive Definition length [X:Set] : (list X) -> nat :=
   (Nil X)       => O
 | (Cons X x xs) => (S (length X xs)).

Recursive Definition append [X:Set;ys:(list X)] : (list X) -> (list X) :=
   (Nil X)       => ys
 | (Cons X x xs) => (Cons X x (append X ys xs)).

(* == Proprietes d'un tri == *)

Recursive Definition sorted : (list nat) -> Prop :=
   (Nil nat)                   => True
 | (Cons nat n (Nil nat))      => True
 | (Cons nat n (Cons nat m x)) => 
       (eq bool (inf_egal n m) true) /\ (sorted (Cons nat m x)).

Recursive Definition nocc [n:nat] : (list nat) -> nat :=
   (Nil nat)      => O
 | (Cons nat m x) => (si nat (egal_nat n m) (S (nocc n x)) (nocc n x)).

(* == Un lemme utile == *)
(* -- Sur les booleens *)

Theorem  diff_true_false : ~true=false.
Proof.
Discriminate.
Qed.

(* -- Sur la conditionnelle *)
Theorem si_intro : 
	(X:Set)(P:X->Prop)(x,y:X)(b:bool)
        ((eq bool b true) -> (P x)) ->
        ((eq bool b false) -> (P y)) ->
        (P (si X b x y)).
Proof.
Induction b.

 Intros.
 Rewrite si_eq1.
 Apply H.
 Trivial.

 Intros.
 Rewrite si_eq2.
 Apply H0.
 Trivial.

Qed.

(* -- Sur la comparaison des entiers *)
Theorem  inf_false_true :
 (n,m:nat)(inf_egal n m)=false->(inf_egal m n)=true.
Proof.
Induction n.
 Intros.
 Apply False_ind.
 Apply diff_true_false.
 Assumption.
 Induction m.
  Intro.
  Trivial.
  Intros.
  Rewrite inf_egal_eq3.
  Apply H.
  Assumption.
Qed.

(* -- Sur le predicat "sorted" *)

Theorem sorted_cdr :
 (n:nat)(x:(list nat))(sorted (Cons nat n x)) -> (sorted x).
Proof.
Induction x.
 Intro. Unfold sorted. Trivial.
 Intro. Intro. Intro. 
 Rewrite sorted_eq3.
 Tauto.
Qed.
 
Theorem sorted_inf :
 (n,m:nat)(x:(list nat))(sorted (Cons nat n (Cons nat m x))) 
        -> ((inf_egal n m)=true).
Proof.
Intro. Intro. Intro.
Rewrite sorted_eq3.
Tauto.
Qed.

(* == Tri par insertion *)

Recursive Definition ins [n:nat] : (list nat) -> (list nat) :=
   (Nil nat)      => (Cons nat n (Nil nat))
 | (Cons nat m x) => (si (list nat)
	                 (inf_egal n m) 
	                 (Cons nat n (Cons nat m x))
	                 (Cons nat m (ins n x))).

Recursive Definition tri_ins : (list nat) -> (list nat) :=
   (Nil nat)      => (Nil nat)
 | (Cons nat n x) => (ins n (tri_ins x)).

(* -- Preuve de correction *)
(* -- 1 : Le resultat de "tri_ins" est une liste sorted *)

Theorem sorted_ins_Cons :
 (x:(list nat))(n,m:nat)((inf_egal n m)=false) ->
    (sorted (Cons nat m x)) ->
    (sorted (Cons nat m (ins n x))).
Proof.
Induction x.
 Intros.
 Rewrite ins_eq1.
 Rewrite sorted_eq3.
 Split.
  Apply inf_false_true. Assumption.
  Trivial.
 Intros.
 Rewrite ins_eq2.
 Apply (si_intro (list nat) ([x:(list nat)](sorted(Cons nat m x)))).
  Intro. 
  Rewrite sorted_eq3.
  Split.
   Apply inf_false_true. Assumption.
   Rewrite sorted_eq3.
   Split.
    Apply H2.
    Apply (sorted_cdr m). Assumption.
  Intro.
  Rewrite sorted_eq3.
  Split.
   Apply (sorted_inf m y l). Assumption.
   Apply H.
    Assumption.
    Apply (sorted_cdr m). Assumption.
Qed.

Theorem sorted_ins : 
  (n:nat)(x:(list nat))(sorted x)->(sorted (ins n x)).
Proof.
Induction x.
 Trivial.
 Intros.
 Rewrite ins_eq2.
 Apply si_intro.
  Intros.
  Rewrite sorted_eq3.
  Split. Assumption. Assumption.
  Intro.
  Apply sorted_ins_Cons. Assumption. Assumption.
Qed.

(* -- 2 : Le resultat de "tri_ins" est une permutation de son entree *)
(*   ie : meme nombre d'occurrences de chaque element                *)

Theorem nocc_Cons_Cons :
 (n,m,p:nat)(x:(list nat))
     (nocc n (Cons nat m (Cons nat p x))) =
     (nocc n (Cons nat p (Cons nat m x))).
Proof.
Intros.
Rewrite (nocc_eq2 n p (Cons nat m x)).
Apply si_intro.
 Intro.
 Rewrite nocc_eq2.
 Apply (si_intro nat ([p:nat]p=(S (nocc n (Cons nat m x))))).
  Intro.
  Rewrite nocc_eq2.
  Rewrite H.
  Rewrite si_eq1.
  Rewrite nocc_eq2.
  Rewrite H0.
  Rewrite si_eq1.
  Apply refl_equal.
  Intro.
  Rewrite nocc_eq2.
  Rewrite H.
  Rewrite si_eq1.
  Rewrite nocc_eq2.
  Rewrite H0.
  Rewrite si_eq2.
  Apply refl_equal.
 Intro.
 Rewrite (nocc_eq2 n m x).
 Apply si_intro.
  Intro.
  Rewrite nocc_eq2.
  Rewrite H0.
  Rewrite si_eq1.
  Rewrite nocc_eq2.
  Rewrite H.
  Rewrite si_eq2.
  Apply refl_equal.
  Intro.
  Rewrite nocc_eq2.
  Rewrite H0.
  Rewrite si_eq2.
  Rewrite nocc_eq2.
  Rewrite H.
  Rewrite si_eq2.
  Apply refl_equal.
Qed.

Theorem nocc_Cons :
 (n,m:nat)(x,y:(list nat))((nocc n x)=(nocc n y)) ->
   (nocc n (Cons nat m x))=(nocc n (Cons nat m y)).
Proof.
Intros.
Unfold nocc.
Apply si_intro.
 Intro.
 Rewrite H0.
 Simpl.
 Apply eq_S.
 Assumption.
 Intro.
 Rewrite H0.
 Simpl.
 Assumption.
Qed.

Theorem nocc_ins :
 (x:(list nat))(n,m:nat)
   (nocc n (ins m x))=(nocc n (Cons nat m x)).
Proof.
Induction x.
 Trivial.
 Intros.
 Rewrite ins_eq2.
 Apply (si_intro (list nat) 
                 [t:(list nat)]((nocc n t)=
                                (nocc n (Cons nat m (Cons nat y l))))).
  Trivial.
  Intro.
  Rewrite nocc_Cons_Cons.
  Apply nocc_Cons.
  Apply H.
Qed.

Theorem nocc_tri_ins :
   (x:(list nat))(n:nat)(nocc n x)=(nocc n (tri_ins x)).
Proof.
Induction x.
 Trivial.
 Intros.
 Rewrite tri_ins_eq2.
 Rewrite nocc_ins.
 Apply nocc_Cons.
 Apply H.
Qed.

(* == Tri bubble *)

Recursive Definition bubble : (list nat) -> (list nat) :=
   (Nil nat)                   => (Nil nat)
 | (Cons nat n (Nil nat))      => (Cons nat n (Nil nat))
 | (Cons nat n (Cons nat m x)) =>
    (si (list nat)
        (inf_egal n m)
        (Cons nat n (bubble (Cons nat m x)))
        (Cons nat m (bubble (Cons nat n x)))).

Recursive Definition bubble_sort0 : nat -> (list nat) -> (list nat) :=
   O x => x
 | (S n) x => (bubble_sort0 n (bubble x)).

Recursive Definition bubble_sort : (list nat) -> (list nat) :=
 x => (bubble_sort0 (length nat x) x).

(* == Tri fusion *)
Recursive Definition fusion : (list nat) -> (list nat) -> (list nat) :=
   (Nil nat) ms                      => ms
 | (Cons nat  n ns) (Nil nat)        => (Cons nat  n ns)
 | (Cons nat  n ns) (Cons nat  m ms) =>
     (si (list nat) 
	 (inf_egal n m)
	 (Cons nat  n (fusion ns (Cons nat  m ms)))
	 (Cons nat  m (fusion (Cons nat  n ns) ms))).

Recursive Definition tri_merge0 : nat -> (list (list nat)) -> (list nat) :=
   O     xss                                           => (Nil nat)
 | (S l) (Nil (list nat))                              => (Nil nat)
 | (S l) (Cons (list nat) ns (Nil (list nat)))         => ns
 | (S l) (Cons (list nat) ns (Cons (list nat) ms xss)) =>
    (fusion (fusion ns ms) (tri_merge0 l xss)).

Recursive Definition l2ll [X:Set] : (list X) -> (list (list X)) :=
   (Nil  X)      => (Nil (list X))
 | (Cons X x xs) => (Cons (list X) (Cons X x (Nil X)) (l2ll X xs)).

Recursive Definition tri_merge : (list nat) -> (list nat) :=
  l => (tri_merge0 (length nat l) (l2ll nat l)).

(* == Tri par abrbre binaire de recherche *)

Inductive Set arbin :=
   Fe : arbin
 | Br : nat -> arbin -> arbin -> arbin.

Recursive Definition abr2Ln : arbin -> (list nat) :=
   Fe           => (Nil nat)
 | (Br n a1 a2) => (append nat (abr2Ln a1) (Cons nat n (abr2Ln a2))).

Recursive Definition insabr : nat -> arbin -> arbin :=
   n Fe           => (Br n Fe Fe)
 | n (Br m a1 a2) =>
     (si arbin 
	 (inf_egal n m) 
	 (Br m (insabr n a1) a2) 
	 (Br m a1 (insabr n a2))).

Recursive Definition Ln2abr : (list nat) -> arbin :=
   (Nil  nat)      => Fe
 | (Cons nat n ns) => (insabr n (Ln2abr ns)).

Recursive Definition tri_abr : (list nat) -> (list nat) :=
  ns => (abr2Ln (Ln2abr ns)).

(* == Tri par tas *)

Recursive Definition Tas2Ln : arbin -> (list nat) :=
   Fe           => (Nil nat)
 | (Br n a1 a2) => (Cons nat n (fusion (Tas2Ln a1) (Tas2Ln a2))).

Recursive Definition insTas : nat -> arbin -> arbin :=
   n Fe           => (Br n Fe Fe)
 | n (Br m a1 a2) =>
     (si arbin
	 (inf_egal n m)
         (Br n a2 (insTas m a1))
         (Br m a2 (insTas n a1))). 

Recursive Definition Ln2Tas : (list nat) -> arbin :=
   (Nil  nat)      => Fe
 | (Cons nat n ns) => (insTas n (Ln2Tas ns)).

Recursive Definition tri_heap : (list nat) -> (list nat) :=
 l => (Tas2Ln (Ln2Tas l)).
