(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              ProPre_test.v                               *)
(****************************************************************************)

Require ProPre.


(* Inductive Set bool := true : bool | false : bool. *)
Recursive Definition ifbool [X:Set] : bool -> X -> X -> X :=
  true x y => x
| false x y => y
.

(* Inductive Set nat : O:nat | S:nat->nat. *)
Recursive Definition add_acc [m:nat] : nat -> nat :=
 O => m
|(S n) => (add_acc (S m) n).

Recursive Definition ack3 : nat -> nat -> nat -> nat :=
 O y O => (S y)
|O y (S z) => (ack3 O (S y) z)
|(S x) O z => (ack3 x (S O) (S z))
|(S x) (S y) z => (ack3 x (ack3 (S x) y (S z)) (S z))
.

Recursive Definition eq_nat : nat -> nat -> bool :=
  O O => true
| O (S m) => false
| (S n) O => false
| (S n) (S m) => (eq_nat n m)
.

Recursive Definition less_eq_nat : nat -> nat -> bool :=
  O m => true
 |(S n) O => false
 |(S n) (S m) => (less_eq_nat n m)
.

Inductive Set list [X:Set] :=
  Nil : (list X)
| Cons : X -> (list X) -> (list X)
.

Recursive Definition length [X:Set] : (list X) -> nat :=
  (Nil X) => O
| (Cons X x xs) => (S (length X xs))
.

Recursive Definition append [X:Set;ys:(list X)] : (list X) -> (list X) :=
  (Nil X) => ys
| (Cons X x xs) => (Cons X x (append X ys xs))
.

Recursive Definition it_list [X,Y:Set;b:X;f:X->Y->X] : (list Y) -> X :=
 (Nil Y) => b
|(Cons Y a x) => (it_list X Y (f b a) f x)
.

Recursive Definition list_it [X,Y:Set;b:Y;f:X->Y->Y] : (list X) -> Y :=
 (Nil X) => b
|(Cons X a x) => (f a (list_it X Y b f x))
.

Recursive Definition map [X,Y:Set;f:X->Y] : (list X) -> (list Y) :=
 (Nil X) => (Nil Y)
|(Cons X x xs) => (Cons Y (f x) (map X Y f xs))
.

Recursive Definition soml : (list nat) -> nat :=
 (Nil nat) => O
|(Cons nat O ns) => (soml ns)
|(Cons nat (S n) ns) => (S (soml (Cons nat n ns)))
.

Recursive Definition is_sorted : (list nat) -> Prop :=
  (Nil nat) => True
| (Cons nat n (Nil nat)) => True
| (Cons nat n (Cons nat m x))
  => (<bool>(less_eq_nat n m)=true) /\ (is_sorted (Cons nat m x))
.

Recursive Definition nocc [n:nat] : (list nat) -> nat :=
  (Nil nat) => O
| (Cons nat m x) => (ifbool nat (eq_nat n m) (S (nocc n x)) (nocc n x))
.

Recursive Definition Isfalse : bool -> Prop :=
  true => False
| false => True
.

Recursive Definition ins [n:nat] : (list nat) -> (list nat) :=
  (Nil nat) => (Cons nat n (Nil nat))
| (Cons nat m x)
  => (ifbool(list nat)
	(less_eq_nat n m) 
	(Cons nat n (Cons nat m x))
	(Cons nat m (ins n x)))
.

Recursive Definition ins_sort : (list nat) -> (list nat) :=
  (Nil nat) => (Nil nat)
 |(Cons nat n x) => (ins n (ins_sort x))
.

Recursive Definition bubble : (list nat) -> (list nat) :=
  (Nil nat) => (Nil nat)
| (Cons nat n (Nil nat)) => (Cons nat n (Nil nat))
| (Cons nat n (Cons nat m x))
  => (ifbool(list nat)
        (less_eq_nat n m)
        (Cons nat n (bubble (Cons nat m x)))
        (Cons nat m (bubble (Cons nat n x))))
.

Recursive Definition bubble_sort_rec : nat -> (list nat) -> (list nat) :=
  O x => x
| (S n) x => (bubble_sort_rec n (bubble x))
.

Recursive Definition bubble_sort : (list nat) -> (list nat) :=
  x => (bubble_sort_rec (length nat x) x)
.

Recursive Definition pseudo_bubble_sort_rec [zs:(list nat)] :
  (list nat) -> (list nat) -> (list nat) :=
 (Cons nat x xs) (Cons nat y ys)
 => (ifbool (list nat) (less_eq_nat x y)
            (pseudo_bubble_sort_rec (Cons nat y zs) (Cons nat x xs) ys)
            (pseudo_bubble_sort_rec (Cons nat x zs) (Cons nat y xs) ys))
|(Cons nat x xs) (Nil nat)
 => (Cons nat x (pseudo_bubble_sort_rec (Nil nat) xs zs))
|(Nil nat) ys => ys.

Definition pseudo_bubble_sort := 
 [xs:(list nat)](pseudo_bubble_sort_rec (Nil nat) xs xs).

Recursive Definition merge : (list nat) -> (list nat) -> (list nat) :=
  (Nil nat) ms => ms
| (Cons nat  n ns) (Nil nat) => (Cons nat  n ns)
| (Cons nat  n ns) (Cons nat  m ms)
  => (ifbool(list nat) 
	 (less_eq_nat n m)
	 (Cons nat  n (merge ns (Cons nat  m ms)))
	 (Cons nat  m (merge (Cons nat  n ns) ms)))
.

Recursive Definition merge_sort_rec : nat -> (list (list nat)) -> (list nat) :=
  O xss => (Nil nat)
| (S l) (Nil (list nat)) => (Nil nat)
| (S l) (Cons (list nat) ns (Nil (list nat))) => ns
| (S l) (Cons (list nat) ns (Cons (list nat) ms xss))
  => (merge (merge ns ms) (merge_sort_rec l xss))
.

Definition x2l := [X:Set][x:X](Cons X x (Nil X))
.

Definition l2ll : (X:Set)(list X) -> (list (list X)) := 
 [X:Set][xs:(list X)](map X (list X) (x2l X) xs)
.

Definition merge_sort : (list nat) -> (list nat) :=
  [l:(list nat)](merge_sort_rec (length nat l) (l2ll nat l))
.

Inductive Set bintree :=
 Lf : bintree
|Br : nat -> bintree -> bintree -> bintree
.

Recursive Definition bst2ln : bintree -> (list nat) :=
  Lf => (Nil nat)
 |(Br n a1 a2) => (append nat (bst2ln a1) (Cons nat n (bst2ln a2)))
.

Recursive Definition bst_ins : nat -> bintree -> bintree :=
  n Lf => (Br n Lf Lf)
 |n (Br m a1 a2) 
  => (ifbool bintree 
	 (less_eq_nat n m) 
	 (Br m (bst_ins n a1) a2) 
	 (Br m a1 (bst_ins n a2)))
.

Recursive Definition ln2bst : (list nat) -> bintree :=
  (Nil nat) => Lf
 |(Cons nat n ns) => (bst_ins n (ln2bst ns))
.

Definition bst_sort : (list nat) -> (list nat) :=
  [ns:(list nat)] (bst2ln (ln2bst ns))
.

Recursive Definition heap2ln : bintree -> (list nat) :=
  Lf => (Nil nat)
 |(Br n a1 a2) => (Cons nat n (merge (heap2ln a1) (heap2ln a2)))
.

Recursive Definition heap_ins : nat -> bintree -> bintree :=
  n Lf => (Br n Lf Lf)
 |n (Br m a1 a2)
  => (ifbool bintree
	 (less_eq_nat n m)
         (Br n a2 (heap_ins m a1))
         (Br m a2 (heap_ins n a1)))
. 

Recursive Definition ln2heap : (list nat) -> bintree :=
  (Nil nat) => Lf
 |(Cons nat n ns) => (heap_ins n (ln2heap ns))
.

Definition heap_sort : (list nat) -> (list nat) :=
 [l:(list nat)] (heap2ln (ln2heap l))
.
 



     




















