(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Marks.v                                  *)
(*                                                                          *)
(*                                Gerard Huet                               *)
(*                                                                          *)
(* Developed in V5.8  January 1993                                          *)
(* Ported    to V5.10 January 1995                                          *)
(****************************************************************************)

Require Arith.
Require Terms.
Require Reduction.
Require Redexes.
Require Test.

(* Translation from terms to redexes *)

Recursive Definition mark : lambda -> redexes :=
    (Ref n)   => (Var n)
  | (Abs M)   => (Fun (mark M))
  | (App M N) => (Ap false (mark M)(mark N)). 

(* Reverse translation : erasing the marks *)
Recursive Definition unmark : redexes -> lambda :=
    (Var n)    => (Ref n)
  | (Fun U)    => (Abs (unmark U))
  | (Ap b U V) => (App (unmark U)(unmark V)).

Lemma inverse : (M:lambda) M = (unmark (mark M)).
Proof.
Induction M; Simpl; Trivial; Induction 1; Trivial.
Induction 1; Trivial.
Qed.

Lemma comp_unmark_eq : (U,V:redexes)(comp U V) -> (unmark U)=(unmark V).
Proof.
Induction 1; Simpl; Trivial.
Induction 2; Trivial.
Induction 2; Induction 2; Trivial.
Qed.

(* The converse is true, but not needed in the rest of the development *)

