(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Simulation.v                               *)
(*                                                                          *)
(*                                Gerard Huet                               *)
(*                                                                          *)
(* Developed in V5.8  January 1993                                          *)
(* Ported    to V5.10 January 1995                                          *)
(****************************************************************************)

(* Reduction of a term by a set of redexes *)

Require Arith.
Require Terms.
Require Reduction.
Require Redexes.
Require Test.
Require Marks.
Require Substitution.
Require Residuals.

(* Commuting mark and subst *)

Lemma mark_lift_rec : (M:lambda)(n,k:nat)
      (lift_rec_r (mark M) k n)=(mark (lift_rec M k n)).
Proof.
Induction M; Simpl; Intros.
Elim (test k n); Simpl; Intros; Trivial.
Elim H; Trivial.
Elim H; Elim H0; Trivial.
Qed.

Lemma mark_lift : 
      (M:lambda)(n:nat)(lift_r n (mark M))=(mark (lift n M)).
Proof.
Unfold lift; Unfold lift_r; Intros; Apply mark_lift_rec.
Qed.

Lemma mark_subst_rec : (N,M:lambda)(n:nat)
      (subst_rec_r (mark M) (mark N) n)=(mark (subst_rec M N n)).
Proof.
Induction M; Simpl; Intros.
Unfold insert_Var insert_Ref.
Elim (compare n0 n); Intro H.
Elim H; Intro H'.
Simpl; Trivial.
Rewrite (mark_lift N n0); Trivial.
Simpl; Trivial.
Elim H; Trivial.
Elim H; Elim H0; Trivial.
Qed.

Lemma mark_subst : 
      (M,N:lambda)(subst_r (mark M) (mark N))=(mark (subst M N)).
Proof.
Unfold subst; Unfold subst_r; Intros; Apply mark_subst_rec.
Qed.

(* residuals simulates par_red1 *)

Lemma simulation : (M,M':lambda)(par_red1 M M')->(Ex [V:redexes]
                   (residuals (mark M) V (mark M'))).
Proof.
Induction 1; Simpl; Intros.
Elim H1; Intros V1 P1.
Elim H3; Intros V2 P2.
Exists (Ap true (Fun V1) V2).
Elim mark_subst; Auto.
Exists (Var n); Trivial.
Elim H1; Intros V1 P1.
Exists (Fun V1); Auto.
Elim H1; Intros V1 P1.
Elim H3; Intros V2 P2.
Exists (Ap false V1 V2); Auto.
Qed.

(* Commuting unmark and subst *)

Lemma unmark_lift_rec : (U:redexes)(n,k:nat)
      (lift_rec (unmark U) k n) = (unmark (lift_rec_r U k n)).
Proof.
Induction U; Simpl; Intros.
Elim (test k n); Trivial.
Elim H; Trivial.
Elim H; Elim H0; Trivial.
Qed.

Lemma unmark_lift : (U:redexes)(n:nat)
      (lift n (unmark U))=(unmark (lift_r n U)).
Proof.
Unfold lift; Unfold lift_r; Intros; Apply unmark_lift_rec.
Qed.

Lemma unmark_subst_rec : (V,U:redexes)(n:nat)
      (subst_rec (unmark U) (unmark V) n)=(unmark (subst_rec_r U V n)).
Proof.
Induction U; Simpl; Intros.
Unfold insert_Var insert_Ref.
Elim (compare n0 n); Intro H; Simpl; Trivial.
Elim H; Trivial.
Rewrite (unmark_lift V n0); Trivial.
Elim H; Trivial.
Elim H; Elim H0; Trivial.
Qed.

Lemma unmark_subst : (U,V:redexes)
     (subst (unmark U) (unmark V))=(unmark (subst_r U V)).
Proof.
Unfold subst; Unfold subst_r; Intros; Apply unmark_subst_rec.
Qed.

Lemma completeness : 
     (U,V,W:redexes)(residuals U V W) -> (par_red1 (unmark U) (unmark W)).
Proof.
Induction 1; Simpl; Auto.
Intros; Elim unmark_subst; Auto.
Qed.


(**************************************************)
(* Reduction of a lambda term by a set of redexes *)
(**************************************************)

Definition reduction := 
    [M:lambda][U:redexes][N:lambda](residuals (mark M) U (mark N)).

Lemma reduction_function : 
   (M,N,P:lambda)(U:redexes)(reduction M U N) -> (reduction M U P) -> N=P.
Proof.
Unfold reduction; Intros; Cut (comp (mark N) (mark P)).
Intro; Rewrite (inverse N); Rewrite (inverse P); Apply comp_unmark_eq; Trivial.
Apply mutual_residuals_comp with U (mark M) (mark M); Trivial.
Qed.
