(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)

(****************************************************************************)
(*                              Substitution.v                              *)
(*                                                                          *)
(*                                Gerard Huet                               *)
(*                                                                          *)
(****************************************************************************)
(* Developed in V5.8  January 1993                                          *)
(* Ported    to V5.10 January 1995                                          *)
(****************************************************************************)

Require Arith.
Require Terms.
Require Reduction.
Require Redexes.
Require Test.

(****************************)
(*  Substitution of redexes *)
(****************************)

(* Similar to lift_rec of Terms *)

Recursive Definition lift_rec_r : redexes -> nat -> nat -> redexes :=
   (Var i) k n    => (Var (relocate i k n))
 | (Fun M) k n    => (Fun (lift_rec_r M (S k) n))
 | (Ap b M N) k n => (Ap b (lift_rec_r M k n) (lift_rec_r N k n)).

Definition lift_r := [n:nat][N:redexes](lift_rec_r N O n).

Definition insert_Var := [N:redexes][i,k:nat]<redexes>Case (compare k i) of
           [C:{(gt i k)}+{k=i}]<redexes>Case C of
              (* k<i *) [H:(lt k i)](Var (pred i))
              (* k=i *) [H:k=i]     (lift_r k N) end
              (* k>i *) [H:(gt k i)](Var i) end.

(* Similar to subst_rec of Terms *)

Recursive Definition subst_rec_r : redexes -> redexes -> nat -> redexes :=
   (Var i) N k     => (insert_Var N i k)
 | (Fun M) N k     => (Fun (subst_rec_r M N (S k)))
 | (Ap b M M') N k => (Ap b (subst_rec_r M N k) (subst_rec_r M' N k)).

Definition subst_r := [N,M:redexes](subst_rec_r M N O).


(* Lifting lemmas *)

Lemma lift_le : (n,i,k:nat)(le k i)->
     (lift_rec_r (Var i) k n) = (Var (plus n i)).
Proof.
Simpl; Unfold relocate.
Intros; Elim (test k i); Intro P; Trivial.
Absurd (gt k i); Trivial.
Apply le_not_gt; Trivial.
Qed.

Lemma lift_gt : (n,i,k:nat)(gt k i)->
      (lift_rec_r (Var i) k n) = (Var i).
Proof.
Simpl; Unfold relocate.
Intros; Elim (test k i); Intro P; Trivial.
Absurd (gt k i); Trivial.
Apply le_not_gt; Trivial.
Qed.

Lemma lift1 : (U:redexes)(j,i,k:nat)
     (lift_rec_r (lift_rec_r U i j) (plus j i) k) =
     (lift_rec_r U i (plus j k)).
Proof.
Induction U; Simpl; Intros.
Unfold relocate; Elim (test i n); Simpl.
Elim (test (plus j i) (plus j n)); Simpl; Intros.
Elim plus_permute; Elim plus_assoc_l; Trivial.
Absurd (gt i n); Auto.
Apply simpl_gt_plus_l with j; Trivial.
Elim (test (plus j i) n); Simpl; Intros; Trivial.
Absurd (le i n); Auto.
Apply le_trans with (plus j i); Trivial.
Rewrite (plus_n_Sm j i); Elim H; Trivial.
Elim H; Elim H0; Trivial.
Qed.

Lemma lift_lift_rec : (U:redexes)(k,p,n,i:nat)(le i n)->
     (lift_rec_r (lift_rec_r U i p) (plus p n) k)=
     (lift_rec_r (lift_rec_r U n k) i p).
Proof.
Induction U; Simpl; Intros.
(* Var *)
Unfold relocate.
Elim (test n0 n); Elim (test i n); Simpl.
Elim (test (plus p n0) (plus p n)); Elim (test i (plus k n)); Simpl; Intros.
Rewrite plus_permute; Trivial.
Absurd (gt i n); Auto.
Apply gt_le_trans with (plus k n); Trivial.
Absurd  (gt n0 n); Auto.
Apply simpl_gt_plus_l with p; Trivial.
Absurd  (gt n0 n); Auto.
Apply simpl_gt_plus_l with p; Trivial.
Intros; Absurd (gt i n); Trivial.
Apply le_not_gt; Apply le_trans with n0; Trivial.
Intros; Elim (test (plus p n0) (plus p n)); Simpl; Intros; Trivial.
Absurd (gt n0 n); Trivial.
Apply le_not_gt; Apply simpl_le_plus_l with p; Trivial.
Intros; Elim (test (plus p n0) n); Simpl; Intros; Trivial.
Absurd (gt n0 n); Trivial.
Apply le_not_gt; Apply le_trans with (plus p n0); Trivial.
(* Fun *)
Rewrite (plus_n_Sm p n); Rewrite H; Trivial.
Elim (plus_n_Sm k n); Auto.
(* Ap *)
Rewrite H; Trivial.
Rewrite H0; Trivial.
Qed.

Lemma lift_lift : (U:redexes)(k,p,n:nat)
     (lift_rec_r (lift_r p U) (plus p n) k)=(lift_r p (lift_rec_r U n k)).
Proof.
Unfold lift_r; Intros; Apply lift_lift_rec; Trivial.
Qed.

Lemma liftrecO : (U:redexes)(n:nat)(lift_rec_r U n O)=U.
Proof.
Induction U; Simpl; Intros.
Unfold relocate; Elim (test n0 n); Trivial.
Rewrite H; Trivial.
Rewrite H; Rewrite H0; Trivial.
Qed.

Lemma liftO : (U:redexes)(lift_r O U)=U.
Proof.
Unfold lift_r; Intro U; Apply liftrecO.
Qed.

Lemma lift_rec_lift_rec : 
    (U:redexes)(n,p,k,i:nat)(le k (plus i n))->(le i k)->
    (lift_rec_r (lift_rec_r U i n) k p)=(lift_rec_r U i (plus p n)).
Proof.
Induction U; Simpl; Intros.
(* Var *)
Unfold relocate; Elim (test i n); Intro P.
Elim (test k (plus n0 n)); Intro Q.
Rewrite plus_assoc_r; Trivial.
Absurd (gt k (plus n0 n)); Trivial.
Apply le_not_gt; Apply le_trans with (plus i n0); Trivial.
Replace (plus i n0) with (plus n0 i); Auto; 
Apply le_reg_l; Trivial.
Elim (test k n); Intro Q; Trivial.
Absurd (gt i k).
Apply le_not_gt; Trivial.
Apply gt_le_trans with n; Trivial.
(* Fun *)
Rewrite H; Trivial; Simpl; Apply le_n_S; Trivial.
(* Ap *)
Rewrite H; Trivial; Rewrite H0; Trivial.
Qed.

Lemma lift_rec_lift : (U:redexes)(n,p,k,i:nat)(le k n)->
     (lift_rec_r (lift_r n U) k p)=(lift_r (plus p n) U).
Proof.
Unfold lift_r; Intros; Rewrite lift_rec_lift_rec; Trivial.
Qed.


(* The three cases of substitution of U for (Var n) *)

Lemma subst_eq : (U:redexes)(n:nat)(subst_rec_r (Var n) U n) = (lift_r n U).
Proof.
Simpl; Unfold insert_Var.
Intros; Elim (compare n n); Intro P.
Elim P; Intro Q; Simpl; Trivial.
Absurd (gt n n); Trivial.
Absurd (gt n n); Trivial.
Qed.

Lemma subst_gt : (U:redexes)(n,p:nat)(gt n p) ->
    (subst_rec_r (Var n) U p) = (Var (pred n)).
Proof.
Simpl; Unfold insert_Var.
Intros; Elim (compare p n); Intro P.
Elim P; Intro Q; Trivial.
Absurd (gt n p); Trivial; Rewrite Q; Trivial.
Absurd (gt n p); Auto.
Qed.

Lemma subst_lt : (U:redexes)(n,p:nat)(gt p n) ->
    (subst_rec_r (Var n) U p) = (Var n).
Proof.
Simpl; Unfold insert_Var.
Intros; Elim (compare p n); Intro P; Trivial.
Absurd (gt p n); Trivial; Elim P; Intro Q; Auto.
Rewrite Q; Trivial.
Qed.

(* Substitution lemma *)

Lemma lift_rec_subst_rec : (U,V:redexes)(k,p,n:nat)
     (lift_rec_r (subst_rec_r V U p) (plus p n) k)=
     (subst_rec_r (lift_rec_r V (S (plus p n)) k) (lift_rec_r U n k) p).
Proof.
Induction V.
(* 2 Fun *) 
2: Simpl; Intros; Replace (S (plus p n)) with (plus (S p) n); Trivial.
2: Elim H; Trivial.
(* 3 Ap *)
2: Simpl; Intros; Elim H; Elim H0; Trivial.
(* 1 Var *)
Intros; Simpl.
Unfold insert_Var relocate.
Elim (compare p n); Intro P.
(* 1.1  P : {(gt n p)}+{p=n} *)
Elim P; Intro P1.
(* 1.1.1 P1 : (gt n p) *)
Cut n=(S (pred n)).
2: Apply S_pred with p; Trivial.
Intro E; Elim (test (S (plus p n0)) n); Intro Q.
(* 1.1.1.1 Q : (le (S (plus p n0)) n) *)
Elim (compare p (plus k n)); Intro R.
(* 1.1.1.1.1 R : {(lt p (plus k n))}+{p=(plus k n)} *)
Elim R; Intro R1; Simpl.
(* 1.1.1.1.1.1 R1 : (lt p (plus k n)) *)
Unfold relocate.
Rewrite E; Simpl.
Elim (test (plus p n0) (pred n)); Intro S.
Elim (plus_n_Sm k (pred n)); Simpl; Trivial.
Absurd (gt (plus p n0) (pred n)); Trivial.
Apply le_not_gt; Apply le_S_n; Elim E; Trivial.
(* 1.1.1.1.1.2 R1 : p=(plus k n) *)
Absurd (gt n p); Trivial.
Apply le_not_gt; Rewrite R1; Trivial.
(* 1.1.1.1.2 R : (gt p (plus k n)) *)
Absurd (gt p n); Auto.
Apply gt_le_trans with (plus k n); Trivial.
(* 1.1.1.2 Q : (gt (S (plus p n0)) n) *)
Simpl; Unfold relocate.
Elim (compare p n); Intro R.
(* 1.1.1.2.1  R : {(lt p n)}+{p=n} *)
Elim R; Intro R1.
(* 1.1.1.2.1.1  R1 : (lt p n) *)
Elim (test (plus p n0) (pred n)); Intro C.
Absurd (gt (S (plus p n0)) n); Trivial.
Apply le_not_gt; Rewrite E; Auto.
Trivial.
(* 1.1.1.2.1.2  R1 : p=n *)
Absurd (gt n p); Trivial.
Rewrite R1; Trivial.
(* 1.1.1.2.2  R : (gt p n) *)
Absurd (gt n p); Auto.
(* 1.1.2 P1 : p=n *)
Rewrite P1.
Elim (test (S (plus n n0)) n); Intro Q.
(* 1.1.2.1  Q : (le (S (plus n n0)) n) *)
Absurd (le (S (plus n n0)) n); Auto.
(* 1.1.2.2  Q : (gt (S (plus n n0)) n) *)
Elim (compare n n); Intro R.
(* 1.1.2.2.1  R : {(lt n n)}+{n=n} *)
Elim R; Intro R1.
(* 1.1.2.2.1.1  R1 : (lt n n) *)
Absurd (lt n n); Trivial.
(* 1.1.2.2.1.2  R1 : n=n *)
Apply lift_lift; Trivial.
(* 1.1.2.2.2  R : (gt n n) *)
Absurd (gt n n); Trivial.
(* 1.2  P : (gt p n) *)
Elim (test (S (plus p n0)) n); Intro Q.
(* 1.2.1  Q : (le (S (plus p n0)) n) *)
Absurd (gt n p); Auto.
Apply gt_le_trans with (plus p n0); Auto.
(* 1.2.2  Q : (gt (S (plus p n0)) n) *)
Elim (compare p n); Intro R.
(* 1.2.2.1  R : {(lt p n)}+{p=n} *)
Elim R; Intro R1.
(* 1.2.2.1.1 R1 : (lt p n) *)
Absurd (gt p n); Auto.
(* 1.2.2.1.2 R1 : p=n *)
Absurd (gt p n); Trivial.
Rewrite R1; Trivial.
(* 1.2.2.2 R : (gt p n) *)
Apply lift_gt.
Apply le_gt_trans with p; Trivial.
Qed.

Lemma lift_subst : (U,V:redexes)(k,n:nat)
    (lift_rec_r (subst_r U V) n k) =
    (subst_r (lift_rec_r U n k) (lift_rec_r V (S n) k)).
Proof.
Unfold subst_r; Intros.
Replace (S n) with (S (plus O n)).
Elim lift_rec_subst_rec; Trivial.
Simpl; Trivial.
Qed.

Lemma subst_rec_lift_rec1 : (U,V:redexes)(n,p,k:nat)(le k n)->
       (subst_rec_r (lift_rec_r U k p) V (plus p n)) =
       (lift_rec_r (subst_rec_r U V n) k p).
Proof.
Induction U; Intros; Simpl.
2: Rewrite plus_n_Sm; Rewrite H; Trivial.
2: Apply le_n_S; Trivial.
2: Rewrite H; Trivial; Rewrite H0; Trivial.
Unfold relocate insert_Var; Elim (test k n); Intro P.
(* 1 P : (le k n) *)
Elim (compare n0 n); Elim (compare (plus p n0) (plus p n)); Intros Q R.
(* 1.1   Q : {(lt (plus p n0) (plus p n))}+{(plus p n0)=(plus p n)}
         R : {(lt n0 n)}+{n0=n} *)
Elim Q; Elim R; Intros R1 Q1.
Cut n=(S (pred n)).
2: Apply S_pred with n0; Trivial.
Intro E; Rewrite lift_le.
Rewrite E; Elim plus_n_Sm; Simpl; Trivial.
Apply le_trans with n0; Trivial.
Apply gt_S_le; Elim E; Trivial.
Absurd (gt n n0).
Rewrite R1; Trivial.
Apply simpl_gt_plus_l with p; Trivial.
Absurd (gt (plus p n) (plus p n0)); Auto.
Apply le_not_gt; Rewrite Q1; Trivial.
Rewrite lift_rec_lift; Trivial.
(* 1.2   Q : (gt (plus p n0) (plus p n))
         R : {(lt n0 n)}+{n0=n} *)
Absurd (gt n0 n).
Apply le_not_gt; Elim R; Intro S.
Apply gt_S_le; Apply gt_trans with n; Trivial. 
Rewrite S; Trivial.
Apply simpl_gt_plus_l with p; Trivial.
Elim Q; Intro S.
Absurd (gt n0 n); Trivial.
Apply gt_not_sym; Apply simpl_gt_plus_l with p; Trivial.
Absurd (gt n0 n); Trivial.
Apply le_not_gt; Replace n0 with n; Trivial.
Apply simpl_plus_l with p; Auto.
Rewrite lift_le; Trivial.
(* 2 P : (gt k n) *)
Elim (compare n0 n); Intro Q.
(* 2.1 Q : {(lt n0 n)}+{n0=n} *)
Absurd (gt n0 n).
Apply le_not_gt; Elim Q; Intro R.
Apply gt_S_le; Apply gt_trans with n; Trivial.
Rewrite R; Trivial.
Apply le_gt_trans with k; Trivial.
(* 2.2 Q : (gt n0 n) *)
Elim (compare (plus p n0) n); Intro R.
Absurd (gt n0 n); Trivial.
Apply le_not_gt; Elim R; Intro S.
Apply le_trans with (plus p n0); Trivial.
Apply gt_S_le; Apply gt_trans with n; Trivial.
Elim S; Trivial.
Rewrite lift_gt; Trivial.
Qed.

Lemma subst_rec_lift1 : (U,V:redexes)(n,p:nat)
       (subst_rec_r (lift_r p U) V (plus p n)) =
       (lift_r p (subst_rec_r U V n)).
Proof.
Unfold lift_r; Intros; Rewrite subst_rec_lift_rec1; Trivial.
Qed.


Lemma subst_rec_lift_rec : (U,V:redexes)(p,q,n:nat)
     (le q (plus p n)) -> (le n q) -> 
     (subst_rec_r (lift_rec_r U n (S p)) V q)=(lift_rec_r U n p).
Proof.
Induction U; Intros; Simpl.
2: Rewrite H; Trivial.
2: Elim plus_n_Sm; Simpl; Apply le_n_S; Trivial.
2: Apply le_n_S; Trivial.
2: Rewrite H; Trivial; Rewrite H0; Trivial.
Unfold insert_Var relocate; Simpl.
Elim (test n0 n); Intro P.
(* 1  P : (le n0 n) *)
Elim (compare q (S (plus p n))); Intro Q.
(* 1.1  Q : {(lt q (S (plus p n)))}+{q=(S (plus p n))} *)
Elim Q; Intro Q1; Simpl; Trivial.
(* 1.1.1  Q1 : q=(S (plus p n)) *)
Absurd (le q (plus p n0)); Trivial; Rewrite Q1; Auto.
(* 1.1.2  Q1 : (gt q (S (plus p n))) *)
Absurd (gt q (S (plus p n))); Trivial.
Apply le_not_gt; Apply le_trans with (plus p n); Trivial.
Apply le_trans with (plus p n0); Auto.
(* 2  P : (gt n0 n) *)
Elim (compare q n); Intro Q; Trivial.
(* 2.1 Q : {(lt n q)}+{q=n} *)
Elim Q; Intro Q1; Simpl; Trivial.
(* 2.1.1  Q1 : (lt n q) *)
Absurd (gt n0 q); Auto.
Apply gt_trans with n; Trivial.
(* 2.1.2  Q1 : q=n *)
Absurd (gt n0 q); Auto.
Rewrite Q1; Trivial.
Qed.

Lemma subst_rec_lift : (U,V:redexes)(p,q:nat)(le q p) ->
     (subst_rec_r (lift_r (S p) U) V q)=(lift_r p U).
Proof.
Unfold lift_r; Intros; Rewrite subst_rec_lift_rec; Trivial.
Elim plus_n_O; Trivial.
Qed.

(* subst_rec_subst_rec *)

Lemma subst_rec_subst_rec : (U,V,W:redexes)(n,p:nat)
       (subst_rec_r (subst_rec_r V U p) W (plus p n)) =
       (subst_rec_r (subst_rec_r V W (S (plus p n))) (subst_rec_r U W n) p).
Proof.
Induction V.
2: Simpl; Intros; Replace (S (plus p n)) with (plus (S p) n); Trivial.
2: Elim H; Trivial.
2: Simpl; Intros; Elim H; Elim H0; Trivial.
Unfold 2 4 subst_rec_r; Unfold insert_Var.
Intros n W i p; Elim (compare p n); Intro C.
(* 1.1  C : {(lt p n)}+{p=n} *)
Elim C; Intro D.
(* 1.1.1  D : (lt p n) *) 
Elim (compare (S (plus p i)) n); Intro P.
(* 1.1.1.1  P : {(lt (S (plus p i)) n)}+{(S (plus p i))=n} *)
Elim P; Intro P1.
(* 1.1.1.1.1  P1 : (lt (S (plus p i)) n) *)
Rewrite subst_gt; Auto. 
Rewrite subst_gt; Auto.
Apply gt_pred; Apply gt_le_trans with (S (plus p i)); Auto.
(* 1.1.1.1.2  P1 : (S (plus p i))=n *)
Rewrite (subst_rec_lift W (subst_rec_r U W i) (plus p i) p (le_plus_l p i)).
Replace (pred n) with (plus p i).
Apply subst_eq.
Elim P1; Simpl; Trivial.
(* 1.1.1.1  P : (gt (S (plus p i)) n) *)
Rewrite subst_lt; Trivial.
Rewrite subst_gt; Trivial.
Apply gt_S_n; Replace (S (pred n)) with n; Trivial.
Apply S_pred with p; Trivial.
(* 1.1.2 D : p=n *)
Rewrite D; Elim (compare (S (plus n i)) n); Intro P.
(* 1.1.2.1  P : {(lt (S (plus n i)) n)}+{(S (plus n i))=n} *)
Absurd (lt (plus n i) n); Auto.
Elim P; Intro Q.
Apply lt_trans with (S (plus n i)); Trivial.
Apply lt_S_n; Rewrite Q; Trivial.
(* 1.1.2.2  P : (gt (S (plus n i)) n) *)
Rewrite subst_eq.
Apply subst_rec_lift1.
(* 1.2  C : (gt p n) *)
Rewrite subst_lt.
Elim (compare (S (plus p i)) n); Intro P.
(* 1.2.1  P : {(lt (S (plus p i)) n)}+{(S (plus p i))=n} *)
Absurd (lt n p); Trivial.
Apply lt_not_sym; Elim P; Intro Q.
Apply lt_trans with (S (plus p i)); Auto.
Elim Q.
Apply le_lt_n_Sm; Apply le_plus_l.
(* 1.2.2  P : (gt (S (plus p i)) n) *)
Rewrite subst_lt; Trivial.
Apply le_gt_trans with p; Trivial.
Qed. 

Lemma subst_rec_subst_0 : (U,V,W:redexes)(n:nat)
   (subst_rec_r (subst_rec_r V U O) W n) =
   (subst_rec_r (subst_rec_r V W (S n)) (subst_rec_r U W n) O).
Proof.
Intros; Pattern 1 3 n.
Replace n with (plus O n).
Rewrite (subst_rec_subst_rec U V W n O); Trivial.
Simpl; Trivial.
Qed.

(**************************)
(* The Substitution Lemma *)
(**************************)

Lemma substitution : 
  (U,V,W:redexes)(n:nat)(subst_rec_r (subst_r U V) W n) =
                        (subst_r (subst_rec_r U W n) (subst_rec_r V W (S n))).
Proof.
Unfold subst_r; Intros; Apply subst_rec_subst_0; Trivial.
Qed.

(* Substitution preserves compatibility *)

Lemma lift_rec_preserve_comp : (U1,V1:redexes)(comp U1 V1) -> 
     (n,m:nat)(comp (lift_rec_r U1 m n) (lift_rec_r V1 m n)).
Proof.
Induction 1; Unfold lift_rec_r; Auto.
Qed.

Lemma subst_rec_preserve_comp : 
     (U1,V1,U2,V2:redexes)(comp U1 V1) -> (comp U2 V2) -> 
     (n:nat)(comp (subst_rec_r U1 U2 n) (subst_rec_r V1 V2 n)).
Proof.
Induction 1; Simpl; Auto.
Intros n C n0; Unfold insert_Var; Elim (compare n0 n); Trivial.
Induction y; Trivial.
Intro; Unfold lift_r; Apply lift_rec_preserve_comp; Trivial.
Qed.

Lemma subst_preserve_comp : 
    (U1,V1,U2,V2:redexes)(comp U1 V1) -> (comp U2 V2) -> 
    (comp (subst_r U2 U1) (subst_r V2 V1)).
Proof.
Intros; Unfold subst_r; Apply subst_rec_preserve_comp; Trivial.
Qed.

(* Substitution preserves regularity *)

Lemma lift_rec_preserve_regular : 
      (U:redexes)(regular U) -> (n,m:nat)(regular (lift_rec_r U m n)).
Proof.
Induction U; Unfold lift_rec_r; Simpl; Auto.
Induction b; Induction r; Try Contradiction.
Intros; Elim H2; Auto.
Intros; Elim H1; Auto.
Intros; Elim H2; Auto.
Intros; Elim H3; Auto.
Qed.

Lemma subst_rec_preserve_regular : 
  (U,V:redexes)(regular U) -> (regular V) -> 
  (n:nat)(regular (subst_rec_r U V n)).
Proof.
Intros U V; Elim U; Unfold subst_rec_r; Simpl; Auto.
Intros; Unfold insert_Var; Elim (compare n0 n).
Induction y; Simpl; Trivial.
Intro; Unfold lift_r; Apply lift_rec_preserve_regular; Trivial.
Simpl; Trivial.
Induction b; Induction r; Try Contradiction.
Intros; Elim H2; Auto.
Intros; Elim H1; Auto.
Intros; Elim H2; Auto.
Intros; Elim H3; Auto.
Qed.

Lemma subst_preserve_regular : 
      (U,V:redexes)(regular U) -> (regular V) -> (regular (subst_r U V)).
Proof.
Unfold subst_r; Intros; Apply subst_rec_preserve_regular; Trivial.
Qed.
