(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)

(****************************************************************************)
(*                                 Terms.v                                  *)
(*                                                                          *)
(*                                Gerard Huet                               *)
(*                                                                          *)
(* Developed in V5.8  January 1993                                          *)
(* Ported    to V5.10 January 1995                                          *)
(****************************************************************************)

Require Arith.
Require Test.

(* Lambda terms with de Bruijn's indexes *)

Inductive lambda : Set :=
    Ref : nat -> lambda
  | Abs : lambda -> lambda
  | App : lambda -> lambda -> lambda.


(* Lifting *)

Definition relocate := [i,k,n:nat]<nat>Case (test k i) of
        (* k<=i *) [H:(le k i)] (plus n i)
        (* k>i  *) [H:(gt k i)] i end.

Recursive Definition lift_rec : lambda -> nat -> nat -> lambda :=
   (Ref i) k n   => (Ref (relocate i k n))
 | (Abs M) k n   => (Abs (lift_rec M (S k) n))
 | (App M N) k n => (App (lift_rec M k n) (lift_rec N k n)).

Definition lift := [n:nat][N:lambda](lift_rec N O n).

(* Substitution *)

Definition insert_Ref := [N:lambda][i,k:nat]<lambda>Case (compare k i) of
           [C:{(gt i k)}+{k=i}]<lambda>Case C of
              (* k<i *) [H:(lt k i)](Ref (pred i))
              (* k=i *) [H:k=i]     (lift k N) end
              (* k>i *) [H:(gt k i)](Ref i) end.

Recursive Definition subst_rec : lambda -> lambda -> nat -> lambda :=
   (Ref i) N k    => (insert_Ref N i k)
 | (Abs M) N k    => (Abs (subst_rec M N (S k)))
 | (App M M') N k => (App (subst_rec M N k) (subst_rec M' N k)).

Definition subst := [N,M:lambda](subst_rec M N O).

