(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  INTSYNTAX.v                                                             *)
(*                                                                          *)
(*  LL1 grammar for arithmetic on integers (look at file                    *)
(*  NATURALS/NATSYNTAX.v for more details                                   *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                             INTSYNTAX.v                                  *)
(****************************************************************************)



(*
coqc -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "."  -I "$REA/NATURALS" -I "$REA/INTEGERS" INTSYNTAX
*)

Require intSyntax.
Require NATSYNTAX.

(* Declaration of non-terminal symbols *)
Grammar integer ident := [ prim:ident($id) ] -> [$0 = $id].
Grammar integer formula :=.
Grammar integer atom[$p] :=.
Grammar integer expr :=.
Grammar integer exprsum[$p] :=.
Grammar integer fact :=.
Grammar integer factprod[$p]:=.
Grammar integer final :=.
Grammar integer application[$p] :=.
Grammar integer lexpr[$p]:=.

(* LL1 Grammar for atomic formulas *)

Grammar integer formula :=
  [ expr($c) atom[[$0 = $c]]($r) ] -> [$0 = $r].

Token "<=".

Grammar integer atom[$p]:=
  [ ] -> [$0 = $p]
| [ "=" expr($c)] -> [$0 = <<(eq Z $p $c)>>]
| [ "<=" expr($c)] -> [$0 = <<(leZ $p $c)>>]
| [ "<" expr($c)] -> [$0 = <<(ltZ $p $c)>>]
| [ "<" ">" expr($c)] -> [$0 = <<~(eq Z $p $c)>>].

(* LL1 Grammar rules for arithmetic*)

Grammar integer expr :=
  [ fact($c) exprsum[[$0 = $c]]($r)] -> [$0 = $r].

Grammar integer exprsum[$p] := 
  [ ] -> [$0 = $p]
| [ "+" fact($c) exprsum[[$0 =<<(plusZ $p $c)>>]]($r)] -> [$0 = $r].

Grammar integer fact :=
  [ final($c) factprod[[$0 = $c]]($r) ] -> [$0 = $r].

Grammar integer factprod[$p] :=
  [ ] -> [$0 = $p]
| [ "*" final($c) factprod[[$0 =<<(multZ $p $c)>>]]($r)] -> [$0 = $r].

Token "-".

Grammar integer final :=
  [ ident($c) ] -> [$0 = ($VAR $c)]
| [ "(" expr($a) application[[$0 = $a]]($r) ] -> [$0 = $r]
| [ "{" natural:expr($c) "}" ] -> [$0 = <<(|($c,O)|z)>>]
| [ "-"  integer:expr($c)] -> [$0 = <<(unary_minus $c)>>]
| [ "[" command:command($c) "]" ] -> [$0 = $c]
| [ "|" integer:expr($c) "|" ] -> [$0 = <<(ABSZ $c)>>]
| [ "0" ] -> [$0 = <<(|(O,O)|z)>>]
| [ "1" ] -> [$0 = <<(|((S O),O)|z)>>].


Grammar integer lexpr[$p] :=
  [ ")" ] -> [$0 = $p]
| [ expr($c1)  lexpr[[$0 = <<($p $c1)>>]]($r) ] -> [$0 = $r].

Grammar integer application[$p] :=
 [ lexpr[[$0 = $p]]($r)] -> 
     [$0 = $r].


Syntax constr plusZ <<(plus $n1 $n2)>> 1
                 <$n1 : "left" : E> " + " <$n2 : "left" : E>  .

Syntax constr multZ <<(mult $n1 $n2)>> 1
                 <$n1 : "left" : E> "*" <$n2 : "left" : E>  .

Syntax constr unary_minus <<(unary_minus $n2)>> 1
                "-" <$n2 : "left" : E>  .

Syntax constr ABSZ <<(ABSZ $n)>> 1
                "|" <$n2 : "left" : E> "|" .

Syntax constr leZ <<(le $n1 $n2)>> 1
                 <$n1 : "left" : E> "<=" <$n2 : "left" : E>  .

Syntax constr ltZ <<(lt $n1 $n2)>> 1
                 <$n1 : "left" : E> "<" <$n2 : "left" : E>  .

Syntax constr fst <<(fst nat nat $c)>> 1
                 <$c : "left" : E> ".1".

Syntax constr snd <<(snd nat nat $c)>> 1
                 <$c : "left" : E> ".2".
