(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  MultZ.v                                                                 *)
(*                                                                          *)
(*  Basic properties of the multiplication on rational numbers              *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                              MultZ.v                                     *)
(****************************************************************************)




(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" MultZ
*)


Require AC.
Require HS.
Require quotient.
Require INTEGER.
Require PlusZ.

Section MultZ.

Require nat.
Require INTSYNTAX.
Require Int_in_com.
Require intSyntax.


Lemma multZ_sym : (n,m:Z) [| n*m = m*n |].

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Unfold multZ.
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (a1,a2)).
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (b1,b2)).
Unfold multZ_X;Simpl.
Repeat Rewrite -> Reduce.
Unfold multZ_XX;Simpl.
Repeat Rewrite -> Reduce.
Unfold multZ_XXX;Simpl.
Apply (From_R_to_L Z_typ Z_rel).
Red; Simpl.
Repeat Rewrite (mult_sym b1).
Repeat Rewrite (mult_sym b2).
ACNa.
Save.


Lemma multZ_assoc_l : (n,m,p:Z) [| n*(m*p) = (n*m)*p |].

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intro p; Pattern p; Apply (Closure_prop Z_typ Z_rel p).
Induction x; Intros c1 c2.
Unfold multZ;Simpl.
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (b1,b2)).
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (a1,a2)).
Unfold multZ_X;Simpl.
Repeat Rewrite (Reduce Z_typ Z_rel Z).
Unfold multZ_XX;Simpl.
Repeat Rewrite (Reduce Z_typ Z_rel Z).
Unfold multZ_XXX;Simpl.
Rewrite -> (Reduce Z_typ Z_rel Z->Z 
        [x:Z_typ]
          (lift Z_typ Z_rel Z 
           [y:Z_typ]
      | [! (x.1*y.1 + x.2*y.2 , x.1*y.2 + x.2*y.1) !] |z
            (Compat_multZ_XX x))
     Compat_multZ_X [! (a1*b1 + a2*b2 , a1*b2 + a2*b1) !]).
Repeat Rewrite (Reduce Z_typ Z_rel Z).
Simpl.
Apply (From_R_to_L Z_typ Z_rel).
Red; Simpl.
(* Apply distributivity to obtain normal form (a sum of products)       *)
Repeat Rewrite -> mult_plus_distr_l.
Repeat Rewrite -> mult_plus_distr.
ANm.
ANa.
(*         everything atom of the form a*b*c       *)
ACNa.
Save.

Lemma multZ_permute : (n,m,p:Z) [| n*(m*p)  =  m*(n*p) |].

(* almost exactly as previous lemma ! *)
Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intro p; Pattern p; Apply (Closure_prop Z_typ Z_rel p).
Induction x; Intros c1 c2.
Unfold multZ;Simpl.
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (b1,b2)).
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (a1,a2)).
Unfold multZ_X;Simpl.
Repeat Rewrite (Reduce Z_typ Z_rel Z).
Unfold multZ_XX;Simpl.
Repeat Rewrite (Reduce Z_typ Z_rel Z).
Unfold multZ_XXX;Simpl.
Apply (From_R_to_L Z_typ Z_rel).
Red; Simpl.
Repeat Rewrite -> mult_plus_distr_l.
Repeat Rewrite -> mult_plus_distr.
ANm.
ANa.
Repeat Elim (mult_permute a1).
Repeat Elim (mult_permute a2).
ACNa.
Save.


Lemma multZ_assoc_r : (n,m,p:Z)  [| (n*m)*p = n*(m*p) |].

Intros.
Elim multZ_sym.
Elim (multZ_sym [|n*m|]).
Elim (multZ_assoc_l n m p).
Auto.
Save.


Lemma multZ_O : (n:Z) [| n*0 = 0 |].

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Unfold multZ.
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (a1,a2)).
Unfold multZ_X.
Repeat Rewrite -> Reduce.
Unfold multZ_XX.
Unfold multZ_XXX;Simpl.
Repeat Elim (mult_sym O).
Simpl.
Auto.
Save.

(* Voici un lemme faux qu'on peut adapter avec n=/ 0 un de ces jours
Lemma multZ_mult_l  : (n,m,p:Z) [|  (n*m) = (n*p) |] ->  m = p.

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intro p; Pattern p; Apply (Closure_prop Z_typ Z_rel p).
Induction x; Intros c1 c2.
Unfold multZ.
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (a1,a2)).
Unfold multZ_X.
Repeat Rewrite -> Reduce.
Unfold multZ_XX.
Repeat Rewrite -> Reduce.
Unfold multZ_XXX;Simpl.
Intro HH.
Cut (Z_rel [!(a1 + b1,a2 + b2)!]  [!(a1 + c1,a2 + c2)!]).
Unfold Z_rel; Simpl.
Intro H'.
Apply (From_R_to_L Z_typ Z_rel).
Red;Simpl.
Apply (simpl_plus_l a2).
Apply (simpl_plus_l a1).
Rewrite (plus_permute a2).
Rewrite plus_assoc_l.
Rewrite H'.
ACNa.


Red;Simpl.
Apply (simpl_mult_l a2).
Apply (simpl_mult_l a1).
Rewrite (mult_permute a2).
Rewrite mult_assoc_l.
Rewrite H'.
ACNa.

Apply (axiom_quotient Z_typ  Z_rel).
Subtree proved!
Save.

*)


Lemma distribZ : (n,m,p:Z) [| n*(m + p) =  n*m + n*p |].

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intro p; Pattern p; Apply (Closure_prop Z_typ Z_rel p).
Induction x; Intros c1 c2.
Unfold multZ.
Rewrite (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (a1,a2)).
Unfold multZ_X.
Repeat Rewrite -> Reduce.
Unfold multZ_XX.
Repeat Rewrite -> Reduce.
Unfold multZ_XXX;Simpl.
Unfold plusZ.
Rewrite (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (b1,b2)).
Rewrite (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X
                      [! (a1*b1 + a2*b2 , a1*b2 + a2*b1) !]).
Unfold plusZ_X.
Repeat Rewrite -> Reduce.
Unfold plusZ_XX.
Repeat Rewrite -> Reduce.
Unfold plusZ_XXX;Simpl.
Apply (From_R_to_L Z_typ Z_rel).
Red;Simpl.
Repeat Rewrite -> mult_plus_distr_l.
ACNa.
Save.

Lemma distribZ_l : (n,m,p:Z) [| (m + p)*n = m*n + p*n |].

Intros.
Repeat Elim (multZ_sym n).
Apply distribZ.
Save.

(* TODO:

Lemma proof_plusQ : (n,m:Z)(Pos n)->(Pos m) ->(Pos n #z m).

Unfold Nonzero;Simpl.
Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intros.
Unfold multZ;Simpl.
Rewrite -> (Reduce Z_typ Z_rel Z->Z multZ_X Compat_multZ_X (a1,a2)).
Unfold multZ_X;Simpl.
Rewrite -> (Reduce Z_typ Z_rel).
Unfold multZ_XX;Simpl.
Unfold multZ_XXX;Simpl.

*)
Axiom mult_minus : (x,y:Z) [| (-x) * y = - (x * y) |].


Lemma sign_simpl : (x,y:Z) (x = y) -> [| -x = -y |].
Intros.Elim H.Auto.
Save.

End MultZ.
