(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  PlusZ.v                                                                 *)
(*                                                                          *)
(*  Basic properties of the addition on integers                            *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               PlusZ.v                                    *)
(****************************************************************************)


(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" PlusZ
*)

Require AC.
Require HS.
Require quotient.
Require INTEGER.


Section PlusZ.

Require nat.
Require INTSYNTAX.
Require Int_in_com.
Require intSyntax.

Lemma plusZ_sym : (n,m:Z) [| n + m  =  m + n |].

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Unfold plusZ .
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (a1,a2)).
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (b1,b2)).
Unfold plusZ_X;Simpl.
Repeat Rewrite -> Reduce.
Unfold plusZ_XX;Simpl.
Unfold plusZ_XXX;Simpl.
Apply (From_R_to_L Z_typ Z_rel).
Red; Simpl.
ACNa.
Save.

Lemma plusZ_assoc_l : (n,m,p:Z) [| n + (m + p) = (n + m) + p |].

(* almost identical proof as previous lemma *)
Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intro p; Pattern p; Apply (Closure_prop Z_typ Z_rel p).
Induction x; Intros c1 c2.
Unfold plusZ;Simpl.
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (b1,b2)).
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (a1,a2)).
Unfold plusZ_X;Simpl.
Repeat Rewrite -> (Reduce Z_typ Z_rel Z).
Unfold plusZ_XX;Simpl.
Repeat Rewrite -> (Reduce Z_typ Z_rel Z).
Unfold plusZ_XXX;Simpl.
Rewrite -> (Reduce Z_typ Z_rel Z->Z 
        [x:Z_typ]
          (lift Z_typ Z_rel Z [y:Z_typ]
                       |([! (x.1+y.1,x.2+y.2) !])|z
            (Compat_plusZ_XX x))
                 Compat_plusZ_X ([! (a1+b1,a2+b2) !])).
Repeat Rewrite -> (Reduce Z_typ Z_rel Z).
Simpl.
Apply (From_R_to_L Z_typ Z_rel).
Red; Simpl.
ACNa.
Save.

Lemma plusZ_permute : (n,m,p:Z) [| n + (m + p) = m + (n + p) |].

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intro p; Pattern p; Apply (Closure_prop Z_typ Z_rel p).
Induction x; Intros c1 c2.
Unfold plusZ;Simpl.
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (b1,b2)).
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (a1,a2)).
Unfold plusZ_X;Simpl.
Repeat Rewrite -> (Reduce Z_typ Z_rel Z).
Unfold plusZ_XX;Simpl.
Repeat Rewrite -> (Reduce Z_typ Z_rel Z).
Unfold plusZ_XXX;Simpl.
Apply (From_R_to_L Z_typ Z_rel).
Red; Simpl.
ACNa.
Save.

Lemma plusZ_assoc_r : (n,m,p:Z) [| (n + m) + p = n + (m + p) |].

Intros.
Elim plusZ_sym.
Elim (plusZ_sym [|n + m|]).
Rewrite <- (plusZ_assoc_l n m p).
Auto.
Save.


Lemma plusZ_plus_l  : (n,m,p:Z)   [| n + m = n + p |] ->  m = p.

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Intro m; Pattern m; Apply (Closure_prop Z_typ Z_rel m).
Induction x; Intros b1 b2.
Intro p; Pattern p; Apply (Closure_prop Z_typ Z_rel p).
Induction x; Intros c1 c2.
Unfold plusZ.
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (a1,a2)).
Unfold plusZ_X.
Repeat Rewrite -> Reduce.
Unfold plusZ_XX.
Repeat Rewrite -> Reduce.
Unfold plusZ_XXX;Simpl.
Intro HH.
Cut (Z_rel [!(a1 + b1,a2 + b2)!]  [!(a1 + c1,a2 + c2)!]).
Unfold Z_rel; Simpl.
Intro H'.
Apply (From_R_to_L Z_typ Z_rel).
Red;Simpl.
Apply (simpl_plus_l a2).
Apply (simpl_plus_l a1).
Rewrite (plus_permute a2).
Rewrite plus_assoc_l.
Rewrite H'.
ACNa.
Apply (From_L_to_R Z_typ Z_rel).
Exact HH.
Save.

Lemma plusZ_O : (n:Z) [| n + 0 = n |].

Intro n; Pattern n; Apply (Closure_prop Z_typ Z_rel n).
Induction x; Intros a1 a2.
Unfold plusZ.
Rewrite -> (Reduce Z_typ Z_rel Z->Z plusZ_X Compat_plusZ_X (a1,a2)).
Unfold plusZ_X.
Repeat Rewrite -> Reduce.
Unfold plusZ_XX.
Repeat Rewrite -> Reduce.
Unfold plusZ_XXX;Simpl.
Repeat Elim (plus_sym O).
Simpl.
Auto.
Save.

End PlusZ.


