(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  int.v                                                                   *)
(*                                                                          *)
(*  Summary of basic arihmetic on integers                                  *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               int.v                                      *)
(****************************************************************************)



(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" int
*)


Require quotient.
Require Export INTEGER.
Require Export PlusZ.
Require Export MultZ.
Require AC.
Require HS.


Section INT.

Require intSyntax.
Require NATSYNTAX.
Require INTSYNTAX.
Require Int_in_com.




(*******************************************************************
PROPERTIES OF Z
********************************************************************


plusZ_sym     : (n,m:Z)                       n + m = m + n
plusZ_assoc_l : (n,m,p:Z)              n + (m + p) = (n + m) + p
plusZ_permute : (n,m,p:Z)              n + (m + p) = m + (n + p)
plusZ_assoc_r : (n,m,p:Z)              (n + m) + p = n + (m + p)
plusZ_plus_l  : (n,m,p:Z)               n + m = n + p =>  m = p
plusZ_O       : (n:Z)                        n + 0 = n      

multZ_sym         : (n,m:Z)                   n*m = m*n
multZ_assoc_l     : (n,m,p:Z)             n*(m*p) = (n*m)*p
multZ_permute     : (n,m,p:Z)             n*(m*p) = m*(n*p)
multZ_assoc_r     : (n,m,p:Z)             (n*m)*p = n*(m*p)
multZ_O           : (n:Z)                    n*0 = 0

distribZ_l : (n,m,p:Z)           (n + m)*p = (n*p) + (m*p)
distribZ   : (n,m,p:Z)           p*(n + m) = (p*n) + (p*m).

*********************************************************************)

(* Tactiques de d'ecision AC pour l'addition sur Z.*)

Axiom plusZ_simpl_l : (n,m,p:Z) m = p ->  [| n + m = n + p |].

Axiom multZ_simpl_l : (n,m,p:Z) m = p -> [| n * m = n * p |].

End INT.

Grammar tactic simple_tactic :=
  ["CZa"] ->
  [$0 = <:tactic:<Intros; AC_OF eq plusZ plusZ_sym plusZ_permute; Auto>>].

Grammar tactic simple_tactic :=
  ["AZa"] ->
  [$0 = <:tactic:<Intros;Repeat Elim plusZ_assoc_l>>].

Grammar tactic simple_tactic :=
  ["ACZa"] ->
  [$0 = <:tactic:<Intros;Repeat Elim plusZ_assoc_l;
                   AC_OF eq plusZ plusZ_sym plusZ_permute; Auto>>].

Grammar tactic simple_tactic :=
  ["HSZa"] ->
            [$0 = 
  <:tactic:<Intros; HS_OF eq plusZ plusZ_sym plusZ_permute plusZ_simpl_l>>].

(* Tactiques de d'ecision AC pour la multiplication sur Z.*)

Grammar tactic simple_tactic :=
  ["CZm"] ->
  [$0 = <:tactic:< Intros;
                   AC_OF eq multZ multZ_sym multZ_permute;
                   Auto>>].

Grammar tactic simple_tactic :=
  ["AZm"] ->
  [$0 = <:tactic:<Intros;
                  Repeat Elim multZ_assoc_l>>].

Grammar tactic simple_tactic :=
  ["ACZm"] ->
  [$0 = <:tactic:< Intros;
                   Repeat Elim multZ_assoc_l;
                   AC_OF eq multZ multZ_sym multZ_permute;
                   Auto>>].

Grammar tactic simple_tactic :=
  ["HSZm"] ->
  [$0 = <:tactic:<Intros;
                  HS_OF eq multZ multZ_sym multZ_permute multZ_simpl_u>>].
