(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  leZ.v                                                                   *)
(*                                                                          *)
(*  Definition of ``less or equal'' on integers                             *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               leZ.v                                      *)
(****************************************************************************)




(*
coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/SUBSETS"  leZ
*)

Require nat.
Require quotient.
Require INTEGER.
(*Require productSyntax.*)


Section LEZ.
Require INTSYNTAX.
Require Int_in_com.

Definition leZ_XX
           : Z_typ -> Z_typ -> Prop
           := [y,x:Z_typ] [! x.1 + y.2 <= y.1 + x.2 !].

Require Le.

Lemma le_with_plus1:
    (p,n,m:nat) [! n <= m !] -> [! p+n <= p+m !].
Auto.
Save.

Lemma le_with_plus2:
    (p,n,m:nat) [! p+n <= p+m !] -> [! n <= m !].
Exact simpl_le_plus_l.
Save.

Lemma tech1 :
        (a1,a2,b1,b2,c1,c2:nat)    
      [! a1 + b2 = a2 + b1 !] ->
         [! c1 + a2 <= c2 + a1 !]->
            [! c1 + b2 <= c2 + b1 !].

Intros.
Apply (le_with_plus2 a1).
Elim (plus_permute c1).
Rewrite -> H.
Repeat Elim (plus_sym b1).
Repeat Elim (plus_permute b1).
Apply (le_with_plus1 b1).
Elim (plus_sym c2).
Auto.
Save.


Lemma Compat_leZ_XX:
      (x:Z_typ)(Compat_prop Z_typ Z_rel (leZ_XX x)).

Induction x; Intros c2 c1.
Red.
Induction x; Intros a2 a1.
Induction y; Intros b2 b1.
Unfold leZ_XX;Simpl.
Unfold Z_rel;Simpl.
Elim (plus_sym a1).
Elim (plus_sym c1).
Split.
Intro.
Elim (plus_sym c1).
Apply (tech1 a1 a2 b1 b2 c1 c2).
Elim H;Auto.
Auto.
Elim (plus_sym c1).
Intro.
Apply (tech1 b1 b2 a1 a2 c1 c2).
Elim (plus_sym a1).
Elim H.
Elim (plus_sym a2).
Auto.
Auto.
Save.

Definition leZ_X : Z -> Z_typ -> Prop
  := [z:Z][x:Z_typ] ((lift_prop Z_typ Z_rel (leZ_XX x) (Compat_leZ_XX x)) z).


Lemma Compat_leZ_X: (z:Z)(Compat_prop Z_typ Z_rel (leZ_X z)).

Red.
Induction x; Intros a1 a2.
Induction y; Intros b1 b2.
Unfold Z_rel; Simpl.
Apply (Closure_prop Z_typ Z_rel z).
Unfold leZ_X.
Induction x; Intros c1 c2.
Repeat Rewrite -> (Reduce_prop Z_typ Z_rel).
Unfold leZ_XX;Simpl.
Intro.
Repeat Elim (plus_sym c2).
Split.
Intro.
Apply (tech1 a1 a2 b1 b2 c1 c2).
Elim (plus_sym b1).
Elim H.
Auto.
Auto.
Intro.
Apply (tech1 b1 b2 a1 a2 c1 c2).
Elim H.
Elim (plus_sym b2).
Auto.
Auto.
Save.

Definition leZ : Z -> Z -> Prop
    := [z:Z](lift_prop Z_typ Z_rel (leZ_X z) (Compat_leZ_X z)).


Definition ltZ : Z -> Z -> Prop
   := [z1,z2:Z] [| z1 <= z2 |] /\ ~(z1 = z2).


End LEZ.