(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  leZ.v                                                                   *)
(*                                                                          *)
(*  Basic properties of ``less or equal'' on integers                       *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               leZ.v                                      *)
(****************************************************************************)


(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" leZproperties
*)

Require leZ.
Require int.
Require quotient.
Require Arith.

Section leZproperties.

Require nat.
Require intSyntax.
Require Int_in_com.
Require INTSYNTAX.


Lemma Z_partition:
   (z:Z) [| z <=  - 1 |] \/ 
           [| z = 0 |]   \/
         [| 1 <= z |].

Intro.
Unfold unary_minus;Simpl.
Unfold swap;Simpl.
Rewrite Reduce;Simpl.
Apply (Closure_prop Z_typ Z_rel z).
Induction x.
Intros.
Elim (le_or_lt y y0);Intro.
Elim (le_lt_or_eq y y0 H);Intro.
Left.
Red.
Simpl.
Unfold leZ_X;Unfold leZ_XX.
Repeat Rewrite (Reduce_prop Z_typ Z_rel).
Simpl.
Elim plus_sym;Simpl.
Exact (lt_le_S y y0 H0).
Right.Left.
Elim H0.
Apply  (From_R_to_L Z_typ Z_rel).
Red;Simpl.
Elim plus_sym;Simpl;Auto.
Right.Right.
Red.
Simpl.
Unfold leZ_X;Unfold leZ_XX.
Repeat Rewrite (Reduce_prop Z_typ Z_rel).
Simpl.
Elim plus_sym;Simpl.
Exact (lt_le_S y0 y H).
Save.


Require Compare_dec.

Lemma Constructive_Z_partition:
  (z:Z) {[| z <=  - 1 |]} + {[| z = 0 |]} + {[| 1 <= z |]}.

Intro.
Unfold unary_minus;Simpl.
Unfold swap;Simpl.
Rewrite Reduce;Simpl.
Apply (Closure Z_typ Z_rel z).
Induction x.
Intros.
Elim (le_lt_dec y y0);Intro.
Elim (le_lt_eq_dec y y0 y1);Intro.
Left.
Left.
Red.
Simpl.
Unfold leZ_X;Unfold leZ_XX.
Repeat Rewrite (Reduce_prop Z_typ Z_rel).
Simpl.
Elim plus_sym;Simpl.
Exact (lt_le_S y y0 y2).
Left.Right.
Elim y2.
Apply  (From_R_to_L Z_typ Z_rel).
Red;Simpl.
Elim plus_sym;Simpl;Auto.
Right.
Red.
Simpl.
Unfold leZ_X;Unfold leZ_XX.
Repeat Rewrite (Reduce_prop Z_typ Z_rel).
Simpl.
Elim plus_sym;Simpl.
Exact (lt_le_S y0 y y1).
Save.

Syntactic Definition H1 := [z:Z] [| z <= - 1 |].

Syntactic Definition H2 := [z:Z] [| z = 0 |].
 
Syntactic Definition H3 := [z:Z] [| 1 <= z |].


Lemma S1 : (z:Z)(H1 z)/\(H2 z) -> False.

Unfold unary_minus;Simpl.
Unfold swap;Simpl.
Rewrite Reduce;Simpl.
Simpl;Intro.
Apply (Closure_prop Z_typ Z_rel z).
Induction x.
Intros.
Elim H.
Unfold leZ.
Rewrite (Reduce_prop Z_typ Z_rel).
Unfold leZ_X;Unfold leZ_XX.
Repeat Rewrite (Reduce_prop Z_typ Z_rel).
Simpl.
Intros.
Cut (Z_rel (y,y0) (O,O)).
Unfold Z_rel;Simpl.
Elim plus_sym;Simpl.
Intro.
Cut [!  y + 1  <= y0 !].
Elim plus_sym.
Simpl.
Elim H5.
Intro.
Elim (le_Sn_n y).
Auto.
Auto.
Exact (From_L_to_R Z_typ Z_rel (y,y0) (O,O) H4).
Save.



Lemma S2 : (z:Z)(H2 z)/\(H3 z)-> False.

Unfold unary_minus;Simpl.
Unfold swap;Simpl.
Simpl;Intro.
Apply (Closure_prop Z_typ Z_rel z).
Induction x.
Intros.
Elim H.
Unfold leZ.
Rewrite (Reduce_prop Z_typ Z_rel).
Unfold leZ_X;Unfold leZ_XX.
Repeat Rewrite (Reduce_prop Z_typ Z_rel).
Simpl.
Intros.
Cut (Z_rel (y,y0) (O,O)).
Unfold Z_rel;Simpl.
Elim plus_sym;Simpl.
Intro.
Cut [!  1 + y0  <= y + 0 !].
Elim (plus_sym O).
Simpl.
Elim H5.
Intro.
Elim (le_Sn_n y).
Auto.
Auto.
Exact (From_L_to_R Z_typ Z_rel (y,y0) (O,O) H0).
Save.


Lemma le_false : (y,y0:nat) [! 1 + y0 <= y !] ->
                              [! 1 + y <= y0 !] ->
                                False.

Intros.
Cut [! y <= 1 + y !].
Intro.
Generalize (le_trans ? ? ? H H4).
Intro.
Generalize (le_trans ? ? ? H5 H0).
Simpl;Intro.
Elim (le_Sn_n y0 H6).
Simpl.
Apply le_n_Sn.
Save.


Lemma S3 : (z:Z)(H1 z)/\ (H3 z) -> False.

Unfold unary_minus;Simpl.
Unfold swap;Simpl.
Rewrite Reduce.
Simpl;Intro.
Apply (Closure_prop Z_typ Z_rel z).
Induction x.
Intros.
Elim H.
Unfold leZ.
Rewrite (Reduce_prop Z_typ Z_rel).
Unfold leZ_X;Unfold leZ_XX.
Repeat Rewrite (Reduce_prop Z_typ Z_rel).
Simpl.
Intros.
Generalize H0.
Elim plus_sym;Simpl.
Intro.
Generalize H4.
Elim plus_sym;Simpl.
Intro.
Apply (le_false y y0);Auto.
Save.


Inductive EX [A:Prop;P:A->Prop] : Prop 
    := EX_intro : (x:A)(P x)->(EX A P).


Lemma z_lt_O : (z:Z)(H1 z) ->
   (EX (H1 z)
         [h:(H1 z)]
    ((Constructive_Z_partition z) = 
       (inleft ({(H1 z)} + {(H2 z)}) (H3 z) (left (H1 z) (H2 z) h)))).

Intros.
Elim (Constructive_Z_partition z).
Induction y;Intros.
Exists y0.
Auto.
Elim (S1 z).
Split;Auto.
Intros.
Elim (S3 z).
Split;Auto.
Save.


Lemma z_eq_O : (z:Z)(H2 z) ->
   (EX (H2 z)
         [h:(H2 z)]
    ((Constructive_Z_partition z) = 
       (inleft ({(H1 z)} + {(H2 z)}) (H3 z) (right (H1 z) (H2 z) h)))).

Simpl.
Intros.
Elim (Constructive_Z_partition z).
Induction y;Intros.
Elim (S1 z).Split;Auto.
Exists y0.
Auto.
Intros.
Elim (S2 z).
Split;Auto.
Save.

Lemma O_lt_z : (z:Z)(H3 z) ->
   (EX (H3 z)
         [h:(H3 z)]
    ((Constructive_Z_partition z) = 
       (inright ({(H1 z)} + {(H2 z)}) (H3 z) h))).

Simpl.
Intros.
Elim (Constructive_Z_partition z).
Induction y;Intros.
Elim (S3 z).Split;Auto.
Elim (S2 z).
Split;Auto.
Intro.
Exists y.
Unfold unary_minus;Simpl.
Unfold swap;Simpl.
Rewrite Reduce;Simpl.
Auto.
Save.

End leZproperties.