(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  extensionality.v                                                        *)
(*                                                                          *)
(*  Functional extensionality can be deduced from the axioms on quotient    *)
(*  together with the xi-rule as shown in this file                         *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                            extensionality.v                              *)
(****************************************************************************)



(*   WARNING:
If the axiom Reduce on quotients 
(Axiom Reduce : (A:Set)(R:A->A->Prop)(B:Set)(f:A->B) 
      (c:(Compat A R B f))(a:A)
     (lift A R B f c (In A R a)) = (f a).)
was defined for DEFINITIONAL IDENTITY and not LEIBNIZ IDENTITY
then eta rule is sufficient to derive extensionality from the
axioms for quotient as this is mention by Jacobs or Hofmann
in his thesis
*)


Require quotient.
Require quoSyntax.


Section Extensionality.

Variable A:Set.
Variable B:Set.

Definition U:= A -> B.

Definition eq_ext := [f,g:U] ((x:A) (f x) = (g x)).

Definition E:= U/eq_ext.

Definition PX := [x:A][f:U](f x).

(* We now prove that for x:A, (P x) induces a function from E to B *)

Lemma Compat_PX :(x:A)(Compat U eq_ext B (PX x)).

Intro.
Unfold Compat.
Unfold eq_ext.
Unfold PX;Simpl.
Intros.
Exact (H x).
Save.

Definition P:=[x:A](lift U eq_ext B (PX x) (Compat_PX x)).

Definition Reform := [f:E][x:A](P x f).

(* A formulation of (xi)-rule in case eta is given *)
Axiom xi: (f,g:U)((x:A) (f x)=(g x)) -> f = g.

Lemma Rf_is_f : (f:U) (Reform (In U eq_ext f)) = f.

Intros.
Unfold Reform.
Unfold P.
Apply (xi [x:A](lift U eq_ext B (PX x) (Compat_PX x) (In U eq_ext f)) f).
(* Here if the axiom Reduce was defined for definitional identity,
eta reduction would be sufficient to conclude this lemma *)
Intro.
Rewrite Reduce.
Unfold PX.
Auto.
Save.

Lemma extensionality: (f,g:A->B)(eq_ext f g)-> f = g.

Intros.
Rewrite <- (Rf_is_f f).
Rewrite <- (Rf_is_f g).
Elim (From_R_to_L U eq_ext f g).
Auto.
Assumption.
Save.

End Extensionality.

