(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  MultQ.v                                                                 *)
(*                                                                          *)
(*  Basic properties of the multiplication on rational numbers              *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               MultQ.v                                    *)
(****************************************************************************)

(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" MultQ
*)


Require AC.
Require HS.
Require quotient.
Require subset.
Require RATIONAL.
Require PlusQ.

Section MultQ.

Require intnumbers.
Require ratSyntax.
Require RATSYNTAX.
Require Rat_in_com.


Lemma multQ_sym : (n,m:Q) [| n * m = m * n |].

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Unfold multQ .
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X x).
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X y).
Unfold multQ_X;Simpl.
Repeat Rewrite -> Reduce.
Unfold multQ_XX.
Unfold multQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red; Simpl.
Repeat Rewrite -> Out_In.
ACZm.
Save.


Lemma multQ_assoc_l : (n,m,p:Q) [| n * (m * p) = (n * m) * p |].

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Intro p; Pattern p; Apply (Closure_prop Q_typ Q_rel p).
Intro z.
Unfold multQ .
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X x).
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X y).
Unfold multQ_X;Simpl.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold multQ_XX.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Rewrite -> (Reduce 
         Q_typ Q_rel Q->Q
       [x0:Q_typ]
        (lift Q_typ Q_rel Q [y0:Q_typ]|(multQ_XXX x0 y0)|q
          (Compat_multQ_XX x0))
           Compat_multQ_X   (multQ_XXX x y))  .
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold multQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red; Simpl.
Repeat Rewrite -> Out_In.
ACZm.
Save.


Lemma multQ_permute : (n,m,p:Q) [| n * (m * p) = m * (n * p) |].

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Intro p; Pattern p; Apply (Closure_prop Q_typ Q_rel p).
Intro z.
Unfold multQ .
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X x).
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X y).
Unfold multQ_X;Simpl.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold multQ_XX.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold multQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red; Simpl.
Repeat Rewrite -> Out_In.
ACZm.
Save.


Lemma multQ_assoc_r : (n,m,p:Q)  [| (n * m) * p = n * (m * p) |].

Intros.
Elim multQ_sym.
Elim (multQ_sym [| n * m |]).
Elim (multQ_assoc_l n m p).
Auto.
Save.

(*  TODO Define Oq and prove:

Lemma multQ_O : (n:Q)  [| n * 0 = 0 |].

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Induction x; Intros a1 a2.
Unfold multQ.
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X (a1,a2)).
Unfold multQ_X.
Repeat Rewrite -> Reduce.
Unfold multQ_XX.
Unfold multQ_XXX;Simpl.
Repeat Elim (mult_sym O).
Simpl.
Auto.
Save.
*)



Lemma distribQ : (n,m,p:Q) [| n * (m + p) = n * m + n * p |].

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Intro p; Pattern p; Apply (Closure_prop Q_typ Q_rel p).
Intro z.
Unfold multQ .
Rewrite -> (Reduce Q_typ Q_rel Q->Q multQ_X Compat_multQ_X x).
Unfold multQ_X.
Repeat Rewrite -> Reduce.
Unfold multQ_XX.
Repeat Rewrite -> Reduce.
Unfold multQ_XXX;Simpl.
Unfold plusQ.
Rewrite (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X y).
Rewrite (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X
([! x.1 * y.1 !],
 (In_subset(Z)(Pos) [! x.2 * y.2 !] (plusQ_XXX_prelim x y)))).
Unfold plusQ_X.
Repeat Rewrite -> Reduce.
Unfold plusQ_XX.
Repeat Rewrite -> Reduce.
Unfold plusQ_XXX;Simpl.
Apply (From_R_to_L Q_typ Q_rel).
Red;Simpl.
Repeat Rewrite -> Out_In.
AZm.
Repeat Rewrite -> distribZ_l.
Repeat Rewrite -> distribZ.
AZm.
Rewrite (multZ_permute [!z.2!] [!x.2!]).
Rewrite (multZ_permute [!y.2!] [!x.2!]).
HSZa.
ACZm.
Save.



Lemma distribQ_l : (n,m,p:Q) [| (m + p) * n = m * n + p * n |].

Intros.
Repeat Elim (multQ_sym n).
Apply distribQ.
Save.



End MultQ.
