(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  PlusQ.v                                                                 *)
(*                                                                          *)
(*  Basic properties of the addition on rational numbers                    *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                               PlusQ.v                                    *)
(****************************************************************************)


(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" PlusQ
*)

(*Require Arith.*)
Require AC.
Require HS.
Require quotient.
Require subset.
Require RATIONAL.


Section plusQ.

Require ratSyntax.
Require productSyntax.
Require intnumbers.
Require RATSYNTAX.
Require Rat_in_com.


Lemma plusQ_sym : (n,m:Q) [| n + m = m + n |].

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Unfold plusQ .
Rewrite -> (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X x).
Rewrite -> (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X y).
Unfold plusQ_X;Simpl.
Repeat Rewrite -> Reduce.
Unfold plusQ_XX.
Unfold plusQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red; Simpl.
Repeat Rewrite -> Out_In.
Repeat Rewrite -> distribZ_l.
AZm.
Repeat Elim (multZ_sym  [! y.2 !]).
ACZa.
Save.



Lemma plusQ_assoc_l : (n,m,p:Q) [| n + (m + p) = (n + m) + p |].


Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Intro p; Pattern p; Apply (Closure_prop Q_typ Q_rel p).
Intro z.
Unfold plusQ .
Rewrite -> (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X x).
Rewrite -> (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X y).
Unfold plusQ_X;Simpl.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold plusQ_XX.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Rewrite -> (Reduce 
         Q_typ Q_rel Q->Q
       [x0:Q_typ]
        (lift Q_typ Q_rel Q [y0:Q_typ]|(plusQ_XXX x0 y0)|q
          (Compat_plusQ_XX x0))
           Compat_plusQ_X   (plusQ_XXX x y))  .
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold plusQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red; Simpl.
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
Repeat Rewrite -> distribZ_l.
AZm.
AZa.
HSZa.
Rewrite -> (multZ_permute [! z.2 !]).
HSZa.
ACZm.
Save.



Lemma plusQ_permute : (n,m,p:Q)[| n + (m + p) = m + (n + p) |].

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Intro p; Pattern p; Apply (Closure_prop Q_typ Q_rel p).
Intro z.
Unfold plusQ .
Rewrite -> (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X x).
Rewrite -> (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X y).
Unfold plusQ_X;Simpl.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold plusQ_XX.
Repeat Rewrite -> (Reduce Q_typ Q_rel Q).
Unfold plusQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red; Simpl.
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
Pattern 1 (Out_subset Z Pos
           (In_subset Z Pos
             (multZ (Out_subset Z Pos y^2) (Out_subset Z Pos z^2))
             (plusQ_XXX_prelim y z))).
Rewrite -> Out_In .
AZm.
Rewrite -> (multZ_permute [! x.2 !]).
Repeat Rewrite -> (multZ_sym [! x.2 !]).
Repeat Rewrite -> distribZ_l.
AZm.
AZa.
HSZa.
Rewrite -> (multZ_permute [! y.2 !]).
HSZa.
ACZm.
Save.


Lemma plusQ_assoc_r : (n,m,p:Q) [| (n + m) + p = n + (m + p) |].

Intros.
Elim plusQ_sym.
Elim (plusQ_sym [|n + m|]).
Rewrite <- (plusQ_assoc_l n m p).
Auto.
Save.

Axiom plusQ_plus_l  : (n,m,p:Q)  [| n + m = n + p |] ->  m = p.

(*
Lemma plusQ_plus_l  : (n,m,p:Q) [| n + m = n + p |] ->  m = p.

Intro n; Pattern n; Apply (Closure_prop Q_typ Q_rel n).
Intro x.
Intro m; Pattern m; Apply (Closure_prop Q_typ Q_rel m).
Intro y.
Intro p; Pattern p; Apply (Closure_prop Q_typ Q_rel p).
Intro z.
Unfold plusQ.
Rewrite -> (Reduce Q_typ Q_rel Q->Q plusQ_X Compat_plusQ_X x).
Unfold plusQ_X.
Repeat Rewrite -> Reduce.
Unfold plusQ_XX.
Repeat Rewrite -> Reduce.
Unfold plusQ_XXX;Simpl.
Intro HH.


Unfold Q_rel; Simpl.
Apply (From_R_to_L Q_typ Q_rel).
Red;Simpl.


Apply (simpl_plus_l a2).
Apply (simpl_plus_l a1).
Rewrite (plus_permute a2).
Rewrite plus_assoc_l.
Rewrite H'.
ACNa.
Apply (From_L_to_R Q_typ Q_rel).
Exact HH.
Save.
*)
End plusQ.