(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  RATIONAL.v                                                              *)
(*                                                                          *)
(*  In this we define rational numbers using quotient types and             *)
(*  subset types. We alos define addition and multiplication on             *)
(*  rational numbers                                                        *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                             RATIONAL.v                                   *)
(****************************************************************************)



(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" RATIONAL
*)


Require quotient.
Require subset.
Require HS.
Require AC.

Section Rationals.

Require intnumbers.
Require productSyntax.
Require Int_in_com.
Require ratSyntax.
Require RATSYNTAX.
Require quoSyntax.

Definition Pos : Z->Prop 
               := [z:Z]  [| 1 <= z |].

Definition Q_typ := Z * Z!Pos.

Definition Q_rel := [x,y:Q_typ] 
           [|  x.1 * y.2  =  y.1 * x.2 |].

Definition Q := Q_typ/Q_rel.


(*****************************************************************************
 plusQ : Q -> Q -> Q
*****************************************************************************)

Axiom plusQ_XXX_prelim : (x,y:Q_typ)
                          [| 1 <= x.2 * y.2 |] .

Definition plusQ_XXX := [x,y:Q_typ] 
     ([| x.1 * y.2 + y.1 * x.2 |],
        (%+ [| x.2 * y.2 |] (plusQ_XXX_prelim x y))).

Definition plusQ_XX : Q_typ -> Q_typ -> Q
                    := [x,y:Q_typ] |(plusQ_XXX x y)|q.

Lemma Compat_plusQ_XX: 
       (x:Q_typ) (Compat Q_typ Q_rel Q (plusQ_XX x)).

Red.
Unfold  Q_rel.
Unfold plusQ_XX.
Unfold plusQ_XXX.
Intros x x0 y H.
Apply (From_R_to_L Q_typ Q_rel).
Red;Simpl.
Rewrite -> Out_In.
Rewrite -> Out_In.
Repeat Rewrite -> distribZ_l.
AZm.
Elim (multZ_sym [| y.2 |]).
Repeat Elim (multZ_permute  [| y.2 |]).
Repeat Elim (multZ_permute [| x0.1|]).
Rewrite -> (multZ_assoc_l [|x0.1|]).
Rewrite -> H.
AZm.
Elim (multZ_sym [| x0.2 |]).
Repeat Elim (multZ_permute [| x0.2 |]).
HSZa.
Auto.
Save.


Definition plusQ_X : Q_typ -> Q -> Q
                   := [x:Q_typ]
              (lift Q_typ Q_rel Q (plusQ_XX x) (Compat_plusQ_XX x)).

Lemma Compat_plusQ_X: 
              (Compat Q_typ Q_rel (Q->Q) plusQ_X ).

Red.
Unfold  Q_rel.
Intros x1 y0 H0.
Apply Ext.
Unfold plusQ_X.
Intro q.
Pattern q.
Apply (Closure_prop Q_typ Q_rel q).
Intro.
Repeat Rewrite -> Reduce.
Unfold plusQ_XX;Simpl.
Unfold plusQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red;Simpl.
Repeat Rewrite -> Out_In.
Repeat Rewrite -> distribZ_l.
AZm.
Repeat Elim (multZ_permute [| y0.2 |]).
Repeat Elim (multZ_permute [| x1.1 |]).
Rewrite -> (multZ_assoc_l [| x1.1 |]).
Rewrite -> H0.
AZm.
Elim (multZ_permute [| x1.2 |]).
HSZa.
Repeat Elim (multZ_permute [| y0.1 |]).
AZm.
Repeat Elim (multZ_permute [| x1.2 |]).
Auto.
Save.

Definition plusQ : Q -> Q -> Q
                 := (lift Q_typ Q_rel (Q->Q) plusQ_X Compat_plusQ_X ).



(*****************************************************************************
 multQ : Q -> Q -> Q
*****************************************************************************)

Definition multQ_XXX := [x,y:Q_typ] 
  ([| x.1 * y.1 |] , (%+ [| x.2 * y.2 |] (plusQ_XXX_prelim x y))).

Definition multQ_XX : Q_typ -> Q_typ -> Q
                    := [x,y:Q_typ] |(multQ_XXX x y)|q.

Lemma Compat_multQ_XX: 
       (x:Q_typ) (Compat Q_typ Q_rel Q (multQ_XX x)).

Red.
Unfold  Q_rel.
Unfold multQ_XX.
Unfold multQ_XXX.
Intros x x2 y0 H0.
Apply (From_R_to_L Q_typ Q_rel).
Red;Simpl.
Repeat Rewrite -> Out_In.
AZm.
Elim (multZ_sym [| y0.2 |]).
Repeat Elim (multZ_permute [| y0.2 |]).
Repeat Elim (multZ_permute [| x2.1 |]).
Rewrite -> (multZ_assoc_l [| x2.1 |]).
Rewrite -> H0.
ACZm.
Save.


Definition multQ_X : Q_typ -> Q -> Q
                   := [x:Q_typ]
              (lift Q_typ Q_rel Q (multQ_XX x) (Compat_multQ_XX x)).

Lemma Compat_multQ_X: 
              (Compat Q_typ Q_rel (Q->Q) multQ_X ).

Red.
Unfold  Q_rel.
Intros x1 y0 H0.
Apply Ext.
Unfold multQ_X.
Intro q.
Pattern q.
Apply (Closure_prop Q_typ Q_rel q).
Intro.
Repeat Rewrite -> Reduce.
Unfold multQ_XX;Simpl.
Unfold multQ_XXX.
Apply (From_R_to_L Q_typ Q_rel).
Red;Simpl.
Repeat Rewrite -> Out_In.
AZm.
Elim (multZ_permute [| y0.2 |]).
Rewrite -> (multZ_assoc_l  [| x1.1 |]).
Rewrite -> H0.
ACZm.
Save.


Definition multQ : Q -> Q -> Q
                 := (lift Q_typ Q_rel (Q->Q) multQ_X Compat_multQ_X ).



End Rationals.
