(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  RATSYNTAX.v                                                             *)
(*                                                                          *)
(*  Like in NATSYNTAX.v and INTSYNTAX.v, we define an LL1 grammar for       *)
(*  arithmetic on rational numbers, some extensions of this grammar         *)
(*  are done further when we define Cauchy sequences for instance           *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                            RATSYNTAX.v                                   *)
(****************************************************************************)



(*

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" RATSYNTAX
*)

Require subset.
Require subsetSyntax.
Require productSyntax.
Require intSyntax.
Require NATSYNTAX.
Require INTSYNTAX.


(* Declaration of non-terminal symbols *)

Grammar rational ident := [ prim:ident($id) ] -> [$0 = $id].
Grammar rational formula :=.
Grammar rational atom[$p] :=.
Grammar rational expr :=.
Grammar rational exprsum[$p] :=.
Grammar rational fact :=.
Grammar rational factprod[$p]:=.
Grammar rational final :=.
Grammar rational application[$p] :=.
Grammar rational lexpr[$p]:=.

(* LL1 Grammar for atomic formulas *)

Grammar rational formula :=
  [ expr($c) atom[[$0 = $c]]($r) ] -> [$0 = $r].

Token "<=".

Grammar rational atom[$p]:=
  [ ] -> [$0 = $p]
| [ "=" expr($c)] -> [$0 = <<(eq Q $p $c)>>]
| [ "<=" expr($c)] -> [$0 = <<(leQ $p $c)>>]
| [ "<" expr($c)] -> [$0 = <<(ltQ $p $c)>>]
| [ "<" ">" expr($c)] -> [$0 = <<~(eq Q $p $c)>>].

(* LL1 Grammar rules for arithmetic*)

Grammar rational expr :=
  [ fact($c) exprsum[[$0 = $c]]($r)] -> [$0 = $r].

Token "-".

Grammar rational exprsum[$p] := 
  [ ] -> [$0 = $p]
| [ "+" fact($c) exprsum[[$0 =<<(plusQ $p $c)>>]]($r)] -> [$0 = $r]
| [ "-" fact($c) exprsum[[$0 =<<(binary_minusQ $p $c)>>]]($r)] -> [$0 = $r].

Grammar rational fact :=
  [ final($c) factprod[[$0 = $c]]($r) ] -> [$0 = $r].

Grammar rational factprod[$p] :=
  [ ] -> [$0 = $p]
| [ "*" final($c) factprod[[$0 =<<(multQ $p $c)>>]]($r)] -> [$0 = $r].

Grammar rational final :=
  [ ident($c) ] -> [$0 = ($VAR $c)]
| [ "(" expr($a) application[[$0 = $a]]($r) ] -> [$0 = $r]
| [ "{" integer:expr($c1) "/" integer:expr($c2) ";" command:command($P) "}"]
               -> [$0 = <<($c1,(%+ $c2 $P))>>]
| [ "{" integer:expr($c) "}" ] -> [$0 = <<(fromZ_to_Q $c)>>]
| [ "[" command:command($c) "]" ] -> [$0 = $c]
| [ "|" expr($c) "|" ] -> [$0 = <<(AbsQ $c)>>]
| [ "-" expr($a) ] -> [$0 =<<(unary_minusQ $a)>>].

(* Extension of integer grammar *)

Grammar integer proj[$c]:=.

Grammar integer final :=
  [ ident($c) proj[[$0=($VAR $c)]]($r) ] -> [$0 = $r].

Grammar integer proj[$c]:=
  [ "." "1"] -> [$0 = <<(fst Z (Z!Pos) $c)>>]
| [ "." "2"] -> [$0 = <<(Out_subset Z Pos (snd Z (Z!Pos) $c))>>].

(* End*)

Grammar rational lexpr[$p] :=
  [ ")" ] -> [$0 = $p]
| [ expr($c1)  lexpr[[$0 = <<($p $c1)>>]]($r) ] -> [$0 = $r].

Grammar rational application[$p] :=
  [ lexpr[[$0 = $p]]($r)] -> [$0 = $r]
| [ "," expr($c) ")"] -> [$0 = <<($p,$c)>>].

Syntax constr plusQ <<(plusQ $n1 $n2)>> 1
                 <$n1 : "left" : E> " + " <$n2 : "left" : E>  .

Syntax constr multQ <<(multQ $n1 $n2)>> 1
                 <$n1 : "left" : E> "*" <$n2 : "left" : E>  .

Syntax constr binary_minusQ <<(binary_minusQ $n1 $n2)>> 1
             <$n1 : "left" : E>  "-" <$n2 : "left" : E>  .

Syntax constr leQ <<(leQ $n1 $n2)>> 1
                 <$n1 : "left" : E> "<=" <$n2 : "left" : E>  .

Syntax constr ltQ <<(leQ $n1 $n2)>> 1
                 <$n1 : "left" : E> "<" <$n2 : "left" : E>  .

Syntax constr unary_minusQ <<(unary_minusQ $n2)>> 1
                "-" <$n2 : "left" : E>  .

Syntax constr from_Z_to_Q <<(from_Z_to_Q $n2)>> 1
                               <$n2 : "left" : E>  .

Syntax constr ABSQ <<(ABSZ $n)>> 1
                "|" <$n2 : "left" : E> "|" .

Syntax constr fst <<(fst nat nat $c)>> 1
                 <$c : "left" : E> ".1".

Syntax constr snd <<(snd nat nat $c)>> 1
                 <$c : "left" : E> ".2".
