(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*  rat.v                                                                   *)
(*                                                                          *)
(*  Summary of rational arithmetic                                          *)
(*                                                                          *)
(*                                                                          *)
(*  Samuel Boutin                                                           *)
(*  Coq V5.10                                                               *)
(*  June  1995                                                              *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                                rat.v                                     *)
(****************************************************************************)


(*
coq -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" -I "$REA/QUOTIENTS"

coqc  -I $COQ/theories/INIT -I "$QUO/Tactics/ML_struct" -I "$QUO/Tactics/AC" -I "$QUO/Tactics/HS" -I "$REA/NATURALS" -I "$REA/QUOTIENTS" -I "$REA/INTEGERS" -I "$REA/UTIL" -I "$REA/RATIONALS"  -I "$REA/SUBSETS" -I "$REA/ORDERING" rat
*)


Require quotient.
Require subset.
Require Export RATIONAL.
Require Export PlusQ.
Require Export MultQ.
Require AC.
Require HS.

Section rat.

Require intnumbers.
Require ratSyntax.
Require productSyntax.
Require RATSYNTAX.
Require Rat_in_com.


(*******************************************************************
PROPERTIES OF Q
********************************************************************

plusQ_sym     : (n,m:Q)                       n + m = m + n
plusQ_assoc_l : (n,m,p:Q)              n + (m + p) = (n + m) + p
plusQ_permute : (n,m,p:Q)              n + (m + p) = m + (n + p)
plusQ_assoc_r : (n,m,p:Q)              (n + m) + p = n + (m + p)
plusQ_plus_l  : (n,m,p:Q)               n + m = n + p =>  m = p

multQ_sym         : (n,m:Q)                   n*m = m*n
multQ_assoc_l     : (n,m,p:Q)             n*(m*p) = (n*m)*p
multQ_permute     : (n,m,p:Q)             n*(m*p) = m*(n*p)
multQ_assoc_r     : (n,m,p:Q)             (n*m)*p = n*(m*p)

distribQ_l : (n,m,p:Q)           (n + m)*p = (n*p) + (m*p)
distribQ   : (n,m,p:Q)           p*(n + m) = (p*n) + (p*m).

*********************************************************************)

(* Tactiques de d'ecision AC pour l'addition sur Q.*)

Lemma  plusQ_simpl_l : (n,m,p:Q)  (m = p) -> [|  n + m = n + p |].
Intros.Elim H.Auto.Save.

Lemma  multQ_simpl_l : (n,m,p:Q) m = p -> [| n * m = n * p |].
Intros.Elim H.Auto.Save.

End rat.

Grammar tactic simple_tactic :=
  ["CQa"] ->
  [$0 = <:tactic:<Intros; AC_OF eq plusQ plusQ_sym plusQ_permute; Auto>>].

Grammar tactic simple_tactic :=
  ["AQa"] ->
  [$0 = <:tactic:<Intros;Repeat Elim plusQ_assoc_l>>].

Grammar tactic simple_tactic :=
  ["ACQa"] ->
  [$0 = <:tactic:<Intros;Repeat Elim plusQ_assoc_l;
                   AC_OF eq plusQ plusQ_sym plusQ_permute; Auto>>].

Grammar tactic simple_tactic :=
  ["HSQa"] ->
            [$0 = 
  <:tactic:<Intros; HS_OF eq plusQ plusQ_sym plusQ_permute plusQ_simpl_l>>].

(* Tactiques de d'ecision AC pour la multiplication sur Z.*)

Grammar tactic simple_tactic :=
  ["CQm"] ->
  [$0 = <:tactic:< Intros;
                   AC_OF eq multQ multQ_sym multQ_permute;
                   Auto>>].

Grammar tactic simple_tactic :=
  ["AQm"] ->
  [$0 = <:tactic:<Intros;
                  Repeat Elim multQ_assoc_l>>].

Grammar tactic simple_tactic :=
  ["ACQm"] ->
  [$0 = <:tactic:< Intros;
                   Repeat Elim multQ_assoc_l;
                   AC_OF eq multQ multQ_sym multQ_permute;
                   Auto>>].

Grammar tactic simple_tactic :=
  ["HSQm"] ->
  [$0 = <:tactic:<Intros;
                  HS_OF eq multQ multQ_sym multQ_permute multQ_simpl_u>>].
