(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                                                                          *)
(****************************************************************************)
(*                               Alternate.v                                *)
(****************************************************************************)
(* G. Huet - V5.8  Nov. 1994 *)
(*    ported V5.10 June 1995 *)

Require Bool "$COQTH/BOOL/Bool".
Require Words.

(*********************)
(* Alternating words *)
(*********************)

(* (alt b w) == w = [b ~b b ~b ...]  *)

Inductive alt : bool -> word -> Prop :=
  alt_empty   : (b:bool)(alt b empty)
| alt_bit     : (b:bool)(w:word)(alt (neg b) w)->(alt b (bit b w)).

Hint alt_empty alt_bit.

Lemma alt_neg_intro : (b,b':bool)(w:word)(alt b (bit b' w))->(alt (neg b) w).
Proof.
Intros b b' w al.
Inversion al; Trivial.
Qed.

Lemma alt_neg_elim : (b,b':bool)(w:word)(alt (neg b) (bit b' w))->(alt b w).
Proof.
Intros; Elim (neg_intro b); Apply alt_neg_intro with b'; Trivial.
Qed.

Lemma alt_eq : (b,b':bool)(w:word)(alt b (bit b' w))->b=b'.
Proof.
Intros b b' w al.
Inversion al; Trivial.
Qed.

Lemma alt_back : 
      (b,b':bool)(w:word)(alt b (bit b' w))->(b=b' /\ (alt (neg b) w)).
Proof.
Intros; Split.
Apply alt_eq with w; Trivial.
Apply alt_neg_intro with b'; Trivial.
Qed.

Inductive alternate [w:word] : Prop :=
  alter : (b:bool)(alt b w)->(alternate w).

(* (alternate w) iff Exists b  (alt b w)  *)

(*********************************************)
(* Subwords of alternate words are alternate *)
(*********************************************)

Lemma alt_conc_l : (u,v,w:word)(conc u v w)->(b:bool)(alt b w)->(alt b u).
Proof.
Induction 1; Auto; Intros.
Elim alt_back with b0 b w0.
Intros eq A.
Elim eq; Auto.
Trivial.
Qed.

Lemma alt_conc_r : (u,v,w:word)(conc u v w) -> (b:bool)(alt b w) ->
     (odd u) /\ (alt (neg b) v)
  \/ (even u) /\ (alt b v).
Proof.
Induction 1; Intros.
Right; Split; Intros; Auto.
Elim H1 with (neg b0).
Elim neg_elim with b0; Intro.
Right; Split; Elim H3; Auto.
Intro; Left; Split; Elim H3; Auto.
Apply alt_neg_intro with b; Trivial.
Qed.

Lemma alt_conc : (u,v,w:word)(conc u v w) -> (alternate w) ->
                 (alternate u) /\ (alternate v).
Proof.
Induction 2; Intros b ab; Split.
Apply alter with b; Apply alt_conc_l with v w; Trivial.
Elim alt_conc_r with u v w b; Intros; Trivial.
Elim H1; Intros; Apply alter with (neg b); Trivial.
Elim H1; Intros; Apply alter with b; Trivial.
Qed. (* unused in Shuffle *)


