(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                                                                          *)
(****************************************************************************)
(*                               Gilbreath.v                                 *)
(****************************************************************************)
(* G. Huet - V5.8  Nov. 1994 *)
(*    ported V5.10 June 1995 *)

Require Bool "$COQTH/BOOL/Bool".
Require Words.
Require Alternate.
Require Opposite.
Require Paired.
Require Shuffle.

(****************************)
(* The Gilbreath card trick *)
(****************************)

Section Context.

Variable x:word.
Hypothesis Even_x : (even x).
Variable b:bool. (* witness for (alternate x) *)
Hypothesis A : (alt b x).
Variables u,v:word.
Hypothesis C : (conc u v x).
Variable w:word.
Hypothesis S : (shuffle u v w).

Lemma Alt_u : (alt b u).
Proof.
Apply alt_conc_l with v x.
Apply C.
Apply A.
Qed.

Section Case1_.
Hypothesis Odd_u : (odd u).

Lemma Not_even_u : ~(even u).
Proof.
Red; Intro.
Elim not_odd_and_even with u; Trivial.
(*Apply Odd_u. *)
Qed.

Lemma Odd_v : (odd v).
Proof.
Elim even_conc with u v x.
Intro H; Elim H; Trivial.
Intro H; Elim H; Intro; Elim Not_even_u; Trivial.
Apply C.
Apply Even_x.
Qed.

Remark Alt_v : (alt (neg b) v).
Proof.
Elim alt_conc_r with u v x b.
Intro H; Elim H; Trivial.
Intro H; Elim H; Intro; Elim Not_even_u; Trivial.
Apply C.
Apply A.
Qed.

Lemma Opp_uv : (opposite u v).
Proof.
Apply alt_neg_opp with b.
Apply Odd_u.
Apply Alt_u.
Apply Odd_v.
Apply Alt_v.
Qed.

Lemma Case1 : (paired w).
Proof.
Elim Shuffling with u v w b.
Induction 1; Induction 2; Induction 1; Induction 2; Intros P1 P2.
Apply P1.
Apply Alt_v.
Elim not_odd_and_even with v; Trivial.
Apply Odd_v.
Induction 1; Intro; Elim Not_even_u; Trivial.
Apply S.
Apply Alt_u.
Qed.
End Case1_.

Section Case2_.
Hypothesis Even_u : (even u).

Lemma Not_odd_u : ~(odd u).
Proof.
Red; Intro; Elim not_odd_and_even with u; Trivial.
(*Apply Even_u.*)
Qed.

Lemma Even_v : (even v).
Proof.
Elim even_conc with u v x.
Intro H; Elim H; Intro; Elim Not_odd_u; Trivial.
Intro H; Elim H; Trivial.
Apply C.
Apply Even_x.
Qed.

Remark Alt_v : (alt b v).
Proof.
Elim alt_conc_r with u v x b.
Intro H; Elim H; Intro; Elim Not_odd_u; Trivial.
Intro H; Elim H; Trivial.
Apply C.
Apply A.
Qed.

Lemma Not_opp_uv : ~(opposite u v).
Proof.
Apply alt_not_opp with b.
Apply Alt_u.
Apply Alt_v.
Qed.

Lemma Case2 : (paired (rotate w)).
Proof.
Apply paired_rotate with b.
Elim Shuffling with u v w b.
Induction 1; Intro; Elim Not_odd_u; Trivial.
Induction 1; Induction 2.
Induction 1; Intros; Elim not_odd_and_even with v; Trivial.
Apply Even_v.
Induction 1; Induction 2; Intros P1 P2.
Apply P1.
Apply Alt_v.
Apply S.
Apply Alt_u.
Qed.

End Case2_.

(* We recall from the prelude the definition of the conditional :
Definition IF := [P,Q,R:Prop](P /\ Q) \/ ((~P) /\ R)
Syntax IF "if _ then _ else _" *)

Lemma Main : if (opposite u v) then (paired w) else (paired (rotate w)).
Proof.
Unfold IF; Elim odd_or_even with u; Intros.
(* (odd u) : Case 1 *)
Left; Split.
Apply Opp_uv; Trivial.
Apply Case1; Trivial.
(* (even u) : Case 2 *)
Right; Split.
Apply Not_opp_uv; Trivial.
Apply Case2; Trivial.
Qed.

End Context.


(*********************)
(* Gilbreath's trick *)
(*********************)


Theorem Gilbreath : (x:word)(even x)
  -> (alternate x)
  -> (u,v:word)(conc u v x)
  -> (w:word)(shuffle u v w)
  -> if (opposite u v) then (paired w) else (paired (rotate w)).

Proof.
Induction 2; Intros. (* Existential intro *)
Apply Main with x b; Trivial.
Qed.
