(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                                                                          *)
(****************************************************************************)
(*                               Opposite.v                                 *)
(****************************************************************************)
(* G. Huet - V5.8  Nov. 1994 *)
(*    ported V5.10 June 1995 *)

Require Bool "$COQTH/BOOL/Bool".
Require Words.
Require Alternate.


(************)
(* Opposite *)
(************)

Inductive opposite : word -> word -> Prop :=
  opp : (u,v:word)(b:bool)(opposite (bit b u) (bit (neg b) v)).

Hint opp.

Lemma not_opp_empty_r : (u:word)~(opposite u empty).
Proof.
Unfold not; Intros u op.
Inversion op.
Qed.

Lemma not_opp_empty_l : (u:word)~(opposite empty u).
Proof.
Unfold not; Intros u op.
Inversion op.
Qed.

Lemma not_opp_same : (u,v:word)(b:bool)~(opposite (bit b u) (bit b v)).
Proof.
Unfold not; Intros u v b op.
Inversion op.
Apply (no_fixpoint_neg b); Trivial.
Qed.

Lemma alt_neg_opp : (u,v:word)(b:bool)(odd u)->(alt b u)->
                (odd v)->(alt (neg b) v)->(opposite u v).
Proof.
Induction u.
Intros v b odd_empty;
Absurd (odd empty); Trivial.
Intros b u' H v; Elim v.
Intros b' H1 H2 odd_empty.
Absurd (odd empty); Trivial.
Intros b' v' H' b'' H1 H2 H3 H4.
Elim (alt_eq (neg b'') b' v'); Trivial.
Elim (alt_eq b'' b u'); Trivial.
Qed.

Lemma alt_not_opp : (u,v:word)(b:bool)(alt b u)->(alt b v)->~(opposite u v).
Proof.
Induction u.
Intros; Apply not_opp_empty_l.
Intros b u' H v; Elim v.
Intros; Apply not_opp_empty_r.
Intros b' v' H1 b'' H2 H3.
Elim (alt_eq b'' b' v'); Trivial.
Elim (alt_eq b'' b u'); Trivial.
Apply not_opp_same.
Qed.

