(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                                                                          *)
(****************************************************************************)
(*                                 Paired.v                                 *)
(****************************************************************************)
(* G. Huet - V5.8  Nov. 1994 *)
(*    ported V5.10 June 1995 *)

Require Bool "$COQTH/BOOL/Bool".
Require Words.

(****************)
(* Paired words *)
(****************)

(* (paired w) ==  w = [b1 ~b1 b2 ~b2 ... bn ~bn] *)
Inductive paired : word -> Prop :=
  paired_empty : (paired empty)
| paired_bit : (w:word)(paired w)->(b:bool)(paired (bit (neg b) (bit b w))).

(* paired_odd_l b w == w = [b b1 ~b1 b2 ~b2 ... bn ~bn] *)
Definition paired_odd_l := [b:bool][w:word](paired (bit (neg b) w)).

Lemma paired_odd_l_intro : 
      (b:bool)(w:word)(paired w)->(paired_odd_l b (bit b w)).
Proof.
Unfold paired_odd_l; Intros.
Apply paired_bit; Trivial.
Qed.

Lemma paired_odd_l_elim : 
      (b:bool)(w:word)(paired_odd_l (neg b) w)->(paired (bit b w)).
Proof.
Unfold paired_odd_l; Intros.
Elim (neg_intro b); Trivial.
Qed.

(* paired_odd_r b w == w = [b1 ~b1 b2 ~b2 ... bn ~bn ~b] *)
Definition paired_odd_r := [b:bool][w:word](paired (append w (single b))).

(* paired_rot b w == w = [b b2 ~b2 ... bn ~bn ~b] *)
Inductive paired_rot : bool -> word -> Prop :=
  paired_rot_empty : (b:bool)(paired_rot b empty)
| paired_rot_bit : (b:bool)(w:word)(paired_odd_r b w)
                                 ->(paired_rot b (bit b w)).

Lemma paired_odd_r_from_rot : 
      (w:word)(b:bool)(paired_rot b w)->(paired_odd_r b (bit (neg b) w)).
Proof.
Induction 1.
Intro; Unfold paired_odd_r; Simpl.
Unfold single; Apply paired_bit.
Apply paired_empty.
Intros b0 b' w'; Unfold paired_odd_r; Intros.
Simpl; Apply paired_bit; Auto.
Qed.

(* paired_bet b w == w = [b b2 ~b2 ... bn ~bn b] *)
Inductive paired_bet [b:bool] :  word -> Prop :=
  paired_bet_bit : (w:word)(paired_odd_r (neg b) w)->(paired_bet b (bit b w)).

Lemma paired_odd_r_from_bet : 
      (b:bool)(w:word)(paired_bet (neg b) w)->(paired_odd_r b (bit b w)).
Proof.
Intros b w pb.
Elim (neg_intro b).
Elim pb.
Unfold paired_odd_r. (* Unfolds twice *)
Intros; Simpl.
Apply paired_bit; Trivial.
Qed.


(************)
(* Rotation *)
(************)

Recursive Definition rotate : word -> word := 
    empty     => empty
  | (bit b w) => (append w (single b)).

Lemma paired_rotate : (w:word)(b:bool)(paired_rot b w)->(paired (rotate w)).
Proof.
Induction 1.
Intro; Simpl; Apply paired_empty.
Intros b' w'; Simpl.
Unfold paired_odd_r; Trivial.
Qed.

