(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                                                                          *)
(****************************************************************************)
(*                                Shuffle.v                                 *)
(****************************************************************************)
(* G. Huet - V5.8  Nov. 1994 *)
(*    ported V5.10 June 1995 *)

Require Bool "$COQTH/BOOL/Bool".
Require Words.
Require Alternate.
Require Opposite.
Require Paired.


(***********************)
(* Shuffling two words *)
(***********************)

Inductive shuffle : word -> word -> word -> Prop :=
  shuffle_empty :    (shuffle empty empty empty)
| shuffle_bit_left : (u,v,w:word)(shuffle u v w) ->
                            (b:bool)(shuffle (bit b u) v (bit b w))
| shuffle_bit_right : (u,v,w:word)(shuffle u v w) ->
                            (b:bool)(shuffle u (bit b v) (bit b w)).


(***********************)
(* The shuffling lemma *)
(***********************)

Lemma Shuffling : (u,v,w:word)(shuffle u v w) -> (b:bool)(alt b u) ->
  ( (odd u) /\ ( (odd v)  /\ ((alt (neg b) v) -> (paired w))
                          /\ ((alt b v)       -> (paired_bet b w))
              \/ (even v) /\ ((alt b v)       -> (paired_odd_l b w))
                          /\ ((alt (neg b) v) -> (paired_odd_r (neg b) w)))
 \/ (even u) /\ ( (odd v) /\ ((alt (neg b) v) -> (paired_odd_r b w))
                          /\ ((alt b v)       -> (paired_odd_l b w))
              \/ (even v) /\ ((alt b v)       -> (paired_rot b w))
                          /\ ((alt (neg b) v) -> (paired w)))).
Proof.
Induction 1; Intros.
(* 0. empty case *)
Right.
Split; Auto.
Right.
Split; Auto.
Split; Intro.
Apply paired_rot_empty.
Apply paired_empty.
(* 1. u before v *)
Elim (alt_eq b0 b u0); Trivial.
Elim (H1 (neg b0)); Intros.
(* 1.1. *) Right.
Elim H3; Intros.
Split; Auto.
Elim H5; Intros.
Elim H6; Intros.
(* 1.1.1. *) Left.
Elim H8; Intros. 
Split; Auto.
Split; Intro.
Apply paired_odd_r_from_bet; Auto.
Apply paired_odd_l_intro; Apply H9; Elim (neg_elim b0); Auto.
Elim H6; Intros.
Elim H8; Intros.
(* 1.1.2. *) Right.
Split; Auto.
Split; Intro. 
Apply paired_rot_bit; Elim (neg_intro b0); Apply H10; Elim (neg_elim b0); Auto.
Apply paired_odd_l_elim; Auto.
(* 1.2. *) Left.
Elim H3; Intros.
Split; Auto.
Elim H5; Intros.
(* 1.2.1. *) Left. 
Elim H6; Intros. 
Elim H8; Intros.
Split; Auto. 
Split; Intro. 
Apply paired_odd_l_elim; Auto.
Apply paired_bet_bit; Apply H9; Elim (neg_elim b0); Auto.
(* 1.2.2. *) Right.
Elim H6; Intros. 
Elim H8; Intros.
Split; Auto.
Split; Intro.
Apply paired_odd_l_intro; Apply H10; Elim (neg_elim b0); Auto.
Pattern 2 b0; Elim (neg_intro b0).
Apply paired_odd_r_from_rot; Auto.
Apply alt_neg_intro with b; Trivial.
(* 2. v before u *)
Elim (H1 b0); Intros.
(* 2.1. *) Left.
Elim H3; Intros.
Split; Auto.
Elim H5; Intros.
(* 2.1.1. *) Right.
Elim H6; Intros.
Elim H8; Intros.
Split; Auto.
Split; Intro.
Elim (alt_eq b0 b v0); Trivial.
Apply paired_odd_l_intro; Apply H9; Apply alt_neg_intro with b; Auto.
Elim (alt_eq (neg b0) b v0); Trivial.
Apply paired_odd_r_from_bet; Elim (neg_elim b0); Apply H10; Apply alt_neg_elim with b; Auto.
(* 2.1.2. *) Left.
Elim H6; Intros.
Elim H8; Intros.
Split; Auto.
Split; Intro.
Apply paired_odd_l_elim.
Elim (alt_eq (neg b0) b v0); Trivial.
Elim (neg_elim b0).
Apply H9.
Elim (neg_intro b0).
Apply alt_neg_intro with b; Auto.
Elim (alt_eq b0 b v0); Trivial.
Apply paired_bet_bit; Apply H10; Apply alt_neg_intro with b; Auto.
(* 2.2. *) Right.
Elim H3; Intros.
Split; Auto.
Elim H5; Intros.
(* 2.2.1. *) Right.
Elim H6; Intros.
Elim H8; Intros.
Split; Auto.
Split; Intro.
Elim (alt_eq b0 b v0); Trivial.
Apply paired_rot_bit; Apply H9; Apply alt_neg_intro with b; Auto.
Elim (alt_eq (neg b0) b v0); Trivial.
Apply paired_odd_l_elim.
Elim (neg_elim b0); Apply H10; Elim (neg_intro b0); Apply alt_neg_intro with b; Auto.
(* 2.2.2. *) Left.
Elim H6; Intros.
Elim H8; Intros.
Split; Auto.
Split; Intro.
Elim (alt_eq (neg b0) b v0); Trivial.
Apply paired_odd_r_from_rot; Apply H9; Elim (neg_intro b0); Apply alt_neg_intro with b; Auto.
Elim (alt_eq b0 b v0); Trivial.
Apply paired_odd_l_intro; Apply H10; Apply alt_neg_intro with b; Auto.
Trivial.
Qed.

