(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Pol1.v                                  *)
(****************************************************************************)


(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


       (* Preuve de terminaison: Polynome P1 *)

Require Le.
Require Lt.
Require Plus.
Require Gt.
Require Minus.
Require Mult.
Require TS. 
Require sigma_lift.  
Require comparith.

Definition e_P1: (b:wsort)(TS b)->nat := 
[b:wsort][U:(TS b)] <[b:wsort]nat>Match U with
  (* var *) [n:nat] (power2 (S n))
  (* app *) [a:terms][P1a:nat][b:terms][P1b:nat](plus P1a P1b)
  (* lam *) [a:terms][P1a:nat](plus P1a (S(S O)))
  (* env *) [a:terms][P1a:nat][s:sub_explicits][P1s:nat](mult P1a P1s)
  (*  id *) (S (S O)) 
  (*  |  *) (S (S O))
  (*  .  *) [a:terms][P1a:nat][s:sub_explicits][P1s:nat](plus P1a P1s)
  (*  o  *) [s:sub_explicits][P1s:nat][t:sub_explicits][P1t:nat]
            (mult P1s P1t)
  (*  || *) [s:sub_explicits][P1s:nat]P1s
  (*  X  *) [n:nat](S(S O))
  (*  x  *) [n:nat](S(S O))   end. 

Syntactic Definition P1 := (e_P1 ?).
Syntax constr P1 <<(e_P1 $1 $t1)>> 10
  [<hov 0> "P1 " <$t1:"term":L>].


Theorem gt_P1_1: (b:wsort)(M:(TS b))(gt (P1 M) (S O)).
Proof.
Induction M;Intros;Simpl;Auto.
(* var *)
Elim plus_n_O;Elim n;Simpl.
Auto.
Intros;Elim plus_n_O;Auto.
Qed.
Hint gt_P1_1.

Theorem P1_app: (M,N:terms)(reg_app M N)->((P1 M)=(P1 N)).
Proof. 
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_app.

Theorem P1_lambda: (M,N:terms)(reg_lambda M N)->(gt (P1 M) (P1 N)).
Proof. 
Induction 1;Intros;Simpl;Rewrite mult_plus_distr;Auto.
Qed.
Hint P1_lambda.

Theorem P1_clos: (M,N:terms)(reg_clos M N)->((P1 M)=(P1 N)).
Proof. 
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_clos.
 
Theorem P1_varshift1: (M,N:terms)(reg_varshift1 M N)->((P1 M)=(P1 N)).
Proof. 
Induction 1;Intros.
Change ((mult (power2 (S n)) (S(S O)))=(mult (S(S O)) (power2 (S n)))).
Auto.
Qed.
Hint P1_varshift1.

Theorem P1_varshift2: (M,N:terms)(reg_varshift2 M N)->((P1 M)=(P1 N)).
Proof. 
Induction 1;Intros.
Change ((mult (power2 (S n)) (mult (S(S O)) (P1 s)))=
            (mult (mult (S(S O)) (power2 (S n))) (P1 s))).
Elim mult_permut;Auto.
Qed.
Hint P1_varshift2.

Theorem P1_fvarcons: (M,N:terms)(reg_fvarcons M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros;Simpl;Elim plus_n_O;Auto.
Qed.
Hint P1_fvarcons.

Theorem P1_fvarlift1: (M,N:terms)(reg_fvarlift1 M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (S(S O)) (P1 s)) (S(S O))).
Auto.
Qed.
Hint P1_fvarlift1.

Theorem P1_fvarlift2: (M,N:terms)(reg_fvarlift2 M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (S(S O)) (mult (P1 s) (P1 t)))
           (mult (S(S O)) (P1 t))).
Auto.
Qed.
Hint P1_fvarlift2.

Theorem P1_rvarcons: (M,N:terms)(reg_rvarcons M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (mult (S(S O)) (power2 (S n))) (plus (P1 a) (P1 s)))
           (mult (power2 (S n)) (P1 s))).
Rewrite mult_plus_distr_r;Auto.
Qed.
Hint P1_rvarcons.

Theorem P1_rvarlift1: (M,N:terms)(reg_rvarlift1 M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros.
Change ((mult (mult (S(S O)) (power2 (S n))) (P1 s))=
            (mult (power2 (S n)) (mult (P1 s) (S(S O))))).
Elim mult_assoc_l;Elim (mult_permut (power2 (S n)) (S (S O)) (P1 s)).
Auto.
Qed.
Hint P1_rvarlift1.

Theorem P1_rvarlift2: (M,N:terms)(reg_rvarlift2 M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros.
Change ((mult (mult (S(S O)) (power2 (S n))) (mult (P1 s) (P1 t)))
           =(mult (power2 (S n)) (mult (P1 s) (mult (S(S O)) (P1 t))))).
Elim (mult_sym (power2 (S n)) (S(S O)));Elim mult_assoc_l;Auto.
Qed.
Hint P1_rvarlift2.

Theorem P1_assenv: (M,N:sub_explicits)(reg_assenv M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_assenv.

Theorem P1_mapenv: (M,N:sub_explicits)(reg_mapenv M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_mapenv.

Theorem P1_shiftcons: (M,N:sub_explicits)
                      (reg_shiftcons M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros;Simpl;Elim plus_n_O;Auto.
Qed.
Hint P1_shiftcons.

Theorem P1_shiftlift1: (M,N:sub_explicits)
                       (reg_shiftlift1 M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros;Simpl;Elim plus_n_O;Auto.
Qed.
Hint P1_shiftlift1.

Theorem P1_shiftlift2: (M,N:sub_explicits)(reg_shiftlift2 M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros;Simpl;Do 2 (Elim plus_n_O);Auto.
Qed.
Hint P1_shiftlift2.

Theorem P1_lift1: (M,N:sub_explicits)(reg_lift1 M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_lift1.

Theorem P1_lift2: (M,N:sub_explicits)(reg_lift2 M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_lift2.

Theorem P1_liftenv: (M,N:sub_explicits)(reg_liftenv M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros;Simpl;Rewrite mult_plus_distr_r;Auto.
Qed.
Hint P1_liftenv.

Theorem P1_idl: (M,N:sub_explicits)(reg_idl M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros;Simpl;Elim plus_n_O;Auto.
Qed.
Hint P1_idl.

Theorem P1_idr: (M,N:sub_explicits)(reg_idr M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_idr.

Theorem P1_liftid: (M,N:sub_explicits)(reg_liftid M N)->((P1 M)=(P1 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_liftid.

Theorem P1_id: (M,N:terms)(reg_id M N)->(gt (P1 M) (P1 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P1_id.

