(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Pol2.v                                  *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


           (* Preuve de terminaison: polynome P2 *)

Require Le.
Require Lt.
Require Plus.
Require Gt.
Require Minus.
Require Mult.
Require TS.  
Require sigma_lift.
Require comparith.

Definition e_P2: (b:wsort)(TS b)->nat:= 
[b:wsort][U:(TS b)] <[b:wsort]nat>Match U with
      (* var *) [n:nat](S O)
      (* app *) [a:terms][P2a:nat][b:terms][P2b:nat](S (plus P2a P2b))
      (* lam *) [a:terms][P2a:nat](mult (S(S O)) P2a)
      (* env *) [a:terms][P2a:nat][s:sub_explicits][P2s:nat](mult P2a (S P2s))
      (*  id *) (S O)
      (*  |  *) (S O)
      (*  .  *) [a:terms][P2a:nat][s:sub_explicits][P2s:nat](S (plus P2a P2s))
      (*  o  *) [s:sub_explicits][P2s:nat][t:sub_explicits][P2t:nat]
                (mult P2s (S P2t))
      (*  || *) [s:sub_explicits][P2s:nat](mult (S(S(S(S O)))) P2s)
      (*  X  *) [n:nat](S O)
      (*  x  *) [n:nat](S O)  end.


Syntactic Definition P2 := (e_P2 ?).
Syntax constr P2 <<(e_P2 $1 $t1)>> 10
  [<hov 0> "P2 " <$t1:"term":L>].

Theorem P2_pos : (b:wsort)(M:(TS b))(gt (P2 M) O).
Proof.
Induction M;Simpl;Intros;Auto.
Qed.
Hint P2_pos.

Theorem P2_app : (M,N:terms)(reg_app M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Elim mult_plus_distr;Auto.
Qed.
Hint P2_app.

Theorem P2_lambda : (M,N:terms)(reg_lambda M N)->(lt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (S(S O)) (mult (P2 a) (S (mult (S(S(S(S O)))) (P2 s)))))
           (mult (mult (S(S O)) (P2 a)) (S (P2 s)))).
Elim mult_assoc_r;Auto.
Qed.
Hint P2_lambda.

Theorem P2_clos : (M,N:terms)(reg_clos M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl.
Elim mult_assoc_l;Apply gt_mult_reg_l.
Auto.
Simpl;Auto.
Qed.
Hint P2_clos.

Theorem P2_varshift1 : (M,N:terms)(reg_varshift1 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Simpl;Auto.
Qed.
Hint P2_varshift1.

Theorem P2_varshift2 : (M,N:terms)(reg_varshift2 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Repeat (Elim plus_n_O);Auto.
Qed.
Hint P2_varshift2.

Theorem P2_fvarcons : (M,N:terms)(reg_fvarcons M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Elim plus_n_O;Auto.
Qed.
Hint P2_fvarcons.

Theorem P2_fvarlift1 : (M,N:terms)(reg_fvarlift1 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (S O) (S (mult (S(S(S(S O)))) (P2 s)))) (S O)).
Auto.
Qed.
Hint P2_fvarlift1.

Theorem P2_fvarlift2 : (M,N:terms)(reg_fvarlift2 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Repeat (Elim plus_n_O);Auto.
Qed.
Hint P2_fvarlift2.

Theorem P2_rvarcons : (M,N:terms)(reg_rvarcons M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Repeat (Elim plus_n_O);Auto.
Qed.
Hint P2_rvarcons.

Theorem P2_rvarlift1 : (M,N:terms)(reg_rvarlift1 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Repeat (Elim plus_n_O).
Elim mult_n_2;Auto.
Qed.
Hint P2_rvarlift1.

Theorem P2_rvarlift2 : (M,N:terms)(reg_rvarlift2 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (S O) (S (mult (mult (S(S(S(S O)))) (P2 s))
            (S (P2 t)))))
(mult (S O) (S (mult (P2 s) (S (mult (S O) (S (P2 t)))))))).
Unfold 1 mult;Unfold 3 mult;Unfold 4 mult;Repeat (Elim plus_n_O).
Apply gt_n_S;Repeat (Elim mult_n_Sm);Elim plus_assoc_l.
Apply gt_plus_plus.
Elim mult_assoc_l;Auto.
Elim mult_n_2;Elim mult_sym;Auto.
Qed.
Hint P2_rvarlift2.

Theorem P2_assenv : (M,N:sub_explicits)(reg_assenv M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl.
Rewrite mult_assoc_r;Simpl;Auto.
Qed.
Hint P2_assenv.

Theorem P2_mapenv : (M,N:sub_explicits)(reg_mapenv M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Elim mult_plus_distr;Auto.
Qed.
Hint P2_mapenv.

Theorem P2_shiftcons : (M,N:sub_explicits)(reg_shiftcons M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Elim plus_n_O;Auto.
Qed.
Hint P2_shiftcons.

Theorem P2_shiftlift1 : (M,N:sub_explicits)(reg_shiftlift1 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (S O) (S (mult (S(S(S(S O)))) (P2 s))))
           (mult (P2 s) (S(S O)))).
Unfold 1 mult;Elim plus_n_O.
Apply gt_S_l;Elim mult_sym;Auto.
Qed.
Hint P2_shiftlift1.

Theorem P2_shiftlift2 : (M,N:sub_explicits)(reg_shiftlift2 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (S O) (S (mult (mult (S(S(S(S O)))) (P2 s)) 
        (S (P2 t))))) 
       (mult (P2 s) (S (mult (S O) (S (P2 t)))))).
Unfold 1 mult;Elim plus_n_O;Unfold 4 mult;Elim plus_n_O.
Apply gt_S_l;Repeat (Elim mult_n_Sm);Elim plus_assoc_l.
Apply gt_plus_plus.
Elim mult_assoc_l;Auto.
Elim mult_n_2;Elim mult_sym;Auto.
Qed.
Hint P2_shiftlift2.

Theorem P2_lift1 : (M,N:sub_explicits)(reg_lift1 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (mult (S(S(S(S O)))) (P2 s)) 
           (S (mult (S(S(S(S O)))) (P2 t))))
           (mult (S(S(S(S O)))) (mult (P2 s)(S (P2 t))))).
Elim mult_assoc_r;Auto.
Qed.
Hint P2_lift1.

Theorem P2_lift2 : (M,N:sub_explicits)(reg_lift2 M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (mult (S(S(S(S O)))) (P2 s)) 
   (S (mult (mult (S(S(S(S O)))) (P2 t)) (S (P2 u)))))
       (mult (mult (S(S(S(S O)))) (mult (P2 s) (S (P2 t))))
         (S (P2 u)))).
Elim mult_assoc_r;
  Elim (mult_assoc_l (mult (S (S (S (S O)))) (P2 s))
                     (S (P2 t)) (S (P2 u)));
  Apply gt_mult_reg_l.
Auto.
Apply gt_S_l;Apply gt_mult_reg_r.
Auto.
Apply gt_trans with (mult (S(S(S O))) (P2 t)).
Auto.
Simpl;Elim plus_n_O;Rewrite S_plus;Apply gt_reg_l.
Elim mult_n_2;Auto.
Qed.
Hint P2_lift2.

Theorem P2_liftenv : (M,N:sub_explicits)(reg_liftenv M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros.
Change (gt (mult (mult (S(S(S(S O)))) (P2 s)) 
            (S(S (plus (P2 a) (P2 t)))))
           (S (plus (P2 a) (mult (P2 s) (S (P2 t)))))).
Cut ((S (S (plus (P2 a) (P2 t)))) =
     (plus (P2 a) (plus (P2 t) (S(S O)))) ).
Intro H1;Rewrite H1. 
Rewrite (S_plus (plus (P2 a) (mult (P2 s) (S (P2 t)))));
  Rewrite mult_plus_distr_r.
Elim plus_assoc_l;Apply gt_plus_plus.
Apply gt_mult_l.
Auto.
Auto.
Replace (plus (P2 t) (S (S O))) 
  with (plus (plus (P2 t) (S O)) (S O)).
Rewrite mult_plus_distr_r;Apply gt_plus_plus.
Elim S_plus;Elim mult_assoc_l.
Apply gt_mult_l;Auto.
Apply gt_mult_l;Auto.
Elim (plus_n_Sm (P2 t) (S O));Auto.
Rewrite plus_assoc_l;Elim plus_n_Sm;Elim plus_n_Sm;
Elim plus_n_O;Auto.
Qed.
Hint P2_liftenv.

Theorem P2_idl : (M,N:sub_explicits)(reg_idl M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Simpl;Intros;Elim plus_n_O;Auto.
Qed.
Hint P2_idl.

Theorem P2_idr : (M,N:sub_explicits)(reg_idr M N)->(gt (P2 M) (P2 N)).
Proof. 
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P2_idr.

Theorem P2_liftid : (M,N:sub_explicits)(reg_liftid M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P2_liftid.

Theorem P2_id : (M,N:terms)(reg_id M N)->(gt (P2 M) (P2 N)).
Proof.
Induction 1;Intros;Simpl;Auto.
Qed.
Hint P2_id.






