(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                           SLstar_bpar_SLstar.v                           *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)

         
                   (* relation SL* o B|| o SL* *)

Require TS.
Require sur_les_relations.
Require sigma_lift.
Require lambda_sigma_lift.
Require betapar.

Definition e_slstar_bp_slstar := [b:wsort]
 (comp_rel (e_relSLstar b) (comp_rel (e_beta_par b) (e_relSLstar b))).

Syntactic Definition slstar_bp_slstar := (e_slstar_bp_slstar ?).
Syntax constr slstar_bp_slstar <<(e_slstar_bp_slstar  $1 $t1 $t2)>> 10
  [<hov 0> "slstar_bp_slstar " <$t1:"term":L> " " <$t2:"term":L>].

Hint Unfold e_slstar_bp_slstar.

Goal (a,a',b,b':terms)(beta_par b b')->(slstar_bp_slstar a a')->
            (slstar_bp_slstar (app a b) (app a' b')).
Induction 2;Intros.
Red;Apply comp_2rel with (app y b).
Auto.
Elim H2;Intros.
Apply comp_2rel with (app y0 b');Auto.
Save slbpsl_context_app_l.
Hint slbpsl_context_app_l.

Goal (a,a',b,b':terms)(beta_par a a')->(slstar_bp_slstar b b')->
            (slstar_bp_slstar (app a b) (app a' b')).
Induction 2;Intros.
Red;Apply comp_2rel with (app a y).
Auto.
Elim H2;Intros.
Apply comp_2rel with (app a' y0);Auto.
Save slbpsl_context_app_r.
Hint slbpsl_context_app_r.

Goal (a,b,a',b':terms)(beta_par b b')->(slstar_bp_slstar a a')->
        (slstar_bp_slstar (app (lambda a) b) (env a' (cons b' id))).
Induction 2;Intros.
Red;Apply comp_2rel with (app (lambda y) b).
Auto.
Elim H2;Intros.
Apply comp_2rel with (env y0 (cons b' id));Auto.
Save slbpsl_context_beta_l.
Hint slbpsl_context_beta_l.

Goal (a,b,a',b':terms)(beta_par a a')->(slstar_bp_slstar b b')->
      (slstar_bp_slstar (app (lambda a) b) (env a' (cons b' id))).
Induction 2;Intros.
Red;Apply comp_2rel with (app (lambda a) y).
Auto.
Elim H2;Intros.
Apply comp_2rel with (env a' (cons y0 id));Auto.
Save slbpsl_context_beta_r.
Hint slbpsl_context_beta_r.

Goal (a,a':terms)(slstar_bp_slstar a a')->
               (slstar_bp_slstar (lambda a) (lambda a')).
Induction 1;Intros.
Red;Apply comp_2rel with (lambda y).
Auto.
Elim H1;Intros.
Apply comp_2rel with (lambda y0);Auto.
Save slbpsl_context_lambda .
Hint slbpsl_context_lambda .

Goal (a,a':terms)(s,s':sub_explicits)(beta_par s s')->
   (slstar_bp_slstar a a')->(slstar_bp_slstar (env a s) (env a' s')).
Induction 2;Intros.
Red;Apply comp_2rel with (env y s).
Auto.
Elim H2;Intros.
Apply comp_2rel with (env y0 s');Auto.
Save slbpsl_context_env_t.
Hint slbpsl_context_env_t.

Goal (a,a':terms)(s,s':sub_explicits)(beta_par a a')->
   (slstar_bp_slstar s s')->(slstar_bp_slstar (env a s) (env a' s')).
Induction 2;Intros.
Red;Apply comp_2rel with (env a y).
Auto.
Elim H2;Intros.
Apply comp_2rel with (env a' y0);Auto.
Save slbpsl_context_env_s.
Hint slbpsl_context_env_s.

Goal (a,a':terms)(s,s':sub_explicits)(beta_par s s')->
   (slstar_bp_slstar a a')->(slstar_bp_slstar (cons a s) (cons a' s')).
Induction 2;Intros.
Red;Apply comp_2rel with (cons y s).
Auto.
Elim H2;Intros.
Apply comp_2rel with (cons y0 s');Auto.
Save slbpsl_context_cons_t.
Hint slbpsl_context_cons_t.

Goal (a,a':terms)(s,s':sub_explicits)(beta_par a a')->
   (slstar_bp_slstar s s')->(slstar_bp_slstar (cons a s) (cons a' s')).
Induction 2;Intros.
Red;Apply comp_2rel with (cons a y).
Auto.
Elim H2;Intros.
Apply comp_2rel with (cons a' y0);Auto.
Save slbpsl_context_cons_s.
Hint slbpsl_context_cons_s.

Goal (s,s',t,t':sub_explicits)(beta_par t t')->
   (slstar_bp_slstar s s')->(slstar_bp_slstar (comp s t) (comp s' t')).
Induction 2;Intros.
Red;Apply comp_2rel with (comp y t).
Auto.
Elim H2;Intros.
Apply comp_2rel with (comp y0 t');Auto.
Save slbpsl_context_comp_l.
Hint slbpsl_context_comp_l.

Goal (s,s',t,t':sub_explicits)(beta_par s s')->
   (slstar_bp_slstar t t')->(slstar_bp_slstar (comp s t) (comp s' t')).
Induction 2;Intros.
Red;Apply comp_2rel with (comp s y).
Auto.
Elim H2;Intros.
Apply comp_2rel with (comp s' y0);Auto.
Save slbpsl_context_comp_r.
Hint slbpsl_context_comp_r.

Goal (s,s':sub_explicits)(slstar_bp_slstar s s')->
               (slstar_bp_slstar (lift s) (lift s')).
Induction 1;Intros.
Red;Apply comp_2rel with (lift y).
Auto.
Elim H1;Intros.
Apply comp_2rel with (lift y0);Auto.
Save slbpsl_context_lift.
Hint slbpsl_context_lift.

Goal (b:wsort)(M,N:(TS b))(beta_par M N)->(slstar_bp_slstar M N).
Intros;Red;Apply comp_2rel with M.
Red;Auto.
Apply comp_2rel with N;Auto.
Save betapar_slbpsl.
Hint betapar_slbpsl.

Goal (b:wsort)(M:(TS b))(slstar_bp_slstar M M).
Auto.
Save refl_slbpsl.
Hint refl_slbpsl.

(* LSL inclus dans SL*B||SL* *)

Goal (b:wsort)(inclus (e_relLSL b) (e_slstar_bp_slstar b)).
Red;Induction 1;Auto.
Induction 1;Auto.
 (* regle beta *)
Induction 1;Auto.
Intros b1 M0 N0 H1;Red;Apply comp_2rel with N0.
Auto. 
Apply comp_2rel with N0;Auto.
Save relLSL_inclus_slbpsl.
Hint relLSL_inclus_slbpsl.

(* SL*B||SL* inclus dans LSL* *)

Goal (b:wsort)(inclus (e_beta_par b) (e_relLSLstar b)).
Red;Induction 1;Intros;Auto.
(* beta_bpar *)
Red;Apply star_trans1 with (env M (cons N id)).
Auto.
Change (relLSLstar (env M (cons N id)) (env M' (cons N' id)));Auto.
Save betapar_inclus_relSLstar.
Hint betapar_inclus_relSLstar.

Goal (b:wsort)(inclus (e_relSL b) (e_relLSL b)).
Red;Induction 1;Auto.
Save relSL_inclus_relLSL.
Hint relSL_inclus_relLSL.

Goal (b:wsort)(inclus (e_slstar_bp_slstar b) (e_relLSLstar b)).

Unfold e_slstar_bp_slstar;Intro b.
Apply inclus_comp.
(* SL* incl LSL* *)
Change (inclus (star (e_relSL b)) (star (e_relLSL b)));Auto.
Apply inclus_comp.
(* B|| incl LSL* *)
Auto.
(* SL* incl LSL* *)
Change (inclus (star (e_relSL b)) (star (e_relLSL b)));Auto.
Intros;Red;Apply star_trans with y;Assumption.
Intros;Red;Apply star_trans with y;Assumption.
Save slbpsl_inclus_relLSLstar.
Hint slbpsl_inclus_relLSLstar.



