(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                   TS.v                                   *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


              (* Termes et Substitutions *)


Inductive wsort : Set := ws : wsort | wt : wsort.

Inductive TS : wsort->Set
:=   var:nat->(TS wt)
  | app:(TS wt)->(TS wt)->(TS wt)
  | lambda:(TS wt)->(TS wt)
  | env:(TS wt)->(TS ws)->(TS wt)
  | id:(TS ws)
  | shift:(TS ws)
  | cons:(TS wt)->(TS ws)->(TS ws)
  | comp:(TS ws)->(TS ws)->(TS ws)
  | lift:(TS ws)->(TS ws)
  | meta_X:nat->(TS wt)
  | meta_x:nat->(TS ws).

Definition terms := (TS wt).

Definition sub_explicits := (TS ws).
 
(* Principe d induction pour les terms *)
 
Goal (terms->Prop)->(b:wsort)(TS b)->Prop.
Intros P b;Elim b.
Exact [x:(TS ws)]True.
Exact P.
Save Pterms.
Transparent Pterms.

Lemma terms_ind: (P:terms->Prop)
                 ((n:nat)(P (var n)))->
                 ((a,b:terms)(P a)->(P b)->(P (app a b)))->
                 ((a:terms)(P a)->(P (lambda a)))->
                 ((a:terms)(P a)->(s:sub_explicits)(P (env a s)))->
                 ((n:nat)(P (meta_X n)))->
                 (a:terms)(P a).
Intros;Change (Pterms P wt a);Elim a;Simpl;Auto.
Save.

(* Principe d induction pour les sub_explicits *)

Goal (sub_explicits->Prop)->(b:wsort)(TS b)->Prop.
Intros P b;Elim b.
Exact P.
Exact [x:(TS wt)]True.
Save Psubst.
Transparent Psubst.

Lemma sub_explicits_ind: (P:sub_explicits->Prop)
                         (P id)->
                         (P shift)->
                         ((s:sub_explicits)(P s)->(a:terms)(P (cons a s)))->
                         ((s,t:sub_explicits)(P s)->(P t)->(P (comp s t)))->
                         ((s:sub_explicits)(P s)->(P (lift s)))->
                         ((n:nat)(P (meta_x n)))->
                         (s:sub_explicits)(P s).
Intros;Change (Psubst P ws s);Elim s;Simpl;Auto.
Save.




