(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                betapar.v                                 *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


         (* relation bata_par: Beta|| *)

Require TS.
Require sur_les_relations.

Inductive e_beta_par: (b:wsort)(TS b)->(TS b)->Prop
:=  var_bpar: (n:nat)(e_beta_par wt (var n)(var n))
 | id_bpar: (e_beta_par ws id id)
 | shift_bpar: (e_beta_par ws shift shift)
 | app_bpar: (M,N,M',N':terms)(e_beta_par wt M M')->(e_beta_par wt N N')->
                               (e_beta_par wt (app M N) (app M' N'))
 | lambda_bpar: (M,M':terms)(e_beta_par wt M M')->
                      (e_beta_par wt (lambda M) (lambda M'))
 | env_bpar: (M,M':terms)(s,s':sub_explicits)(e_beta_par wt M M')->
             (e_beta_par ws s s')->(e_beta_par wt (env M s) (env M' s'))
 | beta_bpar: (M,N,M',N':terms)(e_beta_par wt M M')->(e_beta_par wt N N')->
                (e_beta_par wt (app (lambda M) N) (env M' (cons N' id)))
 | cons_bpar:(M,M':terms)(s,s':sub_explicits)(e_beta_par wt M M')->
            (e_beta_par ws s s')->(e_beta_par ws (cons M s) (cons M' s')) 
 | lift_bpar: (s,s':sub_explicits)(e_beta_par ws s s')->
                           (e_beta_par ws (lift s) (lift s'))
 | comp_bpar: (s,s',t,t':sub_explicits)(e_beta_par ws s s')->
       (e_beta_par ws t t')->(e_beta_par ws (comp s t) (comp s' t'))
 | metaX_bpar: (n:nat)(e_beta_par wt (meta_X n)(meta_X n))
 | metax_bpar: (n:nat)(e_beta_par ws (meta_x n)(meta_x n)).

Hint var_bpar id_bpar shift_bpar app_bpar lambda_bpar env_bpar beta_bpar
      cons_bpar lift_bpar comp_bpar metaX_bpar metax_bpar.


Syntactic Definition beta_par := (e_beta_par ?).
Syntax constr beta_par <<(e_beta_par $1 $t1 $t2)>> 10
  [<hov 0> "beta_par " <$t1:"term":L> " " <$t2:"term":L>].


Goal (b:wsort)(M:(TS b))(beta_par M M).
Induction M;Auto.
Save refl_betapar.
Hint refl_betapar.

Definition e_betapar_inv:=[b:wsort][M:(TS b)][N:(TS b)]
 (<[b:wsort]Prop>Case M of
 (* var *) [n:nat] (<[b:wsort]Prop>Case N of
                   (* var *) [m:nat] (n=m)
                   (* app *) [N1,N2:terms]False 
                   (* lam *) [N1:terms]False
                   (* env *) [N1:terms][N2:sub_explicits]False
                   (* id  *) False
                   (*  |  *) False
                   (*  .  *) [N1:terms][N2:sub_explicits]False       
                   (*  o  *) [N1,N2:sub_explicits]False
                   (*  || *) [N1:sub_explicits]False
                   (*  X  *) [n:nat]False
                   (*  x  *) [n:nat]False  end)
 (* app *) [M1,M2:terms]
           (<[b:wsort]Prop>Case N of
            (* var *) [n:nat]False
            (* app *) [N1,N2:terms]
                       ((beta_par M1 N1) /\ (beta_par M2 N2))
            (* lam *) [N1:terms]False                               
            (* env *) [N1:terms][N2:sub_explicits]
                      (Ex( ([M3:terms] (Ex( ([N3:terms]
                             (M1=(lambda M3)) /\ (beta_par M3 N1) /\ 
                             (N2=(cons N3 id)) /\ (beta_par M2 N3) ))) )))
            (* id  *) False
            (*  |  *) False
            (*  .  *) [N1:terms][N2:sub_explicits]False
            (*  o  *) [N1,N2:sub_explicits]False
            (* ||  *) [N1:sub_explicits]False
            (*  X  *) [n:nat]False
            (*  x  *) [n:nat]False  end)
 (* lam *) [M1:terms]
           (<[b:wsort]Prop>Case N of
            (* var *) [n:nat]False
            (* app *) [N1,N2:terms]False 
            (* lam *) [N1:terms](beta_par M1 N1)
            (* env *) [N1:terms][N2:sub_explicits]False
            (* id  *) False
            (*  |  *) False
            (*  .  *) [N1:terms][N2:sub_explicits]False
            (*  o  *) [N1,N2:sub_explicits]False
            (* ||  *) [N1:sub_explicits]False
            (*  X  *) [n:nat]False
            (*  x  *) [n:nat]False  end)
 (* env *) [M1:terms][M2:sub_explicits]
           (<[b:wsort]Prop>Case N of
            (* var *) [n:nat]False
            (* app *) [N1,N2:terms]False 
            (* lam *) [N1:terms]False 
            (* env *) [N1:terms][N2:sub_explicits]
                      ((beta_par M1 N1) /\ (beta_par M2 N2))
            (* id  *) False
            (*  |  *) False
            (*  .  *) [N1:terms][N2:sub_explicits]False
            (*  o  *) [N1,N2:sub_explicits]False
            (* ||  *) [N1:sub_explicits]False
            (*  X  *) [n:nat]False
            (*  x  *) [n:nat]False  end)
 (* id  *) (<[b:wsort]Prop>Case N of
           (* var *) [n:nat]False
           (* app *) [N1,N2:terms]False 
           (* lam *) [N1:terms]False
           (* env *) [N1:terms][N2:sub_explicits]False
           (* id  *) True
           (*  |  *) False
           (*  .  *) [N1:terms][N2:sub_explicits]False
           (*  o  *) [N1,N2:sub_explicits]False
           (*  || *) [N1:sub_explicits]False
           (*  X  *) [n:nat]False
           (*  x  *) [n:nat]False end)
 (*  |  *) (<[b:wsort]Prop>Case N of
           (* var *) [n:nat]False
           (* app *) [N1,N2:terms]False 
           (* lam *) [N1:terms]False
           (* env *) [N1:terms][N2:sub_explicits]False
           (* id  *) False
           (*  |  *) True 
           (*  .  *) [N1:terms][N2:sub_explicits]False
           (*  o  *) [N1,N2:sub_explicits]False
           (*  || *) [N1:sub_explicits]False
           (*  X  *) [n:nat]False
           (*  x  *) [n:nat]False  end)          
 (*  .  *) [M1:terms][M2:sub_explicits]
           (<[b:wsort]Prop>Case N of
            (* var *) [n:nat]False
            (* app *) [N1:terms][N2:terms]False 
            (* lam *) [N1:terms]False 
            (* env *) [N1:terms][N2:sub_explicits]False
            (* id  *) False
            (*  |  *) False
            (*  .  *) [N1:terms][N2:sub_explicits]
                      ((beta_par M1 N1) /\ (beta_par M2 N2))
            (*  o  *) [N1,N2:sub_explicits]False
            (* ||  *) [N1:sub_explicits]False
            (*  X  *) [n:nat]False
            (*  x  *) [n:nat]False  end)
 (*  o  *) [M1,M2:sub_explicits]
           (<[b:wsort]Prop>Case N of
            (* var *) [n:nat]False
            (* app *) [N1,N2:terms]False 
            (* lam *) [N1:terms]False
            (* env *) [N1:terms][N2:sub_explicits]False
            (* id  *) False
            (*  |  *) False
            (*  .  *) [N1:terms][N2:sub_explicits]False
            (*  o  *) [N1,N2:sub_explicits] 
                      ((beta_par M1 N1) /\ (beta_par M2 N2))
            (* ||  *) [N1:sub_explicits]False
            (*  X  *) [n:nat]False
            (*  x  *) [n:nat]False  end) 
 (* ||  *) [M1:sub_explicits]
           (<[b:wsort]Prop>Case N of
            (* var *) [n:nat]False
            (* app *) [N1,N2:terms]False 
            (* lam *) [N1:terms]False
            (* env *) [N1:terms][N2:sub_explicits]False
            (* id  *) False
            (*  |  *) False
            (*  .  *) [N1:terms][N2:sub_explicits]False
            (*  o  *) [N1,N2:sub_explicits]False
            (* ||  *) [N1:sub_explicits](beta_par M1 N1)
            (*  X  *) [n:nat]False
            (*  x  *) [n:nat]False  end)
 (*  X  *) [n:nat] (<[b:wsort]Prop>Case N of
                   (* var *) [n:nat]False
                   (* app *) [N1,N2:terms]False 
                   (* lam *) [N1:terms]False
                   (* env *) [N1:terms][N2:sub_explicits]False
                   (* id  *) False
                   (*  |  *) False
                   (*  .  *) [N1:terms][N2:sub_explicits]False
                   (*  o  *) [N1:sub_explicits][N2:sub_explicits]False
                   (*  || *) [N1:sub_explicits]False
                   (*  X  *) [m:nat] (n=m)
                   (*  x  *) [m:nat]False  end)
 (*  x  *) [n:nat] (<[b:wsort]Prop>Case N of
                   (* var *) [n:nat]False
                   (* app *) [N1,N2:terms]False 
                   (* lam *) [N1:terms]False
                   (* env *) [N1:terms][N2:sub_explicits]False
                   (* id  *) False
                   (*  |  *) False
                   (*  .  *) [N1:terms][N2:sub_explicits]False
                   (*  o  *) [N1:sub_explicits][N2:sub_explicits]False
                   (*  || *) [N1:sub_explicits]False
                   (*  X  *) [m:nat]False
                   (*  x  *) [m:nat] (n=m)  end)  end).


Syntactic Definition betapar_inv := (e_betapar_inv ?).
Syntax constr betapar_inv <<(e_betapar_inv $1 $t1 $t2)>> 10
  [<hov 0> "betapar_inv " <$t1:"term":L> " " <$t2:"term":L>].


Goal (b:wsort)(M,N:(TS b))(beta_par M N)->(betapar_inv M N).
Induction 1;Intros;Simpl;Auto.
(* beta *)Exists M0;Exists N';Auto.
Save lemma1_inv_betapar.
Hint lemma1_inv_betapar.

Goal (P:terms->Prop)(n:nat)
     (P (var n))->
     (M:terms)(beta_par (var n) M)->(P M).
Intros P n H M H0;Cut (betapar_inv (var n) M).
2:Auto.
Pattern M;Apply terms_ind.
(* var *)
Induction 1;Assumption.
(* app *)
Induction 3.
(* lam *)
Induction 2.
(* env *)
Induction 2.
(* X *)Induction 1.
Save case_bvar.

Goal (P:terms->Prop)(a,b:terms)
     ((a',b':terms)(beta_par a a')->(beta_par b b')->(P (app a' b')))->
     ((a1,a1',b':terms)(a=(lambda a1))->(beta_par a1 a1')->
         (beta_par b b')->(P (env a1' (cons b' id))))->
     (M:terms)(beta_par (app a b) M)->(P M).
Intros P a b H H0 M H1;Cut (betapar_inv (app a b) M).
2:Auto.
Pattern M;Apply terms_ind.
(* var *)
Induction 1.
(* app *)
Unfold 3 e_betapar_inv;Intros a' b' H2 H3 H4.
Elim H4;Intros H5 H6.
Apply H;Assumption.
(* lam *)
Induction 2.
(* env *)
Unfold 2 e_betapar_inv;Intros a1' H2 s H3.
Elim H3;Intros a1 H4;Elim H4;Intros b' H5.
Elim H5;Intros H6 H7;Elim H7;Intros H8 H9;Elim H9;Intros H10 H11.
Rewrite H6;Rewrite H10;Apply (H0 a1);Assumption.
(* X *)Induction 1.
Save case_bapp.

Goal (P:terms->Prop)(a:terms)
     ((a':terms)(beta_par a a')->(P (lambda a')))->
     (M:terms)(beta_par (lambda a) M)->(P M).
Intros P a H M H0;Cut (betapar_inv (lambda a) M).
2:Auto.
Pattern M;Apply terms_ind.
(* var *)
Induction 1.
(* app *)
Induction 3.
(* lam *)
Unfold 2 e_betapar_inv;Intros a' H1 H2.
Apply H;Assumption.
(* env *)
Induction 2.
(* X *)Induction 1.
Save case_blambda.

Goal (P:terms->Prop)(a:terms)(s:sub_explicits)
     ((a':terms)(s':sub_explicits)(beta_par a a')->(beta_par s s')->
                  (P (env a' s')))->
     (M:terms)(beta_par (env a s) M)->(P M).
Intros P a s H M H0;Cut (betapar_inv (env a s) M).
2:Auto.
Pattern M;Apply terms_ind.
(* var *)
Induction 1.
(* app *)
Induction 3.
(* lam *)
Induction 2.
(* env *)
Unfold 2 e_betapar_inv;Intros a' H1  s' H2.
Elim H2;Intros;Apply H;Assumption.
(* X *)Induction 1.
Save case_benv.

Goal(P:sub_explicits->Prop)(P id)-> 
    (M:sub_explicits)(beta_par id M)->(P M).
Intros P H M H0;Cut (betapar_inv id M).
2:Auto.
Pattern M;Apply sub_explicits_ind.
(* id *)
Intro;Assumption.
(* |  *)
Induction 1.
(* .  *)
Induction 2.
(*  o *)
Induction 3.
(* || *)
Induction 2.
(* x *)Induction 1.
Save case_bid.

Goal(P:sub_explicits->Prop)(P shift)-> 
    (M:sub_explicits)(beta_par shift M)->(P M).
Intros P H M H0;Cut (betapar_inv shift M).
2:Auto.
Pattern M;Apply sub_explicits_ind.
(* id *)
Induction 1.
(* |  *)
Intro;Assumption.
(* .  *)
Induction 2.
(*  o *)
Induction 3.
(* || *)
Induction 2.
(* x *)Induction 1.
Save case_bshift.

Goal (P:sub_explicits->Prop)(a:terms)(s:sub_explicits)
     ((a':terms)(s':sub_explicits)(beta_par a a')->(beta_par s s')->
                  (P (cons a' s')))->
     (M:sub_explicits)(beta_par (cons a s) M)->(P M).
Intros P a s H M H0;Cut (betapar_inv (cons a s) M).
2:Auto.
Pattern M;Apply sub_explicits_ind.
(* id  *)
Induction 1.
(*  |  *)
Induction 1.
(*  .  *)
Unfold 2 e_betapar_inv;Intros s' H1 a' H2.
Elim H2;Intros.
Apply H;Assumption.
(*  o  *)
Induction 3.
(*  || *)
Induction 2.
(* x *)Induction 1.
Save case_bcons.

Goal (P:sub_explicits->Prop)(s,t:sub_explicits)
     ((s',t':sub_explicits)(beta_par s s')->(beta_par t t')->
                  (P (comp s' t')))->
     (M:sub_explicits)(beta_par (comp s t) M)->(P M).
Intros P s t H M H0;Cut (betapar_inv (comp s t) M).
2:Auto.
Pattern M;Apply sub_explicits_ind.
(* id  *)
Induction 1.
(*  |  *)
Induction 1.
(*  .  *)
Induction 2.
(*  o  *)
Unfold 3 e_betapar_inv.
Intros s' t' H1 H2 H3;Elim H3;Intros;Apply H;Assumption.
(*  || *)
Induction 2.
(* x *)Induction 1.
Save case_bcomp.

Goal (P:sub_explicits->Prop)(s:sub_explicits)
     ((s':sub_explicits)(beta_par s s')->(P (lift s')))->
     (M:sub_explicits)(beta_par (lift s) M)->(P M).
Intros P s H M H0;Cut (betapar_inv (lift s) M).
2:Auto.
Pattern M;Apply sub_explicits_ind.
(* id  *)
Induction 1.
(*  |  *)
Induction 1.
(*  .  *)
Induction 2.
(*  o  *)
Induction 3.
(*  || *)
Unfold 2 e_betapar_inv.
Intros s' H1 H2;Apply H;Assumption.
(* x *)Induction 1.
Save case_blift.

Goal (P:terms->Prop)(n:nat)
     (P (meta_X n))->
     (M:terms)(beta_par (meta_X n) M)->(P M).
Intros P n H M H0;Cut (betapar_inv (meta_X n) M).
2:Auto.
Pattern M;Apply terms_ind.
(* var *)
Induction 1.
(* app *)
Induction 3.
(* lam *)
Induction 2.
(* env *)
Induction 2.
(* X *)Induction 1;Assumption.
Save case_bmetaX.

Goal(P:sub_explicits->Prop)(n:nat)
    (P (meta_x n))-> 
    (M:sub_explicits)(beta_par (meta_x n) M)->(P M).
Intros P n H M H0;Cut (betapar_inv (meta_x n) M).
2:Auto.
Pattern M;Apply sub_explicits_ind.
(* id *)
Induction 1.
(* |  *)
Induction 1.
(* .  *)
Induction 2.
(*  o *)
Induction 3.
(* || *)
Induction 2.
(* x *)Induction 1;Assumption.
Save case_bmetax.





