(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              commutation.v                               *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


                 (*  SL commute avec B|| de la maniere suivante:
      
                                           B||
                                       x ---------> z
                                       |            |
                                    SL |            |SL*
                                       |            | 
                                       V            V
                                       y ----------> u
                                         SL*B||SL*              *)
Require sur_les_relations.
Require TS.
Require egaliteTS.
Require sigma_lift.
Require betapar.
Require SLstar_bpar_SLstar.
Require determinePC_SL.

Definition e_diag1:=[b:wsort][x:(TS b)][y:(TS b)](z:(TS b))(beta_par x z)->
      (Ex( ([u:(TS b)] (slstar_bp_slstar y u) /\ (relSLstar z u)))).

Syntactic Definition diag1 := (e_diag1 ?).
Syntax constr diag1 <<(e_diag1 $1 $t1 $t2)>> 10
  [<hov 0> "diag1 " <$t1:"term":L> " " <$t2:"term":L>].


(* les regles du systeme sigma-lift (SL) verifient le diagramme *)

Goal (x,y:terms)(reg_app x y)->(diag1 x y).
Induction 1;Red;Intros a b0 s z H0.
Pattern z;Apply case_benv with (app a b0) s.
2:Assumption.
Intros x' s' H1 H2;Pattern x';Apply case_bapp with a b0.
3:Assumption.
(* 1-regle B||: app *)
Intros a' b0' H3 H4;Exists (app (env a' s') (env b0' s'));Auto 6.
 (* (a[s])(b0[s]) SL*B||SL* (a'[s'])(b0'[s']) *)
 (* (a'b0')[s'] SL* (a'[s'])(b0'[s']) *)
(* 2-regle B||: beta *)
Intros a1 a1' b0' H3 H4 H5;Rewrite H3.
Exists (env a1' (cons (env b0' s') s'));Split.
 (* ((L a1)[s])(b0[s]) SL*B||SL* a1'[b0'[s'].s'] *)
Red;Apply comp_2rel with (app (lambda (env a1 (lift s))) (env b0 s)).
   (* ((L a1)[s])(b0[s]) SL* (L (a1[||S]))(b0[s]) *) 
Auto 6.
   (* (L (a1[||S]))(b0[s]) B|| (a1'[||s'])[b0'[s'].id] *)
Apply comp_2rel with (env (env a1' (lift s')) (cons (env b0' s') id)).
Auto.
   (* (a1'[||s'])[b0'[s'].id] SL* a1'[b0'[s'].s'] *)
Red;Apply star_trans1 with (env a1' (comp (lift s') (cons (env b0' s') id))).
Auto.
Apply star_trans1 with (env a1' (cons (env b0' s') (comp s' id)));Auto 6.
 (* (a1'[b0'id])[s'] SL* a1'[b0'[s'].s'] *)
Red;Apply star_trans1 with (env a1' (comp (cons b0' id) s')).
Auto.
Apply star_trans1 with (env a1' (cons (env b0' s') (comp id s')));Auto 6.
Save commut_app.
Hint commut_app.

Goal (x,y:terms)(reg_lambda x y)->(diag1 x y).
Induction 1;Red;Intros a s z H0.
Pattern z;Apply case_benv with (lambda a) s.
2:Assumption.
Intros x' s' H1 H2;Pattern x';Apply case_blambda with a.
2:Assumption.
Intros a' H3;Exists (lambda (env a' (lift s')));Auto 6.
(* L(a[||s]) SL*B||*SL L(a'[||s']) *)
(* (L a0')[s'] SL* L(a0'[||s']) *)
Save commut_lambda.
Hint commut_lambda.

Goal (x,y:terms)(reg_clos x y)->(diag1 x y).
Induction 1;Red;Intros a s t z H0.
Pattern z;Apply case_benv with (env a s) t.
2:Assumption.
Intros x' t' H1 H2;Pattern x';Apply case_benv with a s.
2:Assumption.
Intros a' s' H3 H4;Exists (env a' (comp s' t'));Auto 6.
(*  a[sot] SL*B||SL* a'[s'ot'] *)
(* (a'[s'])[t'] SL* a'[s'ot'] *)
Save commut_clos.
Hint commut_clos.

Goal (x,y:terms)(reg_varshift1 x y)->(diag1 x y).
Induction 1;Red;Intros n z H0.
Pattern z;Apply case_benv with (var n) shift.
2:Assumption.
Intros x' s' H1 H2;Pattern x';Apply case_bvar with n.
2:Assumption.
Pattern s';Apply case_bshift.
2:Assumption.
Exists (var (S n));Auto 6.
(* n+1 SL*B||SL* n+1 *)
(* n[|] SL* n+1 *)
Save commut_varshift1.
Hint commut_varshift1.

Goal (x,y:terms)(reg_varshift2 x y)->(diag1 x y).
Induction 1;Red;Intros n s z H0.
Pattern z;Apply case_benv with (var n) (comp shift s).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bvar with n.
2:Assumption.
Pattern y';Apply case_bcomp with shift s.
2:Assumption.
Intros t' s' H3 H4;Pattern t';Apply case_bshift.
2:Assumption.
Exists (env (var (S n)) s');Auto 6.
(* n+1[s] SL*B||SL* n+1[s'] *)
(* n[|os'] SL* n+1[s'] *)
Save commut_varshift2.
Hint commut_varshift2.

Goal (x,y:terms)(reg_fvarcons x y)->(diag1 x y).
Induction 1;Red;Intros a s z H0.
Pattern z;Apply case_benv with (var O) (cons a s).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bvar with O.
2:Assumption.
Pattern y';Apply case_bcons with a s.
2:Assumption.
Intros a' s' H3 H4;Exists a';Auto 6.
(* a SL*B||SL* a' *)
(* 0[a'.s'] SL* a' *)
Save commut_fvarcons.
Hint commut_fvarcons.

Goal (x,y:terms)(reg_fvarlift1 x y)->(diag1 x y).
Induction 1;Red;Intros s z H0.
Pattern z;Apply case_benv with (var O) (lift s).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bvar with O.
2:Assumption.
Pattern y';Apply case_blift with s.
2:Assumption.
Intros s' H3;Exists (var O);Auto 6.
(* 0 SL*B||SL* 0 *)
(* 0[||s'] SL* 0 *)
Save commut_fvarlift1.
Hint commut_fvarlift1.

Goal (x,y:terms)(reg_fvarlift2 x y)->(diag1 x y).
Induction 1;Red;Intros s t z H0.
Pattern z;Apply case_benv with (var O) (comp (lift s) t).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bvar with O.
2:Assumption.
Pattern y';Apply case_bcomp with (lift s) t.
2:Assumption.
Intros z' t' H3 H4;Pattern z';Apply case_blift with s.
2:Assumption.
Intros s' H5;Exists (env (var O) t');Auto 6.
(* 0[t] SL*B||SL* 0[t'] *)
(* 0[||s'ot'] SL* 0[t'] *)
Save commut_fvarlift2.
Hint commut_fvarlift2.

Goal (x,y:terms)(reg_rvarcons x y)->(diag1 x y).
Induction 1;Red;Intros n a s z H0.
Pattern z;Apply case_benv with (var (S n)) (cons a s).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bvar with (S n).
2:Assumption.
Pattern y';Apply case_bcons with a s.
2:Assumption.
Intros a' s' H3 H4;Exists (env (var n) s');Auto 6.
(* n[s] SL*B||SL* n[s'] *)
(* n+1[a'.s'] SL* n[s'] *)
Save commut_rvarcons.
Hint commut_rvarcons.

Goal (x,y:terms)(reg_rvarlift1 x y)->(diag1 x y).
Induction 1;Red;Intros n s z H0.
Pattern z;Apply case_benv with (var (S n)) (lift s).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bvar with (S n).
2:Assumption.
Pattern y';Apply case_blift with s.
2:Assumption.
Intros s' H3;Exists (env (var n) (comp s' shift));Auto 6.
(* n[so|] SL*B||SL* n[s'o|] *)
(* n+1[||s'] SL* n[s'o|] *)
Save commut_rvarlift1.
Hint commut_rvarlift1.

Goal (x,y:terms)(reg_rvarlift2 x y)->(diag1 x y).
Induction 1;Red;Intros n s t z H0.
Pattern z;Apply case_benv with (var (S n)) (comp (lift s) t).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bvar with (S n).
2:Assumption.
Pattern y';Apply case_bcomp with (lift s) t.
2:Assumption.
Intros z' t' H3 H4;Pattern z';Apply case_blift with s.
2:Assumption.
Intros s' H5;Exists (env (var n) (comp s' (comp shift t')));Auto 6.
(* n[so(|ot)] SL*B||SL* n[s'o(|ot')] *)
(* n+1[||s'ot'] SL* n[s'o(|ot')] *)
Save commut_rvarlift2.
Hint commut_rvarlift2.

Goal (x,y:sub_explicits)(reg_assenv x y)->(diag1 x y).
Induction 1;Red;Intros s t u z H0.
Pattern z;Apply case_bcomp with (comp s t) u.
2:Assumption.
Intros x' u' H1 H2;Pattern x';Apply case_bcomp with s t.
2:Assumption.
Intros s' t' H3 H4;Exists (comp s' (comp t' u'));Auto 6.
(* so(tou) SL*B||SL* s'o(t'ou') *)
(*  (s'ot')ou' SL* s'o(t'ou') *)
Save commut_assenv.
Hint commut_assenv.

Goal (x,y:sub_explicits)(reg_mapenv x y)->(diag1 x y).
Induction 1;Red;Intros a s t z H0.
Pattern z;Apply case_bcomp with (cons a s) t.
2:Assumption.
Intros x' t' H1 H2;Pattern x';Apply case_bcons with a s.
2:Assumption.
Intros a' s' H3 H4;Exists (cons (env a' t') (comp s' t'));Auto 6.
(* a[t].(sot) SL*B||SL a'[t'].(s'ot') *)
(* (a'.s')ot' SL* a'[t'].(s'ot') *)
Save commut_mapenv.
Hint commut_mapenv.

Goal (x,y:sub_explicits)(reg_shiftcons x y)->(diag1 x y).
Induction 1;Red;Intros a s z H0.
Pattern z;Apply case_bcomp with shift (cons a s).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bshift.
2:Assumption.
Pattern y';Apply case_bcons with a s.
2:Assumption.
Intros a' s' H3 H4;Exists s';Auto 6.
(* s SL*B||SL* s' *)
(* shift o(a'.s') SL* s' *)
Save commut_shiftcons.
Hint commut_shiftcons.

Goal (x,y:sub_explicits)(reg_shiftlift1 x y)->(diag1 x y).
Induction 1;Red;Intros s z H0.
Pattern z;Apply case_bcomp with shift (lift s).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bshift.
2:Assumption.
Pattern y';Apply case_blift with s.
2:Assumption.
Intros s' H3;Exists (comp s' shift);Auto 6.
(* so| SL*B||SL* s'o| *)
(* |o(|| s') SL* s'o| *)
Save commut_shiftlift1.
Hint commut_shiftlift1.

Goal (x,y:sub_explicits)(reg_shiftlift2 x y)->(diag1 x y).
Induction 1;Red;Intros s t z H0.
Pattern z;Apply case_bcomp with shift (comp (lift s) t).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_bshift.
2:Assumption.
Pattern y';Apply case_bcomp with (lift s) t.
2:Assumption.
Intros z' t' H3 H4;Pattern z';Apply case_blift with s.
2:Assumption.
Intros s' H5;Exists (comp s' (comp shift t'));Auto 6.
(* so(|ot) SL*B||SL* s'o(|ot') *)
(* (|| s')ot' SL* s'o(|ot') *)
Save commut_shiftlift2.
Hint commut_shiftlift2.

Goal (x,y:sub_explicits)(reg_lift1 x y)->(diag1 x y).
Induction 1;Red;Intros s t z H0.
Pattern z;Apply case_bcomp with (lift s) (lift t).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_blift with s.
2:Assumption.
Intros s' H3;Pattern y';Apply case_blift with t.
2:Assumption.
Intros t' H4;Exists (lift (comp s' t'));Auto 6.
(* ||(sot) SL*B||SL* ||(s'ot') *)
(* ||s' o ||t' SL* ||(s'ot') *)
Save commut_lift1.
Hint commut_lift1.

Goal (x,y:sub_explicits)(reg_lift2 x y)->(diag1 x y).
Induction 1;Red;Intros s t u z H0.
Pattern z;Apply case_bcomp with (lift s) (comp (lift t) u).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_blift with s.
2:Assumption.
Intros s' H3;Pattern y';Apply case_bcomp with (lift t) u.
2:Assumption.
Intros z' u' H4 H5;Pattern z';Apply case_blift with t.
2:Assumption.
Intros t' H6;Exists (comp (lift (comp s' t')) u');Auto 6.
(* ||(sot)ou SL*B||SL* ||(s'ot')ou' *) 
(* ||s'o(||t'ou') SL* ||(s'ot')ou' *) 
Save commut_lift2.
Hint commut_lift2.

Goal (x,y:sub_explicits)(reg_liftenv x y)->(diag1 x y).
Induction 1;Red;Intros a s t z H0.
Pattern z;Apply case_bcomp with (lift s) (cons a t).
2:Assumption.
Intros x' y' H1 H2;Pattern x';Apply case_blift with s.
2:Assumption.
Intros s' H3;Pattern y';Apply case_bcons with a t.
2:Assumption.
Intros a' t' H4 H5;Exists (cons a' (comp s' t'));Auto 6.
(* a.(sot) SL*B||SL* a'.(s'ot') *)
(* ||s'o(a'.t') SL* a'.(s'ot') *)
Save commut_liftenv.
Hint commut_liftenv.

Goal (x,y:sub_explicits)(reg_idl x y)->(diag1 x y).
Induction 1;Red;Intros s z H0.
Pattern z;Apply case_bcomp with id s.
2:Assumption.
Intros x' s' H1 H2;Pattern x';Apply case_bid.
2:Assumption.
Exists s';Auto 6.
(* s SL*B||SL* s' *)
(* idos' SL* s' *)
Save commut_idl.
Hint commut_idl.

Goal (x,y:sub_explicits)(reg_idr x y)->(diag1 x y).
Induction 1;Red;Intros s z H0.
Pattern z;Apply case_bcomp with s id.
2:Assumption.
Intros s' x' H1 H2;Pattern x';Apply case_bid.
2:Assumption.
Exists s';Auto 6.
(* s SL*B||SL* s' *)
(* s'oid SL* s' *)
Save commut_idr.
Hint commut_idr.

Goal (x,y:sub_explicits)(reg_liftid x y)->(diag1 x y).
Induction 1;Red;Intros z H0.
Pattern z;Apply case_blift with id.
2:Assumption.
Intros x' H1;Pattern x';Apply case_bid.
2:Assumption.
Exists id;Auto 6.
(* id SL*B||SL* id *)
(* ||id SL* id *)
Save commut_liftid.
Hint commut_liftid.

Goal (x,y:terms)(reg_id x y)->(diag1 x y).
Induction 1;Red;Intros a z H0.
Pattern z;Apply case_benv with a id.
2:Assumption.
Intros a' x' H1 H2;Pattern x';Apply case_bid.
2:Assumption.
Exists a';Auto 6.
(* a SLB||SL* a' *)
(* a'[id] SL* a' *)
Save commut_id.
Hint commut_id.
 
Goal (b:wsort)(x,y:(TS b))(systemSL x y)->(diag1 x y).
Induction 1;Intros;Auto.
Save commut_systemSL.

(* lemmes techniques *)

Goal (P:terms->Prop)(a:terms)
     ((a':terms)(relSLstar a a')->(P (lambda a')))->
     (M,N:terms)(relSLstar N M)->(N=(lambda a))->(P M).
Intros P a H M N H0;Generalize a H;Elim H0.
Intros x a0 H1 H2;Rewrite H2;Apply (H1 a0);Red;Apply star_refl.
Intros x y z H1 H2 H3 a0 H4 H5;Generalize H1;Rewrite H5;Intro H6.
Cut (y=y).
2:Trivial.
Pattern 2 y;Apply case_SLlambda with a0.
2:Assumption.
Intros a0' H7 H8;Apply (H3 a0').
Intros a' H9;Apply H4;Red;Apply star_trans1 with a0';Assumption.
Assumption.
Save case_SLstar_lambda'.

Goal (P:terms->Prop)(a:terms)
      ((a':terms)(relSLstar a a')->(P (lambda a')))->
      (M:terms)(relSLstar (lambda a) M)->(P M).
Intros; Pattern M;Apply case_SLstar_lambda' with a (lambda a);Auto 6.
Save case_SLstar_lambda.

Goal (P:terms->Prop)(a:terms)
     ((a':terms)(slstar_bp_slstar a a')->(P (lambda a')))->
     (M:terms)(slstar_bp_slstar (lambda a) M)->(P M).
Intros P a H M H0.
Elim (comp_case terms (e_relSLstar wt) (comp_rel (e_beta_par wt) 
              (e_relSLstar wt)) (lambda a) M H0).
Intros x H1;Elim H1;Intros H2.
Pattern x;Apply case_SLstar_lambda with a.
2:Assumption.
Intros a' H3 H4.
Elim (comp_case terms (e_beta_par wt) (e_relSLstar wt) (lambda a') M H4).
Intros y H5;Elim H5;Intros H6.
Pattern y;Apply case_blambda with a'.
2:Assumption.
Intros a'' H7 H8.
Pattern M;Apply case_SLstar_lambda with a''.
2:Assumption.
Intros a_ H9;Apply H.
Red;Apply comp_2rel with a'.
Assumption.
Apply comp_2rel with a'';Assumption.
Save case_slbpsl_lambda.

Goal (a,a':terms)(diag1 (lambda a) (lambda a'))->(diag1 a a').
Red;Intros a a' H z H0.
Elim (H (lambda z)).
2:Apply lambda_bpar;Assumption.
Intros u1 H1;Elim H1;Intros H2 H3.
Cut (u1=u1).
2:Trivial.
Pattern 1 u1;Apply case_SLstar_lambda with z.
2:Assumption.
Intros z' H4;Pattern u1;Apply case_slbpsl_lambda with a'.
2:Assumption.
Intros a'' H5 H6;Exists a'';Split.
Assumption.
Elim (proj_lambda z' a'' H6);Assumption.
Save diag1_lambda.

Theorem commut: (b:wsort)(x,y:(TS b))(relSL x y)->(diag1 x y).
Induction 1;Intros.
(* regles de reecriture *)
Apply commut_systemSL;Assumption.
(* contexte app droit *)
Red;Intros z H2;Generalize H0 H1.
Pattern z;Apply case_bapp with a b0.
3:Assumption.
 (* regle B||: app *)
Intros a'' b0'' H3 H4 H5 H6.
Elim (H6 a'' H3);Intros a_ H7;Elim H7;Intros H8 H9.
Exists (app a_ b0'');Auto.
 (* regle B||: beta *)
Intros a1 a1'' b0'' H3 H4 H5;Rewrite H3.
Intro H6;Pattern a';Apply case_SLlambda with a1.
2:Assumption.
Intros a1' H7 H8.
Elim (diag1_lambda a1 a1' H8 a1'' H4);Intros a_ H9;Elim H9;Intros H10 H11.
Exists (env a_ (cons b0'' id));Auto.
(* contexte app gauche *)
Red;Intros z H2;Pattern z;Apply case_bapp with a b0.
3:Assumption.
 (* regle B||: app *)
Intros a'' b0'' H3 H4.
Elim (H1 b0'' H4);Intros b0_ H5;Elim H5;Intros H6 H7.
Exists (app a'' b0_);Auto.
 (* regle B||: beta *)
Intros a1 a1'' b0'' H3 H4 H5;Rewrite H3.
Elim (H1 b0'' H5);Intros b0_ H6;Elim H6;Intros H7 H8.
Exists (env a1'' (cons b0_ id));Auto.
(* contexte lambda *)
Red;Intros z H2.
Pattern z;Apply case_blambda with a.
2:Assumption.
Intros a'' H3.
Elim (H1 a'' H3);Intros a_ H4;Elim H4;Intros H5 H6.
Exists (lambda a_);Auto.
(* contexte env droit *)
Red;Intros z H2.
Pattern z;Apply case_benv with a s.
2:Assumption.
Intros a'' s'' H3 H4.
Elim (H1 a'' H3);Intros a_ H5;Elim H5;Intros H6 H7.
Exists (env a_ s'');Auto.
(* contexte env gauche *)
Red;Intros z H2.
Pattern z;Apply case_benv with a s.
2:Assumption.
Intros a'' s'' H3 H4.
Elim (H1 s'' H4);Intros s_ H5;Elim H5;Intros H6 H7.
Exists (env a'' s_);Auto.
(* contexte cons droit *)
Red;Intros z H2.
Pattern z;Apply case_bcons with a s.
2:Assumption.
Intros a'' s'' H3 H4.
Elim (H1 a'' H3);Intros a_ H5;Elim H5;Intros H6 H7.
Exists (cons a_ s'');Auto.
(* contexte cons gauche *)
Red;Intros z H2.
Pattern z;Apply case_bcons with a s.
2:Assumption.
Intros a'' s'' H3 H4.
Elim (H1 s'' H4);Intros s_ H5;Elim H5;Intros H6 H7.
Exists (cons a'' s_);Auto.
(* contexte comp droit *)
Red;Intros z H2.
Pattern z;Apply case_bcomp with s t.
2:Assumption.
Intros s'' t'' H3 H4.
Elim (H1 s'' H3);Intros s_ H5;Elim H5;Intros H6 H7.
Exists (comp s_ t'');Auto.
(* contexte comp gauche *)
Red;Intros z H2.
Pattern z;Apply case_bcomp with s t.
2:Assumption.
Intros s'' t'' H3 H4.
Elim (H1 t'' H4);Intros t_ H5;Elim H5;Intros H6 H7.
Exists (comp s'' t_);Auto.
(* contexte lift *)
Red;Intros z H2.
Pattern z;Apply case_blift with s.
2:Assumption.
Intros s'' H3.
Elim (H1 s'' H3);Intros s_ H4;Elim H4;Intros H5 H6.
Exists (lift s_);Auto.
Save.


(***************************************************)
(*    SL verifie le diagramme ci-dessus            *)
(***************************************************)

Theorem commutation: (b:wsort)(x,y,z:(TS b))(relSL x y)->(beta_par x z)->
       (Ex( ([u:(TS b)](relSLstar z u) /\ (slstar_bp_slstar y u)))).
Intros b x y z H H0;Apply Ex_PQ;Generalize z H0.
Change (diag1 x y).
Apply commut;Assumption.
Save.


