(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               comparith.v                                *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)



              (*         complements d arithmetique            *)


Require Le.
Require Lt.
Require Plus.
Require Gt.
Require Minus.
Require Mult.

Hint mult_n_O mult_n_Sm le_plus_plus. 

(* gt *)

Goal (n,m:nat)(gt n m)->(gt (S n) m).
Auto.
Save gt_S_l.
Hint gt_S_l.

Goal (n,m:nat)(gt n (S m))->(gt n m).
Auto.
Save gt_S_r.
Hint gt_S_r.
 
(* plus *)

Goal (n,m,p:nat)(n=m)->((plus p n)=(plus p m)).
Intros;Elim p;Elim H;Auto.
Save eq_plus_reg_r.
Hint eq_plus_reg_r.

Goal (n,m,p:nat)(n=m)->((plus n p)=(plus m p)).
Intros;Elim p;Elim H;Auto.
Save eq_plus_reg_l.
Hint eq_plus_reg_l.

Goal (n,m,p:nat)(gt n m)->(gt (plus n p) (plus m p)).
Intros;Elim (plus_sym p m);Elim (plus_sym p n);Auto.
Save gt_reg_r.
Hint gt_reg_r.

Goal (n,m,p,q:nat)(gt n m)->(gt p q)->(gt (plus n p)(plus m q)).
Induction 1;Intros;Simpl;Auto.
Save gt_plus_plus.
Hint gt_plus_plus.

Goal (p:nat)(gt p O)->(n:nat)(gt (plus p n) n).
Induction 1;Intros.
Auto.
Simpl;Auto.
Save gt_plus_l.
Hint gt_plus_l.

Goal (p:nat)(gt p O)->(n:nat)(gt (plus n p) n).
Intros;Elim (plus_sym p n);Auto.
Save gt_plus_r.
Hint gt_plus_r.

Goal (n,m,p:nat)(gt n m)->(gt (plus n p) m).
Auto.
Save gt_plus_trans_r.
Hint gt_plus_trans_r.

Goal (n,m,p:nat)(gt n m)->(gt (plus p n) m).
Intros;Elim (plus_sym n p);Auto.
Save gt_plus_trans_l.
Hint gt_plus_trans_l.

Goal (n:nat)((S n)=(plus n (S O))).
Induction n.
Auto.
Simpl;Auto.
Save S_plus.
Hint S_plus.
 
(* mult *)

Goal (n,m:nat)((mult n m)=(mult m n)).
Induction n;Intros.
Auto.
Simpl;Elim mult_n_Sm;Elim H;Auto.
Save mult_sym.
Hint mult_sym.

Goal (n,m,p:nat)((mult n (mult m p))=(mult (mult n m) p)).
Intros n m p;Elim n;Simpl.
Auto.
Intros y H;Rewrite mult_plus_distr;Elim H;Auto.
Save mult_assoc_l.
Hint mult_assoc_l.

Goal (n,m,p:nat)((mult n (mult m p))=(mult m (mult n p))).
Intros;Rewrite (mult_assoc_l m n p);Rewrite (mult_sym m n);Auto.
Save mult_permut.
Hint mult_permut. 

Goal (n,m,p:nat)((mult p (plus n m))=(plus (mult p n)(mult p m))).
Intros n m p;Elim p.
Auto.
Simpl;Intros y H;Rewrite H;Elim plus_assoc_l;Elim plus_assoc_l.
Pattern (plus (mult y n) (plus m (mult y m)));Elim plus_permute;Trivial.
Save mult_plus_distr_r.
Hint mult_plus_distr_r.

Goal (n:nat)((mult n (S(S O)))=(plus n n)).
Induction n.
Auto.
Intros;Simpl;Rewrite H;Auto.
Save mult_n_2. 
Hint mult_n_2.
 
Goal (n:nat)(n=(mult n (S O))).  
Induction n.
Auto.
Simpl;Auto.
Save mult_n_1. 
Hint mult_n_1.

Goal (n,m,p:nat)(n=m)->((mult p n)=(mult p m)).
Intros;Elim p;Elim H;Auto.
Save eq_mult_reg_r.
Hint eq_mult_reg_r.

Goal (n,m,p:nat)(n=m)->((mult n p)=(mult m p)).
Intros;Elim p;Elim H;Auto.
Save eq_mult_reg_l.
Hint eq_mult_reg_l.

Goal (p:nat)(gt p O)->(n,m:nat)(gt n m)->(gt (mult p n) (mult p m)).
Induction p.
Intros H n m H0;Absurd (gt O O);Auto.
Intros y H H0 n m H1;Elim (gt_O_eq y);Intros.
Simpl;Auto.
Elim H2;Simpl;Elim (plus_n_O n);Elim (plus_n_O m);Trivial.
Save gt_mult_reg_l.
Hint gt_mult_reg_l.

Goal (p:nat)(gt p O)->(n,m:nat)(gt n m)->(gt (mult n p)(mult m p)).
Intros;Elim (mult_sym p n);Elim (mult_sym p m);Auto.
Save gt_mult_reg_r.
Hint gt_mult_reg_r.

Goal (p:nat)(gt p (S O))->(n:nat)(gt n O)->(gt (mult p n) n).
Induction p.
Intros H n H0;Absurd (gt O (S O));Auto.
Intros y H H0 n H1;Simpl;Apply gt_plus_r;Replace O with (mult y O);Auto.
Save gt_mult_l.
Hint gt_mult_l.

Goal (p:nat)(gt p (S O))->(n:nat)(gt n O)->(gt (mult n p) n).
Intros;Elim (mult_sym p n);Auto.
Save gt_mult_r.
Hint gt_mult_r.

Goal (p:nat)(gt p O)->(n,m:nat)(gt n m)->(gt (mult p n) m).
Induction p.
Intros H n m H0;Simpl;Absurd (gt O O);Auto.
Intros y H H0 n m H1;Simpl;Elim (gt_O_eq y);Intro H2.
Auto.
Elim H2;Simpl;Elim plus_n_O;Assumption.
Save gt_mult_trans_r.
Hint gt_mult_trans_r.

Goal (p:nat)(gt p O)->(n,m:nat)(gt n m)->(gt (mult n p) m).
Intros;Elim (mult_sym p n);Auto.
Save gt_mult_trans_l.
Hint gt_mult_trans_l.

(* power2: (power2 n)=2^n *)

Fixpoint power2 [n:nat] : nat :=
 <nat>Case n of
            (* O *) (S O)
            (* S *) [p:nat](mult (S (S O)) (power2 p))  end.

Goal (n:nat)(gt (power2 n) O).
Induction n;Simpl;Intros.
Auto.
Elim plus_n_O;Auto.
Save gt_power2_O.
Hint gt_power2_O.

 

