(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             conf_local_SL.v                              *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)



                             (* Confluence locale de sigma-lift *)

Require TS.
Require sur_les_relations.
Require sigma_lift.
Require determinePC_SL.
Require resoudPC_SL.


Definition e_local1 := [b:wsort][x,y:(TS b)](z:(TS b))(relSL x z)->
                  (Ex( ([u:(TS b)] (relSLstar y u) /\ (relSLstar z u)))).

Syntactic Definition local1 := (e_local1 ?).
Syntax constr local1 <<(e_local1 $1 $t1 $t2)>> 10
  [<hov 0> "local1 " <$t1:"term":L> " " <$t2:"term":L>].

(* app *)

Goal (x,y:terms)(reg_app x y)->(local1 x y).
Induction 1;Red;Intros a b0 s z H0.
Pattern s z;Apply case_SL_reg_app with a b0;Auto.
Exists (app (env a s) (env b0 s));Auto.
Save local_app.
Hint local_app.

(* lambda *)

Goal (x,y:terms)(reg_lambda x y)->(local1 x y).
Induction 1;Red;Intros a s z H0.
Pattern s z;Apply case_SL_reg_lambda with a;Auto.
Exists (lambda (env a (lift s)));Auto.
Save local_lambda.
Hint local_lambda.

(* clos *)

Goal (x,y:terms)(reg_clos x y)->(local1 x y).
Induction 1;Red;Intros a s t z H0.
Pattern t z;Apply case_SL_clos with a s;Auto.
Exists (env a (comp s t));Auto.
Save local_clos.
Hint local_clos.

(* varshift1 *)

Goal (x,y:terms)(reg_varshift1 x y)->(local1 x y).
Induction 1;Red;Intros n z H0.
Pattern z;Apply case_SL_varshift1 with n;Auto.
Exists (var (S n));Auto.
Save local_varshift1.
Hint local_varshift1.

(* varshift2 *)

Goal (x,y:terms)(reg_varshift2 x y)->(local1 x y).
Induction 1;Red;Intros n s z H0.
Pattern z;Apply case_SL_varshift2 with n s;Auto.
Exists (env (var (S n)) s);Auto.
Save local_varshift2.
Hint local_varshift2.

(* fvarcons *)

Goal (x,y:terms)(reg_fvarcons x y)->(local1 x y).
Induction 1;Red;Intros a s z H0.
Pattern z;Apply case_SL_fvarcons with a s;Intros.
3:Assumption.
Exists a;Auto.
Apply PC_fvarcons_ctxt_r with s;Assumption.
Save local_fvarcons.
Hint local_fvarcons.

(* fvarlift1 *)

Goal (x,y:terms)(reg_fvarlift1 x y)->(local1 x y).
Induction 1;Red;Intros s z H0.
Pattern z;Apply case_SL_fvarlift1 with s;Intros.
3:Assumption.
Exists (var O);Auto.
Apply PC_fvarlift1_ctxt_r' with s;Assumption.
Save local_fvarlift1.
Hint local_fvarlift1.

(* fvarlift2 *)

Goal (x,y:terms)(reg_fvarlift2 x y)->(local1 x y).
Induction 1;Red;Intros s t z H0.
Pattern z;Apply case_SL_fvarlift2 with s t;Intros.
3:Assumption.
Exists (env (var O) t);Auto.
Apply PC_fvarlift2_ctxt_r with s;Assumption.
Save local_fvarlift2.
Hint local_fvarlift2.

(* rvarcons *)

Goal (x,y:terms)(reg_rvarcons x y)->(local1 x y).
Induction 1;Red;Intros n a s z H0.
Pattern z;Apply case_SL_rvarcons with n a s;Intros.
3:Assumption.
Exists (env (var n) s);Auto.
Apply PC_rvarcons_ctxt_r with a;Assumption.
Save local_rvarcons.
Hint local_rvarcons.

(* rvarlift1 *)

Goal (x,y:terms)(reg_rvarlift1 x y)->(local1 x y).
Induction 1;Red;Intros n s z H0.
Pattern z;Apply case_SL_rvarlift1 with n s;Auto.
Exists (env (var n) (comp s shift));Auto.
Save local_rvarlift1.
Hint local_rvarlift1.

(* rvarlift2 *)

Goal (x,y:terms)(reg_rvarlift2 x y)->(local1 x y).
Induction 1;Red;Intros n s t z H0.
Pattern z;Apply case_SL_rvarlift2 with n s t;Auto.
Exists (env (var n) (comp s (comp shift t)));Auto.
Save local_rvarlift2.
Hint local_rvarlift2.

(* assenv *)

Goal (x,y:sub_explicits)(reg_assenv x y)->(local1 x y).
Induction 1;Red;Intros s t u z H0.
Pattern u z;Apply case_SL_assenv with s t;Auto.
Exists (comp s (comp t u));Auto.
Save local_assenv.
Hint local_assenv.

(* mapenv *)

Goal (x,y:sub_explicits)(reg_mapenv x y)->(local1 x y).
Induction 1;Red;Intros a s t z H0.
Pattern t z;Apply case_SL_mapenv with a s;Auto.
Exists (cons (env a t) (comp s t));Auto.
Save local_mapenv.
Hint local_mapenv.

(* shiftcons *)

Goal (x,y:sub_explicits)(reg_shiftcons x y)->(local1 x y).
Induction 1;Red;Intros a s z H0.
Pattern z;Apply case_SL_shiftcons with a s;Intros.
3:Assumption.
Exists s;Auto.
Apply PC_shiftcons_ctxt_r with a;Assumption.
Save local_shiftcons.
Hint local_shiftcons.

(* shiftlift1 *)

Goal (x,y:sub_explicits)(reg_shiftlift1 x y)->(local1 x y).
Induction 1;Red;Intros s z H0.
Pattern z;Apply case_SL_shiflift1 with s;Auto.
Exists (comp s shift);Auto.
Save local_shiftlift1.
Hint local_shiftlift1.

(* shiftlift2 *)

Goal (x,y:sub_explicits)(reg_shiftlift2 x y)->(local1 x y).
Induction 1;Red;Intros s t z H0.
Pattern z;Apply case_SL_shiflift2 with s t;Auto.
Exists (comp s (comp shift t));Auto.
Save local_shiftlift2.
Hint local_shiftlift2.

(* lift1 *)

Goal (x,y:sub_explicits)(reg_lift1 x y)->(local1 x y).
Induction 1;Red;Intros s t z H0.
Pattern z;Apply case_SL_lift1 with s t;Auto.
Exists (lift (comp s t));Auto.
Save local_lift1.
Hint local_lift1.

(* lift2 *)

Goal (x,y:sub_explicits)(reg_lift2 x y)->(local1 x y).
Induction 1;Red;Intros s t u z H0.
Pattern z;Apply case_SL_lift2 with s t u;Auto.
Exists (comp (lift (comp s t)) u);Auto.
Save local_lift2.
Hint local_lift2.

(* liftenv *)

Goal (x,y:sub_explicits)(reg_liftenv x y)->(local1 x y).
Induction 1;Red;Intros a s t z H0.
Pattern z;Apply case_SL_liftenv with a s t;Auto.
Exists (cons a (comp s t));Auto.
Save local_liftenv.
Hint local_liftenv.

(* idl *)

Goal (x,y:sub_explicits)(reg_idl x y)->(local1 x y).
Induction 1;Red;Intros s z H0.
Pattern s z;Apply case_SL_idl;Auto.
Exists s;Auto.
Save local_idl.
Hint local_idl.

(* idr *)

Goal (x,y:sub_explicits)(reg_idr x y)->(local1 x y).
Induction 1;Red;Intros s z H0.
Apply Ex_PQ;Pattern s z;Apply case_SL_idr;Auto.
Exists s;Auto.
Save local_idr.
Hint local_idr.

(* liftid *)

Goal (x,y:sub_explicits)(reg_liftid x y)->(local1 x y).
Induction 1;Red;Intros z H0.
Pattern z;Apply case_SL_liftid;Auto.
Save local_liftid.
Hint local_liftid.

(* id *)

Goal (x,y:terms)(reg_id x y)->(local1 x y).
Induction 1;Red;Intros a z H0.
Apply Ex_PQ;Pattern a z;Apply case_SL_reg_id;Auto.
Exists a;Auto 6.
Save local_id.
Hint local_id.

(* systeme SL *)

Goal (b:wsort)(x,y:(TS b))(systemSL x y)->(local1 x y).
Induction 1;Auto.
Save local_systemSL.

Goal (b:wsort)(x,y:(TS b))(relSL x y)->(local1 x y).
Induction 1.
(* systemSL *)
Intros;Apply local_systemSL;Assumption.
(* contexte app gauche *)
Red;Intros a a' b0 H0 H1 z H2.
Pattern z;Apply case_SLapp with a b0.
3:Assumption.
Intros a'' H3;Elim (H1 a'' H3);Intros a_ H4.
Elim H4;Intros H5 H6.
Exists (app a_ b0);Auto.
Intros b0' H3;Exists (app a' b0');Auto.
(* contexte app droit *)
Red;Intros a b0 b0' H0 H1 z H2.
Pattern z;Apply case_SLapp with a b0.
3:Assumption.
Intros a' H3;Exists (app a' b0');Auto.
Intros b0'' H3;Elim (H1 b0'' H3);Intros b0_ H4.
Elim H4;Intros H5 H6.
Exists (app a b0_);Auto.
(* contexte lambda *)
Red;Intros a a' H0 H1 z H2.
Pattern z;Apply case_SLlambda with a.
2:Assumption.
Intros a'' H3;Elim (H1 a'' H3);Intros a_ H4;Elim H4;Intros H5 H6.
Exists (lambda a_);Auto.
(* contexte env gauche *)
Red;Intros a a' s H0 H1 z H2.
Apply Ex_PQ;Generalize H0;Pattern a s z;Apply case_SLenv;Auto.
Intros n H3;Elim (case_SLvar n a' H3).
Intros n s1 H3;Elim (case_SLvar n a' H3).
Intros a1 s1 H3;Elim (case_SLvar O a' H3).
Intros s1 H3;Elim (case_SLvar O a' H3).
Intros s1 s2 H3;Elim (case_SLvar O a' H3).
Intros n a1 s1 H3;Elim (case_SLvar (S n) a' H3).
Intros n s1 H3;Elim (case_SLvar (S n) a' H3).
Intros n s1 s2 H3;Elim (case_SLvar (S n) a' H3).
Intros a'' H3 H4;Elim (H1 a'' H3);Intros a_ H5;Elim H5;Intros H6 H7.
Exists (env a_ s);Auto.
Intros s' H3 H4;Exists (env a' s');Auto.
(* contexte env droit *)
Red;Intros a s s' H0 H1 z H2.
Apply Ex_PQ;Generalize H0;Pattern a s z;Apply case_SLenv;Auto.
Intros n H3;Elim (case_SLshift s' H3).
Intros;Apply PC_fvarcons_ctxt_r with s1;Assumption.
Intros;Apply PC_fvarlift1_ctxt_r' with s1;Assumption.
Intros;Apply PC_fvarlift2_ctxt_r with s1;Assumption.
Intros;Apply PC_rvarcons_ctxt_r with a1;Assumption.
Intro H3;Elim (case_SLid s' H3).
Intros a' H3 H4;Exists (env a' s');Auto.
Intros s'' H3 H4;Elim (H1 s'' H3);Intros s_ H5;Elim H5;Intros H6 H7.
Exists (env a s_);Auto.
(* contexte cons gauche *)
Red;Intros a a' s H0 H1 z H2.
Pattern z;Apply case_SLcons with a s;Auto.
Intros a'' H3;Elim (H1 a'' H3);Intros a_ H4;Elim H4;Intros H5 H6.
Exists (cons a_ s);Auto.
Intros s' H3;Exists (cons a' s');Auto.
(* contexte cons droit *)
Red;Intros a s s' H0 H1 z H2.
Pattern z;Apply case_SLcons with a s;Auto.
Intros a' H3;Exists (cons a' s');Auto.
Intros s'' H3;Elim (H1 s'' H3);Intros s_ H4;Elim H4;Intros H5 H6.
Exists (cons a s_);Auto.
(* contexte comp gauche *)
Red;Intros s s' t H0 H1 z H2.
Apply Ex_PQ;Generalize H0;Pattern s t z;Apply case_SLcomp;Auto.
Intros a t1 H3;Elim (case_SLshift s' H3).
Intros t1 H3;Elim (case_SLshift s' H3).
Intros t1 t2 H3;Elim (case_SLshift s' H3).
Intro H3;Elim (case_SLid s' H3).
Intros s'' H3;Elim (H1 s'' H3);Intros s_ H4;Elim H4;Intros H5 H6.
Exists (comp s_ t);Auto.
Intros t' H3;Exists (comp s' t');Auto.
(* contexte comp droit *)
Red;Intros s t t' H0 H1 z H2.
Apply Ex_PQ;Generalize H0;Pattern s t z;Apply case_SLcomp;Auto.
Intros;Apply PC_shiftcons_ctxt_r with a;Assumption.
Intro H3;Elim (case_SLid t' H3).
Intros s' H3;Exists (comp s' t');Auto.
Intros t'' H3;Elim (H1 t'' H3);Intros t_ H4;Elim H4;Intros H5 H6.
Exists (comp s t_);Auto.
(* contexte lift *)
Red;Intros s s' H0 H1 z H2.
Generalize H0;Pattern s z;Apply case_SLlift.
3:Assumption.
Intro H3;Elim (case_SLid s' H3).
Intros s'' H3;Elim (H1 s'' H3);Intros s_ H4;Elim H4;Intros H5 H6.
Exists (lift s_);Auto.
Save local_relSL.


(*************************************************************)
(*           sigma-lift est localement confluente            *)
(*************************************************************)


Theorem conf_local_SL: (b:wsort)(local_confluence (e_relSL b)).
Red;Red;Intros b x y z H H0.
Generalize z H0.
Change (local1 x y);Apply local_relSL;Assumption.
Save.


