(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               egaliteTS.v                                *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)

      
                      (* egalite dans les termes-sub_explicits (TS) *)

Require TS.

(***********************************************)
(*            Inegalites dans TS               *)
(***********************************************)

Goal (n:nat)(a,b:terms)~((var n)=(app a b)).
Intros;Discriminate.
Save diff_var_app.

Goal (n:nat)(a:terms)~((var n)=(lambda a)).
Intros;Discriminate.
Save diff_var_lambda.

Goal (n:nat)(a:terms)(s:sub_explicits)~((var n)=(env a s)).
Intros;Discriminate.
Save diff_var_env.

Goal (a,b,c:terms)~((app a b)=(lambda c)).
Intros;Discriminate.
Save diff_app_lambda.

Goal (a,b,c:terms)(s:sub_explicits)~((app a b)=(env c s)).
Intros;Discriminate.
Save diff_app_env.

Goal (a,b:terms)(s:sub_explicits)~((lambda a)=(env b s)).
Intros;Discriminate.
Save diff_lambda_env.

Goal ~(id=shift).
Intros;Discriminate.
Save diff_id_shift.

Goal (a:terms)(s:sub_explicits)~(id=(cons a s)).
Intros;Discriminate.
Save diff_id_cons.

Goal (s,t:sub_explicits)~(id=(comp s t)).
Intros;Discriminate.
Save diff_id_comp.

Goal (s:sub_explicits)~(id=(lift s)).
Intros;Discriminate.
Save diff_id_lift.

Goal (a:terms)(s:sub_explicits)~(shift=(cons a s)).
Intros;Discriminate.
Save diff_shift_cons.

Goal (s,t:sub_explicits)~(shift=(comp s t)).
Intros;Discriminate.
Save diff_shift_comp.

Goal (s:sub_explicits)~(shift=(lift s)).
Intros;Discriminate.
Save diff_shift_lift.

Goal (a:terms)(s,t,u:sub_explicits)~((cons a s)=(comp t u)).
Intros;Discriminate.
Save diff_cons_comp.

Goal (a:terms)(s,t:sub_explicits)~((cons a s)=(lift t)).
Intros;Discriminate.
Save diff_cons_lift.

Goal (s,t,u:sub_explicits)~((comp s t)=(lift u)).
Intros;Discriminate.
Save diff_comp_lift.

(***********************************************)
(*             Egalite dans TS                 *)
(***********************************************)

Goal (n1,n2:nat)((var n1)=(var n2))->(n1=n2). 
Intros n1 n2 H; Injection H; Trivial. 
Save proj_var.

Goal (a1,b1,a2,b2:terms)((app a1 b1)=(app a2 b2))->(a1=a2).
Intros a1 b1 a2 b2 H; Injection H; Trivial. 
Save proj_app1.

Goal (a1,b1,a2,b2:terms)((app a1 b1)=(app a2 b2))->(b1=b2).
Intros a1 b1 a2 b2 H; Injection H; Trivial. 
Save proj_app2.

Goal (a,b:terms)((lambda a)=(lambda b))->(a=b).
Intros a b H; Injection H; Trivial. 
Save proj_lambda.

Goal (a,b:terms)(s,t:sub_explicits)((env a s)=(env b t))->(a=b).
Intros a b s t H; Injection H; Trivial. 
Save proj_env1.

Goal (a,b:terms)(s,t:sub_explicits)((env a s)=(env b t))->(s=t).
Intros a b s t H; Injection H; Trivial. 
Save proj_env2.

Goal (a,b:terms)(s,t:sub_explicits)((cons a s)=(cons b t))->(a=b).
Intros a b s t H; Injection H; Trivial. 
Save proj_cons1.

Goal (a,b:terms)(s,t:sub_explicits)((cons a s)=(cons b t))->(s=t).
Intros a b s t H; Injection H; Trivial. 
Save proj_cons2.

Goal (s1,s2,t1,t2:sub_explicits)((comp s1 t1)=(comp s2 t2))->(s1=s2).
Intros s1 s2 t1 t2 H; Injection H; Trivial. 
Save proj_comp1.

Goal (s1,s2,t1,t2:sub_explicits)((comp s1 t1)=(comp s2 t2))->(t1=t2).
Intros s1 s2 t1 t2 H; Injection H; Trivial. 
Save proj_comp2.

Goal (s,t:sub_explicits)((lift s)=(lift t))->(s=t).
Intros s t H; Injection H; Trivial. 
Save proj_lift.





