(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                           lambda_sigma_lift.v                            *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


               (* theorie lambda-sigma-lift-calcul *) 

Require TS.
Require sur_les_relations.
Require sigma_lift.


(* regle beta *)

Inductive reg_beta: terms->terms->Prop 
:= reg1_beta: (a,b:terms)(reg_beta (app (lambda a) b) (env a (cons b id))).
Hint reg1_beta.

(* systeme lambda-sigma-lift *)

Inductive e_systemLSL: (b:wsort)(TS b)->(TS b)->Prop
:=  beta1: (M,N:terms)(reg_beta M N)->(e_systemLSL wt M N)
 | SL1: (b:wsort)(M,N:(TS b))(systemSL M N)->(e_systemLSL b M N). 
Hint beta1 SL1.

Syntactic Definition systemLSL := (e_systemLSL ?).
Syntax constr systemLSL <<(e_systemLSL $1 $t1 $t2)>> 10
  [<hov 0> "systemLSL " <$t1:"term":L> " " <$t2:"term":L>].

(* relation engendree par le systeme lambda-sigma-lift *)

Inductive e_relLSL: (b:wsort)(TS b)->(TS b)->Prop
:=  LSL_one_regle: (b:wsort)(M,N:(TS b))(systemLSL M N)->(e_relLSL b M N)
 | LSL_context_app_l: (a,a',b:terms)
    (e_relLSL wt a a')->(e_relLSL wt (app a b) (app a' b))
 | LSL_context_app_r: (a,b,b':terms)(e_relLSL wt b b')->
                                      (e_relLSL wt (app a b) (app a b'))  
 | LSL_context_lambda: (a,a':terms)(e_relLSL wt a a')->
                                      (e_relLSL wt (lambda a) (lambda a'))
 | LSL_context_env_t: (a,a':terms)(s:sub_explicits)(e_relLSL wt a a')->
                                               (e_relLSL wt (env a s) (env a' s))
 | LSL_context_env_s: (a:terms)(s,s':sub_explicits)(e_relLSL ws s s')->
					       (e_relLSL wt (env a s) (env a s'))
 | LSL_context_cons_t: (a,a':terms)(s:sub_explicits)(e_relLSL wt a a')->
	       (e_relLSL ws (cons a s) (cons a' s)) 
 | LSL_context_cons_s: (a:terms)(s,s':sub_explicits)(e_relLSL ws s s')->
			(e_relLSL ws (cons a s) (cons a s'))
 | LSL_context_comp_l: (s,s',t:sub_explicits)(e_relLSL ws s s')->
				         (e_relLSL ws (comp s t) (comp s' t))
 | LSL_context_comp_r: (s,t,t':sub_explicits)(e_relLSL ws t t')->
                                         (e_relLSL ws (comp s t) (comp s t'))
 | LSL_context_lift: (s,s':sub_explicits)(e_relLSL ws s s')->
                                         (e_relLSL ws (lift s) (lift s')).

Syntactic Definition relLSL := (e_relLSL ?).
Syntax constr relLSL <<(e_relLSL $1 $t1 $t2)>> 10
  [<hov 0> "relLSL " <$t1:"term":L> " " <$t2:"term":L>].

Hint LSL_one_regle LSL_context_app_l LSL_context_app_r LSL_context_lambda
     LSL_context_env_t LSL_context_env_s LSL_context_cons_t LSL_context_cons_s
     LSL_context_comp_l LSL_context_comp_r LSL_context_lift.

(* fermeture reflexive-transitive de la relation lambda-sigma-lift *)

Definition e_relLSLstar := [b:wsort] (star (e_relLSL b)).

Syntactic Definition relLSLstar := (e_relLSLstar ?).
Syntax constr relLSLstar <<(e_relLSLstar $1 $t1 $t2)>> 10
  [<hov 0> "relLSLstar " <$t1:"term":L> " " <$t2:"term":L>].

Hint Unfold e_relLSLstar.

(* un exemple *)

Goal (relLSLstar (lambda (app (lambda (app (var O)(var O))) 
                                    (lambda (app (var O)(var (S O))))))
                               (lambda (app (var O)(var O)))).
Red;Apply star_trans1 with (lambda (env (app (var O)(var O)) 
                                (cons (lambda (app (var O)(var (S O)))) id ))).
Auto.
Apply star_trans1 with (lambda (app (env (var O)(cons (lambda 
                                                    (app (var O)(var (S O)))) id))
                                        (env (var O) (cons (lambda 
                                              (app (var O)(var (S O)))) id)))).
Auto.
Apply star_trans1 with (lambda (app (lambda (app (var O)(var (S O))))
                     (env (var O) (cons (lambda (app (var O)(var (S O)))) id)))).
Auto 6.
Apply star_trans1 with (lambda (app (lambda (app (var O)(var (S O))))
                                    (lambda (app (var O)(var (S O)))))).
Auto 6.
Apply star_trans1 with (lambda (env (app (var O)(var (S O))) 
                                    (cons (lambda (app (var O)(var (S O)))) id))).
Auto.
Apply star_trans1 with (lambda (app (env (var O)(cons (lambda (app (var O)
                                                          (var (S O)))) id))
                              (env (var (S O))(cons (lambda (app (var O)
                                                            (var (S O)))) id)))).
Auto.
Apply star_trans1 with (lambda (app (lambda (app (var O)(var (S O))))
                             (env (var (S O))(cons (lambda (app (var O)
                                                (var (S O)))) id)))).
Auto 6.


Apply star_trans1 with (lambda (app (lambda (app (var O)(var (S O))))
                           (env (var O) id))).
Auto 6.
Apply star_trans1 with (lambda (env (app (var O)(var (S O)))
                  (cons (env (var O) id) id))). 
Auto.
Apply star_trans1 with (lambda (app (env (var O)(cons (env (var O) id) id))   
                                    (env (var (S O))(cons (env (var O) id) id)))).
Auto 6.
Apply star_trans1 with (lambda (app (env (var O) id)
                                    (env (var (S O))(cons (env (var O) id) id)))). 
Auto 6.
Apply star_trans1 with (lambda (app (env (var O) id)(env (var O) id))).
Auto 6.
Apply star_trans1 with (lambda (app (var O)(env (var O) id))).
Auto 6.
Apply star_trans1 with (lambda (app (var O)(var O)));Auto 6.
Save exemple.

(* *)

Goal (a,a',b:terms)(relLSLstar a a')->(relLSLstar (app a b) (app a' b)).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (app y b);Auto.
Save LSLstar_context_app_l.
Hint LSLstar_context_app_l.

Goal (a,b,b':terms)(relLSLstar b b')->(relLSLstar (app a b) (app a b')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (app a y);Auto.
Save LSLstar_context_app_r.
Hint LSLstar_context_app_r.

Goal (a,a',b,b':terms)(relLSLstar a a')->(relLSLstar b b')->
         (relLSLstar (app a b) (app a' b')).
Intros;Red.
Apply star_trans with (app a' b).
Change (relLSLstar (app a b) (app a' b));Auto.
Change (relLSLstar (app a' b) (app a' b'));Auto.
Save LSLstar_context_app.
Hint LSLstar_context_app.

Goal (a,a':terms)(relLSLstar a a')->(relLSLstar (lambda a) (lambda a')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (lambda y);Auto.
Save LSLstar_context_lambda.
Hint LSLstar_context_lambda.

Goal (a,a':terms)(s:sub_explicits)(relLSLstar a a')->
                                        (relLSLstar (env a s) (env a' s)).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (env y s);Auto.
Save LSLstar_context_env_t.
Hint LSLstar_context_env_t.

Goal (a:terms)(s,s':sub_explicits)(relLSLstar s s')->
                                        (relLSLstar (env a s) (env a s')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (env a y);Auto.
Save LSLstar_context_env_s.
Hint LSLstar_context_env_s.

Goal (a,a':terms)(s,s':sub_explicits)(relLSLstar a a')->(relLSLstar s s')->
                                        (relLSLstar (env a s) (env a' s')).
Intros;Red.
Apply star_trans with (env a' s).
Change (relLSLstar (env a s) (env a' s));Auto.
Change (relLSLstar (env a' s) (env a' s'));Auto.
Save LSLstar_context_env.
Hint LSLstar_context_env.

Goal (a,a':terms)(s:sub_explicits)(relLSLstar a a')->
                (relLSLstar (cons a s) (cons a' s)). 
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (cons y s);Auto.
Save LSLstar_context_cons_t.
Hint LSLstar_context_cons_t.

Goal (a:terms)(s,s':sub_explicits)(relLSLstar s s')->
                (relLSLstar (cons a s) (cons a s')). 
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (cons a y);Auto.
Save LSLstar_context_cons_s.
Hint LSLstar_context_cons_s.

Goal (a,a':terms)(s,s':sub_explicits)(relLSLstar a a')->(relLSLstar s s')->
                                        (relLSLstar (cons a s) (cons a' s')).
Intros;Red.
Apply star_trans with (cons a' s).
Change (relLSLstar (cons a s) (cons a' s));Auto.
Change (relLSLstar (cons a' s) (cons a' s'));Auto.
Save LSLstar_context_cons.
Hint LSLstar_context_cons.

Goal (s,s',t:sub_explicits)(relLSLstar s s')->
			  (relLSLstar (comp s t) (comp s' t)).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (comp y t);Auto.
Save LSLstar_context_comp_l.
Hint LSLstar_context_comp_l.

Goal (s,t,t':sub_explicits)(relLSLstar t t')->
			  (relLSLstar (comp s t) (comp s t')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (comp s y);Auto.
Save LSLstar_context_comp_r.
Hint LSLstar_context_comp_r.

Goal (s,s',t,t':sub_explicits)(relLSLstar t t')->(relLSLstar s s')->
                                        (relLSLstar (comp s t) (comp s' t')).
Intros;Red.
Apply star_trans with (comp s' t).
Change (relLSLstar (comp s t) (comp s' t));Auto.
Change (relLSLstar (comp s' t) (comp s' t'));Auto.
Save LSLstar_context_comp.
Hint LSLstar_context_comp.

Goal  (s,s':sub_explicits)(relLSLstar s s')->
                                  (relLSLstar (lift s) (lift s')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (lift y);Auto.
Save LSLstar_context_lift.
Hint LSLstar_context_lift.


