(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              resoudPC_SL.v                               *)
(****************************************************************************)


      (* confluence locale de sigma_lift: resolution des paires critiques *)

Require TS.
Require sur_les_relations.
Require sigma_lift.
Require determinePC_SL.

(*** app ***)

Goal (a,b:terms)
     (Ex( ([u:terms] (relSLstar (app (env a id) (env b id)) u) /\
                        (relSLstar (app  a b) u) ))).
Intros; Exists (app a b);Split;Red.
Apply star_trans1 with (app a (env b id));Auto.
Auto.
Save PC_app_id.
Hint PC_app_id.

Goal (a,a',b:terms)(s:sub_explicits)(relSL a a')->
     (Ex(([u:terms] ((relSLstar (app (env a s) (env b s)) u) /\
                        (relSLstar (env (app a' b) s) u))))).
Intros; Exists (app (env a' s) (env b s));Auto 6.
Save PC1_app_ctxt_l.
Hint PC1_app_ctxt_l.

Goal (a,b,b':terms)(s:sub_explicits)(relSL b b')->
     (Ex(([u:terms](relSLstar (app (env a s) (env b s)) u)/\
                        (relSLstar (env (app a b') s) u)))).
Intros; Exists (app (env a s) (env b' s));Auto 6.
Save PC2_app_ctxt_l.
Hint PC2_app_ctxt_l.

Goal (a,b:terms)(s,s':sub_explicits)(relSL s s')->
     (Ex(([u:terms](relSLstar (app (env a s) (env b s)) u)/\
                        (relSLstar (env (app a b) s') u)))).
Intros; Exists (app (env a s') (env b s'));Split;Red.
Apply star_trans1 with (app (env a s') (env b s));Auto.
Auto.
Save PC_app_ctxt_r.
Hint PC_app_ctxt_r.

Goal (a,b,x':terms)(s:sub_explicits)(relSL (app a b) x')->
     (Ex(([u:terms](relSLstar (app (env a s) (env b s)) u)/\
                        (relSLstar (env x' s) u)))).
Intros a b x' s H;Pattern x';Apply case_SLapp with a b;Auto.
Save PC_app_ctxt_l.
Hint PC_app_ctxt_l.

(*** lambda ***)

Goal (a:terms)
     (Ex(([u:terms](relSLstar (lambda (env a (lift id))) u)/\
                        (relSLstar (lambda a) u)))).
Intro; Exists (lambda a);Split;Red.
Apply star_trans1 with (lambda (env a id));Auto.
Auto.
Save PC_lambda_id.
Hint PC_lambda_id.

Goal (a,x':terms)(s:sub_explicits)(relSL (lambda a) x')->
     (Ex(([u:terms](relSLstar (lambda (env a (lift s))) u)/\
                        (relSLstar (env x' s) u)))).
Intros a x' s H;Pattern x';Apply case_SLlambda  with a;Intros.
2:Assumption.
Exists (lambda (env a' (lift s)));Auto 6.
Save PC_lambda_ctxt_l.
Hint PC_lambda_ctxt_l.

Goal (a:terms)(s,s':sub_explicits)(relSL s s')->
     (Ex(([u:terms](relSLstar (lambda (env a (lift s))) u)/\
                        (relSLstar (env (lambda a) s') u)))).
Intros; Exists (lambda (env a (lift s')));Auto 8.
Save PC_lambda_ctxt_r.
Hint PC_lambda_ctxt_r.

(*** Clos ***)

Goal (a:terms)(s:sub_explicits)
     (Ex(([u:terms](relSLstar (env a (comp s id)) u)/\
                        (relSLstar (env a s) u)))).
Intros; Exists (env a s);Split;Red;Auto.
Save PC1_clos_id.
Hint PC1_clos_id.

Goal (a,b:terms)(s,t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (app a b) (comp s t)) u)/\
                        (relSLstar (env (app (env a s) (env b s)) t) u)))).
Intros; Exists (app (env a (comp s t)) (env b (comp s t)));Split;Red.
Auto.
Apply star_trans1 with (app (env (env a s) t) (env (env b s) t)).
Auto.
Apply star_trans1 with (app (env a (comp s t)) (env (env b s) t));Auto.
Save PC_clos_app.
Hint PC_clos_app.

Goal (a:terms)(s,t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (lambda a) (comp s t)) u)/\
                        (relSLstar (env (lambda (env a (lift s))) t) u)))).
Intros; Exists (lambda (env a (lift (comp s t))));Split;Red.
Auto.
Apply star_trans1 with (lambda (env (env a (lift s)) (lift t))).
Auto.
Apply star_trans1 with (lambda (env a (comp (lift s) (lift t))));Auto 6.
Save PC_clos_lambda.
Hint PC_clos_lambda.

Goal (a:terms)(s,s1,t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (env a s1) (comp s t)) u)/\
                        (relSLstar (env (env a (comp s1 s)) t) u)))).
Intros; Exists (env a (comp s1 (comp s t)));Split;Red.
Auto.
Apply star_trans1 with (env a (comp (comp s1 s) t));Auto.
Save PC_clos_clos.
Hint PC_clos_clos.

Goal (n:nat)(t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var n) (comp shift t)) u)/\
                        (relSLstar (env (var (S n)) t) u)))).
Intros; Exists (env (var (S n)) t);Auto 6.
Save PC_clos_varshift1.
Hint PC_clos_varshift1.

Goal (n:nat)(s,t:sub_explicits)(Ex(([u:terms]
        (relSLstar (env (var n) (comp (comp shift s) t)) u)/\
        (relSLstar (env (env (var (S n)) s) t) u)))).
Intros; Exists (env (var (S n)) (comp s t));Split;Red.
Apply star_trans1 with (env (var n) (comp shift (comp s t)));Auto.
Auto.
Save PC_clos_varshift2.
Hint PC_clos_varshift2.

Goal (a:terms)(s,t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) (comp (cons a s) t)) u)/\
                        (relSLstar (env a t) u)))).
Intros; Exists (env a t);Split;Red.
Apply star_trans1 with (env (var O) (cons (env a t) (comp s t)));Auto.
Auto.
Save PC_clos_fvarcons.
Hint PC_clos_fvarcons.

Goal (s,t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) (comp (lift s) t)) u)/\
                        (relSLstar (env (var O) t) u)))).
Intros; Exists (env (var O) t);Auto 6.
Save PC_clos_fvarlift1.
Hint PC_clos_fvarlift1.

Goal (s1,s2,t:sub_explicits)(Ex(([u:terms]
       (relSLstar (env (var O) (comp (comp (lift s1) s2) t)) u)/\
                        (relSLstar (env (env (var O) s2) t) u)))).
Intros; Exists (env (var O) (comp s2 t));Split;Red.
Apply star_trans1 with (env (var O) (comp (lift s1) (comp s2 t)));Auto.
Auto.
Save PC_clos_fvarlift2.
Hint PC_clos_fvarlift2.

Goal (n:nat)(a:terms)(s,t:sub_explicits)(Ex(([u:terms]
        (relSLstar (env (var (S n)) (comp (cons a s) t)) u)/\
        (relSLstar (env (env (var n) s) t) u)))).
Intros; Exists (env (var n) (comp s t));Split;Red.
Apply star_trans1 with (env (var (S n)) (cons (env a t) (comp s t)));Auto.
Auto.
Save PC_clos_rvarcons.
Hint PC_clos_rvarcons.

Goal (n:nat)(s,t:sub_explicits)(Ex(([u:terms]
          (relSLstar (env (var (S n)) (comp (lift s) t)) u)/\
          (relSLstar (env (env (var n) (comp s shift)) t) u)))).
Intros; Exists (env (var n) (comp s (comp shift t)));Split;Red.
Auto.
Apply star_trans1 with (env (var n) (comp (comp s shift) t));Auto.
Save PC_clos_rvarlift1.
Hint PC_clos_rvarlift1.

Goal (n:nat)(s1,s2,t:sub_explicits)(Ex(([u:terms]
           (relSLstar (env (var (S n)) (comp (comp (lift s1) s2) t)) u)/\
           (relSLstar (env (env (var n) (comp s1 (comp shift s2))) t) u)))).
Intros; Exists (env (var n) (comp s1 (comp shift (comp s2 t))));Split;Red.
Apply star_trans1 with (env (var (S n)) (comp (lift s1) (comp s2 t)));Auto.
Apply star_trans1 with (env (var n) (comp (comp s1 (comp shift s2)) t)).
Auto.
Apply star_trans1 with (env (var n) (comp s1 (comp (comp shift s2) t)));Auto 6.
Save PC_clos_rvarlift2.
Hint PC_clos_rvarlift2.

Goal (a:terms)(t:sub_explicits)
     (Ex(([u:terms](relSLstar (env a (comp id t)) u)/\
                        (relSLstar (env a t) u)))).
Intros; Exists (env a t);Auto 8.
Save PC2_clos_id.
Hint PC2_clos_id.

Goal (a,a':terms)(s,t:sub_explicits)(relSL a a')->
     (Ex(([u:terms](relSLstar (env a (comp s t)) u)/\
                        (relSLstar (env (env a' s) t) u)))).
Intros; Exists (env a' (comp s t));Auto 8.
Save PC1_clos_ctxt_l.
Hint PC1_clos_ctxt_l.

Goal (a:terms)(s,s',t:sub_explicits)(relSL s s')->
     (Ex(([u:terms](relSLstar (env a (comp s t)) u)/\
                        (relSLstar (env (env a s') t) u)))).
Intros; Exists (env a (comp s' t));Auto 6.
Save PC2_clos_ctxt_l.
Hint PC2_clos_ctxt_l.

Goal (a:terms)(s,t,t':sub_explicits)(relSL t t')->
     (Ex(([u:terms](relSLstar (env a (comp s t)) u)/\
                        (relSLstar (env (env a s) t') u)))).
Intros; Exists (env a (comp s t'));Auto 6.
Save PC_clos_ctxt_r.
Hint PC_clos_ctxt_r.

Goal (a,x':terms)(s,t:sub_explicits)(relSL (env a s) x')->
     (Ex(([u:terms](relSLstar (env a (comp s t)) u)/\
                        (relSLstar (env x' t) u)))).
Intros a x' s t H;Pattern a s x';Apply case_SLenv;Auto.
Save PC_clos_ctxt_l.
Hint PC_clos_ctxt_l.

(*** varshift1 ***)

 (* aucune PC *)

(*** varshift2 ***)

Goal (n:nat)(a:terms)(s:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var (S n)) (cons a s)) u)/\
                        (relSLstar (env (var n) s) u)))).
Intros; Exists (env (var n) s);Auto 6.
Save PC_varshift2_shiftcons.
Hint PC_varshift2_shiftcons.

Goal (n:nat)(s:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var (S n)) (lift s)) u)/\
                        (relSLstar (env (var n) (comp s shift)) u)))).
Intros; Exists (env (var n) (comp s shift));Auto 6.
Save PC_varshift2_shiftlift1.
Hint PC_varshift2_shiftlift1.

Goal (n:nat)(s,t:sub_explicits)(Ex(([u:terms]
     (relSLstar (env (var (S n)) (comp (lift s) t)) u)/\
     (relSLstar (env (var n) (comp s (comp shift t))) u)))).
Intros; Exists (env (var n) (comp s (comp shift t)));Auto 6.
Save PC_varshift2_shiftlift2.
Hint PC_varshift2_shiftlift2.

Goal (n:nat)
     (Ex(([u:terms](relSLstar (env (var (S n)) id) u)/\
                        (relSLstar (env (var n) shift) u)))).
Intros; Exists (var (S n));Auto 6.
Save PC_varshift2_idr.
Hint PC_varshift2_idr.

Goal (n:nat)(s,s':sub_explicits)(relSL s s')->
     (Ex(([u:terms](relSLstar (env (var (S n)) s) u)/\
                        (relSLstar (env (var n) (comp shift s')) u)))).
Intros; Exists (env (var (S n)) s');Auto 6.
Save PC_varshift2_ctxt_r.
Hint PC_varshift2_ctxt_r.

Goal (n:nat)(s,x':sub_explicits)(relSL (comp shift s) x')->
     (Ex(([u:terms](relSLstar (env (var (S n)) s) u)/\
                        (relSLstar (env (var n) x') u)))).
Intros n s x' H;Pattern s x';Apply case_SLcomp1;Auto.
Save PC_varshift2_ctxt_r'.
Hint PC_varshift2_ctxt_r'.

(*** fvarcons ***)

Goal (a,a':terms)(s:sub_explicits)(relSL a a')->
     (Ex(([u:terms](relSLstar a u)/\
                        (relSLstar (env (var O) (cons a' s)) u)))).
Intros; Exists a';Auto 6.
Save PC1_fvarcons_ctxt_r.
Hint PC1_fvarcons_ctxt_r.

Goal (a:terms)(s':sub_explicits)
     (Ex(([u:terms](relSLstar a u)/\
                        (relSLstar (env (var O) (cons a s')) u)))).
Intros; Exists a;Auto 6.
Save PC2_fvarcons_ctxt_r.
Hint PC2_fvarcons_ctxt_r.

Goal (a:terms)(s,x':sub_explicits)(relSL (cons a s) x')->
     (Ex(([u:terms](relSLstar a u)/\
                        (relSLstar (env (var O) x') u)))).
Intros a s x' H;Pattern x';Apply case_SLcons with a s;Auto.
Save PC_fvarcons_ctxt_r. 

(*** fvarlift1 ***)

Goal (Ex(([u:terms](relSLstar (var O) u)/\
                        (relSLstar (env (var O) id) u)))).
Intros; Exists (var O);Auto 6.
Save PC_fvarlift1_liftid.
Hint PC_fvarlift1_liftid.

Goal (s':sub_explicits)
     (Ex(([u:terms](relSLstar (var O) u)/\
                        (relSLstar (env (var O) (lift s')) u)))).
Intros; Exists (var O);Auto 6.
Save PC_fvarlift1_ctxt_r.
Hint PC_fvarlift1_ctxt_r.

Goal (s,x':sub_explicits)(relSL (lift s) x')->
     (Ex(([u:terms](relSLstar (var O) u)/\
                        (relSLstar (env (var O) x') u)))).
Intros s x' H;Pattern s x';Apply case_SLlift;Auto.
Save PC_fvarlift1_ctxt_r'.

(*** fvarlift2 ***)

Goal (s,t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) (lift t)) u)/\
                        (relSLstar (env (var O) (lift (comp s t))) u)))).
Intros; Exists (var O);Auto 6.
Save PC_fvarlift2_lift1.
Hint PC_fvarlift2_lift1.

Goal (s,t,v:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) (comp (lift t) v)) u)/\
      (relSLstar (env (var O) (comp (lift (comp s t)) v)) u)))).
Intros; Exists (env (var O) v);Auto 6.
Save PC_fvarlift2_lift2.
Hint PC_fvarlift2_lift2.

Goal (a:terms)(s,t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) (cons a t)) u)/\
                        (relSLstar (env (var O) (cons a (comp s t))) u)))).
Intros; Exists a;Auto 6.
Save PC_fvarlift2_liftenv.
Hint PC_fvarlift2_liftenv.

Goal (s:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) id) u)/\
                        (relSLstar (env (var O) (lift s)) u)))).
Exists (var O);Auto 6.
Save PC_fvarlift2_idr.
Hint PC_fvarlift2_idr.

Goal (t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) t) u)/\
                        (relSLstar (env (var O) (comp id t)) u)))).
Intros; Exists (env (var O) t);Auto 7.
Save PC_fvarlift2_liftid.
Hint PC_fvarlift2_liftid.

Goal (s',t:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var O) t) u)/\
                        (relSLstar (env (var O) (comp (lift s') t)) u)))).
Intros; Exists (env (var O) t);Auto 6.
Save PC1_fvarlift2_ctxt_r.
Hint PC1_fvarlift2_ctxt_r.

Goal (s,t,t':sub_explicits)(relSL t t')->
     (Ex(([u:terms](relSLstar (env (var O) t) u)/\
                        (relSLstar (env (var O) (comp (lift s) t')) u)))).
Intros; Exists (env (var O) t');Auto 6.
Save PC2_fvarlift2_ctxt_r.
Hint PC2_fvarlift2_ctxt_r.

Goal (s,t,x':sub_explicits)(relSL (comp (lift s) t) x')->
     (Ex(([u:terms](relSLstar (env (var O) t) u)/\
                        (relSLstar (env (var O) x') u)))).
Intros s t x' H;Pattern t x';Apply case_SLcomp2 with s;Auto.
Intros;Pattern s x'0;Apply case_SLlift;Auto.
Save PC_fvarlift2_ctxt_r.

(*** rvarcons ***)

Goal (n:nat)(a':terms)(s:sub_explicits)
     (Ex(([u:terms](relSLstar (env (var n) s) u)/\
                        (relSLstar (env (var (S n)) (cons a' s)) u)))).
Intros; Exists (env (var n) s);Auto 6.
Save PC1_rvarcons_ctxt_r.
Hint PC1_rvarcons_ctxt_r.

Goal (n:nat)(a:terms)(s,s':sub_explicits)(relSL s s')->
     (Ex(([u:terms](relSLstar (env (var n) s) u)/\
                        (relSLstar (env (var (S n)) (cons a s')) u)))).
Intros; Exists (env (var n) s');Auto 6.
Save PC2_rvarcons_ctxt_r.
Hint PC2_rvarcons_ctxt_r.

Goal (n:nat)(a:terms)(s,x':sub_explicits)(relSL (cons a s) x')->
     (Ex(([u:terms](relSLstar (env (var n) s) u)/\
                        (relSLstar (env (var (S n)) x') u)))).
Intros n a s x' H;Pattern x';Apply case_SLcons with a s;Auto.
Save PC_rvarcons_ctxt_r.

(*** rvarlift1 ***)

Goal (n:nat)
     (Ex(([u:terms](relSLstar (env (var n) (comp id shift)) u)/\
                        (relSLstar (env (var (S n)) id) u)))).
Intros; Exists (var (S n));Split;Red.
Apply star_trans1 with (env (var n) shift);Auto.
Auto.
Save PC_rvarlift1_id.
Hint PC_rvarlift1_id.

Goal (n:nat)(s,s':sub_explicits)(relSL s s')->
     (Ex(([u:terms](relSLstar (env (var n) (comp s shift)) u)/\
                        (relSLstar (env (var (S n)) (lift s')) u)))).
Intros; Exists (env (var n) (comp s' shift));Auto 6.
Save PC_rvarlift1_ctxt_r.
Hint PC_rvarlift1_ctxt_r.

Goal (n:nat)(s,x':sub_explicits)(relSL (lift s) x')->
     (Ex(([u:terms](relSLstar (env (var n) (comp s shift)) u)/\
                        (relSLstar (env (var (S n)) x') u)))).
Intros n s x' H;Pattern s x';Apply case_SLlift;Auto.
Save PC_rvarlift1_ctxt_r'.
Hint PC_rvarlift1_ctxt_r'.

(*** rvarlift2 ***)

Goal (n:nat)(s,t:sub_explicits)(Ex(([u:terms]
      (relSLstar (env (var n) (comp s (comp shift (lift t)))) u)/\
      (relSLstar (env (var (S n)) (lift (comp s t))) u)))).
Intros; Exists (env (var n) (comp s (comp t shift)));Split;Red.
Auto 6.
Apply star_trans1 with (env (var n) (comp (comp s t) shift));Auto.
Save PC_rvarlift2_lift1.
Hint PC_rvarlift2_lift1.

Goal (n:nat)(s,t,v:sub_explicits)(Ex(([u:terms]
  (relSLstar (env (var n) (comp s (comp shift (comp (lift t) v)))) u)/\
  (relSLstar (env (var (S n)) (comp (lift (comp s t)) v)) u)))).
Intros; Exists (env (var n) (comp s (comp t (comp shift v))));Split;Red.
Auto 6.
Apply star_trans1 with (env (var n) (comp (comp s t) (comp shift v)));Auto.
Save PC_rvarlift2_lift2.
Hint PC_rvarlift2_lift2.

Goal (n:nat)(a:terms)(s,t:sub_explicits)(Ex(([u:terms]
      (relSLstar (env (var n) (comp s (comp shift (cons a t)))) u)/\
      (relSLstar (env (var (S n)) (cons a (comp s t))) u)))).
Intros; Exists (env (var n) (comp s t));Auto 8.
Save PC_rvarlift2_liftenv.
Hint PC_rvarlift2_liftenv.

Goal (n:nat)(s:sub_explicits)(Ex(([u:terms]
      (relSLstar (env (var n) (comp s (comp shift id))) u)/\
      (relSLstar (env (var (S n)) (lift s)) u)))).
Intros; Exists (env (var n) (comp s shift));Auto 8.
Save PC_rvarlift2_idr.
Hint PC_rvarlift2_idr.

Goal (n:nat)(t:sub_explicits)(Ex(([u:terms]
      (relSLstar (env (var n) (comp id (comp shift t))) u)/\
      (relSLstar (env (var (S n)) (comp id t)) u)))).
Intros; Exists (env (var (S n)) t);Split;Red.
Apply star_trans1 with (env (var n) (comp shift t));Auto.
Auto.
Save PC_rvarlift2_liftid.
Hint PC_rvarlift2_liftid.

Goal (n:nat)(s,s',t:sub_explicits)(relSL s s')->
 (Ex(([u:terms](relSLstar (env (var n) (comp s (comp shift t))) u)/\
                    (relSLstar (env (var (S n)) (comp (lift s') t)) u)))).
Intros; Exists (env (var n) (comp s' (comp shift t)));Auto 6.
Save PC1_rvarlift2_ctxt_r.
Hint PC1_rvarlift2_ctxt_r.

Goal (n:nat)(s,t,t':sub_explicits)(relSL t t')->
 (Ex(([u:terms](relSLstar (env (var n) (comp s (comp shift t))) u)/\
                    (relSLstar (env (var (S n)) (comp (lift s) t')) u)))).
Intros; Exists (env (var n) (comp s (comp shift t')));Auto 7.
Save PC2_rvarlift2_ctxt_r.
Hint PC2_rvarlift2_ctxt_r.

Goal (n:nat)(s,t,x':sub_explicits)(relSL (comp (lift s) t) x')->
 (Ex(([u:terms](relSLstar (env (var n) (comp s (comp shift t))) u)/\
                    (relSLstar (env (var (S n)) x') u)))).
Intros n s t x' H;Pattern t x';Apply case_SLcomp2 with s;Auto.
Intros;Pattern s x'0;Apply case_SLlift;Auto.
Save PC_rvarlift2_ctxt_r.
Hint PC_rvarlift2_ctxt_r.

(*** assenv ***) 

Goal (s1,s2,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (comp s1 s2) (comp t v)) u)/\
     (relSLstar (comp (comp s1 (comp s2 t)) v) u)))).
Intros; Exists (comp s1 (comp s2 (comp t v)));Split;Red.
Auto.
Apply star_trans1 with (comp s1 (comp (comp s2 t) v));Auto.
Save PC_assenv_assenv.
Hint PC_assenv_assenv.

Goal (a:terms)(s,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (cons a s) (comp t v)) u)/\
     (relSLstar (comp (cons (env a t) (comp s t)) v) u)))).
Intros; Exists (cons (env a (comp t v)) (comp s (comp t v)));Split;Red.
Auto.
Apply star_trans1 with (cons (env (env a t) v) (comp (comp s t) v)).
Auto.
Apply star_trans1 with (cons (env a (comp t v)) (comp (comp s t) v));Auto.
Save PC_assenv_mapenv.
Hint PC_assenv_mapenv.

Goal (a:terms)(s,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp shift (comp (cons a s) v)) u)/\
     (relSLstar (comp s v) u)))).
Intros; Exists (comp s v);Split;Red.
Apply star_trans1 with (comp shift (cons (env a v) (comp s v)));Auto.
Auto.
Save PC_assenv_shiftcons.
Hint PC_assenv_shiftcons.

Goal (s,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp shift (comp (lift s) v)) u)/\
     (relSLstar (comp (comp s shift) v) u)))).
Intros; Exists (comp s (comp shift v));Auto 6.
Save PC_assenv_shiftlift1.
Hint PC_assenv_shiftlift1.

Goal (s,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp shift (comp (comp (lift s) t) v)) u)/\
     (relSLstar (comp (comp s (comp shift t)) v) u)))).
Intros; Exists (comp s (comp shift (comp t v)));Split;Red.
Apply star_trans1 with (comp shift (comp (lift s) (comp t v)));Auto.
Apply star_trans1 with (comp s (comp (comp shift t) v));Auto.
Save PC_assenv_shiftlift2.
Hint PC_assenv_shiftlift2.

Goal (s,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift s) (comp (lift t) v)) u)/\
     (relSLstar (comp (lift (comp s t)) v) u)))).
Intros; Exists (comp (lift (comp s t)) v);Auto 6.
Save PC_assenv_lift1.
Hint PC_assenv_lift1.

Goal (s,t1,t2,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift s) (comp (comp (lift t1) t2) v)) u)/\
     (relSLstar (comp (comp (lift (comp s t1)) t2) v) u)))).
Intros; Exists (comp (lift (comp s t1)) (comp t2 v));Split;Red.
Apply star_trans1 with (comp (lift s) (comp (lift t1) (comp t2 v)));Auto.
Auto.
Save PC_assenv_lift2.
Hint PC_assenv_lift2.

Goal (a:terms)(s,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift s) (comp (cons a t) v)) u)/\
     (relSLstar (comp (cons a (comp s t)) v) u)))).
Intros; Exists (cons (env a v) (comp s (comp t v)));Split;Red.
Apply star_trans1 with (comp (lift s) (cons (env a v) (comp t v)));Auto.
Apply star_trans1 with (cons (env a v) (comp (comp s t) v));Auto.
Save PC_assenv_liftenv.
Hint PC_assenv_liftenv.

Goal (t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp id (comp t v)) u)/\
     (relSLstar (comp t v) u)))).
Intros; Exists (comp t v);Auto 6.
Save PC_assenv_idl.
Hint PC_assenv_idl.

Goal (s,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp s (comp id v)) u)/\
     (relSLstar (comp s v) u)))).
Intros; Exists (comp s v);Auto 7.
Save PC1_assenv_idr.
Hint PC1_assenv_idr.

Goal (s,t:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp s (comp t id)) u)/\
     (relSLstar (comp s t) u)))).
Intros; Exists (comp s t);Auto 7.
Save PC2_assenv_idr.
Hint PC2_assenv_idr.

Goal (s,s',t,v:sub_explicits)(relSL s s')->
 (Ex(([u:sub_explicits](relSLstar (comp s (comp t v)) u)/\
                                    (relSLstar (comp (comp s' t) v) u)))).
Intros; Exists (comp s' (comp t v));Auto 6.
Save PC_assenv_ctxt_l.
Hint PC_assenv_ctxt_l.

Goal (s,t,t',v:sub_explicits)(relSL t t')->
 (Ex(([u:sub_explicits](relSLstar (comp s (comp t v)) u)/\
                                    (relSLstar (comp (comp s t') v) u)))). 
Intros; Exists (comp s (comp t' v));Auto 6.
Save PC1_assenv_ctxt_r.
Hint PC1_assenv_ctxt_r.

Goal (s,t,v,v':sub_explicits)(relSL v v')->
  (Ex(([u:sub_explicits](relSLstar (comp s (comp t v)) u)/\
                                   (relSLstar (comp (comp s t) v') u)))). 
Intros; Exists (comp s (comp t v'));Auto 6.
Save PC2_assenv_ctxt_r.
Hint PC2_assenv_ctxt_r.

Goal (s,t,v,x':sub_explicits)(relSL (comp s t) x')->
  (Ex(([u:sub_explicits](relSLstar (comp s (comp t v)) u)/\
                                   (relSLstar (comp x' v) u)))).
Intros s t v x' H;Pattern s t x';Apply case_SLcomp;Auto.
Save PC_assenv_ctxt_r.
Hint PC_assenv_ctxt_r.

(*** mapenv ***)

Goal (a:terms)(s:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (cons (env a id) (comp s id)) u)/\
     (relSLstar (cons a s) u)))). 
Intros; Exists (cons a s);Split;Red.
Apply star_trans1 with (cons a (comp s id));Auto.
Auto.
Save PC_mapenv_idr.
Hint PC_mapenv_idr.

Goal (a,a':terms)(s,t:sub_explicits)(relSL a a')->
     (Ex(([u:sub_explicits]
            (relSLstar (cons (env a t) (comp s t)) u)/\
            (relSLstar (comp (cons a' s) t) u)))). 
Intros; Exists (cons (env a' t) (comp s t));Auto 6.
Save PC1_mapenv_ctxt_l.
Hint PC1_mapenv_ctxt_l.

Goal (a:terms)(s,s',t:sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits]
       (relSLstar (cons (env a t) (comp s t)) u)/\
       (relSLstar (comp (cons a s') t) u)))). 
Intros; Exists (cons (env a t) (comp s' t));Auto 6.
Save PC2_mapenv_ctxt_l.
Hint PC2_mapenv_ctxt_l.

Goal (a:terms)(s,t,t':sub_explicits)(relSL t t')->
     (Ex(([u:sub_explicits]
       (relSLstar (cons (env a t) (comp s t)) u)/\
       (relSLstar (comp (cons a s) t') u)))). 
Intros; Exists (cons (env a t') (comp s t'));Split;Red.
Apply star_trans1 with (cons (env a t') (comp s t));Auto.
Auto.
Save PC_mapenv_ctxt_r.
Hint PC_mapenv_ctxt_r.

Goal (a:terms)(s,t,x':sub_explicits)(relSL (cons a s) x')->
     (Ex(([u:sub_explicits]
       (relSLstar (cons (env a t) (comp s t)) u)/\
       (relSLstar (comp x' t) u)))). 
Intros a s t x' H;Pattern x';Apply case_SLcons with a s;Auto.
Save PC_mapenv_ctxt_l.
Hint PC_mapenv_ctxt_l.

(*** shiftcons ***)

Goal (a':terms)(s:sub_explicits)
     (Ex(([u:sub_explicits]
          (relSLstar s u)/\
          (relSLstar (comp shift (cons a' s)) u)))). 
Intros; Exists s;Auto 6.
Save PC1_shiftcons_ctxt_r.
Hint PC1_shiftcons_ctxt_r.

Goal (a:terms)(s,s':sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits]
          (relSLstar s u)/\
          (relSLstar (comp shift (cons a s')) u)))).
Intros; Exists s';Auto 6.
Save PC2_shiftcons_ctxt_r.
Hint PC2_shiftcons_ctxt_r.

Goal (a:terms)(s,x':sub_explicits)(relSL (cons a s) x')->
     (Ex(([u:sub_explicits]
          (relSLstar s u)/\
          (relSLstar (comp shift x') u)))).
Intros a s x' H;Pattern x';Apply case_SLcons with a s;Auto.
Save PC_shiftcons_ctxt_r.

(*** shiftlift1 ***)

Goal (Ex(([u:sub_explicits](relSLstar (comp id shift) u)/\
                                        (relSLstar (comp shift id) u)))).
Intros; Exists shift;Auto 6.
Save PC_shiftlift1_liftid.
Hint PC_shiftlift1_liftid.

Goal (s,s':sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits]
        (relSLstar (comp s shift) u)/\
        (relSLstar (comp shift (lift s')) u)))).
Intros; Exists (comp s' shift);Auto 6.
Save PC_shiftlift1_ctxt_r.
Hint PC_shiftlift1_ctxt_r.

Goal (s,x':sub_explicits)(relSL (lift s) x')->
     (Ex(([u:sub_explicits]
        (relSLstar (comp s shift) u)/\
        (relSLstar (comp shift x') u)))).
Intros s x' H;Pattern s x';Apply case_SLlift;Auto.
Save PC_shiftlift1_ctxt_r'.
Hint PC_shiftlift1_ctxt_r'.

(*** shiftlift2 ***)

Goal (s,t:sub_explicits)(Ex(([u:sub_explicits]
      (relSLstar (comp s (comp shift (lift t))) u)/\
      (relSLstar (comp shift (lift (comp s t))) u)))).
Intros; Exists (comp s (comp t shift));Split;Red.
Auto.
Apply star_trans1 with (comp (comp s t) shift);Auto.
Save PC_shiftlift2_lift1.
Hint PC_shiftlift2_lift1.

Goal (s,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp s (comp shift (comp (lift t) v))) u)/\
     (relSLstar (comp shift (comp (lift (comp s t)) v)) u)))).
Intros; Exists (comp s (comp t (comp shift v)));Split;Red.
Auto.
Apply star_trans1 with (comp (comp s t) (comp shift v));Auto.
Save PC_shiftlift2_lift2.
Hint PC_shiftlift2_lift2.

Goal (a:terms)(s,t:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp s (comp shift (cons a t))) u)/\
     (relSLstar (comp shift (cons a (comp s t))) u)))).
Intros; Exists (comp s t);Auto 7.
Save PC_shiftlift2_liftenv.
Hint PC_shiftlift2_liftenv.

Goal (t:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp id (comp shift t)) u)/\
     (relSLstar (comp shift (comp id t)) u)))).
Intros; Exists (comp shift t);Auto 7.
Save PC_shiftlift2_liftid.
Hint PC_shiftlift2_liftid.

Goal (s:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp s (comp shift id))  u)/\
     (relSLstar (comp shift (lift s)) u)))).
Intros; Exists (comp s shift);Auto 7.
Save PC_shiftlift2_idr.
Hint PC_shiftlift2_idr.

Goal (s,s',t:sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits]
       (relSLstar (comp s (comp shift t))  u)/\
       (relSLstar (comp shift (comp (lift s') t)) u)))).
Intros; Exists (comp s' (comp shift t));Auto 6.
Save PC1_shiftlift2_ctxt_r.
Hint PC1_shiftlift2_ctxt_r.

Goal (s,t,t':sub_explicits)(relSL t t')->
     (Ex(([u:sub_explicits]
       (relSLstar (comp s (comp shift t))  u)/\
       (relSLstar (comp shift (comp (lift s) t')) u)))).
Intros; Exists (comp s (comp shift t'));Auto 6.
Save PC2_shiftlift2_ctxt_r.
Hint PC2_shiftlift2_ctxt_r.

Goal (s,t,x':sub_explicits)(relSL (comp (lift s) t) x')->
     (Ex(([u:sub_explicits]
       (relSLstar (comp s (comp shift t))  u)/\
       (relSLstar (comp shift x') u)))).
Intros s t x' H;Pattern t x';Apply case_SLcomp2 with s;Auto.
Intros;Pattern s x'0;Apply case_SLlift;Auto.
Save PC_shiftlift2_ctxt_r.
Hint PC_shiftlift2_ctxt_r.
  
(*** lift1 ***)

Goal (t:sub_explicits)
     (Ex(([u:sub_explicits](relSLstar (lift (comp id t)) u)/\
                                  (relSLstar (comp id (lift t)) u)))).
Intros; Exists (lift t);Auto 7.
Save PC1_lift1_liftid.
Hint PC1_lift1_liftid.

Goal (s:sub_explicits)
     (Ex(([u:sub_explicits](relSLstar (lift (comp s id)) u)/\
                                  (relSLstar (comp (lift s) id) u)))).
Intros; Exists (lift s);Auto 7.
Save PC2_lift1_liftid.
Hint PC2_lift1_liftid.

Goal (s,s',t:sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits](relSLstar (lift (comp s t)) u)/\
                                  (relSLstar (comp (lift s') (lift t)) u)))).
Intros; Exists (lift (comp s' t));Auto 6.
Save PC_lift1_ctxt_l.
Hint PC_lift1_ctxt_l.

Goal (s,t,t':sub_explicits)(relSL t t')->
     (Ex(([u:sub_explicits](relSLstar (lift (comp s t)) u)/\
                                  (relSLstar (comp (lift s) (lift t')) u)))).
Intros; Exists (lift (comp s t'));Auto 6.
Save PC_lift1_ctxt_r.
Hint PC_lift1_ctxt_r.

Goal (s,t,x':sub_explicits)(relSL (lift s) x')->
     (Ex(([u:sub_explicits](relSLstar (lift (comp s t)) u)/\
                                  (relSLstar (comp x' (lift t)) u)))).
Intros s t x' H;Pattern s x';Apply case_SLlift;Auto.
Save PC_lift1_ctxt_l'.
Hint PC_lift1_ctxt_l'.

Goal (s,t,x':sub_explicits)(relSL (lift t) x')->
     (Ex(([u:sub_explicits](relSLstar (lift (comp s t)) u)/\
                                  (relSLstar (comp (lift s) x') u)))).
Intros a t x' H;Pattern t x';Apply case_SLlift;Auto.
Save PC_lift1_ctxt_r'.
Hint PC_lift1_ctxt_r'.

(*** lift2 ***)

Goal (s,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift (comp s t)) (lift v)) u)/\
     (relSLstar (comp (lift s) (lift (comp t v))) u)))).
Intros; Exists (lift (comp s (comp t v)));Split;Red.
Apply star_trans1 with (lift (comp (comp s t) v));Auto.
Auto.
Save PC_lift2_lift1.
Hint PC_lift2_lift1.

Goal (s,t1,t2,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift (comp s t1)) (comp (lift t2) v)) u)/\
     (relSLstar (comp (lift s) (comp (lift (comp t1 t2)) v))  u)))).
Intros; Exists (comp (lift (comp s (comp t1 t2))) v);Split;Red.
Apply star_trans1 with (comp (lift (comp (comp s t1) t2)) v);Auto 6.
Auto.
Save PC_lift2_lift2.
Hint PC_lift2_lift2.

Goal (a:terms)(s,t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift (comp s t)) (cons a v)) u)/\
     (relSLstar (comp (lift s) (cons a (comp t v)))  u)))).
Intros; Exists (cons a (comp s (comp t v)));Split;Red.
Apply star_trans1 with (cons a (comp (comp s t) v));Auto.
Auto.
Save PC_lift2_liftenv.
Hint PC_lift2_liftenv.

Goal (t,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift (comp id t)) v) u)/\
     (relSLstar (comp id (comp (lift t) v))  u)))).
Intros; Exists (comp (lift t) v);Auto 8.
Save PC1_lift2_liftid.
Hint PC1_lift2_liftid.

Goal (s,v:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift (comp s id)) v) u)/\
     (relSLstar (comp (lift s) (comp id v))  u)))).
Intros; Exists (comp (lift s) v);Auto 8.
Save PC2_lift2_liftid.
Hint PC2_lift2_liftid.

Goal (s,t:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (comp (lift (comp s t)) id) u)/\
     (relSLstar (comp (lift s) (lift t)) u)))).
Intros; Exists (lift (comp s t));Auto 6.
Save PC_lift2_idr.
Hint PC_lift2_idr.

Goal (s,s',t,v:sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits]
       (relSLstar (comp (lift (comp s t)) v) u)/\
       (relSLstar (comp (lift s') (comp (lift t) v)) u)))).
Intros; Exists (comp (lift (comp s' t)) v);Auto 7.
Save PC_lift2_ctxt_l.
Hint PC_lift2_ctxt_l.

Goal (s,t,t',v:sub_explicits)(relSL t t')->
     (Ex(([u:sub_explicits]
        (relSLstar (comp (lift (comp s t)) v) u)/\
        (relSLstar (comp (lift s) (comp (lift t') v)) u)))).
Intros; Exists (comp (lift (comp s t')) v);Auto 7.
Save PC1_lift2_ctxt_r.
Hint PC1_lift2_ctxt_r.

Goal (s,t,v,v':sub_explicits)(relSL v v')->
     (Ex(([u:sub_explicits]
        (relSLstar (comp (lift (comp s t)) v) u)/\
        (relSLstar (comp (lift s) (comp (lift t) v')) u)))).
Intros; Exists (comp (lift (comp s t)) v');Auto 6.
Save PC2_lift2_ctxt_r.
Hint PC2_lift2_ctxt_r.

Goal (s,t,v,x':sub_explicits)(relSL (lift s) x')->
     (Ex(([u:sub_explicits]
        (relSLstar (comp (lift (comp s t)) v) u)/\
        (relSLstar (comp x'  (comp (lift t) v)) u)))).
Intros s t v x' H;Pattern s x';Apply case_SLlift;Auto.
Save PC_lift2_ctxt_l'. 
Hint PC_lift2_ctxt_l'. 

Goal (s,t,v,x':sub_explicits)(relSL (comp (lift t) v) x')->
     (Ex(([u:sub_explicits]
        (relSLstar (comp (lift (comp s t)) v) u)/\
        (relSLstar (comp (lift s) x') u)))).
Intros s t v x' H;Pattern v x';Apply case_SLcomp2 with t;Auto.
Intros;Pattern t x'0;Apply case_SLlift;Auto.
Save PC_lift2_ctxt_r.
Hint PC_lift2_ctxt_r.

(*** liftenv ***)

Goal (a:terms)(t:sub_explicits)(Ex(([u:sub_explicits]
     (relSLstar (cons a (comp id t)) u)/\
     (relSLstar (comp id (cons a t)) u)))).
Intros; Exists (cons a t);Auto 7.
Save PC_liftenv_liftid.
Hint PC_liftenv_liftid.

Goal (a:terms)(s,s',t:sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits](relSLstar (cons a (comp s t)) u)/\
                    (relSLstar (comp (lift s') (cons a t)) u)))).
Intros; Exists (cons a (comp s' t));Auto 6.
Save PC_liftenv_ctxt_l.
Hint PC_liftenv_ctxt_l.

Goal (a,a':terms)(s,t:sub_explicits)(relSL a a')->
     (Ex(([u:sub_explicits](relSLstar (cons a (comp s t)) u)/\
                    (relSLstar (comp (lift s) (cons a' t)) u)))).
Intros; Exists (cons a' (comp s t));Auto 6.
Save PC1_liftenv_ctxt_r.
Hint PC1_liftenv_ctxt_r.

Goal (a:terms)(s,t,t':sub_explicits)(relSL t t')->
     (Ex(([u:sub_explicits](relSLstar (cons a (comp s t)) u)/\
                    (relSLstar (comp (lift s) (cons a t')) u)))).
Intros; Exists (cons a (comp s t'));Auto 6.
Save PC2_liftenv_ctxt_r.
Hint PC2_liftenv_ctxt_r.

Goal (a:terms)(s,t,x':sub_explicits)(relSL (lift s) x')->
     (Ex(([u:sub_explicits](relSLstar (cons a (comp s t)) u)/\
                    (relSLstar (comp x' (cons a t)) u)))).
Intros a s t x' H;Pattern s x';Apply case_SLlift;Auto.
Save PC_liftenv_ctxt_l'.
Hint PC_liftenv_ctxt_l'.

Goal (a:terms)(s,t,x':sub_explicits)(relSL (cons a t) x')->
     (Ex(([u:sub_explicits](relSLstar (cons a (comp s t)) u)/\
                    (relSLstar (comp (lift s) x') u)))).
Intros a s t x' H;Pattern x';Apply case_SLcons with a t;Auto.
Save PC_liftenv_ctxt_r.
Hint PC_liftenv_ctxt_r.

(*** idl ***)

Goal (Ex(([u:sub_explicits](relSLstar id  u)/\
                    (relSLstar id u)))).
Intros; Exists id;Auto.
Save PC_idl_idr.
Hint PC_idl_idr.

Goal (s,s':sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits](relSLstar s  u)/\
                    (relSLstar (comp id s') u)))).
Intros; Exists s';Auto 6.
Save PC_idl_ctxt_r.
Hint PC_idl_ctxt_r.

(*** idr ***)

Goal (s,s':sub_explicits)(relSL s s')->
     (Ex(([u:sub_explicits](relSLstar s  u)/\
                    (relSLstar (comp s' id) u)))).
Intros; Exists s';Auto 6.
Save PC_idr_ctxt_l.
Hint PC_idr_ctxt_l.

(*** liftid ***)

 (* aucune PC *)

(*** id ***)

Goal (a,a':terms)(relSL a a')->
     (Ex(([u:terms](relSLstar a  u)/\
                        (relSLstar (env a' id) u)))).
Intros; Exists a';Auto 6.
Save PC_id_ctxt_l.
Hint PC_id_ctxt_l.



