(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               sigma_lift.v                               *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


                    (* Systeme sigma-lift *)

Require TS.
Require sur_les_relations.

(* regles de reecriture *)

Inductive reg_app: terms->terms->Prop
:=  reg1_app: (a,b:terms)(s:sub_explicits)
             (reg_app (env (app a b) s) (app (env a s) (env b s))). 
Hint reg1_app.

Inductive reg_lambda: terms->terms->Prop
:=  reg1_lambda: (a:terms)(s:sub_explicits)
                (reg_lambda (env (lambda a) s) (lambda (env a (lift s)))). 
Hint reg1_lambda.

Inductive reg_clos: terms->terms->Prop
:=  reg1_clos: (a:terms)(s,t:sub_explicits)
              (reg_clos (env (env a s) t) (env a (comp s t))). 
Hint reg1_clos.

Inductive reg_varshift1: terms->terms->Prop
:=  reg1_varshift1: (n:nat)(reg_varshift1 (env (var n) shift) (var (S n))). 
Hint reg1_varshift1.

Inductive reg_varshift2: terms->terms->Prop
:=  reg1_varshift2: (n:nat)(s:sub_explicits)
        (reg_varshift2  (env (var n) (comp shift s)) (env (var (S n)) s)). 
Hint reg1_varshift2.

Inductive reg_fvarcons: terms->terms->Prop 
:=  reg1_fvarcons: (a:terms)(s:sub_explicits)
              (reg_fvarcons (env (var O) (cons a s)) a). 
Hint reg1_fvarcons.

Inductive reg_fvarlift1: terms->terms->Prop 
:=  reg1_fvarlift1: (s:sub_explicits)
           (reg_fvarlift1 (env (var O) (lift s)) (var O)). 
Hint reg1_fvarlift1.

Inductive reg_fvarlift2: terms->terms->Prop 
:=  reg1_fvarlift2: (s,t:sub_explicits)
                   (reg_fvarlift2 (env (var O) (comp (lift s) t)) 
                                           (env (var O) t)). 
Hint reg1_fvarlift2.

Inductive reg_rvarcons: terms->terms->Prop 
:=  reg1_rvarcons: (n:nat)(a:terms)(s:sub_explicits)
           (reg_rvarcons (env (var (S n)) (cons a s)) (env (var n) s)). 
Hint reg1_rvarcons.

Inductive reg_rvarlift1: terms->terms->Prop
:=  reg1_rvarlift1: (n:nat)(s:sub_explicits)
 (reg_rvarlift1 (env (var (S n)) (lift s)) (env (var n) (comp s shift))). 
Hint reg1_rvarlift1.

Inductive reg_rvarlift2: terms->terms->Prop
:=  reg1_rvarlift2: (n:nat)(s,t:sub_explicits)
                   (reg_rvarlift2 (env (var (S n)) (comp (lift s) t)) 
                                  (env (var n) (comp s (comp shift t)))). 
Hint reg1_rvarlift2.

Inductive reg_assenv: sub_explicits->sub_explicits->Prop
:=  reg1_assenv: (s,t,u:sub_explicits)(reg_assenv 
              (comp (comp s t) u) (comp s (comp t u))). 
Hint reg1_assenv.

Inductive reg_mapenv: sub_explicits->sub_explicits->Prop
:=  reg1_mapenv: (a:terms)(s,t:sub_explicits)
         (reg_mapenv (comp (cons a s) t) (cons (env a t) (comp s t))). 
Hint reg1_mapenv.

Inductive reg_shiftcons: sub_explicits->sub_explicits->Prop
:=  reg1_shiftcons: (a:terms)(s:sub_explicits)
         (reg_shiftcons (comp shift (cons a s)) s). 
Hint reg1_shiftcons.

Inductive reg_shiftlift1: sub_explicits->sub_explicits->Prop
:=  reg1_shiftlift1: (s:sub_explicits)
     (reg_shiftlift1 (comp shift (lift s)) (comp s shift)). 
Hint reg1_shiftlift1.

Inductive reg_shiftlift2: sub_explicits->sub_explicits->Prop
:=  reg1_shiftlift2: (s,t:sub_explicits)
 (reg_shiftlift2 (comp shift (comp (lift s) t)) (comp s (comp shift t))). 
Hint reg1_shiftlift2. 

Inductive reg_lift1: sub_explicits->sub_explicits->Prop
:=  reg1_lift1: (s,t:sub_explicits)
       (reg_lift1 (comp (lift s) (lift t)) (lift (comp s t))). 
Hint reg1_lift1.

Inductive reg_lift2: sub_explicits->sub_explicits->Prop
:=  reg1_lift2: (s,t,u:sub_explicits)
(reg_lift2 (comp (lift s) (comp (lift t) u)) (comp (lift (comp s t)) u)). 
Hint reg1_lift2.

Inductive reg_liftenv: sub_explicits->sub_explicits->Prop
:=  reg1_liftenv: (a:terms)(s,t:sub_explicits)
         (reg_liftenv (comp (lift s) (cons a t)) (cons a (comp s t))). 
Hint reg1_liftenv.

Inductive reg_idl: sub_explicits->sub_explicits->Prop
:=  reg1_idl: (s:sub_explicits)(reg_idl (comp id s) s). 
Hint reg1_idl.

Inductive reg_idr: sub_explicits->sub_explicits->Prop
:=  reg1_idr: (s:sub_explicits)(reg_idr (comp s id) s). 
Hint reg1_idr.

Inductive reg_liftid: sub_explicits->sub_explicits->Prop
:=  reg1_liftid: (reg_liftid (lift id) id). 
Hint reg1_liftid.

Inductive reg_id: terms->terms->Prop
:=  reg1_id: (a:terms)(reg_id (env a id) a). 
Hint reg1_id.

(* systeme sigma-lift *)

Inductive e_systemSL: (b:wsort)(TS b)->(TS b)->Prop
:=  regle_app: (a,b:terms)(reg_app a b)->(e_systemSL wt a b) 
 | regle_lambda: (a,b:terms)(reg_lambda a b)->(e_systemSL wt a b) 
 | regle_clos: (a,b:terms)(reg_clos a b)->(e_systemSL wt a b) 
 | regle_varshift1: (a,b:terms)(reg_varshift1 a b)->(e_systemSL wt a b)  
 | regle_varshift2: (a,b:terms)(reg_varshift2 a b)->(e_systemSL wt a b) 
 | regle_fvarcons: (a,b:terms)(reg_fvarcons a b)->(e_systemSL wt a b) 
 | regle_fvarlift1: (a,b:terms)(reg_fvarlift1 a b)->(e_systemSL wt a b) 
 | regle_fvarlift2: (a,b:terms)(reg_fvarlift2 a b)->(e_systemSL wt a b) 
 | regle_rvarcons: (a,b:terms)(reg_rvarcons a b)->(e_systemSL wt a b) 
 | regle_rvarlift1: (a,b:terms)(reg_rvarlift1 a b)->(e_systemSL wt a b) 
 | regle_rvarlift2: (a,b:terms)(reg_rvarlift2 a b)->(e_systemSL wt a b) 
 | regle_assenv: (s,t:sub_explicits)(reg_assenv s t)->(e_systemSL ws s t) 
 | regle_mapenv: (s,t:sub_explicits)(reg_mapenv s t)->(e_systemSL ws s t)
 | regle_shiftcons: (s,t:sub_explicits)
      (reg_shiftcons s t)->(e_systemSL ws s t)
 | regle_shiftlift1: (s,t:sub_explicits)
          (reg_shiftlift1 s t)->(e_systemSL ws s t) 
 | regle_shiftlift2: (s,t:sub_explicits)
           (reg_shiftlift2 s t)->(e_systemSL ws s t)
 | regle_lift1: (s,t:sub_explicits)(reg_lift1 s t)->(e_systemSL ws s t)
 | regle_lift2: (s,t:sub_explicits)(reg_lift2 s t)->(e_systemSL ws s t) 
 | regle_liftenv: (s,t:sub_explicits)(reg_liftenv s t)->(e_systemSL ws s t)
 | regle_idl: (s,t:sub_explicits)(reg_idl s t)->(e_systemSL ws s t)
 | regle_idr: (s,t:sub_explicits)(reg_idr s t)->(e_systemSL ws s t)
 | regle_liftid: (s,t:sub_explicits)(reg_liftid  s t)->(e_systemSL ws s t) 
 | regle_id: (a,b:terms)(reg_id a b)->(e_systemSL wt a b).

Syntactic Definition systemSL := (e_systemSL ?).
Syntax constr systemSL <<(e_systemSL $1 $t1 $t2)>> 10
  [<hov 0> "systemSL " <$t1:"term":L> " " <$t2:"term":L>].

Hint regle_app regle_lambda regle_clos regle_varshift1 regle_varshift2
      regle_fvarcons regle_fvarlift1 regle_fvarlift2 regle_rvarcons
      regle_rvarlift1 regle_rvarlift2 regle_assenv
      regle_mapenv regle_shiftcons regle_shiftlift1 regle_shiftlift2
      regle_lift1 regle_lift2 regle_liftenv regle_idl regle_idr
      regle_liftid regle_id.

(* relation engendree par le systeme sigma-lift *)

Inductive e_relSL: (b:wsort)(TS b)->(TS b)->Prop
:=  SL_one_regle: (b:wsort)(M,N:(TS b))(systemSL M N)->(e_relSL b M N)
 | SL_context_app_l: (a,a',b:terms)
         (e_relSL wt a a')->(e_relSL wt (app a b) (app a' b))
 | SL_context_app_r: (a,b,b':terms)(e_relSL wt b b')->
                                      (e_relSL wt (app a b) (app a b'))  
 | SL_context_lambda: (a,a':terms)(e_relSL wt a a')->
                                      (e_relSL wt (lambda a) (lambda a'))
 | SL_context_env_t: (a,a':terms)(s:sub_explicits)(e_relSL wt a a')->
                                               (e_relSL wt (env a s) (env a' s))
 | SL_context_env_s: (a:terms)(s,s':sub_explicits)(e_relSL ws s s')->
					       (e_relSL wt (env a s) (env a s'))
 | SL_context_cons_t: (a,a':terms)(s:sub_explicits)(e_relSL wt a a')->
	     (e_relSL ws (cons a s) (cons a' s)) 
 | SL_context_cons_s: (a:terms)(s,s':sub_explicits)(e_relSL ws s s')->
						(e_relSL ws (cons a s) (cons a s'))
 | SL_context_comp_l: (s,s',t:sub_explicits)(e_relSL ws s s')->
				         (e_relSL ws (comp s t) (comp s' t))
 | SL_context_comp_r: (s,t,t':sub_explicits)(e_relSL ws t t')->
                                         (e_relSL ws (comp s t) (comp s t'))
 | SL_context_lift: (s,s':sub_explicits)(e_relSL ws s s')->
                                         (e_relSL ws (lift s) (lift s')).

Syntactic Definition relSL := (e_relSL ?).
Syntax constr relSL <<(e_relSL $1 $t1 $t2)>> 10
  [<hov 0> "relSL " <$t1:"term":L> " " <$t2:"term":L>].

Hint  SL_one_regle SL_context_app_l SL_context_app_r SL_context_lambda
      SL_context_env_t SL_context_env_s SL_context_cons_t SL_context_cons_s
      SL_context_comp_l SL_context_comp_r SL_context_lift.

(* fermeture reflexive-transitive de la relation sigma-lift *)

Definition e_relSLstar := [b:wsort] (star (e_relSL b)).

Syntactic Definition relSLstar := (e_relSLstar ?).
Syntax constr relSLstar <<(e_relSLstar $1 $t1 $t2)>> 10
  [<hov 0> "relSLstar " <$t1:"term":L> " " <$t2:"term":L>].

Hint Unfold e_relSLstar.

(* *)

Goal (a,a',b:terms)(relSLstar a a')->(relSLstar (app a b) (app a' b)).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (app y b);Auto.
Save SLstar_context_app_l.
Hint SLstar_context_app_l.

Goal (a,b,b':terms)(relSLstar b b')->(relSLstar (app a b) (app a b')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (app a y);Auto.
Save SLstar_context_app_r.
Hint SLstar_context_app_r.

Goal (a,a':terms)(relSLstar a a')->(relSLstar (lambda a) (lambda a')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (lambda y);Auto.
Save SLstar_context_lambda.
Hint SLstar_context_lambda.

Goal (a,a':terms)(s:sub_explicits)(relSLstar a a')->
                                        (relSLstar (env a s) (env a' s)).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (env y s);Auto.
Save  SLstar_context_env_t.
Hint  SLstar_context_env_t.

Goal (a:terms)(s,s':sub_explicits)(relSLstar s s')->
                                        (relSLstar (env a s) (env a s')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (env a y);Auto.
Save  SLstar_context_env_s.
Hint  SLstar_context_env_s.

Goal (a,a':terms)(s:sub_explicits)(relSLstar a a')->
                (relSLstar (cons a s) (cons a' s)). 
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (cons y s);Auto.
Save SLstar_context_cons_t.
Hint SLstar_context_cons_t.

Goal (a:terms)(s,s':sub_explicits)(relSLstar s s')->
                (relSLstar (cons a s) (cons a s')). 
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (cons a y);Auto.
Save SLstar_context_cons_s.
Hint SLstar_context_cons_s.

Goal (s,s',t:sub_explicits)(relSLstar s s')->
			  (relSLstar (comp s t) (comp s' t)).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (comp y t);Auto.
Save SLstar_context_comp_l.
Hint SLstar_context_comp_l.

Goal (s,t,t':sub_explicits)(relSLstar t t')->
			  (relSLstar (comp s t) (comp s t')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (comp s y);Auto.
Save SLstar_context_comp_r.
Hint SLstar_context_comp_r.

Goal  (s,s':sub_explicits)(relSLstar s s')->
                                  (relSLstar (lift s) (lift s')).
Red;Induction 1;Intros.
Auto.
Apply star_trans1 with (lift y);Auto.
Save SLstar_context_lift.
Hint SLstar_context_lift.



