(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             terminaison_SL.v                             *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq  - Calculus of Inductive Constructions V5.8           *)
(*****************************************************************************)
(*                                                                           *)
(*      Meta-theory of the explicit substitution calculus lambda-env         *)
(*      Amokrane Saibi                                                       *)
(*                                                                           *)
(*      September 1993                                                       *)
(*                                                                           *)
(*****************************************************************************)


              (* Preuve de terminaison *)  


Require Le.
Require Lt.
Require Plus.
Require Gt.
Require Minus.
Require Mult.
Require sur_les_relations.
Require TS. 
Require sigma_lift.  
Require comparith.
Require Pol1.
Require Pol2.

Section ordre.

Variable A : Set.
Variable f,g: A->nat.

Definition e_lexfg := [a,b:A] (gt (f a) (f b)) \/
                  ( ((f a)=(f b)) /\ (gt (g a) (g b))).

Lemma lexfg_notherian: (noetherian e_lexfg). 
Proof.
Unfold explicit_noetherian;Unfold universal;Unfold hereditary;
  Unfold adjoint;Unfold sub;Unfold a_set.
Intros P H.
Cut (n,m:nat)(a:A)(gt n (f a)) \/
    ( (n=(f a)) /\ (gt m (g a)))->(P a).
Intros H0 x;Apply (H0 (S (f x)) O).
Auto.
Induction n;Induction m.
(* m=n=0 *)
Induction 1;Intro H1.
Absurd (gt O (f a));Auto.
Elim H1;Intros.
Absurd (gt O (g a));Auto.
(* n=0, m=(S y) *)
Intros y H' a H0.
Apply H;Intros b lexfgab. 
Apply H';Right.
Elim  H0;Intro H1.
Absurd (gt O (f a));Auto.
Elim H1;Intros H2 H3;Elim lexfgab;Intro H4.
Absurd (gt O (f b)).
Auto.
Rewrite H2;Assumption.
Elim H4;Intros.
Split.
Rewrite H2;Assumption.
Apply le_gt_trans with (g a);Auto.
(* n=(S y), m=0 *)
Intros a H0';Apply H;Intros b lexfgab.
Apply (H0 (g a) b);Elim H0';Intro H1.
Elim lexfgab;Intro H2.
Left;Apply le_gt_trans with (f a);Auto.
Elim H2;Intros H3 H4;Elim (gt_S n0 (f a) H1);Intro H5.
Left;Elim H3;Assumption.
Right;Split.
Elim H3;Auto.
Assumption.
Elim H1;Intros H2 H3.
Absurd (gt O (g a));Auto.
(* n=(S y), m=(S y0) *)
Intros y0 H0' a H1;Apply H;Intros b lexfgab.
Apply H0';Elim H1;Elim lexfgab;Intros H2 H3.
Left;Apply le_gt_trans with (f a);Auto.
Elim H2;Intros H4 H5;Left;Elim H4;Assumption.
Elim H3;Intros H4 H5;Left;Rewrite H4;Assumption.
Elim H2;Intros H4 H5;Elim H3;Intros H6 H7.
Right;Split.
Apply trans_equal with (f a);Assumption.
Apply le_gt_trans with (g a);Auto.
Qed.

End ordre.

Syntactic Definition lexfg := (e_lexfg ?).
Syntax constr lexfg <<(e_lexfg $1 $t1 $t2 $t3 $t4)>> 10
  [<hov 0> "lexfg " <$t1:"term":L> " " <$t2:"term":L> " " 
                    <$t3:"term":L> " " <$t4:"term":L>  ].

Theorem lexfg_systemSL: (b:wsort)(M,N:(TS b))
                        (systemSL M N)->(lexfg (e_P1 b) (e_P2 b) M N).
Proof.
Red;Induction 1;Auto.
Qed.
Hint lexfg_systemSL.
 
Theorem lexfg_app_l: (a,a',b:terms)(lexfg (e_P1 wt) (e_P2 wt) a a')->
                     (lexfg (e_P1 wt) (e_P2 wt) (app a b) (app a' b)).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_app_l.

Theorem lexfg_app_r: (a,b,b':terms)(lexfg (e_P1 wt) (e_P2 wt) b b')->
                     (lexfg (e_P1 wt) (e_P2 wt) (app a b) (app a b')).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_app_r.

Theorem lexfg_lambda: (a,a':terms)(lexfg (e_P1 wt) (e_P2 wt) a a')->
                      (lexfg (e_P1 wt) (e_P2 wt) (lambda a) (lambda a')).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_lambda.

Theorem lexfg_env_t: (a,a':terms)(s:sub_explicits)
                     (lexfg (e_P1 wt) (e_P2 wt) a a')->
                     (lexfg (e_P1 wt) (e_P2 wt) (env a s) (env a' s)).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_env_t.

Theorem lexfg_env_s: (a:terms)(s,s':sub_explicits)
                     (lexfg (e_P1 ws) (e_P2 ws) s s')->
                     (lexfg (e_P1 wt) (e_P2 wt) (env a s) (env a s')).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_env_s.

Theorem lexfg_cons_t: (a,a':terms)(s:sub_explicits)
                      (lexfg (e_P1 wt) (e_P2 wt) a a')->
                      (lexfg (e_P1 ws) (e_P2 ws) (cons a s) (cons a' s)).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_cons_t.

Theorem lexfg_cons_s: (a:terms)(s,s':sub_explicits)
                      (lexfg (e_P1 ws) (e_P2 ws) s s')->
                      (lexfg (e_P1 ws) (e_P2 ws) (cons a s) (cons a s')).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_cons_s.

Theorem lexfg_comp_l: (s,s',t:sub_explicits)(lexfg (e_P1 ws) (e_P2 ws) s s')->
                      (lexfg (e_P1 ws) (e_P2 ws) (comp s t) (comp s' t)).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_comp_l.

Theorem lexfg_comp_r: (s,t,t':sub_explicits)(lexfg (e_P1 ws) (e_P2 ws) t t')->
                      (lexfg (e_P1 ws) (e_P2 ws) (comp s t) (comp s t')).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Auto.
Intros;Elim H0;Auto.
Qed.
Hint lexfg_comp_r.

Theorem lexfg_lift: (s,s':sub_explicits)(lexfg (e_P1 ws) (e_P2 ws) s s')->
                    (lexfg (e_P1 ws) (e_P2 ws) (lift s) (lift s')).
Proof.
Unfold e_lexfg;Induction 1;Simpl;Intros.
Auto.
Elim H0;Intros;Right;Split.
Assumption.
Change (gt (mult (S(S(S(S O)))) (P2 s))
           (mult (S(S(S(S O)))) (P2 s'))).
Auto.
Qed.
Hint lexfg_lift.

Theorem lexfg_relSL: (b:wsort)(M,N:(TS b))
                     (relSL M N)->(lexfg (e_P1 b) (e_P2 b) M N).
Proof.
Induction 1;Auto.
Qed.


(***************************************************)
(*   la relation sigma-lift (SL) est noetherienne  *)
(***************************************************)

Theorem relSL_noetherian: (b:wsort)(noetherian (e_relSL b)).
Proof.
Intro b;Apply noether_inclus with (lexfg (e_P1 b) (e_P2 b)).
Apply lexfg_notherian.
Exact (lexfg_relSL b).
Qed.


