(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*      Initial Version: Frederic Prost, July 1993                          *)
(*      Revised Version: Gilles Kahn, September 1993                        *)
(*      Revised Version: Gilles Kahn, November 1993                         *)
(*      Revised Version: Gilles Kahn, April 1994                            *)
(*                               INRIA Sophia-Antipolis, FRANCE             *)
(*      Revised Version: Jean-Christophe Filliatre, January 1995            *)
(*                               ENS-Lyon, FRANCE                           *)
(*                                                                          *)
(****************************************************************************)
(*                               Ensembles.v                                *)
(****************************************************************************)

Require Classical.

Section Ensembles.
   Variable U: Type.
   
   Definition Ensemble :=  U -> Prop.
   
   Definition In : Ensemble -> U -> Prop :=  [B: Ensemble] [x: U] (B x).
   
   Definition Included : Ensemble -> Ensemble -> Prop :=
      [B: Ensemble] [C: Ensemble] (x: U) (In B x) -> (In C x).
   
   Inductive Empty_set : Ensemble :=
      .
   
   Inductive Singleton [x: U]: Ensemble :=
         In_singleton: (In (Singleton x) x).
   
   Inductive Union [B,C: Ensemble]: Ensemble :=
         Union_introl: (x: U) (In B x) -> (In (Union B C) x)
      | Union_intror: (x: U) (In C x) -> (In (Union B C) x).
   
   Inductive Intersection [B,C: Ensemble]: Ensemble :=
         Intersection_intro:
           (x: U) (In B x) -> (In C x) -> (In (Intersection B C) x).
   
   Inductive Couple [x,y: U]: Ensemble :=
         Couple_l: (In (Couple x y) x)
      | Couple_r: (In (Couple x y) y).
   
   Inductive Setminus [B,C: Ensemble]: Ensemble :=
         Setminus_intro: (x: U) (In B x) -> ~ (In C x) -> (In (Setminus B C) x).
   
   Inductive Disjoint [B,C: Ensemble]: Prop :=
         Disjoint_intro: ((x: U) ~ (In (Intersection B C) x)) -> (Disjoint B C).
   
   Inductive Non_empty [B: Ensemble]: Prop :=
         Non_empty_intro: (x: U) (In B x) -> (Non_empty B).
   
   Inductive Finite : Ensemble -> Prop :=
         Empty_is_finite: (Finite Empty_set)
      | Union_is_finite:
          (A: Ensemble) (Finite A) ->
           (x: U) ~ (In A x) -> (Finite (Union (Singleton x) A)).

 (*************************************************************************)
 (*** Initialement, la definition de Same_set etait :
   
   Inductive Same_set [B,C: Ensemble]: Prop :=
         Same_set_intro: (Included B C) -> (Included C B) -> (Same_set B C).

  *** mais dans le lemme Same_set_equivalence de ps.v (par exemple)
      	(Same_set U)
      n'est plus unifie avec
      	[x,y:(Ensemble U)](and (Included x y) (Included y x))

  *** on redefinit donc Same_set et Same_Set_intro de la maniere
      suivante : ***)
 (*************************************************************************)

   Definition Same_set : Ensemble -> Ensemble -> Prop :=
      [B,C:Ensemble](and (Included B C) (Included C B)).
      
   Lemma Same_set_intro : (B,C:Ensemble)
      (Included B C) -> (Included C B) -> (Same_set B C).
   Intros B C H H'.
   Unfold Same_set.
   Split; [Exact H | Exact H'].
   Qed.

 (*************************************************************************)

   Axiom Extensionality_Ensembles:
      (A: Ensemble) (B: Ensemble) (Same_set A B) -> <Ensemble> A == B.
   
End Ensembles.
Hint Unfold In.
Hint Unfold Included.
Hint Same_set_intro.
Hint Union_introl Union_intror.
Hint Intersection_intro.
Hint In_singleton.
Hint Couple_l Couple_r.
Hint Setminus_intro.
Hint Disjoint_intro.
Hint Empty_is_finite Union_is_finite.
Hint Extensionality_Ensembles.


