(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*      Initial Version: Frederic Prost, July 1993                          *)
(*      Revised Version: Gilles Kahn, September 1993                        *)
(*                               INRIA Sophia-Antipolis, FRANCE             *)
(*      Revised Version: Jean-Christophe Filliatre, January 1995            *)
(*                               ENS-Lyon, FRANCE                           *)
(*                                                                          *)
(****************************************************************************)
(*                          Partial_order_facts.v                           *)
(****************************************************************************)

Require Ensembles.
Require Relations_1.
Require podefs.
Require podefs_1.
Require ps.

Section Lemmas_on_partial_orders.
   Variable U: Type.
   Variable B: (Ensemble U).
   Variable D: (PO U).
   
   Theorem Rel_of_reflexive: (x: U) (Rel_of U D x x).
   Elim D; Simpl; Auto.
   (Intros C R H' H'0; Elim H'0); Auto 10.
   Qed.
   Hint Rel_of_reflexive.
   
   Theorem Rel_of_antisymmetric: (Antisymmetric U (Rel_of U D)).
   Elim D; Simpl; Auto.
   (Intros C R H' H'0; Elim H'0); Auto 10.
   Qed.
   Hint Rel_of_antisymmetric.
   
   Theorem Rel_of_transitive: (Transitive U (Rel_of U D)).
   Elim D; Simpl; Auto.
   (Intros C R H' H'0; Elim H'0); Auto 10.
   Qed.
   Hint Rel_of_transitive.
   
   Theorem Couple_included_in_carrier: (x: U)
              (y: U) (In U (Carrier_of U D) x) -> (In U (Carrier_of U D) y) ->
               (Included U (Couple U x y) (Carrier_of U D)).
   (Intros x y H' H'0; Red); Auto 20.
   (Intros x0 H'1; Elim H'1); Auto 20.
   Qed.
   Hint Couple_included_in_carrier.
   
   Theorem Lub_is_in_Carrier: (bsup: U)
              (X: (Ensemble U))
              (Included U X (Carrier_of U D)) -> (Lub U D X bsup) ->
               (In U (Carrier_of U D) bsup).
   (Intros bsup X H' H'0; Elim H'0); Auto 20.
   (Intro H'1; Elim H'1); Auto 20.
   Qed.
   
   Theorem Singleton_has_lub: (x: U) (In U (Carrier_of U D) x) -> (Lub U D (Singleton U x) x).
   Intros x H'; Auto 20.
   Apply Lub_definition; Auto 10.
   Apply Upper_Bound_definition; Auto 10.
   (Intros y H'0; Elim H'0); Auto 10.
   (Intros y H'0; Elim H'0); Auto 10.
   Qed.
   Hint Singleton_has_lub.
   
   Theorem Empty_set_has_Upper_Bound: (ExT [maj: U] (Upper_Bound U D (Empty_set U) maj)).
   Elim D.
   Intros C R n o.
   Elim n.
   Intros x H'; Exists x.
   Apply Upper_Bound_definition; Auto.
   (Intros y H'0; Elim H'0); Auto.
   Qed.
   Hint Empty_set_has_Upper_Bound.
   
   Theorem Empty_set_has_lub: (Cpo U D) -> (ExT [bsup: U] (Lub U D (Empty_set U) bsup)).
   Intro H'; Elim H'.
   Intro h; Elim h; Intros bot E; Clear h.
   Intro H'0; Exists bot.
   Apply Lub_definition.
   Apply Upper_Bound_definition.
   Elim E; Auto.
   (Intros y H'1; Elim H'1); Auto.
   (Intros y H'1; Elim H'1); Auto.
   Elim E; Auto.
   Qed.
   Hint Empty_set_has_lub.
 
   Theorem Upper_downward_stable: (A: (Ensemble U))
              (B: (Ensemble U))
              (maj: U)
              (Included U A (Carrier_of U D)) ->
              (Included U B (Carrier_of U D)) ->
              (Included U A B) -> (Upper_Bound U D B maj) ->
               (Upper_Bound U D A maj).
   Clear B.
   (Intros A B maj H' H'0 H'1 H'2; Elim H'2); Auto.
   Qed.
   
   Theorem Conditionally_complete_has_a_bottom: (Conditionally_complete U D) -> (ExT [bot: U] (Bottom U D bot)).
   (Intro H'; Elim H'); Auto 10.
   (Intro H'0; Generalize (H'0 (Empty_set U)); Intro h; LApply h;
     [Intro H'1; LApply H'1;
       [Intro h0; Elim h0; Intros bsup E; Clear h H'1 h0; Elim E | Clear h] |
       Clear h]); Auto 10.
   (Intros H'1 H'2; Exists bsup; Apply Bottom_definition); Auto 10.
   Elim H'1; Auto 10.
   (Intros y H'3; Try Assumption); Auto 10.
   Apply H'2; Auto 10.
   Apply Upper_Bound_definition; Auto 10.
   (Intros y0 H'4; Elim H'4); Auto 10.
   Qed.
   Hint Conditionally_complete_has_a_bottom.

   Theorem Compatible_is_reflexive: (Reflexive U (Compatible U D)).
   Red; Intro x; Red.
   (Exists x; Intros H' H'0; Split; [Idtac | Apply Upper_Bound_definition]);
    Auto.
   (Intros y H'1; Elim H'1); Auto.
   Qed.
   
   Theorem Couple_is_symmetric: (x: U) (y: U) <(Ensemble U)> (Couple U x y) == (Couple U y x).
   Intros x y; Apply Extensionality_Ensembles; Apply Same_set_intro; Red;
    (Intros x0 H'; Elim H'); Auto 10.
   Qed.
   
   Theorem Compatible_is_symmetric: (Symmetric U (Compatible U D)).
   Red; Unfold Compatible.
   Intros x y h; Elim h; Intros z E; Clear h.
   Elim Couple_is_symmetric; Exists z; Auto.
   Qed.
   
End Lemmas_on_partial_orders.
Hint Couple_included_in_carrier.
Hint Singleton_has_lub.
Hint Empty_set_has_Upper_Bound.
Hint Empty_set_has_lub.
Hint Conditionally_complete_has_a_bottom.


