(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                           Relations_2_facts.v                            *)
(****************************************************************************)

Require Relations_1.
Require Relations_2.

Theorem Rstar_reflexive: (U: Type) (R: (Relation U)) (Reflexive U (Rstar U R)).
Auto.
Qed.

Theorem Rplus_contains_R: (U: Type) (R: (Relation U)) (contains U (Rplus U R) R).
Auto.
Qed.

Theorem Rstar_contains_R: (U: Type) (R: (Relation U)) (contains U (Rstar U R) R).
(Intros U R; Red); Intros x y H'; Apply Rstar_n with y; Auto.
Qed.

Theorem Rstar_contains_Rplus: (U: Type) (R: (Relation U)) (contains U (Rstar U R) (Rplus U R)).
Intros U R; Red.
Intros x y H'; Elim H'.
(Generalize Rstar_contains_R; Intro T; Red in T); Auto.
(Intros x0 y0 z H'0 H'1 H'2; Apply Rstar_n with y0); Auto.
Qed.

Theorem Rstar_transitive: (U: Type) (R: (Relation U)) (Transitive U (Rstar U R)).
Intros U R; Red.
(Intros x y z H'; Elim H'); Auto.
Intros x0 y0 z0 H'0 H'1 H'2 H'3; Apply Rstar_n with y0; Auto.
Qed.

Theorem Rstar_cases: (U: Type) (R: (Relation U)) (x,y: U) (Rstar U R x y) ->
            <U> x == y \/ ((ExT [u: U] (R x u) /\ (Rstar U R u y))).
(Intros U R x y H'; Elim H'); Auto.
Intros x0 y0 z H'0 H'1 H'2; Right; Exists y0; Auto.
Qed.

Theorem Rstar_equiv_Rstar1: (U: Type) (R: (Relation U))
            (same_relation U (Rstar U R) (Rstar1 U R)).
Generalize Rstar_contains_R; Intro T; Red in T.
Intros U R; Unfold same_relation contains.
Split; Intros x y H'; Elim H'; Auto.
Generalize Rstar_transitive; Intro T1; Red in T1.
(Intros x0 y0 z H'0 H'1 H'2 H'3; Apply T1 with y0); Auto.
(Intros x0 y0 z H'0 H'1 H'2; Apply Rstar1_n with y0); Auto.
Qed.

Theorem Rsym_imp_Rstarsym: (U: Type) (R: (Relation U)) (Symmetric U R) ->
            (Symmetric U (Rstar U R)).
Intros U R H'; Red.
(Intros x y H'0; Elim H'0); Auto.
Intros x0 y0 z H'1 H'2 H'3.
Generalize Rstar_transitive; Intro T; Red in T.
Apply T with y0; Auto.
Apply Rstar_n with x0; Auto.
Qed.

Theorem Sstar_contains_Rstar: (U: Type) (R,S: (Relation U)) (contains U (Rstar U S) R) ->
            (contains U (Rstar U S) (Rstar U R)).
Unfold contains.
(Intros U R S H' x y H'0; Elim H'0); Auto.
Generalize Rstar_transitive; Intro T; Red in T.
(Intros x0 y0 z H'1 H'2 H'3; Apply T with y0); Auto.
Qed.

Theorem star_monotone: (U: Type) (R,S: (Relation U)) (contains U S R) ->
            (contains U (Rstar U S) (Rstar U R)).
Intros U R S H'.
Apply Sstar_contains_Rstar.
Generalize (Rstar_contains_R U S); Auto.
Qed.

Theorem RstarRplus_RRstar: (U: Type)
           (R: (Relation U)) (x,y,z: U) (Rstar U R x y) -> (Rplus U R y z) ->
            (ExT [u: U] (R x u) /\ (Rstar U R u z)).
Generalize Rstar_contains_Rplus; Intro T; Red in T.
Generalize Rstar_transitive; Intro T1; Red in T1.
Intros U R x y z H'; Elim H'.
Intros x0 H'0; Elim H'0.
Intros; Exists y0; Auto.
Intros; Exists y0; Auto.
Intros; Exists y0; Auto.
Split; [Try Assumption | Idtac].
Apply T1 with z0; Auto.
Qed.

Theorem Lemma1: (U: Type) (R: (Relation U)) (Strongly_confluent U R) ->
            (x,b: U) (Rstar U R x b) ->
             (a: U) (R x a) -> (ExT [z: U] (Rstar U R a z) /\ (R b z)).
Intros U R H' x b H'0; Elim H'0.
Intros x0 a H'1; Exists a; Auto.
Intros x0 y z H'1 H'2 H'3 a H'4.
Red in H'.
(Generalize (H' x0 a y); Intro h; LApply h;
  [Intro H'5; LApply H'5;
    [Intro h0; Elim h0; Intros t h1; Elim h1; Intros H'6 H'7; Clear h H'5 h0 h1;
      Try Exact H'6 | Clear h] | Clear h]); Auto 10.
(Generalize (H'3 t); Intro h; LApply h;
  [Intro h0; Elim h0; Intros z1 h1; Elim h1; Intros H'5 H'8; Clear h h0 h1;
    Try Exact H'5 | Clear h]); Auto 10.
Intros; Exists z1; Split; [Idtac | Assumption].
Apply Rstar_n with t; Auto.
Qed.
