(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                   ps.v                                   *)
(****************************************************************************)

Require Classical.

Require Ensembles.
Require Relations_1.
Require Relations_1_facts.
Require podefs.
Require podefs_1.

Section The_power_set_partial_order.
   Variable U: Type.
   
   Inductive Power_set [A: (Ensemble U)]: (Ensemble (Ensemble U)) :=
         Definition_of_Power_set:
           (X: (Ensemble U)) (Included U X A) ->
            (In (Ensemble U) (Power_set A) X).
   Hint Definition_of_Power_set.
   
   (* Consider a set A, and its powerset P  *)
   Variable A: (Ensemble U).
   
   Theorem Empty_set_minimal: (X: (Ensemble U)) (Included U (Empty_set U) X).
   Proof.
   Red; Intros X x H'; Elim H'.
   Qed.
   Hint Empty_set_minimal.
   
   Theorem Power_set_non_empty: 
           (A: (Ensemble U)) (Non_empty (Ensemble U) (Power_set A)).
   Proof.
   Intro A'; Apply Non_empty_intro with (Empty_set U); Auto.
   Qed.
   Hint Power_set_non_empty.
   
   Theorem Inclusion_is_an_order: (Order (Ensemble U) (Included U)).
   Proof.
   Auto 8.
   Qed.
   Hint Inclusion_is_an_order.
   
   Theorem Inclusion_is_transitive: (Transitive (Ensemble U) (Included U)).
   Elim Inclusion_is_an_order; Auto.
   Qed.
   Hint Inclusion_is_transitive.
   
   Theorem Same_set_equivalence: (Equivalence (Ensemble U) (Same_set U)).
   Proof.
   (Generalize (Equiv_from_order (Ensemble U) (Included U)) ; Intro H'; Elim H');
    Auto.
   Qed.

   Theorem Same_set_reflexive: (Reflexive (Ensemble U) (Same_set U)).
   Proof.
   Elim Same_set_equivalence; Auto.
   Qed.
   Hint Same_set_reflexive.
   
   Theorem Power_set_PO: (PO (Ensemble U)).
   Proof.
   Apply Definition_of_PO with (Power_set A) (Included U); Auto.
   Qed.
   Transparent Power_set_PO.
   
   Theorem Union_minimal: 
       (a,b,X: (Ensemble U)) (Included U a X) -> (Included U b X) ->
               (Included U (Union U a b) X).
   Proof.
   (Intros a b X H' H'0; Red); (Intros x H'1; Elim H'1); Auto.
   Qed.
   Hint Union_minimal.
   
   Theorem Intersection_maximal: 
       (a,b,X: (Ensemble U)) (Included U X a) -> (Included U X b) ->
               (Included U X (Intersection U a b)).
   Proof.
   Auto.
   Qed.
   
   Theorem Union_increases_l: (a,b: (Ensemble U)) (Included U a (Union U a b)).
   Proof.
   Auto.
   Qed.
   
   Theorem Union_increases_r: (a,b: (Ensemble U)) (Included U b (Union U a b)).
   Proof.
   Auto.
   Qed.
   
   Theorem Intersection_decreases_l: 
         (a,b: (Ensemble U)) (Included U (Intersection U a b) a).
   Proof.
   (Intros a b; Red); Auto.
   (Intros x H'; Elim H'); Auto.
   Qed.
   
   Theorem Intersection_decreases_r:
        (a,b: (Ensemble U)) (Included U (Intersection U a b) b).
   Proof.
   (Intros a b; Red); Auto.
   (Intros x H'; Elim H'); Auto.
   Qed.
   Hint Union_increases_l Union_increases_r Intersection_decreases_l
        Intersection_decreases_r.
   
   Theorem Empty_set_is_Bottom: 
           (Bottom (Ensemble U) Power_set_PO (Empty_set U)).
   Proof.
   Apply Bottom_definition; Simpl; Auto.
   Qed.
   Hint Empty_set_is_Bottom.
  
   Theorem Union_is_Lub: 
       (a,b: (Ensemble U)) (Included U a A) -> (Included U b A) ->
               (Lub
                  (Ensemble U) Power_set_PO (Couple (Ensemble U) a b)
                  (Union U a b)).
   Proof.
   Intros a b H' H'0.
   Apply Lub_definition; Simpl.
   Apply Upper_Bound_definition; Simpl.
   Auto.
   (Intros y H'1; Elim H'1); Auto.
   (Intros y H'1; Elim H'1); Simpl; Auto.
   Qed.


   Theorem Intersection_is_Glb: 
       (a,b: (Ensemble U)) (Included U a A) -> (Included U b A) ->
               (Glb
                  (Ensemble U) Power_set_PO (Couple (Ensemble U) a b)
                  (Intersection U a b)).
   Proof.
   Intros a b H' H'0.
   Apply Glb_definition.
   Apply Lower_Bound_definition; Simpl.
   Apply Definition_of_Power_set; Auto.
   Generalize Inclusion_is_transitive; Intro IT; Red in IT; Apply IT with a;
    Auto.
   (Intros y H'1; Elim H'1); Auto.
   (Intros y H'1; Elim H'1); Simpl; Auto.
   Qed.
   
   Theorem Empty_set_zero: 
       (X: (Ensemble U)) (Union U (Empty_set U) X) == X.
   Proof.
   Auto 10.
   Qed.
   
   Theorem Union_commutative: 
    (A: (Ensemble U)) (B: (Ensemble U))(Union U A B) == (Union U B A).
   Proof.
   Auto.
   Qed.
   
   Theorem Union_associative: (A,B,C: (Ensemble U))
                    (Union U (Union U A B) C) == (Union U A (Union U B C)).
   Proof.
   Auto 20.
   Qed.
   
   Theorem Non_disjoint_union: (X: (Ensemble U)) (x: U) (In U X x) ->
                 (Union U (Singleton U x) X) == X.
   Proof.
   (Intros X x H'; Try Assumption); Auto 10.
   Apply Extensionality_Ensembles; Unfold Same_set; Split; Auto.
   Red; Auto 10.
   (Intros x0 H'0; Elim H'0); Auto 10.
   (Intros x1 H'1; Elim H'1); Auto 10.
   Qed.
   
   Theorem Finite_plus_one_is_finite: (X: (Ensemble U)) (x: U) (Finite U X) ->
               (Finite U (Union U (Singleton U x) X)).
   Proof.
   Intros X x H'.
   (Generalize (classic (In U X x)); Intro h; Elim h;
     [Intro H'0; Clear h; Try Exact H'0 | Clear h; Intro H'0]); Auto 10.
   (Generalize (Non_disjoint_union X x); Intro h; LApply h;
     [Intro H'1; Rewrite -> H'1; Clear h | Clear h]); Auto.
   Qed.
   Hint Finite_plus_one_is_finite.
  
   Theorem Singleton_is_finite: (x: U) (Finite U (Singleton U x)).
   Proof.
   Intro x; Generalize (Empty_set_zero (Singleton U x)); Intro h; Rewrite <- h;
    Clear h.
   (Generalize (Union_commutative (Empty_set U) (Singleton U x)); Intro h;
     Rewrite -> h; Clear h); Auto.
   Qed.
   Hint Singleton_is_finite.
   
   Theorem Union_of_finite_is_finite: 
         (X,Y: (Ensemble U)) (Finite U X) -> (Finite U Y) ->
               (Finite U (Union U X Y)).
   Proof.
   Intros X Y H'; Elim H'.
   (Generalize (Empty_set_zero Y); Intro h; Rewrite -> h; Clear h); Auto.
   Clear A.
   Intros A H'0 H'1 x H'2 H'3.
   (Generalize (Union_associative (Singleton U x) A Y); Intro h; Rewrite -> h;
     Clear h); Auto.
   Qed.
   
End The_power_set_partial_order.
Hint Empty_set_minimal.
Hint Power_set_non_empty.
Hint Inclusion_is_an_order.
Hint Inclusion_is_transitive.
Hint Same_set_reflexive.
Hint Union_minimal.
Hint Same_set_reflexive.
Hint Union_increases_l.
Hint Union_increases_r.
Hint Intersection_decreases_l.
Hint Intersection_decreases_r.
Hint Empty_set_is_Bottom.
Hint Empty_set_zero.
Hint Finite_plus_one_is_finite.
Hint Singleton_is_finite.


