(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: Adder.v                                                *)
(* contents: Definition of an adder for two BVs and a carry     *)
(*                                                              *)
(****************************************************************)

Require Export BV.
Require Export FullAdder.

(****************************************************************)

(* Pourquoi ne peut-on ecrire ceci: ?
Recursive Definition BV_full_adder_sum : BV -> BV -> bool -> BV :=
utiliser le type BV en parties gauches ????????? UN BUG DE COQ ??????????
*)

Recursive Definition BV_full_adder_sum: (list bool)->(list bool)->bool->BV :=
    (nil bool) (nil bool) b => nilbv
  | (nil bool) (cons bool vh vt) b =>
	   (consbv (half_adder_sum vh b)
		   (BV_full_adder_sum (nil bool) vt (half_adder_carry vh b)))
  | (cons bool vh vt) (nil bool) b =>
	   (consbv (half_adder_sum vh b)
		   (BV_full_adder_sum vt (nil bool) (half_adder_carry vh b)))
  | (cons bool vh vt) (cons bool wh wt) b =>
	    (consbv (full_adder_sum vh wh b)
		    (BV_full_adder_sum vt wt (full_adder_carry vh wh b))).


Recursive Definition BV_full_adder_carry: (list bool)->(list bool)->bool->bool
									   :=
     (nil bool) (nil bool) b => b
   | (nil bool) (cons bool vh vt) b => 
	(BV_full_adder_carry (nil bool) vt (half_adder_carry vh b))
   | (cons bool vh vt) (nil bool) b =>
	(BV_full_adder_carry vt (nil bool) (half_adder_carry vh b))
   | (cons bool vh vt) (cons bool wh wt) b => 
	(BV_full_adder_carry vt wt (full_adder_carry vh wh b)).

Definition BV_full_adder : BV->BV->bool->BV :=
     [v,w:BV][cin:bool](appbv (BV_full_adder_sum v w cin)
			      (consbv (BV_full_adder_carry v w cin) nilbv)).
Hint Unfold BV_full_adder.

(****************************************************************)

Lemma BV_full_adder_sum_v_nil_false : (v:BV)
	(BV_full_adder_sum v nilbv false)=v.
Unfold nilbv .Induction v.Try Trivial.Intros.
Rewrite -> BV_full_adder_sum_eq3.Rewrite -> half_adder_carry_false.
Rewrite -> half_adder_sum_false.(Rewrite -> H;Auto).
Qed.Hint BV_full_adder_sum_v_nil_false.

Lemma BV_full_adder_carry_v_nil_false : (v:BV)
	(BV_full_adder_carry v nilbv false)=false.
Unfold nilbv.
Induction v.Try Trivial.Intros.
Rewrite -> BV_full_adder_carry_eq3.Rewrite -> half_adder_carry_false.
Try Trivial.
Qed.Hint BV_full_adder_carry_v_nil_false.

Lemma BV_full_adder_sum_sym : (v,w:BV)(cin:bool)
	(BV_full_adder_sum v w cin)=(BV_full_adder_sum w v cin).
Induction v.Induction w.Auto.Intros.
Rewrite -> BV_full_adder_sum_eq2.Rewrite -> BV_full_adder_sum_eq3.
Rewrite -> H.Auto.Induction w.Intro.
Rewrite -> BV_full_adder_sum_eq2.Rewrite -> BV_full_adder_sum_eq3.Rewrite -> H.
Auto.Intros.Repeat Rewrite -> BV_full_adder_sum_eq4.Rewrite -> H.
Do 2 Rewrite -> full_adder_carry_sym1.Do 2 Rewrite -> full_adder_sum_sym1.Auto.
Qed.
				       
Lemma length_BV_full_adder_sum : (v,w:BV)(cin:bool)
	(lengthbv v)=(lengthbv w) ->
	(lengthbv (BV_full_adder_sum v w cin))=(lengthbv v).
Unfold lengthbv .Induction v.Induction w.Intros.Case cin.Simpl.Try Trivial.
Simpl.Try Trivial.
Intros.Absurd (eq ? (length bool (nil bool)) (length bool (cons bool a l))).
Simpl.Discriminate.Exact H0.Induction w.Simpl.Intros.Discriminate H0.
Intros.Simpl.Rewrite -> H.Try Trivial.Generalize H1 .Simpl.Auto.
Qed.

Lemma BV_full_adder_carry_sym : (v,w:BV)(cin:bool)
	(BV_full_adder_carry v w cin)=(BV_full_adder_carry w v cin).
Induction v.Induction w.Auto.Intros.
Rewrite -> BV_full_adder_carry_eq2.Rewrite -> BV_full_adder_carry_eq3.
(Rewrite -> H;Auto).Induction w.Intros.Rewrite -> BV_full_adder_carry_eq2.
Rewrite -> BV_full_adder_carry_eq3.
Rewrite -> H.Auto.Intros.Do 2 Rewrite -> BV_full_adder_carry_eq4.
Rewrite -> H.Rewrite -> full_adder_carry_sym1.Auto.
Qed.

Lemma BV_full_adder_sym : (v,w:BV)(cin:bool)
	(BV_full_adder v w cin)=(BV_full_adder w v cin).
Unfold BV_full_adder .
Intros.
Rewrite -> BV_full_adder_sum_sym.Rewrite -> BV_full_adder_carry_sym.Auto.
Qed.



