(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: AdderProof                                             *)
(* contents: verification of the adder defined in Adder.v       *)
(*                                                              *)
(****************************************************************)

Require Export Adder.

(****************************************************************)

Lemma BV_full_adder_nil_true_ok : (v:BV)
	(BV_to_nat (BV_full_adder v nilbv true)) = (S (BV_to_nat v)).
Unfold nilbv .(Induction v;Auto).Unfold BV_full_adder .Intros.
Rewrite -> BV_full_adder_sum_eq3.Rewrite -> BV_full_adder_carry_eq3.
Rewrite -> (app_eq2 bool).Rewrite -> half_adder_carry_true.
Simpl.Elim a.Unfold appbv  in H.Rewrite -> H.Simpl.Auto.
Rewrite -> BV_full_adder_sum_v_nil_false.
Rewrite -> BV_full_adder_carry_v_nil_false.Rewrite -> BV_to_nat_app2.
Simpl.Elim mult_n_O.Elim plus_n_O.Try Trivial.
Qed.


Lemma BV_full_adder_nil_ok : (v:BV)(cin:bool)
	(BV_to_nat (BV_full_adder v nilbv cin)) =
	(plus (BV_to_nat v) (bool_to_nat cin)).

Induction v.(Induction cin;Auto).
Induction cin.Rewrite -> BV_full_adder_nil_true_ok.Simpl.Auto.
Unfold BV_full_adder .Rewrite -> BV_full_adder_sum_v_nil_false.
Rewrite -> BV_full_adder_carry_v_nil_false.Rewrite -> BV_to_nat_app2.
Simpl.Elim mult_n_O.Elim plus_n_O.Try Trivial.
Qed.

(****************************************************************)

Theorem BV_full_adder_ok : (v,w:BV)(cin:bool)
	(BV_to_nat (BV_full_adder v w cin)) =
	(plus (plus (BV_to_nat v) (BV_to_nat w)) (bool_to_nat cin)).
Induction v.
Intros.
Rewrite -> BV_full_adder_sym.
Simpl.
Rewrite -> BV_full_adder_nil_ok.
Auto.

Unfold BV_full_adder .
Induction w.
Simpl.
Intro.
Rewrite -> H.
Simpl.
Elim plus_n_O.
Elim plus_n_O.
Replace (plus
          (plus (BV_to_nat l) (bool_to_nat (half_adder_carry a cin)))
          (plus (BV_to_nat l) (bool_to_nat (half_adder_carry a cin)))) with (
plus
 (plus (bool_to_nat (half_adder_carry a cin))
   (bool_to_nat (half_adder_carry a cin)))
 (plus (BV_to_nat l) (BV_to_nat l))).
Repeat Rewrite -> plus_assoc_l.
Replace (plus
          (plus (bool_to_nat (half_adder_sum a cin))
            (bool_to_nat (half_adder_carry a cin)))
          (bool_to_nat (half_adder_carry a cin))) with (plus
                                                         (bool_to_nat
                                                           (half_adder_sum
                                                             a cin))
                                                         (plus
                                                           (bool_to_nat
                                                             (half_adder_carry
                                                               a cin))
                                                           (bool_to_nat
                                                             (half_adder_carry
                                                               a cin)))).
Rewrite -> half_adder_ok.
Rewrite -> (plus_permute2 (bool_to_nat a) (bool_to_nat cin)
             (BV_to_nat l)).
Rewrite -> (plus_permute2 (plus (bool_to_nat a) (BV_to_nat l))
             (bool_to_nat cin) (BV_to_nat l)).
Try Trivial.

Try Trivial.

Repeat Rewrite -> plus_assoc_l.
Rewrite -> (plus_permute2 (bool_to_nat (half_adder_carry a cin))
             (bool_to_nat (half_adder_carry a cin)) (BV_to_nat l)).
Rewrite -> (plus_sym (bool_to_nat (half_adder_carry a cin))
             (BV_to_nat l)).
Rewrite -> (plus_permute2
             (plus (BV_to_nat l) (bool_to_nat (half_adder_carry a cin)))
             (bool_to_nat (half_adder_carry a cin)) (BV_to_nat l)).
Try Trivial.

Intros.
Simpl.
Rewrite -> H.
Clear  H.
(Elim cin;Elim a).
Rewrite -> full_adder_carry_sym1.
Rewrite -> full_adder_carry_true.
Rewrite -> full_adder_sum_sym1.
Rewrite -> full_adder_sum_true.
Simpl.
Repeat Rewrite -> plus_n_SO.
Elim plus_n_Sm.
Elim plus_n_Sm.
Simpl.
Elim plus_n_Sm.
Repeat Rewrite -> plus_assoc_l.
Rewrite -> (plus_permute2 (plus (bool_to_nat a0) (BV_to_nat l))
             (BV_to_nat l0) (BV_to_nat l)).
Rewrite -> (plus_sym (bool_to_nat a0) (BV_to_nat l)).
Rewrite -> (plus_permute2 (BV_to_nat l) (bool_to_nat a0) (BV_to_nat l)).
Try Trivial.

Elim a0.
Simpl.
Elim plus_n_Sm.
Simpl.
Elim plus_n_O.
Elim plus_n_Sm.
Elim plus_n_Sm.
Elim plus_n_Sm.
Elim plus_n_O.
Repeat Rewrite -> plus_assoc_l.
Rewrite -> (plus_permute2 (BV_to_nat l) (BV_to_nat l0) (BV_to_nat l)).
Try Trivial.

Simpl.
Repeat Rewrite <- plus_n_Sm.
Repeat Rewrite <- plus_n_O.
Repeat Rewrite -> plus_assoc_l.
Try Try Trivial.
Rewrite -> (plus_permute2 (BV_to_nat l) (BV_to_nat l0) (BV_to_nat l)).
Try Try Trivial.

Elim a0.
Simpl.
Repeat Rewrite <- plus_n_Sm.
Repeat Rewrite <- plus_n_O.
Repeat Rewrite -> plus_assoc_l.
Simpl.
Rewrite -> (plus_permute2 (BV_to_nat l) (BV_to_nat l0) (BV_to_nat l)).
Try Trivial.

Simpl.
Repeat Rewrite <- plus_n_O.
Repeat Rewrite -> plus_assoc_l.
Rewrite -> (plus_permute2 (BV_to_nat l) (BV_to_nat l0) (BV_to_nat l)).
Try Trivial.

Elim a0;Simpl;Repeat Rewrite <- plus_n_Sm;Repeat Rewrite <- plus_n_O;Repeat Rewrite -> plus_assoc_l;Rewrite -> (
plus_permute2 (BV_to_nat l) (BV_to_nat l0) (BV_to_nat l));Try Trivial.

Qed.
(****************************************************************)
