(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: FullAdder.v                                            *)
(* contents: definition and verification of a full adder        *)
(*                                                              *)
(****************************************************************)

(* Full adder sur les bits (bool) : rend la somme ou le carry *)
(* Verification: Theorem full_adder_ok *)

Require Export HalfAdder.

(****************************************************************)

Definition full_adder_sum := [a,b,c:bool]
	(half_adder_sum (half_adder_sum a b) c).

Definition full_adder_carry := [a,b,c:bool]
	(orb (half_adder_carry a b)
	     (half_adder_carry (half_adder_sum a b) c)).

(****************************************************************)

Lemma full_adder_sum_sym1 : (a,b,c:bool)
	(full_adder_sum a b c) = (full_adder_sum b a c).
Induction a;Induction b;Auto.
Qed.Hint full_adder_sum_sym1.

Lemma full_adder_sum_sym2 : (a,b,c:bool)
	(full_adder_sum a b c) = (full_adder_sum a c b).
Induction b.
Induction c.
Auto.
Unfold full_adder_sum .
Rewrite -> half_adder_sum_false.
Rewrite -> half_adder_sum_false.
Auto.
Unfold full_adder_sum .
Rewrite -> half_adder_sum_false.
Intro.
Auto.
Qed.Hint full_adder_sum_sym2.

Lemma full_adder_sum_false : (a:bool)
	(full_adder_sum a false false)=a.
Induction a;Auto.
Qed.Hint full_adder_sum_false.

Lemma full_adder_sum_true : (a:bool)
	(full_adder_sum a true true)=a.
Induction a;Auto.
Qed.Hint full_adder_sum_true.

Lemma full_adder_carry_sym1 : (a,b,c:bool)
	(full_adder_carry a b c) = (full_adder_carry b a c).
Induction a;Induction b;Auto.
Qed.Hint full_adder_carry_sym1.

Lemma full_adder_carry_sym2 : (a,b,c:bool)
	(full_adder_carry a b c) = (full_adder_carry a c b).
Induction b.
Induction c.
Auto.
Unfold full_adder_carry.
Rewrite -> half_adder_sum_false.
Rewrite -> half_adder_carry_false.
Rewrite -> half_adder_carry_false.
Simpl.
Elim (half_adder_carry a true);Auto.
Intros.
Unfold full_adder_carry.
Rewrite -> half_adder_carry_false.
Rewrite -> half_adder_sum_false.
Rewrite -> half_adder_carry_false.
Simpl.
Elim (half_adder_carry a c);Auto.
Qed.Hint full_adder_carry_sym2.

Lemma full_adder_carry_false : (a:bool)
	(full_adder_carry a false false)=false.
Induction a;Auto.
Qed.Hint full_adder_carry_false.

Lemma full_adder_carry_true : (a:bool)
	(full_adder_carry a true true)=true.
Induction a.
Unfold full_adder_carry.
Auto.
Unfold full_adder_carry.
Auto.
Qed.Hint full_adder_carry_true.

Lemma full_adder_carry_true_false : (a:bool)
	(full_adder_carry a true false)=a.
Induction a;Auto.
Qed.Hint full_adder_carry_true_false.

Lemma full_adder_carry_neg : (a,b:bool)
	(full_adder_carry a (negb a) b)=b.
(Induction a;Induction b;Simpl).
Rewrite -> full_adder_carry_sym1.Rewrite -> full_adder_carry_true.Try Trivial.
Rewrite -> full_adder_carry_false.Try Trivial.
Rewrite -> full_adder_carry_true.Try Trivial.
Rewrite -> full_adder_carry_sym1.Rewrite -> full_adder_carry_false.Try Trivial.
Qed.

(****************************************************************)

Theorem full_adder_ok : (a,b,c:bool)
	(plus (bool_to_nat (full_adder_sum a b c))
	      (plus (bool_to_nat (full_adder_carry a b c))
	            (bool_to_nat (full_adder_carry a b c)))) =
	(plus (plus (bool_to_nat a) (bool_to_nat b)) (bool_to_nat c)).
Induction a;Induction b; Induction c;Auto.
Qed.

