(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* file: HalfAdder.v                                            *)
(* contents: definition and verification of an half-adder       *)
(*                                                              *)
(****************************************************************)

(* Half adder sur les bits (bool) : rend la somme ou le carry *)
(* Verification: Theorem half_adder_ok *)

Require Export Arith_compl.
Require Export Bool_compl.

(****************************************************************)

Definition half_adder_sum := [a,b:bool] (xorb a b).
Definition half_adder_carry := [a,b:bool] (andb a b).

(****************************************************************)

Lemma half_adder_sum_sym : (a,b:bool)
	(half_adder_sum a b) = (half_adder_sum b a).
Induction a;Induction b;Auto.
Qed.Hint half_adder_sum_sym.

Lemma half_adder_carry_sym : (a,b:bool)
	(half_adder_carry a b) = (half_adder_carry b a).
Induction a;Induction b;Auto.
Qed.Hint half_adder_carry_sym.

Lemma half_adder_sum_false : (a:bool)
	(half_adder_sum a false)=a.
Induction a;Auto.
Qed.Hint half_adder_sum_false.

Lemma half_adder_carry_false : (a:bool)
	(half_adder_carry a false)=false.
Induction a;Auto.
Qed.Hint half_adder_carry_false.

Lemma half_adder_sum_true : (a:bool)
	(half_adder_sum a true) = (negb a).
Auto.
Qed.Hint half_adder_sum_true.

Lemma half_adder_carry_true : (a:bool)
	(half_adder_carry a true) = a.
Induction a;Auto.
Qed.Hint half_adder_carry_true.

(****************************************************************)

Theorem half_adder_ok : (a,b:bool)
	(plus (bool_to_nat (half_adder_sum a b))
	      (plus (bool_to_nat (half_adder_carry a b))
	            (bool_to_nat (half_adder_carry a b)))) =
	(plus (bool_to_nat a) (bool_to_nat b)).
(Induction a;Induction b;Auto).
Qed.

