(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: october 1995                                           *)
(* file: IncrDecr.v                                             *)
(* contents: definnition of an incrementer and a decrementer    *)
(* on BVs     **** TO BE COMPLETED ****                         *)
(****************************************************************)

Require Export AdderProof.


Recursive Definition BV_increment: (list bool)->BV :=
     (nil bool) => nilbv
   | (cons bool false v) => (cons bool true v)
   | (cons bool true v) => (cons bool false (BV_increment v)).

Recursive Definition BV_increment_carry: (list bool) -> bool :=
     (nil bool) => true
   | (cons bool false v) => false
   | (cons bool true v) => (BV_increment_carry v).

Recursive Definition BV_decrement: (list bool)->BV :=
     (nil bool) => nilbv
   | (cons bool false v) => (cons bool true (BV_decrement v))
   | (cons bool true v) => (cons bool false v).

Recursive Definition BV_decrement_carry: (list bool) -> bool :=
     (nil bool) => true
   | (cons bool false v) => (BV_decrement_carry v)
   | (cons bool true v) => false.






Lemma length_BV_increment : (v:BV) (lengthbv (BV_increment v)) = (lengthbv v).
Induction v.Auto.
Induction a.
Simpl.Intros.(Rewrite -> H;Try Trivial).
Simpl.Intros.(Rewrite -> H;Try Trivial).
Qed.

Lemma length_BV_decrement : (v:BV) (lengthbv (BV_decrement v)) = (lengthbv v).
Induction v.Auto.
Induction a.
Simpl.Intros.(Rewrite -> H;Try Trivial).
Simpl.Intros.(Rewrite -> H;Try Trivial).
Qed.

Lemma BV_increment_limit : (n:nat)
	(BV_increment (list_const bool n true)) = (list_const bool n false).
Induction n.Simpl.Auto.Intros.Simpl.(Rewrite -> H;Try Trivial).
Qed.

Lemma BV_decrement_limit : (n:nat)
	(BV_decrement (list_const bool n false)) = (list_const bool n true).
Induction n.Simpl.Auto.Intros.Simpl.(Rewrite -> H;Try Trivial).
Qed.


Lemma BV_increment_limit_carry : (n:nat)
	(BV_increment_carry (list_const bool n true)) = true.
Induction n.Auto.Intros.Simpl.Exact H.
Qed.

Lemma BV_decrement_limit_carry : (n:nat)
	(BV_decrement_carry (list_const bool n false)) = true.
Induction n.Auto.Intros.Simpl.Exact H.
Qed.


Lemma BV_increment_adder : (v:BV)
  (appbv (BV_increment v) (consbv (BV_increment_carry v) nilbv)) =
  (BV_full_adder v nilbv true).
Induction v.
Simpl.Unfold BV_full_adder .Unfold nilbv consbv appbv .Simpl.Try Trivial.
Induction a.Intros.Unfold consbv nilbv appbv .Simpl.Rewrite -> H.
Unfold consbv nilbv appbv  in H.Rewrite -> H.
Unfold BV_full_adder .Simpl.Replace (half_adder_sum true true) with false.
Try Trivial.
Auto.
Unfold BV_full_adder .Unfold consbv nilbv appbv .Intros.Simpl.
Replace (half_adder_sum false true) with true.
Replace (BV_full_adder_sum l (nil bool) false) with l.
Replace (BV_full_adder_carry l (nil bool) false) with false.
Try Trivial.
Auto.Auto.Auto.
Qed.

Lemma BV_increment_ok : (v:BV)
  (BV_to_nat (appbv (BV_increment v) (consbv (BV_increment_carry v) nilbv))) =
  (S (BV_to_nat v)).
Intro.Rewrite -> BV_increment_adder.Rewrite -> BV_full_adder_ok.
Simpl.Elim plus_n_O.Elim plus_n_Sm.Auto.
Qed.

Lemma BV_decr_incr : (v:BV) (BV_decrement (BV_increment v))=v.
Induction v.Simpl.Try Trivial.
Induction a.Intros.Simpl.(Rewrite -> H;Try Trivial).
Intros.Simpl.Try Trivial.
Qed.

Lemma BV_incr_decr : (v:BV) (BV_increment (BV_decrement v))=v.
Induction v.
Auto.(Induction a;Intros;Simpl).Try Trivial.(Rewrite -> H;Try Trivial).
Qed.

