(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: november 1995                                          *)
(* file: Fill_defs                                              *)
(* contents: proof of a memory block instruction: Fills cx words*)
(* in memory starting at address di with value al.              *)
(* Definitions of operators, sizes...                           *)
(****************************************************************)

Require Export IncrDecr.
Require Export Memo.

Variable a_size, d_size : nat.  (* address and data sizes *)

Variable di_init,cx_init,al_init : BV.
Variable mem_init : Memo.

Axiom di_initsize : (lengthbv di_init)=a_size.
Axiom cx_initsize : (lengthbv cx_init)=a_size.
Axiom al_initsize : (lengthbv al_init)=d_size.
Axiom mem_initsize : (MemoSize mem_init)=a_size.

Fixpoint IsNull [v:BV] : bool :=
  <bool> Case v of
	true
	[b:bool][w:BV] <bool> Case b of
				 false
				(IsNull w) end end.

Lemma IsNull_nil : (IsNull nilbv)=true.
Auto.
Qed.

Lemma IsNull_false : (a:bool)(v:BV) (IsNull (consbv a v))=true -> a=false.
Induction a.Simpl.Auto.Try Trivial.
Qed.

Lemma IsNull_cons : (a:bool)(v:BV) (IsNull (consbv a v))=true ->
					(IsNull v)=true.
Induction a.Simpl.Intros.Absurd (eq ? false true).
Auto.Exact H.Intros v.Auto.
Qed.

Lemma IsNull_null : (n:nat) (IsNull (BV_null n))=true.
Induction n.Simpl.Try Trivial.
Intros.Simpl.Exact H.
Qed.

Lemma IsNull_BV_null : (v:BV) (IsNull v)=true -> v=(BV_null (lengthbv v)).
Induction v.Simpl.Unfold BV_null .Auto.
Intros a l H.Intro.
Change (eq ? (cons bool a l) (cons bool false (BV_null (lengthbv l)))).
Rewrite <- H.Generalize H0 .Replace a with false.Try Trivial.
Symmetry.Apply IsNull_false with v:=l .Exact H0.
Apply IsNull_cons with a:=a .Exact H0.
Qed.


