(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: november 1995                                          *)
(* file: Fill_spec.v                                            *)
(* contents: proof of a memory block instruction: Fills cx words*)
(* in memory starting at address di with value al.              *)
(* Specification level                                          *)
(****************************************************************)

Require Export Fill_defs.

(* Specification:

cx: register a_size
di: register a_size
al: register d_size
mem: memory of d_size bit words, addresses of a_size bits

while not(cx=0) do begin
  mem[di] <- al
  di <- di+1
  cx <- cx-1
end
*)

(****************************************************************)

Fixpoint
di [st:nat] : BV->BV->BV->Memo-> BV := [di0,cx0,al0:BV][mem0:Memo]
  <BV> Case st of
	di0
	[t:nat] <BV> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (di t di0 cx0 al0 mem0)
		       (BV_increment (di t di0 cx0 al0 mem0)) end end
with
cx [st:nat] : BV->BV->BV->Memo-> BV := [di0,cx0,al0:BV][mem0:Memo]
  <BV> Case st of
	cx0
	[t:nat] <BV> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (cx t di0 cx0 al0 mem0)
		       (BV_decrement (cx t di0 cx0 al0 mem0)) end end
with
al [st:nat] : BV->BV->BV->Memo-> BV := [di0,cx0,al0:BV][mem0:Memo]
  <BV> Case st of
	al0
	[t:nat] <BV> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (al t di0 cx0 al0 mem0)
		       (al t di0 cx0 al0 mem0) end end
with
mem [st:nat] : BV->BV->BV->Memo-> Memo := [di0,cx0,al0:BV][mem0:Memo]
  <Memo> Case st of
	mem0
	[t:nat] <Memo> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (mem t di0 cx0 al0 mem0)
		       (MemoWrite (mem t di0 cx0 al0 mem0)
				  (BV_to_nat (di t di0 cx0 al0 mem0))
				  (al t di0 cx0 al0 mem0)) end end.

(****************************************************************)
(* Valeurs generales des registres *)

Lemma di_t : (t:nat) (di0,cx0,al0:BV)(mem0:Memo)
	(di (S t) di0 cx0 al0 mem0) =
	(<BV> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (di t di0 cx0 al0 mem0)
		       (BV_increment (di t di0 cx0 al0 mem0)) end).
Auto.
Qed.

Lemma cx_t : (t:nat) (di0,cx0,al0:BV)(mem0:Memo)
	(cx (S t) di0 cx0 al0 mem0) =
	(<BV> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (cx t di0 cx0 al0 mem0)
		       (BV_decrement (cx t di0 cx0 al0 mem0)) end).
Auto.
Qed.

Lemma al_t : (t:nat) (di0,cx0,al0:BV)(mem0:Memo)
	(al (S t) di0 cx0 al0 mem0) =	
	(<BV> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (al t di0 cx0 al0 mem0)
		       (al t di0 cx0 al0 mem0) end).
Auto.
Qed.

Lemma al_constant : (t:nat) (di0,cx0,al0:BV)(mem0:Memo)
	(al t di0 cx0 al0 mem0)=al0.
Induction t.Auto.
Intros.Rewrite -> al_t.Elim (IsNull (cx n di0 cx0 al0 mem0)).Apply H.Apply H.
Qed.

Lemma mem_t : (t:nat) (di0,cx0,al0:BV)(mem0:Memo)
	(mem (S t) di0 cx0 al0 mem0) =
	(<Memo> Case (IsNull (cx t di0 cx0 al0 mem0)) of
                       (mem t di0 cx0 al0 mem0)
		       (MemoWrite (mem t di0 cx0 al0 mem0)
				  (BV_to_nat (di t di0 cx0 al0 mem0))
				  (al t di0 cx0 al0 mem0)) end).
Auto.
Qed.
(****************************************************************)
(* Longueurs des registres *)

Lemma length_di : (t:nat)
	(lengthbv (di t di_init cx_init al_init mem_init))=a_size.
Induction t.Simpl.Exact di_initsize.
Intros.
Rewrite -> di_t.Elim (IsNull (cx n di_init cx_init al_init mem_init)).Exact H.
Rewrite -> length_BV_increment.Exact H.
Qed.

Lemma length_cx : (t:nat)
	(lengthbv (cx t di_init cx_init al_init mem_init))=a_size.
Induction t.Simpl.Exact cx_initsize.
Intros.
Rewrite -> cx_t.Elim (IsNull (cx n di_init cx_init al_init mem_init)).Exact H.
Rewrite -> length_BV_decrement.Exact H.
Qed.

Lemma length_al : (t:nat)
	(lengthbv (al t di_init cx_init al_init mem_init))=d_size.
Intro.Rewrite -> al_constant.Exact al_initsize.
Qed.
