(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: july 1995                                              *)
(* file: Definitions.v                                          *)
(* contents: PROOF OF A SEQUENTIAL MULTIPLIER. Definitions of   *)
(* auxilliary functions, axioms and the register-functions.     *)
(****************************************************************)

(****************************************************************)
(* Definitions des operateurs utilises (lowbit et highs)
   des registres (R1 et R2)
   et des lemmes sur les longueurs des registres
*)
   
Require Export BV.
Require Export Adder.
Require Gt.

Definition Mux := (If BV).
Hint Unfold Mux.

Recursive Definition lowbit : (list bool) -> bool :=
	(nil bool) => false
	|(cons bool b _) => b.

Recursive Definition highs : (list bool) -> BV :=
	(nil bool) => (nil bool)
	|(cons bool _ v) => v.

Lemma lowbit_is_trunc : (v:BV)
	~v=nilbv -> (consbv (lowbit v) nilbv)=(truncbv v (S O)).
Induction v.Intro.(Absurd (not (eq ? (nil bool) nilbv));Unfold not ;Auto).
Intros.Simpl.Rewrite -> (trunc_O bool).Try Trivial.
Qed.

Lemma lowbit_is_abit : (v:BV)
	~v=nilbv -> (consbv (lowbit v) nilbv)=(abit v O).
Intros.Unfold abit.Unfold elemlist .Rewrite -> strip_O.
Apply lowbit_is_trunc.Exact H.
Qed.

Lemma highs_is_strip : (v:BV) (highs v)=(stripbv v (S O)).
Induction v.Simpl.Auto.
Intros.Simpl.Rewrite -> (strip_cons_S bool).Rewrite -> strip_O.Try Trivial.
Qed.

Lemma app_lowbit_highs : (v:BV)
	~(v=nilbv) -> (appbv (consbv (lowbit v) nilbv) (highs v))=v.
Intros.Rewrite -> lowbit_is_trunc.Rewrite -> highs_is_strip.
Rewrite -> (app_trunc_strip bool).Try Trivial.Exact H.
Qed.

Lemma length_highs : (v:BV)
	~(v=nilbv) -> (lengthbv (highs v))=(pred (lengthbv v)).
Intros.Rewrite -> highs_is_strip.
Rewrite -> (length_strip bool).Apply minus_n_SO.
Apply (v_not_nil_length bool).Exact H.
Qed.

Lemma length_abit : (v:BV)(i:nat)
	(lt i (lengthbv v)) -> (lengthbv (abit v i))=(S O).
Unfold abit .Exact (length_elemlist bool).
Qed.
(****************************************************************)
(* Defintitions des variables et leurs contraintes *)

Variable size : nat. (* La taille des mots *)
Variable V1 : BV. (* Les deux entrees *)
Variable V2 : BV.

Axiom size_not_O : ~size=O. Hint size_not_O.
Axiom length_V1_size : (lengthbv V1)=size. Hint length_V1_size.
Axiom length_V2_size : (lengthbv V2)=size. Hint length_V2_size.

Lemma le_SO_size : (le (S O) size).
       Generalize size_not_O.Elim size.Intro.Absurd (not (eq ? O O));Unfold not;Auto.
       Intros.Apply le_n_S.Auto. Qed. Hint le_SO_size.

Lemma length_V1	: ~(lengthbv V1)=O.
	Rewrite -> length_V1_size.Auto.Qed. Hint length_V1.
Lemma length_V2	: ~(lengthbv V2)=O.
	Rewrite -> length_V2_size.Auto.Qed. Hint length_V2.
Lemma length_V2_V1 : (lengthbv V2)=(lengthbv V1).
	(Transitivity size;Auto). Qed.
Lemma V1_not_nil : ~V1=nilbv.
	Apply (not_nil bool). Auto. Qed. Hint V1_not_nil.
Lemma V2_not_nil : ~V2=nilbv.
	Apply (not_nil bool). Auto. Qed. Hint V2_not_nil.
Lemma le_SO_length_V1 :	(le (S O) (lengthbv V1)).
	Apply (le_SO_length_v bool).Auto. Qed. Hint le_SO_length_V1.

(****************************************************************)
(* Les registres: *)

Fixpoint R1 [st:nat] : BV :=
  <BV> Case st of
	V1
	[t:nat]
	(appbv (highs (R1 t))
	       (Mux (lowbit (R1 t))
		    (consbv (lowbit (BV_full_adder_sum (R2 t) V2 false)) nilbv)
		    (consbv (lowbit (R2 t)) nilbv)))
	end

with R2 [st:nat] : BV :=
  <BV> Case st of
	(BV_null size)
	[t:nat]
	(appbv (highs (Mux (lowbit (R1 t))
		           (BV_full_adder_sum (R2 t) V2 false)
		           (R2 t)))
	       (Mux (lowbit (R1 t))
		    (consbv (BV_full_adder_carry (R2 t) V2 false) nilbv)
		    (consbv false nilbv)))
	end.


Lemma R1_eq1 : (R1 O)=V1. Auto.Qed.
Lemma R1_eq2 : (t:nat)
	(R1 (S t))=
	(appbv (highs (R1 t))
	       (Mux (lowbit (R1 t))
		    (consbv (lowbit (BV_full_adder_sum (R2 t) V2 false)) nilbv)
		    (consbv (lowbit (R2 t)) nilbv))).
Auto.Qed.

Lemma R2_eq1 : (R2 O)=(BV_null size). Auto.Qed.
Lemma R2_eq2 : (t:nat)
	(R2 (S t))=
	(appbv (highs (Mux (lowbit (R1 t))
		           (BV_full_adder_sum (R2 t) V2 false)
		           (R2 t)))
	       (Mux (lowbit (R1 t))
		    (consbv (BV_full_adder_carry (R2 t) V2 false) nilbv)
		    (consbv false nilbv))).
Auto.Qed.

(****************************************************************)
Lemma length_R1 : (t:nat) (le t size) -> (lengthbv (R1 t))=size.
Induction t.Auto.
Intros.Rewrite -> R1_eq2.Rewrite -> (length_app bool).
Unfold Mux .Rewrite -> (F_If BV nat).Simpl.Rewrite -> If_eq.
Rewrite -> highs_is_strip.Rewrite -> (length_strip bool).Unfold lengthbv  in H.
Rewrite -> H.Symmetry.Rewrite -> plus_sym.Apply le_plus_minus.Auto.
(Apply le_trans_S;Auto).
Unfold lengthbv  in H.Rewrite -> H.Auto.
Apply le_trans_S.Exact H0.
Qed. Hint length_R1.

Lemma length_R2 : (t:nat) (le t size) -> (lengthbv (R2 t))=size.
Induction t.Simpl.
Unfold lengthbv BV_null .Rewrite -> (length_list_const bool).Try Trivial.
Unfold lengthbv .Intros.Rewrite -> R2_eq2.
Rewrite -> (length_app bool).Rewrite -> (F_If BV BV).Rewrite -> (F_If BV nat).
Rewrite -> highs_is_strip.Rewrite -> (length_strip bool).
Rewrite -> length_BV_full_adder_sum.Unfold lengthbv .
Rewrite -> H.Rewrite -> highs_is_strip.Rewrite -> (length_strip bool).
Rewrite -> H.Rewrite -> If_eq.Rewrite -> (F_If BV nat).Simpl.
Rewrite -> If_eq.Symmetry.Rewrite -> plus_sym.Apply le_plus_minus.Auto.
Auto.Rewrite -> H.Auto.
(Apply le_trans_S;Exact H0).(Apply le_trans_S;Exact H0).
Unfold lengthbv .Rewrite -> H.Auto.(Apply le_trans_S;Exact H0).
Rewrite -> length_BV_full_adder_sum.
Unfold lengthbv .Rewrite -> H.Auto.(Apply le_trans_S;Exact H0).
Unfold lengthbv .Rewrite -> H.Auto.(Apply le_trans_S;Exact H0).
Qed. Hint length_R2.

Lemma R1_never_nil : (t:nat)	(le t size) -> ~(R1 t)=nilbv.
Intros.Apply (not_nil bool).Rewrite -> length_R1.Auto.Exact H.
Qed.


