(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: july 1995                                              *)
(* file: LemPrelim.v                                            *)
(* contents: VERIFICATION OF A SEQUENTIAL MULTIPLIER. Proofs of *)
(* some preliminary lemmas.                                     *)
(****************************************************************)

(* Des lemmes preliminaires *)

Require Export Definitions.

Lemma R1_lem : (t:nat)  (* R1(t)[0:n-1-t]=B[t:n-1] *)
	(le t size) -> (truncbv (R1 t) (minus size t))=(stripbv V1 t).
Induction t.Elim minus_n_O.Rewrite <- (length_R1 O).
Intro.Rewrite -> (trunc_all bool).Rewrite -> (strip_O bool).Auto.
Auto.Intros.Rewrite -> R1_eq2.Rewrite -> highs_is_strip.
Rewrite -> (trunc_app bool).Rewrite -> (trunc_strip bool).
Rewrite -> (length_strip bool).Rewrite -> length_R1.
Replace (minus (minus size (S n)) (minus size (S O))) with O.
Rewrite -> (trunc_O bool).
Replace (plus (S O) (minus size (S n))) with (minus size n).
Rewrite -> (app_v_nil bool).Unfold truncbv  in H.
Rewrite -> H.Rewrite -> (strip_strip bool).Rewrite -> plus_sym.Simpl.Auto.
(Apply le_trans_S;Exact H0).
Simpl.Rewrite -> minus_Sn_m.Simpl.Try Trivial.
Exact H0.
Symmetry.Apply minus_le_O.Apply le_minus_minus.Auto.
(Apply le_trans_S;Exact H0).
Rewrite -> length_R1.Apply le_SO_size.(Apply le_trans_S;Exact H0).
Qed.

(*
Lemma R1_lem2 : (t,i:nat) (* R1(t)[i]=B[t+i] *)
	(le t size) -> (lt i size)  ->
	(abit (R1 t) i) = (abit v1 (plus t i)).
*)


Lemma R1_lem3 : (t:nat) (* R1(t)[0]=B[t] *)
	(lt t size) -> (abit (R1 t) O) = (abit V1 t).
Intros.Unfold abit .Unfold elemlist .
Apply (trunc_plus_petit bool) with (minus size t) .
Unfold lt  in H.Inversion H.Rewrite <- minus_Sn_m.Auto.Auto.
Rewrite <- minus_Sn_m.Auto.(Apply le_trans_S;Auto).
Rewrite -> (strip_O bool).Apply R1_lem.Apply lt_le_weak.Exact H.
Qed.

(****************************************************************)

Lemma lem_le_minus_v1_Sn : (n:nat) 
	  (le (S n) size)  -> (le n size) ->
 (le (minus size (S n))
     (lengthbv (appbv (highs (R1 n)) (consbv (lowbit (R2 n)) nilbv)))).

Intros.Rewrite -> (length_app bool).
Rewrite -> highs_is_strip.Rewrite -> (length_strip bool).Rewrite -> length_R1.
Simpl.Rewrite -> plus_sym.Rewrite <- le_plus_minus.Apply minus_le_lem2.
Exact le_SO_size.Exact H0.Rewrite -> length_R1.Exact le_SO_size.Exact H0.
Qed.

Lemma BV_to_nat_lem0 : (v:BV)
	 ~(lengthbv v)=O ->
   (BV_to_nat v)
    =(plus (BV_to_nat (consbv (lowbit v) nilbv))
       (mult (power2 (S O)) (BV_to_nat (highs v)))).

Intros.Rewrite -> lowbit_is_trunc.Rewrite -> highs_is_strip.
Replace (power2 (S O)) with (power2 (length bool (trunc bool v (S O)))).
Rewrite <- BV_to_nat_app2.Rewrite -> (app_trunc_strip bool).Try Trivial.
Rewrite -> (length_trunc bool).Try Trivial.
Apply (le_SO_length_v bool).Exact H.Apply (not_nil bool).Exact H.
Qed.

Lemma BV_nat_lem1 : (v:BV)(n:nat)
	 ~(lengthbv v)=O -> 
	(plus (mult (power2 n) (BV_to_nat (consbv (lowbit v) nilbv)))
	      (mult (power2 (S n)) (BV_to_nat (highs v)))) =
	(mult (power2 n) (BV_to_nat v)).

Intros.Rewrite -> power2_eq2.Rewrite -> mult_plus_distr.
Replace (BV_to_nat v) with (BV_to_nat
                             (appbv (consbv (lowbit v) nilbv) (highs v))).
Rewrite -> BV_to_nat_app2.
Rewrite -> (mult_sym (power2 n)
             (plus (BV_to_nat (consbv (lowbit v) nilbv))
               (mult (power2 (lengthbv (consbv (lowbit v) nilbv)))
                 (BV_to_nat (highs v))))).
Rewrite -> mult_plus_distr.
Simpl.Elim plus_n_O.Elim plus_n_O.Rewrite -> mult_plus_distr.
Rewrite -> (mult_sym (power2 n)).Rewrite -> (mult_sym (power2 n)).Try Trivial.
Rewrite -> app_lowbit_highs.Try Trivial.
Apply (not_nil bool).Exact H.
Qed.

