(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: july 1995                                              *)
(* file: MultSeq.v                                              *)
(* contents: VERIFICATION OF A SEQUENTIAL MULTIPLIER.           *)
(* Proofs of the invariant and the final theorem                *)
(****************************************************************)

Require  Definitions.
Require  LemPrelim.
Require  AdderProof.


(* L'invariant est :
si t<= size alors nat(R1(t)[size-t:size-1]@R2(t))=nat(V2)*nat(V1[0:t-1])
*)

(****************************************************************)
(* D'abord on prouve l'invariant pour t=O: *)

Lemma Invariant_t_O :
	(BV_to_nat (appbv (stripbv (R1 O) size) (R2 O))) =
	(mult (BV_to_nat V2) (BV_to_nat (truncbv V1 O))).
Intros.Simpl.
Rewrite -> BV_to_nat_app2.Rewrite -> (trunc_O bool).Rewrite <- length_V1_size.
Rewrite -> (strip_all bool).Rewrite -> length_V1_size.
Simpl.Elim plus_n_O.Elim mult_n_O.Rewrite -> BV_null_nat.Try Trivial.
Qed.

(****************************************************************)
(* Ensuite, lors de la preuve par induction pour t quelconque,
   on doit faire une preuve par cas selon R1[0].
   On fait la preuve pour R1[0]=false:  (assez long)
*)

Lemma inv_ind_false :	(t,n:nat)
    ( (le n size) ->
      (BV_to_nat (appbv (stripbv (R1 n) (minus size n)) (R2 n)))
       =(mult (BV_to_nat V2) (BV_to_nat (truncbv V1 n))) ) ->
	(le (S n) size)  -> (le n size) ->
   (BV_to_nat
     (appbv
       (stripbv
         (appbv (highs (R1 n))
           (Mux false
             (abit (BV_full_adder_sum (R2 n) V2 false) O)
             (consbv (lowbit (R2 n)) nilbv)))
         (minus size (S n)))
       (appbv
         (highs
           (Mux false
             (BV_full_adder_sum (R2 n) V2 false)
             (R2 n)))
         (Mux false
           (consbv (BV_full_adder_carry (R2 n) V2 false) nilbv)
           (consbv false nilbv))))) =
   (mult (BV_to_nat V2) (BV_to_nat (appbv (truncbv V1 n) (consbv false nilbv)))).
Intros.
Simpl.
Do 3 Rewrite -> BV_to_nat_app2.
Simpl.
Elim mult_n_O.
Elim mult_n_O.
Elim plus_n_O.
Elim plus_n_O.
Rewrite -> (length_strip bool).
Rewrite <- H.
Clear  H.
Rewrite -> (length_app bool).
Rewrite -> highs_is_strip.
Rewrite -> (length_strip bool).
Rewrite -> length_R1.
Simpl.
Replace (plus (minus size (S O)) (S O)) with size.
Rewrite -> minus_minus_lem2.
Rewrite -> (strip_app bool).
Rewrite -> BV_to_nat_app2.
Rewrite -> (strip_strip bool).
Rewrite -> (length_strip bool).
Rewrite -> length_R1.
Rewrite -> (length_strip bool).
Rewrite -> BV_to_nat_app2.
Rewrite -> (length_strip bool).
Rewrite -> length_R1.
Rewrite -> minus_minus_lem2.
Replace (plus (S O) (minus size (S n))) with (minus size n).
Rewrite -> minus_minus_lem2.
Rewrite -> (minus_le_O (minus size (S n)) (minus size (S O))).
Rewrite -> (strip_O bool).
Rewrite -> plus_assoc_r.
Replace (plus
          (mult (power2 n) (BV_to_nat (consbv (lowbit (R2 n)) nilbv)))
          (mult (power2 (S n)) (BV_to_nat (highs (R2 n))))) with (
mult (power2 n) (BV_to_nat (R2 n))).
Try Trivial. Symmetry. Apply BV_nat_lem1.
Rewrite -> length_R2. Exact size_not_O. Exact H1. (Apply le_minus_minus;Auto).
Exact H1. Simpl. Rewrite -> minus_Sn_m. Auto.
Exact H0. Exact H1. Exact H1. Rewrite -> length_R1. Auto. Exact H1. 
Rewrite -> length_R1. Auto. Exact H1. Exact H1. Rewrite -> length_R1.
Simpl. Rewrite -> minus_Sn_m. Simpl. Auto. Exact H0. Exact H1. 
Exact H0. Rewrite -> plus_sym. Simpl. (Rewrite -> minus_Sn_m;Auto).
 Exact H1. Rewrite -> length_R1. Auto. Exact H1. Exact H1. 
Rewrite -> (length_app bool). Rewrite -> length_highs. Rewrite -> length_R1.
Simpl. Rewrite -> plus_n_SO. Replace (pred size) with (minus size (S O)).
Rewrite -> minus_Sn_m. Simpl. Elim minus_n_O. (Apply minus_le_lem2;Auto). Auto.
Apply minus_n_SO. Exact H1. Apply (not_nil bool). (Rewrite -> length_R1;Auto).
Qed.

(****************************************************************)
(* Puis pour le cas ou R1[0]=true (tres tres long !!!!!)
*)

Lemma inv_ind_true :	(t,n:nat)
    ( (le n size)
               ->(BV_to_nat (appbv (stripbv (R1 n) (minus size n)) (R2 n)))
                  =(mult (BV_to_nat V2) (BV_to_nat (truncbv V1 n))) ) ->
	  (le (S n) size)  -> (le n size) ->
   (BV_to_nat
     (appbv
       (stripbv
         (appbv (highs (R1 n))
           (Mux true
             (abit (BV_full_adder_sum (R2 n) V2 false) O)
             (consbv (lowbit (R2 n)) nilbv)))
         (minus size (S n)))
       (appbv
         (highs
           (Mux true
             (BV_full_adder_sum (R2 n) V2 false)
             (R2 n)))
         (Mux true
           (consbv (BV_full_adder_carry (R2 n) V2 false) nilbv)
           (consbv false nilbv)))))
    =(mult (BV_to_nat V2) (BV_to_nat (appbv (truncbv V1 n) (consbv true nilbv)))).

Intros. Simpl. Do 3 Rewrite -> BV_to_nat_app2. Simpl.
Rewrite -> mult_plus_distr2.
Rewrite -> (mult_plus_distr2 (BV_to_nat V2) (BV_to_nat (truncbv V1 n))
             (mult (power2 (lengthbv (truncbv V1 n))) (S O))).
Rewrite <- H. Clear  H. Elim plus_n_O.
Rewrite -> highs_is_strip. Rewrite -> highs_is_strip.
Rewrite -> (length_trunc bool). Rewrite -> (length_strip bool).
 Rewrite -> (length_app bool). Rewrite -> (length_strip bool).
 Rewrite -> (length_strip bool). Rewrite -> length_abit.
 Rewrite -> length_R1. Rewrite -> (plus_sym (minus size (S O))).
 Replace (plus (S O) (minus size (S O))) with size.
Rewrite -> minus_minus_lem2. Rewrite -> length_BV_full_adder_sum.
Rewrite -> length_R2. Rewrite -> (strip_app bool).
 Rewrite -> BV_to_nat_app2. Rewrite -> (strip_strip bool).
 Rewrite -> (length_strip bool). Rewrite -> (length_strip bool).
 Rewrite -> length_R1. Rewrite -> BV_to_nat_app2.
Simpl. Rewrite -> minus_Sn_m.
 Rewrite -> (minus_le_O (minus size (S n)) (minus size (S O))).
Simpl. Rewrite -> minus_minus_lem2.
 Rewrite -> (strip_O bool). Rewrite -> (length_strip bool).
 Rewrite -> length_R1. Rewrite -> minus_minus_lem2.
Replace (plus (power2 n) (power2 n)) with (power2 (S n)).
Replace (mult (power2 n) (S O)) with (power2 n).
 Rewrite <- (mult_assoc_r (power2 (S n))).
Rewrite <- power2_plus. Simpl. Rewrite <- power2_eq2. Rewrite <- power2_eq2.
Replace (S (plus n (minus size (S O)))) with (plus n size).
Repeat Rewrite -> plus_assoc_r.
Replace (plus
          (mult (power2 n)
            (BV_to_nat (abit (BV_full_adder_sum (R2 n) V2 false) O)))
          (plus
            (mult (power2 (S n))
              (BV_to_nat
                (stripbv (BV_full_adder_sum (R2 n) V2 false) (S O))))
            (mult (power2 (plus n size))
              (bool_to_nat (BV_full_adder_carry (R2 n) V2 false))))) with (
plus (mult (power2 n) (BV_to_nat (R2 n)))
 (mult (BV_to_nat V2) (power2 n))).
Try Trivial. Rewrite -> plus_assoc_l. Rewrite <- highs_is_strip.
 Unfold abit . Unfold elemlist . Rewrite -> (strip_O bool).
Rewrite <- lowbit_is_trunc. Rewrite -> BV_nat_lem1. Rewrite -> power2_plus.
 Rewrite -> (mult_assoc_r (power2 n)). Rewrite <- mult_plus_distr2.
Replace size with (lengthbv (BV_full_adder_sum (R2 n) V2 false)).
Replace (bool_to_nat (BV_full_adder_carry (R2 n) V2 false)) with (
BV_to_nat (consbv (BV_full_adder_carry (R2 n) V2 false) nilbv)).
Rewrite <- BV_to_nat_app2.
Replace (appbv (BV_full_adder_sum (R2 n) V2 false)
          (consbv (BV_full_adder_carry (R2 n) V2 false) nilbv)) with (
BV_full_adder (R2 n) V2 false).
Rewrite -> BV_full_adder_ok.
 Rewrite -> mult_plus_distr2. Rewrite -> mult_plus_distr2.
Simpl. Elim mult_n_O. Elim plus_n_O. Rewrite -> (mult_sym (BV_to_nat V2)).
 Try Trivial. Unfold BV_full_adder . Try Trivial. 
Simpl. Elim plus_n_O. Try Trivial. (Rewrite -> length_BV_full_adder_sum;Auto).
(Rewrite -> length_R2;Auto). (Rewrite -> length_BV_full_adder_sum;Auto).
(Rewrite -> length_R2;Auto). (Rewrite -> length_R2;Auto). 
(Apply (not_nil bool);Auto). (Rewrite -> length_BV_full_adder_sum;Auto).
 (Rewrite -> length_R2;Auto). (Rewrite -> length_R2;Auto). 
Replace (S (plus n (minus size (S O)))) with (plus (S n)
                                               (minus size (S O))).
Replace (plus (S n) (minus size (S O))) with (plus n
                                               (S (minus size (S O)))).
Rewrite -> minus_Sn_m. Simpl. Elim minus_n_O. Try Trivial. 
Auto. Rewrite -> plus_sym. Simpl. Rewrite -> plus_sym. Try Trivial.
Simpl. Try Trivial. Rewrite -> mult_sym. Simpl. Elim plus_n_O.
Try Trivial. Auto. Exact H1. Exact H1. (Rewrite -> length_R1;Auto).
 Exact H1. Apply le_minus_minus. Auto. Exact H0. Exact H1.
Rewrite -> length_R1. Auto. Exact H1. (Rewrite -> length_R1;Auto).
Simpl. Rewrite -> minus_Sn_m. Simpl. Auto. Exact H0. Exact H1. 
(Rewrite -> length_R2;Auto). Exact H0. Simpl. Rewrite -> minus_Sn_m.
Auto. Auto. Exact H1. (Rewrite -> length_BV_full_adder_sum;Auto).
Unfold lt . Rewrite -> length_R2. Auto. Exact H1. (Transitivity size;Auto). 
(Rewrite -> length_BV_full_adder_sum;Auto). (Rewrite -> length_R2;Auto).
 (Rewrite -> length_R2;Auto). (Rewrite -> length_R1;Auto). 
Rewrite -> (length_app bool). Rewrite -> (length_strip bool).
(Rewrite -> length_R1;Auto). Rewrite -> length_abit. Rewrite -> plus_sym.
Simpl. Rewrite -> minus_Sn_m. Simpl. Elim minus_n_O. Apply minus_le_lem2. Auto.
(Rewrite -> length_BV_full_adder_sum;Auto). (Rewrite -> length_R2;Auto).
 (Rewrite -> length_R2;Auto). (Rewrite -> length_R1;Auto). 
(Replace (length bool V1) with (lengthbv V1);Auto).
Rewrite -> length_V1_size. Exact H1. Exact H1. 

Qed.

(****************************************************************)
(* Finalement on prouve le cas general de l'invariant:
*)

Lemma Invariant : (t:nat)
	(le t size) -> 
	(BV_to_nat (appbv (stripbv (R1 t) (minus size t)) (R2 t))) =
	(mult (BV_to_nat V2) (BV_to_nat (truncbv V1 t))).

Induction t.Intros.Elim minus_n_O.Apply Invariant_t_O.
Intros.Rewrite <- (app_trunc_elemlist bool).
Replace (elemlist bool V1 n) with (abit V1 n).
Rewrite <- (R1_lem3 n).Rewrite -> R1_eq2.Rewrite -> R2_eq2.
Rewrite <- lowbit_is_abit.Case (lowbit (R1 n)).
Replace (consbv (lowbit (BV_full_adder_sum (R2 n) V2 false)) nilbv) with (
abit (BV_full_adder_sum (R2 n) V2 false) O).
Apply inv_ind_true.Auto.
Intros.Apply H.Exact H1.Exact H0.
(Apply le_trans_S;Exact H0).Unfold abit .Rewrite -> lowbit_is_trunc.
Unfold elemlist .Rewrite -> (strip_O bool).Try Trivial.
Apply (not_nil bool).Rewrite -> length_BV_full_adder_sum.Rewrite -> length_R2.
Exact size_not_O.(Apply le_trans_S;Exact H0).
Rewrite -> length_R2.Rewrite -> length_V2_size.Try Trivial.
(Apply le_trans_S;Exact H0).
Replace (consbv (lowbit (BV_full_adder_sum (R2 n) V2 false)) nilbv) with (
abit (BV_full_adder_sum (R2 n) V2 false) O).
Apply inv_ind_false.Try Trivial.
Exact H.Exact H0.(Apply le_trans_S;Exact H0).Rewrite -> lowbit_is_trunc.
Unfold abit .Unfold elemlist .Rewrite -> (strip_O bool).Try Trivial.
Apply (not_nil bool).
Rewrite -> length_BV_full_adder_sum.(Rewrite -> length_R2;Auto).
Transitivity size.
(Rewrite -> length_R2;Try Trivial).(Apply le_trans_S;Exact H0).Auto.
Apply (not_nil bool).(Rewrite -> length_R1;Auto).Auto.Auto.
Replace (length bool V1) with (lengthbv V1).Rewrite -> length_V1_size.Exact H0.
Auto.

Qed.

(****************************************************************)
(* Et le theoreme final qui est une specialisation de l'invariant
   avec t=size *)

Theorem Correct : (BV_to_nat (appbv (R1 size) (R2 size))) =
                  (mult (BV_to_nat V2) (BV_to_nat V1)).

Intros.Rewrite <- (strip_O bool (R1 size)).
Replace (BV_to_nat V1) with (BV_to_nat (truncbv V1 size)).
Rewrite -> (minus_n_n size).Apply Invariant.Auto.
Rewrite <- length_V1_size.Rewrite -> (trunc_all bool).Try Trivial.
Qed.
