Require Import Ensembles.
Require Import Relations_1.
Require Import Partial_Order.
Require Import Cpo.
Require Import Image.
Require Import Integers.
Section Algebraic_partial_orders.
Variable U:Type.
Variable D:(PO U).

Local D_C := (Carrier_of U D).

Local D_R := (Rel_of U D).

Definition Consistent: (Ensemble U) ->Prop :=
   [A:(Ensemble U)] (Included U A D_C) ->
   (x, y:U) (Included U (Couple U x y) A) ->(Compatible U D x y).

Definition Coherent: Prop :=
   (A:(Ensemble U)) (Included U A D_C) -> (Consistent A) ->
   (ExT [bsup:U] (Lub U D A bsup)).

Inductive Compact[x:U]: Prop :=
   Definition_of_compact:
     (In U D_C x) ->
     ((X:(Ensemble U))
      (Directed U D X) -> (ExT [bsup:U] (Lub U D X bsup) /\ (D_R x bsup)) ->
      (ExT [y:U] (In U X y) /\ (D_R x y))) ->(Compact x).

Inductive Approximant[x, c:U]: Prop :=
   Defn_of_Approximant: (Compact c) -> (D_R c x) ->(Approximant x c).

Inductive Approximants[x:U]: (Ensemble U) :=
   Defn_of_Approximants:
     (c:U) (Compact c) -> (D_R c x) ->(In U (Approximants x) c).

Definition Algebraic: Prop :=
   (x:U)(Directed U D (Approximants x)) /\ (Lub U D (Approximants x) x).

Definition Denumerable: (Ensemble U) ->Prop :=
   [A:(Ensemble U)](ExT [f:U ->Nat] (injective U Nat f)).

Definition Omega_algebraic: Prop := Algebraic /\ (Denumerable [x:U](Compact x)).

Definition Domain: Prop := Coherent /\ Omega_algebraic.
End Algebraic_partial_orders.
Hint Unfold Consistent Coherent.
Hint Definition_of_compact.
Hint Defn_of_Approximant.
Hint Defn_of_Approximants.

