Require Import Ensembles.
Require Import Relations_1.
Require Import Partial_Order.
Require Import Cpo.
Require Import algpodefs.
Require Import Powerset.
Require Import Constructive_sets.
Require Import Finite_sets.
Require Import lpo.
Require Import lpo1.
Section Advanced_lemmas.
Variable U:Type.
Variable D:(PO U).
Hint Lub_definition Upper_Bound_Couple_intro.

Lemma Lub_monotonic:
  (A, B:(Ensemble U)) (Included U A B) -> (Included U B (Carrier_of U D)) ->
  (a, b:U) (Lub U D A a) -> (Lub U D B b) ->(Rel_of U D a b).
Intros A B H' H'0 a b H'1 H'2; Try Assumption.
Elim H'1; Auto.
Intros H'3 H'4; Apply H'4.
Apply Upper_downward_stable with B := B; Auto.
Elim H'2; Auto.
Qed.

Lemma Compatible_lubs:
  (A, B, C:(Ensemble U))
  (Included U A C) -> (Included U B C) -> (Included U C (Carrier_of U D)) ->
  (a, b, c:U) (Lub U D A a) -> (Lub U D B b) -> (Lub U D C c) ->
  (Compatible U D a b).
Intros A B C H' H'0 H'1 a b c H'2 H'3 H'4; Red.
Intros H'5 H'6; Exists c; Split.
Apply Lub_is_in_Carrier with X := C; Auto.
Apply Upper_Bound_Couple_intro; Auto.
Apply Lub_is_in_Carrier with X := C; Auto.
Apply Lub_monotonic with A := A B := C; Auto.
Apply Lub_monotonic with A := B B := C; Auto.
Qed.

Lemma LUaux:
  (A, B:(Ensemble U)) (Included U (Union U A B) (Carrier_of U D)) ->
  (a, b, bsup:U)
  (Lub U D A a) -> (Lub U D B b) -> (Lub U D (Union U A B) bsup) ->
  (Lub U D (Couple U a b) bsup).
Generalize Rel_of_transitive; Intro RoT; Red in RoT.
Intros A B H' a b bsup H'0 H'1 H'2; Try Assumption.
Elim H'0; Elim H'1; Elim H'2.
Intros H'3 H'4 H'5 H'6 H'7 H'8; Try Assumption.
Elim H'3; Elim H'5; Elim H'7.
Intros H'9 H'10 H'11 H'12 H'13 H'14; Try Assumption.
Apply Lub_definition; Auto 8.
Intros y H'15; Apply H'4.
Apply Upper_Bound_definition; Auto.
Elim H'15; Auto.
Elim (Upper_Bound_Couple_inv U D a b y);
 [Intros H'22 H'23; Elim H'23; Intros H'24 H'25; Try Exact H'24; Clear H'23 |
   Idtac]; Auto.
Intros y0 H'16; Elim H'16.
Intros x H'17; Apply RoT with y := a; Auto.
Intros x H'17; Apply RoT with y := b; Auto.
Qed.

Lemma LUaux2:
  (A, B:(Ensemble U)) (Included U (Union U A B) (Carrier_of U D)) ->
  (a, b, bsup:U)
  (Lub U D A a) -> (Lub U D B b) -> (Lub U D (Couple U a b) bsup) ->
  (Lub U D (Union U A B) bsup).
Generalize Rel_of_transitive; Intro RoT; Red in RoT.
Intros A B H' a b bsup H'0 H'1 H'2; Try Assumption.
Elim H'0; Elim H'1; Elim H'2.
Intros H'3 H'4 H'5 H'6 H'7 H'8; Try Assumption.
Elim H'5; Elim H'7.
Elim (Upper_Bound_Couple_inv U D a b bsup);
 [Intros H'15 H'16; Elim H'16; Intros H'17 H'18; Try Exact H'17; Clear H'16 |
   Idtac]; Auto.
Intros H'9 H'10 H'11 H'12; Try Assumption.
Apply Lub_definition; Auto.
Apply Upper_Bound_definition; Auto.
Intros y H'13; Elim H'13.
Intros x H'30; Apply RoT with y := a; Auto.
Intros x H'30; Apply RoT with y := b; Auto.
Intros y H'13; Try Assumption.
Apply H'4.
Elim H'13; Auto 7.
Qed.

Theorem Lub_of_Union:
  (Conditionally_complete U D) ->
  (A, B:(Ensemble U)) (Included U (Union U A B) (Carrier_of U D)) ->
  (x:U) (In U (Carrier_of U D) x) -> (Upper_Bound U D (Union U A B) x) ->
  (a, b:U) (Lub U D A a) -> (Lub U D B b) ->
  (ExT [bsup:U] (Lub U D (Union U A B) bsup) /\ (Lub U D (Couple U a b) bsup)).
Intro H'; Elim H'.
Intros H'0 A B H'1 x H'2 H'3 a b H'4 H'5; Try Assumption.
Elim (H'0 (Union U A B)); [Intros bsup E; Try Exact E | Idtac | Idtac]; Auto.
2:Exists x; Try Assumption.
Exists bsup; Split; [Try Assumption | Idtac].
Apply LUaux with A := A B := B; Auto.
Qed.

Theorem Lub_of_compacts_is_compact:
  (Conditionally_complete U D) ->
  (x, y, z:U)
  (Included U (Couple U x y) (Approximants U D z)) ->
  (In U (Carrier_of U D) z) ->
  (ExT [bsup:U] (Lub U D (Couple U x y) bsup) /\ (Approximant U D z bsup)).
Proof.
Intros H' x y z H'0 K; Red in H'0.
Elim (H'0 x); Auto.
Intros c H'1 H'2.
Elim (H'0 y); Auto.
Intros c' H'3 H'4.
Elim H'.
Intro H'5; Elim (H'5 (Couple U c c')); Auto.
2:Elim H'1; Elim H'3; Exists z; Auto.
Intros bsup H'6; Elim H'6.
Intros H'7 H'8; Exists bsup; Split; Auto.
Apply Defn_of_Approximant; Auto.
Apply Definition_of_compact.
Apply Lub_is_in_Carrier with X := (Couple U c c'); Auto.
Intros X H'9 H'10.
Elim H'1.
Intros H'11 H'12; Try Assumption.
Elim H'3.
Intros H'13 H'14; Try Assumption.
Clear H'1 H'3.
Elim H'10; Intros bsup0 h; Elim h; Clear H'10 h.
Intros H'1 H'3; Try Assumption.
Generalize Rel_of_transitive; Intro RoT; Red in RoT.
Elim (H'14 X); Auto.
Intros x0 H'10; Elim H'10; Intros H'15 H'17; Try Exact H'17; Clear H'10.
Elim (H'12 X); Auto.
Intros x1 H'10; Elim H'10; Intros H'18 H'20; Try Exact H'20; Clear H'10.
Elim H'9.
Intros H'10 H'21 H'22; Elim (H'22 x0 x1); Auto.
Intros y' H'23; Elim H'23; Intros H'24 H'26; Try Exact H'24; Clear H'23.
Exists y'; Split; Auto.
Apply H'8.
Elim (Upper_Bound_Couple_inv U D x0 x1 y');
 [Intros H'29 H'30; Elim H'30; Intros H'31 H'32; Clear H'30 | Idtac]; Auto.
Apply Upper_Bound_Couple_intro; Auto.
Apply RoT with y := x1; Auto.
Apply RoT with y := x0; Auto.
Red.
Intros x2 H'23; Elim H'23; Auto.
Exists bsup0; Split; Auto.
Elim (Upper_Bound_Couple_inv U D c c' bsup);
 [Intros H'22 H'23; Elim H'23; Intros H'24 H'25; Clear H'23 | Idtac]; Auto.
Apply RoT with y := bsup; Auto.
Exists bsup0; Split; Auto.
Elim (Upper_Bound_Couple_inv U D c c' bsup);
 [Intros H'20 H'21; Elim H'21; Intros H'22 H'23; Clear H'21 | Idtac]; Auto.
Apply RoT with y := bsup; Auto.
Qed.

Theorem approximants_closed_by_finite_lubs:
  (Conditionally_complete U D) ->
  (x:U)
  (C:(Ensemble U))
  (In U (Carrier_of U D) x) ->
  (Finite U C) -> (Included U C (Approximants U D x)) ->
  (ExT [bsup:U] (Lub U D C bsup) /\ (Approximant U D x bsup)).
Intros H' x C H H'0; Elim H'0.
Elim (Bottom_is_compact U D);
 [Intros bot E; Elim E; Intros H'4 H'5; Elim H'4; Clear E | Idtac]; Auto.
Exists bot; Split; Auto; Auto.
Apply Lub_definition; Auto.
Intros y H'1; Elim H'1; Auto.
Intros A H'1 H'2 x0 H'3.
Cut (x:U)(Union U A (Singleton U x)) ==(Add U A x); Auto.
Intro H'4; Rewrite <- (H'4 x0).
Clear H'4; Intro H'4.
Elim H'2; [Intros bsup E; Elim E; Intros H'6 H'7; Clear E H'2 | Clear H'2]; Auto.
Red in H'4.
LApply Lub_of_Union;
 [Intro H'2; LApply (H'2 A (Singleton U x0));
   [Intro H'9; LApply (H'9 x);
     [Intro H'11; LApply H'11;
       [Intro H'12; Elim (H'12 bsup x0);
         [Intros bsup0 E; Elim E; Intros H'17 H'18; Try Exact H'17; Clear E H'11 |
           Clear H'11 | Clear H'11] | Clear H'11] | Idtac] | Idtac] | Idtac];
 Auto.
Exists bsup0; Split; [Assumption | Auto].
LApply Lub_of_compacts_is_compact;
 [Intro H'5; Elim (H'5 bsup x0 x);
   [Intros bsup1 E; Elim E; Intros H'15 H'16; Try Exact H'16; Clear E | Idtac |
     Idtac] | Idtac]; Auto.
LApply (Lub_is_unique U D);
 [Intro H'11; Rewrite (H'11 (Couple U bsup x0) bsup0 bsup1) | Idtac]; Auto.
Red.
Intros x1 H'8; Elim H'8; Auto.
Elim H'7; Auto.
Apply Singleton_has_lub.
LApply (H'4 x0); [Intro H'8; Elim H'8 | Idtac]; Auto.
Apply Upper_Bound_definition; Auto.
Intros y H'5; Try Assumption.
LApply (H'4 y); [Intro H'10; Elim H'10 | Idtac]; Auto.
Red.
Intros x1 H'5; Try Assumption.
LApply (H'4 x1); [Intro H'9; Elim H'9 | Idtac]; Auto.
Qed.

Theorem Approximants_directed_set:
  (Conditionally_complete U D) ->
  (x:U) (In U (Carrier_of U D) x) ->(Directed U D (Approximants U D x)).
Proof.
(Intros H' x H'0; Constructor 1); Auto.
Red; Intros x0 H'1; Elim H'1; Auto.
Elim (Bottom_is_compact U D); Auto.
Intros bot H'1; Elim H'1; Intros H'2 H'3; Clear H'1.
Apply Inhabited_intro with x := bot; Auto.
Apply Defn_of_Approximants; Auto.
Elim H'2; Auto.
(Intros x1 x2 H'1; Red in H'1); Auto.
(Generalize (H'1 x1); Intro h; LApply h;
  [Intro H'2; Clear h | Clear h; Try Assumption]); Auto 20.
(Generalize (H'1 x2); Intro h; LApply h;
  [Intro H'3; Clear h | Clear h; Try Assumption]); Auto 20.
Elim H'3.
Intros c H'4 H'5.
Elim H'2.
Intros c' H'6 H'7.
Cut (Compatible U D c' c).
Intro H'8; Try Assumption.
LApply Lub_of_compacts_is_compact; [Intro H'9; Elim (H'9 c' c x) | Idtac]; Auto.
Intros x0 H'10; Elim H'10; Intros H'11 H'12; Elim H'11; Clear H'10.
Intros H'10 H'13; Exists x0; Split; Auto.
Elim H'12; Auto.
Red.
Intros x0 H'10; Elim H'10; Auto.
Red.
Intros H'8 H'9; Exists x; Auto.
Qed.

Theorem Inclusion_of_Approximants:
  (Domain U D) ->
  (x, y:U)
  (In U (Carrier_of U D) x) ->
  (In U (Carrier_of U D) y) ->
  (Included U (Approximants U D x) (Approximants U D y)) ->(Rel_of U D x y).
Proof.
(Intros H' x y H'0 H'1 H'2; Try Assumption); Auto.
Red in H'; Elim H'; Intros H'3 H'4; Clear H'; Red in H'4.
(Elim H'4; Intros H' H'5; Clear H'4; Red in H'); Auto.
(Generalize (H' x); Intro h; Elim h; Intros H'4 H'6; Clear h; Elim H'6); Auto.
Clear H'6; Intros H'7 H'8; Apply H'8.
(Generalize (H' y); Intro h; Elim h; Intros H'6 H'9; Clear h; Try Exact H'9);
 Auto.
Apply Upper_downward_stable with (Approximants U D y); Auto.
Red; (Intros x0 H'10; Elim H'10); Auto.
Red; (Intros x0 H'10; Elim H'10); Auto.
Apply Upper_Bound_definition; Auto.
Intros y0 H'10; Elim H'10; Auto.
Qed.
Hint Inclusion_of_Approximants.

Theorem Less_imp_Inclusion_of_Approximants:
  (Domain U D) ->
  (x, y:U)
  (In U (Carrier_of U D) x) -> (In U (Carrier_of U D) y) -> (Rel_of U D x y) ->
  (Included U (Approximants U D x) (Approximants U D y)).
Proof.
Intros H' x y H'0 H'1 H'2; Red.
Intros x0 H'3; Elim H'3.
Generalize Rel_of_transitive; Intro RoT; Red in RoT.
Intros c H'4 H'5; Try Assumption.
Apply Defn_of_Approximants; Auto.
Apply RoT with y := x; Auto.
Qed.

Theorem Approximants_define_element:
  (Domain U D) ->
  (x, y:U)
  (In U (Carrier_of U D) x) ->
  (In U (Carrier_of U D) y) -> (Approximants U D x) ==(Approximants U D y) ->
  x ==y.
Intros H' x y H'0 H'1 H'2; Try Assumption.
Cut (Same_set U (Approximants U D x) (Approximants U D y)).
Intro H'3; Elim H'3.
Generalize (Rel_of_antisymmetric U D); Auto.
Rewrite H'2; Auto.
Qed.
Hint Approximants_define_element.
Require Import Classical_sets.
Hint Unfold not.

Theorem Corollary1_1:
  (Domain U D) ->
  (x, y:U)
  (Compact U D x) -> (In U (Carrier_of U D) y) -> (Strict_Rel_of U D x y) ->
  (ExT [z:U] (In U (Approximants U D y) z) /\ (Strict_Rel_of U D x z)).
Proof.
Unfold Strict_Rel_of.
Intros H' x y H'0 H'1 H'2; Elim H'2; Intros H'3 H'4; Try Exact H'3; Clear H'2.
LApply Less_imp_Inclusion_of_Approximants;
 [Intro H'2; LApply (H'2 x y);
   [Intro H'7; LApply H'7;
     [Intro H'8; LApply H'8;
       [Intro H'9; Try Exact H'9; Clear H'8 H'7 | Clear H'8 H'7] | Clear H'7] |
     Idtac] | Idtac]; Auto.
Elim (Strict_super_set_contains_new_element
        U (Approximants U D x) (Approximants U D y)); Auto 8.
Intros x0 H'5; Elim H'5; Clear H'5.
Intro H'5; Elim H'5; Clear H'5.
Intros c H'5 H'6 H'7; Try Assumption.
LApply Lub_of_compacts_is_compact; [Intro H'8; Elim (H'8 x c y) | Idtac]; Auto.
Intros x1 H'11; Elim H'11; Intros H'12 H'13; Elim H'12; Clear H'11.
Intros H'11 H'14; Exists x1.
Split; [Idtac | Split; [Try Assumption | Idtac]]; Auto.
Elim H'13; Auto.
Elim H'11; Auto.
Red; Intro H'15; Try Exact H'15.
Apply H'7.
Rewrite H'15; Auto.
Elim (Upper_Bound_Couple_inv U D x c x1);
 [Intros H'21 H'22; Elim H'22; Intros H'23 H'24; Try Exact H'23; Clear H'22 |
   Idtac]; Auto.
Red.
Intros x1 H'11; Elim H'11; Auto.
Apply Coherent_implies_Conditionally_Complete.
Elim H'; Auto.
Qed.

Theorem Corollary1_2:
  (Domain U D) ->
  (x:U) (In U (Carrier_of U D) x) -> ~ (Compact U D x) ->
  ~ (Finite U (Approximants U D x)).
Intros H' x H'0 H'1; Red; Intro H'2; Try Exact H'2.
Apply H'1.
LApply approximants_closed_by_finite_lubs;
 [Intro H'3; Elim (H'3 x (Approximants U D x)) | Idtac]; Auto.
Intros x0 H'4; Elim H'4; Intros H'5 H'6; Try Exact H'5; Clear H'4.
Elim H'.
Intros H'4 H'7; Elim H'7.
Intro H'9; Red in H'9.
Elim (H'9 x); Intros H'11 H'12; Try Exact H'12.
Intro H'10; Clear H'10.
LApply (Lub_is_unique U D);
 [Intro H'14; Rewrite (H'14 (Approximants U D x) x x0) | Idtac]; Auto.
Elim H'6; Auto.
Elim H'; Auto.
Qed.
End Advanced_lemmas.
Hint Inclusion_of_Approximants.
Hint Upper_Bound_Couple_intro.

