Require Import Ensembles.
Require Import Relations_1.
Require Import Partial_Order.
Require Import Cpo.
Require Import algpodefs.
Require Import Powerset.
Section Lemmas_on_partial_orders.
Variable U:Type.
Variable B:(Ensemble U).
Variable D:(PO U).

Theorem Rel_of_reflexive: (x:U)(Rel_of U D x x).
Proof.
Elim D.
Intros Carrier_of Rel_of PO_cond1 PO_cond2; Elim PO_cond2; Auto.
Qed.
Hint Rel_of_reflexive.

Theorem Rel_of_antisymmetric: (Antisymmetric U (Rel_of U D)).
Proof.
Elim D.
Intros Carrier_of Rel_of PO_cond1 PO_cond2; Elim PO_cond2; Auto.
Qed.
Hint Rel_of_antisymmetric.

Theorem Rel_of_transitive: (Transitive U (Rel_of U D)).
Proof.
Elim D.
Intros Carrier_of Rel_of PO_cond1 PO_cond2; Elim PO_cond2; Auto.
Qed.
Hint Rel_of_transitive.

Theorem Couple_included_in_carrier:
  (x, y:U) (In U (Carrier_of U D) x) -> (In U (Carrier_of U D) y) ->
  (Included U (Couple U x y) (Carrier_of U D)).
Proof.
Intros x y H' H'0; Red.
Intros x0 H'1; Elim H'1; Auto.
Qed.
Hint Couple_included_in_carrier.

Theorem Lub_is_in_Carrier:
  (bsup:U)
  (X:(Ensemble U)) (Included U X (Carrier_of U D)) -> (Lub U D X bsup) ->
  (In U (Carrier_of U D) bsup).
Proof.
Intros bsup X H' H'0; Elim H'0.
Intro H'1; Elim H'1; Auto.
Qed.

Theorem Singleton_has_lub:
  (x:U) (In U (Carrier_of U D) x) ->(Lub U D (Singleton U x) x).
Proof.
Intros x H'.
Apply Lub_definition.
Apply Upper_Bound_definition; Auto.
Intros y H'0; Elim H'0; Auto.
Intros y H'0; Elim H'0; Auto.
Qed.
Hint Singleton_has_lub.

Theorem Empty_set_has_Upper_Bound:
  (ExT [maj:U] (Upper_Bound U D (Empty_set U) maj)).
Proof.
Elim D.
Intros C R cond1 cond2.
Elim cond1.
Intros x H'; Exists x.
Apply Upper_Bound_definition; Auto.
Intros y H'0; Elim H'0; Auto.
Qed.
Hint Empty_set_has_Upper_Bound.

Theorem Anyone_is_Upper_Bound_of_Empty_set:
  (x:U) (In U (Carrier_of U D) x) ->(Upper_Bound U D (Empty_set U) x).
Intros x H'; Apply Upper_Bound_definition; Auto.
Intros y H'0; Elim H'0; Auto.
Qed.
Hint Anyone_is_Upper_Bound_of_Empty_set.

Theorem Empty_set_has_lub:
  (C:(Cpo U))(ExT [bsup:U] (Lub U (PO_of_cpo U C) (Empty_set U) bsup)).
Proof.
Intro C; Elim C; Simpl.
Intros PO_of_cpo Cpo_cond; Elim Cpo_cond; Clear Cpo_cond.
Intro H'; Elim H'; Clear H'.
Intros bot is_bot H'0; Exists bot; Simpl.
Elim is_bot; Intros H' H'1.
Apply Lub_definition.
Apply Upper_Bound_definition; Auto.
Intros y H'2; Elim H'2; Auto.
Intros y H'2; Elim H'2; Auto.
Qed.
Hint Empty_set_has_lub.

Theorem Upper_downward_stable:
  (A:(Ensemble U))
  (B:(Ensemble U))
  (maj:U)
  (Included U A (Carrier_of U D)) ->
  (Included U B (Carrier_of U D)) ->
  (Included U A B) -> (Upper_Bound U D B maj) ->(Upper_Bound U D A maj).
Proof.
Intros A B0 maj H' H'0 H'1 H'2; Elim H'2; Auto.
Qed.

Theorem Conditionally_complete_has_a_bottom:
  (Conditionally_complete U D) ->(ExT [bot:U] (Bottom U D bot)).
Proof.
Intro H'; Elim H'.
Intro H'0; Elim (H'0 (Empty_set U)); [Intros bsup E; Elim E | Idtac | Idtac];
 Auto.
Intros H'1 H'2; Exists bsup.
Elim H'1; Auto.
Qed.
Hint Conditionally_complete_has_a_bottom.

Theorem Bottom_is_compact:
  (Conditionally_complete U D) ->
  (ExT [bot:U] (Bottom U D bot) /\ (Compact U D bot)).
Proof.
Intro H'; LApply Conditionally_complete_has_a_bottom;
 [Intro H'0 | Try Assumption].
Elim H'0; Intros bot E; Clear H'0.
Exists bot; Split; Auto.
Apply Definition_of_compact; Auto.
Elim E; Auto.
Intros X H'0; Elim H'0.
Intros H'1 H'2; Elim H'2.
Intros x H'3 H'4 H'5; Try Assumption.
Exists x; Auto.
Elim E; Auto.
Qed.
Hint Bottom_is_compact.

Theorem Compact_is_in_Carrier:
  (x:U) (Compact U D x) ->(In U (Carrier_of U D) x).
Proof.
(Intros x H'; Elim H'); Auto.
Qed.
Hint Compact_is_in_Carrier.

Theorem Compatible_is_reflexive: (x:U)(Compatible U D x x).
Proof.
Intro x; Red.
Intros H'2 H'3; Exists x.
Split; [Assumption | Apply Upper_Bound_definition]; Auto.
Intros y H'; Elim H'; Auto.
Qed.
Hint Compatible_is_reflexive.

Theorem Couple_is_symmetric:
  (x:U) (y:U)<(Ensemble U)> (Couple U x y) ==(Couple U y x).
Proof.
(Intros x y; Apply Extensionality_Ensembles); Red.
(Split; Red); (Intros x0 H'; Elim H'); Auto.
Qed.

Theorem Compatible_is_symmetric:
  (x:U) (y:U) (Compatible U D x y) ->(Compatible U D y x).
Proof.
Unfold Compatible.
Intros x y H' H'0 H'1.
Rewrite <- (Couple_is_symmetric x y); Auto.
Qed.
Immediate Compatible_is_symmetric.

Theorem Compatible_imp_consistent:
  (x:U) (y:U) (Compatible U D x y) ->(Consistent U D (Couple U x y)).
Intros x y H'; Red.
Intros H'0 x0 y0 H'1; Try Assumption.
Red in H'1.
LApply (H'1 x0); [Intro H'3 | Auto].
LApply (H'1 y0); [Intro H'4 | Auto].
Elim H'4.
Elim H'3; Auto.
Elim H'3; Auto.
Qed.

Theorem Consistent_imp_compatible:
  (x:U)
  (y:U)
  (In U (Carrier_of U D) x) ->
  (In U (Carrier_of U D) y) -> (Consistent U D (Couple U x y)) ->
  (Compatible U D x y).
Proof.
(Intros x y H' H'0 H'1; Red in H'1); Auto.
Qed.

Theorem Coherent_implies_Conditionally_Complete:
  (Coherent U D) ->(Conditionally_complete U D).
Proof.
Intro H'; Red in H'.
Apply Definition_of_Conditionally_complete.
Intros X H'0 H'1.
Apply H'; Auto.
Red.
Intros H'2 x y H'3; Red.
Intros H'4 H'5.
Elim H'1; Intros maj E; Try Exact E; Clear H'1.
Exists maj; Elim E; Auto.
Qed.
Hint Coherent_implies_Conditionally_Complete.

Theorem Coherent_has_a_bottom: (Coherent U D) ->(ExT [bot:U] (Bottom U D bot)).
Proof.
Auto.
Qed.
Hint Coherent_has_a_bottom.

Theorem Coherent_implies_Complete: (Coherent U D) ->(Complete U D).
Proof.
Intro H'.
Apply Definition_of_Complete; Auto.
Intros X H'0; Apply H'.
Elim H'0; Auto.
Red.
Intros H'1 x y H'2; Elim H'0.
Intros H'3 H'4 H'5; LApply (H'5 x y); [Intro H'8; Elim H'8 | Auto].
Intros x0 H'6; Red.
Intros H'7 H'9; Exists x0.
Intuition.
Qed.
End Lemmas_on_partial_orders.
Hint Rel_of_reflexive.
Hint Rel_of_antisymmetric.
Hint Rel_of_transitive.
Hint Couple_included_in_carrier.
Hint Singleton_has_lub.
Hint Empty_set_has_Upper_Bound.
Hint Anyone_is_Upper_Bound_of_Empty_set.
Hint Empty_set_has_lub.
Hint Conditionally_complete_has_a_bottom.
Hint Bottom_is_compact.
Hint Compact_is_in_Carrier.
Hint Compatible_is_reflexive.
Immediate Compatible_is_symmetric.
Hint Coherent_implies_Conditionally_Complete.
Hint Coherent_has_a_bottom.


