Require Import Ensembles.
Require Import Relations_1.
Require Import Partial_Order.
Require Import Cpo.
Require Import algpodefs.
Require Import Powerset.
Require Import lpo.
Require Import Constructive_sets.
Require Import Finite_sets.
Section more_lemmas.
Variable U:Type.
Variable B:(Ensemble U).
Variable D:(PO U).

Theorem Upper_Bound_is_in_Carrier:
  (X:(Ensemble U))
  (bound:U) (Included U X (Carrier_of U D)) -> (Upper_Bound U D X bound) ->
  (In U (Carrier_of U D) bound).
Intros X bound H' H'0; Elim H'0; Auto.
Qed.

Theorem Lower_Bound_is_in_Carrier:
  (X:(Ensemble U))
  (bound:U) (Included U X (Carrier_of U D)) -> (Lower_Bound U D X bound) ->
  (In U (Carrier_of U D) bound).
Intros X bound H' H'0; Elim H'0; Auto.
Qed.

Theorem Lub_is_unique:
  (Conditionally_complete U D) ->
  (C:(Ensemble U)) (bsup1, bsup2:U) (Lub U D C bsup1) -> (Lub U D C bsup2) ->
  bsup1 ==bsup2.
Intros H' C bsup1 bsup2 H'0 H'1.
Apply (Rel_of_antisymmetric U D); (Elim H'1; Elim H'0; Auto).
Qed.

Theorem Upper_Bound_Couple_intro:
  (x, y, z:U)
  (In U (Carrier_of U D) x) ->
  (In U (Carrier_of U D) y) ->
  (In U (Carrier_of U D) z) -> (Rel_of U D x z) -> (Rel_of U D y z) ->
  (Upper_Bound U D (Couple U x y) z).
Intros x y z H' H'0 H'1 H'2 H'3.
Apply Upper_Bound_definition; Auto.
Intros y0 H'4; Elim H'4; Auto.
Qed.
Hint Upper_Bound_Couple_intro.

Theorem Upper_Bound_Couple_inv:
  (x, y, z:U) (Upper_Bound U D (Couple U x y) z) ->
  (Rel_of U D x z) /\ ((Rel_of U D y z) /\ (In U (Carrier_of U D) z)).
Intros x y z H'; Elim H'; Auto.
Qed.

Theorem Upper_Bound_Singleton_intro:
  (x, z:U) (In U (Carrier_of U D) x) -> (In U (Carrier_of U D) z) -> x ==z ->
  (Upper_Bound U D (Singleton U x) z).
Intros x z H' H'0 H'1; Rewrite <- H'1.
Apply Upper_Bound_definition; Auto.
Intros y H'2; Elim H'2; Auto.
Qed.
Hint Upper_Bound_Singleton_intro.

Theorem Upper_Bound_Singleton_inv:
  (x, z:U) (Upper_Bound U D (Singleton U x) z) ->(Rel_of U D x z).
Intros x z H'; Elim H'; Auto.
Qed.
Hint Upper_Bound_Singleton_inv.

Theorem Non_empty_has_glb:
  (Conditionally_complete U D) ->
  (X:(Ensemble U)) (Included U X (Carrier_of U D)) -> (Inhabited U X) ->
  (ExT [binf:U] (Glb U D X binf)).
Proof.
Intro H'; Elim H'; Clear H'.
Intros H'0 X H'1 H'2.
Generalize (Lower_Bound_is_in_Carrier X); Intro L.
Elim (H'0 [z:U](Lower_Bound U D X z)); Auto.
Intros bsup H'3; Elim H'3; Intros H'4 H'6.
Exists bsup; Auto.
Apply Glb_definition; Auto.
Apply Lower_Bound_definition; Auto.
Apply Lub_is_in_Carrier with X := [z:U](Lower_Bound U D X z); Auto.
Intros y H'7; Apply H'6.
Apply Upper_Bound_definition; Auto.
Intros y0 H'8; Elim H'8; Auto.
Intros y H'7; Elim H'7; Auto.
Intros H'8 H'9; Elim H'4; Auto.
Elim H'2; Intros x0 H'3; Exists x0.
Apply Upper_Bound_definition; Auto.
Intros y H'4; Elim H'4; Auto.
Qed.

Theorem Bounded_implies_consistent:
  (X:(Ensemble U))
  (Included U X (Carrier_of U D)) -> ((exT U [maj:U](Upper_Bound U D X maj))) ->
  (Consistent U D X).
Intros X H' H'0; Elim H'0; Intros maj E; Try Exact E; Clear H'0.
Red.
Intros H'0 x y H'1; Red.
Intros H'2 H'3; Exists maj; Split.
Elim E; Auto.
Elim E; Auto.
Qed.

Theorem Consistent_downward_stable:
  (B:(Ensemble U)) (Included U B (Carrier_of U D)) -> (Consistent U D B) ->
  (A:(Ensemble U)) (Included U A B) ->(Consistent U D A).
Proof.
Auto 6.
Qed.

Lemma Consistent_triple_intro:
  (x, y, z:U)
  (Compatible U D x y) -> (Compatible U D y z) -> (Compatible U D x z) ->
  (Consistent U D (Triple U x y z)).
Intros x y z H' H'0 H'1; Red.
Intros H'2 x0 y0 H'3; Try Assumption.
Red in H'3.
LApply (H'3 x0); [Intro H'5 | Try Assumption]; Auto.
LApply (H'3 y0); [Intro H'6 | Idtac]; Auto.
Clear H'3.
Elim H'6; Elim H'5; Auto.
Qed.
End more_lemmas.
Hint Upper_Bound_Couple_intro.
Hint Upper_Bound_Singleton_intro.
Hint Upper_Bound_Singleton_inv.
Hint Bounded_implies_consistent.


