Require Import Ensembles.
Require Import Relations_1.
Require Import Partial_Order.
Require Import Cpo.
Require Import algpodefs.
Require Import Powerset.
Require Import Constructive_sets.
Require Import Finite_sets.
Require Import lpo.
Require Import lpo1.
Require Import alpo.
Require Import Finite_sets_facts.
Section First_inductive_lemma.
Variable U:Type.
Variable C:(Cpo U).

Local D: (PO U) := (PO_of_cpo U C).

Theorem Pairs_are_enough_finite_case:
  ((x:U) (y:U) (Compatible U D x y) ->
   (exT U [bsup:U](Lub U D (Couple U x y) bsup))) ->
  (X:(Ensemble U))
  (Finite U X) ->
  (Included U X (Carrier_of U D)) -> ((exT U [maj:U](Upper_Bound U D X maj))) ->
  (exT U [bsup:U](Lub U D X bsup)).
Proof.
Intros H' X H'0; Elim H'0; Auto.
Unfold 3 D; Auto.
Unfold Add.
Intros A H'1 H'2 x H'3 H'4 H'5.
Elim H'5; Intros maj E; Try Exact E; Clear H'5.
Elim H'2; [Intros bsupA E0; Elim E0; Clear H'2 | Clear H'2 | Clear H'2]; Auto.
2:Exists maj; Apply Upper_downward_stable with B := (Union U A (Singleton U x));
   Auto.
Intros H'2 H'6.
Elim (H' bsupA x); [Intros bsup E1; Elim E1 | Idtac]; Auto.
2:Red.
2:Intros H'7 H'8; Exists maj; Elim E; Auto 7.
Intros H'5 H'7; Try Assumption.
Exists bsup; Try Assumption.
Apply LUaux2 with a := bsupA b := x; Auto.
Qed.

Inductive Lubs_of_finite_parts[X:(Ensemble U)]: (Ensemble U) :=
   Lubs_of_finite_parts_def:
     (bsup:U)
     (Y:(Ensemble U)) (Included U Y X) -> (Finite U Y) -> (Lub U D Y bsup) ->
     (In U (Lubs_of_finite_parts X) bsup).

Theorem LFP_directed:
  (X:(Ensemble U))
  ((Y:(Ensemble U)) (Included U Y X) -> (Finite U Y) ->
   (exT ? [bsup:U](Lub U D Y bsup))) ->
  (Included U X (Carrier_of U D)) -> (Consistent U D X) ->
  (Directed U D (Lubs_of_finite_parts X)).
Intros X K H' L; Apply Definition_of_Directed.
Red; Intros x H'0; Elim H'0; Auto.
Intros bsup Y H'1 H'2 H'3; Apply Lub_is_in_Carrier with X := Y; Auto.
Elim (Empty_set_has_lub U C); Intros bsup E.
Apply Inhabited_intro with x := bsup.
Apply Lubs_of_finite_parts_def with Y := (Empty_set U); Auto.
Intros x1 x2 H'4; Red in H'4.
LApply (H'4 x1); [Intro H'1; Elim H'1 | Idtac]; Auto.
Intros bsup Y H'0 H'2 H'3; Try Assumption.
LApply (H'4 x2); [Intro H'6; Elim H'6 | Idtac]; Auto.
Intros bsup0 Y0 H'5 H'7 H'8; Try Assumption.
LApply (Union_preserves_Finite U Y Y0);
 [Intro H'12; LApply H'12; [Intro H'13; Clear H'12 | Clear H'12] | Idtac]; Auto.
Elim (K (Union U Y Y0)); [Intros bsup1 E | Idtac | Idtac]; Auto.
Exists bsup1; Split.
Apply Lubs_of_finite_parts_def with Y := (Union U Y Y0); Auto.
Apply Upper_Bound_Couple_intro.
Apply Lub_is_in_Carrier with X := Y; Auto.
Apply Lub_is_in_Carrier with X := Y0; Auto.
Apply Lub_is_in_Carrier with X := (Union U Y Y0); Auto.
Elim H'3.
Intros H'10 H'11; Apply H'11.
Elim E; Intros H'12 H'14; Apply Upper_downward_stable with B := (Union U Y Y0);
 Auto.
Elim H'8.
Intros H'10 H'11; Apply H'11.
Elim E; Intros H'12 H'14; Apply Upper_downward_stable with B := (Union U Y Y0);
 Auto.
Qed.

Theorem Lub_is_LFP:
  (X:(Ensemble U)) (Included U X (Carrier_of U D)) ->
  (bsup:U) (Lub U D (Lubs_of_finite_parts X) bsup) ->(Lub U D X bsup).
Intros X K bsup E0; Elim E0.
Intros H'4 H'5; Apply Lub_definition; Auto.
Apply Upper_Bound_definition; Auto.
Apply Lub_is_in_Carrier with X := (Lubs_of_finite_parts X); Auto.
Red; Intros x H'8; Elim H'8.
Intros bsup0 Y H'9 H'10 H'11; Apply Lub_is_in_Carrier with Y; Auto.
Intros y H'8; Elim H'4.
Intros H'9 H'10; Apply H'10.
Apply Lubs_of_finite_parts_def with Y := (Singleton U y); Auto.
Red.
Intros x H'11; Elim H'11; Auto.
Apply Singleton_is_finite.
Intros y H'8; Apply H'5.
Apply Upper_Bound_definition.
Elim H'8; Auto.
Intros y0 H'9; Elim H'9.
Intros bsup1 Y H'10 H'11 H'12; Elim H'12.
Intros H'13 H'14; Apply H'14.
Apply Upper_downward_stable with B := X; Auto.
Qed.
Hint Lub_is_LFP.
Hint Pairs_are_enough_finite_case.

Theorem Pairs_are_enough:
  ((x:U) (y:U) (Compatible U D x y) ->
   (exT U [bsup:U](Lub U D (Couple U x y) bsup))) ->(Conditionally_complete U D).
Proof.
Intro H'; Apply Definition_of_Conditionally_complete.
Intros X H'1 H'2.
Cut (Complete U D).
2:Exact (Cpo_cond U C).
Intro H'3; Elim H'3; Intros H'6 H'7.
Elim (H'7 (Lubs_of_finite_parts X)); [Intros bsup0 E0 | Apply LFP_directed];
 Auto.
Exists bsup0; Auto.
Auto.
Intros Y H'4 H'5; Apply Pairs_are_enough_finite_case; Auto.
Elim H'2; Intros maj E; Try Exact E; Clear H'2.
Exists maj.
Apply Upper_downward_stable with B := X; Auto.
Qed.
End First_inductive_lemma.
Hint Lub_is_LFP.


