(****************************************************************************)
(*                                                                          *)
(*                          Constructive Geometry                           *)
(*                             according to                                 *)
(*                            Jan von Plato                                 *)
(*                                                                          *)
(*                               Coq V5.10                                  *)
(*									    *)
(*			        Gilles Kahn 				    *)
(*				   INRIA                                    *)
(*		              Sophia-Antipolis		 	            *)
(*									    *)
(*                                                                          *)
(* Acknowledgments: This work started in Turin in May 95 with Yves Bertot.  *)
(* Thanks to the Newton Institute for providing an exceptional work         *)
(* environment in Summer 1995 so that I could complete this.                *)
(****************************************************************************)

(****************************************************************************)
(*                             affinity.v                                   *)
(* little lemmas at the end to handle strict parallels                      *)
(****************************************************************************)

Require Import basis.
Require Import part1.
Require Import part2.
Require Import part3.

Axiom constructed_parallel :
   (l: Line) (a: Point) {l': Line | (Par l' l) /\ (Incident a l')}.

Definition par: Line -> Point -> Line.
Proof.
Intros l a; Elim (constructed_parallel l a).
Intros x H; Exact x.
Defined.

Axiom constructive_uniqueness_for_parallels :
   (l, m: Line) (a: Point) (DiLn l m) ->
    ((Apart a l) \/ (Apart a m)) \/ (ConLn l m).

Theorem Ax1_i : (l: Line) (a: Point) (Par (par l a) l).
Proof.
Intros l a.
Unfold 1 par.
Elim (constructed_parallel l a).
Simpl; Tauto.
Qed.

Theorem Ax1_ii : (l: Line) (a: Point) (Incident a (par l a)).
Proof.
Intros l a.
Unfold 1 par.
Elim (constructed_parallel l a).
Simpl; Tauto.
Qed.
Hint Ax1_i Ax1_ii.

Theorem thm7_1 :
  (l, m: Line) (a: Point) (Incident a l) /\ (Par l m) -> (EqLn l (par m a)).
Proof.
Intros l m a H'; Elim H'; Intros H'0 H'1; Try Exact H'1; Clear H'.
Generalize (Ax1_i m a); Intro H'2.
Elim equiv_Par.
Intros H'3 H'4 H'5.
Red in H'5.
Specialize 3 H'5 with x := (par m a) y := m z := l; Intro H'8; LApply H'8;
 [Intro H'9; LApply H'9; [Intro H'10; Clear H'9 H'8 | Clear H'9 H'8] | Clear H'8];
 Auto.
Red; Red; Red; Intro H'6.
LApply (constructive_uniqueness_for_parallels l (par m a) a);
 [Intro H'11 | Assumption].
Elim H'11; [Intro H'; Elim H'; [Idtac | Intro H'7; Clear H' H'11] | Idtac]; Auto.
Elim (Ax1_ii m a); Auto.
Qed.
Hint thm7_1.

Theorem thm7_3 :
  (l, m: Line) (a: Point) (Incident a l) -> (Incident a m) -> (Par l m) ->
   (EqLn l m).
Proof.
Elim equiv_EqLn.
Intros H' H'0 H'1 l m a H'2 H'3 H'4.
Unfold 1 Transitive in H'1.
Apply H'1 with y := (par m a); Auto.
Apply H'0.
Elim equiv_Par; Auto.
Qed.

Theorem thm7_4 :
  (l, m, n: Line)
  (a: Point)
  (Apart a l) -> (Incident a m) -> (Incident a n) -> (Par n l) -> (Par m l) ->
   (EqLn m n).
Proof.
Intros l m n a H' H'0 H'1 H'2 H'3; Try Assumption.
Apply thm7_3 with a := a; Auto.
Elim equiv_Par.
Intros H'4 H'5 H'6; Red in H'6.
Apply H'6 with y := l; Auto.
Qed.

Theorem DiLn_qimp_con :
  (l, m: Line)
  (DiLn l m) -> ((ex Point [b: Point] (Incident b l) /\ (Incident b m))) ->
   (ConLn l m).
Proof.
Unfold Incident Negation.
Intros l m H' H'0; Elim H'0; Intros b E; Elim E; Intros H'1 H'2; Clear E H'0.
LApply (constructive_uniqueness_for_parallels l m b); [Intro H'5 | Trivial].
Tauto.
Qed.

Theorem strict_parallel : (a: Point) (l: Line) (Apart a l) -> (DiLn l (par l a)).
Proof.
Intros a l H'.
LApply (cmp_apt_diln a l (par l a)); [Intro H'3 | Trivial].
Elim H'3; [Trivial | Intro H'0; Clear H'3].
Elim (Ax1_ii l a); Auto.
Qed.
Hint strict_parallel.

Theorem spar : (a: Point) (l: Line) (Apart a l) -> (SPar l (par l a)).
Proof.
Intros a l H'; Red; Auto.
Qed.
Hint spar.



