(****************************************************************************)
(*                                                                          *)
(*                          Constructive Geometry                           *)
(*                             according to                                 *)
(*                            Jan von Plato                                 *)
(*                                                                          *)
(*                               Coq V5.10                                  *)
(*									    *)
(*			        Gilles Kahn 				    *)
(*				   INRIA                                    *)
(*		              Sophia-Antipolis		 	            *)
(*									    *)
(*                                                                          *)
(* Acknowledgments: This work started in Turin in May 95 with Yves Bertot.  *)
(* Thanks to the Newton Institute for providing an exceptional work         *)
(* environment in Summer 1995 so that I could complete this.                *)
(****************************************************************************)


(****************************************************************************)
(*                               part1.v                                    *)
(*                                                                          *)
(* Congruence lemmas near the end slightly more explicit than in von Plato  *)
(****************************************************************************)
Require Import basis.

Theorem Uniqueness_of_constructed_lines :
  (x: Segment)
  (l: Line) (Incident (origin x) l) -> (Incident (extremity x) l) ->
   (EqLn l (ln x)).
Proof.
Intros x l.
Generalize (inc_ln2 x); Generalize (inc_ln1 x).
Unfold Incident EqLn Negation.
Intros H' H'0 H'1 H'2; Red; Intro H'3.
LApply (el_ax x l (ln x)); Trivial.
Tauto.
Qed.

Theorem Convergent_imp_distinct : (l, m: Line) (ConLn l m) -> (DiLn l m).
Proof.
Intros l m H'.
LApply (cmp_con_diln l m l); Trivial.
Intro H'0; Elim H'0; Auto.
Intro H'1; Elim apart_con.
Intro H'2; Elim (H'2 l); Trivial.
Qed.
Hint Convergent_imp_distinct.

Theorem Uniqueness_of_constructed_points :
  (x: Twolines) (a: Point) (Incident a (line1 x)) -> (Incident a (line2 x)) ->
   (EqPt a (pt x)).
Proof.
Intro x.
Generalize (inc_pt2 x); Generalize (inc_pt1 x).
Unfold Incident EqPt Negation.
Intros H' H'0 a H'1 H'2; Red; Intro H'3.
LApply (el_ax (Seg a (pt x) H'3) (line1 x) (line2 x)); Simpl.
Tauto.
Elim x; Auto.
Qed.

Theorem cong_eqpt_apt :
  (a, b: Point) (l: Line) (Apart a l) -> (EqPt a b) -> (Apart b l).
Proof.
Intros a b l H' H'0.
Elim (cmp_apt_dipt a b l); Auto.
Intro H'1; Elim H'0; Trivial.
Qed.

Theorem cong_eqln_apt :
  (a: Point) (l, m: Line) (Apart a l) -> (EqLn l m) -> (Apart a m).
Proof.
Intros a l m H' H'0.
Elim (cmp_apt_diln a l m); Auto.
Intro H'1; Elim H'0; Trivial.
Qed.

Theorem cong_eqpt_inc :
  (a, b: Point) (l: Line) (Incident a l) -> (EqPt a b) -> (Incident b l).
Proof.
Unfold Incident.
Intros a b l H' H'0; Red; Intro H'1; Apply H'.
Apply cong_eqpt_apt with a := b; Auto.
Qed.

Theorem cong_eqln_inc :
  (a: Point) (l, m: Line) (Incident a l) -> (EqLn l m) -> (Incident a m).
Proof.
Unfold Incident.
Intros a l m H' H'0; Red; Intro H'1; Apply H'.
Apply cong_eqln_apt with l := m; Auto.
Qed.

Theorem cong_eqln_con : (l, m, n: Line) (ConLn l m) -> (EqLn m n) -> (ConLn l n).
Proof.
Intros l m n H' H'0.
Elim (cmp_con_diln l m n); Auto.
Intro H'1; Elim H'0; Trivial.
Qed.

Theorem cong_eqln_par : (l, m, n: Line) (Par l m) -> (EqLn m n) -> (Par l n).
Proof.
Unfold Par Negation.
Intros l m n H' H'0; Red; Intro H'1; Apply H'.
Apply cong_eqln_con with m := n; Auto.
Qed.

Theorem cong_eqpt_dipt : (a, b, c: Point) (DiPt a b) -> (EqPt b c) -> (DiPt a c).
Proof.
Intros a b c H' H'0; Elim apart_dipt.
Unfold 1 Separating.
Intros H'1 H'2; Elim (H'2 a b c); Trivial.
Intro H'3; Elim H'0; Trivial.
Qed.

Theorem cong_eqln_diln : (l, m, n: Line) (DiLn l m) -> (EqLn m n) -> (DiLn l n).
Proof.
Intros l m n H' H'0; Elim apart_diln.
Unfold 1 Separating.
Intros H'1 H'2; Elim (H'2 l m n); Trivial.
Intro H'3; Elim H'0; Trivial.
Qed.

Theorem eqln_imp_par : (l, m: Line) (EqLn l m) -> (Par l m).
Proof.
Unfold Par EqLn Negation; Red; Auto.
Qed.

Theorem cong_par_con : (l, m, n: Line) (ConLn l m) -> (Par m n) -> (ConLn l n).
Proof.
Intros l m n H' H'0.
Elim apart_con.
Unfold 1 Separating.
Intros H'1 H'2; Elim (H'2 l m n); Trivial.
Intro H'3; Elim H'0; Trivial.
Qed.

Theorem sym_SPar : (x, y: Line) (SPar x y) -> (SPar y x).
Proof.
Unfold SPar.
Intuition.
Qed.
Hint sym_SPar.

Theorem cong_eqln_spar : (l, m, n: Line) (SPar l m) -> (EqLn m n) -> (SPar l n).
Proof.
Unfold SPar.
Intros l m n H'; Elim H'; Intros H'0 H'1; Try Exact H'0; Clear H'.
Intro H'; Split.
Apply cong_eqln_par with m := m; Trivial.
Apply cong_eqln_diln with m := m; Trivial.
Qed.

Definition reverse : Segment -> Segment.
Proof.
Intro H'; Elim H'.
Intros a b H'0.
Apply (Seg b a); Auto.
Defined.

Theorem orig_rev : (x: Segment) (origin x) = (extremity (reverse x)).
Proof.
(Intro x; Elim x; Simpl); Auto.
Qed.

Theorem ext_rev : (x: Segment) (extremity x) = (origin (reverse x)).
Proof.
(Intro x; Elim x; Simpl); Auto.
Qed.

Theorem rev_defines_sameln : (x: Segment) (EqLn (ln x) (ln (reverse x))).
Proof.
Intro x; Apply Uniqueness_of_constructed_lines.
Rewrite <- (ext_rev x); Auto.
Rewrite <- (orig_rev x); Auto.
Qed.
Hint rev_defines_sameln.

Definition flip : Twolines -> Twolines.
Proof.
Intro H'; Elim H'.
Intros l m H'0.
Apply (Twol m l); Auto.
Defined.

Theorem line1_flip : (x: Twolines) (line1 x) = (line2 (flip x)).
Proof.
(Intro x; Elim x; Simpl); Auto.
Qed.

Theorem line2_flip : (x: Twolines) (line2 x) = (line1 (flip x)).
Proof.
(Intro x; Elim x; Simpl); Auto.
Qed.

Theorem flip_defines_samept : (x: Twolines) (EqPt (pt x) (pt (flip x))).
Proof.
Intro x; Apply Uniqueness_of_constructed_points.
Rewrite <- (line2_flip x); Auto.
Rewrite <- (line1_flip x); Auto.
Qed.
Hint rev_defines_sameln flip_defines_samept.

Definition colinear : Segment -> Segment -> Prop :=
   [x, y: Segment] (EqLn (ln x) (ln y)).

Theorem Colinearity_is_equivalence : (Equivalence Segment colinear).
Proof.
Cut (Equivalence Line EqLn); Auto.
Intro H'; Elim H'.
Intros H'0 H'1 H'2; Apply Definition_of_equivalence; Unfold colinear; Auto.
Red.
Intros x y z H'3 H'4.
Red in H'2.
Apply H'2 with y := (ln y); Auto.
Qed.



