(****************************************************************************)
(*                                                                          *)
(*                          Constructive Geometry                           *)
(*                             according to                                 *)
(*                            Jan von Plato                                 *)
(*                                                                          *)
(*                               Coq V5.10                                  *)
(*									    *)
(*			        Gilles Kahn 				    *)
(*				   INRIA                                    *)
(*		              Sophia-Antipolis		 	            *)
(*									    *)
(*                                                                          *)
(* Acknowledgments: This work started in Turin in May 95 with Yves Bertot.  *)
(* Thanks to the Newton Institute for providing an exceptional work         *)
(* environment in Summer 1995 so that I could complete this.                *)
(****************************************************************************)

(****************************************************************************)
(*                                part2.v                                   *)
(****************************************************************************)
Require Import basis.
Require Import part1.

Theorem thm4_1a :
  (x: Segment) (l: Line) (DiLn l (ln x)) ->
   (Apart (origin x) l) \/ (Apart (extremity x) l).
Proof.
Intros x l.
Generalize (inc_ln2 x); Generalize (inc_ln1 x).
Unfold Incident.
Generalize (el_ax x l (ln x)).
Tauto.
Qed.

Theorem thm4_1b :
  (x: Segment) (l: Line) (Apart (origin x) l) \/ (Apart (extremity x) l) ->
   (DiLn l (ln x)).
Proof.
Intros x l.
Generalize (inc_ln2 x); Generalize (inc_ln1 x).
Unfold Incident.
Intros H' H'0 H'1; Elim H'1; Intro H'2; Clear H'1.
Elim (cmp_apt_diln (origin x) l (ln x)); Tauto.
Elim (cmp_apt_diln (extremity x) l (ln x)); Tauto.
Qed.
Hint thm4_1a thm4_1b.

Theorem thm4_1c :
  (x: Twolines) (a: Point) (DiPt a (pt x)) ->
   (Apart a (line1 x)) \/ (Apart a (line2 x)).
Proof.
Intros x a.
Generalize (inc_pt2 x); Generalize (inc_pt1 x).
Unfold Incident.
Intros H' H'0 H'1.
Generalize (el_ax (Seg a (pt x) H'1) (line1 x) (line2 x)); Simpl.
Cut (DiLn (line1 x) (line2 x)).
Tauto.
Elim x; Auto.
Qed.

Theorem thm4_1d :
  (x: Twolines) (a: Point) (Apart a (line1 x)) \/ (Apart a (line2 x)) ->
   (DiPt a (pt x)).
Proof.
Intros x a.
Generalize (inc_pt2 x); Generalize (inc_pt1 x).
Unfold Incident.
Intros H' H'0 H'1; Elim H'1; Intro H'2; Clear H'1.
Generalize (cmp_apt_dipt a (pt x) (line1 x)); Tauto.
Generalize (cmp_apt_dipt a (pt x) (line2 x)); Tauto.
Qed.

Theorem Symmetry_of_Apart :
  (x, y: Segment) (Apart (origin x) (ln y)) \/ (Apart (extremity x) (ln y)) ->
   (Apart (origin y) (ln x)) \/ (Apart (extremity y) (ln x)).
Intros x y H'.
Apply thm4_1a.
Apply sym_DiLn; Auto.
Qed.

Theorem thm4_3a :
  (x: Segment) (c: Point) (Apart c (ln x)) -> (DiPt c (origin x)).
Proof.
Intros x c H'.
Elim (cmp_apt_dipt c (origin x) (ln x)); Trivial.
Intro H0; Elim (inc_ln1 x); Trivial.
Qed.

Theorem thm4_3b :
  (x: Segment) (c: Point) (Apart c (ln x)) -> (DiPt c (extremity x)).
Proof.
Intros x c H'.
Elim (cmp_apt_dipt c (extremity x) (ln x)); Trivial.
Intro H0; Elim (inc_ln2 x); Trivial.
Qed.

Definition Side1 : Triangle -> Segment.
Proof.
Intro H'; Elim H'; Clear H'.
Intros summit base H'.
Apply (Seg summit (origin base)).
Apply thm4_3a; Trivial.
Defined.

Definition Side2 : Triangle -> Segment.
Proof.
Intro H'; Elim H'; Clear H'.
Intros summit base H'.
Apply (Seg summit (extremity base)).
Apply thm4_3b; Trivial.
Defined.

Theorem auxs1 : (t: Triangle) (origin (base t)) = (extremity (Side1 t)).
Proof.
Intro t; Elim t; Auto.
Qed.

Theorem auxs2 : (t: Triangle) (extremity (base t)) = (extremity (Side2 t)).
Proof.
Intro t; Elim t; Auto.
Qed.

Theorem auxs3 : (t: Triangle) (summit t) = (origin (Side1 t)).
Proof.
Intro t; Elim t; Auto.
Qed.

Theorem auxs4 : (t: Triangle) (summit t) = (origin (Side2 t)).
Proof.
Intro t; Elim t; Auto.
Qed.

Theorem thm4_3c : (t: Triangle) (DiLn (ln (base t)) (ln (Side1 t))).
Proof.
Intro H'; Elim H'; Clear H'.
Intros summit base Tri_cond.
Elim (cmp_apt_diln summit (ln base) (ln (Side1 (Tri summit base Tri_cond))));
 Auto.
Qed.

Theorem thm4_3d : (t: Triangle) (DiLn (ln (base t)) (ln (Side2 t))).
Proof.
Intro H'; Elim H'; Clear H'.
Intros summit base Tri_cond.
Elim (cmp_apt_diln summit (ln base) (ln (Side2 (Tri summit base Tri_cond))));
 Auto.
Qed.

Hint thm4_3c thm4_3d.



