(****************************************************************************)
(*                                                                          *)
(*                          Constructive Geometry                           *)
(*                             according to                                 *)
(*                            Jan von Plato                                 *)
(*                                                                          *)
(*                               Coq V5.10                                  *)
(*									    *)
(*			        Gilles Kahn 				    *)
(*				   INRIA                                    *)
(*		              Sophia-Antipolis		 	            *)
(*									    *)
(*                                                                          *)
(* Acknowledgments: This work started in Turin in May 95 with Yves Bertot.  *)
(* Thanks to the Newton Institute for providing an exceptional work         *)
(* environment in Summer 1995 so that I could complete this.                *)
(****************************************************************************)

(****************************************************************************)
(*                               part3.v                                    *)
(* I don't follow exactly the original in the proofs of the Triangle axioms *)
(****************************************************************************)
Require Import basis.
Require Import part1.
Require Import part2.

Theorem Triangle_axioms_i :
  (t: Triangle) (Apart (origin (base t)) (ln (Side2 t))).
Proof.
Intro t.
Generalize (thm4_3d t); Intro H'.
LApply (el_ax (base t) (ln (base t)) (ln (Side2 t))); Trivial.
Intro H.
Elim H; Intro H'0; Clear H.
Generalize (inc_ln2 (base t)); Generalize (inc_ln1 (base t)).
Unfold Incident; Tauto.
Generalize H'0; Clear H'0.
Rewrite (auxs2 t).
Intro H'2.
Elim H'2; [Trivial | Intro H'0; Clear H'2].
Elim (inc_ln2 (Side2 t)); Trivial.
Qed.

Theorem Triangle_axioms_ii :
  (t: Triangle) (Apart (extremity (base t)) (ln (reverse (Side1 t)))).
Proof.
Intro t.
Generalize (thm4_3c t); Intro H'.
LApply (el_ax (base t) (ln (base t)) (ln (reverse (Side1 t)))); Trivial.
Intro H; Elim H; (Clear H; Intro H'0).
Generalize (inc_ln2 (base t)); Generalize (inc_ln1 (base t)).
Unfold Incident; Tauto.
Elim H'0; [Intro H'1; Clear H'0 | Trivial].
Generalize (inc_ln1 (reverse (Side1 t))).
Rewrite <- (ext_rev (Side1 t)).
Rewrite <- (auxs1 t).
Intro H'3; Elim H'3; Auto.
Apply cong_eqln_diln with m := (ln (Side1 t)); Auto.
Qed.

Theorem Triangle_axioms_iii :
  (t: Triangle) (Apart (summit t) (ln (reverse (base t)))).
Proof.
Intro t; Elim t.
Intros summit base Tri_cond.
Apply cong_eqln_apt with l := (ln base); Auto.
Qed.
Hint Triangle_axioms_i Triangle_axioms_ii Triangle_axioms_iii.

Theorem thm4_5ia :
  (x: Segment) (l: Line) (EqLn l (ln x)) ->
   (Incident (origin x) l) /\ (Incident (extremity x) l).
Proof.
Unfold EqLn Incident Negation.
Intuition.
Qed.

Theorem thm4_5ib :
  (x: Segment)
  (l: Line) (Incident (origin x) l) /\ (Incident (extremity x) l) ->
   (EqLn l (ln x)).
Proof.
Intuition.
Intros H' H'0; Apply Uniqueness_of_constructed_lines; Trivial.
Qed.
Hint thm4_1c thm4_1d.

Theorem thm4_5iia :
  (x: Twolines) (a: Point) (EqPt a (pt x)) ->
   (Incident a (line1 x)) /\ (Incident a (line2 x)).
Proof.
Split; Apply cong_eqpt_inc with a := (pt x); Auto.
Qed.

Theorem thm4_5iib :
  (x: Twolines) (a: Point) (Incident a (line1 x)) /\ (Incident a (line2 x)) ->
   (EqPt a (pt x)).
Proof.
Intuition.
Intros H' H'0; Apply Uniqueness_of_constructed_points; Trivial.
Qed.

Theorem thm4_6 :
  (x, y: Segment)
  (Incident (origin x) (ln y)) /\ (Incident (extremity x) (ln y)) ->
   (Incident (origin y) (ln x)) /\ (Incident (extremity y) (ln x)).
Proof.
Unfold Incident.
Intros x y H'; Generalize (Symmetry_of_Apart y x).
Tauto.
Qed.

Theorem thm4_7i :
  (a, b, c: Point)
  (H1: (DiPt a b)) (H2: (DiPt b c)) (Incident c (ln (Seg a b H1))) ->
   (Incident a (ln (reverse (Seg b c H2)))).
Proof.
Intros a b c H1 H2 H'.
LApply (thm4_6 (reverse (Seg b c H2)) (Seg a b H1));
 [Intro H'1; Elim H'1; Trivial | Idtac].
Generalize (inc_ln2 (Seg a b H1)); Auto.
Qed.

Theorem thm4_7ii :
  (a, b, c: Point)
  (H1: (DiPt a b)) (H2: (DiPt a c)) (Incident c (ln (Seg a b H1))) ->
   (Incident b (ln (Seg a c H2))).
Proof.
Intros a b c H1 H2 H'.
LApply (thm4_6 (Seg a c H2) (Seg a b H1));
 [Intro H'1; Elim H'1; Trivial | Idtac].
Generalize (inc_ln1 (Seg a b H1)); Auto.
Qed.

Theorem thm4_7iii:
  (x: Segment) (c: Point) (Incident c (ln x)) -> (Incident c (ln (reverse x))).
Proof.
Intros x c H'.
Apply cong_eqln_inc with l := (ln x); Auto.
Qed.

Theorem Symmetry_of_Apart':
  (x, y: Twolines) (Apart (pt y) (line1 x)) \/ (Apart (pt y) (line2 x)) ->
   (Apart (pt x) (line1 y)) \/ (Apart (pt x) (line2 y)).
Proof.
Intros x y H'.
Apply thm4_1c.
Apply sym_DiPt; Auto.
Qed.

Theorem thm4_9a: (x: Twolines) (c: Line) (Apart (pt x) c) -> (DiLn c (line1 x)).
Proof.
Intros x c H'.
LApply (cmp_apt_diln (pt x) c (line1 x));
 [Intro H'3; Elim H'3; [Trivial | Intro H'4; Clear H'3] | Idtac]; Trivial.
Elim (inc_pt1 x); Trivial.
Qed.

Theorem thm4_9b: (x: Twolines) (c: Line) (Apart (pt x) c) -> (DiLn c (line2 x)).
Proof.
Intros x c H'.
LApply (cmp_apt_diln (pt x) c (line2 x));
 [Intro H'3; Elim H'3; [Trivial | Intro H'4; Clear H'3] | Idtac]; Trivial.
Elim (inc_pt2 x); Trivial.
Qed.

Theorem thm5_3 :
  (x, y: Segment)
  (z: Twolines)
  (origin x) = (origin y) -> (line1 z) = (ln x) -> (line2 z) = (ln y) ->
   (EqPt (pt z) (origin x)).
Proof.
Intros x y z; Elim z; Simpl.
Intros line1 line2 Twol_cond H' H'0 H'1.
LApply (Convergent_imp_distinct line1 line2); Trivial.
Intro H'2; Red; Red; Red; Intro H.
LApply (el_ax (Seg (pt (Twol line1 line2 Twol_cond)) (origin x) H) line1 line2);
 [Intro H'7 | Trivial].
Simpl in H'7.
Elim H'7; Clear H'7; (Intro H'3; Elim H'3; (Clear H'3; Intro H'4)).
Elim (inc_pt1 (Twol line1 line2 Twol_cond)); Auto.
Apply (inc_ln1 x); Rewrite <- H'0; Assumption.
Elim (inc_pt2 (Twol line1 line2 Twol_cond)); Auto.
Apply (inc_ln1 y); Rewrite <- H'1; Rewrite <- H'; Assumption.
Qed.

Theorem thm5_4 :
  (x, y: Twolines)
  (z: Segment)
  (line1 x) = (line1 y) -> (origin z) = (pt x) -> (extremity z) = (pt y) ->
   (EqLn (ln z) (line1 x)).
Proof.
Intros x y z; Elim z; Simpl.
Intros origin extremity Seg_cond H' H'0 H'1.
Red; Red; Red; Intro H'2.
LApply (el_ax
          (Seg origin extremity Seg_cond)
          (ln (Seg origin extremity Seg_cond))
          (line1 x)); [Intro H'7 | Trivial].
Simpl in H'7.
Elim H'7; Clear H'7; (Intro H'3; Elim H'3; (Clear H'3; Intro H'4)).
Elim (inc_ln1 (Seg origin extremity Seg_cond)); Auto.
Elim (inc_ln2 (Seg origin extremity Seg_cond)); Auto.
Apply (inc_pt1 x); Rewrite <- H'0; Assumption.
Apply (inc_pt1 y); Rewrite <- H'1; Rewrite <- H'; Assumption.
Qed.

Theorem thm5_5 :
  (a, b, c: Point) (H1: (DiPt a b)) (H2: (EqPt b c))
   (EqLn (ln (Seg a b H1)) (ln (Seg a c (cong_eqpt_dipt a b c H1 H2)))).
Proof.
Intros a b c H1 H2; Apply Uniqueness_of_constructed_lines; Simpl.
Exact (inc_ln1 (Seg a b H1)).
Generalize (inc_ln2 (Seg a b H1)); Simpl; Intro H'.
Apply cong_eqpt_inc with a := b; Trivial.
Qed.

Theorem thm5_6 :
  (l, m, n: Line) (H1: (ConLn l m)) (H2: (EqLn m n))
   (EqPt (pt (Twol l m H1)) (pt (Twol l n (cong_eqln_con l m n H1 H2)))).
Proof.
Intros l m n H1 H2; Apply Uniqueness_of_constructed_points; Simpl.
Exact (inc_pt1 (Twol l m H1)).
Generalize (inc_pt2 (Twol l m H1)); Simpl; Intro H'.
Apply cong_eqln_inc with l := m; Trivial.
Qed.



