(****************************************************************************)
(*                                                                          *)
(*                          Constructive Geometry                           *)
(*                             according to                                 *)
(*                            Jan von Plato                                 *)
(*                                                                          *)
(*                               Coq V5.10                                  *)
(*									    *)
(*			        Gilles Kahn 				    *)
(*				   INRIA                                    *)
(*		              Sophia-Antipolis		 	            *)
(*									    *)
(*                                                                          *)
(* Acknowledgments: This work started in Turin in May 95 with Yves Bertot.  *)
(* Thanks to the Newton Institute for providing an exceptional work         *)
(* environment in Summer 1995 so that I could complete this.                *)
(****************************************************************************)


(****************************************************************************)
(*      			problems.v                                  *)
(****************************************************************************)
Require Import basis.
Require Import part1.
Require Import part2.
Require Import part3.
Require Import affinity.
Require Import orthogonality.

Theorem pb9_1 : (a: Point) (l: Line) (ex Point [b: Point] (Incident b l)).
Proof.
Intros a l.
Generalize (O3_i l a); Intro H'.
Elim (O1 (ort l a) l); Intro H'2.
Exists (pt (Twol (ort l a) l H'2)).
Exact (inc_pt2 (Twol (ort l a) l H'2)).
Elim H'; Auto.
Qed.

Section construction9_2.
Variable t: Triangle.

Local C : Point :=  (summit t).

Local A : Point :=  (origin (base t)).

Local B : Point :=  (extremity (base t)).

Local Base : Line :=  (ln (base t)).

Local L1 : Line :=  (ln (Side1 t)).

Local L2 : Line :=  (ln (Side2 t)).

Local L3 : Line :=  (par Base C).

Local L4 : Line :=  (par L1 B).

Local lemma1 : (ConLn Base L1).
Proof.
Unfold Base L1.
Apply DiLn_qimp_con; Auto.
Exists (origin (base t)); Split; Auto.
Rewrite (auxs1 t); Auto.
Qed.
Hint lemma1.

Local lemma2 : (DiLn Base L3).
Proof.
Unfold L3 Base C; Auto.
Qed.
Hint lemma2.

Local lemma5' : (Apart B L1).
Proof.
Unfold B L1.
Apply cong_eqln_apt with l := (ln (reverse (Side1 t))); Auto.
Qed.
Hint lemma5'.

Local lemma2' : (DiLn L1 L4).
Proof.
Unfold L1 L4; Auto.
Qed.
Hint lemma2'.

Local lemma3 : (ConLn L1 L3).
Proof.
Unfold 1 L3.
Apply cong_par_con with m := Base; Auto.
Qed.
Hint lemma3.

Local lemma4 : (ConLn L3 L4).
Proof.
Unfold 1 L4.
Apply cong_par_con with m := L1; Auto.
Qed.
Hint lemma4.

Local D : Point :=  (pt (Twol L3 L4 lemma4)).

Local lemma5 : (Apart B L3).
Proof.
LApply (constructive_uniqueness_for_parallels Base L3 B);
 [Intro H'2; Elim H'2;
   [Intro H'3; Elim H'3; [Intro H'4; Clear H'3 H'2 | Trivial] |
     Intro H'3; Clear H'2] | Idtac]; Auto.
Unfold B Base in H'4.
Elim (inc_ln2 (base t)); Auto.
Unfold L3 in H'3.
Elim (Ax1_i Base C); Auto.
Qed.
Hint lemma5.

Local lemma6 : (Apart C L4).
Proof.
LApply (constructive_uniqueness_for_parallels L1 L4 C);
 [Intro H'4; Elim H'4;
   [Intro H'5; Elim H'5; [Intro H'6; Clear H'5 H'4 | Trivial] |
     Intro H'5; Clear H'4] | Idtac]; Auto.
Unfold C L1 in H'6.
Generalize H'6; Rewrite (auxs3 t); Intro H'.
Elim (inc_ln1 (Side1 t)); Auto.
Elim (Ax1_i L1 B); Auto.
Qed.
Hint lemma6.

Local lemma7: (DiPt C D).
Proof.
Unfold D; Auto.
Qed.

Local lemma8: (DiPt B D).
Proof.
Unfold D; Auto.
Qed.
Hint lemma7 lemma8.

Local S1 : Segment :=  (base t).

Local S3 : Segment :=  (reverse (Side1 t)).

Local S2 : Segment :=  (Seg C D lemma7).

Local S4 : Segment :=  (Seg B D lemma8).

Local lemma9: (EqLn L1 (ln (reverse (Side1 t)))).
Proof.
Unfold 1 L1; Auto.
Qed.
Hint lemma9.

Local lemma10: (EqLn L3 (ln S2)).
Proof.
Apply Uniqueness_of_constructed_lines.
Unfold S2 L1 L3; Simpl; Auto.
Unfold L3 S2 D; Simpl.
Exact (inc_pt1 (Twol L3 L4 lemma4)).
Qed.
Hint lemma10.

Local lemma11: (EqLn L4 (ln S4)).
Proof.
Apply Uniqueness_of_constructed_lines.
Unfold S4 L4; Simpl; Auto.
Unfold S4 D; Simpl.
Exact (inc_pt2 (Twol L3 L4 lemma4)).
Qed.
Hint lemma11.

Theorem thm9_2: Parallelogram.
Proof.
Apply (Pgram S1 S2 S3 S4).
Unfold S1 S2 S3.
Rewrite <- (ext_rev (Side1 t)).
Rewrite <- (auxs1 t).
Rewrite <- (orig_rev (Side1 t)).
Simpl.
Rewrite <- (auxs3 t).
Unfold C; Auto.
Unfold S1 S2 S4 B; Simpl; Auto.
Apply cong_eqln_spar with m := L3; Auto.
Unfold L3 S1 Base C; Auto.
Apply cong_eqln_spar with m := L4; Auto.
Unfold L4.
Apply sym_SPar.
Apply cong_eqln_spar with m := L1; Auto.
Qed.

End construction9_2.



