(****************************************************************************)
(*                                                                          *)
(*                           Group Theory in Coq                            *)
(*                                                                          *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*									    *)
(*			         Gilles Kahn 				    *)
(*									    *)
(*                                  INRIA                                   *)
(*                             Sophia-Antipolis                             *)
(*									    *)
(*				 January 1996				    *)
(*                                                                          *)
(****************************************************************************)

Require Import Ensembles.
Require Import Laws.
Require Import Group_definitions.
Require Import gr.
Require Import g1.

Theorem auxsub:
  (H:(Group U)) (x:U) (subgroup U H Gr) ->(In U (G_ U H) x) ->(In U G x).
Proof.
Intros H x H'; Elim H'; Auto.
Qed.

Section Trois.

Variables H, K: (Group U).
Variable subH: (subgroup U H Gr).
Variable subK: (subgroup U K Gr).

Inductive Prod: (Ensemble U) :=
   Definition_of_Prod:
     (x, y, z:U) (In U (G_ U H) x) ->(In U (G_ U K) y) ->(star x y) ==z ->
     (In U Prod z).
End Trois.

Section Quatre.
Variables H, K: (Group U).
Variable subH: (subgroup U H Gr).
Variable subK: (subgroup U K Gr).

Theorem T4: (Same_set U (Prod H K) (Prod K H)) ->(Setsubgroup U (Prod H K) Gr).
Proof.
Generalize (auxsub H); Intro tH; Generalize (auxsub K); Intro tK.
Intro H'.
Apply T_1_6_3 with witness := e; Auto.
Rewrite <- (G2c' e); Auto.
Apply Definition_of_Prod with x := e y := e; Auto.
Rewrite <- (eh_is_e H); Auto.
Rewrite <- (eh_is_e K); Auto.
Red.
Intros x H'0; Elim H'0.
Intros x0 y z H'1 H'2 H'3; Rewrite <- H'3; Auto.
Intros a b H'0 H'1.
Generalize H'1; Clear H'1.
Elim H'0.
Intros x y z H'1 H'2 H'3; Rewrite <- H'3.
Intro H'4; Elim H'4.
Intros x0 y0 z0 H'5 H'6 H'7; Rewrite <- H'7.
Rewrite <- (inv_star' x0 y0); Auto.
Rewrite <- (G1' x y (star (inv y0) (inv x0))); Auto.
Red in H'.
Elim H'; Intros H'8 H'9; Red in H'9; Clear H'.
Rewrite (G1' y (inv y0) (inv x0)); Auto.
LApply (H'9 (star (star y (inv y0)) (inv x0))); [Intro H'10; Elim H'10 | Idtac].
Intros x1 y1 z1 H' H'11 H'12; Rewrite <- H'12.
Rewrite (G1' x x1 y1); Auto.
Apply Definition_of_Prod with x := (star x x1) y := y1; Auto.
Rewrite <- (starH_is_star H); Auto.
Apply Definition_of_Prod with x := (star y (inv y0)) y := (inv x0).
Rewrite <- (starH_is_star K); Auto.
Rewrite <- (invH_is_inv K); Auto.
Rewrite <- (invH_is_inv H); Auto.
Trivial.
Qed.

Theorem T4R: (Setsubgroup U (Prod H K) Gr) ->(Included U (Prod H K) (Prod K H)).
Proof.
Generalize (auxsub H); Intro tH; Generalize (auxsub K); Intro tK.
Intro H'; Elim H'.
Intros x H'0; Red.
Intros x0 H'1.
Elim H'0; Intros L1 L2; Clear H'0.
Generalize (auxsub x); Intro tx.
Cut (ExT [t:U] (In U (Prod H K) t) /\ t == (inv x0)).
Intro H'2; Elim H'2.
Intros x1 H'3; Elim H'3; Intros H'4 H'5; Try Exact H'4; Clear H'3.
Generalize H'5.
Elim H'4.
Intros x2 y z H'3 H'6 H'7; Rewrite <- H'7.
Intro H'8.
Rewrite (inv_involution' x0); Auto.
Rewrite <- H'8.
Rewrite <- (inv_star' x2 y); Auto.
Apply Definition_of_Prod with x := (inv y) y := (inv x2); Auto.
Rewrite <- (invH_is_inv K subK y); Auto.
Rewrite <- (invH_is_inv H subH x2); Auto.
Generalize H'1.
Rewrite <- L2; Auto.
Intro H'0.
Apply exT_intro with x := (inv x0).
Split; [Idtac | Trivial].
Rewrite <- (invH_is_inv x L1 x0); Auto.
Qed.

Theorem T4R1:
  (Setsubgroup U (Prod H K) Gr) ->(Included U (Prod K H) (Prod H K)).
Proof.
Generalize (auxsub H); Intro tH; Generalize (auxsub K); Intro tK.
Intro H'; Elim H'.
Intros x H'0; Red.
Intros x0 H'1.
Elim H'0; Intros L1 L2; Clear H'0.
Generalize (auxsub x); Intro tx.
Cut (ExT [t:U] (In U (Prod K H) t) /\ t == (inv (inv x0))).
Intro H'0; Elim H'0.
Intros x1 H'2.
Elim H'2; Intros H'3 H'4; Generalize H'4; Clear H'2.
Elim H'3.
Intros x2 y z H'2 H'5 H'6; Rewrite <- H'6.
Intro H'7.
Rewrite (inv_involution' x0).
Rewrite <- H'7.
Rewrite (inv_involution' (star x2 y)).
Rewrite <- (inv_star' x2 y).
Rewrite <- (invH_is_inv x L1 (star (inv y) (inv x2))).
Rewrite <- L2; Auto.
Elim L1; Simpl; Auto.
Intros H'15 H'16; Apply (G3a_ U x); Auto.
Rewrite L2.
Apply Definition_of_Prod with x := (inv y) y := (inv x2); Auto.
Rewrite <- (invH_is_inv H subH y); Auto.
Rewrite <- (invH_is_inv K subK x2); Auto.
Rewrite L2.
Apply Definition_of_Prod with x := (inv y) y := (inv x2); Auto.
Rewrite <- (invH_is_inv H subH y); Auto.
Rewrite <- (invH_is_inv K subK x2); Auto.
Elim H'1; Intros x3 y0 z0 H'8 H'9 H'10; Rewrite <- H'10.
Apply exT_intro with x := (star x3 y0).
Split; [Idtac | Auto].
Apply Definition_of_Prod with x := x3 y := y0; Trivial.
Qed.
Hint T4 T4R T4R1.

Theorem T_1_6_8:
  (iff (Same_set U (Prod H K) (Prod K H)) (Setsubgroup U (Prod H K) Gr)).
Proof.
Red; Auto.
Qed.
End Quatre.

