(****************************************************************************)
(*                                                                          *)
(*                           Group Theory in Coq                            *)
(*                                                                          *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*									    *)
(*			         Gilles Kahn 				    *)
(*									    *)
(*                                  INRIA                                   *)
(*                             Sophia-Antipolis                             *)
(*									    *)
(*				 January 1996				    *)
(*                                                                          *)
(****************************************************************************)

Require Import Ensembles.
Require Import Laws.
Require Import Group_definitions.
Section group_trivialities.
Variable U: Type.
Variable Gr: (Group U).

Local G : (Ensemble U) :=  (G_ U Gr).

Local star : U -> U -> U :=  (star_ U Gr).

Local inv : U -> U :=  (inv_ U Gr).

Local e : U :=  (e_ U Gr).

Definition G0 : (a, b: U) (In U G a) -> (In U G b) -> (In U G (star a b)) :=
   (G0_ U Gr).

Definition G1 : (a, b, c: U) (star a (star b c)) == (star (star a b) c) :=
   (G1_ U Gr).

Definition G2a : (In U G e) :=  (G2a_ U Gr).

Definition G2b : (a: U) (star e a) == a :=  (G2b_ U Gr).

Definition G2c : (a: U) (star a e) == a :=  (G2c_ U Gr).

Definition G3a : (a: U) (In U G a) -> (In U G (inv a)) :=  (G3a_ U Gr).

Definition G3b : (a: U) (star a (inv a)) == e :=  (G3b_ U Gr).

Definition G3c : (a: U) (star (inv a) a) == e :=  (G3c_ U Gr).
Hint G1.
Hint G2a G2b G2c.
Hint G3a G3b G3c.
Hint G0.

Theorem triv1: (a, b: U) (star (inv a) (star a b)) == b.
Intros a b; Try Assumption.
Rewrite (G1 (inv a) a b); Auto.
Rewrite G3c; Auto.
Qed.

Theorem triv2: (a, b: U) (star (star b a) (inv a)) == b.
Intros a b; Try Assumption.
Rewrite <- (G1 b a (inv a)); Auto.
Rewrite (G3b a); Auto.
Qed.

Theorem resolve: (a, b: U) (star b a) == e -> b == (inv a).
Intros a b H'1.
Cut (star (star b a) (inv a)) == (inv a).
Rewrite <- (G1 b a (inv a)); Auto.
Rewrite (G3b a); Auto.
Rewrite (G2c b); Auto.
Rewrite H'1.
Rewrite (G2b (inv a)); Auto.
Qed.

Theorem self_inv: e == (inv e).
Apply resolve; Auto.
Qed.

Theorem inv_star: (a, b: U) (star (inv b) (inv a)) == (inv (star a b)).
Intros a b.
Apply resolve.
Rewrite <- (G1 (inv b) (inv a) (star a b)).
Rewrite (G1 (inv a) a b).
Rewrite (G3c a).
Rewrite (G2b b); Auto.
Qed.

Theorem cancellation: (a, b: U) (star a b) == a -> b == e.
Intros a b H'.
Cut (star (inv a) (star a b)) == b.
Rewrite H'.
Rewrite (G3c a); Auto.
Rewrite (G1 (inv a) a b).
Rewrite (G3c a); Auto.
Qed.

Theorem inv_involution: (a: U) a == (inv (inv a)).
Intro a; Apply resolve; Auto.
Qed.
End group_trivialities.
Hint G1.
Hint G2a G2b G2c.
Hint G3a G3b G3c.
Hint G0.
Hint triv1 triv2 resolve self_inv inv_star inv_involution.


