Require Import Arith.
Require Import Nat_complements.
Require Import Zbase.
Require Import Z_succ_pred.

Recursive Definition addZ : Z -> Z -> Z :=
      OZ   y => y
    | (pos O)   y => (succZ y)
    | (pos (S n1))   y => (succZ (addZ (pos n1) y))
    | (neg O)   y => (predZ y)
    | (neg (S n1))   y => (predZ (addZ (neg n1) y)).

Lemma succ_addZ_l: (x, y: Z) (addZ (succZ x) y) = (succZ (addZ x y)).
Intro x; Elim x; Auto.
Intro n; Elim n; Auto.
Simpl.
Intro y.
Specialize 1 succ_predZ with x := y; Intro H'; Rewrite H'; Auto.
Intros; Symmetry; Rewrite addZ_eq5.
Apply succ_predZ.
Qed.

Lemma pred_addZ_l: (x, y: Z) (addZ (predZ x) y) = (predZ (addZ x y)).
Intros; Elim x.
Reflexivity.
Destruct n.
Simpl; Rewrite pred_succZ; Trivial.
Intros; Rewrite addZ_eq3; Rewrite pred_succZ; Trivial.
Trivial.
Qed.

Lemma tech_add_pos_succZ:
  (x: nat) (y: Z) (addZ (pos (S x)) y) = (succZ (addZ (pos x) y)).
Proof addZ_eq3.

Lemma tech_add_neg_predZ:
  (x: nat) (y: Z) (addZ (neg (S x)) y) = (predZ (addZ (neg x) y)).
Proof addZ_eq5.

Lemma succ_addZ_r: (x, y: Z) (addZ x (succZ y)) = (succZ (addZ x y)).
Intros; Elim x.
Reflexivity.
Induction n.
Reflexivity.
Intros.
Do 2 Rewrite (tech_add_pos_succZ n0).
Elim H; Reflexivity.
Induction n.
Simpl; Symmetry; Apply succ_pred_pred_succZ.
Intros.
Do 2 Rewrite (tech_add_neg_predZ n0).
Rewrite H.
Symmetry; Apply succ_pred_pred_succZ.
Qed.

Lemma pred_addZ_r: (x, y: Z) (addZ x (predZ y)) = (predZ (addZ x y)).
Intros; Elim x.
Reflexivity.
Induction n.
Simpl; Apply succ_pred_pred_succZ.
Intros.
Do 2 Rewrite (tech_add_pos_succZ n0).
Rewrite H; Apply succ_pred_pred_succZ.
Induction n.
Reflexivity.
Intros.
Do 2 Rewrite (tech_add_neg_predZ n0).
Elim H; Reflexivity.
Qed.

Lemma add_OZ: (x: Z) (addZ x OZ) = x.
Induction x.
Reflexivity.
Induction n.
Reflexivity.
Intros; Rewrite tech_add_pos_succZ; Rewrite H; Reflexivity.
Induction n.
Reflexivity.
Intros; Rewrite tech_add_neg_predZ; Rewrite H; Reflexivity.
Qed.

Lemma add_IZ_succZ: (x: Z) (addZ x IZ) = (succZ x).
Intros.
Cut (succZ OZ) = IZ; Intros.
Elim H.
Rewrite (succ_addZ_r x OZ); Rewrite (add_OZ x); Reflexivity.
Reflexivity.
Qed.

Lemma add_mIZ_predZ: (x: Z) (addZ x (neg O)) = (predZ x).
Intros.
Cut (predZ OZ) = (neg O); Intros.
Elim H.
Rewrite (pred_addZ_r x OZ); Rewrite (add_OZ x); Reflexivity.
Reflexivity.
Qed.

Definition commutative :=
   [U: Set] [op: U -> U -> U] (x, y: U) (op x y) = (op y x).

Theorem addZ_commutativity: (commutative Z addZ).
Unfold commutative; Intros; Elim x.
Simpl; Symmetry; Exact (add_OZ y).
Induction n.
Simpl; Symmetry; Exact (add_IZ_succZ y).
Intros; Rewrite (tech_add_pos_succZ n0 y).
Rewrite H.
Cut (succZ (pos n0)) = (pos (S n0)); Intros.
Elim H0.
Rewrite (succ_addZ_r y (pos n0)); Reflexivity.
Reflexivity.
Induction n.
Simpl; Symmetry; Exact (add_mIZ_predZ y).
Intros; Rewrite (tech_add_neg_predZ n0 y).
Rewrite H.
Cut (predZ (neg n0)) = (neg (S n0)); Intros.
Elim H0.
Rewrite (pred_addZ_r y (neg n0)); Reflexivity.
Reflexivity.
Qed.

Lemma tech_add_pos_neg_posZ:
  (n1, n2: nat) (lt n2 n1) -> (addZ (pos n1) (neg n2)) = (pos (minus n1 (S n2))).
Induction n2.
Intros; Elim (addZ_commutativity (neg O) (pos n1)).
Rewrite addZ_eq4.
Elim minus_n_Sm; Trivial.
Elim minus_n_O.
Apply tech_pred_posZ; Trivial.
Intros; Elim (addZ_commutativity (neg (S n)) (pos n1)).
Rewrite tech_add_neg_predZ.
Elim (addZ_commutativity (pos n1) (neg n)).
Rewrite H; Auto.
Elim (minus_n_Sm n1 (S n) H0).
Apply tech_pred_posZ.
Apply lt_minus2; Trivial.
Qed.

Definition associative :=
   [U: Set] [op: U -> U -> U] (x, y, z: U) (op x (op y z)) = (op (op x y) z).

Theorem addZ_associativity: (associative Z addZ).
Unfold associative; Intros; Elim x.
Unfold addZ; Reflexivity.
Intros; Elim n.
Simpl; Symmetry; Exact (succ_addZ_l y z).
Intros.
Do 2 Rewrite (tech_add_pos_succZ n0).
Rewrite (succ_addZ_l (addZ (pos n0) y) z); Elim H; Reflexivity.
Induction n.
Simpl; Symmetry; Exact (pred_addZ_l y z).
Intros.
Do 2 Rewrite (tech_add_neg_predZ n0).
Rewrite (pred_addZ_l (addZ (neg n0) y) z); Elim H; Reflexivity.
Qed.

Definition IdZ :=  [x: Z] True.

Definition neutral :=
   [S: Set] [G: S -> Prop] [Add: S -> S -> S] [O: S]
    (G O) /\ ((x: S) (G x) -> (Add x O) = x /\ (Add O x) = x).

Theorem addZ_neutral: (neutral Z IdZ addZ OZ).
Unfold neutral; Intros.
Split.
Exact I.
Intros.
Split.
Exact (add_OZ x).
Unfold addZ; Reflexivity.
Qed.

Definition oppZ :=
   [x: Z] <Z>Case x of OZ [n: nat] (neg n) [n: nat] (pos n) end.

Lemma opp_succZ: (x: Z) (oppZ (succZ x)) = (predZ (oppZ x)).
Destruct x.
Reflexivity.
Intros; Reflexivity.
Destruct n; Intros; Reflexivity.
Qed.

Lemma opp_predZ: (x: Z) (oppZ (predZ x)) = (succZ (oppZ x)).
Destruct x.
Reflexivity.
Destruct n; Intros; Reflexivity.
Intros; Reflexivity.
Qed.

Lemma tech_add_pos_negZ: (n: nat) (addZ (pos n) (neg n)) = OZ.
Induction n.
Reflexivity.
Intros; Rewrite (tech_add_pos_succZ n0).
Elim succ_addZ_r; Exact H.
Qed.

Lemma tech_add_neg_posZ: (n: nat) (addZ (neg n) (pos n)) = OZ.
Intros; Elim (addZ_commutativity (pos n) (neg n)); Exact (tech_add_pos_negZ n).
Qed.

Lemma tech_add_pos_posZ:
  (n: nat) (m: nat) (addZ (pos n) (pos m)) = (pos (S (plus n m))).
Intros; Elim n.
Reflexivity.
Intros; Rewrite (tech_add_pos_succZ n0 (pos m)); Rewrite H; Reflexivity.
Qed.

Lemma tech_add_neg_negZ:
  (n: nat) (m: nat) (addZ (neg n) (neg m)) = (neg (S (plus n m))).
Induction n.
Reflexivity.
Intros; Rewrite (tech_add_neg_predZ n0 (neg m)); Rewrite H; Reflexivity.
Qed.

Theorem abs_eq_or_oppZ: (x: Z) {(absZ x) = x}+{(absZ x) = (oppZ x)}.
Destruct x; Auto.
Qed.

Lemma tech_opp_pos_negZ:
  (n: nat) (oppZ (pos n)) = (neg n) /\ (oppZ (neg n)) = (pos n).
Induction n; Auto.
Qed.


