Require Import Arith.
Require Import Zbase.
Require Import Z_succ_pred.
Require Import Zadd.

Definition leZ :=
   [x, y: Z]
    <Prop>Case x of
      <Prop>Case y of True [n: nat] True [n: nat] False end
      [n: nat] <Prop>Case y of False [m: nat] (le n m) [m: nat] False end
      [n: nat] <Prop>Case y of True [m: nat] True [m: nat] (le m n) end end.

Lemma sign_absZ: (x: Z) (leZ OZ (absZ x)).
Intros; Elim x; Simpl.
Exact I.
Intro; Simpl.
Exact I.
Intro; Simpl.
Exact I.
Qed.

Lemma tech_le_pos_abs: (x: Z) (leZ OZ x) -> (absZ x) = x.
Intro x; Elim x.
Unfold absZ; Reflexivity.
Unfold absZ; Reflexivity.
Intros; Elim H.
Qed.

Theorem leZ_antisymmetric: (x, y: Z) (leZ x y) -> (leZ y x) -> x = y.
Intros x y; Elim x.
Elim y.
Reflexivity.
Intros; Elim H0.
Intros; Elim H.
Intro n; Elim y.
Intros; Elim H.
Simpl; Intros; Elim (le_antisym n n0 H H0); Reflexivity.
Intros; Elim H.
Intro n; Elim y.
Intros; Elim H0.
Intros; Elim H0.
Simpl; Intros; Elim (le_antisym n0 n H H0); Reflexivity.
Qed.

Definition ltZ :=  [x, y: Z] (leZ (succZ x) y).

Definition lt_absZ :=  [x, y: Z] (ltZ (absZ x) (absZ y)).

Lemma tech_lt_abs_OZ: (x: Z) (lt_absZ x (pos O)) -> x = OZ.
Induction x.
Reflexivity.
Unfold lt_absZ; Unfold absZ; Unfold ltZ; Unfold leZ; Intros.
Elim (le_Sn_O n H).
Unfold lt_absZ; Unfold absZ; Unfold ltZ; Unfold leZ; Intros.
Elim (le_Sn_O n H).
Qed.

Lemma tech_posOZ_pos: (n: nat) (leZ OZ (posOZ n)).
Intros; Elim n.
Simpl; Exact I.
Simpl; Intros; Exact I.
Qed.

Lemma le_opp_OZ_l: (x: Z) (leZ OZ x) -> (leZ (oppZ x) OZ).
Intro x; Elim x.
Simpl; Intros; Exact I.
Simpl; Intros; Exact I.
Intros; Elim H.
Qed.

Lemma le_opp_OZ: (x, y: Z) x = (oppZ y) -> (leZ OZ x) -> (leZ OZ y) -> x = OZ.
Intros.
Apply (leZ_antisymmetric x OZ).
Rewrite H.
Exact (le_opp_OZ_l y H1).
Exact H0.
Qed.

Local opp_inv: (x, y: Z) x = (oppZ y) -> y = (oppZ x).
Intros x y; Elim y; Simpl.
Intro H'; Rewrite H'; Auto.
Intros n H'; Rewrite H'; Auto.
Intros n H'; Rewrite H'; Auto.
Qed.

Lemma le_opp_OZ2: (x, y: Z) x = (oppZ y) -> (leZ OZ x) -> (leZ OZ y) -> x = y.
Intros.
Rewrite (le_opp_OZ x y H H0 H1).
Cut y = (oppZ x).
Intro H'; Rewrite H'.
Rewrite (le_opp_OZ x y H H0 H1); Simpl; Reflexivity.
Apply opp_inv; Trivial.
Qed.



