(****************************************************************************)
(*                                                                          *)
(*                          Boolean  Functions                              *)
(*           of arity n,  variables of which are named x1 ....xn            *)
(*                  and supposed ordered  x1 < x2 ....< xn                  *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                            Boolean_functions.v                           *)
(****************************************************************************)

Require Export Bool.
Require Export Peano_dec.
Require Export Compare_dec.
Require Export Compare.
Require Export Le.
Require Export Gt.
Require Export Prelude0.
Require Export Prelude1.
Require Export Finite_sets.
Require Export Vars.


(*-------------------   Valuation of Boolean Variables    ------------------*)


Definition Assign := [n:nat](Var n)->bool.

Definition Cst_Ass := [n:nat][b:bool][i:(Var n)]b.


Lemma Assign_n_inhabited : (n:nat)(inhabited (Assign n) (all_ (Assign n))).
Proof.
Intro n.
Apply Def_inh
with v:= (Cst_Ass n true).
Unfold In all_;Auto.
Qed.
Hint Assign_n_inhabited.


Inductive Assign_eq [n:nat] : (Assign n)->(Assign n)->Prop :=
Def_Assign_eq: (A1,A2:(Assign n))
               ((xi:(Var n))(A1 xi)=(A2 xi)) -> (Assign_eq n A1 A2).


Lemma Assign_eq_refl : (n:nat)(A:(Assign n))(Assign_eq n A A).
Proof.
Intros n A.
Apply Def_Assign_eq;Auto.
Qed.


Lemma Assign_eq_sym : 
   (n:nat)(A1,A2:(Assign n))(Assign_eq n A1 A2) -> (Assign_eq n A2 A1).
Proof.
Intros n A1 A2 H_A1_eq_A2.
Apply  Def_Assign_eq.
Intro xi.
Elim H_A1_eq_A2;Auto.
Qed.
Immediate Assign_eq_sym.


Lemma Assign_eq_trans : (n:nat)(A1,A2,A3:(Assign n))
         (Assign_eq n A1 A2)->(Assign_eq n A2 A3)->(Assign_eq n A2 A3).
Proof.
Intros n A1 A2 A3 H_eq_A1A2 H_eq_A2A3.
Apply Def_Assign_eq.
Intro xi.
Apply trans_equal with y:=(A2 xi).
Elim H_eq_A1A2; Auto.
Elim H_eq_A2A3; Auto.
Qed.
 

(* Extensionality on Assignments *)
(*----------------------------------------------------------------------*)
Axiom Assign_subst : (n:nat)(X:(Assign n))
         (P:(Assign n)->Prop)(P X)->(A:(Assign n))(Assign_eq n X A)->(P A).
(*----------------------------------------------------------------------*)


Definition Upd : (n:nat)(Assign n)->(Var n)->bool->(Assign n) :=
[n:nat][A:(Assign n)][xi:(Var n)][b:bool]
    [xk:(Var n)]<bool> Match (eq_Var_dec n xi xk) with 
	(* Inl H *) [H:(eq_Var n xi xk)]b
        (* Inr H *) [H:~(eq_Var n xi xk)](A xk) end.


Definition Upd_le : (n:nat)(Assign n)->nat->bool->(Assign n) :=
[n:nat][A:(Assign n)][k:nat][b:bool]
    [xi:(Var n)]<bool> Match (le_gt_dec (Order n xi) k) with 
	(* Inl H *) [H:(le (Order n xi) k)]b
        (* Inr H *) [H:(gt (Order n xi) k)](A xi) end.


Lemma Upd_le_n_Cst : 
    (n:nat)(b:bool)(A:(Assign n))(Assign_eq n (Upd_le n A n b) (Cst_Ass n b)).
Proof.
Intros n b A.
Apply Def_Assign_eq.
Induction xi.
Intros i gi li.
Unfold Upd_le Cst_Ass; Simpl; Auto.
Elim (le_gt_dec i n); Auto.
Intro H_i_gt_n; Absurd (le i n); Auto.
Qed.
Hint Upd_le_n_Cst.


Lemma Upd_le_O_Id : (n:nat)(b:bool)(A:(Assign n))(Assign_eq n (Upd_le n A O b) A).
Proof.
Intros n b A.
Apply Def_Assign_eq.
Induction xi.
Intros i gi li.
Unfold Upd_le;Simpl.
Elim (le_gt_dec i O); Auto.
Intro H_absurde.
Absurd (le i O); Auto.
Qed.
Hint Upd_le_O_Id.


Lemma Upd_xi_xj : (n:nat)(xi,xj:(Var n))
    ~(eq_Var n xi xj) -> (A:(Assign n))(b:bool) ((Upd n A xj b) xi)=(A xi).
Proof.
Intros n xi xj
xi_xj_distinct A b.
Unfold Upd.
Elim (eq_Var_dec n xj xi); Auto.
Intro H_absurde.
Absurd (eq_Var n xi xj); Auto; Try Apply eq_Var_sym; Auto.
Qed.


Lemma eq_Var_eq_values : (n:nat)(xi,xj:(Var n))
      (A:(Assign n))(eq_Var n xi xj) -> (A xi)=(A xj).
Proof.
Intros  n xi xj A xi_eq_xj.
Pattern xj; Apply Proof_irrelevance with n:=n x:=xi; Auto.
Qed.


Definition Assign_gt := [n,k:nat][A:(Assign n)]
                       (xi:(Var n))(gt (Order n xi) k)->(A xi)=true.


Definition Upd' : (n:nat)(Var n)->bool->(Assign n)->(Assign n) :=
                   [n:nat][xi:(Var n)][b:bool][A:(Assign n)](Upd n A xi b).


Lemma Assign_k_finite : (n,k:nat)(le k n)->(finite (Assign n) (Assign_gt n k)).
Proof.
Intro n.
Induction k.
(*  k = O  *)
Intro H_triv.
(* Apply extensionality with V:=(Assign n) V0:=(add (Assign n) (empty (Assign n)) (Cst_Ass n true)). PB *)
Apply (extensionality (Assign n) 
                      (add (Assign n) (empty (Assign n)) (Cst_Ass n true))
                      (Assign_gt n O)).
Apply Def_set_eq; Unfold incl empty add In.
Intro A.
Induction 1; Try Contradiction.
Unfold Assign_gt.
Induction 1; Unfold Cst_Ass; Auto.
Intro A.
Right.
(* Apply Assign_subst with n:=n X:=(Cst_Ass n true); Auto. PB *)
Apply (Assign_subst n (Cst_Ass n true) [A:(Assign n)](Cst_Ass n true)=A); Auto.
Apply Def_Assign_eq; Unfold Cst_Ass.
Unfold Assign_gt in H.
Intro xi; Symmetry; Auto.
Apply finite_add; Auto.
(* k -> (S k) *)
Clear k.
Intros k Hk Sk_le_n.
ElimType (Ex [x:(Var n)]<nat>(S k)=(Order n x)); Auto.
Intros x Order_x.
Apply (extensionality (Assign n) 
                      (union (Assign n) 
           (Im (Assign n) (Assign_gt n k) (Upd' n x true))
           (Im (Assign n) (Assign_gt n k) (Upd' n x false)))
                      (Assign_gt n (S k))).
(* (set_eq (union (Im (Assign_gt n k) (Upd' x true))
                  (Im (Assign_gt n k) (Upd' x false))) (Assign_gt n (S k))) *)
Apply Def_set_eq; Unfold incl union In.
Intro A.
Induction 1; Unfold Im Assign_gt;
  Induction 1; Intro A0; Unfold In; Simpl;
  Induction 1; Intros H_A0 Def_A; Elim Def_A;
  Intros xi Order_xi; Unfold Upd' Upd;
  Elim (eq_Var_dec n x xi); Auto.
Intro H_absurde.
Absurd (gt (Order n xi) (S k)); Auto.
Pattern xi.
Apply Proof_irrelevance with n:=n x:=x; Auto.
Elim Order_x; Auto.
Intros A Prop_A.
Unfold Assign_gt in Prop_A.
ElimType true=(A x) \/ false=(A x); [Idtac | Idtac | Elim (A x); Auto].
(* true=(A x)->(Im (Assign n) (Assign_gt n k) (Upd' n x true) A) *)
Left.
Unfold Im Assign_gt In Upd'.
Exists A.
Split.
Intro xi.
Elim (le_or_gt_Var n xi x).
Unfold le_Var.
Elim Order_x.
Intros Oxi_le_Sk Oxi_gt_k.
Pattern xi.
Apply Proof_irrelevance with n:=n x:=x; Auto.
Unfold eq_Var.
Apply trans_equal with y:=(S k); Auto.
Unfold gt_Var.
Elim Order_x; Auto.
Pattern (Upd n A x true).
Apply Assign_subst with n:=n X:=A; Auto.
Apply Def_Assign_eq.
Unfold Upd.
Intro xi; Elim (eq_Var_dec n x xi); Auto.
Intro xi_eq_x. 
Pattern xi.
Apply Proof_irrelevance with n:=n x:=x; Auto.
(* false=(A x)->(Im (Assign n) (Assign_gt n k) (Upd' n x false) A) *)
Right.
Unfold Im Assign_gt In Upd'.
Exists (Upd n A x true).
Split.
Intro xi.
Elim (le_or_gt_Var n xi x).
Unfold le_Var.
Elim Order_x.
Intros Oxi_le_Sk Oxi_gt_k.
Pattern xi.
Apply Proof_irrelevance with n:=n x:=x.
Unfold Upd.
Elim (eq_Var_dec n x x); Auto.
Intro H_abs.
Absurd (eq_Var n x x); Auto.
Unfold eq_Var.
Apply trans_equal with y:=(S k); Auto.
Unfold gt_Var.
Elim Order_x.
Intros H1 H2.
Apply trans_equal with y:=(A xi); Auto.
Unfold Upd.
Elim (eq_Var_dec n x xi); Auto.
Intro H_abs.
Absurd (gt (Order n xi) (S k)); Auto.
Pattern xi.
Apply Proof_irrelevance with n:=n x:=x; Auto.
Elim Order_x; Auto.
Pattern (Upd n (Upd n A x true) x false).
Apply Assign_subst with n:=n X:=A; Auto.
Apply Def_Assign_eq.
Unfold Upd.
Intro xi;Elim (eq_Var_dec n x xi); Auto.
Intro xi_eq_x. 
Pattern xi.
Apply Proof_irrelevance with n:=n x:=x; Auto.
(*  (finite (Assign n) (union (Im (Assign_gt n k) (Upd' n x true)) 
                              (Im (Assign_gt n k) (Upd' n x false))))  *)
Apply finite_union; Apply finite_image; Auto.
Qed.

Lemma Assign_n_finite : (n:nat)(finite (Assign n) (all_ (Assign n))).
Proof.
Intro n.
Apply (extensionality (Assign n) (Assign_gt n n) (all_ (Assign n))).
Apply Def_set_eq; Unfold incl Assign_gt all_ In; Auto.
Intros A Triv xi H_abs.
Absurd (le (Order n xi) n); Auto.
Apply Assign_k_finite; Auto.
Qed.
Hint Assign_n_finite.


(*--------------------      Boolean Functions      -------------------------*)


Definition BF := [n:nat](Assign n)->bool.


(*------------------     Equality of functions      ------------------------*)


Definition BF_eq : (n:nat)(BF n)->(BF n)->Prop :=
                   [n:nat][f1,f2:(BF n)](A:(Assign n))(<bool>(f1 A)=(f2 A)).


Lemma BF_eq_refl : (n:nat)(f:(BF n))(BF_eq n f f).
Proof.
Intros n f.
Unfold BF_eq; Auto.
Qed.
Hint BF_eq_refl.


Lemma BF_eq_sym : (n:nat)(f1,f2:(BF n))(BF_eq n f1 f2)->(BF_eq n f2 f1).
Proof.
Intros n f1 f2.
Unfold BF_eq; Auto.
Qed.
Immediate BF_eq_sym.


Lemma  BF_eq_trans : 
  (n:nat)(f1,f2,f3:(BF n))(BF_eq n f1 f2)->(BF_eq n f2 f3)->(BF_eq n f1 f3).
Proof.
Intros  n f1 f2 f3. 
Unfold BF_eq.
Intros H12 H23 A.
Apply trans_equal with y:=(f2 A); Auto.
Qed.


Lemma BF_eq_congr :
  (n:nat)(f1,f'1,f2,f'2:(BF n))(BF_eq n f1 f'1)->(BF_eq n f2 f'2)
      -> (A:(Assign n))(f1 A)=(f2 A) -> (f'1 A)=(f'2 A).
Proof.
Intros n 
f1 f'1 f2 f'2
eq1 eq2 A H.
Unfold BF_eq in eq1.
Unfold BF_eq in eq2.
Apply trans_equal with y:=(f2 A);Auto.
Apply trans_equal with y:=(f1 A);Auto.
Qed.


(*------------------    Restriction on a variable   ------------------------*)


Definition Restr : (n:nat)(BF n)->(Var n)->bool->(BF n) :=
                  [n:nat][f:(BF n)][xi:(Var n)][b:bool]
                      [A:(Assign n)](f (Upd n A xi b)).


Lemma f_Restr_f : (n:nat)(A:(Assign n))(f:(BF n))(xi:(Var n))(b:bool)
      b=(A xi) -> ((Restr n f xi b) A)=(f A).
Proof.
Intros n A f xi b Def_b.
Unfold Restr.
Pattern (Upd n A xi b).
Apply Assign_subst with n:=n X:=A; Auto.
Apply Def_Assign_eq.
Intro xj.
Unfold Upd.
Elim (eq_Var_dec n xi xj); Auto.
Intro xi_eq_xj.
Apply Proof_irrelevance with n:=n x:=xi a:=xj ; Auto; Elim Def_b; Trivial.
Qed.


Lemma BF_eq_Restr : (n:nat)(f1,f2:(BF n))(BF_eq n f1 f2)
      -> (x:(Var n))(b:bool)(BF_eq n (Restr n f1 x b) (Restr n f2 x b)).
Proof.
Intros n
f1 f2 
f1_eq_f2 x b .
Unfold BF_eq in f1_eq_f2.
Unfold BF_eq Restr; Auto.
Qed.
Hint BF_eq_Restr.


Lemma Restr_xi_Restr_xi : (n:nat)(f:(BF n))(xi:(Var n))(b1,b2:bool)
      (BF_eq n (Restr n (Restr n f xi b1) xi b2) (Restr n f xi b1)).
Proof.
Intros
n f xi b1 b2.
Unfold BF_eq.
Intro A.
Unfold Restr.
Pattern (Upd n (Upd n A xi b2) xi b1).
Apply Assign_subst with n:=n X:=(Upd n A xi b1); Auto.
Apply Def_Assign_eq.
Intro x.
Unfold Upd.
Elim (eq_Var_dec n xi x); Auto.
Qed.
Hint Restr_xi_Restr_xi.


Lemma Restr_xi_Restr_xj : (n:nat)(f:(BF n))(xi,xj:(Var n))(bi,bj:bool)
       ~(eq_Var n xi xj)->(BF_eq n (Restr n (Restr n f xi bi) xj bj) 
                                   (Restr n (Restr n f xj bj) xi bi)).
Proof.
Intros n f xi xj bi bj xi_diff_xj.
Unfold BF_eq.
Intro A.
Unfold Restr.
Pattern (Upd n (Upd n A xj bj) xi bi).
Apply Assign_subst with n:=n X:=(Upd n (Upd n A xi bi) xj bj); Auto.
Apply Def_Assign_eq.
Intro x.
Unfold Upd.
Elim (eq_Var_dec n xi x); Elim (eq_Var_dec n xj x); Auto.
Intros H1 H2.
Absurd (eq_Var n xi xj); Auto.
Apply eq_Var_trans with x2:=x; Try Assumption; Apply eq_Var_sym; Assumption.
Qed.
Hint Restr_xi_Restr_xj.


(*--------------          Dependent variables         ----------------------*)

Definition Dep_Var : (n:nat)(BF n)->(Var n)->Prop :=
   [n:nat][f:(BF n)][xi:(Var n)]~(BF_eq n (Restr n f xi true) 
                                          (Restr n f xi false)).


Lemma xi_non_dep_Restr_xi_Id : (n:nat)(f:(BF n))(xi:(Var n))
       ~(Dep_Var n f xi) -> (b:bool)(BF_eq n f (Restr n f xi b)).
Proof.
Intros n f xi xi_non_dep b.
Unfold BF_eq Restr.
Intro A.
Elim (eq_bool_dec (f A) (f (Upd n A xi b))); Auto.
Intro Def_A.
Absurd (Dep_Var n f xi); Auto.
(* (Dep_Var n f xi) *)
Unfold Dep_Var BF_eq not.
Intro H_absurde.
Cut (Assign_eq n A (Upd n A xi (neg b))).
Intro Axi_eq_negb.
Unfold not in Def_A.
Apply Def_A.
Pattern 1 A.
Apply Assign_subst with n:=n X:=(Upd n A xi (neg b)); Auto.
Change (Restr n f xi (neg b) A) = (Restr n f xi b A).
Elim b; Simpl; Auto.
(* (Assign_eq n A (Upd n A xi (neg b))) *)
Apply Def_Assign_eq.
Intro x.
Unfold Upd.
Elim (eq_Var_dec n xi x); Auto.
Intro x_eq_xi.
Cut (A x)=(neg b) \/ (A x)=b.
Induction 1;Auto.
Intro Cas_absurde_Ax.
Absurd (f A)=(f (Upd n A xi b)); Auto.
Pattern 1 A.
Apply Assign_subst 
with n:=n X:=(Upd n A xi b); Auto.
Apply Def_Assign_eq.
Intro y.
Unfold Upd.
Elim (eq_Var_dec n xi y); Auto.
Intro xi_eq_y.
Pattern y.
Apply Proof_irrelevance with n:=n x:=x; Auto.
Apply eq_Var_trans with x2:=xi; Auto.
(* (A x)=(neg b) \/ (A x)=b *)
Elim (A x); Elim b; Auto.
Qed.
                          

Lemma xi_non_dep_Restr_xi : 
  (n:nat)(f:(BF n))(xi:(Var n))(b:bool)~(Dep_Var n (Restr n f xi b) xi).
Proof.
Intros n f xi b.
Unfold not.
Intro H_absurde.
Unfold Dep_Var in H_absurde.
Absurd (BF_eq n (Restr n (Restr n f xi b) xi true)
                (Restr n (Restr n f xi b) xi false)); Auto.
Apply BF_eq_trans with f2:=(Restr n f xi b); Auto.
(* Use : Apply Restr_xi_Restr_xi *)
Qed.
Hint xi_non_dep_Restr_xi.


Lemma BF_eq_same_dep_var : (n:nat)(f1,f2:(BF n))(xi:(Var n))
      (BF_eq n f1 f2)->(Dep_Var n f1 xi)->(Dep_Var n f2 xi).
Proof.
Intros
n f1 f2 xi
f1_eq_f2 f1_dep_xi.
Unfold Dep_Var.
Unfold Dep_Var in f1_dep_xi.
Apply Contra with Q:=(BF_eq n (Restr n f1 xi true) (Restr n f1 xi false));Auto.
Intro H2.
Apply BF_eq_trans with f2:=(Restr n f2 xi true); Auto.
Apply BF_eq_trans with f2:=(Restr n f2 xi false); Auto.
Qed.


Lemma Dep_Restr_f_Dep_f : (n:nat)(f:(BF n))(xi,x:(Var n))(b:bool)
      (Dep_Var n (Restr n f xi b) x) -> (Dep_Var n f x).
Proof.
Intros
n f xi x b.
Elim (eq_Var_dec n xi x).
(* Case xi = x *)
Intro xi_eq_x. 
Pattern x .
Apply Proof_irrelevance with n:=n x:=xi; Auto.
Intro H_absure.
Absurd (Dep_Var n (Restr n f xi b) xi); Auto.
(* Case xi diff x *)
Intro xi_diff_x.
Intro H_dep.
Unfold Dep_Var in H_dep.
Unfold Dep_Var.
Apply Contra with Q:=(BF_eq n (Restr n (Restr n f xi b) x true)
                              (Restr n (Restr n f xi b) x false)); Auto.
Intro H'.
Apply BF_eq_trans with n:=n f2:=(Restr n (Restr n f x true) xi b); Auto.
(* Use : Apply Restr_xi_Restr_xj ; Assumption *)
Apply BF_eq_trans with n:=n f2:=(Restr n (Restr n f x false) xi b); Auto.
(* Use : Apply BF_eq_Restr *)
Apply Restr_xi_Restr_xj.
Apply Contra with Q:=(eq_Var n xi x); Auto.
(* Use : Apply eq_Var_sym  *)
Qed.


Definition Dep_set : (n:nat)(BF n)->(Varset n) :=
                    [n:nat][f:(BF n)][xi:(Var n)](Dep_Var n f xi).


Lemma A_Dep_xi_dec : (n:nat)(f:(BF n))(xi:(Var n))
      (decidable (Assign n) [A:(Assign n)]<bool>(Restr n f xi true A) =
                                                (Restr n f xi false A)).
Proof.
Intros n f xi.
Apply Def_dec.
Intro A.
Elim (eq_bool_dec (Restr n f xi true A) (Restr n f xi false A));
Unfold In; Auto.
Qed.
Hint A_Dep_xi_dec.


Lemma Dep_set_decidable : (n:nat)(f:(BF n))(decidable (Var n) (Dep_set n f)).
Proof.
Intros n f.
Apply Def_dec.
Unfold In Dep_set.
Intro xi.
Cut ((A:(Assign n))
 ([X:(Assign n)](Restr n f xi true X)=(Restr n f xi false X) A)) \/
   ~((A:(Assign n))
 ([X:(Assign n)](Restr n f xi true X)=(Restr n f xi false X) A)).
Induction 1; Intro H1.
Right.
Unfold Dep_Var not; Auto.
Left.
Unfold Dep_Var; Auto.
Apply P_dec_AllP_dec with V:=(Assign n) 
     P:=[A:(Assign n)](Restr n f xi true A)=(Restr n f xi false A); Auto.
Qed.


Lemma Dep_set_finite :  
   (n:nat)(f:(BF n))(<nat>Ex([c:nat](cardinal (Var n) (Dep_set n f) c))).
Proof.
Intros n f.
Apply finite_cardinal.
Apply decidable_finite.
Apply Var_n_finite.
Apply Dep_set_decidable.
Qed.


(*--------      Existence of a greatest dependent variable       ----------*)


Inductive Greatest_Dep_Var [n:nat;f:(BF n);xi:(Var n)] : Prop :=
  Def_GreatestDV : (Dep_Var n f xi)
                -> ((xj:(Var n))(Dep_Var n f xj)->(le_Var n xj xi))
                           -> (Greatest_Dep_Var n f xi).


Lemma Gst_Dep_Var_inv_Dep : (n:nat)(f:(BF n))(xi:(Var n))
      (Greatest_Dep_Var n f xi) -> (Dep_Var n f xi).
Proof.
Induction 1; Auto.
Qed.
Hint Gst_Dep_Var_inv_Dep.


Lemma Gst_Dep_Var_inv_Greatest : (n:nat)(f:(BF n))
   (xi:(Var n))(Greatest_Dep_Var n f xi) -> 
   (xj:(Var n))(Dep_Var n f xj) -> (le_Var n xj xi).
Proof.
Induction 1; Auto.
Qed.

Lemma CardSk_Greatest : 
   (n,c:nat)(f:(BF n))(cardinal (Var n) (Dep_set n f) c) ->
          (gt c O) -> (Ex [xi:(Var n)](Greatest_Dep_Var n f xi)).
Proof.
Intros n c f Def_c c_gt_O.
Cut (Ex [xi:(Var n)](greatest (Var n) (le_Var n) (Dep_set n f) xi)).
2: Apply Card_gtO_exist_greatest with V_eq:=(eq_Var n) c:=c; Auto.
Induction 1.
Intros xi Def_xi.
Exists xi.
Apply Def_GreatestDV; Elim Def_xi; Auto. 
Qed.

Lemma Dep_Restrf_incl_Depf : (n:nat)(f:(BF n))(xi:(Var n))(b:bool)
      (Dep_Var n f xi) -> (incl_st (Var n) (Dep_set n (Restr n f xi b)) 
                                           (Dep_set n f)).
Proof.
Intros n f xi b H.
Unfold incl_st.
Split. 
Unfold incl Dep_set.
Intro x; Unfold In.
Intro H1.
Apply Dep_Restr_f_Dep_f with xi:=xi b:=b; Trivial.
Exists xi.
Split; Unfold In Dep_set; Auto.
(*  Use : Apply xi_non_dep_Restr_xi *)
Qed.
Hint Dep_Restrf_incl_Depf.


Lemma CardSk_Card_Restr : (n,c:nat)(f:(BF n))(cardinal (Var n) (Dep_set n f) c)
      ->(xi:(Var n))(Dep_Var n f xi)
      ->(b:bool)(cr:nat)
         (cardinal (Var n) (Dep_set n (Restr n f xi b)) cr)->(gt c cr).
Proof.
Intros n c
f Def_c xi
f_dep_xi b 
cr Def_cr.
Apply incl_st_card_lt
with V:=(Var n) E1:=(Dep_set n (Restr n f xi b))
E2 :=(Dep_set n f);Auto.
(* Use : Apply Dep_Restrf_incl_Depf ; Assumption *)
Qed.


(*---------------          Constant functions          ---------------------*)


Inductive Constant_fct [n:nat;f:(BF n)] : Prop :=
Def_cst_fct: (b:bool)((A:(Assign n))(<bool>b=(f A)))->(Constant_fct n f).


Definition Fcst : (n:nat)(BF n)->bool->(BF n) := 
                 [n:nat][f:(BF n)][b:bool][A:(Assign n)](f (Cst_Ass n b)).


Definition Restr_le : (n:nat)(BF n)->nat->bool->(BF n) :=
           [n:nat][f:(BF n)][k:nat][b:bool][A:(Assign n)](f (Upd_le n A k b)).


Lemma Constant_fct_inv : (n:nat)(f:(BF n))(Constant_fct n f)->
               (<bool>Ex([b:bool](A:(Assign n))(<bool>b=(f A)))).
Proof.
Induction 1.
Intros b H_f.
Exists b;Auto.
Qed.
Hint Constant_fct_inv.


Lemma CardO_no_Dep_Var : (n:nat)(f:(BF n))
      (cardinal (Var n) (Dep_set n f) O)->(xi:(Var n))~(Dep_Var n f xi).
Proof.
Intros n f H_cardO xi.
Cut (set_eq (Var n) (Dep_set n f) (empty (Var n))).
2: Apply cardinalO_empty; Trivial.
Intro H_Dep_set.
Cut (incl (Var n) (Dep_set n f) (empty (Var n))) /\
    (incl (Var n) (empty (Var n)) (Dep_set n f)).
2: Apply set_eq_inv; Trivial.
Induction 1.
Unfold not; Intros H1 H2 H3.
Apply H1 with x:=xi; Trivial.
Qed.


Lemma Restr_le_k_Restr_le_Sk : (n,k:nat)(f:(BF n))(x:(Var n))
  (Order n x)=(S k) -> ~(Dep_Var n f x) ->
  (b:bool)(BF_eq n f (Restr_le n f k b)) -> (BF_eq n f (Restr_le n f (S k) b)).
Proof.
Intros n k f x Order_x x_non_dep b f_eq_Rk_f.
Apply BF_eq_trans with n:=n f2:= (Restr n (Restr_le n f k b) x b).
Apply BF_eq_trans with f2:= (Restr n f x b).
Apply xi_non_dep_Restr_xi_Id; Trivial.
Apply BF_eq_Restr; Trivial.
Unfold BF_eq.
Intro A.
Unfold Restr_le Restr.
Pattern (Upd_le n (Upd n A x b) k b).
Apply Assign_subst with n:=n X:=(Upd_le n A (S k) b); Auto.
Apply Def_Assign_eq.
Intro xi.
Unfold Upd_le Upd.
Elim (le_gt_dec (Order n xi) (S k)); Elim (le_gt_dec (Order n xi) k);
     Elim (eq_Var_dec n x xi); Auto.
(* (gt (Order n xi) k) (le (Order n xi) (S k)) ~(eq_Var n x xi) *)
Intros H1 H2 H_absurde.
Absurd (eq_Var n x xi); Auto.
Apply same_order_eq_Var with k:=(S k); Auto.
(* (gt (Order n xi) (S k)) (le (Order n xi) k) (eq_Var n x xi) *)
Intros H1 H2 H3.
Absurd (gt k (S k)); Auto.
Apply le_gt_trans with m:=(Order n xi); Auto.
(* (gt (Order n xi) (S k)) (le (Order n xi) k) ~(eq_Var n x xi) *)
Intros H1 H2 H3.
Absurd (gt k (S k)); Auto.
Apply le_gt_trans with m:=(Order n xi); Auto.
(* (gt (Order n xi) k) (gt (Order n xi) (S k)) (eq_Var n x xi) *)
Intros H1 Order_xi H_absurde. 
Cut (Order n x)=(Order n xi); Try Apply P_Var_P_nat; Auto.
Intro H_absurde2.
Absurd (gt (Order n xi) (S k)); Auto.
Elim H_absurde2;Elim Order_x; Auto.
Qed.


Lemma no_Dep_Var_Cstk : (n:nat)(f:(BF n))((xi:(Var n))~(Dep_Var n f xi))
      ->(k:nat)(le k n)->(b:bool)(BF_eq n f (Restr_le n f k b)).
Proof.
Intros n f no_dep_var.
Induction k.
(* k = O *)
Unfold BF_eq Restr_le.
Intros H_triv b A.
Pattern (Upd_le n A O b).
Apply Assign_subst with n:=n X:=A; Auto.
(* k -> (S k) *)
Clear k.
Intros k Hk Sk_le_n b.
ElimType (<(Var n)>Ex([x:(Var n)]<nat>(S k)=(Order n x))); Auto.
Intros x Order_x.
Apply Restr_le_k_Restr_le_Sk with x:=x; Auto.
Qed.


Lemma Restr_le_n_Fcst :
   (n:nat)(f:(BF n))(b:bool)(BF_eq n (Restr_le n f n b) (Fcst n f b)).
Proof.
Intros n f b.
Unfold BF_eq Restr_le.
Intro A.
Pattern (Upd_le n A n b).
Apply Assign_subst with n := n X:=(Cst_Ass n b); Auto.
Qed.
Hint Restr_le_n_Fcst.


Lemma CardO_cst_fct:
(n:nat)(f:(BF n))(cardinal (Var n) (Dep_set n f) O) -> (Constant_fct n f).
Proof.
Intros n f H_card.
Apply Def_cst_fct with b:=(f (Cst_Ass n true)).
Change (BF_eq n (Fcst n f true) f).
Apply BF_eq_sym.
Apply BF_eq_trans with f2:=(Restr_le n f n true); Auto.
Apply no_Dep_Var_Cstk with k:=n; Auto.
Intro xi.
Apply CardO_no_Dep_Var; Auto.
Qed.
