(****************************************************************************)
(*                                                                          *)
(*                                                                          *)
(*                         Canonicity of BDTs                               *)
(*                                                                          *)
(*                                                                          *)
(*                                                                          *)
(****************************************************************************)
(*                             Canonicity_BDT.v                             *)
(****************************************************************************)

Require Boolean_functions.
Require BDTs.
Require Complete_induction.

Variable n:nat.

(****************     Existence of BDT Canonical form    ******************)

Lemma Cst_fct_Fzero_Fone : 
(f:(BF n))(Constant_fct n f) -> (BF_eq n f (Fun n (zero n))) \/
                                (BF_eq n f (Fun n (one n))).
Proof.
Intros f H_f_Cst.
Cut (Ex [b:bool](A:(Assign n))b=(f A)); Auto.
Induction 1.
Induction x.
Intro f_Cst_true.
Right; Simpl.
Unfold BF_eq; Auto.
Intro f_Cst_false.
Left; Simpl.
Unfold BF_eq; Auto.
Qed.

Lemma Exist_DepO:  
 (f:(BF n))(Constant_fct n f)->(Ex [bdt:(BDT n)](Denote n f bdt)).
Proof.
Intros f f_Cst.
Cut (BF_eq n f (Fun n (zero n))) \/ (BF_eq n f (Fun n (one n)));
  Try Apply Cst_fct_Fzero_Fone; Auto.
Induction 1.
      (*  f cst false *)
Intro f_eq_Fzero.
Exists (zero n).
Unfold Denote; Auto.
      (*  f cst true *)
Intro f_eq_Fone.
Exists (one n).
Unfold Denote; Auto.
Qed.

Lemma Denote_true_false :
  (f:(BF n))(xi:(Var n))(Greatest_Dep_Var n f xi) -> (bl,bh:(BDT n))
  (Denote n (Restr n f xi false) bl) -> (Denote n (Restr n f xi true) bh)
                      -> (Denote n f (node n xi bl bh)).
Proof.
Intros f xi Def_xi bl bh Def_bl Def_bh.
Unfold Denote.
Split.
(*--- (OBDT n (node n xi bl bh)) ----*)
Apply order_node.
Apply Denote_inv_OBDT with (Restr n f xi false); Assumption.
Apply Denote_inv_OBDT with (Restr n f xi true); Assumption.
     (* (gt (Order n xi) (Dimension n bl)) *)
Elim (gt_O_eq (Dimension n bl)).
2:Induction 1; Auto.
Intro Dim_low_gt_O.
Cut (Ex [x:(Var n)]((Order n x)=(Dimension n bl) 
                 /\ (Greatest_Dep_Var n (Fun n bl) x))).
Induction 1.
Intro xl.
Induction 1.
Intros Order_xl Def_xl.
Elim Order_xl.
Apply le_ne_gt.
Change (le_Var n xl xi).
Apply Gst_Dep_Var_inv_Greatest with f:=f; Auto.
  (* (Dep_Var n f xl) *)
Apply Dep_Restr_f_Dep_f with xi:=xi b:=false.
Apply BF_eq_same_dep_var with f1:=(Fun n bl); 
   [Apply BF_eq_sym; Auto|Apply Gst_Dep_Var_inv_Dep; Auto].
  (*  ~(Order n xl)=(Order n xi) *)
Unfold not.
Intro H_absurde.
Absurd (Dep_Var n (Fun n bl) xi).
             (*  ~(Dep_Var n (Fun n bl) xi) *)
Apply Contra with Q:= (Dep_Var n (Restr n f xi false) xi); Auto.
(*  Use: Apply xi_non_dep_Restr_xi *)
Intro Flow_dep_xi.
Apply BF_eq_same_dep_var with f1:=(Fun n bl); Try Apply BF_eq_sym; Auto.
             (* (Dep_Var n (Fun n bl) xi) *)
(* Apply Proof_irrelevance with n:=n x:=xl; Auto. PB *)
Apply (Proof_irrelevance n xl [a:(Var n)](Dep_Var n (Fun n bl) a)); Auto.
(* Use : Apply Gst_Dep_Var_inv_Dep ; Assumption *)
   (*    (Ex [x:(Var n)](Order n x)=(Dimension n bl)
                     /\ (Greatest_Dep_Var n (Fun n bl) x))   *)
Apply Dim_is_Greatest_Dep_Var; Auto. 
Apply Denote_inv_OBDT with f:=(Restr n f xi false); Trivial.
    (*  (gt (Order n xi) (Dimension n bh)) *)
Elim (gt_O_eq (Dimension n bh)).
2: Induction 1; Auto.
Intro Dim_high_gt_O.
Cut (Ex [x:(Var n)]((Order n x)=(Dimension n bh)
                 /\ (Greatest_Dep_Var n (Fun n bh) x))).
Induction 1.
Intro x_high.
Induction 1.
Intros Order_x_high Def_x_high.
Elim Order_x_high.
Apply le_ne_gt.
Change (le_Var n x_high xi).
Apply Gst_Dep_Var_inv_Greatest with f:=f; Auto.
  (* (Dep_Var n f x_high) *)
Apply Dep_Restr_f_Dep_f with xi:=xi b:=true.
Apply BF_eq_same_dep_var with f1:=(Fun n bh); 
  [Apply BF_eq_sym; Auto|Apply Gst_Dep_Var_inv_Dep; Auto].
  (*  ~(Order n x_high)=(Order n xi) *)
Unfold not.
Intro H_absurde.
Absurd (Dep_Var n (Fun n bh) xi).
             (*  ~(Dep_Var n (Fun n bh) xi) *)
Apply Contra with Q:= (Dep_Var n (Restr n f xi true) xi); Auto.
(*  Use: Apply xi_non_dep_Restr_xi *)
Intro Fhigh_dep_xi.
Apply BF_eq_same_dep_var with f1:=(Fun n bh); Try Apply BF_eq_sym; Auto.
             (* (Dep_Var n (Fun n bh) xi) *)
(* Apply Proof_irrelevance with n:=n x:=x_high; Auto. PB *)
Apply (Proof_irrelevance n x_high [a:(Var n)](Dep_Var n (Fun n bh) a)); Auto.
(* Use : Apply Gst_Dep_Var_inv_Dep ; Assumption *)
   (*    (Ex [x:(Var n)](Order n x)=(Dimension n bh)
                     /\ (Greatest_Dep_Var n (Fun n bh) x))       *)
Apply Dim_is_Greatest_Dep_Var; Auto. 
Apply Denote_inv_OBDT with f:=(Restr n f xi true); Trivial.
   (*  ~(BDT_eq n bl bh)  *)
Unfold not.
Intro H_absurde.
Absurd (BF_eq n (Restr n f xi true) (Restr n f xi false)).
    (*  ~(BF_eq n (Restr n f xi true) (Restr n f xi false)) *)
Change (Dep_Var n f xi); Auto. (* Use : Apply Gst_Dep_Var_inv_Dep *)
    (*   (BF_eq n (Restr n f xi true) (Restr n f xi false)) *)
Apply BF_eq_trans with f2:= (Fun n bh); Auto.
Apply BF_eq_trans with f2:= (Fun n bl); Apply BF_eq_sym; Auto.
(* Use : Apply BDT_iso_BDF_eq *)
      (* (BF_eq n f (Fun n (node n xi bl bh))) *)
Unfold BF_eq.
Intro A; Simpl.
ElimType true=(A xi) \/ false=(A xi); Try Induction 1.
      (*  (f A)=(Fun n bh A) *)
Elim (f_Restr_f n A f xi true); Auto.
Cut (BF_eq n (Restr n f xi true) (Fun n bh)); Auto. 
(* Use : Apply Denote_inv_eq *)
Unfold BF_eq; Auto.
      (*  (f A)=(Fun n bl A)  *)
Elim (f_Restr_f n A f xi false); Auto.
Cut (BF_eq n (Restr n f xi false) (Fun n bl)); Auto. 
(* Use : Apply Denote_inv_eq *)
Unfold BF_eq; Auto.
      (*   true=(A xi) \/ false=(A xi) *)
Elim (A xi); Auto.
Qed.


(*--------------------------------------------------------------------------*)
   Theorem Existence: (f:(BF n))(Ex [bdt:(BDT n)](Denote n f bdt)).
(*--------------------------------------------------------------------------*)

Proof.
Intro f.
Elim (Dep_set_finite n f).
Intros  kf Def_kf.
Cut (k:nat)(f:(BF n))(cardinal (Var n) (Dep_set n f) k)
                        -> (Ex [bdt:(BDT n)](Denote n f bdt)) .
Intro H; Apply H with k:=kf; Trivial.
Intro k; Pattern k; Apply nat_wf_ind.
(****  |Dep(f)|=O     ****)
Intros fO H_CardO.
Apply Exist_DepO; Apply CardO_cst_fct; Assumption.
(****  |Dep(f)|=(S l) ****)
Intros l H_Exist_Dep_l f_Sl H_CardSl.
Elim (CardSk_Greatest n (S l) f_Sl); Trivial. 
Intros xi Def_xi.
Cut (Ex [bdt:(BDT n)](Denote n (Restr n f_Sl xi true) bdt)).
Intro H_rec_true; Elim H_rec_true.
Intros bh Def_bh.
Cut (Ex [bdt:(BDT n)](Denote n (Restr n f_Sl xi false) bdt)).
Intro H_rec_false; Elim H_rec_false.
Intros bl Def_bl.
Exists (node n xi bl bh).
Apply Denote_true_false; Trivial.
      (*   Proof of Ex([bdt:(BDT n)](Denote n (Restr n f_Sl xi false) bdt))  *)
Elim (Dep_set_finite n (Restr n f_Sl xi false)).
Intros  l_false Def_l_false.
Apply H_Exist_Dep_l with l0:=l_false; Trivial.
Apply gt_S_le.
Apply CardSk_Card_Restr
      with n:=n c:=(S l) f:=f_Sl xi:=xi b:=false cr:=l_false; Auto.
      (*   Proof of Ex([bdt:(BDT n)](Denote n (Restr n f_Sl xi true) bdt))  *)
Elim (Dep_set_finite n (Restr n f_Sl xi true)).
Intros  l_true Def_l_true.
Apply H_Exist_Dep_l with l0:=l_true; Trivial.
Apply gt_S_le.
Apply CardSk_Card_Restr
      with n:=n c:=(S l) f:=f_Sl xi:=xi b:=true cr:=l_true; Auto.
Qed.





(****************               Uniqueness                   ******************)



(*----------------------------------------------------------------------------*)
   Theorem Uniqueness: 

   (f:(BF n))(bdt1,bdt2:(BDT n))
    (Denote n f bdt1)->(Denote n f bdt2)->(BDT_eq n bdt1 bdt2).
(*----------------------------------------------------------------------------*)

Proof.
Intros f bdt1 bdt2 Def_bdt1 Def_bdt2.
Apply Fun_eq_OBDT_eq; Try Apply Denote_inv_OBDT with f; Auto.
Apply BF_eq_trans with f; Auto.
Apply BF_eq_sym; Auto.
Qed.



(*----------------------------------------------------------------------------*)
  Remark Canonicity_BF_BDT: 

  (f1,f2:(BF n))(bdt1,bdt2:(BDT n))
  (Denote n f1 bdt1)->(Denote n f2 bdt2)->(BF_eq n f1 f2)->(BDT_eq n bdt1 bdt2).
(*----------------------------------------------------------------------------*)
Proof.
Intros f1 f2 bdt1 bdt2 Def_bdt1 Def_bdt2 H_eq_f1_f2.
Cut (Denote n f1 bdt2).
Intro H.
Apply Uniqueness with f:=f1; Assumption.
Unfold Denote.
Split.
Elim Def_bdt2; Auto.
Apply BF_eq_trans with f2:=f2; Try Assumption.
Elim Def_bdt2; Auto.
Qed.

(*--------------------------------------------------------------------------*)
  Remark Canonicity_BDT_BF:
  (f1,f2:(BF n))(bdt1,bdt2:(BDT n))
  (Denote n f1 bdt1) -> (Denote n f2 bdt2) -> 
  (BDT_eq n bdt1 bdt2) -> (BF_eq n f1 f2).                     
(*--------------------------------------------------------------------------*)
Proof.
Intros f1 f2 bdt1 bdt2 Def_bdt1 Def_bdt2 bdt1_iso_bdt2.
Apply BF_eq_trans with f2:=(Fun n bdt2).
Apply BF_eq_trans with f2:=(Fun n bdt1); Auto.
Apply BF_eq_sym; Auto.
Qed.

