(****************************************************************************)
(* Course of values induction                                               *)
(* To be integrated in ARITH library                                        *)
(*                                                                          *)
(****************************************************************************)
(*                           Complete_induction.v                           *)
(****************************************************************************)

Require Lt.
Require Le.

Definition lt_hereditary := [P:nat->Prop]
       (n:nat)((m:nat)(lt m n)->(P m)) -> (P n).

Lemma le_split : (m,n:nat)(le m (S n)) -> (le m n) \/ m=(S n).
Proof.
Intros m n H; Elim (le_lt_or_eq m (S n)); Auto.
Qed.

Lemma course_of_values :
      (P:nat->Prop)(lt_hereditary P) -> (n,m:nat)(le m n) -> (P m).
Proof.
Unfold lt_hereditary; Induction n.
Intros; Apply H.
Elim (le_n_O_eq m); Trivial.
Intros p abs; Elim lt_n_O with p; Trivial.
Intros p lemp m lemSp; Elim le_split with m p; Auto.
Intro E; Rewrite E; Auto. (* using lt_n_Sm_le *)
Qed.

Lemma complete_induction : (P:nat->Prop)(lt_hereditary P) -> (n:nat)(P n).
Proof.
Intros; Apply course_of_values with m:=n n:=n; Auto.
Qed.

(* Version Ledinot *)
Lemma nat_wf_ind : (P:nat->Prop)
      (P O) -> ((k:nat)((l:nat)(le l k)->(P l)) -> (P (S k))) -> (n:nat)(P n).
Proof.
Intros P H0 Hk.
Cut (n,k:nat)(le k n)->(P k).
Intros HC n.
Apply HC with n:=n; Auto.
Induction n.
Intros k Def_k.
ElimType O=k; Auto.
Clear n.
Intros n Hn k k_le_Sn.
Elim (le_split k n); Auto.
Intro Def_k.
Rewrite -> Def_k.
Auto.
Qed.

