(****************************************************************************)
(* This contribution was updated for Coq V5.10 by the COQ workgroup.        *)
(* January 1995                                                             *)
(****************************************************************************)
(*                                                                          *)
(*      Coq V5.8                                                            *)
(*                                                                          *)
(*                                                                          *)
(*      Alternating Bit Protocol                                            *)
(*                                                                          *)
(*      Jan Friso Groote                                                    *)
(*      Utrecht University                                                  *)
(*                                                                          *)
(*      November 1992                                                       *)
(*                                                                          *)
(* From bezem@phil.ruu.nl Fri Apr  2 13:14:31 1993                          *)
(*                                                                          *)
(****************************************************************************)
(*                                abp_defs.v                                *)
(****************************************************************************)

Require abp_base.

(* HERE STARTS THE APPLICATION SECTION *) 

(* First we define the datatypes with a number of
   elementary axioms about them *)

Section BOOL.
Variable b:bool.
Parameter andb,orb:bool->bool->bool.
Parameter notb:bool->bool.
Axiom andb1:b=(andb true b).
Axiom andb2:false=(andb false b).
Axiom orb1:true=(orb true b).
Axiom orb2:b=(orb false b).
Axiom notb1:false=(notb true).
Axiom notb2:true=(notb false).
End BOOL.

Section BIT.
Parameter bit : Set.
Parameter e0,e1:bit. 
Parameter eqb:bit->bit->bool.
Parameter toggle:bit->bit.
Variable b:bit.
Axiom Toggle1:e1=(toggle e0).
Axiom Toggle2:e0=(toggle e1).
Axiom bit1:true=(eqb b b).
Axiom bit2:false=(eqb b (toggle b)). 
Axiom bit3:false=(eqb (toggle b) b).
End BIT.

Section DATA.
Parameter D : Set.
Parameter eqD : D->D->bool.
Parameter ifD : bool->D->D->D.
Variable d,e:D.
Axiom eqD5 : d=(ifD true d e).
Axiom eqD6 : e=(ifD false d e).
Axiom eqD7 : true=(eqD d d).
Axiom eqD8 : (ifD(eqD d e) d e)=e.

Axiom EQDi : (~(EQ D one)).
End DATA.

Goal (d,e:D)((d=e)->(true=(eqD d e))).
Intros.
Elim H.
Apply eqD7.

Save eqD_elim.

Goal (d,e:D)((true=(eqD d e)) -> (d=e)).

Intros.
Elim (eqD8 d e).
Elim H.
Elim eqD5.
Apply refl_equal.
Save eqD_intro.

Goal (d,e:D)((false=(eqD d e)) -> ~(d=e)).

Intros.
Red ; Intro.
Cut (P:bool->Prop)(P true)->(P false) ; Intro L.
Apply (L ([P:bool](<Prop>Case P of True False end))).
Exact I.
Intros.
ElimType (eqD d e)=false.
Elim H0.
Elim eqD7.
Assumption.
Apply sym_equal.
Assumption.
Save eqD_intro'.

Section FRAME1.
Parameter frame : Set.
Variable b,b1,b2:bit.
Variable d,e:frame.
Parameter tuple : bit -> frame. 
Parameter sce:frame.
Parameter eqf : frame->frame->bool.
Parameter iff : bool->frame->frame->frame.
Axiom eqf1:true=(eqf sce sce).
Axiom eqf2:false=(eqf sce (tuple b)).
Axiom eqf3:false=(eqf (tuple b) sce).
Axiom eqf4:(eqb b1 b2)=(eqf (tuple b1) (tuple b2)).

Axiom eqf5 : d=(iff true d e).
Axiom eqf6 : e=(iff false d e).
Axiom eqf7 : true=(eqf d d).
Axiom eqf8 : (iff(eqf d e) d e)=e.

Axiom EQfi:(~(EQ frame one)).
Axiom EQfD:(~(EQ frame D)).
End FRAME1.

 
Goal (d,e:frame)((d=e)->(true=(eqf d e))).
Intros.
Elim H.
Apply eqf7.
 
Save eqf_elim.
 
Goal (d,e:frame)((true=(eqf d e)) -> (d=e)).
 
Intros.
Elim (eqf8 d e).
Elim H.
Elim eqf5.
Apply refl_equal.
Save eqf_intro.
 
Goal (d,e:frame)((false=(eqf d e)) -> ~(d=e)).
 
Intros.
Red ; Intro.
Cut (P:bool->Prop)(P true)->(P false) ; Intro L.
Apply (L ([P:bool](<Prop>Case P of True False end))).
Exact I.
Intros.
ElimType (eqf d e)=false.
Elim H0.
Elim eqf7.
Assumption.
Apply sym_equal.
Assumption.
Save eqf_intro'.
 

Section FRAME2.
Parameter Frame : Set.
Variable b,b1,b2:bit.
Variable d,d1,d2:D.

Variable e,e':Frame.
Parameter Tuple : bit->D->Frame. 
Parameter lce:Frame.
Parameter eqF : Frame->Frame->bool.
Parameter ifF : bool->Frame->Frame->Frame. 
Axiom eqF1:true=(eqF lce lce).
Axiom eqF2:false=(eqF lce (Tuple b d)).
Axiom eqF3:false=(eqF (Tuple b d) lce).
Axiom eqF4:(andb (eqb b1 b2) (eqD d1 d2))=
              (eqF (Tuple b1 d1) (Tuple b2 d2)).
 
Axiom eqF5 : e'=(ifF true e' e).
Axiom eqF6 : e=(ifF false e' e). 
Axiom eqF7 : true=(eqF e e). 
Axiom eqF8 : (ifF(eqF e e') e e')=e'. 
 
Axiom EQFi:(~(EQ Frame one)).
Axiom EQFD:(~(EQ Frame D)).
Axiom EQFf:(~(EQ Frame frame)).
End FRAME2.


Hint EQFf (*EQfF*).


Goal (d,e:Frame)((d=e)->(true=(eqF d e))).
Intros.
Elim H.
Apply eqF7.

Save eqF_elim.

Goal (d,e:Frame)((true=(eqF d e)) -> (d=e)).

Intros.
Elim (eqF8 d e).
Elim H.
Elim eqF5.
Apply refl_equal.
Save eqF_intro.

Goal (d,e:Frame)((false=(eqF d e)) -> ~(d=e)).

Intros.
Red ; Intro.
Cut (P:bool->Prop)(P true)->(P false) ; Intro L.
Apply (L ([P:bool](<Prop>Case P of True False end))).
Exact I.
Intros.
ElimType (eqF d e)=false.
Elim H0.
Elim eqF7.
Assumption.
Apply sym_equal.
Assumption.
Save eqF_intro'.
 
(* Below the processes used in the alternating bit protocol
   are defined *)

Parameter K:one->proc.
Parameter L:one->proc.
Parameter S:one->proc.
Parameter Sn:bit -> proc.
Parameter Sn_d:D->bit->proc.
Parameter Tn_d:D->bit->proc.
Parameter R:one->proc.
Parameter Rn:bit->proc.

Section PROC.
Variable b:bit.
Variable j:one.
Variable d:D.

Axiom ChanK:
   
    (sum Frame ([x:Frame] 
     (seq (ia Frame r2 x)
      (seq 
       (alt 
        (seq (ia one int i) (ia Frame s3 x)) 
        (seq (ia one int i) (ia Frame s3 lce)))  
       (K i)))))=(K j).

Axiom ChanL:
   
    (sum frame ([n:frame] 
     (seq (ia frame r5 n)
      (seq 
       (alt 
        (seq (ia one int i) (ia frame s6 n)) 
        (seq (ia one int i) (ia frame s6 sce)))  
       (L i)))))=(L j).

Axiom ProcS:
    (seq ( Sn e0) (seq (Sn e1) (S i)))=(S j).

Axiom ProcSn:
    (sum D ([d:D] (seq (ia D r1 d) (Sn_d d b))))=(Sn b).

Axiom ProcSn_d:
    (seq (ia Frame s2 (Tuple b d)) (Tn_d d b))=(Sn_d d b).

Axiom ProcTn_d:
    
       (alt 
        (seq 
          (alt 
            (ia frame r6 (tuple(toggle b)))
            (ia frame r6 sce)) 
          (Sn_d d b)) 
        (ia frame r6 (tuple b)))=(Tn_d d b).

Axiom ProcR:
    (seq (Rn e1) (seq (Rn e0) (R i)))=(R j).

Axiom ProcRn:
    
       (alt 
        (seq 
         (alt 
          (sum D ([d:D] (ia Frame r3 (Tuple b d)))) 
          (ia Frame r3 lce)) 
         (seq (ia frame s5 (tuple b)) (Rn b)))
        (sum D ([d:D]
         (seq 
          (ia Frame r3 (Tuple (toggle b) d)) 
          (seq 
           (ia D s4 d) 
           (ia frame s5 (tuple (toggle b))))))))=(Rn b).
End PROC.

Definition H:=(ehcons r2 
               (ehcons r3
                (ehcons r5
                 (ehcons r6
                  (ehcons s2
                   (ehcons s3
                    (ehcons s5
                     (ehcons s6 ehnil)))))))).

Definition ABP := (enc H (mer (S i) (mer (K i) (mer (L i) (R i))))).
Definition X := (enc H (mer (S i) (mer (K i) (mer (L i) (R i))))).

Definition X1 := ([d:D](enc H (mer 
                        (seq (Sn_d d e0) (seq (Sn e1) (S i)))
                        (mer (K i) (mer (L i) (R i)))))).

Definition X2 := ([d:D](enc H (mer 
                        (seq (Tn_d d e0) (seq (Sn e1) (S i)))
                        (mer (K i) 
                         (mer 
                          (L i) 
                           (seq 
                            (ia frame s5 (tuple e0)) 
                            (seq (Rn e0) (R i)))))))).

Definition Y := (enc H (mer  
                        (seq (Sn e1) (S i))
                        (mer (K i) 
                         (mer 
                          (L i) 
                           (seq (Rn e0) (R i)))))).

Definition Y1 := ([d:D](
                    (enc H (mer  
                        (seq (Sn_d d e1) (S i))
                        (mer (K i) 
                         (mer 
                          (L i) 
                           (seq (Rn e0) (R i)))))))).

Definition Y2 := ([d:D](
                    (enc H (mer      
                        (seq (Tn_d d e1) (S i)) 
                        (mer (K i)    
                         (mer     
                          (L i)  
                           (seq (ia frame s5 (tuple e1)) (R i)))))))).


(* Here the section on equality of actions starts *)

Goal ~(r1=r2).
Discriminate.
Save neqr1r2.

Goal ~(r1=r3).
Discriminate.
Save neqr1r3. 

Goal ~(r1=r5).
Discriminate.
Save neqr1r5. 

Goal ~(r1=r6).
Discriminate.
Save neqr1r6. 

Goal ~(r1=s2).
Discriminate.
Save neqr1s2.

Goal ~(r1=s3).
Discriminate.
Save neqr1s3.

Goal ~(r1=s4).
Discriminate.
Save neqr1s4.

Goal ~(r1=s5).
Discriminate.
Save neqr1s5.

Goal ~(r1=s6).
Discriminate.
Save neqr1s6.

Goal ~(r1=c2).
Discriminate.
Save neqr1c2.

Goal ~(r1=c3).
Discriminate.
Save neqr1c3.

Goal ~(r1=c5).
Discriminate.
Save neqr1c5.

Goal ~(r1=c6).
Discriminate.
Save neqr1c6.

Goal ~(r1=int).
Discriminate.
Save neqr1int.

Goal ~(r1=tau).
Discriminate.
Save neqr1tau.

Hint neqr1r2 neqr1r3 neqr1r5 neqr1r6 neqr1s2 neqr1s3 neqr1s4 neqr1s5
neqr1s6 neqr1c2 neqr1c3 neqr1c5 neqr1c6 neqr1int neqr1tau.

Goal ~(r2=r1).
Discriminate.
Save neqr2r1.

Goal ~(r2=r3).
Discriminate.
Save neqr2r3. 

Goal ~(r2=r5).
Discriminate.
Save neqr2r5. 

Goal ~(r2=r6).
Discriminate.
Save neqr2r6. 

Goal ~(r2=s2).
Discriminate.
Save neqr2s2.

Goal ~(r2=s3).
Discriminate.
Save neqr2s3.

Goal ~(r2=s4).
Discriminate.
Save neqr2s4.

Goal ~(r2=s5).
Discriminate.
Save neqr2s5.

Goal ~(r2=s6).
Discriminate.
Save neqr2s6.

Goal ~(r2=c2).
Discriminate.
Save neqr2c2.

Goal ~(r2=c3).
Discriminate.
Save neqr2c3.

Goal ~(r2=c5).
Discriminate.
Save neqr2c5.

Goal ~(r2=c6).
Discriminate.
Save neqr2c6.

Goal ~(r2=int).
Discriminate.
Save neqr2int.

Goal ~(r2=tau).
Discriminate.
Save neqr2tau.

Hint neqr2r1 neqr2r3 neqr2r5 neqr2r6 neqr2s2 neqr2s3 neqr2s4 neqr2s5
neqr2s6 neqr2c2 neqr2c3 neqr2c5 neqr2c6 neqr2int neqr2tau.


Goal ~(r3=r1).
Discriminate.
Save neqr3r1.

Goal ~(r3=r2).
Discriminate.
Save neqr3r2.

Goal ~(r3=r5).
Discriminate.
Save neqr3r5. 

Goal ~(r3=r6).
Discriminate.
Save neqr3r6. 

Goal ~(r3=s2).
Discriminate.
Save neqr3s2.

Goal ~(r3=s3).
Discriminate.
Save neqr3s3.

Goal ~(r3=s4).
Discriminate.
Save neqr3s4.

Goal ~(r3=s5).
Discriminate.
Save neqr3s5.

Goal ~(r3=s6).
Discriminate.
Save neqr3s6.

Goal ~(r3=c2).
Discriminate.
Save neqr3c2.

Goal ~(r3=c3).
Discriminate.
Save neqr3c3.

Goal ~(r3=c5).
Discriminate.
Save neqr3c5.

Goal ~(r3=c6).
Discriminate.
Save neqr3c6.

Goal ~(r3=int).
Discriminate.
Save neqr3int.

Goal ~(r3=tau).
Discriminate.
Save neqr3tau.

Hint neqr3r2 neqr3r1 neqr3r5 neqr3r6 neqr3s2 neqr3s3 neqr3s4 neqr3s5
neqr3s6 neqr3c2 neqr3c3 neqr3c5 neqr3c6 neqr3int neqr3tau.


Goal ~(r5=r1).
Discriminate.
Save neqr5r1.

Goal ~(r5=r2).
Discriminate.
Save neqr5r2.

Goal ~(r5=r3).
Discriminate.
Save neqr5r3. 

Goal ~(r5=r6).
Discriminate.
Save neqr5r6. 

Goal ~(r5=s2).
Discriminate.
Save neqr5s2.

Goal ~(r5=s3).
Discriminate.
Save neqr5s3.

Goal ~(r5=s4).
Discriminate.
Save neqr5s4.

Goal ~(r5=s5).
Discriminate.
Save neqr5s5.

Goal ~(r5=s6).
Discriminate.
Save neqr5s6.

Goal ~(r5=c2).
Discriminate.
Save neqr5c2.

Goal ~(r5=c3).
Discriminate.
Save neqr5c3.

Goal ~(r5=c5).
Discriminate.
Save neqr5c5.

Goal ~(r5=c6).
Discriminate.
Save neqr5c6.

Goal ~(r5=int).
Discriminate.
Save neqr5int.

Goal ~(r5=tau).
Discriminate.
Save neqr5tau.

Hint neqr5r2 neqr5r3 neqr5r1 neqr5r6 neqr5s2 neqr5s3 neqr5s4 neqr5s5
neqr5s6 neqr5c2 neqr5c3 neqr5c5 neqr5c6 neqr5int neqr5tau.


Goal ~(r6=r1).
Discriminate.
Save neqr6r1.

Goal ~(r6=r2).
Discriminate.
Save neqr6r2.

Goal ~(r6=r3).
Discriminate.
Save neqr6r3. 

Goal ~(r6=r5).
Discriminate.
Save neqr6r5. 

Goal ~(r6=s2).
Discriminate.
Save neqr6s2.

Goal ~(r6=s3).
Discriminate.
Save neqr6s3.

Goal ~(r6=s4).
Discriminate.
Save neqr6s4.

Goal ~(r6=s5).
Discriminate.
Save neqr6s5.

Goal ~(r6=s6).
Discriminate.
Save neqr6s6.

Goal ~(r6=c2).
Discriminate.
Save neqr6c2.

Goal ~(r6=c3).
Discriminate.
Save neqr6c3.

Goal ~(r6=c5).
Discriminate.
Save neqr6c5.

Goal ~(r6=c6).
Discriminate.
Save neqr6c6.

Goal ~(r6=int).
Discriminate.
Save neqr6int.

Goal ~(r6=tau).
Discriminate.
Save neqr6tau.

Hint neqr6r2 neqr6r3 neqr1r5 neqr6r1 neqr6s2 neqr6s3 neqr6s4 neqr6s5
neqr6s6 neqr6c2 neqr6c3 neqr6c5 neqr6c6 neqr6int neqr6tau.


Goal ~(s2=r1).
Discriminate.
Save neqs2r1.

Goal ~(s2=r2).
Discriminate.
Save neqs2r2.

Goal ~(s2=r3).
Discriminate.
Save neqs2r3. 

Goal ~(s2=r5).
Discriminate.
Save neqs2r5. 

Goal ~(s2=r6).
Discriminate.
Save neqs2r6. 

Goal ~(s2=s3).
Discriminate.
Save neqs2s3.

Goal ~(s2=s4).
Discriminate.
Save neqs2s4.

Goal ~(s2=s5).
Discriminate.
Save neqs2s5.

Goal ~(s2=s6).
Discriminate.
Save neqs2s6.

Goal ~(s2=c2).
Discriminate.
Save neqs2c2.

Goal ~(s2=c3).
Discriminate.
Save neqs2c3.

Goal ~(s2=c5).
Discriminate.
Save neqs2c5.

Goal ~(s2=c6).
Discriminate.
Save neqs2c6.

Goal ~(s2=int).
Discriminate.
Save neqs2int.

Goal ~(s2=tau).
Discriminate.
Save neqs2tau.

Hint neqs2r2 neqs2r3 neqs2r5 neqs2r6 neqs2r1 neqs2s3 neqs2s4 neqs2s5
neqs2s6 neqs2c2 neqs2c3 neqs2c5 neqs2c6 neqs2int neqs2tau.

Goal ~(s3=r1).
Discriminate.
Save neqs3r1.

Goal ~(s3=r2).
Discriminate.
Save neqs3r2.

Goal ~(s3=r3).
Discriminate.
Save neqs3r3. 

Goal ~(s3=r5).
Discriminate.
Save neqs3r5. 

Goal ~(s3=r6).
Discriminate.
Save neqs3r6. 

Goal ~(s3=s2).
Discriminate.
Save neqs3s2.

Goal ~(s3=s4).
Discriminate.
Save neqs3s4.

Goal ~(s3=s5).
Discriminate.
Save neqs3s5.

Goal ~(s3=s6).
Discriminate.
Save neqs3s6.

Goal ~(s3=c2).
Discriminate.
Save neqs3c2.

Goal ~(s3=c3).
Discriminate.
Save neqs3c3.

Goal ~(s3=c5).
Discriminate.
Save neqs3c5.

Goal ~(s3=c6).
Discriminate.
Save neqs3c6.

Goal ~(s3=int).
Discriminate.
Save neqs3int.

Goal ~(s3=tau).
Discriminate.
Save neqs3tau.

Hint neqs3r2 neqs3r3 neqs3r5 neqs3r6 neqs3s2 (*neqs3s1*) neqs3s4 neqs3s5
neqs3s6 neqs3c2 neqs3c3 neqs3c5 neqs3c6 neqs3int neqs3tau.


Goal ~(s4=r1).
Discriminate.
Save neqs4r1.

Goal ~(s4=r2).
Discriminate.
Save neqs4r2.

Goal ~(s4=r3).
Discriminate.
Save neqs4r3. 

Goal ~(s4=r5).
Discriminate.
Save neqs4r5. 

Goal ~(s4=r6).
Discriminate.
Save neqs4r6. 

Goal ~(s4=s2).
Discriminate.
Save neqs4s2.

Goal ~(s4=s3).
Discriminate.
Save neqs4s3.

Goal ~(s4=s5).
Discriminate.
Save neqs4s5.

Goal ~(s4=s6).
Discriminate.
Save neqs4s6.

Goal ~(s4=c2).
Discriminate.
Save neqs4c2.

Goal ~(s4=c3).
Discriminate.
Save neqs4c3.

Goal ~(s4=c5).
Discriminate.
Save neqs4c5.

Goal ~(s4=c6).
Discriminate.
Save neqs4c6.

Goal ~(s4=int).
Discriminate.
Save neqs4int.

Goal ~(s4=tau).
Discriminate.
Save neqs4tau.

Hint neqs4r2 neqs4r3 neqs4r5 neqs4r6 neqs4s2 neqs4s3 (*neqs4s1*) neqs4s5
neqs4s6 neqs4c2 neqs4c3 neqs4c5 neqs4c6 neqs4int neqs4tau.


Goal ~(s5=r1).
Discriminate.
Save neqs5r1.

Goal ~(s5=r2).
Discriminate.
Save neqs5r2.

Goal ~(s5=r3).
Discriminate.
Save neqs5r3. 

Goal ~(s5=r5).
Discriminate.
Save neqs5r5. 

Goal ~(s5=r6).
Discriminate.
Save neqs5r6. 

Goal ~(s5=s2).
Discriminate.
Save neqs5s2.

Goal ~(s5=s3).
Discriminate.
Save neqs5s3.

Goal ~(s5=s4).
Discriminate.
Save neqs5s4.

Goal ~(s5=s6).
Discriminate.
Save neqs5s6.

Goal ~(s5=c2).
Discriminate.
Save neqs5c2.

Goal ~(s5=c3).
Discriminate.
Save neqs5c3.

Goal ~(s5=c5).
Discriminate.
Save neqs5c5.

Goal ~(s5=c6).
Discriminate.
Save neqs5c6.

Goal ~(s5=int).
Discriminate.
Save neqs5int.

Goal ~(s5=tau).
Discriminate.
Save neqs5tau.

Hint neqs5r2 neqs5r3 neqs5r5 neqs5r6 neqs5s2 neqs5s3 neqs5s4 neqs5r1
neqs5s6 neqr1c2 neqs5c3 neqs5c5 neqs5c6 neqs5int neqs5tau.

Goal ~(s6=r1).
Discriminate.
Save neqs6r1.

Goal ~(s6=r2).
Discriminate.
Save neqs6r2.

Goal ~(s6=r3).
Discriminate.
Save neqs6r3. 

Goal ~(s6=r5).
Discriminate.
Save neqs6r5. 

Goal ~(s6=r6).
Discriminate.
Save neqs6r6. 

Goal ~(s6=s2).
Discriminate.
Save neqs6s2.

Goal ~(s6=s3).
Discriminate.
Save neqs6s3.

Goal ~(s6=s4).
Discriminate.
Save neqs6s4.

Goal ~(s6=s5).
Discriminate.
Save neqs6s5.

Goal ~(s6=c2).
Discriminate.
Save neqs6c2.

Goal ~(s6=c3).
Discriminate.
Save neqs6c3.

Goal ~(s6=c5).
Discriminate.
Save neqs6c5.

Goal ~(s6=c6).
Discriminate.
Save neqs6c6.

Goal ~(s6=int).
Discriminate.
Save neqs6int.

Goal ~(s6=tau).
Discriminate.
Save neqs6tau.

Hint neqs6r2 neqs6r3 neqs6r5 neqs6r6 neqs6s2 neqs6s3 neqs6s4 neqs6s5
neqs6r1 neqs6c2 neqs6c3 neqs6c5 neqs6c6 neqs6int neqs6tau.


Goal ~(c2=r1).
Discriminate.
Save neqc2r1.

Goal ~(c2=r2).
Discriminate.
Save neqc2r2.

Goal ~(c2=r3).
Discriminate.
Save neqc2r3. 

Goal ~(c2=r5).
Discriminate.
Save neqc2r5. 

Goal ~(c2=r6).
Discriminate.
Save neqc2r6. 

Goal ~(c2=s2).
Discriminate.
Save neqc2s2.

Goal ~(c2=s3).
Discriminate.
Save neqc2s3.

Goal ~(c2=s4).
Discriminate.
Save neqc2s4.

Goal ~(c2=s5).
Discriminate.
Save neqc2s5.

Goal ~(c2=s6).
Discriminate.
Save neqc2s6.

Goal ~(c2=c3).
Discriminate.
Save neqc2c3.

Goal ~(c2=c5).
Discriminate.
Save neqc2c5.

Goal ~(c2=c6).
Discriminate.
Save neqc1c6.

Goal ~(c2=int).
Discriminate.
Save neqc2int.

Goal ~(c2=tau).
Discriminate.
Save neqc2tau.

Hint neqc2r2 neqc2r3 neqc2r5 neqc2r6 neqc2s2 neqc2s3 neqc2s4 neqc2s5
neqc2s6 (*neqc2s1*) neqc2c3 neqc2c5 (*neqc2c6*) neqc2int neqc2tau.


Goal ~(c3=r1).
Discriminate.
Save neqc3r1.

Goal ~(c3=r2).
Discriminate.
Save neqc3r2.

Goal ~(c3=r3).
Discriminate.
Save neqc3r3. 

Goal ~(c3=r5).
Discriminate.
Save neqc3r5. 

Goal ~(c3=r6).
Discriminate.
Save neqc3r6. 

Goal ~(c3=s2).
Discriminate.
Save neqc3s2.

Goal ~(c3=s3).
Discriminate.
Save neqc3s3.

Goal ~(c3=s4).
Discriminate.
Save neqc3s4.

Goal ~(c3=s5).
Discriminate.
Save neqc3s5.

Goal ~(c3=s6).
Discriminate.
Save neqc3s6.

Goal ~(c3=c2).
Discriminate.
Save neqc3c2.

Goal ~(c3=c5).
Discriminate.
Save neqc3c5.

Goal ~(c3=c6).
Discriminate.
Save neqc3c6.

Goal ~(c3=int).
Discriminate.
Save neqc3int.

Goal ~(c3=tau).
Discriminate.
Save neqc3tau.

Hint neqc3r2 neqc3r3 neqc3r5 neqc3r6 neqc3s2 neqc3s3 neqc3s4 neqc3s5
neqc3s6 neqc3c2 neqc3r1 neqc3c5 neqc3c6 neqc3int neqc3tau.


Goal ~(c5=r1).
Discriminate.
Save neqc5r1.

Goal ~(c5=r2).
Discriminate.
Save neqc5r2.

Goal ~(c5=r3).
Discriminate.
Save neqc5r3. 

Goal ~(c5=r5).
Discriminate.
Save neqc5r5. 

Goal ~(c5=r6).
Discriminate.
Save neqc5r6. 

Goal ~(c5=s2).
Discriminate.
Save neqc5s2.

Goal ~(c5=s3).
Discriminate.
Save neqc5s3.

Goal ~(c5=s4).
Discriminate.
Save neqc5s4.

Goal ~(c5=s5).
Discriminate.
Save neqc5s5.

Goal ~(c5=s6).
Discriminate.
Save neqc5s6.

Goal ~(c5=c2).
Discriminate.
Save neqc5c2.

Goal ~(c5=c3).
Discriminate.
Save neqc5c3.

Goal ~(c5=c6).
Discriminate.
Save neqc5c6.

Goal ~(c5=int).
Discriminate.
Save neqc5int.

Goal ~(c5=tau).
Discriminate.
Save neqc5tau.

Hint neqc5r2 neqc5r3 neqc5r5 neqc5r6 neqc5s2 neqc5s3 neqc5s4 neqc5s5
neqc5s6 neqc5c2 neqc5c3 neqc5r1 neqc5c6 neqc5int neqc5tau.


Goal ~(c6=r1).
Discriminate.
Save neqc6r1.

Goal ~(c6=r2).
Discriminate.
Save neqc6r2.

Goal ~(c6=r3).
Discriminate.
Save neqc6r3. 

Goal ~(c6=r5).
Discriminate.
Save neqc6r5. 

Goal ~(c6=r6).
Discriminate.
Save neqc6r6. 

Goal ~(c6=s2).
Discriminate.
Save neqc6s2.

Goal ~(c6=s3).
Discriminate.
Save neqc6s3.

Goal ~(c6=s4).
Discriminate.
Save neqc6s4.

Goal ~(c6=s5).
Discriminate.
Save neqc6s5.

Goal ~(c6=s6).
Discriminate.
Save neqc6s6.

Goal ~(c6=c2).
Discriminate.
Save neqc6c2.

Goal ~(c6=c3).
Discriminate.
Save neqc6c3.

Goal ~(c6=c5).
Discriminate.
Save neqc6c5.

Goal ~(c6=int).
Discriminate.
Save neqc6int.

Goal ~(c6=tau).
Discriminate.
Save neqc6tau.

Hint neqc6r2 neqc6r3 neqc6r5 neqc6r6 neqc6s2 neqc6s3 neqc6s4 neqc6s5
neqc6s6 neqc6c2 neqc6c3 neqc6c5 neqc6r1 neqc6int neqc6tau.


Goal ~(int=r1).
Discriminate.
Save neqintr1.

Goal ~(int=r2).
Discriminate.
Save neqintr2.

Goal ~(int=r3).
Discriminate.
Save neqintr3. 

Goal ~(int=r5).
Discriminate.
Save neqintr5. 

Goal ~(int=r6).
Discriminate.
Save neqintr6. 

Goal ~(int=s2).
Discriminate.
Save neqints2.

Goal ~(int=s3).
Discriminate.
Save neqints3.

Goal ~(int=s4).
Discriminate.
Save neqints4.

Goal ~(int=s5).
Discriminate.
Save neqints5.

Goal ~(int=s6).
Discriminate.
Save neqints6.

Goal ~(int=c2).
Discriminate.
Save neqintc2.

Goal ~(int=c3).
Discriminate.
Save neqintc3.

Goal ~(int=c5).
Discriminate.
Save neqintc5.

Goal ~(int=c6).
Discriminate.
Save neqintc6.

Goal ~(int=tau).
Discriminate.
Save neqinttau.

Hint neqintr2 neqintr3 neqintr5 neqintr6 neqints2 neqints3 neqints4 neqints5
neqints6 neqintc2 neqintc3 neqintc5 neqintc6 neqintr1 neqinttau.


Goal ~(tau=r1).
Discriminate.
Save neqtaur1.

Goal ~(tau=r2).
Discriminate.
Save neqtaur2.

Goal ~(tau=r3).
Discriminate.
Save neqtaur3. 

Goal ~(tau=r5).
Discriminate.
Save neqtaur5. 

Goal ~(tau=r6).
Discriminate.
Save neqtaur6. 

Goal ~(tau=s2).
Discriminate.
Save neqtaus2.

Goal ~(tau=s3).
Discriminate.
Save neqtaus3.

Goal ~(tau=s4).
Discriminate.
Save neqtaus4.

Goal ~(tau=s5).
Discriminate.
Save neqtaus5.

Goal ~(tau=s6).
Discriminate.
Save neqtaus6.

Goal ~(tau=c2).
Discriminate.
Save neqtauc2.

Goal ~(tau=c3).
Discriminate.
Save neqtauc3.

Goal ~(tau=c5).
Discriminate.
Save neqtauc5.

Goal ~(tau=c6).
Discriminate.
Save neqtauc6.

Goal ~(tau=int).
Discriminate.
Save neqtauint.

Hint neqtaur2 neqtaur3 neqtaur5 neqtaur6 neqtaus2 neqtaus3 neqtaus4 neqtaus5
neqtaus6 neqtauc2 neqtauc3 neqtauc5 neqtauc6 neqtauint neqtaur1.

(*Here the section on equality of actions ends *)

(*Here the section on actions in the set H starts *)

Goal (a:act)(a=r2) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto.
Save HLemmar2.

Goal (a:act)(a=r3) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto.
Save HLemmar3.

Goal (a:act)(a=r5) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto.
Save HLemmar5.

Goal (a:act)(a=r6) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto.
Save HLemmar6.

Goal (a:act)(a=s2) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto 10.
Save HLemmas2.

Goal (a:act)(a=s3) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto 10.
Save HLemmas3.

Goal (a:act)(a=s5) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto 10.
Save HLemmas5.

Goal (a:act)(a=s6) ->(In_ehlist a H).
Intros.
Unfold In_ehlist H.
Auto 10.
Save HLemmas6.


Goal (a:act)(~a=r2)->(~a=r3)->(~a=r5)->(~a=r6)->
  (~a=s2)-> (~a=s3)->(~a=s5)->(~a=s6)->~(In_ehlist a H).
Intros.
Red . Unfold In_ehlist H. 
Intro I1. Elim I1. Assumption.
Intro I2. Elim I2. Assumption.
Intro I3. Elim I3. Assumption.
Intro I4. Elim I4. Assumption.
Intro I5. Elim I5. Assumption.
Intro I6. Elim I6. Assumption.
Intro I7. Elim I7. Assumption.
Intro I8. Elim I8. Assumption.
Intro . Assumption.
Save HLemma.

Hint HLemmar2 HLemmar3 HLemmar5 HLemmar6 HLemmas2 HLemmas3 HLemmas5
HLemmas6 HLemma.

Goal ~(In_ehlist r1 H).
Apply HLemma; Auto.
Save Inr1H.

Goal (In_ehlist r2 H).
Auto.
Save Inr2H.

Goal (In_ehlist r3 H).
Auto.
Save Inr3H.

Goal (In_ehlist r5 H).
Auto.
Save Inr5H.

Goal (In_ehlist r6 H).
Auto.
Save Inr6H.

Goal (In_ehlist s2 H).
Auto.
Save Ins2H.

Goal (In_ehlist s3 H).
Auto.
Save Ins3H.

Goal ~(In_ehlist s4 H).
Apply HLemma; Auto.
Save Ins4H.

Goal (In_ehlist s5 H).
Auto.
Save Ins5H.

Goal (In_ehlist s6 H).
Auto.
Save Ins6H.

Goal ~(In_ehlist int H).
Apply HLemma; Auto.
Save InintH.

Goal ~(In_ehlist c2 H).
Apply HLemma; Auto.
Save Inc2H.

Goal ~(In_ehlist c3 H). 
Apply HLemma; Auto.
Save Inc3H. 

Goal ~(In_ehlist c5 H). 
Apply HLemma; Auto.
Save Inc5H. 

Goal ~(In_ehlist c6 H). 
Apply HLemma; Auto.
Save Inc6H. 

Definition I':=(ehcons c2 (ehcons c3 (ehcons c5 (ehcons c6 ehnil)))).
Definition I'':=(ehcons int ehnil).

Goal (In_ehlist c2 I').
Unfold In_ehlist I'. 
Left; Apply refl_equal. 
Save Inc2I. 

Goal (In_ehlist c3 I').
Unfold In_ehlist I'. 
Right; Left; Apply refl_equal. 
Save Inc3I. 

Goal (In_ehlist c5 I').
Unfold In_ehlist I'. 
Right; Right; Left; Apply refl_equal. 
Save Inc5I. 

Goal (In_ehlist c6 I').
Unfold In_ehlist I'.
Right; Right; Right; Left; Apply refl_equal.
Save Inc6I.

Goal ~(In_ehlist int I').
Red . Unfold In_ehlist I'.
Intro; Elim H0 . Intro . Apply neqintc2 . Assumption.
Intro; Elim H1 . Intro . Apply neqintc3 . Assumption.
Intro; Elim H2 . Intro . Apply neqintc5 . Assumption.
Intro; Elim H3 . Intro . Apply neqintc6 . Assumption.
Intro; Assumption.
Save InintI.

Goal ~(In_ehlist s4 I').
Red . Unfold In_ehlist I'.
Intro; Elim H0 . Intro . Apply neqs4c2 . Assumption.
Intro; Elim H1 . Intro . Apply neqs4c3 . Assumption.
Intro; Elim H2 . Intro . Apply neqs4c5 . Assumption.
Intro; Elim H3 . Intro . Apply neqs4c6 . Assumption.
Intro; Assumption.
Save Ins4I.

Goal ~(In_ehlist r1 I').
Red . Unfold In_ehlist I'.
Intro; Elim H0 . Intro . Apply neqr1c2 . Assumption.
Intro; Elim H1 . Intro . Apply neqr1c3 . Assumption.
Intro; Elim H2 . Intro . Apply neqr1c5 . Assumption.
Intro; Elim H3 . Intro . Apply neqr1c6 . Assumption.
Intro; Assumption.
Save Inr1I.

Goal (In_ehlist int I'').
Unfold In_ehlist I''.
Left. Apply refl_equal.
Save InintI''.


Goal ~(In_ehlist s4 I'').
Red . Unfold In_ehlist I''.
Intro; Elim H0 . Intro . Apply neqints4 . 
Apply sym_equal. Assumption.
Intro . Assumption.
Save Ins4I''.

Goal ~(In_ehlist r1 I'').
Red; Unfold In_ehlist I''.
Intro; Elim H0 . Intro . Apply neqintr1 . 
Apply sym_equal. Assumption.
Intro; Assumption.
Save Inr1I''.

Goal ~(In_ehlist tau I'').
Red; Unfold In_ehlist I''.
Intro; Elim H0 . Intro . Apply neqinttau . 
Apply sym_equal. Assumption.
Intro; Assumption.
Save IntauI''.


(*  End of basic axioms about the sets H and I   *)
