(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              abstraction.ml                              *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "vectops";;
#open "names";;
#open "more_util";;
#open "generic";;
#open "term";;
#open "libobject";;
#open "reintern0";;
#open "reintern1";;

let cache_abstraction = function
    (sp,ao) ->
    nametab__push (basename sp) (coerce_path ao.abs_kind sp)
(*  | _ -> invalid_arg "abstraction__cache_abstraction" *)
;;

let reintern_abstraction ao =
    {abs_kind = ao.abs_kind;
     abs_arity = ao.abs_arity;
     abs_rhs = ri_constr ao.abs_rhs}
;;

let specification_abstraction x = x;;

let (inAbstraction,outAbstraction) =
    declare_object ("ABSTRACTION",
                    {cache_function = cache_abstraction;
                     reintern_function = reintern_abstraction;
                     specification_function = specification_abstraction});;


let abst_of_path sp =
    match lib__leaf_object_tag (objsp_of sp) with
    "ABSTRACTION" ->
    let ao = outAbstraction(lib__map_leaf (objsp_of sp))
    in (sp,ao)

  | _ -> invalid_arg "abst_of_path called with non-abstraction"
;;

let abst_option_of_path sp =
try SOME(abst_of_path sp)
with Not_found -> NONE
;;

let rec count_dlam = function
    DLAM(_,c) -> 1+(count_dlam c)
  | _ -> 0
;;

let execute_abstraction pk arity rhs =
    if for_all_vect (fun i -> i >= 0) arity &
        closed rhs & global_vars rhs = [] then
        {abs_kind = pk;
         abs_arity = arity;
         abs_rhs = rhs}
    else error "malformed abstraction object"
;;

let is_abstraction sp =
    lib__leaf_object_tag sp = "ABSTRACTION"
;;

let contract_abstraction = function
    DOPN(Abst sp,args) ->
    let (_,ao) = abst_of_path sp
    in if for_all2eq_vect (fun c i ->
                               (count_dlam c) = i) args ao.abs_arity then
        sosub__soexecute(it_vect SAPP ao.abs_rhs args)
       else failwith "contract_abstraction"
  | _ -> invalid_arg "contract_abstraction"
;;

let make_abstraction sp args =
    let (_,ao) = abst_of_path sp
    in if for_all2eq_vect (fun c i ->
                               (count_dlam c) = i) args ao.abs_arity then
        DOPN(Abst sp,args)
       else failwith "make_abstraction"
;;

(* $Id: abstraction.ml,v 1.3 1996/02/12 15:37:21 cpaulin Exp $ *)
