(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              constrtypes.ml                              *)
(****************************************************************************)


#open "std";;
#open "vectops";;
#open "names";;
#open "more_util";;
#open "generic";;
#open "pp";;
#open "stdpp";;
#open "term";;

let index_pred P v = indrec 0
    where rec indrec n =
    if n = vect_length v then failwith "index_pred"
    else if P v.(n) then n
         else indrec (n+1)
;;
    
let mind_type_name_index mib id =
    let mipvect = mib.MINDPACKETS
    in index_pred (fun mip -> id = mip.MINDTYPENAME) mipvect
;;

let mind_nth_type_packet mib n = mib.MINDPACKETS.(n);;

let mind_singl mib = mib.MINDSINGL;;
                

let first_result_i = fun f start -> aux start
where rec aux n = function
    [] -> NONE
  | (h::t) -> try (SOME (f n h))
              with Failure _ | UserError _ -> aux (n+1) t
;;

(* gives the first (f k v.k) which succedds *)

let first_result_vec_i f v = let n = vect_length v in aux 0
where rec aux k = if k = n then failwith "first_result_vec_i"
                  else  try (f k v.(k))
                        with Failure _ | UserError _ -> aux (k+1)
;;

let mind_oper_of_id sp mib id =
   first_result_vec_i
   (fun tyi mip ->
	if id = mip.MINDTYPENAME then (MutInd(sp,tyi),mib.MINDHYPS)
	else first_result_vec_i
	(fun cidx cid -> if id = cid then (MutConstruct((sp,tyi),cidx+1),mib.MINDHYPS)
	                 else failwith "caught")
	     mip.MINDCONSNAMES)
   mib.MINDPACKETS
;;

let specification_of_constant cmap =
    map (fun (pk,cb) ->
             (pk,
              if not cb.CONSTOPAQUE then cb
              else {CONSTKIND = cb.CONSTKIND;
                    CONSTHYPS = cb.CONSTHYPS;
                    CONSTBODY = NONE;
                    CONSTTYPE = cb.CONSTTYPE;
                    CONSTOPAQUE = true;
                    CONSTEVAL = NONE}))
    cmap
;;

let specification_of_minductive x = x;;

let mind_type_finite mib i  = mib.MINDPACKETS.(i).MINDFINITE;;

let is_recursive listind = 
    let rec one_is_rec rvec = 
       exists (function Mrec(i) -> mem i listind 
                            | Imbr(lvec) -> exists_vect one_is_rec lvec
                            | Norec -> false) rvec
    in exists_vect one_is_rec;;


(* $Id: constrtypes.ml,v 1.4 1996/02/12 15:37:30 cpaulin Exp $ *)
