(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             constrtypes.mli                              *)
(****************************************************************************)

#open "std";;
#open "vectops";;
#open "names";;
#open "more_util";;
#open "generic";;
#open "pp";;
#open "stdpp";;
#open "term";;

(****************************************************************************)
(*            The Type of Constructions Constant Bodies                     *)
(****************************************************************************)

type discharge_recipe = {d_expand : section_path list;
                         d_modify : (sorts oper * sorts oper modification) list;
                         d_abstract : identifier list;
                         d_from : section_path}
;;


type recipe =
    COOKED of constr
  | RECIPE of discharge_recipe
;;

type constant_body = {CONSTKIND : path_kind;
                      CONSTHYPS : identifier list * constr list;
                      mutable CONSTBODY : recipe nref option;
                      CONSTTYPE : constr;
                      mutable CONSTOPAQUE : bool;
                      mutable CONSTEVAL : ((int * constr) list * int * bool) option option}
;;

type const_entry == section_path * constant_body;;
type constdecl_body == (path_kind,constant_body) listmap__t;;

value specification_of_constant : constdecl_body -> constdecl_body;;


(****************************************************************************)
(*       The Type of Constructions Mutually Inductive Bodies                *)
(****************************************************************************)

type recarg = Norec | Mrec of int | Imbr of (recarg list) vect;;

(* Modified by E.G 11/10/94.
   A boolean MINDFINITE tels us if the recursive type is inductive (=true) or
   co-inductive  (=false) *)

type mutual_inductive_packet =
    {MINDCONSNAMES : identifier vect;
     MINDTYPENAME : identifier;
     MINDLC : constr;
     MINDSTAMP : name;
     MINDARITY : constr;
     MINDLAMEXP : constr option;
     MINDKD : sorts list;
     MINDKN : sorts list;
     MINDLISTREC : (recarg list) vect;
     MINDFINITE : bool
     }

and mutual_inductive_body =
    {MINDKIND : path_kind;
     MINDNTYPES : int;
     MINDHYPS : constr signature;
     MINDPACKETS : mutual_inductive_packet vect;
     MINDSINGL : constr option;
     MINDNPARAMS : int}
;;

type mutual_ind_entry == section_path * mutual_inductive_body;;
type mutual_inddecl_body == (path_kind,mutual_inductive_body) listmap__t;;

(* Based at zero *)
value mind_type_name_index : mutual_inductive_body -> identifier -> int;;
value mind_nth_type_packet : mutual_inductive_body -> int -> mutual_inductive_packet;;
value mind_singl : mutual_inductive_body -> constr option;;
value mind_oper_of_id : section_path -> mutual_inductive_body -> identifier -> (sorts oper * constr signature);;
value specification_of_minductive : mutual_inddecl_body -> mutual_inddecl_body;;
value mind_type_finite : mutual_inductive_body -> int -> bool;;
value is_recursive : int list -> recarg list vect -> bool;;

(* $Id: constrtypes.mli,v 1.7 1996/02/12 15:37:32 cpaulin Exp $ *)
