(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                nametab.ml                                *)
(****************************************************************************)
#open "std";;
#open "lomsg";;
#open "names";;
#open "more_util";;
#open "pp";;
#open "stdpp";;

let id_tab = (mhs__new (17,NONE) : (identifier,section_path) mhs__t);;

type frozen_t == (identifier,section_path) mhs__frozen_t;;

(*** old version
let sp_of_id kind id =
  let sp = match mhs__map id_tab id with
             h::_ -> h
           | []   -> raise Not_found
  in coerce_path kind sp
;;
***)

let fw_sp_of_id id = 
    let l = mhs__map id_tab id in 
let rec look_for_kind_fw = function
    sp::l -> if kind_of_path sp <> FW then look_for_kind_fw l else sp
  | []    -> raise Not_found in
look_for_kind_fw l;;

let sp_of_id kind id =
  let rec look_for_kind_fw = function
    sp::_ -> coerce_path FW sp
  | []    -> raise Not_found in
  let rec look_for_other_kind = function
    sp::l -> if kind_of_path sp <> FW then coerce_path kind sp
	                              else look_for_other_kind l
  | []    -> raise Not_found in
  let l = mhs__map id_tab id in
  match kind with
    FW -> look_for_kind_fw l
  | _  -> look_for_other_kind l
;;

let push id sp =
    mhs__add id_tab (id,sp)
;;

let print_name_using desired_k sp =
let (_,id,k) = repr_path sp in
    try (if eq(sp,sp_of_id desired_k id) then
             string_of_id id
         else (string_of_id id)^"."^(string_of_kind k))
    with Not_found -> if (SA__intern "?") = (atompart_of_id (basename sp))
                          then string_of_id(basename sp)
                      else 
                          (warning "Undeclared constants in print_name";
                           string_of_path sp)
;;

let print_name sp =
let (_,_,k) = repr_path sp in
    print_name_using k sp;;

let print_name_mind sp mindid =
let (_,id,k) = repr_path sp in
    try (if eq(sp,sp_of_id k id) then
             string_of_id mindid
         else (string_of_id mindid)^"."^(string_of_kind k))
    with Not_found -> if (SA__intern "?") = (atompart_of_id (basename sp))
                          then string_of_id mindid
                      else 
                          (warning "Undeclared constants in print_name";
                           string_of_path_mind sp mindid)
;;

let init () = (mhs__empty id_tab);;

let freeze () = mhs__freeze id_tab;;

let unfreeze fidt =
    (mhs__unfreeze fidt id_tab);;


library__declare_summary "names"
{summary__freeze_function = freeze;
 summary__unfreeze_function = unfreeze;
 summary__init_function = init;
 summary__sync_function = (fun () -> false)}
;;

let rollback f () =
    let fs = freeze()
    in try f ()
       with e -> (unfreeze fs; raise e)
;;

(* $Id: nametab.ml,v 1.4 1996/02/12 15:37:43 cpaulin Exp $ *)
