(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               reintern1.ml                               *)
(****************************************************************************)
#open "system";;
#open "std";;
#open "more_util";;
#open "names";;
#open "generic";;
#open "term";;
#open "constrtypes";;
#open "unparsings";;
#open "lexer";;
#open "ast";;
#open "impuniv";;
#open "reintern0";;

let ri_sign ri_val (idl,vall) = (map ri_id idl,map ri_val vall);;
let ri_dbsign ri_dbval = map (fun (na,val) -> (ri_name na,ri_dbval val));;
let ri_env ri_val ri_dbval (ENVIRON(sign,dbsign)) =
    ENVIRON(ri_sign ri_val sign,
            ri_dbsign ri_dbval dbsign)
;;

let ri_term ri_oper = rirec
    where rec rirec = function
    DOP0 op -> DOP0(ri_oper op)
  | DOP1(op,c) -> DOP1(ri_oper op,rirec c)
  | DOP2(op,c1,c2) -> DOP2(ri_oper op,rirec c1,rirec c2)
  | DOPN(op,cl) -> DOPN(ri_oper op,map_vect rirec cl)
  | DOPL(op,cl) -> DOPL(ri_oper op,map rirec cl)
  | DLAM(na,c) -> DLAM(ri_name na,rirec c)
  | DLAMV(na,cl) -> DLAMV(ri_name na,map_vect rirec cl)
  | VAR id -> VAR (ri_id id)
  | Rel _ as x -> x
;;

let ri_oper ri_sorts oper =
    match oper with    
    Meta _ -> oper
  | XTRA(s,pl) -> XTRA(s,map ri_tok pl)
  | Sort s -> Sort(ri_sorts s)
  | Implicit -> oper
  | Cast -> oper
  | Prod -> oper
  | Lambda -> oper
  | AppL -> oper
  | Const sp -> Const(ri_sp sp)
  | Abst sp -> Abst(ri_sp sp)
  | MutInd(sp,i) -> MutInd(ri_sp sp,i)
  | MutConstruct((sp,i),j) -> MutConstruct((ri_sp sp,i),j)
  | MutCase -> MutCase
  | Fix x -> Fix x
  | CoFix x -> CoFix x
;;

let hash_cons_constr (c:constr) =
    (if !use_hash_cons then hashcons__hcons c else c)
;;

let ri_universe u =
    {u_sp = ri_sp u.u_sp;
     u_num = u.u_num}
;;

let ri_sorts = function 
    Type(cts,u) -> Type (cts,ri_universe u)
   | x          -> x;;

let ri_constr c =
    hash_cons_constr
    ((ri_term (ri_oper ri_sorts)) c);;


let rec ri_relation = function
    Terminal -> Terminal
  | Equiv u -> Equiv(ri_universe u)
  | Greater(b,u,r) -> Greater(b,ri_universe u,ri_relation r)
;;

let ri_arc = function
    Arc(u,r) -> Arc(ri_universe u,ri_relation r)
  | Undef -> Undef
;;

let ri_universes UNIV =
    ri_avm_frozen_t uni_ord ri_universe ri_arc UNIV;;


let ri_modification = function
    NOT_OCCUR -> NOT_OCCUR
  | DO_ABSTRACT(oper,al) -> DO_ABSTRACT(ri_oper ri_sorts oper,al)
  | DO_REPLACE -> DO_REPLACE
;;

let ri_recipe = function
    COOKED c -> COOKED(ri_constr c)
  | RECIPE dr -> RECIPE{d_expand = map ri_sp dr.d_expand;
                        d_modify = map (fun (oper,modif) ->
                                            (ri_oper ri_sorts oper,ri_modification modif))
                                       dr.d_modify;
                        d_abstract = map ri_id dr.d_abstract;
                        d_from = ri_sp dr.d_from}
;;

let ri_eval = fun (l,n,b) -> (map (fun (i,c) -> (i,ri_constr c)) l,n,b);;

let ri_cb cb =
    {CONSTKIND = cb.CONSTKIND;
     CONSTHYPS = ri_sign ri_constr cb.CONSTHYPS;
     CONSTBODY = option_app (ri_nref ri_recipe) cb.CONSTBODY;
     CONSTTYPE = ri_constr cb.CONSTTYPE;
     CONSTOPAQUE = cb.CONSTOPAQUE;
     CONSTEVAL = option_app (option_app ri_eval) cb.CONSTEVAL};;

let ri_mip mip = 
     {MINDSTAMP = ri_name mip.MINDSTAMP;
      MINDCONSNAMES = map_vect ri_id mip.MINDCONSNAMES;
      MINDARITY = ri_constr mip.MINDARITY;
      MINDTYPENAME = ri_id mip.MINDTYPENAME;
      MINDLC = ri_constr mip.MINDLC;
      MINDKD = mip.MINDKD;
      MINDKN = mip.MINDKN;
      MINDLISTREC = mip.MINDLISTREC;
      MINDLAMEXP = option_app ri_constr mip.MINDLAMEXP;
      MINDFINITE = mip.MINDFINITE};;
     
let ri_mib mib =
     {MINDKIND = mib.MINDKIND;
      MINDNPARAMS = mib.MINDNPARAMS;
      MINDHYPS = ri_sign ri_constr mib.MINDHYPS;
      MINDNTYPES = mib.MINDNTYPES;
      MINDSINGL = option_app ri_constr mib.MINDSINGL;
      MINDPACKETS = map_vect ri_mip mib.MINDPACKETS};;



(* $Id: reintern1.ml,v 1.4 1996/02/12 15:37:51 cpaulin Exp $ *)
