(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 term.mli                                 *)
(****************************************************************************)

#open "more_util";;
#open "names";;
#open "generic";;

(****************************************************************************)
(*               The Type of Constructions Terms                            *)
(****************************************************************************)

type contents = Pos | Null;;

(* Coq abstract syntax with deBruijn variables; 'a is the type of sorts *)
(* TODO: Document arities *)

type 'a oper = 

    (* injection: DOP0 *)
    Meta of int                        (* meta-variables (for synthesis) *)
  | XTRA of string * ast__tok list
                  (* an extra slot, for putting in whatever sort of
                      operator we need for whatever sort of application *)
  | Sort of 'a
  | Implicit

    (* injection: DOP2 *)
  | Cast | Prod | Lambda

    (* injection: DOPN *)
  | AppL | Const of section_path | Abst of section_path

  | MutInd of section_path * int
  | MutConstruct of (section_path * int) * int
  | MutCase
  | Fix of int vect * int
  | CoFix of int
;;

type sorts =
    Prop of contents                      (* proposition types *)
  | Type of contents * impuniv__universe
;;

type constr == sorts oper term;;

(* TODO: Obsolete *)
type fterm == constr;;

type 'b assumptions == (constr,'b) env;;

type judgement = {VAL : constr ;
                  TYP : constr;
                  KIND : constr};;

type information = Logic | Inf of judgement;;

value sort_hdchar : sorts -> string;;
value sort_eq_x : sorts -> sorts -> bool;;

value j_pop : judgement -> judgement;;
value j_lift : int -> judgement -> judgement;;
value j_subst_var : identifier -> judgement -> judgement;;

value mkVAR : identifier -> constr;;

value cast_type : constr -> constr;;
value strip_outer_cast : constr -> constr;;
value isprop : constr -> bool;;
value istype : constr -> bool;;
value iskind : constr -> bool;;
value same_kind : constr -> constr -> bool;;
value le_kind : constr -> constr -> bool;;

value isMETA : constr -> bool;;


value path_of_abst : constr -> section_path;;
value args_of_abst : constr -> constr vect;;

value path_of_const : constr -> section_path;;
value args_of_const : constr -> constr vect;;

value op_of_mind : constr -> section_path * int;;
value args_of_mind : constr -> constr vect;;

value op_of_mconstr : constr -> (section_path * int) * int;;
value args_of_mconstr : constr -> constr vect;;


value contents_of_kind : constr -> contents;;

value typep : sorts;;
value types : sorts;;
value spec : sorts;;
value prop : sorts;;
value applist : constr * constr list -> constr;;
value applistc : constr -> constr list -> constr;;
value appvect : constr * constr vect -> constr;;
value appvectc : constr -> constr vect -> constr;;
value telescope_appl : constr -> constr;;
value collapse_appl : constr -> constr;;
value strip_head_cast : constr -> constr;;
value whd_castapp_stack : constr -> constr list -> constr * constr list;;
value whd_castapp : constr -> constr;;
value ensure_appl : constr -> constr;;
value produit : identifier -> constr -> constr -> constr;;
value lambda : identifier -> constr -> constr -> constr;;
value occur_const : section_path -> constr -> bool;;
value subst_term_occ : int list -> constr -> constr -> constr;;
value eq_constr : constr * constr -> bool;;
value eta_eq_constr : constr * constr -> bool;;
value subst_term : constr -> constr -> constr;;
value subst_term_eta_eq : constr -> constr -> constr;;
value replace_consts : (section_path * (identifier list * constr) option) list -> constr -> constr;;
value push_and_lift : name * constr -> (name * constr) list -> (int * constr) list -> (name * constr) list * (int * constr) list;;
value push_and_liftl : int -> (name * constr) list -> constr -> (int * constr) list -> (name * constr) list * constr * (int * constr) list;;
value prod_and_pop : (name * constr) list -> constr -> (int * constr) list -> (name * constr) list * constr * (int * constr) list;;
value prod_and_popl : int -> (name * constr) list -> constr -> (int * constr) list -> (name * constr) list * constr * (int * constr) list;;
value lam_and_pop : (name * constr) list -> constr -> (int * constr) list -> (name * constr) list * constr * (int * constr) list;;
value lam_and_popl : int -> (name * constr) list -> constr -> (int * constr) list -> (name * constr) list * constr * (int * constr) list;;
value nb_lam : constr -> int;;
value nb_prod : constr -> int;;
value extract_lifted : int * constr -> constr;;
value insert_lifted : constr -> int * constr;;
value apply : constr -> constr -> constr;;
value occur_meta : constr -> bool;;
value assumptions_for_print : name list -> unit assumptions;;
value rel_vect : int -> int -> constr vect;;
value hd_of_prod : sorts oper term -> sorts oper term;;



(* $Id: term.mli,v 1.4 1996/02/12 15:38:03 cpaulin Exp $ *)
