(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                vartab.ml                                 *)
(****************************************************************************)
#open "more_util";;
#open "names";;
#open "term";;

(* Sections and Sticky Stack Summary *)
let AUTO_ABS_VARS = ref ([] : identifier list);;

let auto_save_variables () = !AUTO_ABS_VARS;;

let VARIABLES = ref (nil_sign : (constr * constr option) signature);;

let initial_sign () = map_sign_typ fst !VARIABLES;;
let initial_fsign () =
sign_it (fun id (_,SOME fty) sign -> add_sign (id,fty) sign
           | _ _ sign -> sign)
        !VARIABLES nil_sign
;;
let initial_assumptions () = (initial_sign(),initial_fsign());;

let push id sticky vbody =
    let (idl,tyl) = !VARIABLES
    in (VARIABLES := (id::idl,vbody::tyl);
        if sticky then
            (AUTO_ABS_VARS := id::!AUTO_ABS_VARS) else ())
;;

let init () = (VARIABLES:=nil_sign;
               AUTO_ABS_VARS := []);;

type frozen_t == (constr * constr option) signature
               * identifier list
;;

let freeze  () = (!VARIABLES,!AUTO_ABS_VARS);;
let unfreeze(v,a) = (VARIABLES := v;
                       AUTO_ABS_VARS := a);;

library__declare_summary "var"
{summary__freeze_function = freeze;
 summary__unfreeze_function = unfreeze;
 summary__init_function = init;
 summary__sync_function = (fun () -> false)}
;;

let rollback f () =
    let fs = freeze()
    in try f ()
       with e -> (unfreeze fs; raise e)
;;

(* $Id: vartab.ml,v 1.2 1996/02/12 15:38:06 cpaulin Exp $ *)
