(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                clenv.mli                                 *)
(****************************************************************************)

#open "names";;
#open "more_util";;
#open "term";;
#open "tacmach";;
#open "proof";;
#open "proof_trees";;
#open "pp";;
#open "stdpp";;


(****************************************************************************)
(*         The Type of Constructions Clausale Environments                  *)
(****************************************************************************)

type 'a freelisted = {rebus : 'a;
                      freemetas : intavs__t};;
type clbinding = CLTYP of constr freelisted
               | CLVAL of constr freelisted * constr freelisted;;
type 'a clausenv = {templval : constr freelisted;
                    templtyp : constr freelisted;
                    namenv : identifier intavm__t;
                    env : clbinding intavm__t;
                    hook : 'a};;
(* templval is the template which we are trying to fill out.
 * templtyp is its type.
 * namenv is a mapping from metavar numbers to names, for
 *        use in instanciating metavars by name.
 * env is the mapping from metavar numbers to their types
 *     and values.
 * hook is the pointer to the current walking context, for
 *      integrating existential vars and metavars.
 *)

value new_existential_in_sign : constr signature -> constr;;
value occur_existential : constr -> bool;;
value whd_deltat : walking_constraints -> constr -> constr;;
value ApplyHead : int -> constr -> walking_constraints -> walking_constraints * constr;;
value mimick_evar : constr -> int -> section_path -> walking_constraints -> walking_constraints;;
value unify : (int * constr) list -> walking_constraints -> constr -> constr -> (int * constr) list * (constr * constr) list;;
value unirec_rec : walking_constraints -> (int * constr) list * (constr * constr) list -> constr -> constr -> (int * constr) list * (constr * constr) list;;
value w_Unify : constr -> constr -> (int * constr) list -> walking_constraints -> walking_constraints * (int * constr) list;;
value UnifyTerms : constr -> constr -> tactic;;
value Unify : constr -> tactic;;

value pr_clenv : 'a clausenv -> std_ppcmds;;
value collect_metas : constr -> int list;;
value metavars_of : constr -> intavs__t;;
value mk_freelisted : constr -> constr freelisted;;
value mentions : 'a clausenv -> int -> int -> bool;;
value mk_clenv : 'a -> constr -> 'a clausenv;;
value mk_clenv_from : 'a -> constr * constr -> 'a clausenv;;
value mk_clenv_rename_from : 'a -> constr * constr -> 'a clausenv;;
value bounded_mk_clenv_from : int -> 'a -> constr * constr -> 'a clausenv;;
value mk_clenv_hnf_constr_type_of : walking_constraints -> constr -> walking_constraints clausenv;;
value mk_clenv_rename_type_of : walking_constraints -> constr -> walking_constraints clausenv;;
value mk_clenv_printable_type_of : walking_constraints -> constr -> walking_constraints clausenv;;
value mk_clenv_type_of : walking_constraints -> constr -> walking_constraints clausenv;;
value mk_clenv_rename_hnf_constr_type_of : walking_constraints -> constr -> walking_constraints clausenv;;
value connect_clenv : walking_constraints -> 'a clausenv -> walking_constraints clausenv;;
value clenv_assign : int -> constr -> 'a clausenv -> 'a clausenv;;
value clenv_val_of : 'a clausenv -> int -> constr;;
value clenv_instance : 'a clausenv -> constr freelisted -> constr;;
value clenv_instance_term : 'a clausenv -> constr -> constr;;
value clenv_pose : name * int * constr -> 'a clausenv -> 'a clausenv;;
value clenv_defined : 'a clausenv -> int -> bool;;
value clenv_value : 'a clausenv -> int -> constr freelisted;;
value clenv_type : 'a clausenv -> int -> constr freelisted;;
value clenv_template : 'a clausenv -> constr freelisted;;
value clenv_template_type : 'a clausenv -> constr freelisted;;
value clenv_instance_value : 'a clausenv -> int -> constr;;
value clenv_instance_type : 'a clausenv -> int -> constr;;
value clenv_instance_template : 'a clausenv -> constr;;
value clenv_instance_template_type : 'a clausenv -> constr;;
value clenv_unify : constr -> constr -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_bchain : int -> 'a clausenv -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_fchain : int -> 'a clausenv -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_refine : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value res_pf : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value res_pf1 : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value elim_res_pf : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value elim_res_pf1 : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value res_pf_cast : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value elim_res_pf_cast : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value constrain_clenv_to_subterm : walking_constraints clausenv -> constr * constr -> walking_constraints clausenv * constr;;
value constrain_clenv_using_subterm_list : bool -> walking_constraints clausenv -> constr list -> constr -> walking_constraints clausenv * constr list;;
value clenv_metavars : 'a clausenv -> int -> intavs__t;;
value clenv_template_metavars : 'a clausenv -> intavs__t;;
value dependent_metas : 'a clausenv -> int list -> intavs__t -> intavs__t;;
value clenv_dependent : 'a clausenv -> constr freelisted * constr freelisted -> int list;;
value clenv_independent : 'a clausenv -> constr freelisted * constr freelisted -> int list;;
value clenv_missing : 'a clausenv -> constr freelisted * constr freelisted -> int list;;
value clenv_constrain_missing_args : constr list -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_constrain_dep_args : constr list -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_constrain_dep_args_of : int -> constr list -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_lookup_name : 'a clausenv -> identifier -> int list;;
value clenv_match_args : (BindOcc * constr) list -> walking_constraints clausenv -> walking_constraints clausenv;;
value subterm : constr -> int list -> constr;;
value rev_subterm : constr -> int list -> constr;;
value clenv_pose_dependent_evars : walking_constraints clausenv -> walking_constraints clausenv;;
value e_res_pf : (walking_constraints -> tactic) -> walking_constraints clausenv -> tactic;;
value clenv_add_sign : (identifier * constr) -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_type_of : walking_constraints clausenv -> constr -> constr;;

value clenv_typed_unify : constr -> constr -> walking_constraints clausenv -> walking_constraints clausenv;;
value clenv_unique_resolver : bool -> walking_constraints clausenv -> goal sigma -> walking_constraints clausenv ;;
value abstract_list_all : goal sigma -> constr -> constr -> constr list -> constr ;;


(* $Id: clenv.mli,v 1.3 1996/02/12 15:38:34 cpaulin Exp $ *)
