(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               constants.ml                               *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "names";;
#open "generic";;
#open "constrtypes";;
#open "term";;
#open "reduction";;
#open "pp";;
#open "stdpp";;
#open "himsg";;
#open "more_util";;
#open "mach";;
#open "proof";;
#open "dischcore";;
#open "environ";;
#open "termenv";;

let make_constant_body k opacity (hyps,j) =
   { CONSTKIND   = k;
     CONSTHYPS   = hyps;
     CONSTBODY   = (SOME (nref(COOKED j.VAL)));
     CONSTEVAL   = NONE;
     CONSTOPAQUE = opacity;
     CONSTTYPE   = DOP2(Cast,j.TYP,hnf_constr mt_evc j.KIND) }
;;

let make_constant opacity ((hyps,j),(fhyps,inf)) =
  let cci = [(CCI,make_constant_body CCI opacity (hyps,j))] and
      fw = (match inf with
              Logic    -> []
            | Inf infj -> [(FW,make_constant_body FW false (fhyps,infj))])
  in cci@fw
;;

let make_cmap (hyps,fhyps) opacity (j,infj) =
  let ids = (auto_save_variables()) in
  let hyps' = thin_hyps ids (hyps,j) in
  let fhyps' = (match infj with
                  Logic -> nil_sign
                | Inf _ -> thin_to_match hyps' fhyps) in
  let fhyps'' = (match infj with
                   Logic    -> nil_sign
                 | Inf jinf -> thin_to_level fhyps' jinf.KIND)
  in make_constant opacity ((hyps',j),(fhyps'',infj))
;;

let expmod_recipe modlist = function
    COOKED c -> COOKED(expmod_constr modlist c)
  | RECIPE dr ->
    let new_expands = map_succeed (fun (Const sp,DO_REPLACE) -> sp
                                     | _ -> failwith "caught") modlist and
        new_modifs = map_succeed (fun ((_,DO_ABSTRACT _) as x) -> x
                                    | _ -> failwith "caught") modlist
    in RECIPE
       {d_expand = new_expands@ dr.d_expand;
        d_modify = new_modifs@ dr.d_modify;
        d_abstract = dr.d_abstract;
        d_from = dr.d_from}
;;

let expmod_constant modlist cb =
   {CONSTKIND = cb.CONSTKIND;
    CONSTHYPS= map_sign_typ (expmod_constr modlist) cb.CONSTHYPS;
    CONSTBODY= option_app (nref_app (expmod_recipe modlist)) cb.CONSTBODY;
    CONSTEVAL = NONE;
    CONSTOPAQUE = cb.CONSTOPAQUE;
    CONSTTYPE= expmod_constr modlist cb.CONSTTYPE}
;;

let expmod_cmap modlist cmap =
    map (fun (k,cb) ->
             (k,(expmod_constant modlist cb))) cmap
;;

let abstract_recipe_nref_option ((hyps,rpno),modl) id =
    if isnull_sign hyps or id <> fst(hd_sign hyps) then
        ((hyps,rpno),modl)
    else
        match (fst rpno) with
        (SOME{v=COOKED c}) ->
        let j = {VAL=c;TYP=fst(snd rpno);KIND=snd(snd rpno)} in
        let (hyps_j,modif) = machops__abs_var mt_evc id (snd(hd_sign hyps)) j
        in ((tl_sign hyps,(SOME{v=COOKED hyps_j.VAL},
                           (hyps_j.TYP,hyps_j.KIND))),modif::modl)

      | (SOME{v=RECIPE dr}) ->
        let j = {VAL=DOP2(Cast,DOP0 Implicit,DOP0 Implicit);
                 TYP = fst(snd rpno);
                 KIND = snd(snd rpno)} in
        let (hyps_j,modif) = machops__abs_var mt_evc id (snd(hd_sign hyps)) j in
        let dr' = {d_from = dr.d_from;
                   d_expand = dr.d_expand;
                   d_modify = dr.d_modify;
                   d_abstract = dr.d_abstract@[id]}
        in ((tl_sign hyps,(SOME{v=RECIPE dr'},(hyps_j.TYP,hyps_j.KIND))),modif::modl)

      | NONE ->
        let j = {VAL=DOP2(Cast,DOP0 Implicit,DOP0 Implicit);
                 TYP = fst(snd rpno);
                 KIND = snd(snd rpno)} in
        let (hyps_j,modif) = machops__abs_var mt_evc id (snd(hd_sign hyps)) j
        in ((tl_sign hyps,(NONE,(hyps_j.TYP,hyps_j.KIND))),modif::modl)
;;

let abstract_constant ids_to_abs cb =
let b = cb.CONSTBODY in
let (DOP2(Cast,typ,kind)) = cb.CONSTTYPE in
let rp = (b,(typ,kind)) in
let ((hyps',rp'),revmodl) =
    it_list abstract_recipe_nref_option ((cb.CONSTHYPS,rp),[]) ids_to_abs in
let modl = rev revmodl
in ({CONSTKIND = cb.CONSTKIND;
    CONSTHYPS = hyps';
    CONSTBODY = fst rp';
    CONSTTYPE = DOP2(Cast,fst(snd rp'),snd(snd rp'));
    CONSTOPAQUE = cb.CONSTOPAQUE;
    CONSTEVAL = NONE},
    modl)
;;

let abstract_cmap (osecsp,nsecsp) ids_to_abs cmap =
let (mods,cmap) =
     it_list (fun (mods,cmap) (k,cb) ->
                 match k with
                 CCI ->
                 let (cb',modl) = abstract_constant ids_to_abs cb
                 in ((Const(ccisp_of osecsp),DO_ABSTRACT(Const(ccisp_of nsecsp),modl))::mods,(k,cb')::cmap)
               | FW ->
                 let (cb',modl) = abstract_constant ids_to_abs cb
                 in ((Const(fwsp_of osecsp),DO_ABSTRACT(Const(fwsp_of nsecsp),modl))::mods,(k, cb')::cmap))
    ([],[]) cmap
in (mods,cmap)
;;

let recipe_cmap (sp,cmap) =
    map (fun (k,cb) ->
             let dr = {d_from = coerce_path k sp;
                       d_expand = [];
                       d_modify = [];
                       d_abstract = []}
             in (k,{CONSTKIND = cb.CONSTKIND;
                    CONSTHYPS = cb.CONSTHYPS;
                    CONSTBODY = option_app (fun _ -> nref(RECIPE dr))
      	       	       	       	       	   cb.CONSTBODY;
                    CONSTEVAL = NONE;
                    CONSTOPAQUE = true;
                    CONSTTYPE = cb.CONSTTYPE})) cmap
;;

let rec execute_recipe dr =
  let _ = cook_constant dr.d_from in
  let from_cb = snd(const_of_path dr.d_from) in
  let expands = dr.d_expand in
  let _ = do_list cook_constant expands in
  let expmod_cb = expmod_constant
      	      ((map (fun sp -> (Const sp,DO_REPLACE)) expands)@dr.d_modify)
      	      from_cb in
  let (abs_cb,_) = abstract_constant dr.d_abstract expmod_cb
  in match abs_cb.CONSTBODY with
       SOME{v=COOKED c} -> c
     | _ -> anomaly "execute_recipe"

and cook_constant sp =
  let cb = (snd (const_of_path sp))
  in match cb.CONSTBODY with
       SOME({v=RECIPE dr} as r) ->
             (MSGNL [< 'S"[Cooking " ; 'S(string_of_path sp) ; 'S"]" >];
              r.v <- (COOKED (execute_recipe dr)))
     | SOME{v=COOKED _} -> ()
     | NONE -> anomaly "cook_constant"
;;

let process_recipe_cmap (sp,newsp) (ids_to_discard,work_alist) cmap =
  let raw_cmap = recipe_cmap (sp,cmap) in
  let expmod_raw_cmap = expmod_cmap work_alist raw_cmap
  in abstract_cmap (sp,newsp) ids_to_discard expmod_raw_cmap
;;

let infexecute_constant (sign,fsign) opacity c =
  let c = (instantiate_universes c) in
  let thmj = infexecute mt_evc (sign,fsign) c
  in make_cmap (sign,fsign) opacity thmj
;;

let make_parameter_body k (hyps,j) =
match hnf_constr mt_evc j.TYP with DOP0(Sort(_)) ->
let typ = j_val j
in  {CONSTKIND = k;
     CONSTHYPS=hyps;
     CONSTBODY=NONE;
     CONSTEVAL = NONE;
     CONSTOPAQUE = true;
     CONSTTYPE=(match typ with
                DOP2(Cast,c,t) -> DOP2(Cast,c,hnf_constr mt_evc t)
              | _ -> typ)}
  | _ -> errorlabstrm "make_parameter" 
                      [< pTERM (j_val j); 'S " is not a type" >]
;;

let make_parameter ((hyps,j),(fhyps,inf)) =
let cci = [(CCI,make_parameter_body CCI (hyps,j))] and
    fw = (match inf with
          Logic -> []
        | Inf infj -> [(FW,make_parameter_body FW (fhyps,infj))])
in cci@fw
;;

let make_paramap (hyps,fhyps) (j,infj) =
let ids = (auto_save_variables()) in
let hyps' = thin_hyps ids (hyps,j) in
let fhyps' = (match infj with
              Logic -> nil_sign
            | Inf _ -> thin_to_match hyps' fhyps) in
let fhyps'' = (match infj with
               Logic -> nil_sign
             | Inf jinf -> thin_to_level fhyps' jinf.TYP)
in make_parameter ((hyps',j),(fhyps'',infj))
;;

let infexecute_parameter (sign,fsign) T =
let T = (instantiate_universes T) in
let (j,infj) = infexecute mt_evc (sign,fsign) T
in make_paramap (sign,fsign) (j,infj)
;;

(* $Id: constants.ml,v 1.3 1996/02/12 15:38:40 cpaulin Exp $ *)
