(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              coqtoplevel.ml                              *)
(****************************************************************************)

#open "pp";;
#open "stdpp";;
#open "himsg";;
#open "std";;
#open "sys";;
#open "initial";;
#open "system";;
#open "errors";;
#open "vkey";;
#open "vernac";;
#open "vernacinterp";;
#open "lexer";;
#open "more_util";;

(* read_loop reads vernacular expressions while no error occurs.
   it raises various exceptions if an error occurs,
   and raises Drop if the last expression is Drop. *)

let rec discard_dots ts =
    let rec aux = function
        [< 'Tdot >]  -> [< 'S"." >]
      | [< 't ; s >] -> [< 'S(string_of_token rev_vernac_keywords t) ;
                           'S" " ; aux s >]
      | [< >]        -> [< >]
    in MSGNL [< 'S"Discarding " ; aux ts >]
;;

let read_loop ts =
    while true do
    print_newline(); flush std_out; flush std_err;
    parse_vernac  ts
    done;;

let top_loop_handler read_loop discarder ts =
    try while true do
        (try read_loop ts
         with Parse_error   -> (try discarder ts; print_syntax_error()
                                with Lexer_error n -> print_lexer_error n)   
            | Lexer_error n -> print_lexer_error n
            | Break         -> message "Interrupted while parsing vernac"
            | Parse_failure -> raise Parse_failure
            | End_of_file   -> raise End_of_file
            | Drop          -> raise Drop
            | e             -> (MSG(errors__explain_sys_exn e);
      	       	       	       	reraise_sys_exn e))
        done; 0
    with Drop          -> 0
       | Parse_failure -> raise Parse_failure
       | End_of_file   -> raise End_of_file
       | Sys_error msg -> message("OS error: " ^ msg); raise (Sys_error msg)
       | e             -> MSG (errors__explain_sys_exn e); 1
;;

let constr_loop () = 
    let ts = token_stream vernac_keywords (stream_of_channel std_in) in
    top_loop_handler read_loop discard_dots ts;;

(* constr_loop reads vernacular expressions while no fatal error occurs.
   Returns 0 at the end, if everything was successful, otherwise 1 -
   so it's compatible with UNIX return codes. *)

let coq () = set_prompt pfedit__proof_prompt;
             try constr_loop ()
             with (End_of_file|Parse_failure) -> sys__exit 0 
                | _ -> (*set_prompt (fun () -> "ml #");*) 0
;;

(* coq has to be used in interactive mode. Finishes if a fatal error occurs, 
   if Drop. is encountered, if EOF is encountered. Prints an error message 
   and gives a new prompt "Coq < " if a non-fatal error occurs. *)

let load_vernacular = (load_vernac false);;
let load_vernacular_noisily = (load_vernac true);;

let go () = coq ();;

(**
set_prompt (fun () -> "ml #");;

let handle_errors f x =
  try f x with
    Drop          -> error "Drop"
  | Parse_failure -> error "Parse_failure"
  | End_of_file   -> error "End_of_file"
  | Break         -> message "Interrupted while parsing vernac";error "handled"
  | Sys_error msg -> message("OS error: " ^ msg); raise (Sys_error msg)
  | e             -> MSG (errors__explain_sys_exn e); error "handled"
;;
**)

(* $Id: coqtoplevel.ml,v 1.4 1996/02/12 15:38:43 cpaulin Exp $ *)
