(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                pfedit.mli                                *)
(****************************************************************************)

#open "std";;
#open "names";;
#open "more_util";;
#open "term";;
#open "pp";;
#open "stdpp";;
#open "proof";;
#open "proof_constraints";;
#open "tacmach";;
#open "libobject";;
#open "states";;

type proof_topstate = 
    {top_hyps : constr signature * constr signature;
     top_goal : goal;
     top_strength : strength }
;;

value PROOFS : (string, pftreestate,proof_topstate) edit__t;;

value undo_limit : int ref;;
value set_undo : int -> unit;;

value msg_proofs : bool -> std_ppcmds;;
value unset_undo : unit -> unit;;

value get_state : unit -> pftreestate * proof_topstate;;
value get_topstate : unit -> proof_topstate;;
value get_pftreestate : unit -> pftreestate;;
value set_proof : string option -> unit;;
value get_proof : unit -> string;;
value list_proofs : unit -> string list;;
value add_proof : string * pftreestate * proof_topstate -> unit;;
value del_proof : string -> unit;;
value init_proofs : unit -> unit;;

value mutate : (pftreestate -> pftreestate) -> unit;;
value rev_mutate : (pftreestate -> pftreestate) -> unit;;
value start : string * proof_topstate -> unit;;
value restart : unit -> unit;;
value proof_prompt : unit -> string;;
value proof_term : unit -> constr;;

value undo : int -> unit;;
value resume_last : unit -> unit;;

value refining : unit -> bool;;
value abort_goal : string -> unit;;
value abort_cur_goal : unit -> unit;;
value abort_goals : unit -> unit;;
value abort_refine : ('a -> 'b) -> 'a -> 'b;;
value reset_name : identifier -> unit;;
value reset_keeping_name : identifier -> unit;;
value save_state : string -> string -> unit;;
value restore_state : string -> unit;;
value restore_last_saved_state : unit -> unit;;
value reset_all : unit -> unit;;
value reset_section : string -> unit;;
value reset_prelude : unit -> unit;;
value reset_initial : unit -> unit;;

value start_proof : string -> strength -> command -> unit;;

value save_named : bool -> unit;;
value save_anonymous : bool -> string -> 'a -> unit;;
value save_anonymous_thm : bool -> string -> unit;;
value save_anonymous_remark : bool -> string -> unit;;

value solve_nth : int -> tactic -> unit;;
value by : tactic -> unit;;
value traverse : int -> unit;;
value traverse_to : int list -> unit;;

(* $Id: pfedit.mli,v 1.5 1996/02/12 15:38:59 cpaulin Exp $ *)
