(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               tacmach.mli                                *)
(****************************************************************************)

#open "initial";;
#open "names";;
#open "more_util";;
#open "term";;
#open "mach";;
#open "pp";;
#open "stdpp";;
#open "proof";;
#open "proof_trees";;
#open "proof_constraints";;
#open "refiner";;
#open "evar_refiner";;
#open "reduction";;

type 'a sigma;;
value sig_it : 'a sigma -> 'a;;
value Project : goal sigma -> readable_constraints;;
type validation == (proof list -> proof);;
type tactic == goal sigma -> (goal list sigma * validation);;
type transformation_tactic == proof -> (goal list * validation);;

type walking_constraints == evar_refiner__walking_constraints;;
type 'a result_w_tactic == walking_constraints -> walking_constraints * 'a;;
type w_tactic == walking_constraints -> walking_constraints;;
value StartWalk : goal sigma -> walking_constraints * (walking_constraints -> tactic);;
value Walking_THEN : 'a result_w_tactic -> ('a -> tactic) -> tactic;;
value Walking : w_tactic -> tactic;;
value w_Focusing_THEN : section_path -> 'a result_w_tactic -> ('a -> w_tactic) -> w_tactic;;
value w_Declare : section_path -> constr -> w_tactic;;
value w_Declare_At : section_path -> section_path -> constr -> w_tactic;;
value w_Define : section_path -> constr -> w_tactic;;

value w_Underlying : walking_constraints -> readable_constraints;;
value w_hyps : walking_constraints -> constr signature;;
value w_type_of : walking_constraints -> constr -> constr;;
value w_add_sign : (identifier * constr) -> walking_constraints -> walking_constraints;;

value w_IDTAC : w_tactic;;
value w_ORELSE : w_tactic -> w_tactic -> w_tactic;;
value ctxt_type_of : readable_constraints -> constr -> constr;;

value IDTAC : tactic;;
value ORELSE : tactic -> tactic -> tactic;;
value THEN : tactic -> tactic -> tactic;;
value THEN_i : tactic -> (int -> tactic) -> int -> tactic;;
(*value THENF : tactic -> tactic -> tactic;;*)
value THENL : tactic -> tactic -> tactic;;
value THENS : tactic -> tactic list -> tactic;;
value REPEAT : tactic -> tactic;;
value FIRST : tactic list -> tactic;;
value TRY : tactic -> tactic;;
value COMPLETE : tactic -> tactic;;
value AT_LEAST_ONE : tactic -> tactic;;
value FAILTAC : tactic;;
value DO : int -> tactic -> tactic;;
value PROGRESS : tactic -> tactic;;

value unTAC : tactic -> goal sigma -> proof sigma;;
value Tactic : tactic_expression -> tactic;;
value Context : ctxt -> tactic;;

type pftreestate == refiner__pftreestate;;
value proof_of_pftreestate : pftreestate -> proof;;
value cursor_of_pftreestate : pftreestate -> int list;;
value evc_of_pftreestate : pftreestate -> global_constraints;;

value traverse : int -> pftreestate -> pftreestate;;
value solve_nth_pftreestate : int -> tactic -> pftreestate -> pftreestate;;
value solve_pftreestate : tactic -> pftreestate -> pftreestate;;
value mk_pftreestate : goal -> pftreestate;;
value extract_pftreestate : pftreestate -> constr;;
value first_unproven : pftreestate -> pftreestate;;
value last_unproven : pftreestate -> pftreestate;;
value node_prev_unproven : int -> pftreestate -> pftreestate;;
value node_next_unproven : int -> pftreestate -> pftreestate;;
value next_unproven : pftreestate -> pftreestate;;
value prev_unproven : pftreestate -> pftreestate;;
value top_of_tree : pftreestate -> pftreestate;;

value frontier : transformation_tactic;;

value pr_gls : goal sigma -> std_ppcmds;;
value pr_glls : goal list sigma -> std_ppcmds;;

value pr_com : readable_constraints -> goal -> command -> std_ppcmds;;
value pr_binding : readable_constraints -> goal -> BindOcc * command -> std_ppcmds;;
value pr_bindings_list : readable_constraints -> goal -> tactic_arg list -> std_ppcmds;;
value pr_with_bindings_list : readable_constraints -> goal -> tactic_arg list -> std_ppcmds;;

value register_comarg_tactic : string -> (command -> tactic) -> (command -> tactic);;
value register_comarglist_tactic : string -> string -> (command list -> tactic) -> (command list -> tactic);;
value register_numarg_tactic : string -> (int -> tactic) -> (int -> tactic);;
value register_atomic_tactic : string -> tactic -> tactic;;
value register_ident_tactic : string -> (identifier -> tactic) -> (identifier -> tactic);;
value register_string_tactic : string -> (string -> tactic) -> (string -> tactic);;
value register_bindinglist_tactic : string -> ((BindOcc * command) list -> tactic) -> ((BindOcc * command) list -> tactic);;
value register_combindinglist_tactic : string -> (command * (BindOcc * command) list -> tactic) -> (command * (BindOcc * command) list -> tactic);;

value overwriting_register_comarg_tactic : string -> (command -> tactic) -> (command -> tactic);;
value overwriting_register_numarg_tactic : string -> (int -> tactic) -> (int -> tactic);;
value overwriting_register_atomic_tactic : string -> tactic -> tactic;;
value overwriting_register_ident_tactic : string -> (identifier -> tactic) -> (identifier -> tactic);;
value overwriting_register_string_tactic : string -> (string -> tactic) -> (string -> tactic);;
value overwriting_register_comarglist_tactic : string -> string -> (command list -> tactic) -> (command list -> tactic);;
value overwriting_register_bindinglist_tactic : string -> ((BindOcc * command) list -> tactic) -> ((BindOcc * command) list -> tactic);;
value overwriting_register_combindinglist_tactic : string -> (command * (BindOcc * command) list -> tactic) -> (command * (BindOcc * command) list -> tactic);;


value register_tactic : string ->
                       (tactic_arg list -> tactic) ->
                       (readable_constraints -> goal -> tactic_expression -> std_ppcmds) ->
                       (tactic_arg list -> tactic);;

value overwriting_register_tactic : string ->
                       (tactic_arg list -> tactic) ->
                       (readable_constraints -> goal -> tactic_expression -> std_ppcmds) ->
                       (tactic_arg list -> tactic);;

value pf_global : goal sigma -> identifier -> constr;;
value pf_type_of : goal sigma -> constr -> constr;;
value pf_constr_of_com : goal sigma -> command -> constr;;
value pf_fexecute : goal sigma -> constr -> judgement;;
value pf_infexecute : goal sigma -> constr -> judgement * information;;
value pf_whd_betadeltaiota : goal sigma -> constr -> constr;;
value pf_whd_betadeltaiota_stack : goal sigma -> stack_reduction_function;;
value pf_hnf_constr : goal sigma -> constr -> constr;;
value pf_red_product : goal sigma -> constr -> constr;;
value pf_nf : goal sigma -> constr -> constr;;
value pf_one_step_reduce : goal sigma -> constr -> constr;;
value pf_compute : goal sigma -> constr -> constr;;
value pf_const_value : goal sigma -> constr -> constr;;
value pf_unfoldn : goal sigma -> int list -> section_path -> constr -> constr;;
value pf_conv_x : goal sigma -> constr -> constr -> bool;;
value pf_hyps : goal sigma -> constr signature;;
value pf_concl : goal sigma -> constr;;
value pf_h : goal sigma -> int -> identifier * constr;;
value pf_ctxt : goal sigma -> ctxt;;

value rename_bound_var : identifier list -> constr -> constr;;
value rename_bound_var_goal : tactic;;

value refiner : rule -> tactic;;

value introduction : identifier -> tactic;;
value intro_after : identifier -> identifier -> tactic;;
value intro_replacing : identifier -> tactic;;
value Refine : constr -> tactic;;
value convert_concl : constr -> tactic;;
value convert_hyp : identifier -> constr -> tactic;;
value thin : identifier list -> tactic;;
value fix : identifier -> int -> tactic;;
value mutual_fix : identifier list -> int list -> constr list -> tactic;;
value cofix : identifier -> tactic;;
value mutual_cofix : identifier list -> constr list -> tactic;;

value print_rule : readable_constraints -> goal -> rule -> std_ppcmds;;
value print_proof : readable_constraints -> constr signature -> proof -> std_ppcmds;;
value print_script : bool -> readable_constraints -> constr signature -> proof -> std_ppcmds;;
value print_treescript : readable_constraints -> constr signature -> proof -> std_ppcmds;;

value hnf_type_of : goal sigma -> constr -> constr;;
value w_whd_betadeltaiota : walking_constraints -> constr -> constr;;
value w_hnf_constr : walking_constraints -> constr -> constr;;
value w_conv_x : walking_constraints -> constr -> constr -> bool;;
value w_const_value : walking_constraints -> constr -> constr;;
value w_defined_const : walking_constraints -> constr -> bool;;


(* $Id: tacmach.mli,v 1.5 1996/02/12 15:39:17 cpaulin Exp $ *)
