(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               tactics.mli                                *)
(****************************************************************************)

#open "names";;
#open "more_util";;
#open "term";;
#open "tacmach";;
#open "proof";;
#open "proof_trees";;
#open "clenv";;

value tactic_com : (constr -> tactic) -> command -> tactic;;
value tactic_com_bind_list : (constr * (BindOcc * constr) list -> tactic)
      -> command * (BindOcc * command) list -> tactic;;
value tactic_com_list : (constr list -> tactic) -> command list -> tactic;;
value tactic_bind_list : ((BindOcc * constr) list -> tactic) 
                      -> (BindOcc * command) list -> tactic;;
value intro : tactic;;
value intros : tactic;;
value intros_with : identifier list -> tactic;;
value intros_until : identifier -> tactic;;
value intro_using : identifier -> tactic;;
value cut : command -> tactic;;
value new_hyp : command -> tactic;;
value new_hyp_with : command -> (BindOcc * command) list -> tactic;;
value new_hyp_with_num : int -> command -> (BindOcc * command) list -> tactic;;
value generalize : command list -> tactic;;
value resolve : command -> tactic;;
value resolve_with : command list -> tactic;;
value resolve_with_bindings : (command * (BindOcc * command) list) -> tactic;;
value imp_elim_tac : command -> tactic;;
value assumption : tactic;;
value give_exact_com : command -> tactic;;
value change : command -> tactic;;
value red : tactic;;
value hnf_tac : tactic;;
value simpl_tac : tactic;;
value compute_tac : tactic;;
value pattern : (int list * command) list -> tactic;;
value unfold_nth_tac : (int list * identifier) list -> tactic;;
value unfold_nth : (int list * section_path) list -> tactic;;
value clear_hyp : identifier list -> tactic;;
value red_hyp : identifier -> tactic;;
value hnf_tac_hyp : identifier -> tactic;;
value change_hyp : command -> identifier -> tactic;;
value Change : constr -> tactic;;
value ChangeHyp : constr -> identifier -> tactic;;
value simpl_hyp : identifier -> tactic;;
value unfold_nth_hyp : (int list * identifier) list -> identifier -> tactic;;
value elim : command -> tactic;;
value elim1 : command -> tactic;;
value elim_with : command list -> tactic;;
value elim_with_using : (command * (BindOcc * command) list) ->  (command * (BindOcc * command) list) -> tactic;;
value elim_with_bindings : (command * (BindOcc * command) list) -> tactic;;
value case_with_bindings : (command * (BindOcc * command) list) -> tactic;;
value case : command -> tactic;;
value case_type : command -> tactic;;
value case_analysis : (BindOcc * constr) list -> constr -> tactic;;
value case_analysis_type : constr -> tactic;;
value elim_last : tactic;;
value elim_type : command -> tactic;;
value induct : identifier -> tactic;;
value induct_nodep : int -> tactic;;
value destruct : identifier -> tactic;;
value destruct_nodep : int -> tactic;;
value replace_tac : command -> command -> command -> command -> command -> command -> tactic;;
value equality_tac : (command * command * command) -> (command * command * command) -> command -> tactic;;
value string_head : constr -> string;;
exception BOUND;;
value head_constr : constr -> constr;;
value head_constr_bound : constr -> constr;;

value decomp_app : constr -> constr * constr list;;
value cut_tac : constr -> tactic;;
value pattern_list : (int list * constr) list -> tactic;;
value elimination : constr list -> constr -> tactic;;
value elimination_type : constr -> tactic;;
value change_pf : constr -> tactic;;
value generalize_tac : constr list -> tactic;;
value absurd : command -> tactic;;

value apply_type : constr -> constr list -> tactic;;
value apply_term : constr -> constr list -> tactic;;
value exact : constr -> tactic;;
value give_exact : constr -> tactic;;
value Fresh : identifier -> name -> (identifier -> tactic) -> tactic;;

value resolve_term: constr->tactic;;
value resolve_with_bindings_tac : constr * (BindOcc * constr) list -> tactic;;

value make_clenv_binding_apply :
walking_constraints ->
 constr * constr -> (BindOcc * constr) list -> walking_constraints clausenv;;

value constructor_with_bindings : int ->  (BindOcc * command) list  -> tactic;;
value left_with_bindings  :  (BindOcc * command) list  -> tactic;;
value right_with_bindings  :  (BindOcc * command) list  -> tactic;;
value split_with_bindings  :  (BindOcc * command) list  -> tactic;;

value sort_of_goal : goal sigma -> sorts;;

(* $Id: tactics.mli,v 1.7 1996/02/12 15:39:22 cpaulin Exp $ *)
