(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 stdpp.ml                                 *)
(****************************************************************************)
#open "pp_control";;
#open "pp";;

let string_of_bool = function true -> "true" | _ -> "false";;
let string_of_char = char_for_read;;

let (std_fp:(int * string) pp_formatter_defaults) =
    {output = output_substring std_out;
     flush_out = (fun () -> flush std_out);
     print = (function (n,s) -> (n,[< 's >]))}
;;

let (err_fp:(int * string) pp_formatter_defaults) =
    {output = output_substring std_err;
     flush_out = (fun () -> flush std_err);
     print = (fun (n,s) -> (n,[< 's >]))}
;;

let biggest_int = 1073741822;;

let with_depth gp n =
    (gp.pp_control__max_depth <- n;
     if gp.pp_control__limit_depth < n then
         gp.pp_control__limit_depth <- succ n)
;;

let fully_pp gp = with_depth gp biggest_int;;

let full_print strm =
    let gp = copy_gp dflt_gp in
        (fully_pp gp;
         PP_WITH gp std_fp strm)
;;

let PP cmdstream = PP_WITH dflt_gp std_fp cmdstream;;
let PPNL cmdstream = PPNL_WITH dflt_gp std_fp cmdstream;;

let PPERR strm =
let gp = copy_gp dflt_gp
in (fully_pp gp;
    PP_WITH gp err_fp strm)
;;

let PPERRNL strm =
let gp = copy_gp dflt_gp
in (fully_pp gp;
    PPNL_WITH gp err_fp strm)
;;

let PPDIR dirstream = PPDIR_WITH  dflt_gp std_fp dirstream;;

let PPDIR_FULL dirstream =
    let gp = copy_gp dflt_gp in
        (fully_pp gp;
         PPDIR_WITH gp std_fp dirstream)
;;

let AsNull s = PR(0,s);;
let STRING s = PR(string_length s,s);;
let S = STRING;;
let ID id = STRING id;;
let QSTRING s = STRING("\""^(string_for_read s)^"\"");;
let QS = QSTRING;;

let INT n = STRING(string_of_int n);;
let REAL r = STRING(string_of_float r);;
let BOOL b = STRING(string_of_bool b);;

let implode sl =
  let len = it_list (function a -> function b -> a+(string_length b)) 0 sl in
  let dest = create_string len in
      (it_list (function start -> function src ->
                let src_len = string_length src in
                    (blit_string src 0 dest start src_len;
                     start + src_len))
               0 sl;
       dest)
;;

let STRINGPP cmdstream =
let strbuf = ref [] in
let outs s n m = strbuf := (sub_string s n m)::!strbuf in
let (std_stringfp:(int * string) pp_formatter_defaults) =
    {output = outs;
     flush_out = (fun () -> ());
     print = (function (n,s) -> (n,[< 's >]))}
in (PP_WITH dflt_gp std_stringfp cmdstream;
    implode (rev !strbuf))
;;

(* $Id: stdpp.ml,v 1.2 1996/02/12 15:40:20 cpaulin Exp $ *)
