(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  avm.ml                                  *)
(****************************************************************************)

#open "std";;
#open "more_util";;

type ('a,'b) t == ('a,'b) map__t;;
type ('a,'b) frozen_t == ('a,'b) map__frozen_t;;

let new ord = map__empty ord;;

let toList m =
let r = ref []
in (map__iter (fun k v -> r := (k,v)::!r) m;
    !r)
;;

let dom m = map fst (toList m);;
let rng m = map snd (toList m);;

let map m k = map__find k m;;

let in_dom m k =
    try (map m k;true)
    with Not_found -> false
;;

let rmv m k = map__remove k m;;

let add m (k,v) =
    if in_dom m k then raise (Failure "avm__add")
    else map__add k v m
;;

let remap m k v = (add (rmv m k) (k,v));;

let app f m = map__iter (fun k v -> (f(k,v))) m;;

let ofList ord l = it_list add (new ord) l;;

let inv l b = list__map fst (filter (fun (a',b') -> b' = b) (toList l));;

let freeze m = map__read_tree m;;
let unfreeze fm m = map__set_tree fm m;;

let clone_empty m = map__clone_empty m;;

(* $Id: avm.ml,v 1.2 1996/02/12 15:40:36 cpaulin Exp $ *)
