(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                bstack.ml                                 *)
(****************************************************************************)
#open "std";;
#open "more_util";;

type 'a t = {mutable depth : int option;
             mutable stack : 'a list};;

let new depth = {depth = depth;
                 stack = []};;

let set_depth bs n = bs.depth <- n;;

let safe_chop_list len l =
    if length l <= len then (l,[]) else chop_list len l
;;

let push bs e =
    match bs.depth with
    NONE -> bs.stack <- e::bs.stack
  | SOME d -> bs.stack <- fst(safe_chop_list d (e::bs.stack))
;;

let pop bs =
    match bs.stack with
    [] -> NONE
  | h::tl -> (bs.stack <- tl; SOME h)
;;

let top bs =
    match bs.stack with
    [] -> NONE
  | h::_ -> SOME h
;;

let app_push bs f =
    match bs.stack with
    [] -> error "Nothing on the stack"
  | h::_ -> push bs (f h)
;;

let app_repl bs f =
    match bs.stack with
    [] -> error "Nothing on the stack"
  | h::t -> bs.stack <- (f h)::t
;;

let is_empty bs = bs.stack = [];;

let depth bs = length bs.stack;;

(* $Id: bstack.ml,v 1.2 1996/02/12 15:40:55 cpaulin Exp $ *)
