(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 edit.ml                                  *)
(****************************************************************************)

#open "std";;
#open "stdpp";;
#open "more_util";;


type ('a,'b,'c) t = {mutable focus : 'a option;
                     mutable last_focused_stk : 'a list;
                     buf : ('a, 'b bstack__t * 'c) mlm__t}
;;

let mt () = {focus = NONE;
             last_focused_stk = [];
             buf = mlm__new()};;

let focus e nd =
  (match nd with
     NONE -> ()
   | SOME f -> if not(mlm__in_dom e.buf f) then
                  invalid_arg "edit__focus");
  (match e.focus with
     NONE -> ()
   | SOME foc -> if e.focus <> nd then
                  e.last_focused_stk <- (foc::(except foc e.last_focused_stk))
  );
  e.focus <- nd;;

let last_focused e =
  match e.last_focused_stk with
    [] -> NONE
  | (f::_) -> (SOME f)
;;

let restore_last_focus e = focus e (last_focused e);;
(*
  match e.last_focused_stk with
    [] -> e.focus <- NONE
  | (f::r) -> (e.last_focused_stk <- r ; e.focus <- (SOME f))
;;
*)

let focusedp e =
    match e.focus with
    NONE -> false
  | _    -> true
;;

let read e =
    match e.focus with
    NONE -> NONE
  | SOME d ->
    let (bs,c) = (mlm__map e.buf d)
    in (match bstack__top bs with
        NONE -> anomaly "edit__read"
      | SOME v -> SOME(d,v,c))
;;

let mutate e f =
    match e.focus with
    NONE -> invalid_arg "edit__mutate"
  | SOME d ->
    let (bs,c) = (mlm__map e.buf d)
    in bstack__app_push bs (f c)
;;

let rev_mutate e f =
    match e.focus with
    NONE -> invalid_arg "edit__rev_mutate"
  | SOME d ->
    let (bs,c) = (mlm__map e.buf d)
    in bstack__app_repl bs (f c)
;;

let undo e n =
    match e.focus with
    NONE -> invalid_arg "edit__undo"
  | SOME d ->
    let (bs,_) = mlm__map e.buf d
    in if bstack__depth bs <= n then
        errorlabstrm "edit__undo" [< 'S"Undo stack would be exhausted" >]
       else
           repeat n (fun () -> bstack__pop bs)()
;;

let create e (d,b,c,udepth) =
    if mlm__in_dom e.buf d then
        errorlabstrm "edit__create" 
      	       	         [< 'S"Already editing something of that name" >]
    else let bs = bstack__new udepth
         in (bstack__push bs b;
             mlm__add e.buf (d,(bs,c)))
;;

let delete e d =
    if not(mlm__in_dom e.buf d) then
        errorlabstrm "edit__delete" [< 'S"No such editor" >]
    else (mlm__rmv e.buf d;
          e.last_focused_stk <- (except d e.last_focused_stk);
          match e.focus with
          SOME d' -> if d = d' then (e.focus <- NONE ; (restore_last_focus e))
        | NONE -> ())
;;

let dom e = mlm__dom e.buf;;

let empty e =
    e.focus <- NONE;
    e.last_focused_stk <- [];
    mlm__empty e.buf
;;


(* $Id: edit.ml,v 1.3 1996/02/12 15:41:03 cpaulin Exp $ *)
