(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 edit.mli                                 *)
(****************************************************************************)
#open "std";;
#open "more_util";;

(* The type of editors:
 *
 * An editor is a finite map, 'a -> 'b, which knows how to apply
 * modification functions to the value in the rane, and how to
 * focus on a member of the range.
 *
 * It also supports the notion of a limited-depth undo, and that certain
 * modification actions do not push onto the undo stack, since they are
 * reversible.
 *)

type ('a,'b,'c) t;;
value mt : unit -> ('a,'b,'c) t;;

(* sets the focus to the specified domain element, or if NONE,
 * unsets the focus *)
value focus : ('a,'b,'c) t -> 'a option -> unit;;

(* gives the last focused element or NONE if none *)
value last_focused : ('a,'b,'c) t -> 'a option;;

(* are we focused ? *)
value focusedp : ('a,'b,'c) t -> bool;;

(* If we are focused, then return the current domain,range pair. *)
value read : ('a,'b,'c) t -> ('a * 'b * 'c) option;;

(* mutates the currently-focused range element, pushing its
 * old value onto the undo stack
 *)
value mutate : ('a,'b,'c) t -> ('c -> 'b -> 'b) -> unit;;

(* mutates the currently-focused range element, in place. *)
value rev_mutate : ('a,'b,'c) t -> ('c -> 'b -> 'b) -> unit;;

(* Pops the specified number of elements off of the undo stack,
 * reinstating the last popped element
 *
 * the undo stack is independently managed for each range element.
 *)
value undo : ('a,'b,'c) t -> int -> unit;;

value create : ('a,'b,'c) t -> 'a * 'b * 'c * int option -> unit;;
value delete : ('a,'b,'c) t -> 'a -> unit;;

value dom : ('a,'b,'c) t -> 'a list;;

value empty : ('a,'b,'c) t -> unit;;

(* $Id: edit.mli,v 1.3 1996/02/12 15:41:05 cpaulin Exp $ *)
