(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 fmavm.ml                                 *)
(****************************************************************************)


let new (ord,n) = (mavm__new ord,mhm__new n);;

let empty (am,hm) = (mavm__empty am;mhm__empty hm);;
let map (am,hm) k =
    try mhm__map hm k
    with Not_found ->
        let v = mavm__map am k
        in (mhm__add hm (k,v);
            v)
;;

let rmv (am,hm) k =
    if mhm__in_dom hm k then
        (mhm__rmv hm k; mavm__rmv am k)
    else (mavm__rmv am k)
;;

let add (am,hm) (k,v) = mavm__add am (k,v);;
let dom (am,hm) = mavm__dom am;;
let rng (am,hm) = mavm__rng am;;
let in_dom (am,hm) k = mavm__in_dom am k;;
let inv (am,hm) v = mavm__inv am v;;
let remap (am,hm) k v =
    (rmv (am,hm) k;
     add (am,hm) (k,v));;
let app f (am,hm) = mavm__app f am;;
let toList (am,hm) = mavm__toList am;;

let freeze ((am,hm):('a,'b) t) = mavm__freeze am;;

(* WARNING: PHYSEQ: BUG:
 * This code relies on the implementation of physical equality
 * in a safe way.  But if there are any bugs, then the following
 * true-branch can be killed off, at a price in speed.
 *)
let unfreeze fm (am,hm) =
    let oam = mavm__freeze am
    in if oam == fm then ()
       else (empty (am,hm);
             mavm__unfreeze fm am)
;;

(* $Id: fmavm.ml,v 1.2 1996/02/12 15:41:07 cpaulin Exp $ *)
