(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 fthm.ml                                  *)
(****************************************************************************)
#open "std";;

type ('a,'b) tree =
    NODE of 'b mls__t * ('a,('a,'b) tree) mhm__t;;

type ('a,'b) t =
    WRITABLE of ('a list * 'b) list ref * ('a,'b) tree
  | READONLY of ('a,'b) tree
;;

let new () = WRITABLE (ref [],NODE(mls__mt(),mhm__new 17));;

let contents_of = function
    WRITABLE (_,NODE t) -> t
  | READONLY (NODE t) -> t
;;

let add_log ent = function
    WRITABLE(l,_) -> l:= ent:: !l
  | READONLY _ -> anomaly "fthm: add_log"
;;

let rmv_log ent = function
    WRITABLE(l,_) -> l := listset__rmv !l ent
  | READONLY _ -> anomaly "fthm: add_log"
;;


let map tm lbl = READONLY(mhm__map (snd(contents_of tm)) lbl);;

let xtract tm = !(fst (contents_of tm));;

let dom tm = mhm__dom (snd (contents_of tm));;

let in_dom tm lbl = mhm__in_dom (snd (contents_of tm)) lbl;;

let assure_arc tm lbl =
    let m = snd(contents_of tm)
    in if mhm__in_dom m lbl then ()
       else mhm__add m (lbl,NODE (mls__mt(),mhm__new 17))
;;

let rec at_path f = atrec
    where rec atrec tm = function
    [] -> f(fst(contents_of tm))
  | h::t -> (assure_arc tm h;atrec (map tm h) t)
;;

let add tm (path,v) =
    (match tm with READONLY _ -> anomaly "fthm: add" | _ -> ());
    at_path (fun hereset -> mls__add hereset v) tm path;
    add_log (path,v) tm
;;

let rmv tm (path,v) =
    (match tm with READONLY _ -> anomaly "fthm: add" | _ -> ());
    at_path (fun hereset -> mls__rmv hereset v) tm path;
    rmv_log (path,v) tm
;;

let app f tm = apprec [] tm
    where rec apprec pfx tm =
    let hereset = fst(contents_of tm) and
        m = snd(contents_of tm) in
    let path = rev pfx
    in (mls__app (fun v -> f(path,v)) hereset;
        mhm__app (fun (l,_) -> apprec (l::pfx) (map tm l)) m)
;;

let toList tm = torec [] tm
    where rec torec pfx tm =
    let hereset = fst(contents_of tm) and
        m = snd(contents_of tm) in
    let path = rev pfx
    in join((list__map (fun v -> (path,v)) (mls__toList hereset))::
            (list__map (fun l -> torec (l::pfx) (map tm l)) (mhm__dom m)))
;;

type ('a,'b) frozen_t == ('a list * 'b) list;;
let freeze = function
    WRITABLE(l,_) -> !l
  | READONLY _ -> anomaly "fthm: freeze"
;;

let unfreeze fl tm =
    match tm with
    READONLY _ -> anomaly "fthm: unfreeze"
  | WRITABLE(log,m) -> (log := [];
                        mhm__empty (snd(contents_of tm));
                        do_list (add tm) fl)
;;

let empty tm =
    match tm with
    READONLY _ -> anomaly "fthm: empty"
  | WRITABLE _ -> ((fst(contents_of tm)) := [];
                   mhm__empty (snd(contents_of tm)))
;;

(* $Id: fthm.ml,v 1.2 1996/02/12 15:41:16 cpaulin Exp $ *)
