(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               hashcons.ml                                *)
(****************************************************************************)
(* Hash consing of datastructures *)

(* The algorithm for hash-consing is simple.  We maintain a
   hash-table, indexed by "obj", and storing "obj".  The equality in
   this hash-table is structural.  Each time we hash-cons an object,
   we hash it into the table.  If we get a hit, we return the stored
   value.  If we do not get a hit, and the object is a vector, then we
   recursively hash-cons the sub-objects, updating them in place, and
   then store the object into the hash-table and return it.
 *)

type t == (obj__obj,obj__obj) mhm__t;;

let hctab = (mhm__new 569 : t);;

let init () =
    (mhm__empty hctab;
     mhm__set_maxlen hctab 1000000000)
;;

(*#define Num_tags (1 << 8)*)
let Num_tags = 256;;
let No_scan_tag = Num_tags - 4;;
let tuple_p obj = obj__is_block obj & (obj__obj_tag obj < No_scan_tag);;

let do_hash_cons = hcrec
    where rec hcrec o =
    try mhm__map hctab o
    with Not_found ->
        if tuple_p o then
            (let n = obj__obj_size o
             in
                 for i = 0 to pred n do
                   obj__set_obj_field o i (hcrec (obj__obj_field o i))
                 done);
        mhm__add hctab (o,o);
        o
;;

let hcons e = obj__magic_obj(do_hash_cons (obj__repr e));;
let hash1 c =
    init();
    let r = hcons c
    in (init();
        r)
;;

(* $Id: hashcons.ml,v 1.2 1996/02/12 15:41:21 cpaulin Exp $ *)
