(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                intavm.ml                                 *)
(****************************************************************************)

#open "std";;
#open "more_util";;

type 'b t == 'b intmap__t;;

let new () = intmap__empty ;;

let toList m =
let r = ref []
in (intmap__iter (fun k v -> r := (k,v)::!r) m;
    !r)
;;

let dom m = map fst (toList m);;
let rng m = map snd (toList m);;

let map m k = intmap__find k m;;

let in_dom m k =
    try (map m k;true)
    with Not_found -> false
;;

let rmv m k = intmap__remove k m;;

let add m (k,v) =
    if in_dom m k then raise (Failure "intavm__add")
    else intmap__add k v m
;;

let remap m k v = (add (rmv m k) (k,v));;

let app f m = intmap__iter (fun k v -> (f(k,v))) m;;

let ofList l = it_list add (new()) l;;

let inv l b = list__map fst (filter (fun (a',b') -> b' = b) (toList l));;

(* $Id: intavm.ml,v 1.2 1996/02/12 15:41:29 cpaulin Exp $ *)
