(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                intmap.ml                                 *)
(****************************************************************************)
(* Association tables over int *)

#open "int";;
#open "eq";;
#open "list";;
#open "baltree";;

type 'b binding =
  { key: int; data: 'b; prev: 'b list };;

type 'b t == 'b binding baltree__t;;

let order x y = (prefix -) x y.key;;

let empty = Empty;;

let bind x y b =
  Something
    { key = x;
      data = y;
      prev = match b with Nothing -> [] | Something b -> b.data :: b.prev };;

let add x y m =
  baltree__modify (order x) (bind x y) m;;

let find x m =
  (baltree__find (order x) m).data;;

let unbind = function
    Something({prev = x::l} as b) ->
      Something { key = b.key; data = x; prev = l }
  | _ -> Nothing;;

let remove x m =
  baltree__modify (order x) unbind m;;

let iter f m =
  let rec iter = function
    Empty -> ()
  | Node(l, b, r, _) ->
      iter l;
      f b.key b.data;
      do_list (f b.key) b.prev;
      iter r
  in iter m;;

(* $Id: intmap.ml,v 1.2 1996/02/12 15:41:39 cpaulin Exp $ *)
