(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 lavm.ml                                  *)
(****************************************************************************)

type 'a refbit = {v : 'a;
                  mutable rb : bool};;

type ('a,'b) t == ('a,'b refbit) avm__t;;

let new ord = avm__new ord;;

let deref rb =
    (rb.rb <- true;
     rb.v)
;;

let mk v' = {v=v'; rb = false};;
let mkref v' = {v=v'; rb = true};;

let upd rb v' =
    {rb = rb.rb;
     v = v'}
;;

let map m k = deref(avm__map m k);;

let dom m = avm__dom m;;

let in_dom m k = avm__in_dom m k;;

let add m (k,v) = avm__add m (k,mk v);;

let log_add m (k,v) = avm__add m (k,mkref v);;

let rmv m k = avm__rmv m k;;

let remap m k v' =
    let rb = avm__map m k
    in avm__remap m k (upd rb v')
;;

let app f m =
avm__app (fun (k,rb) -> f(k,deref rb)) m
;;

let toList m =
    list__map (fun (k,rb) -> (k,deref rb)) (avm__toList m);;

let nolog_toList m =
    list__map (fun (k,rb) -> (k,rb.v)) (avm__toList m);;

let read_log m =
let l = avm__toList m
in std__map_succeed (fun (k,rb) -> if rb.rb then k else failwith "caught") l
;;

let clear_log m =
    avm__app (fun (k,rb) -> rb.rb <- false) m
;;

(* $Id: lavm.ml,v 1.2 1996/02/12 15:41:48 cpaulin Exp $ *)
