(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 lavm.mli                                 *)
(****************************************************************************)

(* This module implements association tables where readings are logged. *)

(* The function read_log gives the list of a such as an association
(a,b) as been stored in the table, and that the association of a has
been accessed. clear_log forgets all accesses to the associations *)

type ('a,'b) t;;

value new : ('a -> 'a -> int) -> ('a,'b) t;;
value map : ('a,'b) t -> 'a -> 'b;;
value dom : ('a,'b) t -> 'a list;;(* does not log*)
(*value rng : ('a,'b) t -> 'b list;;*)
value in_dom : ('a,'b) t -> 'a -> bool;;(*does not log*)
value add : ('a,'b) t -> 'a * 'b -> ('a,'b) t;;
value log_add : ('a,'b) t -> 'a * 'b -> ('a,'b) t;;(* add and logs *)
value rmv : ('a,'b) t -> 'a -> ('a,'b) t;;(* forgets an association *)
value remap : ('a,'b) t -> 'a -> 'b -> ('a,'b) t;;(* same log value kept*)
value app : (('a * 'c) -> 'b) -> ('a,'c) t -> unit;;(* logs every access *)
value toList : ('a,'b) t -> ('a * 'b) list;;(* logs every access *)
value nolog_toList : ('a,'b) t -> ('a * 'b) list;;(* does not log accesses*)
(*value ofList : ('a -> 'a -> int) -> ('a * 'b) list -> ('a,'b) t;;*)
(*value inv : ('a,'b) t -> 'b -> 'a list;;*)

(*value freeze : ('a,'b) t -> ('a,'b) frozen_t;;
value unfreeze : ('a,'b) frozen_t -> ('a,'b) t -> ('a,'b) t;;*)

(*value clone_empty : ('a,'b) t -> ('a,'b) t;;*)

value read_log : ('a,'b) t -> 'a listset__t;;(* gives the accessed elements *)
value clear_log : ('a,'b) t -> unit;;(* clear all logs*)

(* $Id: lavm.mli,v 1.2 1996/02/12 15:41:50 cpaulin Exp $ *)
