(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                listset.ml                                *)
(****************************************************************************)
#open "std";;
#open "more_util";;

let rec uniquize = function
    [] -> []
  | (h::t) -> if mem h t then uniquize t else h::(uniquize t)
;;

let add fs a = if mem a fs then fs else (a::fs)
;;
let rec rmv = fun
    (h::t) a -> if h = a then t else h::(rmv t a)
  | _ _ -> failwith "listset__rmv"
;;
let memb ls a = mem a ls
;;
let choose P fs = hd(filter P fs)
;;
let intersect l1 l2 = filter (fun x -> mem x l2) l1
;;

let union l1 l2 =
  let htab = hashtbl__new 151 in
  let rec add_elt l = function
      [] -> l
    | x::ll -> try hashtbl__find htab x; add_elt l ll
               with Not_found -> hashtbl__add htab x ();
                                 add_elt (x::l) ll
  in 
  let l' = add_elt [] l1
  in add_elt l' l2
;;

let diff l1 l2 = filter (fun x -> not (mem x l2)) l1
;;
let symdiff l1 l2 = diff(union l1 l2) (intersect l1 l2)
;;

let subset l1 l2 =
  let t2 = hashtbl__new 151 in
  do_list (fun x-> hashtbl__add t2 x ()) l2;
  let rec look = function
     [] -> true
   | x::ll -> try hashtbl__find t2 x; look ll
              with Not_found -> false
  in look l1
;;

let equal s1 s2 = subset s1 s2 & subset s2 s1
;;

let mt = [];;
let ofList l = it_list add mt l;;
let toList l = l;;
let is_empty = function [] -> true | _ -> false;;

let app f l = do_list f l;;

(* $Id: listset.ml,v 1.2 1996/02/12 15:42:06 cpaulin Exp $ *)
