(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  mhm.ml                                  *)
(****************************************************************************)

type ('a,'b) t == ('a,'b) hashtbl__t;;
type ('a,'b) frozen_t == ('a,'b) hashtbl__frozen_t;;

let new = hashtbl__new;;
let empty = hashtbl__clear;;
let map = hashtbl__find;;

let in_dom m k =
    try (map m k;true) with Not_found -> false;;

let add m (k,v) =
    if not in_dom m k then
        hashtbl__add m k v
    else failwith "mhm__add";;

let rmv = hashtbl__remove;;
let app f m = hashtbl__do_table (fun k v -> f(k,v)) m;;
let freeze = hashtbl__freeze;;
let unfreeze = hashtbl__unfreeze;;
let dom m =
    let l = ref []
    in app (fun (k,_) -> if not(mem k !l) & in_dom m k then l := k::!l) m;
       !l
;;

let toList m =
let l = dom m
in list__map (fun k -> (k,map m k)) l
;;

let remap m k v =
    rmv m k;
    add m (k,v)
;;

let set_maxlen t n = hashtbl__set_maxlen t n;;

(* $Id: mhm.ml,v 1.2 1996/02/12 15:42:34 cpaulin Exp $ *)
