(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  mhs.ml                                  *)
(****************************************************************************)

#open "std";;
#open "more_util";;

(* Mutable Hash-Summaries *)

type ('a,'b) t = {tab : ('a,'b list) mhm__t ;
                  mutable stk : ('a * 'b) list;
                  ord : ('b -> 'b -> int) option};;

type ('a,'b) frozen_t == ('a,'b list) mhm__frozen_t;;

let new (n,ord) =
    {tab = mhm__new n;
     stk = [];
     ord = ord};;

let empty s =
    (if s.stk <> [] then
         anomaly "cannot be emptying a summary which contains locally pushed values");
    mhm__empty s.tab
;;

let map s k = mhm__map s.tab k;;

let rec insert ordopt v l =
    match ordopt with
    NONE -> v::l
  | SOME ord -> (insrec l
                 where rec insrec = function
                 [] -> [v]
               | h::tl -> if ord v h <= 0 then v::h::tl
                          else h::(insrec tl))
;;

let add s (k,v) =
    (if s.stk <> [] then
         anomaly "cannot be adding to a summary which contains locally pushed values");
    try let oval = mhm__map s.tab k
        in mhm__rmv s.tab k;
           mhm__add s.tab (k,insert s.ord v oval)
    with Not_found -> mhm__add s.tab (k,[v])
;;

let rmv s (k,v) =
    anomaly "mutable hash-summaries do not allow removal"
;;

let app f s = mhm__app f s.tab;;

let in_dom s k = mhm__in_dom s.tab k;;

let freeze s =
    (if s.stk <> [] then
         anomaly "cannot be emptying a summary which contains locally pushed values");
    mhm__freeze s.tab;;

let unfreeze fs s =
    (if s.stk <> [] then
         anomaly "cannot be emptying a summary which contains locally pushed values");
    mhm__unfreeze fs s.tab;;

let push s (k,v) =
    s.stk <- (k,v)::s.stk;
    try let oval = mhm__map s.tab k
        in mhm__rmv s.tab k;
           mhm__add s.tab (k,insert s.ord v oval)
    with Not_found -> mhm__add s.tab (k,[v])
;;

let pop s kopt =
let k = (match kopt with NONE -> fst(hd s.stk) | SOME k -> k)
in (if k <> fst(hd s.stk) then
    anomaly "the summary's local-stack is out of sync with its user");
   let oval = mhm__map s.tab k in
   let nval = except (snd (hd s.stk)) oval
   in mhm__rmv s.tab k;
      mhm__add s.tab (k,nval);
      s.stk <- tl s.stk
;;

let pop_all s =
    while s.stk <> [] do
        pop s NONE
    done
;;

let dom s = mhm__dom s.tab;;
let toList s = mhm__toList s.tab;;

(* $Id: mhs.ml,v 1.2 1996/02/12 15:42:45 cpaulin Exp $ *)
