(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 mhs.mli                                  *)
(****************************************************************************)
#open "more_util";;

type ('a,'b) t;;
type ('a,'b) frozen_t;;

(* All destructive operations below are only allowed when the local-push
 * stack is empty
 *)

value new : (int * ('b -> 'b -> int) option) -> ('a,'b) t;;
value empty : ('a,'b) t -> unit;;
value map : ('a,'b) t -> 'a -> 'b list;;
value add : ('a,'b) t -> 'a * 'b -> unit;;
value rmv : ('a,'b) t -> 'a * 'b -> unit;;
value app : (('a * 'b list) -> 'c) -> ('a,'b) t -> unit;;
value in_dom : ('a,'b) t -> 'a -> bool;;
value dom : ('a,'b) t -> 'a list;;
value toList : ('a,'b) t -> ('a * 'b list) list;;

(*
value remap : ('a,'b) t -> 'a -> 'b -> unit;;
value rng : ('a,'b) t -> 'b list;;
value inv : ('a,'b) t -> 'b -> 'a list;;
*)

value freeze : ('a,'b) t -> ('a,'b) frozen_t;;
value unfreeze : ('a,'b) frozen_t -> ('a,'b) t -> unit;;

(* the option on "pop" is so that we can ask to check
 * that what we are popping is what we originally pushed,
 * to assure that we are not by chance out of sync.
 *)
value push : ('a,'b) t -> 'a * 'b -> unit;;
value pop : ('a,'b) t -> 'a option -> unit;;
value pop_all : ('a,'b) t -> unit;;



(* $Id: mhs.mli,v 1.2 1996/02/12 15:42:47 cpaulin Exp $ *)
