(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 std.mli                                  *)
(****************************************************************************)

#open "pp";;
#open "stdpp";;

exception Anomaly of string * std_ppcmds;;  (* System errors *)
value anomaly : string -> 'a;;
value anomalylabstrm : string -> std_ppcmds -> 'a;;

exception UserError of string * std_ppcmds;; (* User errors *)

value error : string -> 'a;;
value errorlabstrm : string -> std_ppcmds -> 'a;;

value enforce : string -> ('a -> bool) -> 'a -> unit;;
value enforce_error : string -> ('a -> bool) -> 'a -> unit;;

type 'a nref = {mutable v : 'a};;
value nref : 'a -> 'a nref;;
value nref_app : ('a -> 'b) -> 'a nref -> 'b nref;;
value forward : 'a -> 'b;;
value o : ('a -> 'b) -> ('c -> 'a) -> 'c -> 'b;;
value length : 'a list -> int;;
value num_of_string : string -> int;;
value string_of_num : int -> string;;
value date : string -> string;;
value succ_int : int -> int;;
value modify_vect : ('a -> 'a) -> 'a vect -> unit;;
value explode : string -> string list;;
value implode : string list -> string;;
value eq_string : string * string -> bool;;
value explode_ascii : string -> int list;;
value item : 'a list -> int -> 'a;;
value tl : 'a list -> 'a list;;
value hd : 'a list -> 'a;;
value car : 'a list -> 'a;;
value cdr : 'a list -> 'a list;;
value cddr : 'a list -> 'a list;;
value cdddr : 'a list -> 'a list;;
value safe_cdr : 'a list -> 'a list;;
value safe_cddr : 'a list -> 'a list;;
value safe_cdddr : 'a list -> 'a list;;
value cadr : 'a list -> 'a;;
value null : 'a list -> bool;;
value except_assoc : 'a -> ('a * 'b) list -> ('a * 'b) list;;
value append : 'a list -> 'a list -> 'a list;;
value union : 'a list -> 'a list -> 'a list;;
value try_find : ('a -> 'b) -> 'a list -> 'b;;
value try_find_i : (int -> 'a -> 'b) -> int -> 'a list -> 'b;;
value index : 'a -> 'a list -> int;;
value curry : ('a * 'b -> 'c) -> 'a -> 'b -> 'c;;
value for_all_i : (int -> 'a -> bool) -> int -> 'a list -> bool;;
value for_all2 : ('a -> 'b -> bool) -> 'a list -> 'b list -> bool;;
value for_all2eq : ('a -> 'b -> bool) -> 'a list -> 'b list -> bool;;
value gt : int * int -> bool;;
value eq : 'a * 'a -> bool;;
exception Identity;;
value share : ('a -> 'a) -> 'a -> 'a;;
value filter : ('a -> bool) -> 'a list -> 'a list;;
value neg : ('a -> bool) -> 'a -> bool;;
value I : 'a -> 'a;;
value C : ('a -> 'b -> 'c) -> 'b -> 'a -> 'c;;
value make_set : 'a list -> 'a list;;
value subtractq : 'a list -> 'a list -> 'a list;;
value distinct : 'a list -> bool;;
value chop_list : int -> 'a list -> 'a list * 'a list;;
value unionq : 'a list -> 'a list -> 'a list;;
value iterate : ('a -> 'a) -> int -> 'a -> 'a;;
value nth : 'a list -> int -> 'a;;
value map : ('a -> 'b) -> 'a list -> 'b list;;
value map_i : (int -> 'a -> 'b) -> int -> 'a list -> 'b list;;
value map2_i : (int -> 'a -> 'b -> 'c) -> int -> 'a list -> 'b list -> 'c list;;
value app_i : (int -> 'a -> 'b) -> int -> 'a list -> unit;;
value it_list_i : (int -> 'a -> 'b -> 'a) -> int -> 'a -> 'b list -> 'a;;
value sort : ('a * 'a -> bool) -> 'a list -> 'a list;;
value ascii_code : string -> int;;
value ascii : int -> string;;
value rev_append : 'a list -> 'a list -> 'a list;;
value it_vect : ('a -> 'b -> 'a) -> 'a -> 'b vect -> 'a;;
value K : 'a -> 'b -> 'a;;
value first_n : int -> 'a list -> 'a list;;
value flat : 'a list list -> 'a list;;
value last : 'a list -> 'a;;
value sep_last : 'a list -> 'a * 'a list;;
value words : string -> string list;;
value scan_string : string -> string -> int -> int;;
value interval : int -> int -> int list;;
value range : int -> int list;;
value add_set : 'a -> 'a list -> 'a list;;
value filter_pos : ('a -> bool) -> 'a list -> 'a list;;
value intersect : 'a list -> 'a list -> 'a list;;
value hash_clear : 'a list vect -> unit;;
value old_vect_item : 'a vect * int -> 'a;;
value old_vect_assign : 'a vect * int * 'a -> 'a;;
value print_flush : unit -> unit;;
value length_string : string -> int;;
value input_char : in_channel -> char;;
value set_prompt : (unit -> string) -> unit;;
value get_prompt : unit -> (unit -> string);;
exception break;;
value open_append : string -> out_channel;;
value hash_assoc : 'a -> ('a * 'b) list vect -> 'b;;
value hash_remove_assoc : 'a * 'b -> ('a * 'b) list vect -> unit;;
value hash_add_assoc : 'a * 'b -> ('a * 'b) list vect -> unit;;
value min : int -> int -> int;;
value max : int -> int -> int;;
value repeat : int -> ('a -> 'b) -> 'a -> unit;;
value break_string : string -> string -> string list;;
value skip_string : char list -> string -> string;;
value skip_space_return : string -> string;;
value first_n_string : int -> string -> string;;
value last_n_string : int -> string -> string;;
value fst3 : ('a * 'b * 'c) -> 'a;;
value map_succeed : ('a -> 'b) -> 'a list -> 'b list;;
value map_succeed_match : ('a -> 'b) -> 'a list -> 'b list;;
value make_set : 'a list -> 'a list;;
value caml_assoc : 'a -> ('a * 'b) list -> 'b;;
value do_listRL : ('a -> unit) -> 'a list -> unit;;
value do_listLR : ('a -> unit) -> 'a list -> unit;;
value join : 'a list list -> 'a list;;
value join_map_list : ('a -> 'b list) -> 'a list -> 'b list;;
value firstn : int -> 'a list -> 'a list;;
value lastn : int -> 'a list -> 'a list;;
value nthtail0 : int -> 'a list -> 'a list;;
value push : 'a list nref -> 'a -> unit;;
value pop : 'a list nref -> unit;;
value top : 'a list nref -> 'a;;
value truncate : int -> 'a list nref -> unit;;

type 'a lazy;;
value delay : (unit -> 'a) -> 'a lazy;;
value force : 'a lazy -> 'a;;

type ('a,'b) union = inl of 'a | inr of 'b;;
value outl : ('a,'b) union -> 'a;;
value outr : ('a,'b) union -> 'b;;

value prefix_of : 'a list -> 'a list -> bool;;
value lexico : ('a -> 'a -> int) -> ('a list -> 'a list -> int);;
value rev_lexico : ('a -> 'a -> int) -> ('a list -> 'a list -> int);;

value union_exn : ('a -> 'b) -> 'a -> ('b,exn) union;;
value compare : string -> ('b -> 'b -> bool) -> ('a -> stdpp__std_ppcmds) -> ('a -> 'b) -> ('a -> 'b) -> 'a -> 'b;;

value tabulate_list : (int -> 'b) -> int -> 'b list;;
value list_assign : 'a list -> int -> 'a -> 'a list;;
value bool_ord : bool -> bool -> int;;

(* $Id: std.mli,v 1.2 1996/02/12 15:43:15 cpaulin Exp $ *)
