(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                system.ml                                 *)
(****************************************************************************)
#open "std";;
#open "more_util";;
#open "sys";;
#open "unix";;
#infix "o";;
#open "pp";;
#open "stdpp";;

(* Random Global Variables *)
let LOADPATH = ref ([]:string list);;

let add_path dir =
    if not(listset__memb (!LOADPATH) dir) then
        LOADPATH := listset__add (!LOADPATH) dir
;;

let del_path dir =
    if not(listset__memb (!LOADPATH) dir) then
        error (dir ^ " not on loadpath")
    else LOADPATH := listset__rmv (!LOADPATH) dir
;;

let search_paths () = !LOADPATH
;;

let safe_getenv n =
    try getenv n with Not_found -> "$" ^ n
;;

let safe_getlogin () =
    try getlogin()
    with _ -> (getpwuid(getuid())).pw_name
;;

let glob fname =
    let get_revexp_env n =
        rev(explode_chars(safe_getenv n)) in
    let get_revexp_pwdir n =
        rev(explode_chars(getpwnam n).pw_dir) in
    let rec aux sofar = function
        [< '`/` ;
         (function
          [< '`~` ;
           (function
            [< p_atom n >] -> n
          | [< >] -> safe_getlogin()) uname ;
           s >] -> aux (get_revexp_pwdir uname) s
        | [< s >] -> aux (`/`::sofar) s) rslt >] -> rslt
      | [< '`$` ; p_atom a ; s >] ->
        aux ((get_revexp_env a)@sofar) s
      | [< 'c ; s >] -> aux (c::sofar) s
      | [< >] -> sofar
    in
        (implode_chars o rev)
        ((function
          [< '`~` ; s >] -> aux [] [< '`/` ; '`~` ; s >]
        | [< s >] -> aux [] s)
         (string_to_char_stream fname))
;;

let open_trapping_failure open_fun name suffix =
    try open_fun (glob (name^suffix))
    with _ -> error("Can't open " ^ name)
;;

let open_with_suffix open_fun name suffix =
    try open_fun (glob(name ^ suffix))
    with _ -> try open_fun (glob name)
              with _ -> error("Can't open " ^ name)
;;


let file_readable_p na =
    try access (glob na) [R_OK];true
    with Unix_error _ -> false
;;

let open_in_maybe_compressed s =
    try open_in s
    with e -> (if file_readable_p (s^".z") then
                   open_process_in ("gzcat "^s^".z")
               else raise e)
;;

(* Rendu portable entre Unix/Mac/PC par emploi judicieux
   du module filename. -XL. *)

let open_with_suffix_from_path open_fun path name suffix =
    (if filename__is_absolute (glob name) then
        open_with_suffix open_fun name suffix
    else open_rec path)
where rec open_rec = function
(dir :: rest) -> (try open_with_suffix open_fun (filename__concat dir name) suffix
                  with _ -> open_rec rest)
| [] -> error ("Can't find file " ^ name ^ " on loadpath")
;;
let timestamp () = (time(),times());;
let fmt_time_difference (startreal,start) (stopreal,stop) =
    [< 'INT(stopreal - startreal); 'S" secs ";
       'S"(";
       'REAL(sub_float stop.tms_utime start.tms_utime); 'S"u";
       'S",";
       'REAL(sub_float stop.tms_stime start.tms_stime); 'S"s";
       'S")" >]
;;

(* devraient etre dans unix.ml *)

let mode_append = 16;;
let l_xtnd = 2;;

(* *)

let comline c =
  let child = fork () in
  if child < 0 then
    prerr_endline "fork failed"
  else if child > 0 then (wait ();())
  else (
    execv "/bin/sh" [| "sh"; "-c"; c |];
    prerr_endline "execv failed";
    exit 1
  )
;;

(***
let Noisy = ref false
;;

let cocorico() = 
  try (if (getenv "ARCH") = "sun4" & !Noisy
       then comline ("cp " ^ (filename__concat (getenv "COQTOP") 
                     (filename__concat "RELEASED" 
                     (filename__concat (getenv "ARCH") "rooster.au"))) 
                     ^ " /dev/audio")
       else ())
  with _ -> ()
;;
***)

let (extern_intern :
      	(int * string) -> ((string -> 'a -> unit) * (string -> 'a)))
    = function (magic,suffix) ->

let extern_state name val = 
  try
  let (expname,channel) =
        open_trapping_failure (fun n -> n,open_out n) name suffix in
  try
    output_binary_int channel magic;
    output_compact_value channel val;
    close_out channel
  with
  e -> ((try remove expname with _ -> PPNL
                              [< 'S"Warning: " ; 'S"Could not remove file " ;
                                 'S expname ; 'S" which is corrupted !!" >]);
       raise e)
  with sys__Sys_error s -> error("System error: " ^ s)

and intern_state name = 
  try
  let channel = open_with_suffix_from_path open_in_maybe_compressed 
      	       	       	       	       	   (!LOADPATH) name suffix in
  begin try
    if input_binary_int channel <> magic then error "Bad magic number";
    let v = input_value(channel)
    in (close_in channel;
        v)
  with sys__Sys_error s -> error("System error: " ^ s)
  end
  with sys__Sys_error s -> error("System error: " ^ s)

in (extern_state,intern_state)
;;

(* $Id: system.ml,v 1.3 1996/02/12 15:43:17 cpaulin Exp $ *)
