(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  tlm.ml                                  *)
(****************************************************************************)
#open "std";;

type ('a,'b) t =
    NODE of 'b listset__t * ('a,('a,'b) t) listmap__t;;

let new () = NODE(listset__mt,listmap__new());;

let map (NODE tm) lbl = listmap__map (snd tm) lbl;;
let xtract (NODE tm) = fst tm;;
let dom (NODE tm) = listmap__dom (snd tm);;
let in_dom (NODE tm) lbl = listmap__in_dom (snd tm) lbl;;

let is_empty_node = function
    NODE([],[]) -> true
  | _ -> false
;;

let assure_arc m lbl =
    if listmap__in_dom m lbl then m
    else listmap__add m (lbl,NODE (listset__mt,listmap__new()))
;;

let cleanse_arcs (NODE (hereset,m)) =
let l = listmap__rng m
in NODE(hereset,if for_all is_empty_node l then listmap__new() else m)
;;

let rec at_path f (NODE (hereset,m)) = function
    [] -> cleanse_arcs(NODE(f hereset,m))
  | h::t ->
    let m = assure_arc m h
    in cleanse_arcs(NODE(hereset,
                         listmap__remap m h (at_path f (listmap__map m h) t)))
;;

let add tm (path,v) =
    at_path (fun hereset -> listset__add hereset v) tm path
;;

let rmv tm (path,v) =
    at_path (fun hereset -> listset__rmv hereset v) tm path
;;

let app f (NODE tm) = apprec [] (NODE tm)
    where rec apprec pfx (NODE(hereset,m)) =
    let path = rev pfx
    in (listset__app (fun v -> f(path,v)) hereset;
        listmap__app (fun (l,tm) -> apprec (l::pfx) tm) m)
;;

let toList (NODE tm) = torec [] (NODE tm)
    where rec torec pfx (NODE(hereset,m)) =
    let path = rev pfx
    in join((list__map (fun v -> (path,v)) (listset__toList hereset))::
            (list__map (fun (l,tm) -> torec (l::pfx) tm) (listmap__toList m)))
;;

(* $Id: tlm.ml,v 1.2 1996/02/12 15:43:29 cpaulin Exp $ *)
