(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                vectops.ml                                *)
(****************************************************************************)
#open "std";;

let vect_it f v a = vect_it_f 0
 where rec vect_it_f n =
  if n >= vect_length v then a else f v.(n) (vect_it_f (succ n))
;;

let vect_it2 f v1 v2 a =
if not(vect_length v1 = vect_length v2) then invalid_arg "vect_it2"
else vect_it_f 0
 where rec vect_it_f n =
  if n >= vect_length v1 then a else f v1.(n) v2.(n) (vect_it_f (succ n))
;;

let it_vect f a v = it_vect_f a 0
 where rec it_vect_f a n =
  if n >= vect_length v then a
   else it_vect_f (f a v.(n)) (succ n)
;;

let it_vect2 f a v1 v2 =
if not(vect_length v1 = vect_length v2) then invalid_arg "it_vect2"
else it_vect_f a 0
 where rec it_vect_f a n =
  if n >= vect_length v1 then a
   else it_vect_f (f a v1.(n) v2.(n)) (succ n)
;;

let vlist_length l =
    it_list (fun sofar v -> sofar+(vect_length v)) 0 l
;;

let join_vect l =
if l = [] then vect_of_list[] else
let len = vlist_length l in
if len = 0 then vect_of_list[] else
let result = make_vect len (hd l).(0) in
let rec aux n = function
    [] -> ()
  | h::t -> let vlen = vect_length h
            in (blit_vect h 0 result n (vect_length h);
                aux (n+vlen) t)
in aux 0 l; result
;;

let vlitem l n = if n<0 then failwith "item" else
 (item_op (l,n) where rec item_op = 
   fun (hd::t,n) -> let hdlen = vect_length hd
                    in if n < hdlen then hd.(n)
                       else item_op(t,n-hdlen)
   |   ([],_) -> failwith "item");;

let vlnth l n = vlitem l (pred n);;

let chop_vect n v =
    let vlen = vect_length v
    in if n > vlen  then failwith "chop_vect"
    else (sub_vect v 0 n,sub_vect v n (vlen-n))
;;

let safe_chop_vect n v =
    let vlen = vect_length v
    in if n > vlen  then (v,inr(n-vlen))
    else (sub_vect v 0 n,inl(sub_vect v n (vlen-n)))
;;

let chop_vect_list = (fun n l -> chop_aux n ([],l))
 where rec chop_aux =
  fun 0 (l1,l2) -> rev l1,l2
    | _ (_,[]) -> failwith "chop_list"
    | n (l1,h::t) ->
      match safe_chop_vect n h with
      (v,inl v') -> v::(rev l1),v'::t
    | (v,inr more) -> chop_aux more (v::l1,t)
;;

let exists_vect f v = exrec ((vect_length v)-1) where rec exrec = function
    (-1) -> false
    | n -> (f v.(n)) or (exrec (n-1))
;;

let exists2_i_vect_LR f start v1 v2 =
if vect_length v1 <> vect_length v2 then
    invalid_arg "exists2_i_LR"
else (exrec 0 where rec exrec n =
      if n = vect_length v1 then false
      else f (start+n) v1.(n) v2.(n) or exrec (n+1))
;;

let exists2_vect_LR f v1 v2 =
if vect_length v1 <> vect_length v2 then
    invalid_arg "exists2_i_LR"
else (exrec 0 where rec exrec n =
      if n = vect_length v1 then false
      else f v1.(n) v2.(n) or exrec (n+1))
;;

let for_all_vect f v = allrec ((vect_length v)-1) where rec allrec = function
    (-1) -> true
    | n -> (f v.(n)) & (allrec (n-1))
;;

let for_all_vect_i f base v = allrec ((vect_length v)-1) where rec allrec = function
    (-1) -> true
    | n -> (f (base+n) v.(n)) & (allrec (n-1))
;;

let for_all2eq_vect f v1 v2 =
  let vlv1 = (vect_length v1)
  in
    vlv1 = (vect_length v2)
    & allrec (pred vlv1) where rec allrec = function
    (-1) -> true
    | n -> (f v1.(n) v2.(n)) & (allrec (n-1))
;;

let list_of_vectlist vl =
let rec aux = function
    [] -> []
  | v::vl -> (list_of_vect v)@(aux vl)
in aux vl
;;

let vstack1 v =
    if vect_length v = 0 then [] else [(v,0)]
;;

let push_vstack v vs =
    if vect_length v = 0 then vs
    else (v,0)::vs
;;

let vstack_empty = function
    [] -> true
    | _ -> false
;;

let pop_vstack = function
    (v,n)::vs ->
    (v.(n),if n = (vect_length v) then vs else (v,n+1)::vs)
  | _ -> invalid_arg "pop_vstack"
;;

let vstack_length l =
    it_list (fun sofar (v,n) -> sofar+(vect_length v)-n) 0 l
;;

let safe_chop_vfrag n (v,start) =
    let vlen = (vect_length v)-start
    in if n > vlen  then ((v,start),inr(n-vlen))
    else ((sub_vect v start n,0),inl(v,n+start))
;;

let chop_vstack = (fun n l -> chop_aux n ([],l))
 where rec chop_aux =
  fun 0 (l1,l2) -> rev l1,l2
    | _ (_,[]) -> failwith "chop_list"
    | n (l1,h::t) ->
      match safe_chop_vfrag n h with
      (v,inl v') -> v::(rev l1),v'::t
    | (v,inr more) -> chop_aux more (v::l1,t)
;;

let join_vstack l =
if l = [] then vect_of_list[] else
let len = vstack_length l in
let result = make_vect len (fst(hd l)).(0) in
let rec aux n = function
    [] -> ()
  | (h,st)::t -> let vlen = (vect_length h)-st
            in (blit_vect h st result n (vect_length h);
                aux (n+vlen) t)
in aux 0 l; result
;;

let tabulate_vect f n =
    if n = 0 then vect_of_list []
    else
        let zeroval = f 0 in
        let rslt = make_vect n zeroval in
        let rec aux = function
            0 -> ()
          | n -> (rslt.(n) <- (f n);
                  aux (n-1))
        in aux (n-1);rslt
;;

let rec map_vstack f = function
    [] -> []
    | (v,n)::l ->
      (tabulate_vect (fun i -> f v.(n+i)) ((vect_length v)-n),0)::(map_vstack f l)
;;

let rec map_i_vstack f startnum = map_rec startnum where rec map_rec idx = function
    [] -> []
    | (v,n)::l ->
      let vlen = ((vect_length v)-n)
      in (tabulate_vect (fun i -> f (idx+i) v.(n+i)) vlen,0)::(map_rec (idx+vlen) l)
;;

let rec map_vstack_list f = function
    [] -> []
    | vs -> let e,rs = pop_vstack vs
            in (f e)::(map_vstack_list f rs)
;;

let for_all_vstack f = allrec where rec allrec = function
    [] -> true
    | vs -> let (e,rs) = pop_vstack vs
            in f e & (allrec rs)
;;

let for_all2eq_vstack f = allrec where rec allrec = fun
    [] [] -> true
  | [] vs2 -> vs2=[]
  | vs1 [] -> vs1=[]
  | vs1 vs2 -> let e1,rs1 = pop_vstack vs1
               and e2,rs2 = pop_vstack vs2
               in f e1 e2 & allrec rs1 rs2
;;

let concat_vstack vs1 vs2 = vs1@vs2;;

let app_vect v l =
    vect_it (fun e l -> e::l) v l
;;

let it_vect_from n f a v = it_vect_f a n
 where rec it_vect_f a n =
  if n >= vect_length v then a
   else it_vect_f (f a v.(n)) (succ n)
;;

let vect_it_from n f v a = vect_it_f n
 where rec vect_it_f n =
  if n >= vect_length v then a else f v.(n) (vect_it_f (succ n))
;;

let it_vect2_from n f a v1 v2 =
if not(vect_length v1 = vect_length v2) then invalid_arg "it_vect2"
else it_vect_f a n
 where rec it_vect_f a n =
  if n >= vect_length v1 then a
   else it_vect_f (f a v1.(n) v2.(n)) (succ n)
;;

let app_tl_vect v l =
if vect_length v = 0 then invalid_arg "app_tl_vect"
else vect_it_from 1 (fun e l -> e::l) v l
;;

let list_of_tl_vect v =
if vect_length v = 0 then invalid_arg "app_tl_vect"
else vect_it_from 1 (fun e l -> e::l) v []
;;

let app_vectpair (v1,v2) l =
    vect_it2 (fun e1 e2 l -> (e1,e2)::l) v1 v2 l
;;

let hd_vect v = v.(0);;
let tl_vect v = sub_vect v 1 ((vect_length v) - 1);;
let cons_vect e v2 = concat_vect [|e|] v2;;
let car_vect = hd_vect;;
let cdr_vect = tl_vect;;
let cadr_vect v = v.(1);;
let cddr_vect v = sub_vect v 2 ((vect_length v) - 2);;
let last_vect v = v.((vect_length v) - 1);;

let map2_vect f v1 v2 =
  if vect_length v1 <> vect_length v2 then
      invalid_arg "map2_vect"
  else if vect_length v1 == 0 then [| |] else begin
    let res = make_vect (vect_length v1) (f v1.(0)v2.(0)) in
      for i = 1 to pred(vect_length v1) do
        res.(i) <- f v1.(i) v2.(i)
      done;
      res
    end
;;

let map3_vect f v1 v2 v3 =
  if vect_length v1 <> vect_length v2 or vect_length v1 <> vect_length v3 then
      invalid_arg "map3_vect"
  else if vect_length v1 == 0 then [| |] else begin
    let res = make_vect (vect_length v1) (f v1.(0)v2.(0) v3.(0)) in
      for i = 1 to pred(vect_length v1) do
        res.(i) <- f v1.(i) v2.(i) v3.(i)
      done;
      res
    end
;;

let map4_vect f v1 v2 v3 v4 =
   if vect_length v1 <> vect_length v2 or
      vect_length v1 <> vect_length v3 or
      vect_length v1 <> vect_length v4 
   then invalid_arg "map4_vect"
   else if vect_length v1 == 0 then [| |] else begin
     let res = make_vect (vect_length v1) (f v1.(0)v2.(0) v3.(0) v4.(0)) in
       for i = 1 to pred(vect_length v1) do
         res.(i) <- f v1.(i) v2.(i) v3.(i) v4.(i)
      done;
      res
    end
;;

let map_i_vect f start v =
    tabulate_vect (fun i -> f (start+i) v.(i)) (vect_length v)
;;

let split_vect v =
    let n = vect_length v in
        (tabulate_vect (fun i -> fst v.(i)) n,
         tabulate_vect (fun i -> snd v.(i)) n)
;;

let combine_vect_list (v1,v2) =
    if vect_length v1 <> vect_length v2 then
        invalid_arg "combine_vect_list"
    else (comb 0 where rec comb n =
          if n = vect_length v1 then [] else (v1.(n),v2.(n))::(comb (n+1)))
;;

let map_tl_vect_list f v =
  let rec map i =
    if i >= vect_length v then [] else f v.(i) :: map (succ i)
  in
    map 1
;;

let first_vect P v =
    firstrec 0 where rec
    firstrec n =
    if n = vect_length v then raise Not_found
    else if P v.(n) then n
         else firstrec (n+1)
;;

let vect_index e v = indrec 0
    where rec indrec n =
    if n = vect_length v then failwith "vect_index"
    else if e = v.(n) then n+1 else indrec (succ n)
;;


let vect_index_pred P v = indrec 0
    where rec indrec n =
    if n = vect_length v then failwith "index_pred"
    else if P v.(n) then n
         else indrec (n+1)
;;

let first_result_vect_i f start v = let n = vect_length v in aux start
where rec aux k = if k > n then failwith "first_result_vect_i"
                  else  try (f k v.(k-1))
                        with Failure _ | UserError _ -> aux (k+1)
;;

(* $Id: vectops.ml,v 1.2 1996/02/12 15:43:33 cpaulin Exp $ *)
