(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               vectops.mli                                *)
(****************************************************************************)
#open "std";;

(* Functions on vectors *)

value vect_it : ('a -> 'b -> 'b) -> 'a vect -> 'b -> 'b;;
value vect_it2 : ('a -> 'b -> 'c -> 'c) -> 'a vect -> 'b vect -> 'c -> 'c;;
value it_vect : ('a -> 'b -> 'a) -> 'a -> 'b vect -> 'a;;
value it_vect2 : ('a -> 'b -> 'c -> 'a) -> 'a -> 'b vect -> 'c vect -> 'a;;
value vlnth : 'a vect list -> int -> 'a;;
value chop_vect : int -> 'a vect -> 'a vect * 'a vect;;
value safe_chop_vect : int -> 'a vect -> 'a vect * ('a vect, int) union;;
value exists_vect : ('a -> bool) -> 'a vect -> bool;;
value exists2_i_vect_LR : (int -> 'a -> 'b -> bool) -> int -> 'a vect -> 'b vect -> bool;;
value exists2_vect_LR : ('a -> 'b -> bool) -> 'a vect -> 'b vect -> bool;;
value for_all_vect : ('a -> bool) -> 'a vect -> bool;;
value for_all_vect_i : (int -> 'a -> bool) -> int -> 'a vect -> bool;;
value for_all2eq_vect : ('a -> 'b -> bool) -> 'a vect -> 'b vect -> bool;;
value tabulate_vect : (int -> 'a) -> int -> 'a vect;;
value app_vect : 'a vect -> 'a list -> 'a list;;
value it_vect_from : int -> ('a -> 'b -> 'a) -> 'a -> 'b vect -> 'a;;
value vect_it_from : int -> ('a -> 'b -> 'b) -> 'a vect -> 'b -> 'b;;
value it_vect2_from : int -> ('a -> 'b -> 'c -> 'a) -> 'a -> 'b vect -> 'c vect -> 'a;;
value app_tl_vect : 'a vect -> 'a list -> 'a list;;
value list_of_tl_vect : 'a vect -> 'a list;;
value app_vectpair : 'a vect * 'b vect -> ('a * 'b) list -> ('a * 'b) list;;
value hd_vect : 'a vect -> 'a;;
value tl_vect : 'a vect -> 'a vect;;
value cons_vect : 'a -> 'a vect -> 'a vect;;
value car_vect : 'a vect -> 'a;;
value cdr_vect : 'a vect -> 'a vect;;
value cadr_vect : 'a vect -> 'a;;
value cddr_vect : 'a vect -> 'a vect;;
value last_vect : 'a vect -> 'a;;
value map2_vect : ('a -> 'b -> 'c) -> 'a vect -> 'b vect -> 'c vect;;
value map3_vect : ('a -> 'b -> 'c -> 'd) -> 'a vect -> 'b vect -> 'c vect -> 'd vect;;
value map4_vect : ('a -> 'b -> 'c -> 'd -> 'e) -> 'a vect -> 'b vect -> 'c vect -> 'd vect -> 'e vect;;
value map_i_vect : (int -> 'a -> 'b) -> int -> 'a vect -> 'b vect;;
value split_vect : ('a * 'b) vect -> 'a vect * 'b vect;;
value combine_vect_list : 'a vect * 'b vect -> ('a * 'b) list;;
value map_tl_vect_list : ('a -> 'b) -> 'a vect -> 'b list;;
value first_vect : ('a -> bool) -> 'a vect -> int;;
value vect_index : 'a -> 'a vect -> int;;

(* Functions on lists of vectors *)
type 'a vectlist == 'a vect list;;
value join_vect : 'a vect list -> 'a vect;;
value vlist_length : 'a vect list -> int;;
value chop_vect_list : int -> 'a vect list -> 'a vect list * 'a vect list;;
value vlitem : 'a vect list -> int -> 'a;;
value list_of_vectlist : 'a vect list -> 'a list;;

(* Functions on lists of vector fragments *)
type 'a vectstack == ('a vect * int) list;;
value vstack1 : 'a vect -> ('a vect * int) list;;
value push_vstack : 'a vect -> ('a vect * int) list -> ('a vect * int) list;;
value vstack_empty : 'a list -> bool;;
value pop_vstack : ('a vect * int) list -> 'a * ('a vect * int) list;;
value vstack_length : ('a vect * int) list -> int;;
value safe_chop_vfrag : int -> 'a vect * int -> ('a vect * int) * ('a vect * int, int) union;;
value chop_vstack : int -> ('a vect * int) list -> ('a vect * int) list * ('a vect * int) list;;
value join_vstack : ('a vect * int) list -> 'a vect;;
value map_vstack : ('a -> 'b) -> ('a vect * int) list -> ('b vect * int) list;;
value map_i_vstack : (int -> 'a -> 'b) -> int -> ('a vect * int) list -> ('b vect * int) list;;
value map_vstack_list : ('a -> 'b) -> ('a vect * int) list -> 'b list;;
value for_all_vstack : ('a -> bool) -> ('a vect * int) list -> bool;;
value for_all2eq_vstack : ('a -> 'b -> bool) -> ('a vect * int) list -> ('b vect * int) list -> bool;;
value concat_vstack : 'a list -> 'a list -> 'a list;;

(* $Id: vectops.mli,v 1.2 1996/02/12 15:43:36 cpaulin Exp $ *)
