(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               arg_parse.ml                               *)
(****************************************************************************)

let implode l =
  let s = create_string (list_length l) in
  let rec loop i = function 
     (c::l) -> set_nth_char s i c; loop (i+1) l
   | [] -> ()
  in loop 0 l; s
;;

let isolate_tokens arg =
  let stream = stream_of_string arg in
  let separator c = c = ` ` or c = `\t` or c = `\n` in
  let sep  = stream_check separator and
      char = stream_check (fun c -> not (separator c)) in
  let rec end_word  = 
    function [< char c; end_word l >] -> c :: l 
           | [< >] -> [] in
  let word = function [< char c; end_word l >] -> c :: l in
  let rec blank = function [< sep _; blank _ >] -> ()
                         | [< >] -> () in
  let rec end_line = function 
     [< word w; blank _; end_line l >] -> implode w :: l
   | [< >] -> [] in
  let line = function [< blank _; end_line l >] -> l in
  line stream
;;

let parse spec default args =
  let rec loop = function
     (arg :: l) ->
       if string_length arg = 0  then loop l
       else if nth_char arg 0 = `-` then begin
         let handler = 
           try assoc arg spec 
           with Not_found ->  raise (Bad ("Unknown option " ^ arg))
         in
         match handler with
            Unit f -> f (); loop l
          | String f ->
              begin match l with 
                 (x::l') -> f x; loop l' 
               | [] -> raise (Bad "Unexpected end")
              end
          | Int f ->
              begin match l with 
                 (x::l') -> 
                    let i = try int_of_string x 
                            with _ -> raise (Bad "Integer expected")
                    in f i; loop l' 
               | [] -> raise (Bad "Unexpected end")
              end
          | Float f ->
              begin match l with 
                 (x::l') -> 
                    let r = try float_of_string x 
                            with _ ->  raise (Bad "Integer expected")
                    in f r; loop l' 
               | [] -> raise (Bad "Unexpected end")
              end
       end else begin
         default arg; loop l
       end
   | [] -> ()
  in loop (isolate_tokens args)
;;

(* $Id: arg_parse.ml,v 1.2 1996/02/12 15:44:09 cpaulin Exp $ *)
