(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  SA.ml                                   *)
(****************************************************************************)
#open "std";;
#open "pp";;
#open "stdpp";;

(* this package implements a hash-table for strings, converting
 * them into unique integers.
 *)

type t == int;;

let new_counter = ref 0;;
let int_of_string_tab = (fmavm__new (compare_strings,17) : (string,int) fmavm__t);;
let string_of_int_tab = (fmavm__new (prefix -, 17) : (int,string) fmavm__t);;

let freeze () = (!new_counter,
                 fmavm__freeze int_of_string_tab,
                 fmavm__freeze string_of_int_tab);;

let unfreeze (ctr,ist,sit) =
    (new_counter := ctr;
     fmavm__unfreeze ist int_of_string_tab;
     fmavm__unfreeze sit string_of_int_tab)
;;

let intern s =
    try fmavm__map int_of_string_tab s
    with Not_found ->
        let newv = (incr new_counter ; !new_counter)
        in (fmavm__add int_of_string_tab (s,newv);
            fmavm__add string_of_int_tab (newv,s);
            newv)
;;

let un_intern n =
    try fmavm__map string_of_int_tab n
    with Not_found -> anomaly "string-atom table is out of sync"
;;

let pr_atom n = [< 'S(un_intern n) >];;
let ord a1 a2 = a1 - a2;;

let inspect a = (a:int);;

(* $Id: SA.ml,v 1.2 1996/02/12 15:44:42 cpaulin Exp $ *)
