(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 ast.mli                                  *)
(****************************************************************************)

(* Abstract syntax trees *)
#open "names";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "unparsings";;
#open "lexer";;
#open "vkey";;

type tok = 
    TOKnum of int
  | TOKid of identifier
  | TOKstr of string
  | TOKpath of section_path
  | TOKdyn of pridyn__t

;;

type node_name == string;;

type ast =
    ASTprim of tok
  | ASTnode of node_name * ast list
  | ASTnvar of identifier
  | ASTslam of identifier option * ast
;;

type frag =
    FRAGast of ast
  | FRAGastlist of ast list
  | FRAGprim of tok
;;

type patvar == identifier;;

type spat =
    SPATmeta of patvar
  | SPATnvar of patvar
  | SPATnil
  | SPATcons of spat * spat
  | SPATappend of spat list
  | SPATlist of spat list
  | SPATprim of patvar * string option
  | SPATnode of node_name * spat list
  | SPATbreaknode of spat * spat

  | SPATslamlist of spat * spat
  | SPATslam of patvar option * spat
  | SPATquoteslam of identifier * spat
  | SPATquote of ast
;;

type tyexp =
    TYprim of string
  | TYlist of tyexp
  | TYoption of tyexp
  | TYcon of string
  | TYbound of tyexp
  | TYboundlist of tyexp
;;

value injection : tok -> string;;
value parse_tok : 'a lexer__token stream -> tok;;
value print_tok : tok -> std_ppcmds;;
value parse_ast : 'a lexer__token stream -> ast;;
value print_ast : ast -> std_ppcmds;;
value print_frag : frag -> std_ppcmds;;
value mkNODE : node_name -> ast list -> ast;;
value dest_slam_as_ast : ast -> ast * ast;;
value mk_slam_as_ast : ast * ast -> ast;;
value isNVAR : ast -> bool;;
value mkNVAR : identifier -> ast;;
value alpha_eq : ast * ast -> bool;;
value smatch : spat -> frag -> (patvar * frag) list;;
value ssub : (patvar, frag) listmap__t -> spat -> frag;;
value spat_of_ast : ast -> spat;;
value ast_of_spat : spat -> ast;;
value raise_spat_bound_indices : spat -> spat;;

value print_spat : spat -> std_ppcmds;;
value parse_spat : 'a lexer__token stream -> spat;;


value ri_ast : ast -> ast;;
value ri_frag : frag -> frag;;
value ri_spat : spat -> spat;;
value ri_tyexp : tyexp -> tyexp;;
value ri_tok : tok -> tok;;

value inSPAT : spat -> pridyn__t;;
value outSPAT : pridyn__t -> spat;;

value OP : (string * tok list) * ast list -> ast;;
value SLAM : identifier option * ast -> ast;;

type gram_key == identifier * identifier;;
type first_val =
    LITERAL of string * vernac_keyword token
  | PRIMITIVE of gram_key
  | EPSILON
;;
value fv_ord : first_val -> first_val -> int;;
value gk_ord : gram_key -> gram_key -> int;;


(* $Id: ast.mli,v 1.2 1996/02/12 15:44:49 cpaulin Exp $ *)
