(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                errors.ml                                 *)
(****************************************************************************)
#open "sys";;
#open "std";;
#open "lexer";;
#open "pp";;
#open "stdpp";;

let debug = ref false;;

(* assumption : explain_sys_exn ends with a 'FNL *)
let explain_sys_exn_default e =
    match e with
    Parse_failure -> [< 'S"The parser failed to parse input" ; 'FNL >]

  | Parse_error -> fmt_syntax_error()

  | Sys_error msg -> [< 'S"OS error: " ; 'S msg ; 'FNL >]

  | UserError(s,pps) -> (if !debug then
                          [< 'S"Error: " ; 'S s ; 'FNL ; pps ; 'FNL >]
                         else [< 'S"Error: " ; pps ; 'FNL >])

  | Anomaly (s,pps) ->
            [< 'S"System Error (Anomaly) "; 'S s ; 'SPC ; pps ; 
               'S". Please report." ; 'FNL >]

  | Match_failure(filename,pos1,pos2) ->
                  [< 'S"System Error Match in file " ;
                     'S filename ; 'S " from char #" ;
                     'INT pos1 ; 'S " to #" ; 'INT pos2 ;
                     'S ": Please report."; 'FNL >]

  | Not_found -> [< 'S"System Error search. Please Report." ; 'FNL >]

  | Failure(s)          -> [< 'S "System Error (Failure) " ; 'S s ;
                              'S ". Please report." ; 'FNL >]

  | Invalid_argument(s) -> [< 'S"System Error (Invalid arguments) " ; 'S s ;
                              'S ". Please report." ; 'FNL >]

  | Format(s)            -> [< 'S "Formatter Error " ; 'S s ;
                               'S ". Please report." ; 'FNL >]
  | Break                -> [< 'S"User Interrupt." ; 'FNL >]

  | reraise   -> [< 'S "Unexplainable exception. Please report." ; 'FNL >]
;;

let fmt_disclaimer() = 
    [< 'CUT ; 'S"If this is in user-written tactic code, then" ; 'SPC ;
       'S"it needs to be modified." ; 'SPC ; 'S"If it is in system" ; 'SPC ;
       'S"code then this needs to be reported." ; 'FNL >]
;;

(* assumption : explain_user_exn ends with a 'FNL *)
let explain_user_exn_default e =
    match e with
    Parse_failure -> [< 'S"The parser failed to parse input" ; 'FNL >]

  | Parse_error -> fmt_syntax_error()

  | Sys_error msg -> [< 'S"OS error: " ; 'S msg ; 'FNL >]

  | UserError(s,pps) -> [< 'S"Error: " ; pps ; 'FNL >]


  | Anomaly (s,pps) ->
    [< 'S"System Error (Anomaly) "; 'S s ; 'SPC ; pps ; 'S"." ;
       fmt_disclaimer() >]

  | Match_failure(filename,pos1,pos2) ->
                  [< 'S"System Error Match in file " ;
                     'S filename ; 'S " from char #" ;
                     'INT pos1 ; 'S " to #" ; 'INT pos2 ;
                     'S "." ; fmt_disclaimer() >]

  | Not_found -> [< 'S"System Error search. Please Report." ; 'FNL >]

  | Failure(s)          ->
    [< 'S "Somebody raised a Failure exception " ; 'S s ;
       'S "." ; fmt_disclaimer() >]

  | Invalid_argument(s) -> [< 'S"System Error (Invalid arguments) " ; 'S s ;
                              'S "." ; fmt_disclaimer() >]

  | Format(s)            -> [< 'S "Formatter Error " ; 'S s ;
                               'S ". Please report." ; 'FNL >]

  | Break                -> [< 'S"User Interrupt." ; 'FNL >]

  | reraise              ->
    [< 'S "Unexplainable exception." ; fmt_disclaimer() >]
;;

let reraise_user_exn = function
    Match_failure _ -> error "match_failure"
  | Not_found -> error "search"
  | Failure s -> error s
  | Invalid_argument s -> error s
  | e -> raise e
;;

let reraise_sys_exn = function
    Match_failure _ -> ()
  | Not_found -> ()
  | Failure s -> ()
  | Invalid_argument s -> ()
  | UserError _ -> ()
  | e -> raise e
;;

let raise_if_debug e =
  if !debug then raise e;;


let EXPLAIN_SYS_EXN = ref explain_sys_exn_default;;

let explain_sys_exn e = !EXPLAIN_SYS_EXN e;;

let EXPLAIN_USER_EXN = ref explain_user_exn_default;;

let explain_user_exn e = !EXPLAIN_USER_EXN e;;

(* $Id: errors.ml,v 1.4 1996/02/12 15:44:51 cpaulin Exp $ *)
