(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               generic.mli                                *)
(****************************************************************************)

#open "std";;
#open "vectops";;
#open "names";;
#open "pp";;
#open "stdpp";;
#open "more_util";;

exception FreeVar;;
exception Occur;;

type 'a substituend =
    CLOSEDTERM of 'a
  | OPENTERM of 'a
  | DANGLINGREF
;;

type modification_action = ABSTRACT | ERASE;;


type 'oper term =
    DOP0 of 'oper                            (* atomic terms *)
  | DOP1 of 'oper * 'oper term               (* operator of arity 1 *)
  | DOP2 of 'oper * 'oper term * 'oper term  (* operator of arity 2 *)
  | DOPN of 'oper * 'oper term vect          (* operator of variadic arity *)
  | DOPL of 'oper * 'oper term list          (* operator of variadic arity *)
  | DLAM of name * 'oper term                (* deBruijn binder on one term*)
  | DLAMV of name * 'oper term vect          (* deBruijn binder on many terms*)
  | VAR of identifier                        (* named variable *)
  | Rel of int                               (* variable as deBruijn index *)
;;

value free_rels : 'a term -> int list;;
value closedn : int -> 'a term -> unit;;
value closed : 'a term -> bool;;
value noccurn : int -> 'a term -> bool;;
value noccur_bet : int -> int -> 'a term -> bool;;
value liftn : int -> int -> 'a term -> 'a term;;
value lift : int -> 'a term -> 'a term;;
value pop : 'a term -> 'a term;;
value lift_substituend : int -> 'a term substituend -> 'a term;;
value make_substituend : 'a term -> 'a term substituend;;
value substn_many : 'a term substituend list -> int -> 'a term -> 'a term;;
value substl : 'a term list -> 'a term -> 'a term;;
value subst1 : 'a term -> 'a term -> 'a term;;
value occur_opern : 'a -> 'a term -> bool;;
value occur_oper0 : 'a -> 'a term -> bool;;
value occur_var : identifier -> 'a term -> bool;;
value occur_oper : 'a -> 'a term -> bool;;
value process_opers_of_term : ('a -> bool) -> ('a -> 'b) -> 'b list -> 'a term -> 'b list;;
value dependent : 'a term -> 'a term -> bool;;
value global_varsl : identifier list -> 'a term -> identifier list;;
value global_vars : 'a term -> identifier list;;
value subst_var : identifier -> 'a term -> 'a term;;
value subst_varn : identifier -> int -> 'a term -> 'a term;;
value replace_vars : (identifier * 'a term substituend) list -> 'a term -> 'a term;;
value id_of_var : 'a term -> identifier;;

value id_of_name : identifier -> name -> identifier;;
value dunbind : identifier -> identifier list * 'a list -> 'a -> 'b term -> (identifier list * 'a list) * 'b term;;
value dunbindv : identifier -> identifier list * 'a list -> 'a -> 'b term -> (identifier list * 'a list) * 'b term vect;;
value dbind : identifier list * 'a list -> 'b term -> 'a * 'b term;;
value dbindv : identifier list * 'a list -> 'b term vect -> 'a * 'b term;;

value SAPP : 'a term -> 'a term -> 'a term;;
value SAPPV : 'a term -> 'a term -> 'a term vect;;
value SAPPVi : int -> 'a term -> 'a term -> 'a term;;

value SAPPList : 'a term -> 'a term list -> 'a term;;
value SAPPVList : 'a term -> 'a term list-> 'a term vect;;
value SAPPViList : int -> 'a term -> 'a term list -> 'a term;;
value under_dlams : ('a term -> 'a term) -> 'a term -> 'a term;;
value eq_term : 'a term -> 'a term -> bool;;

type 'a modification =
    NOT_OCCUR
  | DO_ABSTRACT of 'a * modification_action list
  | DO_REPLACE
;;

value modify_opers : ('a term -> 'a term) -> ('a term -> 'a term -> 'a term) -> ('a * 'a modification) list -> 'a term -> 'a term;;

value put_DLAMSV : name list -> 'a term vect -> 'a term;;
value decomp_DLAMV : int -> 'a term -> 'a term vect;;
value decomp_DLAMV_name : int -> 'a term -> name list * 'a term vect;;
value decomp_all_DLAMV_name : 'a term -> name list * 'a term vect;;
value put_DLAMSV_subst : identifier list -> 'a term vect -> 'a term;;
value rel_vect : int -> int -> 'a term vect;;
value rel_list : int -> int -> 'a term list;;

(* $Id: generic.mli,v 1.2 1996/02/12 15:45:06 cpaulin Exp $ *)
