(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                initial.ml                                *)
(****************************************************************************)

#open "std";;
#open "lomsg";;
#open "more_util";;

(* A list of the areas of the system where "unsafe" operation
 * has been requested *)
let UNSAFE = (mls__mt() : string mls__t);;
let add_unsafe s = mls__add UNSAFE s;;
let is_unsafe s = mls__memb UNSAFE s;;

(* Silent *)
let SILENT = ref false;;
let make_silent flag = SILENT:=flag; ();;
let is_silent () = !SILENT;;

let silently f x =
  let osilent = is_silent() in
  try make_silent true;
      let rslt = f x in (make_silent osilent; rslt)
  with e -> (make_silent osilent; raise e)
;;

(* Counter of generated variable names *)
let METACTR = ref 0;;
let setMETA n = (METACTR:=n; warning "gensym counter reset");;
let newMETA () = (incr METACTR; !METACTR);;

(* Time stamps *)
type 'a timestamped = {stamp : int; ed : 'a};;
let ts_stamp st = st.stamp;;
let ts_mod f st = {stamp = newMETA();
                        ed = f st.ed};;
let ts_it st = st.ed;;
let ts_mk v = {stamp = newMETA();
               ed    = v};;
let ts_eq st1 st2 = (st1.stamp = st2.stamp);;

type 'a idstamped == 'a timestamped;;
let ids_mod f st = {stamp = st.stamp;
                    ed    = f st.ed};;
let ids_it = ts_it;;
let ids_mk = ts_mk;;
let ids_eq = ts_eq;;

(* The goal focused on *)
let FOCUS = ref 0;;
let make_focus n = FOCUS:=n;;
let focus () = !FOCUS;;

(* The number of printed hypothesis in a goal *)

let PRINT_HYPS_LIMIT = ref (NONE : int option);;
let set_print_hyps_limit n = PRINT_HYPS_LIMIT := SOME n;;
let unset_print_hyps_limit () = PRINT_HYPS_LIMIT := NONE;;
let print_hyps_limit () = !PRINT_HYPS_LIMIT;;

(* $Id: initial.ml,v 1.4 1996/02/12 15:45:16 cpaulin Exp $ *)
