(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                lexer.mli                                 *)
(****************************************************************************)

#open "stdpp";;

exception Lexer_error of int;;

type ttree;;
type frozen;;

type 'a token =
 (* predefined tokens *) 
     Tbar | Tcolon | Tcolonequal | Tequal | Tgreater | Tgreatergreater 
   | Tlessgreater | Tlessless | Tless | Tminusgreater | Tsemi | Tsharp
   | Tstar | Tcomma
 (* identifiers *)
   | Tident of string
 (* strings *)
   | Tstring of string 
 (* user's tokens *)
   | Tspecial of string
 (* integers *)
   | Tint of int 
 (* keywords *)
   | Tkw of 'a 
 (* reserved characters *)
   | Tdot | Tlbrace | Trbrace | Tlbracket | Trbracket | Tlparen | Trparen
   | Tminus | Tlparencolon | Tlbracketlbrace
;;

value freeze : unit -> frozen;;
value unfreeze : frozen -> unit;;
value add_token : string -> unit;;

value token_stream :
  (string * vkey__vernac_keyword) list vect -> char stream -> vkey__vernac_keyword token stream;;
value stream_of_string : string -> char stream;;
value stream_of_channel : in_channel -> char stream;;
value fmt_lexer_error : int -> std_ppcmds;;
value print_lexer_error : int -> unit;;
value fmt_syntax_error : unit -> std_ppcmds;;
value print_syntax_error : unit -> unit;;
value reset_lexer : char stream -> unit;;
value string_of_token : (vkey__vernac_keyword * string) list vect -> vkey__vernac_keyword token -> string;;

value fmt_token_stream :
  (vkey__vernac_keyword * string) list vect -> vkey__vernac_keyword token stream -> std_ppcmds;;
value with_tokens :
  (string * vkey__vernac_keyword) list vect -> (vkey__vernac_keyword token stream -> 'b) -> string -> 'b;;

(* $Id: lexer.mli,v 1.4 1996/02/12 15:45:21 cpaulin Exp $ *)
