(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 lib.mli                                  *)
(****************************************************************************)

#open "std";;
#open "names";;
#open "more_util";;
#open "libobject";;

type node =
    LEAF of object
  | OpenDir of string * open_dir_contents
  | ClosedDir of
      	  string * open_dir_contents * closed_dir_contents * library_segment
  | Import of string * section_path * bool

and open_dir_contents = {module_p : bool;
                         imports : section_path list}

and closed_dir_contents = {exports : section_path list;
                           mutable export_fn : string option}

and library_segment == (section_path * node) list;;
type library_entry == section_path * node;;
type frozen_t == library_segment;;

value init : unit -> unit;;

value add : (identifier * path_kind) * node -> (section_path * node);;
value add_leaf : (identifier * path_kind) * object -> (section_path * node);;
value add_module : section_path * node -> unit;;
(* Adds an object to the library.  If the identifier is provided, then
 * the object is added to the library with the section_path gotten by
 * taking the current path-prefix, and adding the identifier and path_kind.
 *)

value reset_to : section_path -> unit;;
value reset_keeping : section_path -> unit;;
value reset_to_P : (section_path * node -> bool) -> unit;;
value reset_keeping_P : (section_path * node -> bool) -> unit;;
(* Resets the library to everything before the specified object.
 * This object cannot be in a closed directory - that is an error.
 *)

value app : ((section_path * node) -> unit) -> unit;;

value leaf_object_tag : section_path -> string;;
value map : section_path -> node;;
value map_leaf : section_path -> object;;
(* Produces the library-object referenced by the supplied section_path *)

value find_entry_P : ((section_path * node) -> bool) -> section_path;;
value find_dir : string option -> section_path;;
(* finds the section_path of the named directory, which must be open.
 * if there is no name, then it finds the nearest still-open directory.
 *)

value open_dir : string -> open_dir_contents -> (section_path * node);;
(* opens a directory with the desired name. *)

value close_dir : section_path -> closed_dir_contents -> unit;;
(* Closes the named directory, verifying that there are no intervening
 * open directories.
 *)

value contents_after : section_path option -> library_segment;;
(* Given a section_path, returns the list of objects which follow it. *)

value make_path : path_kind -> identifier -> section_path;;
value cwd : unit -> string list;;

value freeze : unit -> frozen_t;;
value unfreeze : frozen_t -> unit;;

(* The operation of section closing consists of (1) discovering the
   section_path corresponding to the nearest open directory, (2)
   getting the contents which follows this OpenDir, (3) closing the
   directory, (4) re-installing the entries which remain.

   The operation of resetting to a section or an entry is just the
   reset_to command.

 *)

value node_name : (section_path * node) -> identifier;;

value next_anonymous_id : unit -> identifier;;

value last_entry : unit -> section_path option;;

(* $Id: lib.mli,v 1.3 1996/02/12 15:45:25 cpaulin Exp $ *)
