(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              libobject.mli                               *)
(****************************************************************************)

#open "std";;
#open "names";;
#open "more_util";;

(* the closure at which the object will be discharged *)
type strength = DischargeAt of section_path | NeverDischarge;;

value make_strength : string list -> strength;;

type object;;

value object_tag : object -> string;;

(* given the object-name, the "caching" function, and the "reintern"
 * function, this function
 * will hand back two functions, the "injection" and "projection"
 * functions for dynamically typed library-objects.
 *)

type 'a object_declaration =
    {cache_function : (section_path * 'a -> unit);
     reintern_function : ('a -> 'a);
     specification_function : 'a -> 'a}
;;

value declare_object : (string * 'a object_declaration)
                    -> ('a -> object) * (object -> 'a);;

value cache_object : (section_path * object) -> unit;;
value reintern_object : object -> object;;
value extract_object_specification : object -> object;;

(* $Id: libobject.mli,v 1.2 1996/02/12 15:45:29 cpaulin Exp $ *)
