(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               library.mli                                *)
(****************************************************************************)
#open "std";;
#open "pp";;
#open "stdpp";;
#open "names";;
#open "more_util";;
#open "libobject";;

(* Modules *)

value reset_library : unit -> unit;;
value rebuild_caches : unit -> unit;;

value add_anonymous_object : object -> unit;;
value add_named_object : (identifier * path_kind) -> object -> unit;;
value open_section : string -> unit;;
value open_module : string -> unit;;
value import_module : string -> unit;;
value export_module : string -> unit;;
value close_module : section_path -> section_path list -> unit;;

value raw_reset_section : string -> unit;;
value reset_keeping : identifier -> unit;;
value reset_to : identifier -> unit;;

value cwd : unit -> string list;;

value extern_module : string -> unit;;
value extern_module_to : string -> string -> unit;;
(*value extern_module_specification : string -> unit;;*)
(*value extern_module_specification_to : string -> string -> unit;;*)
value intern_module : string -> unit;;
value intern_module_from : string -> string -> unit;;
(*value intern_module_specification : string -> unit;;*)
(*value intern_module_specification_from : string -> string -> unit;;*)

value read_module : string -> unit;;
(*value read_module_specification : string -> unit;;*)

value read_module_from : string -> string -> unit;;
(*value read_module_specification_from : string -> string -> unit;;*)

value require_module : string -> bool -> unit;;
value require_module_from : string -> string -> bool -> unit;;
(*value require_module_specification : string -> bool -> unit;;*)
(*value require_module_specification_from : string -> string -> bool ->unit;;*)

value add_import_watcher : string * (section_path -> unit) -> unit;;

value fmt_module_state : unit -> std_ppcmds;;
value init_caches : unit -> unit;;
value resynchronize_caches : unit -> unit;;
value backtrack_caches : unit -> unit;;
value force_freeze_caches : unit -> unit;;

value declare_summary : string -> 'a summary__summary_declaration -> unit;;

value search_modules : unit -> (string * section_path) list;;
value search_imports : unit -> section_path list;;
value with_heavy_rollback : ('a -> 'b) -> ('a -> 'b);;

(* This exception is raised by find_dir when the dir is not found *)
exception unknown_dir;;
value find_dir : string option -> section_path;;
value find_nearest_module : unit -> section_path;;
value find_nearest_section : unit -> section_path;;
value is_section_p : section_path -> bool;;
value is_module_p : section_path -> bool;;

(* $Id: library.mli,v 1.2 1996/02/12 15:45:34 cpaulin Exp $ *)
