(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                names.mli                                 *)
(****************************************************************************)
#open "stdpp";;
#open "lexer";;

type identifier;;

value make_ident : SA__t -> int -> identifier;;
value repr_ident : identifier -> SA__t * int;;
value lift_ident : identifier -> identifier;;

type name = Name of identifier | Anonymous;;

value next_ident_away_from : identifier -> identifier list -> identifier;;
value next_ident_away : identifier -> identifier list -> identifier;;
value next_name_away : name -> identifier list -> identifier;;

value explode_id : identifier -> string list;;
value print_id : identifier -> std_ppcmds;;
value string_of_id : identifier -> string;;
value id_of_string : string -> identifier;;
value atompart_of_id : identifier -> SA__t;;
value index_of_id : identifier -> int;;
value id_ord : identifier -> identifier -> int;;

type path_kind = CCI | FW | OBJ;;
value string_of_kind : path_kind -> string;;
value kind_of_string : string -> path_kind;;

type section_path;;

value make_path : PA__t -> identifier -> path_kind -> section_path;;
value repr_path : section_path -> PA__t * identifier * path_kind;;

value kind_of_path : section_path -> path_kind;;
value fwsp_of_ccisp : section_path -> section_path;;
value ccisp_of_fwsp : section_path -> section_path;;
value coerce_path : path_kind -> section_path -> section_path;;
value fwsp_of : section_path -> section_path;;
value ccisp_of : section_path -> section_path;;
value objsp_of : section_path -> section_path;;
value sp_ord : section_path -> section_path -> int;;
value sp_gt : section_path * section_path -> bool;;
value string_of_path : section_path -> string;;
value string_of_path_mind : section_path -> identifier -> string;;
value path_of_string : string -> section_path;;
value qualid : 'a token stream -> section_path;;
value basename : section_path -> identifier;;
value dirpath : section_path -> string list;;
value append_to_path : section_path -> string -> section_path;;

value sp_of_wd : string list -> section_path;;

type 'a signature == identifier list * 'a list;;
type 'a db_signature == (name * 'a) list;;
type ('a,'b) env = ENVIRON of 'a signature * 'b db_signature;;

value nil_sign : 'a signature;;
value rev_sign : 'a signature -> 'a signature;;
value map_sign_typ : ('a -> 'b) -> 'a signature -> 'b signature;;
value isnull_sign : 'a signature -> bool;;
value hd_sign : 'a signature -> identifier * 'a;;
value tl_sign : 'a signature -> 'a signature;;

value sign_it : (identifier -> 'a -> 'b -> 'b) -> 'a signature -> 'b -> 'b;;
value it_sign : ('b -> identifier -> 'a -> 'b) -> 'b -> 'a signature -> 'b;;
value add_sign : (identifier * 'a) -> 'a signature -> 'a signature;;
value dbenv_it : (name -> 'b -> 'c -> 'c) -> ('a,'b) env -> 'c -> 'c;;
value it_dbenv : ('c -> name -> 'b -> 'c) -> 'c -> ('a,'b) env -> 'c;;
value GLOB : 'b signature -> ('b,'a) env;;
value get_globals : ('b,'a) env -> 'b signature;;
value get_rels : ('b,'a) env -> 'a db_signature;;
value ids_of_sign : 'a signature -> identifier list;;
value vals_of_sign : 'a signature -> 'a list;;
value add_rel : (name * 'a) -> ('b,'a) env -> ('b,'a) env ;;
value add_glob : (identifier * 'b) -> ('b,'a) env -> ('b,'a) env;;
type ('b,'a) search_result =
    GLOBNAME of identifier  * 'b
  | RELNAME of int * 'a
;;

value lookup_glob : identifier -> ('b,'a) env -> (identifier * 'b);;
value lookup_rel : int -> ('b,'a) env  -> (name * 'a);;
value lookup_id : identifier -> ('b,'a) env -> ('b,'a) search_result;;
value map_rel_env : ('a -> 'b) -> ('c,'a) env -> ('c,'b) env;;
value map_var_env : ('c -> 'b) -> ('c,'a) env -> ('b,'a) env;;
value nth_sign : 'a signature -> int -> (identifier * 'a);;
value map_sign_graph : (identifier -> 'a -> 'b) -> 'a signature -> 'b list;;
value make_sign : (identifier * 'a) list -> 'a signature;;
value do_sign : (identifier -> 'a -> unit) -> 'a signature -> unit;;
value uncons_sign : 'a signature -> (identifier * 'a) * 'a signature;;
value lookup_sign : identifier -> 'a signature -> (identifier * 'a);;
value sign_length : 'a signature -> int;;
value mem_sign : 'a signature -> identifier -> bool;;
value modify_sign : identifier -> 'a -> 'a signature -> 'a signature;;

value add_envp_rel : (name * 'a * 'b) -> ('c,'a) env * ('d,'b) env -> ('c,'a) env * ('d,'b) env;;

value isnull_rel_env : ('a,'b) env -> bool;;
value uncons_rel_env : ('a,'b) env -> (name * 'b) * ('a,'b) env;;
value exists_sign : (identifier -> 'a -> bool) -> 'a signature -> bool;;
value sign_prefix : identifier -> 'a signature -> 'a signature;;
value add_sign_after : identifier -> (identifier * 'a) -> 'a signature -> 'a signature;;
value add_sign_replacing : identifier -> (identifier * 'a) -> 'a signature -> 'a signature;;
value prepend_sign : 'a signature -> 'a signature -> 'a signature;;

(* $Id: names.mli,v 1.3 1996/02/12 15:45:38 cpaulin Exp $ *)
