(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                nonedn.ml                                 *)
(****************************************************************************)

#open "std";;
#open "more_util";;

type ('name,'key,'pat,'val) t ==
    ('name,'key,'pat,'val) args *
    ('name,'pat) fmavm__t *
    ('key,'pat) mhmfs__t
;;

type ('name,'key,'pat,'val) frozen_t ==
    ('name,'pat) fmavm__frozen_t *
    ('key,'pat) mhmfs__frozen_t
;;

let freeze (A,nm,km) = (fmavm__freeze nm,mhmfs__freeze km);;

let unfreeze (fnm,fkm) (A,nm,km) = (fmavm__unfreeze fnm nm;
                                    mhmfs__unfreeze fkm km);;

let new A =
    ((A,fmavm__new(A.name_ord,17),mhmfs__new 17) : ('name,'key,'pat,'val) t);;

let add (A,nm,km) (na,pat) =
    let patkl = A.pat_keys pat
    in (fmavm__add nm (na,pat);
        do_list (fun k -> mhmfs__add km (k,pat)) patkl)
;;

let rmv (A,nm,km) na =
    let pat = fmavm__map nm na in
    let patkl = A.pat_keys pat in
        (fmavm__rmv nm na;
         do_list (fun k -> (mhmfs__rmv km (k,pat))) patkl)
;;

let remap ndn na pat =
    rmv ndn na;
    add ndn (na,pat)
;;

let map (A,nm,km) valu =
    let valkl = A.val_keys valu
    in join(list__map
            (fun k ->
                 try mhmfs__map km k
                 with Not_found -> [])
            valkl)
;;

let in_dn (A,nm,km) na = fmavm__in_dom nm na;;

let empty (A,nm,km) = (fmavm__empty nm;mhmfs__empty km);;




(* $Id: nonedn.ml,v 1.3 1996/02/12 15:45:40 cpaulin Exp $ *)
