(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                nonedn.mli                                *)
(****************************************************************************)

(* named one-level discrimination nets

   Each entry in the net is named, and we can insert and remove
   entries in the net with with this name.

   Each 'pat entered into the net is stored under every key given by
   the pat_keys function.  Later lookup with a given 'val will
   retrieve all patterns stored under the keys given by the val_keys
   function, in the order in which the val_keys function gives the
   keys, and then in the order in which the keys are actually stored.

 *)

type ('name,'key,'pat,'val) t;;
type ('name,'key,'pat,'val) frozen_t;;

type ('name,'key,'pat,'val) args =
    {name_ord : 'name -> 'name -> int;
     pat_keys : 'pat -> 'key list;
     val_keys : 'val -> 'key list}
;;

value new : ('name,'key,'pat,'val) args -> ('name,'key,'pat,'val) t;;

value add : ('name,'key,'pat,'val) t -> ('name * 'pat) -> unit;;

value remap : ('name,'key,'pat,'val) t -> 'name -> 'pat -> unit;;

value rmv : ('name,'key,'pat,'val) t -> 'name -> unit;;

value map : ('name,'key,'pat,'val) t -> 'val -> 'pat list;;

value in_dn : ('name,'key,'pat,'val) t -> 'name -> bool;;

value freeze : ('name,'key,'pat,'val) t -> ('name,'key,'pat,'val) frozen_t;;

value unfreeze :
  ('name,'key,'pat,'val) frozen_t -> ('name,'key,'pat,'val) t -> unit;;

value empty : ('name,'key,'pat,'val) t -> unit;;

(* $Id: nonedn.mli,v 1.3 1996/02/12 15:45:42 cpaulin Exp $ *)
