(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              pureparse.mli                               *)
(****************************************************************************)
#open "std";;
#open "names";;
#open "more_util";;
#open "lexer";;
#open "pp";;
#open "stdpp";;
#open "vkey";;
#open "ast";;
#open "termact";;
#open "libobject";;

type gram_key == identifier * identifier;;

type gram_call == gram_key * action list;;

type production_item =
    NONTERM of gram_call * identifier option
  | TERM of vernac_keyword token;;

type production == production_item list * action;;

type gram_defn == identifier list * production list;;

type grammar_entry == gram_key * gram_defn;;

type tokstream == vernac_keyword token stream;;


type compiled_grammar =
    c_ACTION of action
  | c_SWITCH of (vernac_keyword token list,
                 primitive_grammar list * compiled_grammar) listmap__t
              * (primitive_grammar list * compiled_grammar) list

and primitive_grammar =
    c_NTERM of gram_call * identifier option
  | c_TERM of vernac_keyword token
    
;;

type compiled_grammar_entry =
    {g_args : identifier list;
     g_prodns : production list;
     g_compiled : compiled_grammar;
     g_firsts : fvavs__t;
     g_mentions : gkavs__t;
     g_mentioned_by : gkavs__t}
;;

type frozen_t == (gram_key,gram_defn) mhs__frozen_t
               * (gram_key,compiled_grammar_entry) fmavm__frozen_t
               * (gram_key*gram_defn) list;;

value isMeta : identifier -> bool;;
value lex_string : string -> vernac_keyword token;;
value parse_bar : 'a token stream -> unit;;
value parse_prod_item : (tokstream -> action) -> identifier -> vernac_keyword token stream -> production_item;;
value parse_production : (tokstream -> action) -> identifier -> vernac_keyword token stream -> production_item list * action;;
value parse_grammar_entry : (tokstream -> action) -> identifier -> vernac_keyword token stream -> grammar_entry;;
value print_bar : unit -> (int * string) ppcmd_token stream;;
value print_gk : identifier -> identifier * identifier -> (int * string) ppcmd_token stream;;
value print_prod_item : identifier -> production_item -> (int * string) ppcmd_token stream;;
value print_production : identifier -> production_item list * action -> (int * string) ppcmd_token stream;;
value print_grammar_entry : identifier -> grammar_entry -> (int * string) ppcmd_token stream;;
value dollarzero : identifier;;
value prim_grams : (gram_key, (vernac_keyword token stream -> frag) * first_val list) mhm__t;;
value lookup_prim_gram : gram_key -> vernac_keyword token stream -> frag;;
value gk_ord : identifier * identifier -> identifier * identifier -> int;;
value grams : (gram_key, gram_defn) mhs__t;;
value compiled_grams : (gram_key, compiled_grammar_entry) fmavm__t;;
value freeze : unit -> frozen_t;;
value unfreeze : frozen_t -> unit;;
value init : unit -> unit;;
value update_firsts : gram_key -> fvavs__t -> unit;;
value update_mby : gram_key -> gkavs__t -> unit;;
value update_mentions : gram_key -> gkavs__t -> unit;;
value update_compiled : gram_key -> compiled_grammar -> unit;;
value nonterminals : (production_item list * 'a) list -> gkavs__t;;
value productions_of : gram_key -> production list;;
value nonterminals_of : gram_key -> gkavs__t;;
value params_of : gram_key -> identifier list;;
value firsts_of_key : gram_key -> fvavs__t;;
value mby_of_key : gram_key -> gkavs__t;;
value firsts_of_prodn : fvavs__t -> production_item list * 'a -> fvavs__t;;
value firsts : (production_item list * 'a) list -> fvavs__t;;
value literals_only : fvavs__t * 'a -> bool;;
value prims_plus_literals : fvavs__t * 'a -> bool;;
value epsilon : fvavs__t * 'a -> bool;;
value sort_list : (fvavs__t * production) list -> (fvavs__t * production) list * (fvavs__t * production) list * (fvavs__t * production) list * (fvavs__t * production) list;;
value common_prefix : production_item list -> production_item list list -> production_item list * production_item list list;;
value compile_prodl : (production_item list * action) list -> compiled_grammar;;
value compile_fs : fvavs__t -> (fvavs__t * (production_item list * action)) list -> primitive_grammar list * compiled_grammar;;
value compile_literals : (fvavs__t * (production_item list * action)) list -> (vernac_keyword token list * (primitive_grammar list * compiled_grammar)) list;;
value compile_prims : (fvavs__t * (production_item list * action)) list -> (primitive_grammar list * compiled_grammar) list;;
value compile_eps : (fvavs__t * (production_item list * action)) list -> (primitive_grammar list * compiled_grammar) list;;
value compile_gkey : gram_key -> compiled_grammar;;
value recompile_nonterminals : gram_key list -> unit;;
value add_productions : (gram_key * (identifier list * production list)) list -> unit;;
value choose : ('a stream -> 'b) list -> 'a stream -> 'b;;
value comp_gram_of : gram_key -> compiled_grammar;;
value build_call_env : nonterm_map -> 'a list -> action list -> gram_key -> ('a * frag) list;;
value interpc : (identifier * frag) list -> compiled_grammar -> vernac_keyword token stream -> frag;;
value interpc_seq : (identifier * frag) list -> primitive_grammar list * compiled_grammar -> vernac_keyword token stream -> frag;;
value interpc_pg : nonterm_map -> primitive_grammar -> vernac_keyword token stream -> nonterm_map;;
value push : grammar_entry list -> unit;;
value interpc_gram : nonterm_map -> action list -> gram_key -> tokstream -> frag;;
value lookup_cgram : gram_key -> compiled_grammar_entry;;
value recheck_because_of : gram_key -> gram_key list -> (first_val list * first_val list) -> unit;;

(*=================
LaTeX output
Claude
*)

value LaTeX_print_nonterminal_definition : 
    gram_key -> std_ppcmds;;

value LaTeX_print_grammar_definition : 
    identifier -> std_ppcmds;;

(*==============*)
value print_nonterminal_definition : gram_key -> std_ppcmds;;
value print_nonterminal_code : gram_key -> std_ppcmds;;
value print_cgram : compiled_grammar -> std_ppcmds;;
value print_pgram : primitive_grammar -> std_ppcmds;;

value ri_grammar_entry : grammar_entry -> grammar_entry;;
value inGRAMMAR_ENTRY : grammar_entry -> pridyn__t;;
value outGRAMMAR_ENTRY : pridyn__t -> grammar_entry;;

type grammar_object == grammar_entry list;;
value inGrammar : grammar_object -> object;;
value outGrammar : object -> grammar_object;;

value inToken : string -> libobject__object;;

(* $Id: pureparse.mli,v 1.4 1996/02/12 15:45:50 cpaulin Exp $ *)
