(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               reintern0.ml                               *)
(****************************************************************************)

#open "system";;
#open "std";;
#open "more_util";;
#open "names";;
#open "libobject";;
#open "pp";;
#open "stdpp";;
#open "lomsg";;

let sa_fmavm = (fmavm__new(SA__ord,17) : (SA__t,string) fmavm__t);;
let pa_fmavm = (fmavm__new(PA__ord,17) : (PA__t,string list) fmavm__t);;

let use_hash_cons = ref false;;

let install_saved_tables ((_,_,sa),(_,_,pa)) =
     fmavm__unfreeze sa sa_fmavm;
     fmavm__unfreeze pa pa_fmavm;
     if !use_hash_cons then hashcons__init()
;;

let erase_tables () =
    fmavm__empty sa_fmavm;
    fmavm__empty pa_fmavm;
    hashcons__init()
;;

let do_reintern ri_function hash_cons_p ((sa,pa),thing) =
    install_saved_tables(sa,pa);
    use_hash_cons := hash_cons_p;
    try let ri_thing = ri_function thing
        in (erase_tables();
            ri_thing)
    with e -> (MSG [< 'S"Error in reinterning an object: " ; 
                      errors__explain_sys_exn e >];
               raise e)
;;

let prepare_extern thing =
    ((SA__freeze(),PA__freeze()),thing)
;;

let ri_nref f r = {v = f r.v};;

let ri_sa sa = SA__intern (fmavm__map sa_fmavm sa);;
let ri_pa pa = PA__intern (fmavm__map pa_fmavm pa);;

let ri_id id = 
  let (oatom,n) = repr_ident id
  in make_ident (ri_sa oatom) n
;;

let ri_sp sp = 
  let (pa,id,pk) = repr_path sp
  in make_path(ri_pa pa) (ri_id id) pk
;;

let ri_name = function
    Anonymous -> Anonymous
  | Name id   -> Name(ri_id id)
;;

let ri_sign ri_val (idl,vall) = (map ri_id idl,map ri_val vall);;
let ri_dbsign ri_dbval = map (fun (na,val) -> (ri_name na,ri_dbval val));;
let ri_env ri_val ri_dbval (ENVIRON(sign,dbsign)) =
    ENVIRON(ri_sign ri_val sign,
            ri_dbsign ri_dbval dbsign)
;;

let ri_spavs s = spavs__ofList(map ri_sp (spavs__toList s));;

let ri_avm dom_ord ri_dom ri_rng avm =
  avm__ofList dom_ord
      	    (map (fun (d,r) -> (ri_dom d,ri_rng r)) (avm__toList avm))
;;

let ri_intavm ri_rng avm =
    intavm__ofList(map (fun (d,r) -> (d,ri_rng r)) (intavm__toList avm));;

let ri_avm_frozen_t dom_ord ri_dom ri_rng frozen =
  let avm = avm__unfreeze frozen (avm__new dom_ord) in
  let avm' = ri_avm dom_ord ri_dom ri_rng avm
  in avm__freeze avm'
;;

let ri_object lobj = reintern_object lobj;;

let rec ri_node = function
    lib__LEAF obj                  -> lib__LEAF(ri_object obj)
  | lib__Import(s,isp,locally)     -> lib__Import(s,ri_sp isp,locally)
  | lib__OpenDir(s,odc)            -> lib__OpenDir(s,ri_odc odc)
  | lib__ClosedDir(s,odc,cdc,ctxt) -> lib__ClosedDir(s,ri_odc odc,
                                                     ri_cdc cdc,
                                                     ri_ctxt ctxt)

and ri_ctxt l = map (fun (spopt,n) -> (ri_sp spopt,ri_node n)) l

and ri_cdc cdc =
    {lib__exports = map ri_sp cdc.lib__exports;
     lib__export_fn = cdc.lib__export_fn}

and ri_odc odc =
    {lib__module_p = odc.lib__module_p;
     lib__imports = map ri_sp odc.lib__imports}
;;


(* $Id: reintern0.ml,v 1.3 1996/02/12 15:45:52 cpaulin Exp $ *)
