(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                states.ml                                 *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "lomsg";;
#open "names";;
#open "system";;
#open "pp";;
#open "stdpp";;
#open "more_util";;
#open "library";;
#open "summary";;

let STATES = ref ([]:(string * (string * state)) list);;

type atom_state == SA__frozen_t * PA__frozen_t;;

type externable_states == (string * (string * state)) list * atom_state;;

(* D2| State saving/restoring *)

let raw_save_state name desc = 
    if listmap__in_dom !STATES name then error(name ^ " already exists");
    force_freeze_caches();
    STATES:=
        (listmap__add (!STATES)
                     (name,(desc,(lib__freeze(),
                                  summary__get_frozen_summaries()))));
    ();;

let reset_state (env,sum) = (lib__unfreeze env;
                             summary__set_frozen_summaries sum;
                             backtrack_caches ());;

let raw_restore_state name = 
    (try reset_state(snd(listmap__map (!STATES) name))
     with Not_found -> error(name ^ ": unknown state"));;

let raw_restore_last_saved_state () =
    match !STATES with
      ((_,(_,s))::_) -> reset_state s
    | _ -> errorlabstrm "states__raw_restore_last_saved_state"
      	       	       	[< 'S"Can not restore the last saved state,"; 'SPC;
			   'S"since no state has been saved." >]
;;

let forget_state verbose name =
    if listmap__in_dom (!STATES) name then
        (STATES := listmap__rmv (!STATES) name;
         if verbose then warning(name ^ " forgotten"))
    else warning ("There is no state named " ^ name)
;;

let list_saved_states () =
    map (fun (n,(desc,_)) -> (n,desc)) (!STATES);;

(* Total resetting *)

let MAGIC = 19755;;
let ATOM_MAGIC = 19600;;

let (extern_state,intern_state) =
let (raw_extern_state,raw_intern_state) = system__extern_intern(MAGIC,".coq") in
let extern_state s =
    raw_extern_state s (!STATES,(SA__freeze(),PA__freeze()))

and intern_state s =
    if !STATES <> [] then
        error "cannot intern a state when we already have some";
    let ((statelist,(sa,pa)):externable_states) =
        raw_intern_state s
    in STATES := statelist;
       SA__unfreeze sa;
       PA__unfreeze pa

in (extern_state,intern_state)
;;


(* $Id: states.ml,v 1.3 1996/02/12 15:46:10 cpaulin Exp $ *)
