(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                syntax.ml                                 *)
(****************************************************************************)
#open "names";;
#open "std";;
#open "more_util";;
#open "unparsings";;
#open "ast";;
#open "pp";;
#open "stdpp";;
#open "lomsg";;

let ast_key = function
    ASTnode("APPLIST",(ASTnode("CONST",(ASTprim(TOKpath sp))::_))::_) ->
    ASTprim(TOKpath sp)

  | ASTnode(s,_) -> ASTprim(TOKid (id_of_string s))
  | _ -> ASTnode("NONE",[])
;;

let short_ast_key = function
    ASTnode(s,_) -> ASTprim(TOKid(id_of_string s))
  | _ -> ASTnode("NONE",[])
;;

let spat_key = function
    SPATnode("APPLIST",(SPATnode("CONST",(SPATquote(ASTprim(TOKpath sp)))::_))::_) ->
    ASTprim(TOKpath sp)

  | SPATbreaknode(SPATquote(ASTprim(TOKid id)),_) ->
    ASTprim(TOKid id)

  | SPATnode(na,_) -> ASTprim(TOKid(id_of_string na))

  | SPATquote ast -> ast_key ast

  | _ -> ASTnode("NONE",[])
;;

let se_keys se = [(se.syn_universe,spat_key se.syn_astpat)];;

let ast_keys (whatfor,ast) = uniquize [(whatfor,ast_key ast);
                                       (whatfor,short_ast_key ast)];;

let compare_names (a1,a2) (b1,b2) =
    match id_ord a1 b1 with
    0 -> compare_strings a2 b2
  | x -> x
;;

let SYNTAX = nonedn__new{nonedn__name_ord = compare_names;
                         nonedn__pat_keys = se_keys;
                         nonedn__val_keys = ast_keys}
;;

let map whatfor t = nonedn__map SYNTAX (whatfor,t);;

let add (sp,se) =
let name = (se.syn_universe,se.syn_id) in
    if nonedn__in_dn SYNTAX name then
        (MSGNL [< 'S"Warning [Overriding Syntax Entry " ; 
                  'S(string_of_id se.syn_universe) ; 'S":" ;
                  'S se.syn_id ; 'S"]" >];
         nonedn__remap SYNTAX name se)
    else nonedn__add SYNTAX (name,se)
;;

let init () = nonedn__empty SYNTAX;;

type frozen_t ==
    (identifier * string,
     identifier * ast,
     syntax_entry,
     identifier * ast) nonedn__frozen_t
;;

let freeze () = nonedn__freeze SYNTAX;;
let unfreeze fdn = nonedn__unfreeze fdn SYNTAX;;

library__declare_summary "syntax"
{summary__freeze_function = freeze;
 summary__unfreeze_function = unfreeze;
 summary__init_function = init;
 summary__sync_function = (fun () -> false)}
;;

#open "reintern0";;
let ri_syntax_entry se =
    {syn_id = se.syn_id;
     syn_universe = ri_id se.syn_universe;
     syn_astpat = ri_spat se.syn_astpat;
     syn_recbindings = list__map (fun (id,smc) -> (ri_id id,ri_spat smc)) se.syn_recbindings;
     syn_unparsing = ri_unparsing se.syn_unparsing}
;;


(* $Id: syntax.ml,v 1.2 1996/02/12 15:46:24 cpaulin Exp $ *)
