(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              unparsings.mli                              *)
(****************************************************************************)
#open "std";;
#open "names";;
#open "more_util";;
#open "pp";;
#open "stdpp";;

type ppcmd =
    ppHB of int
  | ppHOVB of int
  | ppHVB of int
  | ppVB of int
  | ppCLOSE
  | ppBRK of int * int
  | ppFNL
;;

value pp_ppcmd : ppcmd -> 'a ppcmd_token;;
type ParenRelation = L | E | Any;;
type unparsing_hunk = 
    PH of identifier * string * ParenRelation
  | RO of string
  | UNP_PPTOKEN of ppcmd
;;
type precedence == int * int * int;;
value compare_prec : precedence -> precedence -> int;;
type unparsing = {unp: unparsing_hunk list ; prec : precedence };;
value unparsing : 'a lexer__token stream -> unparsing;;

type unparsing_subfunction ==
    (string * precedence) * ParenRelation -> std_ppcmds;;

value apply_unparsing : std_ppcmds -> string * unparsing -> (identifier, unparsing_subfunction) listmap__t -> std_ppcmds;;

type lattice;;
type compiled_lattice;;
value mt_lattice : lattice;;
value compile_lattice : lattice -> string list -> compiled_lattice;;
type irrit_factor = Less | Greater | Equal | Unrelated;;
value irrit_factor : compiled_lattice -> string -> string -> irrit_factor;;
value parse_lattice : 'a lexer__token stream -> lattice;;
value print_lattice : lattice -> (int * string) ppcmd_token stream;;
value tolerable : compiled_lattice option -> (string * precedence) -> ((string * precedence) * ParenRelation) -> bool;;

value ri_unparsing : unparsing -> unparsing;;
value inUNPARSING : unparsing -> pridyn__t;;
value outUNPARSING : pridyn__t -> unparsing;;

(* $Id: unparsings.mli,v 1.2 1996/02/12 15:46:35 cpaulin Exp $ *)
