(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                termast.ml                                *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "lomsg";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "environ";;
#open "termenv";;
#open "impuniv";;
#open "ast";;
#open "termast";;

let absolutize absfun sigma = absrec
    where rec absrec env = function
    VAR id -> absfun sigma env id
  | Rel _ as t -> t
  | DOP0 _ as t -> t
  | DOP1(oper,c) -> DOP1(oper,absrec env c)
  | DOP2(oper,c1,c2) -> DOP2(oper,absrec env c1,absrec env c2)
  | DOPN(oper,cl) -> DOPN(oper,map_vect (absrec env) cl)
  | DOPL(oper,cl) -> DOPL(oper,map (absrec env) cl)
  | DLAM(na,c) -> DLAM(na,absrec (add_rel (na,()) env) c)
  | DLAMV(na,cl) -> DLAMV(na,map_vect (absrec (add_rel (na,()) env)) cl)
;;

let absolutize_cci =
    absolutize
    (fun sigma env id ->
    if (atompart_of_id id = SA__intern "?") then
        machops__lookup_exist sigma env (make_path(PA__intern ["evar"]) id CCI)
    else (try VAR(fst(lookup_glob id env))
          with Not_found ->
         (try machops__search_reference env id
          with UserError _ -> (try search_synconst CCI id
                             with Not_found -> error((string_of_id id)^"  not declared")))))
;;

let absolutize_fw =
    absolutize
    (fun sigma env id ->
    if (atompart_of_id id = SA__intern "?") then
        error "existentials in fterms not implemented"
    else (try VAR(fst(lookup_glob id env))
          with Not_found ->
          (try machops__search_freference env id
           with UserError _ -> (try search_synconst FW id
                             with Not_found -> error((string_of_id id)^"  not declared")))))
;;

let dbize_cci sigma env c =
let c = try dbize env c
        with e ->
            errorlabstrm "dbize_cci"
            [< 'S"During conversion from explicit-names to" ; 'SPC ;
               'S"debruijn-indices, the exception " ;
               errors__explain_sys_exn e ; 'S" was raised" ; 'SPC ; 
               'S"perhaps the input is malformed" >] in

let c = try absolutize_cci sigma env c
        with Not_found ->
            errorlabstrm "absolutize_cci"
            [< 'S"During the relocation of global references," ; 'SPC ;
               'S"the variable(s) [" ; 'SPC ;
               prlist (fun id ->
                           try (absolutize_cci sigma env (VAR id);
                                [< >])
                           with _ -> [< 'S (string_of_id id) ; 'SPC >])
               (global_vars c) ;
               'S"]" ; 'SPC ; 'S"were not found in the current environment" >]
         | e ->
            errorlabstrm "absolutize_cci"
            [< 'S"During the relocation of global references," ; 'SPC ;
               'S"The error " ; errors__explain_sys_exn e ;
               'S"was raised.  Perhaps the input is malformed" >]
in c
;;

let dbize_fw sigma env c =
let c = try dbize env c
        with e ->
            errorlabstrm "dbize_fw"
            [< 'S"During conversion from explicit-names to" ; 'SPC ;
               'S"debruijn-indices, the exception " ;
               errors__explain_sys_exn e ; 'S" was raised" ; 'SPC ; 
               'S"perhaps the input is malformed" >] in

let c = try absolutize_fw sigma env c
        with Not_found ->
            errorlabstrm "absolutize_fw"
            [< 'S"During the relocation of global references," ; 'SPC ;
               'S"the variable(s) [" ; 'SPC ;
               prlist (fun id ->
                           try (absolutize_fw sigma env (VAR id);
                                [< >])
                           with _ -> [< 'S (string_of_id id) ; 'SPC >])
               (global_vars c) ;
               'S"]" ; 'SPC ; 'S"were not found in the current environment" >]
         | e ->
            errorlabstrm "absolutize_fw"
            [< 'S"During the relocation of global references," ; 'SPC ;
               'S"The error " ; errors__explain_sys_exn e ;
               'S"was raised.  Perhaps the input is malformed" >]
in c
;;

let ast_adjust_consts = dbrec
    where rec dbrec env = function
    ASTnvar id ->
    if (atompart_of_id id = SA__intern "?") then ASTnvar id
    else (try (lookup_id id env ; ASTnvar id)
          with Not_found ->
         (try bdize env (machops__search_reference env id)
          with Not_found -> ASTnvar id))

  | ASTslam(na,t) ->
    let na' = (match na with SOME id -> Name id | NONE -> Anonymous)
    in ASTslam(na,dbrec (add_rel (na',()) env) t)

  | (ASTnode("CONST",(ASTprim(TOKid id))::tl)) ->
    (try let sp = nametab__sp_of_id CCI id
        in ASTnode("CONST",(ASTprim(TOKpath sp))::(map (dbrec env) tl))
    with Not_found -> error((string_of_id id)^"  not declared"))

  | (ASTnode(opn,tl)) -> ASTnode(opn,map (dbrec env) tl)

  | (ASTprim _ as x) -> x
;;

let spat_adjust_consts env =
    spat_of_ast o (ast_adjust_consts env) o ast_of_spat
;;

(* $Id: astterm.ml,v 1.2 1996/02/12 15:47:11 cpaulin Exp $ *)
