(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               dischcore.ml                               *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "names";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "typing";;
#open "pp";;
#open "stdpp";;
#open "himsg";;
#open "more_util";;
#open "constrtypes";;
#open "environ";;
#open "termenv";;
#open "proof";;

let thin_hyps init_globs (hyps,j) =
let vars = init_globs@(global_vars(DOPN(AppL,[|j.VAL;j.TYP|]))) in
    rev_sign(fst(it_sign (fun ((hyps,globs) as sofar) id a ->
                         if mem id globs then
                             (add_sign (id,a) hyps,(global_vars a)@globs)
                         else sofar) (nil_sign,vars) hyps))
;;

let thin_hyps_glob init_globs (hyps,globals) =
let vars = init_globs@globals in
    rev_sign(fst(it_sign (fun ((hyps,globs) as sofar) id a ->
                         if mem id globs then
                             (add_sign (id,a) hyps,(global_vars a)@globs)
                         else sofar) (nil_sign,vars) hyps))
;;

let thin_to_match sign fsign =
    sign_it (fun id ty fsign ->
                 if mem_sign sign id then
                     (add_sign (id,ty) fsign)
                 else fsign)
    fsign nil_sign
;;

let rec thin_to_level fsign k =
    if isnull_sign fsign then nil_sign
    else let (na,T),tls = uncons_sign fsign
         in if le_kind k (cast_type T) then
             let res_k = type_of_product mt_evc (cast_type T) k
             in add_sign (na,T) (thin_to_level tls res_k)
            else thin_to_level tls k
;;

let rec thin_to_type_level fsign =
    if isnull_sign fsign then nil_sign
    else let (na,T),tls = uncons_sign fsign
         in if istype (cast_type T) then
              add_sign (na,T) (thin_to_type_level tls)
            else thin_to_type_level tls
;;

let expmod_constr modlist c =
let simpfun = if modlist = [] then fun x -> x else strong whd_betaiota in
let expfun (DOPN(Const sp,_) as c) =
    if not(cookable_constant sp) then
        anomaly "expmod_constr: an uncookable constant was discharged";
    try const_value mt_evc c
    with Failure _ as e ->
        (MSGERRNL [< 'S"Cannot unfold the value of " ; 'S(string_of_path sp) ; 'SPC;
                 'S"You cannot declare local lemmas as being opaque" ; 'SPC ;
                 'S"and then require that theorems which use them" ; 'SPC ;
                 'S"be transparent" >];
         (snd (const_of_path sp)).CONSTOPAQUE <- false;
         try
         let v = const_value mt_evc c
         in ((snd (const_of_path sp)).CONSTOPAQUE <- true; v)
         with _ -> anomaly "expmod_constr") in

let under_casts f = function
    (DOP2(Cast,c,t)) -> (DOP2(Cast,f c,f t))
  | c -> f c in
let c' = modify_opers expfun apply modlist c
in match c' with
    DOP2(Cast,val,typ) -> DOP2(Cast,simpfun val,simpfun typ)
  | DOP2(XTRA("IND",[]),c,DLAMV(na,lc)) ->
    DOP2(XTRA("IND",[]),under_casts simpfun c,
         DLAMV(na,map_vect (under_casts simpfun) lc))
  | _ -> strong whd_betaiota c'
;;


(* $Id: dischcore.ml,v 1.2 1996/02/12 15:47:15 cpaulin Exp $ *)
