(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             evar_refiner.ml                              *)
(****************************************************************************)

#infix "o";;
#open "std";;
#open "initial";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "himsg";;
#open "termenv";;
#open "mach";;
#open "more_util";;
#open "proof_constraints";;
#open "proof_trees";;
#open "proof";;
#open "pp";;
#open "stdpp";;
#open "logic";;
#open "refiner";;

#infix "ORELSE";;
#infix "NEXT";;
#infix "THEN";;
#infix "THENF";;
#infix "THENL";;
#infix "THENS";;

let rc_of_pfsigma sigma = rc_of_gc sigma.sigma sigma.it.goal;;
let rc_of_glsigma sigma = rc_of_gc sigma.sigma sigma.it;;

type walking_constraints == readable_constraints idstamped;;

let Local_Constraints lc gs = refiner (LOCAL_CONSTRAINTS lc) gs;;

let on_wc f wc = ids_mod f wc;;

let StartWalk gls =
    let evc = Project gls in
    let wc = (ids_mk evc)
    in (wc,
        (fun wc' gls' ->
             if ids_eq wc wc' & gls.it = gls'.it then
                 if evc__lc_equal gls.it.lc (evc__get_focus (ids_it wc')) then
                     IDTAC {it=gls'.it; sigma = evc__get_gc (ids_it wc')}
                 else
                 (Local_Constraints (evc__get_focus (ids_it wc'))
                  {it=gls'.it; sigma = evc__get_gc (ids_it wc')})
             else error "Walking"))
;;

let Walking_THEN wt rt gls =
    let (wc,KONT) = StartWalk gls in
    let (wc',rslt) = wt wc
    in (KONT wc' THEN rt rslt) gls
;;
(*
let Walking_THEN wt rt gls =
    let evc = Project gls in
    let wc = ids_mk evc in
    let (wc',rslt) = wt wc
    in if ids_eq wc wc' then
        (Local_Constraints (ids_it wc').focus
         THEN rt rslt) {it=gls.it; sigma = evc__get_gc (ids_it wc')}
       else error "Walking"
;;
*)

let Walking wt = Walking_THEN (fun wc -> (wt wc,())) (fun () -> IDTAC);;


let extract_decl sp evc =
    let evd = evc__map evc sp
    in abs_rc(ts_mk{sign = evd.evgoal.hyps;
              focus = evd.evgoal.lc;
              decls = lavm__rmv (ts_it (repr_rc evc)).decls sp})
;;

let restore_decl sp evd evc =
    let newgoal = {hyps=evd.evgoal.hyps; concl = evd.evgoal.concl;
                   lc = (ts_it (repr_rc evc)).focus;
                   ctxt = evd.evgoal.ctxt}
    in (evc__log_add evc (sp,{evgoal = newgoal;
                              body = evd.body}))
;;

(* [w_Focusing sp wt wc]
 *
 * Focuses the walking context WC onto the declaration SP, given that
 * this declaration is UNDEFINED.  Then, it runs the walking_tactic,
 * WT, on this new context.  When the result is returned, we recover
 * the resulting focus (access list) and restore it to SP's declaration.
 *
 * It is an error to cause SP to change state while we are focused on it.
 *)
let w_Focusing_THEN sp (wt:'a result_w_tactic) (wt':'a -> w_tactic)
                       (wc:walking_constraints) =
    let focus = evc__get_focus (ids_it wc)
    and sign = evc__get_hyps (ids_it wc)
    and evd = evc__map (ids_it wc) sp in
    let (wc':walking_constraints) = (ids_mod (extract_decl sp) wc) in
    let (wc'',rslt) = wt wc'
    in if ids_eq wc wc'' then 
        if evc__equal (ids_it wc') (ids_it wc'') then wt' rslt wc
        else let wc''' = (ids_mod (restore_decl sp evd) wc'')
             in wt' rslt
                 ((ids_mod
                   (abs_rc o
                    (ts_mod(fun evc ->
                                {sign = sign;
                                 focus = focus;
                                 decls = evc.decls})) o
                    repr_rc)
                    wc'''))
       else error "w_saving_focus"
;;

let w_add_sign (id,T) (wc:walking_constraints) =
  ids_mk(abs_rc(ts_mod
         (fun evr ->
              {focus = evr.focus;
               sign = add_sign (id,T) evr.sign;
               decls = evr.decls})
          (repr_rc (ids_it wc))))
;;

let ctxt_type_of evc c = type_of evc (ts_it (repr_rc evc)).sign c;;

let w_IDTAC wc = wc;;
let w_Focusing sp wt = w_Focusing_THEN sp (fun wc -> (wt wc,())) (fun _ -> w_IDTAC);;

let w_Focus sp (wc:walking_constraints) =
    (ids_mod (extract_decl sp) wc);;

let w_Underlying wc = (ids_it wc);;
let w_type_of wc c = ctxt_type_of (w_Underlying wc) c;;
let w_hyps wc = evc__get_hyps (w_Underlying wc);;
let w_ORELSE wt1 wt2 wc = try wt1 wc with UserError _ -> wt2 wc;;

let w_Declare sp c (wc:walking_constraints) =
(match c with (DOP2(Cast,_,_)) -> ()
      | _ -> error "Cannot declare an un-casted evar");
let _ = w_type_of wc c in
let access = (evc__get_focus (w_Underlying wc))
and sign = evc__get_hyps (w_Underlying wc) in

let newdecl = {evgoal=mkGOAL access mt_ctxt sign c;
               body=EVAR_EMPTY}
in ((ids_mod (fun evc -> (evc__add evc (sp,newdecl))) wc)
    : walking_constraints)
;;

let w_Declare_At sp sp' c = w_Focusing sp (w_Declare sp' c);;

let evars_of sigma c =
    spavs__ofList
    (uniquize(process_opers_of_term
             (function (Const sp) -> evc__is_evar sigma sp
                     | _ -> false)
             (function (Const sp) -> sp) [] c))
;;

let w_Define sp c wc =
let spdecl = evc__map (w_Underlying wc) sp in
let cty = ctxt_type_of (ids_it (w_Focus sp wc))
                       (DOP2(Cast,c,spdecl.evgoal.concl))
in match spdecl.body with
    EVAR_EMPTY ->
    let access = abs_lc(evars_of (w_Underlying wc) c) in
    let newgoal = mkGOAL access mt_ctxt spdecl.evgoal.hyps spdecl.evgoal.concl in
    let spdecl' = {evgoal = newgoal;
                   body = EVAR_DEFINED c}
    in (ids_mod (fun evc -> (evc__remap evc sp spdecl')) wc)

  | _ -> error "define_evar"
;;

(* $Id: evar_refiner.ml,v 1.2 1996/02/12 15:47:20 cpaulin Exp $ *)
