(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  evc.ml                                  *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "initial";;
#open "proof";;
#open "proof_trees";;
#open "proof_constraints";;

let lc_add lc x = abs_lc (spavs__add (repr_lc lc) x);;
let lc_memb lc x = spavs__memb (repr_lc lc) x;;
let lc_exists f lc = spavs__exists f (repr_lc lc);;
let lc_equal lc1 lc2 = spavs__equal (repr_lc lc1) (repr_lc lc2);;

let lc_toList lc = spavs__toList (repr_lc lc);;
let lc_ofList l = abs_lc(spavs__ofList l);;

let toList evc = lavm__toList (ts_it (repr_rc  evc)).decls;;
let dom evc = lavm__dom (ts_it (repr_rc evc)).decls;;
let map evc k = lavm__map (ts_it (repr_rc evc)).decls k;;
let in_dom evc k = lavm__in_dom (ts_it (repr_rc evc)).decls k;;

(* add a new declaration to evc. Fail if a declaration with the same
section path already exists *)
let add evc (k,v) = abs_rc(ts_mod
    (fun evc -> {focus=(lc_add evc.focus k);
                     sign = evc.sign;
                     decls = lavm__add evc.decls (k,v)})
         (repr_rc evc));;

let log_add evc (k,v) = abs_rc(ts_mod
    (fun evc -> {focus=(lc_add evc.focus k);
                         sign = evc.sign;
                         decls = lavm__log_add evc.decls (k,v)})
         (repr_rc evc));;


let remap evc k v = abs_rc(ts_mod
    (fun evc -> {focus = evc.focus;
                     sign = evc.sign;
                     decls = lavm__remap evc.decls k v})
         (repr_rc evc));;

let app f evc = lavm__app f (ts_it (repr_rc evc)).decls;;

let set_focus evc f = abs_rc(ts_mod
    (fun evc -> {focus= f;
                 sign = evc.sign;
                 decls = evc.decls})
         (repr_rc evc))
;;

let get_focus evc = (ts_it (repr_rc evc)).focus;;

let equal evc1 evc2 = ts_eq (repr_rc evc1) (repr_rc evc2);;

let saving_focus f evc =
    let focus = (ts_it (repr_rc evc)).focus in
    let evc' = f evc
    in if equal evc evc' then evc
       else abs_rc(ts_mk {sign = (ts_it (repr_rc evc)).sign;
                   focus = focus;
                   decls = (ts_it (repr_rc evc')).decls})
;;

(* MENTIONS SIGMA SP LOC is true exactly when LOC is defined, and SP is
 * on LOC's access list, or an evar on LOC's access list mentions SP.
 *)
let rec mentions sigma sp loc =
let loc_evd = map sigma loc
in match loc_evd.body with 
    EVAR_DEFINED _ -> (lc_memb loc_evd.evgoal.lc sp
                       or lc_exists (mentions sigma sp) loc_evd.evgoal.lc)
  | _ -> false
;;

(* ACCESSIBLE SIGMA SP LOC is true exactly when SP is on LOC's access list,
 * or there exists a LOC' on LOC's access list such that
 * MENTIONS SIGMA SP LOC' is true.
 *)
let rec accessible sigma sp loc =
let loc_evd = map sigma loc
in lc_exists (fun loc' -> sp = loc' or mentions sigma sp loc') loc_evd.evgoal.lc
;;

(* [ctxt_access sigma sp] is true when SIGMA is accessing a current
 * accessibility list ACCL, and SP is either on ACCL, or is mentioned
 * in the body of one of the ACCL.
 *)
let ctxt_access sigma sp =
    lc_exists (fun sp' -> sp' = sp or mentions sigma sp sp') (ts_it (repr_rc sigma)).focus
;;

let is_evar sigma sp = in_dom sigma sp;;

let get_hyps evc = (ts_it (repr_rc evc)).sign;;

let get_gc evc = abs_gc(ts_mk (ts_it (repr_rc evc)).decls);;

(* $Id: evc.ml,v 1.2 1996/02/12 15:47:24 cpaulin Exp $ *)
