#infix "o";;
#open "std";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "environ";;
#open "constrtypes";;
#open "reduction";;
#open "typing";;
#open "termenv";;
#open "himsg";;
#open "stdpp";;
#open "more_util";;
#open "proof";;
#open "pp";;

let whd_betadeltaiota_empty = whd_betadeltaiota mt_evc;;

let it_lambda_name = it_list (fun c (n,t) -> lambda_name (n,t,c)) ;;
let make_lambda_string s t c = DOP2(Lambda,t,DLAM(Name(id_of_string s),c));;

let it_prod_name = it_list (fun c (n,t) -> prod_name (n,t,c)) ;;
let make_prod_string s t c = DOP2(Prod,t,DLAM(Name(id_of_string s),c));;

let lift_context n l = let k = length l in 
          map_i (fun i (name,c) -> (name,liftn n (k-i) c)) 0 l;;


(*******************************************)
(* Building curryfied elimination          *)
(*******************************************)



(*********************************************)
(* lc is the list of the constructors of ind *)
(*********************************************)

let mis_make_case_com depopt sigma mispec kinds =
 let sp = mispec.sp and tyi = mispec.tyi and cl = mispec.args in 
 let nparams = mis_nparams mispec in
 let mip = mispec.mip in
 let mind = DOPN(MutInd(mispec.sp,mispec.tyi),mispec.args) in
 let kd = mis_kd mispec and
     kn = mis_kn mispec and t = mis_arity mispec and
     (lc,lct) = mis_type_mconstructs mispec in
     let lnames,sort = splay_prod sigma t in
     let nconstr = vect_length lc in
     let dep = match depopt with NONE -> (sort<>DOP0(Sort(Prop Null)))
                               | SOME d -> d
     in let _ =  (if dep then 
           (if not (exists (sort_eq_x kinds) kd) then 
         errorlabstrm "Case analysis"
         [< 'S "Dependent case analysis on sort: " ;pTERM (DOP0(Sort kinds));
            'FNL; 'S "is not allowed for inductive definition: "; pTERM mind>])
      else (if not (exists (sort_eq_x kinds)) kn then 
         errorlabstrm "Case analysis"
         [< 'S "Non Dependent case analysis on sort: " ;
                pTERM (DOP0(Sort kinds)); 'FNL;
               'S "is not allowed for inductive definition: "; pTERM mind>]))
     in 

       let lnamesar,lnamespar = chop_list (length lnames - nparams) lnames in
       let lgar = length lnamesar in
       let ar = hnf_prod_appvect sigma "make_case_dep" t (rel_vect 0 nparams)
                in
       let typP = if dep then make_arity_dep (DOP0(Sort kinds)) ar 
                              (appvect (mind,rel_vect 0 nparams))
                  else make_arity_nodep (DOP0(Sort kinds)) ar in 
       let rec add_branch k = 
        if k = nconstr then 
           it_lambda_name 
           (lambda_create 
             (appvect (mind,
                      (concat_vect (rel_vect (nconstr+lgar+1) nparams)
                                   (rel_vect 0 lgar))),
               (DOPN(MutCase,concat_vect [|Rel (nconstr+lgar+2);Rel 1|]
                             (rel_vect (lgar+1) nconstr)))))
           (lift_context (nconstr+1) lnamesar)
        else make_lambda_string "f" 
            (if dep then 
              type_one_branch_dep 
             (sigma,nparams,(rel_list (k+1) nparams),Rel (k+1)) lc.(k) lct.(k)
             else type_one_branch_nodep 
             (sigma,nparams,(rel_list (k+1) nparams),Rel (k+1)) lct.(k))
             (add_branch (k+1))
       in (it_lambda_name (make_lambda_string "P" typP (add_branch 0)) 
                          lnamespar)
     ;;

let make_case_com depopt sigma mind kinds =
 let (DOPN(MutInd (sp,tyi),cl)) as ity = mrectype_spec sigma mind in
 let mispec = mind_specif_of_mind ity in 
 mis_make_case_com depopt sigma mispec kinds;;

let make_case_dep = make_case_com (SOME true);;
let make_case_nodep = make_case_com (SOME false);;
let make_case_gen = make_case_com NONE;;


(* check if the type depends recursively on one of the inductive scheme *)

(* Building the recursive elimination *)

let simple_prod (n,t,c) = DOP2(Prod,t,DLAM(n,c));;

(***********************************************************************
* t is the type of the constructor co and recargs is the information on 
* the recursive calls.                                                  
* build the type of the corresponding branch of the recurrence principle
* assuming f has this type, branch_rec gives also the term 
*   [x1]..[xk](f xi (F xi) ...) to be put in the corresponding branch of 
* the case operation
* FPvect gives for each inductive definition if we want an elimination 
* on it with which predicate and which recursive function. 
************************************************************************)

let make_prod_dep dep = if dep then prod_name else simple_prod;;

let type_rec_branch dep (sigma,vargs,depPvect,decP) co t recargs = 
let make_prod = make_prod_dep dep and
    nparams = vect_length vargs and
    st = hnf_prod_appvect sigma "type_rec_branch" t vargs in

let process_pos depK Pk = prec 0 where rec prec i P = 
match whd_betadeltaiota_stack sigma P [] with 
      (DOP2(Prod,t,DLAM(n,c))),[] -> make_prod (n,t,prec (i+1) c)
    | (DOPN(MutInd _,_),largs) -> let (_,realargs) = chop_list nparams largs
      in let base = applist (lift i Pk,realargs) in       
         if depK then apply base (appvect (Rel (i+1),rel_vect 0 i))
         else base in

let rec process_constr i C recargs co = 
match whd_betadeltaiota_stack sigma C [] with 
      (DOP2(Prod,t,DLAM(n,c)),[]) -> 
          let (optionpos,rest) = (match recargs with 
              [] -> NONE,[] 
            | (Norec::rest) -> NONE,rest 
            | (Imbr _::rest) -> warning "Ignoring recursive call";
                                     NONE,rest 
            |(Mrec j::rest) -> depPvect.(j),rest) in 
          (match optionpos with 
                NONE -> make_prod (n,t,process_constr (i+1) c rest 
                                       (apply (lift 1 co) (Rel 1)))
              | SOME(dep',P) -> 
             let nP = lift (i+1+decP) P 
             in let T = process_pos dep' nP (lift 1 t) in 
          make_prod_dep (dep or dep')
                    (n,t,arrow T (process_constr (i+2) (lift 1 c) rest
                                        (apply (lift 2 co) (Rel 2)))))
    | (DOPN(MutInd(_,tyi),_),largs) -> 
      let nP = (match depPvect.(tyi) with SOME(_,P) -> lift (i+decP) P) in
      let (_,realargs) = chop_list nparams largs 
      in let base = applist (nP,realargs) in
         if dep then apply base co else base

in process_constr 0 st recargs (appvect(co,vargs));;

let rec_branch_arg (sigma,vargs,Fvect,decF) f t recargs = 
let nparams = vect_length vargs and
    st = hnf_prod_appvect sigma "type_rec_branch" t vargs in

let process_pos Fk  = prec 0 where rec prec i P = 
match whd_betadeltaiota_stack sigma P [] with 
      (DOP2(Prod,t,DLAM(n,c))),[] -> lambda_name (n,t,prec (i+1) c) 
    | (DOPN(MutInd _,_),largs) -> 
         let (_,realargs) = chop_list nparams largs
         and arg = appvect (Rel (i+1),rel_vect 0 i) in 
          applist(lift i Fk,realargs@[arg]) in

let rec process_constr i C f recargs = 
match whd_betadeltaiota_stack sigma C [] with 
      (DOP2(Prod,t,DLAM(n,c)),[]) -> 
          let (optionpos,rest) = (match recargs with 
              [] -> NONE,[] | (Norec::rest) -> NONE,rest 
                  | (Imbr _::rest) -> NONE,rest 
                  |(Mrec i::rest) -> Fvect.(i),rest) in 
          (match optionpos with 
                NONE -> lambda_name (n,t,process_constr (i+1) c 
                        (applist(whd_beta_stack (lift 1 f) [(Rel 1)])) rest)
             | SOME(_,F) -> 
             let nF = lift (i+1+decF) F 
             in let arg = process_pos nF (lift 1 t) in 
                lambda_name (n,t,process_constr (i+1) c 
                    (applist(whd_beta_stack (lift 1 f) [(Rel 1); arg])) rest))
    | (DOPN(MutInd(_,tyi),_),largs) -> f

in process_constr 0 st f recargs ;;


let mis_make_indrec sigma listdepkind mispec =

 let nparams =  mis_nparams mispec and
     recargsvec = mis_recargs mispec and
     ntypes = mis_ntypes mispec in
 let mind_arity = mis_arity mispec in 
 let (lnames,DOP0(Sort kind)) = splay_prod sigma mind_arity in
 let lnamespar = lastn nparams lnames in

 let listdepkind = if listdepkind = [] then 
     let dep = kind <> Prop Null in [(mispec,dep,kind)] 
 else listdepkind in

 let nrec = length listdepkind in
 let depPvec = make_vect ntypes (NONE : (bool * constr) option) in 
 let _ = assign nrec listdepkind where rec 
 assign k = function [] -> ()
                 |(mispeci,dep,_)::rest -> 
        (vect_assign depPvec mispeci.tyi (SOME(dep,Rel k));
         assign (k-1) rest) 

in let make_one_rec p = 

let makefix nbconstruct = mrec 0 [] [] [] where rec mrec i ln ltyp ldef = 
 function (mispeci,dep,_)::rest -> 
     let tyi = mispeci.tyi in
     let mind = DOPN(MutInd (mispeci.sp,tyi),mispeci.args) in
     let (_,lct) = mis_type_mconstructs mispeci in
     let nctyi = vect_length lct  (* nb constructeurs du type *) 
     in let realar =  hnf_prod_appvect sigma "make_branch" 
               (mis_arity mispeci) 
               (rel_vect (nrec+nbconstruct) nparams) in
            (* arity in the contexte P1..Prec f1..f_nbconstruct *)
     let lnames,_ = splay_prod sigma realar in 
     let nar = length lnames in 
     let decf = nar+nrec+nbconstruct+nrec in 
     let dect = nar+nrec+nbconstruct in
     let vecfi = rel_vect (dect+1-i-nctyi) nctyi in
     let branches = map3_vect 
               (rec_branch_arg (sigma,rel_vect (decf+1) nparams,depPvec,nar+1))
                  vecfi lct recargsvec.(tyi) in
     let (SOME (_,Rel j)) = depPvec.(tyi) in
     let deftyi = 
     it_lambda_name 
    (lambda_create (appvect (mind,(concat_vect (rel_vect decf nparams)
                                               (rel_vect 0 nar))),
             DOPN(MutCase,concat_vect [|Rel (decf-nrec+j+1);Rel 1|] branches)))
     (lift_context nrec lnames)
     and typtyi = it_prod_name 
    (prod_create (appvect (mind,(concat_vect (rel_vect dect nparams)
                                             (rel_vect 0 nar))),
              (if dep then appvect (Rel (dect-nrec+j+1),rel_vect 0 (nar+1)) 
               else appvect (Rel (dect-nrec+j+1),rel_vect 1 nar))))
          lnames
     in mrec (i+nctyi) (nar::ln) (typtyi::ltyp) (deftyi::ldef) rest
        | [] -> let fixn = vect_of_list (rev ln) 
                and fixtyi = vect_of_list (rev ltyp)
                and fixdef = vect_of_list (rev ldef) in 
        let makefixdef = 
        put_DLAMSV (tabulate_list (fun _ -> Name(id_of_string "F")) nrec) fixdef in 
        let fixspec = concat_vect fixtyi [|makefixdef|] 
        in DOPN(Fix(fixn,p),fixspec)

in let rec make_branch i =  function 
  (mispeci,dep,_)::rest -> 
     let tyi = mispeci.tyi in
     let (lc,lct) = mis_type_mconstructs mispeci
     in onerec 0
     where rec onerec j = 
     if j = vect_length lc then make_branch (i+j) rest 
     else let co = lc.(j) and t = lct.(j) and recarg = recargsvec.(tyi).(j)
          and vargs = rel_vect (nrec+i+j) nparams in
     let P = type_rec_branch dep (sigma,vargs,depPvec,i+j) co t recarg
     in DOP2(Lambda,P,DLAM(Name (id_of_string "f"),onerec (j+1)))
| [] -> makefix i listdepkind

in let rec put_arity i = function 
   ((mispeci,dep,kinds)::rest) -> 
   let mind = DOPN(MutInd (mispeci.sp,mispeci.tyi),mispeci.args) in 
   let arity = mis_arity mispeci
   in let ar =  hnf_prod_appvect sigma "put_arity" arity 
                    (rel_vect i nparams)
   in let typP = if dep then make_arity_dep (DOP0(Sort kinds)) ar 
                    (appvect (mind,rel_vect i nparams))
                  else make_arity_nodep (DOP0(Sort kinds)) ar in 
   DOP2(Lambda,typP,DLAM(Name(id_of_string "P"),put_arity (i+1) rest))
  | [] -> make_branch 0 listdepkind 

in let (mispeci,dep,kind) = nth listdepkind (p+1) in
   if is_recursive (map (fun (mispec,_,_) -> mispec.tyi) listdepkind)
      recargsvec.(mispeci.tyi) then 
   it_lambda_name (put_arity 0 listdepkind) lnamespar
   else mis_make_case_com (SOME dep) sigma mispeci kind 

in tabulate_vect make_one_rec nrec;;

let make_indrec sigma listdepkind mind =
 let (DOPN(MutInd (sp,tyi),largs)) as ity = minductype_spec sigma mind in
 let mispec = mind_specif_of_mind ity in  
 mis_make_indrec sigma listdepkind mispec ;;

let change_sort_arity sort = drec where rec drec
  = function (DOP2(Cast,c,t)) -> drec c 
           | (DOP2(Prod,t,DLAM(n,c))) -> DOP2(Prod,t,DLAM(n,drec c))
           | (DOP0(Sort(_))) -> DOP0(Sort(sort));;

let instanciate_indrec_scheme sort = drec where rec drec npar elim =
    let (DOP2(Lambda,T,DLAM(n,c))) = strip_outer_cast elim in
    if npar = 0 then DOP2(Lambda,change_sort_arity sort T,DLAM(n,c))
                else DOP2(Lambda,T,DLAM(n,drec (npar-1) c));;
 

let check_arities listdepkind = 
    do_list (function (mispeci,dep,kinds) -> 
            let mip = mispeci.mip  in
            if dep then let kd = mis_kd mispeci
            in if exists (sort_eq_x kinds) kd then ()
               else errorlabstrm "Bad Induction"
     [<'S "Dependent induction for type "; print_id mip.MINDTYPENAME;
       'S " and sort "; pTERM (DOP0(Sort kinds)); 'S "is not allowed">]
            else let kn = mis_kn mispeci
            in if exists (sort_eq_x kinds) kn then ()
               else errorlabstrm "Bad Induction"
     [<'S "Non dependent induction for type "; print_id mip.MINDTYPENAME;
       'S " and sort "; pTERM (DOP0(Sort kinds)); 'S "is not allowed">])
    listdepkind;;

let build_indrec sigma ((mind,dep,s)::lrecspec) = 
let (DOPN(MutInd(sp,tyi),_) as redind) = minductype_spec sigma mind in 
let listdepkind = (mind_specif_of_mind redind,dep,s)::
   (map (function (mind',dep',s') -> 
        let (DOPN(MutInd(sp',_),_) as redind') = minductype_spec sigma mind'
        in if sp=sp' then (mind_specif_of_mind redind',dep',s') 
           else error 
     "Induction schemes concern mutually inductive types" ) lrecspec) in
let _ = check_arities listdepkind in 
make_indrec sigma listdepkind mind;;

(* In order to interpret the Match operator *)

let type_mutind_rec env sigma ct Pt P c = 
      let (MI,largs as mind) = find_minductype sigma ct
   in let mispec = mind_specif_of_mind MI in 
      let recargs =  mis_recarg mispec in
     if is_recursive [mispec.tyi] recargs then
        let dep = find_case_dep_mis env sigma mispec (c,P) mind Pt in 
        let ntypes = mis_nconstr mispec 
        and tyi = mispec.tyi 
        and nparams = mis_nparams mispec in
        let depPvec = make_vect ntypes (NONE : (bool * constr) option) in 
        let _ = vect_assign depPvec mispec.tyi (SOME(dep,P)) in 
        let (pargs,realargs) = chop_list nparams largs in
        let vargs = vect_of_list pargs in
        let (constrvec,typeconstrvec) = mis_type_mconstructs mispec 
        in let lft = map3_vect (type_rec_branch dep (sigma,vargs,depPvec,0)) 
                     constrvec typeconstrvec recargs
        in (lft, if dep then applist(P,realargs@[c]) 
                        else applist(P,realargs) )
     else type_case_branches env sigma ct Pt P c;;

let is_mutind sigma ct =  
    try let _ = find_minductype sigma ct in true
    with Induc -> false;;

let type_rec_branches recursive sigma env ct Pt P c = 
    match whd_betadeltaiota_stack sigma ct [] with 
          (DOPN(MutInd _,_),_) -> if recursive then type_mutind_rec env sigma ct Pt P c
                                  else type_case_branches env sigma ct Pt P c
        | _ -> error"Elimination on a non-inductive type 1";;

(* Awful special reduction function which skips abstraction on Xtra in order to 
   be safe for Program ... *)

let stacklamxtra recfun = lamrec where rec lamrec sigma = fun
    stack (DOP2(Lambda,DOP1(XTRA("COMMENT",[]),_),DLAM(_,c)) as t) ->
          recfun stack (substl sigma t)
  | (h::t) (DOP2(Lambda,_,DLAM(_,c))) -> lamrec (h::sigma) t c
  | stack t -> recfun stack (substl sigma t)
;;

let rec whrec x stack =
    match x with   
    DOP2(Lambda,DOP1(XTRA("COMMENT",[]),c),DLAM(name,t)) ->
    let t' = applist (whrec t (map (lift 1) stack))
    in DOP2(Lambda,DOP1(XTRA("COMMENT",[]),c),DLAM(name,t')),[]
  | DOP2(Lambda,c1,DLAM(name,c2)) ->
    (match stack with
     [] -> (DOP2(Lambda,c1,DLAM(name,whd_betaxtra c2)),[])
   | a1::rest -> stacklamxtra (fun l x -> whrec x l) [a1] rest c2)

  | DOPN(AppL,cl)      -> whrec (hd_vect cl) (app_tl_vect cl stack)
  | DOP2(Cast,c,_)       ->  whrec c stack
  | x -> x,stack

and whd_betaxtra x = applist(whrec x []);;

let transform_rec env sigma cl (ct,Pt) = 
      let (MI,largs as mind) = find_minductype sigma ct
   in  let P = cl.(0)
       and c = cl.(1)
       and lf = sub_vect cl 2 ((vect_length cl) - 2) in
       let mispec = mind_specif_of_mind MI in 
       let recargs =  mis_recarg mispec in
       let expn = vect_length recargs in
       if vect_length lf <> expn
     then errorlabstrm "Case with wrong number of cases"
 (HOV 0 [< 'S "Match"; 'S " on term "; pTERMINENV(env,c); 'WS 1 ; 'S "of type"; 'WS 1;
           pTERMINENV(env,ct); 'WS 1; 'S " expects " ; 'INT expn; 'S " branches" >]) 

     else

     if is_recursive [mispec.tyi] recargs then
        let dep = find_case_dep_mis env sigma mispec (c,P) mind Pt in 
        let ntypes = mis_nconstr mispec 
        and tyi = mispec.tyi 
        and nparams = mis_nparams mispec in
        let depFvec = make_vect ntypes (NONE : (bool * constr) option) in 
        let _ = vect_assign depFvec mispec.tyi (SOME(dep,Rel 1)) in 
        let (pargs,realargs) = chop_list nparams largs in
        let vargs = vect_of_list pargs in
        let (_,typeconstrvec) = mis_type_mconstructs mispec in
(* build now the fixpoint *)
        let realar =  hnf_prod_appvect sigma "make_branch" (mis_arity mispec) vargs in
        let lnames,_ = splay_prod sigma realar in 
        let nar = length lnames in
        let branches = map3_vect 
           (fun f t reca -> 
                 whd_betaxtra
         (rec_branch_arg (sigma,(map_vect (lift (nar+2)) vargs),depFvec,nar+1) f t reca))
                       (map_vect (lift (nar+2)) lf) typeconstrvec recargs in 
        let deffix = it_lambda_name 
             (lambda_create 
             (appvect (MI,concat_vect (map_vect (lift (nar+1)) vargs)
                           (rel_vect 0 nar)),
             DOPN(MutCase,concat_vect [|(lift (nar+2) P);Rel 1|] branches)))
                    (lift_context 1 lnames) in
        if noccurn 1 deffix then whd_beta (applist (pop deffix,realargs@[c]))
        else
        let typPfix = it_prod_name  
          (prod_create (appvect (MI,(concat_vect 
                                    (map_vect (lift nar) vargs) (rel_vect 0 nar))),
              (if dep then applist (whd_beta_stack (lift (nar+1) P) (rel_list 0 (nar+1)))
               else applist (whd_beta_stack (lift (nar+1) P) (rel_list 1 nar))))) lnames
        in let fix = DOPN(Fix([|nar|],0),[|typPfix;
                             DLAMV(Name(id_of_string "F"),[|deffix|])|])
        in applist (fix,realargs@[c])

     else DOPN(MutCase,concat_vect [|P;c|] lf);;


(* $Id: indrec.ml,v 1.8 1996/02/12 15:47:36 cpaulin Exp $ *)
