(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 logic.ml                                 *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "initial";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "himsg";;
#open "termenv";;
#open "machops";;
#open "mach";;
#open "more_util";;
#open "proof_trees";;
#open "proof";;
#open "pp";;
#open "stdpp";;
#open "typing";;

let pr_prim_rule = function
    {name="INTRO";newids=[id];_} -> [< 'S"intro " ; print_id id >]

  | {name="INTRO-AFTER";newids=[id];_} -> [< 'S"intro after " ; print_id id >]

  | {name="INTRO-REPLACING";newids=[id];_} -> [< 'S"intro replacing " ; print_id id >]

  | {name="FIX";newids=[f];params=[ast__TOKnum n];_} -> 
         [< 'S"Fix "; print_id f; 'S"/"; 'INT n>]

  | {name="FIX";newids=(f::lf);params=(ast__TOKnum n)::ln;terms=lar;_} -> 
        let rec print_mut = 
           function (f::lf),((ast__TOKnum n)::ln),(ar::lar) -> 
       [< print_id f; 'S"/"; 'INT n; 'S" : "; pTERM ar; print_mut (lf,ln,lar)>]
                  | _ -> [<>] in
   [< 'S"Fix "; print_id f; 'S"/"; 'INT n; 'S" with "; print_mut (lf,ln,lar) >]

  | {name="COFIX";newids=[f];terms=[];_} -> 
         [< 'S"Cofix "; print_id f >]

  | {name="COFIX";newids=(f::lf);terms=lar;_} -> 
        let rec print_mut = 
           function (f::lf),(ar::lar) -> 
       [< print_id f; 'S" : "; pTERM ar; print_mut (lf,lar)>]
                  | _ -> [<>] in
   [< 'S"Cofix "; print_id f;  'S" with "; print_mut (lf,lar) >]

  | {name="REFINE";terms=[c];_} -> [< 'S"refine " ; pTERM c >]

  | {name="CONVERT_CONCL";terms=[c];_} ->
    [< 'S"convert_concl " ; pTERM c >]

  | {name="CONVERT_HYP";hypspecs=[id];terms=[c];_} ->
    [< 'S"convert_hyp " ; print_id id ; 'SPC ; pTERM c >]

  | {name="THIN";hypspecs=ids;_} ->
    [< 'S"thin " ; prlist_with_sep (fun () -> [< 'SPC >]) print_id ids >]

  | _ -> anomaly "pr_prim_rule: Unrecognized primitive rule"
;;

let rec mkREFGOALS sigma goal goalacc conclty = function

    DOP0(Meta mv) ->
    if occur_meta conclty then
        error "Cannot refine to conclusions with meta-variables"
    else 
        {hyps=goal.hyps;ctxt=goal.ctxt;lc=goal.lc;
         concl=strong whd_betaiota conclty}::goalacc,conclty

  | DOP2(Cast,DOP0(Meta mv),ty) ->
    let _ = type_of sigma goal.hyps ty
    in if conv_x sigma ty conclty then
        {hyps=goal.hyps;ctxt=goal.ctxt;lc=goal.lc;
         concl=strong whd_betaiota ty}::goalacc,conclty
       else failwith "refine"

  | DOPN(AppL,cl) ->
    let (acc',hdty) = mkHDGOALS sigma goal goalacc (hd_vect cl) in
    let (acc'',conclty') = mkARGGOALS sigma goal acc' hdty (list_of_tl_vect cl)
    in if conv_x sigma conclty' conclty then
        (acc'',conclty')
       else failwith "refine"

  | DOPN(MutCase,cl) -> 
    let (acc',lbrty,conclty') = mkCASEGOALS sigma goal goalacc cl.(0) cl.(1) in
    let acc'' = it_vect2 
                (fun lacc ty fi -> fst (mkREFGOALS sigma goal lacc ty fi))
                acc' lbrty (sub_vect cl 2 ((vect_length cl) - 2))
    in if conv_x sigma conclty' conclty then
        (acc'',conclty')
       else failwith "refine"

  | t -> if occur_meta t then failwith "refine" else
    let t'ty = type_of sigma goal.hyps t
    in if conv_x sigma t'ty conclty then
        goalacc,t'ty
       else failwith "refine"

and mkHDGOALS sigma goal goalacc = function
    DOP2(Cast,DOP0(Meta mv),ty) ->
    let _ = type_of sigma goal.hyps ty
    in {hyps=goal.hyps;ctxt=goal.ctxt;lc=goal.lc;
        concl=strong whd_betaiota ty}::goalacc,ty

  | DOPN(AppL,cl) ->
    let (acc',hdty) = mkHDGOALS sigma goal goalacc (hd_vect cl)
    in mkARGGOALS sigma goal acc' hdty (list_of_tl_vect cl)

  | DOPN(MutCase,cl) -> 
    let (acc',lbrty,conclty') = mkCASEGOALS sigma goal goalacc cl.(0) cl.(1) in
    let acc'' = it_vect2 
                (fun lacc ty fi -> fst (mkREFGOALS sigma goal lacc ty fi))
                acc' lbrty (sub_vect cl 2 ((vect_length cl) - 2))
    in (acc'',conclty')

  | t -> goalacc,type_of sigma goal.hyps t

and mkARGGOALS sigma goal goalacc funty = function
    [] -> goalacc,funty
  | harg::tlargs ->
    (match whd_betadeltaiota sigma funty with
     DOP2(Prod,c1,B) ->
     let (acc',hargty) = mkREFGOALS sigma goal goalacc c1 harg
     in mkARGGOALS sigma goal acc' (SAPP B harg) tlargs
   | _ -> failwith "refine")

and mkCASEGOALS sigma goal goalacc P c= 
   let (acc',ct) = mkHDGOALS sigma goal goalacc c in 
   let (acc'',Pt) = mkHDGOALS sigma goal acc' P in
   let (lbrty,conclty) = type_case_branches (GLOB goal.hyps) sigma ct Pt P c in
   (acc'',lbrty,conclty)
;;

let collect_meta_variables c = rev(collrec [] c)
    where rec collrec acc = function
    DOP0(Meta mv) -> mv::acc
  | DOP2(Cast,c,t) -> collrec acc c
  | DOPN(AppL,cl) -> it_vect collrec acc cl
  | DOPN(MutCase,cl) -> it_vect collrec acc cl
  | _ -> acc;;

let new_meta_variables = newrec
    where rec newrec = function
    DOP0(Meta mv) -> DOP0(Meta (newMETA()))
  | DOP2(Cast,c,t) -> DOP2(Cast,newrec c,t)
  | DOPN(AppL,cl) -> DOPN(AppL,map_vect newrec cl)
  | DOPN(MutCase,cl) -> DOPN(MutCase,map_vect newrec cl)
  | x -> x;;

let mk_assumption sigma sign c =
    assumption_of_judgement sigma (fexecute sigma sign c);;

let sign_before id = aux
    where rec aux = function
    [],[] -> error "no such hypothesis"
  | sign -> if fst(hd_sign sign) = id then tl_sign sign
            else aux (tl_sign sign)
;;

let prim_refiner r sigma goal =
    match r with
    {name="INTRO";newids=[id];_} ->
    let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal
    in if not(mem_sign sign id) then
        (match strip_outer_cast cl with
         DOP2(Prod,c1,B) ->
         if occur_meta c1 then error "Cannot intro when there are open metavars in the domain type - use Instantiate"
         else
         let a = mk_assumption sigma sign c1
         and v = VAR id in
         let sg = mkGOAL lc ctxt (add_sign (id,a) sign) (SAPP B v)
         in [sg]
       | _ -> error "INTRO used non-product")
    else error "new variable is already declared"

  | {name="INTRO-AFTER";newids=[id];hypspecs=[whereid];_} ->
    let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal
    in if not(mem_sign sign id) then
        (match strip_outer_cast cl with
         DOP2(Prod,c1,B) ->
         if occur_meta c1 then
             error "Cannot intro when there are open metavars in the domain type - use Instantiate"
         else if not(for_all (mem_sign (sign_prefix whereid sign)) (global_vars c1)) then
             error "Can't introduce at that location: free variable conflict"
         else
         let a = mk_assumption sigma sign c1
         and v = VAR id in
         let sg = mkGOAL lc ctxt (add_sign_after whereid (id,a) sign) (SAPP B v)
         in [sg]
       | _ -> error "INTRO used non-product")
    else error "new variable is already declared"

  | {name="INTRO-REPLACING";newids=[];hypspecs=[id];_} ->
    let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal
    in (match strip_outer_cast cl with
         DOP2(Prod,c1,B) ->
         if occur_meta c1 then
             error "Cannot intro when there are open metavars in the domain type - use Instantiate"
         else if not(for_all (mem_sign (tl_sign (sign_prefix id sign))) (global_vars c1))
                  or exists (occur_var id) (vals_of_sign sign) then
             error "Can't introduce at that location: free variable conflict"
         else
         let a = mk_assumption sigma sign c1
         and v = VAR id in
         let sg = mkGOAL lc ctxt (add_sign_replacing id (id,a) sign) (SAPP B v)
         in [sg]
       | _ -> error "INTRO used non-product")

  | {name="FIX";hypspecs=[];terms=[];newids=[f];params=[ast__TOKnum n]} ->
     let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal
     in let rec check_ind k cl = 
        match whd_castapp cl with 
            DOP2(Prod,c1,DLAM(_,B)) -> 
            if k = 1 then (try find_minductype sigma c1 
              with Induc -> error "cannot do a fixpoint on a non inductive type")
            else check_ind (k-1) B
          | _ -> error "not enough products"
     in let _ = check_ind n cl
        in if not(mem_sign sign f) then 
        let a = mk_assumption sigma sign cl
        in let sg = mkGOAL lc ctxt (add_sign (f,a) sign) cl
        in [sg]
           else error "name already used in the environment"
    
  | {name="FIX";hypspecs=[];terms=lar;newids=lf;params=ln} ->
     let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal
     in let rec check_ind k cl = 
        match whd_castapp cl with 
            DOP2(Prod,c1,DLAM(_,B)) -> 
            if k = 1 then (try find_minductype sigma c1 
              with Induc -> error "cannot do a fixpoint on a non inductive type")
            else check_ind (k-1) B
          | _ -> error "not enough products" in
     let (ast__TOKnum n) = hd ln in
     let (DOPN(MutInd(sp,_),_),_) = check_ind n cl
     in let rec mk_sign sign = function 
        (ar::lar'),(f::lf'),((ast__TOKnum n)::ln')->
        let (DOPN(MutInd(sp',_),_),_)  = check_ind n ar
        in if not (sp=sp') then error 
         "fixpoints should be on the same mutual inductive declaration"
        else
        if not(mem_sign sign f) then 
        let a = mk_assumption sigma sign ar
        in mk_sign (add_sign (f,a) sign) (lar',lf',ln')
           else error "name already used in the environment"
      | ([],[],[]) -> map (mkGOAL lc ctxt sign) (cl::lar)
      | _ -> error "not the right number of arguments"
     in mk_sign sign (cl::lar,lf,ln)

  | {name="COFIX";hypspecs=[];terms=lar;newids=lf;params=[]} ->
     let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal
     in let rec check_is_coind cl = 
                 match whd_castapp cl with 
                   DOP2(Prod,c1,DLAM(_,B)) -> check_is_coind  B
          |      B  -> ((try find_mcoinductype sigma B
                        with Induc -> error "All methods must construct elements in coinductive types");true)
     in let _ = for_all check_is_coind (cl::lar)
        in let rec mk_sign sign = function 
        (ar::lar'),(f::lf') ->
            if not(mem_sign sign f) then 
               let a = mk_assumption sigma sign ar
               in mk_sign (add_sign (f,a) sign) (lar',lf')
            else error "name already used in the environment"
      | ([],[]) -> map (mkGOAL lc ctxt sign) (cl::lar)
      |  _      -> error "not the right number of arguments"
     in mk_sign sign (cl::lar,lf)


  | {name="REFINE";terms=[c]} ->
   (try
    let {hyps=sign;concl=cl;ctxt=ctxt;_} = goal in
    let c = new_meta_variables c in
    let r = PRIM{name="REFINE";terms=[c];hypspecs=[];newids=[];params=[]} in
    let (sgl,cl') = mkREFGOALS sigma goal [] cl c in
    let sgl = rev sgl
    in sgl
    with Failure "refine" ->
        errorlabstrm "refine"
        [< 'S"Refine rule passed ill-formed term." ; 'FNL ;
           'S"Goal: " ; pSEQ goal ; 'FNL ;
           'S"Term: " ; pTERM c >])

  | {name="CONVERT_CONCL";terms=[cl'];_} ->
    let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal in
    let cl'ty = type_of sigma sign cl'
    in if conv_x sigma cl' cl then
        let sg = mkGOAL lc ctxt sign (DOP2(Cast,cl',cl'ty))
        in [sg]
     else error "convert-concl rule passed non-converting term"

  | {name="CONVERT_HYP";hypspecs=[id];terms=[ty'];_} ->
    let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal in
    let ty'ty = type_of sigma (sign_before id sign) ty'
    in if conv_x sigma ty' (snd(lookup_sign id sign)) then
        let sg = mkGOAL lc ctxt (modify_sign id (DOP2(Cast,ty',ty'ty)) sign) cl
        in [sg]
    else error "convert-hyp rule passed non-converting term"

   | {name="THIN";hypspecs=ids;_} ->
     let {hyps=sign;concl=cl;ctxt=ctxt;lc=lc;_} = goal in
     let rec remove_pair s = function
         ([],[]) -> error ((string_of_id s) ^ " is not among the assumptions.")
       | sign -> let (s',ty),tlsign = uncons_sign sign
                 in if s = s' then tlsign
                    else if occur_var s ty then
                        error ((string_of_id s) ^ " is used in the hypothesis " ^ (string_of_id s'))
                    else add_sign (s',ty) (remove_pair s tlsign)
     | _ -> anomaly "clear_hyp: signature not well-formed" in
     let rec clear_hyp_aux sign s =
         if (occur_var s cl) then
             error ((string_of_id s) ^ " is used in the conclusion.")
         else remove_pair s sign in
     let sg = mkGOAL lc ctxt (it_list clear_hyp_aux sign ids) cl
     in [sg]

   | _ -> error "prim_refiner: Unrecognized primitive rule"
;;


let extract_constr = crec where rec crec vl = crecvl where rec
 crecvl = function
 VAR str ->
 (try (match lookup_id str vl with
      GLOBNAME (id,_) -> VAR id
    | RELNAME (n,_) -> Rel n)
 with Not_found -> 
     (try search_reference vl str with Not_found -> error((string_of_id str)^"  not declared")))

| (Rel _) as val -> val

| DOP2(Lambda,c1,DLAM(na,c2)) ->
  let u1 = crecvl c1 in
      DOP2(Lambda,u1,DLAM(na,crec (add_rel (Anonymous,u1) vl) c2))
| DOPN(Fix par,cl) -> let listar = sub_vect cl 0 ((vect_length cl) -1) 
      and def = last_vect cl in 
      let newar = map_vect crecvl listar in
      let newenv = it_vect (fun env ar -> add_rel (Anonymous,ar) env) vl newar 
      in let newdef = under_dlams (crec newenv) def in 
      DOPN(Fix par,concat_vect newar [|newdef|])
| DOPN(CoFix par,cl) -> let listar = sub_vect cl 0 ((vect_length cl) -1) 
      and def = last_vect cl in 
      let newar = map_vect crecvl listar in
      let newenv = it_vect (fun env ar -> add_rel (Anonymous,ar) env) vl newar 
      in let newdef = under_dlams (crec newenv) def in 
      DOPN(CoFix par,concat_vect newar [|newdef|])
| DOP2(Prod,c1,DLAM(na,c2)) ->
  let u1 = crecvl c1 in
      DOP2(Prod,u1,DLAM(na,crec (add_rel (Anonymous,u1) vl) c2))
| DOP2(Cast,c,t)    -> DOP2(Cast,crecvl c,crecvl t)
|(DOPN(Abst _,_) as val) -> crecvl (abst_value val)
| DOPN(oper,cl)     -> DOPN(oper,map_vect crecvl cl)
| DOPL(oper,cl)     -> DOPL(oper,map crecvl cl)
| DOP0(Implicit) -> anomaly "found a term Implicit in a construction"
| DOP0(Meta _) as c -> c
| DOP0 _ as c -> c
;;

let prim_extractor subfun vl = function

    {ref=SOME(PRIM{name="INTRO";newids=[id];_},[spf]);
     goal={concl=cl;_};_} ->
    (match strip_outer_cast cl with
     DOP2(Prod,ty,B) ->
     let cty = extract_constr vl ty in
         let na = Name id
         in DOP2(Lambda,cty,DLAM(na,subfun (add_rel (na,cty) vl) spf))
   | _ -> error "incomplete proof!")

   | {ref=SOME(PRIM{name="INTRO-AFTER";newids=[id];_},[spf]);
     goal={concl=cl;_};_} ->
    (match strip_outer_cast cl with
     DOP2(Prod,ty,B) ->
     let cty = extract_constr vl ty in
         let na = Name id
         in DOP2(Lambda,cty,DLAM(na,subfun (add_rel (na,cty) vl) spf))
   | _ -> error "incomplete proof!")

   | {ref=SOME(PRIM{name="INTRO-REPLACING";hypspecs=[id];_},[spf]);
     goal={concl=cl;_};_} ->
    (match strip_outer_cast cl with
     DOP2(Prod,ty,B) ->
     let cty = extract_constr vl ty in
         let na = Name id
         in DOP2(Lambda,cty,DLAM(na,subfun (add_rel (na,cty) vl) spf))
   | _ -> error "incomplete proof!")

   | {ref=SOME(PRIM{name="FIX";newids=[f];params=[ast__TOKnum n];_},[spf]);
     goal={concl=cl;_};_} ->
     let cty = extract_constr vl cl in 
     let na = Name f in 
         DOPN(Fix([|n-1|],0),[| cty ; 
                             DLAMV(na,[|subfun (add_rel (na,cty) vl) spf|])|])

   | {ref=SOME(PRIM{name="FIX";newids=lf;terms=lar;params=ln;_},spfl);
     goal={concl=cl;_};_} ->
     let lcty = map (extract_constr vl) (cl::lar) in 
     let vn = vect_of_list (map (function ast__TOKnum n -> n-1
                                       | _ -> anomaly "mutual_fix_refiner")
                            ln) in 
     let lna = map (fun f -> Name f) lf in
     let newvl = it_list2 (fun sign na ar -> (add_rel (na,ar) sign)) 
                 vl lna lcty in 
     let lfix =map_vect (subfun newvl) (vect_of_list spfl) in
         DOPN(Fix(vn,0),vect_of_list (lcty@[put_DLAMSV (rev lna) lfix]))

  | {ref=SOME(PRIM{name="COFIX";newids=lf;terms=lar;_},spfl);
     goal={concl=cl;_};_} ->
     let lcty = map (extract_constr vl) (cl::lar) in 
     let lna  = map (fun f -> Name f) lf in
     let newvl = it_list2 (fun sign na ar -> (add_rel (na,ar) sign)) 
                 vl lna lcty in 
     let lfix =map_vect (subfun newvl) (vect_of_list spfl) in
         DOPN(CoFix(0),vect_of_list (lcty@[put_DLAMSV (rev lna) lfix]))

   | {ref=SOME(PRIM{name="REFINE";terms=[c];_},spfl);
      goal={concl=cl;_};_} ->
     let mvl = collect_meta_variables c in
     let metamap = combine(mvl,map (subfun vl) spfl) in
     let cc = extract_constr vl c
     in instance metamap cc

   | {ref=SOME(PRIM{name="CONVERT_CONCL";terms=[c];_},[pf]);_} ->
     subfun vl pf

   | {ref=SOME(PRIM{name="CONVERT_HYP";hypspecs=[id];terms=[c];_},[pf]);_} ->
     subfun vl pf

   | {ref=SOME(PRIM{name="THIN";hypspecs=ids;_},[pf]);_} ->
     subfun vl pf

   | {ref=SOME(PRIM _,_);_} ->
     error "prim_extractor handed unrecognizable primitive proof"

   | {ref=NONE;_} -> error "prim_extractor handed incomplete proof"
   | _ -> anomaly "prim_extractor"
;;



(* $Id: logic.ml,v 1.7 1996/02/12 15:47:45 cpaulin Exp $ *)
