(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                printer.ml                                *)
(****************************************************************************)

#open "initial";;
#open "std";;
#open "pp";;
#open "stdpp";;
#open "names";;
#open "proof";;
#open "more_util";;
#open "unparsings";;
#open "term";;
#open "ast";;
#open "termast";;
#open "termenv";;
#open "proof";;
#open "proof_trees";;
#open "proof_constraints";;

let print_arguments = ref false;;
let print_casts = ref false;;

(******
let pr_global dflt k oper =
    if is_existential_oper oper & not !print_arguments then
        [< 'S"?" >]
    else
    try let id = id_of_global oper in
        let (oper',_) = global_operator (nametab__sp_of_id k id) id
        in if oper = oper' then
            [< 'S(string_of_id id) >]
           else dflt
    with (Failure _)|Not_found ->
        [< 'S"[Error printing " ; dflt ; 'S"]" >]
       | _ -> [< 'S"Error [Nasty error printing " ; dflt ; 'S"]" >]
;;
******************)

let pr_global dflt k oper =
    try let id = id_of_global oper in
    if is_existential_oper oper then
        [< 'S (string_of_id id) >]
    else let (oper',_) = global_operator (nametab__sp_of_id k id) id
        in if oper = oper' then
            [< 'S(string_of_id id) >]
           else dflt
    with (Failure _)|Not_found ->
        [< 'S"[Error printing " ; dflt ; 'S"]" >]
       | _ -> [< 'S"Error [Nasty error printing " ; dflt ; 'S"]" >]
;;



let first_result = fun f -> aux
where rec aux = function
    [] -> NONE
  | (h::t) -> try (SOME (f h))
              with Failure _ | UserError _ -> aux t
;;

let apply_prec = (("Term",(9,0,0)),L);;

(**)
let apply_tacprec = (("Tactic",(9,0,0)),L);;
(**)


(* [genprint unpmap dflt gterm]
 * prints out the term GTERM, following the indications in the
 * unparsing-map, UNPMAP.  If the term is not matched by anything in
 * unpmap, then it will call dflt on it, which is responsible
 * for printing out the term, with an error message, if needed.
 *)
let genprint inhprec whatfor primpr dflt =
let rec pr inherited gt =
try
let unpmap = syntax__map whatfor gt
in match first_result (fun se ->
                           (se,smatch se.syntax__syn_astpat (FRAGast gt))) unpmap with
    NONE -> dflt gt
  | SOME (se,bind) -> 
    let gtbind' = map (fun (id,gt) -> (id,ssub bind gt)) se.syntax__syn_recbindings in
    let bind = gtbind'@bind in
    let subprims = map_succeed (fun (id,FRAGprim p) -> (id,primpr p)
                                  | _ -> failwith "caught") bind
    and subgts = map_succeed (fun (id,FRAGast gt) -> (id,(fun inh -> pr (SOME inh) gt))
                        | _ -> failwith "caught") bind
    in (match inherited with
        NONE -> apply_unparsing (dflt gt) (se.syntax__syn_id,se.syntax__syn_unparsing) (subprims@subgts)
      | SOME(par,reln) ->
        if tolerable NONE par ((se.syntax__syn_id,se.syntax__syn_unparsing.prec),reln) then
            apply_unparsing (dflt gt) (se.syntax__syn_id,se.syntax__syn_unparsing) (subprims@subgts)
        else [< 'S"(" ; apply_unparsing (dflt gt) (se.syntax__syn_id,se.syntax__syn_unparsing) (subprims@subgts) ; 'S")" >])
with Failure _ -> [< 'S"[Pretty-printing failure: " ; dflt gt ; 'S"]" >]
   | Not_found -> [< 'S"[Search failure during pretty-printing: " ; dflt gt ; 'S"]" >]
in pr inhprec
;;

let dfltpr ast = [< 'S"#GENTERM " ; print_ast ast >];;

let globpr k = fun
    (ASTnvar id) -> [< 'S(string_of_id id) >]

  | (ASTnode("CONST",
             (ASTprim(TOKpath sp))::_) as gt) ->
     if !print_arguments then
         dfltpr gt
     else
         pr_global (dfltpr gt) k (Const sp)

  | (ASTnode("MUTIND",
             (ASTprim(TOKpath sp))::(ASTprim(TOKnum tyi))::_) as gt) ->
     if !print_arguments then
         (dfltpr gt)
     else pr_global (dfltpr gt) k (MutInd(sp,tyi))

  | (ASTnode("MUTCONSTRUCT",
             (ASTprim(TOKpath sp))::(ASTprim(TOKnum tyi))::(ASTprim(TOKnum i))::_) as gt) ->
     if !print_arguments then
         (dfltpr gt)
     else pr_global (dfltpr gt) k (MutConstruct((sp,tyi),i))

  | gt -> [< 'S"#GENTERM " ; print_ast gt >]
;;


let rec gencompr k gt =
let uni = (match k with
           (CCI |FW) -> (id_of_string "constr")
(*         | FW -> (id_of_string "fterm") *)
         | _ -> id_of_string "<undefined>") in
let rec gpr gt = genprint (SOME apply_prec) uni
    (fun (TOKpath sp) _ -> [< 'S(nametab__print_name sp) >]
       | p _ -> print_tok p)
    (fun (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "CCI");c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "FW");c])) -> gencompr FW c
       | gt -> globpr k gt) gt
in gpr gt
;;

let gentermpr k env t =
let gt = (if !print_casts then bdize else bdize_no_casts) (unitize_env env) t in gencompr k gt;;

(*****
let uni = (match k with
           (CCI | FW) -> (id_of_string "constr")
   (*      | FW -> (id_of_string "fterm") *)
         | _ -> id_of_string "<undefined>") in
let rec gpr gt = genprint (SOME apply_prec) uni
    (fun (TOKpath sp) _ -> [< 'S(nametab__print_name sp) >]
       | p _ -> print_tok p)
    (fun (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "CCI");c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "FW");c])) -> gencompr FW c
       | gt -> globpr k gt)
    gt
in gpr gt
;;
****)

let term0 a = gentermpr CCI a;;
let prterm = term0 (GLOB nil_sign);;
let fterm0 a = gentermpr FW a;;
let fprterm = fterm0 (GLOB nil_sign);;

(***********
let gentacpr gt =
let rec gpr gt = genprint (SOME apply_prec) (id_of_string "tactic")
    (fun (TOKpath sp) _ -> [< 'S(nametab__print_name sp) >]
       | p _ -> print_tok p)
    (fun (ASTnvar id) -> [< 'S(string_of_id id) >]
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "CCI");c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "FW");c])) -> gencompr FW c

       | gt -> globpr OBJ gt) gt
in gpr gt
;;
**********)

let gentacpr gt =
let rec gpr gt = genprint (SOME apply_tacprec) (id_of_string "tactic")
    (fun (TOKpath sp) _ -> [< 'S(nametab__print_name sp) >]
       | p _ -> print_tok p)
    (fun (ASTnvar id) -> [< 'S(string_of_id id) >]
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "CCI");c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "FW");c])) -> gencompr FW c

       | gt -> globpr OBJ gt) gt
in gpr gt
;;


let genvernacpr gt =
let rec gpr gt = genprint (SOME apply_prec) (id_of_string "vernac")
    (fun (TOKpath sp) _ -> [< 'S(nametab__print_name sp) >]
       | p _ -> print_tok p)
    (fun (ASTnvar id) -> [< 'S(string_of_id id) >]
       | (ASTnode("PPUNI$TACTIC",[c])) -> gentacpr c
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "CCI");c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[c])) -> gencompr CCI c
       | (ASTnode("PPUNI$COMMAND",[ASTprim(TOKstr "FW");c])) -> gencompr FW c

       | gt -> globpr OBJ gt) gt
in gpr gt
;;

let print_decl sign (s,typ) =
    [< print_id s ; 'S" : "; term0 (GLOB sign) typ >];;

let print_binding env (na,typ) =
    match na with
    Anonymous -> [< 'S"<>" ; 'S" : " ; term0 env typ >]
  | Name id -> [< print_id id ; 'S" : "; term0 env typ >]
;;

(* Prints out an "env" in a nice format.  We print out the
 * signature,then a horizontal bar, then the debruijn environment.
 * It's printed out from outermost to innermost, so it's readable. *)

let sign_it_with f sign e =
    snd(sign_it (fun id T (sign,e) -> (add_sign (id,T) sign,
                                   f id T sign e))
            sign (nil_sign,e))
;;

let sign_it_with_i f sign e =
   let (_,_,e') = (sign_it (fun id T (i,sign,e) -> (i+1,add_sign (id,T) sign,
                                   f i id T sign e))
                    sign (0,nil_sign,e))
   in e'
;;


let dbenv_it_with f env e =
    snd(dbenv_it (fun na T (env,e) -> (add_rel (na,T) env,
                                       f na T env e))
                 env (GLOB(get_globals env),e))
;;

let pr_term_env env =
[< (sign_it_with (fun id T sign pps ->
                      [<  pps ; 'FNL ; print_decl sign (id,T) >])
    (get_globals env) [< >]) ; 

 (dbenv_it_with (fun na T env pps ->
                     [<  pps ; 'FNL ; print_binding env (na,T) >])
  env [< >]) >]
;;

let pr_term_env_limit n env =
let sign = get_globals env 
in let lgsign = sign_length sign 
in if n>=lgsign then pr_term_env env else let k = lgsign-n in
[< (sign_it_with_i (fun i id T sign pps -> 
                    if i < k then [< pps ;'S "." >] else
                      [<  pps ; 'FNL ; print_decl sign (id,T) >])
    (get_globals env) [< >]) ; 

 (dbenv_it_with (fun na T env pps ->
                     [<  pps ; 'FNL ; print_binding env (na,T) >])
  env [< >]) >]
;;

let pr_term_env_opt env =  match print_hyps_limit () with 
  NONE -> pr_term_env env | SOME n -> pr_term_env_limit n env;;

let pr_goal g = 
  [< V 2 [< pr_term_env_opt (GLOB g.hyps); 'CUT;
           'S "============================"; 'FNL ;
           'S" " ; prterm g.concl >] ; 'FNL>]
;;

let pr_concl n g =
 [< 'S "subgoal ";'INT n;'S " is:";'CUT;'S" " ; prterm g.concl >];;

let pr_subgoals = function
    [] -> [< 'S"Subtree proved!" ; 'FNL >]
  | [g] ->
      V 0 [< 'S ("1 "^"subgoal");'CUT;pr_goal g >]

  | g1::rest ->
      let rec pr_rec n = function
          [] -> [< >]
        | g::rest -> [< 'CUT;pr_concl n g;pr_rec (n+1) rest >] in
       V 0
       [< 'INT(length rest+1) ; 'S" subgoals" ;'CUT;
       pr_goal g1;pr_rec 2 rest >];;

let pr_subgoal n =
 let rec prrec p = function
     [] -> error "No such goal"
   | g::rest ->
       if p = 1
        then (V 0
            [< 'S "subgoal ";'INT n;'S " is:";'CUT;pr_goal g >])
        else prrec (p-1) rest in
  prrec n;;

let pr_idl idl =
    prlist_with_sep (fun () -> [< 'S" " >]) print_id idl
;;

let pr_pgm ctxt =
    match ctxt.pgm with
    NONE -> [< >]
  | SOME pgm -> [< 'S"Realizer " ; fprterm pgm >]
;;

let pr_ctxt ctxt =
    [< 'S"[" ; pr_pgm ctxt ; 'S"]" >]
;;

let pr_seq {hyps=sign;concl=cl;ctxt=ctxt} =
let sign = rev(combine sign)
in HOV 0
    [< pr_ctxt ctxt ; 
       prlist_with_sep (fun () -> [< 'SPC >])
                      (fun (id,ty) -> HOV 0 [< print_id id; 'S":";prterm ty >])
                       sign;
      'SPC ; HOV 0 [< 'S"|- " ; prterm cl >] >]
;;

let prgl gl =
    [< 'S"[[" ; pr_idl (map basename (evc__lc_toList gl.lc)) ; 'S"]" ;
       pr_seq gl ; 'S"]" ; 'SPC >]
;;

let pr_evgl gl =
    [< 'S"[[" ; pr_idl (map basename (evc__lc_toList gl.lc)) ; 'S"]" ;
       'S" " ; pr_idl (ids_of_sign gl.hyps) ;
       'S" |- " ; 'SPC ; prterm gl.concl ; 'S"]" >]
;;

let pr_evd evd =
    H 0 [< 'S"[" ; pr_idl (map basename (evc__lc_toList evd.evgoal.lc)) ; 'S"]" ;
         H 0 [< pr_evgl evd.evgoal ;
              (match evd.body with
               EVAR_EMPTY -> [< 'FNL >]
             | EVAR_DEFINED c -> [< 'S" => " ; prterm c >]) >] >]
;;

let pr_focus accl = pr_idl (map basename (evc__lc_toList accl));;

let pr_evc evc =
let evc = ts_it (repr_rc evc) and
    stamp = ts_stamp (repr_rc evc) in
    [< 'S"#" ; 'INT stamp ; 'S"[" ; pr_focus evc.focus ; 'S"]=" ;
    prlist_with_sep
    (fun () -> [< 'FNL >])
    (fun (sp,evd) -> H 0 [< print_id (basename sp) ; 'S"==" ; pr_evd evd >])
    (lavm__nolog_toList evc.decls) >]
;;

let pr_decls decls =
    prlist_with_sep
    (fun () -> [< 'FNL >])
    (fun (sp,evd) -> H 0 [< print_id (basename sp) ; 'S"==" ; pr_evd evd >])
    (lavm__nolog_toList (ts_it (repr_gc decls)))
;;


(* $Id: printer.ml,v 1.6 1996/02/12 15:47:57 cpaulin Exp $ *)
