(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              proof_trees.ml                              *)
(****************************************************************************)


#open "initial";;
#open "names";;
#open "more_util";;

let mkGOAL lc ctxt sign cl = {hyps=sign;concl=cl;ctxt=ctxt;lc=lc};;

let mt_ctxtty = {pgm=NONE; mimick = NONE};;

let get_pgm ctxt = ctxt.pgm;;
let set_pgm pgm ctxt = {pgm = pgm; mimick = ctxt.mimick};;

let get_mimick ctxt = ctxt.mimick;;
let set_mimick mimick ctxt = {mimick = mimick; pgm = ctxt.pgm};;

let ref_of_proof pf =
    match pf.ref with
    NONE -> failwith"rule_of_proof"
  | SOME r -> r
;;

let rule_of_proof pf =
let (r,_) = ref_of_proof pf in r
;;

let children_of_proof pf = 
let (_,cl) = ref_of_proof pf in cl;;

let goal_of_proof pf = pf.goal;;
let subproof_of_proof pf =
    match pf.subproof with
    NONE -> failwith "subproof_of_proof"
  | SOME pf -> pf
;;

let status_of_proof pf = pf.status;;

let is_complete_proof pf = pf.status = COMPLETE_PROOF;;

let is_leaf_proof pf =
    match pf.ref with
    NONE -> true
  | SOME _ -> false
;;

let is_tactic_proof pf =
    match pf.subproof with
    SOME _ -> true
  | NONE -> false
;;

(* $Id: proof_trees.ml,v 1.2 1996/02/12 15:48:12 cpaulin Exp $ *)
