(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              reduction.mli                               *)
(****************************************************************************)

#open "std";;
#open "impuniv";;
#open "names";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "term";;
#open "proof";;

exception Redelimination;;
exception Induc;;
exception Elimconst;;

(* Term Manipulation Functions *)

value lambda_implicit : constr -> constr;;
value lambda_implicit_lift : int -> constr -> constr;;
value prod_name : name * constr * constr -> constr;;
value hdchar : constr -> string;;
value named_hd : constr -> name -> name;;
value arrow : constr -> constr -> constr;;
value lambda_name : name * constr * constr -> constr;;
value lambda_create : constr * constr -> constr;;
value prod_create : constr * constr -> constr;;


(* Reduction Functions *)

type reduction_function == constr -> constr;;
type stack_reduction_function == constr -> constr list -> constr * constr list;;

value strong : reduction_function -> reduction_function;;
value under_casts : reduction_function -> reduction_function;;

value whd_meta : (int * constr) list -> reduction_function;;
value whd_beta_stack : stack_reduction_function;;
value whd_beta : reduction_function;;
value whd_const_stack : section_path list -> readable_constraints -> stack_reduction_function;;
value whd_const : section_path list -> readable_constraints -> reduction_function;;
value whd_delta_stack : readable_constraints -> stack_reduction_function;;
value whd_delta : readable_constraints -> reduction_function;;
value whd_betaiota_stack : stack_reduction_function;;
value whd_betaiota : reduction_function;;
value whd_betadelta_stack : readable_constraints -> stack_reduction_function;;
value whd_betadelta : readable_constraints -> reduction_function;;
value whd_betadeltat_stack : readable_constraints -> stack_reduction_function;;
value whd_betadeltat : readable_constraints -> reduction_function;;
value whd_betadeltaiota_stack : readable_constraints -> stack_reduction_function;;
value whd_betadeltaiota : readable_constraints -> reduction_function;;
value whd_betadeltatiota_stack : readable_constraints -> stack_reduction_function;;
value whd_betadeltatiota : readable_constraints -> reduction_function;;
value whd_betadeltaiotaeta_stack : readable_constraints -> stack_reduction_function;;
value whd_betadeltaiotaeta : readable_constraints -> reduction_function;;

value beta_applist : (constr * constr list) -> constr;;

value hnf_prod_app : readable_constraints -> string -> constr -> constr -> constr;;
value hnf_prod_appvect : readable_constraints -> string -> constr -> constr vect -> constr;;
value hnf_prod_applist : readable_constraints -> string -> constr -> constr list -> constr;;
value hnf_lam_app : readable_constraints -> string -> constr -> constr -> constr;;
value hnf_lam_appvect : readable_constraints -> string -> constr -> constr vect -> constr;;
value hnf_lam_applist : readable_constraints -> string -> constr -> constr list -> constr;;
value splay_prod : readable_constraints -> constr -> (name * constr) list * constr;;
value decomp_prod : readable_constraints -> constr -> int * constr;;
value decomp_n_prod : readable_constraints -> int -> constr -> ((name * constr) list) * constr;;

(* Obsolete Reduction Functions *)

value hnf : readable_constraints -> constr -> constr * constr list;;
value apprec : readable_constraints -> constr list -> constr -> constr * constr list;;
value redapplist : constr list -> constr -> constr;;
value red_product : readable_constraints -> constr -> constr;;
value find_mrectype : readable_constraints -> constr -> constr * constr list;;
value find_minductype : readable_constraints -> constr -> constr * constr list;;
value find_mcoinductype : readable_constraints -> constr -> constr * constr list;;
value check_mrectype_spec : readable_constraints -> constr -> constr;;
value minductype_spec : readable_constraints -> constr -> constr;;
value mrectype_spec : readable_constraints -> constr -> constr;;
value find_Inductype : readable_constraints -> constr -> constr * constr list;;

(* Conversion Functions *)

value sort_eq : sorts -> sorts -> bool;;
value sort_leq : sorts -> sorts -> bool;;
value conv : readable_constraints -> constr -> constr -> bool;;
value conv_leq : readable_constraints -> constr -> constr -> bool;;
value conv_x : readable_constraints -> constr -> constr -> bool;;

(* Special-Purpose Reduction Functions *)

value instance : (int * constr) list -> constr -> constr;;
value red_elim_const : readable_constraints -> constr -> constr list -> constr * constr list;;
value hnf_constr : readable_constraints -> constr -> constr;;
value nf : readable_constraints -> constr -> constr;;
value unfoldn : readable_constraints -> int list -> section_path -> constr -> constr;;
value one_step_reduce : readable_constraints -> constr -> constr;;
value reduce : readable_constraints -> constr -> constr;;
value compute : readable_constraints -> reduction_function;;
value is_info_sort : constr -> bool;;

(* $Id: reduction.mli,v 1.5 1996/02/12 15:48:21 cpaulin Exp $ *)
