(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               refiner.mli                                *)
(****************************************************************************)

#open "initial";;
#open "names";;
#open "more_util";;
#open "term";;
#open "mach";;
#open "pp";;
#open "stdpp";;
#open "proof";;
#open "proof_trees";;
#open "proof_constraints";;

type 'a sigma = {it : 'a ; sigma : global_constraints};;

value sig_it : 'a sigma -> 'a;;
value sig_sig : 'a sigma -> global_constraints;;

value Project : goal sigma -> readable_constraints;;

type validation == (proof list -> proof);;
type tactic == goal sigma -> (goal list sigma * validation);;
type transformation_tactic == proof -> (goal list * validation);;

value add_tactic : string -> ((tactic_arg list -> tactic)
                              * (readable_constraints -> goal -> tactic_expression -> std_ppcmds)) -> unit;;

value overwriting_add_tactic : string -> ((tactic_arg list -> tactic)
                              * (readable_constraints -> goal -> tactic_expression -> std_ppcmds)) -> unit;;

value lookup_tactic : string -> tactic_arg list -> tactic;;

value register_tactic : string ->
                       (tactic_arg list -> tactic) ->
                       (readable_constraints -> goal -> tactic_expression -> std_ppcmds) ->
                       (tactic_arg list -> tactic);;

value overwriting_register_tactic : string ->
                       (tactic_arg list -> tactic) ->
                       (readable_constraints -> goal -> tactic_expression -> std_ppcmds) ->
                       (tactic_arg list -> tactic);;

value pr_tactic : readable_constraints -> goal -> tactic_expression -> std_ppcmds;;
value refiner : rule -> tactic;;

value pf_status : proof -> pf_status;;
value is_complete : proof -> bool;;
value and_status : pf_status list -> pf_status;;

value list_pf : proof -> goal list;;

value IDTAC : tactic;;
value ORELSE : tactic -> tactic -> tactic;;
value THEN : tactic -> tactic -> tactic;;
value THEN_i : tactic -> (int -> tactic) -> int -> tactic;;
(*value THENF : tactic -> tactic -> tactic;;*)
value THENL : tactic -> tactic -> tactic;;
value THENS : tactic -> tactic list -> tactic;;
value REPEAT : tactic -> tactic;;
value FIRST : tactic list -> tactic;;
value TRY : tactic -> tactic;;
value COMPLETE : tactic -> tactic;;
value AT_LEAST_ONE : tactic -> tactic;;
value FAILTAC : tactic;;
value DO : int -> tactic -> tactic;;
value PROGRESS : tactic -> tactic;;

value refiner : rule -> tactic;;
value Context : ctxt -> tactic;;
value Tactic : tactic_expression -> tactic;;
value frontier : transformation_tactic;;

value list_pf : proof -> goal list;;
value unTAC : tactic -> goal sigma -> proof sigma;;

type pfstate;;
value proof_of_pfstate : pfstate -> proof;;
value evc_of_pfstate : pfstate -> global_constraints;;
value cursor_of_pfstate : pfstate -> int list;;

value solve_subgoal : int -> tactic -> proof sigma -> proof sigma;;
value solve_pf : int -> tactic -> pfstate -> pfstate;;
value mk_pfstate : goal -> pfstate;;
value extract_pfstate : pfstate -> constr;;

type pftreestate;;
value proof_of_pftreestate : pftreestate -> proof;;
value cursor_of_pftreestate : pftreestate -> int list;;
value evc_of_pftreestate : pftreestate -> global_constraints;;

value traverse : int -> pftreestate -> pftreestate;;
value solve_nth_pftreestate : int -> tactic -> pftreestate -> pftreestate;;
value solve_pftreestate : tactic -> pftreestate -> pftreestate;;
value mk_pftreestate : goal -> pftreestate;;
value extract_pftreestate : pftreestate -> constr;;
value extract_open_proof : constr signature -> proof -> constr * (int * constr) list;;
value first_unproven : pftreestate -> pftreestate;;
value last_unproven : pftreestate -> pftreestate;;
value node_prev_unproven : int -> pftreestate -> pftreestate;;
value node_next_unproven : int -> pftreestate -> pftreestate;;
value next_unproven : pftreestate -> pftreestate;;
value prev_unproven : pftreestate -> pftreestate;;
value top_of_tree : pftreestate -> pftreestate;;

(* $Id: refiner.mli,v 1.2 1996/02/12 15:48:26 cpaulin Exp $ *)
