(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                termast.ml                                *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "lomsg";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "environ";;
#open "termenv";;
#open "impuniv";;
#open "ast";;

let contents_of_id id =
    match string_of_id id with
    "Pos" -> Pos
  | "Null" -> Null
  | _ -> error "Malformed kind-constructor"
;;

let id_of_contents c =
    match c with
    Pos -> id_of_string "Pos"
  | Null -> id_of_string "Null"
;;

let rec iterated_binder oper ty = function
    DLAM(na,body) ->
    DOP2(oper,ty,DLAM(na,iterated_binder oper (lift 1 ty) body))
  | body -> body
;;

exception NoFactor;;

let rec factorize_binder oper ty = function
    (DOP2(oper',ty',DLAM(Name _ as na',c))) -> let ty1 = lift 1 ty in
    if oper=oper' & eq_constr(ty1,ty') & dependent (Rel 1) c
    then let c' = try factorize_binder oper ty1 c with NoFactor -> c 
    in DLAM(na',c') else raise NoFactor
  | body -> raise NoFactor
;;

let indexdlam iddef = indrec 0 where rec indrec i = 
    function DLAM(Name(id),C) -> if id=iddef then i else indrec (i+1) C
           | DLAMV(Name(id),C) -> if id=iddef then i else 
        errorlabstrm "indexdlam" 
       [< print_id id; 'S "is not declared in the recursive specification" >];;


let dbize_op opn =
    match opn with
    "META" -> (fun [TOKnum n] [] -> DOP0(Meta n)
                 | _ _ -> error "dbize_op")
  | "XTRA" -> (fun ((TOKstr s)::pl) ->
                   (fun [] -> DOP0(XTRA(s,pl))
                      | [c] -> DOP1(XTRA(s,pl),c)
                      | [c1;c2] -> DOP2(XTRA(s,pl),c1,c2)
                      | cl -> DOPN(XTRA(s,pl),vect_of_list cl))
                 | _ -> error "dbize_op")

  | "PROP" -> (fun [TOKid c] [] -> DOP0(Sort(Prop (contents_of_id c)))
                 | _ _ -> error "dbize_op")
  | "TYPE" -> (fun [TOKid c] [] -> DOP0(Sort(Type(contents_of_id c,Dummy_univ)))
                 | _ _ -> error "dbize_op")
  | "IMPLICIT" -> (fun [] [] -> DOP0(Implicit)
                 | _ _ -> error "dbize_op")

  | "CAST" -> (fun [] [c1;c2] -> DOP2(Cast,c1,c2)
                 | _ _ -> error "dbize_op")
  | "PROD" -> (fun [] [c1;(DLAM _ as c2)] -> DOP2(Prod,c1,c2)
                 | _ _ -> error "dbize_op")
  | "LAMBDA" -> (fun [] [c1;(DLAM _ as c2)] -> DOP2(Lambda,c1,c2)
                 | _ _ -> error "dbize_op")
  | "PRODLIST" -> (fun [] [c1;(DLAM _ as c2)] -> iterated_binder Prod c1 c2
                     | _ _ -> error "dbize_op")
  | "LAMBDALIST" -> (fun [] [c1;(DLAM _ as c2)] -> iterated_binder Lambda c1 c2
                     | _ _ -> error "dbize_op")
  | "APPLIST" -> (fun [] tl -> DOPN(AppL,vect_of_list tl)
                 | _ _ -> error "dbize_op")
  | "CONST" -> (fun [TOKpath sp] tl -> DOPN(Const sp,vect_of_list tl)
                 | _ _ -> error "dbize_op")
  | "ABST" -> (fun [TOKpath sp] tl -> global_abst sp (vect_of_list tl)
                 | _ _ -> error "dbize_op")
  | "REL" -> error "dbize_op called on a name-term containing a REL"

  | "MUTIND" -> (fun [TOKpath sp;TOKnum tyi] cl ->
                     DOPN(MutInd (sp,tyi),vect_of_list cl)
                | _ _ -> error "dbize_op")

  | "MUTCASE" -> (fun [] tl -> DOPN(MutCase,vect_of_list tl)
                 | _ _ -> error "dbize_op")

  | "MUTCONSTRUCT" -> (fun [TOKpath sp;TOKnum tyi;TOKnum n] cl ->
                           DOPN(MutConstruct ((sp,tyi),n),vect_of_list cl)
                 | _ _ -> error "dbize_op")

  | _ -> error "dbize_op"
;;

let unitize_env env = map_rel_env (fun _ -> ()) env;;

let split_params = sprec []
    where rec sprec acc = function
    (ASTprim p)::l -> sprec (p::acc) l
  | l -> (rev acc,l)
;;

let dbize = dbrec
    where rec dbrec env = function
    ASTnvar id ->
    (try (match lookup_id id env with
          RELNAME(n,_) -> Rel n
        | _ -> VAR id)
     with Not_found -> VAR id)

  | ASTslam(na,ASTnode("V$",l)) ->
    let na = (match na with NONE -> Anonymous | SOME id -> Name id)
    in DLAMV(na,vect_of_list (map (dbrec (add_rel (na,()) env)) l))

  | ASTslam(na,t) ->
    let na = (match na with NONE -> Anonymous | SOME id -> Name id)
    in DLAM(na,dbrec (add_rel (na,()) env) t)

  | ASTnode("FIX",astl) -> let ([TOKid iddef],ldecl) = split_params astl in 
     let (rlf,lf,ln,lA,lt) = decrec [] [] [] [] ldecl where rec decrec lf ln lA lt = 
         (function [] -> (lf,rev lf, rev ln, rev lA, rev lt)
             | ASTnode("FDECL",fl)::rest -> 
            let ([TOKid fi;TOKnum ni],[astA; astT]) = split_params fl in
            decrec (fi::lf) (ni-1::ln) ((dbrec env astA)::lA) (astT::lt) rest)
     in let n = (index iddef lf) -1 
     in let new_env = it_list (fun env fid -> add_rel (Name fid,()) env) env lf 
     in let defs = vect_of_list (map (dbrec new_env) lt)
     in let c = it_list (fun c fi -> DLAM(Name fi,c)) (DLAMV(Name (hd rlf),defs)) 
                        (tl rlf)
     in DOPN(Fix(vect_of_list ln,n),concat_vect (vect_of_list lA) [|c|])

  | ASTnode("COFIX",astl) -> let ([TOKid iddef],ldecl) = split_params astl in 
     let (rlf,lf,lA,lt) = decrec [] [] [] ldecl where rec decrec lf lA lt = 
         (function [] -> (lf,rev lf, rev lA, rev lt)
             | ASTnode("CFDECL",fl)::rest -> 
            let ([TOKid fi],[astA; astT]) = split_params fl in
            decrec (fi::lf) ((dbrec env astA)::lA) (astT::lt) rest)
     in let n = (index iddef lf) -1 
     in let new_env = it_list (fun env fid -> add_rel (Name fid,()) env) env lf 
     in let defs = vect_of_list (map (dbrec new_env) lt)
     in let c = it_list (fun c fi -> DLAM(Name fi,c)) (DLAMV(Name (hd rlf),defs)) 
                        (tl rlf)
     in DOPN(CoFix(n),concat_vect (vect_of_list lA) [|c|])

  | ASTnode(opn,tl) ->
    let (pl,tl) = split_params tl in
        dbize_op opn pl (map (dbrec env) tl)
;;

let bdize_op = function
    (Meta n) -> ("META",[TOKnum n])
  | (XTRA(s,pl)) -> ("XTRA",(TOKstr s)::pl)
  | (Sort(Prop c)) -> ("PROP",[TOKid (id_of_contents c)])
  | (Sort(Type(c,u))) -> ("TYPE",[TOKid (id_of_contents c);TOKpath u.u_sp;TOKnum u.u_num])
  | Implicit -> ("IMPLICIT",[])
  | Cast -> ("CAST",[])
  | Prod -> ("PROD",[])
  | Lambda -> ("LAMBDA",[])
  | AppL -> ("APPLIST",[])
  | (Const sp) -> ("CONST",[TOKpath sp])
  | (Abst sp) -> ("ABST",[TOKpath sp])
  | (MutConstruct ((sp,tyi),n)) -> ("MUTCONSTRUCT",[TOKpath sp;TOKnum tyi;TOKnum n])
  | MutInd (sp,tyi) -> ("MUTIND",[TOKpath sp;TOKnum tyi])
  | MutCase -> ("MUTCASE",[])
  | _ -> anomaly "bdize_op" (* Fix and CoFix are transformed globally in bdize *)
;;

let decomp_DLAM_new_names = findrec [] where rec findrec lna avoid = function 
   DLAMV(na,vt) -> let id = next_name_away na avoid in (rev (id::lna),vt) 
 | DLAM(na,t) -> let id = next_name_away na avoid in findrec (id::lna) (id::avoid) t
 | _          -> anomaly "decomp_DLAM_new_names";;


let bdize env t =
let fv_t = global_vars t
and fv_env =
    map_succeed (fun (Name id,_) -> id | _ -> failwith "caught") (get_rels env)
in bdrec (fv_env@fv_t) env t
    where rec bdrec avoid env t = 
    match collapse_appl t with
    VAR id -> ASTnvar id
  | Rel n ->
    (try (match fst(lookup_rel n env) with
          Name id -> ASTnvar id
        | Anonymous -> (warning "bdize was fed an incomplete environment";
                        raise Not_found))
     with Not_found -> OP(("REL",[TOKnum n]),[]))
(* TODO: Change this to subtract the current depth *)
  | DOP0 op -> OP(bdize_op op,[])
  | DOP1(op,c) -> OP(bdize_op op,[bdrec avoid env c])
  | DOP2(op,c1,c2) -> 
      (* try first to factorize types in binders *)
      (try match c2 with 
           (DLAM(Name _ as na,c)) -> if dependent (Rel 1) c then
           let oplist = match op with Prod -> ("PRODLIST",[])
                                     | Lambda -> ("LAMBDALIST",[])
                                     | _ -> raise NoFactor
           in let c' = factorize_binder op c1 c in 
           OP(oplist,[bdrec avoid env c1;bdrec avoid env (DLAM(na,c'))])
           else raise NoFactor
          | _ -> raise NoFactor
      with NoFactor -> OP(bdize_op op,[bdrec avoid env c1;bdrec avoid env c2]))
  | DOPN(Fix(nv,n),cl) -> 
          let (lfi,vt) = decomp_DLAM_new_names avoid (last_vect cl) in 
          let def_env = it_list (fun env id -> add_rel (Name id,()) env) env lfi in
          let def_avoid = lfi@avoid in
          let f = nth lfi (n+1) in 
          let listdecl = 
          map_i (fun i fi -> OP(("FDECL",[TOKid fi;TOKnum (nv.(i)+1)]),
                      [bdrec avoid env cl.(i); bdrec def_avoid def_env vt.(i)]))
                0 lfi 
          in OP(("FIX",[TOKid f]),listdecl)
  | DOPN(CoFix(n),cl) -> 
          let (lfi,vt) = decomp_DLAM_new_names avoid (last_vect cl) in 
          let def_env = it_list (fun env id-> add_rel (Name id,()) env) env lfi in
          let def_avoid = lfi@avoid in
          let f = nth lfi (n+1) in 
          let listdecl = 
          map_i (fun i fi -> OP(("CFDECL",[TOKid fi]),
                      [bdrec avoid env cl.(i); bdrec def_avoid def_env vt.(i)]))
                0 lfi
      in OP(("COFIX",[TOKid f]),listdecl) 
  | DOPN(op,cl) -> OP(bdize_op op,map_vect_list (bdrec avoid env) cl)
  | DOPL(op,cl) -> OP(bdize_op op,map (bdrec avoid env) cl)
  | DLAM(na,c) ->
    if not(dependent (Rel 1) c) then
        ASTslam(NONE,bdrec avoid env (pop c))
    else
    let id = next_name_away na avoid
    in ASTslam(SOME id,bdrec (id::avoid) (add_rel (Name id,()) env) c)

  | DLAMV(na,cl) ->
    if not(exists_vect (dependent (Rel 1)) cl) then
        ASTslam(NONE,OP(("V$",[]),map_vect_list (fun c -> bdrec avoid env (pop c)) cl))
    else
    let id = next_name_away na avoid
    in ASTslam(SOME id,
            OP(("V$",[]),map_vect_list (bdrec (id::avoid) (add_rel (Name id,()) env)) cl))
;;

let bdize_no_casts env t =
let fv_t = global_vars t
and fv_env =
    map_succeed (fun (Name id,_) -> id | _ -> failwith "caught") (get_rels env)
in bdrec (fv_env@fv_t) env t
    where rec bdrec avoid env t =
    match collapse_appl t with
    VAR id -> ASTnvar id
  | Rel n ->
    (try (match fst(lookup_rel n env) with
          Name id -> ASTnvar id
        | Anonymous -> (warning "bdize was fed an incomplete environment";
                        raise Not_found))
     with Not_found -> OP(("REL",[TOKnum n]),[]))
(* TODO: Fix this so that the number is offset by the depth *)

  | DOP0 op -> OP(bdize_op op,[])
  | DOP1(op,c) -> OP(bdize_op op,[bdrec avoid env c])
  | DOP2(Cast,c,_) -> bdrec avoid env c
  | DOP2(op,c1,c2) -> 
      (* try first to factorize types in binders *)
      (try match c2 with 
           (DLAM(Name _ as na,c)) -> if dependent (Rel 1) c then
           let oplist = match op with Prod -> ("PRODLIST",[])
                                     | Lambda -> ("LAMBDALIST",[])
                                     | _ -> raise NoFactor
           in let c' = factorize_binder op c1 c in 
           OP(oplist,[bdrec avoid env c1;bdrec avoid env (DLAM(na,c'))])
           else raise NoFactor
          | _ -> raise NoFactor
      with NoFactor -> OP(bdize_op op,[bdrec avoid env c1;bdrec avoid env c2]))
    | DOPN(Fix(nv,n),cl) -> 
          let (lfi,vt) = decomp_DLAM_new_names avoid (last_vect cl) in 
          let def_env = it_list (fun env id -> add_rel (Name id,()) env) env lfi in
          let def_avoid = lfi@avoid in
          let f = nth lfi (n+1) in 
          let listdecl = 
          map_i (fun i fi -> OP(("FDECL",[TOKid fi;TOKnum (nv.(i)+1)]),
                      [bdrec avoid env cl.(i); bdrec def_avoid def_env vt.(i)]))
                0 lfi 
          in OP(("FIX",[TOKid f]),listdecl)
  | DOPN(CoFix(n),cl) -> 
          let (lfi,vt) = decomp_DLAM_new_names avoid (last_vect cl) in 
          let def_env = it_list (fun env id-> add_rel (Name id,()) env) env lfi in
          let def_avoid = lfi@avoid in
          let f = nth lfi (n+1) in 
          let listdecl = 
          map_i (fun i fi -> OP(("CFDECL",[TOKid fi]),
                      [bdrec avoid env cl.(i); bdrec def_avoid def_env vt.(i)]))
                0 lfi
      in OP(("COFIX",[TOKid f]),listdecl) 
  | DOPN(op,cl) -> OP(bdize_op op,map_vect_list (bdrec avoid env) cl)
  | DOPL(op,cl) -> OP(bdize_op op,map (bdrec avoid env) cl)
  | DLAM(na,c) ->
    if not(dependent (Rel 1) c) then
        ASTslam(NONE,bdrec avoid env (pop c))
    else
    let id = next_name_away na avoid
    in ASTslam(SOME id,bdrec (id::avoid) (add_rel (Name id,()) env) c)

  | DLAMV(na,cl) ->
    if not(exists_vect (dependent (Rel 1)) cl) then
        ASTslam(NONE,OP(("V$",[]),map_vect_list (fun c -> bdrec avoid env (pop c)) cl))
    else
    let id = next_name_away na avoid
    in ASTslam(SOME id,
            OP(("V$",[]),map_vect_list (bdrec (id::avoid) (add_rel (Name id,()) env)) cl))
;;


(* $Id: termast.ml,v 1.8 1996/02/12 15:48:28 cpaulin Exp $ *)
