(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               termenv.mli                                *)
(****************************************************************************)
#open "more_util";;
#open "names";;
#open "term";;
#open "constrtypes";;
#open "proof";;

(* Variables *)

value initial_sign : unit -> constr signature;;
value initial_fsign : unit -> constr signature;;
value initial_assumptions : unit -> constr signature * constr signature;;

value auto_save_variables : unit -> identifier list;;

value global_operator : section_path -> identifier -> sorts oper * constr signature;;
value id_of_global : sorts oper -> identifier;;
value const_value : readable_constraints -> constr -> constr;;
value is_existential : constr -> bool;;
value is_existential_oper : sorts oper -> bool;;
value evaluable_const : readable_constraints -> constr -> bool;;
value cookable_constant : section_path -> bool;;
value opaque_const : readable_constraints -> constr -> bool;;
value translucent_const : readable_constraints -> constr -> bool;;
value defined_const : readable_constraints -> constr -> bool;;
value const_type : readable_constraints -> constr -> constr;;
value informative_construction : constr -> bool;;
value informative_reference : constr -> bool;;
value informative_constant : constr -> bool;;

type mind_specif = 
{sp:section_path;
 mib:mutual_inductive_body;
 tyi:int;
 args:constr vect;
 mip:mutual_inductive_packet};;

value mind_specif_of_mind : constr -> mind_specif;;

value mind_path : constr -> section_path;;
value mind_arity : constr -> constr;;
value mind_lamexp : constr -> constr option;;
value mind_recargs : constr -> recarg list vect vect;;
value mind_nparams : constr -> int;;

value mis_arity : mind_specif -> constr;;
value mis_lc : mind_specif -> constr;;
value mis_lc_arity : mind_specif -> constr * constr;;
value mis_kd : mind_specif -> sorts list;;
value mis_kn : mind_specif -> sorts list;;
value mis_lamexp : mind_specif -> constr option;;
value mis_nparams : mind_specif -> int;;
value mis_ntypes : mind_specif -> int;;
value mis_consnames : mind_specif -> identifier vect;;
value mis_nconstr : mind_specif -> int;;
value mis_singl : mind_specif -> constr option;;
value mis_typename : mind_specif  -> identifier;;
value mis_recargs : mind_specif -> (recarg list) vect vect;;
value mis_recarg : mind_specif -> (recarg list) vect;;

(* Abstractions *)
value abst_value : constr -> constr;;
value global_abst : section_path -> constr vect -> constr;;
value search_abst : path_kind -> identifier -> constr vect -> constr;;
value evaluable_abst : constr -> bool;;
value translucent_abst : constr -> bool;;




(* $Id: termenv.mli,v 1.4 1996/02/12 15:48:34 cpaulin Exp $ *)
