(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                Command.v                                 *)
(****************************************************************************)
Grammar command ident := [ prim:ident($id) ] -> [$0 = $id].
Grammar command raw_command := [ prim:raw_command($c) ] -> [$0 = $c].

Grammar vernac vernac :=.
Grammar command command :=.
Grammar command lcommand :=.
Grammar command ne_command_list :=.

Grammar command
ne_command_list :=
  [ command($c1) ne_command_list($cl) ] -> [$0 = ($CONS $c1 $cl)]
| [ command($c1) ] -> [$0 = ($CONS $c1 ($LIST))]
.

Grammar command command0 :=.
Grammar command command1 := [ command0($c) ] -> [$0 = $c].
Grammar command command2 := [ command1($c) ] -> [$0 = $c].
Grammar command command3 := [ command2($c1) ] -> [$0 = $c1].

Grammar command lassoc_command4_tail[$c1] := [ ] -> [$0 = $c1].
Grammar command lassoc_command4 :=
  [ command3($c1) lassoc_command4_tail[[$0 = $c1]]($r) ] -> [$0 = $r]
.

Grammar command command5 := [ lassoc_command4($c1) ] -> [$0 = $c1].
Grammar command command6 := [ command5($c1) ] -> [$0 = $c1].
Grammar command command7 := [ command6($c1) ] -> [$0 = $c1].
Grammar command command8 := [ command7($c1) ] -> [$0 = $c1].
Grammar command command9 := [ command8($c1) ] -> [$0 = $c1].
Grammar command command10 := [ command9($f) ] -> [$0 = $f].

Grammar command command := [ command8($c) ] -> [$0 = $c].
Grammar command lcommand := [ command10($c) ] -> [$0 = $c].

Grammar command binder :=
  [ ident($id) "," binder($b) ] ->
  case $b of
  ($OPER {IDBINDER} $L) -> [$0 = ($OPER {IDBINDER} ($CONS (SOME $id) $L))]
  esac
| [ ident($id) ] -> [$0 = ($OPER {IDBINDER} ($LIST (SOME $id)))]
.

Grammar command command0 :=
  [ "?" ] -> [$0 = (XTRA {"ISEVAR"})]

| [ "?" prim:number($n) ] -> [$0 = (META {$n})]

| [ "[" ident($id1) ":" command($c) "]" command($c2) ]
  -> [$0 = (LAMBDA $c [$id1]$c2)]

| [ "[" ident($id1) "," binder($idl) ":" command($c) "]" command($c2) ]
  -> case $idl of
 	 ($OPER{IDBINDER} $L) ->
     [$0 = (LAMBDALIST $c ($SLAML ($CONS (SOME $id1) $L) $c2))]
     esac

| [ "[" ident($id1) "=" command($c) "]" command($c2) ]
  -> [$0 = (ABST {#Core#let.cci} $c [$id1]$c2)]

| [ "<<" ident($id1) ">>" command($c) ] -> [$0 = [$id1]$c]

| [ "(" lcommand($lc1) ":" command($c) ")" command($body) ]
  -> case $lc1 of ($VAR $id) -> [$0 = (PROD $c [$id]$body)] esac

| [ "(" lcommand($lc1) "," lcommand($lc2) ":" command($c) ")" command($body) ]
  -> case $lc1 of ($VAR $id1) ->
     case $lc2 of ($VAR $id2) ->
          [$0 = (PRODLIST $c ($SLAML ($LIST (SOME $id1) (SOME $id2)) $body))]
     esac esac

| [ "(" lcommand($lc1) "," lcommand($lc2) "," binder($idl) ":" command($c) ")" command($body) ]
  -> case $lc1 of ($VAR $id1) ->
     case $lc2 of ($VAR $id2) ->
     case $idl of ($OPER{IDBINDER} $L) ->
          [$0 = (PRODLIST $c ($SLAML ($CONS (SOME $id1) ($CONS (SOME $id2) $L)) $body))]
     esac esac esac

| [ "(" lcommand($lc1) ")" ]
  -> [$0 = $lc1]

| [ "(" lcommand($lc1) ")" "@" "[" ne_command_list($cl) "]" ] ->
	[$0 = ($OPER {XTRA} ($CONS {"$SOAPP"} ($CONS $lc1 $cl)))]

| [ "Prop" ] -> [$0 = (PROP {Null})]
| [ "Set" ] -> [$0 = (PROP {Pos})]
(* | [ "Data" ]  -> [$0 = (PROP {Pos})] *)
| [ "Type" ] -> [$0 = (TYPE {Null})]
| [ "Typeset" ] -> [$0 = (TYPE {Pos})]
| [ ident($v) ] -> [$0 = ($VAR $v)]
.

Grammar command command1 :=
  [ "<" ":"  "ast" ":" "<" raw_command($c) ">>" ] -> [$0 = $c]
| [ "<" lcommand($l1) ">" "Match" command($c) "with" ne_command_list($cl) "end" ]
  -> [$0 = ($OPER {XTRA} ($CONS {"REC"} ($APPEND ($LIST $l1) ($LIST $c) $cl)))]

| [ "<" lcommand($l1) ">" "Match" command($c) "with" "end" ]
  -> [$0 = ($OPER {XTRA} ($CONS {"REC"} ($APPEND ($LIST $l1) ($LIST $c))))]

| [ "<" lcommand($l1) ">" "Case" command($c) "of" ne_command_list($cl) "end" ]
  -> [$0 = ($OPER {MUTCASE} ($APPEND ($LIST $l1) ($LIST $c) $cl))]

| [ "<" lcommand($l1) ">" "Case" command($c) "of" "end" ]
  -> [$0 = ($OPER {MUTCASE} ($APPEND ($LIST $l1) ($LIST $c)))]
.

Grammar command command5 :=
  [ lassoc_command4($c1) "::" command5($c2) ] -> [$0 = (CAST $c1 $c2)].

Grammar command ne_command9_list :=
  [ command9($c1) ne_command9_list($cl) ] -> [$0 = ($CONS $c1 $cl)]
| [ command9($c1) ] -> [$0 = ($CONS $c1 ($LIST))]
.

Grammar command command10 :=
  [ command9($f) ne_command9_list($args) ] ->
	[$0 = ($OPER {APPLIST} ($APPEND ($LIST $f) $args))]
.
Grammar command command8 :=
  [ command7($c1) "->" command8($c2) ] -> [$0 = (PROD $c1 [<>]$c2)]
.

Grammar command fixbinder :=  
 [ ident($id) "/" prim:number($recarg) ":" command($type) ":=" command($def) ]
  -> [$0 = (FDECL $id $recarg $type $def)].

Grammar command fixbinders :=
  [ fixbinder($fb) "with" fixbinders($fbs) ]
  -> [$0 = ($CONS $fb $fbs)]
| [ fixbinder($fb) ]  -> [$0 = ($LIST $fb)].

Grammar command command0 :=
  [ "Fix" ident($id) "{" fixbinders($fbinders) "}"  ]
  -> [$0 = ($OPER{FIX} ($CONS $id $fbinders))]
.

Grammar command cofixbinder :=  
 [ ident($id)  ":" command($type) ":=" command($def) ]
  -> [$0 = (CFDECL $id $type $def)].

Grammar command cofixbinders :=
  [ cofixbinder($fb) "with" cofixbinders($fbs) ]
  -> [$0 = ($CONS $fb $fbs)]
| [ cofixbinder($fb) ]  -> [$0 = ($LIST $fb)].

Grammar command command0 :=
  [ "CoFix" ident($id) "{" cofixbinders($fbinders) "}"  ]
  -> [$0 = ($OPER{COFIX} ($CONS $id $fbinders))]
.



