(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               PPCommand.v                                *)
(****************************************************************************)

Syntax constr path ($PRIM $sp (SOME{PATH})) 1 <$sp:"name_type":*>.
Syntax constr ind_types (MUTIND {$sp}) 1 <$sp:"name_type":*>.
Syntax constr constants (CONST {$sp}) 1 <$sp:"name_type":*>.


Syntax constr  ne_command_listcons ($OPER{NECOMMANDLIST} ($CONS $c1 $cl)) 0
        <$c1:"Command":*> [1 0] <$TAIL:"ListCommand":*>
        with $TAIL := ($OPER{NECOMMANDLIST} $cl).
Syntax constr  ne_command_listone ($OPER{NECOMMANDLIST} ($CONS $c1 ($LIST))) 0
        <$c1:"Command":*>.


(* Things parsed in binder *)
(* ======================= *)

Syntax constr  bindercons ($OPER{IDBINDER} ($CONS (SOME $id) $L))  0
       <$ID:"Ident":*> "," <$B:"Binder":*>
        with  $ID:=($VAR $id) $B:=($OPER{IDBINDER} $L).
Syntax constr  binderone ($OPER{IDBINDER} ($LIST (SOME $id)))  0
       <$ID:"Ident":*> with  $ID:=($VAR $id)  .
Syntax constr  bindernil ($OPER{IDBINDER} ($LIST)) 0.

(* Things parsed in command2 *)

(* Things parsed in command3 *)

(* Things parsed in command4 *)

(* Things parsed in command5 *)

Syntax constr   cast <<($C :: $T)>> 5
          <$C:"term":L> [0 0] "::" [0 0] <$T:"type":E>.

(* Things parsed in command6 *)

(* Things parsed in command7 *)
(* Things parsed in command8 *)

Syntax constr   imp <<$A->$B>> 8
    [<hov 0> <$A:"Dom":L> [0 1] "->"  <$B:"Rng":E>].

(* Things parsed in command9 *)

(* Things parsed in command10 *)


Syntax constr   app ($OPER{APPLIST} ($CONS $H $T)) 10
   [<hov 0>  <$H:"Function":E>  <$P2:"Argument":E>  ] 
   with $P2:=($OPER{APPTAIL} $T).

Syntax constr   apptailcons ($OPER{APPTAIL} ($CONS $H $T)) 10
   [1 1] <$H:"Arg":L>  <$TL:"Tail":E> with $TL:=($OPER{APPTAIL} $T).
Syntax constr   apptailnil ($OPER{APPTAIL} ($LIST)) 10.


(* Things parsed in command0 *)
(* ========================= *)

Syntax constr   evar <<?>> 1 "?".

Syntax constr   meta (META ($PRIM $n)) 1 [<hov 0> "?" <$n:"Num":*>].

Syntax constr implicit (IMPLICIT) 1 "<Implicit>".

Syntax constr   lam_not_bind (LAMBDA $Dom [<>]$body) 8
   [<hov 0> "[_:" <$Dom:"Dom":E> "]" [0 1] <$body:"Body":E>].

Syntax constr   lam <<[$x:$Dom]$body>> 8
   [<hov 0> "[" <$x:"Var":*> ":" <$Dom:"Dom":E> "]" [0 1]<$body:"Body":E>].

Syntax constr lambdalist (LAMBDALIST $c ($SLAML ($CONS (SOME $id1) $L) $c2)) 8
   [<hov 0>  "[" <$ID1:"Ident":*> "," <$TAILBIND:"ListBind":*> ":"
       <$c:"Command":E> "]"  [0 1] <$c2:"Command":E> ] 
    with $TAILBIND := ($OPER{IDBINDER} $L) 
         $ID1:= ($VAR $id1).

Syntax constr   let_K (ABST{#Core#let.cci} $M [<>]$N) 1
   [<hov 0> "[" "_" "=" <$M:"M":*> "]" [0 1]<$N:"N":E>].


Syntax constr   let <<[$x=$M]$N>> 1
   [<hov 0> "[" <$x:"Var":*> "=" <$M:"M":E> "]" [0 1] <$N:"N":E>].
 

Syntax constr   abstpat [$id1]$c  1
   [<hov 0> "<<" <$id1:"Ident":*> ">>" [0 1] <$c:"Command":E> ].

Syntax constr   prod_nodep (PROD  $c [<>]$b) 8
  [<hov 0>  <$c:"Command":L> [0 1] "->"  <$b:"Rng":E>] .

Syntax constr   pi <<($x:$A)$B>> 8
	[<hov 0> "(" <$x:"Var":*> ":" <$A:"Dom":E> ")" [0 1] <$B:"Rng":E>].

Syntax constr prodlist (PRODLIST $c ($SLAML ($CONS (SOME $id1) $L) $c2)) 8
   [<hov 0>  "(" <$ID1:"Ident":*> "," <$TAILBIND:"ListBind":*> ":"
       <$c:"Command":E> ")"  [0 1] <$c2:"Command":E> ] 
    with $TAILBIND := ($OPER{IDBINDER} $L) 
         $ID1:= ($VAR $id1).

Syntax constr   paircomm 
  (APPLIST pair (XTRA{"ISEVAR"}) (XTRA{"ISEVAR"}) $lc1 $lc2) 1
  [<hov 0> "("  <$lc1:"Command":L> ","  <$lc2:"Command":L>  ")" ].
 
Syntax constr   soap ($OPER {XTRA} ($CONS {"$SOAPP"} ($CONS $lc1 $cl))) 1
  [<hov 0> "(" <lc1:"Command":*> ")" "@" "[" <$CL:"CommandList":*> "]"]
  with $CL := ($OPER{NECOMMANDLIST} $cl).

Syntax constr   prop (PROP {Null}) 1 
        "Prop".
Syntax constr   set (PROP {Pos}) 1 
        "Set".
Syntax constr   type (TYPE {Null}) 1 
        "Type".
Syntax constr   type_sp (TYPE {Null} $SP $N) 1
        "Type".
Syntax constr   type_set (TYPE {Pos}) 1 
        "Typeset".

Syntax constr   typeset_sp (TYPE {Pos} $SP $N) 1 
        "Typeset".

(* Things parsed in command1 *)
(* ========================= *)

Syntax constr   recpr ($OPER{XTRA} ($CONS {"REC"} $RECARGS)) 8
   <$RECPRINT:"Rec":*> with $RECPRINT:=($OPER{RECTERM} $RECARGS).

Syntax constr   recterm ($OPER{RECTERM} ($CONS $P ($CONS $c $BL))) 8
 [<hov 0>
    [<hov 0> "<" <$P:"Result":E> ">"
    		 [0 2]
    		 [<hov 0> "Match" [1 0] <$c:"Head":E>  [1 0] "with" ]]
	[1 3]
    [<hov 0> <$BRANCHES:"Branches":E> ]
	"end"]
 with $BRANCHES:=($OPER{RECBRANCHES} $BL).

Syntax constr   recbranchescons ($OPER{RECBRANCHES} ($CONS $B $T)) 8
    [<hov 0> [<hov 0> <$B:"Branch":E>] FNL] <$BTAIL:"Rest":E>
    with $BTAIL:=($OPER{RECBRANCHES} $T).
Syntax constr   recbranchesnil ($OPER{RECBRANCHES} ($LIST)) 8.


Syntax constr   matchbranchescons ($OPER{MATCHBRANCHES} ($CONS $B $T)) 8
    [<hov 0> [<hov 0> <$B:"Branch":E>] FNL] <$BTAIL:"Rest":E>
    with $BTAIL:=($OPER{MATCHBRANCHES} $T).
Syntax constr   matchbranchesnil ($OPER{MATCHBRANCHES} ($LIST)) 8.

Syntax constr   casepr ($OPER{MUTCASE} $MATCHARGS) 8
   <$MATCHPRINT:"Case":*> with $MATCHPRINT:=($OPER{CASETERM} $MATCHARGS).
Syntax constr   caseterm ($OPER{CASETERM} ($CONS $P ($CONS $c $BL))) 8
 [<hov 0>
    [<hov 0> "<" <$P:"Result":E> ">"
    		 [0 2]
    		 [<hov 0> "Case" [1 0] <$c:"Head":E>  [1 0] "of" ]]
	[1 3]
    [<hov 0> <$BRANCHES:"Branches":E> ]
	"end"]
 with $BRANCHES:=($OPER{MATCHBRANCHES} $BL).

Syntax constr onefixdecl (FDECL{$f}{$x} $A $t) 1 
  [<hov 0>  <$F:"Id":*>"/"<$X:"index":*>" : "<$A:"type":*>" := " 
        [0 2] <$t:"body":*>]
with $F := ($VAR $f) $X:= ($PRIM $x).

Syntax constr fix ($OPER{FIX} ($CONS $f ($CONS $def $lfs))) 1 
 [<hov 0> "Fix "<$F:"Id":*> "{" 
       [<hov 0> <$def:"decl":*> <$LFS:"Declaration list":E>] "}"]
   with $F := ($VAR $f) $LFS:=($OPER{FIXDECLS} $lfs).

Syntax constr nofixdefs (FIXDECLS) 1.

Syntax constr fixdefs ($OPER{FIXDECLS} ($CONS $def1 $defs)) 1
   FNL "with"  [1 0] <$def1:"def":*> <$DEFS:"defs":*> 
   with $DEFS:= ($OPER{FIXDECLS} $defs).

Syntax constr onecofixdecl (CFDECL{$f} $A $t) 1 
  [<hov 0>  <$F:"Id":*> " : " <$A:"type":*>" := " [0 2] <$t:"body":*>]
with $F := ($VAR $f).

Syntax constr cofix ($OPER{COFIX} ($CONS $f ($CONS $def $lfs))) 1 
 [<hov 0> "CoFix "<$F:"Id":*> "{" 
       [<hov 0> <$def:"decl":*> <$LFS:"Declaration list":E>] "}"]
   with $F := ($VAR $f) $LFS:=($OPER{FIXDECLS} $lfs).











