(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                PPFTerm.v                                 *)
(****************************************************************************)

Syntax fterm path ($PRIM $sp (SOME{PATH})) 1 <$sp:"name_type":*>.
Syntax fterm ind_types (IND {$sp}) 1 <$sp:"name_type":*>.
Syntax fterm constants (CONST {$sp}) 1 <$sp:"name_type":*>.


Syntax fterm  ne_command_listcons ($OPER{NECOMMANDLIST} ($CONS $c1 $cl)) 0
        <$c1:"Command":*> [1 0] <$TAIL:"ListCommand":*>
        with $TAIL := ($OPER{NECOMMANDLIST} $cl).
Syntax fterm  ne_command_listone ($OPER{NECOMMANDLIST} ($CONS $c1 ($LIST))) 0
        <$c1:"Command":*>.


(* Things parsed in binder *)
(* ======================= *)

Syntax fterm  bindercons ($OPER{IDBINDER} ($CONS (SOME $id) $L))  0
       <$ID:"Ident":*> "," <$B:"Binder":*>
        with  $ID:=($VAR $id) $B:=($OPER{IDBINDER} $L).
Syntax fterm  binderone ($OPER{IDBINDER} ($LIST (SOME $id)))  0
       <$ID:"Ident":*> with  $ID:=($VAR $id)  .
Syntax fterm  bindernil ($OPER{IDBINDER} ($LIST)) 0.

(* Things parsed in command2 *)

(* Things parsed in command3 *)

(* Things parsed in command4 *)

(* Things parsed in command5 *)

Syntax fterm   cast <<($C :: $T)>> 5
          <$C:"term":L> [0 0] "::" [0 0] <$T:"type":E>.

(* Things parsed in command6 *)

(* Things parsed in command7 *)

(* Things parsed in command8 *)

Syntax fterm   imp <<$A->$B>> 8
    [<hov 0> <$A:"Dom":L>[0 1] "->" <$B:"Rng":E>].

(* Things parsed in command9 *)

(* Things parsed in command10 *)

Syntax fterm   app ($OPER{APPLIST} ($CONS $H $T)) 10
   [<hov 0> <$H:"Function":E> <$P2:"Argument":E>] 
   with $P2:=($OPER{APPTAIL} $T).

Syntax fterm   apptailcons ($OPER{APPTAIL} ($CONS $H $T)) 10
   [1 1] <$H:"Arg":L>  <$TL:"Tail":E> with $TL:=($OPER{APPTAIL} $T).
Syntax fterm   apptailnil ($OPER{APPTAIL} ($LIST)) 10.



(* Things parsed in command0 *)
(* ========================= *)

Syntax fterm   evar <<?>> 1 "?".

Syntax fterm   meta (META{$PRIM $n}) 1 [<hov 0> "?" <$n:"Num":*>].


Syntax fterm   lam_K (LAMBDA $Dom [<>]$body) 8
   [<hov 0> "[_:" <$Dom:"Dom":E> "]" [0 1]<$body:"Body":L>].

Syntax fterm   lam <<[$x:$Dom]$body>> 8
   [<hov 0> "[" <$x:"Var":*> ":" <$Dom:"Dom":E> "]" [0 1] <$body:"Body":E>].

Syntax fterm   lam2 <<[$x:$Dom][$y:$Dom]$body>> 8
   [<hov 0> "["<$x:"Var":*>","<$y:"Var":*>":"<$Dom:"Dom":E>"]"
        [0 1]<$body:"Body":E>].

Syntax fterm   lambdalist (LAMBDALIST $c ($SLAML ($CONS (SOME $id1) $L) $c2)) 8
   [<hov 0>  "[" <$ID1:"Ident":*> "," <$TAILBIND:"ListBind":*> ":"
       <$c:"Command":E> "]"  [0 1] <$c2:"Command":E> ] 
    with $TAILBIND := ($OPER{IDBINDER} $L)
         $ID1:= ($VAR $id1).

Syntax fterm   let_K (ABST{#Core#let.cci} $M [<>]$N) 1
   [<hov 0> "[" "_" "=" <$M:"M":*> "]" [0 1]<$N:"N":E>].


Syntax fterm   let <<[$x=$M]$N>> 1
   [<hov 0> "[" <$x:"Var":*> "=" <$M:"M":*> "]" [0 1] <$N:"N":E>].
 
(******************
Syntax fterm   abstpat ([$id1]$c)  1
   [<hov 0> "[" <$id1:"Ident":*> "]" [0 1] <$c:"Command":E> ].

*************)

Syntax fterm   prod (PROD  $c [$id]$b) 8
   [<hov 0> "(" <$id:"Ident":*> ":" <$c:"Command":E> ")" [1 0] <$b:"Rng":E>] .

Syntax fterm   pi <<($x:$A)$B>> 8
    [<hov 0> "(" <$x:"Var":*> ":" <$A:"Dom":E> ")" [0 1]<$B:"Rng":E>].    


Syntax fterm   prodlist_binder
 (PRODLIST $c ($SLAML ($CONS (SOME $id1) ($CONS (SOME $id2) $L)) $body)) 8
    [<hov 0>   "(" <$ID1:"Ident":*> "," <$ID2:"Ident":*> "," 
      <$TAILBIND:"Binder":*> ":" <$c:"Dom":E> ")" [0 1] <$body:"Rng":E> ]
    with $TAILBIND := ($OPER{IDBINDER} $L)
         $ID1 := ($VAR $id1)
         $ID2 := ($VAR $id2).


Syntax fterm   prodlist_two 
  (PRODLIST $c ($SLAML ($LIST (SOME $id1) (SOME $id2)) $body)) 8
    [<hov 0>   "(" <$ID1:"Ident":*> "," <$ID2:"Ident":*> 
        ":" <$c:"Dom":E> ")" [0 1] <$body:"Rng":E> ]
     with $ID1 := ($VAR $id1) $ID2 :=($VAR $id2)  .


Syntax fterm   paircomm 
  (APPLIST pair (XTRA{"ISEVAR"}) (XTRA{"ISEVAR"}) $lc1 $lc2) 1
  [<hov 0> "("  <$lc1:"Command":L> ","  <$lc2:"Command":L>  ")" ].
 
Syntax fterm   soap ($OPER {XTRA} ($CONS {"$SOAPP"} ($CONS $lc1 $cl))) 1
  [<hov 0> "(" <$lc1:"Command":*> ")" "@" "[" <$CL:"CommandList":*> "]"]
  with $CL := ($OPER{NECOMMANDLIST} $cl).

Syntax fterm   Prop (PROP {Null}) 1 
        "Prop".
Syntax fterm   Set (PROP {Pos}) 1 
        "Set".
Syntax fterm   Type (TYPE {Null}) 1 
        "Type".
Syntax fterm   typesp (TYPE {Null} $SP $N) 1 
        "Type".
Syntax fterm   typeset (TYPE {Pos} $SP $N) 1 
        "Typeset".






(* Things parsed in command1 *)
(* ========================= *)

Syntax fterm   recpr ($OPER{XTRA} ($CONS{"REC"} $RECARGS)) 1
   <$RECPRINT:"Rec":*> with $RECPRINT:=($OPER{RECTERM} $RECARGS).
Syntax fterm   recterm ($OPER{RECTERM} ($CONS $P ($CONS $c $BL))) 1
 [<hov 0>
    [<hov 0> "<" <$P:"Result":E> ">"
    		 [0 2]
    		 [<hov 0> "Match" [1 0] <$c:"Head":*>  [1 0] "with" ]]
	[1 3]
    [<hov 0> <$BRANCHES:"Branches":E> ]
	"end"]
 with $BRANCHES:=($OPER{RECBRANCHES} $BL).
Syntax fterm   recbranchescons ($OPER{RECBRANCHES} ($CONS $B $T)) 1
    [<hov 0> [<hov 0> <$B:"Branch":E>] FNL] <$BTAIL:"Rest":E>
    with $BTAIL:=($OPER{RECBRANCHES} $T).
Syntax fterm   recbranchesnil ($OPER{RECBRANCHES} ($LIST)) 1.

Syntax fterm   matchpr ($OPER{MATCH} $MATCHARGS) 1
   <$MATCHPRINT:"Match":*> with $MATCHPRINT:=($OPER{MATCHTERM} $MATCHARGS).
Syntax fterm   matchterm ($OPER{MATCHTERM} ($CONS $P ($CONS $c $BL))) 1
 [<hov 0>
    [<hov 0> "<" <$P:"Result":*> ">"
    		 [0 2]
    		 [<hov 0> "Case" [1 0] <$c:"Head":*>  [1 0] "with" ]]
	[1 3]
    [<hov 0> <$BRANCHES:"Branches":E> ]
	"end"]
 with $BRANCHES:=($OPER{MATCHBRANCHES} $BL)
.
Syntax fterm   matchbranchescons ($OPER{MATCHBRANCHES} ($CONS $B $T)) 8
    [<hov 0> [<hov 0> <$B:"Branch":E>] FNL] <$BTAIL:"Rest":E>
    with $BTAIL:=($OPER{MATCHBRANCHES} $T).
Syntax fterm   matchbranchesnil ($OPER{MATCHBRANCHES} ($LIST)) 8.

Syntax fterm   casepr ($OPER{MUTCASE} $MATCHARGS) 8
   <$MATCHPRINT:"Case":*> with $MATCHPRINT:=($OPER{CASETERM} $MATCHARGS).
Syntax fterm   caseterm ($OPER{CASETERM} ($CONS $P ($CONS $c $BL))) 8
 [<hov 0>
   [<hov 0> "<" <$P:"Result":E> ">"
    		 [0 2]
    		 [<hov 0> "Case" [1 0] <$c:"Head":E>  [1 0] "of" ]]
	[1 3]
    [<hov 0> <$BRANCHES:"Branches":E> ]
	"end"]
 with $BRANCHES:=($OPER{MATCHBRANCHES} $BL).

Syntax fterm index0 (INDEX 0) 1 "1".
Syntax fterm index1 (INDEX 1) 1 "2".
Syntax fterm index2 (INDEX 2) 1 "3".
Syntax fterm index3 (INDEX 3) 1 "4".
Syntax fterm index4 (INDEX 4) 1 "5".
Syntax fterm index5 (INDEX 5) 1 "6".
Syntax fterm index6 (INDEX 6) 1 "7".
Syntax fterm index7 (INDEX 7) 1 "8".

Syntax fterm select0 ($OPER{SELECT} ($CONS 0 ($CONS $f $T))) 1 <$f:"ident":*>.
Syntax fterm select1 ($OPER{SELECT} ($CONS 1 ($CONS $f ($CONS $g $T)))) 1 <$g:"ident":*>.
Syntax fterm select2 ($OPER{SELECT} ($CONS 2 ($CONS $f ($CONS $g ($cons $h $T)))))
                       1 <$h:"ident":*>.
Syntax fterm select3 
        ($OPER{SELECT} ($CONS 3 ($CONS $f ($CONS $g ($cons $h ($CONS $i $T))))))
                       1 <$i:"ident":*>.

Syntax fterm onefixdecl (ONEFIXDECL $x $f $A $t) 1 
  [<hov 0>  <$f:"Id":*>"/"<$index:"index":*>":"<$A:"type":*>" :=" [0 1] <$t:"body":*>]
with $index:=(INDEX $x).

Syntax fterm fixdecl ($OPER{FIXDECL} ($CONS $ONE $T)) 1 
  [<hov 0> "{" <$ONE:"declaration":E> FNL] <$BTAIL:"Rest":E> 
  with $BTAIL:=($OPER{FIXDECLWHERE} $T).

Syntax fterm fixdeclwhere ($OPER{FIXDECLWHERE} ($CONS $ONE $T)) 1 
  [<hov 0> "with" [1 0] <$ONE:"declaration":E> FNL] <$BTAIL:"Rest":E> 
  with $BTAIL:=($OPER{FIXDECLWHERE} $T).
  
Syntax fterm fixdeclnil  (FIXDECLWHERE) 1  "}".
  
Syntax fterm fix1 (FIX{$i}{$x} $A [$f](V$ $t)) 1 
 [<hov 0> "Fix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(FIXDECL (ONEFIXDECL $x ($VAR $f) $A $t)) 
      $SEL:=(SELECT $i ($VAR $f)).

Syntax fterm fix2 (FIX{$i}{$x}{$y} $A $B [$f][$g](V$ $t $u)) 1 
 [<hov 0> "Fix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(FIXDECL (ONEFIXDECL $x ($VAR $f) $A $t)
                      (ONEFIXDECL $y ($VAR $g) $B $u))
      $SEL:=(SELECT $i ($VAR $f) ($VAR $g)).

Syntax fterm fix3 (FIX{$i}{$x}{$y}{$z} $A $B $C [$f][$g][$h](V$ $t $u $v)) 1 
 [<hov 0> "Fix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(FIXDECL (ONEFIXDECL $x ($VAR $f) $A $t)
                      (ONEFIXDECL $y ($VAR $g) $B $u)
                      (ONEFIXDECL $z ($VAR $h) $C $v))
      $SEL:=(SELECT $i ($VAR $f) ($VAR $g) ($VAR $h)).

Syntax fterm fix4 (FIX{$i}{$x}{$y}{$z}{$t} $A $B $C [$f][$g][$h][$j](V$ $t $u $v $w)) 1 
 [<hov 0> "Fix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(FIXDECL (ONEFIXDECL $x ($VAR $f) $A $t)
                      (ONEFIXDECL $y ($VAR $g) $B $u)
                      (ONEFIXDECL $z ($VAR $h) $C $v)
                      (ONEFIXDECL $t ($VAR $j) $D $w))
      $SEL:=(SELECT $i ($VAR $f) ($VAR $g) ($VAR $h) ($VAR $j)).







(* Rules for printing CoFix expressions *)


Syntax fterm onecofixdecl (ONECOFIXDECL $f $A $t) 1 
  [<hov 0>  <$f:"Id":*> ":"<$A:"type":*>":="<$t:"body":*>].

Syntax fterm cofixdecl ($OPER{COFIXDECL} ($CONS $ONE $T)) 1 
  [<hov 0> "{" <$ONE:"declaration":E> FNL] <$BTAIL:"Rest":E> 
  with $BTAIL:=($OPER{COFIXDECLWHERE} $T).

Syntax fterm cofixdeclwhere ($OPER{COFIXDECLWHERE} ($CONS $ONE $T)) 1 
  [<hov 0> "with" [1 0] <$ONE:"declaration":E> FNL] <$BTAIL:"Rest":E> 
  with $BTAIL:=($OPER{COFIXDECLWHERE} $T).
  
Syntax fterm cofixdeclnil  (COFIXDECLWHERE) 1  "}".
  

Syntax fterm cofix1 (COFIX{$i} $A [$f](V$ $t)) 1 
 [<hov 0> "CoFix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(FIXDECL (ONECOFIXDECL ($VAR $f) $A $t)) 
      $SEL:=(SELECT $i ($VAR $f)).

Syntax fterm cofix2 (COFIX{$i} $A $B [$f][$g](V$ $t $u)) 1 
 [<hov 0> "CoFix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(COFIXDECL (ONECOFIXDECL ($VAR $f) $A $t)
                        (ONECOFIXDECL ($VAR $g) $B $u))
      $SEL:=(SELECT $i ($VAR $f) ($VAR $g)).

Syntax fterm cofix3 (COFIX{$i}  $A $B $C [$f][$g][$h](V$ $t $u $v)) 1 
 [<hov 0> "CoFix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(COFIXDECL (ONECOFIXDECL  ($VAR $f) $A $t)
                        (ONECOFIXDECL  ($VAR $g) $B $u)
                        (ONECOFIXDECL  ($VAR $h) $C $v))
      $SEL:=(SELECT $i ($VAR $f) ($VAR $g) ($VAR $h)).

Syntax fterm cofix4 (COFIX{$i} $A $B $C $D [$f][$g][$h][$j](V$ $t $u $v $w)) 1 
 [<hov 0> "CoFix "<$SEL:"Id":*> <$DECL:"Declaration list":E>]
 with $DECL:=(COFIXDECL (ONECOFIXDECL  ($VAR $f) $A $t)
                        (ONECOFIXDECL  ($VAR $g) $B $u)
                        (ONECOFIXDECL  ($VAR $h) $C $v)
                        (ONECOFIXDECL  ($VAR $j) $D $w))
      $SEL:=(SELECT $i ($VAR $f) ($VAR $g) ($VAR $h) ($VAR $j)).







