(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                PPTactic.v                                *)
(****************************************************************************)
(* ======================================= *)
(*   PP rules for basic elements           *)
(* ======================================= *)

Syntax tactic idargnil ($OPER{IDARGLIST} ($LIST)) 0.
Syntax tactic idargcons ($OPER{IDARGLIST} ($CONS $id $L)) 0
    <$id:"id":*> " "  <$IDARGTAIL:"Tail":*>
    with $IDARGTAIL:=($OPER{IDARGLIST} $L).

Syntax tactic ne_identarg_list_nil ($OPER{NEIDENTARGLIST} ($LIST)) 0.
Syntax tactic ne_identarg_list_one 
  ($OPER{NEIDENTARGLIST} ($LIST $id))  0   
        <$id :"Ident":*>.
Syntax tactic ne_identarg_list_cons 
  ($OPER{NEIDENTARGLIST} ($CONS $id $l)) 0
        " "  <$id :"Ident":*>  <$TAIL:"Tail":*>
         with $TAIL := ($OPER{NEIDENTARGLIST} $l).

Syntax tactic ne_num_list_nil ($OPER{NENUMLIST} ($LIST)) 0.
Syntax tactic nenumlistcons ($OPER{NENUMLIST} ($CONS ($PRIM $n) $l))  0
       <$n:"int":*> " " <$INTTAIL:"Tail":*>
        with  $INTTAIL:= ($OPER{NENUMLIST} $l).
Syntax tactic ne_num_list_one ($OPER{NENUMLIST} ($LIST ($PRIM $n)))  0 
        <$n:"int":*> .

Syntax tactic bindinglistnil ($OPER{BINDINGLIST} ($LIST)) 0.
Syntax tactic bindinglistcons ($OPER{BINDINGLIST} ($CONS $b $bl)) 0
        " " <$b:"Binding":*> <$BINDTAIL:"Tail":*>
        with $BINDTAIL:= ($OPER{BINDINGLIST} $bl).

Syntax tactic bindingid (BINDING $id $c) 0
        <$id:"Ident":*> ":=" <$c:"Command":*>.
Syntax tactic bindingnum (BINDING ($PRIM $n) $c) 0
        <$n:"Numarg":*> ":=" <$c:"Command":*>.
Syntax tactic bindingcom (BINDING $c) 0 <$c:"Command":*>.

Syntax tactic patternarg ($OPER{PATTERN} ($CONS $c $OCCL)) 0
    <$OCCLIST:"Occurrences":*> <$c:"Pattern Term":*>
    with $OCCLIST:=($OPER{PATTERNOCCLIST} $OCCL).
Syntax tactic patternoccnil ($OPER{PATTERNOCCLIST} ($LIST)) 0.
Syntax tactic patternocccons ($OPER{PATTERNOCCLIST} ($CONS ($PRIM $n) $T)) 0
    <$n:"Occurrence Number":*> " " <$OCCTAIL:"Tail":*>
    with $OCCTAIL:=($OPER{PATTERNOCCLIST} $T).


Syntax tactic patternnil ($OPER{PATTERNLIST} ($LIST)) 0.
Syntax tactic patterncons ($OPER{PATTERNLIST} ($CONS $H $T)) 0
    <$H:"Pattern Occurrence":*> " " <$PATTERNTAIL:"Tail":*>
    with $PATTERNTAIL:=($OPER{PATTERNLIST} $T).


Syntax tactic comargnil ($OPER{COMARGLIST} ($LIST)) 0.
Syntax tactic comargcons ($OPER{COMARGLIST} ($CONS $c $L)) 0
    <$c:"Com":*> " " <$COMARGTAIL:"Tail":*>
    with $COMARGTAIL:=($OPER{COMARGLIST} $L).


Syntax tactic unfoldarg ($OPER{UNFOLD} ($CONS $id $OCCL)) 0
   <$OCCLIST:"Occurrences":*> <$id:"Unfold Id":*>
    with $OCCLIST:=($OPER{UNFOLDOCCLIST} $OCCL).

Syntax tactic unfold_occ_nil ($OPER{UNFOLDOCCLIST} ($LIST)) 0 .
Syntax tactic unfold_occ_cons ($OPER{UNFOLDOCCLIST} ($CONS ($PRIM $n) $T)) 0
  <$n:"Occurrence Number":*> " " <$OCCTAIL:"Tail":*>
    with $OCCTAIL:=($OPER{UNFOLDOCCLIST} $T).

Syntax tactic unfoldnil ($OPER{UNFOLDLIST} ($LIST)) 0.
Syntax tactic unfoldcons ($OPER{UNFOLDLIST} ($CONS $H $T)) 0
   <$H:"Unfold Occurrence":*> " " <$UNFOLDTAIL:"Tail":*>
    with $UNFOLDTAIL:=($OPER{UNFOLDLIST} $T).

Syntax tactic  tactic_com_list_tail_cons 
        ($OPER{TACTICCOMLISTTAIL} ($CONS  $t1 $l)) 0
         [<hov 0> <$t1:"TacticCom":*>] ";" [1 0] <$TAIL:"ListTacCom":*> 
          with $TAIL :=  ($OPER{TACTICCOMLISTTAIL} $l).

Syntax tactic  tactic_com_list_tail_single 
     ($OPER{TACTICCOMLISTTAIL} ($LIST $t1))  0  
         [<hov 0> <$t1:"TacticCom":*>].
  
  

Syntax tactic tactic_com_seq ($OPER{TACLIST} $l) 0
     [<hv 0> "[ " <$L:"TacSeqBody":*> " ]" ] 
     with $L := ($OPER{TACTICCOMSEQBODY} $l).
  
  
Syntax tactic tactic_com_seq_body_cons ($OPER{TACTICCOMSEQBODY} ($CONS $t $l))0
         [<hov 0> <$t:"TacticComList":*>] [1 0] "| " <$SEQ:"TacComSeq":*>
          with $SEQ := ($OPER{TACTICCOMSEQBODY} $l).
Syntax tactic tactic_com_seq_body_single ($OPER{TACTICCOMSEQBODY} ($LIST $t)) 0
         [<hov 0> <$t:"TacticComList":*>].

(* ========================================== *)
(*       PP rules for simple tactics          *)
(* ========================================== *)

Syntax tactic split ($OPER{Split} $bl) 0
    "Split" <$COMARG:"BindingList":*>
    with $COMARG:=($OPER{BINDINGLIST} $bl)
.

Syntax tactic auto (Auto) 0   
     "Auto"
.
Syntax tactic auton (Auto ($PRIM $n)) 0
     "Auto " <$n:"Int":*>
.

Syntax tactic hnf (Hnf) 0
        "Hnf"
.
Syntax tactic hnfhyp (HnfHyp $id) 0
        "Hnf in " <$id:"NameHyp":*>
.

Syntax tactic assumption (Assumption) 0
    "Assumption"
.


Syntax tactic tacsemilist ($OPER{TACTICLIST} $L) 100
   [<hov 0> <$TACCOMLIST:"Tactics":E> ]
   with $TACCOMLIST:=($OPER{TACTICCOMLISTTAIL} $L).
  
Syntax tactic tacseminolist  (TACTICLIST $y) 1
        [<hov 0> <$y:"TacticCom":E> ].

Syntax tactic tacseminil ($OPER{TACSEMILIST} ($LIST)) 0  .

Syntax tactic tacsemicons ($OPER{TACSEMILIST} ($CONS $H $T)) 100
   [<hov 0> <$H:"Tactic":L>] ";" [0 1]  <$TACLTAIL:"Tactic":E>
      with $TACLTAIL:=($OPER{TACSEMILIST} $T).

Syntax tactic tacsemisingle ($OPER{TACSEMILIST} ($LIST $T)) 100
     [<hov 0> <$T:"Tactic":E>].
  

Syntax tactic unfold ($OPER{Unfold} $L) 0
    "Unfold " <$UNFOLDL:"Unfoldargs":*>
    with $UNFOLDL:=($OPER{UNFOLDLIST} $L).


Syntax tactic unfoldhyp ($OPER{UnfoldHyp} ($CONS $t $ul)) 0
   "Unfold " <$UL:"Unfoldocclist":*> [1 0] "in" [1 0] <$t:"Ident":*>
    with $UL:=($OPER{UNFOLDLIST} $ul).

Syntax tactic intro (Intro) 0
     "Intro".

Syntax tactic introswith1 (IntrosWith $id) 0 
     "Intro " <$id:"identifier":*>.

Syntax tactic intros (Intros) 0
    "Intros".

Syntax tactic introsuntil (IntrosUntil $id)  0
     "Intros Until " <$id:"identifier":*>.

Syntax tactic introswith ($OPER{IntrosWith} $L) 0
    "Intros" <$IDARGLIST:"identifiers":*>
    with $IDARGLIST:=($OPER{NEIDENTARGLIST} $L).

Syntax tactic contradiction (Contradiction) 0 
        "Contradiction". 

Syntax tactic oldelim (Elim1 $C) 0
    "OldElim " <$C:"Command":*>.

Syntax tactic elimwithbindings ($OPER{ElimWithBindings}  ($CONS $c $bl)) 0 
     "Elim " <$c:"Command":*>  <$BL:"List":*> 
     with $BL := ($OPER{WITHARGS} $bl).

Syntax tactic elimwithusing_with
    ($OPER{ElimWithUsing} ($CONS $c ($CONS (BINDING $b) $bl)))  0
    "Elim " <$c:"Command":*>  " with " <$BL:"List":*> 
     with $BL := ($OPER{WITHUSINGARGS} ($CONS (BINDING $b) $bl)).

Syntax tactic elimwithusing_using
    ($OPER{ElimWithUsing} ($CONS $c ($CONS (COMMAND $e) $bl)))  0
    "Elim " <$c:"Command":*>  " using " <$e:"Command":*> <$BL:"List":*> 
     with $BL := ($OPER{WITHARGS} $bl).

Syntax tactic elimwithusing1 ($OPER{WITHUSINGARGS} ($CONS (COMMAND $c) $bl))  0
    " using " <$c:"Command":*>  <$BL:"List":*>
     with $BL:=($OPER{WITHARGS} $bl).

Syntax tactic elimwithusing2 ($OPER{WITHUSINGARGS} ($CONS (BINDING $b) $bl))  0
     <$B:"binding":*>  <$BL:"List":*> 
     with $B:=(BINDING $b) $BL := ($OPER{WITHUSINGARGS} $bl).

Syntax tactic withargs_ne ($OPER{WITHARGS} ($CONS $b $bl)) 0
     " with" <$BL:"List":*> 
     with $BL:=($OPER{BINDINGLIST} ($CONS $b $bl)).

Syntax tactic withargs_empty ($OPER{WITHARGS} ($LIST)) 0.
     
Syntax tactic elimtype  (ElimType $c) 0
    "ElimType " <$c:"Command":*>.

Syntax tactic casewith  ($OPER{CaseWithBindings} ($CONS $c $bl)) 0
    "Case " <$c:"Comarg":*> <$CL:"Bindinglist":*> 
     with $CL := ($OPER{WITHARGS} $bl).

Syntax tactic casetype (CaseType $c) 0
        "CaseType " <$c:"Comarg":*>. 

Syntax tactic specialize (Specialize $c) 0
     "Specialize " <$c:"Command":*>.

Syntax tactic specializewith ($OPER{SpecializeWith} ($CONS $c $l)) 0
      "Specialize "  <$c:"Command":*> " with " <$BINDL:"BindList":*>
       with  $BINDL := ($OPER{BINDINGLIST} $l).

Syntax tactic specializewithnum 
        ($OPER{SpecializeWithNum} ($CONS ($PRIM $n) ($CONS $c $bl))) 0
     "Specialize " <$n:"Numarg":*> " " <$c:"Command":*> 
     " with "  <$BINDTAIL:"Tail":*>
     with $BINDTAIL:= ($OPER{BINDINGLIST} $bl).

Syntax tactic generalize ($OPER{Generalize} $lc) 0 
        "Generalize " <$COMARGL:"Command list":*>
        with  $COMARGL := ($OPER{COMARGLIST} $lc).
Syntax tactic use (ImpElim $c) 0
        "LApply " <$c:"Command":*>.

Syntax tactic clear ($OPER{Clear} $l) 0
        "Clear " <$IDARGLIST:"Listident":*>
        with $IDARGLIST:= ($OPER{NEIDENTARGLIST} $l).


Syntax tactic exists ($OPER{Exists} $bl) 0
    "Exists " <$BL:"Command":*> 
        with $BL := ($OPER{BINDINGLIST} $bl).

Syntax tactic constructor_with_bindings
       ($OPER{ConstructorWithBindings} ($CONS ($PRIM $n) $bl)) 0
        "Constructor " <$n:"num":*> <$BL:"Bindinglist":*> 
        with $BL:=($OPER{WITHARGS} $bl).

Syntax tactic trivial (Trivial) 0
    "Trivial".

Syntax tactic change (Change $C) 0
    "Change " <$C:"Command":*>.

Syntax tactic changein (ChangeHyp $C $id) 0
    "Change " <$C:"Command":*> " in " <$id:"namehyp":*>.

Syntax tactic red (Red) 0
    "Red".

Syntax tactic redin (RedHyp $id)  0
     "Red in " <$id:"namehyp":*> .


Syntax tactic rewritelr (RewriteLR $C) 0
    "Rewrite -> " <$C:"Command":*>.

Syntax tactic rewriterl (RewriteRL $C) 0
    "Rewrite <- " <$C:"Command":*>.


Syntax tactic inductionid (Induction $id) 0
    "Induction " <$id:"Id":*>.

Syntax tactic inductionnum (Induction ($PRIM $n)) 0
    "Induction " <$n:"Number":*>.

Syntax tactic destructid (Destruct $id) 0
    "Destruct " <$id:"Id":*>.

Syntax tactic destructnum (Destruct ($PRIM $n)) 0
    "Destruct " <$n:"Number":*>.


Syntax tactic simpl (Simpl) 0
    "Simpl".

Syntax tactic simplin (SimplHyp $id) 0
     "Simpl in " <$id:"namehyp":*>.

Syntax tactic pattern ($OPER{Pattern} $L) 0
    "Pattern " <$PATTERNL:"Patterns":*>
    with $PATTERNL:=($OPER{PATTERNLIST} $L).

Syntax tactic exact (Exact $C) 0
    "Exact " <$C:"Command":*>.

Syntax tactic absurd (Absurd $C) 0
    "Absurd " <$C:"Command":*>.

Syntax tactic cut (Cut $C) 0
    "Cut " <$C:"Command":*>.

Syntax tactic left ($OPER{Left} $bl) 0
    "Left" <$COMARG:"BindingList":*>
    with $COMARG:=($OPER{BINDINGLIST} $bl).

Syntax tactic right ($OPER{Right} $bl) 0
    "Right" <$COMARG:"BindingList":*>
    with $COMARG:=($OPER{BINDINGLIST} $bl).

Syntax tactic applywith ($OPER{ApplyWithBindings} ($CONS $c $bl)) 0
    "Apply " <$c:"Com":*>  <$COMARG:"BindingList":*>
    with $COMARG:=($OPER{WITHARGS} $bl).

Syntax tactic replace (Replace $C1 $C2) 0
    "Replace " <$C1:"Command":*> " with " <$C2:"Command":*>.

Syntax tactic discriminate (Discriminate) 0
     "Simple Discriminate" .

Syntax tactic injection ($OPER{Injection} ($CONS $c $l)) 0
      "Injection " <$c:"Command":*> " " <$NUMARGL:"listint":*> 
       with $NUMARGL:=($OPER{NENUMLIST} $l)
.



Syntax tactic reflexivity (Reflexivity) 0
     "Reflexivity".

Syntax tactic symmetry (Symmetry) 0
     "Symmetry".

Syntax tactic transitivity (Transitivity $C) 0
     "Transitivity " <$C:"Command":*> .

Syntax tactic idtac (IDTAC) 0  
        "Idtac".

Syntax tactic command (COMMAND $c) 8
	<$C:"Command":E> with $C:=(PPUNI$COMMAND $c).

(* ============================================== *)
(*      PP rules for composed tactics             *)
(* ============================================== *)

Syntax tactic tactic_com_orelse (ORELSE $st $tc) 1
        <$st:"SimpleTac":L> " Orelse " <$tc:"TacComp":E>.

Syntax tactic do (DO ($PRIM $n) $tc) 0
        "Do " <$n:"Int":*> " "  <$tc:"TacComp":E>.

Syntax tactic try (TRY $T) 0
    "Try " <$T:"Tactic":E>.

Syntax tactic repeat (REPEAT $tc) 0
    "Repeat " <$tc:"Comptac":E>.
