(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                PPVernac.v                                *)
(****************************************************************************)


(* ========================================== *)
(*      PP rules for basic elements           *)
(* ========================================== *)
Syntax vernac identarg ($VAR $id)  0
        <$id:"Ident":*>.


Syntax vernac ne_identarg_listnil ($OPER{NEIDENTARGLIST} ($LIST)) 0.
Syntax vernac ne_identarg_listone ($OPER{NEIDENTARGLIST} ($LIST $id)) 0
        <$id:"Ident":*>.
Syntax vernac ne_identarg_listcons ($OPER{NEIDENTARGLIST} ($CONS $id $idl)) 0
        <$id:"Ident":*> " " <$TAIL:"Tail":*>
        with  $TAIL := ($OPER{NEIDENTARGLIST} $idl).


Syntax vernac ne_identarg_comma_listcons 
    ($OPER{NEIDENTARGCOMMALIST} ($CONS $id $idl)) 0
        <$id:"Ident":*> "," <$TAIL:"Tail":*>
        with  $TAIL := ($OPER{NEIDENTARGCOMMALIST} $idl).
Syntax vernac ne_identarg_comma_listone 
        ($OPER{NEIDENTARGCOMMALIST} ($LIST $id)) 0
        <$id:"Ident":*>.


Syntax vernac ne_ident_list_commacons 
    ($OPER{NEIDENTCOMMALIST} ($CONS $id $idl)) 0
        <$id:"Ident":*> "," <$TAIL:"Tail":*>
        with  $TAIL := ($OPER{NEIDENTCOMMALIST} $idl).
Syntax vernac ne_ident_comma_listone 
    ($OPER{NEIDENTCOMMALIST} ($LIST $id)) 0
        <$id:"Ident":*>.


Syntax vernac ne_numarg_list_commacons 
    ($OPER{NENUMARGLIST} ($CONS ($PRIM $n) $idl)) 0
        <$n:"Int":*> "," <$TAIL:"Tail":*>
        with  $TAIL := ($OPER{NENUMARGLIST} $idl).
Syntax vernac ne_numarg_listone ($OPER{NENUMARGLIST} ($LIST ($PRIM $n))) 0
        <$n:"Int":*>.



Syntax vernac spat_binder (SPAT_BINDER $id ($PRIM $c)) 0
        <$id:"Ident":*> ":=" <$c:"SpatArg":*> .

Syntax vernac ne_spat_binder_list_cons 
      ($OPER{NESPATBINDERLIST} ($CONS $b $lb)) 0
        <$b:"SpatBinder":*> " " <$TAIL:"ListSpatBind":*>
        with  $TAIL := ($OPER{NESPATBINDERLIST} $lb) .
Syntax vernac ne_spat_binder_list_one ($OPER{NESPATBINDERLIST} ($LIST $b)) 0
        <$b:"SpatBinder":*>.
 


Syntax vernac theorem_body_line_solven (VERNACCALL {SOLVE} ($PRIM $n) $tac) 0
        <$n:"Int":*> ":" <$tac:"Tactic":*> "."  .

Syntax vernac theorem_body_line_solve1 (VERNACCALL {SOLVE} {1}  $tac) 0
        <$tac:"Tactic":*> "."  .
Syntax vernac theorem_body_line_solveby (VERNACCALL {BY} $tac)  0
        "BY " <$tac:"Tactic":*> ".".

Syntax vernac theorem_body_line_list_nil ($OPER{THEOREMBODYLINELIST} ($LIST)) 0.
Syntax vernac theorem_body_line_list_cons  
      ($OPER{THEOREMBODYLINELIST} ($CONS $bl $bll)) 0 
        [<v 0> <$bl:"ThmBodyLine":*> [0 0] <$TAIL:"ThmBodyLineL":*> ]
        with $TAIL:= ($OPER{THEOREMBODYLINELIST} $bll).


Syntax vernac theorem_body ($OPER{VERNACARGLIST} $l) 0
        <$LIST:"ThmBodyLineList":*> 
        with $LIST:= ($OPER{THEOREMBODYLINELIST} $l).
        

(* ============================================== *)
(*      PP rules for Vernac                       *)
(* ============================================== *)

Syntax vernac pwd  (PWD) 0  
        "Pwd." FNL  .

Syntax vernac quit  (QUIT) 0 
        "Quit." FNL .

Syntax vernac drop (DROP) 0
        "Drop." FNL .

Syntax vernac cd1 (CD) 0
        "Cd." FNL .

Syntax vernac cd2 (CD ($PRIM $dir)) 0
       "Cd " <$dir:"name":*> .

Syntax vernac goal (GOAL) 0 
       "Goal." FNL  .

Syntax vernac goalc (GOAL $c) 0 
       "Goal "  <$c:"com":*> "." FNL .

Syntax vernac abort (ABORT) 0
        "Abort." FNL  .

Syntax vernac abortid (ABORT $id) 0
       "Abort " <$id:"ident":*> "." FNL   . 

Syntax vernac abortall (ABORTALL) 0
	"Abort All." FNL .

Syntax vernac restart (RESTART) 0
        "Restart." FNL  .

Syntax vernac  proof (PROOF $c) 0
        "Proof " <$c:"term":*> "." FNL  .

Syntax vernac focus (FOCUS)  0 
        "Focus." FNL .

Syntax vernac unfocus (UNFOCUS)  0
        "Unfocus." FNL .

Syntax vernac setundo (SETUNDO ($PRIM $n)) 0 
        "Set Undo" <$n:"num":*>  "." FNL  .

Syntax vernac unsetundo (UNSETUNDO) 0 
        "Unset Undo" "." FNL  .

Syntax vernac sethypslimit (SETHYPSLIMIT ($PRIM $n)) 0 
        "Set Hyps_limit" <$n:"num":*>  "." FNL  .

Syntax vernac unsethypslimit  (UNSETHYPSLIMIT) 0 
        "Unset Hyps_limit" "." FNL  .

Syntax vernac savenamed (SaveNamed) 0 
        "Save." FNL .

Syntax vernac defined (DefinedNamed) 0
        "Defined." FNL .

Syntax vernac savermark (SaveAnonymousRmk $id) 0 
        "Save Remark " <$id:"ident":*> "." FNL .

Syntax vernac savethm (SaveAnonymousThm $id) 0
        "Save Theorem " <$id:"ident":*> "." FNL .


Syntax vernac savestate_str  (SaveState $id ($PRIM $s))  0
        "Save State "  <$id:"ident":*> " "  <$s:"ident":*> "." FNL .


Syntax vernac savestate (SaveState $id {""}) 0
        "Save State " <$id:"ident":*> "." FNL  .


Syntax vernac  save (SaveAnonymousThm $id) 0
        "Save " <$id:"Ident":*> "." FNL.

Syntax vernac transparent (TRANSPARENT $id) 0
        "Transparent " <$id:"ident":*> "." FNL .

Syntax vernac opaque (OPAQUE $id) 0
        "Opaque " <$id:"ident":*> "." FNL .


Syntax vernac writestate (WriteStates $id) 0
         "Write States " <$id:"ident":*> "." FNL  .


Syntax vernac wirtestate_str (WriteStates (STRING{$id})) 0
         "Write States " <$id:"ident":*> "." FNL  .


Syntax vernac writemodule (WriteModule $id)  0
         "Write Module " <$id:"ident":*> "." FNL  .

Syntax vernac writemodulefile (WriteModuleTo $id ($PRIM $file)) 0
         "Write Module " <$id:"ident":*> " " <$file:"string":*>  "." FNL  .

Syntax vernac remove (RemoveState $id) 0
        "Remove State "  <$id:"ident":*> "." FNL  .


Syntax vernac undo (UNDO {1}) 0
        "Undo." FNL .

Syntax vernac undon (UNDO ($PRIM $n))  0
        "Undo " <$n:"num":*>  "." FNL  .

Syntax vernac show (SHOW) 0  
        "Show." FNL  .

Syntax vernac shown (SHOW ($PRIM $n)) 0
        "Show " <$n:"num":*>  "." FNL  .

Syntax vernac shownode (ShowNode) 0
        "Show Node." FNL  .

Syntax vernac go (Go ($PRIM $n)) 0 
        "Go " <$n:"num":*>  "." FNL  .

Syntax vernac gotop (Go {"top"}) 0 
        "Go top." FNL .

Syntax vernac goprev (Go {"prev"}) 0 
        "Go prev." FNL .

Syntax vernac gonext (Go {"next"}) 0 
        "Go next." FNL .

Syntax vernac showscript (ShowScript) 0
	"Show Script." FNL .

Syntax vernac showproof  (ShowProof) 0
        "Show Proof." FNL  .

Syntax vernac showtree  (ShowTree) 0
        "Show Tree." FNL  .

Syntax vernac showproofs  (ShowProofs) 0
        "Show Proofs." FNL  .


Syntax vernac printall (PrintAll) 0
        "Print All." FNL  .

Syntax vernac print  (PRINT) 0
        "Print." FNL .

Syntax vernac printhint  (PrintHint) 0
        "Print Hint." FNL  .

(*
Syntax vernac printhintgoal  (PrintHintGoal) 0
        "Print Hint Goal." FNL  .
*)

Syntax vernac printhintid  (PrintHintId $id) 0
        "Print Hint " <$id:"Ident":*> "." FNL  .

Syntax vernac printsection  (PrintSec $s) 0
        "Print Section " <$s:"Ident":*>  "." FNL  .

Syntax vernac printstates  (PrintStates) 0
        "Print States." FNL  .


Syntax vernac printgrammar (PrintGrammar  $whatfor $id) 0
        "Print Grammar " <$whatfor:"Ident":*> " " <$id:"Ident":*> "." FNL .

Syntax vernac print_latex_grammar_rules (PrintLaTeXGrammarRules $whatfor $id) 0
       "Print LaTeX Grammar " <$whatfor:"Ident":*> " " <$id:"Ident":*> "." FNL .

Syntax vernac print_latex_grammar  (PrintLaTeXGrammar $whatfor) 0
        "Print LaTeX Grammar " <$whatfor:"Ident":*> "." FNL .


Syntax vernac printloadpath  (PrintPath) 0
        "Print LoadPath." FNL .

Syntax vernac addpath (ADDPATH ($PRIM $dir)) 0
        "AddPath " <$dir:"String":*>.

Syntax vernac delpath (DELPATH ($PRIM $dir)) 0
        "DelPath " <$dir:"String":*>.

Syntax vernac printmodules (PrintModules) 0
        "Print Modules." FNL .

Syntax vernac printopaqueid (PrintOpaqueId $id) 0
        "Print Proof " <$id:"Ident":*> "." FNL .

Syntax vernac printid (PrintId $id) 0
        "Print " <$id:"Ident":*> "." FNL .

(*
Syntax vernac requireimport (RequireImportSpec $id) 0
        "Require " <$id:"Ident":*> "." FNL .

Syntax vernac  requireimportspecfrom 
        (RequireImportSpecFrom $id ($PRIM $f)) 0
        "Require Import " <$id:"Ident":*> " " <$f:"File":*> "." FNL .

Syntax vernac requireexport (RequireExportSpec $id) 0
        "Require Export " <$id:"Ident":*> "." FNL  .

Syntax vernac  requireexportspecfrom  (RequireExportSpecFrom $id ($PRIM $f)) 0
        "Require Export " <$id:"Ident":*> " " <$f:"File":*> "." FNL .
*)

Syntax vernac requireimplementation  (RequireImport $id) 0
        "Require Implementation "  <$id:"Ident":*> "." FNL .

Syntax vernac  requireimplementationfrom (RequireImportFrom $id ($PRIM $f)) 0
        "Require Implementation " <$id:"Ident":*> " " <$f:"File":*> "." FNL .

Syntax vernac requireexportimplementation  (RequireExport $id) 0
        "Require Export "  <$id:"Ident":*> "." FNL .

Syntax vernac  requireexportimplementationfrom (RequireExportFrom $id ($PRIM $f)) 0
        "Require Export " <$id:"Ident":*> " " <$f:"File":*> "." FNL .

Syntax vernac syntaxwith
 ($OPER{SYNTAX} ($CONS $wf ($CONS $s ($CONS ($PRIM $spat) ($CONS ($PRIM $unp) $consl)))))  0
    "Syntax " <$wf:"FamilyRule":*> " " <$s:"RuleName":*> " "<$spat:"Pattern":*>
     " "  <$unp:"Unparsing":*> " with " <$CONSL:"WithBindings":*> "." FNL 
     with $CONSL := ($OPER{NESPATBINDERLIST} $consl).

Syntax vernac syntax  (SYNTAX $wf $s ($PRIM $spat) $unp) 0 
  "Syntax " <$wf:"FamilyRule":*> " " <$s:"RuleName":*> " " <$spat:"Pattern":*> "." FNL .


Syntax vernac  grammar    (GRAMMAR ($PRIM $ge)) 0
        "Grammar " <$ge:"GrammarEntry":*>.

Syntax vernac token (TOKEN ($PRIM $s)) 0
      	"Token " <$s:"String":*> "." FNL .

Syntax vernac infix (INFIX ($PRIM $n) ($PRIM $s) $pref) 0
     "Infix " <$n:"Int":*> " " <$s:"String":*> " " <$pref:"Ident":*> "." FNL .

Syntax vernac hintunfold ($OPER{HintUnfold} $l) 0
        "Hint Unfold " <$IDENTARG:"ListIdent":*> "." FNL 
        with  $IDENTARG := ($OPER{NEIDENTARGLIST} $l).

Syntax vernac hintresolve ($OPER{HintResolve} $l) 0
        "Hint " <$IDENTARG:"ListIdent":*> "." FNL 
        with  $IDENTARG := ($OPER{NEIDENTARGLIST} $l).

Syntax vernac immediate ($OPER{HintImmediate} $l) 0
        "Immediate " <$IDENTARG:"ListIdent":*> "." FNL 
        with  $IDENTARG := ($OPER{NEIDENTARGLIST} $l).

Syntax vernac section (BeginSection $id) 0
        "Section " <$id:"Ident":*> "." FNL .

Syntax vernac chapter (BeginChapter $id) 0  (** Chapter o Section? **)
        "Chapter " <$id:"Ident":*> "." FNL .

Syntax vernac module (BeginModule $id) 0
        "Module " <$id:"Ident":*> "." FNL .

Syntax vernac beginsilent (BeginSilent) 0
        "Begin Silent." FNL .

Syntax vernac endsilent (EndSilent) 0
        "End Silent." FNL .

Syntax vernac endsection (EndSection $id) 0
        "End " <$id:"Ident":*> "." FNL .

Syntax vernac suspend (SUSPEND) 0 
        "Suspend." FNL .

Syntax vernac resume (RESUME) 0
	"Resume." FNL .

Syntax vernac resumeid (RESUME $id) 0
        "Resume " <$id:"Ident":*> "." FNL .

Syntax vernac extraction (PrintExtractId $id) 0
        "Extraction " <$id:"Ident":*> "." FNL .


Syntax vernac extractionid  (PrintExtract) 0
        "Extraction." FNL .

Syntax vernac info (INFO $id) 0
        "Info "  <$id:"Ident":*> "." FNL .

Syntax vernac search (SEARCH  $id) 0
        "Search "  <$id:"Ident":*> "." FNL .

Syntax vernac inspect (INSPECT ($PRIM $n)) 0
        "Inspect "  <$n:"num":*> "." FNL .

Syntax vernac  syntaxmacro (SyntaxMacro $id $c) 0
        "Syntactic Definition "   <$id:"Ident":*> " := "  <$c:"Command":*> "." FNL .

Syntax vernac  syntaxmacroid (SyntaxMacro $id $c ($PRIM $n)) 0
     "Syntactic Definition "   <$id:"Ident":*> " := " <$c:"Command":*> 
        " | " <$n:"Int":*> "." FNL .

Syntax vernac  abstraction  ($OPER{ABSTRACTION} ($CONS $id ($CONS $c $l))) 0
     "Abstraction "  <$id:"Ident":*>  
      " [" <$ARGLIST:"ListInt":*> "]" " := " <$c:"Command":*> "." FNL 
     with $ARGLIST := ($OPER{NENUMARGLIST} $l).

  
Syntax vernac startproof_thm (StartProof $thm  $id $c) 0
      <$thm:"def_tok":*> " " <$id:"ThmName":*> " : " <$c:"Command":*> "." FNL .


Syntax vernac def_theorem (TheoremProof {"THEOREM"} $id $c $tb) 0
   [<hov 0>  "Theorem " <$id:"ThmName":*> " : " <$c:"Command":*> 
     " := "  [0 1] [<v 0> "Proof" [0 1] <$tb:"ThmBody":*>  "Qed." FNL ]] .


Syntax vernac def_lemma (TheoremProof {"LEMMA"}  $id $c $tb) 0
   [<hov 0>  "Lemma " <$id:"ThmName":*> " : " <$c:"Command":*> 
     " := "  [0 1] [<v 0> "Proof" [0 1] <$tb:"ThmBody":*>  "Qed." FNL ]] .

Syntax vernac def_fact (TheoremProof {"FACT"}  $id $c $tb) 0
   [<hov 0>  "Fact  " <$id:"ThmName":*> " : " <$c:"Command":*> 
     " := "  [0 1] [<v 0> "Proof" [0 1] <$tb:"ThmBody":*>  "Qed." FNL ]] .

Syntax vernac def_remark (TheoremProof  {"REMARK"} $id $c $tb) 0
   [<hov 0>  "Remark " <$id:"ThmName":*> " : " <$c:"Command":*> 
     " := "  [0 1] [<v 0> "Proof" [0 1] <$tb:"ThmBody":*>  "Qed." FNL ]] .



Syntax vernac def_proof (TheoremProof {"DEFINITION"} $id $c $tb) 0
   [<hov 0>  "Definition " <$id:"ThmName":*> " : " <$c:"Command":*> 
     " := "  [0 1] [<v 0> "Proof" [0 1] <$tb:"ThmBody":*>  "Qed." FNL ]] .

Syntax vernac def_let (TheoremProof  {"LET"}  $id $c $tb) 0
   [<hov 0>  "Let " <$id:"ThmName":*> " : " <$c:"Command":*> 
     " := "  [0 1] [<v 0> "Proof" [0 1] <$tb:"ThmBody":*>  "Qed." FNL ]] .

Syntax vernac def_local (TheoremProof {"LOCAL"} $id $c $tb) 0
   [<hov 0>  "Local " <$id:"ThmName":*> " : " <$c:"Command":*> 
     " := "  [0 1] [<v 0> "Proof" [0 1] <$tb:"ThmBody":*>  "Qed." FNL ]] .



(* ===  Two rules for the cast abst stx. Both have absolutely different meanings ==== *)
(* ==    how to choose the correct one?. Correct the grammar to eliminate ambuity == *)
(******
Syntax vernac definitioncast 
    (DEFINITION $def $id (COMMAND (CAST $c1 $c2))) 0
       <$def:"TypeOfDef":*>  " " <$id:"Ident":*> " := " 
            <$c1:"Command":*>  ":"   <$c2:"Command":*> "." FNL . 


Syntax vernac definitioncast2
    (DEFINITION $def $id (COMMAND (CAST $c1 $c2))) 0
       <$def:"TypeOfDef":*>  " " <$id:"Ident":*> ":" 
            <$c1:"Command":*>  ":="   <$c2:"Command":*> "." FNL . 
********)

Syntax vernac definition (DEFINITION $def $id  $c)  0
     <$def:"TypeOfDef":*> " " <$id:"Ident":*> ":=" <$c:"CommandArg":*> "." FNL .


Syntax vernac variable  (VARIABLE $hyp $b) 0
        <$hyp:"TypeOfHyp":*> " " <$b:"Binder":*> "." FNL .

Syntax vernac parameter (PARAMETER $hyp $b)0
        <$hyp:"TypeOfHyp":*> " " <$b:"Binder":*> "." FNL .  

(*****
Syntax vernac parameter (PARAMETER $_ $b)0
        "Parameter " <$b:"Binder":*> "." FNL . 
****)

Syntax vernac solvetac (SOLVE ($PRIM $n) $tac) 0
        <$n:"Num":*> ":" <$tac:"TacticLComList":*> "." FNL .

Syntax vernac solve1 (SOLVE {1} $tac) 0
        <$tac:"TacticComList":*> "." FNL .


Syntax vernac by (BY $tac) 0
        "BY "  <$tac:"Tactic":*> "." FNL   .

Syntax vernac check (CHECK $check $c) 0
        <$check:"TypeOfEval":*> " " <$c:"CommArg":*> "." FNL .


Syntax vernac oneinductive (ONEINDUCTIVE $f $id $c $ip $lidcom) 0
   <$f:"Finite":*>  <$id:"Name":*> " " <$IP:"IndParam":*>
    " : " <$c:"Command":*>  " := "   FNL <$lidcom:"Command":*> "." FNL
    with  $IP:=(INDPAR $ip).

Syntax vernac inductiveSet
   (ONEINDUCTIVE  $f $id (COMMAND (PROP{Pos})) $ip $lidcom) 0
     <$f:"Finite":*> " Set " 
       <$id:"Name":*> " " <$IP:"IndParam":*>  
        " := "   FNL <$lidcom:"Command":*> "." FNL 
      with  $IP:=(INDPAR $ip).

Syntax vernac inductiveProp
   (ONEINDUCTIVE $f $id (COMMAND (PROP{Null})) $ip $lidcom) 0
     <$f:"Finite":*> " Prop " 
       <$id:"Name":*> " " <$IP:"IndParam":*>  
        " := "   FNL <$lidcom:"Command":*> "." FNL 
      with  $IP:=(INDPAR $ip) .

Syntax vernac inductiveType
   (ONEINDUCTIVE  $f $id (COMMAND (TYPE{Null})) $ip $lidcom) 0
      <$f:"Finite":*> " Type " 
       <$id:"Name":*> " " <$IP:"IndParam":*>  
        " := "   FNL <$lidcom:"Command":*> "." FNL 
      with  $IP:=(INDPAR $ip) .

Syntax vernac inductiveTypeset
   (ONEINDUCTIVE $f $id (COMMAND (TYPE{Pos})) $ip $lidcom) 0
     <$f:"Finite":*> " Typeset " 
       <$id:"Name":*> " " <$IP:"IndParam":*>  
        " := "   FNL <$lidcom:"Command":*> "." FNL  
      with  $IP:=(INDPAR $ip).

Syntax vernac mutualindcoind
     (MUTUALINDUCTIVE $ip ($OPER{VERNACARGLIST} $inds) ($OPER{VERNACARGLIST} $coinds))0
     [<v 0> "Mutual " <$IP:"IndParam":*>  <$IND:"SpecifInd":*> <$COIND:"SpecifCoInd":*>] "." FNL 
        with $IND := ($OPER{SPECIFIND} $inds)
             $COIND := ($OPER{SPECIFCOIND} $coinds)
             $IP  := (INDPAR $ip).

Syntax vernac mutualrec (MUTUALRECURSIVE ($OPER{VERNACARGLIST} $recs)) 0
        "Fixpoint " <$RECS:"SpecifRec":*> "." FNL 
         with $RECS := ($OPER{SPECIFREC} $recs).

Syntax vernac mutualcorec (MUTUALCORECURSIVE ($OPER{VERNACARGLIST} $corecs)) 0
        "CoFixpoint " <$RECS:"SpecifRec":*> "." FNL 
         with $RECS := ($OPER{SPECIFCOREC} $corecs).

Syntax vernac scheme (INDUCTIONSCHEME ($OPER{VERNACARGLIST} $sch)) 0
        "Scheme " <$SCH:"Schemes":*> "." FNL 
        with $SCH:= ($OPER{SPECIFSCHEME} $sch).

Syntax vernac sortprop (PROP{Null}) 0
        "Prop".
Syntax vernac sortset  (PROP{Pos}) 0 
        "Set". 
Syntax vernac sorttype (TYPE{Null}) 0
        "Type".
Syntax vernac sorttypeset (TYPE{Pos}) 0
        "Typeset".
 
Syntax vernac thm_tok_Thm {"THEOREM"}  0 
        "Theorem" .
Syntax vernac thm_tok_Lem {"LEMMA"}  0
        "Lemma".
Syntax vernac thm_tok_Fact  {"FACT"}  0
        "Fact".
Syntax vernac thm_tok_rem  {"REMARK"} 0
        "Remark".

Syntax vernac def_tok_def {"DEFINITION"} 0
        "Definition".
Syntax vernac def_tok_let {"LET"} 0
        "Let".
Syntax vernac def_tok_local {"LOCAL"} 0
        "Local".

(* === Three possible rules for VARIABLE === *)

Syntax vernac hyp_tok_variable {"VARIABLE"} 0
        "Variable".

Syntax vernac hyp_tok_variables {"VARIABLES"} 0
        "Variables".

Syntax vernac hyp_tok_hypothesis {"HYPOTHESIS"} 0
        "Hypothesis".

(* === Three possible rules for AXIOM  === *)

Syntax vernac param_tok_axiom {"AXIOM"} 0
        "Axiom".

Syntax vernac param_tok_parameter {"PARAMETER"} 0
        "Parameter".

Syntax vernac param_tok_parameters {"PARAMETERS"} 0
        "Parameters".

Syntax vernac oneind
       (ONEIND ($OPER{VERNACARGLIST} ($CONS $id ($CONS $c ($LIST $lidcom))))) 0
    [<hov 0>    <$id:"Ident":*> [1 4] ": " <$c:"ComArg":*>  " :=" FNL <$lidcom:"ListCom":*>].

Syntax vernac finite_tok_inductive {"Inductive"} 0 
	"Inductive ".

Syntax vernac finite_tok_coinductive {"CoInductive"} 0 
	"CoInductive ".

Syntax vernac specifind_something ($OPER{SPECIFIND} ($CONS $ind $indl)) 0
       "Inductive " FNL "       " <$IND:"OneInd":*>  <$INDL:"SpecifInd":*>
        with $IND  := (ONEIND $ind) 
	     $INDL := ($OPER{SPECIFINDLIST} $indl).

Syntax vernac specifind_nil ($OPER{SPECIFIND} ($LIST)) 0.

Syntax vernac specif_ind_list ($OPER{SPECIFINDLIST}  ($CONS $ind $indl)) 0
       FNL "  with " <$IND:"OneInd":*>  <$INDL:"SpecifInd":*>
        with $IND  := (ONEIND $ind) 
	     $INDL := ($OPER{SPECIFINDLIST} $indl).

Syntax vernac specif_ind_nil ($OPER{SPECIFINDLIST} ($LIST)) 0 FNL.

Syntax vernac specif_coind ($OPER{SPECIFCOIND}  ($CONS $coind $coindl)) 0
       "CoInductive " FNL "       " <$COIND:"OneCoInd":*> 
                     <$COINDL:"SpecifCoInd":*>
           with $COIND  := (ONEIND $coind) 
	        $COINDL := ($OPER{SPECIFCOINDLIST} $coindl).

Syntax vernac specif_coind_nil ($OPER{SPECIFCOIND} ($LIST)) 0 .

Syntax vernac specif_coind_list ($OPER{SPECIFCOINDLIST}  ($CONS $coind $coindl)) 0
       FNL "  with " <$COIND:"OneCoInd":*> 
                         <$COINDL:"SpecifInd":*>
        with $COIND  := (ONEIND $coind)
	     $COINDL := ($OPER{SPECIFCOINDLIST} $coindl).
Syntax vernac specif_coind_list_nil ($OPER{SPECIFCOINDLIST} ($LIST)) 0 FNL.


Syntax vernac onerec (ONEREC ($OPER{VERNACARGLIST} 
 ($CONS $id ($CONS ($OPER{BINDERLIST} $idl) ($CONS $c ($LIST $def)))))) 0
   <$id:"Ident":*> "[" <$IDL:"IdentList":*> "]" ":"  <$c:"ComArg":*> 
     ":="  [0 1]   <$def:"ComArg":*>
   with $IDL := ($OPER{NEBINDERSEMILIST} $idl) .

Syntax vernac  specifrec_cons ($OPER{SPECIFREC} ($CONS $rec $recl)) 0
     <$REC:"Onerec":*> " with " <$RECL:"ListRec":*>
     with $REC:=(ONEREC $rec) $RECL := ($OPER{SPECIFREC} $recl).
Syntax vernac  specifrec_one ($OPER{SPECIFREC} ($LIST $rec)) 0
        <$REC:"Onerec":*> with $REC:=(ONEREC $rec).  

Syntax vernac onecorec (ONECOREC ($OPER{VERNACARGLIST} 
 ($CONS $id  ($CONS $c ($LIST $def))))) 0
   <$id:"Ident":*>  ":"  <$c:"ComArg":*> ":=" <$def:"ComArg":*> .

Syntax vernac  specifcorec_cons ($OPER{SPECIFCOREC} ($CONS $corec $corecl)) 0
     <$COREC:"OneCorec":*> [0 0] " with " <$CORECL:"ListCoRec":*>
     with $COREC:= (ONECOREC $corec) $CORECL := ($OPER{SPECIFCOREC} $corecl).
Syntax vernac  specifcorec_one ($OPER{SPECIFCOREC} ($LIST $corec)) 0
        <$COREC:"OneCorec":*> with $COREC:=(ONECOREC $corec).   

Syntax vernac onescheme (ONESCHEME ($OPER{VERNACARGLIST} ($CONS $id ($CONS $dep ($CONS $c ($LIST $s)))))) 0
    <$id:"Ident":*> " := " <$dep:"Dependency":*> 
    <$c:"ComArg":*> " Sort " <$s:"Sort":*>.
     

Syntax vernac  specifschemecons ($OPER{SPECIFSCHEME} ($CONS $rec $recl)) 0
     <$REC:"Onescheme":*> [1 1] " with " <$RECL:"ListRec":*>
     with $REC:=(ONESCHEME $rec) $RECL :=   ($OPER{SPECIFSCHEME} $recl).
Syntax vernac  specifschemeone ($OPER{SPECIFSCHEME} ($LIST $rec)) 0
        <$REC:"Onerec":*> with $REC:=(ONESCHEME $rec).  

Syntax vernac dependant {"DEP"} 0 
        "Induction for " .

Syntax vernac nodependant {"NODEP"} 0 
        "Minimality for " .

Syntax vernac  eval_tok_eval  {"EVAL"} 0
        "Eval".

Syntax vernac eval_tok_compute  {"COMPUTE"} 0
        "Compute".

Syntax vernac eval_tok_simpl  {"SIMPLIFY"} 0
        "Simplify".

Syntax vernac eval_tok_check {"CHECK"} 0
        "Check".

Syntax vernac eval_tok_type {"PRINTTYPE"} 0
        "Type".

Syntax vernac binder ($OPER{BINDER} ($CONS $c $idl)) 0
        <$IDENTLIST:"ListIdent":*> ":" <$C:"Command":*>
        with $IDENTLIST := ($OPER{NEIDENTARGCOMMALIST} $idl)
             $C := (COMMAND $c).



Syntax vernac ne_binder_semi_list_cons 
        ($OPER{NEBINDERSEMILIST} ($CONS $bind $lb)) 0
        <$bind:"Binder":*> ";" <$TAIL:"ListBind":*>
        with $TAIL := ($OPER{NEBINDERSEMILIST} $lb).
Syntax vernac ne_binder_semi_list_one ($OPER{NEBINDERSEMILIST} ($LIST $bind)) 0
        <$bind:"Binder":*>.


Syntax vernac indparlist (INDPAR ($OPER{BINDERLIST} $l)) 0
        "[" <$L:"Binderlist":*> "]" 
        with $L:=($OPER{NEBINDERSEMILIST} $l).
Syntax vernac indpar (INDPAR (BINDERLIST)) 0.
       

Syntax vernac idcom (BINDER $c $id) 0
        <$id:"Ident":*> ":" <$C:"Command":*>
        with $C := (PPUNI$COMMAND $c).


Syntax vernac ne_lidcom_cons ($OPER{NELIDCOM} ($CONS $idcom $l)) 0 
       [<v 0> "  " <$idcom:"Binder":*> FNL <$TAIL:"ListIdCom":*>]
        with $TAIL := ($OPER{NELIDCOMBARE} $l).
Syntax vernac ne_lidcom_one ($OPER{NELIDCOM} ($LIST $idcom)) 0 
        "  "<$idcom:"Binder":*> .
Syntax vernac ne_lidcom_nil ($OPER{NELIDCOM} ($LIST)) 0.


Syntax vernac ne_lidcom_cons_bare ($OPER{NELIDCOMBARE} ($CONS $idcom $l)) 0 
        [<h 0>  "| " <$idcom:"Binder":*> FNL <$TAIL:"ListIdCom":*>]
        with $TAIL := ($OPER{NELIDCOMBARE} $l).
Syntax vernac ne_lidcom_nil_bare ($OPER{NELIDCOMBARE} ($LIST)) 0.          

Syntax vernac lidcom_nolist (BINDERLIST) 0.
Syntax vernac lidcom_list ($OPER{BINDERLIST} $l) 0
        <$NELIST:"Listcom":*> with $NELIST := ($OPER{NELIDCOM} $l). 



Syntax vernac constr (COMMAND $c) 8
	<$C:"Command":E> with $C:=(PPUNI$COMMAND $c).
Syntax vernac tactic (TACTIC $t) 100
	<$T:"Tactic":E> with $T:=(PPUNI$TACTIC $t).


Syntax vernac nestringarglist_cons ($OPER{NESTRINGARGLIST} ($CONS ($PRIM $s) $l)) 0
    <$s:"stringarg":*> [1 0] <$TAIL:"stringarglist":*>
    with $TAIL:= ($OPER{NESTRINGARGLIST} $l).
Syntax vernac nestringarglist_nil ($OPER{NESTRINGARGLIST} ($LIST)) 0.


Syntax vernac print_ML_path (PrintMLPath) 0 
   "Print ML Path.".

Syntax vernac print_ML_modules (PrintMLModules) 0
   "Print ML Modules.".

Syntax vernac add_ML_path (AddMLPath ($PRIM $dir)) 0
   "Add ML Path "  <$dir:"stringarg":*>.


Syntax vernac declare_ML_module ($OPER{DeclareMLModule} $l) 0
    "Declare ML Module " <$LMOD:"list_string":*> 
    with $LMOD :=($OPER{NESTRINGARGLIST} $l).


Syntax vernac import_ML_module (ImportMLModule ($PRIM $fname)) 0
   "Import ML Module " <$fname:"string":*>.

Syntax vernac require_ML_module ($OPER{RequireMLModule} $l) 0
   "Require ML Module "  <$LMOD:"list_string":*> 
    with $LMOD :=($OPER{NESTRINGARGLIST} $l).

Syntax tactic ML (MLTACTIC ($PRIM $s)) 0 
   "ML " <$s:"string":*>.





