(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Tactic.v                                 *)
(****************************************************************************)
Grammar tactic tactic_com_list :=.
Grammar tactic tactic_com_list_tail :=.
Grammar tactic tactic_com :=.
Grammar tactic tactic_com_seq :=.
Grammar tactic tactic_com_seq_body :=.
Grammar tactic simple_tactic :=.

(* tactic_com_list : a list of either tactic_com_orelses, or
   bar-separated tactic_com_lists, which must start with a
   tactic_com_orelse
 *)
Grammar tactic
identarg :=
  [ prim:ident($id) ] -> [$0 = ($VAR $id)]

with
numarg :=
  [ prim:number($n) ] -> [$0 = $n]

with
comarg :=
  [ command:command($c) ] -> [$0 = (COMMAND $c)]

with
lcomarg :=
  [ command:lcommand($c) ] -> [$0 = (COMMAND $c)]

with
ne_identarg_list :=
  [ identarg($id) ne_identarg_list($idl) ] -> [$0 = ($CONS $id $idl)]
| [ identarg($id) ] -> [$0 = ($LIST $id)]

with
ne_num_list :=
  [ numarg($n) ne_num_list($l) ] -> [$0 = ($CONS $n $l)]
| [ numarg($n) ] -> [$0 = ($LIST $n)]

with
pattern_occ :=
  [ ne_num_list($nl) comarg($c) ] ->
	[$0 = ($OPER{PATTERN} ($CONS $c $nl))]
| [ comarg($c) ] -> [$0 = ($OPER{PATTERN} ($LIST $c))]

with
ne_pattern_list :=
  [ pattern_occ($p) ne_pattern_list($l) ] -> [$0 = ($CONS $p $l)]
| [ pattern_occ($p) ] -> [$0 = ($LIST $p)]

with
simple_binding :=
  [ identarg($id) ":=" comarg($c) ] -> [$0 = (BINDING $id $c)]
| [ numarg($n) ":=" comarg($c) ] -> [$0 = (BINDING $n $c)]

with
simple_binding_list :=
  [ simple_binding($b) simple_binding_list($l) ] -> [$0 = ($CONS $b $l)]
| [ ] -> [$0 = ($LIST)]

with 
com_binding_list := 
  [ comarg($c) com_binding_list($bl) ] -> [$0 = ($CONS (BINDING $c) $bl)]
| [ ] -> [$0 = ($LIST)]

with
binding_list :=
  [ comarg($c1) ":=" comarg($c2) simple_binding_list ($bl) ] -> 
   case $c1 of (COMMAND ($VAR $id)) -> [$0 = ($CONS (BINDING ($VAR $id) $c2) $bl)] esac
| [ numarg($n) ":=" comarg($c) simple_binding_list ($bl) ] -> 
         [$0 =($CONS  (BINDING $n $c) $bl)]
| [ comarg($c1) com_binding_list($bl) ] -> [$0 = ($CONS (BINDING $c1) $bl)]
| [ ] -> [$0 = ($LIST)]

with 
with_binding_list := 
  ["with" binding_list($l) ] -> [$0 = $l]
| [ ] -> [$0 = ($LIST)]

with
comarg_binding_list :=
  [ comarg($c) with_binding_list($l) ] -> [$0 = ($CONS $c $l)]

with
comarg_list :=
  [ comarg($c) comarg_list($l) ] -> [$0 = ($CONS $c $l)]
| [ ] -> [$0 = ($LIST)]

with
unfold_occ :=
  [ ne_num_list($nl) identarg($c) ] ->
	[$0 = ($OPER{UNFOLD} ($APPEND ($LIST $c) $nl))]
| [ identarg($c) ] -> [$0 = ($OPER{UNFOLD} ($LIST $c))]

with
ne_unfold_occ_list :=
  [ unfold_occ($p) ne_unfold_occ_list($l) ] -> [$0 = ($CONS $p $l)]
| [ unfold_occ($p) ] -> [$0 = ($LIST $p)]

with tactic :=
  [ tactic_com_list($tac) ] -> [$0 = $tac]

with
tactic_com_list :=
  [ tactic_com($y) ";" tactic_com_list_tail($l) ] -> [$0 = ($OPER{TACTICLIST} ($CONS $y $l))]
| [ tactic_com($y) ] -> [$0 = (TACTICLIST $y)]
.
Grammar tactic
tactic_com_list_tail :=
  [ tactic_com($t1) ";" tactic_com_list_tail($l) ] -> [$0 = ($CONS $t1 $l)]
| [ tactic_com($t1) ] -> [$0 = ($LIST $t1)]
| [ tactic_com_seq($seq) ";" tactic_com_list_tail($l) ] -> [$0 = ($CONS $seq $l)]
| [ tactic_com_seq($seq) ] -> [$0 = ($LIST $seq)]

with
tactic_com_seq :=
  [ "[" tactic_com_seq_body($l) "]" ] -> [$0 = ($OPER{TACLIST} $l)]

with
tactic_com_seq_body :=
  [ tactic_com_list($t) "|" tactic_com_seq_body($l) ] -> [$0 = ($CONS $t $l)]
| [ tactic_com_list($t) ] -> [$0 = ($LIST $t)]

with
tactic_com :=
  [ simple_tactic($st) "Orelse" tactic_com($tc) ] -> [$0 = (ORELSE $st $tc)]
| [ simple_tactic($st) ] -> [$0 = $st]

with
simple_tactic :=
  [ "Assumption" ] -> [$0 = (Assumption)]
| [ "Contradiction" ] -> [$0 = (Contradiction)]
| [ "Exact" comarg($c) ] -> [$0 = (Exact $c)]

| [ "Intros" "until" identarg($id) ] -> [$0 = (IntrosUntil $id)]
| [ "Intros" ne_identarg_list($idl) ] -> [$0 = ($OPER{IntrosWith} $idl)]
| [ "Intros" ] -> [$0 = (Intros)]
| [ "Intro" identarg($id)] -> [$0 = (IntrosWith $id)]
| [ "Intro" ] -> [$0 = (Intro)]

| [ "Red" "in" identarg($t) ] -> [$0 = (RedHyp $t)]
| [ "Red" ] -> [$0 = (Red)]

| [ "Simpl" "in" identarg($t) ] -> [$0 = (SimplHyp $t)]
| [ "Simpl" ] -> [$0 = (Simpl)]

| [ "Change" comarg($c) "in" identarg($t) ] -> [$0 = (ChangeHyp $c $t)]
| [ "Change" comarg($c) ] -> [$0 = (Change $c)]

| [ "Pattern" ne_pattern_list($pl) ] -> [$0 = ($OPER{Pattern} $pl)]

| [ "Apply" comarg_binding_list($cl) ] -> [$0 = ($OPER{ApplyWithBindings} $cl)]

| [ "Simple" "Discriminate" ] -> [$0 = (Discriminate)]

| [ "Injection" identarg($c)  ne_num_list($l) ] ->
	 [$0 = ($OPER{Injection} ($CONS $c $l))]

| [ "Elim"  comarg_binding_list($cl) "using" comarg_binding_list($el) ] 
	-> [$0 = ($OPER{ElimWithUsing} ($APPEND $cl $el))]

| [ "Elim" comarg_binding_list($cl) ] ->
	[$0 = ($OPER{ElimWithBindings} $cl)]

| [ "OldElim" comarg($c) ] -> [$0 = (Elim1 $c)]

| [ "ElimType" comarg($c) ] -> [$0 = (ElimType $c)]

| [ "Case" comarg_binding_list($cl) ] -> [$0 = ($OPER{CaseWithBindings} $cl)]

| [ "CaseType" comarg($c) ] -> [$0 = (CaseType $c)]

| [ "Induction" identarg($s) ] -> [$0 = (Induction $s)]
| [ "Induction" numarg($n) ] -> [$0 = (Induction $n)]

| [ "Destruct" identarg($s) ] -> [$0 = (Destruct $s)]
| [ "Destruct" numarg($n) ] -> [$0 = (Destruct $n)]

| [ "Cut" comarg($c)] -> [$0 = (Cut $c)]

| [ "Specialize" numarg($n) lcomarg($c) "with" binding_list($l) ] ->
	[$0 = ($OPER{SpecializeWithNum} ($CONS $n ($CONS $c $l)))]

| [ "Specialize" lcomarg($c) "with" binding_list($l) ] ->
	[$0 = ($OPER{SpecializeWith} ($CONS $c $l))]

| [ "Specialize" lcomarg($c) ] ->
	[$0 = (Specialize $c)]

| [ "Generalize" comarg_list($lc) ] -> [$0 = ($OPER{Generalize} $lc)]

| [ "LApply" comarg($c)] -> [$0 = (ImpElim $c)]

| [ "Clear" ne_identarg_list($l) ] -> [$0 = ($OPER{Clear} $l)]

| [ "Unfold" ne_unfold_occ_list($ul) "in" identarg($t) ] ->
	 [$0 = ($OPER{UnfoldHyp} ($APPEND ($LIST $t) $ul))]

| [ "Unfold" ne_unfold_occ_list($ul) ] ->
	 [$0 = ($OPER{Unfold} $ul)]

| [ "Trivial" ] -> [$0 = (TRY (Trivial))]

| [ "Auto" numarg($n) ] -> [$0 = (Auto $n)]
| [ "Auto" ] -> [$0 = (Auto)]

| [ "Hnf" ] -> [$0 = (Hnf)]
| [ "Hnf" "in" identarg($t) ] -> [$0 = (HnfHyp $t)]

| [ "Do" numarg($n) tactic_com($tc) ] -> [$0 = (DO $n $tc)]

| [ "Try" tactic_com($tc) ] -> [$0 = (TRY $tc)]

| [ "Repeat" tactic_com($tc) ] -> [$0 = (REPEAT $tc)]

| [ "Left" binding_list($bl)] -> [$0 = ($OPER{Left} $bl)]

| [ "Right" binding_list($bl)] -> [$0 = ($OPER{Right} $bl)]

| [ "Split" binding_list($bl)] -> [$0 = ($OPER{Split} $bl)]

| [ "Exists" binding_list($bl) ] -> [$0 = ($OPER{Exists} $bl)]

| [ "Constructor" numarg($n) with_binding_list($bl) ] ->
	 [$0 = ($OPER{ConstructorWithBindings} ($CONS $n $bl))]

| [ "Replace" comarg($c1) "with" comarg($c2) ] -> [$0 = (Replace $c1 $c2)]

| [ "Rewrite" "->" comarg($c) ] -> [$0 = (RewriteLR $c)]
| [ "Rewrite" "<-" comarg($c) ] -> [$0 = (RewriteRL $c)]
| [ "Rewrite" comarg($c) ] ->      [$0 = (RewriteLR $c)]

| [ "Reflexivity" ] -> [$0 = (Reflexivity)]
| [ "Symmetry" ] -> [$0 = (Symmetry)]
| [ "Transitivity" comarg($c) ] -> [$0 = (Transitivity $c)]

| [ "Absurd" comarg($c) ] -> [$0 = (Absurd $c)]

| [ "Idtac" ] -> [$0 = (IDTAC)]

| [ "(" tactic_com_list($tcl) ")" ] -> [$0 = $tcl]
.
