(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Vernac.v                                 *)
(****************************************************************************)

Grammar vernac
identarg :=
  [ prim:ident($id) ] -> [$0 = ($VAR $id)]

with
ne_identarg_list :=
  [ identarg($id) ne_identarg_list($idl) ] -> [$0 = ($CONS $id $idl)]
| [ identarg($id) ] -> [$0 = ($LIST $id)]

with
ne_identarg_comma_list :=
  [ identarg($id) "," ne_identarg_comma_list($idl) ] -> [$0 = ($CONS $id $idl)]
| [ identarg($id) ] -> [$0 = ($LIST $id)]


with
numarg :=
  [ prim:number($n) ] -> [$0 = $n]


with
ne_numarg_list :=
  [ numarg($n) ne_numarg_list($l) ] -> [$0 = ($CONS $n $l)]
| [ numarg($n) ] -> [$0 = ($LIST $n)]

with
numarg_list :=
  [ ne_numarg_list($l) ] -> [$0 = $l]
| [ ] -> [$0 = ($LIST)]

with
stringarg :=
  [ prim:string($s) ] -> [$0 = $s]


with
comarg :=
  [ command:command($c) ] -> [$0 = (COMMAND $c)]


with
lcomarg :=
  [ command:lcommand($c) ] -> [$0 = (COMMAND $c)]


with
tacarg :=
  [ tactic:tactic_com_list($tcl) ] -> [$0 = $tcl]


with
lvernac :=
  [ vernac($v) lvernac($l) ] -> [$0 = ($CONS $v $l)]
| [ ] -> [$0 = ($LIST)]


with
unparg :=
  [ prim:unparsing($u) ] -> [$0 = $u]

with
grammar_entry_arg :=
  [ prim:grammar_entry($u) ] -> [$0 = $u]

with
spatarg :=
  [ prim:spat($u) ] -> [$0 = $u]


with
spat_binder :=
  [ identarg($id) ":=" spatarg($c) ]
  -> [$0 = (SPAT_BINDER $id $c)]


with
ne_spat_binder_list :=
  [ spat_binder($b) ne_spat_binder_list($l) ] -> [$0 = ($CONS $b $l)]
| [ spat_binder($b) ] -> [$0 = ($LIST $b)]

with
theorem_body_line :=
  [ numarg($n) ":" tacarg($tac) "." ]
  -> [$0 = (VERNACCALL {SOLVE} $n (TACTIC $tac))]
| [ tacarg($tac) "." ]
  -> [$0 = (VERNACCALL {SOLVE} {1} (TACTIC $tac))]
| [ "BY" tacarg($tac) "." ]
  -> [$0 = (VERNACCALL {BY} (TACTIC $tac))]

with
theorem_body_line_list :=
  [ theorem_body_line($t) ] -> [$0 = ($LIST $t)]
| [ theorem_body_line($t) theorem_body_line_list($l) ] -> [$0 = ($CONS $t $l)]

with
theorem_body :=
  [ theorem_body_line_list($l) ] -> [$0 = ($OPER{VERNACARGLIST} $l)]

with
vernac :=
  [ "Pwd" "." ] -> [$0 = (PWD)]
| [ "Quit" "." ] -> [$0 = (QUIT)]
| [ "Drop" "." ] -> [$0 = (DROP)]
| [ "Cd" "." ] -> [$0 = (CD)]
| [ "Cd" stringarg($dir) "." ] -> [$0 = (CD $dir)]
| [ "Goal" comarg($c) "." ] -> [$0 = (GOAL $c)]
| [ "Goal" "." ] -> [$0 = (GOAL)]
| [ "Proof" "." ] -> [$0 = (GOAL)]
| [ "Abort" "." ] -> [$0 = (ABORT)]
| [ "Abort" "All" "." ] -> [$0 = (ABORTALL)]
| [ "Abort" identarg($id) "." ] -> [$0 = (ABORT $id)]
| [ "Restart" "." ] -> [$0 = (RESTART)]
| [ "Proof" comarg($c) "." ] -> [$0 = (PROOF $c)]
| [ "Focus" "." ] -> [$0 = (FOCUS)]
| [ "Unfocus" "." ] -> [$0 = (UNFOCUS)]
| [ "Save" "." ] -> [$0 = (SaveNamed)]
| [ "Qed" "." ] -> [$0 = (SaveNamed)]
| [ "Defined" "."] -> [$0=(DefinedNamed)]
| [ "Save" "Remark" identarg($id) "." ] -> [$0 = (SaveAnonymousRmk $id)]
| [ "Save" "Theorem" identarg($id) "." ] -> [$0 = (SaveAnonymousThm $id)]
| [ "Save" "State" identarg($id) "." ] -> [$0 = (SaveState $id {""})]
| [ "Save" "State" identarg($id) stringarg($s) "." ] -> [$0 = (SaveState $id $s)]
| [ "Save" identarg($id) "." ] -> [$0 = (SaveAnonymousThm $id)]
| [ "Transparent" identarg($id) "." ] -> [$0 = (TRANSPARENT $id)]
| [ "Opaque" identarg($id) "." ] -> [$0 = (OPAQUE $id)]
| [ "Write" "States" identarg($id) "." ] -> [$0 = (WriteStates $id)]
| [ "Write" "States" stringarg($id) "." ] -> [$0 = (WriteStates $id)]
| [ "Write" "Module" identarg($id) "." ] -> [$0 = (WriteModule $id)]
| [ "Write" "Module" identarg($id) stringarg($file) "." ]
    -> [$0 = (WriteModuleTo $id $file)]
| [ "Remove" "State" identarg($id) "." ] -> [$0 = (RemoveState $id)]
| [ "Undo" "." ] -> [$0 = (UNDO {1})]
| [ "Undo" numarg($n) "." ] -> [$0 = (UNDO $n)]
| [ "Set" "Undo" numarg($n) "." ] -> [$0 = (SETUNDO $n)]
| [ "Unset" "Undo" "." ] -> [$0 = (UNSETUNDO)]
| [ "Set" "Hyps_limit" numarg($n) "." ] -> [$0 = (SETHYPSLIMIT $n)]
| [ "Unset" "Hyps_limit" "." ] -> [$0 = (UNSETHYPSLIMIT)]
| [ "Show" "." ] -> [$0 = (SHOW)]
| [ "Show" "Node" "." ] -> [$0 = (ShowNode)]
| [ "Show" "Script" "." ] -> [$0 = (ShowScript)]
| [ "Explain" "Proof" numarg_list($l) "." ] -> [$0 = ($OPER{ExplainProof} $l)]
| [ "Explain" "Proof" "Tree" numarg_list($l) "." ] -> [$0 = ($OPER{ExplainProofTree} $l)]
| [ "Go" numarg($n) "." ] -> [$0 = (Go $n)]
| [ "Go" "top" "." ] -> [$0 = (Go {"top"})]
| [ "Go" "prev" "." ] -> [$0 = (Go {"prev"})]
| [ "Go" "next" "." ] -> [$0 = (Go {"next"})]
| [ "Show" numarg($n) "." ] -> [$0 = (SHOW $n)]
| [ "Show" "Proof" "." ] -> [$0 = (ShowProof)]
| [ "Guarded"   "." ] -> [$0 = (CheckGuard)]
| [ "Show" "Tree" "." ] -> [$0 = (ShowTree)]
| [ "Show" "Proofs" "." ] -> [$0 = (ShowProofs)]
| [ "Print" "All" "." ] -> [$0 = (PrintAll)]
| [ "Print" "." ] -> [$0 = (PRINT)]
| [ "Print" "Hint" "." ] -> [$0 = (PrintHint)]
(* | [ "Print" "Hint" "Goal" "." ] -> [$0 = (PrintHintGoal)] *)
| [ "Print" "Hint" identarg($s) "." ] -> [$0 = (PrintHintId $s)]
| [ "Print" "Section" identarg($s) "." ] -> [$0 = (PrintSec $s)]
| [ "Print" "States" "." ] -> [$0 = (PrintStates)]
| [ "Print" "Grammar" identarg($whatfor) identarg($id) "." ] -> [$0 = (PrintGrammar $whatfor $id)]


(*====================
LateX Output
Claude
=====================*)

| [ "Print" "LaTeX" "Grammar" identarg($whatfor) identarg($id) "." ] -> [$0 = (PrintLaTeXGrammarRules $whatfor $id)]
| [ "Print" "LaTeX" "Grammar" identarg($whatfor) "." ] -> [$0 = (PrintLaTeXGrammar $whatfor)]
(*====================*)

| [ "Print" "LoadPath" "." ] -> [$0 = (PrintPath)]
| [ "AddPath" stringarg($dir) "." ] -> [$0 = (ADDPATH $dir)]
| [ "DelPath" stringarg($dir) "." ] -> [$0 = (DELPATH $dir)]
| [ "Print" "Modules" "." ] -> [$0 = (PrintModules)]
| [ "Print" "Proof" identarg($id) "." ] -> [$0 = (PrintOpaqueId $id)]
| [ "Print" identarg($id) "." ] -> [$0 = (PrintId $id)]

(* RequireSpec does not exists any longer... at least for the moment *)
| [ "Require" "Import" identarg($id) "." ]
   -> [$0 = ((*RequireImportSpec*)RequireImport $id)]

| [ "Require" "Import" identarg($id) stringarg($file) "." ]
   -> [$0 = ((*RequireImportSpecFrom*)RequireImportFrom $id $file)]

| [ "Require" "Export" identarg($id) "." ]
   -> [$0 = ((*RequireExportSpec*)RequireExport $id)]

| [ "Require" "Export" identarg($id) stringarg($file) "." ]
   -> [$0 = ((*RequireExportSpecFrom*)RequireExportFrom $id $file)]

| [ "Require" identarg($id) "." ]
   -> [$0 = ((*RequireImportSpec*)RequireImport $id)]

| [ "Require" identarg($id) stringarg($file) "." ]
   -> [$0 = ((*RequireImportSpecFrom*)RequireImportFrom $id $file)]

| [ "Require" "Implementation" identarg($id) "." ]
   -> [$0 = (RequireImport $id)]

| [ "Require" "Implementation" identarg($id) stringarg($file) "." ]
   -> [$0 = (RequireImportFrom $id $file)]

| [ "Syntax" identarg($whatfor) identarg($s) spatarg($spat) unparg($unp)
    "with" ne_spat_binder_list($consl) "." ]
   -> [$0 = ($OPER{SYNTAX} ($CONS $whatfor ($CONS $s ($CONS $spat ($CONS $unp $consl)))))]
| [ "Syntax" identarg($whatfor) identarg($s) spatarg($spat) unparg($unp) "." ]
   -> [$0 = (SYNTAX $whatfor $s $spat $unp)]
| [ "Grammar" grammar_entry_arg($ge) "." ] -> [$0 = (GRAMMAR $ge)]
| [ "Token" prim:string($s) "." ] -> [$0 = (TOKEN $s)]
| [ "Infix" numarg($n) prim:string($op) identarg($p) "." ]
   -> [$0 = (INFIX $n $op $p)]
| [ "Hint" "Unfold" ne_identarg_list($l) "." ] -> [$0 = ($OPER{HintUnfold} $l)]
| [ "Hint" ne_identarg_list($l) "." ] -> [$0 = ($OPER{HintResolve} $l)]
| [ "Immediate" ne_identarg_list($l) "." ] -> [$0 = ($OPER{HintImmediate} $l)]
| [ "Section" identarg($id) "." ] -> [$0 = (BeginSection $id)]
| [ "Chapter" identarg($id) "." ] -> [$0 = (BeginSection $id)]
| [ "Module" identarg($id) "." ] -> [$0 = (BeginModule $id)]
| [ "Begin" "Silent" "." ] -> [$0 = (BeginSilent)]
| [ "End" "Silent" "." ] -> [$0 = (EndSilent)]
| [ "End" identarg($id) "." ] -> [$0 = (EndSection $id)]
| [ "Suspend" "." ] -> [$0 = (SUSPEND)]
| [ "Resume" "." ] -> [$0 = (RESUME)]
| [ "Resume" identarg($id) "." ] -> [$0 = (RESUME $id)]

| [ "Extraction" identarg($id) "." ] -> [$0 = (PrintExtractId $id)]
| [ "Extraction" "." ] -> [$0 = (PrintExtract)]
| [ "Info" identarg($id) "." ] -> [$0 = (INFO $id)]
| [ "Search" identarg($id) "." ] -> [$0 = (SEARCH $id)]
| [ "Inspect" numarg($n) "." ] -> [$0 = (INSPECT $n)]

| [ "Syntactic" "Definition" identarg($id) ":=" comarg($com) "." ]
  -> [$0 = (SyntaxMacro $id $com)]

| [ "Syntactic" "Definition" identarg($id) ":=" comarg($com) "|" numarg($n) "." ]
  -> [$0 = (SyntaxMacro $id $com $n)]

| [ "Abstraction" identarg($id) "[" ne_numarg_list($l) "]" ":=" comarg($c) "." ]
  -> [$0 = ($OPER{ABSTRACTION} ($CONS $id ($CONS $c $l)))]

| [ thm_tok($thm) identarg($id) ":" comarg($c) "." ]
  -> [$0 = (StartProof $thm $id $c)]

| [ thm_tok($thm) identarg($id) ":" comarg($c) ":=" "Proof" theorem_body($tb) "Qed" "." ]
  -> [$0 = (TheoremProof $thm $id $c $tb)]

| [ def_tok($def) identarg($s) ":" command:command($c1) "." ]
  -> [$0 = (StartProof $def $s (COMMAND $c1))]

| [ def_tok($def) identarg($s) ":=" command:command($c1) "." ]
  -> [$0 = (DEFINITION $def $s (COMMAND $c1))]

| [ def_tok($def) identarg($s) ":=" command:command($c1) ":" command:command($c2) "." ]
  -> [$0 = (DEFINITION $def $s (COMMAND (CAST $c1 $c2)))]

| [ def_tok($def) identarg($s) ":" command:command($c1) ":=" command:command($c2) "." ]
  -> [$0 = (DEFINITION $def $s (COMMAND (CAST $c2 $c1)))]

| [ def_tok($def) identarg($s) comarg($c) "." ]
  -> [$0 = (DEFINITION $def $s $c)]

| [ hyp_tok($hyp) binder($b) "." ]
  -> [$0 = (VARIABLE $hyp $b)]

| [ param_tok($hyp) binder($b) "." ]
  -> [$0 = (PARAMETER $hyp $b)]

| [ numarg($n) ":" tacarg($tac) "." ] -> [$0 = (SOLVE $n (TACTIC $tac))]
| [ tacarg($tac) "." ] -> [$0 = (SOLVE {1} (TACTIC $tac))]
| [ "BY" tacarg($tac) "." ] -> [$0 = (BY (TACTIC $tac))]

| [ eval_tok($check) comarg($c) "." ]
  -> [$0 = (CHECK $check $c)]

(* Parsing inductive definitions *)

|  [ finite_tok($f) "Set"
    identarg($id) indpar($indpar) ":=" lidcom($lidcom) "." ]
  -> [$0 = (ONEINDUCTIVE $f $id (COMMAND (PROP{Pos})) $indpar $lidcom)]

| [ finite_tok($f) "Type"
    identarg($id) indpar($indpar) ":=" lidcom($lidcom) "." ]
  -> [$0 = (ONEINDUCTIVE $f $id (COMMAND (TYPE{Null})) $indpar $lidcom)]

| [ finite_tok($f) "Prop"
    identarg($id) indpar($indpar) ":=" lidcom($lidcom) "." ]
  -> [$0 = (ONEINDUCTIVE $f $id (COMMAND (PROP{Null})) $indpar $lidcom)]

| [  finite_tok($f) "Typeset"
    identarg($id) indpar($indpar) ":=" lidcom($lidcom) "." ]
  -> [$0 = (ONEINDUCTIVE $f $id (COMMAND (TYPE{Pos})) $indpar $lidcom)]

| [  finite_tok($f) identarg($id) indpar($indpar) ":" comarg($c) 
    ":=" lidcom($lidcom) "." ]
  -> [$0 = (ONEINDUCTIVE $f $id $c $indpar $lidcom)] 

| [ "Mutual" indpar($indpar) "Inductive" block($indl) extracoindblock($coindl) "."]
  -> [$0 = (MUTUALINDUCTIVE $indpar 
	                   ($OPER{VERNACARGLIST} $indl) 
	                   ($OPER{VERNACARGLIST} $coindl))]

| [ "Mutual" indpar($indpar) "CoInductive" block($coindl) extraindblock($indl) "."]
  -> [$0 = (MUTUALINDUCTIVE $indpar 
	                   ($OPER{VERNACARGLIST} $indl) 
	                   ($OPER{VERNACARGLIST} $coindl))]

| [ "Fixpoint" specifrec($recs) "."]
  -> [$0 = (MUTUALRECURSIVE ($OPER{VERNACARGLIST} $recs))]

| [ "CoFixpoint" specifcorec($corecs) "."]
  -> [$0 = (MUTUALCORECURSIVE ($OPER{VERNACARGLIST} $corecs))]

| [ "Scheme" specifscheme($schemes) "."]
  -> [$0 = (INDUCTIONSCHEME ($OPER{VERNACARGLIST} $schemes))]

with
finite_tok := 
   [ "Inductive" ]   -> [$0 = "Inductive"]
 | [ "CoInductive" ] -> [$0 = "CoInductive"]

with
extraindblock := 
  [ "Inductive" block($indl) ]-> [$0 = $indl]
| [] -> [$0 = ($LIST)]

with 
extracoindblock := 
  [ "CoInductive" block($indl) ]-> [$0 = $indl]
| [] -> [$0 = ($LIST)]

with
block := 
  [ oneind($ind) "with" block($indl) ] -> [$0 = ($CONS $ind $indl)]
| [ oneind($ind) ] -> [$0 = ($LIST $ind)]

with 
oneind := 
  [identarg($id) ":" comarg($c) ":=" lidcom($lidcom)] -> 
      [$0 = ($OPER{VERNACARGLIST} ($CONS $id ($CONS $c ($LIST $lidcom))))]

with 
onerec := 
    [identarg($id) "[" ne_binder_semi_list($idl) "]" ":" comarg($c) ":=" 
     comarg($def)] -> 
      [$0 = ($OPER{VERNACARGLIST} 
        ($CONS $id ($CONS ($OPER{BINDERLIST} $idl) ($CONS $c ($LIST $def)))))]

with 
specifrec := 
   [onerec($rec) "with" specifrec($recl)] -> [$0 = ($CONS $rec $recl)]
 | [onerec($rec)] -> [$0 = ($LIST $rec)]

with onecorec := 
    [identarg($id)  ":" comarg($c) ":=" comarg($def)] -> 
      [$0 = ($OPER{VERNACARGLIST} ($CONS $id  ($CONS $c ($LIST $def))))]

with specifcorec := 
   [onecorec($corec) "with" specifcorec($corecl)] -> 
                         [$0 =($CONS $corec $corecl)]
 | [onecorec($corec)] -> [$0 = ($LIST $corec)]

with onescheme :=  
 [identarg($id) ":=" dep($dep) comarg($c) "Sort" sortdef($s)] ->
  [$0 = ($OPER{VERNACARGLIST} 
         ($CONS $id ($CONS $dep ($CONS $c ($LIST (COMMAND $s))))))]

with specifscheme :=  
   [onescheme($rec) "with" specifscheme($recl)] -> [$0 = ($CONS $rec $recl)]
 | [onescheme($rec)] -> [$0 = ($LIST $rec)]

with dep := 
 ["Dependent" "Elimination" "of"] -> [$0="DEP"]
 |["Non" "Dependent" "Elimination" "of"] -> [$0="NODEP"]
 |["Induction" "for"] -> [$0="DEP"]
 |["Minimality" "for"] -> [$0="NODEP"]

with
ne_binder_semi_list :=
  [ binder($id) ";" ne_binder_semi_list($idl) ] -> [$0 = ($CONS $id $idl)]
| [ binder($id) ] -> [$0 = ($LIST $id)]

with
indpar :=
  [ "[" ne_binder_semi_list($bl) "]" ] -> [$0 = ($OPER{BINDERLIST} $bl)]
| [ ] -> [$0 = (BINDERLIST)]

with
sortdef :=
  [ "Set" ] -> [$0 = (PROP{Pos})]
| [ "Prop" ] -> [$0 = (PROP{Null})]
| [ "Type" ] -> [$0 = (TYPE{Null})]
| [ "Typeset" ] -> [$0 = (TYPE{Pos})]


with
thm_tok :=
  ["Theorem"] -> [$0 = {"THEOREM"}]
| ["Lemma"] -> [$0 = {"LEMMA"}]
| ["Fact"] -> [$0 = {"FACT"}]
| ["Remark"] -> [$0 = {"REMARK"}]


with
def_tok :=
  [ "Definition" ] -> [$0 = {"DEFINITION"}]
| [ "Let" ] -> [$0 = {"LET"}]
| [ "Local" ] -> [$0 = {"LOCAL"}]


with
hyp_tok :=
  [ "Hypothesis" ] -> [$0 = {"HYPOTHESIS"}]
| [ "Variable" ] -> [$0 = {"VARIABLE"}]
| [ "Variables" ] -> [$0 = {"VARIABLES"}]

with
param_tok :=
  [ "Axiom" ] -> [$0 = {"AXIOM"}]
| [ "Parameter" ] -> [$0 = {"PARAMETER"}]
| [ "Parameters" ] -> [$0 = {"PARAMETERS"}]

with
eval_tok :=
  ["Eval"] -> [$0 = {"EVAL"}]
| ["Compute"] -> [$0 = {"COMPUTE"}]
| ["Simplify"] -> [$0 = {"SIMPLIFY"}]
| ["Check"] -> [$0 = {"CHECK"}]
| ["Type"] -> [$0 = {"PRINTTYPE"}]


with
binder :=
  [ ne_identarg_comma_list($idl) ":" command:command($c) ]
  -> [$0 = ($OPER{BINDER} ($CONS $c $idl))]

with
idcom :=
  [ prim:ident($id) ":" command:command($c) ] -> [$0 = (BINDER $c ($VAR $id))]


with
ne_lidcom :=
  [ idcom($idc) "|" ne_lidcom($l) ] -> [$0 = ($CONS $idc $l)]
| [ idcom($idc) ] -> [$0 = ($LIST $idc)]

with
lidcom :=
  [ ne_lidcom($l) ] -> [$0 = ($OPER{BINDERLIST} $l)]
| [] -> [$0 = (BINDERLIST)]
.

Grammar vernac ne_stringarg_list :=
  [ stringarg($n) ne_stringarg_list($l) ] -> [$0 = ($CONS $n $l)]
| [ stringarg($n) ] -> [$0 = ($LIST $n)]
.
 
Grammar vernac vernac :=
  [ "Print" "ML" "Path" "." ] -> [$0 = (PrintMLPath)]
| [ "Print" "ML" "Modules" "." ] -> [$0 = (PrintMLModules)]
| [ "Add" "ML" "Path" stringarg($dir) "." ]
  -> [$0 = (AddMLPath $dir)]
 
| [ "Declare" "ML" "Module" ne_stringarg_list($l) "." ]
  -> [$0 = ($OPER{DeclareMLModule} $l)]
 
| [ "Import" "ML" "Module" stringarg($fname) "." ]
  -> [$0 = (ImportMLModule $fname)]
 
| [ "Require" "ML" "Module" ne_stringarg_list($l) "." ]
  -> [$0 = ($OPER{RequireMLModule} $l)]
.
 
Grammar tactic simple_tactic :=
  [ "ML" prim:string($s) ] -> [$0 = (MLTACTIC {$s})]
.
