(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 auto.ml                                  *)
(****************************************************************************)

#infix "o";;
#open "std";;
#open "more_util";;
#open "names";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "himsg";;
#open "machops";;
#open "mach";;
#open "proof";;
#open "proof_trees";;
#open "pfedit";;
#open "tacmach";;
#open "tactics";;
#open "pp";;
#open "stdpp";;
#open "clenv";;
#open "termenv";;
#open "tactics";;
#open "reintern0";;
#open "reintern1";;
#open "libobject";;
#open "library";;
#open "vernacinterp";;


(****************************************************************************)
(*            The Type of Constructions Autotactic Hints                    *)
(****************************************************************************)

let pri_ord {pri=pri1;_} {pri=pri2;_} = pri1 - pri2;;

let insert v l = insrec l
    where rec insrec = function
    [] -> [v]
  | h::tl -> if pri_ord v h <= 0 then v::h::tl
             else h::(insrec tl)
;;

(****************************************************************)
(* Essai d'automatisation, on range les tactiques de resolution *)
(*  suivant leur constante de tete et une priorite.         *) 
(****************************************************************)

(*
type search_entry == pri_auto_tactic list;;

let add_tac t l = insert t l;;
let mt_se = [];;
let lookup_tacs (hdc,c) l = l;;
*)

type stored_data == pri_auto_tactic;;

type search_entry == stored_data list * stored_data list * stored_data btermdn__t;;

let mt_se = ([],[],btermdn__new ());;

let add_tac t (l,l',dn) =
    match t.pat with
    NONE -> (insert t l,l',dn)
  | SOME pat -> (l,t::l',btermdn__add dn (pat,t))
;;

let lookup_tacs (hdc,c) (l,l',dn) =
    sort (fun (t1,t2) -> (pri_ord t1 t2 < 0)) (l@(list__map snd(btermdn__lookup dn c)))
;;

type frozen_t == (constr,search_entry) mhs__frozen_t;; 

let searchtable = (mhs__new (17,NONE) : (constr, search_entry) mhs__t);;

let map_all k =
let (l,l',_) = (try hd (mhs__map searchtable k) with (Not_found|Failure _) -> mt_se)
in sort (fun (t1,t2) -> (pri_ord t1 t2 < 0)) (l@l')
;;

let map (k,c) =
let l = lookup_tacs (k,c) (try hd (mhs__map searchtable k) with (Not_found|Failure _) -> mt_se)
in l
;;

let init () = (mhs__empty searchtable);;

let freeze () = (mhs__freeze searchtable);;

let unfreeze fs = (mhs__unfreeze fs searchtable);;

let add_one (k,v) =
    let oval = try hd(mhs__map searchtable k) with (Not_found|Failure _) -> mt_se
    in mhs__add searchtable (k,add_tac v oval)
;;

let push_one (k,v) =
    let oval = try hd(mhs__map searchtable k) with (Not_found|Failure _) -> mt_se
    in mhs__push searchtable (k,add_tac v oval)
;;

let add l = (do_list add_one l);;

let push (k,v) = (push_one (k,v));;

let pop k = (mhs__pop searchtable (SOME k));;

let app f = (mhs__app (fun (k,[]) -> () | (k,(l,l',_)::_) -> f(k,(l@l'))) searchtable);;

library__declare_summary "search"
{summary__freeze_function = freeze;
 summary__unfreeze_function = unfreeze;
 summary__init_function = init;
 summary__sync_function = (fun () -> false)}
;;


let fmt_autotactic = function
    Res_pf (c,clenv) -> [< 'S"Apply "; pTERM c >]
  | Give_exact c -> [< 'S"Exact " ; pTERM c >]
  | Res_pf_THEN_trivial_fail (c,clenv) -> [< 'S"Apply "; pTERM c ; 'S" ; Trivial" >]
  | Unfold_nth sp -> [< 'S"Unfold " ; 'S(nametab__print_name sp) >]
;;
let fmt_namedtac = fun
    headconstr ntlist ->
    [< 'S"For " ; pTERM headconstr ; 'S" -> " ;
                      (HOV 0 [<
                       prlist (function v ->
                               [< fmt_autotactic v.code ; 'S"(" ; 'INT v.pri;
                                'S")"; 'SPC >]) ntlist >]) ;
                       'FNL >]
;;

let print_search () = app (fun (a,b) -> MSG(fmt_namedtac a b));;

let fmt_hint_constr constr = 
  try let v = mhs__map searchtable constr in match v with 
                [] -> [<'S "No hint declared for :"; pTERM constr >]
              | ((l,l',_)::_) -> fmt_namedtac constr (l@l')
  with (Not_found|Failure _) -> [<'S "No hint declared for :"; pTERM constr >];;

let fmt_hint_id id = let sign = GLOB (initial_sign()) in 
try let c = search_reference sign id in fmt_hint_constr c
with Not_found -> [<print_id id; 'S " not declared" >];;

let print_hint_id id =  PPNL(fmt_hint_id id);;


let fmt_hint_term cl = 
  try let hdc = head_constr_bound cl in
  let tacl =
    if occur_existential cl then (map_all hdc)
    else (map (hdc,cl))
  in match tacl with 
                [] -> [<'S "No hint applicable for current goal" >]
              | l -> [< 'S "Applicable Hints :"; (HOV 0 (prlist 
                            (function v ->
                               [< fmt_autotactic v.code ; 'S"(" ; 'INT v.pri;
                                'S")"; 'FNL >]) tacl)) >]
  with (BOUND|Failure _) -> [<'S "No hint applicable for current goal" >];;

let print_hint_term cl = PPNL (fmt_hint_term cl);;
let print_applicable_hint () = let pts = get_pftreestate () in 
    let pf = proof_of_pftreestate pts in  print_hint_term pf.goal.concl;;

#infix "ORELSE";;
#infix "NEXT";;
#infix "THEN";;
#infix "THENF";;
#infix "THENL";;
#infix "THENS";;

let ri_freelisted ri_rebus fl =
    {rebus = ri_rebus fl.rebus;
     freemetas = fl.freemetas};;

let ri_cfl = ri_freelisted ri_constr;;

let ri_clbinding = function
    CLTYP cfl -> CLTYP(ri_cfl cfl)
  | CLVAL(cfl,ctyfl) -> CLVAL(ri_cfl cfl,ri_cfl ctyfl)
;;

let ri_clenv (ce : unit clausenv) =
    {templval = ri_cfl ce.templval;
     templtyp = ri_cfl ce.templtyp;
     namenv = ri_intavm ri_id ce.namenv;
     env = ri_intavm ri_clbinding ce.env;
     hook = ()}
;;

let ri_auto_tactic = function
    Res_pf(c,ce) -> Res_pf(ri_constr c,ri_clenv ce)
  | Give_exact c -> Give_exact (ri_constr c)
  | Res_pf_THEN_trivial_fail(c,ce) ->
    Res_pf_THEN_trivial_fail(ri_constr c,ri_clenv ce)
  | Unfold_nth sp -> Unfold_nth(ri_sp sp)
;;

let ri_pri_auto_tactic pat =
    {pri = pat.pri;
     pat = option_app ri_constr pat.pat;
     code = ri_auto_tactic pat.code}
;;

let cache_autohint (_,ntl) =
    (try add ntl
    with _ -> anomalylabstrm "add_auto"
              [< 'S"The code which adds auto-tactic hints broke;" ;
                 'SPC ; 'S"this is never supposed to happen" >])
;;

let reintern_autohint l =
    (list__map (fun (c,pat) -> (ri_constr c,ri_pri_auto_tactic pat)) l)
;;

let specification_autohint x = x;;

let (inAutoHint,outAutoHint) =
    declare_object ("AUTOHINT",
                    {cache_function = cache_autohint;
                     reintern_function = reintern_autohint;
                     specification_function = specification_autohint});;

(*********************************************)
(* Automatic tactics                         *)
(*********************************************)

let rec nb_hyp = function
     DOP2(Prod,_,DLAM(_,c2)) -> if dependent (Rel 1) c2 then nb_hyp c2 else 1+(nb_hyp c2)
   | _            -> 0 ;;

(* adding and removing tactics in the search table *)

(* A tactic which is Hint'ed will be:
   (1) used as an Exact, if it does not start with a product
   (2) used as an Apply, if its HNF starts with a product, and
       has no missing arguments.
 *)

let make_exact_entry (na,c,cty) =
    match strip_outer_cast cty with
    DOP2(Prod,_,_) -> failwith "exact"
  | cty -> (head_constr cty,{pri=0;pat=NONE;code=Give_exact c})
;;

let make_apply_entry (na,c,cty) =
    match hnf_constr mt_evc cty with

    DOP2(Prod,_,_) as cty ->
    let hdconstr = (try head_constr_bound cty with BOUND -> failwith "apply") in
    let ce = mk_clenv_from () (c,cty)
    in if null (clenv_missing ce (clenv_template ce,
                                  clenv_template_type ce)) then
        (hdconstr,
         {pri=nb_hyp cty;
          pat=SOME (clenv_template_type ce).rebus;
          code=Res_pf(c,ce)})

        else failwith "apply"

    | _      -> failwith "apply"
;;

let make_resolves (na,c,cty) =
let ents = map_succeed (fun f -> f(na,c,cty)) [make_exact_entry;make_apply_entry]
in if ents = [] then
    error (na^" cannot be used as a hint")
   else ents
;;

let add_resolves idlist =
    add_anonymous_object
    (inAutoHint
     (join(list__map (fun id -> 
               let c = global (GLOB(initial_sign())) id
               in make_resolves (string_of_id id,c,type_of mt_evc (initial_sign()) c)) idlist)))
;;

let make_unfold id =
let hdconstr = (try search_reference (GLOB(initial_sign())) id
                with Not_found -> 
                errorlabstrm  "make_unfold" [<print_id id; 'S " not declared" >])
in hdconstr,{pri=4;
             pat = NONE;
             code=Unfold_nth (nametab__sp_of_id CCI id)}
;;

let add_unfolds l =
    add_anonymous_object (inAutoHint(list__map make_unfold l));;

let make_sign_resolves l =
    join(map_succeed
    (function (str,t,tyt) ->
     try (make_resolves (string_of_id str,t,tyt))
          with UserError _ -> failwith "caught") l)
;;

let save_sign_resolves l =
    do_list push l
;;

let erase_sign_resolves l =
    do_list (pop o fst) (rev l)
;;

(* tactics with a trace mechanism for automatic search *)

let priority l = list__map snd (filter (fun (pr,_) -> pr = 0) l)
;;

let monitored_res_pf newl opatac KONT clenv gls =
let clenv' = (clenv_unify (clenv_instance_template_type clenv) (pf_concl gls) clenv) in
let r = clenv_refine KONT clenv' gls
in if not(mem opatac newl) then
    MSGNL [< 'S"WARNING: A successful auto-tactic which is not in the new-list" ; 'FNL ;
          'S"Goal: " ; pTERM (pf_concl gls) ; 'FNL ;
          'S"Tac : " ; fmt_autotactic opatac.code ; 'SPC ;
          pTERM(outSOME opatac.pat) >];
   r
;;

let rec trivial_fail_sign add_sign goal =
  let sign_resolves = make_sign_resolves (map_sign_graph (fun id a -> (id,VAR(id),a)) add_sign) in
  save_sign_resolves sign_resolves;
  try let res = FIRST (list__map COMPLETE
               (assumption::
               (intro
                THEN (function t'->
                                     (trivial_fail_sign (make_sign[hd_sign (pf_hyps t')]) t')))::
               (trivial_resolve (pf_concl goal)))) goal
  in erase_sign_resolves sign_resolves;res
  with reraise -> (erase_sign_resolves sign_resolves); raise(reraise)

and trivial_fail x = (trivial_fail_sign nil_sign) x

and my_find_search (hdc,concl) =
let tacl =
    if occur_existential concl then (map_all hdc)
    else (map (hdc,concl))
in list__map (fun ({pri=b;code=t;_} as patac) -> (b,
    match (t) with
      (Res_pf (_,c)) ->
       (fun gl -> let (wc,KONT) = StartWalk gl
                  in res_pf KONT (connect_clenv wc c)  gl)

    | (Give_exact c) -> (fun gl -> give_exact c gl)

    | (Res_pf_THEN_trivial_fail (_,c)) ->
      (fun gl -> let (wc,KONT) = StartWalk gl
                 in (res_pf KONT (connect_clenv wc c) THEN trivial_fail) gl)

    | (Unfold_nth str) -> unfold_nth [([],str)])) tacl

and trivial_resolve cl= 
   try priority (my_find_search (head_constr_bound cl,cl))
   with BOUND | Not_found -> []
;;

let possible_resolve gl =
    try list__map snd (my_find_search (head_constr_bound gl,gl))
    with BOUND | Not_found -> [];;

let rec search n add_sign t =
  if n=0 then error "BOUND 2"
  else let sign_resolves = make_sign_resolves (map_sign_graph (fun id a -> (id,VAR(id),a)) add_sign) in
       save_sign_resolves sign_resolves;
       try
       let res = FIRST
                 (assumption::
                  (intro THEN (function t' -> search n (make_sign [hd_sign (pf_hyps t')]) t'))
                  ::(list__map (fun ntac g -> (ntac THEN search (n-1) nil_sign) g)
                          (possible_resolve (pf_concl t)))) t
       in erase_sign_resolves sign_resolves; res
       with reraise -> erase_sign_resolves sign_resolves; raise(reraise);;

let search_auto n t = search n (pf_hyps t) t;;

let automatic n = (TRY (COMPLETE (search_auto n)));;

let SEARCH_DEPTH = ref(5);;

let auto =
let gentac = register_tactic "Auto"
    (fun [] -> automatic !SEARCH_DEPTH
       | [INTEGER n] -> automatic n)
    (fun _ _ (_,[]) -> [< 'S"Auto" >]
       | _ _ (_,[INTEGER n]) -> [< 'S"Auto" ; 'SPC ; 'INT n >])
in function NONE -> gentac [] | SOME n -> gentac [INTEGER n]
;;

let real_failing_trivial_tactic gl =
    trivial_fail_sign (pf_hyps gl) gl;;

let failing_trivial_tactic =
    register_atomic_tactic "Trivial" real_failing_trivial_tactic;;

let trivial t = (TRY failing_trivial_tactic) t;;

let make_trivial id =
let c = global (GLOB(initial_sign())) id in
let t = type_of mt_evc (initial_sign()) c in
let hdconstr = head_constr t in
let ce = mk_clenv_from () (c,hnf_constr mt_evc t)
in hdconstr,{pri=1;
             pat = SOME(clenv_template_type ce).rebus;
             code=Res_pf_THEN_trivial_fail(c,ce)}
;;


let add_trivials l =
    add_anonymous_object (inAutoHint(list__map make_trivial l))
;;


vinterp_add("PrintHintId",
    fun [(VARG_IDENTIFIER id)] -> fun () -> print_hint_id id);;


vinterp_add("HintUnfold",
    fun argl ->
        fun () ->
        let l = list__map (fun (VARG_IDENTIFIER id) -> id) argl
        in add_unfolds l);;

vinterp_add("HintResolve",
    fun argl ->
        fun () ->
        let l = list__map (fun (VARG_IDENTIFIER id) -> id) argl
        in add_resolves l);;

vinterp_add("HintImmediate",
    fun argl ->
        fun () ->
        let l = list__map (fun (VARG_IDENTIFIER id) -> id) argl
        in add_trivials l);;

vinterp_add("PrintHint",
    fun [] -> fun () -> print_search());;

vinterp_add("PrintHintGoal",
    fun [] -> fun () -> print_applicable_hint());;

(* $Id: auto.ml,v 1.5 1996/02/12 15:48:59 cpaulin Exp $ *)
