(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 discr.ml                                 *)
(****************************************************************************)

#infix "o";;
#open "std";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "ast";;
#open "term";;
#open "reduction";;
#open "typing";;
#open "termenv";;
#open "mach";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "tacmach";;
#open "proof";;
#open "proof_trees";;

(***
let pf_get_new_ids ids gls =
    list_it
    (fun id acc ->
         (next_ident_away id (acc@(ids_of_sign (pf_hyps gls))))::acc)
    ids []
;;
**)

let pf_acc_get_new_ids ids acc gls =
    list_it
    (fun id acc ->
         (next_ident_away id (acc@(ids_of_sign (pf_hyps gls))))::acc)
    ids acc
;;


let pf_get_new_ids ids gls = pf_acc_get_new_ids ids [] gls;;


exception constructor_not_found;;
let find_constructor sigma c =
    match whd_betadeltaiota_stack sigma c [] with
    DOPN(MutConstruct _,_) as hd,stack -> (hd,stack)
  | _ -> raise constructor_not_found
;;

(* T is (x1:A1)..(xn:An)u build [x1:A1]..[xn:An]endpt *)
let lambda_from_prod_end endpt = drec where rec drec T = 
  match strip_outer_cast T with 
    (DOP2(Prod,C,DLAM(n,T'))) -> (DOP2(Lambda,C,DLAM(n,drec T')))
   | x                        -> endpt;;


(* prod is (x1:A1)..(xn:An)u build [x1:A1]..[xn:An]endpt *)
let lambda_from_prod_end_named endpt prod gls  = drec [] prod 
where rec drec acc_ids prod  = 
  match strip_outer_cast prod with 
    (DOP2(Prod,ty,DLAM(n,prod'))) -> 
          if n<>Anonymous then (DOP2(Lambda,ty,DLAM(n,drec acc_ids prod')))
            else let (id::l)=pf_acc_get_new_ids [id_of_string "a"] acc_ids gls 
                 in (DOP2(Lambda,ty,DLAM(Name id,drec (id::acc_ids) prod' )))
   | x                        -> endpt;;

(* [lambda_ize n T endpt]
 * will pop off the first n products in T, then stick in endpt,
 * properly lifted, and then push back the products, but as lambda-
 * abstractions
 *)
let lambda_ize n T endpt =
let env = [] and carry = [insert_lifted endpt] in
let env,_,[endpt] = push_and_liftl n env T carry in
let T = extract_lifted endpt in
let _,T,[] = lam_and_popl n env T []
in T
;;

let make_case P c lf = DOPN(MutCase,vect_of_list (P::c::lf));;

(* [discriminate T t1 t2]
 * will produce a term of type ~(eq T t1 t2)
 *)
let make_discriminator T t1 t2 gls =
let sigma = Project gls and
    sign = pf_hyps gls in
let env = GLOB sign in 
let [e] = pf_get_new_ids [id_of_string "e"] gls in
let (case_function,type_constr,pararity,c1i,c2i,nparams,consnamev,largs) = 
match (find_constructor sigma t1,find_constructor sigma t2) with 
   (DOPN(MutConstruct(_,c1i),_),_),(DOPN(MutConstruct(_,c2i),_),_) -> 
    let (ity,largs) = find_mrectype sigma T in
    let mispec = mind_specif_of_mind ity in 
    let nparams = mis_nparams mispec and consnamev = mis_consnames mispec in
    (make_case,snd (mis_type_mconstructs mispec),
    mis_arity mispec,c1i,c2i,nparams,consnamev,largs) 
in 
let nconstructors = vect_length consnamev and
    (globargs,largs) = chop_list nparams largs in

let arity = hnf_prod_applist sigma "discriminate" pararity globargs and
    True = pf_constr_of_com gls (ASTnvar(id_of_string "True")) and
    False = pf_constr_of_com gls (ASTnvar(id_of_string "False")) and
    eq_ind = pf_constr_of_com gls (ASTnvar(id_of_string "eq_ind")) and
    eq = pf_constr_of_com gls (ASTnvar(id_of_string "eq")) and
    I = pf_constr_of_com gls (ASTnvar(id_of_string "I")) in
let P = lambda_from_prod_end (DOP0(Sort (Prop Null))) arity and
    c = (VAR e)
in
let build_branch i =
    let endpt = if i = c1i then True else False
    in lambda_from_prod_end endpt 
          (hnf_prod_applist sigma "discriminate" type_constr.(i-1) globargs) in

let build_match  =
    case_function P c (map build_branch (interval 1 nconstructors)) in
              
let pf = applist(eq_ind,[T;t1;lambda e T build_match;I;t2])

in Refine pf gls
;;
(********
let real_discriminate_tac gls =
let equation =
    (match pf_hnf_constr gls (pf_concl gls) with
     DOP2(Prod,c,DLAM(na,r)) -> c
   | _ -> error "discriminate") in
let (DOPN(AppL,cl)) = ensure_appl(collapse_appl equation) in
    match list_of_vect cl with
    [_;T;t1;t2] -> make_discriminator T t1 t2 gls
  | _ -> error "discriminate"
;;
***********)
let discr_error_msg = "Simple Discriminate should be applied to a pair of terms built with different constructors" ;;

let are_discriminable sigma t1 t2 = 
try (let c1,_ = find_constructor sigma t1 and
         c2,_ = find_constructor sigma t2
     in c1 <> c2)
with constructor_not_found -> error discr_error_msg
;; 

let real_discriminate_tac gls =
let equation =
    (match pf_hnf_constr gls (pf_concl gls) with
     DOP2(Prod,c,DLAM(na,r)) -> c
   | _ -> error discr_error_msg) in
let (DOPN(AppL,cl)) = ensure_appl(collapse_appl equation) in
    match list_of_vect cl with
    [_;T;t1;t2] ->  if are_discriminable (Project gls) t1 t2
                     then make_discriminator T t1 t2 gls
                     else error discr_error_msg
  | _ -> error discr_error_msg
;;

let rec whd_cast = function
    DOP2(Cast,c,_) -> whd_cast c
  | c -> c
;;

let rec range_of_prod = function
    DOP2(Prod,_,DLAM(_,c)) -> c
  | DOP2(Cast,c,_) -> range_of_prod c
  | _ -> failwith "range_of_prod"
;;


let discriminate_tac =
    register_atomic_tactic "Discriminate" real_discriminate_tac;;

let iterate_succeed f x = iterec x
    where rec iterec x =
    try iterec (f x)
    with Failure _ -> x
;;


(* [proj_function t j]
 * Assuming that t is of the form (MutConstruct(i,I) args),
 * proj_function will construct a term which, when applied to t,
 * j-th argument of t.
 *)

exception incorrect_path;;

let proj_function gls t j =
let sigma = Project gls in
let sign = pf_hyps gls in
let env = GLOB sign in 
let [e] = pf_get_new_ids [id_of_string "e"] gls in
let (case_fun,ci,pararity,nparams,consnamev,cargs,typeconstructs) = 
  (let ((DOPN(MutConstruct(sp,ci),cl) as c1),cargs) 
         = find_constructor sigma t in 
      if j <= length cargs then 
      let ity = DOPN(MutInd sp,cl) in
      let mispec = mind_specif_of_mind ity in 
      let nparams = mis_nparams mispec and consnamev = mis_consnames mispec in
      (make_case,ci,mis_arity mispec,nparams,consnamev,
        cargs,snd (mis_type_mconstructs mispec))
     else raise incorrect_path)
in  let (globargs,cargs) = chop_list nparams cargs in
    let T = pf_type_of gls t in
    let nconstructors = vect_length consnamev in
    let arity = hnf_prod_applist sigma "injection" pararity globargs and
    Ci = hnf_prod_applist sigma "injection" typeconstructs.(ci-1) globargs in
    let Tj = (match whd_cast (iterate range_of_prod (j-nparams-1) Ci) with
            DOP2(Prod,c,_) -> c
          | _ -> error "injection") in
          if not(closed Tj) then error "injection only works on non-dependent types";

    let P = lambda_from_prod_end Tj  arity in

    let c = VAR e in

    let m = nth cargs (j-nparams) in

    let build_branch k  =
        let endpt = if k <> ci then m else 
           let nlams = nb_prod typeconstructs.(k-1) in Rel(nlams-j+1)
           in lambda_from_prod_end_named endpt 
           (hnf_prod_applist sigma "injection" typeconstructs.(k-1) globargs) gls                  

    in let build_match () =
    case_fun P c (map build_branch (interval 1 nconstructors)) in

    let pf =  lambda e T (build_match ()) in (Tj,pf)
;;

let injection_step j c gls =
try(
let eq = pf_constr_of_com gls (ASTnvar (id_of_string"eq")) in
let f_equal = pf_constr_of_com gls (ASTnvar (id_of_string "f_equal")) in
let (DOPN(AppL,cl) as T) = ensure_appl(pf_hnf_constr gls (pf_type_of gls c)) in
if not(eq_constr(hd_vect cl,eq)) then error "can not perform injection in the specified hypothesis";
    match list_of_tl_vect cl with
    [T;t1;t2] ->
    let (resT,f) = proj_function gls t1 j in
    let pf = applist(f_equal,[T;resT;f;t1;t2;c])
    in pf
  | _ -> error "injection: a malformed equality")
with incorrect_path -> error "incorrect path"
;;


let injection l c gls = injrec l c
    where rec injrec = fun
    [] c -> c
  | (j::l) c -> injrec l (injection_step j c gls)
;;

let real_injection_tac l com gls =
let c = pf_constr_of_com gls com in
let pf = (injection l c gls) in
let ty = type_of (Project gls)  (pf_hyps gls) pf  
in if conv (Project gls) (pf_concl gls)  ty
   then Refine pf gls
   else error "the result of the injection does not correspond to the current subgoal"
;;

(***** Works with commands instead of identifiers

let vernac_injection_tac =
let gentac = register_tactic "Injection"
    (fun ((COMMAND c)::l) -> real_injection_tac (map (fun (INTEGER i) -> i) l) c)
    (fun sigma goal (_,(COMMAND c)::l) ->
         [< 'S"Injection"; 'SPC ; pr_com sigma goal c;
            prlist_with_sep (fun () -> [< 'SPC >])
                            (fun (INTEGER i) -> [< 'INT i >]) l >])
in fun l c -> gentac ((COMMAND c)::(map (fun i -> INTEGER i) l))
;;
***********)





let vernac_injection_tac  =
let gentac = register_tactic "Injection"
    (fun ((IDENTIFIER id)::l) -> real_injection_tac (map (fun (INTEGER i) -> i) l) (ASTnvar id))
    (fun sigma goal (_,(IDENTIFIER id)::l) ->
         [< 'S"Injection"; 'SPC ; print_id id;
            prlist_with_sep (fun () -> [< 'SPC >])
                            (fun (INTEGER i) -> [< 'INT i >]) l >])
in fun l c -> gentac ((IDENTIFIER c)::(map (fun i -> INTEGER i) l))
;;







(* $Id: discr.ml,v 1.6 1996/02/12 15:49:04 cpaulin Exp $ *)
