(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 fhdn.ml                                  *)
(****************************************************************************)
#open "std";;
#open "more_util";;
#open "vectops";;

let new dna = {tm = fthm__new(); args = dna};;

let path_of dna =
let rec path_of_deferred l =
    match l with
    [] -> []
  | h::tl -> pathrec tl h

and pathrec deferred t =

    match dna t with

    NONE -> NONE::(path_of_deferred deferred)

  | SOME (lbl,v) ->
    if length v = 0 then
        (SOME (lbl,0))::(path_of_deferred deferred)
    else let h = hd v and
             def_subl = tl v
         in (SOME (lbl,length v))::(pathrec (def_subl@deferred) h)

in pathrec []
;;

let tm_of tm lbl =
    try [fthm__map tm lbl]
    with Not_found -> []
;;

let lookup dnm dna' t =
let rec lookrec t tm =
    (tm_of tm NONE)@
    (match dna' t with
     NONE -> []
   | SOME(lbl,v) ->
     it_list (fun l c -> join(map (lookrec c) l))
             (tm_of tm (SOME(lbl,length v)))
             v)

in join(map fthm__xtract (lookrec t dnm.tm))
;;

let upd dnm f = f dnm.args dnm.tm;;

let add dnm (pat,inf) =
    upd dnm
    (fun dna tm ->
         let p = path_of dna pat
         in fthm__add tm (p,(pat,inf)))
;;

let rmv dnm (pat,inf) =
    upd dnm
    (fun dna tm ->
         let p = path_of dna pat
         in fthm__rmv tm (p,(pat,inf)))
;;

let freeze dnm = fthm__freeze dnm.tm;;
let unfreeze fm dnm = fthm__unfreeze fm dnm.tm;;

let empty dnm = fthm__empty dnm.tm;;

let app f dnm =
    fthm__app (fun (_,p) -> f p) dnm.tm
;;

(* $Id: fhdn.ml,v 1.2 1996/02/12 15:49:09 cpaulin Exp $ *)
