(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               nbtermdn.ml                                *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "more_util";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "himsg";;
#open "pp";;
#open "stdpp";;
#open "reintern0";;
#open "reintern1";;
#open "libobject";;
#open "library";;

(* Named, bounded-depth, term-discrimination nets *)

(* Implementation:

   Term-patterns are stored in discrimination-nets, which are
   themselves stored in a hash-table, indexed by the first label.

   They are also stored by name in a table on-the-side, so that we can
   override them if needed.

 *)


type ('na,'a) t ==
    ('na,constr * 'a) fmavm__t *
    (termdn__lbl option,'a btermdn__t) mhm__t
;;

type ('na,'a) frozen_t ==
    ('na,constr * 'a) fmavm__frozen_t *
    (termdn__lbl option,'a btermdn__t) mhm__frozen_t
;;

let new A =
    (fmavm__new(A.name_ord,17),
     mhm__new 17)
;;


let get_dn dnm hkey =
    try mhm__map dnm hkey
    with Not_found -> btermdn__new ()
;;

let add ((nm,dnm):('na,'a) t) (na,(pat,valu)) =
    let hkey = option_app fst (termdn__constr_pat_discr pat)
    in (fmavm__add nm (na,(pat,valu));
        mhm__remap dnm hkey (btermdn__add (get_dn dnm hkey) (pat,valu)))
;;

let rmv ((nm,dnm):('na,'a) t) na =
let (pat,valu) = fmavm__map nm na in
let hkey = option_app fst (termdn__constr_pat_discr pat)
in (fmavm__rmv nm na;
    mhm__remap dnm hkey (btermdn__rmv (get_dn dnm hkey) (pat,valu)))
;;

let in_dn (nm,dnm) na = fmavm__in_dom nm na;;

let remap (ndn:('na,'a) t) na (pat,valu) =
    rmv ndn na;
    add ndn (na,(pat,valu))
;;

let lookup (nm,dnm) valu =
let hkey = option_app fst (termdn__constr_val_discr valu)
in try btermdn__lookup (mhm__map dnm hkey) valu
   with Not_found -> []
;;

let app f (nm,dnm) = fmavm__app f nm;;

let dnet_depth = btermdn__dnet_depth;;

let freeze (nm,dnm) = (fmavm__freeze nm,mhm__freeze dnm);;
let unfreeze (fnm,fdnm) (nm,dnm) =
    (fmavm__unfreeze fnm nm;
     mhm__unfreeze fdnm dnm)
;;

let empty (nm,dnm) = (fmavm__empty nm;
                      mhm__empty dnm)
;;


(* $Id: nbtermdn.ml,v 1.2 1996/02/12 15:49:13 cpaulin Exp $ *)
