(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               tactics0.ml                                *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "initial";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "names";;
#open "generic";;
#open "term";;
#open "printer";;
#open "termenv";;
#open "reduction";;
#open "mach";;
#open "trad";;
#open "tacmach";;
#open "proof";;
#open "clenv";;
#open "tactics";;
#open "lexer";;

#infix "ORELSE";;
#infix "NEXT";;
#infix "THEN";;
#infix "THENF";;
#infix "THENL";;
#infix "THENS";;

let get_new_ids ids sign =
    list_it
    (fun id acc ->
         (next_ident_away id (acc@(ids_of_sign sign)))::acc)
    ids []
;;

let pf_get_new_ids ids gls = get_new_ids ids (pf_hyps gls);;

type arg_binder =
    DEP of identifier
  | NODEP of int
  | ABS of int
;;

type arg_bindings == (arg_binder * constr) list;;

let clenv_constrain_with_bindings bl clause =
if bl = [] then clause else
let all_mvs = collect_metas (clenv_template clause).rebus and
    ind_mvs = clenv_independent clause (clenv_template clause,
                                        clenv_template_type clause)

in matchrec clause bl
    where rec matchrec clause = function
    [] -> clause
  | (b,c)::t ->
    let k =
        (match b with
         DEP s ->
         let l = clenv_lookup_name clause s
         in if length l > 1 then
             error "Multiple instances of this name"
            else hd l
       | NODEP n -> (try
                    if n > 0 then nth ind_mvs n
                    else if n < 0 then nth (rev ind_mvs) (-n)
                    else error "clenv_constrain_with_bindings"
                     with Failure _ ->
                         errorlabstrm "clenv_constrain_with_bindings"
                         [< 'S"Clause did not have " ; 'INT n ; 'S"-th" ;
                            'S" unnamed argument" >])
       | ABS n -> (try
                   if n > 0 then nth all_mvs n
                  else if n < 0 then nth (rev all_mvs) (-n)
                  else error "clenv_constrain_with_bindings" 
                 with Failure _ ->
                     errorlabstrm "clenv_constrain_with_bindings"
                     [< 'S"Clause did not have " ; 'INT n ; 'S"-th" ;
                      'S" absolute argument" >])) in

    let k_typ = strong whd_betaiota (clenv_instance_type clause k)
    and c_typ = strong whd_betaiota (w_type_of clause.hook c)
    in matchrec (clenv_assign k c (clenv_unify k_typ c_typ clause)) t
;;

let strong_prodspine redfun = strec
    where rec strec c =
    match redfun c with
    DOP2(Prod,A,DLAM(na,B)) -> DOP2(Prod,A,DLAM(na,strec B))
  | x -> x
;;


let add_prod_rel sigma (T,env) =
    match T with
    DOP2(Prod,c1,(DLAM(na,B))) ->
    (B,add_rel (na,assumption_of_judgement sigma (execute_rec sigma env c1)) env)
  | _ -> failwith "add_prod_rel"
;;

let rec add_prods_rel sigma (T,env) =
    try add_prods_rel sigma (add_prod_rel sigma (whd_betadeltaiota sigma T,env))
    with Failure "add_prod_rel" -> (T,env)
;;

let add_prod_sign sigma (T,sign) =
    match T with
    DOP2(Prod,c1,(DLAM(na,_) as B)) ->
    let (Name id) = named_hd T na in
    (SAPP B (VAR id),
     add_sign (id,assumption_of_judgement sigma (fexecute sigma sign c1)) sign)
  | _ -> failwith "add_prod_sign"
;;

let rec add_prods_sign sigma (T,sign) =
    try add_prods_sign sigma (add_prod_sign sigma (whd_betadeltaiota sigma T,sign))
    with Failure "add_prod_sign" -> (T,sign)
;;

(* $Id: tactics0.ml,v 1.2 1996/02/12 15:49:22 cpaulin Exp $ *)
