(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               tactics2.ml                                *)
(****************************************************************************)

#infix "o";;
#open "std";;
#open "initial";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "names";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "printer";;
#open "tacmach";;
#open "proof_trees";;
#open "clenv";;
#open "ast";;
#open "tactics";;
#open "pattern";;
#open "tactics1";;

#infix "ORELSE";;
#infix "NEXT";;
#infix "THEN";;
#infix "THENF";;
#infix "THENL";;
#infix "THENS";;

let Unfolds idl cls gl =
    match cls with
    NONE -> unfold_nth_tac (map (fun x -> ([],x)) idl) gl
  | SOME id -> unfold_nth_hyp (map (fun x -> ([],x)) idl) id gl
;;

let mmk = make_module_marker ["#Prelude.obj"];;

let Intro = tactics__intro;;
let IntroUsing id = tactics__intro_using id;;
let IntroReplacing id = tacmach__intro_replacing id;;
let IntroErasing id = tactics__clear_hyp [id] THEN tacmach__introduction id;;

let ImpIntro = Intro;;

let Exact = tactics__exact;;
let ExactClause (SOME id) = Exact (VAR id);;

let False_term = put_pat mmk "False";;

let Absurdity id gl =
    if matches gl (h (SOME id) gl) False_term then
    elimination [] (VAR id) gl
    else FAILTAC gl
;;

let and_pattern = put_pat mmk "(and ? ?)";;
let prod_pattern = put_pat mmk "(prod ? ?)";;

let AndIntro gl =
    if matches gl (pf_concl gl) and_pattern then
        resolve (ASTnvar (id_of_string"conj")) gl
    else if matches gl (pf_concl gl) prod_pattern then
        resolve (ASTnvar (id_of_string"pair")) gl
    else
        error "Not a predefined conjunction"
;;

let sumbool_pattern = put_pat mmk "(sumbool ? ?)";;
let sumor_pattern = put_pat mmk "(sumor ? ?)";;
let or_pattern = put_pat mmk "(or ? ?)";;
let sum_pattern = put_pat mmk "(sum ? ?)";;

let IRight gl = 
    if matches gl (pf_concl gl) sumbool_pattern then
        resolve (ASTnvar (id_of_string"right")) gl
    else if matches gl (pf_concl gl) sumor_pattern then
        resolve (ASTnvar (id_of_string"inright")) gl
    else if matches gl (pf_concl gl) or_pattern then
        resolve (ASTnvar (id_of_string"or_intror")) gl
    else if matches gl (pf_concl gl) sum_pattern then
        resolve (ASTnvar (id_of_string"inr")) gl
    else 
        error "Not a predefined disjunction";;

let ILeft gl = 
    if matches gl (pf_concl gl) sumbool_pattern then
        resolve (ASTnvar (id_of_string"left")) gl
    else if matches gl (pf_concl gl) sumor_pattern then
        resolve (ASTnvar (id_of_string"inleft")) gl
    else if matches gl (pf_concl gl) or_pattern then
        resolve (ASTnvar (id_of_string"or_introl")) gl
    else if matches gl (pf_concl gl) sum_pattern then
        resolve (ASTnvar (id_of_string"inl")) gl
    else 
        error "Not a predefined disjunction";;

let Cut T = cut_tac T THENS [ImpIntro;IDTAC];;
let CutReplacing id T = cut_tac T THENS [
                                         (IntroReplacing id
                     ORELSE
                      (IntroErasing id
                       ORELSE
                       IntroUsing id)
                     )
                     ;
                     IDTAC]
;;

let ParallelCut L = prec (rev L)
    where rec prec = function
    [] -> IDTAC
  | h::t -> Cut h THENS [prec t;IDTAC]
;;

let is_and_term gl T =
    exists (matches gl T) [and_pattern;prod_pattern]
;;

let is_or_term gl T =
    exists (matches gl T)
    [sumbool_pattern;sumor_pattern;or_pattern;sum_pattern]
;;

let is_imp_term T =
    match T with
    DOP2(Prod,_,DLAM(_,B)) -> not(dependent (Rel 1) B)
  | _ -> false
;;

let AndE id gl =
let T = snd(lookup_sign id (pf_hyps gl))
in if is_and_term gl (pf_hnf_constr gl T) then
    ((elimination [] (VAR id)) THEN (DO 2 ImpIntro)) gl
    else FAILTAC gl
;;

let DAnd cls gl =
    match cls with
    NONE -> AndIntro gl
  | SOME id -> AndE id gl
;;

let OrE id gl =
let T = snd(lookup_sign id (pf_hyps gl))
in if is_or_term gl (pf_hnf_constr gl T) then
    ((elimination [] (VAR id)) THEN ImpIntro) gl
else FAILTAC gl
;;

let DOr selopt cls gl =
    match cls with
    SOME id -> OrE id gl
  | NONE ->
    if selopt = 1 then
        ILeft gl
    else if selopt = 2 then
        IRight gl
    else error "DOr"
;;

let ImpE id gl =
let T = snd(lookup_sign id (pf_hyps gl))
in if is_imp_term (pf_hnf_constr gl T) then
    let (DOP2(Prod,Dom,DLAM(_,Rng))) = pf_hnf_constr gl T
    in (Cut Rng
        THENS [IDTAC;
               apply_term (VAR id) [DOP0(Meta(newMETA()))]]) gl
   else FAILTAC gl
;;
                        
let DImp cls gl =
    match cls with
    NONE -> ImpIntro gl
  | SOME id -> ImpE id gl
;;
(*
let res_pf_THEN KONT clenv tac gls =
let clenv' = (clenv_unify (clenv_instance_template_type clenv) (pf_concl gls) clenv)
in (clenv_refine KONT clenv' THEN tac clenv') gls
;;

let res_pf_THEN_i KONT clenv tac i gls =
let clenv' = (clenv_unify (clenv_instance_template_type clenv) (pf_concl gls) clenv)
in THEN_i (clenv_refine KONT clenv') (tac clenv') i gls
;;
*)

let res_pf_THEN KONT clenv tac gls =
let clenv' = (clenv_unique_resolver false clenv gls)
in (clenv_refine KONT clenv' THEN tac clenv') gls
;;

let res_pf_THEN_i KONT clenv tac i gls =
let clenv' = (clenv_unique_resolver false clenv gls)
in THEN_i (clenv_refine KONT clenv') (tac clenv') i gls
;;

let elim_res_pf_THEN_i KONT clenv tac i gls =  
let clenv' = (clenv_unique_resolver true clenv gls) in
    THEN_i (clenv_refine KONT clenv') (tac clenv') i gls
;;

let BringHyps clsl gl = 
let ids = map (fun (SOME id) -> id | NONE -> error "BringHyps") clsl in
    let newcl = list_it 
        (fun id cl' ->
             let t = pf_type_of gl (VAR id)
             in produit id t cl')
             ids (pf_concl gl)
    in apply_type newcl (map (fun id -> VAR id) ids) gl
;;

let ThinClauses clsl =
    thin (map (fun (SOME id) -> id
                 | NONE -> error "ThinClauses") clsl);;

let ThinClause cls = ThinClauses [cls];;

let rec IntrosThinning  = fun
    [] gl -> IDTAC gl
  | (false::tl) gl -> (Intro THEN (IntrosThinning tl)) gl
  | (true::tl) gl -> (Intro THEN
                       OnLastHyp ThinClause THEN
                       IntrosThinning tl) gl
;;

let ChangeClause c cls gl =
    match cls with
    NONE -> Change c gl
  | SOME id -> ChangeHyp c id gl
;;

let HnfConcl = hnf_tac;;
let HnfHyp = hnf_tac_hyp;;

let HnfClause cls gl =
    match cls with
    NONE -> HnfConcl gl
  | SOME id -> HnfHyp id gl
;;


let clenv_change_head (templ,templty) clenv =
  {templval = mk_freelisted templ;
  templtyp = mk_freelisted templty;
  namenv = clenv.namenv;
  env = clenv.env;
  hook = clenv.hook}
;;

let rec assign_term (DOP0(Meta mv)) c ce =
if not(occur_meta c) then
    let cty = w_type_of ce.hook c
    in clenv_assign mv c (clenv_unify cty (clenv_instance_type ce mv) ce)

else
let (hd,args) = whd_betadeltaiota_stack (w_Underlying ce.hook) c [] in
let hdty = clenv_type_of ce hd in
let subce = bounded_mk_clenv_from (length args) () (hd,hdty) in
let ce' = clenv_bchain mv subce ce in
let (_,templargs) = 
    whd_betadeltaiota_stack (w_Underlying ce'.hook) (clenv_value ce mv).rebus []
in
    list_it2 assign_term templargs args ce'
;;

let rec build_args acc ce = fun
    (DOP2(Prod,A,(DLAM(na,_) as B))) (a::Bargs) ->
    let (newa,ce') = (build_term ce (na,SOME A) a)
    in build_args (newa::acc) ce' (SAPP B a) Bargs

  | _ [] -> (rev acc,ce)

  | _ (_::_) -> failwith "mk_clenv_using"

and build_term ce = fun
    (na,SOME T) (DOP0(Meta mv)) -> (*(DOP0(Meta mv),
                                    clenv_pose (na,mv,T) ce)*)
    build_term ce (na,SOME T) (DOP0(XTRA("ISEVAR",[])))

  | (na,SOME T) (DOP0(XTRA("ISEVAR",[]))) ->
    let mv = newMETA()
    in (DOP0(Meta mv),
        clenv_pose (na,mv,T) ce)

  | (na,_) (DOP2(Cast,c,t)) -> build_term ce (na,SOME t) c

  | (na,SOME T) c ->
    if not(occur_meta c) then (c,ce)
    else
        let (hd,args) = whd_betadeltaiota_stack (w_Underlying ce.hook) c [] in
        let hdty = w_type_of ce.hook hd in
        let (args,ce') = build_args [] ce (w_whd_betadeltaiota ce.hook hdty) args in
        let newc = applist(hd,args) in
        let T' = clenv_type_of ce' newc in
            if w_conv_x ce'.hook T T' then (newc,ce')
            else failwith "mk_clenv_using"

  | (na,NONE) c ->
    if not(occur_meta c) then (c,ce)
    else
        let (hd,args) = whd_betadeltaiota_stack (w_Underlying ce.hook) c [] in
        let hdty = w_type_of ce.hook hd in
        let (args,ce') = build_args [] ce (w_whd_betadeltaiota ce.hook hdty) args in
        let newc = applist(hd,args) in
            (newc,ce')
;;


let mk_clenv_using wc c =
let ce = mk_clenv wc (DOP0 Implicit) in
let (newc,ce') = (try build_term ce (Anonymous,NONE) c
           with Failure _ ->
               error "mk_clenv_using"
               [< 'S"The term " ; prterm c ; 'S" is not well-typed" >]) in
    clenv_change_head (newc,clenv_type_of ce' newc) ce'
;;


let ApplyUsing c gl =
    let (wc,KONT) = StartWalk gl in
    let clause = mk_clenv_using wc c
    in res_pf KONT clause gl
;;

let clenv_apply_n_times n ce =
let templtyp = clenv_instance_template_type ce and
    templval = (clenv_template ce).rebus in
    apprec ce [] n (whd_betadeltaiota (w_Underlying ce.hook) templtyp)
    where rec apprec ce argacc = fun
    0 templtyp ->
    clenv_change_head (applist(templval,rev argacc),
                       templtyp) ce

  | n (DOP2(Prod,Dom,(DLAM(na,_) as Rng))) ->
    let mv = newMETA() in
    let newce = clenv_pose (na,mv,Dom) ce in
      apprec newce ((DOP0(Meta mv))::argacc) (n-1) (SAPP Rng (DOP0(Meta mv)))

  | n _ -> failwith "clenv_apply_n_times"
;;

(* $Id: tactics2.ml,v 1.3 1996/02/12 15:49:26 cpaulin Exp $ *)
