
(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               tactics3.ml                                *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "initial";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "constrtypes";;
#open "reduction";;
#open "termenv";;
#open "machops";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "tacmach";;
#open "proof";;
#open "proof_trees";;
#open "clenv";;
#open "tactics";;
#open "tactics0";;
#open "tactics1";;
#open "tactics2";;

#infix "ORELSE";;
#infix "NEXT";;
#infix "THEN";;
#infix "THENF";;
#infix "THENL";;
#infix "THENS";;

let prod_it = it_list (fun c (n,t)  ->  DOP2(Prod,t,DLAM(n,c)));;

let reduce_to_ind gl t = elimrec t []
    where rec elimrec t l = match decomp_app(t) with
               (DOPN(MutInd (sp,_),_) as mind,_) -> (mind,mind_path mind,t,prod_it t l)
             | (DOPN(Const _,_),_) -> elimrec (strong whd_betaiota (pf_one_step_reduce gl t)) l
             | (DOP2(Cast,c,_),[]) -> elimrec c l
             | (DOP2(Prod,ty,DLAM(n,t')),[]) -> elimrec t' ((n,ty)::l)
             | _ -> error "Not an inductive product";;



let reduce_to_mind gl t = elimrec t []
 where rec elimrec t l = match whd_castapp_stack (t) [] with
    (DOPN(MutInd sp,_) as mind,_) -> (mind,prod_it t l)
  | (DOPN(Const _,_),_) -> 
           elimrec (strong whd_betaiota (pf_one_step_reduce gl t)) l
  | (DOP2(Cast,c,_),[]) -> elimrec c l
  | (DOP2(Prod,ty,DLAM(n,t')),[]) -> elimrec t' ((n,ty)::l)
  | _ -> error "Not an inductive product";;




(* Find the right elimination suffix corresponding to the sort of the goal *)
 let suff gl cl = match hnf_type_of gl cl with
                  DOP0(Sort(Type(Null,_))) -> "_rect"
                | DOP0(Sort(Type(Pos,_))) -> "_recs"
                | DOP0(Sort(Type(_,_)))    -> error "elimination not allowed on this sort"
                | DOP0(Sort(Prop(Null)))   -> "_ind"
                | DOP0(Sort(Prop(Pos)))    -> "_rec"
                | _            -> error "goal should be a type";;

(* c should be of type A1->.. An->B with B an inductive definition *)


let lookup_eliminator sign path suff =
let path_elim = append_to_path path suff
in try global_reference (GLOB sign) path_elim (basename path_elim)
   with UserError _ ->
        global (GLOB sign) (basename path_elim)
;;

let case_constructor_sign ity i = 
  let (DOPN(MutInd (_,j),_)) = ity in
   let rec analrec acc = function 
          [] -> acc 
        | (c::rest) -> analrec (false::acc) rest  in
   let recarg = mis_recarg (mind_specif_of_mind ity) 
   in analrec [] recarg.(i-1)
;;

let analyze_constructor_type ity i =
  let (DOPN(MutInd (_,j),_)) = ity in
   let rec analrec acc = function 
          (Norec::rest) -> analrec (false::acc) rest
        | (Imbr(_)::rest) -> analrec (false::acc) rest
        | (Mrec k::rest) -> analrec ((j=k) ::acc) rest
        | [] -> rev acc in 
   let recarg = mis_recarg (mind_specif_of_mind ity) 
   in analrec [] recarg.(i-1)
;;

let nb_branch_args ity i =
let sign = analyze_constructor_type ity i
in it_list (fun n true -> 2+n
              | n false -> 1+n)
           0 sign
;;

let sort_of_goal gl = match hnf_type_of gl (pf_concl gl) with 
     DOP0(Sort s) -> s
  |  _            -> anomaly "goal should be a type";;

let elimination_then_using tac predicate (indbindings,elimbindings) c gl =
  let (ity,t,elim) = 
     let (ity,path_name,_,t) = reduce_to_ind gl (pf_type_of gl c) in
       (ity,t,lookup_eliminator (pf_hyps gl) path_name (suff gl (pf_concl gl)))
 in let name_elim =
      (match elim with
      DOPN(Const sp,_) -> id_of_string(string_of_path sp)
    | VAR _ -> id_of_var elim
    | _ -> id_of_string " ") in
  let (wc,KONT) = StartWalk gl in

  let indclause = mk_clenv_from wc (c,t) in

  let indclause' = clenv_constrain_with_bindings indbindings indclause in

  let elimclause = mk_clenv_from () (elim,w_type_of wc elim) in

  let indmv = (match rev_subterm (clenv_template elimclause).rebus [0] with
               DOP0(Meta mv) -> mv
             | _ -> error "elimination") in

  let pmv = 
      (match decomp_app (clenv_template_type elimclause).rebus with
            (DOP0(Meta(p)),oplist) -> p
          | _ -> error ((string_of_id name_elim)^" not an elimination combinator")) in

  let elimclause' = clenv_fchain indmv elimclause indclause' in

  let elimclause' = clenv_constrain_with_bindings elimbindings elimclause' in

  let after_tac ce i gl =
      let (hd,largs) = decomp_app (clenv_template_type ce).rebus in
      let branchsign = analyze_constructor_type ity i in
      let ba = {branchsign = branchsign;
                nassums = it_list (fun acc true -> acc+2
                                     | acc false -> acc+1) 0 branchsign;
                branchnum = i;
                ity = ity;
                largs = map (clenv_instance_term ce) largs;
                pred = clenv_instance_term ce hd}
      in tac ba gl in

  let elimclause' =
      (match predicate with
       NONE -> elimclause'
     | SOME P -> clenv_assign pmv P elimclause')

  in if (predicate = NONE) then
      (elim_res_pf_THEN_i KONT elimclause' after_tac 1 gl)
     else elim_res_pf_THEN_i KONT elimclause' after_tac 1 gl
;;

let elimination_then tac bindings c gl =
    elimination_then_using tac NONE bindings c gl;;


let sort_of_constr gl c = 
 match hnf_type_of gl c with 
     DOP0(Sort s) -> s
  |  _            -> anomaly "constr should be a type"
;;

let case_then_using tac predicate (indbindings,elimbindings) c gl =
(* finding the case combinator *)
  let (ity,_,_,t) = reduce_to_ind gl (pf_type_of gl c) in
  let sigma = Project gl in 
  let sort = sort_of_goal gl  in
  let elim = indrec__make_case_gen sigma ity sort in
(* applying elimination_scheme just a little modified *)
  let (wc,KONT) = StartWalk gl in

  let indclause = mk_clenv_from wc (c,t) in

  let indclause' = clenv_constrain_with_bindings indbindings indclause in

  let elimclause = mk_clenv_from () (elim,w_type_of wc elim) in

  let indmv = (match rev_subterm (clenv_template elimclause).rebus [0] with
               DOP0(Meta mv) -> mv
             | _ -> error "elimination") in

  let pmv = 
      (match decomp_app (clenv_template_type elimclause).rebus with
            (DOP0(Meta(p)),oplist) -> p
          | _ -> error (" not an elimination combinator")) in

  let elimclause' = clenv_fchain indmv elimclause indclause' in

  let elimclause' = clenv_constrain_with_bindings elimbindings elimclause' in

  let after_tac ce i gl =
      let (hd,largs) = decomp_app (clenv_template_type ce).rebus in
      let branchsign = case_constructor_sign ity i in
      let ba = {branchsign = branchsign;
                nassums = it_list (fun acc true -> acc+2
                                     | acc false -> acc+1) 0 branchsign;
                branchnum = i;
                ity = ity;
                largs = map (clenv_instance_term ce) largs;
                pred = clenv_instance_term ce hd}
      in tac ba gl in

  let elimclause' =
      (match predicate with
       NONE -> elimclause'
     | SOME P -> clenv_assign pmv P elimclause')

  in if (predicate = NONE) then
      (elim_res_pf_THEN_i KONT elimclause' after_tac 1 gl)
     else elim_res_pf_THEN_i KONT elimclause' after_tac 1 gl
;;


let case_nodep_then_using tac predicate (indbindings,elimbindings) c gl =
(* finding the case combinator *)
  let (ity,_,_,t) = reduce_to_ind gl (pf_type_of gl c) in
  let sigma = Project gl in 
  let sort = sort_of_goal gl  in
  let elim = indrec__make_case_nodep sigma ity sort in
(* applying elimination_scheme just a little modified *)
  let (wc,KONT) = StartWalk gl in

  let indclause = mk_clenv_from wc (c,t) in

  let indclause' = clenv_constrain_with_bindings indbindings indclause in

  let elimclause = mk_clenv_from () (elim,w_type_of wc elim) in

  let indmv = (match rev_subterm (clenv_template elimclause).rebus [0] with
               DOP0(Meta mv) -> mv
             | _ -> error "elimination") in

  let pmv = 
      (match decomp_app (clenv_template_type elimclause).rebus with
            (DOP0(Meta(p)),oplist) -> p
          | _ -> error (" not an elimination combinator")) in

  let elimclause' = clenv_fchain indmv elimclause indclause' in

  let elimclause' = clenv_constrain_with_bindings elimbindings elimclause' in

  let after_tac ce i gl =
      let (hd,largs) = decomp_app (clenv_template_type ce).rebus in
      let branchsign = case_constructor_sign ity i in
      let ba = {branchsign = branchsign;
                nassums = it_list (fun acc true -> acc+2
                                     | acc false -> acc+1) 0 branchsign;
                branchnum = i;
                ity = ity;
                largs = map (clenv_instance_term ce) largs;
                pred = clenv_instance_term ce hd}
      in tac ba gl in

  let elimclause' =
      (match predicate with
       NONE -> elimclause'
     | SOME P -> clenv_assign pmv P elimclause')

  in if (predicate = NONE) then
      (elim_res_pf_THEN_i KONT elimclause' after_tac 1 gl)
     else elim_res_pf_THEN_i KONT elimclause' after_tac 1 gl
;;




let IntroCaseAssums ba gls =
let case_thin_sign = join (map (fun true -> [false;true]
                                  | false -> [false]) ba.branchsign)
in IntrosThinning case_thin_sign gls
;;

let IntroElimAssums ba gls =
let nassums = it_list (fun acc true -> acc+2
                         | acc false -> acc+1) 0 ba.branchsign
in RepeatFor nassums Intro gls
;;

let make_elim_branch_assumptions ba gl =
    makerec ([],[],[],[],[]) ba.branchsign
            (error_first_n ba.nassums (ids_of_sign (pf_hyps gl)))
    where rec makerec (assums,cargs,constargs,recargs,indargs) = fun
    [] _ -> {ba = ba;
             assums = assums;
             cargs = cargs;
             constargs = constargs;
             recargs = recargs;
             indargs = indargs}
  | (true::tl) (recarg::indarg::idtl) ->
    makerec (recarg::indarg::assums,
             recarg::cargs,
             recarg::recargs,
             constargs,
             indarg::indargs) tl idtl

  | (false::tl) (constarg::idtl) ->
    makerec (constarg::assums,
             constarg::cargs,
             constarg::constargs,
             recargs,
             indargs) tl idtl

  | _ _ -> error "make_elim_branch_assumptions"
;;

let IntroElimAssumsThen tac ba gls =
    (IntroElimAssums ba THEN
     (fun gl -> tac (make_elim_branch_assumptions ba gl) gl)) gls
;;

let make_case_branch_assumptions ba gl =
    makerec ([],[],[],[]) ba.branchsign
            (error_first_n ba.nassums (ids_of_sign (pf_hyps gl)))
    where rec makerec (assums,cargs,constargs,recargs) = fun
    [] _ -> {ba = ba;
             assums = assums;
             cargs = cargs;
             constargs = constargs;
             recargs = recargs;
             indargs = []}
  | (true::tl) (recarg::idtl) ->
    makerec (recarg::assums,
             recarg::cargs,
             recarg::recargs,
             constargs) tl idtl

  | (false::tl) (constarg::idtl) ->
    makerec (constarg::assums,
             constarg::cargs,
             recargs,
             constarg::constargs) tl idtl

  | _ _ -> error "make_case_branch_assumptions"
;;

let simple_elimination_then tac = elimination_then tac ([],[]);;

let IntroCaseAssumsThen tac ba gls =
    (IntroCaseAssums ba THEN
     (fun gl -> tac (make_case_branch_assumptions ba gl) gl)) gls
;;

let CaseIndTrailer = 
    (IntroCaseAssumsThen (fun ba -> let cl = map inSOME ba.assums in
                  BringHyps cl THEN ThinClauses cl))
;;

let CaseIndUsing = elimination_then_using CaseIndTrailer;;
let CaseInd = CaseIndUsing NONE;;
let SimpleCaseInd = CaseInd ([],[]);;


(* $Id: tactics3.ml,v 1.5 1996/02/12 15:49:28 cpaulin Exp $ *)
