(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               tactics3.mli                               *)
(****************************************************************************)
#open "std";;
#open "more_util";;
#open "term";;
#open "tacmach";;
#open "names";;
#open "proof";;
#open "pattern";;
#open "tactics0";;
#open "tactics1";;
#open "tactics2";;
#open "clenv";;

type branch_args =
    {ity : constr; (* the type we were eliminating on *)
     largs : constr list; (* its arguments *)
     branchnum : int; (* the branch number *)
     pred : constr; (* the predicate we used *)
     nassums : int; (* the number of assumptions to be introduced *)
     branchsign : bool list} (* the signature of the branch (true=recursive argument, false=constant) *)
;;

type branch_assumptions =
    {ba : branch_args; (* the branch args *)
     assums : identifier list; (* the list of assumptions which have been introduced *)
     cargs : identifier list; (* the constructor arguments *)
     constargs : identifier list; (* the CONSTANT constructor arguments *)
     recargs : identifier list; (* the RECURSIVE constructor arguments *)
     indargs : identifier list} (* the inductive arguments *)
;;

value sort_of_goal:goal sigma -> sorts;;
value analyze_constructor_type : constr -> int -> bool list;;
value nb_branch_args : constr -> int -> int;;

value elimination_then_using :
  (branch_args -> tactic) -> constr option -> (arg_bindings * arg_bindings) -> constr -> tactic;;

value elimination_then :
  (branch_args -> tactic) -> (arg_bindings * arg_bindings) -> constr -> tactic;;
value case_then_using :
  (branch_args -> tactic) -> constr option -> (arg_bindings * arg_bindings) -> constr -> tactic;;

value case_nodep_then_using :
  (branch_args -> tactic) -> constr option -> (arg_bindings * arg_bindings) -> constr -> tactic;;

value simple_elimination_then :
  (branch_args -> tactic) -> constr -> tactic;;

value IntroCaseAssums : branch_args -> tactic;;
value IntroElimAssums : branch_args -> tactic;;

value make_elim_branch_assumptions :
   branch_args -> goal sigma -> branch_assumptions;;
value IntroElimAssumsThen :
  (branch_assumptions -> tactic) -> branch_args -> tactic;;

value make_case_branch_assumptions :
  branch_args -> goal sigma -> branch_assumptions;;
value IntroCaseAssumsThen :
  (branch_assumptions -> tactic) -> branch_args -> tactic;;

value CaseIndTrailer : branch_args -> tactic;;
value CaseIndUsing : constr option -> (arg_bindings * arg_bindings) -> constr -> tactic;;
value CaseInd : (arg_bindings * arg_bindings) -> constr -> tactic;;
value SimpleCaseInd : constr -> tactic;;


(* $Id: tactics3.mli,v 1.4 1996/02/12 15:07:13 cpaulin Exp $ *)
