(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                termdn.ml                                 *)
(****************************************************************************)
#open "more_util";;
#open "vectops";;
#open "generic";;
#open "term";;

let rec occur_meta = function
    DOP0 (Meta _) -> true
  | Rel _ -> false
  | VAR _ -> false
  | DOP0 _ -> false
  | DOP1(_,c) -> occur_meta c
  | DOP2(_,c1,c2) -> occur_meta c1 or occur_meta c2
  | DOPN(_,cl) -> exists_vect occur_meta cl
  | DOPL(_,cl) -> exists occur_meta cl
  | DLAM(_,c) -> occur_meta c
  | DLAMV(_,cl) -> exists_vect occur_meta cl
;;

let decomp = decrec []
    where rec decrec acc = function
    DOPN(AppL,cl) -> decrec (app_tl_vect cl acc) (hd_vect cl)
  | DOP2(Cast,c1,_) -> decrec acc c1
  | c -> (c,acc)
;;

let constr_pat_discr t =
    if not(occur_meta t) then NONE
    else
        match decomp t with
        DOPN(Const _,_) as c,l -> SOME(TERM c,l)
      | DOPN(MutInd _,_) as c,l -> SOME(TERM c,l)
      | DOPN(MutConstruct _,_) as c,l -> SOME(TERM c,l)
      | VAR _ as c,l -> SOME(TERM c,l)
      | c -> NONE
;;
let constr_val_discr t =
    match decomp t with
    DOPN(Const _,_) as c,l -> SOME(TERM c,l)
  | DOPN(MutInd _,_) as c,l -> SOME(TERM c,l)
  | DOPN(MutConstruct _,_) as c,l -> SOME(TERM c,l)
  | VAR _ as c,l -> SOME(TERM c,l)
  | c -> NONE
;;

let constr_pat_discr t =
    match constr_pat_discr t with
    NONE -> NONE
  | SOME (c,_) -> SOME(c,[])
;;

let constr_val_discr t =
    match constr_val_discr t with
    NONE -> NONE
  | SOME (c,_) -> SOME(c,[])
;;

let new () = dn__new constr_pat_discr;;

let add = dn__add;;
let rmv = dn__rmv;;

let lookup dn t = dn__lookup dn constr_val_discr t;;

let app f dn = dn__app f dn;;

(* $Id: termdn.ml,v 1.4 1996/02/12 15:49:30 cpaulin Exp $ *)
