(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  DHyp.v                                  *)
(****************************************************************************)

Declare ML Module "nbtermdn" "dhyp".

Grammar tactic simple_tactic :=
  [ "DHyp" identarg($id) ] -> [$0 = (DHyp $id)]
| [ "CDHyp" identarg($id) ] -> [$0 = (CDHyp $id)]
| [ "DConcl" ] -> [$0 = (DConcl)]
| [ "D" identarg($id) ] -> [$0 = (DHyp $id)]
| [ "CD" identarg($id) ] -> [$0 = (CDHyp $id)]
| [ "D" ] -> [$0 = (DConcl)]
.

Grammar vernac meta_binding :=
  [ identarg($id) ":=" comarg($c) ] -> [$0 = (TACTIC_ARG (BINDING $id $c))]
.

Grammar vernac meta_binding_list :=
  [ meta_binding($b) meta_binding_list($l) ] -> [$0 = ($CONS $b $l)]
| [ ] -> [$0 = ($LIST)]
.

Grammar vernac destruct_location :=
  [ "Conclusion" ] -> [$0 = (CONCL)]
| [ "Discardable" "Hypothesis" ] -> [$0 = (DiscardableHYP)]
| [ "Hypothesis" ] -> [$0 = (PreciousHYP)]
.

Grammar vernac vernac :=
  [ "Hint" "Destruct" destruct_location($loc) identarg($na) comarg($hyptyp) numarg($pri)
    spatarg($tac) "." ]
  -> [$0 = (HintDestruct $na (ANY $loc) $hyptyp $pri $tac)]
| [ "Hint" "Destruct" destruct_location($loc) identarg($na) comarg($hyptyp) numarg($pri)
    spatarg($tac) "with" meta_binding_list($bl) "." ]
  -> [$0 = ($OPER{HintDestruct} ($APPEND ($LIST $na (ANY $loc) $hyptyp $pri $tac)
	                                        $bl))]
.

(*

Hint DHyp foobar (le (S ?) O) 1 <:tactic:<Inv $$0 goober>>.

*)

Syntax tactic dhyp (DHyp $id)  0
       "DHyp "  <$id:"identarg":*>.

Syntax tactic  cdhyp (CDHyp $id) 0
        "CDHyp "<$id:"identarg":*>. 

Syntax tactic  dconcl (DConcl) 0
        "DConcl".

Syntax vernac meta_binding  (TACTIC_ARG (BINDING $id $c)) 0
        <$id:"identarg":*> ":=" <$c:"Comarg":*>.

Syntax vernac meta_binding_list_nil ($OPER{METABINDINGLIST} ($LIST)) 0.
Syntax vernac meta_binding_list_cons ($OPER{METABINDINGLIST} ($CONS $b $l)) 0
        [<hov 0> <$b:"meta_bind":*> [1 0] <$L:"metabindlist":*> ]
         with $L := ($OPER{METABINDINGLIST} $l).


Syntax vernac concl (CONCL) 0 
        "Conclusion".

Syntax vernac discardablehyp (DiscardableHYP) 0
        "Discardable Hypothesis".

Syntax vernac precioshyp  (PreciousHYP) 0
        "Hypothesis" .

Syntax vernac hint_destruct (HintDestruct $na (ANY $loc) $hyptyp ($PRIM $pri) ($PRIM $tac)) 0
    [<hov 0>   "Hint Destruct " <$loc:"DestrLoc":*> [1 0] <$na:"identarg":*> 
       [1 0] <$hyptyp:"Comarg":*> [1 0] <$pri:"numarg":*> [1 0] <$tac:"spatarg":*>].
 

Syntax vernac hint_destr2 
     ($OPER{HintDestruct} ($CONS $na ($CONS (ANY $loc) ($CONS $hyptyp ($CONS ($PRIM $pri) ($CONS ($PRIM $tac) $bl)))))) 0
        [<hov 0>   "Hint Destruct " <$loc:"DestrLoc":*> [1 0] <$na:"identarg":*> 
            [1 0] <$hyptyp:"Comarg":*> [1 0] <$pri:"numarg":*> [1 0] 
            "with" [1 0] <$BL:"MetaBindList":*>]
        with $BL:=($OPER{METABINDINGLIST} $bl). 










(* $Id: DHyp.v,v 1.3 1996/02/12 15:55:20 cpaulin Exp $ *)
