(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                Equality.v                                *)
(****************************************************************************)

Declare ML Module "equality".

Grammar tactic simple_tactic :=
  [ "Simplify_eq" identarg($id) ]  -> [$0 = (DEqHyp $id)]
| [ "Simplify_eq" ]                -> [$0 = (DEqConcl)]
| [ "Discriminate" identarg($id) ] -> [$0 = (DiscrHyp $id)]
| [ "Discriminate" ]               -> [$0 = (Discr)]
| [ "Injection " identarg($id) ]   -> [$0 = (InjHyp $id)]
| [ "Injection" ]                  -> [$0 = (Inj)]
| [ "Rewrite" comarg($c) "in" identarg($h) ] -> [$0 = (RewriteLRin $c $h)]
| [ "Rewrite" "->" comarg($c) "in" identarg($h) ] -> [$0 = (RewriteLRin $c $h)]
| [ "Rewrite" "<-" comarg($c) "in" identarg($h) ] -> [$0 = (RewriteRLin $c $h)]
.


Grammar tactic simple_tactic :=
  [ "CutRewrite" "->" comarg($eqn) ] -> [$0=(SubstConcl_LR $eqn)]
| [ "CutRewrite" "->" comarg($eqn) "in" identarg($id)] -> [$0=(SubstHyp_LR $eqn $id)]
| [ "CutRewrite" "<-" comarg($eqn) ] -> [$0=(SubstConcl_RL $eqn)]
| [ "CutRewrite" "<-" comarg($eqn) "in" identarg($id)] -> [$0=(SubstHyp_RL $eqn $id)]
| [ "Dependent" "Rewrite" "->"  identarg($id) ] -> [$0=(SubstHypInConcl_LR $id)]
| [ "Dependent" "Rewrite" "<-"  identarg($id) ] -> [$0=(SubstHypInConcl_RL $id)]
.




Syntax tactic deqhypy (DEqHyp $id) 0
        "Simplify_eq " <$id:"identarg":*>.

Syntax tactic deqconcl (DEqConcl) 0
        "Simplify_eq".

Syntax tactic discr_id (DiscrHyp $id) 0
        "Discriminate " <$id:"Ident":*>.

Syntax tactic discr (Discr) 0
        "Discriminate" .

Syntax tactic inj_id (InjHyp $id) 0
        "Injection " <$id:"Ident":*>.

Syntax tactic inj (Inj) 0
        "Injection".

Syntax tactic rewriteLR_in (RewriteLRin $c $h) 0
      	"Rewrite " <$c:"Command":*> " in " <$h:"Ident":*>.

Syntax tactic rewriteRL_in (RewriteRLin $c $h) 0
      	"Rewrite <- " <$c:"Command":*> " in " <$h:"Ident":*>.


Syntax tactic DRewriteLR (SubstHypInConcl_LR $id) 0
    "Dependent Rewrite -> "  <$id:"Ident":*>.

Syntax tactic DRewriteRL (SubstHypInConcl_RL $id) 0
    "Dependent Rewrite <- "  <$id:"Ident":*>.


Syntax  tactic cutrewriteLR (SubstConcl_LR $eqn) 0
  "CutRewrite -> " <$eqn:"Command":*>.

Syntax tactic cutrewriteLRin (SubstHyp_LR $eqn $id) 0
  "CutRewrite -> "  <$eqn:"Command":E>  " in " <$id:"Ident":*> .

Syntax tactic cutrewriteRL  (SubstConcl_RL $eqn) 0
 "CutRewrite <- " <$eqn:"Command":E> .

Syntax tactic cutrewriteRLin (SubstHyp_RL $eqn $id) 0
 "CutRewrite <- "  <$eqn:"Command":E>  " in " <$id:"Ident":*> .










(* $Id: Equality.v,v 1.8 1996/02/12 15:55:29 cpaulin Exp $ *)
