

(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Inv.v                                   *)
(****************************************************************************)

Require Export Equality.

Declare ML Module "inv" "leminv".

Grammar tactic simple_tactic :=
  [ "Inversion_clear" identarg($id) ] -> [$0 = (Inv $id)]
| [ "Inversion_clear" identarg($id) "in" ne_identarg_list($l) ]
  -> [$0 = ($OPER{InvIn} ($CONS $id $l))]


| [ "Dependent" "Inversion_clear" identarg($id) ] -> [$0 = (DInv $id)]
| [ "Dependent" "Inversion_clear" identarg($id) "with" comarg($c) ] 
  -> [$0 = (DInvWith $id $c) ]


| [ "Inversion" identarg($id) ] -> [$0 = (InvNoThining $id)]
| [ "Inversion" identarg($id) "in" ne_identarg_list($l) ]
  -> [$0 = ($OPER{InvInNoThining} ($CONS $id $l))]



| [ "Dependent" "Inversion" identarg($id) ] -> [$0 = (DInvNoThining $id)]
| [ "Dependent" "Inversion" identarg($id) "with" comarg($c) ] 
  -> [$0 = (DInvNoThiningWith $id $c) ]



| [ "Simple" "Inversion" identarg($id) ] -> [$0 = (HalfInv $id)]


| [ "Use" "Inversion" identarg($id) comarg($c) ]
  -> [$0 = (UseInversionLemma $id $c)]

| [ "Use" "Inversion" identarg($id) comarg($c) "in" ne_identarg_list($l) ]
  -> [$0 = ($OPER{UseInversionLemmaIn} ($CONS $id ($CONS $c $l)))]
.

Grammar vernac vernac :=
  [ "Derive" "Inversion_clear"  identarg($na) identarg($id) "." ]
  -> [$0 = (MakeInversionLemmaFromHyp {1} $na $id)]

| [ "Derive" "Inversion_clear" numarg($n) identarg($na) identarg($id) "." ]
  -> [$0 = (MakeInversionLemmaFromHyp $n $na $id)]

| [ "Derive" "Inversion_clear"  identarg($na) 
       "with" comarg($com) "Sort" sortdef($s) "." ]
  -> [$0 = (MakeInversionLemma $na $com (COMMAND $s))]

(* 
| [ "Derive" "Inversion_clear"  identarg($na) "with" comarg($com) "." ]
  -> [$sort=<<Prop>>; 
      $0 = <:vernac:<Derive Inversion_clear $na with $com Sort $sort.>>]
*)
| [ "Derive" "Inversion_clear"  identarg($na) "with" comarg($com) "." ]
  -> [$0 = (MakeInversionLemma $na $com (COMMAND (PROP{Null})))]

| [ "Derive" "Inversion" identarg($na) "with" comarg($com) "Sort" sortdef($s) "." ]
  -> [$0 = (MakeSemiInversionLemma $na $com (COMMAND $s))]

| [ "Derive" "Inversion" identarg($na) "with" comarg($com) "." ]
  -> [$0 = (MakeSemiInversionLemma $na $com (COMMAND (PROP{Null})))]

| [ "Derive" "Inversion" identarg($na) identarg($id) "." ]
  -> [$0 = (MakeSemiInversionLemmaFromHyp {1} $na $id)]

| [ "Derive" "Inversion"  numarg($n) identarg($na) identarg($id) "." ]
  -> [$0 = (MakeSemiInversionLemmaFromHyp $n $na $id)]


| [ "Derive" "Dependent" "Inversion" identarg($na) 
      "with" comarg($com) "Sort" sortdef($s) "." ]
  -> [$0 = (MakeDependentSemiInversionLemma $na $com (COMMAND $s))]

| [ "Derive" "Dependent" "Inversion_clear"  identarg($na) 
       "with" comarg($com)  "Sort" sortdef($s)"." ]
  -> [$0 = (MakeDependentInversionLemma $na $com (COMMAND $s))]


.


Syntax tactic  inv (Inv $id)  0
   "Inversion_clear "   <$id:"ident":*>.

Syntax tactic invin ($OPER{InvIn} ($CONS $id $l)) 0
   "Inversion_clear "  <$id:"ident":*>  " in "  <$l:"idlist":*>
     with $L:=($OPER{NEIDENTARGLIST} $l).

Syntax tactic  dinv (DInv $id)  0
   "Dependent Inversion_clear "   <$id:"ident":*>.

Syntax tactic dinv_with  (DInvWith $id $c) 0
   "Dependent Inversion_clear "  <$id:"ident":*> " with " <$c:"comarg":*>.



Syntax tactic  inv_nothining (InvNoThining $id)  0
        "Inversion "   <$id:"ident":*>.

Syntax tactic invin_nothining ($OPER{InvInNoThining} ($CONS $id $l)) 0
   "Inversion "  <$id:"ident":*>  " in "  <$L:"idlist":*>
     with $L:=($OPER{NEIDENTARGLIST} $l).

Syntax tactic  dinv_nothining (DInvNoThining $id)  0
     "Dependent Inversion "   <$id:"ident":*>.

Syntax tactic dinv_nothining_with  (DInvNoThiningWith $id $c) 0
   "Dependent Inversion "  <$id:"ident":*> " with " <$c:"comarg":*>.



Syntax tactic halfinv (HalfInv $id)   0
   "Simple Inversion " <$id:"ident":*>.




Syntax vernac makeinvlemmafromhyp_n (MakeInversionLemmaFromHyp ($PRIM $n) $na $id)   0
        "Derive Inversion_clear " <$n:"numarg":*> " "  <$na:"ident":*> " "  <$id:"ident":*>.

Syntax vernac makeinvlemmafromhyp_1 (MakeInversionLemmaFromHyp {1} $na $id)   0
        "Derive Inversion_clear " <$na:"ident":*> " "  <$id:"ident":*>.

Syntax vernac makeinvlemma (MakeInversionLemma $na $com $s) 0 
    "Derive Inversion_clear "  <$na:"ident":*> 
      " with " <$com:"comarg":*> " Sort " <$s:"comarg":*>.



Syntax vernac makesemiinvlemma (MakeSemiInversionLemma $na $com $s)   0
        "Derive Inversion " <$na:"ident":*> 
         " with " <$com:"comarg":*> " Sort " <$s:"sort":*>.


Syntax vernac makesemiinvlemmafromhyp_n (MakeSemiInversionLemmaFromHyp ($PRIM $n) $na $id)   0
        "Derive Inversion " <$n:"numarg":*> " "  <$na:"ident":*> " "  <$id:"ident":*>.

Syntax vernac makesemiinvlemmafromhyp_1 (MakeSemiInversionLemmaFromHyp {1} $na $id)   0
        "Derive Inversion " <$na:"ident":*> " "  <$id:"ident":*>.



Syntax vernac make_semi_dependent_invlemma 
 (MakeDependentSemiInversionLemma $na $com $s) 0
  "Derive Dependent Inversion " <$na:"ident":*> 
       " with " <$com:"comarg":*> " Sort " <$s:"sort":*>.



Syntax vernac make_dependent_invlemma (MakeDependentInversionLemma $na $com $s)0
  "Derive Dependent Inversion_clear " <$na:"ident":*> 
       " with " <$com:"comarg":*> " Sort " <$s:"sort":*>.


(* Use rules *)

Syntax tactic useinvlema (UseInversionLemma $id $c) 0
    "Use Inversion"  <$id:"ident":*> [1 0] <$c:"comarg":*>.

Syntax tactic useinvlemain ($OPER{UseInversionLemmaIn} ($CONS $id ($CONS $c $l)))   0
  "Use Inversion "  <$id:"ident":*> " " <$c:"comarg":*> " in "  <$L:"idlist":*>
   with $L:=($OPER{NEIDENTARGLIST} $l).  

(* $Id: Inv.v,v 1.6 1996/02/12 15:55:35 cpaulin Exp $ *)
