(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Point.v                                  *)
(****************************************************************************)

Require Export DHyp.

Declare ML Module "point".

Grammar tactic simple_tactic :=
  [ "PHyp" identarg($id) ] -> [$0 = (PHyp $id)]
| [ "CPHyp" identarg($id) ] -> [$0 = (CPHyp $id)]
| [ "PConcl" ] -> [$0 = (PConcl)]
| [ "P" identarg($id) ] -> [$0 = (PHyp $id)]
| [ "CP" identarg($id) ] -> [$0 = (CPHyp $id)]
| [ "P" ] -> [$0 = (PConcl)]
| [ "PointStart" numarg($n) comarg($c) ] -> [$0 = (PointStart $n $c)]
| [ "PointEnd" ] -> [$0 = (PointEnd)]
| [ "Point" comarg($c) ] -> [$0 = (Point $c)]
| [ "Point" comarg($c) "in" identarg($id)] -> [$0 = (Point $id $c)]
| [ "CPoint" comarg($c) ] -> [$0 = (CPoint $c)]
| [ "CPoint" comarg($c) "in" identarg($id)] -> [$0 = (CPoint $id $c)]
.


Abstraction mark [0] := <<M>>M.

Grammar command command0 :=
  [  "#" "[" command($c) "]" ]
  -> [$0 = (ABST{#Point#mark.cci} $c)]
.

Grammar vernac point_result :=
  [ "Conclusion" ] -> [ $0 = (CONCL) ]
| [ "Hypothesis" numarg($rank) ] -> [ $0 = (HYP $rank)]
| [ "None" ] -> [$0= (NONE)]
.

Grammar vernac point_result_list :=
  [ point_result($r) ] -> [$0 = ($LIST $r)]
| [ point_result($r) point_result_list($l)] -> [$0 = ($CONS $r $l)]
.

Grammar vernac vernac :=
  [ "Hint" "Point" destruct_location($loc) identarg($na) comarg($hyptyp) numarg($pri)
     "[" numarg($sub) point_result_list($rl) "]" spatarg($tac) "." ]
  -> [$0 = (HintPoint $na (ANY $loc) $hyptyp $pri $sub (ANY ($OPER{result_list} $rl)) $tac)]
| [ "Hint" "Point" destruct_location($loc) identarg($na) comarg($hyptyp) numarg($pri)
    "[" numarg($sub) point_result_list($rl) "]" spatarg($tac) "with" meta_binding_list($bl) "." ]
  -> [$0 = ($OPER{HintPoint} ($APPEND ($LIST $na (ANY $loc) $hyptyp $pri $sub (ANY ($OPER{result_list} $rl)) $tac)
	                                        $bl))]
| [ "Print" "Hint" "Point" "." ] -> [$0 = (PrintPointTable)]
.


Syntax tactic phyp (PHyp $id)  0
        "P " <$id:"identarg":*>.

Syntax tactic chyp (CPHyp $id)  0
        "CP " <$id:"identarg":*>.


Syntax tactic pconcl (PConcl)   0
        "P".


Syntax tactic pointstart (PointStart ($PRIM $n) $c)   0
        "PointStart " <$n:"numarg":*> [1 0] <$c:"comarg":*>.

Syntax tactic pointend  (PointEnd)  0
        "PointEnd".

Syntax tactic point  (Point $c)  0
        "Point " <$c:"comarg":*>.

Syntax tactic pointid (Point $id $c)   0
        "Point " <$c:"comarg":*> " in " <$id:"ident":*>.

Syntax tactic cpoint (CPoint $c)  0
        "CPoint " <$c:"comarg":*>.

Syntax tactic cpointin (CPoint $id $c)  0
        "CPoint " <$c:"comarg":*>  " in " <$id:"ident":*>.

Syntax constr pointmark (ABST{#Point#mark.cci} $c) 0
        "#" "[" <$c:"command":*> "]".

Syntax vernac concl (CONCL) 0
        "Conclusion".

Syntax vernac hyp (HYP ($PRIM $rank))  0
        "Hypothesis " <$rank:"numarg":*>.

Syntax vernac none (NONE)  0
        "None".

Syntax vernac point_result_list_cons ($OPER{POINTRESULTLIST}  ($CONS $r $l)) 0
        <$r:"point_res":*> " " <$L:"pointreslist":*>
        with $L:=($OPER{POINTRESULTLIST}  $L).
Syntax vernac point_result_list_one ($OPER{POINTRESULTLIST} ($LIST $r)) 0
        <$r:"point_res":*>.

Syntax vernac hintpoint 
      (HintPoint $na (ANY $loc) $hyptyp ($PRIM $n) ($PRIM $s) (ANY ($OPER{result_list} $rl)) $tac) 0
       "Hint Point " <$loc:"destrloc":*> [1 0] <$na:"ident":*> [1 0] <$hyptyp:"comarg":*>
         [1 0]  <$n:"numarg":*> [1 0] "["  <$s:"numarg":*> [1 0] <$RL:"pointreslist":*> [1 0]
         <$tac:"spatarg":*> "."
        with $RL:=($OPER{POINTRESULTLIST} $rl).

Syntax vernac hintpointlist 
   ($OPER{HintPoint}  ($CONS $na ($CONS (ANY $loc) ($CONS $hyptyp ($CONS ($PRIM $n) ($CONS ($PRIM $s) 
        ($CONS (ANY ($OPER{result_list} $rl)) ($CONS $tac  $bl)))))))) 0
    "Hint Point " <$loc:"destrloc":*> [1 0] <$na:"ident":*> [1 0] <$hyptyp:"comarg":*>
      [1 0]  <$n:"numarg":*> [1 0] "["  <$s:"numarg":*> [1 0] <$RL:"pointreslist":*> [1 0]
       <$tac:"spatarg":*>  [1 0] "with" <$BL:"metabindlist":*> "."
      with $BL := ($OPER{METABINDINGLIST} $bl).
        
(*
Syntax vernac printpointable (PrintPointTable) 0
        "Print Hint Point."
        
*)




(* $Id: Point.v,v 1.3 1996/02/12 15:55:41 cpaulin Exp $ *)
