(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                Program.v                                 *)
(****************************************************************************)

Declare ML Module "progmach" "program".

Grammar command command0 :=
  [ "[{" ident($id1) ":" command($c)"}"  "]" command($c2) ]
  -> [$0 = (LAMBDA (XTRA{"COMMENT"} $c) [$id1]$c2)]
| [ "[{" ident($id1) "," binder($idl) ":" command($c)"}" "]" command($c2) ]
  -> case $idl of
         ($OPER{IDBINDER} $L) ->
     [$0 = (LAMBDALIST (XTRA{"COMMENT"} $c) ($SLAML ($CONS (SOME $id1) $L) $c2))]
     esac
.

(*
Grammar command make_binder :=
  [ binder($b) ":" command($t) ";" make_binder($l) ] 
     -> case $l of ($LIST $c $l1) ->
        case $b of ($OPER{IDBINDER} $idl) ->
 	  [ $0 = ($LIST $c (LAMBDALIST $t ($SLAML $idl $l1))) ]
	esac
	esac
| [ binder($b) ":" command($t) ")" "=" command($c) "in" command($c1) ] 
     -> case $b of ($OPER{IDBINDER} $idl) ->
 	  [ $0 = ($LIST $c (LAMBDALIST $t ($SLAML $idl $c1))) ]
	esac
.
*)
Grammar command make_binder :=
  [ ";" binder($b) ":" command($t) make_binder($l) ] 
     -> case $l of ($LIST $c $l1) ->
        case $b of ($OPER{IDBINDER} $idl) ->
 	  [ $0 = ($LIST $c (LAMBDALIST $t ($SLAML $idl $l1))) ]
	esac
	esac
| [ ";" binder($b) ":" command($t) ")" "=" command($c) "in" command($c1) ] 
     -> case $b of ($OPER{IDBINDER} $idl) ->
 	  [ $0 = ($LIST $c (LAMBDALIST $t ($SLAML $idl $c1))) ]
	esac
.
(*
Grammar command make_binder :=
  [ binder($b) ")" "=" command($c) "in" command($c1) ] 
     -> case $b of ($OPER{IDBINDER} $idl) ->
 	  [ $0 = ($LIST $c (LAMBDALIST (XTRA{"ISEVAR"}) ($SLAML $idl $c1))) ]
	esac
.
*)
Grammar command command0 :=
  [ "[" ident($id1) "," command:binder($id) "]" command($c) ] -> 
  case $id of ($OPER{IDBINDER} $idl) ->
  [ $0 = (LAMBDALIST (XTRA{"ISEVAR"}) ($SLAML ($CONS (SOME $id1) $idl) $c)) ] 
  esac
| [ "[" ident($id1) "]" command($c) ] -> [$0 = (LAMBDA (XTRA{"ISEVAR"}) [$id1]$c)].

(*
Grammar command command1 :=
  [ "<" lcommand($l1) ">" "let" "(" make_binder($l) ]
  -> case $l of ($LIST $c $l2) ->
       [$0 = (MUTCASE $l1 $c $l2)]
     esac.
*)
Grammar command command1 :=
  [ "<" lcommand($l1) ">" "let" "(" binder($b) ")" "=" command($c) "in" command($c1) ]
  -> case $b of ($OPER{IDBINDER} $idl) ->
       [$0 = (MUTCASE $l1 $c (LAMBDALIST (XTRA{"ISEVAR"}) ($SLAML $idl $c1)))]
     esac
| [ "<" lcommand($l1) ">" "let" "(" binder($b) ":" command($t) make_binder($l) ]
  -> case $b of ($OPER{IDBINDER} $idl) ->
      	case $l of ($LIST $c $l2) ->
       	 [$0 = (MUTCASE $l1 $c (LAMBDALIST $t ($SLAML $idl $l2)))]
       	esac
     esac
| [ "<" lcommand($l1) ">" "let" "(" binder($b) ":" command($t) ")" "=" command($c) "in" command($c1) ]
  -> case $b of ($OPER{IDBINDER} $idl) ->
       	 [$0 = (MUTCASE $l1 $c (LAMBDALIST $t ($SLAML $idl $c1)))]
     esac
.

Grammar command command1 :=
 [ "<" lcommand($l1) ">" "if" command($c1) "then" command($c2) "else" command($c3) ]
  -> [$0 = (MUTCASE $l1 $c1 $c2 $c3)]

.

Grammar command command1 :=
 [ "<" lcommand($l1) ">" "rec" ident($id1) "::" "::" "{" command($c1) "}" "[" ident($id2) ":" command($c2) "]" command($c3) ]
  -> [ $wf = (CONST {#Wf#well_founded_induction.fw});
       $L =  (XTRA{"COMMENT"} $c1);
       $B =  (LAMBDA $c2 [$id2](LAMBDA (PROD $c2 [<>]$l1) [$id1]$c3));
       $0 =  ($OPER{APPLIST} ($LIST $wf $c2 $L $l1 $B)) ]
.

Grammar command command5 :=
  [ lassoc_command4($c1) "::" "::" "{" command5($c2) "}" ]
  -> [$0 = (XTRA{"ANNOT"} $c1 $c2)].

Grammar tactic simple_tactic :=
   [ "Realizer" comarg($c) ] -> [$0 = (Realizer $c)]

|  [ "Program" ] -> [$0 = (Program)]

|  [ "Program_Expand" ] -> [$0 = (Program_Expand)]

|  [ "Program_all" ] -> [$0 = (Program_all)]
.

Grammar vernac vernac :=
  [ "Show" "Program" numarg($n) "." ] -> [$0 = (ShowProgram $n)]

| [ "Show" "Program" "." ] -> [$0 = (ShowProgram)]
.

Syntax constr command_comment (XTRA{"COMMENT"} $c) 1 <$c:"command":L>.

Syntax constr comment (LAMBDA (XTRA{"COMMENT"} $c) [$id1]$c2) 1
  [<hov 0> "{" <$id1:"Ident":*> ":" <$c:"command":E> "}" <$c2:"command":E> ].

(***** Does not work
Syntax constr let_prog 
      (MUTCASE $l1 $c2 ($CONS ($OPER{BINDERLIST} $bl) $c)) 1
[<hov 0> "<" <$l1:"lcommand":E> ">" "let" "("  <$BL:"binders":*> ") = "
          <$c2:"command":E> [1 1] [<hov 2> "in "  <$c:"command":E> ] ]
with $BL:=($OPER{NEBINDERSEMILIST} $bl) .
*****)

Syntax constr ne_binder_semi_list_cons 
        ($OPER{NEBINDERSEMILIST} ($CONS $bind $lb)) 0
        <$bind:"Binder":*> ";" <$TAIL:"ListBind":*>
        with $TAIL := ($OPER{NEBINDERSEMILIST} $lb).

Syntax constr ne_binder_semi_list_one ($OPER{NEBINDERSEMILIST} ($LIST $bind)) 0
        <$bind:"Binder":*>.

Syntax constr binder ($OPER{BINDER} ($CONS $c $idl)) 0
        <$IDENTLIST:"ListIdent":*> ":" <$c:"Command":*>
        with $IDENTLIST := ($OPER{NEIDENTARGCOMMALIST} $idl).

Syntax constr ne_identarg_comma_listcons 
    ($OPER{NEIDENTARGCOMMALIST} ($CONS $id $idl)) 0
        <$id:"Ident":*> "," <$TAIL:"Tail":*>
        with  $TAIL := ($OPER{NEIDENTARGCOMMALIST} $idl).

Syntax constr ne_identarg_comma_listone 
        ($OPER{NEIDENTARGCOMMALIST} ($LIST $id)) 0
        <$id:"Ident":*>.

Syntax constr prog_rec
 ($OPER{APPLIST}
   ($LIST (CONST {#Wf#well_founded_induction.fw})
           $c2 (XTRA{"COMMENT"} $c1) $l1 
           (LAMBDA $c2 [$id2](LAMBDA (PROD $c2 [<>]$l1) [$id1]$c3)))) 1 
 [<hov 0> "<" <$l1:"lcommand":E> ">rec " <$id1:"Ident":*> " :: :: " 
    "{" <$c1:"command":E> "}" [0 0] 
          "[" <$id2:"Ident":*> ":" <$c2:"command":E> "]" <$c3:"command":E> ].

(****
Syntax constr annot (XTRA{"ANNOT"} $c1 $c2) 5
        [<hov 0> <$c1:"lassoc":L> " :::: " "{" <$c2:"command":E> "}"] .
***)


Syntax constr  annot (XTRA{"ANNOT"} $c1 $c2) 5
        [<hov 0> <$c1:"lassoc":L> " :::: " "{" <$C2:"command":E> "}"] 
         with $C2:=(PPUNI$COMMAND $c2).


Syntax tactic realizer  (Realizer $c) 0
	"Realizer " <$c:"Term":*>.

Syntax tactic program  (Program) 0
	"Program".

Syntax tactic program_expand  (Program_Expand) 0
	"Program_Expand".

Syntax tactic program_all  (Program_all) 0
	"Program_all".

Syntax vernac show_prog_n (ShowProgram ($PRIM $n)) 0
	"Show Program " <$n:"int":*>.

Syntax vernac show_prog (ShowProgram) 0
	"Show Program".

(*

let parse s = progmach__raw_prog_of_progcom mt_evc (initial_sign()) (with_tokens command s);;
let tck c = progmach__fexecute_prog mt_evc (initial_fsign()) (initial_sign()) c;;

*)


(* $Id: Program.v,v 1.11 1996/02/12 15:55:44 cpaulin Exp $ *)
