(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 cofix.ml                                 *)
(****************************************************************************)
#open "std";;
#open "pp";;
#open "stdpp";;
#open "names";;
#open "proof_trees";;
#open "pfedit";;
#open "names";;
#open "tacmach";;
#open "more_util";;

let cofix_tac = 
    register_tactic "Cofix" 
    (fun [IDENTIFIER id] -> cofix id)
    (fun _ _ (_,[IDENTIFIER id]) ->
	  [< 'S"Cofix" ; 'SPC ; print_id id >])
;;

let simple_cofix_tac = 
    register_tactic "SimpleCofix" 
            (fun [] -> let l = pfedit__list_proofs() in
                       let id = id_of_string (hd l) 
                       in  cofix id)
            (fun _ _ (_,[]) ->
	           [< 'S"Cofix" >])
;;

let mutual_cofix_tac = 
    register_tactic "MutualCofix" 
    (fun ((IDENTIFIER id)::lcofix) -> 
    let rec decomp lid lar = 
   function (COFIXEXP (id,ar)::rest) 
        -> decomp (id::lid) (ar::lar) rest
          | [] -> (rev lid,rev lar) 
   in let (lid,lar) = decomp [] [] lcofix
   in (fun gl -> 
        let trad = pf_constr_of_com gl in 
        let tlar = map trad lar 
        in mutual_cofix (id::lid) tlar gl))
   (fun sigma goal (_,(IDENTIFIER id):: lfd) ->
	  [< 'S"Cofix" ; 'SPC ; print_id id;  'FNL;
          'S"with"; 'SPC ; 
     prlist_with_sep (fun () -> [<'SPC>])
      (function COFIXEXP(f,c) -> 
      (HOV 0 [< print_id f ;'S":"; pr_com sigma goal c >])) lfd >])
;;


(* $Id: cofix.ml,v 1.4 1996/02/12 15:55:54 cpaulin Exp $ *)
