(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  CAML.v                                  *)
(****************************************************************************)

(* Grammar and Syntax for extraction toward Caml light. *)

Declare ML Module "ml_import" "mlterm" "fw_env"
      	       	  "fwtoml" "optimise" "caml" "gofer".

(* Grammar. *)

Grammar vernac vernac :=

  [ "Write" "Caml" "File" stringarg($file) "[" ne_identarg_list($idl) "]" "." ]
     -> [$0 = (WRITECAMLFILE $file ($OPER{VERNACARGLIST} $idl))]

| [ "Write" "Gofer" "File" stringarg($file)
                                           "[" ne_identarg_list($idl) "]" "." ]
     -> [$0 = (WRITEGOFERFILE $file ($OPER{VERNACARGLIST} $idl))]

| [ "ML" "Import" identarg($id) ":" comarg($c) "." ]
     -> [$0 = (ML_IMPORT $id $c)]

| [ "ML" "Inductive" ne_mindnames_list($nl) "=="
    "Inductive" identarg($id) indpar($indpar) ":" comarg($c) 
    ":=" lidcom($lidcom) "." ]
     -> [$0 = (ML_ONEINDUCTIVE ($OPER{VERNACARGLIST} $nl)
      	       	       	       $id $c $indpar $lidcom)]

| [ "ML" "Inductive" ne_mindnames_list($nl) "=="
    "Mutual" indpar($indpar) "Inductive" block($indl) "." ]
     -> [$0 = (ML_MUTUALINDUCTIVE ($OPER{VERNACARGLIST} $nl)
      	       	       	       	  $indpar ($OPER{VERNACARGLIST} $indl))]

| [ "Link" identarg($id) ":=" comarg($c) "." ]
     -> [$0 = (LINK $id $c)]

| [ "Fw" "Print" identarg($id) "." ]
     -> [$0 = (FWPRINT $id)]

| [ "Poly" "Extraction" "." ] ->  [$0 = (LOOSET)]

| [ "ML" "Extraction" "." ] ->  [$0 = (LOOSEF)]

with mindnames :=
  [ identarg($id) "[" ne_identarg_list($idl) "]" ]
     -> [$0 = ($OPER{VERNACARGLIST} ($CONS $id $idl))]

with ne_mindnames_list :=
  [ mindnames($n) ne_mindnames_list($nl) ] -> [ $0 = ($CONS $n $nl) ]
| [ mindnames($n) ] -> [ $0 = ($LIST $n) ]
.


(* Syntax. *)

Syntax vernac write_caml_file (WRITECAMLFILE ($PRIM $file)
      	       	       	       	       	     ($OPER{VERNACARGLIST} $idl)) 0
    "Write Caml File " <$file:"string":*> " [ " <$L:"ListIdent":*> "]." FNL
      	with $L := ($OPER{NEIDENTARGLIST} $idl).

Syntax vernac write_gofer_file (WRITEGOFERFILE ($PRIM $file)
      	       	       	       	       	     ($OPER{VERNACARGLIST} $idl)) 0
    "Write Gofer File " <$file:"string":*> " [ " <$L:"ListIdent":*> "]." FNL
      	with $L := ($OPER{NEIDENTARGLIST} $idl).

Syntax vernac ml_import (ML_IMPORT $id $c) 0
    "ML Import " <$id:"Ident":*> " : " <$c:"CommandArg":*> "." FNL
.

Syntax vernac mindnames_nil ($OPER{MINDNAMES} ($LIST)) 0.
Syntax vernac mindnames_one ($OPER{MINDNAMES} ($LIST $id)) 0
    <$id:"Ident":*> " []".
Syntax vernac mindnames_cons ($OPER{MINDNAMES} ($CONS $id $idl)) 0
    <$id:"Ident":*> " [ " <$CN:"ListIdent":*> "]"
    with $CN := ($OPER{NEIDENTARGLIST} $idl).

Syntax vernac ne_mindnames_listnil ($OPER{NEMINDNAMESLIST} ($LIST)) 0.
Syntax vernac ne_mindnames_listone
      	       	 ($OPER{NEMINDNAMESLIST} ($LIST ($OPER{VERNACARGLIST} $n))) 0
    <$MN:"MindNames":*>
    with $MN := ($OPER{MINDNAMES} $n).
Syntax vernac ne_mindnames_listcons
      	       ($OPER{NEMINDNAMESLIST} ($CONS ($OPER{VERNACARGLIST} $n) $nl)) 0
    <$MN:"MindNames":*> " " <$TAIL:"Tail":*>
    with $MN := ($OPER{MINDNAMES} $n)
      	 $TAIL := ($OPER{NEMINDNAMESLIST} $nl).

Syntax vernac ml_oneinductive (ML_ONEINDUCTIVE ($OPER{VERNACARGLIST} $nl)
      	       	       	       	       	       $id $c $ip $lidcom) 0
    "ML Inductive " <$MNL:"MindNamesList":*>
    " == Inductive " <$id:"Name":*> " " <$IP:"IndParam":*>
    " : " <$c:"Command":*>  " := "   FNL <$lidcom:"Command":*> "." FNL
    with  $IP := (INDPAR $ip)
      	  $MNL := ($OPER{NEMINDNAMESLIST} $nl).

Syntax vernac ml_mutualind (ML_MUTUALINDUCTIVE ($OPER{VERNACARGLIST} $nl)
      	       	       	       	       	   $ip ($OPER{VERNACARGLIST} $inds)) 0
    "ML Inductive " <$MNL:"MindNamesList":*> "== " 
    "Mutual " <$IP:"IndParam":*> " Inductive" <$IND:"SpecifInd":*> "." FNL 
        with $MNL := ($OPER{NEMINDNAMESLIST} $nl)
      	     $IND := ($OPER{SPECIFIND} $inds)
             $IP:=(INDPAR $ip).

Syntax vernac link (LINK $id $c) 0
    "Link " <$id:"Ident":*> " := " <$c:"CommandArg":*> "." FNL
.

Syntax vernac fwprint (FWPRINT $id) 0
    "Fw Print " <$id:"Ident":*> "." FNL
.

(* $Id: Extraction.v,v 1.3 1996/02/12 17:36:38 cpaulin Exp $ *)
