(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                mlterm.mli                                *)
(****************************************************************************)

#open "names";;

(* identifiers of type are either parameters or type names. *)

type typeid = TYPEparam of identifier
            | TYPEname  of identifier
;;

(* ML type expressions. *)

type MLtype = TYvar  of typeid
      	    | TYapp  of MLtype list
	    | TYarr  of MLtype * MLtype
	    | TYglob of identifier
;;

(* ML inductive types. *)

type MLind == identifier list * identifier * (identifier * MLtype list) list;;

(* ML terms. *)

type MLast = MLrel  of int
      	   | MLapp  of MLast * MLast list
	   | MLlam  of identifier * MLast
	   | MLglob of identifier
	   | MLcons of int * identifier * MLast list
	   | MLcase of MLast * (identifier * identifier list * MLast) vect
	   | MLfix  of int * bool * (identifier list) * (MLast list)
	   | MLexn  of identifier
;;

(* ML declarations. *)

type MLdecl = DECLtype   of MLind list
            | DECLabbrev of identifier * (identifier list) * MLtype
            | DECLglob   of identifier * MLast
;;


(* ML names. *)

value reset_caml_names : unit -> unit;;
value is_caml_keyword : string -> bool;;
value caml_name_of : identifier list -> name -> identifier;;
value new_vars : identifier list -> int -> identifier list;;
value rename_list : identifier list -> identifier list -> identifier list;;

(* Lifting and substitution. *)

value ml_lift : int -> MLast -> MLast;;
value ml_pop : MLast -> MLast;;
value ml_subst1 : identifier list -> MLast -> MLast -> MLast;;
value ml_liftn_branch : int -> 'a * 'b list * MLast -> 'a * 'b list * MLast;;

(* Occurences. *)

value occurs : int -> MLast -> bool;;

(* $Id: mlterm.mli,v 1.4 1996/02/12 17:36:56 cpaulin Exp $ *)
