(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  dpc.ml                                  *)
(****************************************************************************)

#open "pp";;
#open "stdpp";;
#open "std";;
#open "proof_trees";;
#open "tacmach";;
#open "tactics";;
#open "tactics1";;

#open "prove";;
#open "ccidpc";;

#infix "THEN";;

let rec intros_forall gls = 
  let T = pf_concl gls
  in if is_forall_term T
     then (ForAllI THEN intros_forall) gls
     else IDTAC gls;;

let DPC_nq gls =
    let f = dpc_of_cci_fmla gls (pf_concl gls) in
    try let pf = prove_f f in 
      	tradpf [] [] pf gls
    with Not_provable_in_DPC s -> errorlabstrm "dpc__DPC_nq"
            [< 'S"Not provable in Direct Predicate Calculus" >]
 
       | No_intuitionnistic_proof n  -> errorlabstrm "dpc__DPC_nq"
            [< 'S ("Found "^(string_of_int n)^" classical proof(s) but"
		                             ^" no intuitionnistic one !") >]
;;

let DPC =
  (intros_forall THEN DPC_nq) 
;;

let DPC_l lcom =
  (intros_forall THEN ( (generalize lcom) THEN DPC) )
;;

let DPC_tac = register_atomic_tactic "DPC" DPC;;

let DPC_l_tac = register_tactic "DPC_l" 
    (fun l -> let lcom = map (fun (COMMAND c) -> c) l
              in DPC_l lcom)
    (fun sigma goal _ -> [< >]);;


(* $Id: dpc.ml,v 1.2 1996/02/12 17:37:11 cpaulin Exp $ *)
