(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                general.ml                                *)
(****************************************************************************)

(*** mem_assoc v l : returns true if (assoc v l) exists, false otherwise ****)

let mem_assoc v l = try let n = (assoc v l) in true 
                    with Not_found -> false;;

(*** glue : make the concatenation of the lists of a lists list *****)

let rec glue = fun
    (l::ll) -> union l (glue ll)
  | [] -> [];;

(*** strict_glue : same as glue, but with append instead of union *****)

let rec strict_glue = function
    (l::ll) -> l@(strict_glue ll)
  | [] -> [];;

(*** disjoint l1 l2 : returns true if the lists l1 and l2 are disjoint ******)

let disjoint l1 l2 = 
  let rec disjoint_rec = function
      (a::ll1) -> if mem a l2 then false else disjoint_rec ll1
    | [] -> true
  in disjoint_rec l1;;

(*** such_that : 'a list -> ('a -> bool) -> 'a list *******)

let such_that l P = 
  let rec such_rec = function
      a::ll -> if (P a) then a::(such_rec ll) else such_rec ll
    | [] -> []
  in such_rec l
;;

(* $Id: general.ml,v 1.2 1996/02/12 17:37:17 cpaulin Exp $ *)
