(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 graph.ml                                 *)
(****************************************************************************)

(* Given a DIRECTED graph G represented by a list of 
   (x , [neighbours of x])
   then (have_cycle G) returns true if G has a nontrivial cycle.
   Remark : G is not necessarly connected. *******)
  
type mark = NotMarked | OnThePath | AlreadyVisited;;

let df_cycle v marks G =
  let rec df_cycle_rec x = 
    (assoc x marks) := OnThePath;
       let nx  = assoc x G
    in let nb_marks = 
           it_list (fun s x -> if !(assoc x marks)=OnThePath 
      	       	       	       then s+1 else s) 0 nx
    in if nb_marks>0 
       then true
       else let isc = it_list (fun r y -> if r then true 
      	       	       	       	     else if !(assoc y marks)=AlreadyVisited 
      	       	       	       	       	  then false
				          else (df_cycle_rec y)) false nx 
	    in if isc then true
	              else ((assoc x marks) := AlreadyVisited; false)
  and df_init ls = match ls with
        (s::lls) -> if (!(assoc s marks)=NotMarked)
      	       	       & ((list_length (assoc s G))>0)
      	       	    then (if df_cycle_rec s
	                  then true
		          else df_init lls)
		    else df_init lls
      | [] -> false
  in df_init v;;

let have_cycle G =
     let v = map (fun (x,_) -> x) G
  in let marks = map (fun x -> (x,ref NotMarked)) v
  in if list_length G<2
     then false
     else let x0 = fst (hd G)
          in df_cycle v marks G;;

(*  Examples...

  have_cycle [(1,[2;3]); (2,[3]); (3,[])];;    		(false)
  
  have_cycle [(1,[2;3]); (2,[1;3]); (3,[1;2])];;	(true)

  have_cycle [(1 , [6]);
      	      (2 , [3;4;5]);
	      (3 , [2;5]);
	      (4 , [2]);
	      (5 , [2;3]);
	      (6 , [1])];;			    	(true)

  have_cycle [(2,[3]); (4,[]); (1,[4]); (3,[])];;       (false)

  have_cycle [(3,[1;2]); (4,[]); (2,[4]); (1,[4])];;    (false)

*)


(* $Id: graph.ml,v 1.2 1996/02/12 17:37:21 cpaulin Exp $ *)
