(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 elim.ml                                  *)
(****************************************************************************)
#infix "o";;
#open "std";;
#open "initial";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "termenv";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "tacmach";;
#open "proof";;
#open "proof_trees";;
#open "clenv";;
#open "tactics";;
#open "tactics1";;
#open "tactics2";;
#open "tactics3";;

#infix "ORELSE";;
#infix "NEXT";;
#infix "THEN";;
#infix "THENF";;
#infix "THENL";;
#infix "THENS";;

let ElimClauseThen tac (SOME id) gl = elimination_then tac ([],[]) (VAR id) gl;;

let rec RepeatE recognizer =
    IfOnClause recognizer
      
    (fun cls ->
         ElimClauseThen
         (IntroElimAssumsThen
          (fun bas ->
               ThinClause cls THEN
               OnClauses (RepeatE recognizer) (map inSOME bas.assums)))
         cls)

    (fun _ -> IDTAC)
;;

let rec head_constr_bound t =
let t = strip_outer_cast(collapse_appl t)
in match t with
     DOP2(Prod,_,DLAM(_,c2)) -> head_constr_bound c2
  |  DOPN(AppL,cl) -> head_constr_bound (hd_vect cl)
  |  DOPN(Const _,_) as x -> x
  |  DOPN(MutInd _,_) as x -> x
  |  DOPN(MutConstruct _,_) as x -> x
  |  VAR(_) as x   -> x
  |  _                       -> error "Bound head variable";;

let head_constr c = head_constr_bound c;;

let head_in l c = listset__memb l (head_constr c);;

let RepeatEOn id l gls =
let l = map (pf_global gls) l in
    RepeatE (fun (_,T) -> head_in l T) (SOME id) gls
;;

let RepeatEOn_tac =
let gentac =
    register_tactic "RepeatE"
    (fun al -> (match (map (fun (IDENTIFIER id) -> id) al) with
                [] -> error "RepeatE needs at least an id argument"
              | (id::l) -> RepeatEOn id l))
    (fun _ _ (_,al) ->
         let (id::l) = (map (fun (IDENTIFIER id) -> id) al) in
         [< 'S"RepeatE" ; 'SPC ;
            'S"[" ; prlist_with_sep (fun () -> [< 'SPC >]) print_id l ; 'S"]" ;
            'SPC ; print_id id >])
in fun ids -> gentac (map (fun id -> IDENTIFIER id) ids)
;;


#open "pattern";;
let mmk = make_module_marker ["#Prelude.obj"];;
let and_pattern = put_pat mmk "(and ? ?)";;
let prod_pattern = put_pat mmk "(prod ? ?)";;

let matches gls N pat =
let M = get_pat pat in
let (wc,_) = StartWalk gls
in try (clenv__unify [] wc M N; true)
    with UserError _ -> false
;;

let matches_list l gls N = exists (matches gls N) l;;

let RepeatAndE id gls =
    RepeatEOn id [id_of_string "and";id_of_string "prod"] gls;;

let RepeatAndE_tac = register_ident_tactic "RepeatAndE" RepeatAndE;;

(* $Id: elim.ml,v 1.3 1996/02/12 15:56:06 cpaulin Exp $ *)
