(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                include.ml                                *)
(****************************************************************************)

#directory ".";;

#infix "o";;
#open "std";;
#open "system";;
#open "more_util";;
#open "pp";;
#open "stdpp";;
#open "initial";;
#open "names";;
#open "lexer";;
#open "impuniv";;
#open "generic";;
#open "term";;
#open "sosub";;
#open "constrtypes";;
#open "unparsings";;
#open "vkey";;
#open "ast";;
#open "termact";;
#open "pureparse";;
#open "proof_trees";;
#open "proof";;
#open "summary";;
#open "libobject";;
#open "reintern1";;
#open "library";;
#open "environ";;
#open "termenv";;
#open "termast";;
#open "parsing";;
#open "printer";;
#open "reduction";;
#open "typing";;
#open "extraction";;
#open "mach";;
#open "states";;
#open "dischcore";;
#open "variables";;
#open "constants";;
#open "indtypes";;
#open "discharge";;
#open "trad";;
#open "command";;
#open "pretty";;
#open "refiner";;
#open "tacmach";;
#open "pfedit";;
#open "clenv";;
#open "tactics";;
#open "auto";;
#open "discr";;
#open "tacinterp";;
#open "tacentries";;
#open "vernacinterp";;
#open "vernacentries";;
#open "vernac";;
#open "coqtoplevel";;

trace "load_object";;

(* TODO: virer cette ligne lorsque stream-pp utilisera format *)
let PP s = format__print_string (STRINGPP s);;

let with_tokens = lexer__with_tokens vernac_keywords;;

(* let ppterm0 = (fun x -> PPNL(term0 (GLOB nil_sign) x));; *)
let ppterm0 = (fun x -> PP(term0 (GLOB nil_sign) x));;
install_printer  (* constr *) "ppterm0";;

let parse_fterm = with_tokens command;;
let parse_ast = with_tokens command;;

let e s = (fconstruct mt_evc (initial_sign()) (parse_ast s));;
let infe s = (infconstruct mt_evc (initial_assumptions()) (parse_ast s));;
let ein gls s = (fconstruct (Project gls) (sig_it gls).hyps (parse_ast s));;
let eingl gl s = (fconstruct mt_evc (sig_it gl).hyps (parse_ast s));;

let prid id = PP [< 'S"#" ; 'S(string_of_id id) >];;
install_printer  "prid";;

let prconst (sp,j) =
    PP [< 'S"#" ; 'S(string_of_path sp) ; 'S"=" ; term0 (GLOB nil_sign) j.VAL >];;

let prvar ((id,a)) =
    PP [< 'S"#" ; 'S(string_of_id id) ; 'S":" ; term0 (GLOB nil_sign) a >];;

let prj j = PP (term0 (GLOB nil_sign) j.VAL);;
install_printer  (*judgement*) "prj";;

let prsp sp = PP[< 'S(string_of_path sp) >];;
install_printer  (* section_path *)  "prsp";;

let prgoal g = PP(prgl g);;
install_printer  (* goal *)  "prgoal";;

let prgls gls = PP(pr_gls gls);;

let prglls glls = PP(pr_glls glls);;

let prctxt ctxt = PP(pr_ctxt ctxt);;
install_printer  (* ctxt *)  "prctxt";;

let pproof p = PP(print_proof (rc_of_gc (evc__get_gc mt_evc) p.goal) nil_sign p);;
install_printer  (* proof *)  "pproof";;

let prevc evc = PP(pr_evc evc);;
install_printer  (* readable_constraints *)  "prevc";;

let prwc wc = PP(pr_evc (w_Underlying wc));;
install_printer  (* walking_constraints *)  "prwc";;

let prevd evd = PP(pr_evd evd);;
install_printer  (* evar_declaration *)  "prevd";;

let prclenv clenv = PP(pr_clenv clenv);;

let prast c = PP(print_ast c);;
install_printer  (* ast *)  "prast";;

let prsmast c = PP(print_spat c);;
install_printer  (* spat *)  "prsmast";;

let prprodn p = PP(print_production (id_of_string"$ill-formed") p);;
install_printer  (* production *)  "prprodn";;

let pr_pitem pi = PP(print_prod_item (id_of_string"$ill-formed") pi);;
install_printer  (* production_item *)  "pr_pitem";;

let pr_cgram cg = PP(print_cgram cg);;
install_printer  (* compiled_grammar *)  "pr_cgram";;

let pr_pgram pg = PP(print_pgram pg);;
install_printer  (* primitive_grammar *)  "pr_pgram";;

let pract a = PP(fmt_act a);;
install_printer  (* action *)  "pract";;

let pr_constobj cobj = PP(print_constant_object 0 cobj);;
install_printer  (* constant_object *)  "pr_constobj";;

let p_uni u = 
    [< 'S(string_of_path u.u_sp) ;
       'S "." ;
      'INT u.u_num >]
;;
let print_uni u = (PP (p_uni u));;

let p_arc = function
    Undef -> [< 'S"<undef>">]
  | Arc(u,r) ->
    let spc = (fun () -> [< 'SPC >]) in
    let rec collrec (ge,gt,equiv) = function
        Terminal -> (ge,gt,equiv)
      | impuniv__Greater(true,v,r) -> collrec (ge,v::gt,equiv) r
      | impuniv__Greater(false,v,r) -> collrec (v::ge,gt,equiv) r in
    let (ge,gt,equiv) = collrec ([],[],[]) r
    in [< p_uni u ; 'SPC ;
        'S"GE: " ; 'S"[" ; prlist_with_sep spc p_uni ge; 'S"]" ; 'SPC ;
        'S"GT: " ; 'S"[" ; prlist_with_sep spc p_uni gt; 'S"]" ; 'SPC ;
        'S"EQ: " ; 'S"[" ; prlist_with_sep spc p_uni equiv; 'S"]" >]
;;
(* let print_arc a = (PPNL(p_arc a));; *)
let print_arc a = PP [< p_arc a ; 'FNL >];;

let print_universes u =
let am = avm__new uni_ord in
let am' = avm__unfreeze u am in
let graph = avm__toList am'
in prlist_with_sep (fun () -> [< 'FNL >])
    (fun (_,Arc(u,r)) -> p_arc(Arc(u,r))
       | (_,Undef) -> anomaly "UNI") graph
;;

let pp_universes u = PP [< 'S"[" ; print_universes u ; 'S"]" >];;
install_printer  (* universe *)  "print_uni";;
install_printer  (* arc *)  "print_arc";;

let pp_libobj obj =
    try let cobj = outConstant obj
        in PP(print_constant_object 0 cobj)
    with Failure _ -> PP [< 'S"#<libobj: " ; 'S(object_tag obj) ; 'S">" >];;
install_printer  (* object *)  "pp_libobj";;

let curgoal = ref (NONE : goal sigma option);;
let getgoal() =
  try (by(fun g -> curgoal := SOME g; failwith "caught");failwith "caught")
  with Failure "caught" -> outSOME !curgoal
;;


(* $Id: include.ml,v 1.5 1996/02/12 15:56:22 cpaulin Exp $ *)
