(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                macros.ml                                 *)
(****************************************************************************)

#infix "o";;
#open "std";;
#open "more_util";;
#open "initial";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "himsg";;
#open "mach";;
#open "proof";;
#open "proof_trees";;
#open "trad";;
#open "tacmach";;
#open "tactics";;
#open "pp";;
#open "stdpp";;
#open "clenv";;
#open "termenv";;
#open "tactics";;
#open "reintern0";;
#open "reintern1";;
#open "libobject";;
#open "library";;
#open "vernacinterp";;
#open "sopattern";;
#open "tactics1";;
#open "termast";;
#open "ast";;

type macro_data =
    {mac_args : identifier list;
     mac_body : ast__spat}
;;

let ri_md m =
    {mac_args = map ri_id m.mac_args;
     mac_body = ast__ri_spat m.mac_body}
;;

type t == (identifier,macro_data) fmavm__t;;
type frozen_t == (identifier,macro_data) fmavm__frozen_t;;

let mactab = (fmavm__new (id_ord,17) : t);;

let init () = fmavm__empty mactab;;

let freeze () = fmavm__freeze mactab;;
let unfreeze fs = fmavm__unfreeze fs mactab;;

let rollback f x =
    let fs = freeze()
    in try f x
       with e -> (unfreeze fs; raise e)
;;

let add (na,md) = fmavm__add mactab (na,md);;
let lookup id = fmavm__map mactab id;;

library__declare_summary "tactic-macro"
{summary__freeze_function = freeze;
 summary__unfreeze_function = unfreeze;
 summary__init_function = init;
 summary__sync_function = (fun () -> false)}
;;

let cache_md (_,(na,md)) = add (na,md);;

let reintern_md (na,md) = (ri_id na,ri_md md);;

let specification_md x = x;;

type macro_data_object == identifier * macro_data;;

let ((inMD:macro_data_object->object),
      (outMD:object->macro_data_object)) =
    declare_object ("TACTIC-MACRO-DATA",
                    {cache_function = cache_md;
                     reintern_function = reintern_md;
                     specification_function = specification_md});;

let add_macro_hint na (ids,body) =
    add_anonymous_object
    (inMD(na,{mac_args = ids;
              mac_body = body}))
;;

vinterp_add
("TacticDefinition",
fun ((VARG_IDENTIFIER na)::
     (VARG_SPAT tacexp)::
     idl) ->
let ids = map (fun (VARG_IDENTIFIER id) -> id) idl in
fun () ->
    add_macro_hint na (ids,tacexp))
;;

let ApplyMacro id argcoms md gls =
if length argcoms <> length md.mac_args then
    errorlabstrm "ApplyMacro"
    [< 'S"ApplyMacro applied to the wrong number of arguments:"; 'SPC;
       'INT (length argcoms) ; 'S" instead of "; 'INT(length md.mac_args) >];

let astb = map2 (fun id argcom -> (id,FRAGast argcom)) md.mac_args argcoms in
let tcom = ast__ssub astb md.mac_body
in match tcom with
    FRAGast ast -> tacentries__vernac_interp ast gls
  | _ -> errorlabstrm "ApplyMacro"
    [< 'S"Malformed user tactic text" >]
;;

(* [Call id argcomlist gls]

   Will find the macro associated with the name [id], and apply it to
   the supplied arguments, and the given goal.

 *)

let CallMacro id argcoms gls =
let md = try lookup id with Not_found -> error "CallMacro"
in ApplyMacro id argcoms md gls
;;

let vernac_callmacro =
let gentac = register_tactic "CallMacro"
    (fun ((IDENTIFIER na)::al) -> CallMacro na (map (fun (COMMAND com) -> com) al))
    (fun sigma goal (_,(IDENTIFIER na)::al) ->
         let coml = map (fun (COMMAND com) -> com) al in
         [< 'S"Call" ; 'SPC ; print_id na ; 'SPC ; 'S"[" ;
            prlist_with_sep (fun () -> [< 'SPC >])
                            (pr_com sigma goal) coml ;
            'S"]" >])
in fun na coms -> gentac ((IDENTIFIER na)::(map (fun com -> (COMMAND com)) coms))
;;


(* $Id: macros.ml,v 1.2 1996/02/12 15:56:43 cpaulin Exp $ *)
