(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                prolog.ml                                 *)
(****************************************************************************)


#infix "o";;
#open "std";;
#open "more_util";;
#open "names";;
#open "vectops";;
#open "generic";;
#open "term";;
#open "reduction";;
#open "proof";;
#open "proof_trees";;
#open "tacmach";;
#open "tactics";;
#open "pp";;
#open "stdpp";;
#open "clenv";;
#open "termenv";;
#open "tacentries";;

#infix "ORELSE";;
#infix "THENS";;
#infix "THEN";;

let Assumption id gl =
    (Unify (pf_type_of gl (VAR id))
     THEN tactics__exact (VAR id)) gl
;;

let e_give_exact c gl =
    (Unify (pf_type_of gl c)
     THEN tactics__exact c) gl
;;

let vernac_e_give_exact_tac = overwriting_register_comarg_tactic "EExact"
    (fun com gls -> e_give_exact (pf_constr_of_com gls com) gls);;

let Intro = tactics__intro;;

let e_resolve_with_bindings_tac  (c,lbind) gl = 
    let (wc,KONT) = StartWalk gl in
    let t = w_hnf_constr wc (w_type_of wc c) in 
    let clause = make_clenv_binding_apply wc (c,t) lbind
    in e_res_pf KONT clause gl;;

let e_resolve_with_bindings 
    = tactic_com_bind_list e_resolve_with_bindings_tac;;

let vernac_e_resolve_with_bindings = 
    register_combindinglist_tactic "EApplyWithBindings" e_resolve_with_bindings;;

let e_resolve c gls = e_resolve_with_bindings_tac  (c,[]) gls;;

(***** Factorises dans vernac_e_resolve_with_bindings
let e_resolve c gls =
    let (wc,KONT) = StartWalk gls in
    let clause = mk_clenv_hnf_constr_type_of wc c
    in e_res_pf KONT clause gls
;;

let vernac_e_resolve = overwriting_register_comarg_tactic "EApply"
    (fun com gls -> e_resolve (pf_constr_of_com gls com) gls);;

(*********************************************************)
(*  Resolve with missing arguments                   *)
(* Now it is coherent with e_resolve, which tactics__resolve_with_tac *)
(* certainly isn't *)
(* TODO: Fix the code for tactics__resolve_with to be coherent *)
(*********************************************************)

let e_resolve_with_tac chd mlist gls =
   let (wc,KONT) = StartWalk gls in
   let clause = mk_clenv_hnf_constr_type_of wc chd in
   let clause' = clenv_constrain_missing_args mlist clause
   in e_res_pf KONT clause' gls
;;

let e_resolve_with = fun
  (c::lc) gl ->
    let translate = pf_constr_of_com gl
    in e_resolve_with_tac (translate c) (map translate lc) gl
| _ _ -> anomaly "resolve_with";;


let e_resolve_with_name_tac chd blist gl =
   let (wc,KONT) = StartWalk gl in
   let clause = mk_clenv_rename_hnf_constr_type_of wc chd in
   let clause' = clenv_match_args blist clause
   in e_res_pf KONT clause' gl
;;

let e_resolve_with_name c lc gl =
   let translate = pf_constr_of_com gl
   in let translate_list = fun (n,val) -> (n,translate val)
   in e_resolve_with_name_tac (translate c)
     (map translate_list lc) gl;;


let vernac_e_resolve_with_name =
let gentac = overwriting_register_tactic "EApplyWithName"
    (fun ((COMMAND com)::bindl) ->
         e_resolve_with_name com (map (function (BINDING b) -> b) bindl))
    (fun sigma goal (_,(COMMAND com)::bindl) ->
         [< 'S"EApply" ; 'SPC ; pr_com sigma goal com ; 'SPC ; 'S"with" ; 'SPC ;
            prlist_with_sep (fun () -> [< 'SPC >])
            (fun (BINDING b) -> pr_binding sigma goal b)
            bindl >])
in fun com bindl -> gentac ((COMMAND com)::(map (fun b -> BINDING b) bindl))
;;

let vernac_e_resolve_with =
let gentac =
    overwriting_register_tactic "EApplyWith"
    (fun al -> e_resolve_with (map (fun (COMMAND com) -> com) al))
    (fun sigma goal (_,al) ->
         let hdcom::coml = map (fun (COMMAND com) -> com) al
         in [< 'S"EApply" ; 'SPC ; pr_com sigma goal hdcom ; 'SPC ; 'S"with" ; 'SPC ;
             prlist_with_sep (fun () -> [< 'SPC >])
                             (pr_com sigma goal)
                             coml >])
in fun ids -> gentac (map (fun id -> COMMAND id) ids)
;;
****)
   
(*#open "damb";;

let Backtrack gl =
    if Amb() then error "Backtrack"
    else IDTAC gl
;;

let Failify tac = (tac THEN Backtrack);;

let one_step l gl =
try
    FIRST (map Failify 
           ([Intro]
            @(map (fun c gl -> e_resolve c gl) (map (fun x -> VAR x) (ids_of_sign (pf_hyps gl))))
            @(map (fun c gl -> e_resolve c gl) l)
            @(map (fun id gl -> Assumption id gl) (ids_of_sign (pf_hyps gl)))
           )) gl
with UserError _ -> Fail()
;;

let msg n = (print_string "[";print_int n;print_string "]";flush std_out);;

let rec prolog l n gl =
    if n <= 0 then Fail()
    else (msg n;(one_step l THEN (prolog l (n-1))) gl)
;;

let Prolog l n gl =
    if Init() then error "prolog - failure"
    else prolog l n gl
;;
*)
let one_step l gl =
([Intro]
@(map (fun c gl -> e_resolve c gl) (map (fun x -> VAR x) (ids_of_sign (pf_hyps gl))))
@(map (fun c gl -> e_resolve c gl) l)
@(map (fun id gl -> Assumption id gl) (ids_of_sign (pf_hyps gl)))
)
;;

let msg n = (print_string "[";print_int n;print_string "]";flush std_out);;

let rec prolog l n gl =
    if n <= 0 then error "prolog - failure"
    else (msg n;let prol = (prolog l (n-1))
    in
    (FIRST (map (fun t -> t THEN prol) (one_step l gl))) gl)
;;

let Prolog l n gl =
    try (prolog l n gl)
    with UserError ("refiner__FIRST",_) -> errorlabstrm "prolog__Prolog"
                                            [< 'S "Prolog failed" >]
;;

let Prolog_tac lcom n gl =
let l = map (pf_constr_of_com gl) lcom
in Prolog l n gl
;;

let evars_of evc c = rev(evrec [] c)
    where rec evrec acc = function
    DOPN(Const sp,cl) as K ->
    if evc__in_dom evc sp then
        K::acc
    else acc
  | DOPN(_,cl) -> vect_it (fun c acc -> evrec acc c) cl acc
  | DOP2(_,c1,c2) -> evrec (evrec acc c2) c1
  | DOP1(_,c) -> evrec acc c
  | DLAM(_,c) -> evrec acc c
  | DLAMV(_,cl) -> vect_it (fun c acc -> evrec acc c) cl acc
  | _ -> acc
;;

let Instantiate n c gl =
let sigma = Project gl in
let evl = evars_of sigma (pf_concl gl) in
let (wc,KONT) = StartWalk gl in
if length evl < n then error "not enough evars";
let (DOPN(Const sp,_) as K) = nth evl n
in if defined_const sigma K then
    error "Instantiate called on already-defined evar"
   else
let wc' = w_Define sp c wc
in KONT wc' gl
;;

let Instantiate_tac n com gl =
    Instantiate n (pf_constr_of_com gl com) gl
;;

let whd_evar sigma = function
    DOPN(Const sp,cl) as K ->
    if evc__in_dom sigma sp & defined_const sigma K then
        const_value sigma K
    else K
  | x -> x
;;

let NormEvars gl =
let sigma = Project gl in
let nf_evar = strong (whd_evar sigma) and
    simplify = strong whd_betaiota
in convert_concl (nf_evar (simplify (pf_concl gl))) gl
;;

let vernac_prolog =
let gentac = overwriting_register_tactic "Prolog"
    (fun ((INTEGER n)::al) -> Prolog_tac (map (fun (COMMAND com) -> com) al) n)
    (fun sigma goal (_,(INTEGER n)::al) ->
         let coml = map (fun (COMMAND com) -> com) al in
         [< 'S"Prolog" ; 'SPC ; 'S"[" ;
            prlist_with_sep (fun () -> [< 'SPC >])
                            (pr_com sigma goal) coml ;
            'S"]" ; 'SPC ; 'INT n >])
in fun coms n -> gentac ((INTEGER n)::(map (fun com -> (COMMAND com)) coms))
;;

let vernac_instantiate =
let gentac = overwriting_register_tactic "Instantiate"
    (fun [(INTEGER n);(COMMAND com)] -> Instantiate_tac n com)
    (fun sigma goal (_,[(INTEGER n);(COMMAND com)]) ->
         [< 'S"Instantiate" ; 'SPC ; 'INT n ; 'SPC ; pr_com sigma goal com >])
in fun n com -> gentac [(INTEGER n);(COMMAND com)]
;;

let vernac_normevars =
    overwriting_register_atomic_tactic "NormEvars" NormEvars;;

(* $Id: prolog.ml,v 1.3 1996/02/12 15:57:00 cpaulin Exp $ *)
