(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                propre.ml                                 *)
(****************************************************************************)

#open "pp";;
#open "stdpp";;
#open "std";;
#open "more_util";;
#open "names";;
#open "vectops";;
#open "term";;
#open "mach";;
#open "trad";;
#open "generic";;
#open "reduction";;
#open "himsg";;
#open "environ";;
#open "termenv";;
#open "machops";;
#open "typing";;
#open "ast";;
#open "proof";;
#open "pfedit";;
#open "command";;
#open "vernacinterp";;

(* opened but not used.
#open "initial";;
#open "lexer";;
#open "parsing";;
#open "termast";; *)

(* ====== De ProPre a Coq ====== *)
(* ============================= *)

(* === A FAIRE :
   - Gerer les variables implicites
   - Utiliser Match lorsque Match_rec est inutile
   - Autoriser les parametres fonctionnels
*)

(* === PREREQUIS :
   - La clause 'Recursive Definition' fait appel aux
   constructeurs 'refl_equal' et 'refl_eqT' (cf fonction
   "refl_eqC")
*)

(* === BUGS :
   ...
*)

(* === Exceptions *)
exception ProPre_error of (int * std_ppcmds)
;;

exception Error_arbrec
;;

exception Error_filtre
;;

let perror (n,pps) = raise (ProPre_error (n,pps))
;;

(* === futil *)
let rec mapapp f = function
  [] -> []
 |x::xs -> (f x)@(mapapp f xs)
;;

let cmap f aas bs = map f (combine (aas,bs))
;;

let coupe_list_n n l = cln_rec [] (n,l)
 where rec cln_rec ys = function
  0,_ -> anomaly "coupe_list_n(0)"
 |1,(x::xs) -> (rev ys,x,xs)
 |n,(x::xs) -> cln_rec (x::ys) (pred n,xs)
 |_ -> anomaly "coupe_list_n"
;;

let map3 f l1 l2 l3  = map3_rec (l1,l2,l3)
where rec map3_rec = function
 [],[],[] -> []
|x1::l1',x2::l2',x3::l3'
 -> (f x1 x2 x3)::(map3_rec (l1',l2',l3'))
| _ -> anomaly "map3"
;;

let map4 f l1 l2 l3 l4 = map4_rec (l1,l2,l3,l4)
where rec map4_rec = function
 [],[],[],[] -> []
|x1::l1',x2::l2',x3::l3',x4::l4'
 -> (f x1 x2 x3 x4)::(map4_rec (l1',l2',l3',l4'))
| _ -> anomaly "map4"
;;

let map5 f l1 l2 l3 l4 l5 = map5_rec (l1,l2,l3,l4,l5)
where rec map5_rec = function
 [],[],[],[],[] -> []
|x1::l1',x2::l2',x3::l3',x4::l4',x5::l5'
 -> (f x1 x2 x3 x4 x5)::(map5_rec (l1',l2',l3',l4',l5'))
| _ -> anomaly "map5"
;;

let split' xyzs = split_rec [] [] [] xyzs
where rec split_rec xs ys zs = function
 [] -> (rev xs,rev ys, rev zs)
|(x,y,z)::xyzs' -> split_rec (x::xs) (y::ys) (z::zs) xyzs'
;;

let rec mapconc s f = function
  [] -> ""
 |x::[] -> f x
 |x::xs -> (f x)^s^(mapconc s f xs)
;;

(* map_or :
*)
let rec map_or f = function
 [] -> false
|x::xs -> (f x)or(map_or f xs)
;;

(* map_and : 
   ('a -> bool) -> 'a list -> bool *)
let rec map_and f = function
 [] -> true
|x::xs -> (f x)&(map_and f xs)
;;

(* map2_and : 
   ('a -> 'b -> bool) -> 'a list -> 'b list -> bool *)
let rec map2_and f = fun
 [] [] -> true
|(x::xs) (y::ys) -> (f x y)&(map2_and f xs ys)
|_ _ -> anomaly "map2_and"
;;

(* rem_assoc : 
   'a -> ('a * 'b) list -> ('a * 'b) list *)
let rec rem_assoc x = function
 [] -> []
|(y,z)::s -> if x=y then s else (y,z)::(rem_assoc x s)
;;

(* except_last : 
   'a list -> 'a list *)
let rec except_last = function
 [] -> []
|[x] -> []
|x::xs -> x::(except_last xs)
;;

(* add_last : 
   'a -> 'a list -> 'a list *)
let rec add_last x = function
 [] -> [x]
|x'::xs -> x'::(add_last x xs)
;;

(* n_tl : 
   int -> 'a list -> 'a list *)
let rec n_tl = fun
 0 xs -> xs
|n (x::xs) -> n_tl (pred n) xs
|_ _ -> anomaly "n_tl"
;;

(* mkset : 
  'a list -> 'a list *)
let rec mkset = function
 [] -> []
|x::xs -> if (mem x xs) then mkset xs else x::(mkset xs)
;;

(* string_of_name : 
   name -> string *)
let string_of_name = function
 Name id -> string_of_id id
|_ -> anomaly "string_of_name"
;;

(* id_of_ints : 
   string -> int list -> identifier *)
let id_of_ints s x =
 id_of_string (s^(mapconc "_" string_of_int x))
;;

(* null_id : 
   identifier *)
let null_id = id_of_string "..."
;;

(* id_of_name : 
   name -> identifier *)
let id_of_name = function
 Anonymous -> null_id
|Name (x) -> x
;;

(* ================================
let rec string_of_constr = function
 Prop Null -> "Prop"
|Prop Pos -> "Set"
|Type(Null,_) -> "Type"
|Type(Pos,_) -> "Type_Set"
|App(t,u) -> "("^(string_of_constr t)^" "^(string_of_constr u)^")"
|Lambda(x,t,u) 
 -> "["^(string_of_name x)^":"^(string_of_constr t)^"]"^(string_of_constr u)
|Prod(Anonymous,t,u) 
 -> (string_of_constr t)^"->"^(string_of_constr u)
|Prod(x,t,u) 
 -> "("^(string_of_name x)^":"^(string_of_constr t)^")"^(string_of_constr u)
|Const(Def(x,_),_) -> (string_of_name x)
|Var(Decl(x,_),_) -> (string_of_name x)
|_ -> "?constr"
;;

let rec string_of_com = function
 RefC x -> string_of_id x
|PropC Null -> "Prop"
|PropC Pos -> "Set"
|TypeC(Null) -> "Type"
|TypeC(Pos) -> "Type_Set"
|AppC(t,u) -> "("^(string_of_com t)^" "^(string_of_com u)^")"
|LambdaC(x,t,u) 
 -> "["^(string_of_id x)^":"^(string_of_com t)^"]"^(string_of_com u)
|ProdC(x,t,u) 
 -> "("^(string_of_id x)^":"^(string_of_com t)^")"^(string_of_com u)
|ArrowC(t,u) -> (string_of_com t)^"->"^(string_of_com u)
|_ -> "?command"
;;
================================ *)

type global_name = GLOBALNAME of constr;;

let mkPropC ctnts = OP(("PROP",[TOKid(id_of_string ctnts)]),[]);;
let mkLambdaC(id,T,U) = OP(("LAMBDA",[]),[T;SLAM(SOME id,U)]);;
let mkProdC(id,T,U) = OP(("PROD",[]),[T;SLAM(SOME id,U)]);;
let mkArrowC(T,U) = OP(("PROD",[]),[T;SLAM(NONE,U)]);;
let mkAppLC(f,l) = OP(("APPLIST",[]),f::l);;
let mkAppC(f,a) = mkAppLC(f,[a]);;
let mkRecC(Pred,c,lf) = OP(("XTRA",[TOKstr"REC"]),Pred::c::lf);;

(* === De 'construct' a 'etype' *)
type etype =
  Eset
 |Eprop
 |Evar of identifier
 |Ecste of global_name * etype list
 |Eprod of identifier * etype
 |Efleche of etype list * etype
;;

(* etype_of_constr_aux : 
   etype list * etype -> etype *)
let etype_of_constr_aux = function
 [],t -> t
|ts,t -> Efleche(rev ts,t)
;;

(* Evar0 : 
   etype *)
let Evar0 = Evar null_id
;;

(* etype_of_constr : 
   etype list -> etype list -> etype list -> sorts oper term 
   -> etype *)
let rec etype_of_constr env args_stack  prem_stack = function
 Rel n -> etype_of_constr_aux(prem_stack,nth env n)
|DOP0(Sort(Prop Pos)) -> etype_of_constr_aux(prem_stack,Eset)
|DOP0(Sort(Prop Null)) -> etype_of_constr_aux(prem_stack,Eprop)
|VAR(id) as X
 ->  etype_of_constr_aux(prem_stack,Ecste(GLOBALNAME X,args_stack))
|DOPN(Const sp,_) as X
 ->  etype_of_constr_aux(prem_stack,Ecste(GLOBALNAME X,args_stack))
|DOPN(MutInd sp,_) as X
 ->  etype_of_constr_aux(prem_stack,Ecste(GLOBALNAME X,args_stack))
|DOPN(AppL,ts) 
 -> let us = list_of_tl_vect ts in
    let u =  hd_vect ts in
     etype_of_constr env ((map (etype_of_constr env [] []) us)@args_stack) prem_stack u
|DOP2(Prod,t1,DLAM(Anonymous,t2))
 -> let t = etype_of_constr env [] [] t1 in
     (etype_of_constr (Evar0::env) args_stack (t::prem_stack) t2)
|DOP2(Prod,t1,DLAM(Name id,t2))
 -> (let x = Evar id in 
     let t = etype_of_constr env [] [] t1 in
     if (dependent (Rel 1) t2) then 
      (etype_of_constr (x::env) args_stack ((Eprod(id,t))::prem_stack) t2)
     else
      (etype_of_constr (Evar0::env) args_stack (t::prem_stack) t2))
|DOP2(Cast,c,_) -> etype_of_constr env args_stack prem_stack c
|_ -> anomaly "etype_of_constr"
;;

(* etype_of : 
   constr signature -> identifier -> etype list * etype *)
let etype_of sign x =
let j = fconstruct mt_evc sign (ASTnvar x) in
 match j.VAL with
  (DOPN(Const _,_)|VAR _ |DOPN(MutInd _,_)|DOPN(MutConstruct _,_))
   -> let t = j.TYP in
      (match (etype_of_constr [] [] [] t) with
        Efleche(ts,t) -> (ts,t)
       |t -> ([],t))
  | _ -> anomaly "etype_of"
;;

(* === Controle types fonctions *)
(* check_params_f : 
   constr signature -> (identifier * ast) list 
   -> (ast -> ast) *) 
let rec check_params_rec sign funC = function
 [] -> funC
|(X,T)::XTs
 -> (fconstruct mt_evc sign (funC T);
     check_params_rec sign (fun x -> funC (mkProdC(X,T,x))) XTs)
;;

let check_params_f sign XSs = 
 check_params_rec sign (fun x -> x) XSs
;;

(* check_type_exprC :
   ast -> ast *)
let rec check_type_exprC = function
 ASTnvar x -> ASTnvar x
|ASTnode("APPLIST",ts) 
 -> ASTnode("APPLIST",map check_type_exprC ts) 
|x -> perror(1,[<'S"bad type expression \""; pCMD(CCI,x); 'S"\"">])
;;

(* check_type_f' : 
   constr signature -> (ast -> ast) -> ast -> ast *)
let rec check_type_f' sign funC = function
 ASTnode("PROP",[ASTprim(TOKid x)]) as T
 -> (match (string_of_id x) with
      "Null" -> funC (mkPropC "Null")
     |_ -> perror(1,[<'S"bad type expression \""; pCMD(CCI,T); 'S"\".">]))
|ASTnode("PROD",[T;ASTslam(NONE,U)])
 -> let T' = funC (check_type_exprC T) in
    (match (fconstruct mt_evc sign T').TYP with
      DOP0(Sort(Prop Pos))
      -> check_type_f' sign (fun x -> funC (mkArrowC(T,x))) U
     |_ -> perror(1,[<'S"type of "; pCMD(CCI,T); 'S" is not of type Set">]))
|T -> let T' = funC (check_type_exprC T) in
      (match (fconstruct mt_evc sign T').TYP with
        DOP0(Sort(Prop Pos)) -> T'
       |_ -> perror(1,[<'S"type of ";  pCMD(CCI,T); 'S" is not of type Set">]))
;;

(* check_type_f : 
   constr signature -> (identifier * ast) list -> ast -> ast *)
let rec check_type_f sign XSs T =
try
  check_type_f' sign (check_params_f sign XSs) T
with
 ProPre_error(0,x) -> perror(20,[<'S"(parameters list) :\n"; x; 'S".">])
|ProPre_error(_,x) -> perror(20,[<'S"(type specification) :\n"; x; 'S".">])
|UserError(_,x) -> perror(20,[<'S"(type specification) :\n"; x; 'S".">])
|_ -> anomaly "check_type_f"
;;
 
(* === Des 'command's aux 'xterm's*)
type xterm = 
  Xvar of identifier
 |Xcc of identifier
 |Xcf of identifier * xterm list
 |Xct of identifier
 |Xft of identifier * xterm list
 |Xfv of identifier * xterm list
 |Xfx of identifier * xterm list
 |Xind of identifier * xterm list
;;

type symb = Is_var | Is_cc | Is_cf | Is_ft | Is_ct | Is_fx | Is_ind | Is_x;;

(* is_var : 
   constr signature -> string -> bool *)
let is_var sign x = 
 try (search_reference (GLOB sign) (id_of_string x);false)
 with (Not_found|UserError _) -> true
;;

(* symb_of_rec : constr signature -> sorts oper term -> symb *)
let rec symb_of_rec sign = function
 DOPN(MutConstruct _,_) as cx 
 -> (match whd_betaiota (type_of mt_evc sign cx) with
      DOP2(Prod,_,_) -> Is_cf
     |_ -> Is_cc)
|DOPN(MutInd _,_) -> Is_ind
|DOPN(Const _,_) as K -> Is_ft
|VAR id ->  let val = (fconstruct mt_evc sign (ASTnvar id)).VAL in
   begin match val with VAR id' -> Is_var | _ -> symb_of_rec sign val end
|_ -> Is_x
;;

(* symb_of : 
   identifier -> identifier -> identifier list 
   -> constr signature -> symb *)
let rec symb_of x fx xs sign = 
try
(if x=fx then
  Is_fx
 else
  if (mem x xs) then
   Is_var
  else
   try 
    symb_of_rec sign (global (GLOB sign) x)
   with 
    _ -> Is_var)
with
 UserError _ 
 -> perror(31, [< 'S"unknown symbol \""; 'S(string_of_id x); 'S"\"" >])
|_ -> anomaly "symb_of"
;;

(* add_last_arg : 
   xterm -> xterm -> xterm *)
let add_last_arg t = function 
 Xcf(x,ts) -> Xcf(x,add_last t ts)
|Xft(x,ts) -> Xft(x,add_last t ts)
|Xfv(x,ts) -> Xfv(x,add_last t ts)
|Xfx(x,ts) -> Xfx(x,add_last t ts)
|Xind(x,ts) -> Xind(x,add_last t ts)
|_ -> anomaly "add_last_arg"
;;

(* xterm_of_command : 
   identifier -> identifier list -> constr signature 
   -> ast -> xterm *)
let rec xterm_of_command fx xs sign = function
 ASTnvar x
 -> (match (symb_of x fx xs sign) with
      Is_var -> Xvar x
     |Is_cc  -> Xcc x
     |Is_ind -> Xind (x,[])
     |Is_ft -> Xct x (* devrait etre Is_ct (cf symb_of_rec) *)
     |_ -> perror(8,[<'S"illegal constant \""; 'S(string_of_id x); 'S"\"">]))
|ASTnode("APPLIST",((ASTnvar x)::ts)) as t
 -> (match (symb_of x fx xs sign) with
      Is_var -> Xfv(x,map (xterm_of_command fx xs sign) ts)
     |Is_cf -> Xcf(x,map (xterm_of_command fx xs sign) ts)
     |Is_ft -> Xft(x,map (xterm_of_command fx xs sign) ts)
     |Is_fx -> Xfx(x,map (xterm_of_command fx xs sign) ts)
     |Is_ind -> Xind(x,map (xterm_of_command fx xs sign) ts)
     |_ -> perror(8,[<'S"illegal constant \""; 'S(string_of_id x); 'S"\" in \""; pCMD(CCI,t); 'S"\"">]))
|t -> perror(3,[<'S"illegal expression \""; pCMD(CCI,t); 'S"\"">])
;; 

(* === Controle equations *)
(* flat_AppC : 
   ast -> string * ast list *)
let flat_AppC t = flat_AppC_rec [] t
 where rec flat_AppC_rec ll = function
  ASTnvar x -> (string_of_id x,ll)
 |ASTnode("APPLIST",t::l) -> flat_AppC_rec (l@ll) t
 |t ->  perror(3,[< 'S"illegal expression " ; pCMD(CCI,t) >])
;;

(* id_of_contents : 
   contents -> identifier *)
let id_of_contents c =
    match c with
    Pos -> id_of_string "Pos"
  | Null -> id_of_string "Null"
;;

(* valC : 
   name list -> sorts oper term -> ast *)
let rec valC e = function
 Rel n -> ASTnvar (id_of_name (nth e n))
|VAR id -> ASTnvar id
|DOP0(Sort(Prop c)) -> OP(("PROP",[TOKid (id_of_contents c)]),[])
|DOP2(Prod,t1,DLAM(Anonymous,t2)) 
 -> ASTnode("PROD",[valC e t1;ASTslam(NONE,valC (Anonymous::e) t2)])
|DOP2(Prod,t1,DLAM(Name id,t2))
 -> ASTnode("PROD",[valC e t1;ASTslam(SOME id,valC ((Name id)::e) t2)])
|DOPN(AppL,cl) -> OP(("APPLIST",[]),map_vect_list (valC e) cl)
|DOPN(MutInd _ as x,_) -> ASTnvar(id_of_global x)
|DOPN((MutConstruct(_) as val),_) -> ASTnvar(id_of_global val)
|DOPN(Const sp,_) -> ASTnvar (basename sp)
(*--- cf V5.9
|Prod(Anonymous,t,u)
 -> ArrowC(valC e t,valC e u)
|Prod(x,t,u)
 -> ProdC(id_of_name x,valC e t,valC e u)
---*)
|t -> perror(2,[< 'S"illegal type expression " ; pTERM t >])
;;

(* flatC_Prod : 
   string -> sorts oper term -> (name * ast) list * ast *)
let rec flatC_Prod_rec e Ts = function
 DOP2(Prod,t,DLAM(x,u))
 -> flatC_Prod_rec (x::e) ((x,valC e t)::Ts) u
|t -> (rev Ts,valC e t)
;;

let flatC_Prod x T = 
try
 flatC_Prod_rec [] [] T
with
  ProPre_error(2,y) 
  -> perror(0,[<y; 'S" while checking type of "; 'S(x)>])
;;

(* Subst_idC : 
   ast -> identifier -> ast -> ast *)
let rec subst_idC y x = function
 ASTnode("PROP",[c]) -> ASTnode("PROP",[c])
|ASTnvar z -> if x=z then y else (ASTnvar z)
|ASTnode("APPLIST",l) -> OP(("APPLIST",[]),map (subst_idC y x) l)
|ASTnode("PROD",[t;ASTslam(NONE,u)]) ->
 OP(("PROD",[]),[subst_idC y x t;SLAM(NONE,subst_idC y x u)])
|ASTnode("PROD",[t;ASTslam(SOME z,u)])
 -> if x=z then
     OP(("PROD",[]),[t;SLAM(SOME z,u)])
    else
     OP(("PROD",[]),[subst_idC y x t;SLAM(SOME z,subst_idC y x u)])
|_ -> anomaly "subst_idC"
;;

(* it_subst_idC :
   ast -> (name * 'a) list -> ast list -> ast *)
let rec it_subst_idC T = fun
 [] [] -> T
|((Name y,_)::xTs) (t::ts)
 -> it_subst_idC (subst_idC t y T) xTs ts
|(_::xTs) (_::ts) -> it_subst_idC T xTs ts
| _ _ -> anomaly "it_subst_idC"
;;

(* typed_vars : 
   constr signature -> ast list -> (name * ast) list 
   -> (identifier * ast) list *)
let rec typed_vars sign = fun
 [] [] -> []
|((ASTnvar y)::ts) ((Name x,T)::xTs) 
 -> let xTs' = map (fun (x',T') -> (x',subst_idC (ASTnvar y) x T')) xTs in
    if (is_var sign (string_of_id y)) then
     (y,T)::(typed_vars sign ts xTs')
    else
     (typed_vars sign ts xTs')
|((ASTnvar y)::ts) ((_,T)::xTs) 
 -> if (is_var sign (string_of_id y)) then
     (y,T)::(typed_vars sign ts xTs)
    else
     (typed_vars sign ts xTs)
|(t::ts) ((Name x,_)::xTs) 
 -> let xTs' = map (fun (x',T') -> (x',subst_idC t x T')) xTs in
    let (f,ts') = flat_AppC t in
    let (xTs'',_) = flatC_Prod f ((fconstruct mt_evc sign (ASTnvar (id_of_string f))).TYP) in
     typed_vars sign (ts'@ts) (xTs''@xTs')
|(t::ts) (_::xTs) 
 -> let (f,ts') = flat_AppC t in
    let (xTs',_) = flatC_Prod f ((fconstruct mt_evc sign (ASTnvar (id_of_string f))).TYP) in
     typed_vars sign (ts'@ts) (xTs'@xTs)
|[] _ -> perror(5,[<'S"missing argument(s)">])
|_ [] -> perror(6,[<'S"extra argument(s)">])
;;

(* AppC_fx :
   identifier -> ast list -> ast *)
let AppC_fx fx = function
 [] -> anomaly "AppC_fx"
|t::ts -> AppC_rec (mkAppC(ASTnvar fx,t)) ts
where rec AppC_rec t = function
 [] -> t
|t'::ts -> AppC_rec (mkAppC(t,t')) ts
;;

(* refl_eqC : 
   sorts oper term -> ast *)
let refl_eqC = function
 DOP0(Sort(Type(Null,_))) -> ASTnvar (id_of_string "refl_eqT")
|DOP0(Sort(Prop Pos)) -> ASTnvar (id_of_string "refl_equal")
|_ -> anomaly "refl_eqC"
;;

(* eqC :  
   ast -> ast -> ast -> sorts oper term -> ast *)
let eqC t u T = function
 DOP0(Sort(Type(Null,_)))
 -> OP(("APPLIST",[]),[ASTnvar (id_of_string "eqT");T; t; u])
|DOP0(Sort(Prop Pos))
 -> OP(("APPLIST",[]),[ASTnvar (id_of_string "eq"); T; t; u])
|_ -> anomaly "eqC"
;;

(* closeC : 
   ast -> (identifier * ast) list -> ast *)
let rec closeC t = function
 [] -> t
|(x,T)::xTs -> mkProdC(x,T,closeC t xTs)
;;

(* rem_premC : 
   ast -> ast *)
let rec rem_premC = function
 ASTnode("PROD",[_;ASTslam(_,t)]) -> rem_premC t
|t -> t
;;

(* check_left : 
   xterm list -> xterm list *)
let rec check_left = function
 [] -> []
|(Xvar x)::ts -> (Xvar x)::(check_left ts)
|(Xcc x)::ts -> (Xcc x)::(check_left ts)
|(Xcf(x,us))::ts -> (Xcf(x,check_left us))::(check_left ts)
|(Xind (x,us))::ts -> (Xind (x,check_left us))::(check_left ts)
|(Xct x)::_ -> perror(9,[< 'S"illegal symbol \"" ; 'S (string_of_id x) ; 'S"\" on left of \"=>\"" >]) 
|(Xft(f,_))::_ -> perror(9,[< 'S"illegal symbol \"" ; 'S (string_of_id f) ; 'S"\" on left of \"=>\"" >])
|(Xfv(f,_))::_ -> perror(9,[< 'S"illegal symbol \"" ; 'S (string_of_id f) ; 'S"\" on left of \"=>\"" >])
|(Xfx(f,_))::_ -> perror(9,[< 'S"illegal symbol \"" ; 'S (string_of_id f) ; 'S"\" on left of \"=>\"" >])
;;

(* check_vars_eq : 
   int -> identifier list -> (identifier * ast) list 
   -> (identifier * ast) list *)
let check_vars_eq i id_params_f typed_vars_eq =
 check_vars_eq_rec [] typed_vars_eq
where rec check_vars_eq_rec xTs_acc = function
 [] -> rev xTs_acc
|(x,(ASTnode("PROP",[p]) as propp))::xTs
 -> if (mem x id_params_f) then
     if (mem_assoc x xTs_acc) then
       check_vars_eq_rec xTs_acc xTs
     else
       check_vars_eq_rec ((x,propp)::xTs_acc) xTs
    else
     perror(0,[<'S"(equation "; 'S(string_of_int i); 'S") :\n variable \""; 'S(string_of_id x); 'S"\" should be declared in parameters list">])
|(x,T)::xTs
 -> if (mem_assoc x xTs_acc) then
     if (mem x id_params_f) then
      perror(0,[< 'S"(equation "; 'S(string_of_int i); 'S") :\n parameter \"";
                  'S(string_of_id x); 'S"\" can't occur on left of \"=>\"">])
     else
      perror(0,[<'S"(equation "; 'S(string_of_int i); 'S") :\n variable \""; 
                 'S(string_of_id x); 
                 'S"\" can't have more than one occurence on left of \"=>\"">])
     else
      check_vars_eq_rec ((x,T)::xTs_acc) xTs
;;

(* check_vars_eqs : 
   constr signature -> identifier list -> (name * ast) list 
   -> (ast list * 'a) list -> (identifier * ast) list list *)
let rec check_vars_eqs sign id_params_f typed_args_fC eqs_fC =
 check_vars_eqs_rec 1 eqs_fC
where rec check_vars_eqs_rec i = function
 [] -> []
|(ts,_)::eqs
 -> try 
     (check_vars_eq i id_params_f (typed_vars sign ts typed_args_fC))
     ::(check_vars_eqs_rec (succ i) eqs)
    with
     ProPre_error(0,x) -> perror(0,x) 
    |ProPre_error(_,x) -> perror(0,[< 'S"(equation "; 'S(string_of_int i); 'S") :\n "; x>])
;;

(* check_type_eqs : 
   constr signature -> ast list -> constr list *)
let check_type_eqs sign eqs_f =
 check_type_eqs_rec 1 eqs_f
where rec check_type_eqs_rec i = function
 [] -> []
|e::es
 ->((try 
      fconstruct mt_evc sign e
    with
     (UserError (_,x)) 
     -> perror(0,[<'S"(equation "; 'S(string_of_int i); 'S") :\n "; x>])
     | _ -> anomaly "check_type_eqs");
      check_type_eqs_rec (succ i) es)
;;

(* pair_of_eq : 
   ast -> ast list * ast *)
(*===
let rec pair_of_eq = function
 ASTnode("PROD",[_;(ASTslam(_,t))]) -> pair_of_eq t
|ASTnode("APPLIST",[_;_;ASTnode("APPLIST",_::ts);u]) -> (ts,u)
|_ -> anomaly "pair_of_eq"
;;
===*)

(* xterm_of_eqs : 
   constr signature -> identifier -> identifier list 
   -> ast list -> (xterm list * xterm) list *)
let xterm_of_eqs sign id_params_f nom_f eqs_fC =
 xterm_of_eqs_rec 1 eqs_fC
where rec xterm_of_eqs_rec i = function
 [] -> []
|(ts,u)::eqs
 -> try 
     (let ts' = map (xterm_of_command id_params_f nom_f sign) ts in
       (check_left ts',xterm_of_command id_params_f nom_f sign u)
       ::(xterm_of_eqs_rec (succ i) eqs))
    with
     ProPre_error(_,x) -> perror(0,[<'S"(equation "; 'S(string_of_int i); 'S") :\n"; x>])
;;     

(* === Etiquetage des equations  *)
type etiqt = L | De | D | C | N
;;

type etiql = Nl | Ll | Vl
;;

type eterm = 
  Qvar of etiqt * identifier
 |Qcc of etiqt * identifier
 |Qcf of etiqt * identifier * eterm list
 |Qct of etiqt * identifier
 |Qft of etiqt * identifier * eterm list
 |Qfv of etiqt * identifier * eterm list
 |Qfx of etiqt * identifier * eterm list
 |Qind of etiqt * identifier * eterm list
;;
 
(* arg_par_rec : 
   'a -> 'b list * 'a list -> 'b list *)
let rec arg_par_rec d = function
 [],[] -> []
|(t::ts),(di::dis)
   -> if d=di then (arg_par_rec d (ts,dis)) else t::(arg_par_rec d (ts,dis))
|_ -> anomaly "arg_par_rec"
;;

(* arg_par : 
   identifier -> 'a list -> constr signature -> 'a list *)
let arg_par c ts sign =
 let (dis,d) = etype_of sign c in
 arg_par_rec d (ts,dis) 
;;

(* arg_ind_rec :
   'a -> 'b list * 'a list -> 'b list *)
let rec arg_ind_rec d = function
 [],[] -> []
|(t::ts),(di::dis)
   -> if d=di then t::(arg_ind_rec d (ts,dis)) else (arg_ind_rec d (ts,dis))
|_ -> anomaly "arg_ind_rec"
;;

(* arg_ind : 
   identifier -> 'a list -> constr signature -> 'a list *)
let arg_ind c ts sign =
 let (dis,d) = etype_of sign c in
 arg_ind_rec d (ts,dis) 
;;

(* appels_ind : 
   xterm -> xterm list list *)
let rec appels_ind = function
 Xfx(_,ts) -> ts::(appels_inds ts)
|Xft(_,ts) -> (appels_inds ts)
|Xfv(_,ts) -> (appels_inds ts)
|Xcf(_,ts) -> (appels_inds ts)
|Xind(_,ts) -> (appels_inds ts)
|_ -> []

(* appels_inds : 
   xterm list -> xterm list list *)
and appels_inds = function
  [] -> []
 |t::ts 
  -> (appels_ind t)@(appels_inds ts)
;;

(* etier : 
   etiqt -> xterm list -> eterm list *)
let rec etier e ts = map etier0  ts
where rec etier0 = function
 Xvar x -> Qvar(e,x)
|Xcc c -> Qcc(e,c)
|Xct c -> Qct(e,c)
|Xcf(c,ts') -> Qcf(e,c,etier e ts')
|Xft(f,ts') -> Qft(e,f,etier e ts')
|Xfv(f,ts') -> Qfv(e,f,etier e ts')
|Xfx(f,ts') -> Qfx(e,f,etier e ts')
|Xind (x,xs) -> Qind(e,x,etier e xs)
;;

(* est_D : 
   eterm -> bool *)
let est_D = function
 Qvar(De,_) -> true
|Qvar(D,_) -> true
|Qcc(De,_) -> true
|Qcc(D,_) -> true
|Qcf(De,_,_) -> true
|Qcf(D,_,_) -> true
|_ -> false
;;

(* est_De : 
   eterm -> bool *)
let est_De = function
  Qvar(De,_) -> true
 |Qcc(De,_) -> true
 |Qcf(De,_,_) -> true
 |_ -> false
;;

(* est_C : 
   eterm -> bool *)
let est_C = function
  Qvar(C,_) -> true
 |Qcc(C,_) -> true
 |Qcf(C,_,_) -> true
 |_ -> false
;;

(* sont_C : 
   eterm list -> bool *)
let sont_C ets = map_and est_C ets
;;

(* un_De : 
   eterm list -> bool *)
let un_De ets = un_De_rec (false,ets)
 where rec un_De_rec = function
   (b,[]) -> b
  |(true,(et::ets))
   -> if (est_C et) then
        un_De_rec (true,ets)
      else
        false
  |(false,(et::ets))
   -> if (est_De et) then
        un_De_rec (true,ets)
      else
        if (est_C et) then
          un_De_rec (false,ets)
        else
          false
;;

(* etik : 
   constr signature -> xterm * xterm -> eterm *)
let rec etik sign = function
  (Xvar x,u)
  -> if (Xvar x)=u then Qvar(C,x) else Qvar(L,x)
 |(Xcc c,u)
  ->  if (Xcc c)=u then Qcc(C,c) else Qcc(L,c)
 |(Xcf(c,ts),Xvar x)
  -> if (mem (Xvar x) ts) then Qcf(De,c,etier N ts) else Qcf(L,c,etier N ts)
 |(Xcf(c,ts),Xcf(c',ts'))
  -> if (c=c') then
       let ets = cmap (etik sign) ts ts' in
       let eis = arg_ind c ets sign in
        if (sont_C eis) then 
          Qcf(C,c,ets)
        else
          if (un_De eis) then
            Qcf(D,c,ets)
          else 
            Qcf(L,c,ets) 
     else
      Qcf(L,c,etier L ts)
 |(Xcf(c,ts),_)
  -> Qcf(L,c,etier L ts)
 |(Xind (x,xs),u) -> Qind(L,x,etier N xs)
 | _ -> anomaly "etik"
;;

(* etikrecs : 
   constr signature -> xterm list -> xterm list list 
   -> eterm list list *)
let rec etikrecs sign ts uss = 
 map (cmap (etik sign) ts) uss
;;

(* etikeqs : 
   constr signature -> int -> (xterm list * xterm) list 
   -> (int * etiql * eterm list) list *)
let rec etikeqs sign n = function
  [] -> []
 |(ts,u)::es 
  -> let uss' = appels_ind u in
      if uss'=[] then
        (n,Nl,etier N ts)::(etikeqs sign (succ n) es)
      else
        (map (fun us' -> (n,Vl,us')) (etikrecs sign ts uss'))
        @(etikeqs sign (succ n) es)
;;

(* etikf : 
   constr signature -> (xterm list * xterm) list 
   -> (int * etiql * eterm list) list *)
let etikf sign es = etikeqs sign 1 es
;;

(* === Matrice de recurrence *)
type etiqc = P | I
;; 

type imatrec == int list * etiqc * etype
;;

type lmatrec == int * etiql * eterm list
;;

(* Qvar0 : 
   eterm *)
let Qvar0 = Qvar(N,null_id)
;;

(* etype_env : 
   etype * etype -> (identifier * etype) list *)
let rec etype_env = function
  (Evar x), d 
  -> [x,d]
 |(Ecste(d,ds)), (Ecste(d',ds'))
  -> mapapp etype_env (combine (ds,ds'))
 |Eset,_ -> []
 |Eprop,_ -> []
 |_ -> anomaly "etype_env"
;;

(* s_etype : 
   (identifier * etype) list -> etype -> etype *)
let rec s_etype env = function
  Eset -> Eset
 |Eprop -> Eprop
 |Evar x 
  -> if (mem_assoc x env) then
       (assoc x env)
     else 
       (Evar x)
 |Ecste(d,ds)
  -> Ecste(d,map (s_etype env) ds)
 |Eprod(x,d) 
  -> if (mem_assoc x env) then
      Eprod(x,d)
     else
      Eprod(x,s_etype env d)
 |Efleche(ds,d)
  -> Efleche(map (s_etype env) ds,d)
;;

(* subst_idE : 
   etype -> identifier -> etype -> etype *)
let rec subst_idE d x = function
 Evar y -> if x=y then d else (Evar y)
|Eprod(y,d') -> Eprod(y,subst_idE d x d')
|Ecste(y,ds) -> Ecste(y,map (subst_idE d x) ds)
|Efleche(ds,d') -> Efleche(map (subst_idE d x) ds,subst_idE d x d')
|d -> d
;;

(* cons_etypes_args_inst : 
*)
let rec cons_etypes_args_inst = fun
 (d::ds) ((Eprod(x,d'))::ds')
 -> d'::(cons_etypes_args_inst ds (map (subst_idE d x) ds'))
|(d::ds) (_::ds') -> anomaly "cons_etypes_args_inst1"
|(d::ds) [] -> anomaly "cons_etypes_args_inst2"
|[] ((Eprod(x,d'))::ds') -> anomaly "cons_etypes_args_inst3"
|[] ds' -> ds'
;;

(* cons_etypes_args_aux : 
   etype list -> etype -> etype list *)
let cons_etypes_args_aux ds = function
 Efleche(us,_) -> cons_etypes_args_inst ds us
|Ecste _ -> []
|_ -> anomaly "cons_etypes_args_aux"
;;

(* esets : 
   'a list -> etype list *)
let rec esets = function
 [] -> []
|x::xs -> Eset::(esets xs)
;;

(* cons_etypes_args_of : 
   etype -> etype list list *)
let cons_etypes_args_of  = function
 Ecste((GLOBALNAME (DOPN(MutInd _,_) as mind),ds)) ->
    let mis = mind_specif_of_mind mind in 
    let (_,typesconstr) = mis_type_mconstructs mis in 
         map (cons_etypes_args_aux ds)
             (map_vect_list (etype_of_constr (rev ds) [] []) typesconstr)
|_ ->  anomaly "cons_etypes_args_of 2"
;;

(* icols_ci' : 
   int list -> int -> 'a -> 'a list 
   -> (int list * etiqc * 'a) list *)
let rec icols_ci' x n d = function
  [] -> []
 |di::dis 
  -> if di=d then
      (add_last n x,I,d)::(icols_ci' x (succ n) d dis)
     else
      (add_last n x,P,di)::(icols_ci' x (succ n) d dis)
;;

(* icols_ci : 
   (int list * etiqc * etype) list 
   -> (int list * etiqc * etype) list 
   -> int list -> etype -> etype list 
   -> (int list * etiqc * etype) list *)
let icols_ci ic1 ic2 x d = function
 [] -> ic1@((add_last 1 x,P,Evar0)::ic2)
|ds -> ic1@(icols_ci' x 1 d ds)@ic2
;;

(* dev_icol : 
   int -> (int list * etiqc * etype) list 
   -> (int list * etiqc * etype) list list *)
let dev_icol i icol =
  (fun (ic1,(x,e,d),ic2) ->
     map (icols_ci ic1 ic2 x d) (cons_etypes_args_of d))
   (coupe_list_n i icol)
;;

(* place_ligne : 
   'a -> int -> 'a list list -> 'a list list *)
let place_ligne l n ssmat = 
  place_ligne_rec (n,ssmat)
where rec place_ligne_rec = function
  1,ls::lss -> (l::ls)::lss
 |n,ls::lss -> ls::(place_ligne_rec (pred n,lss))
 |_ -> anomaly "place_ligne"
;;

(* num_of_cons : 
   identifier -> constr signature -> int *)
let num_of_cons c sign =
 match search_reference (GLOB sign) c with
   DOPN(MutConstruct(_,i),_) -> i
 |_ -> anomaly "num_of_cons"
;;

(* dev_lmat0 : 
   ('a * etiql * eterm list) list list 
   -> 'a -> eterm list -> eterm list 
   -> constr signature -> etiql * eterm 
   -> ('a * etiql * eterm list) list list *)
let dev_lmat0 sslmat ne ts1 ts2 sign = function
  el,Qcc(_,c)
  -> place_ligne (ne,el,ts1@(Qvar0::ts2)) (num_of_cons c sign) sslmat
 |Vl,Qcf(De,c,ts)
  -> place_ligne (ne,Nl,ts1@ts@ts2) (num_of_cons c sign) sslmat
 |Ll,Qcf(De,c,ts)
  -> place_ligne (ne,Nl,ts1@ts@ts2) (num_of_cons c sign) sslmat
 |Vl,Qcf(D,c,ts)
  -> place_ligne (ne,Ll,ts1@ts@ts2) (num_of_cons c sign) sslmat
 |el,Qcf(_,c,ts)
  -> place_ligne (ne,el,ts1@ts@ts2) (num_of_cons c sign) sslmat
 |_ -> anomaly "dev_lmat0"
;;

(* dev_lmat : 
   int -> int -> constr signature 
   -> ('a * etiql * eterm list) list 
   -> ('a * etiql * eterm list) list list *)
let dev_lmat i nc sign lmat =
 (dev_lmat_rec (iterate (fun x -> []::x) nc []) lmat)
  where rec dev_lmat_rec lmat0 = function
    [] -> lmat0
   |(ne,el,ts)::lgs
    -> let (ts1,t,ts2) = coupe_list_n i ts in
        (dev_lmat_rec 
         (dev_lmat0 lmat0 ne ts1 ts2 sign (el,t))
         lgs)
;;

(*  nb_of_cons : 
    identifier ->  constr signature -> int *)
let nb_of_cons d sign =
 match (search_reference (GLOB sign) d) with
 (DOPN(MutInd _,_)) as ind 
  ->  mis_nconstr (mind_specif_of_mind ind)
 |_ -> anomaly "nb_of_cons (2)"
;;

(* id_of_Ecste : 
   etype -> identifier *)
let id_of_Ecste = function
 Ecste(GLOBALNAME (VAR id),_) -> id
|Ecste(GLOBALNAME (DOPN(MutInd _ as mind,_)),_) -> id_of_global mind
|Ecste(GLOBALNAME (DOPN(Const sp,_)),_) -> basename sp
|_ -> anomaly "id_of_ecste"
;;
 
(* numcol : 
   'a -> ('a * 'b * 'c) list -> int *)
let rec numcol x = function
 [] -> anomaly "numcol"
|(y,_,_)::ys 
 -> if x=y then 1 else (succ (numcol x ys))
;;

(* dev_mat : 
   constr signature
   -> int list * etiqc * etype 
   -> (int list * etiqc * etype) list 
     * ('a * etiql * eterm list) list 
   -> ((int list * etiqc * etype) list 
     * ('a * etiql * eterm list) list) list *)
let dev_mat sign (x,_,d) (icol,lmat) =
  let i = numcol x icol in
  let nc = nb_of_cons (id_of_Ecste d) sign in
  let lmats = (dev_lmat i nc sign lmat) in
  if (mem [] lmats) then
    perror(14,[<'S"pattern matching is not exhaustive">])
  else
   combine ((dev_icol i icol), lmats)
;;

(* icolini : 
   int -> (identifier * etype) list 
   -> (int list * etiqc * etype) list *)
let rec icolini n = function
 [] -> []
|d::ds
-> ([n],P,d)::(icolini (succ n) ds)
;;

(* matini :
   State ref -> etype list -> (xterm list * xterm) list 
   -> (int list * etiqc * etype) list 
     * (int * etiql * eterm list) list *)
let matini st ds es =
  (icolini 1 ds,etikf st es)
;;

(* === Variable eligible *)
(* est_Qvar : 
   eterme -> bool *)
let est_Qvar = function
  Qvar(_) -> true
 |_ -> false
;;

(* tout_I_var : 
   ('a * etiqc * 'b) list * eterm list 
   * ('c * etiqc * 'd) list * eterm list 
   -> bool *)
let rec tout_I_var = function
  [],[],[],[] -> true
 |[],[],(_,P,_)::ics,_::ts -> tout_I_var ([],[],ics,ts)
 |[],[],(_,I,_)::ics,(Qvar(_))::ts -> tout_I_var ([],[],ics,ts)
 |[],[],(_,I,_)::ics,_::ts -> false
 |(_,P,_)::ics0,_::ts0,ics,ts -> tout_I_var (ics0,ts0,ics,ts)
 |(_,I,_)::ics0,(Qvar(_))::ts0,ics,ts -> tout_I_var (ics0,ts0,ics,ts)
 |(_,I,_)::ics0,_::ts0,ics,ts -> false
 |_ -> anomaly "tout_i_var"
;;

(* var_elig : 
   ('a * etiqc * 'b) list -> eterm list 
   -> etiql * bool list * eterm list * ('a * etiqc * 'b) list 
   -> bool list *)
let rec var_elig ics0 ts0 = function
  _, [], [], [] -> []
 |el, (false::bs), (t::ts), (ic::ics)
  -> false::(var_elig (ic::ics0) (t::ts0) (el,bs,ts,ics))
 |el, (true::bs), (Qvar(x)::ts), (ic::ics)
  -> false::(var_elig (ic::ics0) ((Qvar(x))::ts0) (el,bs,ts,ics))
 |el, (true::bs), (Qind(x)::ts), (ic::ics)
  -> false::(var_elig (ic::ics0) ((Qind(x))::ts0) (el,bs,ts,ics))
 |Nl, (true::bs), (t::ts), (ic::ics)
  -> true::(var_elig (ic::ics0) (t::ts0) (Nl,bs,ts,ics))
 |el, (true::bs), (Qcc(L,c)::ts), ic::ics
  -> false::(var_elig (ic::ics0) ((Qcc(L,c))::ts0) (el,bs,ts,ics))
 |el, (true::bs), (Qcc(x)::ts), ic::ics
  -> true::(var_elig (ic::ics0) ((Qcc(x))::ts0) (el,bs,ts,ics))
 |el, (true::bs), (Qcf(L,d,ts'))::ts, ic::ics 
  -> false::(var_elig (ic::ics0) ((Qcf(L,d,ts'))::ts0) (el,bs,ts,ics))
 |el, (true::bs), (Qcf(De,d,ts'))::ts, ic::ics
  -> (tout_I_var (ics0,ts0,ics,ts))
     ::(var_elig (ic::ics0) ((Qcf(De,d,ts'))::ts0) (el,bs,ts,ics))
 |Ll, (true::bs), (Qcf(D,d,ts'))::ts, (x,P,d')::ics 
  -> false::(var_elig ((x,P,d')::ics0) ((Qcf(D,d,ts'))::ts0) (Ll,bs,ts,ics))
 |el, (true::bs), (Qcf(e,d,ts'))::ts, ic::ics
  -> true::(var_elig (ic::ics0) ((Qcf(e,d,ts'))::ts0) (el,bs,ts,ics))
 |_ -> anomaly "var_elig"
;;

(* select_var_eligs : 
   bool list * 'a list -> 'a list *)
let rec select_var_eligs = function
 [],[] -> []
|true::bs, x::xs -> x::(select_var_eligs (bs,xs))
|false::bs, x::xs -> select_var_eligs (bs,xs)
|_ -> anomaly "select_var_eligs"
;;

(* vars_elig0 : 
   bool list -> ('a * etiqc * 'b) list 
   -> ('c * etiql * eterm list) list 
   -> ('a * etiqc * 'b) list *)
let rec vars_elig0 bs ics = function
  [] -> select_var_eligs (bs,ics)
 |(n,el,ts)::ls
  -> vars_elig0 (var_elig [] [] (el,bs,ts,ics)) ics ls
;;

(* == Controle Inductive Set *)
(* check_ind_Set : 
   sorts oper term -> bool *)
let rec check_ind_Set = function
 DOP0(Sort(Prop Pos)) -> true
|DOP2(Prod,_,DLAM(_,t)) -> check_ind_Set t
|DOP2(Cast,t,_) -> check_ind_Set t
|_ -> perror(0,[<>])
;;

(* check_ind_arity : 
   int -> sorts oper term -> bool *)
let rec check_ind_arity = fun
 0 t -> check_ind_Set t
|n (DOP2(Cast,t,_)) -> check_ind_arity n t
|n (DOP2(Prod,DOP0(Sort(Prop Pos)),DLAM(_,t)))
 -> check_ind_arity (pred n) t
|n (DOP2(Prod,DOP2(Cast,DOP0(Sort(Prop Pos)),_),DLAM(_,t)))
 -> check_ind_arity (pred n) t
|n (DOP2(Prod,_,DLAM(_,t))) -> perror(0,[<>])
|_ _ -> anomaly "check_ind_arity"
;;

(* rm_nProd : 
   int -> 'a oper term -> 'a oper term *)
let rec rm_nProd = fun
 0 t -> t
|n (DOP2(Cast,t,_)) -> rm_nProd n t
|n (DOP2(Prod,_,DLAM(_,t))) -> rm_nProd (pred n) t
|_ _ -> anomaly "rm_nProd"
;;

(* check_ind_def : 
   sorts oper term -> bool *)
let rec check_ind_def = function
DOPN(MutInd sp,_) as t
 -> let mispec = mind_specif_of_mind t in 
    let n = mis_nparams mispec in
    let (lC,arity) = mis_lc_arity mispec in 
    let (_,cs) = decomp_all_DLAMV_name lC in
    (check_ind_arity n arity;
     map_and check_cons_type (map_vect_list (rm_nProd n) cs))
|DOP2(Cast,t,_) -> check_ind_def t
|_ -> false

(* check_cons_type : 
   sorts oper term -> unit *)
and check_cons_type = function
 DOP2(Prod,t,DLAM(Anonymous,u))
 -> (check_type_expr t; check_cons_type u)
|DOP2(Cast,t,_) -> check_cons_type t
|t -> (check_type_expr t)

(* check_type_expr : 
   sorts oper term -> bool *)
and check_type_expr = function
 Rel _ -> true
|DOP0(Sort(Prop Pos)) -> true
|DOP0(Sort(Prop Null)) -> true
|DOP2(Cast,t,_) -> check_type_expr t
|DOPN(MutInd _,_) as x -> check_ind_def x
|DOPN(AppL,ts) -> map_and check_type_expr (list_of_vect ts)
|DOPN(Const _,_) as c 
 -> let c' = whd_betadelta mt_evc c in
    (c=c') or (check_type_expr c')
|VAR _ as x 
 -> check_type_expr (type_of_rel mt_evc (GLOB (initial_sign())) x)
|t -> false
;;

(* init_vars_eligs : 
    *)
let rec init_vars_eligs = function
 [] -> []
|(_,_,Ecste(GLOBALNAME (DOPN(MutInd _,_) as t),_))::ics 
 -> (check_ind_def t)::(init_vars_eligs ics)
|_::ics 
 -> false::(init_vars_eligs ics)
;;
 
(* vars_elig : 
   ....
   ('a * etiqc * etype) list 
   * ('b * etiql * eterm list) list 
   -> ('a * etiqc * etype) list *)
let vars_elig (icol,lmat) =
  vars_elig0 (init_vars_eligs icol) icol lmat
;;

(* === Arbre de recurrence *)
type arbrec =
  Frec of int 
 |Brec of ((int list * etype) * arbrec list)
;;

(* l_partoutvar : 
   eterm list -> bool *)
let rec l_partoutvar = function
  [] -> true
 |(Qvar(_))::ts -> l_partoutvar ts
 |_ -> false
;;

(* check_dep_params_bis :
*)
let rec check_dep_params_bis = fun
 [] _ -> true
|(true::bs) (t::ts) -> (est_C t)&(check_dep_params_bis bs ts)
|(_::bs) (t::ts) -> (check_dep_params_bis bs ts)
;;

(* check_dep_params :
*)
let rec check_dep_params bs = function
 [] -> []
|(n,Vl,ts)::ls
 -> if (check_dep_params_bis bs ts) then
     (n,Vl,ts)::(check_dep_params bs ls)
    else
     perror(11,[<'S"(equation "; 'S(string_of_int n); 'S") :\n can't change value of dependent parameter">])
|l::ls -> l::(check_dep_params bs ls)
;;

(* contient_decr_bis : 
   constr signature -> eterm list -> bool *)
let rec contient_decr_bis sign = function
 [] -> false
|Qcf(De,_,_)::ts -> true
|Qcf(D,_,_)::ts -> true
|Qcf(C,c,ts')::ts 
 -> (contient_decr_bis sign (arg_par c ts' sign)) 
   or (contient_decr_bis sign ts)
|t::ts -> (contient_decr_bis sign ts)
;;

(* contient_decr : 
   ... *)
let rec contient_decr sign = function
  [] -> []
 |(n,Vl,ts)::ls 
  -> if (contient_decr_bis sign ts) then
      (n,Vl,ts)::(contient_decr sign ls)
     else
      perror(11,[<'S"(equation "; 'S(string_of_int n); 'S") :\n can't find argument on which apply structural induction">])
 |l::ls -> l::(contient_decr sign ls)
;;

(* memeq : 
   ('a * 'b) list -> ('a * 'c * 'b) list -> ('a * 'b) list *)
let rec memeq nts = function
 [] -> nts
|(n,_,ts)::ls
 -> if (mem_assoc n nts) then (memeq nts ls) else (memeq ((n,ts)::nts) ls)
;;

(* numeqs : 
   ('a * 'b) list -> 'a list *)
let numeqs ls = numeqs_rec [] ls
where rec numeqs_rec ns = function
 [] -> (rev ns)
|(n,_)::ls
 -> if (mem n ns) then (numeqs_rec ns ls) else (numeqs_rec (n::ns) ls)
;;

(* unifQ : 
   eterm -> eterm -> bool *)
let rec unifQ = fun
 (Qvar _) _ -> true
|_ (Qvar _) -> true
|(Qcc(_,x)) (Qcc(_,y)) -> x=y
|(Qcf(_,x,ts)) (Qcf(_,y,us)) -> (x=y)&(map2_and unifQ ts us)
|(Qind(_,x,ts)) (Qind(_,y,us)) -> (x=y)&(map2_and unifQ ts us)
|_ _ -> false
;;

(* unif_eq : 
   eterm list -> ('a * eterm list) list -> 'a list *)
let rec unif_eq ts = function
 [] -> []
|(n,us)::nus
 -> if (map2_and unifQ ts us) then n::(unif_eq ts nus) else (unif_eq ts nus)
;;

(* unif_eqs : 
   ('a * eterm list) list -> 'a list *)
let rec unif_eqs = function
 [] -> []
|(n,ts)::nts
 -> match (unif_eq ts nts) with
     [] -> unif_eqs nts
    |ns -> (n::ns)@(unif_eqs nts)
;;

(* cons_Frec : 
   (int * eterm list) list -> arbrec *)
let cons_Frec = function
 [] -> perror(14,[<'S" :\n pattern matching is not exhaustive">])
|(n,_)::[] -> Frec n
|nts 
 -> let ns = unif_eqs nts in
    match ns with
     [] -> perror(13,[<'S"(equation(s) "; 
                       'S(mapconc "," string_of_int (rev (numeqs nts))); 
                       'S") ;\n can't find argument on which apply structural induction">])
    |_ -> perror(12,[< 'S"(equations "; 'S(mapconc "," string_of_int ns); 
                       'S") :\n overlapping">])
;;

(* cons_arbrec0 : 
   constr signature
   -> identifier
   -> ((int list * etiqc * etype) list 
       -> int list * etiqc * etype) 
   -> (int list * etiqc * etype) list 
     * (int * etiql * eterm list) list 
   -> (int list * etiqc * etype) list 
   -> arbrec *)
let rec cons_arbrec0 sign f choix_var mat = function
 [] -> cons_Frec (memeq [] (snd mat))
|vs -> cons_arbrec0s sign f choix_var mat vs
   
(* cons_arbrec0s : 
   constr signature
   -> string 
   -> ((int list * etiqc * etype) list 
       -> int list * etiqc * etype) 
   -> (int list * etiqc * etype) list 
     * (int * etiql * eterm list) list 
   -> (int list * etiqc * etype) list 
   -> arbrec *)
and cons_arbrec0s sign f choix_var mat = function
 [] -> raise Error_arbrec
|vs 
 -> let (x,e,d) = choix_var vs in
    try (Brec((x,d),
              (map (fun m -> (cons_arbrec0 sign f choix_var m (vars_elig m)))
		   (dev_mat sign (x,e,d) mat))))
    with 
     Error_arbrec -> cons_arbrec0s sign f choix_var mat (except (x,e,d) vs)
;;

(* dep_args_f :
*)
let rec dep_args_f_rec = function
 (Eprod _)::ds -> true::(dep_args_f_rec ds)
|_::ds -> false::(dep_args_f_rec ds)
|[] -> []
;;

let dep_args_f = function
 Efleche(ds,d) -> dep_args_f_rec ds
|_ -> anomaly "dep_args_f"
;;

(* cons_arbrec : 
   constr signature 
   -> identifier * etype * (xterm list * xterm) list 
   -> arbrec *)
let cons_arbrec sign f d es = 
 match d with
  Efleche(ds,_)
  -> (let (ics,ls) = (matini sign ds es) in 
      cons_arbrec0 sign f hd 
                  (ics,
                   contient_decr sign (check_dep_params (dep_args_f d) ls))
                  (vars_elig (ics,ls)))
 |_ -> anomaly "cons_arbrec"
;;

(* === Extraction du terme (de 'arbrec' a 'command') *)
type proto_form =
 At of identifier * etype
|Hr of identifier * identifier list * xterm
;;

(* coupe_prem : 
   'a -> ('a * 'b) list -> ('a * 'b) list * ('a * 'b) list *)
let coupe_prem x xTs = coupe_prem_rec [] xTs
where rec coupe_prem_rec xTs' = function
 [] -> (rev xTs',[])
|(_,x',T)::xTs
 -> if (x=x') then
     (rev ((x,T)::xTs'),xTs)
    else
     coupe_prem_rec ((x',T)::xTs') xTs
;;

(* subst_idX : 
   xterm -> identifier -> xterm -> xterm *)
let rec subst_idX t x = function
 Xvar y -> if x=y then t else (Xvar y)
|Xcf(y,ts) -> Xcf(y,map (subst_idX t x) ts)
|Xft(y,ts) -> Xft(y,map (subst_idX t x) ts)
|Xfv(y,ts) -> Xfv(y,map (subst_idX t x) ts)
|Xfx(y,ts) -> Xfx(y,map (subst_idX t x) ts)
|Xind(y,ts) -> Xind(y,map (subst_idX t x) ts)
|t' -> t'
;;

(* etype_actuel : 
   etype list -> etype -> etype list -> etype list -> etype list * etype *)
let rec etype_actuel acc_ds d = fun
 ((Eprod(x,_))::ds') (d''::ds'')
 -> etype_actuel acc_ds (subst_idE d'' x d) 
                 (map (subst_idE d'' x) ds') ds''
|ds' [] -> (((rev acc_ds)@ds'),d)
|(d'::ds') ds'' -> etype_actuel (d'::acc_ds) d ds' ds''
;;

(* etype_actuel_of : 
   constr signature -> etype list -> identifier -> etype list * etype *)
let etype_actuel_of sign ds x =
 let (ds',d) = etype_of sign x in etype_actuel [] d ds' ds
;;

(* hyp_cons : 
   etype -> identifier list ->  xterm -> identifier 
   -> int list -> int -> etype list -> proto_form list *)
(* NB : on suppose que toutes les variables param du type du cons
   sont instanciees *)
let rec hyp_cons d xs t x ns i ds = 
 hyp_cons_rec i ds
where rec hyp_cons_rec i = function
 [] -> []
|(d'::ds)
 -> if d=d' then
     (let nsi = add_last i ns in
      let xi = id_of_ints "x" nsi in
      (At(xi,d'))
      ::(Hr(id_of_ints "h" nsi,xs,subst_idX (Xvar xi) x t))
      ::(hyp_cons_rec (succ i) ds))
    else
     hyp_cons_rec (succ i) ds
;;

(* xterm_of_etype : 
   etype -> xterm *)
let rec xterm_of_etype = function
 Evar x -> Xvar x
|Ecste(x,ds) as e -> Xind(id_of_Ecste e,map xterm_of_etype ds)
|_ -> anomaly "xterm_of_etype"
;;

(* xid_list : 
   int list -> int -> int -> xterm list *)
let rec xid_list ns i = function
 0 -> []
|n 
 -> (Xvar (id_of_ints "x" (add_last i ns)))
    ::(xid_list ns (succ i) (pred n))
;;

(* term_cons : 
   identifier -> xterm list -> xterm list -> xterm *)
let term_cons c = fun
 [] [] -> Xcc c
|xps xs -> Xcf(c,xps@xs)
;;

(* it_AppC : 
   command -> command list -> command *)
let rec it_AppC t us = 
 it_list (fun t u -> mkAppC(t,u)) t us
;;

(* ProdC_of_etypes : 
   ast -> etype list -> command *)
let rec ProdC_of_etypes t = function
 [] -> t
|(Eprod(x,e))::es 
 -> mkProdC(x,com_of_etype e,ProdC_of_etypes t es)
|e::es
 -> mkArrowC(com_of_etype e,ProdC_of_etypes t es) 

(* com_of_etype : 
   etype -> ast *)
and com_of_etype = function
 Evar x -> ASTnvar x
|Eset -> mkPropC "Pos"
|Eprop -> mkPropC "Null"
|Eprod(x,_) -> ASTnvar x
|Ecste(d,[]) as e -> ASTnvar (id_of_Ecste e)
|Ecste(d,es) as e -> it_AppC (ASTnvar (id_of_Ecste e)) (map com_of_etype es)
|Efleche([],e) -> com_of_etype e
|Efleche(es,e) -> ProdC_of_etypes (com_of_etype e) es
;;

(* prem_cons : 
   int -> etype -> ast -> int list -> etype list 
   -> (identifier * ast) list * (bool * identifier * ast) list*)
let prem_cons i d hrC ns dacs = prem_cons_rec [] [] i dacs
where rec prem_cons_rec pcs ppcs i = function
 [] -> (rev pcs, rev ppcs)
|da::das
 -> let nsi = add_last i ns in
    let (x,t) = (id_of_ints "x" nsi,com_of_etype da) in
    if d=da then
     prem_cons_rec ((id_of_ints "h" nsi,hrC)::(x,t)::pcs) ppcs (succ i) das
    else
     prem_cons_rec ((x,t)::pcs) ((false,x,t)::ppcs) (succ i) das
;;

(* hd_const_id_of :
   sorts oper term -> identifier *)
let rec hd_const_id_of = function
 VAR _ as x -> id_of_var x
|DOPN(Const _,_) as x -> basename (path_of_const x)
|DOPN(MutInd _ as x,_)  -> id_of_global x
|DOPN(MutConstruct _ as x,_) ->  id_of_global x
|DOP2(Prod,_,DLAM(_,t)) -> hd_const_id_of t
|DOPN(AppL,cl) -> hd_const_id_of (hd_vect cl) 
|_ -> anomaly "hd_const_id_of"
;;

(* hd_symb_of :
   'a oper term -> 'a oper term *)
let rec hd_symb_of = function
 DOP2(Lambda,_,DLAM(_,t)) -> hd_symb_of t
|DOPN(AppL,cl) -> hd_symb_of (hd_vect cl) 
|t -> t
;;

(* ids_cons_of : 
   etype -> constr signature -> identifier list *)
let ids_cons_of d sign = 
 let x = id_of_Ecste d in
 let n = nb_of_cons x sign in
  match (search_reference (GLOB sign) x) with
   (DOPN(MutInd _,_)) as mind 
    -> let mispec = mind_specif_of_mind mind in 
        list_of_vect (mis_consnames mispec)
  |_ -> anomaly "ids_of_cons"
;; 

(* etype_of_com : 
   constr signature -> ast -> etype *)
let rec etype_of_com_rec sign = function
 ASTnode("PROP",[ASTprim(TOKid c)])
 -> (match string_of_id c with
      "Pos" -> Eset 
     |"Null" -> Eprop
     |_ -> anomaly "etype_of_com_rec (1)")
|ASTnvar x -> Evar x
|ASTnode("APPLIST",_) as X
 -> let (f,ts) = flat_AppC X in
    let glob_f = search_reference (GLOB sign) (id_of_string f) in
        Ecste(GLOBALNAME glob_f,map (etype_of_com_rec sign) ts)
|ASTnode("PROD",[t;ASTslam(NONE,u)])
 -> Efleche([etype_of_com_rec sign t],etype_of_com_rec sign u)
|ASTnode("PROD",[t;ASTslam(SOME x,u)])
 -> Efleche([Eprod(x,etype_of_com_rec sign t)],etype_of_com_rec sign u)
|_ -> anomaly "etype_of_com_rec (2)"
;;

let rec flat_Efleche = function
 Efleche(ts,Efleche(us,v)) -> flat_Efleche (Efleche(ts@us,v))
|Ecste(x,ts) -> Ecste(x,map flat_Efleche ts)
|Eprod(x,t) -> Eprod(x,flat_Efleche t)
|Efleche(ts,t) -> Efleche(map flat_Efleche ts,flat_Efleche t)
|t -> t
;;

let etype_of_com sign t = flat_Efleche (etype_of_com_rec sign t)
;;

exception erreur_instance
;;

(* maj_assoc : 
   'a -> 'b -> ('a * 'b) list -> ('a * 'b) list *)
let rec maj_assoc x y xys = maj_assoc_rec xys
where rec maj_assoc_rec = function
 [] -> []
|(x',y')::xys
 -> if x=x' then (x,y)::xys else (x',y')::(maj_assoc_rec xys)
;;

let Xvar0 = Xvar null_id
;;

(* instance : 
   identifier list -> xterm -> xterm -> (identifier * xterm) list
 NB : (instance [x1;..;xk] t1 t2) -> [(x1,u1);..;(xk,uk)] 
           ssi t2=t1[u1/x1,..,xk/ux] *)
let rec instance xs t1 t2 = instance_rec (map (fun x -> (x,Xvar0)) xs) t1 t2
where rec instance_rec xus = fun
 (Xvar x) t
 -> if (mem_assoc x xus) then
     (let u = assoc x xus in
       if u=Xvar0 then
        maj_assoc x t xus
       else
        if u=t then xus else raise erreur_instance)
    else     
     if t=(Xvar x) then xus else raise erreur_instance
|(Xcc x) (Xcc x') 
 -> if x=x' then xus else raise erreur_instance
|(Xct x) (Xct x') 
 -> if x=x' then xus else raise erreur_instance
|(Xcf(x,ts)) (Xcf(x',ts'))
 -> if x=x' then
     list_it2 (fun t1 t2 x -> instance_rec x t1 t2) ts ts' xus
    else
     raise erreur_instance
|(Xft(x,ts)) (Xft(x',ts'))
 -> if x=x' then
     list_it2 (fun t1 t2 x -> instance_rec x t1 t2) ts ts' xus
    else
     raise erreur_instance
|(Xfv(x,ts)) (Xfv(x',ts'))
 -> if x=x' then
     list_it2 (fun t1 t2 x -> instance_rec x t1 t2) ts ts' xus
    else
     raise erreur_instance
|(Xfx(x,ts)) (Xfx(x',ts'))
 -> if x=x' then
     list_it2 (fun t1 t2 x -> instance_rec x t1 t2) ts ts' xus
    else
     raise erreur_instance
|(Xind(x,ts)) (Xind(x',ts'))
 -> if x=x' then
     list_it2 (fun t1 t2 x -> instance_rec x t1 t2) ts ts' xus
    else
     raise erreur_instance
|_ _ -> raise erreur_instance
;;

(* cherche_instance : 
   xterm -> proto_form list -> identifier * xterm list *)
let rec cherche_instance t = function
 [] -> anomaly "cherche_instance"
|(Hr(hi,xs,u))::hs
 -> (try 
      let xus = instance xs u t in
       (hi,map snd xus)
     with
      erreur_instance -> cherche_instance t hs)
|h::hs -> cherche_instance t hs
;;
 
(* etendre : 
   identifier -> proto_form list -> xterm -> ast *)
let rec etendre f hs = function
 Xvar x -> ASTnvar x
|Xcc x -> ASTnvar x
|Xct x -> ASTnvar x
|Xcf(x,ts) 
 -> it_AppC (ASTnvar x) (map (etendre f hs) ts)
|Xft(g,ts)
 -> if f=g then
     (let (h,us) = cherche_instance (Xft(g,ts)) hs in
       it_AppC (ASTnvar h) (map (etendre f hs) us))
    else
     it_AppC (ASTnvar g) (map (etendre f hs) ts)
|Xfv(x,ts) -> it_AppC (ASTnvar x) (map (etendre f hs) ts)
|Xfx(f',ts)
 -> let (h,us) = cherche_instance (Xft(f',ts)) hs in
      it_AppC (ASTnvar h) (map (etendre f hs) us)
|Xind(x,[]) -> ASTnvar x
|Xind(x,ts)
 -> it_AppC (ASTnvar x) (map (etendre f hs) ts)
;;

(* renomme_idX : 
   identifier -> identifier -> xterm -> xterm *)
let rec renomme_idX x y = function
 Xvar z -> Xvar (if z=y then x else z)
|Xcf(c,ts) -> Xcf(c,map (renomme_idX x y) ts)
|Xft(f,ts) -> Xft(f,map (renomme_idX x y) ts)
|Xfv(z,ts) -> Xfv((if z=y then x else z),map (renomme_idX x y) ts)
|Xfx(f,ts) -> Xfx(f,map (renomme_idX x y) ts)
|Xind(i,ts) -> Xind(i,map (renomme_idX x y) ts)
|t -> t
;;

(* renommeX :
   xterm -> xterm -> xterm -> xterm *)
let rec renommeX u = fun
 (Xvar x) (Xvar y) -> renomme_idX x y u
|(Xcf(_,ts)) (Xcf(_,us))
 -> list_it2 (fun ts us u -> renommeX u ts us) ts us u
|(Xft(_,ts)) (Xft(_,us))
 -> list_it2 (fun ts us u -> renommeX u ts us) ts us u
|(Xfv(_,ts)) (Xfv(_,us))
 -> list_it2 (fun ts us u -> renommeX u ts us) ts us u
|(Xfx(_,ts)) (Xfx(_,us))
 -> list_it2 (fun ts us u -> renommeX u ts us) ts us u
|(Xind(_,ts)) (Xind(_,us))
 -> list_it2 (fun ts us u -> renommeX u ts us) ts us u
|_ _ -> u
;;
     
(* perm_intros : 
   'a -> ('b * 'a * 'c) list -> ('b * 'a * 'c) list * ('b * 'a * 'c) list *)
let rec perm_intros_rec ps1 x = function
 [] -> ([],rev ps1)
|(b,x',t)::ps2
 -> if (x=x') then
     (rev ((b,x',t)::ps1),ps2)
    else
     perm_intros_rec ((b,x',t)::ps1) x ps2
;;

let perm_intros x ps = perm_intros_rec [] x ps
;;

(* perm_elims : 
   constr signature -> (bool * identifier * ast) list 
   -> proto_form list * (bool * identifier * ast) list *)
let rec perm_elims_rec sign hs ps = function
 [] -> (rev hs,rev ps)
|(true,x,t)::ps'
 ->  perm_elims_rec sign ((At(x,etype_of_com sign t))::hs) ps ps'
|p::ps'
 -> perm_elims_rec sign hs (p::ps) ps'
;;

let perm_elims sign ps = perm_elims_rec sign [] [] ps
;;

(* args_cons : 
   int list -> int -> etype list -> xterm list
 NB : on suppose que les parametres du type sont tous en tete *)
let rec args_cons ns i = function
 [] -> []
|(Eprod(x,_))::das
 -> args_cons ns (succ i) das
|da::das 
 -> (Xvar (id_of_ints "x" (add_last i ns)))::(args_cons ns (succ i) das)
;;

(* com_of_arbrec : 
   constr signature -> identifier -> (xterm list * xterm) list
   -> proto_formlist -> (bool * identifier * ast) list -> etype 
   -> xterm -> arbrec -> ast *)
let rec com_of_arbrec sign f es hs ps d t a = 
 com_of_arbrec_rec hs ps d t a
where rec com_of_arbrec_rec hs ps d t = function
 Frec n 
 -> let (ts,u) = nth es n in
    let u' = renommeX u t (Xft(f,ts)) in
    let U = etendre f hs u' in
     list_it (fun (_,x,T) u -> mkLambdaC(x,T,u)) ps U
|Brec((ns,d'),ars)
 -> (* pour Perm *)
    let x = id_of_ints "x" ns in
    let (ps1,ps2) = perm_intros x ps in
    let (hs',ps1') = perm_elims sign (except_last ps1) in
    (* pour Rec *)
    (* liste des arg de d' *)
    let ds = match d' with Ecste(_,ds) -> ds |_ -> [] in
    (* liste des cons de d' *)
    let cs = ids_cons_of d' sign in
    (* liste des types actuels des arg des cons *)
    let dacss = map fst (map (etype_actuel_of sign ds) cs) in
    (* args param actuels des cons *)
    let xpcs = map xterm_of_etype ds in
    (* liste des autres args des cons *)
    let i0 = succ (list_length ds) in
    let xss = map (args_cons ns i0) dacss in
    (* liste des xterms des cons *)
    let cs = map2 (fun c xics -> (term_cons c xpcs xics)) cs xss in
    (* listes des termes pour les cas des cons *)
    let tcs = map (fun c -> subst_idX c x t) cs in
    (* type(commmande) des hyp rec *)
    let hrC 
    = list_it (fun (_,x,T) u -> mkProdC(x,T,u)) (ps1'@ps2) (com_of_etype d) in
    (* liste des premisses et des arg param des cons *)
    let (pcss,ppcss) 
    = split (map (prem_cons i0 d' hrC ns) dacss) in
    (* liste des prem des hr *)
    let xs = map (fun (_,x,_) -> x) (ps1'@ps2) in
    (* liste des hyp rec des cons *)
    let hiscs = map (hyp_cons d' xs t x ns i0) dacss in
    (* liste des res pour les appels rec de com_of_arbrec *)
    let Us 
    = map4 (fun hisc ppcs tc ar 
            -> com_of_arbrec_rec (hisc@hs'@hs) (ppcs@ps1'@ps2) d tc ar)
           hiscs ppcss tcs ars in
    (* liste des res paraclos *)
    let Us' 
    = map3 (fun U pcs ppcs 
            -> let U' = it_list (fun t (_,x,_) -> mkAppC(t,ASTnvar x)) U ppcs in
               list_it (fun (x,T) u -> mkLambdaC(x,T,u)) pcs U')
           Us pcss ppcss in 
    let U =  mkRecC(hrC,ASTnvar x,Us') in
    let U' = it_list (fun t (_,x,_) -> mkAppC(t,ASTnvar x)) U ps1' in
      list_it (fun (_,x,T) u -> mkLambdaC(x,T,u)) ps1 U'
;;

(* === Theoremes equationels *)
(* thm_eq :
   identifier -> identifier -> constr -> command -> State -> State *)
(* === Pour memoire
let thm_eq nom_f nom_th kind_f eq st =
 let st1 = theorem nom_th (Strength 0) st in
 let st2 = statement eq st1 in
  (* silent_by silent_trivial st2 *)
  by (THEN intros (THEN (unfold_nth [[],nom_f]) (apply (refl_eqC kind_f)))) st2
;;
=== *)

(* thm_eqs : 
   identifier -> command list -> unit *)
let rec thm_eqs nom_f eqs_f = 
 thm_eqs_rec 1 eqs_f
where rec thm_eqs_rec i = function
 [] -> ()
|(eq::eqs) 
 -> try
     let nom_th = make_ident (SA__intern((string_of_id nom_f)^"_eq")) i in
     (start_proof (string_of_id nom_th) (libobject__NeverDischarge) eq;
      solve_nth 1 auto__trivial;
      save_named true;
      thm_eqs_rec (succ i) eqs)
    with
     UserError _ -> perror(0,[<'S"(equation "; 'S(string_of_int i);'S") :\n left and right members are not convertible">])
    |e -> raise e
;;

(* === Fonction principale *)
(* thf : 
   constr signature -> ast -> (bool * identifier * ast) list * etype *)
let rec thf sign T = thf_rec [] 1 T
where rec thf_rec ps i = function
 ASTnode("PROD",[T;ASTslam(SOME x,U)])
 -> let x'= id_of_ints "x" [i] in
     thf_rec ((true,x',T)::ps) (succ i) (subst_idC (ASTnvar x') x U)
|ASTnode("PROD",[T;ASTslam(NONE,U)])
 -> thf_rec ((false,id_of_ints "x" [i],T)::ps) (succ i) U
|T -> (rev ps,etype_of_com sign T)
;;

(* close_eq : 
   identifier -> ast -> sorts oper term -> (identifier * ast) list
   -> ast list * ast -> ast *)
let close_eq nom_f type_res_fC kind_f typed_vars_eq eq_fC =
 let (ts,u) = eq_fC in
  closeC (eqC (AppC_fx nom_f ts) u type_res_fC kind_f) typed_vars_eq
;;

(* renomme_rec : 
   etype list -> int -> etype -> etype list -> etype *)
let rec renomme_rec ds i d = function
 [] -> Efleche(rev ds,d)
|(Eprod(x,d'))::ds'
 -> let x'= id_of_ints "x" [i] in
    renomme_rec ((Eprod(x',d'))::ds) (succ i) 
                (subst_idE (Evar x') x d)(map (subst_idE (Evar x') x) ds')
|d'::ds' -> renomme_rec (d'::ds) (succ i) d ds'
;;

(* renommeE : 
   etype -> etype *)
let renommeE = function 
 Efleche(ds,d) -> renomme_rec [] 1 d ds
|_ -> anomaly "renommeE"
;;

(* Pour debuggage ..... *)
let it_ProPre = (ref (ASTnvar (id_of_string "_")));;

(* ast_free_occ :
*)
let rec ast_free_occ x = function
 ASTnvar y -> x=y
|ASTnode("PROD",[t;ASTslam(NONE,u)])
 -> (ast_free_occ x t)or(ast_free_occ x u)
|ASTnode("PROD",[t;ASTslam(SOME y,u)])
 -> (not(x=y))&((ast_free_occ x t)or(ast_free_occ x u))
|ASTnode("APPLIST",ts)
 -> map_or (ast_free_occ x) ts
|ASTnode("PROP",_) -> false
|_ -> anomaly "ast_free_occ"
;;

(* update_ast_dep :
*)
let rec update_ast_dep = function
 ASTnode("PROD",[t;ASTslam(SOME x,u)])
  -> if (ast_free_occ x u) then
       ASTnode("PROD",[t;ASTslam(SOME x,update_ast_dep u)])
     else
       ASTnode("PROD",[t;ASTslam(NONE,update_ast_dep u)])
|ASTnode("PROD",[t;ASTslam(NONE,u)])
 -> ASTnode("PROD",[t;ASTslam(NONE,update_ast_dep u)])
|ASTnode("APPLIST",ts)
 -> ASTnode("APPLIST",map update_ast_dep ts)
|t -> t
;;

(* ProPre_rec_def : 
   (string * ast) list * identifier * ast * (ast list * ast) list 
   -> unit *)
let ProPre_rec_def (nom_f,params_f,type_fC,eqs_fC) =
try
 let sign0 = initial_sign() in
 let params_f' 
  = mapapp (fun (xs,T) -> map (fun x -> (x,T)) xs) params_f in 
 let type_fC' = update_ast_dep (check_type_f sign0 params_f' type_fC) in
 let j1 = fconstruct mt_evc sign0 type_fC' in
 let type_f = j1.VAL in
 let kind_f = j1.TYP in
 let (typed_args_fC,type_res_fC) = flatC_Prod (string_of_id nom_f) type_f in
 let id_params_f = map fst params_f' in
 let params_RefC = map (fun (x,_) -> ASTnvar x) params_f' in
 let eqs_fC' = map (fun (ts,u) -> ((params_RefC@ts),u)) eqs_fC in
 let typed_vars_eqs_f 
  = check_vars_eqs sign0 id_params_f typed_args_fC eqs_fC' in
 let sign1 
  = add_sign (nom_f,assumption_of_judgement mt_evc j1) sign0 in
 let eqs_f 
  = map2 (close_eq nom_f type_res_fC kind_f) typed_vars_eqs_f eqs_fC' in 
 let _ = check_type_eqs sign1 eqs_f in
 let eqs_fX = xterm_of_eqs sign0 nom_f id_params_f eqs_fC' in
 let type_fE = renommeE (etype_of_constr [] [] [] type_f) in
 let arbrec_f = cons_arbrec sign1 nom_f type_fE eqs_fX in
 let (idxtype_args_fC,type_res_fE) = thf sign0 type_fC' in
 let id_args_fX = map (fun (_,x,_) -> Xvar x) idxtype_args_fC in
 let fX = Xft(nom_f,id_args_fX) in 
 let fC = 
  com_of_arbrec sign1 nom_f eqs_fX [] idxtype_args_fC type_res_fE fX arbrec_f in
(* let fj = fconstruct mt_evc (initial_sign()) fC in *)
  (it_ProPre := fC;
   definition_body nom_f libobject__NeverDischarge fC;
   thm_eqs nom_f eqs_f;
   message ((string_of_id nom_f)^" is recursively defined."))
with
 ProPre_error(_,x) ->
 errorlabstrm ""
 [< 'S"in Recursive Definition \"" ; 'S(string_of_id nom_f); 'S"\" " ; x >]
|UserError(s,pps) ->
 errorlabstrm ""
 [< 'S"in Recursive Definition \"" ; 'S(string_of_id nom_f) ; 'S"\" :\n" ; 
    'S s; 'SPC ; pps >]
|e -> raise e
;;

(* === Grammaire *)
vinterp_add
(
"RecursiveDefinition",
fun
 ((VARG_IDENTIFIER f)::(VARG_BINDERLIST prms)::
  (VARG_COMMAND t)::es)
   -> (let pair_es = 
       (fun 
         (VARG_COMMANDLIST (u::ts)) -> (ts,u) 
        |_ -> anomaly "vinterp_add.RecursiveDefinition") 
       in 
       let es' = map pair_es es in
        fun() -> ProPre_rec_def(f,prms,t,es'))
|_ -> anomaly "vinterp_add.RecursiveDefinition"
);;

(* $Id: propre.ml,v 1.6 1996/02/12 15:57:02 cpaulin Exp $ *)
