(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 tauto.ml                                 *)
(****************************************************************************)

(* Autor: Cesar A. Munnoz H *)

#open "tauto2";;
#open "names";;
#open "std";;
#open "generic";;
#open "term";;
#open "termenv";;
#open "machops";;
#open "tacmach";;
#open "reduction";;
#open "tactics";;
#open "tactics1";;
#open "tactics2";; 
#open "pattern";;
#open "auto";;
#open "proof";;
#open "stdpp";;

#infix "ORELSE";;
#infix "NEXT";;
#infix "THEN";;
#infix "THENF";;
#infix "THENL";;
#infix "THENS";;
(* ------------------ Sistema de Gentzen Intuisionista -------------------*)
(* Gama,Delta son metavariables de conjuntos de reglas
   A,B son variables de formulas  *)

let Gama = (TLister "Gama",FLisfor "GAMA");;
let Delta = (TLister "Delta",FLisfor "DELTA");;
let Delta' = (TLister "Delta'",FLisfor "DELTA");;
let Delta'' = (TLister "Delta''",FLisfor "DELTA");;

let Curry(A,B,C,a,b,p) = TFun(a,A,TFun(b,B,TApl(FAnd(A,B),C,p,
                                         TPar(A,B,TVar a,TVar b))));;
let Left(A,B,C,a,p) = TFun(a,A,TApl(FOr(A,B),C,p,TInl(A,B,TVar a)));;
let Right(A,B,C,b,p) = TFun(b,B,TApl(FOr(A,B),C,p,TInr(A,B,TVar b)));;
let Imp2(A,B,C,a,b,p) = TFun(a,A,TApl(FImp(B,A),C,p,TFun(b,B,TVar a)));;

(* Regla inicial   *) 
(*    / A,Gama |- A,Delta *) 
let inic = {
      Tip="inic";
      Heu=false;
      Ant=true;
      Pri= TVar "#x",FAto "#A";
      Pre=[];
      Res=([],[TVar "#x",FVar "#A"]);
      Def=["Delta",TZero(FLisfor "DELTA")];
      Sub=[];
      Ren=["#x"];
      Vardelta = true;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_false   *) 
(*    / Gama,False |- Delta *) 
let l_false = {
      Tip="false";
      Heu=false;
      Ant=true;
      Pri= TVar "#x",FFalse;
      Pre=[];
      Res=([],[]);
      Def=["Delta",TZ(FLisfor "DELTA",TVar "#x")];
      Sub=[];
      Ren=["#x"];
      Vardelta = true;
      Ssi = true;
      Rendelta=[]};;

(* Regla r_true   *) 
(*    / Gama |- True,Delta *) 
let r_true = {
      Tip="true";
      Heu=false;
      Ant=false;
      Pri= TTrue,FTrue;
      Pre=[];
      Res=([],[]);
      Def=["Delta",TZero(FLisfor "DELTA")];
      Sub=[];
      Ren=[];
      Vardelta = true;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_and     *)
(* A,B,Gama |- Delta / FAnd(A,B),Gama |- Delta *)
let l_and = {
      Tip="l_and";
      Heu=false;
      Ant=true;
      Pri= TVar "#xy",FAnd(FVar "#A",FVar "#B");
      Pre=[[TVar "#x",FVar "#A";TVar "#y",FVar "#B";Gama],[Delta]];
      Res=([],[]);
      Def=[];
      Sub=["#x",TFst(FVar "#A",FVar "#B",TVar "#xy");
	   "#y",TSnd(FVar "#A",FVar "#B",TVar "#xy")];
      Ren=["#x";"#y"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla r_and     *)
(* Gama |- A,Delta'  Gama |- B,Delta'' /  Gama |- A/\B,Delta *)
let r_and = {
      Tip="r_and";
      Heu=false;
      Ant=false;
      Pri= TPar(FVar "#A",FVar "#B",TVar "#x",TVar "#y"), 
                FAnd(FVar "#A",FVar "#B");
      Pre=[[Gama],[TVar "#x",FVar "#A";Delta'];[Gama],
           [TVar "#y",FVar "#B";Delta'']];
      Res=([],[]);
      Def=["Delta",TSum(TLister "Delta'",TLister "Delta''")];
      Sub=[];
      Ren=["#x";"#y"];
      Vardelta = true;
      Ssi = true;
      Rendelta=["Delta'";"Delta''"]};;

(* Regla l_or     *)
(* A,Gama |- Delta'  B,Gama |- Delta'' / A\/B,Gama |- Delta *)
let l_or = {
      Tip="l_or";
      Heu=false;
      Ant=true;
      Pri= TVar "#xy",FOr(FVar "#A",FVar "#B");
      Pre=[[TVar "#x",FVar "#A";Gama],[Delta'];
           [TVar "#y",FVar "#B";Gama],[Delta'']];
      Res=([],[]);
      Sub=[];
      Def=["Delta", TCase([FVar "#A";FVar "#B";FLisfor "DELTA"],
			  [TFun("#x",FVar "#A",TLister "Delta'");
			   TFun("#y",FVar "#B",TLister "Delta''");
			  TVar "#xy"])];
      Ren=["#x";"#y";"#xy"];
      Vardelta = true;
      Ssi = true;
      Rendelta=["Delta'";"Delta''"]};;

(* Regla r_or     *)
(* Gama |- A,B,Delta / Gama |- A\/B,Delta *)
let r_or = {
      Tip="r_or";
      Heu=false;
      Ant=false;
      Pri= TSum(TInl(FVar "#A",FVar "#B",TVar "#x"),
               TInr(FVar "#A",FVar "#B",TVar "#y")),
	   FOr(FVar "#A",FVar "#B");
      Pre=[[Gama],
	   [TVar "#x",FVar "#A";TVar "#y",FVar "#B";Delta]];
      Res=([],[]);
      Sub=[];
      Def=[];
      Ren=["#x";"#y"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_imp1    *)
(* A,B,Gama |- Delta / A->B,A,Gama |- Delta (A es un atomo) *)
let l_imp1 = {
      Tip="l_imp1";
      Heu=false;
      Ant=true;
      Pri= TVar "#p",FImp(FAto "#A",FVar "#B");
      Pre=[[TVar "#x",FVar "#B";Gama],
	   [Delta]];
      Res=([TVar "#a",FVar "#A"],[]);
      Def=[];
      Sub=["#x",TApl(FVar "#A",FVar "#B",TVar "#p",TVar "#a")];
      Ren=["#x"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_imp2   *)
(* C->(D->B),Gama |- Delta / C/\D->B,Gama |- Delta *)
let l_imp2 = {
      Tip="l_imp2";
      Heu=false;
      Ant=true;
      Pri= TVar "#p",FImp(FAnd(FVar "#C",FVar "#D"),FVar "#B");
      Pre=[[TVar "#x",FImp(FVar "#C",FImp(FVar "#D",FVar "#B"));Gama],
	   [Delta]];
      Res=([],[]);
      Def=[];
      Sub=["#x",Curry(FVar "#C",FVar "#D",FVar "#B","#c","#d",TVar "#p")];
      Ren=["#x";"#c";"#d"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_imp3   *)
(* C->B,D->B,Gama |- Delta / C\/D->B,Gama |- Delta *)
let l_imp3 = {
      Tip="l_imp3";
      Heu=false;
      Ant=true;
      Pri= TVar "#p",FImp(FOr(FVar "#C",FVar "#D"),FVar "#B");
      Pre=[[TVar "#x",FImp(FVar "#C",FVar "#B");TVar "#y",
            FImp(FVar "#D",FVar "#B");Gama],
	   [Delta]];
      Res=([],[]);
      Def=[];
      Sub=["#x",Left(FVar "#C",FVar "#D",FVar "#B","#c",TVar "#p"); 
	   "#y",Right(FVar "#C",FVar "#D",FVar "#B","#d",TVar "#p")];
      Ren=["#x";"#y";"#c";"#d"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_imp4   *)
(* D->B,Gama |- C->D,Delta   B,Gama |- Delta / (C->D)->B,Gama |- Delta *)
let l_imp4 = {
      Tip="l_imp4";
      Heu=false;
      Ant=true;
      Pri= TVar "#p",FImp(FImp(FVar "#C",FVar "#D"),FVar "#B");
      Pre=[[TVar "#x",FVar "#B";Gama],[Delta];
           [TVar "#z",FImp(FVar "#D",FVar "#B");Gama],[TVar "#y",
	    FImp(FVar "#C",FVar "#D")]];
      Res=([],[]);
      Def=[];
      Sub=["#x",TApl(FImp(FVar "#C",FVar "#D"),FVar "#B",TVar "#p",
                TApl(FVar "#D",FVar "#B",
                     TFun("#z",FImp(FVar "#D",FVar "#B"),TVar "#y"),
		Imp2(FVar "#D",FVar "#C",FVar "#B","#d","#c",TVar "#p")))];
      Ren=["#x";"#y";"#z";"#d";"#c"];
      Vardelta = false;
      Ssi = false;
      Rendelta=[]};;

(* Regla l_imp5   *)
(* (A->False)->B,Gama |- Delta /  Not(A)->B,Gama |- Delta *) 
let l_imp5 = {
      Tip="l_imp5";
      Heu=false;
      Ant=true;
      Pri= TVar "#x",FImp(FNot(FVar "#A"),FVar "#B");
      Pre=[[TVar "#x",FImp(FImp(FVar "#A",FFalse),FVar "#B");Gama],
	   [Delta]];
      Res=([],[]);
      Def=[];
      Sub=[];
      Ren=["#x"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_imp6   *)
(* (C->D/\D->C)->B,Gama |- Delta  / (C<->D)->B,Gama |- Delta *) 
let l_imp6 = {
      Tip="l_imp6";
      Heu=false;
      Ant=true;
      Pri= TVar "#x",FImp(FEqu(FVar "#C",FVar "#D"),FVar "#B");
      Pre=[[TVar "#x",FImp(FAnd(FImp(FVar "#C",FVar "#D"),
            FImp(FVar "#D",FVar "#C")),FVar "#B");Gama],[Delta]];
      Res=([],[]);
      Def=[];
      Sub=[];
      Ren=["#x"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla r_imp     *)
(* A,Gama |- B  / Gama |- A->B,Delta *)
let r_imp = {
      Tip="r_imp";
      Heu=false;
      Ant=false;
      Pri= TFun("#x",FVar "#A",TVar "#y"),FImp(FVar "#A",FVar "#B");
      Pre=[[TVar "#x",FVar "#A";Gama],[TVar "#y",FVar "#B"]];
      Res=([],[]);
      Def=["Delta",TZero(FLisfor "DELTA")];
      Sub=[];
      Ren=["#x";"#y"];
      Vardelta = true;
      Ssi = false;
      Rendelta=[]};;

(* Regla l_not     *)
(* A->False,Gama |- Delta / Not(A),Gama |- Delta *)
let l_not = {
      Tip="l_not";
      Heu=false;
      Ant=true;
      Pri= TVar "#x",FNot(FVar "#A");
      Pre=[[TVar "#x",FImp(FVar "#A",FFalse);Gama],[Delta]];
      Res=([],[]);
      Def=[];
      Sub=[];
      Ren=["#x"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla r_not     *)
(* Gama |- A->False,Delta   / Gama |- Not(A),Delta *)
let r_not = {
      Tip="r_not";
      Heu=false;
      Ant=false;
      Pri= TVar "#x",FNot(FVar "#A");
      Pre=[[Gama],[TVar "#x",FImp(FVar "#A",FFalse);Delta]];
      Res=([],[]);
      Def=[];
      Sub=[];
      Ren=["#x"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla l_equ     *)
(* A->B/\B->A,Gama |- Delta / A<->B,Gama |- Delta *)
let l_equ = {
      Tip="l_equ";
      Heu=false;
      Ant=true;
      Pri= TVar "#x",FEqu(FVar "#A",FVar "#B");
      Pre=[[TVar "#x",FAnd(FImp(FVar "#A",FVar "#B"),
            FImp(FVar "#B",FVar "#A"));Gama],[Delta]];
      Res=([],[]);
      Def=[];
      Sub=[];
      Ren=["#x"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Regla r_equ     *)
(* Gama |- A->B/\B->A,Delta   / Gama |- A<->B,Delta *)
let r_equ = {
      Tip="r_equ";
      Heu=false;
      Ant=false;
      Pri= TVar "#x",FEqu(FVar "#A",FVar "#B");
      Pre=[[Gama],
	   [TVar "#x",FAnd(FImp(FVar "#A",FVar "#B"),
            FImp(FVar "#B",FVar "#A"));Delta]];
      Res=([],[]);
      Def=[];
      Sub=[];
      Ren=["#x"];
      Vardelta = false;
      Ssi = true;
      Rendelta=[]};;

(* Definicion de la regla VACIA *)
let VACIA = {
      Tip="vacia";
      Heu=false;
      Ant=false;
      Pri=Gama;
      Pre=[];
      Res=([],[]);
      Def=[];
      Sub=[];
      Ren=[];
      Vardelta = false;
      Ssi = false;
      Rendelta=[]};;

(*---------------------- Reglas heuristicas ------------------------------*)

(* Regla simetria de igualdad *) 
(*    / a=b,Gama |- b=a,Delta *) 
let sim = {
      Tip="sim";
      Heu=true;
      Ant=true;
      Pri= TVar "#x",FEq(FVar "#A",FVar "#a",FVar "#b");
      Pre=[];
      Res=([],[TSim(FVar "#A",FVar "#a",FVar"#b",TVar"#x"),
               FEq(FVar "#A",FVar "#b",FVar "#a")]);
      Def=["Delta",TZero(FLisfor "DELTA")];
      Sub=[];
      Ren=["#x";"#a";"#b"];
      Vardelta = true;
      Ssi = true;
      Rendelta=[]};;

(* Regla r_refl   *) 
(*    / Gama |- <t>a=a,Delta *) 
let r_refl = {
      Tip="refl";
      Heu=true;
      Ant=false;
      Pri= TRefl(FVar "#A",FVar "#a"),FEq(FVar "#A",FVar "#a",FVar "#a");
      Pre=[];
      Res=([],[]);
      Def=["Delta",TZero(FLisfor "DELTA")];
      Sub=[];
      Ren=["#a"];
      Vardelta = true;
      Ssi = true;
      Rendelta=[]};;

let sistema = [inic;l_false;r_true;l_and;r_and;l_imp1;l_imp2;l_imp3;
	       l_imp5;l_imp6;l_not;r_not;l_equ;r_equ;r_imp;
               l_or;r_or;l_imp4];;
			    
(*----------- Proyecciones del tipos de datos Sequente ----------------------*) 

(* Antecedente de un sequente *)
let ante (a,_)  = a;;

(* Sucedente de un sequente *)
let suce (_,s) = s;;

(*----------- Constructores de los  tipos de datos  ----------------------*) 

(* Simplifica una substitucion es decir elemina las substituciones 
   inutiles *)
let rec simple = function
      [] -> []
    | ((x,_) as A)::z -> if nth_char x 0 = `#` then simple z 
                         else A::simple z;;

(* Construye un node de demostracion *)
let consd a l = {Arb=a;Lisbut=l};;

(* Construye un nodo de un arbol *)
let consa s r sd st = {Seq = ref s;Reg = ref r;Sd = simple sd; 
                       St= simple st};;  

(* Construye un nodo de sustitucion *)
let conss r sf sg sd ren sdef ster =
      SCons({SReg=ref r;SFor=sf;SGam=sg;SDel=sd; 
             SRen=ren;SDef=sdef;STer=ster});;

(*----------------------- Aplicacion de Reglas ------------------------------*)

(* Buscar un nombre de variable en una sustitucion, retorna la lista
   que contiene la formula que la variable sustituye *)
let rec busque n = function
      [] -> []
    | (x,f)::y -> if x=n then [f] else busque n y;; 

(* Aplicar una substitucion a una formula, retorna otra formula *)
let rec apl_f s = function  
      (FVar y) as X -> (match busque y s with 
			 [] -> X
                       | a::_ -> a ) 
    | (FLisfor y) as X -> (match busque y s with
			     [] -> X
			   | a::_ -> a)
    | FAnd (A,B)   -> FAnd(apl_f s A, apl_f s B) 
    | FOr  (A,B)   -> FOr(apl_f s A, apl_f s B) 
    | FImp (A,B)   -> FImp(apl_f s A, apl_f s B) 
    | FEqu (A,B)   -> FEqu(apl_f s A, apl_f s B) 
    | FEq  (A,B,C) -> FEq(apl_f s A, apl_f s B,apl_f s C) 
    | FNot A -> FNot(apl_f s A) 
    | X -> X;;

(* Aplicar una sustitucion a una lista de formulas *)
let apl_lf s l = list__map (apl_f s) l;; 

(* Encuentra un unificador de primer orden de dos formulas proposicionales,
   retorna la pareja (e,u), donde e indica exito o fracaso
   y u es el unificador principal (si no existe es [] vacia ) *)
let rec unif_f = function
      FAnd(A,B),FAnd(X,Y) -> unif_lf([A;B],[X;Y]) 
    | FOr(A,B),FOr(X,Y)   -> unif_lf([A;B],[X;Y]) 
    | FImp(A,B),FImp(X,Y) -> unif_lf([A;B],[X;Y])
    | FEqu(A,B),FEqu(X,Y) -> unif_lf([A;B],[X;Y])
    | FEq(A,B,C),FEq(X,Y,Z) -> unif_lf([A;B;C],[X;Y;Z])
    | FVar(A),X -> (true,[A,X]) 
    | FAto(A),(FPred(_) as X) -> (true,[A,X])
    | FAto(A),(FEq(_) as X)   -> (true,[A,X])
    | FPred(A),FPred(X)   -> (eq_constr(A,X),[])
    | FNot(A),FNot(X)     -> unif_f(A,X)
    | FFalse,FFalse       -> (true,[])
    | FTrue,FTrue         -> (true,[])
    | _                   -> (false,[]) 

and unif_lf = function
      ([],[]) -> (true,[])
    | (x::y,a::b) -> let (e,u) = unif_f (x,a) in
	               if e then 
		         let (e1,u1) = unif_lf (apl_lf u y,b) in
			   if e1 then (true,u@u1)
                           else (false,[])
                     else (false,[]) 
    | _ -> (false,[]);;

(* Aplicar una substitucion a un lamda termino, retorna otro lambda termino *)
let rec apl_t st sf = function
      (TVar y) as X     -> (match busque y st with
		             []   -> X 
                           | a::_ -> a ) 
    | (TLister y) as X  -> (match busque y st with
		             []   -> X 
                           | a::_ -> a ) 
    | TApl(f1,f2,t1,t2) -> TApl (apl_f sf f1,apl_f sf f2,
                               apl_t st sf t1,apl_t st sf t2) 
    | TFun(x,f,y)       -> (match busque x st with
			     []   -> TFun(x,apl_f sf f,apl_t st sf y)
                           | [TVar n] -> TFun(n,apl_f sf f,apl_t st sf y)
			   | _  -> raise TacticFailure)
    | TCase(lf,lt)      -> TCase(list__map (apl_f sf) lf,list__map (apl_t st sf) lt) 
    | TPar(f1,f2,t1,t2) -> TPar(apl_f sf f1,apl_f sf f2,
			        apl_t st sf t1,apl_t st sf t2)
    | TInl(f1,f2,t)     -> TInl(apl_f sf f1,apl_f sf f2,apl_t st sf t)
    | TInr(f1,f2,t)     -> TInr(apl_f sf f1,apl_f sf f2,apl_t st sf t)
    | TFst(f1,f2,t)     -> TFst(apl_f sf f1,apl_f sf f2,apl_t st sf t)
    | TSnd(f1,f2,t)     -> TSnd(apl_f sf f1,apl_f sf f2,apl_t st sf t)
    | TRefl(f1,f2)      -> TRefl(apl_f sf f1,apl_f sf f2)
    | TSim(f1,f2,f3,t) -> TSim(apl_f sf f1,apl_f sf f2,
                               apl_f sf f3,apl_t st sf t)
    | TLis lt           -> TLis (list__map (apl_t st sf) lt) 
    | TSum(t1,t2)       -> TSum (apl_t st sf t1,apl_t st sf t2) 
    | TZ(f,t)           -> TZ(apl_f sf f,apl_t st sf t)
    | (TExi y) as X     -> (match busque y st with
		             []   -> X 
                           | a::_ -> a ) 
    | TZero f           -> TZero(apl_f sf f)
    | T                 -> T;; 

(* Aplicar substitucion gama delta y una substitucion de terminos lambda 
   a una lista de formulasT, retorna una lista de formulasT *)
let rec apl_lft (s,gama,delta) st rendelta = 

(* Aplicar una substitucion gama delta y una substitucion de terminos a una 
   formulaT, retorna una lista de formulasT *)
    let apl_fm = function
          (_,FLisfor "GAMA") -> gama
        | (TLister x,FLisfor "DELTA") ->(match busque x rendelta with 
	       			         [] -> delta 
				       | a::_ -> apl_lft ([],[],[]) a [] delta) 
        | (_,FLisfor "DELTA") -> delta
        | (t,f) -> [apl_t st [] t,apl_f s f] in

    flat_map apl_fm;; 

(* Aplicar substitucion gama delta y una substitucion de terminos lambda a un 
   sequente, retorna un nuevo sequente*)
let apl sf st rendelta = function 
    (l1,l2) -> (apl_lft sf st rendelta l1,apl_lft sf st rendelta l2);;  

(* Aplicar la regla r, dada una substitucion. 
   Retorna una lista de sequentes *) 
let aplr_s subs = list__map (apl (subs.SFor,subs.SGam,subs.SDel)
                           subs.SDef subs.SRen) !(subs.SReg).Pre;; 

(* Componer dos substituciones de lambda terminos. Aplica la primera
   sobre la segunda *)
let rec comp_st st = function 
        [] -> st 
      | (x,y)::z -> (x,apl_t st [] y)::comp_st st z;;

(* Renombrar las variables izquierdas de una sustitucion *) 
let rec ren_izq ren = function 
      [] -> [] 
    | ((x,y) as a)::z -> match busque x ren with
		           [TVar a] ->  (a,y)::ren_izq ren z 
                         | _ -> a::ren_izq ren z;;

(* Indica si dos formulas son iguales *)
let iguales_f f1 f2 =
    let (e,u) = unif_f(f1,f2) in
       if e& u = [] then
	 true
       else   
	 false;;

(*------------------- Unificador para lambda terminos --------------------*)

(* Encuentra un unificador de primer orden de dos lambda terminos,
   retorna la pareja (e,u), donde e indica exito o fracaso
   y u es el unificador principal (si no existe es [] vacia ).
   TPara los terminos que contienen formulas recibe el unificador de
   ellas *)
let rec unif_t sf = function
      (TVar x,((TVar y) as Y))     -> if (x = y) then (true,[])
			              else (true,[x,apl_t [] sf Y])
    | (TVar x, Y)                  -> (true,[x,apl_t [] sf Y])
    | TApl(f,ff,t,tt),TApl(f1,ff1,t1,tt1)          
                                -> unif_lft sf [f;ff][f1;ff1][t;tt][t1;tt1]
    | TZ(f,t), TZ(f1,t1)           -> unif_lft sf [f][f1][t][t1]  
    | (TExi x,((TExi y) as Y))    -> if (x = y) then (true,[])
			           else (true,[x,apl_t [] sf Y])
    | TZero f, TZero f1            -> unif_lft sf [f][f1][][] 
    | TTrue,TTrue                  -> (true,[])
    | TFun(x,f,t),TFun(a,f1,t1)    -> unif_lft sf [f][f1][t][t1] 
    | TPar(f,ff,t,tt),TPar(f1,ff1,t1,tt1) -> unif_lft sf [f;ff][f1;ff1] 
						         [t;tt][t1;tt1]
    | TInl(f,ff,t),TInl(f1,ff1,t1) -> unif_lft sf [f;ff][f1;ff1][t][t1]  
    | TInr(f,ff,t),TInr(f1,ff1,t1) -> unif_lft sf [f;ff][f1;ff1][t][t1] 
    | TFst(f,ff,t),TFst(f1,ff1,t1) -> unif_lft sf [f;ff][f1;ff1][t][t1]
    | TSnd(f,ff,t),TSnd(f1,ff1,t1) -> unif_lft sf [f;ff][f1;ff1][t][t1] 
    | TRefl(f,ff),TRefl(f1,ff1)    -> unif_lft sf [f;ff][f1;ff1][][] 
    | TSim(f1,f2,f3,t),TSim(f1',f2',f3',t')    
                                   -> unif_lft sf [f1;f2;f3]
                                                  [f1';f2';f3'][t][t'] 
    | _                            -> (false,[])

and iguales_lf sf = function
      ([],[]) -> true
    | (x::y,a::b) -> if iguales_f (apl_f sf x) (apl_f sf a) then
         	       iguales_lf sf (y,b)
                     else false 
    | _ -> false 

and unif_lt sf = function
      ([],[]) -> (true,[])
    | (x::y,a::b) -> let (e,u) = unif_t sf (x,a) in
	               if e then 
		         let (e1,u1) = unif_lt sf (y,b) in
			   if e1 then (true,u@u1)
                           else (false,[])
                     else (false,[]) 
    | _ -> (false,[]) 

and unif_lft sf lf lf1 lt lt1 =
      if iguales_lf sf (lf,lf1) then 
        unif_lt sf (lt,lt1)
      else (false,[]);; 

(* Indica si dos terminos son iguales *)
let iguales_t t1 t2 =
    let (e,u) = unif_t [] (t1,t2) in
       if e& u = [] then
	 true
       else   
	 false;;

(* Indica si dos formulas son iguales. Retorna una pareja con el exito
   y un unificador de los dos lambda terminos *)
let iguales_unif uf tr ts fr fs =
    if iguales_f fr fs then
      let (e,ut) = unif_t uf (ts,tr) in
        if e then
	  (true,ut)
        else
	  raise TacticFailure 
    else 
      (false,[]);;

(* Crear una nueva variable *) 
let hipvar = ref ((id_of_string "#")::[]);;

let genvar = fun () -> let id = next_ident_away 
                                 (id_of_string "H") 
                                 !hipvar in
             (hipvar := id::(!hipvar); string_of_id id);;

(* Lista de terminos de una substitucion *)
let listerm = list__map snd ;;

(* Lista de variables de una lista de formulasS *)
let rec lisvar = function 
      [] -> []
    | (TVar x,_)::y -> x::lisvar y
    | (TExi x,_)::y -> x::lisvar y
    | _::y -> lisvar y;;

(* Lista de formulas de una lista de formulasS *)
let rec lisfor = function 
      [] -> []
    | (TVar _,x)::y -> x::lisfor y
    | (TExi _,x)::y -> x::lisfor y
    | _::y -> lisfor y;;

(* Recibe una lista de variables, retorna un renombramiento de ellas *)
let renombra = list__map (function x -> (x,TVar(genvar())));;

(* Obtiene un renombramiento de todas las metavariables delta *)
let renombradelta s rend= 
      let l = lisvar (suce s) in
      list__map (function x -> (x,renombra l)) rend;;

(* Obtiene una substitucion de las metavariables delta, por lista de
   terminos *)
let rec subsdelta = function
      [] -> []
    | (x,y)::Y -> match listerm y with
		    [] -> [] 
		  | [a] -> (x,a)::subsdelta Y
		  | a -> (x,TLis a)::subsdelta Y;;

(* Indica si una formula pertenece a una lista de formulasT.
   Retorna una pareja con el exito y una unificacion de los lambda terminos *)
let rec pertenece uf ant tr fr = function 
      [] -> (false,[])
    | (TLister _,_)::y -> pertenece uf ant tr fr y 
    | (ts,fs)::y -> let (e,ut) = if ant then
				  iguales_unif uf ts tr fr fs 
				else
				  iguales_unif uf tr ts fr fs in
		       if e then 
			 (true,ut)
	               else 
			 pertenece uf ant tr fr y;;

(* Indica si la primera lista de formulasT contiene la segunda.
   Retorna una pareja con el exito y una unificacion de los lambda terminos *)
let rec contiene uf ant l = function 
      [] -> (true,[])
    | (TLister _,_)::y -> contiene uf ant l y 
    | (tr,fr)::y -> let (e1,s1) = pertenece uf ant tr fr l in
		       if e1 then 
			 let (e2,s2) = contiene uf ant l y in
			   if e2 then
			     (true,s1@s2)
                           else
			     (false,[])
	               else 
			 (false,[]);; 

(* Decide si un secuente cumple con las restricciones de aplicacion de una
   regla. Recibe el unificador de la regla con la restriccion. Retorna una
   pareja con el exito y las unificaciones de lambda terminos del antecedente
   y el sucedente del secuente *)
let cumple uf res = function  
      (seql,seqr) -> let (resl,resr) = apl (uf,[],[]) [] [] res  in
		     let (e1,s1) = contiene uf true seql resl in 
		       if e1 then
			 let (e2,s2) = contiene uf false seqr resr in
                           if e2 then
			     (true,s1,s2)
                           else
			     (false,[],[])
                       else
		         (false,[],[]);;

(* Compone una substitucion de formulas con una substitucion de terminos *)
let rec comp_sfst uf = function
      [] -> []
    | (x,y)::z -> (x,apl_t [] uf y)::comp_sfst uf z;;

(* Crea una substitucion para las variables de un lambda termino, basado
   en la regla que aplica *)
let cree_sub s uf ter t ul ur r =
    let lv =  if r.Vardelta then
                ["DELTA",FLis(lisfor (suce s))]
              else [] in      
    let rendelta = renombradelta s r.Rendelta in
    let ren = (renombra r.Ren) @ (subsdelta rendelta) in
    let sd0 = if r.Ant then ur (* Calcular definicion basica *) 
              else 
                match unif_t uf (t,ter) with
                  (false,_) -> raise(TacticFailure)
                | (_,u)     -> ur@u in
    let sd1 = comp_st r.Def sd0 in
    let sd2 = comp_sfst (uf@lv) sd1 in (*Susbstituir var. proposicionales *)
    let sd  = comp_st ren sd2 in (* Componer con un renombramiento *) 
    let st0 = if r.Ant then    (* Calcular sustitucion basica *)
                match unif_t uf (ter,t) with
                  (false,_) -> raise(TacticFailure)
                | (_,u)     -> ul@u  
              else ul in
    let st1 = comp_st st0 r.Sub in
    let st2 = comp_sfst (uf@lv) st1 in (*Susbstituir var. proposicionales *)
    let st3 = ren_izq ren st2 in (* Componer con un renombramiento *) 
    let st = comp_st ren st3 in
    (sd,st,rendelta);;

(* Decide se una regla dada es aplicable sobre un termino (tf)
   de un secuente y un lado de reduccion o. Retorna la sustitucion
   apropiada para la regla o SNil si no existe *)

let rec aplicable s lf tf o = function
      ({Ant=ant;Pri=ter,pri;Res=res}) as r ->
        if o<>ant then SNil
        else
 	  ( match tf with 
	      (TLister _,_) -> SNil
	    | (t,f) ->
                let (ef,uf) = unif_f(pri,f) in
	        if ef then
		  let (et,ul,ur) = cumple uf res s in 
		  if et then
		    let (gam,del) = if ant then (lf,suce s) 
				    else (ante s,lf) in
                    let (sd,st,rn) = cree_sub s uf ter t ul ur r in 
                      conss r uf gam del rn sd st  
                  else SNil
                else SNil);;

(* Dado una regla, retorna una posicion donde la regla sea aplicable. RNil
   si no existe *)
let rec escoja_termino r s o rseq lacum = function
      [] -> if o=0 then
              escoja_termino r s 2 [] lacum rseq
            else if o=1 then
              escoja_termino r s 2 [] [] rseq
          else RNil
    | t::y -> let oo = if o=0 then 1 else o in
            ( match aplicable s (lacum@y) t (oo=1) r with
                SNil -> escoja_termino r s oo rseq (lacum@[t]) y
              | SCons(s) -> 
                  if oo=1 then RCons(s,[],lacum,(y,rseq))
                  else RCons(s,[],lacum,([],y)));;

(* Dado un secuente y un sistema de reglas
   retorna una sustitucion apropiada para la regla, o RNil si no existe *)
let rec escoja_regla s (lrseq,lac) = function  
      []   -> RNil 
    | (r::y) as lreg ->
        ( match escoja_termino r s 0 (suce lrseq) lac (ante lrseq) with
          RNil -> escoja_regla s (s,[]) y
        | RCons(subs,_,lanew,lrnew) -> RCons(subs,lreg,lanew,lrnew));;

(* Si una formula proposicional existe en una lista de formulas *)
let rec existeprop f = function
      [] -> false
    | x::y -> if iguales_f f x then  true
               else existeprop f y;;

(* Buscar una formula proposicional en una lista de formulasT,
   retorna el termino o TZero si no la encuentra *)
let rec busqueprop f = function
      [] -> TZero(FFalse)
    | (tt,ff)::y -> if iguales_f f ff then  tt
               else busqueprop f y;;

(* Crear un termino como aplicaciones sucesivas del subobjetivo sobre las
   hipotesis *)
let rec ter_subobjetivo lisprop subobj = function
      [] -> (fst subobj)
    | (x,f)::y ->
        if existeprop f lisprop then 
          ter_subobjetivo lisprop subobj y 
        else 
          (match snd(subobj) with
             FImp(A,B) -> ter_subobjetivo (f::lisprop)
               (TApl(A,B,fst(subobj),x),B) y
           | _ -> TZero(FFalse));;

(* Convierte la lista del succedente en una disyuncion *)
let rec disyuncion = function
      []   -> FFalse
    | [a]  -> a
    | x::y -> FOr(x,disyuncion y);;
 
(* Convierte la lista del antecedente en una implicacion *)
let rec implicacion dis vp = function 
        [] -> dis
      | x::y -> if (existeprop x vp) then implicacion dis vp y
                    else FImp(x,implicacion dis vp y);;

(* Lista de proposiciones de un secuente sin repetidos *)
let rec it_propos lisacum = function
      [] -> lisacum
    | (_,f)::y -> if (existeprop f lisacum) then it_propos lisacum y
                  else it_propos (lisacum@[f]) y;;

let proposiciones = it_propos [];;

(* Generar una subobjetivo de la demostracion de tal manera que 
   la validez del sequente original sea equivalente a la validez del
   subobjetivo *)
let subobjetivo s vp = 
      let dis = disyuncion (proposiciones (suce s)) in
      let ter = TExi(genvar()) in
        (ter,implicacion dis vp (proposiciones (ante s))),dis;;

(* Crea una substitucion que supone un subobjetivo demostrado *)
let rec termino_caso fapp f = function
      FOr(a,b) ->
        let id1 = genvar() in
        let t1 = TVar(id1) in
        let id2 = genvar() in
        let t2 = TVar(id2) in
        if iguales_f a f then
          TCase([a;b;f],[TFun(id1,a,t1);TFun(id2,b,TZero(f));fapp])
        else
          TCase([a;b;f],[TFun(id1,a,TZero(f));TFun(id2,b,termino_caso t2 f b);
                         fapp])
    | _ -> fapp;;

let rec it_subs_subobj subs sec fapp tip = function 
      [] -> subs
    | ((TVar x,f) as A)::y -> 
        let t = busqueprop f sec in
        if t <> TZero(FFalse) then 
          it_subs_subobj ((x,apl_t subs [] t)::subs) sec fapp tip y
        else 
          it_subs_subobj ((x,termino_caso fapp f tip)::subs) (A::sec) 
                         fapp tip y;;

let subs_subobj fapp tip s = it_subs_subobj [] [] fapp tip s;;

let rec esta_en_case l = function
      TApl(_,_,t1,t2) ->
        (esta_en_case l t1) or (esta_en_case l t2)
    | TFun(_,_,t) -> 
        esta_en_case l t
    | TPar(_,_,t1,t2) -> 
        (esta_en_case l t1) or (esta_en_case l t2)
    | TInl(_,_,t) -> 
        esta_en_case l t
    | TInr(_,_,t) -> 
        esta_en_case l t
    | TFst(_,_,t) -> 
        esta_en_case l t
    | TSnd(_,_,t) -> 
        esta_en_case l t
    | TZ(_,t) -> 
        esta_en_case l t
    | TSum(t1,t2) -> 
        (esta_en_case l t1) or (esta_en_case l t2)
    | TCase([f1;f2;f3],[t1;t2;t3]) ->
      let [ff1;ff2;ff3] = l in
        if (iguales_f f1 ff1) & (iguales_f f2 ff2) & (iguales_f f3 ff3) then
          true 
        else
          (esta_en_case l t1) or (esta_en_case l t2)    
    | _ -> false;;

let rec busque_termino t = function
      [] -> (false,"",false)
    | (x,v,o)::y -> if iguales_t t x then (true,v,o) else busque_termino t y;; 

(* Sistema de reglas para simplificar terminos *)
let rec sistreg lcase = function
      TApl(_,_,TFun (x,_,t),t1)  -> apl_t [x,t1] [] t
    | TFst(_,_,TPar(_,_,t,_)) -> t
    | TSnd(_,_,TPar(_,_,_,t)) -> t
    (* Simplificacion con TZero *)
    | TApl(_,f,TZero _,t) -> TZero f
    | TApl(_,f,t,TZero _) -> TZero f
    | TFun(x,f1,TZero f2) -> TZero (FImp(f1,f2))
    | TPar(f1,f2,TZero _,t2) -> TZero (FAnd(f1,f2)) 
    | TPar(f1,f2,t1,TZero _) -> TZero (FAnd(f1,f2)) 
    | TInl(f1,f2,TZero _) -> TZero (FOr(f1,f2))
    | TInr(f1,f2,TZero _) -> TZero (FOr(f1,f2))
    | TFst(f1,f2,TZero _) -> TZero f1
    | TSnd(f1,f2,TZero _) -> TZero f2
    | TZ(f,TZero _) -> TZero f
    | TSum(TZero _,t) -> t
    | TSum(t,TZero _) -> t
    | TCase([_;_;f],[_;_;TZero _]) -> TZero f
    | TSum(TFun(v1,ff1,t1),TFun(v2,ff2,t2)) ->
        TFun(v1,ff1,TSum(t1,apl_t [v2,(TVar v1)][] t2)) 
    (* Simplificacion del case *)
    | TApl(f1,f2,TCase([A;B;FImp(C,D)],[TFun(v1,ff1,t1);
                                        TFun(v2,ff2,t2);t3]),t) ->
        TCase([A;B;f2],[TFun(v1,ff1,TApl(C,D,t1,t));
                        TFun(v2,ff2,TApl(C,D,t2,t));t3])
    | TApl(f1,f2,t,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3])) ->
        TCase([A;B;f2],[TFun(v1,ff1,TApl(f1,f2,t,t1));
                        TFun(v2,ff2,TApl(f1,f2,t,t2));t3])
    | TPar(f1,f2,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3]),t) ->
        TCase([A;B;FAnd(f1,f2)],[TFun(v1,ff1,TPar(f1,f2,t1,t));
                                 TFun(v2,ff2,TPar(f1,f2,t2,t));t3])
    | TPar(f1,f2,t,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3])) ->
        TCase([A;B;FAnd(f1,f2)],[TFun(v1,ff1,TPar(f1,f2,t,t1));
                                 TFun(v2,ff2,TPar(f1,f2,t,t2));t3])
    | TInl(f1,f2,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3])) ->
        TCase([A;B;FOr(f1,f2)],[TFun(v1,ff1,TInl(f1,f2,t1));
                                TFun(v2,ff2,TInl(f1,f2,t2));t3])
    | TInr(f1,f2,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3])) ->
        TCase([A;B;FOr(f1,f2)],[TFun(v1,ff1,TInr(f1,f2,t1));
                                TFun(v2,ff2,TInr(f1,f2,t2));t3])
    | TFst(f1,f2,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3])) ->
        TCase([A;B;f1],[TFun(v1,ff1,TFst(f1,f2,t1));
                        TFun(v2,ff2,TFst(f1,f2,t2));t3])
    | TSnd(f1,f2,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3])) ->
        TCase([A;B;f2],[TFun(v1,ff1,TSnd(f1,f2,t1));
                        TFun(v2,ff2,TSnd(f1,f2,t2));t3])
    | TZ(f,TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3])) ->
        TCase([A;B;f],[TFun(v1,ff1,TZ(f,t1));
                       TFun(v2,ff2,TZ(f,t2));t3])
    | TSum((TCase([A;B;C],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3]) as TC1),
           (TCase([A';B';C'],
                 [TFun(v1',ff1',t1');TFun(v2',ff2',t2');t3']) as TC2)) ->
      if (iguales_f A A') & (iguales_f B B') then
        TCase([A;B;C],[TFun(v1,ff1,TSum(t1,apl_t [v1',(TVar v1)] [] t1'));
                       TFun(v2,ff2,TSum(t2,apl_t [v2',(TVar v2)] [] t2'));
                       TSum(t3,t3')])
      else if (esta_en_case [A;B;C] t1') or (esta_en_case [A;B;C] t2') then
        TCase([A';B';C'],[TFun(v1',ff1',TSum(t1',TC1));
                          TFun(v2',ff2',TSum(t2',TC1));t3'])
      else
        TCase([A;B;C],[TFun(v1,ff1,TSum(t1,TC2));TFun(v2,ff2,TSum(t2,TC2));t3])
    | TCase([_;_;f],[TFun(_,_,TZero _);TFun(_,_,TZero _);_]) -> TZero(f)
    | TCase([A;B;f],[TFun(v1,f1,t1) as tt1;TFun(v2,f2,t2) as tt2;t]) -> 
        if iguales_t t1 t2 then t2
        else 
          let (exi,var,ori) = busque_termino t lcase in
          if exi then
            if ori then apl_t [v1,TVar var][] t1
            else apl_t [v1, TVar var] [] t2
        else raise(NoAplica)
    | TSum(t1,t2)-> 
        if (iguales_t t1 t2) then t1
        else raise (NoAplica)
    | TPar(_,_,TFst(_,_,t1),TSnd(_,_,t2)) -> 
       if iguales_t t1 t2 then
         t1
       else raise(NoAplica)
    | _ -> raise(NoAplica);;
  
(* Aplicacion de una regla sobre un termino, si no pudo aplicar retorna
   NoAplica. Estrategia mas izquierdo, menos profundo *)

let pr l = hd l;;

let sn l = hd(tl l);;

let rec it_apl_listsistr lcase lacum siapl = function
      [] -> (lacum,siapl)
    | x::y -> let (xp,exi) = try (apl_sistr lcase x,true) with
                                NoAplica -> (x,false) in
              it_apl_listsistr lcase (lacum@[xp]) (exi or siapl) y

and apl_listsistr lcase l = it_apl_listsistr lcase [] false l

and apl_sistr lcase a =
      try sistreg lcase a with 
        NoAplica -> 
          (match a with
             TApl(f1,f2,t1,t2) ->
               let (lt,exi) = apl_listsistr lcase [t1;t2] in
               if exi then TApl(f1,f2,pr lt,sn lt)
               else raise(NoAplica)
           | TFun(x,f,t) -> 
               let (lt,exi) = apl_listsistr lcase [t] in
               if exi then TFun(x,f,pr lt)
               else raise(NoAplica)
           | TPar(f1,f2,t1,t2) -> 
               let (lt,exi) = apl_listsistr lcase [t1;t2] in
               if exi then TPar(f1,f2,pr lt,sn lt)
               else raise(NoAplica)
           | TInl(f1,f2,t) -> 
               let (lt,exi) = apl_listsistr lcase [t] in
               if exi then TInl(f1,f2,pr lt)
               else raise(NoAplica)
           | TInr(f1,f2,t) -> 
               let (lt,exi) = apl_listsistr lcase [t] in
               if exi then TInr(f1,f2,pr lt)
               else raise(NoAplica)
           | TFst(f1,f2,t) -> 
               let (lt,exi) = apl_listsistr lcase [t] in
               if exi then TFst(f1,f2,pr lt)
               else raise(NoAplica)
           | TSnd(f1,f2,t) -> 
               let (lt,exi) = apl_listsistr lcase [t] in
               if exi then TSnd(f1,f2,pr lt)
               else raise(NoAplica)
           | TZ(f,t) -> 
               let (lt,exi) = apl_listsistr lcase [t] in
               if exi then TZ(f,pr lt)
               else raise(NoAplica)
           | TSum(t1,t2) -> 
               let (lt,exi) = apl_listsistr lcase [t1;t2] in
               if exi then TSum(pr lt,sn lt)
               else raise(NoAplica)
           | TCase([f1;f2;f3],[TFun(v1,ff1,t1);TFun(v2,ff2,t2);t3]) ->
               let ([t1'],exi1) = apl_listsistr ((t3,v1,true)::lcase) [t1] in
               let ([t2'],exi2) = apl_listsistr ((t3,v2,false)::lcase) [t2] in
               let ([t3'],exi3) = apl_listsistr lcase [t3] in
               if (exi1 or exi2 or exi3) then 
                 TCase([f1;f2;f3],[TFun(v1,ff1,t1');
                                   TFun(v2,ff2,t2');t3'])
               else raise(NoAplica)
           | _ -> raise(NoAplica));;

(* Indica si hay un zero en el termino *)
let rec tiene_zero = function
      TApl(_,_,t,t1)  -> tiene_zero(t) or tiene_zero(t1)
    | TFun(_,_,t)     -> tiene_zero(t)  
    | TPar(_,_,t,t1)  -> tiene_zero(t) or tiene_zero(t1)
    | TInl(_,_,t)     -> tiene_zero(t)  
    | TInr(_,_,t)     -> tiene_zero(t)  
    | TFst(_,_,t)     -> tiene_zero(t)  
    | TSnd(_,_,t)     -> tiene_zero(t)  
    | TCase(_,[t;t1;t2]) -> tiene_zero (t) or tiene_zero (t1) or 
                         tiene_zero(t2)    
    | TZ(_,t)         -> tiene_zero(t)
    | TZero(f)        -> true
    | A               -> false;;

(* Elemento de la posicion p de una lista *) 
let rec lis_pos p = function
      [] -> raise(TacticFailure)
    | x::y -> if (p=0) then x else lis_pos (p-1) y;;

(* Genera una copia de una formula con reemplazo de los terminos de tipo
   FLis por las formulas que aparecen en la posicion p'esima de las
   listas respectivas *) 
let rec copia_f p = function
      FAnd(A,B) ->  FAnd(copia_f p A,copia_f p B) 
    | FEqu(A,B) ->  FEqu(copia_f p A,copia_f p B)
    | FOr(A,B)  ->  FOr(copia_f p A,copia_f p B) 
    | FImp(A,B) ->  FImp(copia_f p A,copia_f p B) 
    | FNot(A)   ->  FNot(copia_f p A)  
    | FLis lf  ->  lis_pos p lf
    | A        ->  A;;

(* Genera una copia de un termino con reemplazo de los terminos de tipo
   Lista por los terminos que aparecen en la posicion p'esima de las listas
   respectivas *)
let rec copia_t sinplus p = function
      TApl(f,f1,t,t1)  -> TApl(copia_f p f,copia_f p f1,
                             copia_t sinplus p t,copia_t sinplus p t1)
    | TFun(x,f,t) -> TFun(x,copia_f p f,copia_t sinplus p t)  
    | TPar(f,f1,t,t1) -> TPar(copia_f p f,copia_f p f1,
                            copia_t sinplus p t,copia_t sinplus p t1) 
    | TInl(f,f1,t)  -> TInl(copia_f p f,copia_f p f1,copia_t sinplus p t)  
    | TInr(f,f1,t)  -> TInr(copia_f p f,copia_f p f1,copia_t sinplus p t) 
    | TFst(f,f1,t)  -> TFst(copia_f p f,copia_f p f1,copia_t sinplus p t) 
    | TSnd(f,f1,t)  -> TSnd(copia_f p f,copia_f p f1,copia_t sinplus p t) 
    | TLis lt       -> lis_pos p lt 
    | TSum(t,t1)    -> let s = copia_t sinplus p t in
                       let s1 = copia_t sinplus p t1 in
                         if sinplus then
                           if tiene_zero s then s1
                           else s
                         else TSum(s,s1)
    | TCase(lf,lt)  -> TCase(list__map (copia_f p) lf,list__map (copia_t sinplus p) lt) 
    | TZ(f,t)       -> TZ(copia_f p f,copia_t sinplus p t)
    | TZero(f)      -> TZero(copia_f p f)
    | A             -> A;;

(* Reescribe un lambda termino con constructores TZero y TSum a un lambda
   termino *)
let rec normal t =
    try normal(apl_sistr [] t) with
      NoAplica -> copia_t true 0 t;;
 
(*-------------------- Procedimiento de decision  --------------------------*)

(* Indica que no debe buscar mas en el arbol *)
let no_back rev = function
  {Arb=a;Lisbut=l} -> (a <> Nil) & (l=[] or rev);;       
		       
(* Funcion que dice si un sequente es demostrable o no. Retorna 
   retorna un arbol de demostracion del sequente, o vacio. *)
let rec naive intu vp =  function 
      (l,r) as s -> naive_s s intu (s,[]) vp sistema

(* Dado un secuente s y un subsecuente (en el cual busca una formula
   para aplicarle una regla), encuentra un elemento de demostracion. 
   Si intu es true genera subojetivos equivalentes al original en caso 
   de no encontrar la demostracion. Si es false, retorna el arbol Vacio*)

and naive_s s intu seq_acum vp listareg =
    ( match escoja_regla s seq_acum listareg with
        RNil -> 
          if intu then 
            let obj = subobjetivo s vp in
            let fapp = ter_subobjetivo vp (fst obj) (ante s) in 
            let subs_sub = subs_subobj fapp (snd obj) (suce s) in
	      consd (Cons(Nil,consa s VACIA subs_sub [],Nil)) 
                         [fst obj]
          else consd Nil []
      | RCons(subs,lreg,lanew,lrnew) ->
          let reversible = !(subs.SReg).Ssi or subs.SDel = [] in
	  ( match aplr_s subs with 
              []  -> 
                consd(Cons(Nil,
                           consa s !(subs.SReg) subs.SDef subs.STer,
                           Nil)) []
            | [a] -> 
                let {Arb=a1;Lisbut=l1} as al = (naive intu vp a) in
                if no_back reversible al then
                  consd (Cons(a1,
                              consa s !(subs.SReg) subs.SDef subs.STer,
                              Nil)) l1
                else if not reversible then naive_s s intu (lrnew,lanew) vp lreg
                else  consd Nil []
            | a::(b::_) ->  
                let {Arb=a1;Lisbut=l1} as al1 = naive intu vp a in
                let {Arb=a2;Lisbut=l2} as al2 = naive intu vp b in
                if (no_back reversible al1) & (no_back reversible al2) then
                  consd (Cons(a1,
                              consa s !(subs.SReg) subs.SDef subs.STer,
                              a2)) (l1@l2) 
                else if not reversible then naive_s s intu (lrnew,lanew) vp lreg
                else consd Nil []));;

(* Crea nuevas substituciones para cada variable del sucedente *)
let rec nuevas_subs t p = function 
      [] -> [] 
    | x::y -> (x,copia_t false p t) :: nuevas_subs t (p+1) y;; 

(* Busca todos lo Delta que aparecen en el lado izquierdo de la
   sustitucion y lo reemplaza por las variables del sucedente del secuente *)
let rec remplacedelta lisv = function 
      [] -> []
    | ("Delta",t)::y -> nuevas_subs t 0 lisv @ remplacedelta lisv y 
    | x::y -> x :: remplacedelta lisv y;; 

(* Calcula una lista de susbtituciones sobre las variables que aparecen al
   en el sucedente del secuente de un arbol de demostracion. De tal forma
   que al componerlas y aplicarlas se obtienen los lambda terminos que expresan
   la demostracion*) 
let rec subs_t = function
      Nil -> []
    | Cons(a,{Seq=seq;Sd=sd0;St=st0;Reg=r},b) ->
        let sd = if (!r.Rendelta <> []) or (!r.Vardelta) then  
		   remplacedelta (lisvar (suce !seq)) sd0  
		 else sd0 in 
        let st = if (!r.Rendelta <> []) or (!r.Vardelta) then  
		   remplacedelta (lisvar (suce !seq)) st0 
		 else st0 in
        [sd] @ (subs_t a) @ [st] @ (subs_t b);;

(* Funcion que compone recursivamente una substitucion con una lista
   de substituciones *)
let rec componga_r s = function 
      [] -> s
    | x::y -> componga_r (comp_st x s) y;;

(* Dado un arbol de demostracion de un secuente, calcula los lambda terminos 
   que expresan la demostracion *)
let lterm = function
      Nil -> []
    | (Cons(_,{Seq=seq},_)) as a ->
	list__map (function (x,y) -> (x,normal y)) (componga_r [] (subs_t a));;

(*--------------------- Interface con Coq  ---------------------------------*)
(*-- Convierte una formula cci a una formula notacion Tauto --*)

let (TAUTOFAIL:tactic) = fun _ -> errorlabstrm "TAUTOFAIL"
                                          [< 'S "Tauto failed.">]
;;


let mmk = make_module_marker ["#Prelude.obj"];;
let and_pattern = put_pat mmk "(and ? ?)";;
let or_pattern = put_pat mmk "(or ? ?)";;
let not_pattern = put_pat mmk "(not ?)";;
let iff_pattern = put_pat mmk "(iff ? ?)";;
let false_pattern = put_pat mmk "False";;
let true_pattern = put_pat mmk "True";;
let eq_pattern = put_pat mmk "(eq ? ? ?)";;

let is_imp_term T =
    match T with
    DOP2(Prod,_,DLAM(_,B)) -> not(dependent (Rel 1) B)
  | _ -> false
;;

let tauto_of_cci_fmla gls cciterm = tradrec (whd_betaiota cciterm)
    where rec tradrec cciterm =
    if matches gls cciterm and_pattern then
        let [A;B] = dest_match gls cciterm and_pattern
        in FAnd(tradrec A,tradrec B)
    else if matches gls cciterm or_pattern then
        let [A;B] = dest_match gls cciterm or_pattern
        in FOr(tradrec A,tradrec B)
    else if matches gls cciterm iff_pattern then
        let [A;B] = dest_match gls cciterm iff_pattern
        in FEqu(tradrec A,tradrec B)
    else if matches gls cciterm eq_pattern then
        let [A;B;C] = dest_match gls cciterm eq_pattern
        in FEq(FPred A,FPred B, FPred C)
    else if matches gls cciterm not_pattern then
        let [A] = dest_match gls cciterm not_pattern
        in FNot(tradrec A)
    else if matches gls cciterm false_pattern then
        FFalse
    else if matches gls cciterm true_pattern then
        FTrue
    else if is_imp_term cciterm then
        let (DOP2(Prod,A,DLAM(_,B))) = cciterm
        in FImp(tradrec A,tradrec (generic__pop B))
    else FPred cciterm
;;

(*-- Retorna una lista de todas las variables proposicionales que
     aparescan en una lista de formulasS --*)
let rec lisvarprop = function 
      [] -> []
    | (_,((FPred x) as fx))::y -> fx::lisvarprop y
    | _::y -> lisvarprop y;;

(*-- Dado el ambiente COQ construye el lado izquierdo de un secuente
     (hipotesis) --*)
let rec constr_lseq gls = function
     ([],[]) -> []
    |(idx::idy,hx::hy) -> (match (pf_type_of gls hx) with
                            DOP0 (Sort (Prop Null)) -> 
                              (TVar(string_of_id idx),tauto_of_cci_fmla gls hx)
                              :: constr_lseq gls (idy,hy)
                           |_-> constr_lseq gls (idy,hy))
    | _ -> [];;

(*-- Dado un estado COQ construye el lado derecho de un secuente
     (conclusion) --*)
let constr_rseq gls but = [TVar(genvar()),
                           tauto_of_cci_fmla gls but];;

(*-- Calula la posicion de la lista de un elemento --*) 
let rec pos_lis x = function 
      [] -> raise TacticFailure
    | a::r -> if (x=a) then 1
              else 1+(pos_lis x r);; 

(*-- Construye un termino constr dado una formula tauto --*)
let rec cci_of_tauto_fml env = 
  let CAnd = search env (id_of_string "and") 
  and COr =  search env (id_of_string "or")  
  and CFalse = search env (id_of_string "False") 
  and CTrue = search env (id_of_string "True") 
  and CEq = search env (id_of_string "eq") in
  function
      FAnd(A,B) ->  applistc CAnd 
                    [cci_of_tauto_fml env A;cci_of_tauto_fml env B]
    | FOr(A,B)  ->  applistc COr 
                    [cci_of_tauto_fml env A; cci_of_tauto_fml env B]
    | FEq(A,B,C)->  applistc CEq
                    [cci_of_tauto_fml env A; cci_of_tauto_fml env B;
                     cci_of_tauto_fml env C]
    | FImp(A,B) ->  arrow (cci_of_tauto_fml env A) (cci_of_tauto_fml env B) 
    | FPred A   ->  A  
    | FFalse    ->  CFalse
    | FTrue     ->  CTrue
    | FLis lf   ->  raise TacticFailure 
    | FVar A    ->  raise TacticFailure 
    | FAto A    ->  raise TacticFailure
    | FLisfor A ->  raise TacticFailure;;

(*-- Construye un termino constr de un termino tauto --*)
let cci_of_tauto_term env t =
  let CFalse_ind = search env (id_of_string "False_ind")  
  and Cconj = search env (id_of_string "conj")  
  and Cor_ind = search env (id_of_string "or_ind") 
  and Cor_introl = search env (id_of_string "or_introl") 
  and Cor_intror = search env (id_of_string "or_intror") 
  and Cproj1 = search env (id_of_string "proj1") 
  and Cproj2 = search env (id_of_string "proj2")
  and Crefl  = search env (id_of_string "refl_equal") 
  and Csim   = search env (id_of_string "sym_equal") 
  and Ci     = search env (id_of_string "I") in  

  ter_constr [] t 


where rec ter_constr l = function
       TVar x            -> (try (try Rel(pos_lis x l)
                                  with TacticFailure -> 
                                  search env (id_of_string x))
                             with _ -> raise(TacticFailure))
     | TZ(f,x)           -> applistc CFalse_ind
                            [cci_of_tauto_fml env f;ter_constr l x]
     | TSum(t1,t2)       -> ter_constr l t1 
     | TExi (x)          -> (try search env (id_of_string x) with
                             _ -> raise(TacticFailure))
     | TApl(_,_,t1,t2)   -> applistc (ter_constr l t1) [ter_constr l t2]
     | TPar(f1,f2,t1,t2) -> applistc Cconj
                            [cci_of_tauto_fml env f1;cci_of_tauto_fml env f2;
                             ter_constr l t1;ter_constr l t2]
     | TInl(f1,f2,t)     -> applistc Cor_introl
                            [cci_of_tauto_fml env f1;cci_of_tauto_fml env f2;
                             ter_constr l t]
     | TInr(f1,f2,t)     -> applistc Cor_intror
                            [cci_of_tauto_fml env f1;cci_of_tauto_fml env f2;
                             ter_constr l t]
     | TFst(f1,f2,t)     -> applistc Cproj1
                            [cci_of_tauto_fml env f1;cci_of_tauto_fml env f2;
                             ter_constr l t]
     | TSnd(f1,f2,t)     -> applistc Cproj2
                            [cci_of_tauto_fml env f1;cci_of_tauto_fml env f2;
                             ter_constr l t]
     | TRefl(f1,f2)      -> applistc Crefl
                            [cci_of_tauto_fml env f1;cci_of_tauto_fml env f2]
     | TSim(f1,f2,f3,t) -> applistc Csim
                            [cci_of_tauto_fml env f1;cci_of_tauto_fml env f2;
                             cci_of_tauto_fml env f3;ter_constr l t]
     | TCase(lf,lt)      -> applistc Cor_ind
                            ((list__map (cci_of_tauto_fml env) lf)@
                             (list__map (ter_constr l) lt))
     | TFun(n,f,t)       ->  lambda_name(Name(id_of_string n ),
                             cci_of_tauto_fml env f,ter_constr (n::l) t)
     | TTrue             -> Ci
     | TLis _            -> raise TacticFailure
     | TLister _         -> raise TacticFailure
     | TZero _           -> raise TacticFailure ;; 

let CutUsing id T = cut_tac T THENS [IntroUsing id;IDTAC];;

let ParallelCutUsing idlist L = prec (rev idlist) (rev L)
    where rec prec l = function
    [] -> IDTAC
  | h::t -> CutUsing (id_of_string (hd l)) h THENS [prec (tl l) t;IDTAC];;

let Exacto tt gls = 
    match (try cci_of_tauto_term (GLOB (pf_hyps gls)) tt with
             _ -> (DOP0 Prod)) with (* Efectivamente, es cualquier cosa!! *)
      (DOP0 Prod) -> TAUTOFAIL gls  (* Esto confirma el comentario anterior *)
    | t -> (Exact t) gls;;
    
let Subbuts l hyp = ParallelCutUsing
                      (lisvar l) 
                      (list__map (cci_of_tauto_fml (GLOB hyp)) (lisfor l));;

let t_exacto = ref (TVar "#");;

let tautoOR ti gls =
    let hyp = pf_hyps gls in
    hipvar := ids_of_sign hyp; 
    let but = pf_concl gls in
    let seq = (constr_lseq gls hyp,constr_rseq gls but) in 
    let vp = lisvarprop (ante seq) in
    match naive ti vp seq with
      {Arb=Nil} -> 
        TAUTOFAIL gls     
  | {Arb=arb;Lisbut=l} ->
        let se = apl ([],[],[]) (lterm arb) [] seq in
        let tt = fst(hd(suce se)) in
        (t_exacto := tt;
         Subbuts l hyp gls);;

let Exact_Idtac = function 
      0 -> Exacto (!t_exacto)
    | _ -> IDTAC;;

let TautoOR = THEN_i (tautoOR false) Exact_Idtac 0;;

let IntuitionOR = TRY ((THEN_i (tautoOR true) Exact_Idtac 0) THEN  
                       (automatic !SEARCH_DEPTH));;
                

(*--- Fin Code C.A. Munoz...Start Code Chet M. ---*)
#infix "o";;
#open "more_util";; 

let rec Prove  tauto_intu g   =
     ((ApplyToAHyp (ClauseTacThen (dyck_hypothesis o outSOME) 
                                  (Prove tauto_intu))) 
     ORELSE
     (ApplyToAHyp (ClauseTacThen (dyck_absurdity_elim o outSOME) 
                                 (Prove tauto_intu))) 
     ORELSE
     (ApplyToAHyp (ClauseTacThen (dyck_and_elim o outSOME) 
                                 (Prove tauto_intu))) 
     ORELSE
     (ApplyToAHyp (ClauseTacThen (dyck_atomic_imply_elim o outSOME) 
                                 (Prove tauto_intu))) 
     ORELSE
     (ApplyToAHyp (ClauseTacThen (dyck_and_imply_elim o outSOME) 
                                 (Prove tauto_intu))) 
     ORELSE
     (ApplyToAHyp (ClauseTacThen (dyck_or_imply_elim o outSOME) 
                                 (Prove tauto_intu))) 
     ORELSE
     (ApplyToAHyp (ClauseTacThen (dyck_imply_imply_elim o outSOME) 
                                 (Prove tauto_intu))) 
     ORELSE
     (dyck_and_intro THEN (Prove tauto_intu)) 
     ORELSE
     (dyck_imply_intro THEN (Prove tauto_intu)) 
     ORELSE
     (ApplyToAHyp (ClauseTacThen (dyck_or_elim o outSOME) 
                                 (Prove tauto_intu))) 
     ORELSE
     tauto_intu 
     ORELSE
     TAUTOFAIL)  g 
;;

let Tauto gls =
    (OnAllClausesLR (Unfolds [id_of_string "not";id_of_string"iff"]) THEN 
     (Prove TautoOR)) gls;;

let Intuition gls =
    (OnAllClausesLR (Unfolds [id_of_string "not";id_of_string"iff"]) THEN 
    (Prove IntuitionOR)) gls;;

let Tauto_tac = register_atomic_tactic "Tauto" Tauto;;

let Intuition_tac = register_atomic_tactic "Intuition" Intuition;;



(* $Id: tauto.ml,v 1.6 1996/02/12 15:57:07 cpaulin Exp $ *)
