(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	            Relations between UA and Adjunctions      		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export CoUniversalArrow.
Require Export Adjunction.

(* extraction d'un UA et d'un coUA a` partir d'un adjonction *)

Section adj_to_ua.

Variable C,D : Category.
Variable F   : (Functor D C).
Variable G   : (Functor C D).
Variable ad  : (Adj F G).

Variable d   : (Ob D).

(* Prouver que pour tout d, <F(d),phi(Id(F(d)))> est UA de d vers G *)

Definition Unit_ob := (FOb F d).

Definition Unit_arrow := (ApAphi ad (Id (FOb F d))).
 
Definition Unit_arrow_diese := [c:(Ob C)][f:|(Hom d (FOb G c))|] 
                               (ApAphi_inv ad f).

(* qq c:C. qq f:d->Gc. f=phi<d,Fd>(Id(Fd)) o G(phi-1<d,c>(f)) (@) *)
(* remarquons que phi<d,c>(Id(Fd) o phi-1<d,c>(f)) = f *)
(* donc (@) est une instance de Adj_eq3 *)

Lemma Unit_UAlaw1 : (UA_law1 Unit_arrow Unit_arrow_diese).
Proof.
Unfold ua_law1 ua_eq Unit_arrow_diese Unit_arrow.
Intros c1 f.
Apply (Prf_sym (Hom d (FOb G c1))).
Apply (Prf_trans (Hom d (FOb G c1))) with
  y:=(ApAphi ad
        ((Id (FOb F d)) o (ApAphi_inv ad  f))).
Apply (Prf_trans (Hom d (FOb G c1))) with
  y:=(ApAphi ad
         (ApAphi_inv ad f)).
Apply (Prf_sym (Hom d (FOb G c1))).
Exact (Check_Ta_idl ? ? ? ? (Aphi ad) (Aphi_iso ad) 
                   (Build_POb1 d c1) f).
Unfold apAphi; Apply (Pres ? ?  (ApNT (Aphi ad) (Build_POb1 d c1))).     
Apply (Prf_sym (Hom (FOb F d) c1)).
Unfold Comp; Apply (Prf_idl C).
Apply (Adj_eq3 ? ? ? ? ad ? ? ? 
            (Ap (ApNT (Aphi_inv ad) (Build_POb1 d c1)) f)
            (Id (FOb F d))).
Qed.

(* qq c1:C. qq f:d->Gc1. qq f':Fd->c1. *)
(* phi<d,Fd>(Id(Fd)) o G(f') = f (@) *)
(* => f'=phi-1<d,c1>(f) *)
(* appliquer phi-1<d,c1> aux 2 membres de (@) *)
(* En appliquant Adj_eq5, on obtient le re'sultat souhaite' *)

Lemma Unit_UAlaw2 : (UA_law2 Unit_arrow Unit_arrow_diese).
Proof.
Unfold ua_law2 ua_eq Unit_arrow_diese Unit_arrow.
Intros c1 f f' H.
Apply (Prf_trans (Hom (FOb F d) c1)) with
  y:=(ApAphi_inv ad 
          ((ApAphi ad (Id (FOb F d))) o (FMor G f'))).
Apply (Prf_sym (Hom (FOb F d) c1)).
Apply (Prf_trans (Hom (FOb F d) c1)) with
  y:=(ApAphi_inv ad
        (ApAphi ad (Id (FOb F d)))) o f'.
Apply (Adj_eq5 ? ? ? ? ad ? ? ? f' 
           (ApAphi ad (Id (FOb F d)))).
Apply (Prf_trans (Hom (FOb F d) c1)) with
  y:=(Id (FOb F d)) o f'.
Apply (Prf_congr C).
Apply (Check_Ta_idr ? ? ? ? (Aphi ad) (Aphi_iso ad) 
                    (Build_POb1 d (FOb F d)) (Id (FOb F d))).
Unfold Comp; Apply (Prf_idl C).
Unfold apAphi_inv; Apply (Pres ? ? (ApNT (Aphi_inv ad) (Build_POb1 d c1))).
Apply H.
Qed.

Lemma Unit' : (IsUA d G Unit_ob Unit_arrow).
Proof.
Exact (Build_isUA C D d G Unit_ob Unit_arrow Unit_arrow_diese
                Unit_UAlaw1 Unit_UAlaw2).
Defined.

Definition Unit := (Build_ua C D d G Unit_ob Unit_arrow Unit').

(* Prouver que pour tout c, <G(c),phi-1(Id(G(c)))> est UA de c vers F *)

Variable c : (Ob C).

Definition CoUnit_ob := (FOb G c).

Definition CoUnit_arrow := (ApAphi_inv ad (Id (FOb G c))).

Definition CoUnit_arrow_diese := [d:(Ob D)][f:|(Hom (FOb F d) c)|] 
                                 (ApAphi ad f).
                        
Lemma CoUnit_coUAlaw1 : (CoUA_law1 CoUnit_arrow CoUnit_arrow_diese).
Proof.
Unfold coUA_law1 coUA_eq CoUnit_arrow_diese CoUnit_arrow.
Intros d1 f.
(* *) Apply (Prf_trans (Hom (FOb F d1) c)) with
  y:=(ApAphi_inv ad ((ApAphi ad f) o (Id (FOb G c)))).
Apply (Prf_sym (Hom (FOb F d1) c)).
Apply (Adj_eq6 ? ? ? ? ad ? ? ? 
            (ApAphi ad f) (Id (FOb G c))).
(* *) Apply (Prf_trans (Hom (FOb F d1) c)) with
  y:=(ApAphi_inv ad (ApAphi ad f)).
Unfold apAphi_inv; Apply (Pres ? ? (ApNT (Aphi_inv ad) (Build_POb1 d1 c))).
Apply (Prf_sym (Hom d1 (FOb G c))).
Unfold Comp; Apply (Prf_idr D).         
Apply (Check_Ta_idr ? ? ? ? (Aphi ad) (Aphi_iso ad) 
                    (Build_POb1 d1 c) f).
Qed.

Lemma CoUnit_coUAlaw2 : (CoUA_law2 CoUnit_arrow CoUnit_arrow_diese).
Proof.
Unfold coUA_law2 coUA_eq CoUnit_arrow_diese CoUnit_arrow.
Intros d1 f g H.
(* *) Apply (Prf_trans (Hom d1 CoUnit_ob)) with
  y:= (ApAphi ad ((FMor F g) o (ApAphi_inv ad (Id (FOb G c))))).
(* *) Apply (Prf_trans (Hom d1 CoUnit_ob)) with
  y:= g o (ApAphi ad (ApAphi_inv ad (Id (FOb G c)))).
(* *) Apply (Prf_trans (Hom d1 CoUnit_ob)) with
  y:= g o (Id (FOb G c)).
Unfold Comp; Apply (Prf_idr D).         
Apply (Prf_congl D).
Apply (Prf_sym (Hom (FOb G c) (FOb G c))).
Apply (Check_Ta_idl ? ? ? ? (Aphi ad) (Aphi_iso ad) 
                    (Build_POb1 (FOb G c) c) (Id (FOb G c))).
Apply (Prf_sym (Hom d1 CoUnit_ob)).
Apply (Adj_eq4 ? ? ? ? ad ? ? ? g 
            (ApAphi_inv ad (Id (FOb G c)))).
Unfold apAphi; Apply (Pres ? ? (ApNT (Aphi ad) (Build_POb1 d1 c))).
Apply H.
Qed.

Lemma CoUnit' : (IsCoUA c F CoUnit_ob CoUnit_arrow).
Proof.
Exact (Build_isCoUA D C c F CoUnit_ob CoUnit_arrow CoUnit_arrow_diese
                            CoUnit_coUAlaw1 CoUnit_coUAlaw2).
Defined.

Definition CoUnit := (Build_coUA D C c F CoUnit_ob CoUnit_arrow CoUnit').

End adj_to_ua.


(* $Id: Adj_UA.v,v 1.2 1996/02/12 15:22:44 cpaulin Exp $ *)
