(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                       Adjunctions                		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)


Require Export NatIso.
Require Export HomFunctor2.


Section adj_def.

Variable C, D : Category.
Variable F    : (Functor D C).
Variable G    : (Functor C D).

Definition adj := (NatIso (FunSET2_r F) (FunSET2_l G)).

Definition build_Adj := [phi:(NT (FunSET2_r F) (FunSET2_l G))]
                        [phi_iso:(IsNatIso phi)]
                        (Build_iso (CatFunct (PROD (Dual D) C) SET) ? ? phi phi_iso).

 Section adj_elts.

 Variable ad : adj.
                     
 Definition aphi := (Iso_mor (CatFunct (PROD (Dual D) C) SET) ? ? ad).

 Definition aphi_iso := (Prf_isIso (CatFunct (PROD (Dual D) C) SET) ? ? ad).

 Definition aphi_inv := (Inv_iso aphi_iso).

 End adj_elts.
 
Variable d  : (Ob D).
Variable c  : (Ob C).
Variable ad : adj.
 
Definition apAphi := [f:|(Hom (FOb F d) c)|]
                     (Ap (ApNT (aphi ad) (Build_POb1 d c)) f).

Definition apAphi_inv := [f:|(Hom d (FOb G c))|]
                         (Ap (ApNT (aphi_inv ad) (Build_POb1 d c)) f).

End adj_def.


Syntactic Definition Adj := adj | 2.
Syntax constr adj <<(adj $1 $2 $F $G)>> 10
		[<hov 0> "Adj " <$F:"Term":L> " " <$G:"Term":L> ].

Syntactic Definition Build_Adj := build_Adj | 2.
Syntax constr build_Adj <<(build_Adj $1 $2 $F $G $phi $pi)>> 10
		[<hov 0> "Build_Adj " <$F:"Term":L> " " <$G:"Term":L> " " 
                                      <$phi:"Term":L> " " <$pi:"Term":L>].

Syntactic Definition Aphi := aphi | 4.
Syntax constr aphi <<(aphi $1 $2 $3 $4 $ad)>> 10
		[<hov 0> "Aphi " <$ad:"Term":L>].

Syntactic Definition Aphi_iso := aphi_iso | 4.
Syntax constr aphi_iso <<(aphi_iso $1 $2 $3 $4 $ad)>> 10
		[<hov 0> "Aphi_iso " <$ad:"Term":L>].

Syntactic Definition Aphi_inv := aphi_inv | 4.
Syntax constr Aphi_inv <<(aphi_inv $1 $2 $3 $4 $ad)>> 10
		[<hov 0> "Aphi_inv " <$ad:"Term":L>].

Syntactic Definition ApAphi := apAphi | 6.
Syntax constr apAphi <<(apAphi $1 $2 $3 $4 $5 $6 $ad $f)>> 10
       [<hov 0> "ApAphi " <$ad:"term":L> " " <$f:"term":L>].
                   
Syntactic Definition ApAphi_inv := apAphi_inv | 6.
Syntax constr apAphi_inv <<(apAphi_inv $1 $2 $3 $4 $5 $6 $ad $f)>> 10
       [<hov 0> "ApAphi_inv " <$ad:"term":L> " " <$f:"term":L>].


(* Left Adjunction *)

Structure leftAdj [C,D:Category;G:(Functor C D)] : Type :=
 {adjoint : (Functor D C);
  adj_l   : (Adj adjoint G)}.

Syntactic Definition LeftAdj := leftAdj | 2.
Syntax constr leftAdj <<(leftAdj $1 $2 $G)>> 10
	     [<hov 0> "LeftAdj " " " <$G:"term":L> ].

Syntactic Definition Adjoint := adjoint | 3.
Syntax constr adjoint <<(adjoint $1 $2 $3 $la)>> 10
	     [<hov 0> "Adjoint " " " <$la:"term":L> ].

Syntactic Definition Adj_l := adj_l | 3.
Syntax constr adj_l <<(adj_l $1 $2 $3 $la)>> 10
	     [<hov 0> "Adj_l " " " <$la:"term":L> ].

(* Right Adjunction *)

Structure rightAdj [C,D:Category;F:(Functor D C)] : Type :=
 {coadjoint : (Functor C D);
  adj_r     : (Adj F coadjoint)}.

Syntactic Definition RightAdj := rightAdj | 2.
Syntax constr rightAdj <<(rightAdj $1 $2 $F)>> 10
	     [<hov 0> "RightAdj " " " <$F:"term":L> ].

Syntactic Definition Coadjoint := coadjoint | 3.
Syntax constr coadjoint <<(coadjoint $1 $2 $3 $ra)>> 10
	     [<hov 0> "Coadjoint " " " <$ra:"term":L> ].

Syntactic Definition Adj_r := adj_r | 3.
Syntax constr adj_r <<(adj_r $1 $2 $3 $ra)>> 10
	     [<hov 0> "Adj_r " " " <$ra:"term":L> ].

Section eqs.

Variable C,D   : Category.
Variable F     : (Functor D C).
Variable G     : (Functor C D).
Variable ad    : (Adj F G).
Variable d, d' : (Ob D).
Variable c, c' : (Ob C).
Variable h     : |(Hom d' d)|.
Variable k     : |(Hom c c')|.
Variable f     : |(Hom (FOb F d) c)|.

Lemma Adj_eq1 : (ApAphi ad (((FMor F h) o f) o k)) =%S
                ((h o (ApAphi ad f)) o (FMor G k)).
Proof.
Apply (Prf_NT_law ? ? ? ? (Aphi ad) (Build_POb1 d c) (Build_POb1 d' c')
      (build_Pmor (Dual D) C (Build_POb1 d c) (Build_POb1 d' c') h k) f).
Qed.

Variable g    : |(Hom d (FOb G c))|.

Lemma Adj_eq2 : (ApAphi_inv ad ((h o g) o (FMor G k))) =%S
                (((FMor F h) o (ApAphi_inv ad g)) o k).
Proof.
Apply (Prf_NT_law ? ? ? ? (Aphi_inv ad) (Build_POb1 d c) (Build_POb1 d' c')
     (build_Pmor (Dual D) C (Build_POb1 d c) (Build_POb1 d' c') h k) g).      
Qed.
       
End eqs.


Section eqs1.

Variable C, D  : Category.
Variable F     : (Functor D C).
Variable G     : (Functor C D).
Variable ad    : (Adj F G).
Variable d, d' : (Ob D).
Variable c, c' : (Ob C).
Variable h     : |(Hom d' d)|.
Variable k     : |(Hom c c')|.
Variable f     : |(Hom (FOb F d) c)|.
Variable g     : |(Hom d (FOb G c))|.

Lemma Adj_eq3 : (ApAphi ad (f o k)) =%S ((ApAphi ad f) o (FMor G k)).
Proof.
Apply (Prf_trans (Hom d (FOb G c'))) with
  y:=(ApAphi ad (((FMor F (Id d)) o f) o k)).
Unfold apAphi; Apply (Pres ? ? (ApNT (Aphi ad) (Build_POb1 d c'))).
Apply (Prf_congr C (FOb F d) c c' f ((FMor F (Id d)) o f) k).
(* *) Apply (Prf_trans (Hom (FOb F d) c)) with
  y:=(Id (FOb F d)) o f.
Apply (Prf_sym (Hom (FOb F d) c)).
Unfold Comp; Apply (Prf_idl C).
Apply (Prf_congr C).
Apply (Prf_sym (Hom (FOb F d)(FOb F d))).
Unfold fMor; Apply (Prf_Fid_law ? ? F).
(* *) Apply (Prf_trans (Hom d (FOb G c'))) with
  y:=(((Id d) o (ApAphi ad f)) o 
            (FMor G k)).
Apply (Adj_eq1 ? ? ? ? ad d d ? ? (Id d) k f).
Apply (Prf_congr D).
Unfold Comp; Apply (Prf_idl D).
Qed.

Lemma Adj_eq4 : (ApAphi ad ((FMor F h) o f)) =%S (h o (ApAphi ad f)).
Proof.
(* *) Apply (Prf_trans (Hom d' (FOb G c))) with
  y:=(ApAphi ad (((FMor F h) o f) o (Id c))).
Unfold apAphi; Apply (Pres ? ? (ApNT (Aphi ad) (Build_POb1 d' c))).
Unfold Comp; Apply (Prf_idr C ? ? ((FMor F h) o f)).
(* *) Apply (Prf_trans (Hom d' (FOb G c))) with
  y:=(h o (ApAphi ad f)) o (FMor G (Id c)).
Apply (Adj_eq1 ? ? ? ? ad ? ? ? ? h (Id c) f).
(* *) Apply (Prf_trans (Hom d' (FOb G c))) with
  y:=(h o (ApAphi ad f)) o 
     (Id (FOb G c)).
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fid_law ? ? G).
Apply (Prf_sym (Hom d' (FOb G c))).
Unfold Comp; Apply (Prf_idr D).
Qed.

Lemma Adj_eq5 : (ApAphi_inv ad (g o (FMor G k))) =%S ((ApAphi_inv ad g) o k).
Proof.
(* *) Apply (Prf_trans (Hom (FOb F d) c')) with
 y:=(ApAphi_inv ad 
        (((Id d)o g) o (FMor G k))).
Unfold apAphi_inv; Apply (Pres ? ? (ApNT (Aphi_inv ad) (Build_POb1 d c'))). 
Apply (Prf_congr D ? ? ? g ((Id d) o g) (FMor G k)).
Apply (Prf_sym (Hom d (FOb G c))).
Unfold Comp; Apply (Prf_idl D).
(* *) Apply (Prf_trans (Hom (FOb F d) c')) with
  y:=((FMor F (Id d)) o (ApAphi_inv ad g)) o k.
Apply (Adj_eq2 ? ? ? ? ad d d ? ? (Id d) k g).
Apply (Prf_congr C).
(* *) Apply (Prf_trans (Hom (FOb F d) c)) with
  y:=(Id (FOb F d))o (ApAphi_inv ad g).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fid_law ? ? F).
Unfold Comp; Apply (Prf_idl C).
Qed.
        
Lemma Adj_eq6 : (ApAphi_inv ad (h o g)) =%S ((FMor F h) o (ApAphi_inv ad g)).
Proof.
(* *) Apply (Prf_trans (Hom (FOb F d') c)) with
  y:=(ApAphi_inv ad
                   ((h o g) o (FMor G (Id c)))).
Unfold apAphi_inv; Apply (Pres ? ? (ApNT (Aphi_inv ad) (Build_POb1 d' c))).
(* *) Apply (Prf_trans (Hom d' (FOb G c)))with
  y:=(h o g) o (Id (FOb G c)).
Unfold Comp; Apply (Prf_idr D).
Apply (Prf_congl D).
Apply (Prf_sym (Hom (FOb G c) (FOb G c))).
Unfold fMor; Apply (Prf_Fid_law ? ? G).
(* *) Apply (Prf_trans (Hom (FOb F d') c)) with
  y:=((FMor F h) o (ApAphi_inv ad g)) o (Id c).
Apply (Adj_eq2 ? ? ? ? ad ? ? ? ? h (Id c) g).
Apply (Prf_sym (Hom (FOb F d') c)).
Unfold Comp; Apply (Prf_idr C).
Qed.

End eqs1.



(* $Id: Adjunction.v,v 1.2 1996/02/12 15:22:45 cpaulin Exp $ *)
