(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*            2nd definition (Equational) of Adjunctions       		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export IdCAT.
Require Export Adj_UA.

(* lois des NT: eta et epsilon *)

Section adj1_def.

Variable C,D : Category.
Variable F   : (Functor D C).
Variable G   : (Functor C D).

Variable eta : (NT (Id_CAT D) (F o%F G)).
Variable eps : (NT (G o%F F) (Id_CAT C)).

(* Rmq: on utilise idCAT, qui de'finit Id_CAT sans parler
   de Functor_Setoid. Evitant ainsi l'inconsistance des
   univers *)

Definition adj1_law1 := (c:(Ob C))
  ((ApNT eta (FOb G c)) o (FMor G (ApNT eps c))) =%S (Id (FOb G c)).

Definition adj1_law2 := (d:(Ob D)) 
  ((FMor F (ApNT eta d)) o (ApNT eps (FOb F d))) =%S (Id (FOb F d)).

End adj1_def.

Syntactic Definition Adj1_law1 := adj1_law1 | 4.
Syntax constr adj1_law1 <<(adj1_law1 $1 $2 $3 $4 $eta $eps)>> 10
       [<hov 0> "Adj1_law1 " <$eta:"term":L> " " <$eps:"term":L>].

Syntactic Definition Adj1_law2 := adj1_law2 | 4.
Syntax constr adj1_law2 <<(adj1_law2 $1 $2 $3 $4 $eta $eps)>> 10
       [<hov 0> "Adj1_law2 " <$eta:"term":L> " " <$eps:"term":L>].


Structure adj1 [C,D:Category;F:(Functor D C);G:(Functor C D)] : Type := build_Adj1
 {a_eta         : (NT (Id_CAT D) (F o%F G));
  a_eps         : (NT (G o%F F) (Id_CAT C));
  Prf_Adj1_law1 : (Adj1_law1 a_eta a_eps);
  Prf_Adj1_law2 : (Adj1_law2 a_eta a_eps)}.

Syntactic Definition Adj1 := adj1 | 2.
Syntax constr adj1 <<(adj1 $1 $2 $F $G)>> 10
		[<hov 0> "Adj1 " <$F:"Term":L> " " <$G:"Term":L> ].

Syntactic Definition Build_Adj1 := build_Adj1 | 4.
Syntax constr build_Adj1 <<(build_Adj1 $1 $2 $3 $4 $eta $eps $l1 $l2)>> 10
		[<hov 0> "Build_Adj1 " <$eta:"Term":L> " " <$epa:"Term":L> " " 
                                       <$l1:"Term":L> " " <$l2:"Term":L>].    

Syntactic Definition A_eta := a_eta | 4.
Syntax constr a_eta <<(a_eta $1 $2 $3 $4 $ad1)>> 10
		[<hov 0> "A_eta " <$ad1:"Term":L>].

Syntactic Definition A_eps := a_eps | 4.
Syntax constr a_eps <<(a_eps $1 $2 $3 $4 $ad1)>> 10
		[<hov 0> "A_eps " <$ad1:"Term":L>].


Section adj_to_adj1.

Variable C, D : Category.
Variable F    : (Functor D C).
Variable G    : (Functor C D).
Variable ad   : (Adj F G).

(* *)

Definition Unit_tau := [d:(Ob D)](UA_mor (Unit ? ? ? ? ad d)).

Lemma Unit_nt_law : (NT_law (Id_CAT D) (F o%F G) Unit_tau).
Proof.
Unfold nt_law Unit_tau; Simpl.
Unfold Unit_arrow.
Unfold 1 2 fMor; Simpl.
Unfold comp_FMor.
Intros d1 d2 f.
(* *) Apply (Prf_trans (Hom d1 (FOb G (FOb F d2)))) with
  y:=(ApAphi ad ((FMor F f) o  (Id (FOb F d2)))).
Apply (Prf_sym (Hom d1 (FOb G (FOb F d2)))).
Apply (Adj_eq4 ? ? ? ? ad ? ? ? f (Id (FOb F d2))).
(* *) Apply (Prf_trans (Hom d1 (FOb G (FOb F d2)))) with
  y:=(ApAphi ad 
         ((Id (FOb F d1)) o (FMor F f))).
Unfold apAphi; Apply (Pres ? ? (ApNT (Aphi ad) (Build_POb1 d1 (FOb F d2)))).
(* *) Apply (Prf_trans (Hom (FOb F d1) (FOb F d2))) with
  y:=(FMor F f).
Apply (Prf_sym (Hom (FOb F d1) (FOb F d2))).
Unfold Comp; Apply (Prf_idr C).
Apply (Prf_sym (Hom (FOb F d1) (FOb F d2))).
Unfold Comp; Apply (Prf_idl C).
Apply (Adj_eq3 ? ? ? ? ad ? ? ? (FMor F f) (Id (FOb F d1))).
Qed.

Definition Unit_NT := (Build_NT (Id_CAT D) (F o%F G) Unit_tau Unit_nt_law).

(* *)

Definition CoUnit_tau := [c:(Ob C)](CoUA_mor (CoUnit ? ? ? ? ad c)).

Lemma CoUnit_nt_law : (NT_law (G o%F F) (Id_CAT C) CoUnit_tau).
Proof.
Unfold nt_law CoUnit_tau; Simpl.
Unfold CoUnit_arrow.
Unfold 1 2 fMor; Simpl.
Unfold comp_FMor.
Intros c1 c2 f.
(* *) Apply (Prf_trans (Hom (FOb F (FOb G c1)) c2)) with
  y:=(ApAphi_inv ad
         ((FMor G f) o (Id (FOb G c2)))).
Apply (Prf_sym (Hom (FOb F (FOb G c1)) c2)).
Apply (Adj_eq6 ? ? ? ? ad ? ? ? (FMor G f)  (Id (FOb G c2))). 
(* *) Apply (Prf_trans (Hom (FOb F (FOb G c1)) c2)) with
  y:=(ApAphi_inv ad
         ((Id (FOb G c1)) o (FMor G f))).
Unfold apAphi_inv; Apply (Pres ? ? (ApNT (Aphi_inv ad) (Build_POb1 (FOb G c1) c2))).
(* *) Apply (Prf_trans (Hom (FOb G c1) (FOb G c2))) with
  y:=(FMor G f).
Apply (Prf_sym (Hom (FOb G c1) (FOb G c2))).
Unfold Comp; Apply (Prf_idr D).
Apply (Prf_sym (Hom (FOb G c1) (FOb G c2))).
Unfold Comp; Apply (Prf_idl D).
Apply (Adj_eq5 ? ? ? ? ad ? ? ? f (Id (FOb G c1))).
Qed.
 
Definition CoUnit_NT := (Build_NT (G o%F F) (Id_CAT C) CoUnit_tau CoUnit_nt_law).

(* Unit et CoUnit verifient ces deux lois *)

Lemma Unit_and_CoUnit_law1 : (Adj1_law1 Unit_NT CoUnit_NT).
Proof.
Unfold adj1_law1 Unit_NT CoUnit_NT; Simpl.
Unfold Unit_tau CoUnit_tau; Simpl.
Unfold Unit_arrow CoUnit_arrow.
Intro c.
(* *) Apply (Prf_trans (Hom (FOb G c) (FOb G c))) with
  y:= (ApAphi ad
          ((Id (FOb F (FOb G c))) o 
           (ApAphi_inv ad
               (Id (FOb G c))))).
Apply (Prf_sym (Hom (FOb G c) (FOb G c))).
Apply (Adj_eq3 ? ? ? ? ad ? ? ? 
            (ApAphi_inv ad
                (Id (FOb G c)))
             (Id (FOb F (FOb G c)))).
(* *) Apply (Prf_trans (Hom (FOb G c) (FOb G c))) with
  y:= (ApAphi ad
          (ApAphi_inv ad
              (Id (FOb G c)))).
Unfold apAphi; Apply (Pres ? ? (ApNT (Aphi ad) (Build_POb1 (FOb G c) c))).
Unfold 1 Comp; Apply (Prf_idl C (FOb F (FOb G c)) c).
Apply (Check_Ta_idl ? ? ? ? (Aphi ad) (Aphi_iso ad) 
                    (Build_POb1 (FOb G c) c) (Id (FOb G c))).
Qed.

Lemma Unit_and_CoUnit_law2 : (Adj1_law2 Unit_NT CoUnit_NT).
Proof.
Unfold adj1_law2 Unit_NT CoUnit_NT; Simpl.
Unfold Unit_tau CoUnit_tau; Simpl.
Unfold Unit_arrow CoUnit_arrow.
Intro d.
(* *) Apply (Prf_trans (Hom (FOb F d) (FOb F d))) with
  y:=(ApAphi_inv ad
         ((ApAphi ad
              (Id (FOb F d))) o (Id (FOb G (FOb F d))))).
Apply (Prf_sym (Hom (FOb F d) (FOb F d))).
Apply (Adj_eq6 ? ? ? ? ad ? ? ? 
            (ApAphi ad
              (Id (FOb F d))) (Id (FOb G (FOb F d)))).
(* *) Apply (Prf_trans (Hom (FOb F d) (FOb F d))) with
  y:=(ApAphi_inv ad
         (ApAphi ad
              (Id (FOb F d)))).
Unfold apAphi_inv; Apply (Pres ? ? (ApNT (Aphi_inv ad) (Build_POb1 d (FOb F d)))).
Apply (Prf_sym (Hom d (FOb G (FOb F d)))).
Unfold Comp; Apply (Prf_idr D).
Apply (Check_Ta_idr ? ? ? ? (Aphi ad) (Aphi_iso ad) 
                    (Build_POb1 d (FOb F d)) (Id (FOb F d))).
Qed.

Definition Adj_to_Adj1 := (Build_Adj1 Unit_NT CoUnit_NT Unit_and_CoUnit_law1 Unit_and_CoUnit_law2).

End adj_to_adj1.


(* *)

Section adj1_to_adj.

Variable C,D : Category.
Variable F   : (Functor D C).
Variable G   : (Functor C D).
Variable ad  : (Adj1 F G).

(* a` partir de ces elements, construisons une adjunction, i.e.
   un iso teta : C(F-,-) -> D(-,G-) *)

 Section teta_tau_def.

 Variable dxc : (POb (Dual D) C).

 Definition Teta_arrow := [f:|(Hom (FOb F (Ob_l dxc)) (Ob_r dxc))|]
                          (ApNT (A_eta ad) (Ob_l dxc)) o (FMor G f).

 Lemma Teta_arrow_map_law : (Map_law Teta_arrow).
 Proof.
 Unfold map_law Teta_arrow.
 Intros f g H.
 Apply (Prf_congl D).
 Unfold fMor.
 Apply (Prf_FMap_law ? ? G  (FOb F (Ob_l dxc)) (Ob_r dxc)).
 Assumption.
 Qed.

 Definition Teta_tau := (Build_Map Teta_arrow Teta_arrow_map_law).

 End teta_tau_def.

Lemma Teta_tau_NT_law : (NT_law (FunSET2_r F) (FunSET2_l G) Teta_tau).
Proof.
Unfold nt_law Teta_tau Teta_arrow; Simpl.
Unfold Ext; Simpl.
Intros d1xc1 d2xc2 fxg h.
Unfold FunSET2_r_mor1 FunSET2_l_mor1.
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
  y:=(ApNT (A_eta ad) (Ob_l d2xc2)) o 
     ((FMor G ((FMor F (Hom_l fxg)) o h)) o 
        (FMor G (Hom_r fxg))).
Apply (Prf_congl D).
Unfold fMor.
Apply (Prf_Fcomp_law C D G (FOb F (Ob_l d2xc2)) (Ob_r d1xc1) (Ob_r d2xc2)).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
  y:=((ApNT (A_eta ad) (Ob_l d2xc2)) o 
     (FMor G ((FMor F (Hom_l fxg)) o h))) o 
        (FMor G (Hom_r fxg)).
Unfold Comp; Apply (Prf_ass D).
Apply (Prf_congr D).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))) with
 y:= (ApNT (A_eta ad) (Ob_l d2xc2)) o 
     ((FMor G (FMor F (Hom_l fxg))) o (FMor G h)).
Apply (Prf_congl D).
Unfold fMor.
Apply (Prf_Fcomp_law ? ? G (FOb F (Ob_l d2xc2)) (FOb F (Ob_l d1xc1))
                     (Ob_r d1xc1)).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))) with
 y:=((ApNT (A_eta ad) (Ob_l d2xc2)) o 
     (FMor G (FMor F (Hom_l fxg)))) o (FMor G h).
Unfold Comp; Apply (Prf_ass D).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))) with
 y:=((Hom_l fxg) o (ApNT (A_eta ad) (Ob_l d1xc1))) o (FMor G h).
Apply (Prf_congr D).
Apply (Prf_sym (Hom (Ob_l d2xc2) (FOb G (FOb F (Ob_l d1xc1))))).
Apply (Prf_NT_law ? ? ? ? (A_eta ad) ? ? (Hom_l fxg)).
Apply (Prf_sym (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))).
Unfold Comp; Apply (Prf_ass D).
Qed.

Definition Teta := (Build_NT (FunSET2_r F) (FunSET2_l G) Teta_tau Teta_tau_NT_law).

(* Construisons maintenant Teta_1 *)

 Section teta_1_tau_def.

 Variable dxc : (POb (Dual D) C).

 Definition Teta_1_arrow := [g:|(Hom (Ob_l dxc) (FOb G (Ob_r dxc)))|] 
                            (FMor F g) o (ApNT (A_eps ad) (Ob_r dxc)).

 Lemma Teta_1_arrow_Map_law : (Map_law Teta_1_arrow).
 Proof.
 Unfold map_law Teta_1_arrow.
 Intros f g H.
 Apply (Prf_congr C).
 Unfold fMor.
 Apply (Prf_FMap_law ? ? F (Ob_l dxc) (FOb G (Ob_r dxc))).
 Assumption.
 Qed.

 Definition Teta_1_tau := (Build_Map Teta_1_arrow Teta_1_arrow_Map_law).

 End teta_1_tau_def.

Lemma Teta_1_tau_NT_law : (NT_law (FunSET2_l G) (FunSET2_r F) Teta_1_tau).
Proof.
Unfold nt_law; Simpl.
Unfold Ext; Simpl.
Intros d1xc1 d2xc2 fxg h.
Unfold Teta_1_arrow FunSET2_r_mor1 FunSET2_l_mor1.
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2)))  with
  y:=((FMor F ((Hom_l fxg) o h)) o (FMor F (FMor G (Hom_r fxg))))
     o (ApNT (A_eps ad) (Ob_r d2xc2)).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F). 
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2)))  with
  y:=(FMor F ((Hom_l fxg) o h)) o ((FMor F (FMor G (Hom_r fxg)))
     o (ApNT (A_eps ad) (Ob_r d2xc2))).
Apply (Prf_sym (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2)))  with
  y:=((FMor F (Hom_l fxg)) o (FMor F h)) o 
     ((FMor F (FMor G (Hom_r fxg))) o (ApNT (A_eps ad) (Ob_r d2xc2))).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F). 
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2)))  with
  y:=(FMor F (Hom_l fxg)) o ((FMor F h) o 
     ((FMor F (FMor G (Hom_r fxg))) o (ApNT (A_eps ad) (Ob_r d2xc2)))).
Apply (Prf_sym (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2)))  with
  y:=(FMor F (Hom_l fxg)) o 
     (((FMor F h) o (ApNT (A_eps ad) (Ob_r d1xc1))) o (Hom_r fxg)).
Apply (Prf_congl C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d1xc1)) (Ob_r d2xc2))) with
  y:=(FMor F h) o ((ApNT (A_eps ad) (Ob_r d1xc1)) o (Hom_r fxg)).
Apply (Prf_congl C).
Apply (Prf_NT_law ? ? ? ? (A_eps ad) ? ? (Hom_r fxg)).
Unfold Comp; Apply (Prf_ass C).
Unfold Comp; Apply (Prf_ass C).
Qed.

Definition Teta_1 := (Build_NT (FunSET2_l G) (FunSET2_r F) Teta_1_tau Teta_1_tau_NT_law).

(* dem. que Teta et Teta_1 sont iso *)

(* a` prouver: eta(d) o G(Ff o eps(c)) = f *)
(* f o eta(Gc) o G(eps(c)) =f (car eta NT) *)
(* f o Id(Gc) = f                 (car C1)     *)

 Section teta_iso.

 Variable dxc : (POb (Dual D) C).

 Lemma Teta_1_o_Teta : (Iso_law SET ? ? (Teta_tau dxc) (Teta_1_tau dxc)).
 Proof.
 Unfold Iso_law; Simpl; Unfold Ext; Simpl.
 Unfold Teta_arrow Id_fun Teta_1_arrow.
 Intros f.
 (* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with
   y:=(ApNT (A_eta ad) (Ob_l dxc)) o 
      ((FMor G (FMor F f)) o (FMor G (ApNT (A_eps ad) (Ob_r dxc)))).
 Apply (Prf_congl D).
 Unfold fMor.
 Apply (Prf_Fcomp_law ? ? G (FOb F (Ob_l dxc)) 
                      (FOb F (FOb G (Ob_r dxc))) (Ob_r dxc)).
 (* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with
   y:=((ApNT (A_eta ad) (Ob_l dxc)) o 
       (FMor G (FMor F f))) o (FMor G (ApNT (A_eps ad) (Ob_r dxc))).
 Unfold Comp; Apply (Prf_ass D).
 (* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with
   y:=(f o (ApNT (A_eta ad) (FOb G (Ob_r dxc)))) o 
      (FMor G (ApNT (A_eps ad) (Ob_r dxc))).
 Apply (Prf_congr D).
 Apply (Prf_sym (Hom (Ob_l dxc) (FOb G (FOb F (FOb G (Ob_r dxc)))))).
 Apply (Prf_NT_law ? ? ? ? (A_eta ad) ? ? f).
 (* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with
   y:=f o ((ApNT (A_eta ad) (FOb G (Ob_r dxc))) o 
      (FMor G (ApNT (A_eps ad) (Ob_r dxc)))).
 Apply (Prf_sym (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))).
 Unfold Comp; Apply (Prf_ass D).
 (* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with
   y:=f o (Id (FOb G (Ob_r dxc))).
 Apply (Prf_congl D).
 Apply (Prf_Adj1_law1 ? ? ? ? ad (Ob_r dxc)).
 Apply (Prf_sym (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))).
 Unfold Comp; Apply (Prf_idr D).
 Qed.

 Lemma Teta_o_Teta_1 : (Iso_law SET ? ? (Teta_1_tau dxc) (Teta_tau dxc)). 
 Proof.
 Unfold Iso_law; Simpl; Unfold Ext; Simpl.
 Unfold Teta_1_arrow Teta_arrow Id_fun.
 Intros f.
 (* *) Apply (Prf_trans (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))) with
   y:=((FMor F (ApNT (A_eta ad) (Ob_l dxc))) o (FMor F(FMor G f))) o 
      (ApNT (A_eps ad) (Ob_r dxc)).
 Apply (Prf_congr C).
 Unfold fMor; Apply (Prf_Fcomp_law ? ? F).
 (* *) Apply (Prf_trans (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))) with
   y:=(FMor F (ApNT (A_eta ad) (Ob_l dxc))) o ((FMor F(FMor G f)) o 
      (ApNT (A_eps ad) (Ob_r dxc))).
 Apply (Prf_sym (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))).
 Unfold Comp; Apply (Prf_ass C).
 (* *) Apply (Prf_trans (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))) with
   y:=(FMor F (ApNT (A_eta ad) (Ob_l dxc))) o 
      ((ApNT (A_eps ad) (FOb F (Ob_l dxc))) o f).
 Apply (Prf_congl C).
 Apply (Prf_NT_law ? ? ? ? (A_eps ad) ? ? f).
 (* *) Apply (Prf_trans (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))) with
   y:=((FMor F (ApNT (A_eta ad) (Ob_l dxc))) o 
      (ApNT (A_eps ad) (FOb F (Ob_l dxc)))) o f.
 Unfold Comp; Apply (Prf_ass C).
 (* *) Apply (Prf_trans (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))) with
   y:=(Id (FOb F (Ob_l dxc))) o f.
 Apply (Prf_congr C).
 Apply (Prf_Adj1_law2 ? ? ? ? ad (Ob_l dxc)).
 Unfold Comp; Apply (Prf_idl C).
 Qed.

 Definition Teta_iso := (Build_isIso SET ? ? (ApNT Teta dxc) (ApNT Teta_1 dxc) 
                                     Teta_1_o_Teta Teta_o_Teta_1).

 End teta_iso.

Definition Adj1_to_Adj := (Build_Adj F G Teta (NT_isIso ? ? ? ? Teta Teta_iso)).

End adj1_to_adj.
 


(* Les deux defs sont e'quivalentes *)


Section adj_equiv.

Variable C,D : Category.
Variable F   : (Functor D C).
Variable G   : (Functor C D).

Definition Eq_Adj := [ad,ad':(Adj F G)](Aphi ad) =%NT (Aphi ad').

Definition Eq_Adj1 := [ad,ad':(Adj1 F G)]
                      ((A_eta ad) =%NT (A_eta ad')) /\ ((A_eps ad) =%NT (A_eps ad')).

End adj_equiv.



(* $Id: Adjunction1.v,v 1.2 1996/02/12 15:22:48 cpaulin Exp $ *)
