(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Hom (Bi-)Functors (used in Adjunctions)        *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)


Require Export SET.
Require Export Dual.
Require Export PROD.
Require Export Functor.

(* C(-,G-) et D(F-,-) : D^o x C -> SET *)


(* C(F-,-) *)

Section FunSet2_r.

Variable C, D : Category.
Variable F    : (Functor D C).

Definition FunSET2_r_ob := [dxc:(Ob (PROD (Dual D) C))](Hom (FOb F (Ob_l dxc)) (Ob_r dxc)).

 Section funset2_r_map_def.

 Variable d1xc1, d2xc2 : (Ob (PROD (Dual D) C)).

  Section funset2_r_mor_def.

  Variable foxg : (Pmor d1xc1 d2xc2).

  Definition FunSET2_r_mor1 := [h:|(FunSET2_r_ob d1xc1)|]
                               ((FMor F (Hom_l foxg)) o h) o (Hom_r foxg).

  Lemma FunSET2_r_map_law1 : (Map_law FunSET2_r_mor1).
  Proof.
  Unfold map_law FunSET2_r_mor1; Simpl.
  Intros h1 h2 H.
  Apply (Prf_congr C); Apply (Prf_congl C); Assumption.
  Qed.

  Definition FunSET2_r_mor := (Build_Map FunSET2_r_mor1 FunSET2_r_map_law1).

  End funset2_r_mor_def.
  
 Lemma FunSET2_r_map_law : (map_law (hom (PROD (Dual D) C) d1xc1 d2xc2) 
                                    (hom SET (FunSET2_r_ob d1xc1) (FunSET2_r_ob d2xc2))
                                    FunSET2_r_mor).
 Proof.
 Unfold map_law FunSET2_r_mor.
 Intros f1xg1 f2xg2; Elim f1xg1; Intros f1 g1. 
 Elim f2xg2; Intros f2 g2; Simpl.
 Unfold Ext; Simpl.
 Unfold Equal_Pmor; Simpl.
 Unfold FunSET2_r_mor1; Simpl.
 Unfold FunSET2_r_ob.
 Intro H; Elim H; Intros H1 H2 h.
 Apply (Prf_cong C).
 Apply (Prf_congr C); Trivial.
 Unfold fMor; Apply (Prf_FMap_law ? ? F (Ob_l d2xc2) (Ob_l d1xc1)).
 Assumption.
 Assumption.
 Qed.

 Definition FunSET2_r_map := (build_Map (hom (PROD (Dual D) C) d1xc1 d2xc2) 
                                        (hom SET (FunSET2_r_ob d1xc1) (FunSET2_r_ob d2xc2)) 
                                        FunSET2_r_mor FunSET2_r_map_law).
 End funset2_r_map_def.

Lemma Fun2_r_comp_law : (fcomp_law (PROD (Dual D) C) SET FunSET2_r_ob FunSET2_r_map).
Proof.
Unfold fcomp_law FunSET2_r_map; Simpl.
Unfold FunSET2_r_mor Ext; Simpl.
Unfold FunSET2_r_mor1 FunSET2_r_ob; Simpl.
Intros d1xc1 d2xc2 d3xc3 f1xg1 f2xg2 h.
Elim f1xg1; Intros f1 g1.
Elim f2xg2; Intros f2 g2; Simpl.
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d3xc3)) (Ob_r d3xc3))) with
  y:=(((FMor F (f2 o f1)) o h) o g1) o g2.
Unfold Comp; Apply (Prf_ass C).
Apply (Prf_congr C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d3xc3)) (Ob_r d2xc2))) with
  y:=((FMor F f2) o ((FMor F f1) o h)) o g1.
Apply (Prf_congr C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d3xc3)) (Ob_r d1xc1))) with
  y:=((FMor F f2) o (FMor F f1)) o h.
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F).
Apply (Prf_sym (Hom (FOb F (Ob_l d3xc3)) (Ob_r d1xc1))).
Unfold Comp; Apply (Prf_ass C).
Apply (Prf_sym (Hom (FOb F (Ob_l d3xc3)) (Ob_r d2xc2))).
Unfold Comp; Apply (Prf_ass C).
Qed.

Lemma Fun2_r_id_law : (fid_law (PROD (Dual D) C) SET FunSET2_r_ob FunSET2_r_map).
Proof.
Unfold fid_law FunSET2_r_map FunSET2_r_mor; Simpl.
Unfold Ext Id_SET; Simpl.
Unfold FunSET2_r_mor1; Simpl.
Unfold FunSET2_r_ob; Intros dxc f.
Unfold Id_fun.
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))) with
  y:=(FMor F (Id (Ob_l dxc))) o f.
Apply (Prf_sym (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))).
Unfold Comp; Apply (Prf_idr C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))) with
  y:=(Id (FOb F (Ob_l dxc))) o f.
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fid_law ? ? F).
Unfold Comp; Apply (Prf_idl C).
Qed.

Definition funSET2_r := (Build_Functor (PROD (Dual D) C) SET FunSET2_r_ob FunSET2_r_map 
                                       Fun2_r_comp_law Fun2_r_id_law).

End FunSet2_r.

Syntactic Definition FunSET2_r := funSET2_r | 2. 
Syntax constr funSET2_r <<(funSET2_r $1 $2 $F)>> 10
     [<hov 0> "FunSET2_r " <$F:"Term":L> ].



Section FunSet2_l.

Variable C, D : Category.
Variable G    : (Functor C D).

(* D(-,G-) *)

Definition FunSET2_l_ob := [dxc:(Ob (PROD (Dual D) C))](Hom (Ob_l dxc) (FOb G (Ob_r dxc))).

 Section funset2_l_map_def.

 Variable d1xc1, d2xc2 : (Ob (PROD (Dual D) C)).

  Section funset2_l_mor_def.

  Variable foxg : (Pmor d1xc1 d2xc2).

  Definition FunSET2_l_mor1 := [h:|(FunSET2_l_ob d1xc1)|]
                               ((Hom_l foxg) o h) o (FMor G (Hom_r foxg)).

  Lemma FunSET2_l_map_law1 : (Map_law FunSET2_l_mor1).
  Proof.
  Unfold map_law FunSET2_l_mor1; Simpl.
  Intros h1 h2 H.
  Apply (Prf_congr D); Apply (Prf_congl D); Assumption.
  Qed.

  Definition FunSET2_l_mor := (Build_Map FunSET2_l_mor1 FunSET2_l_map_law1).

  End funset2_l_mor_def.

 Lemma FunSET2_l_map_law : (map_law (hom (PROD (Dual D) C) d1xc1 d2xc2) 
                                   (hom SET (FunSET2_l_ob d1xc1) (FunSET2_l_ob d2xc2))
                                   FunSET2_l_mor).
 Proof.
 Unfold map_law FunSET2_l_mor.
 Intros f1xg1 f2xg2; Elim f1xg1; Intros f1 g1. 
 Elim f2xg2; Intros f2 g2; Simpl.
 Unfold Ext; Simpl.
 Unfold Equal_Pmor; Simpl.
 Unfold FunSET2_l_mor1; Simpl.
 Unfold FunSET2_l_ob.
 Intro H; Elim H; Intros H1 H2 h.
 Apply (Prf_cong D).
 Apply (Prf_congr D); Trivial.
 Unfold fMor; Apply (Prf_FMap_law ? ? G (Ob_r d1xc1) (Ob_r d2xc2)).
 Assumption.
 Qed.

 Definition FunSET2_l_map := (build_Map (hom (PROD (Dual D) C) d1xc1 d2xc2) 
                                        (hom SET (FunSET2_l_ob d1xc1) (FunSET2_l_ob d2xc2)) 
                                        FunSET2_l_mor FunSET2_l_map_law).

 End funset2_l_map_def.
 
Lemma Fun2_l_comp_law : (fcomp_law (PROD (Dual D) C) SET FunSET2_l_ob FunSET2_l_map).
Proof.
Unfold fcomp_law FunSET2_l_map; Simpl.
Unfold FunSET2_l_mor Ext; Simpl.
Unfold FunSET2_l_mor1 FunSET2_l_ob; Simpl.
Intros d1xc1 d2xc2 d3xc3 f1xg1 f2xg2 h.
Elim f1xg1; Intros f1 g1.
Elim f2xg2; Intros f2 g2; Simpl.
(* *) Apply (Prf_trans (Hom (Ob_l d3xc3) (FOb G (Ob_r d3xc3)))) with
  y:=((f2 o f1) o h) o ((FMor G g1) o (FMor G g2)).
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
(* *) Apply (Prf_trans (Hom (Ob_l d3xc3) (FOb G (Ob_r d3xc3)))) with
  y:=(((f2 o f1) o h) o (FMor G g1)) o (FMor G g2).
Unfold Comp; Apply (Prf_ass D).
Apply (Prf_congr D).
(* *) Apply (Prf_trans (Hom (Ob_l d3xc3) (FOb G (Ob_r d2xc2)))) with
  y:=(f2 o (f1 o h)) o (FMor G g1).
Apply (Prf_congr D).
Apply (Prf_sym (Hom (Ob_l d3xc3) (FOb G (Ob_r d1xc1)))); Unfold Comp; Apply (Prf_ass D).
Apply (Prf_sym (Hom (Ob_l d3xc3) (FOb G (Ob_r d2xc2)))); Unfold Comp; Apply (Prf_ass D).
Qed.

Lemma Fun2_l_id_law : (fid_law (PROD (Dual D) C) SET FunSET2_l_ob FunSET2_l_map).
Proof.
Unfold fid_law FunSET2_l_map FunSET2_l_mor; Simpl.
Unfold Ext Id_SET; Simpl.
Unfold FunSET2_l_mor1; Simpl.
Unfold FunSET2_l_ob; Intros dxc f.
Unfold Id_fun.
(* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with 
  y:=(Id (Ob_l dxc)) o (f o (FMor G (Id (Ob_r dxc)))).
Apply (Prf_sym (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))).
Unfold Comp; Apply (Prf_ass D).
(* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with 
  y:=f o (FMor G (Id (Ob_r dxc))).
Unfold Comp; Apply (Prf_idl D).
(* *) Apply (Prf_trans (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))) with
  y:=f o (Id (FOb G (Ob_r dxc))).
Apply (Prf_congl D); Unfold fMor; Apply (Prf_Fid_law ? ? G).
Apply (Prf_sym (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))).
Unfold Comp; Apply (Prf_idr D).
Qed.

Definition funSET2_l := (Build_Functor (PROD (Dual D) C) SET FunSET2_l_ob FunSET2_l_map 
                                       Fun2_l_comp_law Fun2_l_id_law).

End FunSet2_l.

Syntactic Definition FunSET2_l := funSET2_l | 2. 
Syntax constr funSET2_l <<(funSET2_l $1 $2 $G)>> 10
     [<hov 0> "FunSET2_l " <$G:"Term":L> ].



(* tre`s utile pour alleger les e'critures *)

Definition build_POb1 := [D,C:Category][d:(Ob D)][c:(Ob C)]
                         (build_POb (Dual D) C d c).

Syntactic Definition Build_POb1 := build_POb1 | 2.
Syntax constr build_POb1 <<(build_POb1 $1 $2 $d $c)>> 10
       [<hov 0> "Build_POb1 " <$d:"term":L> " " <$c:"term":L>].



(* $Id: HomFunctor2.v,v 1.2 1996/02/12 15:22:51 cpaulin Exp $ *)
