(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*	      A Functor which is a Right Adjoint is Continuous		     *)
(*                      (part of Freyd's Theorem)             		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Adj_UA.
Require Export Pres_Limits.

(* *)

Section ladj_pres.

Variable C,D : Category.
Variable G   : (Functor C D).
Variable la  : (LeftAdj G).
Variable J   : Category.
Variable H   : (Functor J C).
Variable l   : (Limit H).

(* pour tout d : D et tau : Delta(d) -> T o G,
   sigma(i) = phi_1<d,Ti>(tau(i)) *)

 Section lp_diese.

 Variable d   :(Ob D).
 Variable tau : (Cone d (H o%F G)).

 Definition Lp_sigma_tau := [i:(Ob J)] (ApAphi_inv (Adj_l la) (ApNT tau i)).

 (* sigma est en fait un cone(T,Fd) ou` F est l'adjoint de G *)

 Lemma Lp_sigma_tau_cone_law : (Cone_law (FOb (Adjoint la) d) H Lp_sigma_tau).
 Proof.
 Unfold cone_law Lp_sigma_tau.
 Intros i j g.
 (* *) Apply (Prf_trans (Hom (FOb (Adjoint la) d) (FOb H j))) with
   y:=(ApAphi_inv (Adj_l la) ((ApNT tau i) o (FMor G (FMor H g)))).
 Unfold apAphi_inv.
 Apply (Pres ? ? (ApNT (Aphi_inv (Adj_l la)) (Build_POb1 d (FOb H j)))).
 Apply (Eq_cone ? ? ? ? tau ? ? g).
 Apply (Adj_eq5 ? ? ? ? (Adj_l la) ? ? ? (FMor H g) (ApNT tau i)).
 Qed.

 Definition Lp_sigma := (Build_Cone (FOb (Adjoint la) d) H Lp_sigma_tau
                                    Lp_sigma_tau_cone_law).

 (* construction de tau# *)
 (* tau# = phi<d,limT>(sigma#) *)

 Definition Lp_diese := (ApAphi (Adj_l la) (CoUA_diese l ? Lp_sigma)).
 
 End lp_diese.
 
Lemma Lp_coUAlaw1 : (Limit_law1 (H o%F G) ? ((Limiting_cone l) o%C G) Lp_diese).
Proof.
Unfold limit_law1 coUA_law1 coUA_eq 1 fMor; Simpl.
Unfold Equal_NT; Intros d tau i.
Unfold Delta_mor; Simpl.
Unfold Comp_tau Comp_cone; Simpl.
Unfold Delta_mor_tau Comp_cone_tau Lp_diese.
(* *) Apply (Prf_trans (Hom d (FOb G (FOb H i)))) with
  y:=(ApAphi (Adj_l la) 
         ((CoUA_diese l ? (Lp_sigma d tau)) o (ApNT (Limiting_cone l) i))).
Apply (Prf_sym (Hom d (FOb G (FOb H i)))).
Apply (Adj_eq3 ? ? ? ? (Adj_l la) ? ? ? 
            (ApNT (Limiting_cone l) i) (CoUA_diese l ? (Lp_sigma d tau))).
(* *) Apply (Prf_trans (Hom d (FOb G (FOb H i)))) with
  y:=(ApAphi (Adj_l la) (ApAphi_inv (Adj_l la) (ApNT tau i))).
Unfold apAphi; Apply (Pres ? ? (ApNT (Aphi (Adj_l la)) 
                                (Build_POb1 d (FOb H i)))).
Apply (Prf_CoUA_law1 ? ? ? ? l ? (Lp_sigma d tau) i).
Apply (Check_Ta_idl ? ? ? ? (Aphi (Adj_l la)) (Aphi_iso (Adj_l la)) 
             (Build_POb1 d (FOb H i)) (ApNT tau i)).
Qed.

Lemma Lp_coUAlaw2 : (Limit_law2 (H o%F G) ? ((Limiting_cone l) o%C G) Lp_diese).
Proof.
Unfold limit_law2 coUA_law2 coUA_eq 1 fMor; Simpl.
Unfold Equal_NT; Intros d tau t.
Unfold Delta_mor; Simpl.
Unfold Comp_tau Comp_cone; Simpl.
Unfold Delta_mor_tau Comp_cone_tau Lp_diese.
Intro H'.
(* *) Apply (Prf_trans (Hom d (FOb G (Lim l)))) with
  y:=(ApAphi (Adj_l la) (ApAphi_inv (Adj_l la) t)).
Apply (Prf_sym (Hom d (FOb G (Lim l)))).
Apply (Check_Ta_idl ? ? ? ? (Aphi (Adj_l la)) (Aphi_iso (Adj_l la)) 
             (Build_POb1 d (Lim l)) t).
Unfold apAphi; Apply (Pres ? ? (ApNT (Aphi (Adj_l la)) 
                                  (Build_POb1 d (Lim l)))).
Apply (Prf_CoUA_law2 ? ? ? ? l ? (Lp_sigma d tau) 
         (ApAphi_inv (Adj_l la) t)).
Unfold coUA_eq fMor; Simpl.
Unfold Delta_mor Equal_NT; Simpl.
Unfold Comp_tau; Simpl.
Intro i.
Unfold Delta_mor_tau Lp_sigma_tau.
(* *) Apply (Prf_trans (Hom (FOb (Adjoint la) d) (FOb H i))) with
  y:=(ApAphi_inv (Adj_l  la) 
         (t o (FMor G (ApNT (Limiting_cone l) i)))).
Apply (Prf_sym (Hom (FOb (Adjoint la) d) (FOb H i))).
Apply (Adj_eq5 ? ? ? ? (Adj_l  la) ? ? ? (ApNT (Limiting_cone l) i) t).
Unfold apAphi_inv; Apply (Pres ? ? 
    (ApNT (Aphi_inv (Adj_l  la)) (Build_POb1 d (FOb H i)))).
Apply (H' i).
Qed.

Lemma Ladj_Pres1 : (Preserves_1limit G l).
Proof.
Apply (Build_IsLimit (H o%F G) (FOb G (Lim l)) ((Limiting_cone l) o%C G)
                     Lp_diese Lp_coUAlaw1 Lp_coUAlaw2).
Defined.

End ladj_pres.

(* Si G admet un adjoint, alors il Preserve toutes les Limites *)

Lemma Ladj_continuous :  (C,D:Category)(G:(Functor C D))
                         (LeftAdj G) -> (Continuous G).
Proof.
Unfold continuous preserves_limits; Intros C D G la J H l.
Apply (Ladj_Pres1 ? ? G la ? ? l).
Defined.







(* $Id: Limit_Adj.v,v 1.2 1996/02/12 15:22:55 cpaulin Exp $ *)
