(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	            Existence of Left Adjoint                		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Adj_UA.

(* Maintenant a` partir d'un UA unit (pour tout d:D), *)
(* construire une adj *)

Section ua_to_ladj.

Variable C, D : Category.
Variable G    : (Functor C D).

Hypothesis UA_of : (d:(Ob D))(UA d G).

Definition AdjointUA_ob := [d:(Ob D)](UA_ob (UA_of d)).

 Section adjoint_ua_map_def.

 Variable d, d' : (Ob D).

 Definition AdjointUA_mor := [f:|(Hom d d')|]
            (UA_diese (UA_of d) (AdjointUA_ob d') (f o (UA_mor (UA_of d')))).

 Lemma AdjointUA_map_law : (Map_law AdjointUA_mor).
 Proof.
 Unfold map_law AdjointUA_mor; Intros f g H.
 Apply (Diese_map ? ? ? ? (UA_of d) ? (f o (UA_mor (UA_of d')))
                  (g o (UA_mor (UA_of d')))).
 Apply (Prf_congr D); Assumption.
 Qed.

 Definition AdjointUA_map := (Build_Map AdjointUA_mor AdjointUA_map_law).

 End adjoint_ua_map_def.
 
(* unicite' de # pour <cd,ud> : *)
(* qq a':C. qq f':d->Ga'. qq g:cd->a'. *)
(* ud o Gg = f' => g=f'# *)

(* appliquer unicite' pour (Id(d) o ud) et Id(cd) *)

Lemma AdjointUA_id_law : (Fid_law AdjointUA_ob AdjointUA_map).
Proof.
Unfold fid_law; Simpl.
Unfold AdjointUA_mor AdjointUA_ob.
Intro d.
Apply (Prf_sym (Hom (UA_ob (UA_of d)) (UA_ob (UA_of d)))).
Apply (Prf_UA_law2 ? ? ? ? (UA_of d) ? ((Id d) o (UA_mor (UA_of d)))
                      (Id (UA_ob (UA_of d)))).
Unfold ua_eq.
(* *) Apply (Prf_trans (Hom d (FOb G (UA_ob (UA_of d))))) with
  y:=(UA_mor (UA_of d)) o (Id (FOb G (UA_ob (UA_of d)))).
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fid_law ? ? G).
(* *) Apply (Prf_trans (Hom d (FOb G (UA_ob (UA_of d))))) with
  y:=(UA_mor (UA_of d)).
(* *) Apply (Prf_sym (Hom d (FOb G (UA_ob (UA_of d))))).
Unfold Comp; Apply (Prf_idr D).
(* *) Apply (Prf_sym (Hom d (FOb G (UA_ob (UA_of d))))).
Unfold Comp; Apply (Prf_idl D).
Qed.

(* revient a` prouver: ((f o g) o ud3)#1 = (f o ud2)#1 o (g o ud3)#2 *)
(* unicite' ud1 avec ((f o g) o ud3) et (f o ud2)#1 o (g o ud3)#2 *)
(* revient a` dem: ud1 o G((f o ud2)#1 o (g o ud3)#2) = (f o g) o ud3 *)
(*                            (1)   *)
(* (1) = (ud1 o G((f o ud2)#1)) o G((g o ud3)#2) (Fcomplaw) *) 
(*     = (f o ud2) o G((g o ud3)#2)              (loi #1) *)
(*     = f o (g o ud3)                           (loi #2) *) 
(* cqfd *)

Lemma AdjointUA_comp_law : (Fcomp_law AdjointUA_ob AdjointUA_map).
Proof.
Unfold fcomp_law; Simpl.
Unfold AdjointUA_mor; Intros d1 d2 d3 f g.
Apply (Prf_sym (Hom (AdjointUA_ob d1) (AdjointUA_ob d3))).
Apply (Prf_UA_law2 ? ? ? ? (UA_of d1) 
           ? ((f o g) o (UA_mor (UA_of d3)))
           ((UA_diese (UA_of d1) ? (f o (UA_mor (UA_of d2))))
             o (UA_diese (UA_of d2) ? (g o (UA_mor (UA_of d3)))))).
Unfold ua_eq.
(* *) Apply (Prf_trans (Hom d1 (FOb G (UA_ob (UA_of d3))))) with
  y:=(UA_mor (UA_of d1)) o 
     ((FMor G (UA_diese (UA_of d1) ? (f o (UA_mor (UA_of d2))))) o
      (FMor G (UA_diese (UA_of d2) ? (g o (UA_mor (UA_of d3)))))). 
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
(* *) Apply (Prf_trans (Hom d1 (FOb G (UA_ob (UA_of d3))))) with
  y:=((UA_mor (UA_of d1)) o 
     (FMor G (UA_diese (UA_of d1) ? (f o (UA_mor (UA_of d2)))))) o
      (FMor G (UA_diese (UA_of d2) ? (g o (UA_mor (UA_of d3))))). 
Unfold Comp; Apply (Prf_ass D).
(* *) Apply (Prf_trans (Hom d1 (FOb G (UA_ob (UA_of d3))))) with
  y:=(f o (UA_mor (UA_of d2))) o 
     (FMor G (UA_diese (UA_of d2) ? (g o (UA_mor (UA_of d3))))).
Apply (Prf_congr D).
Apply (Prf_UA_law1 ? ? ? ? (UA_of d1) ?
                     (f o (UA_mor (UA_of d2)))).
(* *) Apply (Prf_trans (Hom d1 (FOb G (UA_ob (UA_of d3))))) with
  y:=f o ((UA_mor (UA_of d2)) o 
          (FMor G (UA_diese (UA_of d2) ? (g o (UA_mor (UA_of d3)))))).
Apply (Prf_sym (Hom d1 (FOb G (UA_ob (UA_of d3))))).
Unfold Comp; Apply (Prf_ass D).
(* *) Apply (Prf_trans (Hom d1 (FOb G (UA_ob (UA_of d3))))) with
  y:=f o (g o (UA_mor (UA_of d3))).
Apply (Prf_congl D).
Apply (Prf_UA_law1 ? ? ? ? (UA_of d2) ?
                     (g o (UA_mor (UA_of d3)))).
Unfold Comp; Apply (Prf_ass D).
Qed.

Definition AdjointUA := (Build_Functor D C AdjointUA_ob AdjointUA_map 
                                       AdjointUA_comp_law AdjointUA_id_law).

(* definition de phi:C(F-,-) -> D(-,G-) *)

(* construction de phi *)
(* soit f: Fd ->c, phi(f)=ud o Gf : d -> G(Fd) *)

 Section phi_ua_tau_def.

 Variable dxc : (POb (Dual D) C).

 Definition PhiUA_arrow := [f:|(Hom (UA_ob (UA_of (Ob_l dxc))) (Ob_r dxc))|] 
                           (UA_mor (UA_of (Ob_l dxc))) o (FMor G f).

 Lemma PhiUA_arrow_map_law : (Map_law PhiUA_arrow).
 Proof.
 Unfold map_law PhiUA_arrow.
 Intros f g H.
 Apply (Prf_congl D).
 Unfold fMor.
 Apply (Prf_FMap_law ? ? G (UA_ob (UA_of (Ob_l dxc))) (Ob_r dxc)).
 Assumption.
 Qed.

 Definition PhiUA_tau := (Build_Map PhiUA_arrow PhiUA_arrow_map_law).

 End phi_ua_tau_def.

(* on a a` prouver: ud2 o G(F(fo) o h o g) = fo o ud1 o Gh o Gg *)
(* => ud2 o G(Ffo) = fo o ud1 *)
(* or Ffo = (fo o ud1)#2 *)
(* donc: ud2 o G((fo o ud1)#2) = fo o ud *)
(* vrai car c'est #2 law *)

Lemma PhiUA_tau_nt_law : (NT_law (FunSET2_r AdjointUA) (FunSET2_l G) PhiUA_tau).
Proof.
Unfold nt_law PhiUA_tau PhiUA_arrow; Simpl.
Unfold Ext; Simpl.
Intros d1xc1 d2xc2 fxg h.
Unfold FunSET2_r_mor1 FunSET2_l_mor1.
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
  y:=(UA_mor (UA_of (Ob_l d2xc2))) o 
     ((FMor G ((FMor AdjointUA (Hom_l fxg)) o h)) o 
        (FMor G (Hom_r fxg))).
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
  y:=((UA_mor (UA_of (Ob_l d2xc2))) o 
     (FMor G ((FMor AdjointUA (Hom_l fxg)) o h))) o 
        (FMor G (Hom_r fxg)).
Unfold Comp; Apply (Prf_ass D).
Apply (Prf_congr D).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))) with
 y:=(UA_mor (UA_of (Ob_l d2xc2))) o 
     ((FMor G (FMor AdjointUA (Hom_l fxg))) o (FMor G h)).
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))) with
 y:=((UA_mor (UA_of (Ob_l d2xc2))) o 
     (FMor G (FMor AdjointUA (Hom_l fxg)))) o (FMor G h).
Unfold Comp; Apply (Prf_ass D).
(* *) Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))) with
 y:=((Hom_l fxg) o (UA_mor (UA_of (Ob_l d1xc1)))) o (FMor G h).
Apply (Prf_congr D).
Unfold AdjointUA AdjointUA_map AdjointUA_mor. 
Unfold 1 fMor; Simpl.
Apply (Prf_UA_law1 ? ? ? ? (UA_of (Ob_l d2xc2)) ? 
            ((Hom_l fxg) o (UA_mor (UA_of (Ob_l d1xc1))))).
Apply (Prf_sym (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))).
Unfold Comp; Apply (Prf_ass D).
Qed.

Definition PhiUA := (Build_NT (FunSET2_r AdjointUA) (FunSET2_l G)
                             PhiUA_tau PhiUA_tau_nt_law).

(* de'finissons PhiUA_1 *)

 Section phi_ua_1_tau_def.

 Variable dxc : (POb (Dual D) C).

 Definition PhiUA_1_arrow := [f:|(Hom (Ob_l dxc) (FOb G (Ob_r dxc)))|] 
                             (UA_diese (UA_of (Ob_l dxc)) ? f).

 Lemma PhiUA_1_arrow_map_law : (map_law (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))
                                        (Hom (UA_ob (UA_of (Ob_l dxc))) (Ob_r dxc))  
                                        PhiUA_1_arrow).
 Proof.
 Unfold map_law PhiUA_1_arrow.
 Intros f g H.
 Apply (Diese_map ? ? ? ? (UA_of (Ob_l dxc)) ? f g).
 Assumption.
 Qed.

 Definition PhiUA_1_tau := (Build_Map PhiUA_1_arrow PhiUA_1_arrow_map_law).

 End phi_ua_1_tau_def.

(* On doit prouver: ((fo o h) o Gg)#2 = ((fo o ud1)#2 o h#1) o g *)
(* unicite' de #2 avec (fo o h) o Gg et (fo o ud1)#2 o h#1 o g *)
(* reste a` prouver: ud2 o G((fo o ud1)#2 o h#1 o g) = (fo o h) o Gg *)
(*                        (1)         *)
(* (1) = ud2 o G((fo o ud1)#2) o G(h#1) o Gg (Fcomp) *)
(*     = fo o ud1 o G(h#1) o Gg              (loi #2) *)
(*     = fo o h o Gg                         (loi #1) *)

Lemma PhiUA_1_tau_nt_law : (NT_law (FunSET2_l G) (FunSET2_r AdjointUA) 
                                   PhiUA_1_tau).
Proof.
Unfold nt_law; Simpl.
Unfold Ext; Simpl.
Intros d1xc1 d2xc2 fxg h.
Unfold PhiUA_1_arrow FunSET2_r_mor1 AdjointUA FunSET2_l_mor1 AdjointUA_map AdjointUA_mor.
Unfold 2 fMor; Simpl.
Apply (Prf_sym (Hom (AdjointUA_ob (Ob_l d2xc2)) (Ob_r d2xc2))).
Apply (Prf_UA_law2 ? ? ? ? (UA_of (Ob_l d2xc2)) 
           ? (((Hom_l fxg) o h) o (FMor G (Hom_r fxg)))
           (((UA_diese (UA_of (Ob_l d2xc2)) ? 
                   ((Hom_l fxg) o (UA_mor (UA_of (Ob_l d1xc1)))))
            o (UA_diese (UA_of (Ob_l d1xc1)) ? h)) o (Hom_r fxg))).
Unfold ua_eq.
Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
y:=(UA_mor (UA_of (Ob_l d2xc2))) o 
    ((FMor G ((UA_diese (UA_of (Ob_l d2xc2)) ?
                 ((Hom_l fxg))o(UA_mor (UA_of (Ob_l d1xc1))))
             o (UA_diese (UA_of (Ob_l d1xc1)) ? h))) 
     o (FMor G (Hom_r fxg))).
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
y:=((UA_mor (UA_of (Ob_l d2xc2))) o 
    (FMor G ((UA_diese (UA_of (Ob_l d2xc2)) ?
                 ((Hom_l fxg))o(UA_mor (UA_of (Ob_l d1xc1))))
             o (UA_diese (UA_of (Ob_l d1xc1)) ? h))))
     o (FMor G (Hom_r fxg)).
Unfold Comp; Apply (Prf_ass D).
Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
y:=((UA_mor (UA_of (Ob_l d2xc2))) o 
    ((FMor G (UA_diese (UA_of (Ob_l d2xc2)) ?
                 ((Hom_l fxg))o(UA_mor (UA_of (Ob_l d1xc1)))))
     o (FMor G (UA_diese (UA_of (Ob_l d1xc1)) ? h)))) o 
        (FMor G (Hom_r fxg)).
Apply (Prf_congr D).
Apply (Prf_congl D).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
y:=(((UA_mor (UA_of (Ob_l d2xc2))) o 
    (FMor G (UA_diese (UA_of (Ob_l d2xc2)) ?
                 ((Hom_l fxg))o(UA_mor (UA_of (Ob_l d1xc1))))))
     o (FMor G (UA_diese (UA_of (Ob_l d1xc1)) ? h))) o 
        (FMor G (Hom_r fxg)).
Apply (Prf_congr D).
Unfold Comp; Apply (Prf_ass D).
Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d2xc2)))) with
y:=(((Hom_l fxg) o (UA_mor (UA_of (Ob_l d1xc1))))
     o (FMor G (UA_diese (UA_of (Ob_l d1xc1)) ? h))) o 
        (FMor G (Hom_r fxg)).
Apply (Prf_congr D).
Apply (Prf_congr D).
Apply (Prf_UA_law1 ? ? ? ? (UA_of (Ob_l d2xc2)) ?
              ((Hom_l fxg) o (UA_mor (UA_of (Ob_l d1xc1))))).
Apply (Prf_congr D).
Apply (Prf_trans (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))) with
 y:=(Hom_l fxg) o ((UA_mor (UA_of (Ob_l d1xc1)))
     o (FMor G (UA_diese (UA_of (Ob_l d1xc1)) ? h))).
Apply (Prf_sym (Hom (Ob_l d2xc2) (FOb G (Ob_r d1xc1)))).
Unfold Comp; Apply (Prf_ass D).
Apply (Prf_congl D).
Apply (Prf_UA_law1 ? ? ? ? (UA_of (Ob_l d1xc1)) ? h).
Qed.

Definition PhiUA_1 := (Build_NT (FunSET2_l G) (FunSET2_r AdjointUA) 
                                PhiUA_1_tau PhiUA_1_tau_nt_law).


(* PhiUA et PhiUA_1 sont iso *)

 Section phi_ua_iso.

 Variable dxc : (POb (Dual D) C).

 Lemma PhiUA_1_o_PhiUA : (Iso_law SET ? ? (PhiUA_tau dxc) (PhiUA_1_tau dxc)).
 Proof.
 Unfold Iso_law; Simpl.
 Unfold Ext; Simpl.
 Unfold PhiUA_arrow Id_fun PhiUA_1_arrow.
 Intros f.
 Apply (Prf_UA_law1 ? ? ? ? (UA_of (Ob_l dxc)) ? f).
 Qed.

 Lemma PhiUA_o_PhiUA_1 : (Iso_law SET ? ? (PhiUA_1_tau dxc) (PhiUA_tau dxc)).
 Proof.
 Unfold Iso_law; Simpl.
 Unfold Ext; Simpl.
 Unfold PhiUA_arrow Id_fun PhiUA_1_arrow.
 Intros f.
 Apply (Prf_sym (Hom (UA_ob (UA_of (Ob_l dxc))) (Ob_r dxc))).
 Apply (Prf_UA_law2 ? ? ? ? (UA_of (Ob_l dxc)) 
            ? ((UA_mor (UA_of (Ob_l dxc))) o (FMor G f)) f).
 Unfold ua_eq.
  Apply (Prf_refl (Hom (Ob_l dxc) (FOb G (Ob_r dxc)))).
 Qed.

 Definition PhiUA_isIso := (Build_isIso SET ? ? (ApNT PhiUA dxc) (ApNT PhiUA_1 dxc) 
                                        PhiUA_1_o_PhiUA PhiUA_o_PhiUA_1). 

 End phi_ua_iso.

Definition PhiUA_iso := (NT_isIso ? ? ? ? PhiUA PhiUA_isIso).

Definition AdjUA := (Build_Adj AdjointUA G PhiUA PhiUA_iso).

Definition LeftAdjUA := (Build_leftAdj ? ? G AdjointUA AdjUA).

End ua_to_ladj.
                     

(* $Id: Th_Adjoint.v,v 1.2 1996/02/12 15:22:57 cpaulin Exp $ *)
