(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	            Existence of Right Adjoint                		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Adj_UA.

Section ua_to_radj.

Variable C, D : Category.
Variable F:(Functor D C).

Hypothesis UA_of : (c:(Ob C))(CoUA c F).

(* construction de l'adj droit *)

Definition CoadjointUA_ob := [c:(Ob C)](CoUA_ob (UA_of c)).
 
 Section coadjoint_ua_map_def.

 Variable c, c' : (Ob C).

 Definition CoadjointUA_mor := [f:|(Hom c c')|]
                               (CoUA_diese (UA_of c') ? ((CoUA_mor (UA_of c)) o f)).

 Lemma CoadjointUA_map_law : (Map_law CoadjointUA_mor).
 Proof.
 Unfold map_law CoadjointUA_mor; Intros f g H.
 Apply (Codiese_map ? ? ? ? (UA_of c') ? ((CoUA_mor (UA_of c)) o f)
                     ((CoUA_mor (UA_of c)) o g)).
 Apply (Prf_congl C); Assumption.
 Qed.

 Definition CoadjointUA_map := (Build_Map CoadjointUA_mor CoadjointUA_map_law).

 End coadjoint_ua_map_def.

Lemma CoadjointUA_id_law : (Fid_law CoadjointUA_ob CoadjointUA_map).
Proof.
Unfold fid_law; Simpl.
Unfold CoadjointUA_mor CoadjointUA_ob.
Intro c.
Apply (Prf_sym (Hom (CoUA_ob (UA_of c)) (CoUA_ob (UA_of c)))).
Apply (Prf_CoUA_law2 ? ? ? ? (UA_of c)).
Unfold coUA_eq.
(* *) Apply (Prf_trans (Hom (FOb F (CoUA_ob (UA_of c))) c)) with
  y:=(Id (FOb F (CoUA_ob (UA_of c)))) o (CoUA_mor (UA_of c)).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fid_law ? ? F).
(* *) Apply (Prf_trans (Hom (FOb F (CoUA_ob (UA_of c))) c)) with
  y:=(CoUA_mor (UA_of c)).
Unfold Comp; Apply (Prf_idl C).
Unfold Comp; Apply (Prf_idr C).
Qed.

Lemma CoadjointUA_comp_law : (Fcomp_law CoadjointUA_ob CoadjointUA_map).
Proof.
Unfold fcomp_law; Simpl.
Unfold CoadjointUA_mor; Intros c1 c2 c3 f g.
Apply (Prf_sym (Hom (CoadjointUA_ob c1) (CoadjointUA_ob c3))).
Apply (Prf_CoUA_law2 ? ? ? ? (UA_of c3)).
Unfold coUA_eq.
(* *) Apply (Prf_trans (Hom (FOb F (CoUA_ob (UA_of c1))) c3)) with
  y:=((FMor F (coUA_diese D C c2 F (UA_of c2)
                 (CoUA_ob (UA_of c1)) ((CoUA_mor (UA_of c1)) o f))) o
        (FMor F (coUA_diese D C c3 F (UA_of c3)
                    (CoUA_ob (UA_of c2)) ((CoUA_mor (UA_of c2)) o g))))
       o (CoUA_mor (UA_of c3)).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F).
(* *) Apply (Prf_trans (Hom (FOb F (CoUA_ob (UA_of c1))) c3)) with
  y:=(FMor F (coUA_diese D C c2 F (UA_of c2)
                 (CoUA_ob (UA_of c1)) ((CoUA_mor (UA_of c1)) o f))) o
        ((FMor F (coUA_diese D C c3 F (UA_of c3)
                    (CoUA_ob (UA_of c2)) ((CoUA_mor (UA_of c2)) o g)))
       o (CoUA_mor (UA_of c3))).
Apply (Prf_sym (Hom (FOb F (CoUA_ob (UA_of c1))) c3)).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (CoUA_ob (UA_of c1))) c3)) with
  y:=(FMor F (coUA_diese D C c2 F (UA_of c2)
                 (CoUA_ob (UA_of c1)) ((CoUA_mor (UA_of c1)) o f))) o
        ((CoUA_mor (UA_of c2)) o g).
Apply (Prf_congl C).
Apply (Prf_CoUA_law1 ? ? ? ? (UA_of c3) ? ((CoUA_mor (UA_of c2)) o g)).
(* *) Apply (Prf_trans (Hom (FOb F (CoUA_ob (UA_of c1))) c3)) with
  y:=((FMor F (coUA_diese D C c2 F (UA_of c2)
                (CoUA_ob (UA_of c1)) ((CoUA_mor (UA_of c1)) o f)))
       o (CoUA_mor (UA_of c2))) o g.
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (CoUA_ob (UA_of c1))) c3)) with
  y:=((CoUA_mor (UA_of c1)) o f) o g.
Apply (Prf_congr C).
Apply (Prf_CoUA_law1 ? ? ? ? (UA_of c2) ? ((CoUA_mor (UA_of c1)) o f)).
Apply (Prf_sym (Hom (FOb F (CoUA_ob (UA_of c1))) c3)).
Unfold Comp; Apply (Prf_ass C).
Qed.

Definition CoadjointUA := (Build_Functor C D CoadjointUA_ob CoadjointUA_map 
                                         CoadjointUA_comp_law CoadjointUA_id_law).

(* *)

(* definition de PhiUA':C(F-,-) -> D(-,G-) *)

 Section psi_ua_tau_def.

 Variable dxc : (POb (Dual D) C).

 Definition PsiUA_arrow := [f:|(Hom (FOb F (Ob_l dxc)) (Ob_r dxc))|] 
                           (CoUA_diese (UA_of (Ob_r dxc)) ? f).

 Lemma PsiUA_arrow_map_law : (Map_law PsiUA_arrow).
 Proof.
 Unfold map_law PsiUA_arrow.
 Intros f g H.
 Apply (Codiese_map ? ? ? ? (UA_of (Ob_r dxc)) ? f g).
 Assumption.
 Qed.

 Definition PsiUA_tau := (Build_Map PsiUA_arrow PsiUA_arrow_map_law).

 End psi_ua_tau_def.

Lemma PsiUA_tau_nt_law : (NT_law (FunSET2_r F) (FunSET2_l CoadjointUA) PsiUA_tau).
Proof.
Unfold nt_law PsiUA_tau PsiUA_arrow; Simpl.
Unfold Ext; Simpl.
Intros d1xc1 d2xc2 fxg h.
Unfold FunSET2_r_mor1 FunSET2_l_mor1.
Apply (Prf_sym (FunSET2_l_ob ? ? CoadjointUA d2xc2)).
Apply (Prf_CoUA_law2 ? ? ? ? (UA_of (Ob_r d2xc2)) ? 
               (((FMor F (Hom_l fxg)) o h) o (Hom_r fxg))).
Unfold coUA_eq.
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:=((FMor F ((Hom_l fxg) o (coUA_diese ? ? ? ?
                                        (UA_of (Ob_r d1xc1)) ? h)))
       o (FMor F (FMor CoadjointUA (Hom_r fxg))))
       o (CoUA_mor (UA_of (Ob_r d2xc2))).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:=(((FMor F (Hom_l fxg)) o (FMor F (coUA_diese ? ? ? ?
                                        (UA_of (Ob_r d1xc1)) ? h)))
       o (FMor F (FMor CoadjointUA (Hom_r fxg))))
       o (CoUA_mor (UA_of (Ob_r d2xc2))).
Apply (Prf_congr C).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:=((FMor F (Hom_l fxg)) o (FMor F (coUA_diese ? ? ? ?
                                        (UA_of (Ob_r d1xc1)) ? h)))
       o ((FMor F (FMor CoadjointUA (Hom_r fxg)))
       o (CoUA_mor (UA_of (Ob_r d2xc2)))).
Apply (Prf_sym (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:=(FMor F (Hom_l fxg)) o ((FMor F (coUA_diese ? ? ? ?
                                        (UA_of (Ob_r d1xc1)) ? h))
       o ((FMor F (FMor CoadjointUA (Hom_r fxg)))
       o (CoUA_mor (UA_of (Ob_r d2xc2))))).
Apply (Prf_sym (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:=(FMor F (Hom_l fxg)) o (h o (Hom_r fxg)).
Apply (Prf_congl C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d1xc1)) (Ob_r d2xc2))) with
  y:=(FMor F (coUA_diese ? ? ? ? (UA_of (Ob_r d1xc1)) ? h)) o 
     ((CoUA_mor (UA_of (Ob_r d1xc1))) o (Hom_r fxg)).
Apply (Prf_congl C).
Apply (Prf_CoUA_law1 ? ? ? ? (UA_of (Ob_r d2xc2)) ? 
              ((CoUA_mor (UA_of (Ob_r d1xc1))) o (Hom_r fxg))).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d1xc1)) (Ob_r d2xc2))) with
  y:=((FMor F (coUA_diese ? ? ? ? (UA_of (Ob_r d1xc1)) ? h)) o 
     (CoUA_mor (UA_of (Ob_r d1xc1)))) o (Hom_r fxg).
Unfold Comp; Apply (Prf_ass C).
Apply (Prf_congr C).
Apply (Prf_CoUA_law1 ? ? ? ? (UA_of (Ob_r d1xc1)) ? h).
Unfold Comp; Apply (Prf_ass C).
Qed.

Definition PsiUA := (Build_NT (FunSET2_r F) (FunSET2_l CoadjointUA) 
                              PsiUA_tau PsiUA_tau_nt_law).
               
(* PsiUA_1 *)

 Section psi_ua_1_tau_def.

 Variable dxc : (POb (Dual D) C).

 Definition PsiUA_1_arrow := [f:|(Hom (Ob_l dxc) (FOb CoadjointUA (Ob_r dxc)))|]
                             (FMor F f) o (CoUA_mor (UA_of (Ob_r dxc))).

 Lemma PsiUA_1_arrow_map_law : (Map_law PsiUA_1_arrow).                     
 Proof.
 Unfold map_law PsiUA_1_arrow.
 Intros f g H.
 Apply (Prf_congr C).
 Unfold fMor; Apply (Prf_FMap_law ? ? F(Ob_l dxc) (FOb CoadjointUA (Ob_r dxc))).
 Assumption. 
 Qed.

 Definition PsiUA_1_tau := (Build_Map PsiUA_1_arrow PsiUA_1_arrow_map_law).
 
 End psi_ua_1_tau_def.
      
Lemma PsiUA_1_tau_nt_law : (NT_law (FunSET2_l CoadjointUA) (FunSET2_r F) PsiUA_1_tau).
Proof.
Unfold nt_law PsiUA_1_tau PsiUA_1_arrow; Simpl.
Unfold Ext; Simpl.
Intros d1xc1 d2xc2 fxg h.
Unfold FunSET2_r_mor1 FunSET2_l_mor1.
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:= (((FMor F (Hom_l fxg)) o (FMor F h)) o (FMor F (FMor CoadjointUA (Hom_r fxg))))
       o (CoUA_mor (UA_of (Ob_r d2xc2))).
Apply (Prf_congr C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (FOb F (FOb CoadjointUA (Ob_r d2xc2))))) with
  y:=(FMor F ((Hom_l fxg) o h)) o (FMor F (FMor CoadjointUA (Hom_r fxg))).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F).
Apply (Prf_congr C).
Unfold fMor; Apply (Prf_Fcomp_law ? ? F).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:= ((FMor F (Hom_l fxg)) o (FMor F h)) o ((FMor F (FMor CoadjointUA (Hom_r fxg)))
       o (CoUA_mor (UA_of (Ob_r d2xc2)))).
Apply (Prf_sym (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:= (FMor F (Hom_l fxg)) o ((FMor F h) o ((FMor F (FMor CoadjointUA (Hom_r fxg)))
       o (CoUA_mor (UA_of (Ob_r d2xc2))))).
Apply (Prf_sym (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d2xc2)) (Ob_r d2xc2))) with
  y:=(FMor F (Hom_l fxg)) o (((FMor F h) o (CoUA_mor (UA_of (Ob_r d1xc1))))
       o (Hom_r fxg)).
Apply (Prf_congl C).
(* *) Apply (Prf_trans (Hom (FOb F (Ob_l d1xc1)) (Ob_r d2xc2))) with
  y:=(FMor F h) o ((CoUA_mor (UA_of (Ob_r d1xc1))) o (Hom_r fxg)).
Apply (Prf_congl C).
Apply (Prf_CoUA_law1 ? ? ? ? (UA_of (Ob_r d2xc2)) ? 
               ((CoUA_mor (UA_of (Ob_r d1xc1))) o (Hom_r fxg))).
Unfold Comp; Apply (Prf_ass C).
Unfold Comp; Apply (Prf_ass C).
Qed.

Definition PsiUA_1 := (Build_NT (FunSET2_l CoadjointUA) (FunSET2_r F) 
                                PsiUA_1_tau PsiUA_1_tau_nt_law).

(* PsiUA et PsiUA_1 sont iso *)

 Section psi_ua_iso.

 Variable dxc : (POb (Dual D) C).

 Lemma PsiUA_1_o_PsiUA : (Iso_law SET ? ? (PsiUA_tau dxc) (PsiUA_1_tau dxc)).
 Proof.
 Unfold Iso_law; Simpl.
 Unfold Ext; Simpl.
 Unfold PsiUA_arrow Id_fun PsiUA_1_arrow.
 Intros f.
 (* *) Apply (Prf_sym (Hom (Ob_l dxc) (CoadjointUA_ob (Ob_r dxc)))).
 Apply (Prf_CoUA_law2 ? ? ? ? (UA_of (Ob_r dxc))).
 Unfold coUA_eq.
 Apply (Prf_refl (Hom (FOb F (Ob_l dxc)) (Ob_r dxc))).
 Qed.
 
 Lemma PsiUA_o_PsiUA_1 : (Iso_law SET ? ? (PsiUA_1_tau dxc) (PsiUA_tau dxc)).
 Proof.
 Unfold Iso_law; Simpl.
 Unfold Ext; Simpl.
 Unfold PsiUA_arrow Id_fun PsiUA_1_arrow.
 Intros f.
 Apply (Prf_CoUA_law1 ? ? ? ? (UA_of (Ob_r dxc)) ? f).
 Qed.

 Definition PsiUA_isIso := (Build_isIso SET ? ? (ApNT PsiUA dxc) (ApNT PsiUA_1 dxc) 
                                        PsiUA_1_o_PsiUA PsiUA_o_PsiUA_1).

 End psi_ua_iso.

Definition PsiUA_iso := (NT_isIso ? ? ? ? PsiUA PsiUA_isIso).

Definition CoAdjUA := (Build_Adj F CoadjointUA PsiUA PsiUA_iso).

Definition RightAdjUA := (Build_rightAdj ? ? F CoadjointUA CoAdjUA).

End ua_to_radj.


(* $Id: Th_CoAdjoint.v,v 1.2 1996/02/12 15:22:59 cpaulin Exp $ *)
