(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Dual Category                                  *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export Category.


Section d_cat.

Variable C : Category.

Definition DHom := [a,b:(Ob C)](Hom b a).

(* construire un Comp_Type *)

Definition Comp_Darrow := [a,b,c:(Ob C)][df:|(DHom a b)|][dg:|(DHom b c)|]
                          dg o df.
                          
Lemma Comp_dual_congl : (Congl_law (Ob C) DHom Comp_Darrow).
Proof.
Unfold Congl_law; Simpl.
Unfold Comp_Darrow; Simpl.
Intros a b c f g h eqf_g.
Apply (Prf_congr C ? ? ? f g h); Apply eqf_g.
Qed.
                          
Lemma Comp_dual_congr : (Congr_law (Ob C) DHom Comp_Darrow).
Proof.
Unfold Congr_law; Simpl.
Unfold Comp_Darrow; Simpl.
Intros a b c f g h eqf_g.
Apply (Prf_congl C ? ? ? f g h); Apply eqf_g.
Qed.

Definition Comp_Dual := (Build_Comp (Ob C) DHom Comp_Darrow
                         Comp_dual_congl Comp_dual_congr).
            
Lemma Assoc_Dual : (assoc_law (Ob C) DHom Comp_Dual).
Proof.
Unfold assoc_law DHom Cat_comp ap2; Simpl. 
Unfold Comp_Darrow.
Intros a b c d df dg dh.
Apply (Prf_sym (Hom d a)).
Unfold Comp; Apply (Prf_ass C).
Qed.

Lemma Idl_Dual : (idl_law (Ob C) DHom Comp_Dual (id C)).
Proof.
Unfold idl_law DHom Cat_comp ap2; Simpl. 
Unfold Comp_Darrow.
Intros a b df.
Apply (Prf_sym (Hom b a)).
Unfold Comp; Apply (Prf_idr C).
Qed.

Lemma Idr_Dual : (idr_law (Ob C) DHom Comp_Dual (id C)).
Proof.
Unfold idr_law DHom Cat_comp ap2; Simpl. 
Unfold Comp_Darrow.
Intros a b df.
Apply (Prf_sym (Hom a b)).
Unfold Comp; Apply (Prf_idl C).
Qed.

Definition Dual := (Build_Category (Ob C) DHom Comp_Dual (id C) Assoc_Dual Idl_Dual Idr_Dual).

End d_cat.


(* $Id: Dual.v,v 1.2 1996/02/12 15:23:46 cpaulin Exp $ *)
