(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Hom Equality                                   *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export Category.

Inductive Equal_hom [C:Category;a,b:(Ob C);f:|(Hom a b)|] : 
                    (c,d:(Ob C))|(Hom c d)| -> Prop :=
Build_Equal_hom : (g:|(Hom a b)|) f =%S g -> (Equal_hom C a b f a b g). 
Hint Build_Equal_hom.

Token "=%H".

Syntax constr Equal_hom <<(Equal_hom $1 $2 $3 $f $4 $5 $g)>> 1
     [<hov 0>     <$f:"Command":L> " =%H " <$g:"Command":L>].

Grammar command command1 :=
  [ command0($f) "=%H" command0($g) ] -> 
         [$0 = <<(Equal_hom ? ? ? $f ? ? $g)>>].

Section equal_hom_equiv.

Variable C     : Category.
Variable a , b : (Ob C).
Variable f     : |(Hom a b)|.

Lemma Equal_hom_refl : f =%H f.
Proof.
Apply Build_Equal_hom; Apply (Prf_refl (Hom a b)).
Qed.

Variable c, d : (Ob C).
Variable g    : |(Hom c d)|.

Lemma Equal_hom_sym : f =%H g -> g =%H f.
Proof.
Intros eqdep_fg.
Elim eqdep_fg; Intros h eq_fh.
Apply Build_Equal_hom; Apply (Prf_sym (Hom a b)); Assumption.
Qed.

Variable i, j : (Ob C).
Variable h    : |(Hom i j)|.

Lemma Equal_hom_trans : f =%H g -> g =%H h -> f =%H h.
Proof.
Intros eqdep_fg.
Elim eqdep_fg; Intros k eq_fk eqdep_kh.
Elim  eqdep_kh; Intros l eq_kl.
Apply Build_Equal_hom.
Apply (Prf_trans (Hom a b) f k l); Auto. 
Qed.

End equal_hom_equiv.

Hint Equal_hom_refl.
Hint Equal_hom_sym.


(* "set" of all morphisms of a category *)

Section arrs_setoid_def.

Variable C : Category.

Structure Arrs : Type :=
 {dom   : (Ob C);
  codom : (Ob C);
  arrow : |(Hom dom codom)|}.

Definition Equal_Arrs := [f,g:Arrs] (arrow f) =%H (arrow g).

Lemma Equal_Arrs_equiv : (Equivalence Equal_Arrs).
Proof.
Apply Build_equivalence; Red; Unfold Equal_Arrs.
Intro x; Apply Equal_hom_refl. 
Intros f g h; Exact (Equal_hom_trans ? ? ? (arrow f) ? ? (arrow g) ? ? (arrow h)).
Intros f g; Exact (Equal_hom_sym ? ? ? (arrow f) ? ? (arrow g)).
Qed.

Definition Arrs_setoid := (Build_Setoid Arrs Equal_Arrs Equal_Arrs_equiv).

End arrs_setoid_def.

Syntactic Definition Dom := dom | 1.
Syntax constr dom <<(dom $1 $u)>> 10
	     [<hov 0> "Dom " <$u:"term":L>].

Syntactic Definition Codom := codom | 1.
Syntax constr codom <<(codom $1 $u)>> 10
	     [<hov 0> "Codom " <$u:"term":L>].

Syntactic Definition Arrow := arrow | 1.
Syntax constr arrow <<(arrow $1 $u)>> 10
	     [<hov 0> "Arrow " <$u:"term":L>].

(* $Id: Hom_Equality.v,v 1.2 1996/02/12 15:23:51 cpaulin Exp $ *)
