(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                   Monoids Category                		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export SET.
Require Export Monoid.

(* Monoid Morphism composition *)

Section comp_mon.

Variable m1, m2, m3 : Monoid.
Variable f          : (MonMor m1 m2).
Variable g          : (MonMor m2 m3).

Definition Comp_MonMor_map := (MonMap f) o%M (MonMap g).

Lemma Comp_MonMor_unit_law : (MonUnit_law Comp_MonMor_map).
Proof.
Unfold monUnit_law Comp_MonMor_map. 
Simpl; Unfold Comp_fun.
Apply (Prf_trans (Mcarrier m3)) with
  y:=(Ap (MonMap g) (Munit m2)).
Apply (Pres ? ? (MonMap g)).
Apply (Prf_MonUnit_law ? ? f).
Apply (Prf_MonUnit_law ? ? g).
Qed.

Lemma Comp_MonMor_op_law : (MonOp_law Comp_MonMor_map).
Proof.
Unfold monOp_law Comp_MonMor_map.
Simpl; Unfold Comp_fun.
Intros a b.
Apply (Prf_trans (Mcarrier m3)) with
  y:=(Ap (MonMap g) 
         ((Ap (MonMap f) a) +%M (Ap (MonMap f) b))).
Apply (Pres ? ? (MonMap g)).
Apply (Prf_MonOp_law ? ? f).
Apply (Prf_MonOp_law ? ? g).
Qed.
 
Definition Comp_MonMor := (Build_MonMor Comp_MonMor_map Comp_MonMor_unit_law Comp_MonMor_op_law).

End comp_mon.

Lemma Comp_MonMor_congl : (Congl_law Monoid MonMor_setoid Comp_MonMor).
Proof.
Unfold Congl_law; Simpl.
Unfold Equal_MonMor Ext; Simpl.
Unfold Comp_fun.
Intros m1 m2 m3 f g h H a.
Apply H.
Qed.

Lemma Comp_MonMor_congr : (Congr_law Monoid MonMor_setoid Comp_MonMor).
Proof.
Unfold Congr_law; Simpl.
Unfold Equal_MonMor Ext; Simpl.
Unfold Comp_fun.
Intros m1 m2 m3 f g h H a.
Apply (Pres ? ? (MonMap h)).
Apply H.
Qed.

Definition Comp_MON := (Build_Comp Monoid MonMor_setoid Comp_MonMor
                              Comp_MonMor_congl Comp_MonMor_congr).

Lemma Assoc_MON : (assoc_law Monoid MonMor_setoid Comp_MON).
Proof.
Unfold assoc_law; Simpl; Intros m1 m2 m3 m4 f g h.
Unfold Equal_MonMor Ext; Simpl.
Unfold Comp_fun; Intro a.
Apply (Prf_refl (Mcarrier m4)).
Qed.

(* Id *)

Section id_mon_def.

Variable m : Monoid.

Definition Id_MON_map := (Id_SET (Mcarrier m)).

Lemma Id_MON_unit_law : (MonUnit_law Id_MON_map).
Proof.
Unfold monUnit_law Id_MON_map; Simpl.
Unfold Id_fun; Apply (Prf_refl (Mcarrier m)). 
Qed.

Lemma Id_MON_op_law : (MonOp_law Id_MON_map).
Proof.
Unfold monOp_law Id_MON_map; Simpl.
Unfold Id_fun; Intros a b.
Apply (Prf_refl (Mcarrier m)).
Qed.

Definition Id_MON := (Build_MonMor Id_MON_map Id_MON_unit_law Id_MON_op_law).

End id_mon_def.

Lemma Idl_MON : (idl_law Monoid MonMor_setoid Comp_MON Id_MON).
Proof.
Unfold idl_law; Intros m1 m2 f; Simpl.
Unfold Equal_MonMor Ext; Simpl.
Unfold Comp_fun; Intro a.
Unfold Id_MON_map; Simpl; Unfold Id_fun.
Apply (Prf_refl (Mcarrier m2)).
Qed.

Lemma Idr_MON : (idr_law Monoid MonMor_setoid Comp_MON Id_MON).
Proof.
Unfold idr_law; Intros m1 m2 f; Simpl.
Unfold Equal_MonMor Ext; Simpl.
Unfold Comp_fun; Intro a.
Unfold Id_MON_map; Simpl; Unfold Id_fun.
Apply (Prf_refl (Mcarrier m1)).
Qed.

Definition MON := (Build_Category Monoid MonMor_setoid Comp_MON Id_MON
                                  Assoc_MON Idl_MON Idr_MON).

                    

(* $Id: MON.v,v 1.2 1996/02/12 15:23:56 cpaulin Exp $ *)
