(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Product                                                 *)
(*                                                                           *)
(*          Amokrane Saibi,  May 1994                                        *)
(*                                                                           *)
(*****************************************************************************)


Require Export Category. 
Require Export SetoidPROD.

Structure POb [A,B:Category] : Type := build_POb
 {ob_l : (Ob A);
  ob_r : (Ob B)}.

Syntactic Definition Build_POb := build_POb | 2.
Syntax constr build_POb <<(build_POb $1 $2 $a $b)>> 10
       [<hov 0> "Build_POb " <$a:"term":L> " " <$b:"term":L>].

Syntactic Definition Ob_l := ob_l | 2.
Syntax constr ob_l <<(ob_l $1 $2 $axb)>> 10
       [<hov 0> "Ob_l " <$axb:"term":L>].

Syntactic Definition Ob_r := ob_r | 2.
Syntax constr ob_r <<(ob_r $1 $2 $axb)>> 10
       [<hov 0> "Ob_r " <$axb:"term":L>].

Structure pmor [A,B:Category;u,t:(POb A B)] : Type := build_Pmor
 {hom_l : |(Hom (Ob_l u) (Ob_l t))|;
  hom_r : |(Hom (Ob_r u) (Ob_r t))| }.

Syntactic Definition Pmor := pmor | 2.
Syntax constr pmor <<(pmor $1 $2 $a $b)>> 10
       [<hov 0> "Pmor " <$a:"term":L> " " <$b:"term":L>].

Syntactic Definition Build_Pmor := build_Pmor | 4.
Syntax constr build_Pmor <<(build_Pmor $1 $2 $3 $4 $f $g)>> 10
       [<hov 0> "Build_Pmor " <$f:"term":L> " " <$g:"term":L> ].

Syntactic Definition Hom_l := hom_l | 4.
Syntax constr hom_l <<(hom_l $1 $2 $3 $4 $f)>> 10
       [<hov 0> "Hom_l " <$f:"term":L>].

Syntactic Definition Hom_r := hom_r | 4.
Syntax constr hom_r <<(hom_r $1 $2 $3 $4 $f)>> 10
       [<hov 0> "Hom_r " <$f:"term":L>].


Section ProdCat.

Variable A, B : Category.

 Section pmor_setoid_def.

 Variable u, t : (POb A B).

 Definition Equal_Pmor := [f,g:(Pmor u t)]
            ((Hom_l f) =%S (Hom_l g)) /\ ((Hom_r f) =%S (Hom_r g)).

 Lemma Equal_Pmor_equiv : (Equivalence Equal_Pmor).
 Proof.
 Apply Build_equivalence.
 Unfold Reflexive Equal_Pmor; Intro x; Split.  
 Apply (Prf_refl (Hom (Ob_l u) (Ob_l t))).
 Apply (Prf_refl (Hom (Ob_r u) (Ob_r t))).
 Unfold Transitive Equal_Pmor; Intros x1 x2 x3 H H0.
 Elim H; Intros H1 H2; Elim H0; Intros H3 H4; Split.
 (* *) Apply (Prf_trans (Hom (Ob_l u) (Ob_l t))) 
  with y:=(Hom_l x2); Assumption.
 (* *) Apply (Prf_trans (Hom (Ob_r u) (Ob_r t))) 
  with y:=(Hom_r x2); Assumption.
 Unfold Symmetric Equal_Pmor; Intros x1 x2 H.
 Elim H; Intros H1 H2; Split.
 Apply (Prf_sym (Hom (Ob_l u) (Ob_l t))); Assumption.
 Apply (Prf_sym (Hom (Ob_r u) (Ob_r t))); Assumption.
 Qed.

 Definition Prod_Hom := (Build_Setoid (Pmor u t) Equal_Pmor Equal_Pmor_equiv).

 Definition Prod_Hom' := (SPROD (Hom (Ob_l u) (Ob_l t)) (Hom (Ob_r u) (Ob_r t))).

 End pmor_setoid_def.

(* composition *)

Definition Comp_Pmor := [a,b,c:(POb A B)][f:(Pmor a b)][g:(Pmor b c)]
           (Build_Pmor ((Hom_l f) o (Hom_l g)) ((Hom_r f) o (Hom_r g))).

Lemma Comp_Pmor_congl : (Congl_law (POb A B) Prod_Hom Comp_Pmor).
Proof.
Unfold Congl_law; Simpl.
Unfold Equal_Pmor Comp_Pmor; Simpl.
Intros a b c f g h H. 
Elim H; Intros H1 H2; Split.
Apply (Prf_congl A); Assumption.
Apply (Prf_congl B); Assumption.
Qed.

Lemma Comp_Pmor_congr : (Congr_law (POb A B) Prod_Hom Comp_Pmor).
Proof.
Unfold Congr_law; Simpl.
Unfold Equal_Pmor Comp_Pmor; Simpl.
Intros a b c f g.
Elim f; Elim g; Simpl.
Intros gl gr fl fr H h.
Elim h; Simpl.
Intros hl hr.
Elim H; Intros H1 H2; Split.
Apply (Prf_congr A); Assumption.
Apply (Prf_congr B); Assumption.
Qed.

Definition Comp_PROD := (Build_Comp (POb A B) Prod_Hom Comp_Pmor
                         Comp_Pmor_congl Comp_Pmor_congr).

Lemma Assoc_PROD : (assoc_law (POb A B) Prod_Hom Comp_PROD).
Proof.
Unfold assoc_law; Simpl.
Unfold Equal_Pmor; Simpl.
Intros; Split.
Unfold Comp; Apply (Prf_ass A).
Unfold Comp; Apply (Prf_ass B).
Qed.

Definition Id_PROD := [a:(POb A B)]
                      (Build_Pmor (Id (Ob_l a)) (Id (Ob_r a))).

Lemma Idl_PROD : (idl_law (POb A B) Prod_Hom Comp_PROD Id_PROD).
Proof.
Unfold idl_law; Simpl.
Unfold Equal_Pmor; Simpl.
Intros; Split.
Unfold Comp; Apply (Prf_idl A).
Unfold Comp; Apply (Prf_idl B).
Qed.
 
Lemma Idr_PROD : (idr_law (POb A B) Prod_Hom Comp_PROD Id_PROD).
Proof.
Unfold idr_law; Simpl.
Unfold Equal_Pmor; Simpl.
Intros; Split.
Unfold Comp; Apply (Prf_idr A).
Unfold Comp; Apply (Prf_idr B).
Qed.
 
Definition PROD := (Build_Category (POb A B) Prod_Hom Comp_PROD Id_PROD
                           Assoc_PROD Idl_PROD Idr_PROD).

End ProdCat.

(* $Id: PROD.v,v 1.2 1996/02/12 15:24:05 cpaulin Exp $ *)
