(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          SET : The Category of Setoids                                    *)
(*                                                                           *)
(*          Peter Aczel, Dec. 92                                             *)
(*          rev. Gerard Huet,  Aug. 1993                                     *)
(*          rev. Amokrane Saibi,  May 1994                                   *)
(*                                                                           *)
(*****************************************************************************)

Require Export PermCat.


(* Map composition operator *)

Lemma Comp_map_congl : (Congl_law Setoid Map_setoid Comp_map).
Proof.
Unfold Congl_law; Simpl.
Unfold Ext; Simpl.
Unfold Comp_fun; Auto.
Qed.

Lemma Comp_map_congr : (Congr_law Setoid Map_setoid Comp_map).
Proof.
Unfold Congr_law; Simpl.
Unfold Ext; Simpl.
Unfold Comp_fun.
Intros a b c f g h e x.
Apply (Pres ? ? h); Auto.
Qed.

Definition Comp_SET := (Build_Comp Setoid Map_setoid Comp_map
                                      Comp_map_congl Comp_map_congr).

Lemma Assoc_SET : (assoc_law Setoid Map_setoid Comp_SET).
Proof.
Unfold assoc_law; Simpl; Intros.
Unfold Ext; Intro x; Simpl.
Apply (Prf_refl d).
Qed.

(* Id *)

Definition Id_SET := Id_map.
 
Lemma Idl_SET : (idl_law Setoid Map_setoid Comp_SET Id_SET).
Proof.
Unfold idl_law; Intros.
Unfold Comp.
Unfold ap2; Simpl.
Unfold Ext; Simpl.
Intro x; Apply (Prf_refl b).
Qed.

Lemma Idr_SET : (idr_law Setoid Map_setoid Comp_SET Id_SET).
Proof.
Unfold idr_law; Intros.
Unfold Comp.
Unfold ap2; Simpl.
Unfold Ext; Simpl.
Intro x; Apply (Prf_refl a).
Qed.

Definition SET := (Build_Category Setoid Map_setoid Comp_SET Id_SET Assoc_SET 
                                  Idl_SET Idr_SET).

Definition S_Perm_Group := [A:Setoid](Perm_Group SET A).

(* $Id: SET.v,v 1.2 1996/02/12 15:24:12 cpaulin Exp $ *)
