(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 CatCAT.v                                 *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : The Category CAT                               *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)


Require Export IdCAT.

(* Functors Setoid *)

Lemma Equal_Functor_equiv : (C,D:Category)(Equivalence (Equal_Functor C D)).
Proof.
Intros C D; Apply Build_equivalence; Unfold Equal_Functor; Auto.
Unfold Transitive; Intros F G H Ass1 Ass2 a b f.
(* *) Apply Equal_hom_trans with (FOb G a) (FOb G b) (FMor G f); Auto.
Qed.

Definition Functor_setoid:= [C,D:Category]
 (Build_Setoid (Functor C D) (Equal_Functor C D) (Equal_Functor_equiv C D)).


(* Functor composition operator *)

Lemma Comp_Functor_congl : (Congl_law Category Functor_setoid Comp_Functor).
Proof.
Unfold Congl_law; Simpl.
Unfold Equal_Functor Comp_Functor.
Intros A B C F G H eq_FG a1 a2 f.
Unfold fMor; Simpl.
Unfold comp_FOb comp_FMor.
Apply eq_FG.
Qed.

Lemma Comp_Functor_congr : (Congr_law Category Functor_setoid Comp_Functor).
Proof.
Unfold Congr_law; Simpl.
Unfold Equal_Functor Comp_Functor.
Intros A B C F G H eq_FG a1 a2 f.
Unfold fMor; Simpl.
Unfold comp_FOb comp_FMor.
 Elim (eq_FG a1 a2 f); Intros g eq_Ff_g.
Apply Build_Equal_hom.
Unfold fMor; Apply (Prf_FMap_law B C H (FOb F a1) (FOb F a2)).
Assumption.
Qed.

Definition Comp_CAT := (Build_Comp Category Functor_setoid 
                        Comp_Functor Comp_Functor_congl Comp_Functor_congr).
     
Lemma Assoc_CAT : (assoc_law Category Functor_setoid Comp_CAT).
Proof.
Unfold assoc_law Cat_comp; Simpl.
Intros C D E F G H X.
Unfold Equal_Functor; Simpl.
Unfold ap2; Simpl; Auto.
Qed.

(* Id *)

Lemma Idl_CAT : (idl_law Category Functor_setoid Comp_CAT Id_CAT).
Proof.
Unfold idl_law Cat_comp ap2; Simpl.
Intros C D G; Unfold Comp_Functor Equal_Functor.
Intros a b f; Unfold fMor; Simpl.
Unfold comp_FMor; Unfold Id_CAT fMor; Simpl.
Auto.
Qed.

Lemma Idr_CAT : (idr_law Category Functor_setoid Comp_CAT Id_CAT).
Proof.
Unfold idr_law Cat_comp ap2; Simpl.
Intros C D G; Unfold Comp_Functor Equal_Functor.
Intros a b f; Unfold fMor; Simpl.
Unfold comp_FMor; Unfold Id_CAT fMor; Simpl.
Auto.
Qed.

(* The Category CAT *)

Structure Category' : Type :=
 {Ob'      : Type;
  hom'     : Ob' -> Ob' -> Setoid;
  opcomp'  : (a,b,c:Ob')|((hom' a b) => ((hom' b c) => (hom' a c)))|;
  id'      : (a:Ob')|(hom' a a)|;
  Prf_ass' : (Assoc_law opcomp');
  Prf_idl' : (Idl_law opcomp' id');
  Prf_idr' : (Idr_law opcomp' id')}.

Definition CAT : Category' := 
  (Build_Category' Category Functor_setoid Comp_CAT Id_CAT Assoc_CAT 
                   Idl_CAT Idr_CAT).
             

(* $Id: CAT.v,v 1.2 1996/02/12 15:24:52 cpaulin Exp $ *)
