(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                      Comma Category (x|G)            		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Functor.


(* (x|G) *)

Section comma_def.

Variable A,X : Category.
Variable G   : (Functor A X).
Variable x   : (Ob X).

Structure com_ob : Type := build_Com_ob
 {ob_com_ob  : (Ob A);
  mor_com_ob : |(Hom x (FOb G ob_com_ob))|}.

 Section com_arrow_def.

 Variable axf, bxg : com_ob.
 
 Definition com_law := [h:|(Hom (ob_com_ob axf) (ob_com_ob bxg))|]
                       (mor_com_ob bxg) =%S ((mor_com_ob axf) o (FMor G h)).

 Structure com_arrow : Type := build_Com_arrow
  {mor_com_arrow : |(Hom (ob_com_ob axf) (ob_com_ob bxg))|;
   Prf_com_law   : (com_law mor_com_arrow)}.

 Definition Equal_com_arrow := [h,h':com_arrow]
            (mor_com_arrow h) =%S (mor_com_arrow h').

 Lemma Equal_com_arrow_equiv : (Equivalence Equal_com_arrow).
 Proof.
 Apply Build_equivalence; Unfold Equal_com_arrow.
 Unfold Reflexive; Intro f.
 Apply (Prf_refl (Hom (ob_com_ob axf) (ob_com_ob bxg))).
 Unfold Transitive; Intros f g h H1 H2.
 (* *) Apply (Prf_trans (Hom (ob_com_ob axf) (ob_com_ob bxg))) with 
   y:=(mor_com_arrow g);
 Assumption. 
 Unfold Symmetric; Intros f g H.
 Apply (Prf_sym (Hom (ob_com_ob axf) (ob_com_ob bxg))); Assumption.
 Qed.

 Definition Com_arrow_setoid := (Build_Setoid com_arrow Equal_com_arrow Equal_com_arrow_equiv).

 End com_arrow_def. 

(* composition of two arrows in comma *)

 Section comp_com_def.

 Variable axf, bxg, cxh : com_ob.
 Variable f             : (com_arrow axf bxg).
 Variable g             : (com_arrow bxg cxh).

 Definition Comp_com_mor := (mor_com_arrow ? ? f) o (mor_com_arrow ? ? g).

 Lemma Comp_com_law : (com_law ? ? Comp_com_mor).
 Proof.
 Unfold com_law Comp_com_mor.
 (* *) Apply (Prf_trans (Hom x (FOb G (ob_com_ob cxh)))) with
   y:=(mor_com_ob bxg) o (FMor G (mor_com_arrow ? ? g)).
 Apply (Prf_com_law bxg cxh g).
 (* *) Apply (Prf_trans (Hom x (FOb G (ob_com_ob cxh)))) with
   y:=(mor_com_ob axf) o ((FMor G (mor_com_arrow ? ? f)) o
                           (FMor G (mor_com_arrow ? ? g))).
 (* *) Apply (Prf_trans (Hom x (FOb G (ob_com_ob cxh)))) with
   y:=((mor_com_ob axf) o (FMor G (mor_com_arrow ? ? f))) o
                           (FMor G (mor_com_arrow ? ? g)).
 Apply (Prf_congr X).
 Apply (Prf_com_law ? ? f).
 Apply (Prf_sym (Hom x (FOb G (ob_com_ob cxh)))).
 Unfold Comp; Apply (Prf_ass X).
 Apply (Prf_congl X).
 Apply (Prf_sym (Hom (FOb G (ob_com_ob axf)) (FOb G (ob_com_ob cxh)))).
 Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
 Qed.

 Definition Comp_com_arrow := (build_Com_arrow ? ? Comp_com_mor Comp_com_law).

 End comp_com_def.
 
(* composition operator *)

Lemma Comp_com_congl : (Congl_law com_ob Com_arrow_setoid Comp_com_arrow).
Proof.
Unfold Congl_law; Simpl.
Intros a b c f1; Elim f1; Intros h e f2.
Elim f2; Intros h' e' f3; Elim f3; Intros h'' e''.
Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Intro H; Apply (Prf_congl A); Trivial.
Qed.

Lemma Comp_com_congr : (Congr_law com_ob Com_arrow_setoid Comp_com_arrow).
Proof.
Unfold Congr_law; Simpl.
Intros a b c f1; Elim f1; Intros h e f2.
Elim f2; Intros h' e' f3; Elim f3; Intros h'' e''.
Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Intro H; Apply (Prf_congr A); Trivial.
Qed.

Definition Comp_Comma := (Build_Comp com_ob Com_arrow_setoid 
                                  Comp_com_arrow Comp_com_congl Comp_com_congr). 

Lemma Assoc_Comma : (assoc_law com_ob Com_arrow_setoid Comp_Comma).
Proof.
Unfold assoc_law; Intros a b c d f1; Elim f1.
Intros h1 e1 f2; Elim f2; Intros h2 e2 f3.
Elim f3; Intros h3 e3.
Simpl; Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Unfold Comp_com_mor; Simpl.
Unfold Comp; Apply (Prf_ass A).
Qed.

(* Id *)

Lemma Id_com_law : (axf:com_ob)(com_law ? ? (Id (ob_com_ob axf))).
Proof.
Intro axf; Unfold com_law.
(* *) Apply (Prf_trans (Hom x (FOb G (ob_com_ob axf)))) with
  y:=(mor_com_ob axf) o (Id (FOb G (ob_com_ob axf))).
Unfold Comp; Apply (Prf_idr X).
Apply (Prf_congl X).
Apply (Prf_sym (Hom (FOb G (ob_com_ob axf)) (FOb G (ob_com_ob axf)))).
Unfold fMor; Apply (Prf_Fid_law ? ? G).
Qed.

Definition Id_Comma := [axf:com_ob](build_Com_arrow ? ? (Id (ob_com_ob axf)) 
                                                    (Id_com_law axf)).

Lemma Idl_Comma : (idl_law com_ob Com_arrow_setoid Comp_Comma Id_Comma).
Proof.
Unfold idl_law; Intros a b f1; Elim f1; Intros h e.
Simpl; Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Unfold Comp; Apply (Prf_idl A).
Qed.

Lemma Idr_Comma : (idr_law com_ob Com_arrow_setoid Comp_Comma Id_Comma).
Proof.
Unfold idr_law; Intros a b f1; Elim f1; Intros h e.
Simpl; Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Unfold Comp; Apply (Prf_idr A).
Qed.

Definition comma := (Build_Category com_ob Com_arrow_setoid 
                       Comp_Comma Id_Comma Assoc_Comma Idl_Comma Idr_Comma).

End comma_def.


Syntactic Definition Com_ob := com_ob | 2.
Syntax constr com_ob <<(com_ob $1 $2 $G $x)>> 10
	     [<hov 0> "Com_ob " <$G:"term":L> " " <$x:"term":L>].

Syntactic Definition Build_Com_ob := build_Com_ob | 2.
Syntax constr build_Com_ob <<(build_Com_ob $1 $2 $G $x $a $f)>> 10
	     [<hov 0> "Build_Com_ob " <$G:"term":L> " " <$x:"term":L> " "
                                      <$a:"term":L> " " <$f:"term":L>].

Syntactic Definition Ob_com_ob := ob_com_ob | 4.
Syntax constr ob_com_ob <<(ob_com_ob $1 $2 $3 $4 $co)>> 10
	     [<hov 0> "Ob_com_ob " <$co:"term":L>].

Syntactic Definition Mor_com_ob := mor_com_ob | 4.
Syntax constr mor_com_ob <<(mor_com_ob $1 $2 $3 $4 $co)>> 10
	     [<hov 0> "Mor_com_ob " <$co:"term":L>].

Syntactic Definition Com_law := com_law | 4.
Syntax constr com_law <<(com_law $1 $2 $3 $4 $a $b $f)>> 10
	     [<hov 0> "Com_law " <$a:"term":L> " " <$b:"term":L> " " <$f:"term":L>].

Syntactic Definition Com_arrow := com_arrow | 4.
Syntax constr com_arrow <<(com_arrow $1 $2 $3 $4 $a $b)>> 10
	     [<hov 0> "Com_arrow " <$a:"term":L> " " <$b:"term":L>].

Syntactic Definition Build_Com_arrow := build_Com_arrow | 4.
Syntax constr build_Com_arrow <<(build_Com_arrow $1 $2 $3 $4 $a $b $f $l)>> 10
	     [<hov 0> "Build_Com_arrow " <$a:"term":L> " " <$b:"term":L> " " 
                                         <$f:"term":L> " " <$l:"term":L>].

Syntactic Definition Mor_com_arrow := mor_com_arrow | 6.
Syntax constr mor_com_arrow <<(mor_com_arrow $1 $2 $3 $4 $5 $6 $co)>> 10
	     [<hov 0> "Mor_com_arrow " <$co:"term":L>].

Syntactic Definition Comma := comma | 2.
Syntax constr comma <<(comma $1 $2 $G $x)>> 10
	     [<hov 0> "Comma " <$G:"term":L> " " <$x:"term":L>].











(* $Id: Comma.v,v 1.2 1996/02/12 15:24:54 cpaulin Exp $ *)
