(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Dual Functor                                   *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export Dual.
Require Export Functor.


Section dual_functor.

Variable A, B : Category.
Variable F    : (Functor A B).

Definition DFunctor_ob := [a:(Ob (Dual A))](FOb F a).

Definition DFunctor_map := [b,a:(Ob A)](FMap ? ? F a b).

Lemma DFunctor_comp_law : (fcomp_law (Dual A) (Dual B) DFunctor_ob DFunctor_map).
Proof.
Unfold fcomp_law; Simpl.
Intros a b c f g.
Exact (Prf_Fcomp_law ? ? F ? ? ? g f).  
Qed.

Lemma DFunctor_id_law : (fid_law (Dual A) (Dual B) DFunctor_ob DFunctor_map).
Proof.
Unfold fid_law; Simpl.
Intro a; Exact (Prf_Fid_law ? ? F a).     
Qed.

Definition dfunctor := (Build_Functor (Dual A) (Dual B) DFunctor_ob DFunctor_map 
                                      DFunctor_comp_law DFunctor_id_law).

End dual_functor.

Syntactic Definition DFunctor := dfunctor | 2.
Syntax constr dfunctor <<(dfunctor $1 $2 $F)>> 10
     [<hov 0> "DFunctor " <$F:"Term":L>].




(* $Id: Dual_Functor.v,v 1.2 1996/02/12 15:24:58 cpaulin Exp $ *)
