(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                   Full SubCategory : Inclusion functor		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export FullSubCat.
Require Export FunctorProperty.

(* Inclusion Functor *)

Section fsc_inc_def.

Variable C : Category.
Variable I : Type.
Variable a : I -> (Ob C).

Definition Inc_ob := [i:I](a i).

Definition Inc_map := [i,j:I](Id_map (Hom (a i) (a j))).

Lemma Inc_comp_law : (fcomp_law (FullSubCat a) C Inc_ob Inc_map).
Proof.
Unfold fcomp_law; Simpl.
Intros i1 i2 i3 f g; Apply (Prf_refl (Hom (a i1) (a i3))).
Qed.

Lemma Inc_id_law : (fid_law (FullSubCat a) C Inc_ob Inc_map).
Proof.
Unfold fid_law; Simpl.
Intros i; Apply (Prf_refl (Hom (a i) (a i))).
Qed.

Definition inc := (Build_Functor (FullSubCat a) C Inc_ob Inc_map 
                                 Inc_comp_law Inc_id_law).

(* proprie'te's de Inc *)

Lemma Inc_faith : (IsFaithful inc).
Proof.
Apply Build_isFaithful; Unfold Faithful_law; Simpl.
Unfold fMor; Simpl; Auto.
Qed.

Lemma Inc_full : (IsFull inc).
Proof.
Apply Build_isFull with ([i,j:I](Id_fun (Hom (a i) (a j)))).
Unfold Full_law; Simpl; Intros i j f.
Apply (Prf_refl (Hom (a i) ( a j))).
Qed.

End fsc_inc_def.

Syntactic Definition Inc := inc | 2.
Syntax constr inc <<(inc $1 $2 $a)>> 10
	     [<hov 0> "Inc " <$a:"term":L>].


(* $Id: FSC_inc.v,v 1.2 1996/02/12 15:25:00 cpaulin Exp $ *)
