(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	            Functor Free Monoid : SET -> MON                         *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export FreeMonoid.
Require Export Functor.
Require Export MON.

(* SET -> MON *)

Definition FunFreeMon_ob  := [A:Setoid](FreeMonoid A).

Section funfreemon_map_def.

Variable A, B : Setoid.

 Section funfreemon_mor_def.

 Variable f    : (Map A B).

 Fixpoint Dist_fun [l:(Tlist A)] : (Tlist B) :=
            <(Tlist B)>Case l of 
                       (* Empty   *) (Empty B)
                       (* Concat1 *) [a:?][l1:?]
                                     (Concat1 (Ap f a) (Dist_fun l1))     
                       end.

 Lemma Dist_map_law : (map_law (Tlist_setoid A) (Tlist_setoid B) Dist_fun).
 Proof.
 Unfold map_law; Intros l1; Elim l1.
 Intro l2; Elim l2.
 Intros; Apply (Prf_refl (Tlist_setoid B)).
 Intros c t H H1; Elim H1.
 Intros c t H l2; Elim l2.
 Intro H1; Absurd (Equal_Tlist A (Concat1 c t) (Empty A)).
 Apply (Diff_Concat1_Empty A).
 Trivial.
 Intros c0 t0 H1 H2.
 Elim H2; Intros H3 H4.
 Simpl; Split.
 Apply (Pres ? ? f); Trivial.
 Exact (H t0 H4).
 Qed.

 Definition Dist_map := (build_Map (Tlist_setoid A) (Tlist_setoid B) Dist_fun
                                   Dist_map_law).

 Lemma Dist_map_unit_law : (monUnit_law (FreeMonoid A) (FreeMonoid B) Dist_map).
 Proof.
 Unfold monUnit_law; Simpl; Trivial.
 Qed.

 Lemma Dist_map_op_law : (monOp_law (FreeMonoid A) (FreeMonoid B) Dist_map).
 Proof.
 Unfold monOp_law; Simpl.
 Intros l1 l2; Elim l1.
 Unfold ApMop; Simpl; Exact (Prf_refl (Tlist_setoid B) (Dist_fun l2)).
 Unfold ApMop; Simpl; Intros; Split.
 Apply (Pres ? ? f); Apply (Prf_refl A).
 Trivial.
 Qed.

 Definition FunFreeMon_mor := (build_MonMor (FreeMonoid A) (FreeMonoid B) 
                                Dist_map Dist_map_unit_law Dist_map_op_law).

 End funfreemon_mor_def.

Lemma FunFreeMon_map_law : (map_law (A=>B) 
                               (MonMor_setoid (FreeMonoid A) (FreeMonoid B))
                               FunFreeMon_mor).
Proof.
Unfold map_law; Simpl.
Intros f g; Unfold Equal_MonMor; Simpl.
Unfold Ext; Simpl; Intro H; Intro l; Elim l; Simpl.
Trivial.
Intros c t H1; Split.
Apply H.
Trivial.
Qed.

Definition FunFreeMon_map := (build_Map (A=>B) 
                               (MonMor_setoid (FreeMonoid A) (FreeMonoid B))
                               FunFreeMon_mor FunFreeMon_map_law).

End funfreemon_map_def.

Lemma FunFreeMon_fcomp_law : (fcomp_law SET MON FunFreeMon_ob FunFreeMon_map).
Proof.
Unfold fcomp_law; Simpl.
Intros A B C f g; Unfold Equal_MonMor; Simpl.
Unfold Ext; Simpl; Intro l; Elim l; Simpl.
Trivial.
Intros c t H; Split.
Apply (Prf_refl C).
Trivial.
Qed.

Lemma FunFreeMon_fid_law : (fid_law SET MON FunFreeMon_ob FunFreeMon_map).
Proof.
Unfold fid_law; Simpl.
Intro A; Unfold Equal_MonMor; Simpl.
Unfold Ext; Simpl; Intro l; Elim l; Simpl.
Trivial.
Intros c t H; Split.
Apply (Prf_refl A).
Trivial.
Qed.

Definition FunFreeMon := (Build_Functor SET MON FunFreeMon_ob FunFreeMon_map
                                   FunFreeMon_fcomp_law FunFreeMon_fid_law).


(* $Id: FunFreeMon.v,v 1.2 1996/02/12 15:25:04 cpaulin Exp $ *)
