(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Functors                                       *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export Hom_Equality.

(* Functors *) 

Section Functors.

Variable C, D : Category.
Variable FOb  : (Ob C) -> (Ob D).
Variable FMap : (a,b:(Ob C))(Map (Hom a b) (Hom (FOb a) (FOb b))).

Definition fcomp_law := (a,b,c:(Ob C))(f:|(Hom a b)|)(g:|(Hom b c)|)
   (Ap (FMap a c) (f o g)) =%S ((Ap (FMap a b) f) o (Ap (FMap b c) g)).

Definition fid_law := (a:(Ob C))(Ap (FMap a a) (Id a)) =%S  (Id (FOb a)).

End Functors.

Syntactic Definition Fcomp_law := fcomp_law | 2.
Syntax constr fcomp_law <<(fcomp_law $1 $2 $FO $FM)>> 10
  [<hov 0> "Fcomp_law " <$FO:"Term":L> " " <$FM:"Term":L>].

Syntactic Definition Fid_law := fid_law | 2.
Syntax constr fid_law <<(fid_law $1 $2 $FO $FM)>> 10
  [<hov 0> "Fid_law " <$FO:"Term":L> " " <$FM:"Term":L>].

Structure Functor [C,D:Category] : Type := 
 {fOb           : (Ob C) -> (Ob D);
  FMap          : (a,b:(Ob C))(Map (Hom a b) (Hom (fOb a) (fOb b)));
  Prf_Fcomp_law : (Fcomp_law fOb FMap);
  Prf_Fid_law   : (Fid_law  fOb FMap)}.

Definition fMor := [C,D:Category][a,b:(Ob C)]
      [F:(Functor C D)][f:|(Hom a b)|](Ap (FMap ? ? F a b) f).

Syntactic Definition FOb := fOb | 2.
Syntax constr fOb <<(fOb $1 $2 $F $a)>> 10
		[<hov 0> "FOb " <$F:"Term":L> " " <$a:"Term":L>].

Syntactic Definition FMor := fMor | 4.
Syntax constr fMor <<(fMor $1 $2 $3 $4 $F $f)>> 10
		[<hov 0> "FMor " <$F:"Term":L> " " <$f:"Term":L>].

Lemma Prf_FMap_law : (C,D:Category)(F:(Functor C D))(a,b:(Ob C))
                     (Map_law (Ap (FMap ? ? F a b))).
Proof.
Intros C D F a b; Exact (Pres ? ? (FMap C D F a b)).
Qed.

(* Functors equality *)

Definition Equal_Functor := [C,D:Category][F,G:(Functor C D)]
                            (a,b:(Ob C))(f:|(Hom a b)|)
                            (FMor F f) =%H (FMor G f).

Token "=%F".

Syntax constr Equal_Functor <<(Equal_Functor $1 $2 $c1 $c2)>> 1
     [<hov 0>     <$c1:"Command":L> " =%F "  <$c2:"Command":L>].

Grammar command command1 := [ command0($F1) "=%F" command0($F2) ] -> 
                            [$0 = <<(Equal_Functor ? ? $F1 $F2)>>].


(* composition of two functors *)

Section Comp_F.

Variable C, D, E : Category.
Variable G       : (Functor C D).
Variable H       : (Functor D E).

Definition comp_FOb := [a:(Ob C)](FOb H (FOb G a)).

 Section comp_functor_map.

 Variable a, b : (Ob C).

 Definition comp_FMor := [f:|(Hom a b)|](FMor H (FMor G f)). 

 Lemma Comp_FMap_law : (Map_law comp_FMor).
 Proof.
 Unfold map_law comp_FMor fMor; Intros f g eqfg.
 Apply (Prf_FMap_law D E H (FOb G a) (FOb G b)).
 Apply (Prf_FMap_law C D G a b); Assumption.
 Qed.

 Definition Comp_FMap := (Build_Map comp_FMor Comp_FMap_law).

 End comp_functor_map.

Lemma Comp_Functor_comp_law : (Fcomp_law comp_FOb Comp_FMap).
Proof.
Unfold fcomp_law comp_FOb Comp_FMap comp_FMor.
Simpl; Intros a b c f g.
(* H(G(fog)) = H(G(f)))o(H(G(g)) *) 
(* *) Apply (Prf_trans (Hom (FOb H (FOb G a)) (FOb H (FOb G c))))
 with y:=(FMor H ((FMor G f) o (FMor G g))).
(* H(G(fog)) = H(G(f)oG(g)) *)
Unfold fMor; Apply (Prf_FMap_law D E H (FOb G a) (FOb G c)).
(* G(fog) = G(f)oG(g) *)
Apply (Prf_Fcomp_law C D G).
(* H(G(f)o(G g)) = H(G(f))oH(G(g)) *)
Unfold fMor; Apply (Prf_Fcomp_law D E H).
Qed.

Lemma Comp_Functor_id_law : (Fid_law comp_FOb Comp_FMap).
Proof.
Unfold fid_law comp_FOb Comp_FMap comp_FMor.
Simpl; Intro a.
(* H(G(Id(a))) = Id(H(G(a))) *)
(* *) Apply (Prf_trans (Hom (FOb H (FOb G a)) (FOb H (FOb G a))))
   with y:= (FMor H (Id (FOb G a))).
(* H(G(Id(a))) = H(Id(G(a))) *)
Unfold fMor; Apply (Prf_FMap_law D E H (FOb G a) (FOb G a)).
(* G(Id(a)) = Id(G(a)) *)
Apply (Prf_Fid_law C D G).
(* H(Id(G(a))) = Id(H(G(a))) *)
Unfold fMor; Apply (Prf_Fid_law D E H).
Qed.               

Definition Comp_Functor := (Build_Functor C E comp_FOb Comp_FMap 
                                          Comp_Functor_comp_law Comp_Functor_id_law).

End Comp_F.

Syntactic Definition Comp_FOb := comp_FOb | 3.
Syntax constr comp_FOb <<(comp_FOb $1 $2 $3 $F $G)>> 10
	[<hov 0> "Comp_FOb " <$F:"Term":L> " " <$G:"Term":L>].

Syntactic Definition Comp_FMor := comp_FMor | 3.
Syntax constr comp_FMor <<(comp_FMor $1 $2 $3 $F $G)>> 10
	[<hov 0> "Comp_FMor " <$F:"Term":L> " " <$G:"Term":L>].

Token "%F".

Syntax constr Comp_Functor <<(Comp_Functor $1 $2 $3 $F $G)>> 2
		[<hov 0> <$F:"Term":L> [<hov 0> " o" "%F " <$G:"Term":L>]].

Grammar command command2 :=
  [ command1($F) "o" "%F" command2($G) ] -> 
        [$0 = <<(Comp_Functor ? ? ? $F $G)>>].

(* $Id: Functor.v,v 1.2 1996/02/12 15:25:06 cpaulin Exp $ *)
