(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Functor exercises                              *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export Functor.
Require Export CatProperty.

(* exersise *)

Section functor_prop.

Variable C, D : Category. 
Variable F    : (Functor C D).

(* exercises *)

Lemma Functor_preserves_iso : (a,b:(Ob C))(Iso a b) -> (Iso (FOb F a) (FOb F b)).
Proof.
Intros a b H.
Elim H; Intros f H1.
Elim H1; Intros g H2 H2'.
Apply Build_iso with (FMor F f).
Apply Build_isIso with (FMor F g); Unfold Iso_law.
(* *) Apply (Prf_trans (Hom (FOb F b) (FOb F b))) with y:=(FMor F (g o f)).
Apply (Prf_sym (Hom (FOb F b) (FOb F b))).
Unfold fMor; Apply (Prf_Fcomp_law C D F). 
(* *) Apply (Prf_trans  (Hom (FOb F b) (FOb F b))) with y:=(FMor F (Id b)).
Unfold fMor; Apply (Prf_FMap_law C D F b b); Assumption.
Unfold fMor; Apply (Prf_Fid_law C D F). 
(* *) Apply (Prf_trans (Hom (FOb F a) (FOb F a))) with y:=(FMor F (f o g)).
Apply (Prf_sym (Hom (FOb F a) (FOb F a))).
Unfold fMor; Apply (Prf_Fcomp_law C D F). 
(* *) Apply (Prf_trans  (Hom (FOb F a) (FOb F a))) with y:=(FMor F (Id a)).
Unfold fMor; Apply (Prf_FMap_law C D F a a); Assumption.
Unfold fMor; Apply (Prf_Fid_law C D F). 
Qed.

(* faithful *)

Definition Faithful_law := (a,b:(Ob C))(f,g:|(Hom a b)|)
                           ((FMor F f) =%S (FMor F g)) -> (f =%S g).

Structure isFaithful : Type :=
 {Prf_faithful : Faithful_law}.

(* full *)

Definition Full_law := [H:(a,b:(Ob C))|(Hom (FOb F a) (FOb F b))| -> |(Hom a b)|]
                       (a,b:(Ob C))(h:|(Hom (FOb F a) (FOb F b))|)
                       h =%S (FMor F (H a b h)).

Structure isFull : Type :=
 {Full_mor : (a,b:(Ob C))|(Hom (FOb F a) (FOb F b))| -> |(Hom a b)|;
  Prf_full : (Full_law Full_mor) }.

End functor_prop.

Syntactic Definition IsFaithful := isFaithful | 2.
Syntax constr isFaithful <<(isFaithful $1 $2 $F)>> 10
	     [<hov 0> "IsFaithful " <$F:"term":L>].

Syntactic Definition IsFull := isFull | 2.
Syntax constr isFull <<(isFull $1 $2 $F)>> 10
	     [<hov 0> "IsFull " <$F:"term":L>].

Structure Full [C,D:Category] : Type :=
 {Full_functor : (Functor C D);
  Prf_isFull   : (IsFull Full_functor)}.

Structure Faithful [C,D:Category] : Type :=
 {Faithful_functor : (Functor C D);
  Prf_isFaithful   : (IsFaithful Faithful_functor)}.

Section comp_functor_prop.

Variable C, D, E : Category.
Variable F       : (Functor C D).
Variable G       : (Functor D E).

Lemma IsFaithful_comp : (IsFaithful F) -> (IsFaithful G) -> (IsFaithful (F o%F G)).
Proof.
Intros H1 H2.
Elim H1; Intro H12.
Elim H2; Intro H22.
Apply Build_isFaithful.
Unfold Comp_Functor fMor; Simpl.
Unfold comp_FMor.
Unfold Faithful_law; Auto.
Qed.

Lemma IsFull_comp : (IsFull F) -> (IsFull G) -> (IsFull (F o%F G)).
Proof.
Intros H1 H2.
Elim H1; Unfold Full_law; Intros H1f H3.
Elim H2; Unfold Full_law; Intros H2f H4.
Unfold Comp_Functor fMor; Simpl.
Unfold comp_FOb.
Apply Build_isFull with 
       [a,b:(Ob C)]
       [h:|(Hom (FOb G (FOb F a)) (FOb G (FOb F b)))|]
       (H1f ? ? (H2f ? ? h)).
Unfold Full_law; Intros a b h.
Unfold Comp_Functor fMor; Simpl.
Unfold comp_FMor.
(* *) Apply (Prf_trans (Hom (FOb G (FOb F a)) (FOb G (FOb F b))))
 with y:=(FMor G (H2f (FOb F a) (FOb F b) h)).
Auto.
Unfold fMor; Apply (Prf_FMap_law ? ? G (FOb F a) (FOb F b)).
Apply (H3 ? ? (H2f (FOb F a) (FOb F b) h)).
Qed.

End comp_functor_prop.



(* $Id: FunctorProperty.v,v 1.2 1996/02/12 15:25:09 cpaulin Exp $ *)
