(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Hom Functors                                   *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export SET.
Require Export Functor.
Require Export Dual.

(* 1. Hom-Functor : C(a,-) : C --> SET *)

Section funset.

Variable C : Category.
Variable a : (Ob C).

(* to each object b of C, we associate Hom(a,b), a object of SET *)
Definition FunSET_ob := [b:(Ob C)](Hom a b).

 Section funset_map_def.

 Variable b, c : (Ob C).

  Section funset_mor_def.

  Variable f : |(Hom b c)|.
 
  Definition FunSET_mor1 := [g:|(Hom a b)|] (g o f).

 
  Lemma FunSET_map_law1 : (Map_law FunSET_mor1).
  Proof.
  Unfold map_law FunSET_mor1.
  Intros g h H.
  Apply (Prf_congr C); Assumption.
  Qed.

  Definition FunSET_mor := (Build_Map FunSET_mor1 FunSET_map_law1).

  End funset_mor_def.

 Lemma FunSET_map_law : (map_law (Hom b c) (hom SET (FunSET_ob b) (FunSET_ob c)) FunSET_mor).
 Proof.
 Unfold map_law; Simpl.
 Unfold Ext; Simpl.
 Unfold FunSET_mor1 FunSET_ob.
 Intros f g H h.
 Apply (Prf_congl C); Assumption.
 Qed.

 Definition FunSET_map := (build_Map (Hom b c) (hom SET (FunSET_ob b) (FunSET_ob c))
                                     FunSET_mor FunSET_map_law).

 End funset_map_def.

Lemma Fun_comp_law : (fcomp_law C SET FunSET_ob FunSET_map).
Proof.
Unfold fcomp_law; Simpl.
Unfold Ext; Simpl.
Unfold Comp_fun; Simpl.
Unfold FunSET_mor1 FunSET_ob.
Intros b c d f g h.
Unfold Comp; Apply (Prf_ass C).
Qed.

Lemma Fun_id_law : (fid_law C SET FunSET_ob FunSET_map).
Proof.
Unfold fid_law; Simpl.
Unfold Ext; Simpl.
Unfold Id_fun.
Unfold FunSET_mor1 FunSET_ob.
Intros b f; Apply (Prf_sym (Hom a b)); Unfold Comp; Apply (Prf_idr C).
Qed.

Definition funSET := (Build_Functor C SET FunSET_ob FunSET_map Fun_comp_law Fun_id_law).

End funset.


Syntactic Definition FunSET := funSET | 1.
Syntax constr funSET <<(funSET $1 $c)>> 10
                     [<hov 0> "FunSET " <$c:"Term":L> ].



(* $Id: HomFunctor.v,v 1.2 1996/02/12 15:25:11 cpaulin Exp $ *)
