(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Product : Projection Functors                           *)
(*                                                                           *)
(*          Amokrane Saibi,  May 1994                                        *)
(*                                                                           *)
(*****************************************************************************)

Require Export PROD.
Require Export Functor. 


(* Projection Functors *)
Section prod_proj.

Variable A, B : Category.

(* Fst *)

Lemma Fst_map_law : (a,b:(Ob (PROD A B)))
        (map_law (hom (PROD A B) a b) (Hom (Ob_l a) (Ob_l b)) (hom_l A B)).
Proof.
Unfold map_law.
Intros a b; Unfold 1 Equal; Simpl.
Unfold Equal_Pmor; Intros f g H.
Elim H; Intros; Assumption.
Qed.

Definition Fst_map := [a,b:(Ob (PROD A B))]
     (build_Map (hom (PROD A B) a b) (Hom (Ob_l a) (Ob_l b))
                (hom_l A B) (Fst_map_law a b)).

Lemma Fst_comp_law : (fcomp_law (PROD A B) A (ob_l A B) Fst_map).
Proof.
Unfold fcomp_law; Simpl.
Intros a b c f g.
Apply (Prf_refl (Hom (Ob_l a) (Ob_l c))).
Qed.

Lemma Fst_id_law : (fid_law (PROD A B) A (ob_l A B) Fst_map).
Proof.
Unfold fid_law; Simpl.
Intro a; Apply (Prf_refl (Hom (Ob_l a) (Ob_l a))).
Qed.
 
Definition Fst := (Build_Functor (PROD A B) A (ob_l A B) Fst_map Fst_comp_law Fst_id_law).

(* Snd *)

Lemma Snd_map_law : (a,b:(Ob (PROD A B)))(map_law (hom (PROD A B) a b) (Hom (Ob_r a) (Ob_r b)) 
                                            (hom_r A B)).
Proof.
Unfold map_law.
Intros a b; Unfold 1 Equal; Simpl.
Unfold Equal_Pmor; Intros f g H.
Elim H; Intros; Assumption.
Qed.

Definition Snd_map := [a,b:(Ob (PROD A B))]
     (build_Map (hom (PROD A B) a b) (Hom (Ob_r a) (Ob_r b))
                (hom_r A B) (Snd_map_law a b)).

Lemma Snd_comp_law : (fcomp_law (PROD A B) B (ob_r A B) Snd_map).
Proof.
Unfold fcomp_law; Simpl.
Intros a b c f g.
Apply (Prf_refl (Hom (Ob_r a) (Ob_r c))).
Qed.

Lemma Snd_id_law : (fid_law (PROD A B) B (ob_r A B) Snd_map).
Proof.
Unfold fid_law; Simpl.
Intro a; Apply (Prf_refl (Hom (Ob_r a) (Ob_r a))).
Qed.
 
Definition Snd := (Build_Functor (PROD A B) B (ob_r A B) Snd_map Snd_comp_law Snd_id_law).

End prod_proj.

(* $Id: PROD_proj.v,v 1.2 1996/02/12 15:25:15 cpaulin Exp $ *)
