(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                      CoLimits                  		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Delta.
Require Export UniversalArrow.
Require Export Limit.
Require Export Dual_Functor.

Section def_cocone.

Variable J,C : Category.
Variable F   : (Functor J C).
Variable c   : (Ob C).

Definition coCone := (NT F (Const J C c)).

Definition coCones := {F => (Const J C c)}.

 Section cocone_nt. 

 Variable T : (a:(Ob J))|(Hom (FOb F a) (FOb (Const J C c) a))|.

 Definition coCone_law := (i,j:(Ob J))(g:|(Hom i j)|)((FMor F g) o (T j)) =%S (T i).

 (* comment construire un coCone *)

 Lemma Prf_CoCone_nt_law : coCone_law -> (NT_law F (Const J C c) T).
 Proof.
 Unfold nt_law; Intros H a b f.
 Unfold 2 fMor; Simpl; Unfold Const_mor.
 (* *) Apply (Prf_trans (Hom (FOb F a) c)) with 
   y:=(T a).
 Apply (H a b).
 Unfold Comp; Apply (Prf_idr C).
 Qed.

 Definition build_CoCone := [p:coCone_law](Build_NT F (Const J C c) T (Prf_CoCone_nt_law p)).

 End cocone_nt.

Lemma Eq_coCone : (tau:coCone)(coCone_law (ApNT tau)).
Proof.
Unfold coCone_law; Intros tau i j g.
(* *) Apply (Prf_trans (Hom (FOb F i) c)) with 
  y:= (ApNT tau i) o (Id c).
Apply (Prf_NT_law ? ? ? ? tau i j g).
Apply (Prf_sym (Hom (FOb F i) c)).
Unfold Comp; Apply (Prf_idr C).
Qed.

End def_cocone.


Syntactic Definition CoCone := coCone | 2.
Syntax constr coCone <<(coCone $1 $2 $F $c)>> 10
	     [<hov 0> "CoCone " <$F:"term":L> " " <$c:"term":L>].

Syntactic Definition CoCones := coCones | 2.
Syntax constr coCones <<(coCones $1 $2 $F $c)>> 10
	     [<hov 0> "CoCones " <$F:"term":L> " " <$c:"term":L>].

Syntactic Definition CoCone_law := coCone_law | 2.
Syntax constr coCone_law <<(coCone_law $1 $2 $F $c $t)>> 10
       [<hov 0> "CoCone_law " <$F:"term":L> " " <$c:"term":L> " " <$t:"term":L>].

Syntactic Definition Build_CoCone := build_CoCone | 2.
Syntax constr build_CoCone <<(build_CoCone $1 $2 $F $c $t $p)>> 10
       [<hov 0> "Build_CoCone " <$F:"term":L> " " <$c:"term":L> " " 
                                <$t:"term":L> " " <$p:"term":L>].
		

Section def_colimit.

Variable J,C : Category.
Variable F   : (Functor J C).

(* Colimites *)

Definition Colimit' := (Limit (DFunctor F)).

Definition colimit := (ua C (CatFunct J C) F (Delta C J)).

Variable cl : colimit.

Definition colim := (UA_ob cl).

Definition limiting_coCone : (CoCone F colim) := (UA_mor cl).

(* tau(i) = mu(i) o tau# *)

Lemma Eq_colimit : (c:(Ob C))(tau:(CoCone F c))(i:(Ob J))
      (ApNT tau i) =%S ((ApNT limiting_coCone i) o (UA_diese cl c tau)).
Proof.
Intros c tau i.
Apply (Prf_sym (Hom (FOb F i) c)).
Apply (Prf_UA_law1 ? ? ? ? cl ? tau i).
Qed.
 
Variable a : (Ob C).
Variable u : (CoCone F a).
 
Definition isColimit := (isUA C (CatFunct J C) F (Delta C J) a u).

Variable diese : (c:(Ob C))(CoCone F c) -> |(Hom a c)|.

Definition colimit_law1 := (ua_law1 C (CatFunct J C) F (Delta C J) a u diese).

Definition colimit_law2 := (ua_law2 C (CatFunct J C) F (Delta C J) a u diese).

Variable ll1 : colimit_law1.
Variable ll2 : colimit_law2.

Definition build_IsColimit := (Build_isUA C (CatFunct J C) F (Delta C J) a u 
                                          diese ll1 ll2).

Variable p : isColimit.

Definition build_Colimit := (Build_ua C (CatFunct J C) F (Delta C J) a u p).

End def_colimit.

Syntactic Definition Colimit := colimit | 2.
Syntax constr colimit <<(colimit $1 $2 $F)>> 10
	     [<hov 0> "Colimit " <$F:"term":L>].

Syntactic Definition Colim := colim | 3.
Syntax constr colim <<(colim $1 $2 $3 $cl)>> 10
	     [<hov 0> "Colim " <$cl:"term":L>].

Syntactic Definition Limiting_coCone := limiting_coCone | 3.
Syntax constr limiting_coCone <<(limiting_coCone $1 $2 $3 $l)>> 10
	     [<hov 0> "Limiting_coCone " <$cl:"term":L>].

Syntactic Definition IsColimit := isColimit | 2.
Syntax constr isColimit <<(isColimit $1 $2 $F $c $u)>> 10
       [<hov 0> "IsColimit " <$F:"term":L> " " <$c:"term":L> " " <$u:"term":L>].

Syntactic Definition Colimit_law1 := colimit_law1 | 2.
Syntax constr colimit_law1 <<(colimit_law1 $1 $2 $F $a $u $d)>> 10
	     [<hov 0> "Colimit_law1 " <$F:"term":L> " " <$a:"term":L> 
                                    " " <$u:"term":L> " " <$d:"term":L>].

Syntactic Definition Colimit_law2 := colimit_law2 | 2.
Syntax constr colimit_law2 <<(colimit_law2 $1 $2 $F $a $u $d)>> 10
	     [<hov 0> "Colimit_law2 " <$F:"term":L> " " <$a:"term":L> 
                                    " " <$u:"term":L> " " <$d:"term":L>].

Syntactic Definition Build_IsColimit := build_IsColimit | 2.
Syntax constr build_IsColimit <<(build_IsColimit $1 $2 $F $a $u $d $l1 $l2)>> 10
     [<hov 0> "Build_IsColimit " <$F:"term":L> " " <$a:"term":L> " " 
      <$u:"term":L> " " <$d:"term":L> " " <$l1:"term":L> " " <$l2:"term":L>].

Syntactic Definition Build_Colimit := build_Colimit | 2.
Syntax constr build_Colimit <<(build_Colimit $1 $2 $F $a $u $p)>> 10
     [<hov 0> "Build_Colimit " <$F:"term":L> " " <$a:"term":L> " " 
              <$u:"term":L> " " <$p:"term":L>].






(* $Id: CoLimit.v,v 1.2 1996/02/12 15:25:23 cpaulin Exp $ *)
