(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                CoUniversal Arrows               		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                            A.SAIBI   May 95				     *)
(*									     *)
(*****************************************************************************)


Require Export UniversalArrow.
Require Export Dual_Functor.


(* coUA *)

Section co_univarr.

Variable A, B : Category.
Variable b : (Ob B).
Variable F : (Functor A B).

Variable a : (Ob A).
Variable u : |(Hom (FOb F a) b)|.

Variable co_diese : (a':(Ob A))|(Hom (FOb F a') b)| -> |(Hom a' a)|.

Definition coUA_eq := [a':(Ob A)][f:|(Hom (FOb F a') b)|]
                      [g:|(Hom a' a)|] ((FMor F g) o u) =%S f.

Definition coUA_law1 := (a':(Ob A))(f:|(Hom (FOb F a') b)|)
                        (coUA_eq ? f (co_diese ? f)).

Definition coUA_law2 := (a':(Ob A))(f:|(Hom (FOb F a') b)|)(g:|(Hom a' a)|)
                        (coUA_eq a' f g) -> g =%S (co_diese ? f).

End co_univarr.


Syntactic Definition CoUA_eq := coUA_eq | 5.
Syntax constr coUA_eq <<(coUA_eq $1 $2 $3 $4 $5 $u $a' $f' $g)>> 10
	     [<hov 0> "CoUA_eq " <$u:"term":L> " " <$a':"term":L> " " <$f':"term":L> 
                               " " <$g:"term":L>].

Syntactic Definition CoUA_law1 := coUA_law1 | 5.
Syntax constr coUA_law1 <<(coUA_law1 $1 $2 $3 $4 $5 $u $d)>> 10
	     [<hov 0> "CoUA_law1 " <$u:"term":L> " " <$d:"term":L>].

Syntactic Definition CoUA_law2 := coUA_law2 | 5.
Syntax constr coUA_law2 <<(coUA_law2 $1 $2 $3 $4 $5 $u $d)>> 10
	     [<hov 0> "CoUA_law2 " <$u:"term":L> " " <$d:"term":L>].

Structure isCoUA [A,B:Category;b:(Ob B);F:(Functor A B);
                  a:(Ob A);u:|(Hom (FOb F a) b)|] : Type :=
 {isCoUA_diese    : (a':(Ob A))|(Hom (FOb F a') b)|->|(Hom a' a)|;
  Prf_isCoUA_law1 : (CoUA_law1 u isCoUA_diese);
  Prf_isCoUA_law2 : (CoUA_law2 u isCoUA_diese)}.

Syntactic Definition IsCoUA := isCoUA | 2.
Syntax constr isCoUA <<(isCoUA $1 $2 $b $F $a $u)>> 10
	     [<hov 0> "IsCoUA " <$b:"term":L> " " <$F:"term":L>
                      " " <$a:"term":L> " " <$u:"term":L> ].

Syntactic Definition IsCoUA_diese := isCoUA_diese | 6.
Syntax constr isCoUA_diese <<(isCoUA_diese $1 $2 $3 $4 $5 $6 $t $a' $f )>> 10
       [<hov 0> "IsCoUA_diese " <$t:"term":L> " " <$a':"term":L> " " <$f:"term":L>].

Structure coUA [A,B:Category;b:(Ob B);F:(Functor A B)] : Type :=
 {coUA_ob    : (Ob A);
  coUA_mor   : |(Hom (FOb F coUA_ob) b)|;
  prf_IsCoUA : (IsCoUA b F coUA_ob coUA_mor)}.

Syntactic Definition CoUA := coUA | 2.
Syntax constr coUA <<(coUA $1 $2 $b $F)>> 10
	     [<hov 0> "CoUA " <$b:"term":L> " " <$F:"term":L>].

Syntactic Definition CoUA_ob := coUA_ob | 4.
Syntax constr coUA_ob <<(coUA_ob $1 $2 $3 $4 $t1)>> 10
	     [<hov 0> "CoUA_ob " <$t1:"term":L>].

Syntactic Definition CoUA_mor := coUA_mor | 4.
Syntax constr coUA_mor <<(coUA_mor $1 $2 $3 $4 $t1)>> 10
	     [<hov 0> "CoUA_mor " <$t1:"term":L>].

Syntactic Definition Prf_IsCoUA := prf_IsCoUA | 4.
Syntax constr prf_IsCoUA <<(prf_IsCoUA $1 $2 $3 $4 $t1)>> 10
	     [<hov 0> "Prf_IsCoUA " <$t1:"term":L>].

 Section proj_coua.

 Variable A ,B : Category.
 Variable b    : (Ob B).
 Variable F    : (Functor A B).
 Variable u    : (CoUA b F).
               
 Definition coUA_diese := (IsCoUA_diese (Prf_IsCoUA u)). 

 Lemma Prf_CoUA_law1 : (CoUA_law1 (CoUA_mor u) coUA_diese).
 Proof.
 Unfold coUA_diese.
 Exact (Prf_isCoUA_law1 ? ? ? ? ? ? (Prf_IsCoUA u)). 
 Qed.

 Lemma Prf_CoUA_law2 : (CoUA_law2 (CoUA_mor u) coUA_diese).
 Proof.
 Unfold coUA_diese.
 Exact (Prf_isCoUA_law2 ? ? ? ? ? ? (Prf_IsCoUA u)). 
 Qed.

 (* f=g => f# = g# *)

 Lemma Codiese_map : (a':(Ob A))(Map_law (coUA_diese a')).
 Proof.
 Intros a'.
 Unfold map_law; Intros f g H.
 Apply (Prf_CoUA_law2 a' g (coUA_diese a' f)).
 Unfold coUA_eq.
 (* *) Apply (Prf_trans (Hom (FOb F a') b)) with y:=f.
 Apply (Prf_CoUA_law1 a' f).
 Assumption.
 Qed.

 End proj_coua.
          
Syntactic Definition CoUA_diese := coUA_diese | 4.
Syntax constr coUA_diese<<(coUA_diese $1 $2 $3 $4 $u $a' $f)>> 10 
	     [<hov 0> "CoUA_diese " <$u:"term":L> " " <$a':"term":L> " " <$f:"term":L>].

(* 2nd def : par dualite' *)

Section co_univarr1.

Variable A, B :Category.
Variable b    : (Ob B).
Variable F    : (Functor A B).

 Section isco_univarr1.

 Variable a    : (Ob A).
 Variable u    : |(Hom (FOb F a) b)|.

 Definition isCoUA1 := (isUA ? ? b (DFunctor F) a u).
 
 Variable u1 : isCoUA1.

 Definition IsCoUA1_diese :=[a':(Ob A)][f:|(Hom (FOb F a') b)|] 
                            (IsUA_diese u1 ? f).

 Lemma Prf_isCoUA1_law1 : (CoUA_law1 u IsCoUA1_diese).
 Proof.
 Unfold coUA_law1 coUA_eq; Intros a' f.
 Unfold IsCoUA1_diese.
 Exact (Prf_isUA_law1 ? ? ? ? ? ? u1 a' f).
 Qed. 

 Lemma Prf_isCoUA1_law2 : (CoUA_law2 u IsCoUA1_diese).
 Proof.
 Unfold coUA_law2 coUA_eq; Intros a' f g H.
 Unfold IsCoUA1_diese.
 Apply (Prf_isUA_law2 ? ? ? ? ? ? u1 a' f g H).
 Qed.

 Lemma IsCoUA1_to_IsCoUA : (IsCoUA b F a u). 
 Proof.
 Exact (Build_isCoUA ? ? ? ? ? u IsCoUA1_diese Prf_isCoUA1_law1 Prf_isCoUA1_law2).
 Defined.

 End isco_univarr1.

Definition coUA1 := (UA b (DFunctor F)).

Variable u1 : coUA1.

Definition CoUA1_to_CoUA := (Build_coUA ? ? b F (UA_ob u1) (UA_mor u1) 
                              (IsCoUA1_to_IsCoUA ? (UA_mor u1) (Prf_IsUA u1))).


End co_univarr1.

(* $Id: CoUniversalArrow.v,v 1.2 1996/02/12 15:25:25 cpaulin Exp $ *)
