(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	   A Universal Arrow is a Initial Object In Comma Category 	     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Comma.
Require Export UniversalArrow.


Section ua_comma.

Variable A, X : Category.
Variable G    : (Functor A X).
Variable x    : (Ob X).

(* Un UA de x vers G est un obj initial dans (x|G) *)
Variable ua : (UA x G).

Definition UA_com_ob := (Build_Com_ob G x (UA_ob ua) (UA_mor ua)).

 Section ua_com_arrow_def.

 Variable axf:(Ob (Comma G x)).

 Lemma UA_com_law : (Com_law UA_com_ob axf (UA_diese ua ? (Mor_com_ob axf))).
 Proof.
 Unfold com_law.
 Elim axf; Intros a f; Simpl.
 Apply (Prf_sym (Hom x (FOb G a))).
 Apply (Prf_UA_law1 ? ? ? ? ua ? f).
 Qed.

 Definition UA_com_arrow := (Build_Com_arrow UA_com_ob axf (UA_diese ua ? (Mor_com_ob axf))
                                             UA_com_law).

 Lemma UA_com_arrow_unique : (At_most_1mor (Comma G x) UA_com_ob axf).
 Proof.
 Unfold At_most_1mor.
 Intros g h; Simpl.
 Unfold Equal_com_arrow; Elim g; Intros g' p'; Simpl.
 Elim h; Intros h' q'; Simpl.
 (* *) Apply (Prf_trans (Hom (UA_ob ua) (Ob_com_ob axf))) with
   y:=(UA_diese ua ? (Mor_com_ob axf)).
 Apply (Prf_UA_law2 ? ? ? ? ua).
 Unfold ua_eq; Apply (Prf_sym (Hom x (FOb G (Ob_com_ob axf)))).
 Exact p'.
 Apply (Prf_sym (Hom (UA_ob ua) (Ob_com_ob axf))).
 Apply (Prf_UA_law2 ? ? ? ? ua).
 Unfold ua_eq; Apply (Prf_sym (Hom x (FOb G (Ob_com_ob axf)))).
 Exact q'.
 Qed.

 End ua_com_arrow_def.

Lemma UA_isInitial : (isInitial (Comma G x) UA_com_ob).
Proof.
Apply Build_isInitial.
Exact UA_com_arrow.
Exact UA_com_arrow_unique.
Qed.

(* un obj initial dans (x|G) est Un UA de x vers G *)

Variable axu : (Initial (Comma G x)).

Definition Com_diese := [a':(Ob A)][f:|(Hom x (FOb G a'))|]
                        (Mor_com_arrow (MorI (Prf_isInitial ? axu) 
                                       (Build_Com_ob G x a' f))).

Lemma Com_UAlaw1 : (UA_law1 (Mor_com_ob (Initial_ob axu)) Com_diese).
Proof.
Unfold ua_law1 ua_eq; Intros a' f.
Apply (Prf_sym (Hom x (FOb G a'))).
Exact (Prf_com_law ? ? G x ? ? (MorI (Prf_isInitial ? axu) 
                                        (Build_Com_ob G x a' f))).
Qed.

Lemma Com_UAlaw2 : (UA_law2 (Mor_com_ob (Initial_ob axu)) Com_diese).
Proof.
Unfold ua_law2 ua_eq; Intros a' f g H.
Cut (Com_law (Initial_ob axu) (Build_Com_ob G x a' f) g).
Intro H'.
Apply (UniqueI (Comma G x) (Initial_ob axu) (Prf_isInitial ? axu)
        (Build_Com_ob G x a' f) 
        (Build_Com_arrow (Initial_ob axu) (Build_Com_ob G x a' f) g H')
        (MorI (Prf_isInitial ? axu) (Build_Com_ob G x a' f))).
Unfold com_law; Simpl.
Apply (Prf_sym (Hom x (FOb G a'))); Trivial.
Qed.

Lemma Com_isUA : (IsUA x G (Ob_com_ob (Initial_ob axu))
                       (Mor_com_ob (Initial_ob axu))).
Proof.
Apply (Build_isUA ? ? x G (Ob_com_ob (Initial_ob axu))
                       (Mor_com_ob (Initial_ob axu))
                       Com_diese Com_UAlaw1 Com_UAlaw2).
Defined.
         
Lemma Com_UA : (UA x G).
Proof.
Apply (Build_ua ? ? x G (Ob_com_ob (Initial_ob axu))
                       (Mor_com_ob (Initial_ob axu))
                       Com_isUA).
Defined.


End ua_comma.

(* $Id: Comma_UA.v,v 1.2 1996/02/12 15:25:27 cpaulin Exp $ *)
