(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	            Delta Functor (used in Limits)             		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export PROD.
Require Export CatFunct.


(* Const(b) : A->B 
   Const(b)(a) = b 
   Const(b)(f) = Id(b) *)

Section constFun.

Variable A, B : Category.
Variable b    : (Ob B).

Definition Const_ob := [a:(Ob A)]b.

 Section const_map_def.

 Variable a1, a2 : (Ob A).

 Definition Const_mor := [f:|(Hom a1 a2)|](Id b).

 Lemma Const_mor_map_law : (Map_law Const_mor).
 Proof.
 Unfold map_law Const_mor.
 Intros f g H; Apply (Prf_refl (Hom b b)).
 Qed.

 Definition Const_map := (Build_Map Const_mor Const_mor_map_law).

 End const_map_def.

Lemma Const_comp_law : (Fcomp_law Const_ob Const_map).
Proof.
Unfold fcomp_law Const_map Const_mor; Simpl.
Intros a1 a2 a3 f g.
Unfold Comp; Apply (Prf_idr B).
Qed.

Lemma Const_id_law : (Fid_law Const_ob Const_map).
Proof.
Unfold fid_law Const_map Const_mor Const_ob; Simpl.
Intros; Apply (Prf_refl (Hom b b)).
Qed.

Definition Const := (Build_Functor A B Const_ob Const_map
                                   Const_comp_law Const_id_law).

End constFun.

(* Delta : C -> [J=>C]
   Delta(c) = Const(c)
   Delta(f)(i) = f *)


Section delta.

Variable C, J : Category.

Definition Delta_ob := [c:(Ob C)](Const J C c).

 Section delta_map_def.

 Variable c1, c2 : (Ob C).

  Section delta_mor_def.

  Variable f : |(Hom c1 c2)|.
  
  Definition Delta_mor_tau := [i:(Ob J)]f.
                       
  Lemma Delta_mor_tau_nt_law : (NT_law (Const J C c1) (Const J C c2) Delta_mor_tau).
  Proof.
  Unfold nt_law Delta_mor_tau Const Const_map Const_mor fMor.
  Simpl; Intros i j g.
  (* *) Apply (Prf_trans (Hom c1 c2)) with y:=f.
  Unfold Comp; Apply (Prf_idl C).
  Unfold Comp; Apply (Prf_idr C).
  Qed.

  Definition Delta_mor := (Build_NT (Const J C c1) (Const J C c2)  
                                    Delta_mor_tau Delta_mor_tau_nt_law).

  End delta_mor_def.

 Lemma Delta_map_law : (map_law (Hom c1 c2) (hom CatFunct (Delta_ob c1) (Delta_ob c2)) 
                                    Delta_mor).
 Proof.
 Unfold map_law Delta_mor Delta_mor_tau; Simpl.
 Unfold Equal_NT; Simpl; Auto.
 Qed.

 Definition Delta_map := (build_Map (Hom c1 c2) (hom CatFunct (Delta_ob c1) (Delta_ob c2)) 
                                    Delta_mor Delta_map_law).

 End delta_map_def. 

Lemma Delta_comp_law : (fcomp_law C (CatFunct J C) Delta_ob Delta_map).
Proof.
Unfold fcomp_law; Simpl.
Unfold Delta_mor Delta_mor_tau Equal_NT; Simpl.
Unfold Comp_tau; Simpl.
Intros a b c f g i; Apply (Prf_refl (Hom a c)).
Qed.

Lemma Delta_id_law : (fid_law C (CatFunct J C) Delta_ob Delta_map).
Proof.
Unfold fid_law; Simpl.
Unfold Delta_mor Delta_mor_tau Equal_NT; Simpl.
Unfold Id_CatFunct_tau; Simpl.
Unfold Const_ob; Intros a b; Apply (Prf_refl (Hom a a)).
Qed.

Definition Delta := (Build_Functor C (CatFunct J C) Delta_ob Delta_map
                                   Delta_comp_law Delta_id_law).

End delta.




(* $Id: Delta.v,v 1.2 1996/02/12 15:25:29 cpaulin Exp $ *)
