(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	  Discrete category (used in the definition of Products) 	     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Category.

(* Discrete Categories *)

Section discrete.

Variable U : Type.

Inductive Discr_mor : U -> U -> Type :=
 Build_Discr_mor : (a:U)(Discr_mor a a).

 Section disc_mor_setoid.

 Variable a, b : U.

 Definition Equal_Discr_mor := [f,g:(Discr_mor a b)]True.

 Lemma Equal_Discr_mor_equiv : (Equivalence Equal_Discr_mor).
 Proof.
 Unfold Equal_Discr_mor.
 Apply Build_equivalence; Red; Trivial.
 Qed.

 Definition Discr_mor_setoid := (Build_Setoid (Discr_mor a b)
                                       Equal_Discr_mor Equal_Discr_mor_equiv).

 End disc_mor_setoid.

Definition Comp_Discr_mor := [a,b,c:U][f:(Discr_mor a b)] 
 <[d,d0:U][_:(Discr_mor d d0)](Discr_mor d0 c)->(Discr_mor d c)>
 Case f of [x:U][g:(Discr_mor x c)]
           <[d,d0:U][_:(Discr_mor d d0)](Discr_mor d d0)>
           Case g of Build_Discr_mor end end.
   
Lemma Comp_Discr_congl : (Congl_law U Discr_mor_setoid Comp_Discr_mor).
Proof.
Unfold Congl_law; Simpl; Unfold Equal_Discr_mor.
Intros a b c f g h H; Elim H; Trivial.
Qed.

Lemma Comp_Discr_congr : (Congr_law U Discr_mor_setoid Comp_Discr_mor).
Proof.
Unfold Congr_law Comp_Discr_mor; Simpl; Unfold Equal_Discr_mor; Trivial.
Qed.

Definition Comp_Discr := (Build_Comp U Discr_mor_setoid Comp_Discr_mor 
                          Comp_Discr_congl Comp_Discr_congr).

Lemma Assoc_Discr : (assoc_law U Discr_mor_setoid Comp_Discr).
Proof.
Unfold assoc_law; Simpl.
Unfold Comp_Discr Equal_Discr_mor; Trivial.
Qed.

Definition Id_Discr := [a:U](Build_Discr_mor a).

Lemma Idl_Discr : (idl_law U Discr_mor_setoid Comp_Discr Id_Discr).
Proof.
Unfold idl_law; Simpl.
Unfold Comp_Discr Id_Discr Equal_Discr_mor; Trivial.
Qed.

Lemma Idr_Discr : (idr_law U Discr_mor_setoid Comp_Discr Id_Discr).
Proof.
Unfold idr_law; Simpl.
Unfold Comp_Discr Id_Discr Equal_Discr_mor; Trivial.
Qed.

Definition Discr := (Build_Category U Discr_mor_setoid Comp_Discr Id_Discr
                                       Assoc_Discr Idl_Discr Idr_Discr).

End discrete.

Axiom Discr_mor_ind1 : (U:Type)(a:U)
                       (P:(Discr_mor ? a a)->Prop)                          
                       (P (Build_Discr_mor ? a)) ->
                       (x:(Discr_mor ? a a))(P x).







(* $Id: Discr.v,v 1.2 1996/02/12 15:25:30 cpaulin Exp $ *)
