(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                       Equalizers                		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Equalizers.
Require Export PA.
Require Export Limit.

(* PA -> C *)

Section equalizer_limit_def.

Variable C   : Category.
Variable a,b : (Ob C).
Variable I   : Type.
Variable k   : I -> |(Hom a b)|.

Definition FPA_ob := [x:PA_ob]<(Ob C)>Case x of (* PA1 *) a 
                                                (* PA2 *) b end.

Definition FPA_mor := [x,y:PA_ob][f:(PA_mor I x y)]
                      <[x',y':PA_ob]|(Hom (FPA_ob x') (FPA_ob y'))|>
                      Case f of (* PA_I1 *) (Id a) 
                                (* PA_I2 *) (Id b) 
                                (* PA_f  *) [i:I](k i) end.

Lemma FPA_map_law : (x,y:PA_ob)(map_law (PA_mor_setoid ? ? ? ? k x y) ? 
                                        (FPA_mor x y)).
Proof.
Unfold map_law; Simpl.
Intros x y f; Elim f.
(* 1 *)
Simpl; Intros g H; Apply (PA_11_ind I ([x:(PA_mor I PA1 PA1)]
                                        ((Id a) =%S (FPA_mor PA1 PA1 x)))).
Simpl; Apply (Prf_refl (Hom a a)).
(* 2 *)
Simpl; Intros g H; Apply (PA_22_ind I ([x':(PA_mor I PA2 PA2)]
                                        ((Id b) =%S (FPA_mor PA2 PA2 x')))).
Simpl; Apply (Prf_refl (Hom b b)).
(* 3 *)
Intros i g; LApply (PA_12_ind I ([x':(PA_mor I PA1 PA2)]
                            (Equal_PA_mor ? ? ? ? k PA1 PA2 (PA_f I i) x')->
                              (k i) =%S (FPA_mor PA1 PA2 x'))).
Intros H H0; Apply (H g H0).
Simpl; Auto.
Qed.

Definition FPA_map := [x,y:PA_ob](build_Map (PA_mor_setoid ? ? ? ? k x y) 
                                            ? (FPA_mor x y) (FPA_map_law x y)).

Lemma FPA_comp_law : (fcomp_law (PA k) C FPA_ob FPA_map).
Proof.
Unfold fcomp_law; Simpl.
Intros x y z f; Elim f.
(* 1 *)
Simpl; Intro g.
(* *) Apply (Prf_trans (Hom a (FPA_ob z))) with
  y:=(FPA_mor PA1 z g).
Apply (Pres ? ? (FPA_map PA1 z) (Comp_PA_mor I ? ? ? PA_I1 g) g).
Apply (Comp_PA_fact1 ? ? ? ? k PA_I1 ? g).
Apply (Prf_sym (Hom a (FPA_ob z))).
Unfold Comp; Apply (Prf_idl C).
(* 2 *)
Simpl; Intro g.
(* *) Apply (Prf_trans (Hom b (FPA_ob z))) with
  y:=(FPA_mor PA2 z g).
Apply (Pres ? ? (FPA_map PA2 z) (Comp_PA_mor I ? ? ? PA_I2 g) g).
Apply (Comp_PA_fact2 ? ? ? ? k PA_I2 ? g).
Apply (Prf_sym (Hom b (FPA_ob z))).
Unfold Comp; Apply (Prf_idl C).
(* 3 *)
Simpl; Intro i; Elim z; Intro g.
Apply (PA_21_ind I ([x':(PA_mor I PA2 PA1)]
         (FPA_mor PA1 PA1 (Comp_PA_mor I ? ? ? (PA_f I i) g)) =%S 
         ((k i) o (FPA_mor PA2 PA1 g))) g).
Simpl.
Apply (PA_22_ind I ([x:(PA_mor I PA2 PA2)] (k i) =%S ((k i) o (FPA_mor PA2 PA2 x)))).
Simpl; Unfold Comp; Apply (Prf_idr C).       
Qed.

Lemma FPA_id_law : (fid_law (PA k) C FPA_ob FPA_map).
Proof.
Unfold fid_law; Simpl.
Intro x; Elim x.
Simpl; Apply (Prf_refl (Hom a a)).
Simpl; Apply (Prf_refl (Hom b b)).
Qed.

Definition fpa := (Build_Functor (PA k) C FPA_ob FPA_map FPA_comp_law FPA_id_law).

Definition equalizer1 := (Limit fpa).

Variable l : equalizer1.

Definition e1_ob := (Lim l).

Definition e1_mor := (ApNT (Limiting_cone l) PA1).
 
Lemma Prf_E1_law1 : (Equalizer_law1 k e1_ob e1_mor).
Proof.
Unfold equalizer_law1 equalizer_eq e1_mor.
Intros i j.
(* *) Apply (Prf_trans (Hom e1_ob b)) with
  y:=(ApNT (Limiting_cone l) PA2). 
Apply (Prf_sym (Hom e1_ob b)).
Apply (Eq_cone ? C e1_ob fpa (Limiting_cone l) PA1 PA2 (PA_f I i)).
Apply (Eq_cone ? C e1_ob fpa (Limiting_cone l) PA1 PA2 (PA_f I j)).
Qed.

(* construire a` partir de h, un cone r -> fpa *)
(* C'est la` ou` on a besoin que I ne soit pas vide *)

Variable witness : I.

 Section e1_diese_def.

 Variable r   : (Ob C).
 Variable h   : |(Hom r a)|.
 Hypothesis p : (Equalizer_eq k ? h).

 Definition E_tau := [x:(Ob (PA k))]<[x':(Ob (PA k))]|(Hom r (FOb fpa x'))|>
                     Case x of (* PA1 *) h
                               (* PA2 *) (h o (k witness)) end.
 
 Lemma E_tau_cone_law : (Cone_law r fpa E_tau).
 Proof.
 Unfold cone_law; Intros x o2 f.
 Elim f.
 (* 1 *)
 Unfold fMor; Simpl.
 Unfold Comp; Apply (Prf_idr C).
 (* 2 *)
 Unfold fMor; Simpl.
 Unfold Comp; Apply (Prf_idr C).
 (* 3 *)
 Unfold fMor; Simpl.
 Intro i; Apply (p witness i).
 Qed.
 
 Definition E_NT := (Build_Cone r fpa E_tau E_tau_cone_law).

 Definition e1_diese := (CoUA_diese l r E_NT).
 
 End e1_diese_def.


Lemma Prf_E1_law2 : (Equalizer_law2 k e1_ob e1_mor e1_diese).
Proof.
Unfold equalizer_law2 e1_mor equalizer_eq.
Intros r h p.
Apply (Eq_limit ? ? ? l r (E_NT r h p) PA1).
Qed.

Lemma Prf_E1_law3 : (Equalizer_law3 k e1_ob e1_mor e1_diese).
Proof.
Unfold equalizer_law3 e1_mor equalizer_eq.
Intros r h p q H.
Apply (Prf_CoUA_law2 ? ? ? ? l ? (E_NT r h p) q).
Unfold coUA_eq; Simpl.
Unfold Equal_NT; Simpl.
Unfold Comp_tau E_NT; Simpl.
Intro x; Elim x; Simpl.
Apply (Prf_sym (Hom r a)); Apply H.
(* *) Apply (Prf_trans (Hom r b)) with y:=(q o e1_mor) o (k witness).
(* *) Apply (Prf_trans (Hom r b)) with y:=q o (e1_mor o (k witness)).
Apply (Prf_congl C).
Apply (Eq_cone ? C e1_ob fpa (Limiting_cone l)
                PA1 PA2 (PA_f ? witness)).
Unfold Comp; Apply (Prf_ass C).
Apply (Prf_congr C).
Apply (Prf_sym (Hom r a)).
Apply H.
Qed.

Definition Equalizer1_to_Equalizer := (Build_equalizer C a b I k e1_ob e1_mor 
                                        Prf_E1_law1 e1_diese Prf_E1_law2 Prf_E1_law3).

End equalizer_limit_def.
 
Syntactic Definition FPA := fpa | 4.
Syntax constr fpa <<(fpa $1 $2 $3 $4 $k)>> 10
		[<hov 0> "FPA " <$k:"Term":L>].

Syntactic Definition Equalizer1 := equalizer1 | 4.
Syntax constr equalizer1 <<(equalizer $1 $2 $3 $4 $k)>> 10
		[<hov 0> "Equalizer1 " <$k:"Term":L>].

Syntactic Definition E1_ob := e1_ob | 4.
Syntax constr e1_ob <<(e1_ob $1 $2 $3 $4 $k $e)>> 10
		[<hov 0> "E1_ob " <$e:"Term":L> " " <$k:"Term":L>].

Syntactic Definition E1_mor := e1_mor | 4.
Syntax constr e1_mor <<(e1_mor $1 $2 $3 $4 $k $e)>> 10
		[<hov 0> "E1_mor " <$e:"Term":L> " " <$k:"Term":L>].

Syntactic Definition E1_diese := e1_diese | 4.
Syntax constr e1_diese <<(e1_diese $1 $2 $3 $4 $k $e $r $h $p)>> 10
    [<hov 0> "E1_diese " <$k:"Term":L> " " <$e:"Term":L> " " <$r:"Term":L> 
                         " " <$h:"Term":L> " " <$p:"Term":L>].

(* cas particuliers *)

(* de deux morhismes *)

Section equaz_fg.

Variable C    : Category.
Variable a, b : (Ob C).
Variable f, g : |(Hom a b)|.

Definition k_fg := [i:TwoElts]<|(Hom a b)|>Case i of (* Elt1 *) f 
                                                     (* Elt2 *) g end.

Definition j_fg := (PA k_fg).

Definition f_fg := (FPA k_fg).

Definition equalizer1_fg := (Equalizer1 k_fg).

Lemma prf_law1_fg : (r:(Ob C))(h:|(Hom r a)|)((h o f) =%S (h o g)) ->
                    (Equalizer_eq k_fg r h).
Proof.
Intros r h H; Unfold equalizer_eq.
Intros i j; Elim i; Elim j; Simpl.
Apply (Prf_refl (Hom r b)).
Apply H.
Apply (Prf_sym (Hom r b)); Trivial.
Apply (Prf_refl (Hom r b)).
Qed.

End equaz_fg.


Syntactic Definition K_fg := k_fg | 3.
Syntax constr k_fg <<(k_fg $1 $2 $3 $f $g $i)>> 10
		[<hov 0> "K_fg " <$f:"Term":L> " " <$g:"Term":L>
                                      " " <$i:"Term":L>].

Syntactic Definition J_fg := j_fg | 3.
Syntax constr j_fg <<(j_fg $1 $2 $3 $f $g)>> 10
		[<hov 0> "J_fg " <$f:"Term":L> " " <$g:"Term":L>].

Syntactic Definition F_fg := f_fg | 3.
Syntax constr f_fg <<(f_fg $1 $2 $3 $f $g)>> 10
		[<hov 0> "F_fg " <$f:"Term":L> " " <$g:"Term":L>].

Syntactic Definition Equalizer1_fg := equalizer1_fg | 3.
Syntax constr equalizer1_fg <<(equalizer1_fg $1 $2 $3 $f $g)>> 10
		[<hov 0> "Equalizer1_fg " <$f:"Term":L> " " <$g:"Term":L>].

Syntactic Definition Prf_law1_fg := prf_law1_fg | 7.
Syntax constr prf_law1_fg <<(prf_law1_fg $1 $2 $3 $4 $5 $6 $7 $e)>> 10
		[<hov 0> "Prf_law1_fg " <$e:"Term":L>].


(* Equalizer du hom-set C(a,b) *)

Section equaz_hom.

Variable C    : Category.
Variable a, b : (Ob C).

Definition i_hom := |(Hom a b)|.

Definition k_hom := [f:i_hom]f.

Definition j_hom := (PA k_hom).

Definition f_hom := (FPA k_hom).

Definition equalizer1_hom := (Equalizer1 k_hom).

End equaz_hom.

Syntactic Definition I_hom := i_hom | 1.
Syntax constr i_hom <<(i_hom $1 $a $b)>> 10
		[<hov 0> "I_hom " <$a:"Term":L> " " <$b:"Term":L>].

Syntactic Definition K_hom := k_hom | 3.
Syntax constr k_hom <<(k_hom $1 $2 $3 $f)>> 10
		[<hov 0> "K_hom " <$f:"Term":L>].

Syntactic Definition J_hom := j_hom | 1.
Syntax constr j_hom <<(j_hom $1 $a $b)>> 10
		[<hov 0> "J_hom " <$a:"Term":L> " " <$b:"Term":L>].

Syntactic Definition F_hom := f_hom | 1.
Syntax constr f_hom <<(f_hom $1 $a $b)>> 10
		[<hov 0> "F_hom " <$a:"Term":L> " " <$b:"Term":L>].

Syntactic Definition Equalizer1_hom := equalizer1_hom | 1.
Syntax constr equalizer1_hom <<(equalizer1_hom $1 $a $b)>> 10
		[<hov 0> "Equalizer1_hom " <$a:"Term":L> " " <$b:"Term":L>].

(* $Id: Equalizers1.v,v 1.2 1996/02/12 15:25:32 cpaulin Exp $ *)
