(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*            Example of Universal Arrow : Universal Arrow from the          *)
(*		Set A to the Forgetful Functor MON -> SET                    *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)



Require Export FunForget.
Require Export FreeMonoid.
Require Export UniversalArrow.

Section ua_fm.

Variable A : Setoid.

Definition UA_FM_ob := (FreeMonoid A).

Definition UA_FM_fun := [a:|A|](Concat1 a (Empty A)).

Lemma UA_FM_map_law : (map_law A (FOb FunForget (FreeMonoid A)) UA_FM_fun).
Proof.
Unfold map_law; Simpl; Intros a b H; Split; Trivial. 
Qed.

Definition UA_FM_mor := (build_Map A (FOb FunForget (FreeMonoid A)) 
                                   UA_FM_fun UA_FM_map_law).

 Section ua_fm_diese_def.

 Variable B : Monoid.
 Variable f : (Map A (FOb FunForget B)). 

 Fixpoint UA_FM_diese_fun [l:(Tlist A)] : |(Mcarrier B)| := 
 <|(Mcarrier B)|>Case l of (* Empty   *) (Munit B)
                           (* Concat1 *) [a:?][m:?]
                                         ((Ap f a) +%M (UA_FM_diese_fun m))
                 end.

 Lemma UA_FM_diese_map_law : (map_law (Tlist_setoid A) (Mcarrier B)
                                      UA_FM_diese_fun).
 Proof.
 Unfold map_law; Simpl.
 Intros l1; Elim l1.
 Intro l2; Elim l2.
 Simpl; Intro; Apply (Prf_refl (Mcarrier B)).
 Intros c t H1 H2; Elim H2.
 Intros c t H1 l2; Elim l2.
 Intro H2; Absurd (Equal_Tlist A (Concat1 c t) (Empty A)).
 Apply Diff_Concat1_Empty.
 Trivial.
 Intros c0 t0 H2; Simpl; Intro H3; Elim H3; Intros H4 H5.
 Apply (Prf_trans (Mcarrier B)) with
   y:=((Ap f c) +%M (UA_FM_diese_fun t0)).
 Apply (Prf_map2_congl ? ? ? (Mop B) (Ap f c) (UA_FM_diese_fun t) 
                       (UA_FM_diese_fun t0)).
 Apply H1; Trivial.
 Trivial.
 Apply (Prf_map2_congr ? ? ? (Mop B) (Ap f c) (Ap f c0) 
                       (UA_FM_diese_fun t0)).
 Apply (Pres ? ? f); Trivial.
 Qed.
 
 Definition UA_FM_diese_map := (build_Map (Tlist_setoid A) (Mcarrier B)
                                      UA_FM_diese_fun UA_FM_diese_map_law).
 
 Lemma UA_FM_diese_unit_law : (monUnit_law (FreeMonoid A) B UA_FM_diese_map).
 Proof.
 Unfold monUnit_law; Simpl.
 Apply (Prf_refl (Mcarrier B)).
 Qed. 

 Lemma UA_FM_diese_op_law : (monOp_law (FreeMonoid A) B UA_FM_diese_map).
 Proof.
 Unfold monOp_law; Simpl.
 Intro l1; Elim l1.
 Simpl; Intro l2.
 Apply (Prf_trans (Mcarrier B)) with y:=(UA_FM_diese_fun l2).
 Apply (Prf_refl (Mcarrier B)).
 Apply (Prf_sym (Mcarrier B)).
 Apply (Prf_monoid_idl B (UA_FM_diese_fun l2)).
 Simpl; Intros c t H l2.
 Apply (Prf_trans (Mcarrier B)) with 
  y:=((Ap f c) +%M ((UA_FM_diese_fun t) +%M (UA_FM_diese_fun l2))).
 Apply (Prf_map2_congl ? ? ? (Mop B) (Ap f c) (UA_FM_diese_fun (Concat t l2))
                       ((UA_FM_diese_fun t) +%M (UA_FM_diese_fun l2))).
 Exact (H l2).
 Apply (Prf_monoid_ass B (Ap f c) (UA_FM_diese_fun t) (UA_FM_diese_fun l2)).
 Qed.

 Definition UA_FM_diese := (build_MonMor (FreeMonoid A) B UA_FM_diese_map
                                  UA_FM_diese_unit_law UA_FM_diese_op_law).

 End ua_fm_diese_def.

 Lemma UA_FM_law1 : (ua_law1 MON SET A FunForget UA_FM_ob UA_FM_mor UA_FM_diese).
 Proof.
 Unfold ua_law1 FunForget_ob ua_eq; Intros B f.
 Unfold UA_FM_mor; Simpl.
 Unfold Ext; Simpl.
 Intro a; Apply (Prf_sym (Mcarrier B)); Exact (Prf_monoid_idr B (Ap f a)).
 Qed.

 Lemma UA_FM_law2 : (ua_law2 MON SET A FunForget UA_FM_ob UA_FM_mor UA_FM_diese).
 Proof.
 Unfold ua_law2; Simpl.
 Unfold ua_eq Equal_MonMor; Simpl.
 Unfold UA_FM_diese_map Ext; Simpl.
 Unfold Comp_fun UA_FM_fun 1 fMor.
 Simpl; Unfold FunForget_mor.
 Intros B f h H l; Elim l; Simpl.
 Apply (Prf_MonUnit_law ? ? h).
 Intros c t H1. 
 Apply (Prf_trans (Mcarrier B)) with
  y:=((Ap (MonMap h) (Concat1 c (Empty A))) +%M (Ap (MonMap h) t)).
 Apply (Prf_MonOp_law ? ? h (Concat1 c (Empty A)) t).
 Apply (Prf_trans (Mcarrier B)) with
  y:=((Ap f c) +%M (Ap (MonMap h) t)).
 Apply (Prf_map2_congr ? ? ? (Mop B) (Ap (MonMap h) (Concat1 c (Empty A)))
                       (Ap f c) (Ap (MonMap h) t)). 
 Exact (H c). 
 Apply (Prf_map2_congl ? ? ? (Mop B) (Ap f c) (Ap (MonMap h) t) 
                       (UA_FM_diese_fun B f t)).
 Trivial.
 Qed.
  
Lemma IsUA_FM : (isUA MON SET A FunForget UA_FM_ob UA_FM_mor).
Proof.
Exact (Build_isUA MON SET A FunForget UA_FM_ob UA_FM_mor UA_FM_diese
                      UA_FM_law1 UA_FM_law2).
Defined.

Definition UA_FM := (Build_ua MON SET A FunForget UA_FM_ob UA_FM_mor IsUA_FM).

End ua_fm.

(* $Id: FunForget_UA.v,v 1.2 1996/02/12 15:25:34 cpaulin Exp $ *)
