(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                         Limits                  		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Delta.
Require Export CoUniversalArrow.


Section def_cone.

Variable J, C : Category.
Variable c    : (Ob C).
Variable F    : (Functor J C).

Definition cone := (NT (Const J C c) F).

Definition cones := {(Const J C c) => F}.

 Section cone_nt.

 Variable T : (a:(Ob J))|(Hom (FOb (Const J C c) a) (FOb F a))|.

 Definition cone_law := (i,j:(Ob J))(g:|(Hom i j)|)(T j) =%S ((T i) o (FMor F g)).

 Lemma Prf_Cone_nt_law : cone_law -> (NT_law (Const J C c) F T).
 Proof.
 Unfold nt_law; Intros H a b f.
 Unfold 1 fMor; Simpl; Unfold Const_mor.
 (* *) Apply (Prf_trans (Hom c (FOb F b))) with 
   y:=(T b).
 Unfold Comp; Apply (Prf_idl C).
 Apply (H a b).
 Qed.

 Definition build_Cone := [p:cone_law](Build_NT (Const J C c) F T (Prf_Cone_nt_law p)).

 End cone_nt.

Lemma Eq_cone : (tau:cone)(cone_law (ApNT tau)).
Proof.
Unfold cone_law; Intros tau i j g.
(* *) Apply (Prf_trans (Hom c (FOb F j))) with 
  y:= (Id c) o (ApNT tau j).
Apply (Prf_sym (Hom c (FOb F j))).
Unfold Comp; Apply (Prf_idl C).
Apply (Prf_NT_law ? ? ? ? tau i j g).
Qed.
 
End def_cone.

Syntactic Definition Cone := cone | 2.
Syntax constr cone <<(cone $1 $2 $c $F)>> 10
	     [<hov 0> "Cone " <$c:"term":L> " " <$F:"term":L>].

Syntactic Definition Cones := cones | 2.
Syntax constr cones <<(cones $1 $2 $F $c)>> 10
	     [<hov 0> "Cones " <$c:"term":L> " " <$F:"term":L>].

Syntactic Definition Cone_law := cone_law | 2.
Syntax constr cone_law <<(cone_law $1 $2 $c $F $t)>> 10
       [<hov 0> "Cone_law " <$c:"term":L> " " <$F:"term":L> " " <$t:"term":L>].

Syntactic Definition Build_Cone := build_Cone | 2.
Syntax constr build_Cone <<(build_Cone $1 $2 $F $c $t $p)>> 10
       [<hov 0> "Build_Cone " <$c:"term":L> " " <$F:"term":L> " " 
                                <$t:"term":L> " " <$p:"term":L>].
		
Section def_limit.

Variable J, C : Category.
Variable F    : (Functor J C).

Definition limit := (coUA C (CatFunct J C) F (Delta C J)).

Variable l : limit.

Definition lim := (CoUA_ob l).

Definition limiting_cone : (Cone lim F) := (CoUA_mor l).

(* tau(i) = tau# o mu(i) *)

Lemma Eq_limit : (c:(Ob C))(tau:(Cone c F))(i:(Ob J))
      (ApNT tau i) =%S ((CoUA_diese l c tau) o (ApNT limiting_cone i)).
Proof.
Intros c tau i.
Apply (Prf_sym (Hom c (FOb F i))).
Apply (Prf_CoUA_law1 ? ? ? ? l ? tau i).
Qed.

Variable a : (Ob C).
Variable u : (Cone a F).

Definition isLimit := (isCoUA C (CatFunct J C) F (Delta C J) a u).

Variable diese : (c:(Ob C))(Cone c F) -> |(Hom c a)|.

Definition limit_law1 := (coUA_law1 C (CatFunct J C) F (Delta C J) a u diese).

Definition limit_law2 := (coUA_law2 C (CatFunct J C) F (Delta C J) a u diese).

Variable ll1 : limit_law1.
Variable ll2 : limit_law2.

Definition build_IsLimit := (Build_isCoUA C (CatFunct J C) F (Delta C J) a u 
                                          diese ll1 ll2).

Variable p : isLimit.

Definition build_Limit := (Build_coUA C (CatFunct J C) F (Delta C J) a u p).

End def_limit.


Syntactic Definition Limit := limit | 2.
Syntax constr limit <<(limit $1 $2 $F)>> 10
	     [<hov 0> "Limit " <$F:"term":L>].

Syntactic Definition Lim := lim | 3.
Syntax constr lim <<(lim $1 $2 $3 $cl)>> 10
	     [<hov 0> "Lim " <$cl:"term":L>].

Syntactic Definition Limiting_cone := limiting_cone | 3.
Syntax constr limiting_cone <<(limiting_cone $1 $2 $3 $cl)>> 10
	     [<hov 0> "Limiting_cone " <$cl:"term":L>].

Syntactic Definition IsLimit := isLimit | 2.
Syntax constr isLimit <<(isLimit $1 $2 $F $a $u)>> 10
	     [<hov 0> "Islimit " <$F:"term":L> " " <$a:"term":L> " " <$u:"term":L>].

Syntactic Definition Limit_law1 := limit_law1 | 2.
Syntax constr limit_law1 <<(limit_law1 $1 $2 $F $a $u $d)>> 10
	     [<hov 0> "Limit_law1 " <$F:"term":L> " " <$a:"term":L> 
                                    " " <$u:"term":L> " " <$d:"term":L>].

Syntactic Definition Limit_law2 := limit_law2 | 2.
Syntax constr limit_law2 <<(limit_law2 $1 $2 $F $a $u $d)>> 10
	     [<hov 0> "Limit_law2 " <$F:"term":L> " " <$a:"term":L> 
                                    " " <$u:"term":L> " " <$d:"term":L>].

Syntactic Definition Build_IsLimit := build_IsLimit | 2.
Syntax constr build_IsLimit <<(build_IsLimit $1 $2 $F $a $u $d $l1 $l2)>> 10
     [<hov 0> "Build_IsLimit " <$F:"term":L> " " <$a:"term":L> " " 
      <$u:"term":L> " " <$d:"term":L> " " <$l1:"term":L> " " <$l2:"term":L>].

Syntactic Definition Build_Limit := build_Limit | 2.
Syntax constr build_Limit <<(build_Limit $1 $2 $F $a $u $p)>> 10
     [<hov 0> "Build_Limit " <$F:"term":L> " " <$a:"term":L> " " 
              <$u:"term":L> " " <$p:"term":L>].


Definition Complete := [C:Category](J:Category)(F:(Functor J C))(Limit F).


(* $Id: Limit.v,v 1.2 1996/02/12 15:25:38 cpaulin Exp $ *)
