(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	       Category Parallel Arrowd (used in Equalizers)		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)


Require Export BasicTypes.
Require Export Category.

Section pa_def.

Variable C   : Category.
Variable a,b : (Ob C).
Variable I   : Type.
Variable k   : I -> |(Hom a b)|.

Inductive PA_ob : Type := PA1 : PA_ob | PA2 : PA_ob.

Inductive PA_mor : PA_ob -> PA_ob -> Type := 
   PA_I1 : (PA_mor PA1 PA1)
 | PA_I2 : (PA_mor PA2 PA2)
 | PA_f  : I -> (PA_mor PA1 PA2).

(* sche'mas d'e'limination pour PA_mor *)

Lemma PA_11_P : ((PA_mor PA1 PA1)->Prop) ->
                     (x,y:PA_ob)(PA_mor x y) -> Prop.
Proof.
Intros P x y; Elim x; Elim y.
Exact P.
Intro x'; Exact True.
Intro x'; Exact True.
Intro x'; Exact True.
Defined.

Lemma PA_11_ind : (P:(PA_mor PA1 PA1)->Prop) 
                  (P PA_I1) -> (x:(PA_mor PA1 PA1))(P x).
Proof.
Intros P Pi1 x.
Change (PA_11_P P PA1 PA1 x).
Apply (PA_mor_ind ([x,y:PA_ob][f:(PA_mor x y)](PA_11_P P x y f))); Simpl.
Exact Pi1.
Trivial.
Trivial.
Qed.

Lemma PA_22_P : ((PA_mor PA2 PA2)->Prop) ->
                     (x,y:PA_ob)(PA_mor x y) -> Prop.
Proof.
Intros P x y; Elim x; Elim y.
Intro x'; Exact True.
Intro x'; Exact True.
Intro x'; Exact True.
Exact P.
Defined.

Lemma PA_22_ind : (P:(PA_mor PA2 PA2)->Prop) 
                  (P PA_I2) -> (x:(PA_mor PA2 PA2))(P x).
Proof.
Intros P Pi2 x.
Change (PA_22_P P PA2 PA2 x).
Apply (PA_mor_ind ([x,y:PA_ob][f:(PA_mor x y)](PA_22_P P x y f))); Simpl.
Trivial.
Exact Pi2.
Trivial.
Qed.

Lemma PA_12_P : ((PA_mor PA1 PA2)->Prop) ->
                    (x,y:PA_ob)(PA_mor x y) -> Prop.
Proof.
Intros P x y; Elim x; Elim y.
Intro x'; Exact True.
Exact P.
Intro x'; Exact True.
Intro x'; Exact True.
Defined.

Lemma PA_12_ind : (P:(PA_mor PA1 PA2)->Prop) 
                 ((i:I)(P (PA_f i))) -> 
                 (x:(PA_mor PA1 PA2))(P x).
Proof.
Intros P Pf x.
Change (PA_12_P P PA1 PA2 x).
Apply (PA_mor_ind ([x,y:PA_ob][f:(PA_mor x y)](PA_12_P P x y f))); Simpl.
Trivial.
Trivial.
Exact Pf.
Qed.

Lemma PA_21_P : ((PA_mor PA2 PA1)->Prop) ->
                     (x,y:PA_ob)(PA_mor x y) -> Prop.
Proof.
Intros P x y; Elim x; Elim y.
Intro x'; Exact True.
Intro x'; Exact True.
Exact P.
Intro x'; Exact True.
Defined.

Lemma PA_21_ind : (P:(PA_mor PA2 PA1)->Prop) 
                     (x:(PA_mor PA2 PA1))(P x).
Proof.
Intros P x.
Change (PA_21_P P PA2 PA1 x).
Apply (PA_mor_ind ([x,y:PA_ob][f:(PA_mor x y)](PA_21_P P x y f))); Simpl; Auto.
Qed.

Lemma PA_21_T : ((PA_mor PA2 PA1)->Type) ->
                         (x,y:PA_ob)(PA_mor x y) -> Type.
Proof.
Intros P x y; Elim x; Elim y.
Intro x'; Exact UnitType.
Intro x'; Exact UnitType.
Exact P.
Intro x'; Exact UnitType.
Defined.

Lemma PA_21_rect : (P:(PA_mor PA2 PA1)->Type) 
                     (x:(PA_mor PA2 PA1))(P x).
Proof.
Intros P x.
Change (PA_21_T P PA2 PA1 x).
Apply (PA_mor_rect ([x,y:PA_ob][f:(PA_mor x y)](PA_21_T P x y f))); Simpl; Auto.
Qed.


(* egalite' *)

Definition Equal_PA_mor := [x,y:PA_ob][f,g:(PA_mor x y)]
  <[x',y':PA_ob]Prop>Case f of 
  (* PA_I1  *) True
  (* PA_I2  *) True
  (* PA_f i *) [i:I]<[x',y':PA_ob]Prop>Case g of 
                     (* PA_I1  *) False
                     (* PA_I2  *) False
                     (* PA_f j *) [j:I]((k i) =%S (k j)) end end.

Lemma Equal_PA_mor_Equiv : (x,y:PA_ob)(Equivalence (Equal_PA_mor x y)).
Proof.
Intros x y; Apply Build_equivalence; Red.
Intro f; Elim f.
Simpl; Trivial.
Simpl; Trivial.
Intro i; Simpl; Apply (Prf_refl (Hom a b)).
Intros f; Elim f.
Intros g h e1 e2.
Apply (PA_11_ind ([h:(PA_mor PA1 PA1)](Equal_PA_mor PA1 PA1 PA_I1 h))).
Simpl; Trivial.
Intros g h e1 e2.
Apply (PA_22_ind ([h:(PA_mor PA2 PA2)](Equal_PA_mor PA2 PA2 PA_I2 h))).
Simpl; Trivial.
Intros i g h.
LApply (PA_12_ind ([x:(PA_mor PA1 PA2)]
                      (Equal_PA_mor PA1 PA2 (PA_f i) x)->
                          (Equal_PA_mor PA1 PA2 x h)->
                                (Equal_PA_mor PA1 PA2 (PA_f i) h))).
Intros H H0 H1; Apply (H g H0 H1).
Intro j; Unfold 1 Equal_PA_mor; Intro eqij.
LApply (PA_12_ind ([x:(PA_mor PA1 PA2)]
                          (Equal_PA_mor PA1 PA2 (PA_f j) x)->
                                 (Equal_PA_mor PA1 PA2 (PA_f i) x))).
Intros H H0; Apply (H h H0).
Simpl; Intros l eqjl.
(* *) Apply (Prf_trans (Hom a b)) with y:=(k j); Trivial.
Intros f; Elim f.
Intros g e1.
Apply (PA_11_ind ([g:(PA_mor PA1 PA1)](Equal_PA_mor PA1 PA1 g PA_I1))).
Simpl; Trivial.
Intros g e1.
Apply (PA_22_ind ([g:(PA_mor PA2 PA2)](Equal_PA_mor PA2 PA2 g PA_I2))).
Simpl; Trivial.
Intros i g.
LApply (PA_12_ind ([g:(PA_mor PA1 PA2)](Equal_PA_mor PA1 PA2 (PA_f i) g)->
                            (Equal_PA_mor PA1 PA2 g (PA_f i)))).
Intros H H0; Apply (H g H0).
Intro j; Simpl; Intro eqij; Apply (Prf_sym (Hom a b)); Trivial.
Qed.

Definition PA_mor_setoid := [x,y:PA_ob]
 (Build_Setoid (PA_mor x y) (Equal_PA_mor x y) (Equal_PA_mor_Equiv x y)).

(* composition *)

Definition Comp_PA_mor := [x,y,z:PA_ob][f:(PA_mor x y)]
 <[p,p0:PA_ob][_:(PA_mor p p0)](PA_mor p0 z)->(PA_mor p z)>Case f of 
  (* PA_I1 *) <[p:PA_ob](PA_mor PA1 p)->(PA_mor PA1 p)>Case z of
              (* PA1 *) [_:?]PA_I1 
              (* PA2 *) [g:?]g end
  (* PA_I2 *) <[p:PA_ob](PA_mor PA2 p)->(PA_mor PA2 p)>Case z of
              (* PA1 *) [g:?](PA_21_rect ([_:?](PA_mor PA2 PA1)) g)
              (* PA2 *) [_:?]PA_I2 end
  (* PA_f  *) <[p:PA_ob]I->(PA_mor PA2 p)->(PA_mor PA1 p)>Case z of
              (* PA1 *) [_:?][g:?](PA_21_rect ([_:?](PA_mor PA1 PA1)) g)
              (* PA2 *) [i:?][_:?](PA_f i) end end.
(* *)

Lemma Comp_PA_fact1 : (f:(PA_mor PA1 PA1))(x:PA_ob)(g:(PA_mor PA1 x))
                      (Equal_PA_mor ? ? (Comp_PA_mor ? ? ? f g) g).
Proof.
Intros f x g; Apply (PA_11_ind ([x':(PA_mor PA1 PA1)]
                       (Equal_PA_mor PA1 x (Comp_PA_mor PA1 PA1 x x' g) g))).
Generalize g; Elim x.
Simpl; Auto.
Simpl; Intro g1; Apply (Prf_refl (PA_mor_setoid PA1 PA2) g1).
Qed.

Lemma Comp_PA_fact2 : (f:(PA_mor PA2 PA2))(x:PA_ob)(g:(PA_mor PA2 x))
                      (Equal_PA_mor ? ? (Comp_PA_mor ? ? ? f g) g).
Proof.
Intros f x g; Apply (PA_22_ind ([x':(PA_mor PA2 PA2)]
                       (Equal_PA_mor PA2 x (Comp_PA_mor PA2 PA2 x x' g) g))).
Generalize g; Elim x.
Intro g1; Apply (PA_21_ind ([x':(PA_mor PA2 PA1)]
                   (Equal_PA_mor PA2 PA1
                    (PA_21_rect [_:(PA_mor PA2 PA1)](PA_mor PA2 PA1) g1) g1)) g1).
Simpl; Auto.
Qed.

Lemma Comp_PA_fact3 : (i:I)(g:(PA_mor PA2 PA2))
                      (Equal_PA_mor ? ? (Comp_PA_mor ? ? ? (PA_f i) g) (PA_f i)).
Proof.
Intros i g; Apply (PA_22_ind ([x':(PA_mor PA2 PA2)]
           (Equal_PA_mor PA1 PA2 (Comp_PA_mor PA1 PA2 PA2 (PA_f i) x') (PA_f i)))).
Simpl; Apply (Prf_refl (Hom a b)).
Qed.

Lemma Comp_PA_fact4 : (i:I)(g:(PA_mor PA2 PA1))(h:(PA_mor PA1 PA1))
                      (Equal_PA_mor ? ? (Comp_PA_mor ? ? ? (PA_f i) g) h).
Proof.
Intros i g h; Apply (PA_21_ind ([x':(PA_mor PA2 PA1)]
                      (Equal_PA_mor PA1 PA1 (Comp_PA_mor PA1 PA2 PA1 (PA_f i) g) h)) g).
Qed.


(** **)

Lemma Comp_PA_congl : (Congl_law PA_ob PA_mor_setoid Comp_PA_mor).
Proof.
Unfold Congl_law; Simpl.
Intros x y z f g h; Generalize f; Generalize g.
Elim h; Elim z.
(* cas 1*)
(* cas 1.1 *)
Simpl; Auto.
(* cas 1.2 *)
Simpl; Auto.
(* cas 2 *)
(* cas 2.1 *)
Intros f0 g0 H.
Apply (PA_21_ind ([x:(PA_mor PA2 PA1)]
                      (Equal_PA_mor PA2 PA1 (Comp_PA_mor PA2 PA2 PA1 PA_I2 g0)
                                         (Comp_PA_mor PA2 PA2 PA1 PA_I2 f0))) g0).
(* cas 2.2 *)
Simpl; Auto.
(* cas 3 *)
(* cas 3.1 *)
Intros i g0 f0 H.
Apply (PA_21_ind ([x:(PA_mor PA2 PA1)]
                      (Equal_PA_mor PA1 PA1 (Comp_PA_mor PA1 PA2 PA1 (PA_f i) f0)
                                         (Comp_PA_mor PA1 PA2 PA1 (PA_f i) g0))) f0).
(* cas 3.2 *)
Simpl; Intros i g0 f0 H; Apply (Prf_refl (Hom a b)).
Qed.

Lemma Comp_PA_congr : (Congr_law PA_ob PA_mor_setoid Comp_PA_mor).
Proof.
Unfold Congr_law; Simpl.
Intros x y z f; Elim f.
(* cas 1 *)
Intro g; LApply (PA_11_ind ([x':(PA_mor PA1 PA1)](h:(PA_mor PA1 z))
    (Equal_PA_mor PA1 PA1 PA_I1 x')
     ->(Equal_PA_mor PA1 z (Comp_PA_mor PA1 PA1 z PA_I1 h)
         (Comp_PA_mor PA1 PA1 z x' h)))).
Intros H h H0; Apply (H g h H0).
Elim z.
(* cas 1.1 *)
Simpl; Auto.
(* cas 1.2 *)
Simpl; Intros h H; Apply (Prf_refl (PA_mor_setoid PA1 PA2) h).
(* cas 2 *)
Intro g; LApply (PA_22_ind ([x':(PA_mor PA2 PA2)](h:(PA_mor PA2 z))
                          (Equal_PA_mor PA2 PA2 PA_I2 x')->
                          (Equal_PA_mor PA2 z (Comp_PA_mor PA2 PA2 z PA_I2 h)
                                            (Comp_PA_mor PA2 PA2 z x' h)))).
Intros H h H0; Apply (H g h H0).
Elim z.
(* cas 2.1 *)
Intros h H; Apply (PA_21_ind ([x':(PA_mor PA2 PA1)]
                       (Equal_PA_mor PA2 PA1 (Comp_PA_mor PA2 PA2 PA1 PA_I2 h)
                                          (Comp_PA_mor PA2 PA2 PA1 PA_I2 h))) h).
(* cas 2.2 *)
Simpl; Auto.
(* cas 3 *)
Intros i g; LApply (PA_12_ind ([x':(PA_mor PA1 PA2)](h:(PA_mor PA2 z))
                   (Equal_PA_mor PA1 PA2 (PA_f i) x')->
                      (Equal_PA_mor PA1 z (Comp_PA_mor PA1 PA2 z (PA_f i) h)
                                        (Comp_PA_mor PA1 PA2 z x' h)))).
Intros H h H0; Apply (H g h H0).
Elim z.
(* cas 3.1 *)
Intros j h H; Apply (PA_21_ind ([x':(PA_mor PA2 PA1)]
                    (Equal_PA_mor PA1 PA1 (Comp_PA_mor PA1 PA2 PA1 (PA_f i) h)
                                       (Comp_PA_mor PA1 PA2 PA1 (PA_f j) h))) h).
(* cas 3.2 *)
Intros j h H; Simpl; Exact H.
Qed.

(** **)

Definition Comp_PA := (Build_Comp PA_ob PA_mor_setoid Comp_PA_mor
                                     Comp_PA_congl Comp_PA_congr).

Lemma Assoc_PA : (assoc_law PA_ob PA_mor_setoid Comp_PA).
Proof.
Unfold assoc_law; Simpl.
Intros x y z o4 f; Elim f.
(* cas 1 *)
Intros g h.
Apply (Prf_trans (PA_mor_setoid PA1 o4) 
           (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA1 o4 PA_I1
                (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 z o4 g h))
           (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 z o4 g h)
           (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 z o4
                (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA1 z PA_I1 g) h)).
Apply (Comp_PA_fact1 PA_I1 o4 (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 z o4 g h)).
Apply (Comp_PA_congr ? ? ? g (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA1 z PA_I1 g) h).
Apply (Prf_sym (PA_mor_setoid PA1 z)).
Apply (Comp_PA_fact1 PA_I1 z g).
(* cas 2 *)
Intros g h.
Apply (Prf_trans (PA_mor_setoid PA2 o4) 
           (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 PA2 o4 PA_I2
                (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 z o4 g h))
           (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 z o4 g h)
           (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 z o4
                (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 PA2 z PA_I2 g) h)).
Apply (Comp_PA_fact2 PA_I2 o4 (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 z o4 g h)).
Apply (Comp_PA_congr ? ? ? g (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 PA2 z PA_I2 g) h).
Apply (Prf_sym (PA_mor_setoid PA2 z)).
Apply (Comp_PA_fact2 PA_I2 z g).
(* cas 3 *) 
Intro i; Elim z.
Intros g h; Apply (PA_21_ind ([x:(PA_mor PA2 PA1)]
                    (Equal_PA_mor PA1 o4
                     (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA2 o4 (PA_f i)
                      (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 PA1 o4 g h))
                     (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA1 o4
                      (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA2 PA1 (PA_f i) g) h))) g).
Intros g h; Apply (Prf_trans (PA_mor_setoid PA1 o4)
                        (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA2 o4 (PA_f i)
                         (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 PA2 o4 g h))
                        (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA2 o4 (PA_f i) h)
                        (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA2 o4
                         (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA2 PA2 (PA_f i) g) h)).
Apply (Comp_PA_congl ? ? ? (Cat_comp PA_ob PA_mor_setoid Comp_PA PA2 PA2 o4 g h)
                           h (PA_f i)).
Apply (Comp_PA_fact2 g ? h).
Simpl.
Apply (Comp_PA_congr ? ? ? (PA_f i)
                 (Cat_comp PA_ob PA_mor_setoid Comp_PA PA1 PA2 PA2 (PA_f i) g) h).
Apply (Prf_sym (PA_mor_setoid PA1 PA2)).
Apply (Comp_PA_fact3 i g).
Qed.

Definition Id_PA := [x:PA_ob]<[x':PA_ob](PA_mor x' x')>
                              Case x of (* PA1 *) PA_I1 
                                        (* PA2 *) PA_I2 end.

Lemma Idl_PA : (idl_law PA_ob PA_mor_setoid Comp_PA Id_PA).
Proof.
Unfold idl_law; Simpl.
Intros x y; Elim x; Intro f.
Apply (Comp_PA_fact1 PA_I1 ? f).
Apply (Comp_PA_fact2 PA_I2 ? f).
Qed.

Lemma Idr_PA : (idr_law PA_ob PA_mor_setoid Comp_PA Id_PA).
Proof.
Unfold idr_law; Simpl.
Intros x y f; Elim f.
Apply (Comp_PA_fact1 PA_I1 ? PA_I1).
Apply (Comp_PA_fact2 PA_I2 ? PA_I2).
Intro i; Apply (Comp_PA_fact3 i PA_I2).
Qed.

Definition pa := (Build_Category PA_ob PA_mor_setoid Comp_PA Id_PA
                                 Assoc_PA Idl_PA Idr_PA).

End pa_def.

Syntactic Definition PA := pa | 4.
Syntax constr pa <<(pa $1 $2 $3 $4 $k)>> 10
		[<hov 0> "PA " <$k:"Term":L>].

(* $Id: PA.v,v 1.2 1996/02/12 15:25:40 cpaulin Exp $ *)
