(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                       Products                   		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)


Require Export Limit.
Require Export Discr.
Require Export Products.

(* Foncteur de I~ (cate'gorie discrete obtenue a` partir de I) vers C *)

Section products_limit_def.

Variable C : Category.
Variable I : Type.
Variable a : I -> (Ob C).

Definition FunDiscr_ob := [i:I](a i).

Definition FunDiscr_arrow := [i,j:I][f:|(hom (Discr I) i j)|]
                             <[d,d0:I][_:(Discr_mor I d d0)]|(Hom (a d) (a d0))|>
                             Case f of [k:I](Id (a k)) end.

Lemma FunDiscr_map_law : (i,j:I)
      (map_law (hom (Discr I) i j) ? (FunDiscr_arrow i j)).
Proof.
Unfold map_law.
Intros i j f; Elim f; Intros k f1 H.
Apply (Discr_mor_ind1 ? k
       ([f1:(Discr_mor ? k k)](FunDiscr_arrow k k (Build_Discr_mor I k)) =%S 
                                    (FunDiscr_arrow k k f1))).
Simpl; Apply (Prf_refl (Hom (a k) (a k))).
Qed.

Definition FunDiscr_map := [i,j:I]
           (build_Map (hom (Discr I) i j) ? 
                      (FunDiscr_arrow i j) (FunDiscr_map_law i j)).

Lemma FunDiscr_comp_law : (fcomp_law (Discr I) C FunDiscr_ob FunDiscr_map).
Proof.
Unfold fcomp_law; Intros i j k f.
Elim f; Simpl.
Intros i' g; Elim g; Simpl.
Intro i''; Unfold Comp; (Apply (Prf_idr C)).
Qed.

Lemma FunDiscr_id_law : (fid_law (Discr I) C FunDiscr_ob FunDiscr_map).
Proof.
Unfold fid_law; Simpl.
Intro i; Apply (Prf_refl (Hom (a i) (a i))).
Qed.

Definition funDiscr := (Build_Functor (Discr I) C FunDiscr_ob FunDiscr_map
                                      FunDiscr_comp_law FunDiscr_id_law).

(* produit d'une famille d'objets *)

Definition product1 := (Limit funDiscr).

Variable l : product1.

(* ||ai *)

Definition pi1 := (Lim l).

(* pi : ||ai -> ai *)

Definition proj1 := [i:I](ApNT (Limiting_cone l) i).

(* construire a` partir d'une famille fj:c->ai, une NT tau:Delta(c)->F *)

 Section pd1_diese_def.

 Variable c   : (Ob C).
 Variable f   : (j:I)|(Hom c (a j))|.

 Definition Product_tau := [j:I](f j).

 Lemma Product_tau_cone_law : (Cone_law c funDiscr Product_tau).
 Proof.
 Unfold cone_law Product_tau; Simpl.
 Intros a' b g; Elim g; Simpl; Intro i.
 Unfold fMor; Simpl.
 Unfold Comp; Apply (Prf_idr C).
 Qed.

 Definition Product_nt := (Build_Cone c funDiscr Product_tau Product_tau_cone_law).

 Definition pd1_diese := (CoUA_diese l c Product_nt).

 End pd1_diese_def.

Lemma Prf_pd1_law1 : (Product_law1 pi1 proj1 pd1_diese).
Proof.
Unfold product_law1 product_eq; Intros c f j.
Apply (Eq_limit ? ? ? l c (Product_nt c f) j).
Qed.

Lemma Prf_pd1_law2 : (Product_law2 pi1 proj1 pd1_diese).
Proof.
Unfold product_law2 product_eq; Intros c f g H.
Apply (Prf_CoUA_law2 ? ? ? ? l ? (Product_nt c f) g).
Unfold coUA_eq; Simpl; Unfold Equal_NT; Simpl.
Unfold Comp_tau Product_tau fMor; Simpl.
Unfold Delta_mor_tau; Intro i.
Apply (Prf_sym (Hom c (a i))).
Apply (H i).
Qed.

Definition Product1_to_Product := (Build_product ? ? ? pi1 proj1 pd1_diese 
                                                 Prf_pd1_law1 Prf_pd1_law2).

End products_limit_def.

Syntactic Definition FunDiscr := funDiscr | 2.
Syntax constr funDiscr <<(funDiscr $1 $2 $f)>> 10
	        [<hov 0> "FunDiscr " <$f:"Term":L>].

Syntactic Definition Product1 := product1 | 2.
Syntax constr product1 <<(product1 $1 $2 $a)>> 10
		[<hov 0> "Product1 " <$a:"Term":L>].

Syntactic Definition PI1 := pi1 | 3.
Syntax constr pi1 <<(pi1 $1 $2 $3 $p)>> 10
		[<hov 0> "PI1 " <$p:"Term":L>].

Syntactic Definition Proj1 := proj1 | 3.
Syntax constr proj1 <<(proj1 $1 $2 $3 $p $i)>> 10
		[<hov 0> "Proj1 " <$p:"Term":L> " " <$i:"Term":L>].

Syntactic Definition Pd1_diese := pd1_diese | 3.
Syntax constr pd1_diese <<(pd1_diese $1 $2 $3 $p $r $a')>> 10
    [<hov 0> "Pd1_diese " <$p:"Term":L> " " <$r:"Term":L> " " <$a':"Term":L>].







(* $Id: Products1.v,v 1.2 1996/02/12 15:25:46 cpaulin Exp $ *)
