(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                       Pullbacks                  		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)


Require Export Limit.
Require Export PULB.
Require Export Pullbacks.

(* Pullbacks *)

Section Fpullback_limit_def.

Variable C : Category.
Variable G : (Functor PULB C).

Definition fpullback1 := (Limit G).

Variable l : fpullback1.

Definition fibred1_Fprod := (Lim l).

Definition fibred1_Fp := (ApNT (Limiting_cone l) P1).

Definition fibred1_Fq := (ApNT (Limiting_cone l) P3).

Lemma Prf_pb1_Flaw1 : (Pullback_law1 (FMor G F12) (FMor G F32) 
                                     fibred1_Fprod fibred1_Fp fibred1_Fq).
Proof.
Unfold pullback_law1 pullback_eq1.
(* *) Apply (Prf_trans (Hom fibred1_Fprod (FOb G P2))) with
  y:=(ApNT (Limiting_cone l) P2).
Apply (Prf_sym (Hom fibred1_Fprod (FOb G P2))). 
Apply (Eq_cone ? ? fibred1_Fprod ? (Limiting_cone l) 
               P1 P2 F12).
Apply (Eq_cone ? ? fibred1_Fprod ? (Limiting_cone l) 
               P3 P2 F32).
Qed.
 

 Section pb1_Fdiese_def.

 Variable r  : (Ob C).
 Variable t1 : |(Hom r (FOb G P1))|.
 Variable t2 : |(Hom r (FOb G P3))|.

 Hypothesis H : (Pullback_eq1 (FMor G F12) (FMor G F32) ? t1 t2).

 (* construction de r# *) 

 Definition Pulb_tau := [a:PULB_ob]<[a':PULB_ob]|(Hom r (FOb G a'))|>
                                   Case a of (* P1 *) t1 
                                             (* P2 *) (t1 o (FMor G F12)) 
                                             (* P3 *) t2 end.
 
 Lemma Pulb_cone_law : (Cone_law r G Pulb_tau).
 Proof.
 Unfold cone_law; Intros x x1 t.
 Elim t; Simpl; Unfold 1 fMor; Simpl.
 Apply (Prf_trans (Hom r (FOb G P1))) with 
  y:=t1 o (Id (FOb G P1)).
 Unfold Comp; Apply (Prf_idr C).
 Apply (Prf_congl C).
 Apply (Prf_sym (Hom (FOb G P1) (FOb G P1))).
 Exact (Prf_Fid_law ? ? G P1).
 Apply (Prf_trans (Hom r (FOb G P2))) with
  y:=((t1 o (FMor G F12)) o (Id (FOb G P2))).
 Unfold Comp; Apply (Prf_idr C).
 Apply (Prf_congl C).
 Apply (Prf_sym (Hom (FOb G P2) (FOb G P2))).
 Exact (Prf_Fid_law ? ? G P2).
 Apply (Prf_trans (Hom r (FOb G P3))) with 
  y:=t2 o (Id (FOb G P3)).
 Unfold Comp; Apply (Prf_idr C).
 Apply (Prf_congl C).
 Apply (Prf_sym (Hom (FOb G P3) (FOb G P3))).
 Exact (Prf_Fid_law ? ? G P3).
 Apply (Prf_refl (Hom r (FOb G P2))).
 Apply H.
 Qed.
 
 Definition Pulb_cone := (Build_Cone r G Pulb_tau Pulb_cone_law).

 Definition pb1_Fdiese := (CoUA_diese l r Pulb_cone).

 End pb1_Fdiese_def.


Lemma Prf_pb1_Flaw2 : (Pullback_law2 (FMor G F12) (FMor G F32) fibred1_Fprod 
                                    fibred1_Fp pb1_Fdiese).
Proof.
Unfold pullback_law2 pullback_eq1 pullback_eq2; Intros r t1 t2 H.
Apply (Eq_limit ? ? ? l r (Pulb_cone ? t1 t2 H) P1).
Qed.

Lemma Prf_pb1_Flaw3 : (Pullback_law3 (FMor G F12) (FMor G F32) fibred1_Fprod 
                                    fibred1_Fq pb1_Fdiese).
Proof.
Unfold pullback_law3 pullback_eq1 pullback_eq2; Intros r t1 t2 H.
Apply (Eq_limit ? ? ? l r (Pulb_cone ? t1 t2 H) P3).
Qed.

(* unicite' de r# *)

Lemma Prf_pb1_Flaw4 : (Pullback_law4 (FMor G F12) (FMor G F32) fibred1_Fprod 
                                    fibred1_Fp fibred1_Fq pb1_Fdiese).
Proof.
Unfold pullback_law4 pullback_eq1 pullback_eq2; Intros r t1 t2 H h H1 H2.
Unfold pb1_Fdiese.
Apply (Prf_CoUA_law2 ? ? ? ? l).
Unfold coUA_eq; Simpl.
Unfold Equal_NT; Simpl.
Unfold Comp_tau; Simpl.
Unfold Delta_mor_tau; Intro x.
Elim x; Simpl.
Apply (Prf_sym (Hom r (FOb G P1))).
Exact H1.
(* *) Apply (Prf_trans (Hom r (FOb G P2))) 
       with y :=h o (fibred1_Fp o (FMor G F12)).
Apply (Prf_congl C).
Apply (Eq_cone ? ? fibred1_Fprod ? (Limiting_cone l) P1 P2 F12).
(* *) Apply (Prf_trans (Hom r (FOb G P2))) 
       with y :=(h o fibred1_Fp) o (FMor G F12).
Unfold Comp; Apply (Prf_ass C).
Apply (Prf_congr C).
Apply (Prf_sym (Hom r (FOb G P1))).
Exact H1.
Apply (Prf_sym (Hom r (FOb G P3))).
Exact H2.
Qed.

Definition Pullback1_to_Pullback := (Build_pullback ? ? ? ? (FMor G F12) 
           (FMor G F32) fibred1_Fprod fibred1_Fp fibred1_Fq 
           Prf_pb1_Flaw1 pb1_Fdiese Prf_pb1_Flaw2 
           Prf_pb1_Flaw3 Prf_pb1_Flaw4). 

End Fpullback_limit_def.

Syntactic Definition Fpullback1 := fpullback1 | 1.
Syntax constr fpullback1 <<(fpullback1 $1 $G)>> 10
              [<hov 0> "Fpullback1 " <$G:"Term":L>].

Syntactic Definition Fibred1_Fprod := fibred1_Fprod | 2.
Syntax constr fibred1_Fprod <<(fibred1_Fprod $1 $2 $l)>> 10
              [<hov 0> "Fibred1_Fprod " <$l:"Term":L>].

Syntactic Definition Fibred1_Fp := fibred1_Fp | 2.
Syntax constr fibred1_Fp <<(fibred1_Fp $1 $2 $l)>> 10
              [<hov 0> "Fibred1_Fp " <$l:"Term":L>].

Syntactic Definition Fibred1_Fq := fibred1_Fq | 2.
Syntax constr fibred1_Fq <<(fibred1_Fq $1 $2 $l)>> 10
              [<hov 0> "Fibred1_Fq " <$l:"Term":L>].

Syntactic Definition Pb1_Fdiese := pb1_Fdiese | 2.
Syntax constr pb1_Fdiese 
   <<(pb1_Fdiese $1 $2 $l $r $t1 $t2 $H $pe)>> 10
   [<hov 0> "Pb1_Fdiese " <$l:"Term":L> " " <$r:"Term":L> " " 
    <$t1:"Term":L> " " <$t2:"Term":L> " " <$H:"Term":L> " " <$pe:"Term":L>].

(* Functor PULB -> C *)

Section pullback_limit_def.

Variable C       : Category.
Variable a, b, c : (Ob C).
Variable f       : |(Hom a b)|.
Variable g       : |(Hom c b)|.

Definition Fpulb_ob := [x:PULB_ob]<(Ob C)>Case x of (* P1 *) a
                                                    (* P2 *) b 
                                                    (* P3 *) c end.

Definition Fpulb_mor := [x,y:PULB_ob][h:(PULB_mor x y)]
                        <[x',y':PULB_ob]|(Hom (Fpulb_ob x') (Fpulb_ob y'))|>
                        Case h of (* IP1 *) (Id a)
                                  (* IP2 *) (Id b) 
                                  (* IP3 *) (Id c) 
                                  (* F12 *) f 
                                  (* F32 *) g end.

Lemma Fpulb_map_law : (x,y:PULB_ob)(map_law (PULB_mor_setoid x y) ? (Fpulb_mor x y)).
Proof.
Unfold map_law; Intros x y h1.
Elim h1; Simpl.
(* 1 *)
Intro h2; LApply (PULB_11_ind ([t:(PULB_mor P1 P1)]
                                                 (Equal_PULB_mor P1 P1 IP1 t) ->
                                                    (Id a) =%S (Fpulb_mor P1 P1 t))).
Intros H H0; Apply (H h2 H0).
Intro H; Simpl; Apply (Prf_refl (Hom a a)).
(* 2 *)
Intro h2; LApply (PULB_22_ind ([t:(PULB_mor P2 P2)]
                                                 (Equal_PULB_mor P2 P2 IP2 t) ->
                                                    (Id b) =%S (Fpulb_mor P2 P2 t))).
Intros H H0; Apply (H h2 H0).
Intro H; Simpl; Apply (Prf_refl (Hom b b)).
(* 3 *)
Intro h2; LApply (PULB_33_ind ([t:(PULB_mor P3 P3)]
                                                 (Equal_PULB_mor P3 P3 IP3 t) ->
                                                    (Id c) =%S (Fpulb_mor P3 P3 t))).
Intros H H0; Apply (H h2 H0).
Intro H; Simpl; Apply (Prf_refl (Hom c c)).
(* 4 *)
Intro h2; LApply (PULB_12_ind ([t:(PULB_mor P1 P2)]
                                                 (Equal_PULB_mor P1 P2 F12 t) ->
                                                    f =%S (Fpulb_mor P1 P2 t))).
Intros H H0; Apply (H h2 H0).
Intro H; Simpl; Apply (Prf_refl (Hom a b)).
(* 5 *)
Intro h2; LApply (PULB_32_ind ([t:(PULB_mor P3 P2)]
                                                 (Equal_PULB_mor P3 P2 F32 t) ->
                                                    g =%S (Fpulb_mor P3 P2 t))).
Intros H H0; Apply (H h2 H0).
Intro H; Simpl; Apply (Prf_refl (Hom c b)).
Qed.

Definition Fpulb_map := [x,y:PULB_ob]
           (build_Map (PULB_mor_setoid x y) ? (Fpulb_mor x y) (Fpulb_map_law x y)).

Lemma Fpulb_comp_law : (fcomp_law PULB C Fpulb_ob Fpulb_map).
Proof.
Unfold fcomp_law; Simpl; Intros x y z h1.
Elim h1; Simpl.
(* 1 *)
Elim z; Intro h2.
(* 1.1 *)
Simpl; LApply (PULB_11_ind ([t:(PULB_mor P1 P1)]
                         ((Id a) =%S ((Id a) o (Fpulb_mor P1 P1 t))))).
Intro H; Apply (H h2).
Simpl; Unfold Comp; Apply (Prf_idr C).
(* 1.2 *)
Simpl; LApply (PULB_12_ind ([t:(PULB_mor P1 P2)]
                         (f =%S ((Id a) o (Fpulb_mor P1 P2 t))))).
Intro H; Apply (H h2).
Simpl; Apply (Prf_sym (Hom a b)).
Unfold Comp; Apply (Prf_idl C).
(* 1.3 *)
Apply (PULB_13_ind ([t:(PULB_mor P1 P3)]
                              (Fpulb_mor P1 P3 (Comp_PULB_mor ? ? ? IP1 h2))
                              =%S ((Id a) o (Fpulb_mor P1 P3 h2))) h2).
(* 2 *)
Elim z; Intro h2.
(* 2.1 *)
Apply (PULB_21_ind ([a:(PULB_mor P2 P1)]
          (Fpulb_mor P2 P1 (Comp_PULB_mor ? ? ? IP2 h2)) =%S 
          ((Id b) o (Fpulb_mor P2 P1 h2))) h2).
(* 2.2 *)
Simpl; LApply (PULB_22_ind ([t:(PULB_mor P2 P2)]
                     (Id b) =%S ((Id b) o (Fpulb_mor P2 P2 t)))).
Intro H; Apply (H h2).
Simpl; Unfold Comp; Apply (Prf_idr C).
(* 2.3 *)
Apply (PULB_23_ind ([a:(PULB_mor P2 P3)]
         (Fpulb_mor P2 P3 (Comp_PULB_mor ? ? ? IP2 h2)) =%S 
         ((Id b) o (Fpulb_mor P2 P3 h2))) h2).
(* 3 *)
Elim z; Intro h2.
(* 3.1 *)
Apply (PULB_31_ind ([a:(PULB_mor P3 P1)]
         (Fpulb_mor P3 P1 (Comp_PULB_mor ? ? ? IP3 h2)) =%S 
         ((Id c) o (Fpulb_mor P3 P1 h2))) h2).
(* 3.2 *)
Simpl; Apply (PULB_32_ind ([a:(PULB_mor P3 P2)]
                        g =%S ((Id c) o (Fpulb_mor P3 P2 a)))).
Simpl; Apply (Prf_sym (Hom c b)).
Unfold Comp; Apply (Prf_idl C).
(* 3.3 *)
Simpl; LApply (PULB_33_ind ([t:(PULB_mor P3 P3)]
                     (Id c) =%S ((Id c) o (Fpulb_mor P3 P3 t)))).
Intro H; Apply (H h2).
Simpl; Unfold Comp; Apply (Prf_idr C).
(* 4 *)
Elim z; Intro h2.
(* 4.1 *)
Apply (PULB_21_ind ([a:(PULB_mor P2 P1)]
          (Fpulb_mor P1 P1 (Comp_PULB_mor ? ? ? F12 h2)) =%S 
          (f o (Fpulb_mor P2 P1 h2))) h2).
(* 4.2 *)
Simpl; LApply (PULB_22_ind ([t:(PULB_mor P2 P2)]
                     f =%S (f o (Fpulb_mor P2 P2 t)))).
Intro H; Apply (H h2).
Simpl; Unfold Comp; Apply (Prf_idr C).
(* 4.3 *)
Apply (PULB_23_ind ([a:(PULB_mor P2 P3)]
         (Fpulb_mor P1 P3 (Comp_PULB_mor ? ? ? F12 h2)) =%S 
         (f o (Fpulb_mor P2 P3 h2))) h2).
(* 5 *)
Elim z; Intro h2.
(* 5.1 *)
Apply (PULB_21_ind ([a:(PULB_mor P2 P1)]
          (Fpulb_mor P3 P1 (Comp_PULB_mor ? ? ? F32 h2)) =%S 
          (g o (Fpulb_mor P2 P1 h2))) h2).
(* 5.2 *)
Simpl; LApply (PULB_22_ind ([t:(PULB_mor P2 P2)]
                     g =%S (g o (Fpulb_mor P2 P2 t)))).
Intro H; Apply (H h2).
Simpl; Unfold Comp; Apply (Prf_idr C).
(* 5.3 *)
Apply (PULB_23_ind ([a:(PULB_mor P2 P3)]
         (Fpulb_mor P3 P3 (Comp_PULB_mor ? ? ? F32 h2)) =%S 
         (g o (Fpulb_mor P2 P3 h2))) h2).
Qed.

Lemma Fpulb_id_law : (fid_law PULB C Fpulb_ob Fpulb_map).
Proof.
Unfold fid_law; Simpl; Intro x.
Elim x; Simpl.
Apply (Prf_refl (Hom a a)).
Apply (Prf_refl (Hom b b)).
Apply (Prf_refl (Hom c c)).
Qed.

Definition fpulb := (Build_Functor PULB C Fpulb_ob Fpulb_map 
                                   Fpulb_comp_law Fpulb_id_law).

Definition pullback1 := (Fpullback1 fpulb).

Variable l : pullback1.

Definition fibred1_prod := (Fibred1_Fprod l).

Definition fibred1_p := (Fibred1_Fp l).

Definition fibred1_q := (Fibred1_Fq l).

Definition pb1_diese := (Pb1_Fdiese l).

End pullback_limit_def.

Syntactic Definition Fpulb := fpulb | 4.
Syntax constr fpulb <<(fpulb $1 $2 $3 $4 $f $g)>> 10
              [<hov 0> "Fpulb " <$f:"Term":L> " " <$g:"Term":L>].

Syntactic Definition Pullback1 := pullback1 | 4.
Syntax constr pullback1 <<(pullback1 $1 $2 $3 $4 $f $g)>> 10
              [<hov 0> "Pullback1 " <$f:"Term":L> " " <$g:"Term":L>].
    
Syntactic Definition Fibred1_prod := fibred1_prod | 6.
Syntax constr fibred1_prod <<(fibred1_prod $1 $2 $3 $4 $5 $6 $pb)>> 10
              [<hov 0> "Fibred1_prod " <$pb:"Term":L>].

Syntactic Definition Fibred1_p := fibred1_p | 6.
Syntax constr fibred1_p <<(fibred1_p $1 $2 $3 $4 $5 $6 $pb)>> 10
              [<hov 0> "Fibred1_p " <$pb:"Term":L>].

Syntactic Definition Fibred1_q := fibred1_q | 6.
Syntax constr fibred1_q <<(fibred1_q $1 $2 $3 $4 $5 $6 $pb)>> 10
              [<hov 0> "Fibred1_q " <$pb:"Term":L>].

Syntactic Definition Pb1_diese := pb1_diese | 6.
Syntax constr pb1_diese <<(pb1_diese $1 $2 $3 $4 $5 $6 $pb $r $t1 $t2 $pe)>> 10
              [<hov 0> "Pb1_diese " <$pb:"Term":L> " " <$r:"Term":L> " " 
                       <$t1:"Term":L> " " <$t2:"Term":L> " " <$pe:"Term":L>].

(* $Id: Pullbacks1.v,v 1.2 1996/02/12 15:25:47 cpaulin Exp $ *)
