(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                  Universal Arrows                  		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                            A.SAIBI   May 95				     *)
(*									     *)
(*****************************************************************************)

Require Export CatProperty.
Require Export Functor.

Section univarr.

Variable A, B : Category.
Variable b : (Ob B).
Variable G : (Functor A B).

Variable a : (Ob A).
Variable u : |(Hom b (FOb G a))|.

Variable diese : (a':(Ob A))|(Hom b (FOb G a'))| -> |(Hom a a')|.

Definition ua_eq := [a':(Ob A)][f:|(Hom b (FOb G a'))|]
                         [g:|(Hom a a')|] (u o (FMor G g)) =%S f.

Definition ua_law1 := (a':(Ob A))(f:|(Hom b (FOb G a'))|)
                           (ua_eq ? f (diese ? f)).

Definition ua_law2 := (a':(Ob A))(f:|(Hom b (FOb G a'))|)
  (g:|(Hom a a')|)(ua_eq ? f g) -> g =%S (diese ? f).

End univarr.

Syntactic Definition UA_eq := ua_eq | 5.
Syntax constr ua_eq <<(ua_eq $1 $2 $3 $4 $5 $u $a' $f' $g)>> 10
	     [<hov 0> "UA_eq " <$u:"term":L> " " <$a':"term":L> " " <$f':"term":L> 
                               " " <$g:"term":L>].

Syntactic Definition UA_law1 := ua_law1 | 5.
Syntax constr ua_law1 <<(ua_law1 $1 $2 $3 $4 $5 $u $d)>> 10
	     [<hov 0> "UA_law1 " <$u:"term":L> " " <$d:"term":L>].

Syntactic Definition UA_law2 := ua_law2 | 5.
Syntax constr ua_law2 <<(ua_law2 $1 $2 $3 $4 $5 $u $d)>> 10
	     [<hov 0> "UA_law2 " <$u:"term":L> " " <$d:"term":L>].

Structure isUA [A,B:Category;b:(Ob B);G:(Functor A B);a:(Ob A);u:|(Hom b (FOb G a))|] : Type := 
 {isUA_diese    : (a':(Ob A))|(Hom b (FOb G a'))|->|(Hom a a')|;
  Prf_isUA_law1 : (UA_law1 u isUA_diese);
  Prf_isUA_law2 : (UA_law2 u isUA_diese)}.

Syntactic Definition IsUA := isUA | 2.
Syntax constr isUA <<(isUA $1 $2 $b $G $a $u)>> 10
	     [<hov 0> "IsUA " <$b:"term":L> " " <$G:"term":L>
                      " " <$a:"term":L> " " <$u:"term":L> ].

Syntactic Definition IsUA_diese := isUA_diese | 6.
Syntax constr isUA_diese <<(isUA_diese $1 $2 $3 $4 $5 $6 $t $a' $f)>> 10
	     [<hov 0> "IsUA_diese " <$t:"term":L> " " <$a':"term":L> " " <$f:"term":L>].

Structure ua [A,B:Category;b:(Ob B);G:(Functor A B)] : Type :=
 {ua_ob    : (Ob A);
  ua_mor   : |(Hom b (FOb G ua_ob))|;
  prf_IsUA : (IsUA b G ua_ob ua_mor)}.

Syntactic Definition UA := ua | 2.
Syntax constr ua <<(ua $1 $2 $b $G)>> 10
	     [<hov 0> "UA " <$b:"term":L> " " <$G:"term":L>].

Syntactic Definition UA_ob := ua_ob | 4.
Syntax constr ua_ob <<(ua_ob $1 $2 $3 $4 $t1)>> 10
	     [<hov 0> "UA_ob " <$t1:"term":L>].

Syntactic Definition UA_mor := ua_mor | 4.
Syntax constr ua_mor <<(ua_mor $1 $2 $3 $4 $t1)>> 10
	     [<hov 0> "UA_mor " <$t1:"term":L>].

Syntactic Definition Prf_IsUA := prf_IsUA | 4.
Syntax constr prf_IsUA <<(prf_IsUA $1 $2 $3 $4 $t1)>> 10
	     [<hov 0> "Prf_IsUA " <$t1:"term":L>].

 Section proj_ua.

 Variable A ,B : Category.
 Variable b    : (Ob B).
 Variable G    : (Functor A B).
 Variable u    : (UA b G).

 Definition ua_diese := (IsUA_diese (Prf_IsUA u)).
 
 Lemma Prf_UA_law1 : (UA_law1 (UA_mor u) ua_diese).
 Proof.
 Unfold ua_diese.
 Exact (Prf_isUA_law1 ? ? ? ? ? ? (Prf_IsUA u)). 
 Qed.

 Lemma Prf_UA_law2 : (UA_law2 (UA_mor u) ua_diese).
 Proof.
 Unfold ua_diese.
 Exact (Prf_isUA_law2 ? ? ? ? ? ? (Prf_IsUA u)). 
 Qed.

 (* f=g => f# = g# *)
 
 Lemma Diese_map : (a':(Ob A))(Map_law (ua_diese a')).
 Proof.
 Intros a'.
 Unfold map_law; Intros f g H.
 Apply (Prf_UA_law2 a' g (ua_diese a' f)).
 Unfold ua_eq.
 (* *) Apply (Prf_trans (Hom b (FOb G a'))) with y:=f.
 Apply (Prf_UA_law1 a' f).
 Assumption.
 Qed.

 End proj_ua.

Syntactic Definition UA_diese := ua_diese | 4.
Syntax constr ua_diese <<(ua_diese $1 $2 $3 $4 $u $a' $f)>> 10
      	      [<hov 0> "UA_diese " <$u:"term":L> " " <$a':"term":L> " " <$f:"term":L>].


Section ua_iso_def.

Variable A, B   : Category.
Variable b      : (Ob B).
Variable G      : (Functor A B).
Variable u1, u2 : (UA b G).

Definition UA_iso_mor := (UA_diese u1 ? (UA_mor u2)).

Definition UA_iso_mor_1 := (UA_diese u2 ? (UA_mor u1)).

Lemma UA_iso_law1 : (Iso_law ? ? ? UA_iso_mor UA_iso_mor_1).
Proof.
Unfold Iso_law.
(* *) Apply (Prf_trans (Hom (UA_ob u2) (UA_ob u2))) with
  y:=(UA_diese u2 ? (UA_mor u2)).
Apply (Prf_UA_law2 ? ? ? ? u2 ? (UA_mor u2)
        ((UA_diese u2 ? (UA_mor u1)) o (UA_diese u1 ? (UA_mor u2)))).
Unfold ua_eq.
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u2)))) with
  y:=(UA_mor u2) o ((FMor G (UA_diese u2 ? (UA_mor u1))) o
                   (FMor G (UA_diese u1 ? (UA_mor u2)))).
Apply (Prf_congl B).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u2)))) with
  y:=((UA_mor u2) o (FMor G (UA_diese u2 ? (UA_mor u1)))) o
                   (FMor G (UA_diese u1 ? (UA_mor u2))).
Unfold Comp; Apply (Prf_ass B).
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u2)))) with
  y:=(UA_mor u1) o (FMor G (UA_diese u1 ? (UA_mor u2))).
Apply (Prf_congr B).
Apply (Prf_UA_law1 ? ? ? ? u2 ? (UA_mor u1)).
Apply (Prf_UA_law1 ? ? ? ? u1 ? (UA_mor u2)).
Apply (Prf_sym (Hom (UA_ob u2) (UA_ob u2))).
Apply (Prf_UA_law2 ? ? ? ? u2 ? (UA_mor u2) (Id (UA_ob u2))).
Unfold ua_eq.
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u2)))) with
  y:=(UA_mor u2) o (Id (FOb G (UA_ob u2))).
Apply (Prf_congl B).
Unfold fMor; Apply (Prf_Fid_law ? ? G).
Apply (Prf_sym (Hom b (FOb G (UA_ob u2)))).
Unfold Comp; Apply (Prf_idr B).
Qed.

Lemma UA_iso_law2 : (Iso_law ? ? ? UA_iso_mor_1 UA_iso_mor).
Proof.
Unfold Iso_law.
(* *) Apply (Prf_trans (Hom (UA_ob u1) (UA_ob u1))) with
  y:=(UA_diese u1 ? (UA_mor u1)).
Apply (Prf_UA_law2 ? ? ? ? u1 ? (UA_mor u1)
        ((UA_diese u1 ? (UA_mor u2)) o (UA_diese u2 ? (UA_mor u1)))).
Unfold ua_eq.
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u1)))) with
  y:=(UA_mor u1) o ((FMor G (UA_diese u1 ? (UA_mor u2))) o
                   (FMor G (UA_diese u2 ? (UA_mor u1)))).
Apply (Prf_congl B).
Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u1)))) with
  y:=((UA_mor u1) o (FMor G (UA_diese u1 ? (UA_mor u2)))) o
                   (FMor G (UA_diese u2 ? (UA_mor u1))).
Unfold Comp; Apply (Prf_ass B).
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u1)))) with
  y:=(UA_mor u2) o (FMor G (UA_diese u2 ? (UA_mor u1))).
Apply (Prf_congr B).
Apply (Prf_UA_law1 ? ? ? ? u1 ? (UA_mor u2)).
Apply (Prf_UA_law1 ? ? ? ? u2 ? (UA_mor u1)).
Apply (Prf_sym (Hom (UA_ob u1) (UA_ob u1))).
Apply (Prf_UA_law2 ? ? ? ? u1 ? (UA_mor u1) (Id (UA_ob u1))).
Unfold ua_eq.
(* *) Apply (Prf_trans (Hom b (FOb G (UA_ob u1)))) with
  y:=(UA_mor u1) o (Id (FOb G (UA_ob u1))).
Apply (Prf_congl B).
Unfold fMor; Apply (Prf_Fid_law ? ? G).
Apply (Prf_sym (Hom b (FOb G (UA_ob u1)))).
Unfold Comp; Apply (Prf_idr B).
Qed.

Lemma UA_isIso : (IsIso UA_iso_mor).
Proof.
Exact (Build_isIso ? ? ? UA_iso_mor UA_iso_mor_1 UA_iso_law1 UA_iso_law2).
Defined.

Definition UA_iso := (Build_iso ? (UA_ob u1) (UA_ob u2) UA_iso_mor UA_isIso).

End ua_iso_def.

(* $Id: UniversalArrow.v,v 1.2 1996/02/12 15:25:49 cpaulin Exp $ *)
