(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	   A sufficient condition for a Comma to be Complete        	     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Pres_Limits.
Require Export Comma_proj.

Section comma_complete.

Variable X,A : Category.
Variable G   : (Functor A X).
Variable J   : Category.

Hypothesis A_comp_for_J : (F:(Functor J A))(Limit F).
Hypothesis G_pres_JA    : (F:(Functor J A))(Preserves_limits F G).

Variable x : (Ob X).
Variable F : (Functor J (Comma G x)).

Local F' := F o%F (Comma_proj G x).

Local l_F' := (A_comp_for_J F').

Local l_GF' := (G_pres_JA F' l_F').

(* construction d'un cone Delta(x)->F' o G *)

Definition Tc_tau := [i:(Ob J)](Mor_com_ob (FOb F i)).

Lemma Tc_tau_cone_law : (Cone_law x (F' o%F G) Tc_tau).
Proof.
Unfold cone_law; Intros i j g.
Apply (Prf_com_law ? ? G x ? ? (FMor F g)).
Qed.

Definition Tc_cone := (Build_Cone x (F' o%F G) Tc_tau Tc_tau_cone_law).

(* limite objet pour F *)

Definition Tc_limF := (Build_Com_ob G x (Lim l_F') (IsCoUA_diese l_GF' ? Tc_cone)).

(* limiting cone pour F *)

Lemma Tc_limconeF_tau_com_law : (i:(Ob J))
      (Com_law Tc_limF (FOb F i) (ApNT (Limiting_cone l_F') i)).
Proof.
Unfold com_law; Intro i.
Apply (Prf_sym (Hom x (FOb G (Ob_com_ob (FOb F i))))).
Exact (Prf_isCoUA_law1 ? ? ? ? ? ? l_GF' ? Tc_cone i).
Qed.

Definition Tc_limconeF_tau := [i:(Ob J)]
  (Build_Com_arrow Tc_limF (FOb F i) (ApNT (Limiting_cone l_F') i)
                     (Tc_limconeF_tau_com_law i)).

Lemma Tc_limconeF_cone_law : (cone_law J (Comma G x) Tc_limF F Tc_limconeF_tau).
Proof.
Unfold cone_law; Simpl.
Intros i j g.
Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Apply (Eq_cone ? ? ? ? (Limiting_cone l_F') i j g).
Qed.

Definition Tc_limconeF := (build_Cone J (Comma G x) Tc_limF F Tc_limconeF_tau 
                                      Tc_limconeF_cone_law).

(* operation # *)

 Section ctdiese.
  
 Variable axf : (Ob (Comma G x)).
 Variable t   : (cone J (Comma G x) axf F).

 Definition Tc_t_tau := [i:(Ob J)](Mor_com_arrow (ApNT t i)).

 Lemma Tc_t_tau_cone_law : (Cone_law (Ob_com_ob axf) F' Tc_t_tau).
 Proof.
 Unfold cone_law; Intros i j g.
 Generalize (Eq_cone J (Comma G x) axf F t i j g).
 Intro H.
 Apply (Prf_trans (Hom (Ob_com_ob axf) (FOb F' j))) with
  y:=(FMor (Comma_proj G x) (Comp (Comma G x) ? ? ? (ApNT t i) (FMor F g))).
 Exact (Prf_FMap_law (Comma G x) A (Comma_proj G x) axf (FOb F j) 
              (ApNT t j) (Comp (Comma G x) ? ? ? (ApNT t i) (FMor F g)) H).
 Exact (Prf_Fcomp_law (Comma G x) A (Comma_proj G x) axf (FOb F i) (FOb F j) 
              (ApNT t i) (FMor F g)).
 Qed.

 Definition Tc_t_cone := (Build_Cone (Ob_com_ob axf) F' Tc_t_tau Tc_t_tau_cone_law).

 Lemma Tc_t_cone_com_law : (Com_law axf Tc_limF (CoUA_diese l_F' ? Tc_t_cone)).
 Proof.
 Unfold com_law.
 Apply (Prf_sym (Hom x (FOb G (Ob_com_ob Tc_limF)))).
 Apply (Prf_isCoUA_law2 X (CatFunct J X) ((F o%F (Comma_proj G x)) o%F G) 
                     (Delta X J) ? ? l_GF' ? Tc_cone).
 Unfold coUA_eq 1 fMor; Simpl.
 Unfold Delta_mor Equal_NT; Simpl.
 Intro i; Unfold Comp_tau; Simpl.
 Unfold Delta_mor_tau Comp_cone_tau.
 Apply (Prf_trans (Hom x (FOb G (FOb (Comma_proj G x) (FOb F i)))))
  with y:=(Mor_com_ob axf) o 
          ((FMor G (coUA_diese A (CatFunct J A) F' (Delta A J) l_F'
                           (Ob_com_ob axf) Tc_t_cone)) o 
           (FMor G (ApNT (Limiting_cone l_F') i))).
 Apply (Prf_sym (Hom x (FOb G (FOb (Comma_proj G x) (FOb F i))))).
 Unfold Comp ;Apply (Prf_ass X).
 Apply (Prf_trans (Hom x (FOb G (FOb (Comma_proj G x) (FOb F i)))))
  with y:=(Mor_com_ob axf) o (FMor G (Tc_t_tau i)).
 Apply (Prf_congl X).
 Apply (Prf_trans (Hom (FOb G (Ob_com_ob axf)) 
                       (FOb G (Ob_com_ob (FOb F i)))))
  with y:=(FMor G ((coUA_diese A (CatFunct J A) F' (Delta A J) l_F'
                      (Ob_com_ob axf) Tc_t_cone) o
                   (ApNT (Limiting_cone l_F') i))).
 Apply (Prf_sym (Hom (FOb G (Ob_com_ob axf)) 
                       (FOb G (Ob_com_ob (FOb F i))))).
 Unfold fMor; Apply (Prf_Fcomp_law ? ? G).
 Unfold fMor; Apply (Prf_FMap_law ? ? G (Ob_com_ob axf) 
             (Ob_com_ob (FOb F i))). 
 Apply (Prf_sym (Hom (Ob_com_ob axf) (Ob_com_ob (FOb F i)))).
 Apply (Eq_limit ? ? ? l_F' ? Tc_t_cone i).
 Apply (Prf_sym (Hom x (FOb G (FOb (Comma_proj G x) (FOb F i))))).
 Apply (Prf_com_law ? ? G x ? ? (ApNT t i)).
 Qed.

 Definition Tc_diese := (Build_Com_arrow axf Tc_limF (CoUA_diese l_F' ? Tc_t_cone) 
                                         Tc_t_cone_com_law).

 End ctdiese.
  

Lemma Tc_UA_law1 : (Limit_law1 F Tc_limF Tc_limconeF Tc_diese). 
Proof.
Unfold limit_law1 coUA_law1 coUA_eq; Intros axf t.
Unfold 1 fMor; Simpl.
Unfold Delta_mor Equal_NT; Simpl.
Intro i; Unfold Comp_tau; Simpl.
Unfold Delta_mor_tau Comp_cone_tau.
Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Apply (Prf_sym (Hom (Ob_com_ob axf) (Ob_com_ob (FOb F i)))).
Apply (Eq_limit ? ? ? l_F' ? (Tc_t_cone axf t) i).
Qed.

Lemma Tc_UA_law2 : (Limit_law2 F Tc_limF Tc_limconeF Tc_diese). 
Proof.
Unfold limit_law2 coUA_law2 coUA_eq; Intros axf t g.
Unfold 1 fMor; Simpl.
Unfold Delta_mor Equal_NT; Simpl.
Unfold Comp_tau; Simpl.
Unfold Delta_mor_tau Comp_cone_tau.
Unfold Equal_com_arrow; Simpl.
Unfold Comp_com_mor; Simpl.
Intro H.
Apply (Prf_CoUA_law2 ? ? ? ? l_F' ? (Tc_t_cone axf t)).
Exact H.
Qed.

Lemma Comma_l_F : (IsLimit F Tc_limF Tc_limconeF).
Proof.
Exact (Build_IsLimit F Tc_limF Tc_limconeF Tc_diese Tc_UA_law1 Tc_UA_law2). 
Defined.

Definition Comma_l_F1 := (Build_Limit F Tc_limF Tc_limconeF Comma_l_F).

End comma_complete.


Lemma Comma_complete : (X,A:Category)(G:(Functor A X))
                       (Complete A) -> (Continuous G) -> 
                       (x:(Ob X))(Complete (Comma G x)).
Proof.
Intros X A G cA cG x.
Unfold Complete; Intros J F.
Exact (Comma_l_F1 X A G J (cA J) (cG J) x F).
Defined.


        

(* $Id: Comma_Complete.v,v 1.2 1996/02/12 15:25:57 cpaulin Exp $ *)
