(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	             HomFoncteur  Hom(a,-) is Complete        		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)


Require Export HomFunctor.
Require Export Pres_Limits.

(* exemple : les Hom-Foncteurs preservent toutes les Limites *)

Section funset_pres.

Variable J, C : Category.
Variable F    : (Functor J C).
Variable l    : (Limit F).
Variable c    : (Ob C).

Definition FS_lim := (Hom c (Lim l)).

Definition FS_lcone := (Limiting_cone l) o%C (FunSET c).

 Section fs_diese.

(* soit X un setoid, soit tau : cone(X,FoH) (i.e. Delta(X) -> FoH) 
   pour tout x:|X|, nous construisons un 
   Tx : cone(c,F) (i.e. Delta(c) -> F)
   telque Tx(i) = tau(i)(x) *)

 Variable X   : Setoid.
 Variable tau : (Cone X (F o%F (FunSET c))).

  Section fs_diese_mor_def.

  Variable x : |X|.

  Definition FS_cone_tau := [i:(Ob J)] (Ap (ApNT tau i) x).

  (* revient a` prouver: tau(j)(x) = tau(i)(x) o Fg *)
  (* s'obtient apre`s simplification de Eq_cone sur tau *)

  Lemma FS_cone_tau_cone_law : (Cone_law c F FS_cone_tau).
  Proof.
  Unfold cone_law FS_cone_tau.
  Intros i j g.
  Apply (Eq_cone ? ? ? ? tau i j g x).
  Qed.

  Definition FS_cone := (Build_Cone c F FS_cone_tau FS_cone_tau_cone_law).

 (* soit X un setoid, soit tau : cone(X,FoH) (i.e. Delta(X) -> FoH) 
    construisons tau# : X -> C(c,(Lim F))
    de'finissons tau#(x) par (Tx)# car l est une (Limit F) *)

  Definition FS_diese_mor := (CoUA_diese l ? FS_cone).

  End fs_diese_mor_def.  

 Lemma FS_diese_map_law : (Map_law FS_diese_mor).
 Proof.
 Unfold map_law; Intros x y H.
 Unfold FS_diese_mor.
 Apply (Codiese_map ? ? ? ? l ? (FS_cone x) (FS_cone y)).
 Simpl; Unfold Equal_NT; Intro i; Simpl.
 Unfold FS_cone_tau.
 Apply (Pres ? ? (ApNT tau i)).
 Assumption.
 Qed.

 Definition FS_diese := (Build_Map FS_diese_mor FS_diese_map_law).

End fs_diese.

(* ve'rifions que le # qu'on vient de construire peut former
   un (co)universal arrow *)
(* revient a` prouver: 
   qq X un setoid, qq tau : cone(X,FoH) (i.e. Delta(X) -> FoH) 
   qq i : J, qq x : |X|.
   tau(i)(x) = tau#(x) o mu(i) 
   i.e. Tx(i) = (Tx)# o mu(i)  qui n'est rien d'autre que eq_Ucone 
                               pour le cone Tx *)

Lemma FS_coUAlaw1 : (Limit_law1 (F o%F (FunSET c)) FS_lim FS_lcone FS_diese).
Proof.
Unfold limit_law1 coUA_law1 coUA_eq 1 fMor; Simpl.
Unfold Equal_NT; Intros X tau i.
Unfold Delta_mor; Simpl.
Unfold Ext; Intro x.
Unfold Comp_tau Comp_cone; Simpl.
Unfold FunSET_mor1 FS_diese_mor; Simpl.
Apply (Prf_sym (FunSET_ob C c (FOb F i))).
Apply (Eq_limit ? ? ? l ? (FS_cone X tau x) i).
Qed.

(* revient a` prouver que:
   f(x) o mu(i) = (Tx)i  =>
   f(x) = (Tx)#
   correspondant a` l'axiome d'universalite' de l *)
   
Lemma FS_coUAlaw2 : (Limit_law2 (F o%F (FunSET c)) FS_lim FS_lcone FS_diese).
Proof.
Unfold limit_law2 coUA_law2 ; Intros X tau f.
Unfold coUA_eq FS_diese 1 fMor; Simpl.
Unfold Delta_mor Equal_NT; Simpl.
Unfold Ext; Simpl.
Unfold FunSET_mor1 FS_diese_mor Delta_mor_tau.
Intros H x.
Apply (Prf_CoUA_law2 C (CatFunct J C) F (Delta C J) l ? 
           (FS_cone X tau x) (Ap f x)).
Unfold coUA_eq 1 fMor; Simpl.
Unfold Delta_mor Equal_NT; Simpl.
Unfold Comp_tau; Simpl.
Intro i; Apply (H i x).
Qed.

Lemma FunSET_Preserves_l : (Preserves_1limit (FunSET c) l).
Proof.
Exact (Build_IsLimit (F o%F (FunSET c)) (Hom c (Lim l))
         ((Limiting_cone l) o%C (FunSET c)) FS_diese FS_coUAlaw1 FS_coUAlaw2).
Defined.

End funset_pres.


(* on en d'eduit que C(c,-) Preserve toutes les Limites *)

Lemma FunSET_continuous : (C:Category)(c:(Ob C))(Continuous (FunSET c)). 
Proof.
Unfold continuous preserves_limits; Intros C c J F l.
Exact (FunSET_Preserves_l ? ? ? l c).
Defined.



(* $Id: HomFunctor_Continuous.v,v 1.2 1996/02/12 15:26:00 cpaulin Exp $ *)
