(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	             Preservation of Limits               		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)


Require Export Limit.

(* Preservation de limites *)

(* F : J -> C *)
(* T : Delta(r) -> F *)
(* H : C -> D *)
(* T o H : Delta(r) -> F o H *)

Section comp_cone.

Variable J,C,D : Category.
Variable c     : (Ob C).
Variable F     : (Functor J C).
Variable T     : (Cone c F).
Variable G     : (Functor C D).

Definition Comp_cone_tau := [i:(Ob J)](FMor G (ApNT T i)).

(* revient a` prouver que : H(T(j)) = H(T(i)) o H(F g) *)

Lemma Comp_cone_tau_cone_law : (Cone_law (FOb G c) (F o%F G) Comp_cone_tau).
Proof.
Unfold cone_law; Intros i j g; Unfold Comp_cone_tau.
Unfold 3 fMor; Simpl.
Unfold comp_FMor.
(* *) Apply (Prf_trans (Hom (FOb G c) (FOb G (FOb F j)))) with 
  y:=(FMor G ((ApNT T i) o (FMor F g))).
Unfold fMor; Apply (Prf_FMap_law ? ? G c (FOb F j)).
Apply (Eq_cone ? ? ? ? T ? ? g).
Unfold 1  3 4 fMor.
Apply (Prf_Fcomp_law ? ? G ? ? ? (ApNT T i) (FMor F g)).
Qed.

Definition Comp_cone := (Build_Cone (FOb G c) (F o%F G) Comp_cone_tau Comp_cone_tau_cone_law).

End comp_cone.

Token "%C".

Syntax constr Comp_cone <<(Comp_cone $1 $2 $3 $4 $5 $T $H)>> 10
		[<hov 0> <$T:"Term":L> [<hov 0> " o%C " <$H:"Term":L>]].

Grammar command command2 := [ command1($T) "o" "%C" command2($G) ] -> 
                            [$0 = <<(Comp_cone ? ? ? ? ? $T $G)>>].

(* H : C -> D preserve toutes les limites du foncteur F : J -> C *)

Section def_pres_limits.

Variable J, C, D : Category.
Variable F       : (Functor J C).
Variable G       : (Functor C D).

Definition preserves_1limit := [l:(Limit F)]
       (IsLimit ? (FOb G (Lim l)) ((Limiting_cone l) o%C G)).

Definition preserves_limits := (l:(Limit F))(preserves_1limit l).

End def_pres_limits.


Syntactic Definition Preserves_1limit := preserves_1limit | 4.
Syntax constr preserves_1limit <<(preserves_1limit $1 $2 $3 $4 $G $l)>> 10
              [<hov 0> "Preserves_1limit " <$G:"Term":L> " " <$l:"Term":L>].

Syntactic Definition Preserves_limits := preserves_limits | 3.
Syntax constr preserves_limits <<(preserves_limits $1 $2 $3 $F $G)>> 10
              [<hov 0> "Preserves_limits " <$F:"Term":L> " " <$G:"Term":L>].

Definition continuous := [C,D:Category][G:(Functor C D)]
                         (J:Category)(F:(Functor J C))
                         (Preserves_limits F G).

Syntactic Definition Continuous := continuous | 2.
Syntax constr continuous <<(continuous $1 $2 $G)>> 10
              [<hov 0> "Continuous " <$G:"Term":L>].


(* $Id: Pres_Limits.v,v 1.2 1996/02/12 15:26:02 cpaulin Exp $ *)
