(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                   SET is Complete                		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)


Require Export SET.
Require Export Single.
Require Export Limit.


Section set_c.

Variable J : Category.
Variable F : (Functor J SET).

Definition SET_lim := (cones J SET Single F).

 Section set_lcone_tau_def.

 Variable i : (Ob J).

 Definition SET_lcone_tau_fun := [sigma:(cone J SET Single F)]
                                 (Ap (ApNT sigma i) Elt).

 Lemma SET_lcone_tau_map_law : (map_law SET_lim (FOb F i) SET_lcone_tau_fun).
 Proof.
 Unfold map_law; Intros s1 s2 H.
 Unfold SET_lcone_tau_fun.
 Apply (H i Elt).
 Qed.

 Definition SET_lcone_tau := (build_Map SET_lim (FOb F i) SET_lcone_tau_fun
                                        SET_lcone_tau_map_law).

 End set_lcone_tau_def.

Lemma SET_lcone_cone_law : (cone_law J SET SET_lim F SET_lcone_tau).
Proof.
Unfold cone_law; Simpl.
Intros i j g; Unfold Ext; Simpl.
Intros sigma.
Unfold SET_lcone_tau SET_lcone_tau_fun Comp_fun Id_fun.
Simpl.
Apply (Eq_cone J SET ? F sigma i j g Elt).
Qed.

Definition SET_lcone := (build_Cone J SET SET_lim F SET_lcone_tau SET_lcone_cone_law).


(* construction tau# pour tout tau : DElta(X) -> F *)

 Section set_diese.

 Variable X   : Setoid.
 Variable tau : (cone J SET X F).

  Section set_diese1_def.

  Variable x : |X|.

   Section set_diese1_tau_def.

   Variable i : (Ob J).

   (* (h(x)i)(Elt) =  tau(i)(x) *)

   Definition SET_diese1_tau_fun := [a:|Single|]
              <|(FOb F i)|>Case a of (* Elt *) (Ap (ApNT tau i) x) end.

   Lemma SET_diese1_tau_map_law : (map_law Single (FOb F i) SET_diese1_tau_fun).
   Proof.
   Unfold map_law.
   Intros a b; Elim a; Elim b; Simpl; Intro H.
   Apply (Prf_refl (FOb F i)).
   Qed.

   Definition SET_diese1_tau := (build_Map Single (FOb F i) SET_diese1_tau_fun
                                           SET_diese1_tau_map_law).
 
   End set_diese1_tau_def.

 (* h(x) : DElta({*}) -> F *)

  Lemma SET_diese1_tau_cone_law : (cone_law J SET Single F SET_diese1_tau).
  Proof.
  Unfold cone_law; Intros i j g; Simpl.
  Unfold Ext; Simpl.
  Intro a; Elim a; Simpl.
  Unfold Comp_fun; Simpl.
  Apply (Eq_cone ? SET ? F tau i j g x).
  Qed.

  Definition SET_diese1 := (build_Cone J SET Single F SET_diese1_tau SET_diese1_tau_cone_law).

  End set_diese1_def.

 Lemma SET_diese_map_law : (map_law X SET_lim SET_diese1).
 Proof.
 Unfold map_law; Intros x y H; Simpl.
 Unfold Equal_NT SET_diese1; Simpl.
 Unfold Ext; Simpl; Intros i a.
 Elim a; Simpl.
 Apply (Pres ? ? (ApNT tau i)); Trivial.
 Qed.

 Definition SET_diese := (build_Map X SET_lim SET_diese1 SET_diese_map_law).

 End set_diese.
 
(* universalite' *)

Lemma SET_coUAlaw1 : (Limit_law1 F SET_lim SET_lcone SET_diese).
Proof.
Unfold limit_law1 coUA_law1 coUA_eq; Simpl.
Unfold Equal_NT; Intros X tau i.
Simpl; Unfold Ext; Simpl.
Intro x; Unfold SET_lcone_tau_fun.
Unfold SET_diese1; Simpl.
Apply (Prf_refl (FOb F i)).
Qed.

Lemma SET_coUAlaw2 : (Limit_law2 F SET_lim SET_lcone SET_diese).
Proof.
Unfold limit_law2 coUA_law2 coUA_eq; Simpl.
Unfold Equal_NT; Intros X tau g. 
Unfold Ext 1 fMor; Simpl.
Unfold Ext; Simpl.
Unfold SET_lcone_tau_fun Delta_mor_tau.
Unfold Equal_NT; Simpl.
Unfold Ext; Simpl.
Intros H x i a.
Elim a; Simpl.
Apply (H i x).
Qed.

Lemma SET_Limit : (IsLimit F SET_lim SET_lcone).
Proof.
Exact (Build_IsLimit F SET_lim SET_lcone SET_diese SET_coUAlaw1 SET_coUAlaw2).
Defined.

End set_c.

Lemma SET_is_complete : (Complete SET).
Proof.
Unfold Complete; Intros J F.
Exact (Build_Limit F (SET_lim ? F) 
                   (SET_lcone ? F) (SET_Limit ? F)). 
Defined.


(* $Id: SET_Complete.v,v 1.2 1996/02/12 15:26:04 cpaulin Exp $ *)
