(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	    Existence of an Initial Object                 		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                       A.SAIBI         May 95				     *)
(*									     *)
(*****************************************************************************)


Require Export Products1.
Require Export Equalizers1.
Require Export Single.

Section ssc1.

Variable D : Category.

Structure Cond1 [I:Type;k:I->(Ob D);d:(Ob D)] : Type :=
 {Cond1_i : I;
  Cond1_f : |(Hom (k Cond1_i) d)|}.

Structure SSC1 :Type :=
 {SSC1_I : Type;
  SSC1_k : SSC1_I->(Ob D);
  SSC1_p : (d:(Ob D))(Cond1 SSC1_I SSC1_k d)}.

Definition SSC1_i := [s:SSC1][d:(Ob D)]
                     (Cond1_i (SSC1_I s) (SSC1_k s) d (SSC1_p s d)).

Definition SSC1_f := [s:SSC1][d:(Ob D)]
                     (Cond1_f (SSC1_I s) (SSC1_k s) d (SSC1_p s d)).

End ssc1.

(* => *)

Section thi1.

Variable D          : Category.
Variable D_initial  : (Initial D).

Definition Thi1_d1 := (Initial_ob D_initial).

Definition Thi1_d1_is_initial := (Prf_isInitial ? D_initial).

(* trouver I qui verifie SSC1 *)

Definition Thi1_I := UnitType.

Definition Thi1_k := [i:Thi1_I]<(Ob D)>Case i of (* Elt *) Thi1_d1 end.

(* pour tout d, Ex i et f : ki -> d *)
(* prendre Elt pour i, on a ki qui est d1, donc on cherche *)
(* f : d1 -> d, prendre le morphisme d'initialite' *)

Definition Thi1_verif_Cond1 := [d:(Ob D)]
           (Build_Cond1 D Thi1_I Thi1_k d Elt (MorI Thi1_d1_is_initial d)).

Definition Thi1_SSC1 := (Build_SSC1 D Thi1_I Thi1_k Thi1_verif_Cond1).

End thi1.

(* <= *)

Section thi2.

Variable D          : Category.
Variable D_complete : (Complete D).
Variable SSC1_D     : (SSC1 D). 

(* comme D est complet, elle admet aussi les produits *)

Definition Thi2_D_prod := (D_complete (Discr (SSC1_I ? SSC1_D))
                               (FunDiscr (SSC1_k ? SSC1_D))).

Definition Thi2_w := (PI1 Thi2_D_prod).

(* D admet aussi tous les equalizers *)
(* soit v l'equalizer de (Hom w w) *)

Definition Thi2_D_E_hom := (D_complete (J_hom Thi2_w Thi2_w) (F_hom Thi2_w Thi2_w)).

Definition Thi2_v := (E1_ob (k_hom D Thi2_w Thi2_w) Thi2_D_E_hom).

Definition Thi2_e := (E1_mor (k_hom D Thi2_w Thi2_w) Thi2_D_E_hom).
           
(* construire une fle`che de v vers d qque *)

 Section thi2_mor_d_def.

 Variable d : (Ob D).

 Definition Thi2_p_i := (Proj1 Thi2_D_prod (SSC1_i ? SSC1_D d)).
 
 Definition Thi2_f_d := (SSC1_f ? SSC1_D d).   

 Definition Thi2_mor_d := (Thi2_e o Thi2_p_i) o Thi2_f_d.

 End thi2_mor_d_def. 

(* prouver qu'entre v et d qque, il existe au plus un morphisme *)

 Section unique_mor_d.
 
 Variable d    : (Ob D).
 Variable f, g : |(Hom Thi2_v d)|.
 
 (* equalizer de f et g *) 

 Definition Thi2_D_E_fg := (D_complete (J_fg f g) (F_fg f g)).

 Definition Thi2_c := (E1_ob (K_fg f g) Thi2_D_E_fg).
 
 Definition Thi2_e1 := (E1_mor (K_fg f g) Thi2_D_E_fg).
 
 (* s : w -> c *) 

 Definition Thi2_p_j := (Proj1 Thi2_D_prod (SSC1_i ? SSC1_D Thi2_c)).

 Definition Thi2_f_c := (SSC1_f ? SSC1_D Thi2_c).   

 Definition Thi2_s := Thi2_p_j o Thi2_f_c.

 (* s o e1 o e : w -> w *) 
 (* ainsi que Id(w) : w -> w *)
 (* comme e est l'equalizer des fle`ches de C(w,w), on a:
    e o s o e1 o e = e o Id (w) *)
 
 (* ((e o s) o e1) = (Id v) *)

 Lemma Thi2_e1_RightInv : (RightInv Thi2_e1).
 Proof.
 Apply Build_rightInv with (Thi2_e o Thi2_s).
 Unfold Iso_law; Simpl.
 (* *) Apply (Prf_trans (Hom Thi2_v Thi2_v)) with y :=Thi2_e o (Thi2_s o Thi2_e1).
 Apply (Prf_sym (Hom Thi2_v Thi2_v)).
 Unfold Comp; Apply (Prf_ass D).
 Apply (Monic_l ? ? ? ? (E_monic D Thi2_w Thi2_w (I_hom Thi2_w Thi2_w) 
         (k_hom D Thi2_w Thi2_w)
         (Equalizer1_to_Equalizer D Thi2_w Thi2_w (I_hom Thi2_w Thi2_w) 
                (k_hom D Thi2_w Thi2_w) Thi2_D_E_hom (Id Thi2_w)))).
 Simpl.
 (* *) Apply (Prf_trans (Hom Thi2_v Thi2_w)) with 
       y:=Thi2_e o ((Thi2_s o Thi2_e1) o Thi2_e).
 Apply (Prf_sym (Hom Thi2_v Thi2_w)).
 Unfold Comp; Apply (Prf_ass D).
 (* *) Apply (Prf_trans (Hom Thi2_v Thi2_w)) with y:=Thi2_e o (Id Thi2_w).
 Apply (Prf_E1_law1 D Thi2_w Thi2_w (I_hom Thi2_w Thi2_w) (k_hom D Thi2_w Thi2_w) 
                  Thi2_D_E_hom ((Thi2_s o Thi2_e1) o Thi2_e) (Id Thi2_w)).
 (* *) Apply (Prf_trans (Hom Thi2_v Thi2_w)) with y:= Thi2_e.
 Apply (Prf_sym (Hom Thi2_v Thi2_w)). 
 Unfold Comp; Apply (Prf_idr D).
 Apply (Prf_sym (Hom Thi2_v Thi2_w)). 
 Unfold Comp; Apply (Prf_idl D).
 Qed.
 
 (* on en de'duit que e1 est iso et donc f=g *)
 
 Lemma Eq_fg : f =%S g.
 Proof.
 Generalize (Equalizer_iso D Thi2_v d TwoElts (K_fg f g)
        (Equalizer1_to_Equalizer D Thi2_v d TwoElts (K_fg f g) Thi2_D_E_fg Elt1) 
        Thi2_e1_RightInv).  
 Intro H; Elim H.
 Intros e1_1 H2 H3.
 (* *) Apply (Prf_trans (Hom Thi2_v d)) with y:= (e1_1 o Thi2_e1) o f.
 (* *) Apply (Prf_trans (Hom Thi2_v d)) with y:= (Id Thi2_v) o f.
 Apply (Prf_sym (Hom Thi2_v d)).
 Unfold Comp; Apply (Prf_idl D).
 Apply (Prf_congr D).
 Apply (Prf_sym (Hom Thi2_v Thi2_v)); Apply H2.
 (* *) Apply (Prf_trans (Hom Thi2_v d)) with y:= (e1_1 o Thi2_e1) o g.
 (* *) Apply (Prf_trans (Hom Thi2_v d)) with y:= e1_1 o (Thi2_e1 o f). 
 Apply (Prf_sym (Hom Thi2_v d)).
 Unfold Comp; Apply (Prf_ass D).
 (* *) Apply (Prf_trans (Hom Thi2_v d)) with y:= e1_1 o (Thi2_e1 o g). 
 Apply (Prf_congl D).
 Apply (Prf_E1_law1 D Thi2_v d TwoElts (K_fg f g) Thi2_D_E_fg Elt1 Elt2).
 Unfold Comp; Apply (Prf_ass D).
 (* *) Apply (Prf_trans (Hom Thi2_v d)) with y:=(Id Thi2_v) o g.
 Apply (Prf_congr D); Apply H2.
 Unfold Comp; Apply (Prf_idl D).
 Qed.

End unique_mor_d.
 
Lemma Thi2_isInitial : (IsInitial Thi2_v).
Proof.
Exact (Build_isInitial D Thi2_v Thi2_mor_d Eq_fg). 
Defined.

Definition Thi2_initial := (Build_Initial D Thi2_v Thi2_isInitial).

End thi2.

(* $Id: Th_Initial.v,v 1.2 1996/02/12 15:26:07 cpaulin Exp $ *)
