(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          The Functor Category                                             *)
(*                                                                           *)
(*          Amokrane Saibi,  May 1994                                        *)
(*                                                                           *)
(*****************************************************************************)


Require Export Ntransformation.

(* Functor Category *)

Section cat_functor.

Variable C, D : Category.

 Section compnt.

 Variable F, G, H : (Functor C D).
 Variable  T      : (NT F G).
 Variable T'      : (NT G H).

 Definition Comp_tau := [a:(Ob C)](ApNT T a) o (ApNT T' a).

 Lemma Comp_tau_nt_law : (NT_law ? ? [a:(Ob C)](Comp_tau a)).
 Proof.
 Unfold Comp_tau; Unfold nt_law.
 Intros a b f.
 (* F(f)o((tauFG b)o(tauGH b)) = ((tauFG a)o(tauGH a))oH(f) *)
 (* *) Apply (Prf_trans (Hom (FOb F a) (FOb H b)))
 with y:=((FMor F f) o (ApNT T b)) o (ApNT T' b).
 (* F(f)o((ApNT T b)o(ApNT T' b)) = (F(f)o(ApNT T b))o(ApNT T' b) *)
 Unfold Comp; Apply (Prf_ass D).
 (* (F(f)o(ApNT T' b))o(ApNT T' b) = ((ApNT T a)o(ApNT T' a))oH(f) *) 
 (* *) Apply (Prf_trans (Hom (FOb F a) (FOb H b)))
  with y:=((ApNT T a) o (FMor G f)) o (ApNT T' b).        
 (* (F(f)o(ApNT T' b))o(ApNT T' b) = ((ApNT T a)oG(f))o(ApNT T' b) *) 
 Apply (Prf_congr D); Apply (Prf_NT_law ? ? ? ? T).
 (*  ((ApNT T a)oG(f))o(ApNT T' b) = ((ApNT T a)o(ApNT T' a))oH(f) *) 
 (* *) Apply (Prf_trans (Hom (FOb F a) (FOb H b)))
   with y:=(ApNT T a) o ((FMor G f) o (ApNT T' b)).        
 (* ((ApNT T a)oG(f))o(ApNT T' b) = (ApNT T a)o(G(f)o(ApNT T' b)) *)
 Apply (Prf_sym (Hom (FOb F a) (FOb H b))).
 Unfold Comp; Apply (Prf_ass D).
 (* (ApNT T a)o(G(f)o(ApNT T' b)) = ((ApNT T a)o(ApNT T' a))oH(f) *) 
 (* *) Apply (Prf_trans (Hom (FOb F a) (FOb H b)))
   with y:=(ApNT T a) o ((ApNT T' a) o (FMor H f)).
 (* (ApNT T a)o(G(f)o(ApNT T' b)) = (ApNT T a)o((ApNT T' a)oH(f)) *)
 Apply (Prf_congl D).
 (* G(f)o(ApNT T' b) = (ApNT T' a)oH(f) *)
 Apply (Prf_NT_law ? ? ? ? T').
 (* (ApNT T a)o(G(f)o(ApNT T' b)) = ((ApNT T a)o(G(f))o(ApNT T' b) *)
 Unfold Comp; Apply (Prf_ass D).
 Qed.

 Definition CompV_NT := (Build_NT ? ? Comp_tau Comp_tau_nt_law).

 End compnt.

Lemma CompV_NT_congl : (Congl_law (Functor C D) (NT_setoid C D) CompV_NT).  
Proof.
Unfold Congl_law; Simpl.
Unfold Equal_NT CompV_NT; Simpl.
Intros F G H T1 T2 T3 eqT1_T2 a; Unfold Comp_tau.
Apply (Prf_congl D); Apply eqT1_T2.
Qed.

Lemma CompV_NT_congr : (Congr_law (Functor C D) (NT_setoid C D) CompV_NT).  
Proof.
Unfold Congr_law; Simpl.
Unfold Equal_NT CompV_NT; Simpl.
Intros F G H T1 T2 T3 eqT1_T2 a; Unfold Comp_tau.  
Apply (Prf_congr D); Apply eqT1_T2.
Qed.

Definition Comp_CatFunct := (Build_Comp (Functor C D) (NT_setoid C D) 
                                CompV_NT CompV_NT_congl CompV_NT_congr).
   
Lemma Assoc_CatFunct : (assoc_law (Functor C D) (NT_setoid C D) Comp_CatFunct).
Proof.
Unfold assoc_law; Intros a b c f; Elim f; Simpl.
Intros f1 f2 f3 f4 f5 g; Elim g.
Intros T e h; Elim h.
Intros T1 e1.
Unfold Equal_NT; Simpl.
Unfold Comp_CatFunct; Simpl.
Unfold Comp_tau; Simpl.
Unfold Comp_tau; Simpl; Intro a0.
Unfold Comp; Apply (Prf_ass D).
Qed.

(* Id *)

 Section id_catfunct_def.

 Variable F : (Functor C D).

 Definition Id_CatFunct_tau := [a:(Ob C)](Id (FOb F a)).
 
 Lemma Id_CatFunct_nt_law: (NT_law ? ? Id_CatFunct_tau).
 Proof.
 Unfold nt_law Id_CatFunct_tau; Intros a b f. 
 (* *) Apply (Prf_trans (Hom (FOb F a) (FOb F b))) with y:=(FMor F f).
 Apply (Prf_sym (Hom (FOb F a) (FOb F b))).
 Unfold Comp; Apply (Prf_idr D).
 Apply (Prf_sym (Hom (FOb F a) (FOb F b))).
 Unfold Comp; Apply (Prf_idl D).
 Qed.
  
 Definition Id_CatFunct := (Build_NT ? ? Id_CatFunct_tau Id_CatFunct_nt_law).

 End id_catfunct_def.

Lemma Idl_CatFunct : (idl_law (Functor C D) (NT_setoid C D) 
                                          Comp_CatFunct Id_CatFunct).
Proof.
Red; Induction f.
Unfold Comp_CatFunct; Unfold Id_CatFunct; Simpl.
Unfold Equal_NT; Simpl.
Unfold Comp_tau; Simpl.
Unfold Id_CatFunct_tau; Intros.
Unfold Comp; Apply (Prf_idl D).
Qed.

Lemma Idr_CatFunct : (idr_law (Functor C D) (NT_setoid C D) 
                                 Comp_CatFunct Id_CatFunct).
Proof.
Red; Induction f.
Unfold Comp_CatFunct; Unfold Id_CatFunct; Simpl.
Unfold Equal_NT; Simpl.
Unfold Comp_tau; Simpl.
Unfold Id_CatFunct_tau; Intros.
Unfold Comp; Apply (Prf_idr D).
Qed.

Definition CatFunct :=
  (Build_Category (Functor C D) (NT_setoid C D) Comp_CatFunct 
                  Id_CatFunct Assoc_CatFunct 
                  Idl_CatFunct Idr_CatFunct).

End cat_functor.

Token "%NTv".

Syntax constr CompV_NT <<(CompV_NT $1 $2 $3 $4 $5 $f $g)>> 10
		[<hov 0> <$f:"Term":L> [<hov 0> " o%NTv " <$g:"Term":L>]].

Grammar command command2 :=
  [ command1($T) "o" "%NTv" command2($T') ] -> 
        [$0 = <<(CompV_NT ? ? ? ? ? $T $T')>>].


(* $Id: CatFunct.v,v 1.2 1996/02/12 15:26:18 cpaulin Exp $ *)
