(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*  Category Theory : Natural Transformations between Hom-Functors           *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export HomFunctor.
Require Export Ntransformation.

(* Natural transformation Y(f):C(b,-)-->C(a,-), with f:a-->b in C *)

Section funset_nt.

Variable C    : Category.
Variable b, a : (Ob C).
Variable f    : |(Hom a b)|.

Section nth_map_def.

 Variable c : (Ob C).

 Definition NtH_arrow := [h:|(Hom b c)|] f o h.

 Lemma NtH_map_law : (Map_law NtH_arrow).
 Proof.
 Unfold map_law NtH_arrow; Intros.
 Apply (Prf_congl C); Assumption.
 Qed.

 Definition NtH_map := (Build_Map NtH_arrow NtH_map_law).

End nth_map_def.

Lemma NtH_nt_law : (NT_law (FunSET b) (FunSET a) NtH_map).
Proof.
Unfold nt_law; Simpl.
Unfold Ext; Simpl.
Unfold Comp_fun; Simpl.
Unfold NtH_arrow FunSET_mor1 FunSET_ob.
Intros; Unfold Comp; Apply (Prf_ass C).
Qed.

Definition ntH := (Build_NT (FunSET b) (FunSET a) NtH_map NtH_nt_law).

End funset_nt.

Syntactic Definition NtH := ntH | 3.
Syntax constr ntH <<(ntH $1 $2 $3 $fo)>> 10
	     [<hov 0> "NtH " <$fo:"term":L>].

(* $Id: HomFunctor_NT.v,v 1.2 1996/02/12 15:26:19 cpaulin Exp $ *)
