(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          InterChange Law                                                  *)
(*                                                                           *)
(*          Ge'rard Huet and Amokrane Saibi,  June 1994                      *)
(*                                                                           *)
(*****************************************************************************)

Require Export CatFunct. 

Section horz_comp.

Variable A, B, C : Category.
Variable F, G    : (Functor A B).
Variable F', G'  : (Functor B C).
Variable T       : (NT F G).
Variable T'      : (NT F' G').

Definition Ast : (a:(Ob A))|(Hom (FOb (F o%F F') a) (FOb (G o%F G') a))| := 
           [a:(Ob A)](ApNT T' (FOb F a)) o (FMor G' (ApNT T a)).

Lemma Ast_eq : (a:(Ob A)) ((FMor F' (ApNT T a)) o (ApNT T' (FOb G a))) =%S 
                          ((ApNT T' (FOb F a)) o (FMor G' (ApNT T a))).
Proof.
Intro a; Apply (Prf_NT_law ? ? ? ? T' ? ? (ApNT T a)).
Qed.

Lemma Ast_nt_law : (NT_law (F o%F F') (G o%F G') Ast).
Proof.
Unfold nt_law Ast Comp_Functor.
Unfold 1 fMor; Simpl.
Unfold 3 fMor; Simpl.
Unfold comp_FMor.
Intros a b f.
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb G G' b)))
  with y:=((FMor F' (ApNT T a)) o (ApNT T' (FOb G a))) o (FMor G' (FMor G f)).
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb G G' b)))
  with y:=(FMor F' (FMor F f)) o ((FMor F' (ApNT T b)) o (ApNT T' (FOb G b))).
Apply (Prf_congl C).
Apply (Prf_sym (Hom (FOb F' (FOb F b)) (FOb G' (FOb G b)))).
Apply (Ast_eq b).
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb G G' b)))
  with y:=(FMor F' (ApNT T a)) o ((FMor F' (FMor G f)) o (ApNT T' (FOb G b))).
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb G G' b)))
  with y:=((FMor F' (FMor F f)) o (FMor F' (ApNT T b))) o (ApNT T' (FOb G b)).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb G G' b)))
  with y:=((FMor F' (ApNT T a)) o (FMor F' (FMor G f))) o (ApNT T' (FOb G b)).
Apply (Prf_congr C).
(* commutation du diagramme de gauche par fonctorialite' de F'
   et Naturalite' de T (p.18) *)
(* *) Apply (Prf_trans (Hom (FOb F' (FOb F a))(FOb F' (FOb G b))))
  with y:=(FMor F' ((FMor F f) o (ApNT T b))).
Apply (Prf_sym (Hom (FOb F' (FOb F a)) (FOb F' (FOb G b)))).
Unfold 1 3 5 fMor; Apply (Prf_Fcomp_law B C F').     
(* *) Apply (Prf_trans (Hom (FOb F' (FOb F a)) (FOb F' (FOb G b))))
  with y:=(FMor F' ((ApNT T a) o (FMor G f))).
(* Fonctorialite' de F' *)
Unfold 1 3 fMor.
Apply (Prf_FMap_law B C F' (FOb F a) (FOb G b)).
(* T Naturelle *)
Apply (Prf_NT_law ? ? ? ? T).
Unfold 1 3 4 fMor; Apply (Prf_Fcomp_law B C F').
Apply (Prf_sym (Hom (Comp_FOb F F' a) (Comp_FOb G G' b))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb G G' b)))
  with y:= (FMor F' (ApNT T a)) o ((ApNT T' (FOb G a)) o (FMor G' (FMor G f))).
Apply (Prf_congl C).
Apply (Prf_NT_law ? ? ? ? T').
Unfold Comp; Apply (Prf_ass C).
Apply (Prf_congr C).
Apply (Ast_eq a).
Qed.

Definition CompH_NT := (Build_NT (F o%F F') (G o%F G') Ast Ast_nt_law).

End horz_comp.

Token "%NTh".

Syntax constr CompH_NT <<(CompH_NT $1 $2 $3 $4 $5 $6 $7 $f $g)>> 10
		[<hov 0> <$f:"Term":L> [<hov 0> " o%NTh " <$g:"Term":L>]].

Grammar command command2 :=
  [ command1($T1) "o" "%NTh" command2($T2) ] -> 
        [$0 = <<(CompH_NT ? ? ? ? ? ? ? $T1 $T2)>>].

Section interchangelaw.

Variable A, B, C    : Category.
Variable F, G, H    : (Functor A B).
Variable F', G', H' : (Functor B C).
Variable T          : (NT F G).
Variable T'         : (NT F' G').
Variable U          : (NT G H).
Variable U'         : (NT G' H').

Lemma InterChange_law : ((T o%NTh T') o%NTv (U o%NTh U')) =%NT 
                        ((T o%NTv U) o%NTh (T' o%NTv U')).
Proof.
Unfold Equal_NT; Intro a; Simpl.
Unfold CompH_NT Comp_Functor ; Simpl.
Unfold Ast Comp_tau; Simpl.
Unfold Comp_tau.
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb H H' a)))
  with y:=((ApNT T' (FOb F a)) o (ApNT U' (FOb F a))) o
          ((FMor H' (ApNT T a)) o (FMor H' (ApNT U a))).
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb H H' a)))
  with y:=(ApNT T' (FOb F a)) o ((FMor G' (ApNT T a)) o
          ((ApNT U' (FOb G a)) o (FMor H' (ApNT U a)))).
Apply (Prf_sym (Hom (Comp_FOb F F' a) (Comp_FOb H H' a))).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (Comp_FOb F F' a) (Comp_FOb H H' a)))
  with y:=(ApNT T' (FOb F a)) o ((ApNT U' (FOb F a))o
          ((FMor H' (ApNT T a)) o (FMor H' (ApNT U a)))).
Apply (Prf_congl C).
(* *) Apply (Prf_trans (Hom (FOb G' (FOb F a)) (FOb H' (FOb H a))))
 with y:=((FMor G' (ApNT T a)) o (ApNT U' (FOb G a))) o (FMor H' (ApNT U a)).
Unfold Comp; Apply (Prf_ass C).
(* *) Apply (Prf_trans (Hom (FOb G' (FOb F a)) (FOb H' (FOb H a))))
  with y:=((ApNT U' (FOb F a)) o (FMor H' (ApNT T a))) o (FMor H' (ApNT U a)).
Apply (Prf_congr C).
Apply (Prf_NT_law ? ? ? ? U').
Apply (Prf_sym (Hom (FOb G' (FOb F a)) (FOb H' (FOb H a)))).
Unfold Comp; Apply (Prf_ass C).
Unfold Comp; Apply (Prf_ass C).
Apply (Prf_congl C).
Apply (Prf_sym (Hom (FOb H' (FOb F a)) (FOb H' (FOb H a)))).
Unfold CompV_NT Comp_tau; Simpl.
Unfold fMor; Apply (Prf_Fcomp_law B C H').
Qed.
                                           
End interchangelaw.


(* o%NTh is associative *)

Definition EqualH_NT := [C,D:Category][F,G:(Functor C D)]
                        [F',G':(Functor C D)][T:(NT F G)][T':(NT F' G')]
                        (a:(Ob C)) (ApNT T a) =%H (ApNT T' a).

Token "=%NTH".

Syntax constr EqualH_NT <<(EqualH_NT $1 $2 $3 $4 $5 $6 $c1 $c2)>> 1
     [<hov 0>  <$c1:"Command":L> " =%NTH " <$c2:"Command":L>].

Grammar command command1 :=
  [ command0($T) "=%NTH" command0($T') ] -> 
         [$0 = <<(EqualH_NT ? ? ? ? ? ? $T $T')>>].

Section assoc_horz_comp.

Variable A,B,C,D : Category.
Variable F,G     : (Functor A B).
Variable F',G'   : (Functor B C).
Variable F'',G'' : (Functor C D).
Variable T       : (NT F G).
Variable T'      : (NT F' G').
Variable T''     : (NT F'' G'').

Lemma CompH_NT_assoc : ((T o%NTh T') o%NTh T'') =%NTH (T o%NTh (T' o%NTh T'')).
Proof.
Unfold EqualH_NT CompH_NT; Simpl.
Unfold Ast; Simpl.
Unfold comp_FOb.
Unfold 4 fMor; Simpl.
Unfold comp_FMor.
Intro a.
Apply Build_Equal_hom.
(* *) Apply (Prf_trans (Hom (Comp_FOb F' F'' (FOb F a)) (Comp_FOb G' G'' (FOb G a))))
  with y:=(ApNT T'' (FOb F' (FOb F a))) o ((FMor G'' (ApNT T' (FOb F a)))
                                          o (FMor G'' (FMor G' (ApNT T a)))).
Apply (Prf_congl D).
Unfold 1 3 4 fMor; Apply (Prf_Fcomp_law ? ? G'').     
Unfold Comp; Apply (Prf_ass D).
Qed.

End assoc_horz_comp.



(* $Id: InterChangeLaw.v,v 1.2 1996/02/12 15:26:21 cpaulin Exp $ *)
