(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                   Natural Isomorphism            		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)
  

Require Export CatFunct.
Require Export CatProperty.

Section nat_iso.

Variable A, B : Category.
Variable F, G : (Functor A B).

Definition isNatIso := [T:(NT F G)](isIso (CatFunct A B) ? ? T).

Definition natIso := (iso (CatFunct A B) F G).

End nat_iso.

Syntactic Definition IsNatIso := isNatIso | 4.
Syntax constr isNatIso <<(isNatIso $1 $2 $3 $4 $T)>> 10
		[<hov 0> "IsNatIso " <$T:"Term":L>].

Syntactic Definition NatIso := natIso | 2.
Syntax constr natIso <<(natIso $1 $2 $F $G)>> 10
		[<hov 0> "NatIso " <$F:"Term":L> " " <$G:"Term":L>].


Section about_isIso.

Variable A, B : Category.
Variable F, G : (Functor A B).
Variable T    : (NT F G).

(* qq a. Ta isIso => T isIso *)

Hypothesis H : (a:(Ob A))(IsIso (ApNT T a)).
   
Definition NTinv_tau := [a:(Ob A)](Inv_iso (H a)).

Lemma NTinv_nt_law : (NT_law ? ? NTinv_tau).
Proof.
Unfold nt_law NTinv_tau; Intros a b f.
(* *) Apply (Prf_trans (Hom (FOb G a) (FOb F b))) with
 y:=((Inv_iso (H a)) o ((ApNT T a) o (FMor G f))) o (Inv_iso (H b)).
Apply (Prf_congr B).
(* *) Apply (Prf_trans (Hom (FOb G a) (FOb G b))) with
  y:= ((Inv_iso (H a)) o (ApNT T a)) o (FMor G f).
(* *) Apply (Prf_trans (Hom (FOb G a) (FOb G b))) with
  y:= (Id (FOb G a)) o (FMor G f).
Apply (Prf_sym (Hom (FOb G a) (FOb G b))).
Unfold Comp; Apply (Prf_idl B).
Apply (Prf_congr B).
Apply (Prf_sym (Hom (FOb G a) (FOb G a))).
Apply (Idl_inv ? ? ? ? (H a)).
Apply (Prf_sym (Hom (FOb G a) (FOb G b))).
Unfold Comp; Apply (Prf_ass B).
(* *) Apply (Prf_trans (Hom (FOb G a) (FOb F b))) with
  y:= ((Inv_iso (H a)) o ((FMor F f) o (ApNT T b))) o (Inv_iso (H b)).
Apply (Prf_congr B).
Apply (Prf_congl B).
Apply (Prf_sym (Hom (FOb F a) (FOb G b))).
Apply (Prf_NT_law ? ? ? ? T ? ? f).
(* *) Apply (Prf_trans (Hom (FOb G a) (FOb F b))) with
  y:=(Inv_iso (H a)) o (((FMor F f) o (ApNT T b)) o (Inv_iso (H b))).
Apply (Prf_sym (Hom (FOb G a) (FOb F b))).
Unfold Comp; Apply (Prf_ass B).
Apply (Prf_congl B).
(* *) Apply (Prf_trans (Hom (FOb F a) (FOb F b))) with
  y:= (FMor F f) o ((ApNT T b) o (Inv_iso (H b))).
Apply (Prf_sym (Hom (FOb F a) (FOb F b))).
Unfold Comp; Apply (Prf_ass B).
(* *) Apply (Prf_trans (Hom (FOb F a) (FOb F b))) with
  y:= (FMor F f) o (Id ?).
Apply (Prf_congl B).
Apply (Idr_inv ? ? ? ? (H b)).
Apply (Prf_sym (Hom (FOb F a) (FOb F b))).
Unfold Comp; Apply (Prf_idr B).
Qed.

Definition NTinv := (Build_NT ? ? NTinv_tau NTinv_nt_law).

Lemma Check_idl : (Iso_law (CatFunct A B) F G T NTinv).
Proof.
Unfold Iso_law; Simpl; Unfold Equal_NT CompV_NT NTinv; Simpl.
Unfold Comp_tau; Simpl.
Unfold NTinv_tau Id_CatFunct_tau.
Intro a.
Apply (Idl_inv ? ? ? ? (H a)).
Qed.

Lemma Check_idr : (Iso_law (CatFunct A B) G F NTinv T).
Proof.
Unfold Iso_law; Simpl; Unfold Equal_NT CompV_NT NTinv; Simpl.
Unfold Comp_tau; Simpl.
Unfold NTinv_tau Id_CatFunct_tau.
Intro a.
Apply (Idr_inv ? ? ? ? (H a)).
Qed.

Lemma NT_isIso : (IsNatIso T).
Proof.
Exact (Build_isIso (CatFunct A B) F G T NTinv Check_idl Check_idr).
Defined.

(* T isIso => qq a. Ta isIso *)

Hypothesis H' : (IsNatIso T).

Definition Tinv := (inv_iso (CatFunct A B) F G T H').

 Section nat_iso_to_iso.

 Variable a : (Ob A).

 Definition Ta_inv := (ApNT Tinv a).

 Lemma Check_Ta_idl : (Iso_law ? ? ? (ApNT T a) Ta_inv).
 Proof.
 Unfold Iso_law.
 Apply (Idl_inv (CatFunct A B) F G T H' a).
 Qed.

 Lemma Check_Ta_idr : (Iso_law ? ? ? Ta_inv (ApNT T a)).
 Proof.
 Unfold Iso_law.
 Apply (Idr_inv (CatFunct A B) F G T H' a).
 Qed.

 Lemma NTa_isIso : (IsIso (ApNT T a)).
 Proof.
 Exact (Build_isIso ? ? ? (ApNT T a) Ta_inv  
                                  Check_Ta_idl Check_Ta_idr).
 Defined.

 End nat_iso_to_iso.

End about_isIso.




(* $Id: NatIso.v,v 1.2 1996/02/12 15:26:23 cpaulin Exp $ *)
