(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          Category Theory : Natural Transformations                        *)
(*                                                                           *)
(*          Amokrane Saibi May 1994                                          *)
(*                                                                           *)
(*****************************************************************************)

Require Export Functor.

(* Natural Transformations *)

Section nat_transf.

Variable C, D : Category.
Variable F, G : (Functor C D).

Variable T : (a:(Ob C))|(Hom (FOb F a) (FOb G a))|.

Definition nt_law := (a,b:(Ob C))(f:|(Hom a b)|)
                     ((FMor F f) o (T b)) =%S ((T a) o (FMor G f)).

End nat_transf.

Syntactic Definition NT_law := nt_law | 2.
Syntax constr NT_law <<(nt_law $1 $2 $F $G $tau)>> 10
  [<hov 0> "NT_law " <$F:"Term":L> " " <$G:"Term":L> " " <$tau:"Term":L>].

Structure nt [C,D:Category;F,G:(Functor C D)] : Type := build_NT
 {apNT       : (a:(Ob C))|(Hom (FOb F a) (FOb G a))|;
  Prf_NT_law : (NT_law ? ? apNT)}.

Syntactic Definition Build_NT := build_NT | 2.
Syntax constr build_NT <<(build_NT $1 $2 $F $G $T $p)>> 10
		[<hov 0> "Build_NT " <$F:"Term":L> " " <$G:"Term":L> " " 
                                     <$T:"Term":L> " " <$p:"Term":L>].

Syntactic Definition NT := nt | 2.
Syntax constr nt <<(nt $1 $2 $F $G)>> 10
		[<hov 0> "NT " <$F:"Term":L> " " <$G:"Term":L>].

Syntactic Definition ApNT := apNT | 4.
Syntax constr apNT <<(apNT $1 $2 $3 $4 $T $a)>> 10
   [<hov 0> "ApNT " <$T:"Term":L> " " <$a:"Term":L>].


(* Natural Transformations Setoid between F and G*)

Section setoid_nt.

Variable C, D : Category.
Variable F, G : (Functor C D).

Definition Equal_NT := [T,T':(NT F G)](a:(Ob C))(ApNT T a) =%S (ApNT T' a).

Lemma Equal_NT_equiv : (Equivalence Equal_NT).
Proof.
Apply Build_equivalence; Unfold Equal_NT.
Unfold Reflexive; Intros T a; Elim T; 
  Unfold apNT; Simpl; Intros. 
Apply (Prf_refl (Hom (FOb F a) (FOb G a))).
Unfold Transitive; Intros T1 T2 T3; Elim T1; Elim T2; Elim T3.
Unfold apNT; Simpl; Intros tau n tau0 n0 tau1 n1 H H0 a.
(* *) Apply (Prf_trans (Hom (FOb F a) (FOb G a))) with 
  y:=(tau0 a); Auto.
Unfold Symmetric; Intros T1 T2; Elim T1; Elim T2.
Unfold apNT; Simpl; Intros tau n tau0 n0 H a.
Apply (Prf_sym (Hom (FOb F a) (FOb G a))); Apply H.
Qed.

Definition NT_setoid := (Build_Setoid (NT F G) Equal_NT Equal_NT_equiv).

End setoid_nt.

Token "=%NT".

Syntax constr Equal_NT <<(Equal_NT $1 $2 $3 $4 $c1 $c2)>> 1
     [<hov 0>     <$c1:"Command":L> " =%NT " <$c2:"Command":L>].

Grammar command command1 :=
  [ command0($T) "=%NT" command0($T') ] -> 
         [$0 = <<(Equal_NT ? ? ? ? $T $T')>>].

Syntax constr NT_setoid <<(NT_setoid $1 $2 $F $G)>> 10
     [<hov 0>   "{"  <$F:"Command":L> " => " <$G:"Command":L> "}"].

Grammar command command1 :=
  [ "{" command0($F) "=>" command0($G) "}" ] -> 
         [$0 = <<(NT_setoid ? ? $F $G)>>].



(* $Id: Ntransformation.v,v 1.2 1996/02/12 15:26:25 cpaulin Exp $ *)
