(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Coherence.v                                *)
(****************************************************************************)

Hint refl_eqT. (* TO BE REMOVED *)
Require Relations.
Require Confluence.

Theorem Rstar_reflexive :
  (U: Type) (R: (Relation U)) (Reflexive U (Rstar U R)).
Goal.
Auto.
Save.

Theorem Rplus_contains_R :
   (U: Type)(R: (Relation U))(Contains U (Rplus U R) R).
Goal.
Auto.
Save.

Theorem Rstar_contains_R :
    (U: Type)(R: (Relation U))(Contains U (Rstar U R) R).
Goal.
(Intros U R; Red); Intros x y H'; Apply Rstar_n with y; Auto.
Save.

Theorem Rstar_contains_Rplus :
    (U: Type)(R: (Relation U))(Contains U (Rstar U R) (Rplus U R)).
Goal.
Intros U R; Red.
Intros x y H'; Elim H'.
(Generalize Rstar_contains_R; Intro T; Red in T); Auto.
(Intros x0 y0 z H'0 H'1 H'2; Apply Rstar_n with y0); Auto.
Save.

Theorem Rstar_transitive :
    (U: Type)(R: (Relation U)) (Transitive U (Rstar U R)).
Goal.
Intros U R; Red.
(Intros x y z H'; Elim H'); Auto.
Intros x0 y0 z0 H'0 H'1 H'2 H'3; Apply Rstar_n with y0; Auto.
Save.

Theorem Rstar_cases :
   (U: Type)(R: (Relation U))(x,y: U)(Rstar U R x y) ->
            x==y \/  (ExT [u: U] (R x u) /\ (Rstar U R u y)).
Goal.
Intros U R x y H'; Elim H'; Auto.
Intros x0 y0 z H'0 H'1 H'2; Right; Exists y0; Auto.
Save.

Theorem Rstar_equiv_Rstar1 : 
  (U: Type)(R: (Relation U)) (Same_relation U (Rstar U R) (Rstar1 U R)).
Goal.
Generalize Rstar_contains_R; Intro T; Red in T.
Intros U R; Unfold Same_relation Contains.
Split; Intros x y H'; Elim H'; Auto.
Generalize Rstar_transitive; Intro T1; Red in T1.
(Intros x0 y0 z H'0 H'1 H'2 H'3; Apply T1 with y0); Auto.
(Intros x0 y0 z H'0 H'1 H'2; Apply Rstar1_n with y0); Auto.
Save.

Theorem Rsym_imp_Rstarsym :
  (U: Type)(R: (Relation U)) (Symmetric U R) -> (Symmetric U (Rstar U R)).
Goal.
Intros U R H'; Red.
(Intros x y H'0; Elim H'0); Auto.
Intros x0 y0 z H'1 H'2 H'3.
Generalize Rstar_transitive; Intro T; Red in T.
Apply T with y0; Auto.
Apply Rstar_n with x0; Auto.
Save.

Theorem Sstar_contains_Rstar :
   (U: Type) (R,S: (Relation U)) (Contains U (Rstar U S) R) ->
            (Contains U (Rstar U S) (Rstar U R)).
Goal.
Unfold Contains.
(Intros U R S H' x y H'0; Elim H'0); Auto.
Generalize Rstar_transitive; Intro T; Red in T.
(Intros x0 y0 z H'1 H'2 H'3; Apply T with y0); Auto.
Save.

Theorem star_monotone :
    (U: Type) (R,S: (Relation U)) (Contains U S R) ->
            (Contains U (Rstar U S) (Rstar U R)).
Goal.
Intros U R S H'.
Apply Sstar_contains_Rstar.
Generalize (Rstar_contains_R U S); Auto.
Save.

Theorem RstarRplus_RRstar :
  (U: Type)(R: (Relation U)) (x,y,z: U) (Rstar U R x y) -> (Rplus U R y z) ->
             (ExT [u: U] (R x u) /\ (Rstar U R u z)).
Goal.
Generalize Rstar_contains_Rplus; Intro T; Red in T.
Generalize Rstar_transitive; Intro T1; Red in T1.
Intros U R x y z H'; Elim H'.
Intros x0 H'0; Elim H'0.
Intros; Exists y0; Auto.
Intros; Exists y0; Auto.
Intros; Exists y0; Auto.
Split; [Try Assumption | Idtac].
Apply T1 with z0; Auto.
Save.

Theorem Rstar_imp_coherent :
    (U: Type) (R: (Relation U)) (x: U) (y: U) (Rstar U R x y) ->
              (coherent U R x y).
Goal.
Intros U R x y H'; Red.
Exists y; Auto.
Save.
Hint Rstar_imp_coherent.

Theorem coherent_symmetric :
   (U: Type) (R: (Relation U)) (Symmetric U (coherent U R)).
Goal.
Unfold 1 coherent.
Intros U R; Red.
Intros x y h; Elim h; Intros z h0; Elim h0; Intros H' H'0; Clear h h0.
Exists z; Auto.
Save.

(* $Id: Coherence.v,v 1.2 1996/02/12 15:26:48 cpaulin Exp $ *)
