(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Confluence.v                               *)
(****************************************************************************)

Require Relations.

Section Confluence.
   Variable U: Type.
   Variable R: (Relation U).
   
   Inductive Rstar : (Relation U) :=
        Rstar_0: (x: U) (Rstar x x)
      | Rstar_n: (x,y,z: U) (R x y) -> (Rstar y z) -> (Rstar x z).
   
   Inductive Rstar1 : (Relation U) :=
        Rstar1_0: (x: U) (Rstar1 x x)
      | Rstar1_1: (x: U) (y: U) (R x y) -> (Rstar1 x y)
      | Rstar1_n: (x,y,z: U) (Rstar1 x y) -> (Rstar1 y z) -> (Rstar1 x z).
   
   Inductive Rplus : (Relation U) :=
        Rplus_0: (x,y: U) (R x y) -> (Rplus x y)
      | Rplus_n: (x,y,z: U) (R x y) -> (Rplus y z) -> (Rplus x z).
   
   Definition Strongly_confluent : Prop :=
      (x,a,b: U) (R x a) -> (R x b) ->  (ExT [z: U] (R a z) /\ (R b z)).

   Definition coherent : U -> U -> Prop :=
      [x,y: U]  (ExT [z: U] (Rstar x z) /\ (Rstar y z)).
   
   Definition locally_confluent : U -> Prop :=
      [x: U] (y: U) (z: U) (R x y) -> (R x z) -> (coherent y z).
   
   Definition Locally_confluent : Prop :=  (x: U) (locally_confluent x).
   
   Definition confluent : U -> Prop :=
      [x: U] (y: U) (z: U) (Rstar x y) -> (Rstar x z) -> (coherent y z).
   
   Definition Confluent : Prop := (x: U) (confluent x).
   
End Confluence.
   
Hint Rstar_0.
Hint Rstar1_0.
Hint Rstar1_1.
Hint Rplus_0.



(* $Id: Confluence.v,v 1.2 1996/02/12 15:26:51 cpaulin Exp $ *)
