(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Relations.v                                *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	    Relations over a Type : Orderings              		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*          F. Prost, G. Kahn, G. Huet	May 94				     *)
(*									     *)
(*****************************************************************************)

Require Export Structure. 

Section Orderings.
   Variable U: Type.
   
   Definition Relation :=  U -> U -> Prop.

   Variable R: Relation.
   
   Definition Reflexive :=  (x: U) (R x x).
   
   Definition Transitive :=  (x,y,z: U) (R x y) -> (R y z) -> (R x z).
   
   Definition Symmetric :=  (x,y: U) (R x y) -> (R y x).
   
   Definition Antisymmetric :=
      (x,y: U) (R x y) -> (R y x) -> x==y.
   
   Definition Contains :=
      [R,R': Relation] (x,y: U) (R' x y) -> (R x y).
   
   Definition Same_relation :=
      [R,R': Relation] (Contains R R') /\ (Contains R' R).
   
   Structure preorder : Prop :=
    {Prf_po_refl : Reflexive;
     Prf_po_trans : Transitive}.
   
   Structure order : Prop :=
    {Prf_o_refl  : Reflexive;
     Prf_o_trans : Transitive;
     Prf_o_asym  : Antisymmetric}.
   
   Structure equivalence : Prop := 
    {Prf_e_refl  : Reflexive;
     Prf_e_trans : Transitive;
     Prf_e_sym   : Symmetric}.
   
   Structure partial_equivalence : Prop := 
    {Prf_pe_sym   : Symmetric;
     Prf_pe_trans : Transitive}.
   
End Orderings.

Syntactic Definition Preorder := preorder | 1.
Syntax constr preorder <<(preorder $1 $R)>> 10
     [<hov 0> "Preorder " <$R:"Term":L>].

Syntactic Definition Order := order | 1.
Syntax constr order <<(order $1 $R)>> 10
     [<hov 0> "Order " <$R:"Term":L>].

Syntactic Definition Equivalence := equivalence | 1.
Syntax constr equivalance <<(equivalence $1 $R)>> 10
     [<hov 0> "Equivalence " <$R:"Term":L>].

Syntactic Definition Partial_equivalence := partial_equivalence | 1.
Syntax constr partial_equivalence <<(partial_equivalence $1 $R)>> 10
     [<hov 0> "Partial_equivalence " <$R:"Term":L>].



Hint Unfold Reflexive.
Hint Unfold Transitive.
Hint Unfold Antisymmetric.
Hint Unfold Symmetric.
Hint Unfold Contains.
Hint Unfold Same_relation.
Hint Build_preorder.
Hint Build_order.
Hint Build_equivalence.
Hint Build_partial_equivalence.





(* $Id: Relations.v,v 1.2 1996/02/12 15:26:41 cpaulin Exp $ *)
