(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                    Map.v                                 *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          General Algebra : Maps                                           *)
(*                                                                           *)
(*          Peter Aczel, Dec. 92                                             *)
(*          rev. Gerard Huet,  Aug. 1993                                     *)
(*                                                                           *)
(*****************************************************************************)

Require Export Setoid.

(* The set of maps between two Setoid *)

Section maps.

Variables A,B: Setoid.

Definition map_law := [f:|A|->|B|]
    (x,y:|A|) x =%S y -> (f x) =%S (f y).

Structure Map : Type := build_Map
 {ap   : |A|->|B|;
  Pres : (map_law ap)}.

Definition Ext := [f,g:Map](x:|A|) (ap f x) =%S (ap g x).

Lemma Ext_equiv : (Equivalence Ext).
Proof.
Intros; Apply Build_equivalence.
Unfold Reflexive; Unfold Ext; Intros f x.
Apply (Prf_refl B).
Unfold Transitive; Unfold Ext; Intros f g h e1 e2 x.
(* *) Apply (Prf_trans B) with y:=(ap g x).
Apply (e1 x).
Apply (e2 x).
Unfold Symmetric; Unfold Ext; Intros f g e x.
Apply (Prf_sym B); Trivial.
Qed.

Definition Map_setoid := (Build_Setoid Map Ext Ext_equiv).

End maps.


Syntactic Definition Map_law := map_law | 2. 
Syntax constr map_law <<(map_law $1 $2 $f)>> 10
     [<hov 0> "Map_law " <$f:"Term":L>].

Syntactic Definition Build_Map := build_Map | 2. 
Syntax constr build_Map <<(build_Map $1 $2 $f $p)>> 10
     [<hov 0> "Build_Map " <$f:"Term":L> " " <$p:"Term":L>].

Syntactic Definition Ap := ap | 2. 
Syntax constr ap <<(ap $1 $2 $f $x)>> 10
     [<hov 0> "Ap " <$f:"Term":L> " " <$x:"Term":L>].

Token "=%M".
Syntax constr Ext <<(Ext $1 $2 $f $g)>> 10
       [<hov 0>  <$f:"term":L> [<hov 0> " =%M " <$g:"term":L>]].
Grammar command command2 := [ command1($f) "=%M" command2($g) ] -> 
                            [$0 = <<(Ext ? ? $f $g)>>].

Syntax constr Map_setoid <<(Map_setoid $A $B)>> 10
       [<hov 0>  <$A:"term":L> [<hov 0> " => " <$B:"term":L>]].
Grammar command command2 := [ command1($A) "=>" command2($B) ] -> 
                            [$0 = <<(Map_setoid $A $B)>>].

Definition ap2 := [A,B,C:Setoid][f:|(A=>(B=>C))|][a:|A|] (Ap (Ap f a)).

Syntactic Definition Ap2 := ap2 | 3. 
Syntax constr ap2 <<(ap2 $1 $2 $3 $f $x)>> 10
     [<hov 0> "Ap2 " <$f:"Term":L> " " <$x:"Term":L>].

(* Identity Map *)

Section id_map_def.

Variable A : Setoid.

Definition Id_fun := [x:|A|]x.
     
Lemma Id_fun_map_law : (Map_law Id_fun).
Proof.
Unfold map_law; Trivial.
Qed.

Definition Id_map := (Build_Map Id_fun Id_fun_map_law).
  
End id_map_def.
 

(* composition of two Maps *)

Section mcomp.

Variable A, B, C : Setoid.
Variable f       : (Map A B).
Variable g       : (Map B C).

Definition Comp_fun := [x:|A|](Ap g (Ap f x)).

Lemma Comp_fun_map_law : (Map_law Comp_fun).
Proof.
Unfold map_law; Intros a1 a2 H.
Unfold Comp_fun ;Simpl.
Apply (Pres ? ? g).
Apply (Pres ? ? f); Trivial.
Qed.

Definition Comp_map := (Build_Map Comp_fun Comp_fun_map_law).

End mcomp.


Token "%M".
Syntax constr Comp_map <<(Comp_map $1 $2 $3 $f $g)>> 10
		[<hov 0> <$f:"Term":L> [<hov 0> " o%M " <$g:"Term":L>]].
Grammar command command2 := [ command1($f) "o" "%M" command2($g) ] -> 
                            [$0 = <<(Comp_map ? ? ?  $f $g)>>].




