(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                  Binary Maps                     		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export SetoidPROD.

Section fun2_to_map2.

Variable A, B, C : Setoid.

Definition Map2 := |(A => (B => C))|.

Definition Map2' := (Map (SPROD A B) C).

Variable f : |A| -> |B| -> |C|.

Definition map2_congl_law := (a:|A|)(b1,b2:|B|)
                             (b1 =%S b2) -> (f a b1) =%S (f a b2).

Definition map2_congr_law := (a1,a2:|A|)(b:|B|)
                             (a1 =%S a2) -> (f a1 b) =%S (f a2 b).

Definition map2_cong_law := (a1,a2:|A|)(b1,b2:|B|)
           (a1 =%S a2) -> (b1 =%S b2) -> (f a1 b1) =%S (f a2 b2).

Hypothesis pgcl : map2_congl_law.
Hypothesis pgcr : map2_congr_law.

Lemma Map2_map_law1 : (a:|A|)(Map_law (f a)).
Proof.
Intro a; Unfold map_law.
Exact (pgcl a).
Qed.

Definition Map2_map1 := [a:|A|](Build_Map (f a) (Map2_map_law1 a)).

Lemma Map2_map_law2 : (map_law A (B=>C) Map2_map1).
Proof.
Unfold map_law Map2_map1.
Intros a1 a2 H; Simpl; Unfold Ext; Intro b; Simpl.
Apply pgcr; Trivial.
Qed.

Definition Map2_operator := (build_Map A (B=>C) Map2_map1 Map2_map_law2).

End fun2_to_map2.


Syntactic Definition Map2_congr_law := map2_congr_law | 3. 
Syntax constr map2_congr_law <<(map2_congr_law $1 $2 $3 $f)>> 10
     [<hov 0> "Map2_congr_law " <$f:"Term":L>].

Syntactic Definition Map2_congl_law := map2_congl_law | 3. 
Syntax constr map2_congl_law <<(map2_congl_law $1 $2 $3 $f)>> 10
     [<hov 0> "Map2_congl_law " <$f:"Term":L>].

Syntactic Definition Map2_cong_law := map2_cong_law | 3. 
Syntax constr map2_cong_law <<(map2_cong_law $1 $2 $3 $f)>> 10
     [<hov 0> "Map2_cong_law " <$f:"Term":L>].


Section prop_map2.

Variable A, B, C : Setoid.
Variable f : (Map2 A B C).

Lemma Prf_map2_congl : (Map2_congl_law (Ap2 f)).
Proof.
Unfold map2_congl_law; Intros a b1 b2 H.
Unfold ap2; Apply (Pres ? ? (Ap f a)); Trivial.
Qed.

Lemma Prf_map2_congr : (Map2_congr_law (Ap2 f)).
Proof.
Unfold map2_congr_law; Intros a1 a2 b H.
Apply (Pres ? ? f ? ? H b).
Qed.

Lemma Prf_map2_cong : (Map2_cong_law (Ap2 f)).
Proof.
Unfold map2_cong_law; Intros a1 a2 b1 b2 H1 H2.
(* *) Apply (Prf_trans C) with y:=(Ap2 f a2 b1).
Apply (Prf_map2_congr a1 a2 b1 H1).
Apply (Prf_map2_congl a2 b1 b2 H2).
Qed.

End prop_map2.

