(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                            MapProperty.v                                 *)
(*                            A. SAIBI (May 95)                             *)
(****************************************************************************)


Require Export Map.

(* Injective Map *)

Section inj_surj_def.

Variable A, B : Setoid.

 Section isinj_def.

 Variable f : (Map A B).

 Definition Inj_law := (x,y:|A|)((Ap f x) =%S (Ap f y)) -> (x =%S y).

 Structure isInj : Type := 
  {Inj_l : Inj_law}.

 End isinj_def.

Structure Inj  : Type :=
 {Inj_map   : (Map A B);
  Prf_isInj : (isInj Inj_map)}.

(* Surjective Map *)

 Section issurj_def.

 Variable f : (Map A B).

 Definition Surj_law := [h:|B|->|A|](b:|B|) b =%S (Ap f (h b)).

 Structure isSurj : Type :=
  {Surj_elt     : |B| -> |A|;
   Prf_surj_law : (Surj_law Surj_elt) }.

 End issurj_def.

Structure Surj : Type :=
 {Surj_map   : (Map A B);
  Prf_isSurj : (isSurj Surj_map)}.

End inj_surj_def.

Syntactic Definition IsInj := isInj | 2.
Syntax constr isInj <<(isInj $1 $2 $h)>> 10
		[<hov 0> "IsInj " <$h:"Term":L>].

Syntactic Definition IsSurj := isSurj | 2.
Syntax constr isSurj <<(isSurj $1 $2 $h)>> 10
		[<hov 0> "IsSurj " <$h:"Term":L>].

