(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	               Free Monoid (Kleene Closure)                          *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Monoid.


Section freem_def.

Variable A : Setoid.

Inductive Tlist : Type :=
   Empty   : Tlist 
 | concat1 : |A| -> Tlist -> Tlist.

Fixpoint Equal_Tlist [l:Tlist] : Tlist -> Prop :=
   [m:Tlist] <Prop>Case l of 
                   (* Empty   *) <Prop>Case m of 
                                       (* Empty   *) True
                                       (* concat1 *) [b:?][m1:?]False
                                       end
                   (* concat1 *) [a:?][l1:?]
                                 <Prop>Case m of 
                                       (* Empty   *) False 
                                       (* concat1 *) [b:?][m1:?]
                                               (a =%S b) /\ (Equal_Tlist l1 m1)
                                       end
                   end.


Lemma Equal_Tlist_equiv : (Equivalence Equal_Tlist).
Proof.
Apply Build_equivalence.
Unfold Reflexive; Intro l; Elim l; Simpl.
Auto.
Intros; Split; [Apply (Prf_refl A)|Auto].
Unfold Transitive; Intros l1; Elim l1.
Intro l2; Elim l2.
Intro l3; Elim l3; Simpl; Trivial.
Intros c t H z H1; Elim H1.
Intros c t H y; Elim y.
Intros z H1 H2; Elim H1.
Intros c0 t0 H0 z; Elim z.
Intros H1 H2; Elim H2.
Intros c1 t1 H1 H2 H3.
Elim H2; Intros H4 H5.
Elim H3; Intros H6 H7.
Simpl; Split.
Apply (Prf_trans A) with y:=c0; Trivial.
Apply (H t0 t1); Trivial.
Unfold Symmetric; Intro l1; Elim l1.
Intro l2; Elim l2.
Trivial.
Intros c t H H1; Elim H1.
Intros c t H l2; Elim l2.
Simpl; Trivial.
Intros c0 t0 H0 H1.
Elim H1; Intros H2 H3.
Simpl; Split.
Apply (Prf_sym A); Trivial.
Apply H; Trivial.
Qed.

Definition Tlist_setoid := (Build_Setoid Tlist Equal_Tlist Equal_Tlist_equiv).

Fixpoint concat [l:Tlist] : Tlist -> Tlist 
      := [m:Tlist]<Tlist>Case l of
                (* Empty   *) m 
                (* concat1 *) [a:?][l1:?](concat1 a (concat l1 m)) end.


Lemma Diff_Concat1_Empty : (a:|A|)(l:Tlist)~(Equal_Tlist (concat1 a l) Empty).
Proof.
Unfold not; Intros a l; Simpl; Trivial.
Qed.

Lemma Concat_congl : (map2_congl_law Tlist_setoid Tlist_setoid Tlist_setoid concat).
Proof.
Unfold map2_congl_law; Intros l1 l2 l3.
Elim l1; Simpl.
Trivial.
Intros c t H H0; Split.
Apply (Prf_refl A).
Auto.
Qed.

Lemma Concat_congr : (map2_congr_law Tlist_setoid Tlist_setoid Tlist_setoid concat).
Proof.
Unfold map2_congr_law; Intro l1; Elim l1.
Intro l2; Elim l2.
Simpl; Intros l H; Apply (Prf_refl Tlist_setoid).
Intros c t H l3 H1; Elim H1.
Intros c t H l2; Elim l2.
Intros l H1; Absurd (Equal_Tlist (concat1 c t) Empty).
Apply Diff_Concat1_Empty.
Trivial.
Intros c0 t0 H0 l H1; Elim H1; Intros H2 H3.
Simpl; Split.
Trivial.
Apply (H t0 l); Trivial.
Qed.

Definition Concat_map2 := (Map2_operator Tlist_setoid Tlist_setoid Tlist_setoid 
                                         concat Concat_congl Concat_congr).

Lemma Mass_FreeMonoid : (Monoid_ass Concat_map2).
Proof.
Unfold monoid_ass; Intros l m n; Elim l; Simpl.
Apply (Prf_refl Tlist_setoid).
Intros; Split.
Apply (Prf_refl A).
Trivial.
Qed.

Lemma Midl_FreeMonoid : (Monoid_idl Concat_map2 Empty).
Proof.
Unfold monoid_idl; Intro l; Elim l; Simpl. 
Trivial.
Intros; Split.
Apply (Prf_refl A).
Trivial.
Qed.

Lemma Midr_FreeMonoid : (Monoid_idr Concat_map2 Empty).
Proof.
Unfold monoid_idr; Intro l; Elim l; Simpl. 
Trivial.
Intros; Split.
Apply (Prf_refl A).
Trivial.
Qed.

Definition FreeMonoid := (Build_Monoid Tlist_setoid Concat_map2 Empty 
                              Mass_FreeMonoid Midl_FreeMonoid Midr_FreeMonoid).

End freem_def.

Syntactic Definition Concat1 := concat1 | 1.
Syntax constr concat1 <<(concat1 $1 $a $l)>> 10
     [<hov 0> "Concat1 " <$a:"term":L> " " <$l:"term":L>].

Syntactic Definition Concat := concat | 1.
Syntax constr concat <<(concat $1 $l1 $l2)>> 10
     [<hov 0> "Concat " <$l1:"term":L> " " <$l2:"term":L>].

(* $Id: FreeMonoid.v,v 1.2 1996/02/12 15:27:16 cpaulin Exp $ *)
