(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              Inverses_Group.v                            *)
(****************************************************************************)


Require Export Group.
    
(* The Monoid of inverse pairs on a Monoid *)

Section inv_def.

Variable A : Monoid.

Local A_ := (Mcarrier A).

(* The set Inverses_set of inverse pairs on A *)

Structure Inverses : Type := 
 {Inv_elt_l : |A_|;
  Inv_elt_r : |A_|;  
  Prf_invs1 : (Inverses_rel ? Inv_elt_l Inv_elt_r );
  Prf_invs2 : (Inverses_rel ? Inv_elt_r Inv_elt_l)}.

Definition Inverses_eq := [A1,A2:Inverses] <Prop>Case A1 of
  [a1,a1':?][i1:?][i'1:?]
            <Prop>Match A2 with 
  [a2,a2':?][i2:?][i'2:?]
            (a1 =%S a2) end end.

Lemma Inverses_equiv : (Equivalence Inverses_eq).
Proof.
Apply Build_equivalence.
Unfold Reflexive.
Induction x; Simpl.
Intros a a' p p'; Apply (Prf_refl A_ a).
Unfold Transitive.
Induction x; Induction y; Induction z; Simpl.
Intros a2 b2 p2 p3 H H0; Apply (Prf_trans A_ Inv_elt_l0 Inv_elt_l1 a2); Trivial.
Unfold Symmetric.
Induction x; Induction y; Simpl.
Intros; Apply (Prf_sym A_); Trivial.
Qed.

Definition Inverses_setoid := (Build_Setoid Inverses Inverses_eq Inverses_equiv).

(* Composition operation on Inverses_setoid *)

Definition Opposite := [x:Inverses] <Inverses>Case x of
    [a,a':?][p:?][p':?]
    (Build_Inverses a' a p' p) end.


(* General group theory equational reasoning *)

Lemma Eq_abb'a' : (a,b,a',b' : |A_|)(Inverses_rel ? a a') ->
                  (Inverses_rel ? b b') -> 
                  (Inverses_rel ? (a +%M b) (b' +%M a')).
Proof.
Intros a b a' b' i j.
Red.
(* (a.b).b'.a' = e *)
(* *) Apply (Prf_trans A_) with y:= (a +%M (b +%M (b' +%M a'))).
Apply (Prf_sym A_).
Apply (Prf_monoid_ass A a b (b' +%M a')).
(* a.b.b'.a' = e *)
(* *) Apply (Prf_trans A_) with y:= (a +%M a'). 
(* a.b.b'.a' = a.a' *)
Apply (Prf_map2_congl ? ? ? (Mop A) a (b +%M (b' +%M a')) a'). 
(* b.b'.a' = a' *)
(* *) Apply (Prf_trans A_) with y:= ((b +%M b') +%M a').
Apply (Prf_monoid_ass A b b' a').
(* (b.b').a' = a' *)
(* *) Apply (Prf_trans A_) with y:=  ((Munit A) +%M a'). 
Apply (Prf_map2_congr ? ? ? (Mop A) (b +%M b') (Munit A) a'). 
Apply j.
(* e.a' = a' *)
Apply (Prf_monoid_idl A a').
Apply i.
Save.

 Section inverses_comp_def.

 Variable x, y : |Inverses_setoid|.

 Definition Inv_comp_elt_l := (Inv_elt_l x) +%M (Inv_elt_l y).

 Definition Inv_comp_elt_r := (Inv_elt_r y) +%M (Inv_elt_r x).

 Lemma Inv_comp_invs1 : (Inverses_rel ? Inv_comp_elt_l Inv_comp_elt_r).
 Proof.
 Unfold Inverses_rel Inv_comp_elt_l Inv_comp_elt_r.
 Exact (Eq_abb'a' (Inv_elt_l x) (Inv_elt_l y) (Inv_elt_r x) (Inv_elt_r y)
                  (Prf_invs1 x) (Prf_invs1 y)).
 Qed.

 Lemma Inv_comp_invs2 : (Inverses_rel ? Inv_comp_elt_r Inv_comp_elt_l).
 Proof.
 Unfold Inverses_rel Inv_comp_elt_l Inv_comp_elt_r.
 Exact (Eq_abb'a' (Inv_elt_r y) (Inv_elt_r x) (Inv_elt_l y) (Inv_elt_l x)
                  (Prf_invs2 y) (Prf_invs2 x)).
 Qed.

 Definition Inverses_comp := (Build_Inverses Inv_comp_elt_l Inv_comp_elt_r
                                    Inv_comp_invs1 Inv_comp_invs2).

 End inverses_comp_def.

Lemma Inverses_congl : (map2_congl_law Inverses_setoid Inverses_setoid 
                             Inverses_setoid Inverses_comp).
Proof.
Unfold map2_congl_law.
Intros x y z.
Elim x; Elim y; Elim z; Simpl.
Intros a a' p p' a0 a0' p0 p0' a1 a1' p1 p1' H.
Unfold Inv_comp_elt_l Inv_comp_elt_r; Simpl.
Apply (Prf_map2_congl ? ? ? (Mop A) a1 a0 a); Trivial.
Qed.

Lemma Inverses_congr : (map2_congr_law Inverses_setoid Inverses_setoid 
                              Inverses_setoid Inverses_comp).
Proof.
Unfold map2_congr_law; Induction a1; Induction a2; Simpl.
Intros b0 b0' p0 p0' z; Elim z.
Intros b1 b1' p1 p1' H.
Unfold Inv_comp_elt_l Inv_comp_elt_r; Simpl.
Apply (Prf_map2_congr ? ? ? (Mop A) Inv_elt_l0 b0 b1); Trivial.
Qed.

Definition Inverses_op := (Map2_operator Inverses_setoid Inverses_setoid 
                                         Inverses_setoid Inverses_comp 
                                         Inverses_congl Inverses_congr).

(* The Monoid of inverse pairs *)

Lemma Mass_Inverses : (monoid_ass Inverses_setoid Inverses_op).
Proof.
Red.
Induction x; Induction y; Induction z; Simpl; Intros.
Unfold Inverses_comp Inv_comp_elt_l Inv_comp_elt_r; Simpl.
Apply (Prf_monoid_ass A Inv_elt_l0 Inv_elt_l1 Inv_elt_l2).
Qed.

Lemma Inverses_unit_invs : (Inverses_rel ? (Munit A) (Munit A)).
Proof.
Apply (Prf_monoid_idl A (Munit A)).
Qed.

Definition Inverses_unit := (Build_Inverses (Munit A) (Munit A)
                                   Inverses_unit_invs Inverses_unit_invs).

Lemma Midl_Inverses : (monoid_idl Inverses_setoid Inverses_op Inverses_unit).
Proof.
Red.
Induction x; Simpl; Intros.
Unfold Inverses_eq Inv_comp_elt_l; Simpl.
Apply (Prf_monoid_idl ? Inv_elt_l0).
Qed.

Lemma Midr_Inverses : (Monoid_idr Inverses_op Inverses_unit).
Proof.
Red.
Induction x; Simpl; Intros.
Apply (Prf_monoid_idr ? Inv_elt_l0).
Qed.

Definition Inverses_Monoid := 
           (Build_Monoid Inverses_setoid Inverses_op Inverses_unit 
                  Mass_Inverses Midl_Inverses Midr_Inverses).


Lemma Opposite_map : (map_law Inverses_setoid Inverses_setoid Opposite).
Proof.
Red; Intros x y. Elim x; Elim y; Simpl.
Intros a a' aa' a'a b b' bb' b'b ba.
(* *) Apply (Prf_trans A_) with y := (b' +%M (Munit A)).
Apply (Prf_monoid_idr A b').
(* *) Apply (Prf_trans A_) with y:= (b' +%M (a +%M a')).
Apply (Prf_map2_congl ? ? ? (Mop A) b' (Munit A) (a +%M a')).
Apply (Prf_sym A_); Trivial.
(* *) Apply (Prf_trans A_) with y:= ((b' +%M a) +%M a').
Apply (Prf_monoid_ass A b' a a').
(* *) Apply (Prf_trans A_) with y:= ((Munit A) +%M a').
2: Apply (Prf_monoid_idl A a').
Apply (Prf_map2_congr ? ? ? (Mop A) (b' +%M a) (Munit A) a').
(* *) Apply (Prf_trans A_) with y:= (b' +%M b).
2:Assumption.
Apply (Prf_map2_congl ? ? ? (Mop A) b' a b).
Apply (Prf_sym A_); Trivial.
Qed.

Definition Inverses_inv_map := (build_Map Inverses_setoid Inverses_setoid 
                                          Opposite Opposite_map).

Lemma Inverses_invl : (Group_invl Inverses_Monoid Inverses_inv_map).
Proof.
Red; Induction x; Trivial.
Qed.

Lemma Inverses_invr : (Group_invr Inverses_Monoid Inverses_inv_map).
Proof.
Red; Induction x; Trivial.
Qed.

Definition Inverses_group := 
 (Build_Group Inverses_Monoid Inverses_inv_map Inverses_invl Inverses_invr).

End inv_def.


(* $Id: Inverses_Group.v,v 1.2 1996/02/12 15:27:19 cpaulin Exp $ *)
