(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Monoid.v                                 *)
(****************************************************************************)


Require Export Map2.


Definition Bin_op := [A:Setoid](Map2 A A A).

Section monoid_laws.

Variable A  : Setoid.
Variable op : (Bin_op A).
Variable e  : |A|.

Local Ap_op := [x,y:|A|](Ap2 op x y).

Token "+%".
Syntax constr Ap_op <<(Ap_op $x $y)>> 10
		[<hov 0> <$x:"Term":L> " +%M " <$y:"Term":L>].
Grammar command command2 := [ command1($x) "+%" "M" command2($y) ] -> 
                            [$0 = <<(Ap_op $x $y)>>].

Definition monoid_ass := (x,y,z:|A|)(x +%M (y +%M z)) =%S ((x +%M y) +%M z).

Definition monoid_idl := (x:|A|) (e +%M x) =%S x.

Definition monoid_idr := (x:|A|) x =%S (x +%M e).

End monoid_laws.

Syntactic Definition Monoid_ass := monoid_ass | 1.
Syntax constr monoid_ass <<(monoid_ass $1 $op)>> 10
		[<hov 0> "Monoid_ass " <$op:"Term":L>].

Syntactic Definition Monoid_idl := monoid_idl | 1.
Syntax constr monoid_idl <<(monoid_idl $1 $op)>> 10
		[<hov 0> "Monoid_idl " <$op:"Term":L>].

Syntactic Definition Monoid_idr := monoid_idr | 1.
Syntax constr monoid_idr <<(monoid_idr $1 $op)>> 10
		[<hov 0> "Monoid_idr " <$op:"Term":L>].

Structure Monoid : Type :=
 {Mcarrier       : Setoid;
  Mop            : (Bin_op Mcarrier);
  Munit          : |Mcarrier|;
  Prf_monoid_ass : (Monoid_ass Mop);
  Prf_monoid_idl : (Monoid_idl Mop Munit);
  Prf_monoid_idr : (Monoid_idr Mop Munit)}.

Definition ApMop := [m:Monoid][x,y:|(Mcarrier m)|](Ap2 (Mop m) x y).


Token "+%".
Syntax constr ApMop <<(ApMop $1 $x $y)>> 10
		[<hov 0> <$x:"Term":L> " +%M " <$y:"Term":L>].
Grammar command command2 := [ command1($x) "+%" "M" command2($y) ] -> 
                            [$0 = <<(ApMop ? $x $y)>>].

(* Monoid Morphisme *)

Section mon_mors.

Variable m1, m2 : Monoid.

Definition monUnit_law := [f:(Map (Mcarrier m1) (Mcarrier m2))]
                          (Ap f (Munit m1)) =%S (Munit m2).

Definition monOp_law := [f:(Map (Mcarrier m1) (Mcarrier m2))](a,b:|(Mcarrier m1)|)
                        (Ap f (a +%M b)) =%S ((Ap f a) +%M (Ap f b)).

Structure MonMor : Type := build_MonMor
 {monMap          : (Map (Mcarrier m1) (Mcarrier m2));
  Prf_MonUnit_law : (monUnit_law monMap);
  Prf_MonOp_law   : (monOp_law monMap)}.

(* equality of two monoid morphisms *)

Definition Equal_MonMor := [f,g:MonMor](monMap f) =%M (monMap g).

Lemma Equal_MonMor_equiv : (Equivalence Equal_MonMor).
Proof.
Apply Build_equivalence.
Unfold Reflexive Equal_MonMor; Intro f.
Unfold Ext; Intros x; Apply (Prf_refl (Mcarrier m2)).
Unfold Transitive Equal_MonMor; Intros f g h H1 H2. 
Unfold Ext; Intros x.
Apply (Prf_trans (Mcarrier m2)) with y:=(Ap (monMap g) x).
Apply (H1 x).
Apply (H2 x).
Unfold Symmetric Equal_MonMor; Intros f g H1.
Unfold Ext; Intros x.
Apply (Prf_sym (Mcarrier m2)).
Apply (H1 x).
Qed.

Definition MonMor_setoid := (Build_Setoid MonMor Equal_MonMor Equal_MonMor_equiv).

End mon_mors.

Syntactic Definition MonUnit_law := monUnit_law | 2.
Syntax constr monUnit_law <<(monUnit_law $1 $2 $f)>> 10
		[<hov 0> "MonUnit_law " <$f:"Term":L>].

Syntactic Definition MonOp_law := monOp_law | 2.
Syntax constr monOp_law <<(monOp_law $1 $2 $f)>> 10
		[<hov 0> "MonOp_law " <$f:"Term":L>].

Syntactic Definition MonMap := monMap | 2.
Syntax constr monMap <<(monMap $1 $2 $m)>> 10
		[<hov 0> "MonMap " <$m:"Term":L>].

Syntactic Definition Build_MonMor := build_MonMor | 2.
Syntax constr build_MonMor <<(build_MonMor $1 $2 $f $p1 $p2)>> 10
		[<hov 0> "Build_MonMor " <$f:"Term":L> " " <$p1:"Term":L>
                                         " " <$p2:"Term":L>].

(* $Id: Monoid.v,v 1.2 1996/02/12 15:27:21 cpaulin Exp $ *)
