(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Setoid.v                                 *)
(****************************************************************************)

(*****************************************************************************)
(*          Projet Coq - Calculus of Inductive Constructions V5.10           *)
(*****************************************************************************)
(*                                                                           *)
(*          General Algebra : Setoids                                        *)
(*                                                                           *)
(*          Peter Aczel, Dec. 92                                             *)
(*          rev. Gerard Huet,  Aug. 1993                                     *)
(*                                                                           *)
(*****************************************************************************)

Require Export Relations.


Structure Setoid : Type := 
 {Carrier   : Type;
  Equal     : (Relation Carrier);
  Prf_equiv : (Equivalence Equal)}.

Syntax constr Carrier <<(Carrier $s)>> 1
         [<hov 0> "|" <$s:"term":L> "|" ].

Grammar command command1 := [ "|" command0($s) "|"] -> 
                            [$0 = <<(Carrier $s)>>].

Token "=%S".

Syntax constr setoid_Equal <<(Equal $_ $a $b)>> 1
           [<hov 0>  <$a:"term":E> [<hov 0> " =%S " <$b:"term":L>]].
Grammar command command1 := [ command0($a) "=%S" command0($b) ] -> 
                            [$0 = <<(Equal ? $a $b)>>].

Lemma Prf_refl : (A:Setoid)(Reflexive |A| (Equal A)).
Proof.
Induction A; Simpl; Intros S r e; Elim e; Trivial.
Qed.

Lemma Prf_sym : (A:Setoid)(Symmetric |A| (Equal A)).
Proof.
Induction A; Simpl; Intros S r e; Elim e; Trivial.
Qed.

Lemma Prf_trans : (A:Setoid)(Transitive |A| (Equal A)).
Proof.
Induction A; Simpl; Intros S r e; Elim e; Trivial.
Qed.

(* An example *)

Inductive Nat : Type := Z : Nat | Suc : Nat -> Nat.

Definition Eq_Nat := [N1,N2:Nat] N1==N2.

Lemma Eq_Nat_equiv : (Equivalence Eq_Nat).
Proof.
Apply Build_equivalence; Unfold Eq_Nat.
Auto.
Unfold Transitive; Intros x y z H1 H2; Apply (trans_eqT ? ? ? ? H1 H2).
Unfold Symmetric; Intros x y H1; Apply (sym_eqT ? ? ? H1).
Qed.
 
Definition Set_of_nat := (Build_Setoid Nat Eq_Nat Eq_Nat_equiv).

(* Alternative : PERS instead of Equivalences *)

Structure PSetoid : Type := 
 {PCarrier  : Type;
  Coherence : (Relation PCarrier);
  Prf_PER   : (Partial_equivalence Coherence)}.

Definition Total := [A:PSetoid][x:(PCarrier A)](Coherence ? x x).

