(*****************************************************************************)
(*									     *)
(*	                       Setoid Product               		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

Require Export Map.


(* Produit de deux Setoid *)

Section s_prod.

Variable A, B : Setoid.

Structure Sprod :Type := build_Sprod
 {sprod_l : |A|;
  sprod_r : |B|}.

Definition Equal_Sprod :=[a1xb1,a2xb2:Sprod] 
           ((sprod_l a1xb1) =%S (sprod_l a2xb2)) /\
           ((sprod_r a1xb1) =%S (sprod_r a2xb2)).

Lemma Equal_Sprod_equiv : (Equivalence Equal_Sprod).
Proof.
Apply Build_equivalence.
Unfold Reflexive; Induction x; Intros a b; Unfold Equal_Sprod; Simpl.
Split.
Apply (Prf_refl A).
Apply (Prf_refl B).
Unfold Transitive.
Induction x; Intros a b; Induction y; Intros a' b'; Induction z; Intros a'' b''.
Unfold Equal_Sprod; Simpl.
Intros H H0; Elim H; Intros H1 H2; Elim H0; Intros H3 H4.
Split.
Apply (Prf_trans A) with a'; Trivial.
Apply (Prf_trans B) with b'; Trivial.
Unfold Symmetric; Induction x; Intros a b; Induction y; Intros a' b'.
Unfold Equal_Sprod; Simpl.
Intros H ; Elim H; Intros H1 H2.
Split.
Apply (Prf_sym A); Trivial.
Apply (Prf_sym B); Trivial.
Qed.

Definition SPROD := (Build_Setoid Sprod Equal_Sprod Equal_Sprod_equiv).

(* AxB = (S_PROD A B) *) 
(* sprod_l et sprod_r sont en fait des Map *)

Lemma Proj1_SPROD_map_law : (map_law SPROD A sprod_l).
Proof.
Unfold map_law; Simpl.
Induction x; Intros a b; Induction y; Intros a' b'.
Simpl; Unfold Equal_Sprod; Simpl.
Induction 1; Trivial.
Qed.

Definition Proj1_SPROD := (build_Map SPROD A sprod_l Proj1_SPROD_map_law).

Lemma Proj2_SPROD_map_law : (map_law SPROD B sprod_r).
Proof.
Unfold map_law; Simpl.
Induction x; Intros a b; Induction y; Intros a' b'.
Simpl; Unfold Equal_Sprod; Simpl.
Induction 1; Trivial.
Qed.

Definition Proj2_SPROD := (build_Map SPROD B sprod_r Proj2_SPROD_map_law).

End s_prod.

Syntactic Definition Build_Sprod := build_Sprod | 2.
Syntax constr build_Sprod <<(build_Sprod $1 $2 $axb)>> 10
       [<hov 0> "Build_Sprod " <$axb:"term":L>].

Syntactic Definition Sprod_l := sprod_l | 2.
Syntax constr sprod_l <<(sprod_l $1 $2 $axb)>> 10
       [<hov 0> "Sprod_l " <$axb:"term":L>].

Syntactic Definition Sprod_r := sprod_r | 2.
Syntax constr sprod_r <<(sprod_r $1 $2 $axb)>> 10
       [<hov 0> "Sprod_r " <$axb:"term":L>].

