(*****************************************************************************)
(*          Projet Formel - Calculus of Inductive Constructions V5.10        *)
(*****************************************************************************)
(*									     *)
(*	                       SubSetoid                  		     *)
(*									     *)
(*****************************************************************************)
(*									     *)
(*                     A. SAIBI	  May 95                  		     *)
(*									     *)
(*****************************************************************************)

(* (SubSetoid A P) setoid of elements of A with the property P *)

Require Export Map.

Section sub_setoid.

Variable A : Setoid.
Variable P : |A| -> Prop.

Structure SubType : Type := 
 {Elt_sub    : |A|;
  Prf_constr : (P Elt_sub)}.

Definition Equal_SubType := [a,b:SubType](Elt_sub a) =%S (Elt_sub b).

Lemma Equal_SubType_equiv : (Equivalence Equal_SubType).
Proof.
Apply Build_equivalence.
Unfold Reflexive; Intro x; Exact (Prf_refl A (Elt_sub x)).
Unfold Transitive; Intros x y z.
Exact (Prf_trans A (Elt_sub x) (Elt_sub y) (Elt_sub z)).
Unfold Symmetric; Intros x y.
Exact (Prf_sym A (Elt_sub x) (Elt_sub y)).
Qed.

Definition SubSetoid := (Build_Setoid SubType Equal_SubType Equal_SubType_equiv).

End sub_setoid.

Section restricted_map.

Variable A, B : Setoid.
Variable f    : (Map A B).
Variable P    : |A| -> Prop.

Definition Restricted_fun := [a:(SubType A P)](Ap f (Elt_sub A P a)).

Lemma Restricted_map_law : (map_law (SubSetoid A P) B Restricted_fun).
Proof.
Unfold map_law; Simpl.
Unfold Equal_SubType; Intros a1 a2 H.
Unfold Restricted_fun; Apply (Pres ? ? f); Trivial.
Qed.

Definition restrictedMap := (build_Map (SubSetoid A P) B Restricted_fun 
                                       Restricted_map_law).

End restricted_map.

Syntactic Definition RestrictedMap := restrictedMap | 2.
Syntax constr restrictedMap <<(restrictedMap $1 $2 $p $i)>> 10
		[<hov 0> "RestrictedMap " <$p:"Term":L> " " <$i:"Term":L>].
