(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                   Gt.v                                   *)
(****************************************************************************)

Require Le.
Require Lt.

Theorem gt_Sn_O : (n:nat)(gt (S n) O).
Proof.
Auto.
Qed.
Hint gt_Sn_O.

Theorem gt_Sn_n : (n:nat)(gt (S n) n).
Proof.
Auto.
Qed.
Hint gt_Sn_n.

Theorem le_S_gt : (n,m:nat)(le (S n) m)->(gt m n).
Proof.
Auto.
Qed.
Immediate le_S_gt.

Theorem gt_n_S : (n,m:nat)(gt n m)->(gt (S n) (S m)).
Proof.
Auto.
Qed.
Hint gt_n_S.

Theorem gt_trans_S : (n,m,p:nat)(gt (S n) m)->(gt m p)->(gt n p).
Proof.
Red; Intros; Apply lt_le_trans with m; Auto.
Qed.

Theorem le_gt_trans : (n,m,p:nat)(le m n)->(gt m p)->(gt n p).
Proof.
Red; Intros; Apply lt_le_trans with m; Auto.
Qed.

Theorem gt_le_trans : (n,m,p:nat)(gt n m)->(le p m)->(gt n p).
Proof.
Red; Intros; Apply le_lt_trans with m; Auto.
Qed.

Lemma le_not_gt : (n,m:nat)(le n m)->~(gt n m).
Proof le_not_lt.
Hint le_not_gt.

Lemma gt_antirefl : (n:nat)~(gt n n).
Proof lt_n_n.
Hint gt_antirefl.

Lemma gt_not_sym : (n,m:nat)(gt n m)->~(gt m n).
Proof [n,m:nat](lt_not_sym m n).

Lemma gt_not_le : (n,m:nat)(gt n m)->~(le n m).
Proof.
Auto.
Qed.
Hint gt_not_le.

Lemma gt_trans : (n,m,p:nat)(gt n m)->(gt m p)->(gt n p).
Proof.
Red; Intros n m p H1 H2.
Apply lt_trans with m; Auto.
Qed.

Lemma gt_S_n : (n,p:nat)(gt (S p) (S n))->(gt p n).
Proof.
Auto.
Qed.
Immediate gt_S_n.

Lemma gt_S_le : (n,p:nat)(gt (S p) n)->(le n p).
Proof.
Intros n p; Exact (lt_n_Sm_le n p).
Qed.
Immediate gt_S_le.

Lemma gt_le_S : (n,p:nat)(gt p n)->(le (S n) p).
Proof.
Auto.
Qed.
Hint gt_le_S.

Lemma le_gt_S : (n,p:nat)(le n p)->(gt (S p) n).
Proof.
Auto.
Qed.
Hint le_gt_S.

Lemma gt_pred : (n,p:nat)(gt p (S n))->(gt (pred p) n).
Proof.
Auto.
Qed.
Immediate gt_pred.

Theorem gt_S : (n,m:nat)(gt (S n) m)->((gt n m)\/(<nat>m=n)).
Proof.
Intros n m H; Unfold gt; Apply le_lt_or_eq; Auto.
Qed.

Theorem gt_O_eq : (n:nat)((gt n O)\/(<nat>O=n)).
Proof.
Intro n ; Apply gt_S ; Auto.
Qed.

Require Plus.

Lemma simpl_gt_plus_l : (n,m,p:nat)(gt (plus p n) (plus p m))->(gt n m).
Proof.
Red; Intros n m p H; Apply simpl_lt_plus_l with p; Auto.
Qed.

Lemma gt_reg_l : (n,m,p:nat)(gt n m)->(gt (plus p n) (plus p m)).
Proof.
Auto.
Qed.
Hint gt_reg_l.


(* $Id: Gt.v,v 1.2 1996/02/12 15:27:42 cpaulin Exp $ *)
