(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Mult.v                                  *)
(****************************************************************************)

Require Le.
Require Lt.
Require Plus.
Require Minus.

(**********************************************************)
(* Multiplication                                         *)
(**********************************************************)

Lemma mult_plus_distr : 
      (n,m,p:nat)((mult (plus n m) p)=(plus (mult n p) (mult m p))).
Proof.
Intros; Elim n; Simpl; Intros; Trivial.
Elim plus_assoc_l; Elim H; Auto.
Qed.
Hint mult_plus_distr.

Lemma mult_minus_distr : (n,m,p:nat)((mult (minus n m) p)=(minus (mult n p) (mult m p))).
Proof.
Intros; Pattern n m; Apply nat_double_ind; Simpl; Intros; Trivial.
Elim minus_plus_simpl; Auto.
Qed.
Hint mult_minus_distr.

Lemma mult_O_le : (n,m:nat)(m=O)\/(le n (mult m n)).
Proof.
Induction m; Simpl; Auto.
Qed.
Hint mult_O_le.

Lemma mult_assoc_r : (n,m,p:nat)((mult (mult n m) p) = (mult n (mult m p))).
Proof.
Intros; Elim n; Intros; Simpl; Auto.
Rewrite mult_plus_distr.
Elim H; Auto.
Qed.
Hint mult_assoc_r.

Lemma mult_assoc_l : (n,m,p:nat)(mult n (mult m p)) = (mult (mult n m) p).
Auto.
Save.
Hint mult_assoc_l.

Lemma mult_1_n : (n:nat)(mult (S O) n)=n.
Simpl; Auto.
Save.
Hint mult_1_n.

Lemma mult_sym : (n,m:nat)(mult n m)=(mult m n).
Intros; Elim n; Intros; Simpl; Auto.
Elim mult_n_Sm.
Elim H; Apply plus_sym.
Save.
Hint mult_sym.

Lemma mult_n_1 : (n:nat)(mult n (S O))=n.
Intro; Elim mult_sym; Auto.
Save.
Hint mult_n_1.


(* $Id: Mult.v,v 1.3 1996/02/12 15:27:51 cpaulin Exp $ *)
