(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Bool.v                                  *)
(****************************************************************************)


(************)
(* Booleans *)
(************)

(* Inductive bool : Set := true : bool | false : bool (from Prelude) *)

Definition Is_true := [b:bool](<Prop>Case b of
    (* true  *)  True
    (* false *)  False end).

Lemma diff_true_false : ~true=false.
Goal.
Unfold not; Intro contr; Change (Is_true false).
Elim contr; Simpl; Trivial.
Save.
Hint diff_true_false.

(************)
(* Negation *)
(************)

Definition neg := [b:bool](<bool>Case b of
    (* true *)  false
    (* false*)  true end).

Lemma neg_intro : (b:bool)<bool>(neg (neg b))=b.
Goal.
Induction b; Simpl; Auto.
Save.

Lemma neg_elim : (b:bool)<bool>b=(neg (neg b)).
Goal.
Induction b; Simpl; Trivial.
Save.
       
Lemma neg_sym : (b,b':bool)(<bool>b'=(neg b))->(<bool>b=(neg b')).
Goal.
Induction b; Induction b'; Intros; Simpl; Trivial.
Save.

Lemma no_fixpoint_neg : (b:bool)~(neg b)=b.
Goal.
Induction b; Simpl; Unfold not; Intro; Apply diff_true_false; Auto.
Save.

(* Logical boolean functions *)
 
Definition ifb : bool -> bool -> bool -> bool
     := [b1,b2,b3:bool](<bool>Case b1 of (* true *) b2
                                         (* false *) b3 end).

Definition negb : bool -> bool
     := [b:bool](ifb b false true).

Definition andb : bool -> bool -> bool
     := [b1,b2:bool](ifb b1 b2 false).

Definition orb : bool -> bool -> bool
     := [b1,b2:bool](ifb b1 true b2).

Definition implb : bool -> bool -> bool
     := [b1,b2:bool](ifb b1 b2 true).


Definition eqb : bool->bool->bool :=
 [b1,b2:bool]
    <bool>Case b1 of <bool>Case b2 of true  false end 
	             <bool>Case b2 of false true  end 
	  end.

Lemma eqb_subst : 
  (P:bool->Prop)(b1,b2:bool)(eqb b1 b2)=true->(P b1)->(P b2).
Unfold eqb .
Intros P b1.
Intros b2.
Case b1.
Case b2.
Trivial.
Intros H.
Inversion_clear H.
Case b2.
Intros H.
Inversion_clear H.
Trivial.
Qed.
 
Lemma eqb_reflx : 
  (b:bool)(eqb b b)=true.
Intro b.
Case b.
Trivial.
Trivial.
Qed.
 
Lemma eqb_neg1 : (b:bool)(eqb (neg b) b)=false.
Destruct b.
Trivial.
Trivial.
Qed.
 
Lemma eqb_neg2 : (b:bool)(eqb b (neg b))=false.
Destruct b.
Trivial.
Trivial.
Qed.

(* $Id: Bool.v,v 1.4 1996/02/12 15:28:05 cpaulin Exp $ *)
