(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              July 1st 1995                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  IfProp.v                                *)
(****************************************************************************)

Require Bool.

Inductive IfProp [A,B:Prop] : bool-> Prop 
  := Iftrue : A -> (IfProp A B true)
  |  Iffalse : B -> (IfProp A B false).

Hint Iftrue Iffalse.

Lemma Iftrue_inv : (A,B:Prop)(b:bool) (IfProp A B b) -> b=true -> A.
Destruct 1; Intros; Trivial.
Case diff_true_false; Auto.
Save.

Lemma Iffalse_inv : (A,B:Prop)(b:bool) (IfProp A B b) -> b=false -> B.
Destruct 1; Intros; Trivial.
Case diff_true_false; Trivial.
Save.

Lemma IfProp_or : (A,B:Prop)(b:bool)(IfProp A B b) -> A\/B.
Destruct 1; Auto.
Save.

Lemma IfProp_sum : (A,B:Prop)(b:bool)(IfProp A B b) -> {A}+{B}.
Destruct b; Intro H.
Left; Inversion H; Auto.
Right; Inversion H; Auto.
Save.

(* $Id: IfProp.v,v 1.2 1996/02/12 15:28:07 cpaulin Exp $ *)
