(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Datatypes.v                                *)
(****************************************************************************)

Require PreludeSyntax.

Inductive unit : Set := tt : unit.

Inductive bool : Set := true : bool 
                      | false : bool.

Inductive nat : Set := O : nat 
                     | S : nat->nat.

(* Disjoint sum of two sets *)

Inductive sum [A,B:Set] : Set
    := inl : A -> A+B 
     | inr : B -> A+B.

Syntax constr sum <<(sum $t1 $t2)>> 4
  [<hov 0>  <$t1:"term":L> [0 1] "+" <$t2:"term":E>].

(* Product of Sets *)

Inductive prod [A,B:Set] : Set := pair : A -> B -> A*B.

Syntax constr product  <<(prod  $t1  $t2)>>  3 
           [<hov 0>  <$t1:"term":L> [0 1] "*" <$t2:"term":E>].
Syntax constr pair <<(pair $1 $2 $t3 $t4)>>  1
      [<hov 0> "(" <$t3:"term":E> "," [0 1] <$t4:"term":E> ")" ].

Section projections.
   Variables A,B:Set.
   Definition fst := [H:A*B]<A> Case H of [x:A][y:B]x end.
   Definition snd := [H:A*B]<B> Case H of [x:A][y:B]y end.
End projections. 

Syntactic Definition Fst := (fst ? ?).
Syntactic Definition Snd := (snd ? ?).

Syntax constr fst_imp <<(fst $1 $2  $t2)>> 1
   [<hov 0> "(Fst " <$t2:"term":E> ")"].

Syntax constr snd_imp <<(snd $1 $2  $t2)>> 1
   [<hov 0> "(Snd " <$t2:"term":E> ")"].

Hint pair inl inr.

(* $Id: Datatypes.v,v 1.4 1996/02/12 15:28:27 cpaulin Exp $ *)
