(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                              LogicSyntax.v                               *)
(****************************************************************************)

Grammar command command0 :=
[ "(" lcommand($lc1) "," lcommand($lc2) ")" ]
  -> [$0 = (APPLIST pair (XTRA{"ISEVAR"}) (XTRA{"ISEVAR"}) $lc1 $lc2)].

Grammar command command1 :=
  [ "<" lcommand($l1) "," lcommand($c2) ">"
     "{" command($c3) "," command($c4) "}" ]
  -> [$0 = <<(conj $l1 $c2 $c3 $c4)>>]
.

Grammar command command1 :=
  [ "<" lcommand($l1) "," lcommand($c2) ">"
     "Fst" "{" lcommand($l) "}" ]
  -> [$0 = <<(proj1 $l1 $c2 $l)>>]

| [ "<" lcommand($l1) "," lcommand($c2) ">"
     "Snd" "{" lcommand($l) "}" ]
  -> [$0 = <<(proj2 $l1 $c2 $l)>>]

.

Grammar command command1 :=
  [ "if" command($c) "then" command($t) "else" command($e) ]
  -> [$0 = <<(IF $c $t $e)>>]
.

Grammar command command1 :=
  [ "<" lcommand($l1) ">" "All" "(" lcommand($l2) ")" ]
  -> [$0 = <<(all $l1 $l2)>>]

.

Grammar command command1 :=
  [ "<" lcommand($l1) ">" "Ex" "(" lcommand($l2) ")" ]
  -> [$0 = <<(ex $l1 $l2)>>]

.

Grammar command command1 :=
  [ "<" lcommand($l1) ">" "Ex2" "(" command($c1) "," command($c2) ")" ]
  -> [$0 = <<(ex2 $l1 $c1 $c2)>>]

.

Grammar command command1 :=
  [ "<" lcommand($l1) ">" command0($c1) "=" command0($c2) ]
  -> [$0 = <<(eq $l1 $c1 $c2)>>]
.

Grammar command command1 :=
 [ "<" lcommand($l1) "," lcommand($c2) ">"
    "(" command($c3) "," command($c4) ")" ]
  -> [$0 = <<(pair $l1 $c2 $c3 $c4)>>]
.


Grammar command command1 :=
  [ "<" lcommand($l1) "," lcommand($c2) ">"
     "Fst" "(" lcommand($l) ")" ]
  -> [$0 = <<(fst $l1 $c2 $l)>>]

| [ "<" lcommand($l1) "," lcommand($c2) ">"
     "Snd" "(" lcommand($l) ")"] 
  -> [$0 = <<(snd $l1 $c2 $l)>>]

.

Token "~".
Grammar command command2 :=
  [ "~" command2($c) ] -> [$0 = <<(not $c)>>]
.


Grammar command command3 :=
  [ command2($c1) "*" command3($c2) ] -> [$0 = <<(prod $c1 $c2)>>]
.

Token "+".
Grammar command lassoc_command4_tail[$c1] :=
  [ "+" command3($c2)
        lassoc_command4_tail[case $c2 of
                               (APPLIST squash $T2) ->
                                  (case $c1 of
                                       <<(squash $T1)>> -> [$0 = <<(sumbool $T1 $T2)>>]
                                     | $c1 -> [$0 = <<(sumor $c1 $T2)>>]
                                   esac)
                            | $c2 -> [$0 = <<(sum $c1 $c2)>>]
                           esac]($r) ] -> [$0 = $r]
.

Token "/\\".
Grammar command command6 :=
  [ command5($c1) "/\\" command6($c2) ] -> [$0 = <<(and $c1 $c2)>>]
.

Token "\\/".
Grammar command command7 :=
  [ command6($c1) "\\/" command7($c2) ] -> [$0 = <<(or $c1 $c2)>>]
.
Token "<->".
Grammar command command8 :=
  [ command7($c1) "<->" command8($c2) ] -> [$0 = <<(iff $c1 $c2)>>]
.

Grammar command command1 :=
  [ command0($c) "=" command0($c2) ] -> [$0 = <<(eq ? $c $c2)>>]
.




(* $Id: LogicSyntax.v,v 1.3 1996/02/12 15:28:31 cpaulin Exp $ *)
