(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                               Logic_Type.v                               *)
(****************************************************************************)

Require Logic.
Require LogicSyntax.
Require Logic_TypeSyntax.


Definition allT := [A:Type][P:A->Prop](x:A)(P x). 

Syntactic Definition AllT := (allT ?).

Syntax constr  allT <<(allT $_ $l2)>> 10 
        [<hov 0>  "AllT " <$l2:"Command":L>] .


Section universal_quantification.

Variable A : Type.
Variable P : A->Prop.

Theorem inst :  (x:A)(AllT P)->(P x).
Proof.
Auto.
Qed.

Theorem gen : (B:Prop)(f:(y:A)B->(P y))B->(AllT P).
Proof.
Red; Auto.
Qed.

End universal_quantification.

(* Existential Quantification *)

Inductive  exT [A:Type;P:A->Prop] : Prop
    := exT_intro : (x:A)(P x)->(exT A P).

Syntactic Definition ExT := (exT ?).
Syntax constr PP_exT  <<(exT $_ $l2)>>  10
     [<hov 0> "ExT "  <$l2:"Command":L>] .


Inductive exT2 [A:Type;P,Q:A->Prop] : Prop
    := exT_intro2 : (x:A)(P x)->(Q x)->(exT2 A P Q).

Syntactic Definition ExT2 := (exT2 ?).
Syntax constr PP_exT2 <<(exT2 $_ $c1 $c2)>>  10
     [<hov 0> "ExT2 "  [1 1]
             [<hv 0> <$c1:"Command":L> [1 0] <$c2:"Command":L>]] .

(* Leibniz equality : [A:Type][x,y:A] (P:A->Prop)(P x)->(P y) *)

Inductive eqT [A:Type;x:A] : A -> Prop
                       := refl_eqT : (eqT A x x).

Syntax constr eqT <<(eqT $_ $c1 $c2)>> 1
     [<hov 0>     <$c1:"Command":E> [0 1] "==" <$c2:"Command":E>] .

Hint refl_eqT.



Section Equality_is_a_congruence.
Variables A,B : Type.
Variable  f : A->B.

 Variable x,y,z : A.
 
 Lemma sym_eqT : (x==y) -> (y==x).
 Proof.
 Induction 1; Trivial.
 Qed.

 Lemma trans_eqT : (x==y) -> (y==z) -> (x==z).
 Proof.
 Induction 2; Trivial.
 Qed.

 Lemma congr_eqT : (x==y)->((f x)==(f y)).
 Proof.
 Induction 1; Trivial.
 Qed.

 Lemma sym_not_eqT : ~ x==y -> ~ y==x.
 Proof.
 Red; Intros H H'; Apply H; Elim H'; Trivial.
 Qed.


End Equality_is_a_congruence.

Immediate sym_eqT sym_not_eqT.

Definition eqT_ind_r : (A:Type)(x:A)(P:A->Prop)(P x)->(y:A)y==x -> (P y).
Intros A x P H y H0; Case sym_eqT with 1:=H0; Trivial.
Defined.

Definition eqT_rec_r : (A:Type)(x:A)(P:A->Set)(P x)->(y:A)y==x -> (P y).
Intros A x P H y H0; Case sym_eqT with 1:=H0; Trivial.
Defined.


Inductive eqTS [A:Typeset;x:A] : A -> Prop
                       := refl_eqTS : (eqTS A x x).

Hint refl_eqTS.


Section EqualityTS_is_a_congruence.
Variables A,B : Typeset.
Variable  f : A->B.

 Variable x,y,z : A.
 
 Lemma sym_eqTS : (eqTS ? x y) -> (eqTS ? y x).
 Proof.
 Induction 1; Trivial.
 Qed.

 Lemma trans_eqTS : (eqTS ? x y) -> (eqTS ? y z) -> (eqTS ? x z).
 Proof.
 Induction 2; Trivial.
 Qed.

 Lemma congr_eqTS : (eqTS ? x y) -> (eqTS ? (f x) (f y)).
 Proof.
 Induction 1; Trivial.
 Qed.

 Lemma sym_not_eqTS : ~ (eqTS ? x y) -> ~ (eqTS ? y x).
 Proof.
 Red; Intros H H'; Apply H; Elim H'; Trivial.
 Qed.
 

End EqualityTS_is_a_congruence.

Immediate sym_eqTS sym_not_eqTS.


(* $Id: Logic_Type.v,v 1.5 1996/02/12 15:28:32 cpaulin Exp $ *)
