(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Specif.v                                 *)
(****************************************************************************)

(**************************************************************)
(* Basic specifications : Sets containing logical information *)
(**************************************************************)

Require Logic.
Require Datatypes.
Require PreludeSyntax.
Require SpecifSyntax.

Inductive sig [A:Set;P:A->Prop] : Set
    := exist : (x:A)(P x) -> (sig A P).

Syntax constr sig  <<(sig $c1 $c2)>>  1
      <$ABST:"Term":E>  with $ABST:= (ABSTR_B_NB  $c1 $c2).
Syntax constr sig_var (ABSTR_B_NB $c1 $c2) 10
        [<hov 0> "sig " <$c1:"term":L> [1 1] <$c2:"term":L> ].
Syntax constr sig_nb (ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2)) 1
    [<hov 0> "{_:" <$c1:"term":E> [0 1]  " | " [0 1] <$c2:"term":E> "}" ] .
Syntax constr sigma_b (ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2)) 1
   [<hov 0>  "{" <$id:"ident":*> ":" <$c1:"term":E>  
               " | " [0 1] <$c2:"term":E> "}" ] .

Inductive sig2 [A:Set;P,Q:A->Prop] : Set
    := exist2 : (x:A)(P x) -> (Q x) -> (sig2 A P Q).

Syntax constr sig2 <<(sig2 $c1 $c2 $c3)>>  1
     <$ABST:"Term":E> with $ABST:= (Sig2_ABSTR_B_NB  $c1 $c2 $c3).

Syntax constr sig2_var (Sig2_ABSTR_B_NB $c1 $c2) 10
        [<hov 0> "sig2 " <$c1:"term":L> [1 0] <$c2:"term":L> ].
Syntax constr sig2_b_b 
   (Sig2_ABSTR_B_NB $c1 (LAMBDA $c1  [$id]$c2)  (LAMBDA $c1 [$id]$c3))  1
        [ <hov 0>   "{" <$id:"term":*> ":" <$c1:"term":E>  
              "| " [0 1] <$c2:"term":E>  " & " <$c3:"term":E> "}" ] .
Syntax constr sig2_nb_b 
  (Sig2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA  $c1 [$id]$c3))  1
        [ <hov 0>  "{"  <$id:"term":*> ":" <$c1:"term":E>  
              "| " [0 1] <$c2:"term":E>  " & " <$c3:"term":E> "}" ] .
Syntax constr sig2_b_nb 
   (Sig2_ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2) (LAMBDA $c1 [<>]$c3))  1
        [ <hov 0>  "{"  <$id:"term":*> ":" <$c1:"term":E>  
              "| " [0 1] <$c2:"term":E>  " &" <$c3:"term":E> "}" ] .
Syntax constr sig2_nb_nb 
   (Sig2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA $c1 [<>]$c3)) 1
         [ <hov 0>   "{_:" <$c1:"term":E>  
              "| " [0 1] <$c2:"term":E>  " & " <$c3:"term":E> "}" ] .


Inductive sigS [A:Set;P:A->Set] : Set
    := existS : (x:A)(P x) -> (sigS A P).

Syntax constr sigS  <<(sigS $c1 $c2)>> 1
    <$ABST:"Term":E> with $ABST:= (SigS_ABSTR_B_NB  $c1 $c2).

Syntax constr sigS_var (SigS_ABSTR_B_NB $c1 $c2) 10
        [<hov 0> "sigS " <$c1:"term":L> [1 1] <$c2:"term":L> ].
Syntax constr sigS_nb 
  (SigS_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2))  1
    [<hov 0> "{_:" <$c1:"term":E> [0 1]  " & " <$c2:"term":E>  "}" ] .
Syntax constr sigS_b
  (SigS_ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2))  1
    [<hov 0> "{" <$id:"term":*>  ":" <$c1:"term":E> [0 1]  
                        " & " <$c2:"term":E> "}" ].



(***********************)
(* Projections of sigS *)
(***********************)

Section projections.
   Variable A:Set.
   Variable P:A->Set.
   Definition projS1 (* : (A:Set)(P:A->Set)(sigS A P) -> A *)
            := [H:(sigS A P)]<A> Case H of [x:A][h:(P x)]x end.
   Definition projS2 (* : (A:Set)(P:A->Set)(p:(sigS A P))(P (projS1 A P p)) *)
            := [H:(sigS A P)]<[H:(sigS A P)](P (projS1 H))> 
                   Case H of [x:A][h:(P x)]h end.
End projections.

Inductive sigS2 [A:Set;P,Q:A->Set] : Set
    := existS2 : (x:A)(P x) -> (Q x) -> (sigS2 A P Q).

Syntax constr sigS2 <<(sigS2 $c1 $c2 $c3)>> 1 
   <$ABST:"Term":E>  with $ABST:= (SigS2_ABSTR_B_NB  $c1 $c2 $c3).

Syntax constr sigS2_var (SigS2_ABSTR_B_NB $c1 $c2 $c3) 10
  [<hov 0> "sigS2 " <$c1:"term":L> [1 1] <$c2:"term":L> [1 1] <$c3:"term":L>].
Syntax constr sigS2_b_b 
   (SigS2_ABSTR_B_NB $c1 (LAMBDA $c1  [$id]$c2)  (LAMBDA $c1 [$id]$c3))  1
       [ <hov 0> "{"  <$id:"term":*> ":" <$c1:"term":E>  
           [0 1] " & "  <$c2:"term":E> " & "  <$c3:"term":E> "}" ].
Syntax constr sigS2_nb_b 
  (SigS2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA  $c1 [$id]$c3))  1
       [ <hov 0>  "{" <$id:"term":*> ":" <$c1:"term":E>  
           [0 1]   " & "  <$c2:"term":E>  " & " <$c3:"term":E> "}" ] .
Syntax constr sigS2_b_nb 
   (SigS2_ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2) (LAMBDA $c1 [<>]$c3))  1
       [ <hov 0>  "{" <$id:"term":*> ":" <$c1:"term":E>  
             [0 1]  " & "  <$c2:"term":E>  " & " <$c3:"term":E> "}" ] .
Syntax constr sigS2_nb_nb 
   (SigS2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA $c1 [<>]$c3)) 1
       [ <hov 0>   "{_:" <$c1:"term":E>  
            [0 1]  " & "  <$c2:"term":E>  " & " <$c3:"term":E> "}" ] .


(* Syntax sumbool "{_}+{_}". *)
Inductive sumbool [A,B:Prop] : Set
    := left  : A -> ({A}+{B}) 
     | right : B -> ({A}+{B}).

Syntax constr sumbool <<(sumbool $t1 $t2)>> 4
  [<hov 0>  "{" <$t1:"term":L> "}" [0 1] "+" "{" <$t2:"term":E> "}"] .


(* Syntax sumor "_+{_}". *)
Inductive sumor [A:Set;B:Prop] : Set
    := inleft  : A -> (A+{B}) 
     | inright : B -> (A+{B}).

Syntax constr sumor <<(sumor $t1 $t2)>> 4
  [<hov 0>  <$t1:"term":L>  [0 1]  "+" "{" <$t2:"term":E> "}"] .


(***********************)
(* Projections of sigS *)
(***********************)

Section projections_sigma.
Variable A:Set.
Variable P:A->Prop.

Definition proj1_sig :=
 [e:(sig A P)]<A>Case e of [a:A][_:(P a)]a  end.

Definition proj2_sig :=
 [e:(sig A P)]
   <[e:(sig A P)](P (proj1_sig e))>Case e of [a:A][b:(P a)]b  end.
End projections_sigma.




(**********)
(* Choice *)
(**********)

Lemma Choice : (S,S':Set)(R:S->S'->Prop)((x:S){y:S'|(R x y)})
                       -> {f:S->S'|(z:S)(R z (f z))}.
Proof.
Intros S S' R H.
Exists [z:S](<S'>Case (H z) of [y:S'][h:(R z y)]y end).
Intro z; Elim (H z); Trivial.
Qed.

Lemma Choice2 : (S,S':Set)(R:S->S'->Set)((x:S){y:S' & (R x y)})
                       -> {f:S->S' & (z:S)(R z (f z))}.
Proof.
Intros S S' R H.
Exists [z:S](<S'>Case (H z) of [y:S'][h:(R z y)]y end).
Intro z; Elim (H z); Trivial.
Qed.

Lemma bool_choice : (S:Set)(R1,R2:S->Prop)((x:S){(R1 x)}+{(R2 x)}) ->
 {f:S->bool | (x:S)( ((f x)=true  /\ (R1 x)) 
                  \/ ((f x)=false /\ (R2 x)))}.
Proof.
Intros S R1 R2 H.
Exists [z:S](<bool>Case (H z) of [r:(R1 z)]true [r:(R2 z)]false end).
Intro z; Elim (H z); Auto.
Qed.

(* Exceptions *)

Inductive Exc [A:Set] : Set := value : A->(Exc A) 
                             | error : (Exc A).

Syntactic Definition Error := (error ?).
Syntactic Definition Value := (value ?).

Syntax constr Error_imp <<(error $1 $t2)>> 1
   [<hov 0> "Error " <$t2:"term":*> ].

Syntax constr Value_imp <<(value $1 $t2)>> 1
   [<hov 0> "Value " <$t2:"term":*> ].


(*******************************)
(* Self realizing propositions *)
(*******************************)

Axiom False_rec : (P:Set)False->P.
Definition except := False_rec. (* for compatibility with previous versions *)

Syntactic Definition Except := (except ?).

Syntax constr Except_imp <<(except $1 $t2)>> 1
   [<hov 0> "Except " <$t2:"term":*> ].

Theorem absurd_set : (A:Prop)(C:Set)A->(~A)->C.
Proof.
Intros A C h1 h2.
Apply False_rec.
Apply (h2 h1).
Qed.

Theorem and_rec : (A,B:Prop)(C:Set)(A->B->C)->(A/\B)->C.
Proof.
Intros A B C F AB; Apply F; Elim AB; Auto.
Qed.

(** is now a theorem
Axiom eq_rec : (A:Set)(a:A)(P:A->Set)(P a)->(b:A) a=b -> (P b).
**)

Hint left right inleft inright.





(*********************************)
(* Sigma Type at Type level sigT *)
(*********************************)
Inductive sigT [A:Type;P:A->Type] : Type
    := existT : (x:A)(P x) -> (sigT A P).


Section projections_sigT.
   Variable A:Type.
   Variable P:A->Type.

Definition projT1 (* : (A:Type)(P:A->Type)(sigT A P) -> A *)
            := [H:(sigT A P)]<A> Case H of [x:A][h:(P x)]x end.
   
Definition projT2 (* : (A:Type)(P:A->Type)(p:(sigT A P))(P (projT1 A P p)) *)
            := [H:(sigT A P)]<[H:(sigT A P)](P (projT1 H))> 
                   Case H of [x:A][h:(P x)]h end.
End projections_sigT.









(* $Id: Specif.v,v 1.7 1996/02/12 15:28:44 cpaulin Exp $ *)
