(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                   Wf.v                                   *)
(****************************************************************************)

Require PreludeSyntax.
Require Logic.
Require Specif.

(* Well-founded induction principle on Prop *)

Chapter Well_founded.

Variable A : Set.
Variable R : A -> A -> Prop.

(* The accessibility predicate is defined to be non-informative *)

Inductive  Acc : A -> Prop 
   := Acc_intro : (x:A)((y:A)(R y x)->(Acc y))->(Acc x).

Lemma Acc_inv : (x:A)(Acc x) -> (y:A)(R y x) -> (Acc y).
Destruct 1; Trivial.
Defined.

(* the informative elimination :
   let Acc_rec F = let rec wf x = F x wf in wf *)

Section AccRec.
Variable P : A -> Set.
Variable F : (x:A)((y:A)(R y x)->(Acc y))->((y:A)(R y x)->(P y))->(P x).

Fixpoint Acc_rec [x:A;a:(Acc x)] : (P x)
   := (F x (Acc_inv x a) ([y:A][h:(R y x)](Acc_rec y (Acc_inv x a y h)))).

End AccRec.

(* A relation is well-founded if every element is accessible *)

Definition well_founded := (a:A)(Acc a).

(* well-founded induction *)

Theorem well_founded_induction : 
    well_founded ->
        (P:A->Set)((x:A)((y:A)(R y x)->(P y))->(P x))->(a:A)(P a).
Intros; Apply (Acc_rec P); Auto.
Save.

End Well_founded. 

(* Well-founded induction principle on Prop *)

Section Wf_inductor.
Variable A:Set.
Variable R:A->A->Prop.
Theorem well_founded_ind : 
    (well_founded A R) ->
        (P:A->Prop)((x:A)((y:A)(R y x)->(P y))->(P x))->(a:A)(P a).
Proof.
 Intros; Apply (Acc_ind A R P); Auto.
Qed.
End Wf_inductor.


(* $Id: Wf.v,v 1.5 1996/02/13 17:26:41 cpaulin Exp $ *)
