(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                PolyList.v                                *)
(****************************************************************************)

Require Le.

(* Some programs and results about lists *)

Section Lists.
Variable A : Set.

Inductive list : Set := nil : list | cons : A -> list -> list.


Fixpoint app [l:list] : list -> list 
      := [m:list]<list>Case l of
                (* nil *) m 
          (* cons a l1 *) [a:A][l1:list](cons a (app l1 m)) end.


Token "^".
Grammar command command7 :=
   [ command6($l) "^" command7($l') ] -> [$0 =<<(app $l $l')>>]. 

Syntax constr append <<(app $l $l')>> 10 
  [ <hov 0> <$l:"list":L> "^" <$l':"list":L> ].



Lemma app_nil_end : (l:list)l=(l^nil).
Proof. 
	Induction l ; Simpl ; Auto.
	(*  (a:A)(y:list)
	     (<list>y=(app y nil))->(<list>(cons a y)=(cons a (app y nil)))
	    ============================
	      l : list *)
        Induction 1; Auto.
Qed.
Hint app_nil_end.

Lemma app_ass : (l,m,n : list)<list>((l^m)^ n)=(l^(m^n)).
Proof. 
	Intros l m n ; Elim l ; Simpl ; Auto.
	(*  (a:A)(y:list)(<list>(app (app y m) n)=(app y (app m n)))->
	     (<list>(cons a (app (app y m) n))=(cons a (app y (app m n))))
	    ============================
	      n : list
	      m : list
	      l : list *)
	Induction 1; Auto.
Qed.
Hint app_ass.

Lemma ass_app : (l,m,n : list)(l^(m^n))=((l^m)^n).
Proof. 
	Auto.
Qed.
Hint ass_app.

Definition tail := [l:list]<list>Case l of
                 (* nil *) nil 
            (* cons a m *) [a:A][m:list]m end : list->list.

Lemma nil_cons : (a:A)(m:list)~(nil=(cons a m)).
Intros;Discriminate.
Qed.


 


Lemma app_comm_cons : (x,y:list)(a:A) (cons a (x^y))=((cons a x)^y).
Proof.
 Auto.
Qed.


Lemma app_eq_nil: (x,y:list) (x^y)=nil -> x=nil /\ y=nil.
Proof.
 (Destruct x;Destruct y;Simpl;Auto).
 (Intros H;Discriminate H).
 (Intros a0 l0 H;Discriminate H).
Qed.



Lemma app_cons_not_nil: (x,y:list)(a:A)~nil=(x^(cons a y)).
Proof.
Unfold not .
 (Induction x;Simpl;Intros).
 Discriminate H.
 Discriminate H0.
Qed.

Lemma app_eq_unit:(x,y:list)(a:A)
     (x^y)=(cons a nil)-> (x=nil)/\ y=(cons a nil) \/ x=(cons a nil)/\ y=nil.

Proof.
 (Destruct x;Destruct y;Simpl).
 (Intros a H;Discriminate H).
 (Left;Split;Auto).
 (Right;Split;Auto).
 Generalize H .
 (Generalize (app_nil_end  l) ;Intros E).
 (Rewrite <- E;Auto).
 Intros.
 Injection H.
 Intro.
 (Cut  nil=(l^(cons a0 l0));Auto).
 Intro.
 (Generalize (app_cons_not_nil  l l0 a0 H1) ;Intro).
 Elim H2.
Qed.


Lemma app_inj_tail : (x,y:list)(a,b:A)
   (x^(cons a nil))=(y^(cons b nil)) -> x=y /\ a=b.
Proof.
 (Induction x;Induction y;Simpl;Auto).
 Intros.
 Injection H.
 Auto.
 Intros.
 (Injection H0;Intros).
 (Generalize (app_cons_not_nil  l nil b H1) ;Induction 1).
 Intros.
 (Injection H0;Intros).
 (Cut   nil=(l^(cons a0 nil));Auto).
 Intro.
 (Generalize (app_cons_not_nil  l nil a0 H3) ;Induction 1).
 Intros.
 (Injection H1;Intros).
 Generalize (H l0 a1 b H2) .
 (Induction 1;Split;Auto).
 (Rewrite <- H3;Rewrite <- H5;Auto).
Qed.


(****************************************)
(* Length of lists                      *)
(****************************************)

Fixpoint length [l:list] : nat 
   := (<nat>Case l of (* nil *) O
                      (* cons a m *) [a:A][m:list](S (length m)) end).

(******************************)
(* Length order of lists      *)
(******************************)

Section length_order.
Definition lel [l,m:list](le (length l) (length m)).

Variables a,b:A.
Variables l,m,n:list.

Lemma lel_refl : (lel l l).
Proof. 
	Unfold lel ; Auto.
Qed.

Lemma lel_trans : (lel l m)->(lel m n)->(lel l n).
Proof. 
	Unfold lel ; Intros.
	(*  (le (length l) (length n))
	    ============================
	      H0 : (le (length m) (length n))
	      H : (le (length l) (length m)) *)
        Apply le_trans with (length m) ; Auto.
Qed.

Lemma lel_cons_cons : (lel l m)->(lel (cons a l) (cons b m)).
Proof. 
	Unfold lel ; Simpl ; Auto.
Qed.

Lemma lel_cons : (lel l m)->(lel l (cons b m)).
Proof. 
	Unfold lel ; Simpl ; Auto.
Qed.

Lemma lel_tail : (lel (cons a l) (cons b m)) -> (lel l m).
Proof. 
	Unfold lel ; Simpl ; Auto.
Qed.

Lemma lel_nil : (l':list)(lel l' nil)->(nil=l').
Proof. 
	Intro l' ; Elim l' ; Auto.
	Intros a' y H H0.
	(*  <list>nil=(cons a' y)
	    ============================
	      H0 : (lel (cons a' y) nil)
	      H : (lel y nil)->(<list>nil=y)
	      y : list
	      a' : A
	      l' : list *)
	Absurd (le (S (length y)) O); Auto.
Qed.
End length_order.

Hint lel_refl lel_cons_cons lel_cons lel_nil lel_nil nil_cons.

Fixpoint In  [a:A;l:list] : Prop :=
      (<Prop>Case l of
                    (* nil *) False 
                    (* cons b m *) [b:A][m:list](b=a)\/(In a m) end).

Lemma in_eq : (a:A)(l:list)(In a (cons a l)).
Proof. 
	Simpl ; Auto.
Qed.
Hint in_eq.

Lemma in_cons : (a,b:A)(l:list)(In b l)->(In b (cons a l)).
Proof. 
	Simpl ; Auto.
Qed.
Hint in_cons.

Lemma in_app_or : (l,m:list)(a:A)(In a (l^m))->((In a l)\/(In a m)).
Proof. 
	Intros l m a.
	Elim l ; Simpl ; Auto.
	Intros a0 y H H0.
	(*  ((<A>a0=a)\/(In a y))\/(In a m)
	    ============================
	      H0 : (<A>a0=a)\/(In a (app y m))
	      H : (In a (app y m))->((In a y)\/(In a m))
	      y : list
	      a0 : A
	      a : A
	      m : list
	      l : list *)
	Elim H0 ; Auto.
	Intro H1.
	(*  ((<A>a0=a)\/(In a y))\/(In a m)
	    ============================
	      H1 : (In a (app y m)) *)
	Elim (H H1) ; Auto.
Qed.
Immediate in_app_or.

Lemma in_or_app : (l,m:list)(a:A)((In a l)\/(In a m))->(In a (l^m)).
Proof. 
	Intros l m a.
	Elim l ; Simpl ; Intro H.
	(* 1 (In a m)
	    ============================
	      H : False\/(In a m)
	      a : A
	      m : list
	      l : list *)
	Elim H ; Auto ; Intro H0.
	(*  (In a m)
	    ============================
	      H0 : False *)
	Elim H0. (* subProof completed *)
	Intros y H0 H1.
	(*  2 (<A>H=a)\/(In a (app y m))
	    ============================
	      H1 : ((<A>H=a)\/(In a y))\/(In a m)
	      H0 : ((In a y)\/(In a m))->(In a (app y m))
	      y : list *)
	Elim H1 ; Auto 4.
	Intro H2.
	(*  (<A>H=a)\/(In a (app y m))
	    ============================
	      H2 : (<A>H=a)\/(In a y) *)
	Elim H2 ; Auto.
Qed.
Hint in_or_app.

Definition incl [l,m:list](a:A)(In a l)->(In a m).

Hint Unfold incl.

Lemma incl_refl : (l:list)(incl l l).
Proof. 
	Auto.
Qed.
Hint incl_refl.

Lemma incl_tl : (a:A)(l,m:list)(incl l m)->(incl l (cons a m)).
Proof. 
	Auto.
Qed.
Immediate incl_tl.

Lemma incl_tran : (l,m,n:list)(incl l m)->(incl m n)->(incl l n).
Proof. 
	Auto.
Qed.

Lemma incl_appl : (l,m,n:list)(incl l n)->(incl l (n^m)).
Proof. 
	Auto.
Qed.
Immediate incl_appl.

Lemma incl_appr : (l,m,n:list)(incl l n)->(incl l (m^n)).
Proof. 
	Auto.
Qed.
Immediate incl_appr.

Lemma incl_cons : (a:A)(l,m:list)(In a m)->(incl l m)->(incl (cons a l) m).
Proof. 
	Unfold incl ; Simpl ; Intros a l m H H0 a0 H1.
	(*  (In a0 m)
	    ============================
	      H1 : (<A>a=a0)\/(In a0 l)
	      a0 : A
	      H0 : (a:A)(In a l)->(In a m)
	      H : (In a m)
	      m : list
	      l : list
	      a : A *)
	Elim H1.
	(*  1 (<A>a=a0)->(In a0 m) *)
	Elim H1 ; Auto ; Intro H2.
	(*  (<A>a=a0)->(In a0 m)
	    ============================
	      H2 : <A>a=a0 *)
	Elim H2 ; Auto. (* solves subgoal *)
	(*  2 (In a0 l)->(In a0 m) *)
	Auto.
Qed.
Hint incl_cons.

Lemma incl_app : (l,m,n:list)(incl l n)->(incl m n)->(incl (l^m) n).
Proof. 
	Unfold incl ; Simpl ; Intros l m n H H0 a H1.
	(*  (In a n)
	    ============================
	      H1 : (In a (app l m))
	      a : A
	      H0 : (a:A)(In a m)->(In a n)
	      H : (a:A)(In a l)->(In a n)
	      n : list
	      m : list
	      l : list *)
	Elim (in_app_or l m a) ; Auto.
Qed.
Hint incl_app.



(*********************************************)
(*   Reverse Induction Principle on Lists    *)
(*********************************************)


Section Reverse_Induction.
Variable leA: A->A->Prop.

Fixpoint rev [l:list] : list :=
 <list>Case l of 
              nil
              [x:A][l':list] (rev l')^(cons x nil)
   end.



Lemma distr_rev : 
 (x,y:list) (rev (x^y))=((rev y)^(rev x)).
Proof.
 Induction x.
 Induction y.
 Simpl.
 Auto.

 Simpl.
 Intros.
 (Apply app_nil_end;Auto).

 Intros.
 Simpl.
 (Generalize (H y) ;Intros E;Rewrite -> E).
 Apply (app_ass (rev y) (rev l) (cons a nil)).
Qed.



Remark  rev_unit : (l:list)(a:A) (rev  l^(cons a nil))= (cons a (rev l)).
Proof.
 Intros.
( Apply (distr_rev l (cons a nil));Simpl;Auto).
Qed.


Lemma idempot_rev : (l:list)(rev (rev l))=l.
Proof.
 Induction l.
 (Simpl;Auto).

 Intros.
 Simpl.
 Generalize (rev_unit (rev l0) a); Intros.
 Rewrite -> H0.
 Rewrite -> H;Auto.
Qed.



Remark rev_list_ind: (P:list->Prop)
      (P nil)
       ->((a:A)(l:list)(P (rev l))->(P (rev  (cons a l)))) 
             ->(l:list) (P (rev  l)).
Proof.
 Induction l; Auto.
Qed.

Lemma rev_ind : 
 (P:list->Prop)
   (P nil)->
    ((x:A)(l:list)(P l)->(P l^(cons x nil)))
      ->(l:list)(P l).
Proof.
 Intros.
 Generalize (idempot_rev l) .
 (Intros E;Rewrite <- E).
 Apply (rev_list_ind P).
 Auto.

 Simpl.
 Intros.
 Apply (H0 a (rev l0)).
 Auto.
Qed.

End Reverse_Induction.

End Lists.













(* $Id: PolyList.v,v 1.3 1996/02/12 15:28:58 cpaulin Exp $ *)
