(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                Ack_prog.v                                *)
(****************************************************************************)

(*****************************************************************************)
(*                                                                           *)
(*                    		Definitions             	             *)
(*                                                                           *)
(*****************************************************************************)

Require Program.

Inductive Ack : nat->nat->nat->Prop :=
     AckO : (n:nat)(Ack O n (S n))
   | AcknO : (n,p:nat)(Ack n (S O) p)->(Ack (S n) O p)
   | AckSS : (n,m,p,q:nat)
             (Ack (S n) m q)->(Ack n q p)->(Ack (S n) (S m) p).

Hint AckO AcknO.


(*****************************************************************************)
(*                                                                           *)
(*                    Automatic proof from the program                       *)
(*                                                                           *)
(*****************************************************************************)

(* Functional definition of Ackermann :
 (ack 0 n) = (S n)
 (ack (S n) 0) = (ack n (S 0))
 (ack (S n) (S m)) = (ack n (ack (S n) m)) *)

Fixpoint ack_func [n:nat] : nat -> nat :=
  <nat->nat>Case n of
     (* 0 *) [m:nat](S m)
 (* (S n) *) [n':nat]
		Fix ack_func2 {ack_func2/1 : nat -> nat :=
		    [m:nat]<nat>Case m of
              (* 0 *) (ack_func n' (S O))
            (* S m *) [m':nat](ack_func n' (ack_func2 m')) 
		     end} 
    end.

Theorem Ackermann : (n,m:nat){p:nat|(Ack n m p)}.
Realizer ack_func.
Program_all.
Apply AckSS with x; Auto.
Save.



(* $Id: Ack_prog.v,v 1.2 1996/02/12 15:29:21 cpaulin Exp $ *)
