(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             Dijkstra_prog.v                              *)
(****************************************************************************)

Require Program.
Require Arith.
Require Wf_nat.
Require Compare.
Require SetTheory.

Definition eq_spec := [A:Set][a,b:A][H:a=b][P:A->Set][H':(P a)]
                     (eq_rec A a P H' b H).

(******************************************************)
(* Definition du et logique entre une Prop et un Set *)
(******************************************************)

Hint Unfold incl.

Inductive and_s [A:Prop;B:Set] : Set :=
    Pair_s : A->B->(and_s A B).
Hint Pair_s.

(*******************************************************)
(* On met une relation decidable entre deux sommets    *)
(* et il y a une et une seule arete entre deux sommets *)
(*******************************************************)

Variable E : V->V->nat->Prop.

Axiom Edge_un : (x,y:V) (n,m:nat) (E x y n)->(E x y m)->(n=m).


(*********************************************************)
(* Deux sommets sont soit relies par une arete de cout n *)
(* soit par un arc de cout infini                        *)
(*********************************************************)

Inductive binding [x,y:V] : Set
   := Poids : (n:nat) (E x y n) -> (binding x y)
   | Infini: ((n:nat) ~(E x y n))-> (binding x y).
Hint (*Poids*) Infini.

Axiom dec_E : (x,y:V) (binding x y).
Hint dec_E.

(*********************************************)
(* Definition d un chemin entre deux sommets *)
(*********************************************)

Inductive path [W:set;x:V] : V->nat->Set
   := Edge : (y:V) (n:nat) (E x y n) -> (path W x y n)
   | Composed : (y,z:V) (n,m:nat) (path W x y n)->(In W y)->(E y z m)
                                    ->(path W x z (plus n m)).
Hint Edge.

Definition Edgec := [W:set][x:V][y:V][n:nat](Edge W x y n).
Definition Composedc := [W:set][x:V][y,z:V][n,m:nat](Composed W x y z n m).

Lemma Path_incl : (W,W':set) (x,y:V) (n:nat) (incl W W')->(path W x y n)->(path W' x y n).
Realizer [x,y:V][n:nat][H0:path]
             (path_rec x path
		    [y0:V][n0:nat](Edgec x y0 n0)
                    [y0,z:V][n0,m:nat][y1,H1:path](Composedc x y0 z n0 m H1)
	     y n H0).
Program_all.
Save.

Lemma Path_incl_add : (W:set)(x,y,z:V)(n:nat)(path W x y n)->(path (add W z) x y n).
Realizer [x,y,z:V][n:nat][H:path](Path_incl x y n H).
Program_all.
Save.
Hint Path_incl_add.

Lemma Path_match : (W:set) (x,z:V) (n:nat) (path W x z n)->(P:Prop) 
     ((E x z n)->P)->
     ((y:V) (m,p:nat) (path W x y m)->(In W y)->(E y z p)->
                                     ((plus m p)=n)->P)->P.
Induction 1 ; Intros.
Apply H0 ; Assumption.
Apply H2 with y n0 m ; Auto.
Save.

(****************************)
(* Axiome d'extensionnalite *)
(****************************)

Axiom Ext : (V,W:set) (incl V W)->(incl W V)->(P:set->Set)(P V)->(P W).

Axiom Ext0 : (V,W:set) (incl V W)->(incl W V)->(P:set->Prop)(P V)->(P W).


(********************************)
(* Definition sur les ensembles *)
(********************************)

Definition soustr := [Q:set][x,y:V] (In Q y) /\ ~x=y.
    
Definition quo := [Q:set][y:V] (~(In Q y)).

Definition inter := [P,Q:set][x:V] (In P x) /\ (In Q x).

Inductive taille : set->nat->Prop
    := vide : (taille empty O)
    | non_vide : (y:V)(m:nat)(M:set)(~(In M y))
                 ->(taille M m)->(taille (add M y) (S m)).
Hint vide.


(* Proprietes de ces definitions *)

Lemma inter_incl_l : (P,Q:set) (incl (inter P Q) P).
Red ; Intros P Q x H ; Elim H ; Auto.
Save.

Lemma inter_incl_r : (P,Q:set) (incl (inter P Q) Q).
Red ; Intros P Q x H; Elim H ; Auto.
Save.

Lemma inter_intro : (P,Q:set)(x:V)(In P x)->(In Q x)->(In (inter P Q) x).
Red;Red;Auto.
Save.
Hint inter_intro.

Lemma In_quo_add : (P:set)(x,y:V)(~x=y)->(In (quo P) y)->(In (quo (add P x)) y).
Red;Red;Red;Intros.
Elim H1;Intros.
Absurd (In P y);Auto.
Absurd x=y;Auto.
Save.
Hint In_quo_add.

Lemma lemme_taille_1 : (Q:set) (y:V) (n:nat) (taille Q n)->(In Q y)->~n=O.
Intros Q y n H ; Elim H ; Intros.
Elim H0.
Red ; Intro.
Apply (O_S m) ; Auto.
Save.

Lemma incl_soustr_in : (M:set)(y:V)(In M y)->(incl (soustr M y) M).
Red;Intros.
Elim H0; Auto.
Save.
Hint incl_soustr_in.

Lemma incl_soustr : (M,M':set)(y:V)(incl M M')->(incl (soustr M y) (soustr M' y)).
Unfold soustr; Red; Red; Intros.
Elim H0;Intros.
Apply conj;Auto.
(*Apply H;Auto.*)
Save.
Hint incl_soustr.

Lemma not_In_soustr : (M:set)(y,z:V)(~(In M y))->~(In (soustr M z) y).
Red;Intros.
Elim H0; Intros; Apply H; Auto.
Save.
Hint not_In_soustr.

Lemma not_In_soustr_eq : (M:set)(y:V)~(In (soustr M y) y).
Red;Intros.
Elim H; Intros.
Apply H1;Auto.
Save.
Hint not_In_soustr_eq.

Lemma S_pred_m : (m:nat)(~<nat>m=O)-><nat>(S (pred m))=m.
Induction m;Simpl;Auto.
Intro ; Absurd <nat>O=O ;Auto.
Save.
Hint S_pred_m.

Lemma incl_soustr_add_l : (M:set)(y:V)(incl (soustr (add M y) y) M).
Red; Intros.
Elim H; Intros.
Elim H0; Intros; Auto.
Absurd y=x; Auto.
Save.
Hint incl_soustr_add_l.


Lemma incl_soustr_add_r : (M:set)(y:V)(~(In M y))->(incl M (soustr (add M y) y)). 
Red; Intros.
Red;Red.
Apply conj;Auto.
Red;Intro; Absurd (In M x); Auto.
Elim H1; Auto.
Save.
Hint incl_soustr_add_r.

Lemma lemme_taille_2 : (Q:set) (n:nat) (taille Q (S n))->(y:V) (In Q y)->(taille (soustr Q y) n).
Intros Q n H.
Replace n with (pred (S n));Try Reflexivity.
Elim H ; Intros.
Elim H0.
Simpl.
Elim H3 ; Intro.
Cut ~y0=y.
Intro;Pattern (soustr (add M y) y0);Apply Ext0 with (add (soustr M y0) y).
Red ; Intros.
Red; Red.
Elim H6 ; Intro.
Elim H7 ; Auto.
Elim H7 ; Auto.
Red ; Intros.
Red; Red.
Unfold In soustr.
Elim H6 ; Intros.
Elim H7 ; Auto.
Replace m with (S(pred m)).
Apply non_vide ; Auto.
Cut ~m=O;Auto.
Apply (lemme_taille_1 M y0) ; Auto.
Red;Intro;Apply H0.
Elim H5; Auto.
Elim H4.
Pattern (soustr (add M y) y);Apply Ext0 with M;Auto.
Save.
Hint lemme_taille_2.


Lemma add_soustr_2 : (Q:set) (y:V) (In Q y)->(incl Q (add (soustr Q y) y)).
Intros.
Red ; Intros.
Red; Red.
Unfold In soustr.
Elim (eqV y x) ; Intro ; Auto.
Save.
Hint add_soustr_2.

Lemma add_soustr_1 : (Q:set) (y:V) (In Q y)->(incl (add (soustr Q y) y) Q).
Intros.
Red  ; Intros.
Elim H0 ; Intros.
Elim H1 ; Auto.
Elim H1 ; Assumption.
Save.
Hint add_soustr_1.

Lemma incl_add : (Q,Q':set)(y:V)(incl Q Q')->(incl (add Q y) (add Q' y)).
Red;Intros.
Red;Red.
Elim H0; Auto.
(*Intro;Left;Apply H;Auto.*)
Save.
Hint incl_add.

Lemma In_not_In : (Q:set)(x,y:V)(In Q x)->(~(In Q y))->~x=y.
Red; Intros.
Apply H0; Elim H1;Auto.
Save.

Lemma not_eq_sym_V : (x,y:V)(~x=y)->~y=x.
Red; Intros.
Apply H; Auto.
Save.
Immediate not_eq_sym_V.

Lemma In_not_In_sym : (Q:set)(x,y:V)(In Q x)->(~(In Q y))->~y=x.
Intros; Apply not_eq_sym_V; Apply (In_not_In Q); Auto.
Save.

Lemma same_size : (Q:set) (n:nat) (taille Q n)->(Q':set) (taille Q' n)->(incl Q Q')
                      ->(incl Q' Q).
Intros Q n H ; Elim H ; Intros.
Red ; Intros.
Absurd O=O;Auto.
Apply (lemme_taille_1 Q' x) ; Auto.
Cut (In Q' y);Intros.
Pattern Q';Apply Ext0 with (add (soustr Q' y) y);Auto.
Apply incl_add; Apply H2;Auto.
Red; Red; Red; Intros.
Apply conj.
Apply H4; Auto.
Apply (In_not_In_sym M); Auto.
Apply H4; Auto.
Save.

(***************************************)
(* On travaille dans un ensemble fini  *)
(***************************************)

Axiom finit_V : (enumerate allV).

Lemma card_allV : {n:nat|(taille allV n)}.
Realizer <nat>Match finit_V with
		 O
                 [y,y0,H](S H) 
	 end.
Program_all.
Apply non_vide ; Auto.
Save.

(**********************************)
(* et on choisit un sommet source *)
(**********************************)

Variable source : V.

(**********************************)
(* definition d un chemin minimum *)
(**********************************)

Definition path_inf := [W:set][x:V][n:nat]((m:nat) (path W source x m)->(le n m)).

(***********************************************)
(* Definition de la decomposition d'un chemin *)
(***********************************************)

Inductive decomposed [CC:set;m:nat;x:V] : Set :=
    decomp_Intro1 : (y:V) (m':nat) (In (quo CC) y)->(path CC source y m')->
                          (le m' m) ->(decomposed CC m x)
  | decomp_Intro2 : (path CC source x m)->(decomposed CC m x).
Hint decomp_Intro2.

Local decomp_Intro1c :=
	 [CC:set][m:nat][x:V][y:V][m':nat](decomp_Intro1 CC m x y m').

Local decomp_Intro2c := [CC:set][m:nat][x:V](decomp_Intro2 CC m x).

Lemma decomp : (CC:set)(x:V)(m:nat)((x:V) {In CC x}+{In (quo CC) x})
                         ->(path allV source x m)->(decomposed CC m x).
Realizer [x:V][m:nat][H:V->bool][H0:path]
           (path_rec source decomposed 
             [y:V][n:nat](decomp_Intro2c n y (Edgec source y n))
             [y,z:V][n,m0:nat][y0:path][H1:decomposed]
                 <decomposed>Match H1 with
                   [y1,m',p](decomp_Intro1c (plus n m0) z y1 m' p)
                   [p]<decomposed>if (H y)
                             then (decomp_Intro2c (plus n m0) z 
					      (Composedc source y z n m0 p))
                             else (decomp_Intro1c (plus n m0) z y n p)
		 end
	   x m H0).
Program_all.
Save.

(***********************************************)
(* Definition de connection entre deux sommets *)
(* soit il existe un chemin min                 *)
(* soit il n y a pas de chemin                 *)
(***********************************************)

Inductive connect_s [W,W':set;y:V] : Set 
   :=  Min_path : (n:nat) (path W source y n)->(path_inf W' y n)
                         ->(connect_s W W' y)
   |  No_path : ((n:nat) (path W' source y n)->False)->(connect_s W W' y).
Hint No_path.

Definition Min_pathc := [W,W':set][y:V][n:nat](Min_path W W' y n).
Definition No_pathc := [W,W':set][y:V](No_path W W' y).

Lemma connect_incl : (W,W',Q:set) (y:V) (incl W W')->(connect_s W Q y)->(connect_s W' Q y).
Realizer [y:V][H0:connect_s]
              <connect_s>Case H0 of
                     [n,p](Min_pathc y n (Path_incl source y n p))
                     (No_pathc y)
	      end.
Program_all.
Save.

Lemma connect_incl_add : (W,Q:set) (x,y:V) (connect_s W Q y)->(connect_s (add W x) Q y).
Realizer [x,y:V][H:connect_s](connect_incl y H).
Program_all.
Save.
Hint connect_incl_add.

(*********************************************************************)
(* Demonstration de la propriete qui fait marcher l'algo de Dijkstra *)
(*********************************************************************)

Lemma prop_dij : (CC:set)(x:V)(n:nat)((x:V) {In CC x}+{In (quo CC) x})->
     (path CC source x n)->(path_inf CC x n)
     ->((y:V)(In (quo CC) y)->(path_inf CC y n))->(path_inf allV x n).
Unfold path_inf; Intros.
Elim (decomp CC x m H H3); Intros;Auto.
Apply le_trans with m';Auto.
Apply (H2 y) ; Auto.
Save.

(*****************************)
(* Setification de dijkstra *)
(*****************************)

Inductive specif [CC,P:set] : Set
    := path_end : ((x:V)(n:nat) (In P x)->(path CC source x n)->False)->(specif CC P)
    | path_min : (x:V)(n:nat)(In P x)->(path CC source x n)->
                      ((y:V) (In P y)->(path_inf CC y n))->(specif CC P).
Hint path_end.

Lemma lemme_specif : (CC,P:set) (enumerate P)->((x:V) (In P x)->(connect_s CC CC x))
     ->(specif CC P).
Realizer [H:enumerate]<(V->connect_s)->specif>Match H with
                     [H0]path_end
                     [y,y0,H0,H1]
                        <specif>Match (H0 [x:V](H1 x)) with
                            <specif>Match (H1 y) with
                               [n0,p](path_min y n0 p)
                               path_end 
			    end
                            [x,n0,p]
                                <specif>Match (H1 y) with
                                  [n1,p1]<specif>if (le_dec n0 n1)
                                      	    then (path_min x n0 p)
                                      	    else (path_min y n1 p1) 
                                  (path_min x n0 p)
				end
			end
		     end.
Program_all.
(* (y0:V)(In (add Q y) y0)->(path_inf CC y0 n0) *)
Intros; Elim H0; Intro.
Red; Intros.
Elim (f y0 m); Auto.
Elim H2 ; Auto.

(* (x:V)(n0:nat)(In (add Q y) x)->(path CC source x n0)->False *)
Intros; Elim H0; Intro.
Apply (f x n0) ; Auto.
Apply (f0 n0).
Rewrite -> H3 ; Auto.

(* (y0:V)(In (add Q y) y0)->(path_inf CC y0 n0) *)
Intros; Elim H0; Intro; Auto.
Elim H2; Red; Intros.
Apply le_trans with n1; Auto.

(* (y0:V)(In (add Q y) y0)->(path_inf CC y0 n1) *)
Intros; Elim H0; Intro.
Red; Intros.
Apply le_trans with n0 ; Auto.
Apply (p0 y0 H2) ; Auto.
Elim H2 ; Auto.

(* (y0:V)(In (add Q y) y0)->(path_inf CC y0 n0) *)
Intros; Elim H0; Intro; Auto.
Elim H2; Red; Intros.
Elim (f m); Auto.

Save.

(**************************)
(* Definition de Dijkstra *)
(**************************)

Inductive dijkstra [CC:set] : Set :=
    dij_Intro : 
    ((x:V) ((and_s (In CC x) (connect_s CC allV x))
           +(and_s (In (quo CC) x) (connect_s CC CC x))))
    ->(dijkstra CC).

Inductive specification [P:set->Set;C:set] : Set
    := ends : (P allV)->(specification P C)
    | chooses : (y:V)(In (quo C) y)->(P (add C y))->(specification P C).

Definition dij_spec := (specification dijkstra).

Definition  fin := (ends dijkstra).
Definition choose := (chooses dijkstra).

(* Un sous-ensemble decidable d'un ens. fini est fini *)

Axiom Ext_allV : (W:set) (incl (inter allV W) W)->(incl W (inter allV W))->(P:set->Set)(P (inter allV W))->(P W).

Lemma finit : (P:set) ((x:V) {In P x}+{In (quo P) x})->(enumerate P).
Realizer [H:V->bool]
           (Ext_allV enumerate
              <enumerate>Match finit_V with
		(Ext enumerate enu_empty)
                 [y,y0,H0]<enumerate>if (H y)
                      	    then (Ext enumerate (enu_add y H0))
                      	    else (Ext enumerate H0)
	       end).
Program_all.
Apply inter_incl_r.
(* Red;Red;Red; Auto. *)
Apply inter_incl_l.
Red ; Red; Red ;Intros.
Elim H0 ; Intros.
Elim H1 ; Auto.
Elim H1 ; Auto.
Red ; Red; Red ;Intros.
Elim H0 ; Intros.
Elim H1 ; Auto.
Red ; Intro.
Elim H0 ; Intros.
Apply n ; Auto.
Red ; Intros.
Elim H0 ; Auto.
Red ; Intros.
Elim H0 ; Intros.
Elim H1 ; Intros ; Auto.
Absurd (In P x);Auto.
Elim H3; Auto.
Save.

(* Le schema de preuve utilise : *)

Section Induc_scheme.
Variables x,y :V.
Variables n,m,p:nat.
Variable Q : set.

Inductive Dec_add : Prop :=
   Dec_add_direct : (q:nat)(path Q source y q)->(le q p)->Dec_add
 | Dec_add_edge : (q:nat)(E x y q)->(le (plus n q) p)->Dec_add.

Lemma induc_scheme : (path_inf allV x n)->((z:V)(In Q z)->(connect_s Q allV z))
     ->(path (add Q x) source y p)->Dec_add.
Intros.
Apply (Path_match (add Q x) source y p H1); Intros.
Apply (Dec_add_direct p); Auto.
Elim H3; Intros.
Elim (H0 y0 H6).
Intros q H7 H8.
Apply (Dec_add_direct (plus q p0));Intros.
Apply Composedc with y0; Auto.
Elim H5; Apply le_reg_r.
Apply H8.
Apply (Path_incl (add Q x)); Auto.
Intro H7.
Elim (H7 m0).
Apply (Path_incl (add Q x)); Auto.
Apply (Dec_add_edge p0).
Rewrite -> H6; Auto.
Elim H5; Apply le_reg_r.
Apply H.
Rewrite -> H6; Auto.
Apply (Path_incl (add Q x)); Auto.
Save.
End Induc_scheme.
Hint induc_scheme.
     
Inductive specif' [P:set->Set;n:nat] : Set
    := c_all : (P allV)->(specif' P n)
    | part : (C:set)(taille C n)->(P C)->(specif' P n).

Definition c_allc := [P:set->Set][n:nat](c_all P n).
Definition partc := [P:set->Set][n:nat][C:set](part P n C).

Lemma principe_recurrence : (P:set->Set) (P empty)->((C:set) (P C)->(specification P C))->(P allV).
Realizer [P:Set][H:P][H0:P->(specification P)]
	 <P>Case (nat_rec (specif' P) (partc P O H)
             [y:nat][H1:(specif' P)]
                <(specif' P)>Case H1 of
		  [p:P](c_allc P (S y) p)
                  [p:P]<(specif' P)>Case (H0 p) of
                      		[p0:P](c_allc P (S y) p0)
                      		[y0:V][p0:P](partc P (S y) p0)
		       end
	        end card_allV) of
		[p0]p0 
		[p0](Ext P p0)
	 end.
Program_all.
Apply non_vide ; Auto.
Apply same_size with x ; Auto.
Save.

Lemma Path_empty : (x,y:V) (n:nat)(path empty x y n)->(E x y n).
Intros.
Elim H; Intros; Auto.
Elim i.
Save.
Hint Path_empty.

Theorem proof_dij_spec : (CC:set)(dijkstra CC)->(dij_spec CC).
Realizer 
  [{CC:set}][H:V->(connect_s+connect_s)]
   <dij_spec>Case
      (lemme_specif 
         (finit [x:V]<bool>Case (H x) of
                      (* inl *) [_]false
                      (* inr *) [_]true
		     end)
         [x:V]<connect_s>Case (H x) of
                (* inl *) [_](No_pathc x)
                (* inr *) [H2]H2
	      end)
         :: :: { (specif CC (quo CC)) } of
(* path_end *)
  (fin [x:V](inl ? ? <connect_s>Case (H x) of
                        (* inl *) [a](connect_incl x a)
                        (* inr *) [_](No_pathc x)
		     end))
(* path_min *)
   [x,n,p](choose x [x0:V]
         <connect_s+connect_s>Case (H x0) of
   (* inl *) [H1](inl ? ? (connect_incl_add x x0 H1))
   (* inr *) [H1]<connect_s+connect_s>Case (eqV x x0) of
                 (* left *) 
                    (inl ? ? (eq_spec V x x0 connect_s
                               (Min_pathc x n (Path_incl_add source x x n p))))
                 (* right *) 
                    (inr ? ? <connect_s>Case H1 of
                   (* Min_path *)
                      [n0,p1]
                       <connect_s>Case (dec_E x x0) of
                          (* Poids *)
                             [n1]
                              (Min_pathc x0 (min n0 (plus n n1))
                                 (min_case n0 (plus n n1) path
                                      (Path_incl_add source x0 x n0 p1)
                                      (Composedc source x x0 n n1
                                              (Path_incl_add source x x n p))))
                         (* Infini *)
                            (Min_pathc x0 n0 (Path_incl_add source x0 x n0 p1))
			end
                   (* No_path *)
                    <connect_s>Case (dec_E x x0) of 
                          (* Poids *) 
                           [n0](Min_pathc x0 (plus n n0)
                                    (Composedc source x x0 n n0
                                      (Path_incl_add source x x n p)))
                    (* Infini *) (No_pathc x0)
		     end
			  end)
		end
	 end)
  end.
Program_all.
(* (In (quo (quo CC)) x) *)
Red;Red;Red;Intro.
Absurd (In CC x); Auto.

(* (n:nat)(path allV source x n)->False *)
Intros.
Elim (decomp CC x n); Intros.
Apply (f y m'); Auto.
Apply (f x n); Auto.
Elim (H0 x0); Intro H2; Elim H2; Auto.
Assumption.

(* (path_inf allV x n) *)
Apply (prop_dij CC) ; Auto.
Intro; Elim (H0 x1); Intro H3; Elim H3; Auto.

(* (path_inf (add CC x) x0 (min n1 (plus n n2))) *)
Red; Intros.
Elim (induc_scheme x x0 n m CC); Intros; Trivial.
Apply le_trans with q; Auto.
Apply le_trans with n1; Auto.
Replace n2 with q.
Apply le_trans with (plus n q); Auto.
Apply Edge_un with x x0 ; Auto.
Apply (prop_dij CC) ; Auto.
Intro; Elim (H0 x1); Intro H4; Elim H4; Auto.
Elim (H0 z); Intro H5; Elim H5; Intros; Auto.

(* (path_inf (add CC x) x0 n1) *)
Red; Intros.
Elim (induc_scheme x x0 n m CC); Intros; Trivial.
Apply le_trans with q; Auto.
Apply p2; Auto.
Absurd (E x x0 q); Auto.
Apply (prop_dij CC) ; Auto.
Intro; Elim (H0 x1); Intro H4; Elim H4; Auto.
Elim (H0 z); Intro H5; Elim H5; Intros; Auto.

(* (path_inf (add CC x) x0 (plus n n1)) *)
Red; Intros. 
Elim (induc_scheme x x0 n m CC); Intros; Trivial.
Elim (f q); Auto.
Replace n1 with q; Auto.
Apply Edge_un with x x0 ; Auto.
Apply (prop_dij CC) ; Auto.
Intro; Elim (H0 x1); Intro H4; Elim H4; Auto.
Elim (H0 z); Intro H5; Elim H5; Intros; Auto.

(* (n:nat)(path (add CC x) source x0 n)->False *)
Intros; Elim (induc_scheme x x0 n n2 CC); Intros; Trivial.
Apply (f q); Auto.
Absurd (E x x0 q); Auto.
Apply (prop_dij CC) ; Auto.
Intro; Elim (H0 x1); Intro H4; Elim H4; Auto.
Elim (H0 z); Intro H5; Elim H5; Intros; Auto.

Save.


Theorem Dijkstra_final : (dijkstra allV).
Realizer (principe_recurrence V->(connect_s+connect_s)
                         [x:V]
                          (inr connect_s connect_s
                                <connect_s>Case (dec_E source x) of
                                   [n](Min_pathc x n (Edgec source x n))
                                   (No_pathc x)
				end)
                      proof_dij_spec).
Program_all.
Red; Red; Auto.
Red; Intros.
Replace m with n; Auto.
Apply (Edge_un source x); Auto.
Red; Red; Auto.
Intros; Absurd (E source x n0); Auto.
Save.

Theorem dijkstra_res : (x:V)(connect_s allV allV x).
Realizer [x:V]<connect_s>Case (Dijkstra_final x) of
                         [H]H
                         [H]H
	      end.
Program_all.
Save.

Inductive NATW : Set := cout : nat -> NATW | inf : NATW.

Theorem Dijkstra_cout : (x:V)NATW.
Realizer [x:V]<NATW>Case (dijkstra_res x) of
                   [n,p](cout n) 
		   inf
	      end.
Program_all.
Save.

(* $Id: Dijkstra_prog.v,v 1.4 1996/02/12 15:29:35 cpaulin Exp $ *)
