(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             Mergesort_def.v                              *)
(****************************************************************************)

(*  realizes well_founded_recursion by :                                     *)
(*   [x,f](let rec F = [y] (f y  F) in (F x))                                *)

Require List.
Require Wf_nat.
Require Gt.

Variable   inf : A -> A -> Prop.

Hypothesis inf_total : (x,y:A){(inf x y)}+{(inf y x)}.
Hypothesis inf_tran : (x,y,z:A)(inf x y)->(inf y z)->(inf x z).
Hypothesis inf_refl : (x:A)(inf x x).

(*******************************)
(* addition of natural numbers *)
(*******************************)

Fixpoint add [n,m:nat] : nat :=
 <nat>Case m of
	n
	[p:nat](S (add n p))
 end.

Lemma eq_ad_O : (n:nat)((add O n)=n).
Induction n; Simpl; Auto.
Save.

Hint eq_ad_O.

Lemma sym_add : (n:nat)(m:nat)((add n m)=(add m n)).
Induction n; Simpl; Auto.
Intros.
Elim (H m).
Elim m; Simpl; Auto.
Save.

Hint sym_add.

Lemma add_ass : (n,m,p:nat)(add n (add m p))=(add (add n m) p).
Induction p; Intros; Simpl; Auto.
Save.

Hint add_ass.

Lemma ass_add : (n,m,p:nat)(add (add n m) p)=(add n (add m p)).
Auto.
Save.

Hint ass_add.

Lemma add_simpl_d : (n,m,p:nat)(n=m)->(add n p)=(add m p).
Induction p;Intros;Simpl;Auto.
Save.

Hint add_simpl_d.


Lemma add_simpl_g : (n,m,p:nat)(n=m)->(add p n)=(add p m).
Intros.
Elim (sym_add n p).
Elim (sym_add m p).
Auto.
Save.

Hint add_simpl_g.

Lemma add_simpl : (n,m,p,q:nat)(n=m)->(p=q)->(add n p)=(add m q).
Intros.
Elim H.
Elim H0.
Auto.
Save.

Hint add_simpl.

Definition eq_fun := [f1:A->nat][f2:A->nat]((a:A)((f1 a)=(f2 a))).

Definition fun_add := [f1:A->nat][f2:A->nat][a:A](add (f1 a)(f2 a)).

(***************************)
(* Characteristic function *)
(***************************)

Variable carac : A->A->nat.

(**********************)
(* Contents of a list *)
(**********************)

Fixpoint list_content [l:list] : A->nat :=
[a:A]<nat> Case l of
     (* nil *) O
(* cons a l *) [b:A][l:list](fun_add (carac b) (list_content l) a)
end.

(*****************)
(* low for lists *)
(*****************)

Inductive list_Lowert [a:A] : list -> Prop :=
      nil_low : (list_Lowert a nil)
    | cons_low : (b:A)(l:list)(inf a b)->(list_Lowert a (cons b l)).

Hint nil_low cons_low.


Definition list_Lowert2 :=
   [a:A][l:list](<Prop>Case l of (* nil *) True 
                                (* cons b l *) [b:A][l:list](inf a b) end).

Lemma low2_low : (a:A)(l:list)(list_Lowert2 a l)->(list_Lowert a l).
Induction l; Simpl; Auto.
Save.

Lemma low_low2 : (a:A)(l:list)(list_Lowert a l)->(list_Lowert2 a l).
Induction 1; Intros; Simpl; Auto.
Save.

Lemma lowert_cons_inf : (a,b:A)(l:list)(list_Lowert a (cons b l))->(inf a b).
Intros a b l; Exact (low_low2 a (cons b l)).
Save.

Lemma Lowert_cons_cons : (a,b:A)(l,m:list)(list_Lowert a (cons b l))->(list_Lowert a (cons b m)).
Intros a b l m H.
Apply cons_low.
Apply lowert_cons_inf with l;Auto.
Save.


(**************************************)
(* definition for a list to be sorted *)
(**************************************)

Inductive sort : list -> Prop :=
      nil_sort : (sort nil)
    | cons_sort : (a:A)(l:list)(sort l)->(list_Lowert a l)->(sort (cons a l)).
Hint nil_sort cons_sort.

Definition sort_inv
[l:list](<Prop>Case l of
     (* nil *) True 
(* cons a l *) [a:A][l:list](sort l)/\(list_Lowert a l) end).

Lemma sort_sort_inv : (l:list)(sort l)->(sort_inv l).
Induction 1;Simpl;Auto.
Save.

Lemma sort_rec : (P:list->Set)
        (P nil)->
         ((a:A)
           (l:list)(sort l)->(P l)->(list_Lowert a l)->(P (cons a l)))
         ->(y:list)(sort y)->(P y).
Induction y; Trivial.
Intros a l Pl Pc; Elim (sort_sort_inv (cons a l)); Auto.
Save.

(****************************)
(* merging two sorted lists *)
(****************************)

Inductive merge_lem [l1:list;l2:list] :Set :=
merge_exist : (l:list)(sort l)->
                        (eq_fun(list_content l)
                               (fun_add(list_content l1)
                                       (list_content l2)))->
                            ((a:A)(list_Lowert a l1)->(list_Lowert a l2)->
                                    (list_Lowert a l))->
                                            (merge_lem l1 l2).


Lemma merge : (l1:list)(sort l1)->(l2:list)(sort l2)->(merge_lem l1 l2).
Induction 1;Intros.
Apply merge_exist with l2;Auto.
Unfold eq_fun fun_add;Auto.
Elim H3;Intros.
Apply merge_exist with (cons a l);Auto.
Unfold eq_fun fun_add;Auto.
Elim (inf_total a a0);Intros.
Elim (H1 (cons a0 l0)); Intros; Auto.
Apply merge_exist with (cons a l3);Auto.
Simpl; Unfold eq_fun fun_add;Intros.
Elim add_ass.
Apply add_simpl_g.
Exact (e a1).
Intros.
Apply Lowert_cons_cons with l; Auto.
Elim H5;Intros.
Apply merge_exist with (cons a0 l3).
Auto.
Simpl;Unfold eq_fun fun_add;Intros.
Elim (sym_add (list_content l3 a1) (carac a0 a1)).
Elim (sym_add (list_content l0 a1) (carac a0 a1)).
Elim ass_add.
Apply add_simpl_d.
Exact (e a1).
Intros;Apply Lowert_cons_cons with l0; Auto.
Save.


(*****************)
(* list of lists *)
(*****************)
Inductive list_list : Set :=
   list_nil : list -> list_list
 | cons_list : list->list_list->list_list.

Hint list_nil cons_list.

Fixpoint Sort [L:list_list] : Prop :=
 <Prop>Case L of
   (* list_nil l *) [l:list](sort l)
(* cons_list l L *) [l:list][L':list_list]((Sort L') /\ (sort l))
 end.

Lemma Sort_Cons : (L:list_list)(l:list)(Sort L)->(sort l)->(Sort (cons_list l L)).
Intros;Simpl;Auto.
Save.
Hint Sort_Cons.

Lemma Sort_nil : (l:list)(Sort (list_nil l))->(sort l).
Induction 1;Auto.
Save.

Lemma Sort_tl : (l:list)(L:list_list)(Sort(cons_list l L))->(Sort L).
Induction 1;Auto.
Save.

Lemma Sort_hd : (l:list)(L:list_list)(Sort(cons_list l L))->(sort l).
Induction 1;Auto.
Save.

Hint (*Sort_tl Sort_hd*) Sort_nil.

Fixpoint list_list_content [ll:list_list] : A -> nat :=
<A->nat>Case ll of
   (* list_nil l *) [l:list](list_content l)
(* cons_list l L *) [l:list][L:list_list]
                         (fun_add (list_content l) (list_list_content L))
end.

Inductive single_lem [l:list] : Set :=
  ex_single : (L:list_list)
              (Sort L)->(eq_fun (list_content l)(list_list_content L))->
              (single_lem l).

(*Hint ex_single.*)

Lemma split : (l:list)(single_lem l).
Induction l.
Apply ex_single with (list_nil nil);Simpl;Auto.
Unfold eq_fun;Simpl;Auto.
Intro; Intro.
Elim l0;Intros.
Apply ex_single with (list_nil (cons a nil));Simpl;Auto.
Unfold eq_fun;Simpl;Auto.
Elim H0;Intros.
Apply ex_single with (cons_list (cons a nil) L);Auto.
Simpl.
Unfold eq_fun;Intros;Simpl.
Unfold fun_add;Simpl.Unfold fun_add;Simpl.
Unfold fun_add;Simpl.
Apply add_simpl_g.
Apply (e a1).
Save.

Fixpoint Length [ll:list_list] : nat :=
<nat>Case ll of
   (* list_nil l *) [l:list]O 
(* cons_list l L *) [l:list][L:list_list](S (Length L)) 
end.

Definition Lel [L1,L2:list_list](gt (Length L2)(Length L1)).

Inductive Single_Lem [L:list_list] : Set :=
   Ex_single : (L1:list_list)
               (Sort L1)->
               (Sort L)->
               (eq_fun (list_list_content L1) (list_list_content L))->
               (le (Length L1)(Length L))->
                   (Single_Lem L).

Lemma gt_SSn_n : (n:nat)(gt (S (S n)) n).
Auto.
Save.

Lemma constr_list : (L:list_list)(Sort L)->(Single_Lem L).
Intro; Pattern L.
Apply (well_founded_induction list_list Lel).
Exact (well_founded_gtof list_list  Length).
Induction x.
Intros l R S.
Apply Ex_single with (list_nil l);Auto.
Unfold eq_fun;Auto.
Induction l0;Intros.
Elim merge with l l1; Auto.
Intros l2 Sl2 eql2 ll2; Apply Ex_single with (list_nil l2);Auto.
Apply Sort_hd with (list_nil l1); Auto.
Apply Sort_nil; Apply Sort_tl with l; Auto.
Elim (H1 l2).
Intros L1 SL1 Sy0 eqL1 leL1.
Elim merge with l l1.
Intros l3 Sl3 eql3 ll; Apply Ex_single with (cons_list l3 L1);Simpl;Auto.
Unfold eq_fun fun_add; Intro.
Elim (eqL1 a).
Elim ass_add.
Apply add_simpl_d.
Apply (eql3 a).
Apply Sort_hd with (cons_list l1 l2); Auto.
Apply Sort_hd with l2.
Apply Sort_tl with l; Auto.
Unfold Lel; Simpl; Auto.
Apply Sort_tl with l1.
Apply Sort_tl with l; Auto.
Save.

Lemma mergesort_L :(L:list_list)(Sort L)->
            {l:list|(sort l)&(eq_fun (list_content l) (list_list_content L))}.
Intro; Pattern L.
Apply (well_founded_induction list_list Lel).
Exact (well_founded_gtof list_list  Length).
Induction x.
Intros l H H0.
Exists l;Auto.
Unfold eq_fun;Simpl;Auto.
Induction l0;Intros.
Elim merge with l l1.
Intros.
Exists l2;Auto.
Apply Sort_hd with (list_nil l1); Auto.
Apply Sort_nil.
Apply Sort_tl with l; Auto.
Elim merge with l l1; Intros.
Elim (constr_list l2); Intros.
Elim (H1 (cons_list l3 L1)); Intros.
Exists x0;Auto.
Simpl; Unfold eq_fun fun_add; Intro.
Elim (e0 a).
Elim ass_add.
Replace (add (list_content l a) (list_content l1 a)) with (list_content l3 a).
Exact (y0 a).
Exact (e a).
Unfold Lel; Simpl.
Apply le_S_gt; Auto.
Auto.
Apply Sort_tl with l1.
Apply Sort_tl with l; Auto.
Apply Sort_hd with (cons_list l1 l2); Auto.
Apply Sort_hd with l2.
Apply Sort_tl with l; Auto.
Save.

(* $Id: Mergesort_def.v,v 1.2 1996/02/12 15:30:03 cpaulin Exp $ *)
