(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                             Mergesort_prog.v                             *)
(****************************************************************************)

(*  realizes well_founded_recursion by :                                     *)
(*   [x,f](let rec F = [y] (f y  F) in (F x))                                *)

Require List.
Require Wf_nat.
Require Gt.

Variable   inf : A -> A -> Prop.

Hypothesis inf_total : (x,y:A){(inf x y)}+{(inf y x)}.
Hypothesis inf_tran : (x,y,z:A)(inf x y)->(inf y z)->(inf x z).
Hypothesis inf_refl : (x:A)(inf x x).

(*******************************)
(* addition of natural numbers *)
(*******************************)

Fixpoint add [n,m:nat] : nat :=
 <nat>Case m of
	n
	[p:nat](S (add n p))
 end.

Lemma eq_ad_O : (n:nat)((add O n)=n).
Induction n; Simpl; Auto.
Save.

Hint eq_ad_O.

Lemma sym_add : (n:nat)(m:nat)((add n m)=(add m n)).
Induction n; Simpl; Auto.
Intros.
Elim (H m).
Elim m; Simpl; Auto.
Save.

Hint sym_add.

Lemma add_ass : (n,m,p:nat)(add n (add m p))=(add (add n m) p).
Induction p; Intros; Simpl; Auto.
Save.

Hint add_ass.

Lemma ass_add : (n,m,p:nat)(add (add n m) p)=(add n (add m p)).
Auto.
Save.

Hint ass_add.

Lemma add_simpl_d : (n,m,p:nat)(n=m)->(add n p)=(add m p).
Induction p;Intros;Simpl;Auto.
Save.

Hint add_simpl_d.


Lemma add_simpl_g : (n,m,p:nat)(n=m)->(add p n)=(add p m).
Intros.
Elim (sym_add n p).
Elim (sym_add m p).
Auto.
Save.

Hint add_simpl_g.

Lemma add_simpl : (n,m,p,q:nat)(n=m)->(p=q)->(add n p)=(add m q).
Intros.
Elim H.
Elim H0.
Auto.
Save.

Hint add_simpl.

Definition eq_fun := [f1:A->nat][f2:A->nat]((a:A)((f1 a)=(f2 a))).

Definition fun_add := [f1:A->nat][f2:A->nat][a:A](add (f1 a)(f2 a)).

(***************************)
(* Characteristic function *)
(***************************)

Variable carac : A->A->nat.

(**********************)
(* Contents of a list *)
(**********************)

Fixpoint list_content [l:list] : A->nat :=
[a:A]<nat> Case l of
     (* nil *) O
(* cons a l *) [b:A][l:list](fun_add (carac a) (list_content l) a)
end.

(*****************)
(* low for lists *)
(*****************)

Inductive list_Lowert [a:A] : list -> Prop :=
      nil_low : (list_Lowert a nil)
    | cons_low : (b:A)(l:list)(inf a b)->(list_Lowert a (cons b l)).

Hint nil_low cons_low.


Definition list_Lowert2 :=
   [a:A][l:list](<Prop>Case l of (* nil *) True 
                                (* cons b l *) [b:A][l:list](inf a b) end).

Lemma low2_low : (a:A)(l:list)(list_Lowert2 a l)->(list_Lowert a l).
Induction l; Simpl; Auto.
Save.

Lemma low_low2 : (a:A)(l:list)(list_Lowert a l)->(list_Lowert2 a l).
Induction 1; Intros; Simpl; Auto.
Save.

Lemma lowert_cons_inf : (a,b:A)(l:list)(list_Lowert a (cons b l))->(inf a b).
Intros a b l; Exact (low_low2 a (cons b l)).
Save.

Lemma Lowert_cons_cons : (a,b:A)(l,m:list)(list_Lowert a (cons b l))->(list_Lowert a (cons b m)).
Intros a b l m H.
Apply cons_low.
Apply lowert_cons_inf with l;Auto.
Save.


(**************************************)
(* definition for a list to be sorted *)
(**************************************)

Inductive sort : list -> Prop :=
      nil_sort : (sort nil)
    | cons_sort : (a:A)(l:list)(sort l)->(list_Lowert a l)->(sort (cons a l)).
Hint nil_sort cons_sort.

Definition sort_inv :=
[l:list](<Prop>Case l of
     (* nil *) True 
(* cons a l *) [a:A][l:list](sort l)/\(list_Lowert a l) end).

Lemma sort_sort_inv : (l:list)(sort l)->(sort_inv l).
Induction 1;Simpl;Auto.
Save.

Lemma sort_rec : (P:list->Set)
        (P nil)->
         ((a:A)
           (l:list)(sort l)->(P l)->(list_Lowert a l)->(P (cons a l)))
         ->(y:list)(sort y)->(P y).
Realizer [P:Set][H:P][H0:A->list->P->P]
	      [y:list]<P>Match y with
		    	H
                    	[a,l,Pl](H0 a l Pl)
		       end.
Program_all.
Elim (sort_sort_inv (cons a l0)); Auto.
Elim (sort_sort_inv (cons a l0)); Auto.
Elim (sort_sort_inv (cons a l0)); Auto.
Save.

(****************************)
(* merging two sorted lists *)
(****************************)
Inductive merge_lem [l1:list;l2:list] :Set :=
merge_exist : (l:list)(sort l)->
                        (eq_fun(list_content l)
                               (fun_add(list_content l1)
                                       (list_content l2)))->
                            ((a:A)(list_Lowert a l1)->(list_Lowert a l2)->
                                    (list_Lowert a l))->
                                            (merge_lem l1 l2).

Section m.

Local merge_existc := [l1,l2:list][l:list](merge_exist l1 l2 l).

Lemma merge : (l1:list)(sort l1)->(l2:list)(sort l2)->(merge_lem l1 l2).
Realizer [l:list][{H:(sort l)}]
           (sort_rec list->list 
	      [l1:list]l1
              [a:A][l1:list][H1:list->list][l2:list][{H':(sort l2)}]
                  (sort_rec list
                     (cons a l1)
                     [a0:A][l3,l4:list]
                        <list>if (inf_total a a0) then
	               		      (cons a (H1 (cons a0 l3)))
                           else (cons a0 l4)
                     l2)
	    l).
Program_all.
Unfold eq_fun fun_add;Auto.
Unfold eq_fun fun_add;Auto.
Simpl;Unfold eq_fun fun_add;Intros.
Elim add_ass.
Apply add_simpl_g.
Exact (e0 a1).
Intros; Apply Lowert_cons_cons with l1; Auto.
Simpl;Unfold eq_fun fun_add;Intros.
Elim (sym_add (list_content l3 a1) (carac a1 a1)).
Elim (sym_add (list_content l0 a1) (carac a1 a1)).
Elim ass_add.
Apply add_simpl_d.
Exact (e a1).
Intros;Apply Lowert_cons_cons with l3; Auto.
Save.

End m.

(*****************)
(* list of lists *)
(*****************)

Inductive list_list : Set :=
   list_nil : list -> list_list
 | cons_list : list->list_list->list_list.

Hint list_nil cons_list.

Fixpoint Sort [L:list_list] : Prop :=
 <Prop>Case L of
   (* list_nil l *) [l:list](sort l)
(* cons_list l L *) [l:list][L':list_list]((Sort L') /\ (sort l))
 end.

Lemma Sort_Cons : (L:list_list)(l:list)(Sort L)->(sort l)->(Sort (cons_list l L)).
Intros;Simpl;Auto.
Save.
Hint Sort_Cons.

Lemma Sort_nil : (l:list)(Sort (list_nil l))->(sort l).
Induction 1;Auto.
Save.

Lemma Sort_tl : (l:list)(L:list_list)(Sort(cons_list l L))->(Sort L).
Induction 1;Auto.
Save.

Lemma Sort_hd : (l:list)(L:list_list)(Sort(cons_list l L))->(sort l).
Induction 1;Auto.
Save.

Hint (*Sort_tl Sort_hd*) Sort_nil.

Fixpoint list_list_content [ll:list_list] : A -> nat :=
<A->nat>Case ll of
   (* list_nil l *) [l:list](list_content l)
(* cons_list l L *) [l:list][L:list_list]
                         (fun_add (list_content l) (list_list_content L))
end.

Inductive single_lem [l:list] : Set :=
  ex_single : (L:list_list)
              (Sort L)->(eq_fun (list_content l)(list_list_content L))->
              (single_lem l).

(*Hint ex_single.*)

Section splitsecn.

Local ex_singlec := [l:list][L:list_list](ex_single l L).

Lemma split : (l:list)(single_lem l).
Realizer [l:list]
            <list_list>Case l of
	      (list_nil nil)
              [a,y]<list_list>Match y with
                     (list_nil (cons a nil))
                     [a0,y0,H0]
       			(cons_list (cons a0 nil) H0)
		end
	     end.
Program_all.
Simpl;Auto.
Unfold eq_fun;Simpl;Auto.
Simpl;Auto.
Unfold eq_fun;Simpl;Auto.
Simpl; Unfold eq_fun;Intros;Simpl.
Unfold fun_add;Simpl.
Apply add_simpl_g.
Apply (e a1).
Save.

End splitsecn.

Fixpoint Length [ll:list_list] : nat :=
<nat>Case ll of
   (* list_nil l *) [l:list]O 
(* cons_list l L *) [l:list][L:list_list](S (Length L)) 
end.

Definition Lel := [L1,L2:list_list](gt (Length L2)(Length L1)).

Inductive Single_Lem [L:list_list] : Set :=
   Ex_single : (L1:list_list)
               (Sort L1)->
               (Sort L)->
               (eq_fun (list_list_content L1) (list_list_content L))->
               (le (Length L1)(Length L))->
                   (Single_Lem L).


Lemma gt_SSn_n : (n:nat)(gt (S (S n)) n).
Auto.
Save.

Section clistsecn.

Local Ex_singlec := [L:list_list][L1:list_list](Ex_single L L1).

Lemma constr_list : (L:list_list)(Sort L)->(Single_Lem L).
Realizer <list_list>rec constr_l :: :: { Lel }
         	[L:list_list]
                <list_list>Case L of
                     [l](list_nil l)
                     [l,y]<list_list>Case y of
                             [l0](list_nil (merge l l0))
                             [l0,y0](cons_list (merge l l0) (constr_l y0))
			  end
		end.
Program_all.
Exact (well_founded_gtof list_list Length).
Unfold eq_fun;Auto.
Apply Sort_hd with (list_nil l0); Auto.
Apply Sort_nil; Apply Sort_tl with l; Auto.
Unfold Lel; Simpl; Auto.
Apply Sort_tl with l0.
Apply Sort_tl with l; Auto.
Apply Sort_hd with (cons_list l0 y0); Auto.
Apply Sort_hd with y0.
Apply Sort_tl with l; Auto.
Simpl; Unfold eq_fun fun_add; Intro.
Elim (e a0).
Elim ass_add.
Apply add_simpl_d.
Apply (e0 a0).
Simpl; Auto.
Save.

End clistsecn.

Lemma mergesort_L :(L:list_list)(Sort L)->
            {l:list|(sort l)&(eq_fun (list_content l) (list_list_content L))}.
Realizer <list>rec merge_L :: :: { Lel }
               [L:list_list]
                   <list>Case L of
                        [l]l
                        [l,y]<list>Case y of
                               [l0](merge l l0)
                               [l0,y0](merge_L (cons_list (merge l l0) 
					                    (constr_list y0)))
			  end
		   end.
Program_all.
Exact (well_founded_gtof list_list  Length).
Unfold eq_fun;Simpl;Auto.
Apply Sort_hd with (list_nil l0); Auto.
Apply Sort_nil.
Apply Sort_tl with l; Auto.
Apply Sort_tl with l0.
Apply Sort_tl with l; Auto.
Apply Sort_hd with (cons_list l0 y0); Auto. 
Apply Sort_hd with y0.
Apply Sort_tl with l; Auto.
Unfold Lel; Simpl.
Apply le_S_gt; Auto.
Simpl; Unfold eq_fun fun_add; Intro.
Elim (e a0).
Elim ass_add.
Replace (add (list_content l a0) (list_content l0 a0)) with (list_content l2 a0).
Exact (e1 a0).
Exact (e0 a0).
Save.

Theorem mergesort : (l:list)
     {m:list|(sort m)&(eq_fun (list_content l) (list_content m))}.
Realizer [l:list](mergesort_L (split l)).
Program_all.
Unfold eq_fun;Intro.
Apply trans_equal with (list_list_content L a).
Exact (e a).
Apply sym_equal.
Exact (e0 a).
Save.


(* $Id: Mergesort_prog.v,v 1.4 1996/02/12 15:30:06 cpaulin Exp $ *)
