(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.11                                 *)
(*                              Feb 2nd 1996                                *)
(*                                                                          *)
(****************************************************************************)
(*                               Functions.v                                *)
(****************************************************************************)

Require Ensembles.    (* Ensemble, In, Included, Setminus *)
Require Relations_1.  (* Relation *)

(************************************************************************)
(* In contrast with the definition of functions in Relations_1, we      *)
(* consider, here, functions as binary relations                        *)

Section Functions.

Variable U:Type.

(************************************************************************)
(* Characterization of a relation over two sets                         *)
(* Caracterisation d'une relation d'un ensemble donne dans un autre     *)

Inductive Rel [U:Type;A,B:(Ensemble U);R:(Relation U)] : Prop :=
   Rel_intro : ((x,y:U)(R x y)->(In U A x))
             ->((x,y:U)(R x y)->(In U B y))->(Rel U A B R).
 

(************************************************************************)
(*              Image of a set through a relation                       *)

Section Image.

Inductive Im [R:(Relation U);A:(Ensemble U);y:U] : Prop  :=
     Im_intro : (x:U)(R x y)->(In U A x)->(Im R A y).

Lemma Im_stable_par_incl :
  (R:(Relation U))(A1,A2:(Ensemble U))
        (Included U A1 A2)->(Included U (Im R A1) (Im R A2)).
Do 3 Intro; Intros A1_Included_A2; Red; Red; Intros x x_in_Im_A1.
Elim x_in_Im_A1.
Intros.
Apply Im_intro with x0; Try Assumption.
Apply A1_Included_A2; Assumption.
Qed.

End Image.


Variables A,B:(Ensemble U).
Variable f:(Relation U).


Definition au_plus_une_im := (x,y,z:U) (f x y) -> (f x z) -> y==z.

Definition au_moins_une_im := (x:U) (In U A x) -> (ExT (f x)).

Definition au_plus_un_ant := (x,y,z:U) (f x z) -> (f y z) -> x==y.

Definition au_moins_un_ant := (y:U) (In U B y) -> (ExT [x:U](f x y)).


Inductive fonction : Prop :=          (* fun_in *)
   fonction_intro : (Rel U A B f)
                  ->au_plus_une_im
                  ->au_moins_une_im   ->fonction.

Inductive surjection : Prop :=        (* fun_on *)
   surjection_intro :   (Rel U A B f)
                  ->au_plus_une_im
                  ->au_moins_une_im
                  ->au_moins_un_ant   ->surjection.

Inductive injection : Prop :=         (* map_in *)
   injection_intro :   (Rel U A B f)
                  ->au_plus_une_im
                  ->au_moins_une_im
                  ->au_plus_un_ant  ->injection.

Inductive bijection : Prop :=         (* map_on *)
   bijection_intro : (Rel U A B f)
                  ->au_plus_une_im
                  ->au_moins_une_im
                  ->au_plus_un_ant
                  ->au_moins_un_ant  ->bijection.


End Functions.


(* $Id: Functions.v,v 1.2 1996/02/12 15:32:44 cpaulin Exp $ *)
