(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                Streams.v                                 *)
(****************************************************************************)

Section Streams. (* The set of streams : definition *)

Variable A : Set.

CoInductive Set Stream   := Cons : A->Stream->Stream.

Definition head  := 
  [x:Stream]<A>Case x of [a:A][s:Stream]a end.

Definition tail  := 
  [x:Stream]<Stream>Case x of [a:A][s:Stream]s end.

Fixpoint nth [n:nat] : Stream->A := 
  [s:Stream]<A>Case n of
	         (head s) 
		 [m:nat](nth m (tail s))
	    end.

Lemma unfold_Stream :(x:Stream)(x=(<Stream>Case x of Cons end)). 
Intro x.
Case x.
Trivial.
Qed.


(* Extensional Equality between two streams  *)

CoInductive EqSt  : Stream->Stream->Prop := 
	    eqst : (s1,s2:Stream)
		   ((head s1)=(head s2))->
		   (EqSt (tail s1) (tail s2))
                    ->(EqSt s1 s2).

Theorem  EqSt_reflex : (s:Stream)(EqSt s s).
Cofix u.
Intros.
Apply eqst.
Trivial.
Apply u.
Qed.

Theorem EqSt_sym : 
 (s1:Stream)(s2:Stream)(EqSt s1 s2)->(EqSt s2 s1).
Cofix u.
Intros.
Case H.
Intros.
Apply eqst.
Symmetry.
Assumption.
Apply u.
Assumption.
Qed.


Theorem EqSt_trans : 
 (s1,s2,s3:Stream)(EqSt s1 s2)->(EqSt s2 s3)->(EqSt s1 s3).
Cofix u.
Intros.
Apply eqst.
Transitivity (head s2).
Case H.
Intros.
Assumption. 
Case H0.
Intros.
Assumption.
Apply (u (tail s1) (tail s2) (tail s3)).
Case H.
Intros.
Assumption.
Case H0.
Intros.
Assumption.
Qed.

(* The definition given is equivalent to require the elements at each position to be equal *)

Theorem Equiv1 :
  (s1,s2:Stream)(EqSt s1 s2)->(n:nat)(nth n s1)=(nth n s2).
Fix u / 4.
Case H.
Intros.
Case n.
Assumption.
Intros.
Simpl.
Apply u.
Assumption.
Qed.

Theorem Equiv2 : 
  (s1,s2:Stream)((n:nat)(nth n s1)=(nth n s2))->(EqSt s1 s2).
Cofix u.
Intros.
Apply eqst.
Apply (H O).
Apply u.
Intros n.
Apply (H (S n)).
Qed.

End Streams.

Syntactic Definition cons := (Cons ?).
Syntactic Definition hd   := (head ?).
Syntactic Definition tl   := (tail ?).


(* $Id: Streams.v,v 1.7 1996/02/12 15:33:31 cpaulin Exp $ *)
