(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                Coq V5.10                                 *)
(*                              Nov 25th 1994                               *)
(*                                                                          *)
(****************************************************************************)
(*                                coqdep.ml                                 *)
(****************************************************************************)

(*							*)
(*	coqdep.ml     	(JCF 4/5/94)			*)
(*							*)

#open "init";;
#open "unix";;

let option_c = ref false;;

let option_D = ref false;;

let option_w = ref false;;

let traite_fichier_ML f =
  try 
    let chan = open_in f in 
    let buf = lexing__create_lexer_channel chan in 
    let deja_vu = ref ([] : string list) in 
       try while true do
	 let (Use_module str) = lex_cld__action buf in
	   if mem str (!deja_vu) then ()
	   else (
	      deja_vu := str :: !deja_vu;
              try (
                let mlidir = assoc str (!mliKnown) in
                print_string " ";
      	        print_string (file_name (str,mlidir));
                print_string ".zi")
              with Not_found ->
              try (  
                let mldir = assoc str (!mlKnown) in
                print_string " "; print_string (file_name (str,mldir));
                print_string ".zo")
              with Not_found -> () )
         done
       with lex_cld__fin_fichier -> ();
    close_in chan
  with sys__Sys_error _ -> () 
;;

let warning_notfound f s =
  output_string std_err ("*** Warning : in file " ^ f ^ ", the file ");
  output_string std_err ( s ^ ".v is required and has not been found !\n");
  flush std_err
;;

let traite_fichier_Coq f = 
  try 
    let chan = open_in f in 
    let buf = lexing__create_lexer_channel chan in 
    let deja_vu_v = ref ([]: string list)
    and deja_vu_ml = ref ([] : string list) in
       try while true do
	 let tok = lex_coq__action buf in
	 (match tok with
	    Require str -> 
	      if mem str (!deja_vu_v) then ()
	      else (
	        deja_vu_v := str :: !deja_vu_v;
                try (
                  let vdir = assoc str (!vKnown) in
                  print_string " ";
                  print_string (file_name (str,vdir));
                  print_string ".vo")
                with Not_found -> warning_notfound f str
       	     )
	  | RequireString s -> 
	      let str = filename__basename s in
	      if mem str (!deja_vu_v) then ()
	      else (
	        deja_vu_v := str :: !deja_vu_v;
                try (
                  let vdir = assoc str (!vKnown) in
                  print_string " ";
                  print_string (file_name (str,vdir));
                  print_string ".vo")
                with Not_found -> warning_notfound f s
       	     )
	  | Declare sl -> do_list (fun str ->
	      if mem str (!deja_vu_ml) then ()
              else (
                deja_vu_ml := str :: !deja_vu_ml;
               	try (
              	  let mldir = assoc str (!mlKnown) in
              	  print_string " ";
                  print_string (file_name (str,mldir));
              	  print_string ".zo")
              	with Not_found -> () 
	      ) ) sl
      	 )
       done
       with lex_coq__fin_fichier -> ();
    close_in chan
  with sys__Sys_error _ -> () 
;;

let ML_dep_list b f = 
  try hashtbl__find dep_tab f
  with Not_found -> ( 
   let deja_vu = ref ([] : string list) in 
   try 
     let chan = open_in f in 
     let buf = lexing__create_lexer_channel chan in 
     try while true do
	 let (init__Use_module str) = lex_cld__action buf in
	 if (str=b) then (
	   output_string std_err ("*** Warning : in file "^f^" the");
	   output_string std_err (" notation "^b^"__ is useless !\n");
	   flush std_err
	 )
      	 else if (mem str (!deja_vu)) then ()
	 else deja_vu := str :: !deja_vu
       done; []
     with lex_cld__fin_fichier 
      	 -> (close_in chan; 
	     let rl = rev (!deja_vu) in (
      	     hashtbl__add dep_tab f rl;
	     rl))
   with sys__Sys_error _ -> [])
;;

let affiche_Declare f dcl = 
  print_newline ();
  print_string ("*** In file "^f^" : ");
  print_newline ();
  print_string "Declare ML Module";
  do_list (fun str -> print_string " ";
    print_char `"`; print_string str;
    print_char `"`) dcl;
  print_string ".";
  print_newline();
  flush std_out
;;

let warning_Declare f dcl =
  output_string std_err ("*** Warning : in file "^f^", the ML modules");
  output_string std_err " declaration should be\n";
  output_string std_err "*** Declare ML Module";
  do_list (fun str -> output_string std_err " ";
    output_char std_err `"`; output_string std_err str;
    output_char std_err `"`) dcl;
  output_string std_err ".\n";
  flush std_err
;;

let traite_Declare f = 
  let decl_list = ref ([] : string list)
  in
  let rec treat = function
     s::ll -> if (mem_assoc s (!mlKnown)) & (not (mem s (!decl_list))) then (
       	       	let mldir = assoc s (!mlKnown) in
		let fullname = file_name (s,mldir) in
		let depl = ML_dep_list s (fullname^".ml") in
		treat depl;
		decl_list := s :: (!decl_list) );
	      treat ll
   | [] -> ()
  in
  try
    let chan = open_in f in
    let buf = lexing__create_lexer_channel chan in
      try while true do
      	let tok = lex_coq__action buf in
	(match tok with
	   Declare sl -> decl_list := [];
	                 treat sl;
			 decl_list := rev (!decl_list);
			 if (!option_D) then 
			   affiche_Declare f (!decl_list)
			 else if (!decl_list)<>sl then
			   warning_Declare f (!decl_list)
	 | _ -> ()
        )
      done
      with lex_coq__fin_fichier -> ();
    close_in chan
  with sys__Sys_error _ -> ()
;;

let ML_dependencies () =
    do_list 
      (fun ((name,dirname) as pairname) ->
	 let fullname = file_name pairname in
	 print_string fullname; print_string ".zo: "; 
	 print_string fullname; print_string ".ml";
	 (if (mem pairname (!mliAccu)) then (
	    print_string " "; print_string fullname; print_string ".zi"));
	 traite_fichier_ML (fullname^".ml");
	 print_newline ();
         flush std_out)
      (!mlAccu);
    do_list
      (fun ((name,dirname) as pairname) ->
	 let fullname = file_name pairname in
	 print_string fullname; print_string ".zi: "; 
	 print_string fullname;print_string ".mli";
	 traite_fichier_ML (fullname^".mli");
	 print_newline ();
         flush std_out)
      (!mliAccu)
;;

let Coq_dependencies () =
    do_list
      (fun ((name,dirname) as pairname) ->
	 let fullname = file_name pairname in
	 print_string fullname; print_string ".vo: "; 
	 print_string fullname;print_string ".v";
	 traite_fichier_Coq (fullname^".v");
	 print_newline ();
         flush std_out)
      (!vAccu)
;;

let Declare_dependencies () =
  do_list
      (fun ((name,dirname) as pairname) ->
	 let fullname = file_name pairname in
	 traite_Declare (fullname^".v");      
         flush std_out)
   (!vAccu)
;;

let rec warning_mult suf l = 
  let tab = hashtbl__new 151 in
  do_list (fun (f,d) -> 
     try let d' = hashtbl__find tab f in (
	  if (filename__dirname (file_name (f,d)))
      	      <>(filename__dirname (file_name (f,d'))) then (
	     output_string std_err "*** Warning : the file ";
	     output_string std_err (f ^ suf);
	     output_string std_err " is defined twice !\n";
	     flush std_err
	  ))
     with Not_found -> ();
     hashtbl__add tab f d) l
;;

let coqdep () =
  let lg_command = vect_length sys__command_line in
  if lg_command < 2 then (
    print_string "[ usage: coqdep <filename>+ ]\n";
    flush std_out)
  else
    let rec treat old_dirname old_name =
      let name = filename__basename old_name and 
	  new_dirname = filename__dirname old_name in
      let dirname = 
	 match (old_dirname,new_dirname)
	 with (d, ".") -> d
	    | (NONE,d) -> SOME d
	    | (SOME d1,d2) -> SOME (filename__concat d1 d2) in
      let complete_name = file_name (name,dirname) in
      match try (stat (file_name (name,dirname))).st_kind with _ -> S_BLK
      with S_DIR -> (
	     if name <> "." & name <> ".." then
	     let dir=opendir complete_name in
             let newdirname = 
               match dirname with 
                  NONE -> name
                | SOME d -> filename__concat d name in
	     try 
	       while true do treat (SOME newdirname) (readdir dir) done
	       with End_of_file -> closedir dir)
	 | S_REG -> (
	    if filename__check_suffix name ".ml" then (
	      let basename = filename__chop_suffix name ".ml" in
	      addQueue mlAccu (basename,dirname))
	    else if filename__check_suffix name ".mli" then (
	      let basename = filename__chop_suffix name ".mli" in
	      addQueue mliAccu (basename,dirname))
	    else if filename__check_suffix name ".v" then (
	      let basename = filename__chop_suffix name ".v" in
	      addQueue vAccu (basename,dirname))
	    else ())
	 | _ -> ()
    in 
    let add_known dir_name f =
      let complete_name = filename__concat dir_name f in
      match try (stat complete_name).st_kind with _ -> S_BLK
      with S_REG -> (
	    if filename__check_suffix f ".ml" then (
	      let basename = filename__chop_suffix f ".ml" in
	      addQueue mlKnown (basename,SOME dir_name))
	    else if filename__check_suffix f ".mli" then (
	      let basename = filename__chop_suffix f ".mli" in
	      addQueue mliKnown (basename,SOME dir_name))
	    else if filename__check_suffix f ".v" then (
	      let basename = filename__chop_suffix f ".v" in
	      addQueue vKnown (basename,SOME dir_name))
	    else ())
         | _ -> ()
    in
    let add_directory dir_name = 
      match try (stat dir_name).st_kind with _ -> S_BLK
      with S_DIR -> (let dir = opendir dir_name in
      	       	     try
		       while true do add_known dir_name (readdir dir) done
		     with End_of_file -> closedir dir )
         | _ -> ()
    in
    let rec parse = function
        "-c" :: ll -> option_c := true; parse ll
      | "-D" :: ll -> option_D := true; parse ll
      | "-w" :: ll -> option_w := true; parse ll
      | "-I" :: (r :: ll) -> add_directory r; parse ll
      | f :: ll -> treat NONE f; parse ll
      | [] -> ()
    in
    parse (tl (list_of_vect sys__command_line));
    mliKnown := (!mliKnown) @ (!mliAccu);
    mlKnown  := (!mlKnown) @ (!mlAccu);
    vKnown   := (!vKnown) @ (!vAccu);

    warning_mult ".mli" (!mliKnown);
    warning_mult ".ml" (!mlKnown);
    warning_mult ".v" (!vKnown);

    if (!option_c) & (not (!option_D)) then 
      ML_dependencies ();

    if not (!option_D) then
      Coq_dependencies ();

    if (!option_w) or (!option_D) then
      Declare_dependencies ()
     
;;

printexc__f coqdep ();;

