(*
 *   creer un Makefile pour un repertoire des THEORIES automatiquement
 *
 *   syntaxe : entete fichier (.ml ou .mli)
 *)

let vl = ref ([] : string list);;

let print s = output_string std_out s;;

let rec affiche_liste = function
    x::[] -> print (x ^ "vo\n")
  | x::l -> print (x ^ "vo \\\n     "); affiche_liste l
  | [] -> ()
;;

let affiche points =
  print "\n";
  print ("include " ^ points ^ "/config\n");
  print "\n";
  print "COQC=$(COQTOP)/bin/$(ARCH)/coqc\n";
  print "COQDEP=$(COQTOP)/bin/$(ARCH)/coqdep\n";
  print ("COQINCLUDES=-I " ^ points ^ "/theories/ARITH -I " ^ points ^ "/theories/SETS \\\n");
  print ("            -I " ^ points ^ "/theories/BOOL -I " ^ points ^ "/theories/LISTS \\\n");
  print ("            -I " ^ points ^ "/theories/PROGRAMS -I .\n");
  print "\n";
  print "all: "; affiche_liste !vl;
  print "\n";
  print ".SUFFIXES: .v .vo\n";
  print "\n";
  print ".v.vo:\n";
  print "\t$(COQC) -q $(COQINCLUDES) $*\n";
  print "\n";
  print "clean::\n";
  print "\trm -f *.v? *~\n";
  print "\n";
  print "depend::\n";
  print "\trm -f Makefile.bak\n";
  print "\tcp Makefile Makefile.bak\n";
  print "\tsed -n -e '1,/^### DO NOT DELETE THIS LINE/p' Makefile.bak > Makefile\n";
  print "\t$(COQDEP) $(COQINCLUDES) *.v >> Makefile\n";
  print "\n";
  print "### DO NOT DELETE THIS LINE\n"
;;

let main () =
  if vect_length sys__command_line < 2 then 
    (print_string "[ usage: doMakefile file1 file 2 ... ]\n";
     flush std_out)
  else
    (let args = tl (list_of_vect sys__command_line) in
     let points = hd args in
     begin
       (do_list (fun x -> if filename__check_suffix x "v" then
                       vl := (filename__chop_suffix x "v")::!vl)
                (tl args));
       affiche points;
       flush std_out
     end)
;;

printexc__f main ();;




