(*							*)
(*	lex_coq.mll	(JCF  20/5/94)			*)
(*							*)
{
 #open "init";;

 let comment_depth = ref 0;;
 let CRcpt = ref 0;;

 let print s = output_string (!chan_out) s;;
 
 exception fin_fichier;;

}
rule action = parse
    "Theorem" { print "Theorem"; body lexbuf; 
      	       	CRcpt := 1; action lexbuf }
  | "Lemma"   { print "Lemma";   body lexbuf; 
      	       	CRcpt := 1; action lexbuf }
  | "Fact"    { print "Fact";   body lexbuf; 
      	       	CRcpt := 1; action lexbuf }
  | "Remark"  { print "Remark";  body lexbuf; 
      	       	CRcpt := 1; action lexbuf }
  | "Goal"    { print "Goal";    body lexbuf; 
      	       	CRcpt := 1; action lexbuf }
  | "Hint"    	   	{ skip_until_point lexbuf ; action lexbuf }
  | "Transparent"	{ skip_until_point lexbuf ; action lexbuf }
  | "Immediate"		{ skip_until_point lexbuf ; action lexbuf }
  | "Syntax"		{ skip_until_point lexbuf ; action lexbuf }
  | "(*"      { print "(*"; comment_depth := 1;
      	        Comment lexbuf; CRcpt := 0; action lexbuf }
  | [` ` `\t`]* `\n`      { if !CRcpt < 2 then print "\n";
      	       	       	    CRcpt := !CRcpt+1; action lexbuf}
  | eof       { raise fin_fichier}
  | _         { print (get_lexeme lexbuf); CRcpt := 0; action lexbuf }

and Comment = parse
    "(*"  { print "(*"; 
      	    comment_depth := succ !comment_depth; Comment lexbuf }
  | "*)"  { print "*)";
      	    comment_depth := pred !comment_depth;
            if !comment_depth > 0 then Comment lexbuf }
  | eof   { raise fin_fichier } 
  | _     { print (get_lexeme lexbuf); Comment lexbuf }

and skip_Comment = parse
    "(*"  { comment_depth := succ !comment_depth; skip_Comment lexbuf }
  | "*)"  { comment_depth := pred !comment_depth;
            if !comment_depth > 0 then skip_Comment lexbuf }
  | eof   { raise fin_fichier } 
  | _     { skip_Comment lexbuf }

and body = parse
    `.`   { print ".\n"; skip_proof lexbuf }
  | ":="  { print ".\n"; skip_proof lexbuf }
  | "(*"  { print "(*"; comment_depth := 1;
      	    Comment lexbuf; body lexbuf }
  | eof   { raise fin_fichier } 
  | _     { print (get_lexeme lexbuf); body lexbuf }

and skip_until_point = parse
    `.`   { end_of_line lexbuf }
  | "(*"  { comment_depth := 1;
      	    skip_Comment lexbuf; skip_until_point lexbuf }
  | eof   { raise fin_fichier } 
  | _     { skip_until_point lexbuf }

and end_of_line = parse
    [` ` `\t` ]*	{ end_of_line lexbuf }
  | `\n`		{ () }
  | eof   	     	{ raise fin_fichier } 
  | _	  		{ print (get_lexeme lexbuf) }

and skip_proof = parse
    "Save"	{ skip_until_point lexbuf }
  | "Qed"  	{ skip_until_point lexbuf }
  | "Abort" 	{ skip_until_point lexbuf }
  | "Proof"     { skip_until_point lexbuf }
  | "Proof" [` ` `\t` ]* `.`  { skip_proof lexbuf }
  | "(*"    { comment_depth := 1;
      	      skip_Comment lexbuf; skip_proof lexbuf }
  | eof     { raise fin_fichier } 
  | _       { skip_proof lexbuf }
  
;;
