(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: Arith_compl.v                                          *)
(* contents: additionnal lemmas on plus,mult,minus,le...        *)
(* definitions of 2^ and factorial, lemmas about them           *)
(****************************************************************)


Require Export Plus.
Require Export Mult.
Require Minus.
Require Export Lt.
Require Export Le.
Require Export Gt.

(****************************************************************)
(* Complements sur plus et mult *)

Lemma plus_n_SO : (x:nat) (plus x (S O))=(S x).
Intros;Rewrite plus_sym;Auto.
Qed.Hint plus_n_SO.

Lemma plus_permute2 : (x,y,z:nat) (plus (plus x y) z)=(plus (plus x z) y).
Intros.
Rewrite (plus_sym x y).
Rewrite (plus_sym x z).
Rewrite -> plus_sym.
Symmetry.
Rewrite -> plus_sym.
Rewrite -> plus_permute.
Auto.
Qed.Hint plus_permute2.

Lemma mult_sym : (a,b:nat) ((mult a b)=(mult b a)).
Intros a b;Elim a;Simpl;Auto.
Intros y H.
Replace (mult y b) with (mult b y).
Elim (mult_n_Sm b y).
Apply plus_sym.
Qed.Hint mult_sym.

Lemma mult_permute : (a,b,c:nat) ((mult (mult a b) c)=(mult (mult a c) b)).
Intros.
Rewrite -> mult_assoc_r.
Rewrite -> mult_assoc_r.
Replace (mult c b) with (mult b c); Auto.
Qed.Hint mult_permute.

(*
Lemma plus_n_n : (n:nat)((plus n n)=(mult n (S (S O)))).
Intro.
Rewrite -> mult_sym.
Simpl.
Replace (plus n O) with (plus O n);Auto.
Qed.Hint plus_n_n.
*)

Lemma plus_O_O : (n,m:nat)
	(plus n m)=O -> n=O.
Induction n.Intros.Try Trivial.
Intros.Inversion H0.
Qed.

Lemma mult_plus_distr2 : (n,m,p:nat)
	(mult n (plus m p)) = (plus (mult n m) (mult n p)).
Intros.Rewrite -> mult_sym.Rewrite -> mult_plus_distr.Rewrite -> mult_sym.
Replace (mult p n) with (mult n p).Try Trivial.Apply mult_sym.
Qed.

(****************************************************************)
(* La fonction puissance de 2 *)

Recursive Definition power2 : nat -> nat :=
	O => (S O)
	|(S x) => (plus (power2 x) (power2 x)).

(*
Lemma power2_SO : ((S (S O))=(power2 (S O))).
Auto.Qed.Hint power2_SO.

Lemma mult_power2 : (x,y:nat)
		((mult (power2 x) (power2 y))=(power2 (plus x y))).
*)
Lemma power2_plus : (x,y:nat)
	(power2 (plus x y)) = (mult (power2 x) (power2 y)).
Induction x.Intros.Simpl.(Elim plus_n_O;Auto).
Intros.Simpl.Rewrite -> H.Rewrite -> mult_plus_distr.Try Trivial.
Qed.
(****************************************************************)
(* Pour demontrer que ~x=O -> ~(plus x x)=x *)
(*
Lemma lem_logic1 : (P,Q:Prop)  (Q->P)-> (~P->~Q).
Tauto.
Qed.

Lemma lem_plus_x_O : (x,y:nat) (plus y x)=y -> x=O.
Induction y.Auto.Simpl.Intros.Apply H.Inj H0.Try Trivial.
Qed.

Lemma lem_plus_x_O_bis : (x:nat) (plus x x)=x -> x=O.
Intro.Intros.Apply lem_plus_x_O with y:=x .Exact H.
Qed.

Lemma lem_plus_x_O_inv : (x:nat) ~(x=O) -> ~(plus x x)=x.
Intros.Apply lem_logic1 with P:=(eq ? x O) .
Intros.(Apply lem_plus_x_O_bis;Auto).Exact H.
Qed.

Lemma lem_plus_x_O_inv_sym : (x:nat) ~(x=O) -> ~x=(plus x x).
Red.Intros.Absurd (eq ? (plus x x) x).Apply lem_plus_x_O_inv.Exact H.Auto.
Qed.
*)
(****************************************************************)
(* Des lemmes sur is_O *)
(*
Definition is_O : nat->Prop := [x:nat](x=O).

Lemma Sn_is_not_O_ind : (n:nat)  ~(is_O n)->~(is_O (S n)).
Unfold not .Unfold is_O .Induction n.
Auto.Intros.Absurd (eq ? (S (S n0)) O).Discr.Exact H1.
Qed.

Lemma Sn_is_not_O : (n:nat)  ~(is_O (S n)).
Unfold is_O .Intro.Discr.
Qed.

Lemma plus_is_not_O : (n,m:nat) ~(is_O n) -> ~(is_O (plus n m)).
Induction m.Rewrite <- plus_n_O.
Auto.Intros.Simpl.Rewrite <- plus_n_Sm.Apply Sn_is_not_O.
Qed.

Lemma not_is_O_ex_k : (n:nat) ~(is_O n)->(Ex [k:nat]n=(S k)).
Unfold not is_O .Induction n.Intros.
Absurd (eq ? O O)->False.Red.Auto.Exact H.Intros.Exists n0.Try Trivial.
Qed.
*)

(****************************************************************)
(* Des lemmes sur factorielle *)
(*
Lemma FactR_never_O : (n:nat) ~(is_O (FactR n)).
Induction n.Simpl.Apply Sn_is_not_O.Intros.Simpl.Apply plus_is_not_O.Exact H.
Qed.

Lemma FactR_lem1 : (n:nat)
     (Ex [k:nat]n=(S k) -> (FactR (S k))=(mult (S k) (FactR (pred (S k))))).
Simpl.Intro.Exists (pred n).Intros.Try Trivial.
Qed.

Lemma FactR_lem0 : (n:nat) 
	~(is_O n) -> 
     (Ex [k:nat]n=(S k) -> (FactR (S k))=(mult (S k) (FactR (pred (S k))))).
Intros.Apply FactR_lem1.
Qed.
*)

(****************************************************************)
(* Complements sur Lt Le Gt... *)

Theorem le_plus_n_m : (n,m:nat)
	(le n m) -> (le (plus n n) (plus m m)).
Induction n.Auto.
Intros.Inversion H0.Auto.
Simpl.Elim plus_n_Sm.Elim plus_n_Sm.
Apply le_n_S.Apply le_n_S.Apply H.Apply le_trans_S.Exact H1.
Qed. Hint le_plus_n_m.

Theorem lt_plus_n_m : (n,m:nat)
	(lt n m) -> (lt (S (plus n n)) (plus m m)).
Induction n.
Simpl.Induction m.Simpl.Intros.Absurd (lt O O).Apply le_not_lt.Auto.
Exact H.
Intros.Simpl.Elim plus_n_Sm.Apply lt_n_S.Auto.
Intros.Inversion H0.Simpl.Repeat Elim plus_n_Sm.Auto.
Simpl.
Repeat Elim plus_n_Sm.Apply lt_n_S.Apply lt_n_S.Apply H.Inversion H1.Auto.
Apply le_lt_n_Sm.Apply le_trans_S.Apply le_trans_S.Exact H3.
Qed.

Lemma le_plus_lem1 : (a,b,c:nat) (le a b) -> (le (plus c a) (plus c b)).
Intros.Inversion H.Auto.
Elim plus_n_Sm.Apply le_S.Auto.
Qed. Hint le_plus_lem1.

Lemma le_plus_lem2 : (a,b,c:nat) (le (plus c a) (plus c b)) -> (le a b).
Induction c.Auto.
Simpl.Intros.Apply H.Apply le_S_n.Exact H0.
Qed.

Lemma gt_double : (a,b:nat) (gt (plus a a) (plus b b)) -> (gt a b). 
Induction a.Simpl.Intros.Absurd (gt O (plus b b)).Auto.
Auto.Induction b.Simpl.Auto.
Intros.Apply gt_n_S.
Apply H.Generalize H1 .Simpl.Elim plus_n_Sm.Elim plus_n_Sm.Intro.
Cut (gt (S (plus n n)) (S (plus n0 n0))).Intro.Apply gt_S_n.Exact H3.
Apply gt_S_n.Exact H2.
Qed.

Lemma gt_double_inv : (a,b:nat) (gt a b) -> (gt (plus a a) (plus b b)).
Induction a.Intros.Absurd (gt O b).Auto.Exact H.
Induction b.Intros.Simpl.Auto.
Intros.Simpl.Elim plus_n_Sm.
Elim plus_n_Sm.Apply gt_n_S.Apply gt_n_S.Apply H.Apply gt_S_n.Exact H1.
Qed.

Lemma gt_double_n_S : (a,b:nat) (gt a b) -> (gt (plus a a) (S (plus b b))).
Induction a.Intros.Absurd (gt O b).Auto.
Exact H.
Induction b.Simpl.Intro.
Apply gt_n_S.Unfold gt .Unfold lt .Elim plus_n_Sm.Auto.
Intros.Simpl.Apply gt_n_S.
Elim plus_n_Sm.Apply gt_n_S.Elim plus_n_Sm.Apply H.Apply gt_S_n.Exact H1.
Qed.

Lemma gt_double_S_n : (a,b:nat) (gt a b) -> (gt (S (plus a a)) (plus b b)).
Intros.Apply gt_trans with (plus a a) .Auto.Apply gt_double_inv.Exact H.
Qed.

(****************************************************************)
(* Complements sur minus *)

Lemma minus_le_O : (a,b:nat) (le a b) -> (minus a b)=O.
Intros.Pattern a b .Apply le_elim_rel.Auto.
Intros.Simpl.Exact H1.
Exact H.
Qed.

(*
Lemma minus_le_lem2a : (a,b:nat) (le a b) -> (le (minus a b) a).
Intros.Rewrite -> minus_le_O.Auto.Exact H.
Qed.
Lemma minus_le_lem2b : (a,b:nat) (le b a) -> (le (minus a b) a).
Intros.Apply le_plus_lem2 with c:=b .Rewrite <- le_plus_minus.Auto.Exact H.
Qed.
*)

Lemma minus_n_SO : (n:nat) (minus n (S O))=(pred n).
Induction n.Auto.Intros.Simpl.Auto.
Qed.

Lemma minus_le_lem2c : (a,b:nat) (le (minus a (S b)) (minus a b)).
Intros.Pattern a b .Apply nat_double_ind.Auto.
Intro.Elim minus_n_O.Rewrite -> minus_n_SO.Simpl.Auto.
Intros.Simpl.Exact H.
Qed.

Lemma minus_le_lem2 : (a,b:nat) (le (minus a b) a).
Induction b.Elim minus_n_O.Auto.
Intros.Apply le_trans with (minus a n) .Apply minus_le_lem2c.
Exact H.
Qed. Hint minus_le_lem2.

Lemma minus_minus_lem1 : (a,b:nat) (minus (minus a b) a)=O.
Intros.Apply minus_le_O.Apply minus_le_lem2.
Qed.Hint minus_minus_lem1.

Lemma minus_minus_lem2 : (a,b:nat)
			(le b a) -> (minus a (minus a b))=b.
Induction b.Intros.Elim minus_n_O.Elim minus_n_n.Try Trivial.Intros.
Replace (minus a (minus a (S n))) with (minus (S a) (S (minus a (S n)))).
Rewrite <- (minus_Sn_m a (S (minus a (S n)))).Rewrite -> (minus_Sn_m a (S n)).
Simpl.Rewrite <- H.Rewrite -> H.Rewrite -> H.Try Trivial.
Apply le_trans_S.Exact H0.
Apply le_trans_S.Exact H0.Apply le_trans_S.Exact H0.Exact H0.
Rewrite -> minus_Sn_m.Simpl.Apply minus_le_lem2.Exact H0.Simpl.Try Trivial.
Qed.

Lemma le_minus_minus : (a,b,c:nat)
	(le c b) -> (le (minus a b) (minus a c)).
Induction a.Simpl.Auto.
Intros.Generalize H0 .Elim c.Elim minus_n_O.Intro.Apply minus_le_lem2.
Elim b.Intros.Absurd (le (S n0) O).Auto.
Exact H2.
Intros.Simpl.Apply H.Apply le_S_n.Exact H3.
Qed.
