(***************************************************************************)
(* File: ZFrelations.v                                                     *)
(* auteur: alex@sysal.ibp.fr                                               *)
(* date: 19/10/95                                                          *)
(***************************************************************************)
Require Export axs_fundation.

(***************************************************************************)
(***************************************************************************)
(* Une relation est un ensemble de couples                                 *)
(* Nous nous interessons ici aux relations binaires.                       *)
(***************************************************************************)
(***************************************************************************)
Definition rel := [r:E]
  (c:E)(In c r)->(Ex[x:E](Ex[y:E](c=(couple x y))))
.

Definition rel_sig := [r:E][a,b:E]
  (inc r (cartesien a b))
.

Lemma lem_rel_sig_to_rel :
  (r:E)(a,b:E)
    (rel_sig r a b)->(rel r)
.
(* Proof of lem_rel_sig_is_rel                                             *)
(Unfold rel_sig ;Unfold inc ;Intros).
(Unfold rel ;Intros).
(Generalize (H c H0) ;Clear  H H0;Intros).
(Elim (lem_cartesian_propertie a b c);Intros;Generalize (H0 H) ;Clear  H H0 H1;Intros).
(Elim H;Clear  H;Intros;Elim H;Clear  H;Intros;Elim H;Clear  H;Intros;Elim H0;Clear  H0;Intros).
(Exists x;Exists x0;Auto).

Save.
(* End of proof of lem_rel_sig_is_rel                                      *)

Lemma lem_rel_to_rel_sig :
  (r:E)
    (rel r)->(Ex[a:E](Ex[b:E]((rel_sig r a b))))
.
(* Proof of lem_rel_to_rel_sig                                             *)
Intros.
Unfold rel in H.
Exists (subset [z:E](Ex [t:E](In (couple z t) r)) (reunion (reunion r))).
Exists (subset [t:E](Ex [z:E](In (couple z t) r)) (reunion (reunion r))).
(Unfold rel_sig;Unfold inc;Intros).
(Generalize (H v0 H0);Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_cartesian_propertie (subset [z:E](Ex [t:E](In (couple z t) r)) (reunion (reunion r))) (subset [t:E](Ex [z:E](In (couple z t) r)) (reunion (reunion r))) v0);Intros;Apply H3;Clear H2 H3;Intros).
(Exists x;Exists x0;Split;[Idtac|Split;[Idtac|Auto]]).
(Elim (axs_comprehension [z:E](Ex [t:E](In (couple z t) r)) (reunion (reunion r)) x);Intros;Apply H3;Clear H2 H3;Intros).
(Generalize H0;Rewrite -> H1;Intros).
(Split;[Idtac|Exists x0;Auto]).
(Elim (axs_reunion (reunion r) x);Intros;Apply H4;Clear H3 H4;Intros).
(Exists (singleton x);Split;[Idtac|Apply lem_x_in_sing_x]).
(Elim (axs_reunion r (singleton x));Intros;Apply H4;Clear H3 H4;Intros).
(Exists (couple x x0);Split;[Auto|Idtac]).
(Unfold couple;Elim (axs_paire (singleton x) (paire x x0) (singleton x));Intros;Apply H4;Left;Reflexivity).

(Generalize H0;Rewrite -> H1;Intros).
(Elim (axs_comprehension [t:E](Ex [z:E](In (couple z t) r)) (reunion (reunion r)) x0);Intros;Apply H4;Clear H3 H4;Intros).
(Split;[Idtac|Exists x;Auto]).
(Elim (axs_reunion (reunion r) x0);Intros;Apply H4;Clear H3 H4;Intros).
(Exists (paire x x0);Split;[Idtac|Elim (axs_paire x x0 x0);Intros;Apply H4;Right;Reflexivity]).
(Elim (axs_reunion r (paire x x0));Intros;Apply H4;Clear H3 H4;Intros).
(Exists (couple x x0);Split;[Auto|Elim (axs_paire (singleton x) (paire x x0) (paire x x0));Intros;Apply H4;Right;Reflexivity]).

Save.
(* End of proof of lem_rel_to_rel_sig                                      *)

(***************************************************************************)
(***************************************************************************)
(* Notions de domaine et d'image d'une relation binaire                    *)
(***************************************************************************)
(***************************************************************************)
Definition rdom := [r:E][a,b:E][rRel:(rel_sig r a b)]
  (subset [x:E](Ex[y:E]((In (couple x y) r))) a)
.

Definition rImg := [r:E][a,b:E][rRel:(rel_sig r a b)]
  (subset [y:E](Ex[x:E]((In (couple x y) r))) b)
.

Definition rdom2 := [r:E][rRel:(rel r)]
  (subset [x:E](Ex[y:E](In (couple x y) r)) (reunion (reunion r)))
.

Definition rImg2 := [r:E][rRel:(rel r)]
  (subset [y:E](Ex[x:E](In (couple x y) r)) (reunion (reunion r)))
.

Lemma lem_rdom_eq_rdom2 :
  (r:E)(a,b:E)(rRel:(rel_sig r a b))
    (rdom r a b rRel) = (rdom2 r (lem_rel_sig_to_rel r a b rRel))
.
(* Proof of lem_rdom_eq_rdom2                                              *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold rdom in H.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim H0;Clear H0;Intros).
Unfold rdom2.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) (reunion (reunion r)) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists x;Auto]).
(Elim (axs_reunion (reunion r) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Exists (singleton v2);Split;[Idtac|Apply lem_x_in_sing_x]).
(Elim (axs_reunion r (singleton v2));Intros;Apply H2;Clear H1 H2;Intros).
(Exists (couple v2 x);Split;[Auto|Unfold couple;Elim (axs_paire (singleton v2) (paire v2 x) (singleton v2));Intros;Apply H2;Left;Reflexivity]).

Unfold rdom2 in H.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) (reunion (reunion r)) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
Unfold rdom.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists x;Auto]).
(Unfold rel_sig  in rRel;Unfold inc in rRel).
(Generalize (rRel (couple v2 x) H0);Intros).
(Elim (lem_cartesian_propertie a b (couple v2 x));Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros).
(Generalize H1 ;Rewrite -> H3;Tauto).

Save.
(* End of proof of lem_rdom_eq_rdom2                                       *)

Lemma lem_rImg_eq_rImg2 :
  (r:E)(a,b:E)(rRel:(rel_sig r a b))
    (rImg r a b rRel) = (rImg2 r (lem_rel_sig_to_rel r a b rRel))
.
(* Proof of lem_rImg_eq_rImg2                                              *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold rImg in H.
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) b v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
Unfold rImg2.
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) (reunion (reunion r)) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists x;Auto]).
(Elim (axs_reunion (reunion r) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Exists (paire x v2);Split;[Idtac|Elim (axs_paire x v2 v2);Intros;Apply H2;Right;Reflexivity]).
(Elim (axs_reunion r (paire x v2));Intros;Apply H2;Clear  H1 H2;Intros).
(Exists (couple x v2);Split;[Auto|Unfold couple;Elim (axs_paire (singleton x) (paire x v2) (paire x v2));Intros;Apply H2;Right;Reflexivity]).

Unfold rImg2 in H.
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) (reunion (reunion r)) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
Unfold rImg.
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) b v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists x;Auto]).
(Unfold rel_sig in rRel;Unfold inc in rRel).
(Generalize (rRel (couple x v2) H0);Clear rRel H H0;Intros).
(Elim (lem_cartesian_propertie a b (couple x v2));Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_couple_propertie x x0 v2 x1);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Generalize H0;Rewrite -> H2;Tauto).

Save.
(* End of proof of lem_rImg_eq_rImg2                                       *)

(***************************************************************************)
(* Si (x,y) est dans r alors x est dans le domaine de r.                   *)
(***************************************************************************)
Lemma lem_rel_and_rdom :
  (r:E)(a,b:E)(rRel:(rel_sig r a b))(x,y:E)
    (In (couple x y) r) -> (In x (rdom r a b rRel))
.
(* Proof of lem_rel_and_rdom                                               *)
(Intros;Unfold rdom).
(Elim (axs_comprehension [x0:E](Ex [y:E](In (couple x0 y) r)) a x);Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists y;Auto]).
(Unfold rel_sig in rRel;Unfold inc in rRel).
(Generalize (rRel (couple x y) H);Intros).
(Elim (lem_cartesian_propertie a b (couple x y));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros;Generalize H0;Rewrite -> H2;Tauto).

Save.
(* End of proof of lem_rel_and_rdom                                        *)

Lemma lem_rel_and_rdom2 :
  (r:E)(rRel:(rel r))(x,y:E)
    (In (couple x y) r) -> (In x (rdom2 r rRel))
.
(* Proof of lem_rel_and_rdom2                                              *)
(Unfold rdom2;Intros).
(Elim (axs_comprehension [x0:E](Ex [y:E](In (couple x0 y) r)) (reunion (reunion r)) x);Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists y;Auto]).
(Elim (axs_reunion (reunion r) x);Intros;Apply H1;Clear H0 H1;Intros).
(Exists (singleton x);Split;[Idtac|Apply lem_x_in_sing_x]).
(Elim (axs_reunion r (singleton x));Intros;Apply H1;Clear H0 H1;Intros).
(Exists (couple x y);Split;[Auto|Idtac]).
(Elim (axs_paire (singleton x) (paire x y) (singleton x));Intros;Apply H1;Clear H0 H1;Intros).
(Left;Reflexivity).

Save.
(* End of proof of lem_rel_and_rdom2                                       *)

(***************************************************************************)
(* Si (x,y) est dans r alors y est dans l'image de r.                      *)
(***************************************************************************)
Lemma lem_rel_and_rImg :
  (r:E)(a,b:E)(rRel:(rel_sig r a b))(x,y:E)
    (In (couple x y) r) -> (In y (rImg r a b rRel))
.
(* Proof of lem_rel_and_rImg                                               *)
(Unfold rImg;Intros).
(Elim (axs_comprehension [y0:E](Ex [x:E](In (couple x y0) r)) b y);Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists x;Auto]).
(Generalize (rRel (couple x y) H);Intros).
(Elim (lem_cartesian_propertie a b (couple x y));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros).
(Rewrite -> H3;Auto).

Save.
(* End of proof of lem_rel_and_rImg                                        *)

Lemma lem_rel_and_rImg2 :
  (r:E)(rRel:(rel r))(x,y:E)
    (In (couple x y) r) -> (In y (rImg2 r rRel))
.
(* Proof of lem_rel_and_rImg2                                              *)
(Unfold rImg2;Intros).
(Elim (axs_comprehension [y0:E](Ex [x:E](In (couple x y0) r)) (reunion (reunion r)) y);Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists x;Auto]).
(Elim (axs_reunion (reunion r) y);Intros;Apply H1;Clear H0 H1;Intros).
(Exists (paire x y);Split;[Idtac|Elim (axs_paire x y y);Intros;Apply H1;Intros;Right;Reflexivity]).
(Elim (axs_reunion r (paire x y));Intros;Apply H1;Clear H0 H1;Intros).
(Exists (couple x y);Split;[Auto|Elim (axs_paire (singleton x) (paire x y) (paire x y));Intros;Apply H1;Right;Reflexivity]).

Save.
(* End of proof of lem_rel_and_rImg2                                       *)

(***************************************************************************)
(***************************************************************************)
(* Relation partielle vs relation totale                                   *)
(* Nous considerons ici qu'une relation est totale si tous les elements de *)
(* son domaine sont "consommes" par la relation.                           *)
(***************************************************************************)
(***************************************************************************)
Definition rel_total := [r:E][a,b:E][rRel:(rel_sig r a b)]
  (rdom r a b rRel)=a
.

Definition rel_partiel := [r:E][a,b:E][rRel:(rel_sig r a b)]
  (Ex[x:E]((In x a)->(y:E)~(In (couple x y) r)))
.

(***************************************************************************)
(***************************************************************************)
(* Proprietes de base des relations                                        *)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(* Toute application est une relation                                      *)
(***************************************************************************)

Lemma lem_fun_is_rel :
  (f:E)(Af:(fun f))
    (rel f)
.
(* Proof of lem_fun_is_rel                                                 *)
(Unfold rel;Intros).
Unfold fun in Af.
(Elim Af;Clear Af;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold inc in H0.
(Generalize (H0 c H);Clear H H0 H1;Intros).
(Elim (lem_cartesian_propertie x x0 c);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Exists x1;Exists x2;Auto).

Save.
(* End of proof of lem_fun_is_rel                                          *)


(***************************************************************************)
(* Le domaine et l'image d'une relation non vide sont non vides            *)
(***************************************************************************)
(*
Lemma lem_rel_not_empty_so_rdom_not_empty :
  (r:E)(rRel:(rel r))
    ~(r=Vide)
    ->
    ~((rdom2 r rRel)=Vide)
.
(* Proof of lem_rel_not_empty_so_rdom_not_empty                            *)
Save.
(* End of proof of lem_rel_not_empty_so_rdom_not_empty                     *)
*)
(***************************************************************************)
(* Considerons maintenant des relations dont l'image est tout ou partie du *)
(* domaine.                                                                *)
(***************************************************************************)

(***************************************************************************)
(* Reflexivite & irreflexivite                                             *)
(***************************************************************************)
Definition reflexivity := [r:E][a:E][rRel:(rel_sig r a a)]
  (x:E)(In x (rdom r a a rRel))->(In (couple x x) r)
.

Definition irreflexivity := [r:E][a:E][rRel:(rel_sig r a a)]
  (x:E)(In x (rdom r a a rRel))->~(In (couple x x) r)
.

(***************************************************************************)
(* Le domaine d'une relation reflexive est inclus dans l'image             *)
(***************************************************************************)
Lemma lem_refl_dom_inc_img :
  (r:E)(a:E)(rRel:(rel_sig r a a))
    (reflexivity r a rRel)
    ->
    (inc (rdom r a a rRel) (rImg r a a rRel))
.
(* Proof of lem_refl_dom_inc_img                                           *)
(Unfold inc;Intros).
Unfold reflexivity in H.
(Generalize (H v0 H0);Clear H;Intros).
Unfold rImg.
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) a v0);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists v0;Auto]).
Unfold rdom in H0.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a v0);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Auto).

Save.
(* End of proof of lem_refl_dom_inc_img                                    *)

(***************************************************************************)
(* Si le domaine d'une relation n'est pas inclus dans l'image, alors la    *)
(* relation n'est pas reflexive.                                           *)
(***************************************************************************)
Lemma lem_refl_dom_inc_img_recip :
  (r:E)(a:E)(rRel:(rel_sig r a a))
    ~(inc (rdom r a a rRel) (rImg r a a rRel))
    ->
    ~(reflexivity r a rRel)
.
(* Proof of lem_refl_dom_inc_img_recip                                     *)
Intros.
(Unfold not;Intros).
(Generalize (lem_refl_dom_inc_img r a rRel H0);Intros).
(Absurd (inc (rdom r a a rRel) (rImg r a a rRel));Auto).

Save.
(* End of proof of lem_refl_dom_inc_img_recip                              *)

(***************************************************************************)
(* Une relation non vide reflexive n'est pas irreflexive                   *)
(***************************************************************************)

Lemma lem_refl_not_irrefl :
  (r:E)(a:E)(rNe:~(r=Vide))(rRel:(rel_sig r a a))
    (reflexivity r a rRel)
    ->
    ~(irreflexivity r a rRel)
.
(* Proof of lem_refl_not_irrefl                                            *)
(Unfold reflexivity;Unfold irreflexivity;Unfold not;Intros).
(Generalize (axs_fundation r rNe);Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Generalize rRel;Intros;Unfold rel_sig in rRel0;Unfold inc in rRel0;Intros).
(Generalize (rRel0 x H1);Clear rRel0;Intros).
(Elim (lem_cartesian_propertie a a x);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H4;Clear H4;Intros).
(Generalize H1 ;Rewrite -> H5;Intros).
(Cut (In x0 (rdom r a a rRel));Intros).
(Generalize (H x0 H7) (H0 x0 H7);Intros).
(Absurd (In (couple x0 x0) r);Auto).

Unfold rdom.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a x0);Intros;Apply H8;Clear H7 H8;Intros).
(Split;[Auto|Exists x1;Auto]).

Save.
(* End of proof of lem_refl_not_irrefl                                     *)

(***************************************************************************)
(* Une relation non vide irreflexive n'est pas reflexivive                 *)
(***************************************************************************)
Lemma lem_irrefl_not_refl :
  (r:E)(a:E)(rNe:~(r=Vide))(rRel:(rel_sig r a a))
    (irreflexivity r a rRel)
    ->
    ~(reflexivity r a rRel)
.
(* Proof of lem_irrefl_not_refl                                            *)
(Unfold reflexivity;Unfold irreflexivity;Unfold not;Intros).
(Generalize (axs_fundation r rNe);Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Generalize rRel;Intros;Unfold rel_sig in rRel0;Unfold inc in rRel0;Generalize (rRel0 x H1);Clear rRel0;Intros).
(Elim (lem_cartesian_propertie a a x);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H4;Clear H4;Intros).
(Generalize H1;Clear H1;Rewrite -> H5;Intros).
(Cut (In x0 (rdom r a a rRel));Intros).
(Generalize (H x0 H6) (H0 x0 H6);Intros).
(Absurd (In (couple x0 x0) r);Auto).

Unfold rdom.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a x0);Intros;Apply H7;Clear H6 H7;Intros).
(Split;[Auto|Exists x1;Auto]).

Save.
(* End of proof of lem_irrefl_not_refl                                     *)


(***************************************************************************)
(* Symmetrie & antisymetrie                                                *)
(***************************************************************************)
Definition symmetry := [r:E][a:E][rRel:(rel_sig r a a)]
  (x,y:E)(In (couple x y) r)->(In (couple y x) r)
.
Definition antisymmetry := [r:E][a:E][rRel:(rel_sig r a a)]
  (x,y:E)(In (couple x y) r)->(In (couple y x) r)->(x=y)
.

(***************************************************************************)
(* Propriete de la symetrie                                                *)
(***************************************************************************)
Lemma lem_sym_so_rdom_eq_rImg :
  (r:E)(a:E)(rRel:(rel_sig r a a))
    (symmetry r a rRel)
    ->
    (rdom r a a rRel)=(rImg r a a rRel)
.
(* Proof of lem_sym_so_rdom_eq_rImg                                        *)
(Unfold symmetry;Intros).
(Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold rdom in H0.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Generalize (H v2 x H1);Clear H;Intros).
Unfold rImg.
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) a v2);Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Auto|Exists x;Auto]).

Unfold rImg in H0.
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) a v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Generalize (H x v2 H1);Clear H H1;Intros).
Unfold rdom.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Auto|Exists x;Auto]).

Save.
(* End of proof of lem_sym_so_rdom_eq_rImg                                 *)

(***************************************************************************)
(* Transitivite                                                            *)
(***************************************************************************)
Definition transitivity := [r:E][a:E][rRel:(rel_sig r a a)]
  (x,y,z:E)
    (In (couple x y) r)
    ->
    (In (couple y z) r)
    ->
    (In (couple x z) r)
.

(***************************************************************************)
(***************************************************************************)
(* Autres relations                                                        *)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(* Trois relations sur un ensemble donne                                   *)
(***************************************************************************)

(***************************************************************************)
(* La relation vide                                                        *)
(***************************************************************************)
Lemma lem_Vide_rel_sig :
  (a:E)(rel_sig Vide a a)
.
(* Proof of lem_Vide_rel_sig                                               *)
(Unfold rel_sig;Intros).
Exact (lem_vide_is_inc (cartesien a a)).

Save.
(* End of proof of lem_Vide_rel_sig                                        *)

(***************************************************************************)
(* Le domaine de la relation vide est l'ensemble vide                      *)
(***************************************************************************)
Lemma lem_Vide_dom :
  (a:E)(rdom Vide a a (lem_Vide_rel_sig a))=Vide
.
(* Proof of lem_Vide_dom                                                   *)
(Intros;Apply (lem_x_inc_vide (rdom Vide a a (lem_Vide_rel_sig a)));Unfold inc;Intros).
Unfold rdom in H.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) Vide)) a v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Generalize (lem_vide_propertie (couple v0 x) var_vide);Intros;Absurd (In (couple v0 x) Vide);Auto).

Save.
(* End of proof of lem_Vide_dom                                            *)

(***************************************************************************)
(* L'image de la relation vide est l'ensemble vide                         *)
(***************************************************************************)
Lemma lem_Vide_Img :
  (a:E)(rImg Vide a a (lem_Vide_rel_sig a))=Vide
.
(* Proof of lem_Vide_Img                                                   *)
(Intros;Apply (lem_x_inc_vide (rImg Vide a a (lem_Vide_rel_sig a)));Unfold inc;Unfold rImg;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) Vide)) a v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Generalize (lem_vide_propertie (couple x v0) var_vide);Intros;Absurd (In (couple x v0) Vide);Auto).

Save.
(* End of proof of lem_Vide_Img                                            *)

(***************************************************************************)
(* La relation pleine                                                      *)
(***************************************************************************)
Definition rfull := [a:E]
  (subset [c:E](Ex[x:E](Ex[y:E](c=(couple x y)))) (cartesien a a))
.

(***************************************************************************)
(* rfull est une relation sur a                                            *)
(***************************************************************************)
Lemma lem_rfull_rel_sig :
  (a:E)(rel_sig (rfull a) a a)
.
(* Proof of lem_rfull_rel_sig                                              *)
(Unfold rel_sig;Unfold inc;Unfold rfull;Intros).
(Elim (axs_comprehension [c:E](Ex [x:E](Ex [y:E](eq ? c (couple x y)))) (cartesien a a) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_rfull_rel_sig                                       *)

(***************************************************************************)
(* Le domaine de la relation pleine est a                                  *)
(***************************************************************************)
Lemma lem_rfull_dom :
  (a:E)(rdom (rfull a) a a (lem_rfull_rel_sig a))=a
.
(* Proof of lem_rfull_dom                                                  *)
(Unfold rdom;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (rfull a))) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (rfull a))) a v2);Intros;Apply H1;Clear H0 H1;Split;[Auto|Idtac]).
(Exists v2;Unfold rfull).
(Elim (axs_comprehension [c:E](Ex [x:E](Ex [y:E](eq ? c (couple x y)))) (cartesien a a) (couple v2 v2));Intros;Apply H1;Clear H0 H1;Intros;Split;[Idtac|Exists v2;Exists v2;Reflexivity]).
(Elim (lem_cartesian_propertie a a (couple v2 v2));Intros;Apply H1;Clear H0 H1;Exists v2;Exists v2;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_rfull_dom                                           *)

(***************************************************************************)
(* L'image de la relation pleine est a                                     *)
(***************************************************************************)
Lemma lem_rfull_Img :
  (a:E)(rImg (rfull a) a a (lem_rfull_rel_sig a))=a
.
(* Proof of lem_rfull_Img                                                  *)
(Unfold rImg;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) (rfull a))) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) (rfull a))) a v2);Intros;Apply H1;Clear H0 H1;Intros;Split;[Auto|Unfold rfull;Exists v2]).
(Elim (axs_comprehension [c:E](Ex [x:E](Ex [y:E](eq ? c (couple x y)))) (cartesien a a) (couple v2 v2));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Elim (lem_cartesian_propertie a a (couple v2 v2));Intros;Apply H1;Exists v2;Exists v2;Split;[Auto|Split;[Auto|Reflexivity]]|Exists v2;Exists v2;Reflexivity]).

Save.
(* End of proof of lem_rfull_Img                                           *)

(***************************************************************************)
(* La relation pleine sur a n'est autre que a x a                          *)
(***************************************************************************)
Lemma lem_rfull_eq_cart :
  (a:E)(rfull a)=(cartesien a a)
.
(* Proof of lem_rfull_eq_cart                                              *)
(Unfold rfull;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [c:E](Ex [x:E](Ex [y:E](eq ? c (couple x y)))) (cartesien a a) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

(Elim (axs_comprehension [c:E](Ex [x:E](Ex [y:E](eq ? c (couple x y)))) (cartesien a a) v2);Intros;Apply H1;Clear H0 H1;Intros;Split;[Auto|Idtac]).
(Elim (lem_cartesian_propertie a a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Exists x;Exists x0;Auto).

Save.
(* End of proof of lem_rfull_eq_cart                                       *)

(***************************************************************************)
(* La propriete de la relation pleine                                      *)
(***************************************************************************)
Lemma lem_rfull_prop :
  (a,x,y:E)
    (In x a)
    ->
    (In y a)
    ->
    (In (couple x y) (rfull a))
.
(* Proof of lem_rfull_prop                                                 *)
(Unfold rfull;Intros).
(Elim (axs_comprehension [c:E](Ex [x0:E](Ex [y0:E](eq ? c (couple x0 y0)))) (cartesien a a) (couple x y));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Elim (lem_cartesian_propertie a a (couple x y));Intros;Apply H2;Exists x;Exists y;Split;[Auto|Split;[Auto|Reflexivity]]|Exists x;Exists y;Reflexivity]).

Save.
(* End of proof of lem_rfull_prop                                          *)

(***************************************************************************)
(* La relation identite                                                    *)
(***************************************************************************)
Definition rId := [a:E]
  (subset [c:E](Ex[x:E](c=(couple x x))) (cartesien a a))
.

(***************************************************************************)
(* L'identite est une relation                                             *)
(***************************************************************************)
Lemma lem_rId_rel_sig :
  (a:E)(rel_sig (rId a) a a)
.
(* Proof of lem_rId_rel_sig                                                *)
(Unfold rel_sig;Unfold inc;Unfold rId;Intros).
(Elim (axs_comprehension [c:E](Ex [x:E](eq ? c (couple x x))) (cartesien a a) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_rId_rel_sig                                         *)

(***************************************************************************)
(* Le domaine de l'identite est a                                          *)
(***************************************************************************)
Lemma lem_rId_dom :
  (a:E)(rdom (rId a) a a (lem_rId_rel_sig a))=a
.
(* Proof of lem_rId_dom                                                    *)
(Unfold rdom;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (rId a))) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (rId a))) a v2);Intros;Apply H1;Clear H0 H1;Split;[Auto|Exists v2;Unfold rId ]).
(Elim (axs_comprehension [c:E](Ex [x:E](eq ? c (couple x x))) (cartesien a a) (couple v2 v2));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Elim (lem_cartesian_propertie a a (couple v2 v2));Intros;Apply H1;Exists v2;Exists v2;Split;[Auto|Split;[Auto|Reflexivity]]|Exists v2;Reflexivity]).

Save.
(* End of proof of lem_rId_dom                                             *)

(***************************************************************************)
(* L'image de l'identite est a                                             *)
(***************************************************************************)
(*
Lemma lem_rId_Img :
  (a:E)(rImg (rId a) a a (lem_rId_rel_sig a))=a
.
*)
(***************************************************************************)
(* Complementaire d'une relation                                           *)
(***************************************************************************)
Definition rel_Comp := [r,a,b:E][rRel:(rel_sig r a b)]
  (Comp (cartesien a b) r)
.

Lemma lem_rel_Comp_rel_sig :
  (r,a,b:E)(rRel:(rel_sig r a b))
    (rel_sig (rel_Comp r a b rRel) a b)
.
(* Proof of lem_rel_Comp_rel_sig                                           *)
(Unfold rel_sig;Unfold rel_Comp;Unfold Comp;Unfold inc;Intros).
(Elim (axs_comprehension [y:E](not (In y r)) (cartesien a b) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_rel_Comp_rel_sig                                    *)

(***************************************************************************)
(* Reunion de relations                                                    *)
(***************************************************************************)
Definition rel_union :=
  [r1,a1,b1:E][r2,a2,b2:E][rRel1:(rel_sig r1 a1 b1)][rRel2:(rel_sig r2 a2 b2)]
    (union r1 r2)
.

Lemma lem_rel_union_rel_sig :
  (r1,a1,b1:E)(r2,a2,b2:E)(rRel1:(rel_sig r1 a1 b1))(rRel2:(rel_sig r2 a2 b2))
    (rel_sig 
      (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2)
      (union a1 a2) (union b1 b2)
    )
.
(* Proof of lem_rel_union_rel_sig                                          *)
(Unfold rel_union;Unfold rel_sig;Unfold inc;Intros).
(Elim (lem_union_propertie r1 r2 v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim (lem_cartesian_propertie (union a1 a2) (union b1 b2) v0);Intros;Apply H1;Clear H0 H1;Intros).
(Elim H;Clear H;Intros).
(Generalize (rRel1 v0 H);Intros).
(Elim (lem_cartesian_propertie a1 b1 v0);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Exists x;Exists x0;Split;[Elim (lem_union_propertie a1 a2 x);Intros;Apply H4;Left;Auto|Split;[Elim (lem_union_propertie b1 b2 x0);Intros;Apply H4;Left;Auto|Auto]]).

(Generalize (rRel2 v0 H);Intros).
(Elim (lem_cartesian_propertie a2 b2 v0);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Exists x;Exists x0;Split;[Elim (lem_union_propertie a1 a2 x);Intros;Apply H4;Right;Auto|Split;[Elim (lem_union_propertie b1 b2 x0);Intros;Apply H4;Right;Auto|Auto]]).

Save.
(* End of proof of lem_rel_union_rel_sig                                   *)

(***************************************************************************)
(* Le domaine de l'union de deux relations est l'union des deux domaines   *)
(***************************************************************************)
Lemma lem_rel_union_dom :
  (r1,a1,b1:E)(r2,a2,b2:E)
  (rRel1:(rel_sig r1 a1 b1))
  (rRel2:(rel_sig r2 a2 b2))
  (rdom
    (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2)
    (union a1 a2) (union b1 b2)
    (lem_rel_union_rel_sig r1 a1 b1 r2 a2 b2 rRel1 rRel2)
  ) = (union (rdom r1 a1 b1 rRel1) (rdom r2 a2 b2 rRel2))
.
(* Proof of lem_rel_union_dom                                              *)
(Intros;Unfold rdom;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [x:E](Ex[y:E](In (couple x y) (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2))) (union a1 a2) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
Unfold rel_union in H0.
(Elim (lem_union_propertie a1 a2 v2);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim (lem_union_propertie r1 r2 (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim (lem_union_propertie (subset [x:E](Ex [y:E](In (couple x y) r1)) a1) (subset [x:E](Ex [y:E](In (couple x y) r2)) a2) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Elim H;Elim H0;Clear H H0;Intros).
(Left;Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r1)) a1 v2);Intros;Apply H2;Clear H1 H2;Split;[Auto|Exists x;Auto]).

(Generalize (rRel2 (couple v2 x) H);Intros).
(Elim (lem_cartesian_propertie a2 b2 (couple v2 x));Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H1 H2;Rewrite <- H3;Rewrite <- H4;Clear H1 H2 H3 H4;Clear x0 x1;Intros).
(Right;Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r2)) a2 v2);Intros;Apply H4;Split;[Auto|Exists x;Auto]).

(Generalize (rRel1 (couple v2 x) H);Intros).
(Elim (lem_cartesian_propertie a1 b1 (couple v2 x));Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H1 H2;Rewrite <- H3;Rewrite <- H4;Clear H1 H2 H3 H4;Clear x0 x1;Intros).
(Left;Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r1)) a1 v2);Intros;Apply H4;Split;[Auto|Exists x;Auto]).

(Right;Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r2)) a2 v2);Intros;Apply H2;Split;[Auto|Exists x;Auto]).

(Elim (lem_union_propertie (subset [x:E](Ex [y:E](In (couple x y) r1)) a1) (subset [x:E](Ex [y:E](In (couple x y) r2)) a2) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim (axs_comprehension [x:E](Ex[y:E](In (couple x y) (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2))) (union a1 a2) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r1)) a1 v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Split;[Elim (lem_union_propertie a1 a2 v2);Intros;Apply H2;Left;Auto|Exists x;Unfold rel_union]).
(Elim (lem_union_propertie r1 r2 (couple v2 x));Intros;Apply H2;Left;Auto).

(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r2)) a2 v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Split;[Elim (lem_union_propertie a1 a2 v2);Intros;Apply H2;Right;Auto|Unfold rel_union]).
(Exists x;Elim (lem_union_propertie r1 r2 (couple v2 x));Intros;Apply H2;Right;Auto).

Save.
(* End of proof of lem_rel_union_dom                                       *)

(***************************************************************************)
(* L'image de l'union de deux relations est l'union des deux images        *)
(***************************************************************************)
Lemma lem_rel_union_Img :
  (r1,a1,b1:E)(r2,a2,b2:E)
  (rRel1:(rel_sig r1 a1 b1))
  (rRel2:(rel_sig r2 a2 b2))
  (rImg
    (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2)
    (union a1 a2) (union b1 b2)
    (lem_rel_union_rel_sig r1 a1 b1 r2 a2 b2 rRel1 rRel2)
  ) = (union (rImg r1 a1 b1 rRel1) (rImg r2 a2 b2 rRel2))
.
(* Proof of lem_rel_union_Img                                              *)
(Unfold rImg;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [y:E](Ex[x:E](In (couple x y) (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2))) (union b1 b2) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_union_propertie r1 r2 (couple x v2));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim (lem_union_propertie b1 b2 v2);Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim (lem_union_propertie (subset [y:E](Ex [x:E](In (couple x y) r1)) b1) (subset [y:E](Ex [x:E](In (couple x y) r2)) b2) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Elim H0;Elim H;Clear H H0;Intros).
(Left;Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r1)) b1 v2);Intros;Apply H2;Split;[Auto|Exists x;Auto]).

(Generalize (rRel1 (couple x v2) H0);Intros).
(Elim (lem_cartesian_propertie a1 b1 (couple x v2));Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x0 v2 x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H1 H2;Rewrite <- H3;Rewrite <- H4;Clear H1 H2 H3 H4;Clear x0 x1;Intros).
(Left;Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r1)) b1 v2);Intros;Apply H4;Split;[Auto|Exists x;Auto]).

(Generalize (rRel2 (couple x v2) H0);Intros).
(Elim (lem_cartesian_propertie a2 b2 (couple x v2));Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x0 v2 x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H1 H2;Rewrite <- H3;Rewrite <- H4;Clear H1 H2 H3 H4;Clear x0 x1;Intros).
(Right;Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r2)) b2 v2);Intros;Apply H4;Split;[Auto|Exists x;Auto]).

(Right;Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r2)) b2 v2);Intros;Apply H2;Split;[Auto|Exists x;Auto]).

(Elim (lem_union_propertie (subset [y:E](Ex [x:E](In (couple x y) r1)) b1) (subset [y:E](Ex [x:E](In (couple x y) r2)) b2) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim (axs_comprehension [y:E](Ex[x:E](In (couple x y) (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2))) (union b1 b2) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r1)) b1 v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Split;[Elim (lem_union_propertie b1 b2 v2);Intros;Apply H2;Left;Auto|Exists x;Elim (lem_union_propertie r1 r2 (couple x v2));Intros;Apply H2;Left;Auto]).

(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r2)) b2 v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Split;[Elim (lem_union_propertie b1 b2 v2);Intros;Apply H2;Right;Auto|Exists x;Elim (lem_union_propertie r1 r2 (couple x v2));Intros;Apply H2;Right;Auto]).

Save.
(* End of proof of lem_rel_union_Img                                       *)

(***************************************************************************)
(* Des evidences                                                           *)
(***************************************************************************)
Lemma lem_rel_union_prop :
  (r1,a1,b1:E)(r2,a2,b2:E)
  (rRel1:(rel_sig r1 a1 b1))
  (rRel2:(rel_sig r2 a2 b2))
  (c:E)
    (In c (rel_union r1 a1 b1 r2 a2 b2 rRel1 rRel2))
    <-> 
    ((In c r1) \/ (In c r2))
.
(* Proof of lem_rel_union_prop                                             *)
(Intros;Unfold iff;Unfold rel_union;Split;Intros).
(Elim (lem_union_propertie r1 r2 c);Intros;Exact (H0 H)).

(Elim (lem_union_propertie r1 r2 c);Intros;Exact (H1 H)).

Save.
(* End of proof of lem_rel_union_prop                                      *)

(***************************************************************************)
(* Intersection de deux relations                                          *)
(***************************************************************************)
Definition rel_inter := 
  [r1,a1,b1:E][r2,a2,b2:E][rRel1:(rel_sig r1 a1 b1)][rRel2:(rel_sig r2 a2 b2)]
    (inter r1 r2)
.

Lemma lem_rel_inter_rel_sig :
  (r1,a1,b1:E)(r2,a2,b2:E)(rRel1:(rel_sig r1 a1 b1))(rRel2:(rel_sig r2 a2 b2))
    (rel_sig 
      (rel_inter r1 a1 b1 r2 a2 b2 rRel1 rRel2)
      (inter a1 a2) (inter b1 b2)
    )
.
(* Proof of lem_rel_inter_rel_sig                                          *)
(Unfold rel_sig;Unfold inc;Unfold rel_inter;Intros).
(Elim (lem_inter_propertie r1 r2 v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros).
(Elim (lem_cartesian_propertie (inter a1 a2) (inter b1 b2) v0);Intros;Apply H2;Clear H1 H2;Intros).
(Generalize (rRel1 v0 H) (rRel2 v0 H0);Clear rRel1 rRel2 H H0;Intros).
(Elim (lem_cartesian_propertie a1 b1 v0);Intros;Generalize (H1 H);Clear H H1 H2;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_cartesian_propertie a2 b2 v0);Intros;Generalize (H3 H0);Clear H0 H3 H4;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H3;Clear H3;Intros).
(Generalize H4;Rewrite -> H2;Clear H2 H4;Intros).
(Elim (lem_couple_propertie x x1 x0 x2);Intros;Generalize (H2 H4);Clear H2 H4 H5;Intros;Elim H2;Clear H2;Intros;Generalize H0 H3;Rewrite <- H2;Rewrite <- H4;Clear H0 H3 H2 H4;Clear x1 x2;Intros).
(Exists x;Exists x0;Split;[Elim (lem_inter_propertie a1 a2 x);Intros;Apply H4;Split;Auto|Split;[Elim (lem_inter_propertie b1 b2 x0);Intros;Apply H4;Split;Auto|Reflexivity]]).

Save.
(* End of proof of lem_rel_inter_rel_sig                                   *)

(***************************************************************************)
(* Le domaine de l'intersection de deux relations est inclus dans          *)
(* l'intersection des deux domaines.                                       *)
(***************************************************************************)
Lemma lem_rel_inter_dom :
  (r1,a1,b1:E)(r2,a2,b2:E)
  (rRel1:(rel_sig r1 a1 b1))
  (rRel2:(rel_sig r2 a2 b2))
  (inc
    (rdom
      (rel_inter r1 a1 b1 r2 a2 b2 rRel1 rRel2)
      (inter a1 a2) (inter b1 b2)
      (lem_rel_inter_rel_sig r1 a1 b1 r2 a2 b2 rRel1 rRel2)
    )
    (inter (rdom r1 a1 b1 rRel1) (rdom r2 a2 b2 rRel2))
  )
.
(* Proof of lem_rel_inter_dom                                              *)
(Unfold inc;Intros).
Unfold rdom in H.
(Elim (axs_comprehension [x:E](Ex[y:E](In (couple x y) (rel_inter r1 a1 b1 r2 a2 b2 rRel1 rRel2))) (inter a1 a2) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_inter_propertie r1 r2 (couple v0 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros;Elim H0;Clear H0;Intros).
(Generalize (lem_rel_and_rdom r1 a1 b1 rRel1 v0 x H0) (lem_rel_and_rdom r2 a2 b2 rRel2 v0 x H1);Clear H0 H1;Intros).
(Elim (lem_inter_propertie (rdom r1 a1 b1 rRel1) (rdom r2 a2 b2 rRel2) v0);Intros;Apply H3;Split;Auto).

Save.
(* End of proof of lem_rel_inter_dom                                       *)

(***************************************************************************)
(* L'image de l'intersection de deux relations est incluse dans            *)
(* l'intersection des deux images.                                         *)
(***************************************************************************)
Lemma lem_rel_inter_Img :
  (r1,a1,b1:E)(r2,a2,b2:E)
  (rRel1:(rel_sig r1 a1 b1))
  (rRel2:(rel_sig r2 a2 b2))
  (inc
    (rImg
      (rel_inter r1 a1 b1 r2 a2 b2 rRel1 rRel2)
      (inter a1 a2) (inter b1 b2)
      (lem_rel_inter_rel_sig r1 a1 b1 r2 a2 b2 rRel1 rRel2)
    )
    (inter (rImg r1 a1 b1 rRel1) (rImg r2 a2 b2 rRel2))
  )
.
(* Proof of lem_rel_inter_Img                                              *)
(Unfold inc;Intros).
Unfold rImg in H.
(Elim (axs_comprehension [y:E](Ex[x:E](In (couple x y) (rel_inter r1 a1 b1 r2 a2 b2 rRel1 rRel2))) (inter b1 b2) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_inter_propertie r1 r2 (couple x v0));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros;Elim H0;Clear H0;Intros).
(Generalize (lem_rel_and_rImg r1 a1 b1 rRel1 x v0 H0) (lem_rel_and_rImg r2 a2 b2 rRel2 x v0 H1);Clear H0 H1;Intros;Elim (lem_inter_propertie (rImg r1 a1 b1 rRel1) (rImg r2 a2 b2 rRel2) v0);Intros;Apply H3;Split;Auto).

Save.
(* End of proof of lem_rel_inter_Img                                       *)

(***************************************************************************)
(* Une evidence                                                            *)
(***************************************************************************)
Lemma lem_rel_inter_prop :
  (r1,a1,b1:E)(r2,a2,b2:E)
  (rRel1:(rel_sig r1 a1 b1))
  (rRel2:(rel_sig r2 a2 b2))
  (c:E)
    (In c (rel_inter r1 a1 b1 r2 a2 b2 rRel1 rRel2))
    <->
    ((In c r1) /\ (In c r2))
.
(* Proof of lem_rel_inter_prop                                             *)
(Unfold rel_inter;Unfold iff;Intros;Split;Intros).
(Elim (lem_inter_propertie r1 r2 c);Intros;Generalize (H0 H);Clear H H0;Intros;Auto).

(Elim (lem_inter_propertie r1 r2 c);Intros;Apply H1;Auto).

Save.
(* End of proof of lem_rel_inter_prop                                      *)

(***************************************************************************)
(* Relation inverse                                                        *)
(***************************************************************************)
Definition rel_inverse := [r,a,b:E][rRel:(rel_sig r a b)]
  (subset
    [c:E](Ex[x:E](Ex[y:E]((In (couple x y) r) /\ (c=(couple y x)))))
    (cartesien b a)
  )
.

Lemma lem_rel_inv_is_rel_sig :
  (r,a,b:E)(rRel:(rel_sig r a b))
    (rel_sig (rel_inverse r a b rRel) b a)
.
(* Proof of lem_rel_inv_is_rel_sig                                         *)
(Unfold rel_sig;Unfold inc;Unfold rel_inverse;Intros).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex [y:E](and (In (couple x y) r) (eq ? c (couple y x))))) (cartesien b a) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_rel_inv_is_rel_sig                                  *)

(***************************************************************************)
(* Le domaine de la relation inverse est l'image de la relation initiale   *)
(***************************************************************************)
Lemma lem_rel_inv_dom :
  (r,a,b:E)(rRel:(rel_sig r a b))
    (rdom (rel_inverse r a b rRel) b a (lem_rel_inv_is_rel_sig r a b rRel))
    =
    (rImg r a b rRel)
.
(* Proof of lem_rel_inv_dom                                                *)
(Unfold rdom;Unfold rImg;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (rel_inverse r a b rRel))) b v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros).
(Elim H0;Clear H0;Unfold rel_inverse;Intros).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) r) (eq ? c (couple y x0))))) (cartesien b a) (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie v2 x1 x x0);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros).
(Generalize H1;Rewrite <- H2;Rewrite <- H3;Clear H1 H2 H3;Clear x0 x1;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) b v2);Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Auto|Exists x;Auto]).

(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r)) b v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (rel_inverse r a b rRel))) b v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Auto|Unfold rel_inverse;Exists x]).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) r) (eq ? c (couple y x0))))) (cartesien b a) (couple v2 x));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists x;Exists v2;Split;[Auto|Reflexivity]]).
(Generalize (rRel (couple x v2) H0);Intros).
(Elim (lem_cartesian_propertie a b (couple x v2));Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x0 v2 x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H1 H2;Rewrite <- H3;Rewrite <- H4;Clear H1 H2 H3 H4;Clear x0 x1;Intros).
(Elim (lem_cartesian_propertie b a (couple v2 x));Intros;Apply H4;Clear H3 H4;Intros).
(Exists v2;Exists x;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_rel_inv_dom                                         *)

(***************************************************************************)
(* L'image de la relation inverse est le domaine de la relation initiale   *)
(***************************************************************************)
Lemma lem_rel_inv_Img :
  (r,a,b:E)(rRel:(rel_sig r a b))
    (rImg (rel_inverse r a b rRel) b a (lem_rel_inv_is_rel_sig r a b rRel))
    =
    (rdom r a b rRel)
.
(* Proof of lem_rel_inv_Img                                                *)
(Unfold rImg;Unfold rdom;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) (rel_inverse r a b rRel))) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Unfold rel_inverse;Intros).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) r) (eq ? c (couple y x0))))) (cartesien b a) (couple x v2));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x1 v2 x0);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros;Generalize H1;Rewrite <- H2;Rewrite <- H3;Clear H1 H2 H3;Clear x0 x1;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a v2);Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Auto|Exists x;Auto]).

(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r)) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Generalize (rRel (couple v2 x) H0);Intros).
(Elim (lem_cartesian_propertie a b (couple v2 x));Intros;Generalize (H2 H1);Clear H2 H3;Intros).
(Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H3;Clear H3;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H5 H4);Clear H4 H5 H6;Intros;Elim H4;Clear H4;Intros;Generalize H2 H3;Rewrite <- H4;Rewrite <- H5;Clear H2 H3 H4 H5;Clear x0 x1;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) (rel_inverse r a b rRel))) a v2);Intros;Apply H5;Clear H4 H5;Intros).
(Split;[Auto|Exists x;Unfold rel_inverse]).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex [y:E](and (In (couple x0 y) r) (eq ? c (couple y x0))))) (cartesien b a) (couple x v2));Intros;Apply H5;Clear H4 H5;Intros).
(Split;[Elim (lem_cartesian_propertie b a (couple x v2));Intros;Apply H5;Exists x;Exists v2;Split;[Auto|Split;[Auto|Reflexivity]]|Exists v2;Exists x;Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_rel_inv_Img                                         *)

(***************************************************************************)
(* Produit de relations                                                    *)
(***************************************************************************)
Definition rel_prod :=
  [r1,r2,a:E][rRel1:(rel_sig r1 a a)][rRel2:(rel_sig r2 a a)]
  (subset
    [c:E](Ex[x:E](Ex[y:E](Ex[z:E](
      (In (couple x y) r1)
      /\ 
      (In (couple y z) r2)
      /\ 
      (c=(couple x z))
    ))))
    (cartesien a a)
  )
.

Lemma lem_rel_prod_rel_sig :
  (r1,r2,a:E)(rRel1:(rel_sig r1 a a))(rRel2:(rel_sig r2 a a))
  (rel_sig (rel_prod r1 r2 a rRel1 rRel2) a a)
.
(* Proof of lem_rel_prod_rel_sig                                           *)
(Unfold rel_sig;Unfold inc;Unfold rel_prod;Intros).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](Ex[z:E](and (In (couple x y) r1) (and (In (couple y z) r2) (eq ? c (couple x z))))))) (cartesien a a) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_rel_prod_rel_sig                                    *)

(***************************************************************************)
(* Le domaine de r1.r2 est inclus dans le domaine de r1                    *)
(***************************************************************************)
Lemma lem_rel_prod_dom :
  (r1,r2,a:E)(rRel1:(rel_sig r1 a a))(rRel2:(rel_sig r2 a a))
  (inc
    (rdom
      (rel_prod r1 r2 a rRel1 rRel2) a a
      (lem_rel_prod_rel_sig r1 r2 a rRel1 rRel2)
    )
    (rdom r1 a a rRel1)
  )
.
(* Proof of lem_rel_prod_dom                                               *)
(Unfold inc;Unfold rdom;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (rel_prod r1 r2 a rRel1 rRel2))) a v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Unfold rel_prod;Intros).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex[y:E](Ex[z:E](and (In (couple x0 y) r1) (and (In (couple y z) r2) (eq ? c (couple x0 z))))))) (cartesien a a) (couple v0 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie v0 x0 x x2);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H1 H2;Rewrite <- H3;Rewrite <- H4;Clear H1 H2 H3 H4;Clear x0 x2;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) r1)) a v0);Intros;Apply H4;Clear H3 H4;Intros).
(Split;[Auto|Exists x1;Auto]).

Save.
(* End of proof of lem_rel_prod_dom                                        *)

(***************************************************************************)
(* L'image de r1.r2 est incluse dans l'image de r2                         *)
(***************************************************************************)
Lemma lem_rel_prod_Img :
  (r1,r2,a:E)(rRel1:(rel_sig r1 a a))(rRel2:(rel_sig r2 a a))
  (inc
    (rImg
      (rel_prod r1 r2 a rRel1 rRel2) a a
      (lem_rel_prod_rel_sig r1 r2 a rRel1 rRel2)
    )
    (rImg r2 a a rRel2)
  )
.
(* Proof of lem_rel_prod_Img                                               *)
(Unfold rImg;Unfold inc;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) (rel_prod r1 r2 a rRel1 rRel2))) a v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Unfold rel_prod;Intros).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex[y:E](Ex[z:E](and (In (couple x0 y) r1) (and (In (couple y z) r2) (eq ? c (couple x0 z))))))) (cartesien a a) (couple x v0));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x0 v0 x2);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H1 H2;Rewrite <- H3;Rewrite <- H4;Clear H1 H2 H3 H4;Clear x0 x2;Intros).
(Elim (axs_comprehension [y:E](Ex [x:E](In (couple x y) r2)) a v0);Intros;Apply H4;Clear H3 H4;Intros).
(Split;[Auto|Exists x1;Auto]).

Save.
(* End of proof of lem_rel_prod_Img                                        *)

(***************************************************************************)
(* Le produit est associatif                                               *)
(***************************************************************************)
Lemma lem_rel_prod_assoc :
  (r1,r2,r3,a:E)
  (rRel1:(rel_sig r1 a a))
  (rRel2:(rel_sig r2 a a))
  (rRel3:(rel_sig r3 a a))
  (rel_prod 
    (rel_prod r1 r2 a rRel1 rRel2) r3 a
    (lem_rel_prod_rel_sig r1 r2 a rRel1 rRel2) rRel3
  )
  =
  (rel_prod
    r1 (rel_prod r2 r3 a rRel2 rRel3) a
    rRel1 (lem_rel_prod_rel_sig r2 r3 a rRel2 rRel3)
  )
.
(* Proof of lem_rel_prod_assoc                                             *)
(Unfold 1 3 rel_prod;Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](Ex[z:E](and (In (couple x y) (rel_prod r1 r2 a rRel1 rRel2)) (and (In (couple y z) r3) (eq ? c (couple x z))))))) (cartesien a a) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
Unfold rel_prod in H0.
(Elim (axs_comprehension [c:E] (Ex[x1:E](Ex[y:E](Ex[z:E](and (In (couple x1 y) r1) (and (In (couple y z) r2) (eq ? c (couple x1 z))))))) (cartesien a a) (couple x x0));Intros;Generalize (H3 H0);Clear H0 H3 H4;Intros).
(Elim H0;Clear H0;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_couple_propertie x x2 x0 x4);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros;Elim H5;Clear H5;Intros;Generalize H3 H4;Rewrite <- H5;Rewrite <- H6;Clear H3 H4 H5 H6;Clear x2 x4;Intros).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](Ex[z:E] (and (In (couple x y) r1) (and (In (couple y z) (rel_prod r2 r3 a rRel2 rRel3)) (eq ? c (couple x z))))))) (cartesien a a) v2);Intros;Apply H6;Clear H5 H6;Intros).
(Split;[Auto|Exists x;Exists x3;Exists x1;Split;[Auto|Unfold rel_prod;Split;[Idtac|Auto]]]).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](Ex[z:E] (and (In (couple x y) r2) (and (In (couple y z) r3) (eq ? c (couple x z))))))) (cartesien a a) (couple x3 x1));Intros;Apply H6;Clear H5 H6;Intros).
(Split;[Idtac|Exists x3;Exists x0;Exists x1;Split;[Auto|Split;[Auto|Reflexivity]]]).
(Generalize (rRel3 (couple x0 x1) H1) (rRel1 (couple x x3) H3);Clear H H0 H1 H2 H3 H4;Intros).
(Elim (lem_cartesian_propertie a a (couple x0 x1));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x0 x2 x1 x4);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros;Generalize H H1;Rewrite <- H2;Rewrite <- H3;Clear H H1 H2 H3;Clear x2 x4;Intros).
(Elim (lem_cartesian_propertie a a (couple x x3));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x2 x3 x4);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H0 H2;Rewrite <- H3;Rewrite <- H4;Clear H0 H2 H3 H4;Clear x2 x4;Intros).
(Elim (lem_cartesian_propertie a a (couple x3 x1));Intros;Apply H4;Clear H3 H4;Exists x3;Exists x1;Split;[Auto|Split;[Auto|Reflexivity]]).

(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](Ex[z:E](and (In (couple x y) r1) (and (In (couple y z) (rel_prod r2 r3 a rRel2 rRel3)) (eq ? c (couple x z))))))) (cartesien a a) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
Unfold rel_prod in H1.
(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](Ex[z:E] (and (In (couple x y) r2) (and (In (couple y z) r3) (eq ? c (couple x z))))))) (cartesien a a) (couple x0 x1));Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_couple_propertie x0 x2 x1 x4);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros;Elim H5;Clear H5;Intros;Generalize H3 H4;Rewrite <- H5;Rewrite <- H6;Clear H3 H4 H5 H6;Clear x2 x4;Intros).
(Elim (axs_comprehension [c:E](Ex[x:E](Ex[y:E](Ex[z:E](and (In (couple x y) (rel_prod r1 r2 a rRel1 rRel2)) (and (In (couple y z) r3) (eq ? c (couple x z))))))) (cartesien a a) v2);Intros;Apply H6;Clear H5 H6;Intros).
(Split;[Auto|Exists x;Exists x3;Exists x1;Split;[Unfold rel_prod|Split;Auto]]).
(Elim (axs_comprehension [c:E](Ex[x0:E](Ex[y:E](Ex[z:E](and (In (couple x0 y) r1) (and (In (couple y z) r2) (eq ? c (couple x0 z))))))) (cartesien a a) (couple x x3));Intros;Apply H6;Clear H5 H6;Intros).
(Split;[Idtac|Exists x;Exists x0;Exists x3;Split;[Auto|Split;[Auto|Reflexivity]]]).
(Generalize (rRel1 (couple x x0) H0) (rRel2 (couple x0 x3) H3);Clear H H0 H1 H2 H3 H4;Intros).
(Elim (lem_cartesian_propertie a a (couple x x0));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x2 x0 x4);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros;Generalize H H1;Rewrite <- H2;Rewrite <- H3;Clear H H1 H2 H3;Clear x2 x4;Intros).
(Elim (lem_cartesian_propertie a a (couple x0 x3));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x0 x2 x3 x4);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros;Elim H3;Clear H3;Intros;Generalize H0 H2;Rewrite <- H3;Rewrite <- H4;Clear H0 H2 H3 H4;Clear x2 x4;Intros).
(Elim (lem_cartesian_propertie a a (couple x x3));Intros;Apply H4;Exists x;Exists x3;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_rel_prod_assoc                                      *)

(***************************************************************************)
(* L'identite est un element neutre pour le produit                        *)
(***************************************************************************)

(***************************************************************************)
(***************************************************************************)
(* Relations d'equivalence                                                 *)
(***************************************************************************)
(***************************************************************************)
Definition equivalence := [r:E][a:E][rRel:(rel_sig r a a)]
  (reflexivity r a rRel)
  /\ 
  (symmetry r a rRel)
  /\ 
  (transitivity r a rRel)
.

(***************************************************************************)
(* Classe d'equivalence                                                    *)
(***************************************************************************)
Definition equivClass := 
  [r:E][a:E][rRel:(rel_sig r a a)][rEqv:(equivalence r a rRel)][x:E]
    (subset [y:E](In (couple x y) r) a)
.

(***************************************************************************)
(* Si x est en relation avec y alors ils ont meme classe d'equivalence     *)
(***************************************************************************)
Lemma lem_equivClass_eq :
  (r:E)(a:E)(rRel:(rel_sig r a a))(rEqv:(equivalence r a rRel))(x:E)(y:E)
    (In (couple x y) r)
    ->
    (equivClass r a rRel rEqv x)=(equivClass r a rRel rEqv y)
.
(* Proof of lem_equivClass_eq                                              *)
(Intros;Unfold equivClass;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [y:E](In (couple x y) r) a v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
(Elim (axs_comprehension [y0:E](In (couple y y0) r) a v2);Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Auto|Idtac]).
Unfold equivalence in rEqv.
(Elim rEqv;Clear rEqv;Intros;Elim H3;Clear H3;Intros).
Unfold symmetry in H3.
(Generalize (H3 x y H);Clear H H3;Intros).
Unfold transitivity in H4.
Exact (H4 y x v2 H H1).

(Elim (axs_comprehension [y0:E](In (couple y y0) r) a v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros).
(Elim (axs_comprehension [y:E](In (couple x y) r) a v2);Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Auto|Idtac]).
(Elim rEqv;Clear rEqv;Intros;Elim H3;Clear H3;Intros;Unfold transitivity in H4).
Exact (H4 x y v2 H H1).

Save.
(* End of proof of lem_equivClass_eq                                       *)

(***************************************************************************)
(* Si x et y ne sont pas en relation, l'intersection de leurs classes      *)
(* d'equivalence respectives est vide.                                     *)
(***************************************************************************)
Lemma lem_inter_equivClass_empty :
  (r:E)(a:E)(rRel:(rel_sig r a a))(rEqv:(equivalence r a rRel))(x:E)(y:E)
    ~(In (couple x y) r)
    ->
    (inter (equivClass r a rRel rEqv x) (equivClass r a rRel rEqv y))=Vide
.
(* Proof of lem_inter_equivClass_empty                                     *)
Intros.
(Apply lem_inter_is_empty;Unfold not;Intros).
(Unfold equivClass in H0;Unfold equivClass in H1).
(Elim (axs_comprehension [y:E](In (couple x y) r) a x0);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim (axs_comprehension [y0:E](In (couple y y0) r) a x0);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H0;Elim H1;Clear H0 H1;Intros;Clear H2).
(Elim rEqv;Clear rEqv;Intros).
(Elim H4;Clear H4;Unfold symmetry;Unfold transitivity;Intros).
(Generalize (H4 y x0 H1);Clear H1 H4;Intros).
(Generalize (H5 x x0 y H3 H1);Clear H1 H3 H5;Intros).
(Absurd (In (couple x y) r);Auto).

(Unfold equivClass in H0;Unfold equivClass in H1).
(Elim (axs_comprehension [y1:E](In (couple y y1) r) a y0);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim (axs_comprehension [y:E](In (couple x y) r) a y0);Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H0;Elim H1;Clear H0 H1;Intros;Clear H2).
(Elim rEqv;Clear rEqv;Unfold symmetry;Unfold transitivity;Intros;Elim H4;Clear H4;Intros).
(Generalize (H4 y y0 H3);Clear H3 H4;Intros).
(Generalize (H5 x y0 y H1 H3);Clear H1 H3 H5;Intros).
(Absurd (In (couple x y) r);Auto).

Save.
(* End of proof of lem_inter_equivClass_empty                              *)

Lemma lem_equivClass_in_parties_a :
  (r:E)(a:E)(rRel:(rel_sig r a a))(rEqv:(equivalence r a rRel))(x:E)
    (In (equivClass r a rRel rEqv x) (parties a))
.
(* Proof of lem_equivClass_in_parties_a                                    *)
Intros.
(Elim (axs_parties a (equivClass r a rRel rEqv x));Intros;Apply H0;Clear H H0;Intros).
Unfold equivClass in H.
(Elim (axs_comprehension [y:E](In (couple x y) r) a v3);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_equivClass_in_parties_a                             *)

(***************************************************************************)
(* Quotient d'un ensemble par une relation d'equivalence                   *)
(***************************************************************************)
Definition quotient :=
  [r:E][a:E][rRel:(rel_sig r a a)]
  [rEqv:(equivalence r a rRel)][rD:(rdom r a a rRel)=a]
  (subset 
    [x:E](Ex[y:E]((In y a) /\ (x=(equivClass r a rRel rEqv y))))
    (parties a)
  )
.


(***************************************************************************)
(* Toute application induit une relation d'equivalence sur son domaine     *)
(* equiv_fun sera le "noyau d'equivalence" de l'application                *)
(***************************************************************************)
Definition equiv_fun :=
  [f:E][Af:(fun f)]
    (subset 
      [x:E](Ex[a:E](Ex[b:E]((x=(couple a b)) 
      /\ ((App f Af a)=(App f Af b)))))
      (cartesien (dom f) (dom f))
    )
.

Lemma lem_fun_equiv_is_rel :
  (f:E)(Af:(fun f))
    (rel_sig (equiv_fun f Af) (dom f) (dom f))
.
(* Proof of lem_fun_equiv_is_rel                                           *)
(Unfold rel_sig;Unfold inc;Unfold equiv_fun;Intros).
(Elim (axs_comprehension [x:E](Ex[a:E](Ex[b:E](and (eq ? x (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

Save.
(* End of proof of lem_fun_equiv_is_rel                                    *)

Lemma lem_sig_fun_equiv_is_rel :
  (f:E)(a,s:E)(Afs:(fun_sig f a s))
    (rel_sig (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a a)
.
(* Proof of lem_sig_fun_equiv_is_rel                                       *)
(Unfold rel_sig;Unfold inc;Unfold equiv_fun;Intros).
(Elim (axs_comprehension [x:E](Ex[a0:E](Ex[b:E](and (eq ? x (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Elim Afs;Intros;Elim H3;Clear H3;Intros).
(Generalize H;Rewrite -> H3;Tauto).

Save.
(* End of proof of lem_sig_fun_equiv_is_rel                                *)

Lemma lem_fun_equiv_is_equiv :
  (f:E)(Af:(fun f))(equivalence (equiv_fun f Af) (dom f) (lem_fun_equiv_is_rel f Af))
.
(* Proof of lem_fun_equiv_is_equiv                                         *)
(Unfold equivalence;Intros).
(Split;[Unfold reflexivity;Intros|Split;[Unfold symmetry;Intros|Unfold transitivity;Intros]]).
Unfold equiv_fun.
(Elim (axs_comprehension [x0:E](Ex[a:E](Ex[b:E](and (eq ? x0 (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple x x));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists x;Exists x;Split;Reflexivity]).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x x));Intros;Apply H1;Clear H0 H1;Intros).
(Exists x;Exists x).
Unfold rdom in H.
(Elim (axs_comprehension [x0:E](Ex [y:E](In (couple x0 y) (equiv_fun f Af))) (dom f) x);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Split;[Auto|Split;[Auto|Reflexivity]]).

Unfold equiv_fun in H.
(Elim (axs_comprehension [x0:E](Ex[a:E](Ex[b:E](and (eq ? x0 (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple x y));Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros;Elim H0;Clear H0;Intros).
(Generalize H;Rewrite -> H0;Rewrite -> H2;Clear H H0 H2;Intros).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x0 x1));Intros;Generalize (H0 H);Clear H H0 H2;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_couple_propertie x0 x2 x1 x3);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros).
(Generalize H H0;Rewrite <- H2;Rewrite <- H3;Clear H H0 H2 H3;Intros).
Unfold equiv_fun.
(Elim (axs_comprehension [x:E](Ex[a:E](Ex[b:E](and (eq ? x (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple x1 x0));Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Idtac|Exists x1;Exists x0;Split;[Reflexivity|Symmetry;Auto]]).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x1 x0));Intros;Apply H3;Clear H2 H3;Intros).
(Exists x1;Exists x0;Split;[Auto|Split;[Auto|Reflexivity]]).

Unfold equiv_fun in H.
(Elim (axs_comprehension [x0:E](Ex[a:E](Ex[b:E](and (eq ? x0 (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple x y));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros).
(Generalize H2;Rewrite <- H1;Rewrite <- H3;Clear H1 H2 H3;Intros).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x y));Intros;Generalize (H1 H);Clear H H1 H3;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x2 y x3);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros).
(Generalize H H1;Rewrite <- H3;Rewrite <- H4;Clear H H1 H3 H4;Intros).
Unfold equiv_fun in H0.
(Elim (axs_comprehension [x:E](Ex[a:E](Ex[b:E](and (eq ? x (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple y z));Intros;Generalize (H3 H0);Clear H0 H3 H4;Intros).
(Elim H0;Clear H0;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Elim (lem_couple_propertie y x4 z x5);Intros;Generalize (H5 H3);Clear H3 H5 H6;Intros).
(Elim H3;Clear H3;Intros).
(Generalize H4;Rewrite <- H3;Rewrite <- H5;Clear H3 H4 H5;Intros).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple y z));Intros;Generalize (H3 H0);Clear H0 H3 H5;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H3;Clear H3;Intros).
(Elim (lem_couple_propertie y x6 z x7);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Generalize H0 H3;Rewrite <- H5;Rewrite <- H6;Clear H0 H3 H5 H6;Intros).
(Clear H1;Unfold equiv_fun).
(Elim (axs_comprehension [x0:E](Ex[a:E](Ex[b:E](and (eq ? x0 (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple x z));Intros;Apply H5;Clear H1 H5;Intros).
(Split;[Idtac|Exists x;Exists z;Split;[Reflexivity|Rewrite <- H4;Auto]]).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x z));Intros;Apply H5;Clear H1 H5;Intros).
(Exists x;Exists z;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_fun_equiv_is_equiv                                  *)

Lemma lem_sig_fun_equiv_is_equiv :
  (f:E)(a,s:E)(Afs:(fun_sig f a s))
    (equivalence 
      (equiv_fun f (lem_fun_sig_is_fun f a s Afs))
      a (lem_sig_fun_equiv_is_rel f a s Afs)
    )
.
(* Proof of lem_sig_fun_equiv_is_equiv                                     *)
Intros.
(Elim Afs;Intros;Elim H0;Clear H0;Intros).
(Generalize (lem_fun_equiv_is_equiv f H);Intros).
Unfold equivalence.
(Elim H2;Unfold reflexivity symmetry transitivity;Clear H2;Intros;Elim H3;Clear H3;Intros).
(Split;[Intros|Split;Intros]).
Unfold equiv_fun.
(Elim (axs_comprehension [x0:E](Ex[a0:E](Ex[b:E](and (eq ? x0 (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple x x));Intros;Apply H7;Clear H6 H7;Intros).
(Split;[Idtac|Exists x;Exists x;Split;Reflexivity]).
Unfold rdom in H5.
(Elim (axs_comprehension [x0:E](Ex[y:E](In (couple x0 y) (equiv_fun f (lem_fun_sig_is_fun f a s Afs)))) a x);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x x));Intros;Apply H8;Clear H7 H8;Intros).
(Exists x;Exists x;Rewrite -> H0;Split;[Auto|Split;[Auto|Reflexivity]]).

Unfold equiv_fun in H5.
(Elim (axs_comprehension [x0:E](Ex[a0:E](Ex[b:E](and (eq ? x0 (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple x y));Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros;Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H8 H6);Clear H6 H8 H9;Intros).
(Elim H6;Clear H6;Intros).
(Generalize H7;Rewrite <- H6;Rewrite <- H8;Clear H6 H7 H8;Intros).
Unfold equiv_fun.
(Elim (axs_comprehension [x0:E](Ex[a0:E](Ex[b:E](and (eq ? x0 (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple y x));Intros;Apply H8;Clear H6 H8;Intros).
(Split;[Idtac|Exists y;Exists x;Split;[Reflexivity|Symmetry;Auto]]).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x y));Intros;Generalize (H6 H5);Clear H5 H6 H8;Intros).
(Elim H5;Clear H5;Intros;Elim H5;Clear H5;Intros;Elim H5;Clear H5;Intros;Elim H6;Clear H6;Intros).
(Elim (lem_couple_propertie x x2 y x3);Intros;Generalize (H9 H8);Clear H8 H9 H10;Intros;Elim H8;Clear H8;Intros).
(Rewrite -> H8;Rewrite -> H9;Elim (lem_cartesian_propertie (dom f) (dom f) (couple x3 x2));Intros;Apply H11;Clear H10 H11;Intros).
(Exists x3;Exists x2;Split;[Auto|Split;[Auto|Reflexivity]]).

(Unfold equiv_fun in H5;Unfold equiv_fun in H6).
(Elim (axs_comprehension [x0:E](Ex[a0:E](Ex[b:E](and (eq ? x0 (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple x y));Intros;Generalize (H7 H5);Clear H5 H7 H8;Intros).
(Elim (axs_comprehension [x:E](Ex[a0:E](Ex[b:E](and (eq ? x (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple y z));Intros;Generalize (H7 H6);Clear H6 H7 H8;Intros).
(Elim H5;Clear H5;Intros;Elim H7;Clear H7;Intros;Elim H7;Clear H7;Intros;Elim H7;Clear H7;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H9 H7);Clear H7 H9 H10;Intros;Elim H7;Clear H7;Intros).
(Generalize H8;Rewrite <- H7;Rewrite <- H9;Clear H7 H8 H9;Intros).
(Elim H6;Clear H6;Intros;Elim H7;Clear H7;Intros;Elim H7;Clear H7;Intros;Elim H7;Clear H7;Intros).
(Elim (lem_couple_propertie y x2 z x3);Intros;Generalize (H10 H7);Clear H7 H10 H11;Intros;Elim H7;Clear H7;Intros).
(Generalize H9;Rewrite <- H7;Rewrite <- H10;Clear H7 H9 H10;Intros).
(Generalize H9;Rewrite <- H8;Clear H8 H9;Intros).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x y));Intros;Generalize (H7 H5);Clear H5 H7 H8;Intros).
(Elim H5;Clear H5;Intros;Elim H5;Clear H5;Intros;Elim H5;Clear H5;Intros;Elim H7;Clear H7;Intros).
(Elim (lem_couple_propertie x x4 y x5);Intros;Generalize (H10 H8);Clear H8 H10 H11;Intros;Elim H8;Clear H8;Intros).
(Generalize H5;Rewrite <- H8;Clear  H5 H7 H8 H10;Intros).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple y z));Intros;Generalize (H7 H6);Clear H6 H7 H8;Intros).
(Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros;Elim H7;Clear H7;Intros).
(Elim (lem_couple_propertie y x6 z x7);Intros;Generalize (H10 H8);Clear H8 H10 H11;Intros;Elim H8;Clear H8;Intros).
(Generalize H7;Rewrite <- H10;Clear H6 H7 H8 H10;Intros).
Unfold equiv_fun.
(Elim (axs_comprehension [x0:E](Ex[a0:E](Ex[b:E](and (eq ? x0 (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple x z));Intros;Apply H8;Clear H6 H8;Intros).
(Split;[Idtac|Exists x;Exists z;Split;[Reflexivity|Auto]]).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x z));Intros;Apply H8;Clear H6 H8;Intros).
(Exists x;Exists z;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_sig_fun_equiv_is_equiv                              *)

Lemma lem_fun_equiv_dom :
  (f:E)(Af:(fun f))
    (rdom 
      (equiv_fun f Af) (dom f) (dom f) 
      (lem_fun_equiv_is_rel f Af)
    ) = (dom f)
.
(* Proof of lem_fun_equiv_dom                                              *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold rdom in H.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (equiv_fun f Af))) (dom f) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

Unfold rdom.
(Elim (axs_comprehension [x:E](Ex [y:E](In (couple x y) (equiv_fun f Af))) (dom f) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Auto|Exists v2;Unfold equiv_fun]).
(Elim (axs_comprehension [x:E](Ex[a:E](Ex[b:E](and (eq ? x (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple v2 v2));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists v2;Exists v2;Split;Reflexivity]).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple v2 v2));Intros;Apply H1;Clear H0 H1;Intros).
(Exists v2;Exists v2;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_fun_equiv_dom                                       *)

Lemma lem_sig_fun_equiv_dom :
  (f:E)(a,s:E)(Afs:(fun_sig f a s))
    (rdom 
      (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a a
      (lem_sig_fun_equiv_is_rel f a s Afs)
    ) = a
.
(* Proof of lem_sig_fun_equiv_dom                                          *)
(Intros;Unfold rdom;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim (axs_comprehension [x:E](Ex[y:E](In (couple x y) (equiv_fun f (lem_fun_sig_is_fun f a s Afs)))) a v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros;Elim H;Clear H;Intros;Auto).

(Elim (axs_comprehension [x:E](Ex[y:E](In (couple x y) (equiv_fun f (lem_fun_sig_is_fun f a s Afs)))) a v2);Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Auto|Idtac]).
(Exists v2;Unfold equiv_fun).
(Elim (axs_comprehension [x:E](Ex[a0:E](Ex[b:E](and (eq ? x (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple v2 v2));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists v2;Exists v2;Split;Reflexivity]).
(Elim Afs;Intros;Elim H1;Clear H1;Intros;Rewrite -> H1).
(Elim (lem_cartesian_propertie a a (couple v2 v2));Intros;Apply H4;Clear H3 H4;Intros).
(Exists v2;Exists v2;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_sig_fun_equiv_dom                                   *)

Lemma lem_fun_equiv_prop :
  (f:E)(Af:(fun f))(x,y:E)
    (In (couple x y) (equiv_fun f Af))
    ->
    (App f Af x)=(App f Af y)
.
(* Proof of lem_fun_equiv_prop                                             *)
(Unfold equiv_fun;Intros).
(Elim (axs_comprehension [x0:E](Ex[a:E](Ex[b:E](and (eq ? x0 (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple x y));Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros).
(Rewrite -> H0;Rewrite -> H2;Auto).

Save.
(* End of proof of lem_fun_equiv_prop                                      *)

Lemma lem_equiv_fun_make :
  (f:E)(Af:(fun f))(x,y:E)
    (In x (dom f))
    ->
    (In y (dom f))
    ->
    (App f Af x)=(App f Af y)
    ->
    (In (couple x y) (equiv_fun f Af))
.
(* Proof of lem_equiv_fun_make                                             *)
(Intros;Unfold equiv_fun).
(Elim (axs_comprehension [x0:E](Ex[a:E](Ex[b:E](and (eq ? x0 (couple a b)) (eq ? (App f Af a) (App f Af b))))) (cartesien (dom f) (dom f)) (couple x y));Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Idtac|Exists x;Exists y;Split;[Reflexivity|Auto]]).
(Elim (lem_cartesian_propertie (dom f) (dom f) (couple x y));Intros;Apply H3;Clear H2 H3;Intros).
(Exists x;Exists y;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_equiv_fun_make                                      *)

Lemma lem_sig_equiv_fun_make :
  (f,a,s:E)(Afs:(fun_sig f a s))(x,y:E)
    (In x a)
    ->
    (In y a)
    ->
    (App f (lem_fun_sig_is_fun f a s Afs) x)
    =(App f (lem_fun_sig_is_fun f a s Afs) y)
    ->
    (In (couple x y) (equiv_fun f (lem_fun_sig_is_fun f a s Afs)))
.
(* Proof of lem_sig_equiv_fun_make                                         *)
Intros.
(Elim Afs;Intros;Elim H3;Clear H3;Intros).
Unfold equiv_fun.
(Elim (axs_comprehension [x0:E](Ex[a0:E](Ex[b:E](and (eq ? x0 (couple a0 b)) (eq ? (App f (lem_fun_sig_is_fun f a s Afs) a0) (App f (lem_fun_sig_is_fun f a s Afs) b))))) (cartesien (dom f) (dom f)) (couple x y));Intros;Apply H6;Clear H5 H6;Intros).
(Split;[Idtac|Exists x;Exists y;Split;[Reflexivity|Auto]]).
Rewrite -> H3.
(Elim (lem_cartesian_propertie a a (couple x y));Intros;Apply H6;Clear H5 H6;Intros).
(Exists x;Exists y;Split;[Auto|Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_sig_equiv_fun_make                                  *)

(***************************************************************************)
(* Projection sur le quotient                                              *)
(***************************************************************************)
Definition projQuotient :=
  [r:E][a:E][rRel:(rel_sig r a a)]
  [rEqv:(equivalence r a rRel)][rD:(rdom r a a rRel)=a]
  (subset 
  [x:E](Ex[z:E](Ex[t:E]((x=(couple z t)) /\ (t=(equivClass r a rRel rEqv z)))))
  (cartesien a (quotient r a rRel rEqv rD))
  )
.

(***************************************************************************)
(* projQuotient est une application                                        *)
(***************************************************************************)
Lemma lem_projQuotient_is_fun :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (fun (projQuotient r a rRel rEqv rD))
.
(* Proof of lem_projQuotient_is_fun                                        *)
Unfold fun;Intros.
(Split;Intros).
(Exists a;Exists (quotient r a rRel rEqv rD);Unfold inc;Intros).
Unfold projQuotient in H.
(Elim (axs_comprehension [x:E](Ex[z:E](Ex[t:E](and (eq ? x (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Clear H0;Auto).

Unfold projQuotient in H.
(Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x y));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
Unfold projQuotient in H0.
(Elim (axs_comprehension [x0:E](Ex[z0:E](Ex[t:E](and (eq ? x0 (couple z0 t)) (eq ? t (equivClass r a rRel rEqv z0))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x z));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros).
(Generalize H2;Rewrite <- H1;Rewrite <- H3;Clear H1 H2 H3;Intros).
(Rewrite -> H2;Clear H H2).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_couple_propertie x x2 z x3);Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros).
(Generalize H1;Rewrite <- H0;Rewrite <- H2;Clear H0 H1 H2;Intros).
(Rewrite -> H1;Reflexivity).

Save.
(* End of proof of lem_projQuotient_is_fun                                 *)

(***************************************************************************)
(* Le domaine de projQuotient est a                                        *)
(***************************************************************************)
Lemma lem_projQuotient_dom :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (dom (projQuotient r a rRel rEqv rD))=a
.
(* Proof of lem_projQuotient_dom                                           *)
Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros.
Unfold dom in H.
Elim (axs_comprehension [v0:E](fun (projQuotient r a rRel rEqv rD))->(Ex [v1:E](In (couple v0 v1) (projQuotient r a rRel rEqv rD))) (reunion (reunion (projQuotient r a rRel rEqv rD))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros.
Elim H;Clear H;Intros.
Generalize (H0 (lem_projQuotient_is_fun r a rRel rEqv rD));Clear H0;Intros.
Elim H0;Clear H0;Intros.
Unfold projQuotient in H0.
Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros.
Elim H0;Clear H0;Intros.
Clear H1;Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros.
Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros.
Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros;Elim H2;Clear H2;Intros.
Rewrite -> H2;Auto.

Unfold dom.
Elim (axs_comprehension [v0:E](fun (projQuotient r a rRel rEqv rD))->(Ex [v1:E](In (couple v0 v1) (projQuotient r a rRel rEqv rD))) (reunion (reunion (projQuotient r a rRel rEqv rD))) v2);Intros;Apply H1;Clear H0 H1;Intros.
Split;Intros.
Elim (axs_reunion (reunion (projQuotient r a rRel rEqv rD)) v2);Intros;Apply H1;Clear H0 H1;Intros.
Exists (singleton v2);Split;[Idtac|Apply lem_x_in_sing_x].
Elim (axs_reunion (projQuotient r a rRel rEqv rD) (singleton v2));Intros;Apply H1;Clear H0 H1;Intros.
Exists (couple v2 (equivClass r a rRel rEqv v2));Split;[Idtac|Elim (axs_paire (singleton v2) (paire v2 (equivClass r a rRel rEqv v2))
 (singleton v2));Intros;Apply H1;Left;Reflexivity].
Unfold projQuotient.
Elim (axs_comprehension [x:E](Ex[z:E](Ex[t:E](and (eq ? x (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple v2 (equivClass r a rRel rEqv v2)));Intros;Apply H1;Clear H0 H1;Intros.
Split;[Idtac|Exists v2;Exists (equivClass r a rRel rEqv v2);Split;Reflexivity].
Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple v2 (equivClass r a rRel rEqv v2)));Intros;Apply H1;Clear H0 H1;Intros.
Exists v2;Exists (equivClass r a rRel rEqv v2);Split;[Auto|Split;[Idtac|Reflexivity]].
Unfold quotient.
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) (equivClass r a rRel rEqv v2));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Exact (lem_equivClass_in_parties_a r a rRel rEqv v2)|Exists v2;Split;[Auto|Reflexivity]]).

Exists (equivClass r a rRel rEqv v2);Unfold projQuotient.
Elim (axs_comprehension [x:E](Ex[z:E](Ex[t:E](and (eq ? x (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple v2 (equivClass r a rRel rEqv v2)));Intros;Apply H2;Clear H1 H2;Intros.
Split;[Idtac|Exists v2;Exists (equivClass r a rRel rEqv v2);Split;Reflexivity].
Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple v2 (equivClass r a rRel rEqv v2)));Intros;Apply H2;Clear H1 H2;Intros.
Exists v2;Exists (equivClass r a rRel rEqv v2);Split;[Auto|Split;[Idtac|Reflexivity]].
Unfold quotient.
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) (equivClass r a rRel rEqv v2));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Exact (lem_equivClass_in_parties_a r a rRel rEqv v2)|Exists v2;Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_projQuotient_dom                                    *)

(***************************************************************************)
(* L'image de projQuotient est l'ensemble quotient                         *)
(***************************************************************************)
Lemma lem_projQuotient_img :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (Img (projQuotient r a rRel rEqv rD))=(quotient r a rRel rEqv rD)
.
(* Proof of lem_projQuotient_img                                           *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold Img in H.
(Elim (axs_comprehension [v1:E](fun (projQuotient r a rRel rEqv rD))->(Ex[v0:E](In (couple v0 v1) (projQuotient r a rRel rEqv rD))) (reunion (reunion (projQuotient r a rRel rEqv rD))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Generalize (H0 (lem_projQuotient_is_fun r a rRel rEqv rD));Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold projQuotient in H0.
(Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x v2));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Clear H1).
(Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple x v2));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x x0 v2 x1);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros;Generalize H1;Rewrite -> H3;Tauto).

(Generalize H;Unfold quotient;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
Unfold Img.
(Elim (axs_comprehension [v1:E](fun (projQuotient r a rRel rEqv rD))->(Ex[v0:E](In (couple v0 v1) (projQuotient r a rRel rEqv rD)))
        (reunion (reunion (projQuotient r a rRel rEqv rD))) v2);Intros;Apply H4;Clear H3 H4;Intros).
(Split;Intros).
(Elim (axs_reunion (reunion (projQuotient r a rRel rEqv rD)) v2);Intros;Apply H4;Clear H3 H4;Intros).
(Exists (paire x v2);Split;[Idtac|Elim (axs_paire x v2 v2);Intros;Apply H4;Right;Reflexivity]).
(Elim (axs_reunion (projQuotient r a rRel rEqv rD) (paire x v2));Intros;Apply H4;Clear H3 H4;Intros).
(Exists (couple x v2);Split;[Idtac|Elim (axs_paire (singleton x) (paire x v2) (paire x v2));Intros;Apply H4;Right;Reflexivity]).
Unfold projQuotient.
(Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x v2));Intros;Apply H4;Clear H3 H4;Intros).
(Split;[Idtac|Exists x;Exists v2;Split;[Reflexivity|Auto]]).
(Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple x v2));Intros;Apply H4;Clear H3 H4;Intros).
(Exists x;Exists v2;Split;[Auto|Split;[Idtac|Reflexivity]]).
Unfold quotient.
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) v2);Intros;Apply H4;Clear H3 H4;Intros).
(Split;[Auto|Exists x;Split;Auto]).

Exists x.
Unfold projQuotient.
(Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x v2));Intros;Apply H5;Clear H4 H5;Intros).
(Split;[Idtac|Exists x;Exists v2;Split;[Reflexivity|Auto]]).
(Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple x v2));Intros;Apply H5;Clear H4 H5;Intros).
(Exists x;Exists v2;Split;[Auto|Split;[Idtac|Reflexivity]]).
Unfold quotient.
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) v2);Intros;Apply H5;Clear H4 H5;Intros).
(Split;[Auto|Exists x;Split;Auto]).

Save.
(* End of proof of lem_projQuotient_img                                    *)

(***************************************************************************)
(* La projection sur l'ensemble quotient est une surjection                *)
(***************************************************************************)
Lemma lem_projQuotient_is_surj :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (fun_surj 
    (projQuotient r a rRel rEqv rD)
    a
    (quotient r a rRel rEqv rD)
  )
.
(* Proof of lem_projQuotient_is_surj                                       *)
(Intros;Unfold fun_surj).
(Split;[Exact (lem_projQuotient_is_fun r a rRel rEqv rD)|Split;[Exact (lem_projQuotient_dom r a rRel rEqv rD)|Exact (lem_projQuotient_img r a rRel rEqv rD)]]).

Save.
(* End of proof of lem_projQuotient_is_surj                                *)

(***************************************************************************)
(* Le resultat de l'evaluation de x par projQuotient est la classe         *)
(* d'equivalence de x.                                                     *)
(***************************************************************************)
Lemma lem_projQuotient_eval :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (x:E)(In x a)
  ->
  (App (projQuotient r a rRel rEqv rD) 
  (lem_projQuotient_is_fun r a rRel rEqv rD) x)
  = (equivClass r a rRel rEqv x)
.
(* Proof of lem_projQuotient_eval                                          *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold App in H0.
(Elim (axs_comprehension [z:E](Ex[y:E](and (In (couple x y) (projQuotient r a rRel rEqv rD)) (In z y))) (reunion (Img (projQuotient r a rRel rEqv rD))) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Generalize H0;Clear H0;Rewrite -> (lem_projQuotient_img r a rRel rEqv rD);Intros).
Unfold projQuotient in H1.
(Elim (axs_comprehension [x1:E](Ex[z:E](Ex[t:E](and (eq ? x1 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x x0));Intros;Generalize (H3 H1);Clear H1 H3 H4;Intros).
(Elim H1;Clear H1;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Elim (lem_couple_propertie x x1 x0 x2);Intros;Generalize (H5 H3);Clear H3 H5 H6;Intros;Elim H3;Clear H3;Intros).
(Generalize H2;Rewrite -> H5;Rewrite -> H4;Rewrite <- H3;Tauto).

Unfold App.
Rewrite -> (lem_projQuotient_img r a rRel rEqv rD).
(Elim (axs_comprehension [z:E](Ex[y:E](and (In (couple x y) (projQuotient r a rRel rEqv rD)) (In z y))) (reunion (quotient r a rRel rEqv rD)) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists (equivClass r a rRel rEqv x);Split;[Idtac|Auto]]).
(Elim (axs_reunion (quotient r a rRel rEqv rD) v2);Intros;Apply H2;Clear H1 H2;Intros).
(Exists (equivClass r a rRel rEqv x);Split;[Unfold quotient|Auto]).
(Elim (axs_comprehension [x0:E](Ex [y:E](and (In y a) (eq ? x0 (equivClass r a rRel rEqv y)))) (parties a) (equivClass r a rRel rEqv x));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Exact (lem_equivClass_in_parties_a r a rRel rEqv x)|Exists x;Split;[Auto|Reflexivity]]).

Unfold projQuotient.
(Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x (equivClass r a rRel rEqv x)));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Idtac|Exists x;Exists (equivClass r a rRel rEqv x);Split;[Reflexivity|Auto]]).
(Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple x (equivClass r a rRel rEqv x)));Intros;Apply H2;Clear H1 H2;Intros).
(Exists x;Exists (equivClass r a rRel rEqv x);Split;[Auto|Split;[Idtac|Reflexivity]]).
Unfold quotient.
(Elim (axs_comprehension [x0:E](Ex [y:E](and (In y a) (eq ? x0 (equivClass r a rRel rEqv y)))) (parties a) (equivClass r a rRel rEqv x));Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Exact (lem_equivClass_in_parties_a r a rRel rEqv x)|Exists x;Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_projQuotient_eval                                   *)

(***************************************************************************)
(* Le couple compose d'un element et de sa classe d'equivalence funartient *)
(* au graphe de projQuotient                                               *)
(***************************************************************************)
Lemma lem_x_equiv_in_projQ :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (x:E)(In x a)
  ->
  (In 
    (couple x (equivClass r a rRel rEqv x))
    (projQuotient r a rRel rEqv rD)
  )
.
(* Proof of lem_x_equiv_in_projQ                                           *)
Intros.
Unfold projQuotient.
(Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple x (equivClass r a rRel rEqv x)));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Idtac|Exists x;Exists (equivClass r a rRel rEqv x);Split;Reflexivity]).
(Elim (lem_cartesian_propertie a (quotient r a rRel rEqv rD) (couple x (equivClass r a rRel rEqv x)));Intros;Apply H1;Clear H0 H1;Intros).
(Exists x;Exists (equivClass r a rRel rEqv x);Split;[Auto|Split;[Idtac|Reflexivity]]).
Unfold quotient.
(Elim (axs_comprehension [x0:E](Ex [y:E](and (In y a) (eq ? x0 (equivClass r a rRel rEqv y)))) (parties a) (equivClass r a rRel rEqv x));Intros;Apply H1;Clear H0 H1;Intros).
(Split;[Exact (lem_equivClass_in_parties_a r a rRel rEqv x)|Exists x;Split;[Auto|Reflexivity]]).

Save.
(* End of proof of lem_x_equiv_in_projQ                                    *)

(***************************************************************************)
(* Encore un lemme a la XXX: x est membre de sa classe d'equivalence       *)
(***************************************************************************)
Lemma lem_x_in_equivClass_x :
  (r:E)(a:E)(rRel:(rel_sig r a a))(rEqv:(equivalence r a rRel))
  (rD:(rdom r a a rRel)=a)(x:E)
  (In x a)
  ->
  (In x (equivClass r a rRel rEqv x))
.
(* Proof of lem_x_in_equivClass_x                                          *)
(Intros;Unfold equivClass).
(Elim (axs_comprehension [y:E](In (couple x y) r) a x);Intros;Apply H1;Clear H0 H1;Split;[Auto|Idtac]).
(Elim rEqv;Clear rEqv;Intros).
Unfold reflexivity in H0.
(Generalize H ;Rewrite <- rD;Clear H;Intros).
Exact (H0 x H).

Save.
(* End of proof of lem_x_in_equivClass_x                                   *)

(***************************************************************************)
(* Le noyau d'equivalence de projQuotient est r                            *)
(***************************************************************************)
Lemma lem_projQuotient_equiv_fun :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (equiv_fun 
    (projQuotient r a rRel rEqv rD)
    (lem_projQuotient_is_fun r a rRel rEqv rD)
  )
  = r
.
(* Proof of lem_projQuotient_equiv_fun                                     *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold equiv_fun in H.
(Elim (axs_comprehension [x:E](Ex[a0:E](Ex[b:E](and (eq ? x (couple a0 b)) (eq ? (App (projQuotient r a rRel rEqv rD) (lem_projQuotient_is_fun r a rRel rEqv rD) a0) (App (projQuotient r a rRel rEqv rD) (lem_projQuotient_is_fun r a rRel rEqv rD) b))))) (cartesien (dom (projQuotient r a rRel rEqv rD)) (dom (projQuotient r a rRel rEqv rD))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Rewrite -> (lem_projQuotient_dom r a rRel rEqv rD);Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Generalize H;Rewrite -> H0;Clear H H0;Intros).
(Elim (lem_cartesian_propertie a a (couple x x0));Intros;Generalize (H0 H);Clear H H0 H2;Intros).
(Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H;Clear H;Intros;Elim H0;Clear H0;Intros).
(Elim (lem_couple_propertie x x1 x0 x2);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros).
(Generalize H H0;Rewrite <- H2;Rewrite <- H3;Clear H H0 H2 H3;Intros).
(Generalize H1;Rewrite -> (lem_projQuotient_eval r a rRel rEqv rD x H);Rewrite -> (lem_projQuotient_eval r a rRel rEqv rD x0 H0);Clear H1;Intros).
(Generalize (lem_x_in_equivClass_x r a rRel rEqv rD x0 H0);Intros).
(Generalize H2;Rewrite <- H1;Clear H1 H2;Intros).
Unfold equivClass in H2.
(Elim (axs_comprehension [y:E](In (couple x y) r) a x0);Intros;Generalize (H1 H2);Clear H1 H2 H3;Intros).
(Elim H1;Clear H1;Intros;Auto).

Unfold equiv_fun.
(Elim (axs_comprehension [x:E](Ex[a0:E](Ex[b:E](and (eq ? x (couple a0 b)) (eq ? (App (projQuotient r a rRel rEqv rD) (lem_projQuotient_is_fun r a rRel rEqv rD) a0) (App (projQuotient r a rRel rEqv rD) (lem_projQuotient_is_fun r a rRel rEqv rD) b))))) (cartesien (dom (projQuotient r a rRel rEqv rD)) (dom (projQuotient r a rRel rEqv rD))) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Elim rEqv;Intros).
(Elim H1;Clear H1;Intros).
Split.
Rewrite -> (lem_projQuotient_dom r a rRel rEqv rD).
(Unfold rel_sig in rRel;Unfold inc in rRel).
Exact (rRel v2 H).

(Generalize rRel;Intros).
(Unfold rel_sig  in rRel0;Unfold inc in rRel0).
(Generalize (rRel v2 H);Intros).
(Elim (lem_cartesian_propertie a a v2);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H4;Clear H4;Intros).
(Exists x;Exists x0;Split;[Auto|Idtac]).
Rewrite -> (lem_projQuotient_eval r a rRel0 rEqv rD x H3).
Rewrite -> (lem_projQuotient_eval r a rRel0 rEqv rD x0 H4).
(Generalize H;Rewrite -> H5;Intros).
Exact (lem_equivClass_eq r a rRel rEqv x x0 H6).

Save.
(* End of proof of lem_projQuotient_equiv_fun                              *)

(***************************************************************************)
(* D'ou la signature de projQuotient                                       *)
(***************************************************************************)
Lemma lem_projQuotient_sig :
  (r:E)(a:E)(rRel:(rel_sig r a a))(rEqv:(equivalence r a rRel))
  (rD:(rdom r a a rRel)=a)
  (fun_sig
    (projQuotient r a rRel rEqv rD)
    a (quotient r a rRel rEqv rD)
  )
.
(* Proof of lem_projQuotient_sig                                           *)
(Intros;Unfold fun_sig).
(Split;[Exact (lem_projQuotient_is_fun r a rRel rEqv rD)|Split;[Exact (lem_projQuotient_dom r a rRel rEqv rD)|Unfold inc;Intros]]).
(Generalize H;Rewrite -> (lem_projQuotient_img r a rRel rEqv rD);Tauto).

Save.
(* End of proof of lem_projQuotient_sig                                    *)

(***************************************************************************)
(***************************************************************************)
(* Theoreme 6 page 25 Algebre Tome 1 (Mac LANE & BIRKHOFF)                 *)
(* Propriete fondamentale de l'ensemble quotient                           *)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(* Si x et y ont meme classe d'equivalence, alors ils sont en relation     *)
(***************************************************************************)
Lemma lem_equivClass_is_same :
  (r:E)(a:E)(rRel:(rel_sig r a a))(rEqv:(equivalence r a rRel))
  (rD:(rdom r a a rRel)=a)(x,y:E)
  (In x a)
  ->
  (In y a)
  ->
  (equivClass r a rRel rEqv x)=(equivClass r a rRel rEqv y)
  ->
  (In (couple x y) r)
.
(* Proof of lem_equivClass_is_same                                         *)
Intros.
(Generalize (lem_x_in_equivClass_x r a rRel rEqv rD x H) (lem_x_in_equivClass_x r a rRel rEqv rD y H0);Intros).
(Generalize H3;Clear H3;Rewrite <- H1;Intros).
Unfold equivClass in H3.
(Elim (axs_comprehension [y0:E](In (couple x y0) r) a y);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Auto).

Save.
(* End of proof of lem_equivClass_is_same                                  *)

Definition equiv_comp :=
  [r:E][a:E][rRel:(rel_sig r a a)]
  [rEqv:(equivalence r a rRel)][rD:(rdom r a a rRel)=a]
  [s:E][f:E][Afs:(fun_sig f a s)]
  [Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y)]
  (subset
    [z:E](Ex[x:E]((In x a) /\ z=(couple (equivClass r a rRel rEqv x)  (App f (lem_fun_sig_is_fun f a s Afs) x))))
    (cartesien (quotient r a rRel rEqv rD) s)
  )
.

(***************************************************************************)
(* equiv_comp est une application                                          *)
(***************************************************************************)
Lemma lem_equiv_comp_is_fun :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))
  (fun (equiv_comp r a rRel rEqv rD s f Afs Pr))
.
(* Proof of lem_equiv_comp_is_fun                                          *)
(Intros;Unfold fun;Intros).
(Split;Intros).
(Exists (quotient r a rRel rEqv rD);Exists s;Unfold inc;Intros).
Unfold equiv_comp in H.
(Elim (axs_comprehension [z:E](Ex[x:E](and (In x a) (eq ? z (couple (equivClass r a rRel rEqv x) (App f (lem_fun_sig_is_fun f a s Afs) x))))) (cartesien (quotient r a rRel rEqv rD) s) v0);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Auto).

Unfold equiv_comp in H.
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple x y));Intros;Generalize (H1 H);Clear H H1 H2;Intros).
(Elim H;Clear H;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie x (equivClass r a rRel rEqv x0) y (App f (lem_fun_sig_is_fun f a s Afs) x0));Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros).
Unfold equiv_comp in H0.
(Elim (axs_comprehension [z0:E](Ex[x0:E](and (In x0 a) (eq ? z0 (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple x z));Intros;Generalize (H4 H0);Clear H0 H4 H5;Intros).
(Elim H0;Clear H0;Intros;Elim H4;Clear H4;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_couple_propertie x (equivClass r a rRel rEqv x1) z (App f (lem_fun_sig_is_fun f a s Afs) x1));Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros;Elim H5;Clear H5;Intros).
(Generalize H5;Clear H5;Rewrite -> H2;Intros).
(Generalize (lem_equivClass_is_same r a rRel rEqv rD x0 x1 H1 H4 H5);Intros).
(Generalize (Pr x0 x1 H7);Intros).
(Rewrite -> H3;Rewrite -> H6;Rewrite -> H8;Reflexivity).

Save.
(* End of proof of lem_equiv_comp_is_fun                                   *)

(***************************************************************************)
(* Le domaine de equiv_comp est l'ensemble quotient                        *)
(***************************************************************************)
Lemma lem_equiv_comp_dom :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))
  (dom (equiv_comp r a rRel rEqv rD s f Afs Pr))
  =
  (quotient r a rRel rEqv rD)
.
(* Proof of lem_equiv_comp_dom                                             *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
Unfold dom in H.
(Elim (axs_comprehension [v0:E](fun (equiv_comp r a rRel rEqv rD s f Afs Pr))->(Ex[v1:E](In (couple v0 v1) (equiv_comp r a rRel rEqv rD s f Afs Pr))) (reunion (reunion (equiv_comp r a rRel rEqv rD s f Afs Pr))) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros).
(Elim (H0 (lem_equiv_comp_is_fun r a rRel rEqv rD s f Afs Pr));Clear H0;Intros).
Unfold equiv_comp in H0.
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Clear H1).
(Elim (lem_cartesian_propertie (quotient r a rRel rEqv rD) s (couple v2 x));Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie v2 x0 x x1);Intros;Generalize (H3 H2);Clear H2 H3 H4;Intros).
(Elim H2;Clear H2;Intros).
(Generalize H0;Rewrite <- H2;Tauto).

Unfold dom.
(Elim (axs_comprehension [v0:E](fun (equiv_comp r a rRel rEqv rD s f Afs Pr))->(Ex[v1:E](In (couple v0 v1) (equiv_comp r a rRel rEqv rD s f Afs Pr))) (reunion (reunion (equiv_comp r a rRel rEqv rD s f Afs Pr))) v2);Intros;Apply H1;Clear H0 H1;Intros).
(Generalize H;Unfold quotient;Intros).
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Split;Intros).
(Elim (axs_reunion (reunion (equiv_comp r a rRel rEqv rD s f Afs Pr)) v2);Intros;Apply H4;Clear H3 H4;Intros).
(Exists (singleton v2);Split;[Idtac|Apply lem_x_in_sing_x]).
(Elim (axs_reunion (equiv_comp r a rRel rEqv rD s f Afs Pr) (singleton v2));Intros;Apply H4;Clear H3 H4;Intros).
(Exists (couple v2 (App f (lem_fun_sig_is_fun f a s Afs) x));Split;[Idtac|Elim (axs_paire (singleton v2) (paire v2 (App f (lem_fun_sig_is_fun f a s Afs) x)) (singleton v2));Intros;Apply H4;Left;Reflexivity]).
Unfold equiv_comp.
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple v2 (App f (lem_fun_sig_is_fun f a s Afs) x)));Intros;Apply H4;Clear H3 H4;Intros).
(Split;[Idtac|Exists x;Split;[Auto|Rewrite -> H2;Reflexivity]]).
(Elim (lem_cartesian_propertie (quotient r a rRel rEqv rD) s (couple v2 (App f (lem_fun_sig_is_fun f a s Afs) x)));Intros;Apply H4;Clear H3 H4;Intros).
(Exists v2;Exists (App f (lem_fun_sig_is_fun f a s Afs) x);Split;[Unfold quotient|Split;[Idtac|Reflexivity]]).
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) v2);Intros;Apply H4;Clear H3 H4;Intros).
(Split;[Rewrite -> H2;Exact (lem_equivClass_in_parties_a r a rRel rEqv x)|Exists x;Split;Auto]).

(Elim Afs;Intros).
(Elim H4;Clear H4;Intros).
(Cut (In x (dom f));Intros;[Idtac|Rewrite -> H4;Auto]).
(Generalize (lem_eval_prop2 f x (lem_fun_sig_is_fun f a s Afs) H6);Clear H6;Intros).
(Generalize (lem_fun_and_img f x (App f (lem_fun_sig_is_fun f a s Afs) x) (lem_fun_sig_is_fun f a s Afs) H6);Clear H6;Intros).
Unfold inc in H5.
Exact (H5 (App f (lem_fun_sig_is_fun f a s Afs) x) H6).

(Exists (App f (lem_fun_sig_is_fun f a s Afs) x);Unfold equiv_comp).
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple v2 (App f (lem_fun_sig_is_fun f a s Afs) x)));Intros;Apply H5;Clear H4 H5;Intros).
(Split;[Idtac|Exists x;Split;[Auto|Rewrite -> H2;Auto]]).
(Elim (lem_cartesian_propertie (quotient r a rRel rEqv rD) s (couple v2 (App f (lem_fun_sig_is_fun f a s Afs) x)));Intros;Apply H5;Clear H4 H5;Intros).
(Exists v2;Exists (App f (lem_fun_sig_is_fun f a s Afs) x);Split;[Idtac|Split;[Idtac|Reflexivity]]).
Unfold quotient.
(Elim (axs_comprehension [x:E](Ex [y:E](and (In y a) (eq ? x (equivClass r a rRel rEqv y)))) (parties a) v2);Intros;Apply H5;Clear H4 H5;Intros).
(Split;[Rewrite -> H2;Exact (lem_equivClass_in_parties_a r a rRel rEqv x)|Exists x;Split;Auto]).

(Elim Afs;Intros).
(Elim H5;Clear H5;Intros).
(Cut (In x (dom f));Intros;[Idtac|Rewrite -> H5;Auto]).
(Generalize (lem_eval_prop2 f x (lem_fun_sig_is_fun f a s Afs) H7);Intros).
(Generalize (lem_fun_and_img f x (App f (lem_fun_sig_is_fun f a s Afs) x) (lem_fun_sig_is_fun f a s Afs) H8);Clear H8;Intros).
Exact (H6 (App f (lem_fun_sig_is_fun f a s Afs) x) H8).

Save.
(* End of proof of lem_equiv_comp_dom                                      *)

(***************************************************************************)
(* L'image de equiv_comp est une partie de s                               *)
(***************************************************************************)
Lemma lem_equiv_comp_img :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))
  (inc (Img (equiv_comp r a rRel rEqv rD s f Afs Pr)) s)
.
(* Proof of lem_equiv_comp_img                                             *)
(Unfold inc;Intros).
(Elim Afs;Intros;Elim H1;Clear H1;Intros).
Unfold Img in H.
(Elim (axs_comprehension [v1:E](fun (equiv_comp r a rRel rEqv rD s f Afs Pr))->(Ex[v2:E](In (couple v2 v1) (equiv_comp r a rRel rEqv rD s f Afs Pr))) (reunion (reunion (equiv_comp r a rRel rEqv rD s f Afs Pr))) v0);Intros;Generalize (H3 H);Clear H H3 H4;Intros).
(Elim H;Clear H;Intros).
(Elim (H3 (lem_equiv_comp_is_fun r a rRel rEqv rD s f Afs Pr));Clear H3;Intros).
Unfold equiv_comp in H3.
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple x v0));Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Clear H4).
(Elim (lem_cartesian_propertie (quotient r a rRel rEqv rD) s (couple x v0));Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_couple_propertie x x0 v0 x1);Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros;Elim H5;Clear H5;Intros).
(Generalize H4;Rewrite -> H6;Tauto).

Save.
(* End of proof of lem_equiv_comp_img                                      *)

(***************************************************************************)
(* D'ou la signature de equiv_comp ...                                     *)
(***************************************************************************)
Lemma lem_equiv_comp_sig :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))
  (fun_sig
    (equiv_comp r a rRel rEqv rD s f Afs Pr)
    (quotient r a rRel rEqv rD)
    s
  )
.
(* Proof of lem_equiv_comp_sig                                             *)
(Intros;Unfold fun_sig).
(Split;[Exact (lem_equiv_comp_is_fun r a rRel rEqv rD s f Afs Pr)|Split;[Exact (lem_equiv_comp_dom r a rRel rEqv rD s f Afs Pr)|Exact (lem_equiv_comp_img r a rRel rEqv rD s f Afs Pr)]]).

Save.
(* End of proof of lem_equiv_comp_sig                                      *)

(***************************************************************************)
(* Si x est dans a, le couple classe d'equivalence de x et (f x) est un    *)
(* element de equiv_comp.                                                  *)
(***************************************************************************)
Lemma lem_equiv_comp_elem :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))(x:E)
  (In x a)
  ->
  (In
    (couple
      (equivClass r a rRel rEqv x) 
      (App f (lem_fun_sig_is_fun f a s Afs) x)
    )
    (equiv_comp r a rRel rEqv rD s f Afs Pr)
  )
.
(* Proof of lem_equiv_comp_elem                                            *)
Intros.
(Generalize (lem_x_equiv_in_projQ r a rRel rEqv rD x H);Intros).
(Generalize (lem_fun_and_img (projQuotient r a rRel rEqv rD) x (equivClass r a rRel rEqv x) (lem_projQuotient_is_fun r a rRel rEqv rD) H0);Rewrite -> (lem_projQuotient_img r a rRel rEqv rD);Intros).
Unfold equiv_comp.
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple (equivClass r a rRel rEqv x) (App f (lem_fun_sig_is_fun f a s Afs) x)));Intros;Apply H3;Clear H2 H3;Intros).
(Split;[Idtac|Exists x;Split;[Auto|Reflexivity]]).
(Elim (lem_cartesian_propertie (quotient r a rRel rEqv rD) s (couple (equivClass r a rRel rEqv x) (App f (lem_fun_sig_is_fun f a s Afs) x)));Intros;Apply H3;Clear H2 H3;Intros).
(Exists (equivClass r a rRel rEqv x);Exists (App f (lem_fun_sig_is_fun f a s Afs) x);Split;[Auto|Split;[Idtac|Reflexivity]]).
(Elim Afs;Intros;Elim H3;Clear H3;Intros).
(Cut (In x (dom f));Intros;[Idtac|Rewrite -> H3;Auto]).
(Generalize (lem_eval_prop2 f x (lem_fun_sig_is_fun f a s Afs) H5);Intros).
(Generalize (lem_fun_and_img f x (App f (lem_fun_sig_is_fun f a s Afs) x) (lem_fun_sig_is_fun f a s Afs) H6);Intros).
Exact (H4 (App f (lem_fun_sig_is_fun f a s Afs) x) H7).

Save.
(* End of proof of lem_equiv_comp_elem                                     *)

(***************************************************************************)
(* f = equiv_comp o projQuotient                                           *)
(***************************************************************************)
Lemma lem_equiv_comp_prop :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))
  f = 
  (comp 
    (projQuotient r a rRel rEqv rD)
    (equiv_comp r a rRel rEqv rD s f Afs Pr)
    a (quotient r a rRel rEqv rD) s
    (lem_projQuotient_sig r a rRel rEqv rD)
    (lem_equiv_comp_sig r a rRel rEqv rD s f Afs Pr)
  )
.
(* Proof of lem_equiv_comp_prop                                            *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim Afs;Intros;Elim H1;Clear H1;Intros).
(Generalize (lem_x_in_fun_form f H0 v2 H);Intros;Elim H3;Clear H3;Intros;Elim H3;Clear H3;Intros).
(Generalize H;Clear H;Rewrite -> H3;Clear H3;Intros).
(Generalize (lem_fun_and_dom f x x0 H0 H) (lem_fun_and_img f x x0 H0 H);Intros).
(Cut (In x a);Intros;[Idtac|Rewrite <- H1;Auto]).
(Generalize (H2 x0 H4);Intros).
(Generalize (lem_x_equiv_in_projQ r a rRel rEqv rD x H5);Intros).
(Generalize (lem_fun_and_img (projQuotient r a rRel rEqv rD) x (equivClass r a rRel rEqv x) (lem_projQuotient_is_fun r a rRel rEqv rD) H7) ;Rewrite -> (lem_projQuotient_img r a rRel rEqv rD);Intros).
(Generalize (lem_equiv_comp_elem r a rRel rEqv rD s f Afs Pr x H5);Intros).
Unfold comp.
(Elim (axs_comprehension [c:E](Ex[y:E](and (In (couple (first c) y) (projQuotient r a rRel rEqv rD)) (In (couple y (second c)) (equiv_comp r a rRel rEqv rD s f Afs Pr)))) (cartesien a s) (couple x x0));Intros;Apply H11;Clear H10 H11;Intros).
(Split;[Idtac|Exists (equivClass r a rRel rEqv x);Rewrite -> (lem_first_propertie x x0);Rewrite -> (lem_second_propertie x x0);Split;[Auto|Idtac]]).
(Elim (lem_cartesian_propertie a s (couple x x0));Intros;Apply H11;Clear H10 H11;Intros).
(Exists x;Exists x0;Split;[Auto|Split;[Exact (H2 x0 H4)|Reflexivity]]).

(Rewrite -> (lem_eval_propertie f x x0 (lem_fun_sig_is_fun f a s Afs) H);Auto).

Unfold comp in H.
(Elim (axs_comprehension [c:E](Ex[y:E](and (In (couple (first c) y) (projQuotient r a rRel rEqv rD)) (In (couple y (second c)) (equiv_comp r a rRel rEqv rD s f Afs Pr)))) (cartesien a s) v2);Intros;Generalize (H0 H);Clear H H0 H1;Intros).
(Elim H;Clear H;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
Unfold projQuotient in H0.
(Elim (axs_comprehension [x0:E](Ex[z:E](Ex[t:E](and (eq ? x0 (couple z t)) (eq ? t (equivClass r a rRel rEqv z))))) (cartesien a (quotient r a rRel rEqv rD)) (couple (first v2) x));Intros;Generalize (H2 H0);Clear H0 H2 H3;Intros).
(Elim H0;Clear H0;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie (first v2) x0 x x1);Intros;Generalize (H4 H2);Clear H2 H4 H5;Intros;Elim H2;Clear H2;Intros).
(Generalize H3;Rewrite <- H2;Rewrite <- H4;Clear H2 H3 H4;Intros).
Unfold equiv_comp in H1.
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass r a rRel rEqv x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient r a rRel rEqv rD) s) (couple x (second v2)));Intros;Generalize (H2 H1);Clear H1 H2 H4;Intros).
(Elim H1;Clear H1;Intros;Elim H2;Clear H2;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x (equivClass r a rRel rEqv x2) (second v2) (App f (lem_fun_sig_is_fun f a s Afs) x2));Intros;Generalize (H5 H4);Clear H4 H5 H6;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_cartesian_propertie a s v2);Intros;Generalize (H6 H);Clear H6 H7;Intros).
(Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros;Elim H6;Clear H6;Intros;Elim H7;Clear H7;Intros).
(Generalize H H0 H1 H3 H5;Rewrite -> H8;Rewrite -> (lem_first_propertie x3 x4);Rewrite -> (lem_second_propertie x3 x4);Clear H H0 H1 H3 H5;Intros).
(Generalize H4;Rewrite -> H3;Clear H3;Intros).
(Generalize (lem_equivClass_is_same r a rRel rEqv rD x3 x2 H6 H2 H3);Intros).
(Generalize (Pr x3 x2 H9);Intros).
(Rewrite -> H5;Rewrite <- H10).
(Elim Afs;Intros;Elim H12;Clear H12;Intros).
(Cut (In x3 (dom f));Intros;[Idtac|Rewrite -> H12;Auto]).
Exact (lem_eval_prop2 f x3 (lem_fun_sig_is_fun f a s Afs) H14).

Save.
(* End of proof of lem_equiv_comp_prop                                     *)

(***************************************************************************)
(* Si f = g o projQuotient alors g = equiv_comp                            *)
(***************************************************************************)
Lemma lem_equiv_comp_unicite :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))
  (g:E)(Ags:(fun_sig g (quotient r a rRel rEqv rD) s))
  f = 
  (comp
    (projQuotient r a rRel rEqv rD) g
    a (quotient r a rRel rEqv rD) s
    (lem_projQuotient_sig r a rRel rEqv rD) Ags
  )
  ->
  g = (equiv_comp r a rRel rEqv rD s f Afs Pr)
.
(* Proof of lem_equiv_comp_unicite                                         *)
(Intros;Apply axs_extensionnalite;Unfold iff;Split;Intros).
(Elim Afs;Elim Ags;Intros).
(Elim H2;Elim H4;Clear H2 H4;Intros).
(Generalize (lem_x_in_fun_form g H1 v2 H0);Intros).
(Elim H7;Clear H7;Intros;Elim H7;Clear H7;Intros).
(Generalize H0;Rewrite -> H7;Clear  H0 H7;Intros).
(Generalize (lem_fun_and_dom g x x0 H1 H0) (lem_fun_and_img g x x0 H1 H0);Intros).
(Generalize H7;Rewrite -> H5;Intros).
Unfold equiv_comp.
(Elim (axs_comprehension [z:E](Ex[x1:E](and (In x1 a) (eq ? z (couple (equivClass r a rRel rEqv x1) (App f (lem_fun_sig_is_fun f a s Afs) x1))))) (cartesien (quotient r a rRel rEqv rD) s) (couple x x0));Intros;Apply H11;Clear H10 H11;Intros).
(Generalize (H6 x0 H8);Intros).
Split.
(Elim (lem_cartesian_propertie (quotient r a rRel rEqv rD) s (couple x x0));Intros;Apply H12;Clear H11 H12;Intros).
(Exists x;Exists x0;Split;[Auto|Split;[Auto|Reflexivity]]).

Unfold quotient in H9.
(Elim (axs_comprehension [x0:E](Ex [y:E](and (In y a) (eq ? x0 (equivClass r a rRel rEqv y)))) (parties a) x);Intros;Generalize (H11 H9);Clear H9 H11 H12;Intros).
(Elim H9;Clear H9;Intros;Elim H11;Clear H11;Intros;Elim H11;Clear H11;Intros).
(Exists x1;Split;[Auto|Idtac]).
(Elim (lem_couple_propertie x (equivClass r a rRel rEqv x1) x0 (App f (lem_fun_sig_is_fun f a s Afs) x1));Intros;Apply H14;Clear H13 H14;Intros).
(Split;[Auto|Idtac]).
(Generalize (lem_x_equiv_in_projQ r a rRel rEqv rD x1 H11);Rewrite <- H12;Intros).
(Generalize (lem_comp_make (projQuotient r a rRel rEqv rD) g a (quotient r a rRel rEqv rD) s x1 x x0 (lem_projQuotient_sig r a rRel rEqv rD) Ags H13 H0);Rewrite <- H;Intros).
Exact (lem_eval_propertie f x1 x0 H3 H14).

Unfold equiv_comp in H0.
(Elim (axs_comprehension [z:E](Ex[x:E](and (In x a) (eq ? z (couple (equivClass r a rRel rEqv x) (App f (lem_fun_sig_is_fun f a s Afs) x))))) (cartesien (quotient r a rRel rEqv rD) s) v2);Intros;Generalize (H1 H0);Clear H0 H1 H2;Intros).
(Elim H0;Clear H0;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_cartesian_propertie (quotient r a rRel rEqv rD) s v2);Intros;Generalize (H3 H0);Clear H0 H3 H4;Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros;Elim H3;Clear H3;Intros).
(Generalize H2;Rewrite -> H4;Clear H2 H4;Intros).
(Elim (lem_couple_propertie x0 (equivClass r a rRel rEqv x) x1 (App f (lem_fun_sig_is_fun f a s Afs) x));Intros;Generalize (H4 H2);Clear H2 H4 H5;Intros).
(Elim H2;Clear H2;Intros).
(Generalize (lem_x_equiv_in_projQ r a rRel rEqv rD x H1);Rewrite <- H2;Intros).
(Elim Afs;Intros;Elim H7;Clear H7;Intros).
(Generalize H1;Rewrite <- H7;Intros).
(Cut (In (couple x (App f (lem_fun_sig_is_fun f a s Afs) x)) (comp (projQuotient r a rRel rEqv rD) g a (quotient r a rRel rEqv rD) s (lem_projQuotient_sig r a rRel rEqv rD) Ags));Intros).
(Generalize (lem_comp_intermed (projQuotient r a rRel rEqv rD) g a (quotient r a rRel rEqv rD) s x (App f (lem_fun_sig_is_fun f a s Afs) x) (lem_projQuotient_sig r a rRel rEqv rD) Ags H10);Clear H10;Intros).
(Elim H10;Clear H10;Intros;Elim H10;Clear H10;Intros).
(Generalize (lem_projQuotient_is_fun r a rRel rEqv rD);Intros).
(Elim H12;Clear H12;Intros).
(Generalize (H13 x x0 x2 H5 H10);Clear H12 H13;Intros).
(Generalize H11;Rewrite <- H12;Rewrite <- H4;Tauto).

Rewrite <- H.
Exact (lem_eval_prop2 f x (lem_fun_sig_is_fun f a s Afs) H9).

Save.
(* End of proof of lem_equiv_comp_unicite                                  *)

(***************************************************************************)
(* On propose un ennonce plus traditionnel du thm fondamental sur les      *)
(* ensembles quotients.                                                    *)
(***************************************************************************)
Theorem thm_ens_quotient_fond :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_sig f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s Afs) x)=(App f (lem_fun_sig_is_fun f a s Afs) y))
  (ExU[g:E](
    (fun_sig g (quotient r a rRel rEqv rD) s)
    /\ 
    f = (comp2 
      (projQuotient r a rRel rEqv rD) g
      a (quotient r a rRel rEqv rD) s)
  ))
.
(* Proof of thm_ens_quotient_fond                                          *)
(Unfold exUniq;Intros).
Exists (equiv_comp r a rRel rEqv rD s f Afs Pr).
(Generalize (lem_equiv_comp_sig r a rRel rEqv rD s f Afs Pr);Intros).
(Split;[Split;[Auto|Idtac]|Intros;Symmetry]).
(Generalize (lem_projQuotient_sig r a rRel rEqv rD);Intros).
Rewrite <- (lem_comp2_eq_comp (projQuotient r a rRel rEqv rD) (equiv_comp r a rRel rEqv rD s f Afs Pr) a (quotient r a rRel rEqv rD) s H0 H).
Exact (lem_equiv_comp_prop r a rRel rEqv rD s f Afs Pr).

(Elim H0;Clear H0;Intros).
(Generalize (lem_comp2_eq_comp (projQuotient r a rRel rEqv rD) y a (quotient r a rRel rEqv rD) s (lem_projQuotient_sig r a rRel rEqv rD) H0);Intros).
(Generalize H1;Rewrite <- H2;Clear H1 H2;Intros).
Exact (lem_equiv_comp_unicite r a rRel rEqv rD s f Afs Pr y H0 H1).

Save.
(* End of proof of thm_ens_quotient_fond                                   *)

(***************************************************************************)
(* La seconde partie du thm fondamental: si f est une surjection et si la  *)
(* reciproque de Pr est verifiee alors equiv_comp est une bijection        *)
(***************************************************************************)
Lemma lem_equiv_comp_bij_under :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rEqv:(equivalence r a rRel))(rD:(rdom r a a rRel)=a)
  (s:E)(f:E)(Afs:(fun_surj f a s))
  (Pr:(x,y:E)(In (couple x y) r)->(App f (lem_fun_sig_is_fun f a s (lem_surj_to_sig f a s Afs)) x)=(App f (lem_fun_sig_is_fun f a s (lem_surj_to_sig f a s Afs)) y))
  (Prinv:(x,y:E)(App f (lem_fun_sig_is_fun f a s (lem_surj_to_sig f a s Afs)) x)=(App f (lem_fun_sig_is_fun f a s (lem_surj_to_sig f a s Afs)) y)->(In (couple x y) r))
  (fun_bij 
    (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr)
    (quotient r a rRel rEqv rD) s
  )
.
(* Proof of lem_equiv_comp_bij_under                                       *)
(Unfold fun_bij;Intros).
(Split;[Exact (lem_equiv_comp_is_fun r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr)|Idtac]).
(Split;[Exact (lem_equiv_comp_dom r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr)|Idtac]).
(Split;[Apply axs_extensionnalite;Unfold iff;Split;Intros|Intros]).
Exact (lem_equiv_comp_img r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr v2 H).

(Elim Afs;Intros;Elim H1;Clear H1;Intros).
(Cut (In v2 (Img f));Intros;[Idtac|Rewrite -> H2;Auto]).
(Generalize (lem_App_rev f v2 H0 H3);Intros).
(Elim H4;Clear H4;Intros;Elim H4;Clear H4;Intros).
(Cut (In x a);Intros;[Idtac|Rewrite <- H1;Auto]).
(Generalize (lem_eval_propertie f x v2 H0 H5);Intros).
(Generalize H H3 H5;Rewrite -> H7;Clear H H3 H5 H7;Intros).
(Generalize (lem_equiv_comp_prop r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr);Intros).
(Cut (In (couple x (App f H0 x)) (comp (projQuotient r a rRel rEqv rD) (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) a (quotient r a rRel rEqv rD) s (lem_projQuotient_sig r a rRel rEqv rD) (lem_equiv_comp_sig r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr)));Intros;[Idtac|Rewrite <- H7;Auto]).
(Generalize (lem_comp_intermed (projQuotient r a rRel rEqv rD) (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) a (quotient r a rRel rEqv rD) s x (App f H0 x) (lem_projQuotient_sig r a rRel rEqv rD) (lem_equiv_comp_sig r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) H8);Clear H7 H8;Intros).
(Elim H7;Clear H7;Intros;Elim H7;Clear H7;Intros).
Exact (lem_fun_and_img (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) x0 (App f H0 x) (lem_equiv_comp_is_fun r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) H8).

(Unfold exUniq;Intros).
(Generalize (lem_App_rev (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) b (lem_equiv_comp_is_fun r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) H);Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Exists x;Split;[Auto|Intros]).
(Generalize (lem_fun_and_dom (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) y b (lem_equiv_comp_is_fun r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) H2);Intros).
(Generalize H0 H3;Rewrite -> (lem_equiv_comp_dom r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr);Clear H0 H3;Intros).
Unfold quotient in H0.
(Elim (axs_comprehension [x0:E](Ex [y:E](and (In y a) (eq ? x0 (equivClass r a rRel rEqv y)))) (parties a) x);Intros;Generalize (H4 H0);Clear H0 H4 H5;Intros).
Unfold quotient in H3.
(Elim (axs_comprehension [x:E](Ex[y0:E](and (In y0 a) (eq ? x (equivClass r a rRel rEqv y0)))) (parties a) y);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H0;Clear H0;Intros;Clear H0;Elim H4;Clear H4;Intros;Elim H0;Clear H0;Intros).
(Elim H3;Clear H3;Intros;Clear H3;Elim H5;Clear H5;Intros;Elim H3;Clear H3;Intros).
(Generalize H1 H2;Rewrite -> H4;Rewrite -> H5;Clear H1 H2 H4 H5;Intros).
(Generalize (lem_x_equiv_in_projQ r a rRel rEqv rD x0 H0) (lem_x_equiv_in_projQ r a rRel rEqv rD x1 H3);Intros).
(Generalize (lem_comp_make (projQuotient r a rRel rEqv rD) (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) a (quotient r a rRel rEqv rD) s x0 (equivClass r a rRel rEqv x0) b (lem_projQuotient_sig r a rRel rEqv rD) (lem_equiv_comp_sig r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) H4 H1);Intros).
(Generalize (lem_comp_make (projQuotient r a rRel rEqv rD) (equiv_comp r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) a (quotient r a rRel rEqv rD) s x1 (equivClass r a rRel rEqv x1) b (lem_projQuotient_sig r a rRel rEqv rD) (lem_equiv_comp_sig r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr) H5 H2);Intros).
(Generalize (lem_equiv_comp_prop r a rRel rEqv rD s f (lem_surj_to_sig f a s Afs) Pr);Intros).
(Generalize H6 H7;Rewrite <- H8;Clear H6 H7 H8;Intros).
(Generalize (lem_eval_propertie f x0 b (lem_fun_sig_is_fun f a s (lem_surj_to_sig f a s Afs)) H6) (lem_eval_propertie f x1 b (lem_fun_sig_is_fun f a s (lem_surj_to_sig f a s Afs)) H7);Intros).
(Generalize H9;Rewrite -> H8;Intros).
(Generalize (Prinv x0 x1 H10);Intros).
Exact (lem_equivClass_eq r a rRel rEqv x0 x1 H11).

Save.
(* End of proof of lem_equiv_comp_bij_under                                *)

(***************************************************************************)
(***************************************************************************)
(* Deux corollaires au thm fondamental                                     *)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(* Corollaire 1                                                            *)
(***************************************************************************)
(***************************************************************************)
(* On s'interesse a equiv_comp dans le cas ou la relation r est le noyau   *)
(* d'equivalence de f.                                                     *)
(***************************************************************************)
Definition equiv_comp_fun := [f,a,s:E][Afs:(fun_sig f a s)]
  (equiv_comp
    (equiv_fun f (lem_fun_sig_is_fun f a s Afs))
    a
    (lem_sig_fun_equiv_is_rel f a s Afs)
    (lem_sig_fun_equiv_is_equiv f a s Afs)
    (lem_sig_fun_equiv_dom f a s Afs)
    s f Afs
    (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs))
  )
.

(***************************************************************************)
(* Cette nouvelle "instance" de equiv_comp est une injection               *)
(***************************************************************************)
Lemma lem_equiv_comp_fun_is_inj :
  (f,a,s:E)(Afs:(fun_sig f a s))
  (fun_inj
    (equiv_comp_fun f a s Afs)
    (quotient
      (equiv_fun f (lem_fun_sig_is_fun f a s Afs))
      a
      (lem_sig_fun_equiv_is_rel f a s Afs)
      (lem_sig_fun_equiv_is_equiv f a s Afs)
      (lem_sig_fun_equiv_dom f a s Afs)
    )
    s
  )
.
(* Proof of lem_equiv_comp_fun_is_inj                                      *)
(Intros;Unfold fun_inj).
(Split;[Idtac|Split;Intros]).
Unfold equiv_comp_fun.
Exact (lem_equiv_comp_is_fun (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs) s f Afs (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs))).

Exact (lem_equiv_comp_dom (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs) s f Afs (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs))).

(Split;Intros).
Exact (lem_equiv_comp_img (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs) s f Afs (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs))).

(Unfold exUniq;Intros).
(Generalize (lem_App_rev (equiv_comp_fun f a s Afs) b (lem_equiv_comp_is_fun (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs) s f Afs (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs))) H);Intros).
(Elim H0;Clear H0;Intros;Elim H0;Clear H0;Intros).
(Exists x;Split;[Auto|Intros]).
(Generalize (lem_equiv_comp_is_fun (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs) s f Afs (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs)));Intros).
Unfold equiv_comp_fun in H1.
Unfold equiv_comp in H1.
(Elim (axs_comprehension [z:E](Ex[x0:E](and (In x0 a) (eq ? z (couple (equivClass (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) x0) (App f (lem_fun_sig_is_fun f a s Afs) x0))))) (cartesien (quotient (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs)) s) (couple x b));Intros;Generalize (H4 H1);Clear H1 H4 H5;Intros).
(Elim H1;Clear H1;Intros;Elim H4;Clear H4;Intros;Elim H4;Clear H4;Intros).
(Elim (lem_couple_propertie x (equivClass (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) x0) b (App f (lem_fun_sig_is_fun f a s Afs) x0));Intros;Generalize (H6 H5);Clear H5 H6 H7;Intros).
(Elim H5;Clear H5;Intros).
(Clear H1;Unfold equiv_comp_fun in H2;Unfold equiv_comp in H2).
(Elim (axs_comprehension [z:E](Ex[x:E](and (In x a) (eq ? z (couple (equivClass (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) x) (App f (lem_fun_sig_is_fun f a s Afs) x))))) (cartesien (quotient (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs)) s) (couple y b));Intros;Generalize (H1 H2);Clear H1 H2 H7;Intros).
(Elim H1;Clear H1;Intros;Clear H1;Elim H2;Clear H2;Intros;Elim H1;Clear H1;Intros).
(Elim (lem_couple_propertie y (equivClass (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) x1) b (App f (lem_fun_sig_is_fun f a s Afs) x1));Intros;Generalize (H7 H2);Clear H2 H7 H8;Intros).
(Elim H2;Clear H2;Intros).
(Generalize H7;Rewrite -> H6;Clear H6 H7;Intros).
(Generalize (lem_sig_equiv_fun_make f a s Afs x0 x1 H4 H1 H7);Intros).
(Generalize (lem_equivClass_eq (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) x0 x1 H6);Intros).
(Rewrite -> H5;Rewrite -> H2;Auto).

Save.
(* End of proof of lem_equiv_comp_fun_is_inj                               *)

(***************************************************************************)
(* La signature de equiv_comp_fun                                          *)
(***************************************************************************)
Lemma lem_equiv_comp_fun_sig :
  (f,a,s:E)(Afs:(fun_sig f a s))
    (fun_sig 
      (equiv_comp_fun f a s Afs)
      (quotient
        (equiv_fun f (lem_fun_sig_is_fun f a s Afs))
        a
        (lem_sig_fun_equiv_is_rel f a s Afs)
        (lem_sig_fun_equiv_is_equiv f a s Afs)
        (lem_sig_fun_equiv_dom f a s Afs)
      )
      s
    )
.
(* Proof of lem_equiv_comp_fun_sig                                         *)
Intros.
Exact (lem_equiv_comp_sig (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs) s f Afs (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs))).

Save.
(* End of proof of lem_equiv_comp_fun_sig                                  *)

(***************************************************************************)
(* Comme on pouvait s'y attendre, f = equiv_comp_fun o projQuotient        *)
(***************************************************************************)
Lemma lem_equiv_comp_fun_prop :
  (f,a,s:E)(Afs:(fun_sig f a s))
    f = (comp
      (projQuotient
        (equiv_fun f (lem_fun_sig_is_fun f a s Afs))
        a
        (lem_sig_fun_equiv_is_rel f a s Afs)
        (lem_sig_fun_equiv_is_equiv f a s Afs)
        (lem_sig_fun_equiv_dom f a s Afs)
      )
      (equiv_comp_fun f a s Afs)
      a
      (quotient
        (equiv_fun f (lem_fun_sig_is_fun f a s Afs))
        a
        (lem_sig_fun_equiv_is_rel f a s Afs)
        (lem_sig_fun_equiv_is_equiv f a s Afs)
        (lem_sig_fun_equiv_dom f a s Afs)
      )
      s
      (lem_projQuotient_sig
        (equiv_fun f (lem_fun_sig_is_fun f a s Afs))
        a
        (lem_sig_fun_equiv_is_rel f a s Afs)
        (lem_sig_fun_equiv_is_equiv f a s Afs)
        (lem_sig_fun_equiv_dom f a s Afs)
      )
      (lem_equiv_comp_fun_sig f a s Afs)
    )
.
(* Proof of lem_equiv_comp_fun_prop                                        *)
Intros.
Exact (lem_equiv_comp_prop (equiv_fun f (lem_fun_sig_is_fun f a s Afs)) a (lem_sig_fun_equiv_is_rel f a s Afs) (lem_sig_fun_equiv_is_equiv f a s Afs) (lem_sig_fun_equiv_dom f a s Afs) s f Afs (lem_fun_equiv_prop f (lem_fun_sig_is_fun f a s Afs))).

Save.
(* End of proof of lem_equiv_comp_fun_prop                                 *)

(***************************************************************************)
(***************************************************************************)
(* Resultats exotiques                                                     *)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(* Une relation symmetrique dont le domaine contient au moins deux elements*)
(* distincts n'est pas antisymmetrique.                                    *)
(***************************************************************************)
Lemma lem_sym_not_antisym :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rNe:(Ex[x:E](Ex[y:E](~(x=y) /\ (In (couple x y) r)))))
  (symmetry r a rRel)
  ->
  ~(antisymmetry r a rRel)
.
(* Proof of lem_sym_not_antisym                                            *)
(Unfold not;Unfold symmetry;Unfold antisymmetry;Intros).
(Elim rNe;Clear rNe;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros).
(Generalize (H0 x x0 H2 (H x x0 H2));Intros).
(Absurd (eq ? x x0);Auto).

Save.
(* End of proof of lem_sym_not_antisym                                     *)

(***************************************************************************)
(* Une relation antisymmetrique dont le domaine contient au moins deux     *)
(* elements distincts n'est pas symmetrique.                               *)
(***************************************************************************)
Lemma lem_antisym_not_sym :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rNe:(Ex[x:E](Ex[y:E](~(x=y) /\ (In (couple x y) r)))))
  (antisymmetry r a rRel)
  ->
  ~(symmetry r a rRel)
.
(* Proof of lem_antisym_not_sym                                            *)
(Unfold not;Unfold symmetry;Unfold antisymmetry;Intros).
Elim rNe;Clear rNe;Intros;Elim H1;Clear H1;Intros;Elim H1;Clear H1;Intros.
Generalize (H x x0 H2 (H0 x x0 H2));Intros.
Absurd (eq ? x x0);Auto.

Save.
(* End of proof of lem_antisym_not_sym                                     *)

(***************************************************************************)
(* Une relation irreflexive et transitive est antisymmetrique              *)
(***************************************************************************)
Lemma lem_irrefl_trans_so_antisym :
  (r:E)(a:E)(rRel:(rel_sig r a a))
  (rIrrefl:(irreflexivity r a rRel))
  (rTrans:(transitivity r a rRel))
  (antisymmetry r a rRel)
.
(* Proof of lem_irrefl_trans_so_antisym                                    *)
(Unfold irreflexivity;Unfold transitivity;Unfold antisymmetry;Intros).
(Unfold not;Intros).
(Generalize (rTrans x y x H H0);Clear rTrans H0;Intros).
(Cut (In x (rdom r a a rRel));Intros).
(Generalize (rIrrefl x H1);Intros;Absurd (In (couple x x) r);Auto).

Unfold rdom.
(Elim (axs_comprehension [x0:E](Ex [y:E](In (couple x0 y) r)) a x);Intros;Apply H2;Clear H1 H2;Intros).
(Split;[Unfold rel_sig in rRel;Unfold inc in rRel|Exists y;Auto]).
(Generalize (rRel (couple x y) H) ;Intros).
(Elim (lem_cartesian_propertie a a (couple x y));Intros;Generalize (H2 H1);Clear H1 H2 H3;Intros).
(Elim H1;Clear  H1;Intros;Elim H1;Clear  H1;Intros;Elim H1;Clear  H1;Intros;Elim H2;Clear H2;Intros).
(Elim (lem_couple_propertie x x0 y x1);Intros;Generalize (H4 H3);Clear H3 H4 H5;Intros).
(Elim H3;Clear H3;Intros;Generalize H1;Rewrite -> H3;Tauto).

Save.
(* End of proof of lem_irrefl_trans_so_antisym                             *)

(***************************************************************************)
(*                     End of module MSetBasis                             *)
(***************************************************************************)
