(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: BV.v                                                   *)
(* contents: definition of bit-vectors using polymorphic lists  *)
(* abstraction to nat                                           *)
(****************************************************************)



(* Version AVEC les listes polymorphiques *)
(* Les operateurs instancies pour les BVs sont ceux des listes, avec le
   nom terminant par "bv". ex: nilbv, consbv...*)

Require Export Arith_compl.
Require Export Bool_compl.
Require Export Lists_replace.

(********************************************************************)
(* Instanciation des operateurs des listes polymorphiques           *)
Definition BV := (list bool).
Definition nilbv : BV := (nil bool).
Definition consbv : bool -> BV -> BV := (cons bool).
Definition appbv : BV -> BV -> BV := (app bool).
Definition lengthbv : BV -> nat := (length bool).
Definition lenbv : BV -> nat -> Prop := (len bool).
Definition BV_null : nat -> BV := [n:nat](list_const bool n false).
Definition truncbv : BV -> nat -> BV := (trunc bool).
Definition stripbv : BV -> nat -> BV := (strip bool).
Definition field : BV -> nat -> nat -> BV := (sublist bool).
Definition abit : BV -> nat -> BV := (elemlist bool).
Definition bitset : BV -> nat -> bool -> BV := (replace bool).
Definition BVnot : BV -> BV := [v:BV] (map bool bool v negb).
(********************************************************************)

Fixpoint BV_to_nat [v:BV] : nat :=
	<nat> Case v of
		O
		[b:bool][w:BV] (plus (bool_to_nat b) (plus (BV_to_nat w)
				 		           (BV_to_nat w)))
		end.

Lemma BV_to_nat_eq1: (BV_to_nat nilbv) = O.
Auto.Qed.
Lemma BV_to_nat_eq2: (b:bool)(w:BV)
	(BV_to_nat (consbv b w))=(plus (bool_to_nat b) (plus (BV_to_nat w)
				 		           (BV_to_nat w))).
Auto.Qed.

(* In the current version of Coq (V 5.10.14.b) it is impossible to use
recursive definitions with BV's constructors on left of "=>" as in the
following code: *)

(*
Recursive Definition BV_to_nat : BV -> nat :=
     nilbv => O
   | (consbv b w) => (plus (bool_to_nat b) (plus (BV_to_nat w) (BV_to_nat w))).
*)



Lemma BV_to_nat_app : (l,n:BV)(ll:nat)
(******************)
	(lenbv l ll) ->
	(BV_to_nat (appbv l n)) =
	    (plus (BV_to_nat l) (mult (power2 ll) (BV_to_nat n))).
Unfold BV lenbv appbv .Induction l.Intros.Inversion H.Simpl.Auto.
Intros.Simpl.Inversion H0.Simpl.
Rewrite -> (H n l1).Rewrite -> mult_plus_distr.Repeat Rewrite -> plus_assoc_l.
Rewrite -> (plus_permute2 (plus (bool_to_nat a) (BV_to_nat l0))
             (mult (power2 l1) (BV_to_nat n)) (BV_to_nat l0)).
Auto.
Exact H4.
Qed.Hint BV_to_nat_app.

Lemma BV_to_nat_app2 :(l,n:BV)
(*******************)
	(BV_to_nat (appbv l n)) =
	    (plus (BV_to_nat l) (mult (power2 (lengthbv l)) (BV_to_nat n))).
Intros.Apply BV_to_nat_app.Auto.Unfold lenbv lengthbv .Apply len_length.
Qed.Hint BV_to_nat_app2.

Lemma BV_null_nat : (n:nat) (BV_to_nat (BV_null n))=O.
(****************)
Unfold BV_null .
(Induction n;Auto).
Intros.Simpl.Rewrite -> H.Auto.
Qed.Hint BV_null_nat.

Lemma length_BV_null : (n:nat)(lengthbv (BV_null n))=n.
(*******************)
Unfold lengthbv BV_null .Intro.Apply length_list_const.
Qed.Hint length_BV_null.


(*
Lemma BV_to_nat_max : (l:nat)(v:BV)
	(lenbv v l) -> (lt (BV_to_nat v) (power2 l)).
Induction l.Simpl.Intros.Inversion H.Simpl.Auto.
Intros.Inversion H0.Simpl.Case b.Simpl.Apply lt_plus_n_m.Apply H.Exact H1.
Simpl.Apply lt_S_n.Apply lt_S.Apply lt_plus_n_m.Apply H.Exact H1.
Qed.
*)




