(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: Bool_compl.v                                           *)
(* contents: additional lemmas about bool, definition of xor,   *)
(* bool_to_nat and if_then_else                                 *)
(****************************************************************)

Require Export Bool.

(****************************************************************)
Lemma neg_eq : (a,b:bool)
	(neg a)=(neg b) -> a=b.
(Induction a;Induction b;Auto).
Qed.Hint neg_eq.

Lemma false_to_true : false = (neg true).
Auto.
Qed.

Lemma true_to_false : true = (neg false).
Auto.
Qed.

Definition xorb : bool -> bool -> bool
	:= [b1,b2:bool](orb (andb b1 (negb b2)) (andb (negb b1) (b2))).

Lemma xorb_b_b : (b:bool) ((xorb b b)=false).
Induction b;Auto.
Qed.Hint xorb_b_b.

Lemma xorb_b_false : (b:bool) ((xorb b false)=b).
Induction b;Auto.
Qed.Hint xorb_b_false.

Lemma xorb_b_true : (b:bool) ((xorb b true)=(negb b)).
Induction b;Auto.
Qed.Hint xorb_b_true.

(****************************************************************)
Recursive Definition bool_to_nat : bool -> nat :=
	true => (S O)
	|false => O.

Lemma bool_to_nat_all : (b:bool)
	((bool_to_nat b)=O) \/ ((bool_to_nat b)=(S O)).
Induction b;Auto.
Qed.Hint bool_to_nat_all.

(*
Lemma bool_to_nat_negb : (b:bool)
	(bool_to_nat (negb b))=(minus (S O) (bool_to_nat b)).
Induction b;Auto.
Qed.Hint bool_to_nat_negb.
*)


(****************************************************************)
(* Conditionnelle polymorphique: le test est un booleen,        *)
(* les clauses d'un type quelconque (le meme pour les 2 clauses *)

Recursive Definition If [T:Set] : bool->T->T->T :=
	true x y => x
	|false x y => y.

Lemma If_neg : (T:Set)(b:bool)(x,y:T)  (If T (neg b) x y)=(If T b y x).
Induction b;Simpl;Try Trivial.
Qed.Hint If_neg.

Lemma If_eq : (T:Set)(b:bool)(x:T) (If T b x x)=x.
Induction b;Simpl;Try Trivial.
Qed.Hint If_eq.

Lemma IfIf : (T:Set)(b1,b2:bool)(x,x',y,y':T)
	(If T b1 (If T b2 x y) (If T b2 x' y')) =
	(If T b2 (If T b1 x x') (If T b1 y y')).
Induction b2;Auto.
Qed.

Lemma If_cond_true : (T:Set)(a:bool)(x,y:T)
	~(x=y) ->	(If T a x y)=x ->	a=true.
Unfold not .(Induction a;Auto).Simpl.Intros.Absurd (eq ? y x).Red.Auto.
Exact H0.
Qed.

Lemma If_cond_false : (T:Set)(a:bool)(x,y:T)
	~(x=y) ->	(If T a x y)=y ->	a=false.
Unfold not .Induction a.Simpl.Intros.Absurd (eq ? x y).Auto.
Exact H0.Auto.
Qed.

Lemma F_If : (T,T':Set)(a:bool)(x,y:T)(F:T->T')
	(F (If T a x y))=(If T' a (F x) (F y)).
Induction a;Auto.
Qed. Hint F_If.






