(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: Lists_compl.v                                          *)
(* contents: definitions and lemmas on polymorphic lists,       *)
(* length, append, map, reverse, list_constant                  *)
(* decidability of equality on lists *** TO BE PROVED ***       *)
(****************************************************************)

Require Export PolyList.
Require Export Arith_compl.

Section Lists_compl.
Variable A,B : Set.

Lemma length_eq1 : (length A (nil A))=O.
Auto.Qed.
Lemma length_eq2 : (x:A)(l:(list A)) (length A (cons A x l))=(S (length A l)).
Auto.Qed.

Lemma app_eq1: (l':(list A)) (app A (nil A) l')=l'. 
Auto.Qed.
Lemma app_eq2: (x:A)(l,l':(list A))
		(app A (cons A x l) l') = (cons A x (app A l l')).
Auto.Qed.

Inductive len : (list A) -> nat -> Prop :=
(************)
	  len_nil : (len (nil A) O)
	| len_cons : (b:A)(v:(list A))(l:nat)
			(len v l) -> (len (cons A b v) (S l)).
Hint len_nil len_cons.

Lemma nil_cons : (b:A)(v:(list A))~((nil A)=(cons A b v)).
(*************)
Intros.Discriminate.
Qed.

Lemma not_cons_eq : (v:(list A))(b:A) ~v=(cons A b v).
(****************)
Induction v.Intro.Discriminate.
Intros.Injection.Intros.Absurd (eq ? l (cons A a l)).Apply H.
Exact H1.
Qed.

Lemma app_v_nil : (v:(list A))(app A v (nil A))=v.
(**************)
Intro.
(Rewrite <- app_nil_end;Try Trivial).
Qed.Hint app_v_nil.

Lemma app_v_nil_idem : (v,v':(list A))	v=v' -> (app A v (nil A))=v'.
(********************)
Intros.Inversion H.Apply app_v_nil.
Qed.

Lemma app_v_nil_inv : (v1,v2:(list A)) (app A v1 v2)=v1 -> v2=(nil A).
(******************)
Induction v1.Simpl.Try Trivial.
Intros.Generalize H0 .Rewrite -> app_eq2.Intros.Injection H1.Intro.Apply H.Exact H2.
Qed.

Lemma app_v_nil_sym : (v1,v2:(list A)) v1=(nil A)->(app A v1 v2)=(app A v2 v1).
(******************)
Induction v2.Intro.Rewrite -> H.Try Trivial.
Intros.Rewrite -> H0.Auto.
Qed.

Definition app_assoc_r := app_ass.
(*********************)
Definition app_assoc_l := ass_app.
(*********************)

Lemma len_cons_not_O : (b:A)(v:(list A))(l:nat)
(*******************)
	(len (cons A b v) l) -> ~(l=O).
Induction v.Intros.Inversion H.Discriminate.
Intros.Inversion H0.Discriminate.
Qed.

Lemma len_nil_inv : (l:nat) (len (nil A) l) -> l=O.
(****************)
Intros;Inversion H;Try Trivial.
Qed.

Lemma not_len_nil_Sn : (l:nat) ~(len (nil A) (S l)).
(*******************)
Induction l.Red.Intro.Inversion H.Unfold not .Intros.Inversion H0.
Qed.

Lemma len_app : (l1,l2:nat)(v1,v2:(list A))
(************)
	(len v1 l1) -> (len v2 l2) -> (len (app A v1 v2) (plus l1 l2)).
Induction l1.Simpl.Intros.Inversion H.Simpl.Assumption.
Intros.Inversion H0.Simpl.Apply len_cons.Apply H.Exact H4.
Exact H1.
Qed.


Lemma len_length : (v:(list A)) (len v (length A v)).
(***************)
Induction v.Auto.
Intros.Rewrite -> length_eq2.Apply len_cons.Exact H.
Qed.Hint len_length.

Lemma length_eq : (v1,v2:(list A)) v1=v2 -> (length A v1)=(length A v2).
(**************)
Intros.Rewrite -> H.Try Trivial.
Qed.

Lemma not_nil : (v:(list A)) ~(length A v)=O -> ~v=(nil A).
(************)
Induction v.(Simpl;Intro).Absurd (not (eq ? O O)).(Unfold not ;Auto).Exact H.
Simpl.Intros.Discriminate.
Qed.

Lemma length_app : (v1,v2:(list A))
(***************)
	(length A (app A v1 v2))=(plus (length A v1) (length A v2)).
Induction v1.Simpl.Try Trivial.
Intros.Simpl.Rewrite -> H.Try Trivial.
Qed.

Lemma v_not_nil_length : (v:(list A))
(*********************)
	~v=(nil A) -> (le (S O) (length A v)).
Induction v.Intro.Absurd (not (eq ? (nil A) (nil A))).Unfold not .Auto.
Exact H.
Intros.Simpl.Apply le_n_S.Auto.
Qed.

Lemma le_SO_length_v : (v:(list A))
(*******************)
	 ~(length A v)=O -> (le (S O) (length A v)).
Intros.Apply v_not_nil_length.Apply not_nil.Exact H.
Qed.

(****************************************************************)
(* Rend une liste de longueur n dont tous les elements sont x *)

Fixpoint list_const  [n:nat] : A -> (list A) :=
(*****************)
	[x:A] <(list A)> Case n of
				(nil A)
				[n':nat] (cons A x (list_const n' x)) end.

Lemma list_const_eq1 : (x:A) (list_const O x)=(nil A).
Auto.Qed.
Lemma list_const_eq2 : (n':nat)(x:A)
		(list_const (S n') x)=(cons A x (list_const n' x)).
Auto.Qed.

Lemma len_list_const : (n:nat)(x:A) (len (list_const n x) n).
(*******************)
Induction n.Auto.Intros.Simpl.Auto.
Qed.Hint len_list_const.

Lemma length_list_const : (n:nat)(x:A) (length A (list_const n x))=n.
(**********************)
Induction n.Auto.Intros.Simpl.Auto.
Qed.Hint length_list_const.

(****************************************************************)
(* rev: renverse une liste *)

Fixpoint rev [l:(list A)] : (list A) :=
(************************)
	<(list A)> Case l of
			(nil A)
			[b:A][l':(list A)]
				(app A (rev l') (cons A b (nil A))) end.

Lemma rev_eq1: (rev (nil A))=(nil A).
Auto.Qed.
Lemma rev_eq2: (b:A)(l':(list A))
		(rev (cons A b l'))=(app A (rev l') (cons A b (nil A))).
Auto.Qed.

(*
Recursive Definition rev : (list A) -> (list A) :=
	  (nil A) => (nil A)
	| (cons A c l) => (app A (rev l) (cons A c (nil A))).
*)

Lemma rev_eq : (l,n:(list A)) (l=n)->((rev l)=(rev n)).
(***********)
Intros;Replace l with n;Auto.
Qed.

Lemma rev_app : (l,n:(list A)) (rev (app A l n))=(app A (rev n) (rev l)).
(************)
(Induction l;Auto).Intros.Simpl.Rewrite -> H.Apply app_assoc_r.
Qed.

Lemma rev_rev : (l:(list A)) (rev (rev l))=l.
(************)
(Induction l;Auto).Intros;Simpl.Rewrite -> rev_app.Rewrite -> H.Auto.
Qed.

Lemma length_rev : (l:(list A)) (length A (rev l))=(length A l).
(***************)
Induction l;Auto.Intros.Simpl.
Rewrite -> length_app.Simpl.Rewrite -> plus_n_SO.Rewrite -> H.Try Trivial.
Qed.

(****************************************************************)
(* map *)

Fixpoint map [l:(list A)] : (A->B) -> (list B) :=
(************************)
[f:A->B]
	<(list B)> Case l of
			(nil B)
			[b:A][l':(list A)]
                                (cons B (f b) (map l' f)) end.


Lemma length_map : (l:(list A)) (f:A->B) (length B (map l f))=(length A l).
Induction l.Auto.
Intros.Simpl.Rewrite -> H.Try Trivial.
Qed.

(****************************************************************)
(* Decidabilite *)

Lemma eq_cons : (l1,l2:(list A))(a1,a2:A)
		(cons A a1 l1)=(cons A a2 l2) -> l1=l2.
Intros.
Change (eq ? (tail A (cons A a1 l1)) (tail A (cons A a2 l2))).
Apply (f_equal (list A)).
Exact H.
Qed.
Immediate eq_cons.

(*
Lemma eq_cons2 : (l1,l2:(list A))(a1,a2:A)
		(cons A a1 l1)=(cons A a2 l2) -> a1=a2.
*)

Lemma not_eq_cons : (l1,l2:(list A))(a1,a2:A)
	~(l1=l2) -> ~((cons A a1 l1)=(cons A a2 l2)).
Red.Intros.Absurd (eq ? l1 l2).Exact H.
Apply (eq_cons l1 l2 a1 a2).Exact H0.
Qed.

Axiom eq_list_dec : (l,m:(list A)) {l=m}+{~(l=m)}.

(*
(Induction l;Induction m).Auto.
(Right;Discriminate).(Right;Discriminate).Intros.
*)


End Lists_compl.


