(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: may 1995                                               *)
(* file: Lists_field.v                                          *)
(* contents: trunc, strip, sublist and elemlist for polymorphic *)
(* lists.                                                       *)
(****************************************************************)

Require Arith_compl.
Require Export Minus.
Require Export Lists_compl.


Section Lists_field.
Variable A : Set.

Fixpoint trunc [v:(list A)] : nat -> (list A) := 
	[n:nat] <(list A)> Case v of
			(nil A)
			[b:A][w:(list A)] <(list A)> Case n of
					(nil A)
					[p:nat](cons A b (trunc w p)) end end.

Definition strip := [v:(list A)][n:nat]
	(rev A (trunc (rev A v) (minus (length A v) n))).

(* Selection d'une sous liste commencant a la position start, de
   longueur lengt *)
Definition sublist := [v:(list A)][start,lengt:nat]
				 (trunc (strip v start) lengt).

(* Selection de la liste contenant le i-eme element d'une liste *)
Definition elemlist := [v:(list A)][i:nat] (trunc (strip v i) (S O)).

(****************************************************************)
(* lemmes sur trunc *)

Lemma length_trunc : (v:(list A))(i:nat)
(*****************)
		(le i (length A v)) -> (length A (trunc v i))=i.
Induction v.Simpl.Auto.
Intros b b0 H.Induction i.Simpl.Try Trivial.
Simpl.Intros.Apply eq_S.Apply H.Apply le_S_n.Exact H1.
Qed.

Lemma trunc_inv : (v:(list A))(i:nat)(b:A)
(**************)
	(cons A b (trunc v i)) = (trunc (cons A b v) (S i)).
Simpl.Try Trivial.
Qed.

Lemma trunc_all  : (v:(list A))	(trunc v (length A v))=v.
(**************)
Induction v.Simpl.Try Trivial.
Intros.Rewrite -> length_eq2.Simpl.Rewrite -> H.Try Trivial.
Qed. Hint trunc_all.

Lemma trunc_max : (v:(list A))(i:nat)
(**************)
		(le (length A v) i) -> (trunc v i)=v.
Induction v.Simpl.Try Trivial.
Intros.Inversion H0.Rewrite -> trunc_all.Try Trivial.
Simpl.Rewrite -> H.Try Trivial.
Apply le_trans_S.Generalize H1 .Simpl.Try Trivial.
Qed.

Lemma trunc_O : (v:(list A)) (trunc v O)=(nil A).
(************)
(Induction v;Auto).
Qed.Hint trunc_O.

Lemma le_length_trunc : (v:(list A))(i:nat)	(le (length A (trunc v i)) i).
(********************)
Induction v.Simpl.Auto.
Intros.Case i.Rewrite -> trunc_O.Auto.
Intro.Simpl.Apply le_n_S.Apply H.
Qed.Hint le_length_trunc.

Lemma trunc_app : (v1,v2:(list A))(i:nat)
(**************)
	(trunc (app A v1 v2) i) =
	(app A (trunc v1 i) (trunc v2 (minus i (length A v1)))).
Induction v1.Simpl.Auto.Intros.Rewrite <- minus_n_O.Try Trivial.
Intros.Rewrite -> app_eq2.
Rewrite -> length_eq2.Elim i.Simpl.Rewrite -> trunc_O.Try Trivial.
Intros.Simpl.Rewrite -> H.Try Trivial.
Qed.Hint trunc_app.

Lemma app_v_trunc : (v1,v2:(list A))(i:nat)
(****************)
	(app A v1 (trunc v2 i))=(trunc (app A v1 v2) (plus (length A v1) i)).
Intros.Rewrite -> trunc_app.Rewrite -> (trunc_max v1 (plus (length A v1) i)).
Replace (minus (plus (length A v1) i) (length A v1)) with i.Try Trivial.Auto.
Auto.
Qed.

Lemma trunc_eq : (v1,v2:(list A))(i:nat) v1=v2 -> (trunc v1 i)=(trunc v2 i).
(*************)
Intros.Rewrite -> H.Try Trivial.
Qed.Hint trunc_eq.

Lemma trunc_sym : (v:(list A))(i,j:nat) 
(**************)
			(trunc (trunc v i) j) = (trunc (trunc v j) i).
Induction v.Simpl.Try Trivial.
(Induction i;Induction j).Try Trivial.
Repeat Rewrite -> trunc_O.Simpl.Try Trivial.
Repeat Rewrite -> trunc_O.Simpl.Try Trivial.
Intros.Simpl.Rewrite -> H.Try Trivial.
Qed.

Lemma trunc_trunc1 : (v:(list A))(i:nat)
(*****************)
	(trunc (trunc v i) i) = (trunc v i).
Induction v.Simpl.Try Trivial.
Induction i.Repeat Rewrite -> trunc_O.Try Trivial.
Intros.Simpl.Rewrite -> H.Try Trivial.
Qed.Hint trunc_trunc1.

Lemma trunc_trunc2 : (v:(list A))(i,j:nat)
(*****************)
	(le i j) -> (trunc (trunc v i) j) = (trunc v i).
Intros.Rewrite -> (trunc_max (trunc v i) j).Try Trivial.
Apply le_trans with i .Apply le_length_trunc.Exact H.
Qed.

Lemma trunc_trunc3 : (v:(list A))(i,j:nat)
(*****************)
	(le j i) -> (trunc (trunc v i) j) = (trunc v j).
Intros.Rewrite <- (trunc_max (trunc v j) i).Rewrite -> trunc_sym.Try Trivial.
Apply le_trans with j .Apply le_length_trunc.
Exact H.
Qed.

Lemma trunc_plus_petit : (v1,v2:(list A))(i,j:nat)
(*********************)
	(le j i) -> (trunc v1 i)=v2 -> (trunc v1 j)=(trunc v2 j).
Intros.Rewrite <- H0.Rewrite -> trunc_trunc3.Try Trivial.Exact H.
Qed.

(****************************************************************)
(* lemmes sur strip *)

Lemma strip_nil : (i:nat) (strip nil i)=nil.
(**************)
Intro.Auto.
Qed.Hint strip_nil.

Lemma strip_cons_S : (v:(list A))(i:nat)(b:A)
(*****************)
	(strip (cons A b v) (S i))=(strip v i).
Unfold strip .Induction i.Simpl.
Elim minus_n_O.Intro.Replace (length A v) with (length A (rev A v)).
Rewrite -> trunc_all.Rewrite -> trunc_app.Rewrite -> trunc_all.
Elim minus_n_n.Rewrite -> trunc_O.Rewrite -> app_v_nil.Try Trivial.
Apply length_rev.
Intros.Apply rev_eq.Simpl.
Rewrite -> trunc_app.Rewrite -> length_rev.Rewrite -> minus_minus_lem1.
Rewrite -> trunc_O.Rewrite -> app_v_nil.Try Trivial.
Qed.Hint strip_cons_S.

Lemma length_strip : (v:(list A))(i:nat)
(*****************)
	(le i (length A v)) -> (length A (strip v i))=(minus (length A v) i).
Unfold strip .Intros.Rewrite -> length_rev.Rewrite -> length_trunc.Try Trivial.
Rewrite -> length_rev.Apply minus_le_lem2.
Qed.

Lemma le_length_strip : (v:(list A))(i:nat)
(********************)
	(le (length A (strip v i)) (minus (length A v) i)).
Unfold strip .Intros.Rewrite -> length_rev.Apply le_length_trunc.
Qed.

Lemma strip_inv : (v:(list A))(i:nat)
(***************)
	(rev A (trunc (rev A v) (minus (length A v) i))) = (strip v i).
Unfold strip .Try Trivial.
Qed.

Lemma strip_all : (v:(list A))	(strip v (length A v))=(nil A).
(**************)
Unfold strip .Intro.Rewrite <- minus_n_n.Rewrite -> trunc_O.Auto.
Qed.Hint strip_all.

Lemma strip_max : (v:(list A))(i:nat)
(**************)
		(le (length A v) i) -> (strip v i)=(nil A).
Unfold strip .Intros.Rewrite <- rev_eq1.
Apply rev_eq.Rewrite <- length_rev.Rewrite -> minus_le_O.Auto.
Rewrite -> length_rev.Exact H.
Qed.

Lemma strip_O : (v:(list A))	(strip v O)=v.
(************)
Intro.Unfold strip .Rewrite <- minus_n_O.Rewrite <- length_rev.
Rewrite -> trunc_all.Rewrite -> rev_rev.Try Trivial.
Qed.Hint strip_O.

Lemma strip_app : (v1,v2:(list A))(i:nat)
(**************)
	  (strip (app A v1 v2) i)=
	  (app A (strip v1 i) (strip v2 (minus i (length A v1)))).
Induction v1.Simpl.Intros.Elim minus_n_O.Try Trivial.
Induction v2.Intro.Simpl.
Rewrite -> strip_nil.Rewrite -> app_v_nil.Rewrite -> app_v_nil.Try Trivial.
Induction i.
Rewrite -> strip_O.Simpl.Rewrite -> strip_O.Rewrite -> strip_O.Auto.
Intros.Rewrite -> app_eq2.Rewrite -> strip_cons_S.
Rewrite -> strip_cons_S.Rewrite -> length_eq2.Simpl.Apply H.
Qed.

Lemma strip_strip_S : (v:(list A))(i,j:nat)
(******************)
   (strip (strip v (S i)) j)=(strip (strip v i) (S j)).
Induction v.Intros.Rewrite -> strip_nil.Rewrite -> strip_nil.Try Trivial.
Induction i.Intros.Rewrite -> strip_O.
Do 2 Rewrite -> strip_cons_S.Rewrite -> strip_O.Try Trivial.
Induction j.Rewrite -> strip_O.
Repeat Rewrite -> strip_cons_S.Elim n.Rewrite -> strip_O.Try Trivial.
Intros.Rewrite <- H.Rewrite -> strip_O.Try Trivial.
Intros.Do 2 Rewrite -> strip_cons_S.Apply H.
Qed.

Lemma strip_sym : (v:(list A))(i,j:nat)
(**************)
		(strip (strip v i) j)=(strip (strip v j) i).
Induction v.Intros.Rewrite -> strip_nil.Rewrite -> strip_nil.Try Trivial.
Induction i.Intro.Rewrite -> strip_O.Rewrite -> strip_O.Try Trivial.
Induction j.Rewrite -> strip_O.Rewrite -> strip_O.Try Try Trivial.
Intros.Rewrite -> strip_cons_S.Rewrite -> strip_cons_S.
Replace (strip (strip l n) (S n0)) with (strip (strip l (S n)) n0).
Apply H.Apply strip_strip_S.
Qed.

Lemma strip_eq : (v1,v2:(list A))(i:nat) v1=v2 -> (strip v1 i)=(strip v2 i).
(*************)
Intros.Rewrite -> H.Try Trivial.
Qed.Hint strip_eq.

Lemma strip_strip : (v:(list A))(i,j:nat)
(****************)
	(strip (strip v i) j) = (strip v (plus i j)).
Induction v.Intros.Rewrite -> strip_nil.Rewrite -> strip_nil.Try Trivial.
Induction i.Intro.Simpl.Rewrite -> strip_O.Try Trivial.
Induction j.Rewrite -> strip_O.Elim plus_n_O.Try Trivial.
Intros.Rewrite -> strip_cons_S.Simpl.Rewrite -> strip_cons_S.Apply H.
Qed.Hint strip_strip.

(****************************************************************)
(* lemmes sur trunc et strip ensembles *)

Lemma app_trunc_strip : (v:(list A))(i:nat)
(********************)
		(app A (trunc v i) (strip v i))=v.
Induction v.Unfold strip .Simpl.Try Trivial.
Intros.Elim i.Rewrite -> trunc_O.Rewrite -> strip_O.Simpl.Try Trivial.
Intros.Unfold strip .Simpl.
Rewrite -> trunc_app.Rewrite -> rev_app.Rewrite -> length_rev.
Case n.Rewrite <- minus_n_O.
Rewrite <- minus_n_n.Rewrite -> trunc_O.Rewrite -> trunc_O.Simpl.
Rewrite <- length_rev.Rewrite -> trunc_all.Rewrite -> rev_rev.Try Trivial.
Intro.Replace (minus (minus (length A l) (S n0)) (length A l)) with O.
Rewrite -> trunc_O.Simpl.
Replace (rev A (trunc (rev A l) (minus (length A l) (S n0)))) with (
strip l (S n0)).
Rewrite -> H.Try Trivial.
Unfold strip .Try Trivial.
Rewrite -> minus_minus_lem1.Try Trivial.
Qed.

Lemma strip_trunc_i : (v:(list A))(i:nat) (strip (trunc v i) i)=(nil A).
(******************)
Induction v.Auto.
Induction i.Auto.
Intros.Simpl.Rewrite -> strip_cons_S.Apply H.
Qed.Hint strip_trunc_i.

Lemma strip_trunc : (v:(list A))(i,j:nat)
(****************)
	(strip (trunc v i) j) = (trunc (strip v j) (minus i j)).
Induction v.Simpl.Unfold strip .Simpl.Try Trivial.
(Induction i;Induction j).
Simpl.Rewrite -> strip_O.Rewrite -> trunc_O.Try Trivial.
Rewrite -> trunc_O.
Simpl.Intros.Unfold strip .Simpl.Rewrite -> trunc_O.Try Trivial.
Rewrite -> strip_O.Rewrite -> strip_O.Elim minus_n_O.Try Trivial.
Intros.Rewrite -> strip_cons_S.Simpl.Rewrite -> strip_cons_S.Apply H.
Qed.

Lemma trunc_strip : (v:(list A))(i,j:nat)
(****************)
	(trunc (strip v i) j) = (strip (trunc v (plus i j)) i).
Induction v.Unfold strip .Simpl.Try Trivial.
(Induction i;Induction j).Rewrite -> trunc_O.Rewrite -> strip_O.Auto.
Intros.Rewrite -> strip_O.Rewrite -> strip_O.Auto.
Rewrite -> trunc_O.Elim plus_n_O.Rewrite -> strip_trunc_i.Try Trivial.
Intros.
Rewrite -> strip_cons_S.Replace (plus (S n) (S n0)) with (S (plus n (S n0))).
Simpl.Rewrite -> strip_cons_S.Apply H.
Auto.
Qed.

(****************************************************************)
(* lemmes sur elemlist et sublist *)

Lemma elemlist_is_sublist : (v:(list A))(i:nat)
(************************)
	(elemlist v i)=(sublist v i (S O)).
Unfold elemlist sublist .Try Trivial.
Qed.

Lemma elemlist_cons_S : (v:(list A))(i:nat)(b:A)
(********************)
	(elemlist (cons A b v) (S i))=(elemlist v i).
Unfold elemlist .Intros.Rewrite -> strip_cons_S.Try Trivial.
Qed.

Lemma elemlist_cons_O : (v:(list A))(b:A)
(********************)
	(elemlist (cons A b v) O)=(cons A b nil).
Intros.
Unfold elemlist .Rewrite -> strip_O.Simpl.Rewrite -> trunc_O.Try Trivial.
Qed.

Lemma elemlist_inv : (l:(list A))(i:nat)
(*****************)
	(trunc (strip l i) (S O))=(elemlist l i).
Unfold elemlist. Trivial.
Qed.

Lemma app_trunc_elemlist : (v:(list A))(i:nat)
(***********************)
	(le (S i) (length A v)) ->
	(app A (trunc v i) (elemlist v i)) = (trunc v (S i)).
Induction v.Unfold elemlist .Simpl.Try Trivial.
Induction i.Simpl.Unfold elemlist .Rewrite -> trunc_O.
Rewrite -> strip_O.Simpl.Rewrite -> trunc_O.Try Trivial.
Intros.Simpl.Unfold elemlist .
Rewrite -> strip_cons_S.Unfold elemlist in H.Rewrite -> H.Try Trivial.
Generalize H1 .Simpl.Auto.
Qed.


Lemma length_elemlist : (l:(list A))(i:nat)
(********************)
	(lt i (length A l)) -> (length A (elemlist l i))=(S O).
Intros.Unfold elemlist .Rewrite -> length_trunc.Try Trivial.
Rewrite -> length_strip.Inversion H.Rewrite <- minus_Sn_m.Auto.Auto.
Rewrite <- minus_Sn_m.Auto.Apply le_trans_S.Exact H1.Apply lt_le_weak.Exact H.
Qed.


End Lists_field.
