(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: june 1995                                              *)
(* file: Lists_replace                                          *)
(* contents: definition of replace for polymorphic lists        *)
(*                                                              *)
(****************************************************************)

Require Export Lists_field.

Section Lists_replace.
Variable A : Set.

(* Remplace l'element de l a la position par new *)
Fixpoint replace [l:(list A)] : nat -> A -> (list A) := [position:nat][new:A]
	<(list A)> Case l of
		(* nil *) (nil A)
		(* (cons x l') *)[x:A][l':(list A)]
			<(list A)> Case position of
				(* O *) (cons A new l')
				(* (S n') *)[n':nat]
					(cons A x (replace l' n' new))
			end
	end.

Lemma replace_ok : (l:(list A))(i:nat)(x:A)
(***************)
    (lt i (length A l)) -> (elemlist A (replace l i x) i)=(cons A x (nil A)).
Induction l.Simpl.Intros.Absurd (lt i O).Apply lt_n_O.
Exact H.
Induction i.Intros.Simpl.
Unfold elemlist .Rewrite -> strip_O.Simpl.(Rewrite -> trunc_O;Try Trivial).
Intros.Simpl.Clear  H0.Unfold elemlist .Rewrite -> strip_cons_S.
Rewrite elemlist_inv.
Apply H.Apply lt_S_n.Generalize H1 .Simpl.Try Trivial.
Qed.

(* On montre que tous les elements, sauf celui que l'on remplace,
   ne sont pas modifies: *)
Lemma replace_keep_others : (l:(list A))(i,p:nat)(x:A)
(************************)
	(lt i (length A l)) -> (lt p (length A l)) -> ~i=p ->
	(elemlist A (replace l p x) i)=(elemlist A l i).
Induction l.Simpl.Intros.Absurd (lt i O).Apply lt_n_O.
Exact H.
Induction i.
Intros.Unfold elemlist .Rewrite -> elemlist_inv.Rewrite -> elemlist_inv.
Generalize H1 H2 .Elim p.Intros.Absurd (not (eq ? O O)).(Unfold not ;Auto).
Exact H4.
Intros.Simpl.Rewrite -> elemlist_cons_O.Rewrite -> elemlist_cons_O.Try Trivial.
Induction p.Intros.Simpl.
Rewrite -> elemlist_cons_S.Rewrite -> elemlist_cons_S.Try Trivial.
Intros.Clear  H1.Simpl.Do 2 Rewrite -> elemlist_cons_S.
Clear  H0.Apply H.Apply lt_S_n.(Generalize H2 ;Simpl;Try Trivial).
Apply lt_S_n.(Generalize H3 ;Simpl;Try Trivial).
Generalize H4 .Red.Auto.
Qed.

Lemma length_replace : (l:(list A))(p:nat)(x:A)
(*******************)
	(lt p (length A l)) ->
	(length A (replace l p x))=(length A l).
Induction l.Simpl.Try Try Trivial.
Induction p.Intros.Simpl.Apply eq_S.Try Try Trivial.
Intros.Clear  H0.Simpl.Rewrite -> H.Try Try Trivial.
Auto.
Qed.

Lemma replace_sym : (l:(list A))(p,p':nat)(x,x':A)
(****************)
	(lt p (length A l)) -> (lt p' (length A l)) -> ~p=p' ->
	(replace (replace l p' x') p x)=(replace (replace l p x) p' x').
Induction l.Simpl.Try Trivial.
Induction p.Intros.Generalize H1 H2 .
Elim p'.Intros.(Absurd (not (eq ? O O));Unfold not ;Auto).
Intros.Simpl.Try Trivial.
Induction p'.Intros.Simpl.Try Trivial.
Intros.Clear  H1.Clear  H0.Simpl.Rewrite -> H.Try Trivial.
Auto.
Auto.
Red.Auto.
Qed.

Lemma replace_newer : (l:(list A))(p:nat)(x,x':A)
(*****************)
	(lt p (length A l)) ->
	(replace (replace l p x) p x')=(replace l p x').
Induction l.Simpl.Try Trivial.
Induction p.Simpl.Intros.Try Trivial.
Intros.Clear  H0.Simpl.Rewrite -> H.Try Trivial.
Auto.
Qed.

End Lists_replace.
