(****************************************************************)
(*           The Calculus of Inductive Constructions            *)
(*                       COQ v5.10                              *)
(*                                                              *)
(* Laurent Arditi.  Laboratoire I3S. CNRS ura 1376.             *)
(* Universite de Nice - Sophia Antipolis                        *)
(* arditi@unice.fr, http://wwwi3s.unice.fr/~arditi/lolo.html    *)
(*                                                              *)
(* date: june 1995                                              *)
(* file: Memo.v                                                 *)
(* contents: modeling of memory using list of BV.               *)
(*                                                              *)
(****************************************************************)

(* Memoires de BVs. Les adresses sont des nat, les contenus des BVs *)
(* Version AVEC les listes polymorphiques *)

Require Export Arith_compl.
Require Export Lists_replace.
Require Export BV.

Definition addr := nat.			       (* Le type des adresses *)
Definition Memo := (list BV).                  (* Le type des memoires *)
Definition MemoSize := (length BV).            (* taille d'une memoire *)

(* Une memoire de taille donnee, initialisee avec un certain BV *)
Definition MemoEmpty : nat -> BV -> Memo :=
				[n:nat][v:BV](list_const BV n v).

(* Une zone de memoire commencant a une addresse, de longueur donnee *)
Definition MemoZone : Memo -> addr -> nat -> Memo := (sublist BV).

Definition MemoRead : Memo -> addr -> Memo := (elemlist BV).
Definition MemoWrite : Memo -> addr -> BV -> Memo := (replace BV).

(* Adresse OK si elle est strictement inferieure a la taille de la memoire *)
Definition AddrOK : Memo -> addr -> Prop :=
				[m:Memo][a:addr] (lt a (MemoSize m)).

Definition MMemo : BV -> Memo := [v:BV](cons BV v (nil BV)).
(* Cree une memoire d'un seul BV. C'est a utiliser avec MemoRead car MemoRead
ne rend pas un BV mais une memoire. Il faudrait definir "car" *)

Lemma read_write : (m:Memo)(a:addr)(v:BV)
	(AddrOK m a) -> (MemoRead (MemoWrite m a v) a)=(MMemo v).
Unfold Memo AddrOK MemoRead MemoWrite MMemo .
Unfold MemoSize .Intros.(Apply replace_ok;Auto).
Qed.

(********************************************************************)
